import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";
import { isEqual } from '@antv/f-engine';
import { each, findIndex, isArray, deepMix } from '@antv/util';
import { Component } from '../index';
import { Children } from '../index';
import CoordController from '../controller/coord';
import ScaleController from '../controller/scale';
import Theme from '../theme';
// 统计图表
var Chart = /*#__PURE__*/function (_Component) {
  _inherits(Chart, _Component);
  var _super = _createSuper(Chart);
  function Chart(props, context) {
    var _this;
    _classCallCheck(this, Chart);
    _this = _super.call(this, props);
    // 坐标系
    _this.componentsPosition = [];
    var theme = context.theme,
      px2hd = context.px2hd;
    // hack 处理，设置默认的主题样式
    // 目前没想到其他更合适的方式，只能先这样处理
    deepMix(theme, px2hd(Theme));
    var data = props.data;
    _this.scale = new ScaleController(data);
    _this.coord = new CoordController();
    // state
    _this.state = {
      filters: {}
    };
    return _this;
  }
  _createClass(Chart, [{
    key: "getStyle",
    value: function getStyle(props) {
      var context = this.context,
        layout = this.layout;
      var theme = context.theme,
        px2hd = context.px2hd;
      var left = layout.left,
        top = layout.top,
        width = layout.width,
        height = layout.height;
      var customStyle = props.style;
      return px2hd(_objectSpread(_objectSpread({
        left: left,
        top: top,
        width: width,
        height: height
      }, theme.chart), customStyle));
    }
  }, {
    key: "willMount",
    value: function willMount() {
      var props = this.props,
        coord = this.coord,
        scale = this.scale;
      var scaleOptions = props.scale,
        coordOption = props.coord;
      var style = this.getStyle(props);
      coord.updateLayout(style);
      // 初始化 scale
      scale.create(scaleOptions);
      // 初始化 coord
      coord.create(coordOption);
    }
    // props 更新
  }, {
    key: "willReceiveProps",
    value: function willReceiveProps(nextProps) {
      var scale = this.scale,
        coord = this.coord,
        lastProps = this.props;
      var nextStyle = nextProps.style,
        nextData = nextProps.data,
        nextScale = nextProps.scale;
      var lastStyle = lastProps.style,
        lastData = lastProps.data,
        lastScale = lastProps.scale;
      // style 更新
      if (!isEqual(nextStyle, lastStyle)) {
        var style = this.getStyle(nextProps);
        coord.updateLayout(style);
      }
      if (nextData !== lastData) {
        scale.changeData(nextData);
      }
      // scale
      if (!isEqual(nextScale, lastScale)) {
        scale.update(nextScale);
      }
    }
    // 给需要显示的组件留空
  }, {
    key: "layoutCoord",
    value: function layoutCoord(layout) {
      this.coord.useLayout(layout);
    }
  }, {
    key: "resetCoordLayout",
    value: function resetCoordLayout() {
      var coord = this.coord,
        props = this.props;
      var style = this.getStyle(props);
      coord.updateLayout(style);
    }
  }, {
    key: "updateCoordLayout",
    value: function updateCoordLayout(layout) {
      var _this2 = this;
      if (isArray(layout)) {
        layout.forEach(function (item) {
          _this2.layoutCoord(item);
        });
        return;
      }
      this.layoutCoord(layout);
    }
  }, {
    key: "updateCoordFor",
    value: function updateCoordFor(component, layout) {
      var _this3 = this;
      if (!layout) return;
      var componentsPosition = this.componentsPosition;
      var componentPosition = {
        component: component,
        layout: layout
      };
      var existIndex = findIndex(componentsPosition, function (item) {
        return item.component === component;
      });
      // 说明是已经存在的组件
      if (existIndex > -1) {
        componentsPosition.splice(existIndex, 1, componentPosition);
        // 先重置，然后整体重新算一次
        this.resetCoordLayout();
        componentsPosition.forEach(function (componentPosition) {
          var layout = componentPosition.layout;
          _this3.updateCoordLayout(layout);
        });
        return;
      }
      // 是新组件，直接添加
      componentsPosition.push(componentPosition);
      this.updateCoordLayout(layout);
    }
  }, {
    key: "getGeometrys",
    value: function getGeometrys() {
      var children = this.children;
      var geometrys = [];
      // @ts-ignore
      Children.toArray(children).forEach(function (element) {
        if (!element) return false;
        var component = element.component;
        if (component && component.isGeometry) {
          geometrys.push(component);
        }
      });
      return geometrys;
    }
    /**
     * calculate dataset's position on canvas
     * @param  {Object} record the dataset
     * @return {Object} return the position
     */
  }, {
    key: "getPosition",
    value: function getPosition(record) {
      var coord = this.getCoord();
      var xScale = this.getXScales()[0];
      var xField = xScale.field;
      var yScales = this.getYScales();
      // default first
      var yScale = yScales[0];
      var yField = yScale.field;
      for (var i = 0, len = yScales.length; i < len; i++) {
        var scale = yScales[i];
        var field = scale.field;
        if (record[field]) {
          yScale = scale;
          yField = field;
          break;
        }
      }
      var x = xScale.scale(record[xField]);
      var y = yScale.scale(record[yField]);
      return coord.convertPoint({
        x: x,
        y: y
      });
    }
  }, {
    key: "getSnapRecords",
    value: function getSnapRecords(point, inCoordRange) {
      var geometrys = this.getGeometrys();
      if (!geometrys.length) return;
      // @ts-ignore
      return geometrys[0].getSnapRecords(point, inCoordRange);
    }
  }, {
    key: "getLegendItems",
    value: function getLegendItems(point) {
      var geometrys = this.getGeometrys();
      if (!geometrys.length) return;
      // @ts-ignore
      return geometrys[0].getLegendItems(point);
    }
  }, {
    key: "setScale",
    value: function setScale(field, option) {
      this.scale.setScale(field, option);
    }
  }, {
    key: "getScale",
    value: function getScale(field) {
      return this.scale.getScale(field);
    }
  }, {
    key: "getScales",
    value: function getScales() {
      return this.scale.getScales();
    }
  }, {
    key: "getXScales",
    value: function getXScales() {
      var geometrys = this.getGeometrys();
      return geometrys.map(function (component) {
        // @ts-ignore
        return component.getXScale();
      });
    }
  }, {
    key: "getYScales",
    value: function getYScales() {
      var geometrys = this.getGeometrys();
      return geometrys.map(function (component) {
        // @ts-ignore
        return component.getYScale();
      });
    }
  }, {
    key: "getLayout",
    value: function getLayout() {
      return this.coord.layout;
    }
  }, {
    key: "getCoord",
    value: function getCoord() {
      return this.coord.coord;
    }
  }, {
    key: "filter",
    value: function filter(field, condition) {
      var filters = this.state.filters;
      this.setState({
        filters: _objectSpread(_objectSpread({}, filters), {}, _defineProperty({}, field, condition))
      });
    }
  }, {
    key: "_getRenderData",
    value: function _getRenderData() {
      var props = this.props,
        state = this.state;
      var data = props.data;
      var filters = state.filters;
      if (!filters || !Object.keys(filters).length) {
        return data;
      }
      var filteredData = data;
      each(filters, function (condition, field) {
        if (!condition) return;
        filteredData = filteredData.filter(function (record) {
          return condition(record[field], record);
        });
      });
      return filteredData;
    }
  }, {
    key: "render",
    value: function render() {
      var _this4 = this;
      var props = this.props;
      var children = props.children,
        originData = props.data;
      if (!originData) return null;
      var data = this._getRenderData();
      var layout = this.getLayout();
      var coord = this.getCoord();
      return Children.map(children, function (child) {
        return Children.cloneElement(child, {
          chart: _this4,
          layout: layout,
          coord: coord,
          data: data
        });
      });
    }
  }]);
  return Chart;
}(Component);
export default Chart;