import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _get from "@babel/runtime/helpers/esm/get";
import _getPrototypeOf from "@babel/runtime/helpers/esm/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";
import { filter, getRange, head, isNil, last } from '@antv/util';
import Base from '../base';
/**
 * 连续度量的基类
 * @class
 */
var Continuous = /*#__PURE__*/function (_Base) {
  _inherits(Continuous, _Base);
  var _super = _createSuper(Continuous);
  function Continuous() {
    var _this;
    _classCallCheck(this, Continuous);
    _this = _super.apply(this, arguments);
    _this.isContinuous = true;
    return _this;
  }
  _createClass(Continuous, [{
    key: "scale",
    value: function scale(value) {
      if (isNil(value)) {
        return NaN;
      }
      var rangeMin = this.rangeMin();
      var rangeMax = this.rangeMax();
      var max = this.max;
      var min = this.min;
      if (max === min) {
        return rangeMin;
      }
      var percent = this.getScalePercent(value);
      return rangeMin + percent * (rangeMax - rangeMin);
    }
  }, {
    key: "init",
    value: function init() {
      _get(_getPrototypeOf(Continuous.prototype), "init", this).call(this);
      // init 完成后保证 min, max 包含 ticks 的范围
      var ticks = this.ticks;
      var firstTick = head(ticks);
      var lastTick = last(ticks);
      if (firstTick < this.min) {
        this.min = firstTick;
      }
      if (lastTick > this.max) {
        this.max = lastTick;
      }
      // strict-limit 方式
      if (!isNil(this.minLimit)) {
        this.min = firstTick;
      }
      if (!isNil(this.maxLimit)) {
        this.max = lastTick;
      }
    }
  }, {
    key: "setDomain",
    value: function setDomain() {
      var _getRange = getRange(this.values),
        min = _getRange.min,
        max = _getRange.max;
      if (isNil(this.min)) {
        this.min = min;
      }
      if (isNil(this.max)) {
        this.max = max;
      }
      if (this.min > this.max) {
        this.min = min;
        this.max = max;
      }
    }
  }, {
    key: "calculateTicks",
    value: function calculateTicks() {
      var _this2 = this;
      var ticks = _get(_getPrototypeOf(Continuous.prototype), "calculateTicks", this).call(this);
      if (!this.nice) {
        ticks = filter(ticks, function (tick) {
          return tick >= _this2.min && tick <= _this2.max;
        });
      }
      return ticks;
    }
    // 计算原始值值占的百分比
  }, {
    key: "getScalePercent",
    value: function getScalePercent(value) {
      var max = this.max;
      var min = this.min;
      return (value - min) / (max - min);
    }
  }, {
    key: "getInvertPercent",
    value: function getInvertPercent(value) {
      return (value - this.rangeMin()) / (this.rangeMax() - this.rangeMin());
    }
  }]);
  return Continuous;
}(Base);
export { Continuous as default };