import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _get from "@babel/runtime/helpers/esm/get";
import _getPrototypeOf from "@babel/runtime/helpers/esm/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";
import { each, isDate, isNil, isNumber, isString } from '@antv/util';
import { timeFormat, toTimeStamp } from '../util/time';
import Linear from './linear';
/**
 * 时间度量
 * @class
 */
var Time = /*#__PURE__*/function (_Linear) {
  _inherits(Time, _Linear);
  var _super = _createSuper(Time);
  function Time() {
    var _this;
    _classCallCheck(this, Time);
    _this = _super.apply(this, arguments);
    _this.type = 'time';
    return _this;
  }
  /**
   * @override
   */
  _createClass(Time, [{
    key: "getText",
    value: function getText(value, index) {
      var numberValue = this.translate(value);
      var formatter = this.formatter;
      return formatter ? formatter(numberValue, index) : timeFormat(numberValue, this.mask);
    }
    /**
     * @override
     */
  }, {
    key: "scale",
    value: function scale(value) {
      var v = value;
      if (isString(v) || isDate(v)) {
        v = this.translate(v);
      }
      return _get(_getPrototypeOf(Time.prototype), "scale", this).call(this, v);
    }
    /**
     * 将时间转换成数字
     * @override
     */
  }, {
    key: "translate",
    value: function translate(v) {
      return toTimeStamp(v);
    }
  }, {
    key: "initCfg",
    value: function initCfg() {
      this.tickMethod = 'time-pretty';
      this.mask = 'YYYY-MM-DD';
      this.tickCount = 7;
      this.nice = false;
    }
  }, {
    key: "setDomain",
    value: function setDomain() {
      var values = this.values;
      // 是否设置了 min, max，而不是直接取 this.min, this.max
      var minConfig = this.getConfig('min');
      var maxConfig = this.getConfig('max');
      // 如果设置了 min,max 则转换成时间戳
      if (!isNil(minConfig) || !isNumber(minConfig)) {
        this.min = this.translate(this.min);
      }
      if (!isNil(maxConfig) || !isNumber(maxConfig)) {
        this.max = this.translate(this.max);
      }
      // 没有设置 min, max 时
      if (values && values.length) {
        // 重新计算最大最小值
        var timeStamps = [];
        var min = Infinity; // 最小值
        var secondMin = min; // 次小值
        var max = 0;
        // 使用一个循环，计算min,max,secondMin
        each(values, function (v) {
          var timeStamp = toTimeStamp(v);
          if (isNaN(timeStamp)) {
            throw new TypeError("Invalid Time: ".concat(v, " in time scale!"));
          }
          if (min > timeStamp) {
            secondMin = min;
            min = timeStamp;
          } else if (secondMin > timeStamp) {
            secondMin = timeStamp;
          }
          if (max < timeStamp) {
            max = timeStamp;
          }
          timeStamps.push(timeStamp);
        });
        // 存在多个值时，设置最小间距
        if (values.length > 1) {
          this.minTickInterval = secondMin - min;
        }
        if (isNil(minConfig)) {
          this.min = min;
        }
        if (isNil(maxConfig)) {
          this.max = max;
        }
      }
    }
  }]);
  return Time;
}(Linear);
export default Time;