"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _typeof = require("@babel/runtime/helpers/typeof");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _ = _interopRequireWildcard(require("@antv/util"));
var _constant = require("../constant");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
var Adjust = /*#__PURE__*/function () {
  function Adjust(cfg) {
    (0, _classCallCheck2.default)(this, Adjust);
    var xField = cfg.xField,
      yField = cfg.yField,
      _cfg$adjustNames = cfg.adjustNames,
      adjustNames = _cfg$adjustNames === void 0 ? ['x', 'y'] : _cfg$adjustNames,
      dimValuesMap = cfg.dimValuesMap;
    this.adjustNames = adjustNames;
    this.xField = xField;
    this.yField = yField;
    this.dimValuesMap = dimValuesMap;
  }
  /**
   * 查看维度是否是 adjust 字段
   * @param dim
   */
  (0, _createClass2.default)(Adjust, [{
    key: "isAdjust",
    value: function isAdjust(dim) {
      return this.adjustNames.indexOf(dim) >= 0;
    }
  }, {
    key: "getAdjustRange",
    value: function getAdjustRange(dim, dimValue, values) {
      var yField = this.yField;
      var index = values.indexOf(dimValue);
      var length = values.length;
      var pre;
      var next;
      // 没有 y 字段，但是需要根据 y 调整
      if (!yField && this.isAdjust('y')) {
        pre = 0;
        next = 1;
      } else if (length > 1) {
        // 如果以其开头，则取之，否则取他前面一个
        pre = values[index === 0 ? 0 : index - 1];
        // 如果以其结尾，则取之，否则取他后面一个
        next = values[index === length - 1 ? length - 1 : index + 1];
        if (index !== 0) {
          pre += (dimValue - pre) / 2;
        } else {
          pre -= (next - dimValue) / 2;
        }
        if (index !== length - 1) {
          next -= (next - dimValue) / 2;
        } else {
          next += (dimValue - values[length - 2]) / 2;
        }
      } else {
        pre = dimValue === 0 ? 0 : dimValue - 0.5;
        next = dimValue === 0 ? 1 : dimValue + 0.5;
      }
      return {
        pre: pre,
        next: next
      };
    }
  }, {
    key: "adjustData",
    value: function adjustData(groupedDataArray, mergedData) {
      var _this = this;
      // 所有调整维度的值数组
      var dimValuesMap = this.getDimValues(mergedData);
      // 按照每一个分组来进行调整
      _.each(groupedDataArray, function (dataArray, index) {
        // 遍历所有数据集合
        // 每个分组中，分别按照不同的 dim 进行调整
        _.each(dimValuesMap, function (values, dim) {
          // 根据不同的度量分别调整位置
          _this.adjustDim(dim, values, dataArray, index);
        });
      });
    }
    /**
     * 对数据进行分组adjustData
     * @param data 数据
     * @param dim 分组的字段
     * @return 分组结果
     */
  }, {
    key: "groupData",
    value: function groupData(data, dim) {
      // 补齐数据空数据为默认值
      _.each(data, function (record) {
        if (record[dim] === undefined) {
          record[dim] = _constant.DEFAULT_Y;
        }
      });
      // 按照 dim 维度分组
      return _.groupBy(data, dim);
    }
    /** @override */
  }, {
    key: "adjustDim",
    value: function adjustDim(dim, values, data, index) {}
    /**
     * 获取可调整度量对应的值
     * @param mergedData 数据
     * @return 值的映射
     */
  }, {
    key: "getDimValues",
    value: function getDimValues(mergedData) {
      var xField = this.xField,
        yField = this.yField;
      var dimValuesMap = _.assign({}, this.dimValuesMap);
      // 所有的维度
      var dims = [];
      if (xField && this.isAdjust('x')) {
        dims.push(xField);
      }
      if (yField && this.isAdjust('y')) {
        dims.push(yField);
      }
      dims.forEach(function (dim) {
        if (dimValuesMap && dimValuesMap[dim]) {
          return;
        }
        // 在每个维度上，所有的值
        dimValuesMap[dim] = _.valuesOfKey(mergedData, dim).sort(function (v1, v2) {
          return v1 - v2;
        });
      });
      // 只有一维的情况下，同时调整 y，赋予默认值
      if (!yField && this.isAdjust('y')) {
        var dim = 'y';
        dimValuesMap[dim] = [_constant.DEFAULT_Y, 1]; // 默认分布在 y 轴的 0 与 1 之间
      }

      return dimValuesMap;
    }
  }]);
  return Adjust;
}();
exports.default = Adjust;