(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.F2 = {}));
}(this, (function (exports) { 'use strict';

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }
    return obj;
  }

  function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);
    if (Object.getOwnPropertySymbols) {
      var symbols = Object.getOwnPropertySymbols(object);
      enumerableOnly && (symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      })), keys.push.apply(keys, symbols);
    }
    return keys;
  }
  function _objectSpread2(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = null != arguments[i] ? arguments[i] : {};
      i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
    return target;
  }

  /**
   * Common utilities
   * @module glMatrix
   */
  // Configuration Constants
  var EPSILON = 0.000001;
  var ARRAY_TYPE = typeof Float32Array !== 'undefined' ? Float32Array : Array;
  if (!Math.hypot) Math.hypot = function () {
    var y = 0,
        i = arguments.length;

    while (i--) {
      y += arguments[i] * arguments[i];
    }

    return Math.sqrt(y);
  };

  /**
   * 3x3 Matrix
   * @module mat3
   */

  /**
   * Creates a new identity mat3
   *
   * @returns {mat3} a new 3x3 matrix
   */

  function create() {
    var out = new ARRAY_TYPE(9);

    if (ARRAY_TYPE != Float32Array) {
      out[1] = 0;
      out[2] = 0;
      out[3] = 0;
      out[5] = 0;
      out[6] = 0;
      out[7] = 0;
    }

    out[0] = 1;
    out[4] = 1;
    out[8] = 1;
    return out;
  }
  /**
   * Copies the upper-left 3x3 values into the given mat3.
   *
   * @param {mat3} out the receiving 3x3 matrix
   * @param {ReadonlyMat4} a   the source 4x4 matrix
   * @returns {mat3} out
   */

  function fromMat4(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[4];
    out[4] = a[5];
    out[5] = a[6];
    out[6] = a[8];
    out[7] = a[9];
    out[8] = a[10];
    return out;
  }
  /**
   * Create a new mat3 with the given values
   *
   * @param {Number} m00 Component in column 0, row 0 position (index 0)
   * @param {Number} m01 Component in column 0, row 1 position (index 1)
   * @param {Number} m02 Component in column 0, row 2 position (index 2)
   * @param {Number} m10 Component in column 1, row 0 position (index 3)
   * @param {Number} m11 Component in column 1, row 1 position (index 4)
   * @param {Number} m12 Component in column 1, row 2 position (index 5)
   * @param {Number} m20 Component in column 2, row 0 position (index 6)
   * @param {Number} m21 Component in column 2, row 1 position (index 7)
   * @param {Number} m22 Component in column 2, row 2 position (index 8)
   * @returns {mat3} A new mat3
   */

  function fromValues(m00, m01, m02, m10, m11, m12, m20, m21, m22) {
    var out = new ARRAY_TYPE(9);
    out[0] = m00;
    out[1] = m01;
    out[2] = m02;
    out[3] = m10;
    out[4] = m11;
    out[5] = m12;
    out[6] = m20;
    out[7] = m21;
    out[8] = m22;
    return out;
  }

  /**
   * 4x4 Matrix<br>Format: column-major, when typed out it looks like row-major<br>The matrices are being post multiplied.
   * @module mat4
   */

  /**
   * Creates a new identity mat4
   *
   * @returns {mat4} a new 4x4 matrix
   */

  function create$1() {
    var out = new ARRAY_TYPE(16);

    if (ARRAY_TYPE != Float32Array) {
      out[1] = 0;
      out[2] = 0;
      out[3] = 0;
      out[4] = 0;
      out[6] = 0;
      out[7] = 0;
      out[8] = 0;
      out[9] = 0;
      out[11] = 0;
      out[12] = 0;
      out[13] = 0;
      out[14] = 0;
    }

    out[0] = 1;
    out[5] = 1;
    out[10] = 1;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a new mat4 initialized with values from an existing matrix
   *
   * @param {ReadonlyMat4} a matrix to clone
   * @returns {mat4} a new 4x4 matrix
   */

  function clone(a) {
    var out = new ARRAY_TYPE(16);
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[3];
    out[4] = a[4];
    out[5] = a[5];
    out[6] = a[6];
    out[7] = a[7];
    out[8] = a[8];
    out[9] = a[9];
    out[10] = a[10];
    out[11] = a[11];
    out[12] = a[12];
    out[13] = a[13];
    out[14] = a[14];
    out[15] = a[15];
    return out;
  }
  /**
   * Copy the values from one mat4 to another
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the source matrix
   * @returns {mat4} out
   */

  function copy(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[3];
    out[4] = a[4];
    out[5] = a[5];
    out[6] = a[6];
    out[7] = a[7];
    out[8] = a[8];
    out[9] = a[9];
    out[10] = a[10];
    out[11] = a[11];
    out[12] = a[12];
    out[13] = a[13];
    out[14] = a[14];
    out[15] = a[15];
    return out;
  }
  /**
   * Create a new mat4 with the given values
   *
   * @param {Number} m00 Component in column 0, row 0 position (index 0)
   * @param {Number} m01 Component in column 0, row 1 position (index 1)
   * @param {Number} m02 Component in column 0, row 2 position (index 2)
   * @param {Number} m03 Component in column 0, row 3 position (index 3)
   * @param {Number} m10 Component in column 1, row 0 position (index 4)
   * @param {Number} m11 Component in column 1, row 1 position (index 5)
   * @param {Number} m12 Component in column 1, row 2 position (index 6)
   * @param {Number} m13 Component in column 1, row 3 position (index 7)
   * @param {Number} m20 Component in column 2, row 0 position (index 8)
   * @param {Number} m21 Component in column 2, row 1 position (index 9)
   * @param {Number} m22 Component in column 2, row 2 position (index 10)
   * @param {Number} m23 Component in column 2, row 3 position (index 11)
   * @param {Number} m30 Component in column 3, row 0 position (index 12)
   * @param {Number} m31 Component in column 3, row 1 position (index 13)
   * @param {Number} m32 Component in column 3, row 2 position (index 14)
   * @param {Number} m33 Component in column 3, row 3 position (index 15)
   * @returns {mat4} A new mat4
   */

  function fromValues$1(m00, m01, m02, m03, m10, m11, m12, m13, m20, m21, m22, m23, m30, m31, m32, m33) {
    var out = new ARRAY_TYPE(16);
    out[0] = m00;
    out[1] = m01;
    out[2] = m02;
    out[3] = m03;
    out[4] = m10;
    out[5] = m11;
    out[6] = m12;
    out[7] = m13;
    out[8] = m20;
    out[9] = m21;
    out[10] = m22;
    out[11] = m23;
    out[12] = m30;
    out[13] = m31;
    out[14] = m32;
    out[15] = m33;
    return out;
  }
  /**
   * Set the components of a mat4 to the given values
   *
   * @param {mat4} out the receiving matrix
   * @param {Number} m00 Component in column 0, row 0 position (index 0)
   * @param {Number} m01 Component in column 0, row 1 position (index 1)
   * @param {Number} m02 Component in column 0, row 2 position (index 2)
   * @param {Number} m03 Component in column 0, row 3 position (index 3)
   * @param {Number} m10 Component in column 1, row 0 position (index 4)
   * @param {Number} m11 Component in column 1, row 1 position (index 5)
   * @param {Number} m12 Component in column 1, row 2 position (index 6)
   * @param {Number} m13 Component in column 1, row 3 position (index 7)
   * @param {Number} m20 Component in column 2, row 0 position (index 8)
   * @param {Number} m21 Component in column 2, row 1 position (index 9)
   * @param {Number} m22 Component in column 2, row 2 position (index 10)
   * @param {Number} m23 Component in column 2, row 3 position (index 11)
   * @param {Number} m30 Component in column 3, row 0 position (index 12)
   * @param {Number} m31 Component in column 3, row 1 position (index 13)
   * @param {Number} m32 Component in column 3, row 2 position (index 14)
   * @param {Number} m33 Component in column 3, row 3 position (index 15)
   * @returns {mat4} out
   */

  function set(out, m00, m01, m02, m03, m10, m11, m12, m13, m20, m21, m22, m23, m30, m31, m32, m33) {
    out[0] = m00;
    out[1] = m01;
    out[2] = m02;
    out[3] = m03;
    out[4] = m10;
    out[5] = m11;
    out[6] = m12;
    out[7] = m13;
    out[8] = m20;
    out[9] = m21;
    out[10] = m22;
    out[11] = m23;
    out[12] = m30;
    out[13] = m31;
    out[14] = m32;
    out[15] = m33;
    return out;
  }
  /**
   * Set a mat4 to the identity matrix
   *
   * @param {mat4} out the receiving matrix
   * @returns {mat4} out
   */

  function identity(out) {
    out[0] = 1;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = 1;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 1;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Transpose the values of a mat4
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the source matrix
   * @returns {mat4} out
   */

  function transpose(out, a) {
    // If we are transposing ourselves we can skip a few steps but have to cache some values
    if (out === a) {
      var a01 = a[1],
          a02 = a[2],
          a03 = a[3];
      var a12 = a[6],
          a13 = a[7];
      var a23 = a[11];
      out[1] = a[4];
      out[2] = a[8];
      out[3] = a[12];
      out[4] = a01;
      out[6] = a[9];
      out[7] = a[13];
      out[8] = a02;
      out[9] = a12;
      out[11] = a[14];
      out[12] = a03;
      out[13] = a13;
      out[14] = a23;
    } else {
      out[0] = a[0];
      out[1] = a[4];
      out[2] = a[8];
      out[3] = a[12];
      out[4] = a[1];
      out[5] = a[5];
      out[6] = a[9];
      out[7] = a[13];
      out[8] = a[2];
      out[9] = a[6];
      out[10] = a[10];
      out[11] = a[14];
      out[12] = a[3];
      out[13] = a[7];
      out[14] = a[11];
      out[15] = a[15];
    }

    return out;
  }
  /**
   * Inverts a mat4
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the source matrix
   * @returns {mat4} out
   */

  function invert(out, a) {
    var a00 = a[0],
        a01 = a[1],
        a02 = a[2],
        a03 = a[3];
    var a10 = a[4],
        a11 = a[5],
        a12 = a[6],
        a13 = a[7];
    var a20 = a[8],
        a21 = a[9],
        a22 = a[10],
        a23 = a[11];
    var a30 = a[12],
        a31 = a[13],
        a32 = a[14],
        a33 = a[15];
    var b00 = a00 * a11 - a01 * a10;
    var b01 = a00 * a12 - a02 * a10;
    var b02 = a00 * a13 - a03 * a10;
    var b03 = a01 * a12 - a02 * a11;
    var b04 = a01 * a13 - a03 * a11;
    var b05 = a02 * a13 - a03 * a12;
    var b06 = a20 * a31 - a21 * a30;
    var b07 = a20 * a32 - a22 * a30;
    var b08 = a20 * a33 - a23 * a30;
    var b09 = a21 * a32 - a22 * a31;
    var b10 = a21 * a33 - a23 * a31;
    var b11 = a22 * a33 - a23 * a32; // Calculate the determinant

    var det = b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;

    if (!det) {
      return null;
    }

    det = 1.0 / det;
    out[0] = (a11 * b11 - a12 * b10 + a13 * b09) * det;
    out[1] = (a02 * b10 - a01 * b11 - a03 * b09) * det;
    out[2] = (a31 * b05 - a32 * b04 + a33 * b03) * det;
    out[3] = (a22 * b04 - a21 * b05 - a23 * b03) * det;
    out[4] = (a12 * b08 - a10 * b11 - a13 * b07) * det;
    out[5] = (a00 * b11 - a02 * b08 + a03 * b07) * det;
    out[6] = (a32 * b02 - a30 * b05 - a33 * b01) * det;
    out[7] = (a20 * b05 - a22 * b02 + a23 * b01) * det;
    out[8] = (a10 * b10 - a11 * b08 + a13 * b06) * det;
    out[9] = (a01 * b08 - a00 * b10 - a03 * b06) * det;
    out[10] = (a30 * b04 - a31 * b02 + a33 * b00) * det;
    out[11] = (a21 * b02 - a20 * b04 - a23 * b00) * det;
    out[12] = (a11 * b07 - a10 * b09 - a12 * b06) * det;
    out[13] = (a00 * b09 - a01 * b07 + a02 * b06) * det;
    out[14] = (a31 * b01 - a30 * b03 - a32 * b00) * det;
    out[15] = (a20 * b03 - a21 * b01 + a22 * b00) * det;
    return out;
  }
  /**
   * Calculates the adjugate of a mat4
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the source matrix
   * @returns {mat4} out
   */

  function adjoint(out, a) {
    var a00 = a[0],
        a01 = a[1],
        a02 = a[2],
        a03 = a[3];
    var a10 = a[4],
        a11 = a[5],
        a12 = a[6],
        a13 = a[7];
    var a20 = a[8],
        a21 = a[9],
        a22 = a[10],
        a23 = a[11];
    var a30 = a[12],
        a31 = a[13],
        a32 = a[14],
        a33 = a[15];
    out[0] = a11 * (a22 * a33 - a23 * a32) - a21 * (a12 * a33 - a13 * a32) + a31 * (a12 * a23 - a13 * a22);
    out[1] = -(a01 * (a22 * a33 - a23 * a32) - a21 * (a02 * a33 - a03 * a32) + a31 * (a02 * a23 - a03 * a22));
    out[2] = a01 * (a12 * a33 - a13 * a32) - a11 * (a02 * a33 - a03 * a32) + a31 * (a02 * a13 - a03 * a12);
    out[3] = -(a01 * (a12 * a23 - a13 * a22) - a11 * (a02 * a23 - a03 * a22) + a21 * (a02 * a13 - a03 * a12));
    out[4] = -(a10 * (a22 * a33 - a23 * a32) - a20 * (a12 * a33 - a13 * a32) + a30 * (a12 * a23 - a13 * a22));
    out[5] = a00 * (a22 * a33 - a23 * a32) - a20 * (a02 * a33 - a03 * a32) + a30 * (a02 * a23 - a03 * a22);
    out[6] = -(a00 * (a12 * a33 - a13 * a32) - a10 * (a02 * a33 - a03 * a32) + a30 * (a02 * a13 - a03 * a12));
    out[7] = a00 * (a12 * a23 - a13 * a22) - a10 * (a02 * a23 - a03 * a22) + a20 * (a02 * a13 - a03 * a12);
    out[8] = a10 * (a21 * a33 - a23 * a31) - a20 * (a11 * a33 - a13 * a31) + a30 * (a11 * a23 - a13 * a21);
    out[9] = -(a00 * (a21 * a33 - a23 * a31) - a20 * (a01 * a33 - a03 * a31) + a30 * (a01 * a23 - a03 * a21));
    out[10] = a00 * (a11 * a33 - a13 * a31) - a10 * (a01 * a33 - a03 * a31) + a30 * (a01 * a13 - a03 * a11);
    out[11] = -(a00 * (a11 * a23 - a13 * a21) - a10 * (a01 * a23 - a03 * a21) + a20 * (a01 * a13 - a03 * a11));
    out[12] = -(a10 * (a21 * a32 - a22 * a31) - a20 * (a11 * a32 - a12 * a31) + a30 * (a11 * a22 - a12 * a21));
    out[13] = a00 * (a21 * a32 - a22 * a31) - a20 * (a01 * a32 - a02 * a31) + a30 * (a01 * a22 - a02 * a21);
    out[14] = -(a00 * (a11 * a32 - a12 * a31) - a10 * (a01 * a32 - a02 * a31) + a30 * (a01 * a12 - a02 * a11));
    out[15] = a00 * (a11 * a22 - a12 * a21) - a10 * (a01 * a22 - a02 * a21) + a20 * (a01 * a12 - a02 * a11);
    return out;
  }
  /**
   * Calculates the determinant of a mat4
   *
   * @param {ReadonlyMat4} a the source matrix
   * @returns {Number} determinant of a
   */

  function determinant(a) {
    var a00 = a[0],
        a01 = a[1],
        a02 = a[2],
        a03 = a[3];
    var a10 = a[4],
        a11 = a[5],
        a12 = a[6],
        a13 = a[7];
    var a20 = a[8],
        a21 = a[9],
        a22 = a[10],
        a23 = a[11];
    var a30 = a[12],
        a31 = a[13],
        a32 = a[14],
        a33 = a[15];
    var b00 = a00 * a11 - a01 * a10;
    var b01 = a00 * a12 - a02 * a10;
    var b02 = a00 * a13 - a03 * a10;
    var b03 = a01 * a12 - a02 * a11;
    var b04 = a01 * a13 - a03 * a11;
    var b05 = a02 * a13 - a03 * a12;
    var b06 = a20 * a31 - a21 * a30;
    var b07 = a20 * a32 - a22 * a30;
    var b08 = a20 * a33 - a23 * a30;
    var b09 = a21 * a32 - a22 * a31;
    var b10 = a21 * a33 - a23 * a31;
    var b11 = a22 * a33 - a23 * a32; // Calculate the determinant

    return b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;
  }
  /**
   * Multiplies two mat4s
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the first operand
   * @param {ReadonlyMat4} b the second operand
   * @returns {mat4} out
   */

  function multiply(out, a, b) {
    var a00 = a[0],
        a01 = a[1],
        a02 = a[2],
        a03 = a[3];
    var a10 = a[4],
        a11 = a[5],
        a12 = a[6],
        a13 = a[7];
    var a20 = a[8],
        a21 = a[9],
        a22 = a[10],
        a23 = a[11];
    var a30 = a[12],
        a31 = a[13],
        a32 = a[14],
        a33 = a[15]; // Cache only the current line of the second matrix

    var b0 = b[0],
        b1 = b[1],
        b2 = b[2],
        b3 = b[3];
    out[0] = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
    out[1] = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
    out[2] = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
    out[3] = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
    b0 = b[4];
    b1 = b[5];
    b2 = b[6];
    b3 = b[7];
    out[4] = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
    out[5] = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
    out[6] = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
    out[7] = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
    b0 = b[8];
    b1 = b[9];
    b2 = b[10];
    b3 = b[11];
    out[8] = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
    out[9] = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
    out[10] = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
    out[11] = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
    b0 = b[12];
    b1 = b[13];
    b2 = b[14];
    b3 = b[15];
    out[12] = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
    out[13] = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
    out[14] = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
    out[15] = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
    return out;
  }
  /**
   * Translate a mat4 by the given vector
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to translate
   * @param {ReadonlyVec3} v vector to translate by
   * @returns {mat4} out
   */

  function translate(out, a, v) {
    var x = v[0],
        y = v[1],
        z = v[2];
    var a00, a01, a02, a03;
    var a10, a11, a12, a13;
    var a20, a21, a22, a23;

    if (a === out) {
      out[12] = a[0] * x + a[4] * y + a[8] * z + a[12];
      out[13] = a[1] * x + a[5] * y + a[9] * z + a[13];
      out[14] = a[2] * x + a[6] * y + a[10] * z + a[14];
      out[15] = a[3] * x + a[7] * y + a[11] * z + a[15];
    } else {
      a00 = a[0];
      a01 = a[1];
      a02 = a[2];
      a03 = a[3];
      a10 = a[4];
      a11 = a[5];
      a12 = a[6];
      a13 = a[7];
      a20 = a[8];
      a21 = a[9];
      a22 = a[10];
      a23 = a[11];
      out[0] = a00;
      out[1] = a01;
      out[2] = a02;
      out[3] = a03;
      out[4] = a10;
      out[5] = a11;
      out[6] = a12;
      out[7] = a13;
      out[8] = a20;
      out[9] = a21;
      out[10] = a22;
      out[11] = a23;
      out[12] = a00 * x + a10 * y + a20 * z + a[12];
      out[13] = a01 * x + a11 * y + a21 * z + a[13];
      out[14] = a02 * x + a12 * y + a22 * z + a[14];
      out[15] = a03 * x + a13 * y + a23 * z + a[15];
    }

    return out;
  }
  /**
   * Scales the mat4 by the dimensions in the given vec3 not using vectorization
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to scale
   * @param {ReadonlyVec3} v the vec3 to scale the matrix by
   * @returns {mat4} out
   **/

  function scale(out, a, v) {
    var x = v[0],
        y = v[1],
        z = v[2];
    out[0] = a[0] * x;
    out[1] = a[1] * x;
    out[2] = a[2] * x;
    out[3] = a[3] * x;
    out[4] = a[4] * y;
    out[5] = a[5] * y;
    out[6] = a[6] * y;
    out[7] = a[7] * y;
    out[8] = a[8] * z;
    out[9] = a[9] * z;
    out[10] = a[10] * z;
    out[11] = a[11] * z;
    out[12] = a[12];
    out[13] = a[13];
    out[14] = a[14];
    out[15] = a[15];
    return out;
  }
  /**
   * Rotates a mat4 by the given angle around the given axis
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @param {ReadonlyVec3} axis the axis to rotate around
   * @returns {mat4} out
   */

  function rotate(out, a, rad, axis) {
    var x = axis[0],
        y = axis[1],
        z = axis[2];
    var len = Math.hypot(x, y, z);
    var s, c, t;
    var a00, a01, a02, a03;
    var a10, a11, a12, a13;
    var a20, a21, a22, a23;
    var b00, b01, b02;
    var b10, b11, b12;
    var b20, b21, b22;

    if (len < EPSILON) {
      return null;
    }

    len = 1 / len;
    x *= len;
    y *= len;
    z *= len;
    s = Math.sin(rad);
    c = Math.cos(rad);
    t = 1 - c;
    a00 = a[0];
    a01 = a[1];
    a02 = a[2];
    a03 = a[3];
    a10 = a[4];
    a11 = a[5];
    a12 = a[6];
    a13 = a[7];
    a20 = a[8];
    a21 = a[9];
    a22 = a[10];
    a23 = a[11]; // Construct the elements of the rotation matrix

    b00 = x * x * t + c;
    b01 = y * x * t + z * s;
    b02 = z * x * t - y * s;
    b10 = x * y * t - z * s;
    b11 = y * y * t + c;
    b12 = z * y * t + x * s;
    b20 = x * z * t + y * s;
    b21 = y * z * t - x * s;
    b22 = z * z * t + c; // Perform rotation-specific matrix multiplication

    out[0] = a00 * b00 + a10 * b01 + a20 * b02;
    out[1] = a01 * b00 + a11 * b01 + a21 * b02;
    out[2] = a02 * b00 + a12 * b01 + a22 * b02;
    out[3] = a03 * b00 + a13 * b01 + a23 * b02;
    out[4] = a00 * b10 + a10 * b11 + a20 * b12;
    out[5] = a01 * b10 + a11 * b11 + a21 * b12;
    out[6] = a02 * b10 + a12 * b11 + a22 * b12;
    out[7] = a03 * b10 + a13 * b11 + a23 * b12;
    out[8] = a00 * b20 + a10 * b21 + a20 * b22;
    out[9] = a01 * b20 + a11 * b21 + a21 * b22;
    out[10] = a02 * b20 + a12 * b21 + a22 * b22;
    out[11] = a03 * b20 + a13 * b21 + a23 * b22;

    if (a !== out) {
      // If the source and destination differ, copy the unchanged last row
      out[12] = a[12];
      out[13] = a[13];
      out[14] = a[14];
      out[15] = a[15];
    }

    return out;
  }
  /**
   * Rotates a matrix by the given angle around the X axis
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function rotateX(out, a, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad);
    var a10 = a[4];
    var a11 = a[5];
    var a12 = a[6];
    var a13 = a[7];
    var a20 = a[8];
    var a21 = a[9];
    var a22 = a[10];
    var a23 = a[11];

    if (a !== out) {
      // If the source and destination differ, copy the unchanged rows
      out[0] = a[0];
      out[1] = a[1];
      out[2] = a[2];
      out[3] = a[3];
      out[12] = a[12];
      out[13] = a[13];
      out[14] = a[14];
      out[15] = a[15];
    } // Perform axis-specific matrix multiplication


    out[4] = a10 * c + a20 * s;
    out[5] = a11 * c + a21 * s;
    out[6] = a12 * c + a22 * s;
    out[7] = a13 * c + a23 * s;
    out[8] = a20 * c - a10 * s;
    out[9] = a21 * c - a11 * s;
    out[10] = a22 * c - a12 * s;
    out[11] = a23 * c - a13 * s;
    return out;
  }
  /**
   * Rotates a matrix by the given angle around the Y axis
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function rotateY(out, a, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad);
    var a00 = a[0];
    var a01 = a[1];
    var a02 = a[2];
    var a03 = a[3];
    var a20 = a[8];
    var a21 = a[9];
    var a22 = a[10];
    var a23 = a[11];

    if (a !== out) {
      // If the source and destination differ, copy the unchanged rows
      out[4] = a[4];
      out[5] = a[5];
      out[6] = a[6];
      out[7] = a[7];
      out[12] = a[12];
      out[13] = a[13];
      out[14] = a[14];
      out[15] = a[15];
    } // Perform axis-specific matrix multiplication


    out[0] = a00 * c - a20 * s;
    out[1] = a01 * c - a21 * s;
    out[2] = a02 * c - a22 * s;
    out[3] = a03 * c - a23 * s;
    out[8] = a00 * s + a20 * c;
    out[9] = a01 * s + a21 * c;
    out[10] = a02 * s + a22 * c;
    out[11] = a03 * s + a23 * c;
    return out;
  }
  /**
   * Rotates a matrix by the given angle around the Z axis
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function rotateZ(out, a, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad);
    var a00 = a[0];
    var a01 = a[1];
    var a02 = a[2];
    var a03 = a[3];
    var a10 = a[4];
    var a11 = a[5];
    var a12 = a[6];
    var a13 = a[7];

    if (a !== out) {
      // If the source and destination differ, copy the unchanged last row
      out[8] = a[8];
      out[9] = a[9];
      out[10] = a[10];
      out[11] = a[11];
      out[12] = a[12];
      out[13] = a[13];
      out[14] = a[14];
      out[15] = a[15];
    } // Perform axis-specific matrix multiplication


    out[0] = a00 * c + a10 * s;
    out[1] = a01 * c + a11 * s;
    out[2] = a02 * c + a12 * s;
    out[3] = a03 * c + a13 * s;
    out[4] = a10 * c - a00 * s;
    out[5] = a11 * c - a01 * s;
    out[6] = a12 * c - a02 * s;
    out[7] = a13 * c - a03 * s;
    return out;
  }
  /**
   * Creates a matrix from a vector translation
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.translate(dest, dest, vec);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {ReadonlyVec3} v Translation vector
   * @returns {mat4} out
   */

  function fromTranslation(out, v) {
    out[0] = 1;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = 1;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 1;
    out[11] = 0;
    out[12] = v[0];
    out[13] = v[1];
    out[14] = v[2];
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from a vector scaling
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.scale(dest, dest, vec);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {ReadonlyVec3} v Scaling vector
   * @returns {mat4} out
   */

  function fromScaling(out, v) {
    out[0] = v[0];
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = v[1];
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = v[2];
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from a given angle around a given axis
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.rotate(dest, dest, rad, axis);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {Number} rad the angle to rotate the matrix by
   * @param {ReadonlyVec3} axis the axis to rotate around
   * @returns {mat4} out
   */

  function fromRotation(out, rad, axis) {
    var x = axis[0],
        y = axis[1],
        z = axis[2];
    var len = Math.hypot(x, y, z);
    var s, c, t;

    if (len < EPSILON) {
      return null;
    }

    len = 1 / len;
    x *= len;
    y *= len;
    z *= len;
    s = Math.sin(rad);
    c = Math.cos(rad);
    t = 1 - c; // Perform rotation-specific matrix multiplication

    out[0] = x * x * t + c;
    out[1] = y * x * t + z * s;
    out[2] = z * x * t - y * s;
    out[3] = 0;
    out[4] = x * y * t - z * s;
    out[5] = y * y * t + c;
    out[6] = z * y * t + x * s;
    out[7] = 0;
    out[8] = x * z * t + y * s;
    out[9] = y * z * t - x * s;
    out[10] = z * z * t + c;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from the given angle around the X axis
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.rotateX(dest, dest, rad);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function fromXRotation(out, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad); // Perform axis-specific matrix multiplication

    out[0] = 1;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = c;
    out[6] = s;
    out[7] = 0;
    out[8] = 0;
    out[9] = -s;
    out[10] = c;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from the given angle around the Y axis
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.rotateY(dest, dest, rad);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function fromYRotation(out, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad); // Perform axis-specific matrix multiplication

    out[0] = c;
    out[1] = 0;
    out[2] = -s;
    out[3] = 0;
    out[4] = 0;
    out[5] = 1;
    out[6] = 0;
    out[7] = 0;
    out[8] = s;
    out[9] = 0;
    out[10] = c;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from the given angle around the Z axis
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.rotateZ(dest, dest, rad);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function fromZRotation(out, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad); // Perform axis-specific matrix multiplication

    out[0] = c;
    out[1] = s;
    out[2] = 0;
    out[3] = 0;
    out[4] = -s;
    out[5] = c;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 1;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from a quaternion rotation and vector translation
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.translate(dest, vec);
   *     let quatMat = mat4.create();
   *     quat4.toMat4(quat, quatMat);
   *     mat4.multiply(dest, quatMat);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {quat4} q Rotation quaternion
   * @param {ReadonlyVec3} v Translation vector
   * @returns {mat4} out
   */

  function fromRotationTranslation(out, q, v) {
    // Quaternion math
    var x = q[0],
        y = q[1],
        z = q[2],
        w = q[3];
    var x2 = x + x;
    var y2 = y + y;
    var z2 = z + z;
    var xx = x * x2;
    var xy = x * y2;
    var xz = x * z2;
    var yy = y * y2;
    var yz = y * z2;
    var zz = z * z2;
    var wx = w * x2;
    var wy = w * y2;
    var wz = w * z2;
    out[0] = 1 - (yy + zz);
    out[1] = xy + wz;
    out[2] = xz - wy;
    out[3] = 0;
    out[4] = xy - wz;
    out[5] = 1 - (xx + zz);
    out[6] = yz + wx;
    out[7] = 0;
    out[8] = xz + wy;
    out[9] = yz - wx;
    out[10] = 1 - (xx + yy);
    out[11] = 0;
    out[12] = v[0];
    out[13] = v[1];
    out[14] = v[2];
    out[15] = 1;
    return out;
  }
  /**
   * Creates a new mat4 from a dual quat.
   *
   * @param {mat4} out Matrix
   * @param {ReadonlyQuat2} a Dual Quaternion
   * @returns {mat4} mat4 receiving operation result
   */

  function fromQuat2(out, a) {
    var translation = new ARRAY_TYPE(3);
    var bx = -a[0],
        by = -a[1],
        bz = -a[2],
        bw = a[3],
        ax = a[4],
        ay = a[5],
        az = a[6],
        aw = a[7];
    var magnitude = bx * bx + by * by + bz * bz + bw * bw; //Only scale if it makes sense

    if (magnitude > 0) {
      translation[0] = (ax * bw + aw * bx + ay * bz - az * by) * 2 / magnitude;
      translation[1] = (ay * bw + aw * by + az * bx - ax * bz) * 2 / magnitude;
      translation[2] = (az * bw + aw * bz + ax * by - ay * bx) * 2 / magnitude;
    } else {
      translation[0] = (ax * bw + aw * bx + ay * bz - az * by) * 2;
      translation[1] = (ay * bw + aw * by + az * bx - ax * bz) * 2;
      translation[2] = (az * bw + aw * bz + ax * by - ay * bx) * 2;
    }

    fromRotationTranslation(out, a, translation);
    return out;
  }
  /**
   * Returns the translation vector component of a transformation
   *  matrix. If a matrix is built with fromRotationTranslation,
   *  the returned vector will be the same as the translation vector
   *  originally supplied.
   * @param  {vec3} out Vector to receive translation component
   * @param  {ReadonlyMat4} mat Matrix to be decomposed (input)
   * @return {vec3} out
   */

  function getTranslation(out, mat) {
    out[0] = mat[12];
    out[1] = mat[13];
    out[2] = mat[14];
    return out;
  }
  /**
   * Returns the scaling factor component of a transformation
   *  matrix. If a matrix is built with fromRotationTranslationScale
   *  with a normalized Quaternion paramter, the returned vector will be
   *  the same as the scaling vector
   *  originally supplied.
   * @param  {vec3} out Vector to receive scaling factor component
   * @param  {ReadonlyMat4} mat Matrix to be decomposed (input)
   * @return {vec3} out
   */

  function getScaling(out, mat) {
    var m11 = mat[0];
    var m12 = mat[1];
    var m13 = mat[2];
    var m21 = mat[4];
    var m22 = mat[5];
    var m23 = mat[6];
    var m31 = mat[8];
    var m32 = mat[9];
    var m33 = mat[10];
    out[0] = Math.hypot(m11, m12, m13);
    out[1] = Math.hypot(m21, m22, m23);
    out[2] = Math.hypot(m31, m32, m33);
    return out;
  }
  /**
   * Returns a quaternion representing the rotational component
   *  of a transformation matrix. If a matrix is built with
   *  fromRotationTranslation, the returned quaternion will be the
   *  same as the quaternion originally supplied.
   * @param {quat} out Quaternion to receive the rotation component
   * @param {ReadonlyMat4} mat Matrix to be decomposed (input)
   * @return {quat} out
   */

  function getRotation(out, mat) {
    var scaling = new ARRAY_TYPE(3);
    getScaling(scaling, mat);
    var is1 = 1 / scaling[0];
    var is2 = 1 / scaling[1];
    var is3 = 1 / scaling[2];
    var sm11 = mat[0] * is1;
    var sm12 = mat[1] * is2;
    var sm13 = mat[2] * is3;
    var sm21 = mat[4] * is1;
    var sm22 = mat[5] * is2;
    var sm23 = mat[6] * is3;
    var sm31 = mat[8] * is1;
    var sm32 = mat[9] * is2;
    var sm33 = mat[10] * is3;
    var trace = sm11 + sm22 + sm33;
    var S = 0;

    if (trace > 0) {
      S = Math.sqrt(trace + 1.0) * 2;
      out[3] = 0.25 * S;
      out[0] = (sm23 - sm32) / S;
      out[1] = (sm31 - sm13) / S;
      out[2] = (sm12 - sm21) / S;
    } else if (sm11 > sm22 && sm11 > sm33) {
      S = Math.sqrt(1.0 + sm11 - sm22 - sm33) * 2;
      out[3] = (sm23 - sm32) / S;
      out[0] = 0.25 * S;
      out[1] = (sm12 + sm21) / S;
      out[2] = (sm31 + sm13) / S;
    } else if (sm22 > sm33) {
      S = Math.sqrt(1.0 + sm22 - sm11 - sm33) * 2;
      out[3] = (sm31 - sm13) / S;
      out[0] = (sm12 + sm21) / S;
      out[1] = 0.25 * S;
      out[2] = (sm23 + sm32) / S;
    } else {
      S = Math.sqrt(1.0 + sm33 - sm11 - sm22) * 2;
      out[3] = (sm12 - sm21) / S;
      out[0] = (sm31 + sm13) / S;
      out[1] = (sm23 + sm32) / S;
      out[2] = 0.25 * S;
    }

    return out;
  }
  /**
   * Creates a matrix from a quaternion rotation, vector translation and vector scale
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.translate(dest, vec);
   *     let quatMat = mat4.create();
   *     quat4.toMat4(quat, quatMat);
   *     mat4.multiply(dest, quatMat);
   *     mat4.scale(dest, scale)
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {quat4} q Rotation quaternion
   * @param {ReadonlyVec3} v Translation vector
   * @param {ReadonlyVec3} s Scaling vector
   * @returns {mat4} out
   */

  function fromRotationTranslationScale(out, q, v, s) {
    // Quaternion math
    var x = q[0],
        y = q[1],
        z = q[2],
        w = q[3];
    var x2 = x + x;
    var y2 = y + y;
    var z2 = z + z;
    var xx = x * x2;
    var xy = x * y2;
    var xz = x * z2;
    var yy = y * y2;
    var yz = y * z2;
    var zz = z * z2;
    var wx = w * x2;
    var wy = w * y2;
    var wz = w * z2;
    var sx = s[0];
    var sy = s[1];
    var sz = s[2];
    out[0] = (1 - (yy + zz)) * sx;
    out[1] = (xy + wz) * sx;
    out[2] = (xz - wy) * sx;
    out[3] = 0;
    out[4] = (xy - wz) * sy;
    out[5] = (1 - (xx + zz)) * sy;
    out[6] = (yz + wx) * sy;
    out[7] = 0;
    out[8] = (xz + wy) * sz;
    out[9] = (yz - wx) * sz;
    out[10] = (1 - (xx + yy)) * sz;
    out[11] = 0;
    out[12] = v[0];
    out[13] = v[1];
    out[14] = v[2];
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from a quaternion rotation, vector translation and vector scale, rotating and scaling around the given origin
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.translate(dest, vec);
   *     mat4.translate(dest, origin);
   *     let quatMat = mat4.create();
   *     quat4.toMat4(quat, quatMat);
   *     mat4.multiply(dest, quatMat);
   *     mat4.scale(dest, scale)
   *     mat4.translate(dest, negativeOrigin);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {quat4} q Rotation quaternion
   * @param {ReadonlyVec3} v Translation vector
   * @param {ReadonlyVec3} s Scaling vector
   * @param {ReadonlyVec3} o The origin vector around which to scale and rotate
   * @returns {mat4} out
   */

  function fromRotationTranslationScaleOrigin(out, q, v, s, o) {
    // Quaternion math
    var x = q[0],
        y = q[1],
        z = q[2],
        w = q[3];
    var x2 = x + x;
    var y2 = y + y;
    var z2 = z + z;
    var xx = x * x2;
    var xy = x * y2;
    var xz = x * z2;
    var yy = y * y2;
    var yz = y * z2;
    var zz = z * z2;
    var wx = w * x2;
    var wy = w * y2;
    var wz = w * z2;
    var sx = s[0];
    var sy = s[1];
    var sz = s[2];
    var ox = o[0];
    var oy = o[1];
    var oz = o[2];
    var out0 = (1 - (yy + zz)) * sx;
    var out1 = (xy + wz) * sx;
    var out2 = (xz - wy) * sx;
    var out4 = (xy - wz) * sy;
    var out5 = (1 - (xx + zz)) * sy;
    var out6 = (yz + wx) * sy;
    var out8 = (xz + wy) * sz;
    var out9 = (yz - wx) * sz;
    var out10 = (1 - (xx + yy)) * sz;
    out[0] = out0;
    out[1] = out1;
    out[2] = out2;
    out[3] = 0;
    out[4] = out4;
    out[5] = out5;
    out[6] = out6;
    out[7] = 0;
    out[8] = out8;
    out[9] = out9;
    out[10] = out10;
    out[11] = 0;
    out[12] = v[0] + ox - (out0 * ox + out4 * oy + out8 * oz);
    out[13] = v[1] + oy - (out1 * ox + out5 * oy + out9 * oz);
    out[14] = v[2] + oz - (out2 * ox + out6 * oy + out10 * oz);
    out[15] = 1;
    return out;
  }
  /**
   * Calculates a 4x4 matrix from the given quaternion
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {ReadonlyQuat} q Quaternion to create matrix from
   *
   * @returns {mat4} out
   */

  function fromQuat(out, q) {
    var x = q[0],
        y = q[1],
        z = q[2],
        w = q[3];
    var x2 = x + x;
    var y2 = y + y;
    var z2 = z + z;
    var xx = x * x2;
    var yx = y * x2;
    var yy = y * y2;
    var zx = z * x2;
    var zy = z * y2;
    var zz = z * z2;
    var wx = w * x2;
    var wy = w * y2;
    var wz = w * z2;
    out[0] = 1 - yy - zz;
    out[1] = yx + wz;
    out[2] = zx - wy;
    out[3] = 0;
    out[4] = yx - wz;
    out[5] = 1 - xx - zz;
    out[6] = zy + wx;
    out[7] = 0;
    out[8] = zx + wy;
    out[9] = zy - wx;
    out[10] = 1 - xx - yy;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Generates a frustum matrix with the given bounds
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {Number} left Left bound of the frustum
   * @param {Number} right Right bound of the frustum
   * @param {Number} bottom Bottom bound of the frustum
   * @param {Number} top Top bound of the frustum
   * @param {Number} near Near bound of the frustum
   * @param {Number} far Far bound of the frustum
   * @returns {mat4} out
   */

  function frustum(out, left, right, bottom, top, near, far) {
    var rl = 1 / (right - left);
    var tb = 1 / (top - bottom);
    var nf = 1 / (near - far);
    out[0] = near * 2 * rl;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = near * 2 * tb;
    out[6] = 0;
    out[7] = 0;
    out[8] = (right + left) * rl;
    out[9] = (top + bottom) * tb;
    out[10] = (far + near) * nf;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[14] = far * near * 2 * nf;
    out[15] = 0;
    return out;
  }
  /**
   * Generates a perspective projection matrix with the given bounds.
   * The near/far clip planes correspond to a normalized device coordinate Z range of [-1, 1],
   * which matches WebGL/OpenGL's clip volume.
   * Passing null/undefined/no value for far will generate infinite projection matrix.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {number} fovy Vertical field of view in radians
   * @param {number} aspect Aspect ratio. typically viewport width/height
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum, can be null or Infinity
   * @returns {mat4} out
   */

  function perspectiveNO(out, fovy, aspect, near, far) {
    var f = 1.0 / Math.tan(fovy / 2),
        nf;
    out[0] = f / aspect;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = f;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[15] = 0;

    if (far != null && far !== Infinity) {
      nf = 1 / (near - far);
      out[10] = (far + near) * nf;
      out[14] = 2 * far * near * nf;
    } else {
      out[10] = -1;
      out[14] = -2 * near;
    }

    return out;
  }
  /**
   * Alias for {@link mat4.perspectiveNO}
   * @function
   */

  var perspective = perspectiveNO;
  /**
   * Generates a perspective projection matrix suitable for WebGPU with the given bounds.
   * The near/far clip planes correspond to a normalized device coordinate Z range of [0, 1],
   * which matches WebGPU/Vulkan/DirectX/Metal's clip volume.
   * Passing null/undefined/no value for far will generate infinite projection matrix.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {number} fovy Vertical field of view in radians
   * @param {number} aspect Aspect ratio. typically viewport width/height
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum, can be null or Infinity
   * @returns {mat4} out
   */

  function perspectiveZO(out, fovy, aspect, near, far) {
    var f = 1.0 / Math.tan(fovy / 2),
        nf;
    out[0] = f / aspect;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = f;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[15] = 0;

    if (far != null && far !== Infinity) {
      nf = 1 / (near - far);
      out[10] = far * nf;
      out[14] = far * near * nf;
    } else {
      out[10] = -1;
      out[14] = -near;
    }

    return out;
  }
  /**
   * Generates a perspective projection matrix with the given field of view.
   * This is primarily useful for generating projection matrices to be used
   * with the still experiemental WebVR API.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {Object} fov Object containing the following values: upDegrees, downDegrees, leftDegrees, rightDegrees
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum
   * @returns {mat4} out
   */

  function perspectiveFromFieldOfView(out, fov, near, far) {
    var upTan = Math.tan(fov.upDegrees * Math.PI / 180.0);
    var downTan = Math.tan(fov.downDegrees * Math.PI / 180.0);
    var leftTan = Math.tan(fov.leftDegrees * Math.PI / 180.0);
    var rightTan = Math.tan(fov.rightDegrees * Math.PI / 180.0);
    var xScale = 2.0 / (leftTan + rightTan);
    var yScale = 2.0 / (upTan + downTan);
    out[0] = xScale;
    out[1] = 0.0;
    out[2] = 0.0;
    out[3] = 0.0;
    out[4] = 0.0;
    out[5] = yScale;
    out[6] = 0.0;
    out[7] = 0.0;
    out[8] = -((leftTan - rightTan) * xScale * 0.5);
    out[9] = (upTan - downTan) * yScale * 0.5;
    out[10] = far / (near - far);
    out[11] = -1.0;
    out[12] = 0.0;
    out[13] = 0.0;
    out[14] = far * near / (near - far);
    out[15] = 0.0;
    return out;
  }
  /**
   * Generates a orthogonal projection matrix with the given bounds.
   * The near/far clip planes correspond to a normalized device coordinate Z range of [-1, 1],
   * which matches WebGL/OpenGL's clip volume.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {number} left Left bound of the frustum
   * @param {number} right Right bound of the frustum
   * @param {number} bottom Bottom bound of the frustum
   * @param {number} top Top bound of the frustum
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum
   * @returns {mat4} out
   */

  function orthoNO(out, left, right, bottom, top, near, far) {
    var lr = 1 / (left - right);
    var bt = 1 / (bottom - top);
    var nf = 1 / (near - far);
    out[0] = -2 * lr;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = -2 * bt;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 2 * nf;
    out[11] = 0;
    out[12] = (left + right) * lr;
    out[13] = (top + bottom) * bt;
    out[14] = (far + near) * nf;
    out[15] = 1;
    return out;
  }
  /**
   * Alias for {@link mat4.orthoNO}
   * @function
   */

  var ortho = orthoNO;
  /**
   * Generates a orthogonal projection matrix with the given bounds.
   * The near/far clip planes correspond to a normalized device coordinate Z range of [0, 1],
   * which matches WebGPU/Vulkan/DirectX/Metal's clip volume.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {number} left Left bound of the frustum
   * @param {number} right Right bound of the frustum
   * @param {number} bottom Bottom bound of the frustum
   * @param {number} top Top bound of the frustum
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum
   * @returns {mat4} out
   */

  function orthoZO(out, left, right, bottom, top, near, far) {
    var lr = 1 / (left - right);
    var bt = 1 / (bottom - top);
    var nf = 1 / (near - far);
    out[0] = -2 * lr;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = -2 * bt;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = nf;
    out[11] = 0;
    out[12] = (left + right) * lr;
    out[13] = (top + bottom) * bt;
    out[14] = near * nf;
    out[15] = 1;
    return out;
  }
  /**
   * Generates a look-at matrix with the given eye position, focal point, and up axis.
   * If you want a matrix that actually makes an object look at another object, you should use targetTo instead.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {ReadonlyVec3} eye Position of the viewer
   * @param {ReadonlyVec3} center Point the viewer is looking at
   * @param {ReadonlyVec3} up vec3 pointing up
   * @returns {mat4} out
   */

  function lookAt(out, eye, center, up) {
    var x0, x1, x2, y0, y1, y2, z0, z1, z2, len;
    var eyex = eye[0];
    var eyey = eye[1];
    var eyez = eye[2];
    var upx = up[0];
    var upy = up[1];
    var upz = up[2];
    var centerx = center[0];
    var centery = center[1];
    var centerz = center[2];

    if (Math.abs(eyex - centerx) < EPSILON && Math.abs(eyey - centery) < EPSILON && Math.abs(eyez - centerz) < EPSILON) {
      return identity(out);
    }

    z0 = eyex - centerx;
    z1 = eyey - centery;
    z2 = eyez - centerz;
    len = 1 / Math.hypot(z0, z1, z2);
    z0 *= len;
    z1 *= len;
    z2 *= len;
    x0 = upy * z2 - upz * z1;
    x1 = upz * z0 - upx * z2;
    x2 = upx * z1 - upy * z0;
    len = Math.hypot(x0, x1, x2);

    if (!len) {
      x0 = 0;
      x1 = 0;
      x2 = 0;
    } else {
      len = 1 / len;
      x0 *= len;
      x1 *= len;
      x2 *= len;
    }

    y0 = z1 * x2 - z2 * x1;
    y1 = z2 * x0 - z0 * x2;
    y2 = z0 * x1 - z1 * x0;
    len = Math.hypot(y0, y1, y2);

    if (!len) {
      y0 = 0;
      y1 = 0;
      y2 = 0;
    } else {
      len = 1 / len;
      y0 *= len;
      y1 *= len;
      y2 *= len;
    }

    out[0] = x0;
    out[1] = y0;
    out[2] = z0;
    out[3] = 0;
    out[4] = x1;
    out[5] = y1;
    out[6] = z1;
    out[7] = 0;
    out[8] = x2;
    out[9] = y2;
    out[10] = z2;
    out[11] = 0;
    out[12] = -(x0 * eyex + x1 * eyey + x2 * eyez);
    out[13] = -(y0 * eyex + y1 * eyey + y2 * eyez);
    out[14] = -(z0 * eyex + z1 * eyey + z2 * eyez);
    out[15] = 1;
    return out;
  }
  /**
   * Generates a matrix that makes something look at something else.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {ReadonlyVec3} eye Position of the viewer
   * @param {ReadonlyVec3} center Point the viewer is looking at
   * @param {ReadonlyVec3} up vec3 pointing up
   * @returns {mat4} out
   */

  function targetTo(out, eye, target, up) {
    var eyex = eye[0],
        eyey = eye[1],
        eyez = eye[2],
        upx = up[0],
        upy = up[1],
        upz = up[2];
    var z0 = eyex - target[0],
        z1 = eyey - target[1],
        z2 = eyez - target[2];
    var len = z0 * z0 + z1 * z1 + z2 * z2;

    if (len > 0) {
      len = 1 / Math.sqrt(len);
      z0 *= len;
      z1 *= len;
      z2 *= len;
    }

    var x0 = upy * z2 - upz * z1,
        x1 = upz * z0 - upx * z2,
        x2 = upx * z1 - upy * z0;
    len = x0 * x0 + x1 * x1 + x2 * x2;

    if (len > 0) {
      len = 1 / Math.sqrt(len);
      x0 *= len;
      x1 *= len;
      x2 *= len;
    }

    out[0] = x0;
    out[1] = x1;
    out[2] = x2;
    out[3] = 0;
    out[4] = z1 * x2 - z2 * x1;
    out[5] = z2 * x0 - z0 * x2;
    out[6] = z0 * x1 - z1 * x0;
    out[7] = 0;
    out[8] = z0;
    out[9] = z1;
    out[10] = z2;
    out[11] = 0;
    out[12] = eyex;
    out[13] = eyey;
    out[14] = eyez;
    out[15] = 1;
    return out;
  }
  /**
   * Returns a string representation of a mat4
   *
   * @param {ReadonlyMat4} a matrix to represent as a string
   * @returns {String} string representation of the matrix
   */

  function str(a) {
    return "mat4(" + a[0] + ", " + a[1] + ", " + a[2] + ", " + a[3] + ", " + a[4] + ", " + a[5] + ", " + a[6] + ", " + a[7] + ", " + a[8] + ", " + a[9] + ", " + a[10] + ", " + a[11] + ", " + a[12] + ", " + a[13] + ", " + a[14] + ", " + a[15] + ")";
  }
  /**
   * Returns Frobenius norm of a mat4
   *
   * @param {ReadonlyMat4} a the matrix to calculate Frobenius norm of
   * @returns {Number} Frobenius norm
   */

  function frob(a) {
    return Math.hypot(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15]);
  }
  /**
   * Adds two mat4's
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the first operand
   * @param {ReadonlyMat4} b the second operand
   * @returns {mat4} out
   */

  function add(out, a, b) {
    out[0] = a[0] + b[0];
    out[1] = a[1] + b[1];
    out[2] = a[2] + b[2];
    out[3] = a[3] + b[3];
    out[4] = a[4] + b[4];
    out[5] = a[5] + b[5];
    out[6] = a[6] + b[6];
    out[7] = a[7] + b[7];
    out[8] = a[8] + b[8];
    out[9] = a[9] + b[9];
    out[10] = a[10] + b[10];
    out[11] = a[11] + b[11];
    out[12] = a[12] + b[12];
    out[13] = a[13] + b[13];
    out[14] = a[14] + b[14];
    out[15] = a[15] + b[15];
    return out;
  }
  /**
   * Subtracts matrix b from matrix a
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the first operand
   * @param {ReadonlyMat4} b the second operand
   * @returns {mat4} out
   */

  function subtract(out, a, b) {
    out[0] = a[0] - b[0];
    out[1] = a[1] - b[1];
    out[2] = a[2] - b[2];
    out[3] = a[3] - b[3];
    out[4] = a[4] - b[4];
    out[5] = a[5] - b[5];
    out[6] = a[6] - b[6];
    out[7] = a[7] - b[7];
    out[8] = a[8] - b[8];
    out[9] = a[9] - b[9];
    out[10] = a[10] - b[10];
    out[11] = a[11] - b[11];
    out[12] = a[12] - b[12];
    out[13] = a[13] - b[13];
    out[14] = a[14] - b[14];
    out[15] = a[15] - b[15];
    return out;
  }
  /**
   * Multiply each element of the matrix by a scalar.
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to scale
   * @param {Number} b amount to scale the matrix's elements by
   * @returns {mat4} out
   */

  function multiplyScalar(out, a, b) {
    out[0] = a[0] * b;
    out[1] = a[1] * b;
    out[2] = a[2] * b;
    out[3] = a[3] * b;
    out[4] = a[4] * b;
    out[5] = a[5] * b;
    out[6] = a[6] * b;
    out[7] = a[7] * b;
    out[8] = a[8] * b;
    out[9] = a[9] * b;
    out[10] = a[10] * b;
    out[11] = a[11] * b;
    out[12] = a[12] * b;
    out[13] = a[13] * b;
    out[14] = a[14] * b;
    out[15] = a[15] * b;
    return out;
  }
  /**
   * Adds two mat4's after multiplying each element of the second operand by a scalar value.
   *
   * @param {mat4} out the receiving vector
   * @param {ReadonlyMat4} a the first operand
   * @param {ReadonlyMat4} b the second operand
   * @param {Number} scale the amount to scale b's elements by before adding
   * @returns {mat4} out
   */

  function multiplyScalarAndAdd(out, a, b, scale) {
    out[0] = a[0] + b[0] * scale;
    out[1] = a[1] + b[1] * scale;
    out[2] = a[2] + b[2] * scale;
    out[3] = a[3] + b[3] * scale;
    out[4] = a[4] + b[4] * scale;
    out[5] = a[5] + b[5] * scale;
    out[6] = a[6] + b[6] * scale;
    out[7] = a[7] + b[7] * scale;
    out[8] = a[8] + b[8] * scale;
    out[9] = a[9] + b[9] * scale;
    out[10] = a[10] + b[10] * scale;
    out[11] = a[11] + b[11] * scale;
    out[12] = a[12] + b[12] * scale;
    out[13] = a[13] + b[13] * scale;
    out[14] = a[14] + b[14] * scale;
    out[15] = a[15] + b[15] * scale;
    return out;
  }
  /**
   * Returns whether or not the matrices have exactly the same elements in the same position (when compared with ===)
   *
   * @param {ReadonlyMat4} a The first matrix.
   * @param {ReadonlyMat4} b The second matrix.
   * @returns {Boolean} True if the matrices are equal, false otherwise.
   */

  function exactEquals(a, b) {
    return a[0] === b[0] && a[1] === b[1] && a[2] === b[2] && a[3] === b[3] && a[4] === b[4] && a[5] === b[5] && a[6] === b[6] && a[7] === b[7] && a[8] === b[8] && a[9] === b[9] && a[10] === b[10] && a[11] === b[11] && a[12] === b[12] && a[13] === b[13] && a[14] === b[14] && a[15] === b[15];
  }
  /**
   * Returns whether or not the matrices have approximately the same elements in the same position.
   *
   * @param {ReadonlyMat4} a The first matrix.
   * @param {ReadonlyMat4} b The second matrix.
   * @returns {Boolean} True if the matrices are equal, false otherwise.
   */

  function equals(a, b) {
    var a0 = a[0],
        a1 = a[1],
        a2 = a[2],
        a3 = a[3];
    var a4 = a[4],
        a5 = a[5],
        a6 = a[6],
        a7 = a[7];
    var a8 = a[8],
        a9 = a[9],
        a10 = a[10],
        a11 = a[11];
    var a12 = a[12],
        a13 = a[13],
        a14 = a[14],
        a15 = a[15];
    var b0 = b[0],
        b1 = b[1],
        b2 = b[2],
        b3 = b[3];
    var b4 = b[4],
        b5 = b[5],
        b6 = b[6],
        b7 = b[7];
    var b8 = b[8],
        b9 = b[9],
        b10 = b[10],
        b11 = b[11];
    var b12 = b[12],
        b13 = b[13],
        b14 = b[14],
        b15 = b[15];
    return Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) && Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) && Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)) && Math.abs(a3 - b3) <= EPSILON * Math.max(1.0, Math.abs(a3), Math.abs(b3)) && Math.abs(a4 - b4) <= EPSILON * Math.max(1.0, Math.abs(a4), Math.abs(b4)) && Math.abs(a5 - b5) <= EPSILON * Math.max(1.0, Math.abs(a5), Math.abs(b5)) && Math.abs(a6 - b6) <= EPSILON * Math.max(1.0, Math.abs(a6), Math.abs(b6)) && Math.abs(a7 - b7) <= EPSILON * Math.max(1.0, Math.abs(a7), Math.abs(b7)) && Math.abs(a8 - b8) <= EPSILON * Math.max(1.0, Math.abs(a8), Math.abs(b8)) && Math.abs(a9 - b9) <= EPSILON * Math.max(1.0, Math.abs(a9), Math.abs(b9)) && Math.abs(a10 - b10) <= EPSILON * Math.max(1.0, Math.abs(a10), Math.abs(b10)) && Math.abs(a11 - b11) <= EPSILON * Math.max(1.0, Math.abs(a11), Math.abs(b11)) && Math.abs(a12 - b12) <= EPSILON * Math.max(1.0, Math.abs(a12), Math.abs(b12)) && Math.abs(a13 - b13) <= EPSILON * Math.max(1.0, Math.abs(a13), Math.abs(b13)) && Math.abs(a14 - b14) <= EPSILON * Math.max(1.0, Math.abs(a14), Math.abs(b14)) && Math.abs(a15 - b15) <= EPSILON * Math.max(1.0, Math.abs(a15), Math.abs(b15));
  }
  /**
   * Alias for {@link mat4.multiply}
   * @function
   */

  var mul = multiply;
  /**
   * Alias for {@link mat4.subtract}
   * @function
   */

  var sub = subtract;

  var mat4 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    create: create$1,
    clone: clone,
    copy: copy,
    fromValues: fromValues$1,
    set: set,
    identity: identity,
    transpose: transpose,
    invert: invert,
    adjoint: adjoint,
    determinant: determinant,
    multiply: multiply,
    translate: translate,
    scale: scale,
    rotate: rotate,
    rotateX: rotateX,
    rotateY: rotateY,
    rotateZ: rotateZ,
    fromTranslation: fromTranslation,
    fromScaling: fromScaling,
    fromRotation: fromRotation,
    fromXRotation: fromXRotation,
    fromYRotation: fromYRotation,
    fromZRotation: fromZRotation,
    fromRotationTranslation: fromRotationTranslation,
    fromQuat2: fromQuat2,
    getTranslation: getTranslation,
    getScaling: getScaling,
    getRotation: getRotation,
    fromRotationTranslationScale: fromRotationTranslationScale,
    fromRotationTranslationScaleOrigin: fromRotationTranslationScaleOrigin,
    fromQuat: fromQuat,
    frustum: frustum,
    perspectiveNO: perspectiveNO,
    perspective: perspective,
    perspectiveZO: perspectiveZO,
    perspectiveFromFieldOfView: perspectiveFromFieldOfView,
    orthoNO: orthoNO,
    ortho: ortho,
    orthoZO: orthoZO,
    lookAt: lookAt,
    targetTo: targetTo,
    str: str,
    frob: frob,
    add: add,
    subtract: subtract,
    multiplyScalar: multiplyScalar,
    multiplyScalarAndAdd: multiplyScalarAndAdd,
    exactEquals: exactEquals,
    equals: equals,
    mul: mul,
    sub: sub
  });

  /**
   * 3 Dimensional Vector
   * @module vec3
   */

  /**
   * Creates a new, empty vec3
   *
   * @returns {vec3} a new 3D vector
   */

  function create$2() {
    var out = new ARRAY_TYPE(3);

    if (ARRAY_TYPE != Float32Array) {
      out[0] = 0;
      out[1] = 0;
      out[2] = 0;
    }

    return out;
  }
  /**
   * Creates a new vec3 initialized with values from an existing vector
   *
   * @param {ReadonlyVec3} a vector to clone
   * @returns {vec3} a new 3D vector
   */

  function clone$1(a) {
    var out = new ARRAY_TYPE(3);
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    return out;
  }
  /**
   * Calculates the length of a vec3
   *
   * @param {ReadonlyVec3} a vector to calculate length of
   * @returns {Number} length of a
   */

  function length(a) {
    var x = a[0];
    var y = a[1];
    var z = a[2];
    return Math.hypot(x, y, z);
  }
  /**
   * Creates a new vec3 initialized with the given values
   *
   * @param {Number} x X component
   * @param {Number} y Y component
   * @param {Number} z Z component
   * @returns {vec3} a new 3D vector
   */

  function fromValues$2(x, y, z) {
    var out = new ARRAY_TYPE(3);
    out[0] = x;
    out[1] = y;
    out[2] = z;
    return out;
  }
  /**
   * Copy the values from one vec3 to another
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the source vector
   * @returns {vec3} out
   */

  function copy$1(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    return out;
  }
  /**
   * Set the components of a vec3 to the given values
   *
   * @param {vec3} out the receiving vector
   * @param {Number} x X component
   * @param {Number} y Y component
   * @param {Number} z Z component
   * @returns {vec3} out
   */

  function set$1(out, x, y, z) {
    out[0] = x;
    out[1] = y;
    out[2] = z;
    return out;
  }
  /**
   * Adds two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function add$1(out, a, b) {
    out[0] = a[0] + b[0];
    out[1] = a[1] + b[1];
    out[2] = a[2] + b[2];
    return out;
  }
  /**
   * Subtracts vector b from vector a
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function subtract$1(out, a, b) {
    out[0] = a[0] - b[0];
    out[1] = a[1] - b[1];
    out[2] = a[2] - b[2];
    return out;
  }
  /**
   * Multiplies two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function multiply$1(out, a, b) {
    out[0] = a[0] * b[0];
    out[1] = a[1] * b[1];
    out[2] = a[2] * b[2];
    return out;
  }
  /**
   * Returns the minimum of two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function min(out, a, b) {
    out[0] = Math.min(a[0], b[0]);
    out[1] = Math.min(a[1], b[1]);
    out[2] = Math.min(a[2], b[2]);
    return out;
  }
  /**
   * Returns the maximum of two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function max(out, a, b) {
    out[0] = Math.max(a[0], b[0]);
    out[1] = Math.max(a[1], b[1]);
    out[2] = Math.max(a[2], b[2]);
    return out;
  }
  /**
   * Scales a vec3 by a scalar number
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the vector to scale
   * @param {Number} b amount to scale the vector by
   * @returns {vec3} out
   */

  function scale$1(out, a, b) {
    out[0] = a[0] * b;
    out[1] = a[1] * b;
    out[2] = a[2] * b;
    return out;
  }
  /**
   * Normalize a vec3
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a vector to normalize
   * @returns {vec3} out
   */

  function normalize(out, a) {
    var x = a[0];
    var y = a[1];
    var z = a[2];
    var len = x * x + y * y + z * z;

    if (len > 0) {
      //TODO: evaluate use of glm_invsqrt here?
      len = 1 / Math.sqrt(len);
    }

    out[0] = a[0] * len;
    out[1] = a[1] * len;
    out[2] = a[2] * len;
    return out;
  }
  /**
   * Calculates the dot product of two vec3's
   *
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {Number} dot product of a and b
   */

  function dot(a, b) {
    return a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
  }
  /**
   * Computes the cross product of two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function cross(out, a, b) {
    var ax = a[0],
        ay = a[1],
        az = a[2];
    var bx = b[0],
        by = b[1],
        bz = b[2];
    out[0] = ay * bz - az * by;
    out[1] = az * bx - ax * bz;
    out[2] = ax * by - ay * bx;
    return out;
  }
  /**
   * Performs a linear interpolation between two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @param {Number} t interpolation amount, in the range [0-1], between the two inputs
   * @returns {vec3} out
   */

  function lerp(out, a, b, t) {
    var ax = a[0];
    var ay = a[1];
    var az = a[2];
    out[0] = ax + t * (b[0] - ax);
    out[1] = ay + t * (b[1] - ay);
    out[2] = az + t * (b[2] - az);
    return out;
  }
  /**
   * Transforms the vec3 with a mat4.
   * 4th vector component is implicitly '1'
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the vector to transform
   * @param {ReadonlyMat4} m matrix to transform with
   * @returns {vec3} out
   */

  function transformMat4(out, a, m) {
    var x = a[0],
        y = a[1],
        z = a[2];
    var w = m[3] * x + m[7] * y + m[11] * z + m[15];
    w = w || 1.0;
    out[0] = (m[0] * x + m[4] * y + m[8] * z + m[12]) / w;
    out[1] = (m[1] * x + m[5] * y + m[9] * z + m[13]) / w;
    out[2] = (m[2] * x + m[6] * y + m[10] * z + m[14]) / w;
    return out;
  }
  /**
   * Transforms the vec3 with a mat3.
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the vector to transform
   * @param {ReadonlyMat3} m the 3x3 matrix to transform with
   * @returns {vec3} out
   */

  function transformMat3(out, a, m) {
    var x = a[0],
        y = a[1],
        z = a[2];
    out[0] = x * m[0] + y * m[3] + z * m[6];
    out[1] = x * m[1] + y * m[4] + z * m[7];
    out[2] = x * m[2] + y * m[5] + z * m[8];
    return out;
  }
  /**
   * Transforms the vec3 with a quat
   * Can also be used for dual quaternions. (Multiply it with the real part)
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the vector to transform
   * @param {ReadonlyQuat} q quaternion to transform with
   * @returns {vec3} out
   */

  function transformQuat(out, a, q) {
    // benchmarks: https://jsperf.com/quaternion-transform-vec3-implementations-fixed
    var qx = q[0],
        qy = q[1],
        qz = q[2],
        qw = q[3];
    var x = a[0],
        y = a[1],
        z = a[2]; // var qvec = [qx, qy, qz];
    // var uv = vec3.cross([], qvec, a);

    var uvx = qy * z - qz * y,
        uvy = qz * x - qx * z,
        uvz = qx * y - qy * x; // var uuv = vec3.cross([], qvec, uv);

    var uuvx = qy * uvz - qz * uvy,
        uuvy = qz * uvx - qx * uvz,
        uuvz = qx * uvy - qy * uvx; // vec3.scale(uv, uv, 2 * w);

    var w2 = qw * 2;
    uvx *= w2;
    uvy *= w2;
    uvz *= w2; // vec3.scale(uuv, uuv, 2);

    uuvx *= 2;
    uuvy *= 2;
    uuvz *= 2; // return vec3.add(out, a, vec3.add(out, uv, uuv));

    out[0] = x + uvx + uuvx;
    out[1] = y + uvy + uuvy;
    out[2] = z + uvz + uuvz;
    return out;
  }
  /**
   * Returns whether or not the vectors have approximately the same elements in the same position.
   *
   * @param {ReadonlyVec3} a The first vector.
   * @param {ReadonlyVec3} b The second vector.
   * @returns {Boolean} True if the vectors are equal, false otherwise.
   */

  function equals$1(a, b) {
    var a0 = a[0],
        a1 = a[1],
        a2 = a[2];
    var b0 = b[0],
        b1 = b[1],
        b2 = b[2];
    return Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) && Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) && Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2));
  }
  /**
   * Alias for {@link vec3.subtract}
   * @function
   */

  var sub$1 = subtract$1;
  /**
   * Alias for {@link vec3.length}
   * @function
   */

  var len = length;
  /**
   * Perform some operation over an array of vec3s.
   *
   * @param {Array} a the array of vectors to iterate over
   * @param {Number} stride Number of elements between the start of each vec3. If 0 assumes tightly packed
   * @param {Number} offset Number of elements to skip at the beginning of the array
   * @param {Number} count Number of vec3s to iterate over. If 0 iterates over entire array
   * @param {Function} fn Function to call for each vector in the array
   * @param {Object} [arg] additional argument to pass to fn
   * @returns {Array} a
   * @function
   */

  var forEach = function () {
    var vec = create$2();
    return function (a, stride, offset, count, fn, arg) {
      var i, l;

      if (!stride) {
        stride = 3;
      }

      if (!offset) {
        offset = 0;
      }

      if (count) {
        l = Math.min(count * stride + offset, a.length);
      } else {
        l = a.length;
      }

      for (i = offset; i < l; i += stride) {
        vec[0] = a[i];
        vec[1] = a[i + 1];
        vec[2] = a[i + 2];
        fn(vec, vec, arg);
        a[i] = vec[0];
        a[i + 1] = vec[1];
        a[i + 2] = vec[2];
      }

      return a;
    };
  }();

  /**
   * 4 Dimensional Vector
   * @module vec4
   */

  /**
   * Creates a new, empty vec4
   *
   * @returns {vec4} a new 4D vector
   */

  function create$3() {
    var out = new ARRAY_TYPE(4);

    if (ARRAY_TYPE != Float32Array) {
      out[0] = 0;
      out[1] = 0;
      out[2] = 0;
      out[3] = 0;
    }

    return out;
  }
  /**
   * Creates a new vec4 initialized with the given values
   *
   * @param {Number} x X component
   * @param {Number} y Y component
   * @param {Number} z Z component
   * @param {Number} w W component
   * @returns {vec4} a new 4D vector
   */

  function fromValues$3(x, y, z, w) {
    var out = new ARRAY_TYPE(4);
    out[0] = x;
    out[1] = y;
    out[2] = z;
    out[3] = w;
    return out;
  }
  /**
   * Copy the values from one vec4 to another
   *
   * @param {vec4} out the receiving vector
   * @param {ReadonlyVec4} a the source vector
   * @returns {vec4} out
   */

  function copy$2(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[3];
    return out;
  }
  /**
   * Normalize a vec4
   *
   * @param {vec4} out the receiving vector
   * @param {ReadonlyVec4} a vector to normalize
   * @returns {vec4} out
   */

  function normalize$1(out, a) {
    var x = a[0];
    var y = a[1];
    var z = a[2];
    var w = a[3];
    var len = x * x + y * y + z * z + w * w;

    if (len > 0) {
      len = 1 / Math.sqrt(len);
    }

    out[0] = x * len;
    out[1] = y * len;
    out[2] = z * len;
    out[3] = w * len;
    return out;
  }
  /**
   * Transforms the vec4 with a mat4.
   *
   * @param {vec4} out the receiving vector
   * @param {ReadonlyVec4} a the vector to transform
   * @param {ReadonlyMat4} m matrix to transform with
   * @returns {vec4} out
   */

  function transformMat4$1(out, a, m) {
    var x = a[0],
        y = a[1],
        z = a[2],
        w = a[3];
    out[0] = m[0] * x + m[4] * y + m[8] * z + m[12] * w;
    out[1] = m[1] * x + m[5] * y + m[9] * z + m[13] * w;
    out[2] = m[2] * x + m[6] * y + m[10] * z + m[14] * w;
    out[3] = m[3] * x + m[7] * y + m[11] * z + m[15] * w;
    return out;
  }
  /**
   * Perform some operation over an array of vec4s.
   *
   * @param {Array} a the array of vectors to iterate over
   * @param {Number} stride Number of elements between the start of each vec4. If 0 assumes tightly packed
   * @param {Number} offset Number of elements to skip at the beginning of the array
   * @param {Number} count Number of vec4s to iterate over. If 0 iterates over entire array
   * @param {Function} fn Function to call for each vector in the array
   * @param {Object} [arg] additional argument to pass to fn
   * @returns {Array} a
   * @function
   */

  var forEach$1 = function () {
    var vec = create$3();
    return function (a, stride, offset, count, fn, arg) {
      var i, l;

      if (!stride) {
        stride = 4;
      }

      if (!offset) {
        offset = 0;
      }

      if (count) {
        l = Math.min(count * stride + offset, a.length);
      } else {
        l = a.length;
      }

      for (i = offset; i < l; i += stride) {
        vec[0] = a[i];
        vec[1] = a[i + 1];
        vec[2] = a[i + 2];
        vec[3] = a[i + 3];
        fn(vec, vec, arg);
        a[i] = vec[0];
        a[i + 1] = vec[1];
        a[i + 2] = vec[2];
        a[i + 3] = vec[3];
      }

      return a;
    };
  }();

  /**
   * Quaternion
   * @module quat
   */

  /**
   * Creates a new identity quat
   *
   * @returns {quat} a new quaternion
   */

  function create$4() {
    var out = new ARRAY_TYPE(4);

    if (ARRAY_TYPE != Float32Array) {
      out[0] = 0;
      out[1] = 0;
      out[2] = 0;
    }

    out[3] = 1;
    return out;
  }
  /**
   * Sets a quat from the given angle and rotation axis,
   * then returns it.
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyVec3} axis the axis around which to rotate
   * @param {Number} rad the angle in radians
   * @returns {quat} out
   **/

  function setAxisAngle(out, axis, rad) {
    rad = rad * 0.5;
    var s = Math.sin(rad);
    out[0] = s * axis[0];
    out[1] = s * axis[1];
    out[2] = s * axis[2];
    out[3] = Math.cos(rad);
    return out;
  }
  /**
   * Multiplies two quat's
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a the first operand
   * @param {ReadonlyQuat} b the second operand
   * @returns {quat} out
   */

  function multiply$2(out, a, b) {
    var ax = a[0],
        ay = a[1],
        az = a[2],
        aw = a[3];
    var bx = b[0],
        by = b[1],
        bz = b[2],
        bw = b[3];
    out[0] = ax * bw + aw * bx + ay * bz - az * by;
    out[1] = ay * bw + aw * by + az * bx - ax * bz;
    out[2] = az * bw + aw * bz + ax * by - ay * bx;
    out[3] = aw * bw - ax * bx - ay * by - az * bz;
    return out;
  }
  /**
   * Performs a spherical linear interpolation between two quat
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a the first operand
   * @param {ReadonlyQuat} b the second operand
   * @param {Number} t interpolation amount, in the range [0-1], between the two inputs
   * @returns {quat} out
   */

  function slerp(out, a, b, t) {
    // benchmarks:
    //    http://jsperf.com/quaternion-slerp-implementations
    var ax = a[0],
        ay = a[1],
        az = a[2],
        aw = a[3];
    var bx = b[0],
        by = b[1],
        bz = b[2],
        bw = b[3];
    var omega, cosom, sinom, scale0, scale1; // calc cosine

    cosom = ax * bx + ay * by + az * bz + aw * bw; // adjust signs (if necessary)

    if (cosom < 0.0) {
      cosom = -cosom;
      bx = -bx;
      by = -by;
      bz = -bz;
      bw = -bw;
    } // calculate coefficients


    if (1.0 - cosom > EPSILON) {
      // standard case (slerp)
      omega = Math.acos(cosom);
      sinom = Math.sin(omega);
      scale0 = Math.sin((1.0 - t) * omega) / sinom;
      scale1 = Math.sin(t * omega) / sinom;
    } else {
      // "from" and "to" quaternions are very close
      //  ... so we can do a linear interpolation
      scale0 = 1.0 - t;
      scale1 = t;
    } // calculate final values


    out[0] = scale0 * ax + scale1 * bx;
    out[1] = scale0 * ay + scale1 * by;
    out[2] = scale0 * az + scale1 * bz;
    out[3] = scale0 * aw + scale1 * bw;
    return out;
  }
  /**
   * Calculates the inverse of a quat
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a quat to calculate inverse of
   * @returns {quat} out
   */

  function invert$1(out, a) {
    var a0 = a[0],
        a1 = a[1],
        a2 = a[2],
        a3 = a[3];
    var dot = a0 * a0 + a1 * a1 + a2 * a2 + a3 * a3;
    var invDot = dot ? 1.0 / dot : 0; // TODO: Would be faster to return [0,0,0,0] immediately if dot == 0

    out[0] = -a0 * invDot;
    out[1] = -a1 * invDot;
    out[2] = -a2 * invDot;
    out[3] = a3 * invDot;
    return out;
  }
  /**
   * Creates a quaternion from the given 3x3 rotation matrix.
   *
   * NOTE: The resultant quaternion is not normalized, so you should be sure
   * to renormalize the quaternion yourself where necessary.
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyMat3} m rotation matrix
   * @returns {quat} out
   * @function
   */

  function fromMat3(out, m) {
    // Algorithm in Ken Shoemake's article in 1987 SIGGRAPH course notes
    // article "Quaternion Calculus and Fast Animation".
    var fTrace = m[0] + m[4] + m[8];
    var fRoot;

    if (fTrace > 0.0) {
      // |w| > 1/2, may as well choose w > 1/2
      fRoot = Math.sqrt(fTrace + 1.0); // 2w

      out[3] = 0.5 * fRoot;
      fRoot = 0.5 / fRoot; // 1/(4w)

      out[0] = (m[5] - m[7]) * fRoot;
      out[1] = (m[6] - m[2]) * fRoot;
      out[2] = (m[1] - m[3]) * fRoot;
    } else {
      // |w| <= 1/2
      var i = 0;
      if (m[4] > m[0]) i = 1;
      if (m[8] > m[i * 3 + i]) i = 2;
      var j = (i + 1) % 3;
      var k = (i + 2) % 3;
      fRoot = Math.sqrt(m[i * 3 + i] - m[j * 3 + j] - m[k * 3 + k] + 1.0);
      out[i] = 0.5 * fRoot;
      fRoot = 0.5 / fRoot;
      out[3] = (m[j * 3 + k] - m[k * 3 + j]) * fRoot;
      out[j] = (m[j * 3 + i] + m[i * 3 + j]) * fRoot;
      out[k] = (m[k * 3 + i] + m[i * 3 + k]) * fRoot;
    }

    return out;
  }
  /**
   * Creates a quaternion from the given euler angle x, y, z.
   *
   * @param {quat} out the receiving quaternion
   * @param {x} Angle to rotate around X axis in degrees.
   * @param {y} Angle to rotate around Y axis in degrees.
   * @param {z} Angle to rotate around Z axis in degrees.
   * @returns {quat} out
   * @function
   */

  function fromEuler(out, x, y, z) {
    var halfToRad = 0.5 * Math.PI / 180.0;
    x *= halfToRad;
    y *= halfToRad;
    z *= halfToRad;
    var sx = Math.sin(x);
    var cx = Math.cos(x);
    var sy = Math.sin(y);
    var cy = Math.cos(y);
    var sz = Math.sin(z);
    var cz = Math.cos(z);
    out[0] = sx * cy * cz - cx * sy * sz;
    out[1] = cx * sy * cz + sx * cy * sz;
    out[2] = cx * cy * sz - sx * sy * cz;
    out[3] = cx * cy * cz + sx * sy * sz;
    return out;
  }
  /**
   * Creates a new quat initialized with the given values
   *
   * @param {Number} x X component
   * @param {Number} y Y component
   * @param {Number} z Z component
   * @param {Number} w W component
   * @returns {quat} a new quaternion
   * @function
   */

  var fromValues$4 = fromValues$3;
  /**
   * Copy the values from one quat to another
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a the source quaternion
   * @returns {quat} out
   * @function
   */

  var copy$3 = copy$2;
  /**
   * Alias for {@link quat.multiply}
   * @function
   */

  var mul$1 = multiply$2;
  /**
   * Normalize a quat
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a quaternion to normalize
   * @returns {quat} out
   * @function
   */

  var normalize$2 = normalize$1;
  /**
   * Sets a quaternion to represent the shortest rotation from one
   * vector to another.
   *
   * Both vectors are assumed to be unit length.
   *
   * @param {quat} out the receiving quaternion.
   * @param {ReadonlyVec3} a the initial vector
   * @param {ReadonlyVec3} b the destination vector
   * @returns {quat} out
   */

  var rotationTo = function () {
    var tmpvec3 = create$2();
    var xUnitVec3 = fromValues$2(1, 0, 0);
    var yUnitVec3 = fromValues$2(0, 1, 0);
    return function (out, a, b) {
      var dot$1 = dot(a, b);

      if (dot$1 < -0.999999) {
        cross(tmpvec3, xUnitVec3, a);
        if (len(tmpvec3) < 0.000001) cross(tmpvec3, yUnitVec3, a);
        normalize(tmpvec3, tmpvec3);
        setAxisAngle(out, tmpvec3, Math.PI);
        return out;
      } else if (dot$1 > 0.999999) {
        out[0] = 0;
        out[1] = 0;
        out[2] = 0;
        out[3] = 1;
        return out;
      } else {
        cross(tmpvec3, a, b);
        out[0] = tmpvec3[0];
        out[1] = tmpvec3[1];
        out[2] = tmpvec3[2];
        out[3] = 1 + dot$1;
        return normalize$2(out, out);
      }
    };
  }();
  /**
   * Performs a spherical linear interpolation with two control points
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a the first operand
   * @param {ReadonlyQuat} b the second operand
   * @param {ReadonlyQuat} c the third operand
   * @param {ReadonlyQuat} d the fourth operand
   * @param {Number} t interpolation amount, in the range [0-1], between the two inputs
   * @returns {quat} out
   */

  var sqlerp = function () {
    var temp1 = create$4();
    var temp2 = create$4();
    return function (out, a, b, c, d, t) {
      slerp(temp1, a, d, t);
      slerp(temp2, b, c, t);
      slerp(out, temp1, temp2, 2 * t * (1 - t));
      return out;
    };
  }();
  /**
   * Sets the specified quaternion with values corresponding to the given
   * axes. Each axis is a vec3 and is expected to be unit length and
   * perpendicular to all other specified axes.
   *
   * @param {ReadonlyVec3} view  the vector representing the viewing direction
   * @param {ReadonlyVec3} right the vector representing the local "right" direction
   * @param {ReadonlyVec3} up    the vector representing the local "up" direction
   * @returns {quat} out
   */

  var setAxes = function () {
    var matr = create();
    return function (out, view, right, up) {
      matr[0] = right[0];
      matr[3] = right[1];
      matr[6] = right[2];
      matr[1] = up[0];
      matr[4] = up[1];
      matr[7] = up[2];
      matr[2] = -view[0];
      matr[5] = -view[1];
      matr[8] = -view[2];
      return normalize$2(out, fromMat3(out, matr));
    };
  }();

  /**
   * 2 Dimensional Vector
   * @module vec2
   */

  /**
   * Creates a new, empty vec2
   *
   * @returns {vec2} a new 2D vector
   */

  function create$5() {
    var out = new ARRAY_TYPE(2);

    if (ARRAY_TYPE != Float32Array) {
      out[0] = 0;
      out[1] = 0;
    }

    return out;
  }
  /**
   * Creates a new vec2 initialized with the given values
   *
   * @param {Number} x X component
   * @param {Number} y Y component
   * @returns {vec2} a new 2D vector
   */

  function fromValues$5(x, y) {
    var out = new ARRAY_TYPE(2);
    out[0] = x;
    out[1] = y;
    return out;
  }
  /**
   * Copy the values from one vec2 to another
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a the source vector
   * @returns {vec2} out
   */

  function copy$4(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    return out;
  }
  /**
   * Adds two vec2's
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a the first operand
   * @param {ReadonlyVec2} b the second operand
   * @returns {vec2} out
   */

  function add$2(out, a, b) {
    out[0] = a[0] + b[0];
    out[1] = a[1] + b[1];
    return out;
  }
  /**
   * Subtracts vector b from vector a
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a the first operand
   * @param {ReadonlyVec2} b the second operand
   * @returns {vec2} out
   */

  function subtract$2(out, a, b) {
    out[0] = a[0] - b[0];
    out[1] = a[1] - b[1];
    return out;
  }
  /**
   * Returns the minimum of two vec2's
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a the first operand
   * @param {ReadonlyVec2} b the second operand
   * @returns {vec2} out
   */

  function min$1(out, a, b) {
    out[0] = Math.min(a[0], b[0]);
    out[1] = Math.min(a[1], b[1]);
    return out;
  }
  /**
   * Returns the maximum of two vec2's
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a the first operand
   * @param {ReadonlyVec2} b the second operand
   * @returns {vec2} out
   */

  function max$1(out, a, b) {
    out[0] = Math.max(a[0], b[0]);
    out[1] = Math.max(a[1], b[1]);
    return out;
  }
  /**
   * Scales a vec2 by a scalar number
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a the vector to scale
   * @param {Number} b amount to scale the vector by
   * @returns {vec2} out
   */

  function scale$2(out, a, b) {
    out[0] = a[0] * b;
    out[1] = a[1] * b;
    return out;
  }
  /**
   * Calculates the euclidian distance between two vec2's
   *
   * @param {ReadonlyVec2} a the first operand
   * @param {ReadonlyVec2} b the second operand
   * @returns {Number} distance between a and b
   */

  function distance(a, b) {
    var x = b[0] - a[0],
        y = b[1] - a[1];
    return Math.hypot(x, y);
  }
  /**
   * Normalize a vec2
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a vector to normalize
   * @returns {vec2} out
   */

  function normalize$3(out, a) {
    var x = a[0],
        y = a[1];
    var len = x * x + y * y;

    if (len > 0) {
      //TODO: evaluate use of glm_invsqrt here?
      len = 1 / Math.sqrt(len);
    }

    out[0] = a[0] * len;
    out[1] = a[1] * len;
    return out;
  }
  /**
   * Calculates the dot product of two vec2's
   *
   * @param {ReadonlyVec2} a the first operand
   * @param {ReadonlyVec2} b the second operand
   * @returns {Number} dot product of a and b
   */

  function dot$1(a, b) {
    return a[0] * b[0] + a[1] * b[1];
  }
  /**
   * Returns whether or not the vectors exactly have the same elements in the same position (when compared with ===)
   *
   * @param {ReadonlyVec2} a The first vector.
   * @param {ReadonlyVec2} b The second vector.
   * @returns {Boolean} True if the vectors are equal, false otherwise.
   */

  function exactEquals$1(a, b) {
    return a[0] === b[0] && a[1] === b[1];
  }
  /**
   * Alias for {@link vec2.subtract}
   * @function
   */

  var sub$2 = subtract$2;
  /**
   * Perform some operation over an array of vec2s.
   *
   * @param {Array} a the array of vectors to iterate over
   * @param {Number} stride Number of elements between the start of each vec2. If 0 assumes tightly packed
   * @param {Number} offset Number of elements to skip at the beginning of the array
   * @param {Number} count Number of vec2s to iterate over. If 0 iterates over entire array
   * @param {Function} fn Function to call for each vector in the array
   * @param {Object} [arg] additional argument to pass to fn
   * @returns {Array} a
   * @function
   */

  var forEach$2 = function () {
    var vec = create$5();
    return function (a, stride, offset, count, fn, arg) {
      var i, l;

      if (!stride) {
        stride = 2;
      }

      if (!offset) {
        offset = 0;
      }

      if (count) {
        l = Math.min(count * stride + offset, a.length);
      } else {
        l = a.length;
      }

      for (i = offset; i < l; i += stride) {
        vec[0] = a[i];
        vec[1] = a[i + 1];
        fn(vec, vec, arg);
        a[i] = vec[0];
        a[i + 1] = vec[1];
      }

      return a;
    };
  }();

  function clonePath(path) {
      return path.map(function (x) { return (Array.isArray(x) ? [].concat(x) : x); });
  }

  /******************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  var paramsParser = {
      x1: 0,
      y1: 0,
      x2: 0,
      y2: 0,
      x: 0,
      y: 0,
      qx: null,
      qy: null,
  };

  function fixArc(pathArray, allPathCommands, i) {
      if (pathArray[i].length > 7) {
          pathArray[i].shift();
          var pi = pathArray[i];
          // const ni = i + 1;
          var ni = i;
          while (pi.length) {
              // if created multiple C:s, their original seg is saved
              allPathCommands[i] = 'A';
              // @ts-ignore
              pathArray.splice((ni += 1), 0, ['C'].concat(pi.splice(0, 6)));
          }
          pathArray.splice(i, 1);
      }
  }

  var paramsCount = {
      a: 7,
      c: 6,
      h: 1,
      l: 2,
      m: 2,
      r: 4,
      q: 4,
      s: 4,
      t: 2,
      v: 1,
      z: 0,
  };

  /**
   * Iterates an array to check if it's an actual `PathArray`.
   */
  function isPathArray(path) {
      return (Array.isArray(path) &&
          path.every(function (seg) {
              var lk = seg[0].toLowerCase();
              return paramsCount[lk] === seg.length - 1 && 'achlmqstvz'.includes(lk);
          }));
  }

  /**
   * Iterates an array to check if it's a `PathArray`
   * with all absolute values.
   */
  function isAbsoluteArray(path) {
      return (isPathArray(path) &&
          // @ts-ignore -- `isPathArray` also checks if it's `Array`
          path.every(function (_a) {
              var x = _a[0];
              return x === x.toUpperCase();
          }));
  }

  /**
   * Iterates an array to check if it's a `PathArray`
   * with all segments are in non-shorthand notation
   * with absolute values.
   */
  function isNormalizedArray(path) {
      return isAbsoluteArray(path) && path.every(function (_a) {
          var pc = _a[0];
          return 'ACLMQZ'.includes(pc);
      });
  }

  /**
   * Breaks the parsing of a pathString once a segment is finalized.
   */
  function finalizeSegment(path) {
      var pathCommand = path.pathValue[path.segmentStart];
      var LK = pathCommand.toLowerCase();
      var data = path.data;
      while (data.length >= paramsCount[LK]) {
          // overloaded `moveTo`
          // https://github.com/rveciana/svg-path-properties/blob/master/src/parse.ts
          if (LK === 'm' && data.length > 2) {
              // @ts-ignore
              path.segments.push([pathCommand].concat(data.splice(0, 2)));
              LK = 'l';
              pathCommand = pathCommand === 'm' ? 'l' : 'L';
          }
          else {
              // @ts-ignore
              path.segments.push([pathCommand].concat(data.splice(0, paramsCount[LK])));
          }
          if (!paramsCount[LK]) {
              break;
          }
      }
  }

  /**
   * Validates an A (arc-to) specific path command value.
   * Usually a `large-arc-flag` or `sweep-flag`.
   */
  function scanFlag(path) {
      var index = path.index, pathValue = path.pathValue;
      var code = pathValue.charCodeAt(index);
      if (code === 0x30 /* 0 */) {
          path.param = 0;
          path.index += 1;
          return;
      }
      if (code === 0x31 /* 1 */) {
          path.param = 1;
          path.index += 1;
          return;
      }
      path.err = "[path-util]: invalid Arc flag \"" + pathValue[index] + "\", expecting 0 or 1 at index " + index;
  }

  /**
   * Checks if the character is or belongs to a number.
   * [0-9]|+|-|.
   */
  function isDigitStart(code) {
      return ((code >= 48 && code <= 57) /* 0..9 */ || code === 0x2b /* + */ || code === 0x2d /* - */ || code === 0x2e); /* . */
  }
  function isDigit(code) {
      return code >= 48 && code <= 57; // 0..9
  }

  /**
   * Validates every character of the path string,
   * every path command, negative numbers or floating point numbers.
   */
  function scanParam(path) {
      var max = path.max, pathValue = path.pathValue, start = path.index;
      var index = start;
      var zeroFirst = false;
      var hasCeiling = false;
      var hasDecimal = false;
      var hasDot = false;
      var ch;
      if (index >= max) {
          // path.err = 'SvgPath: missed param (at pos ' + index + ')';
          path.err = "[path-util]: Invalid path value at index " + index + ", \"pathValue\" is missing param";
          return;
      }
      ch = pathValue.charCodeAt(index);
      if (ch === 0x2b /* + */ || ch === 0x2d /* - */) {
          index += 1;
          // ch = (index < max) ? pathValue.charCodeAt(index) : 0;
          ch = pathValue.charCodeAt(index);
      }
      // This logic is shamelessly borrowed from Esprima
      // https://github.com/ariya/esprimas
      if (!isDigit(ch) && ch !== 0x2e /* . */) {
          // path.err = 'SvgPath: param should start with 0..9 or `.` (at pos ' + index + ')';
          path.err = "[path-util]: Invalid path value at index " + index + ", \"" + pathValue[index] + "\" is not a number";
          return;
      }
      if (ch !== 0x2e /* . */) {
          zeroFirst = ch === 0x30 /* 0 */;
          index += 1;
          ch = pathValue.charCodeAt(index);
          if (zeroFirst && index < max) {
              // decimal number starts with '0' such as '09' is illegal.
              if (ch && isDigit(ch)) {
                  // path.err = 'SvgPath: numbers started with `0` such as `09`
                  // are illegal (at pos ' + start + ')';
                  path.err = "[path-util]: Invalid path value at index " + start + ", \"" + pathValue[start] + "\" illegal number";
                  return;
              }
          }
          while (index < max && isDigit(pathValue.charCodeAt(index))) {
              index += 1;
              hasCeiling = true;
          }
          ch = pathValue.charCodeAt(index);
      }
      if (ch === 0x2e /* . */) {
          hasDot = true;
          index += 1;
          while (isDigit(pathValue.charCodeAt(index))) {
              index += 1;
              hasDecimal = true;
          }
          ch = pathValue.charCodeAt(index);
      }
      if (ch === 0x65 /* e */ || ch === 0x45 /* E */) {
          if (hasDot && !hasCeiling && !hasDecimal) {
              path.err = "[path-util]: Invalid path value at index " + index + ", \"" + pathValue[index] + "\" invalid float exponent";
              return;
          }
          index += 1;
          ch = pathValue.charCodeAt(index);
          if (ch === 0x2b /* + */ || ch === 0x2d /* - */) {
              index += 1;
          }
          if (index < max && isDigit(pathValue.charCodeAt(index))) {
              while (index < max && isDigit(pathValue.charCodeAt(index))) {
                  index += 1;
              }
          }
          else {
              path.err = "[path-util]: Invalid path value at index " + index + ", \"" + pathValue[index] + "\" invalid integer exponent";
              return;
          }
      }
      path.index = index;
      path.param = +path.pathValue.slice(start, index);
  }

  /**
   * Checks if the character is a space.
   */
  function isSpace(ch) {
      var specialSpaces = [
          0x1680, 0x180e, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005, 0x2006, 0x2007, 0x2008, 0x2009, 0x200a, 0x202f,
          0x205f, 0x3000, 0xfeff,
      ];
      /* istanbul ignore next */
      return (ch === 0x0a ||
          ch === 0x0d ||
          ch === 0x2028 ||
          ch === 0x2029 || // Line terminators
          // White spaces
          ch === 0x20 ||
          ch === 0x09 ||
          ch === 0x0b ||
          ch === 0x0c ||
          ch === 0xa0 ||
          (ch >= 0x1680 && specialSpaces.includes(ch)));
  }

  /**
   * Points the parser to the next character in the
   * path string every time it encounters any kind of
   * space character.
   */
  function skipSpaces(path) {
      var pathValue = path.pathValue, max = path.max;
      while (path.index < max && isSpace(pathValue.charCodeAt(path.index))) {
          path.index += 1;
      }
  }

  /**
   * Checks if the character is a path command.
   */
  function isPathCommand(code) {
      // eslint-disable-next-line no-bitwise -- Impossible to satisfy
      switch (code | 0x20) {
          case 0x6d /* m */:
          case 0x7a /* z */:
          case 0x6c /* l */:
          case 0x68 /* h */:
          case 0x76 /* v */:
          case 0x63 /* c */:
          case 0x73 /* s */:
          case 0x71 /* q */:
          case 0x74 /* t */:
          case 0x61 /* a */:
              // case 0x72/* r */:
              return true;
          default:
              return false;
      }
  }

  /**
   * Checks if the character is an A (arc-to) path command.
   */
  function isArcCommand(code) {
      return (code | 0x20) === 0x61;
  }

  /**
   * Scans every character in the path string to determine
   * where a segment starts and where it ends.
   */
  function scanSegment(path) {
      var max = path.max, pathValue = path.pathValue, index = path.index;
      var cmdCode = pathValue.charCodeAt(index);
      var reqParams = paramsCount[pathValue[index].toLowerCase()];
      path.segmentStart = index;
      if (!isPathCommand(cmdCode)) {
          path.err = "[path-util]: Invalid path value \"" + pathValue[index] + "\" is not a path command";
          return;
      }
      path.index += 1;
      skipSpaces(path);
      path.data = [];
      if (!reqParams) {
          // Z
          finalizeSegment(path);
          return;
      }
      for (;;) {
          for (var i = reqParams; i > 0; i -= 1) {
              if (isArcCommand(cmdCode) && (i === 3 || i === 4))
                  scanFlag(path);
              else
                  scanParam(path);
              if (path.err.length) {
                  return;
              }
              path.data.push(path.param);
              skipSpaces(path);
              // after ',' param is mandatory
              if (path.index < max && pathValue.charCodeAt(path.index) === 0x2c /* , */) {
                  path.index += 1;
                  skipSpaces(path);
              }
          }
          if (path.index >= path.max) {
              break;
          }
          // Stop on next segment
          if (!isDigitStart(pathValue.charCodeAt(path.index))) {
              break;
          }
      }
      finalizeSegment(path);
  }

  /**
   * The `PathParser` is used by the `parsePathString` static method
   * to generate a `pathArray`.
   */
  var PathParser = /** @class */ (function () {
      function PathParser(pathString) {
          this.pathValue = pathString;
          // @ts-ignore
          this.segments = [];
          this.max = pathString.length;
          this.index = 0;
          this.param = 0.0;
          this.segmentStart = 0;
          this.data = [];
          this.err = '';
      }
      return PathParser;
  }());

  /**
   * Parses a path string value and returns an array
   * of segments we like to call `pathArray`.
   */
  function parsePathString(pathInput) {
      if (isPathArray(pathInput)) {
          return clonePath(pathInput);
      }
      var path = new PathParser(pathInput);
      skipSpaces(path);
      while (path.index < path.max && !path.err.length) {
          scanSegment(path);
      }
      return path.err ? path.err : path.segments;
  }

  function path2Absolute(pathInput) {
      if (isAbsoluteArray(pathInput)) {
          return clonePath(pathInput);
      }
      var path = parsePathString(pathInput);
      // if (!path || !path.length) {
      //   return [['M', 0, 0]];
      // }
      var x = 0;
      var y = 0;
      var mx = 0;
      var my = 0;
      // @ts-ignore
      return path.map(function (segment) {
          var values = segment.slice(1).map(Number);
          var pathCommand = segment[0];
          var absCommand = pathCommand.toUpperCase();
          if (pathCommand === 'M') {
              x = values[0], y = values[1];
              mx = x;
              my = y;
              return ['M', x, y];
          }
          var absoluteSegment;
          if (pathCommand !== absCommand) {
              switch (absCommand) {
                  case 'A':
                      absoluteSegment = [
                          absCommand,
                          values[0],
                          values[1],
                          values[2],
                          values[3],
                          values[4],
                          values[5] + x,
                          values[6] + y,
                      ];
                      break;
                  case 'V':
                      absoluteSegment = [absCommand, values[0] + y];
                      break;
                  case 'H':
                      absoluteSegment = [absCommand, values[0] + x];
                      break;
                  default: {
                      // use brakets for `eslint: no-case-declaration`
                      // https://stackoverflow.com/a/50753272/803358
                      var absValues = values.map(function (n, j) { return n + (j % 2 ? y : x); });
                      // for n, l, c, s, q, t
                      // @ts-ignore
                      absoluteSegment = [absCommand].concat(absValues);
                  }
              }
          }
          else {
              // @ts-ignore
              absoluteSegment = [absCommand].concat(values);
          }
          var segLength = absoluteSegment.length;
          switch (absCommand) {
              case 'Z':
                  x = mx;
                  y = my;
                  break;
              case 'H':
                  x = absoluteSegment[1];
                  break;
              case 'V':
                  y = absoluteSegment[1];
                  break;
              default:
                  x = absoluteSegment[segLength - 2];
                  y = absoluteSegment[segLength - 1];
                  if (absCommand === 'M') {
                      mx = x;
                      my = y;
                  }
          }
          return absoluteSegment;
      });
  }

  /**
   * Normalizes a single segment of a `PathArray` object.
   * eg. H/V -> L, T -> Q
   */
  function normalizeSegment(segment, params) {
      var pathCommand = segment[0];
      var px1 = params.x1, py1 = params.y1, px2 = params.x2, py2 = params.y2;
      var values = segment.slice(1).map(Number);
      var result = segment;
      if (!'TQ'.includes(pathCommand)) {
          // optional but good to be cautious
          params.qx = null;
          params.qy = null;
      }
      if (pathCommand === 'H') {
          result = ['L', segment[1], py1];
      }
      else if (pathCommand === 'V') {
          result = ['L', px1, segment[1]];
      }
      else if (pathCommand === 'S') {
          var x1 = px1 * 2 - px2;
          var y1 = py1 * 2 - py2;
          params.x1 = x1;
          params.y1 = y1;
          result = ['C', x1, y1].concat(values);
      }
      else if (pathCommand === 'T') {
          var qx = px1 * 2 - params.qx;
          var qy = py1 * 2 - params.qy;
          params.qx = qx;
          params.qy = qy;
          result = ['Q', qx, qy].concat(values);
      }
      else if (pathCommand === 'Q') {
          var nqx = values[0], nqy = values[1];
          params.qx = nqx;
          params.qy = nqy;
      }
      return result;
  }

  /**
   * @example
   * const path = 'M0 0 H50';
   * const normalizedPath = SVGPathCommander.normalizePath(path);
   * // result => [['M', 0, 0], ['L', 50, 0]]
   */
  function normalizePath(pathInput) {
      if (isNormalizedArray(pathInput)) {
          return clonePath(pathInput);
      }
      var path = path2Absolute(pathInput);
      var params = __assign({}, paramsParser);
      for (var i = 0; i < path.length; i += 1) {
          // Save current path command
          path[i] = normalizeSegment(path[i], params);
          var segment = path[i];
          var seglen = segment.length;
          params.x1 = +segment[seglen - 2];
          params.y1 = +segment[seglen - 1];
          params.x2 = +segment[seglen - 4] || params.x1;
          params.y2 = +segment[seglen - 3] || params.y1;
      }
      return path;
  }

  /**
   * Iterates an array to check if it's a `PathArray`
   * with all C (cubic bezier) segments.
   *
   * @param {string | PathArray} path the `Array` to be checked
   * @returns {boolean} iteration result
   */
  function isCurveArray(path) {
      return isNormalizedArray(path) && path.every(function (_a) {
          var pc = _a[0];
          return 'MC'.includes(pc);
      });
  }

  function rotateVector(x, y, rad) {
      var X = x * Math.cos(rad) - y * Math.sin(rad);
      var Y = x * Math.sin(rad) + y * Math.cos(rad);
      return { x: X, y: Y };
  }

  /**
   * Converts A (arc-to) segments to C (cubic-bezier-to).
   *
   * For more information of where this math came from visit:
   * http://www.w3.org/TR/SVG11/implnote.html#ArcImplementationNotes
   */
  function arcToCubic(X1, Y1, RX, RY, angle, LAF, SF, X2, Y2, recursive) {
      var x1 = X1;
      var y1 = Y1;
      var rx = RX;
      var ry = RY;
      var x2 = X2;
      var y2 = Y2;
      // for more information of where this Math came from visit:
      // http://www.w3.org/TR/SVG11/implnote.html#ArcImplementationNotes
      var d120 = (Math.PI * 120) / 180;
      var rad = (Math.PI / 180) * (+angle || 0);
      /** @type {number[]} */
      var res = [];
      var xy;
      var f1;
      var f2;
      var cx;
      var cy;
      if (!recursive) {
          xy = rotateVector(x1, y1, -rad);
          x1 = xy.x;
          y1 = xy.y;
          xy = rotateVector(x2, y2, -rad);
          x2 = xy.x;
          y2 = xy.y;
          var x = (x1 - x2) / 2;
          var y = (y1 - y2) / 2;
          var h = (x * x) / (rx * rx) + (y * y) / (ry * ry);
          if (h > 1) {
              h = Math.sqrt(h);
              rx *= h;
              ry *= h;
          }
          var rx2 = rx * rx;
          var ry2 = ry * ry;
          var k = (LAF === SF ? -1 : 1) *
              Math.sqrt(Math.abs((rx2 * ry2 - rx2 * y * y - ry2 * x * x) / (rx2 * y * y + ry2 * x * x)));
          cx = (k * rx * y) / ry + (x1 + x2) / 2;
          cy = (k * -ry * x) / rx + (y1 + y2) / 2;
          // eslint-disable-next-line no-bitwise -- Impossible to satisfy no-bitwise
          f1 = Math.asin(((((y1 - cy) / ry) * Math.pow(10, 9)) >> 0) / Math.pow(10, 9));
          // eslint-disable-next-line no-bitwise -- Impossible to satisfy no-bitwise
          f2 = Math.asin(((((y2 - cy) / ry) * Math.pow(10, 9)) >> 0) / Math.pow(10, 9));
          f1 = x1 < cx ? Math.PI - f1 : f1;
          f2 = x2 < cx ? Math.PI - f2 : f2;
          if (f1 < 0)
              f1 = Math.PI * 2 + f1;
          if (f2 < 0)
              f2 = Math.PI * 2 + f2;
          if (SF && f1 > f2) {
              f1 -= Math.PI * 2;
          }
          if (!SF && f2 > f1) {
              f2 -= Math.PI * 2;
          }
      }
      else {
          f1 = recursive[0], f2 = recursive[1], cx = recursive[2], cy = recursive[3];
      }
      var df = f2 - f1;
      if (Math.abs(df) > d120) {
          var f2old = f2;
          var x2old = x2;
          var y2old = y2;
          f2 = f1 + d120 * (SF && f2 > f1 ? 1 : -1);
          x2 = cx + rx * Math.cos(f2);
          y2 = cy + ry * Math.sin(f2);
          res = arcToCubic(x2, y2, rx, ry, angle, 0, SF, x2old, y2old, [f2, f2old, cx, cy]);
      }
      df = f2 - f1;
      var c1 = Math.cos(f1);
      var s1 = Math.sin(f1);
      var c2 = Math.cos(f2);
      var s2 = Math.sin(f2);
      var t = Math.tan(df / 4);
      var hx = (4 / 3) * rx * t;
      var hy = (4 / 3) * ry * t;
      var m1 = [x1, y1];
      var m2 = [x1 + hx * s1, y1 - hy * c1];
      var m3 = [x2 + hx * s2, y2 - hy * c2];
      var m4 = [x2, y2];
      m2[0] = 2 * m1[0] - m2[0];
      m2[1] = 2 * m1[1] - m2[1];
      if (recursive) {
          return m2.concat(m3, m4, res);
          // return [...m2, ...m3, ...m4, ...res];
      }
      res = m2.concat(m3, m4, res);
      // res = [...m2, ...m3, ...m4, ...res];
      var newres = [];
      for (var i = 0, ii = res.length; i < ii; i += 1) {
          newres[i] = i % 2 ? rotateVector(res[i - 1], res[i], rad).y : rotateVector(res[i], res[i + 1], rad).x;
      }
      return newres;
  }
  // const TAU = Math.PI * 2;
  // const mapToEllipse = (
  //   { x, y }: { x: number; y: number },
  //   rx: number,
  //   ry: number,
  //   cosphi: number,
  //   sinphi: number,
  //   centerx: number,
  //   centery: number,
  // ) => {
  //   x *= rx;
  //   y *= ry;
  //   const xp = cosphi * x - sinphi * y;
  //   const yp = sinphi * x + cosphi * y;
  //   return {
  //     x: xp + centerx,
  //     y: yp + centery,
  //   };
  // };
  // const approxUnitArc = (ang1: number, ang2: number) => {
  //   // If 90 degree circular arc, use a constant
  //   // as derived from http://spencermortensen.com/articles/bezier-circle
  //   const a =
  //     ang2 === 1.5707963267948966
  //       ? 0.551915024494
  //       : ang2 === -1.5707963267948966
  //       ? -0.551915024494
  //       : (4 / 3) * Math.tan(ang2 / 4);
  //   const x1 = Math.cos(ang1);
  //   const y1 = Math.sin(ang1);
  //   const x2 = Math.cos(ang1 + ang2);
  //   const y2 = Math.sin(ang1 + ang2);
  //   return [
  //     {
  //       x: x1 - y1 * a,
  //       y: y1 + x1 * a,
  //     },
  //     {
  //       x: x2 + y2 * a,
  //       y: y2 - x2 * a,
  //     },
  //     {
  //       x: x2,
  //       y: y2,
  //     },
  //   ];
  // };
  // const vectorAngle = (ux: number, uy: number, vx: number, vy: number) => {
  //   const sign = ux * vy - uy * vx < 0 ? -1 : 1;
  //   let dot = ux * vx + uy * vy;
  //   if (dot > 1) {
  //     dot = 1;
  //   }
  //   if (dot < -1) {
  //     dot = -1;
  //   }
  //   return sign * Math.acos(dot);
  // };
  // const getArcCenter = (
  //   px: any,
  //   py: any,
  //   cx: any,
  //   cy: any,
  //   rx: number,
  //   ry: number,
  //   largeArcFlag: number,
  //   sweepFlag: number,
  //   sinphi: number,
  //   cosphi: number,
  //   pxp: number,
  //   pyp: number,
  // ) => {
  //   const rxsq = Math.pow(rx, 2);
  //   const rysq = Math.pow(ry, 2);
  //   const pxpsq = Math.pow(pxp, 2);
  //   const pypsq = Math.pow(pyp, 2);
  //   let radicant = rxsq * rysq - rxsq * pypsq - rysq * pxpsq;
  //   if (radicant < 0) {
  //     radicant = 0;
  //   }
  //   radicant /= rxsq * pypsq + rysq * pxpsq;
  //   radicant = Math.sqrt(radicant) * (largeArcFlag === sweepFlag ? -1 : 1);
  //   const centerxp = ((radicant * rx) / ry) * pyp;
  //   const centeryp = ((radicant * -ry) / rx) * pxp;
  //   const centerx = cosphi * centerxp - sinphi * centeryp + (px + cx) / 2;
  //   const centery = sinphi * centerxp + cosphi * centeryp + (py + cy) / 2;
  //   const vx1 = (pxp - centerxp) / rx;
  //   const vy1 = (pyp - centeryp) / ry;
  //   const vx2 = (-pxp - centerxp) / rx;
  //   const vy2 = (-pyp - centeryp) / ry;
  //   const ang1 = vectorAngle(1, 0, vx1, vy1);
  //   let ang2 = vectorAngle(vx1, vy1, vx2, vy2);
  //   if (sweepFlag === 0 && ang2 > 0) {
  //     ang2 -= TAU;
  //   }
  //   if (sweepFlag === 1 && ang2 < 0) {
  //     ang2 += TAU;
  //   }
  //   return [centerx, centery, ang1, ang2];
  // };
  // const arcToBezier = ({ px, py, cx, cy, rx, ry, xAxisRotation = 0, largeArcFlag = 0, sweepFlag = 0 }) => {
  //   const curves = [];
  //   if (rx === 0 || ry === 0) {
  //     return [{ x1: 0, y1: 0, x2: 0, y2: 0, x: cx, y: cy }];
  //   }
  //   const sinphi = Math.sin((xAxisRotation * TAU) / 360);
  //   const cosphi = Math.cos((xAxisRotation * TAU) / 360);
  //   const pxp = (cosphi * (px - cx)) / 2 + (sinphi * (py - cy)) / 2;
  //   const pyp = (-sinphi * (px - cx)) / 2 + (cosphi * (py - cy)) / 2;
  //   if (pxp === 0 && pyp === 0) {
  //     return [{ x1: 0, y1: 0, x2: 0, y2: 0, x: cx, y: cy }];
  //   }
  //   rx = Math.abs(rx);
  //   ry = Math.abs(ry);
  //   const lambda = Math.pow(pxp, 2) / Math.pow(rx, 2) + Math.pow(pyp, 2) / Math.pow(ry, 2);
  //   if (lambda > 1) {
  //     rx *= Math.sqrt(lambda);
  //     ry *= Math.sqrt(lambda);
  //   }
  //   let [centerx, centery, ang1, ang2] = getArcCenter(
  //     px,
  //     py,
  //     cx,
  //     cy,
  //     rx,
  //     ry,
  //     largeArcFlag,
  //     sweepFlag,
  //     sinphi,
  //     cosphi,
  //     pxp,
  //     pyp,
  //   );
  //   // If 'ang2' == 90.0000000001, then `ratio` will evaluate to
  //   // 1.0000000001. This causes `segments` to be greater than one, which is an
  //   // unecessary split, and adds extra points to the bezier curve. To alleviate
  //   // this issue, we round to 1.0 when the ratio is close to 1.0.
  //   let ratio = Math.abs(ang2) / (TAU / 4);
  //   if (Math.abs(1.0 - ratio) < 0.0000001) {
  //     ratio = 1.0;
  //   }
  //   const segments = Math.max(Math.ceil(ratio), 1);
  //   ang2 /= segments;
  //   for (let i = 0; i < segments; i++) {
  //     curves.push(approxUnitArc(ang1, ang2));
  //     ang1 += ang2;
  //   }
  //   return curves.map((curve) => {
  //     const { x: x1, y: y1 } = mapToEllipse(curve[0], rx, ry, cosphi, sinphi, centerx, centery);
  //     const { x: x2, y: y2 } = mapToEllipse(curve[1], rx, ry, cosphi, sinphi, centerx, centery);
  //     const { x, y } = mapToEllipse(curve[2], rx, ry, cosphi, sinphi, centerx, centery);
  //     return { x1, y1, x2, y2, x, y };
  //   });
  // };
  // export function arcToCubic(
  //   x1: number,
  //   y1: number,
  //   rx: number,
  //   ry: number,
  //   angle: number,
  //   LAF: number,
  //   SF: number,
  //   x2: number,
  //   y2: number,
  // ) {
  //   const curves = arcToBezier({
  //     px: x1,
  //     py: y1,
  //     cx: x2,
  //     cy: y2,
  //     rx,
  //     ry,
  //     xAxisRotation: angle,
  //     largeArcFlag: LAF,
  //     sweepFlag: SF,
  //   });
  //   return curves.reduce((prev, cur) => {
  //     const { x1, y1, x2, y2, x, y } = cur;
  //     prev.push(x1, y1, x2, y2, x, y);
  //     return prev;
  //   }, [] as number[]);
  // }

  function quadToCubic(x1, y1, qx, qy, x2, y2) {
      var r13 = 1 / 3;
      var r23 = 2 / 3;
      return [
          r13 * x1 + r23 * qx,
          r13 * y1 + r23 * qy,
          r13 * x2 + r23 * qx,
          r13 * y2 + r23 * qy,
          x2,
          y2, // x,y
      ];
  }

  function midPoint(a, b, t) {
      var ax = a[0];
      var ay = a[1];
      var bx = b[0];
      var by = b[1];
      return [ax + (bx - ax) * t, ay + (by - ay) * t];
  }

  function distanceSquareRoot(a, b) {
      return Math.sqrt((a[0] - b[0]) * (a[0] - b[0]) + (a[1] - b[1]) * (a[1] - b[1]));
  }

  /**
   * Returns a {x,y} point at a given length, the total length and
   * the minimum and maximum {x,y} coordinates of a line (L,V,H,Z) segment.
   */
  function segmentLineFactory(x1, y1, x2, y2, distance) {
      var length = distanceSquareRoot([x1, y1], [x2, y2]);
      var point = { x: 0, y: 0 };
      if (typeof distance === 'number') {
          if (distance <= 0) {
              point = { x: x1, y: y1 };
          }
          else if (distance >= length) {
              point = { x: x2, y: y2 };
          }
          else {
              var _a = midPoint([x1, y1], [x2, y2], distance / length), x = _a[0], y = _a[1];
              point = { x: x, y: y };
          }
      }
      return {
          length: length,
          point: point,
          min: {
              x: Math.min(x1, x2),
              y: Math.min(y1, y2),
          },
          max: {
              x: Math.max(x1, x2),
              y: Math.max(y1, y2),
          },
      };
  }

  function lineToCubic(x1, y1, x2, y2) {
      var t = 0.5;
      var p0 = [x1, y1];
      var p1 = [x2, y2];
      var p2 = midPoint(p0, p1, t);
      var p3 = midPoint(p1, p2, t);
      var p4 = midPoint(p2, p3, t);
      var p5 = midPoint(p3, p4, t);
      var p6 = midPoint(p4, p5, t);
      // const seg1 = [...p0, ...p2, ...p4, ...p6, t];
      // @ts-ignore
      var cp1 = segmentLineFactory(p0[0], p0[1], p2[0], p2[1], p4[0]).point;
      // const seg2 = [...p6, ...p5, ...p3, ...p1, 0];
      // @ts-ignore
      var cp2 = segmentLineFactory(p6[0], p6[1], p5[0], p5[1], p3[0]).point;
      return [cp1.x, cp1.y, cp2.x, cp2.y, x2, y2];
  }

  function segmentToCubic(segment, params) {
      var pathCommand = segment[0];
      var values = segment.slice(1).map(Number);
      var x = values[0], y = values[1];
      var args;
      var px1 = params.x1, py1 = params.y1, px = params.x, py = params.y;
      if (!'TQ'.includes(pathCommand)) {
          params.qx = null;
          params.qy = null;
      }
      switch (pathCommand) {
          case 'M':
              params.x = x;
              params.y = y;
              return segment;
          case 'A':
              args = [px1, py1].concat(values);
              // @ts-ignore
              return ['C'].concat(arcToCubic(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9]));
          case 'Q':
              params.qx = x;
              params.qy = y;
              args = [px1, py1].concat(values);
              // @ts-ignore
              return ['C'].concat(quadToCubic(args[0], args[1], args[2], args[3], args[4], args[5]));
          case 'L':
              // @ts-ignore
              return ['C'].concat(lineToCubic(px1, py1, x, y));
          case 'Z':
              // prevent NaN from divide 0
              if (px1 === px && py1 === py) {
                  return ['C', px1, py1, px, py, px, py];
              }
              // @ts-ignore
              return ['C'].concat(lineToCubic(px1, py1, px, py));
      }
      return segment;
  }

  // import { fixPath } from '../process/fix-path';
  function path2Curve(pathInput, needZCommandIndexes) {
      if (needZCommandIndexes === void 0) { needZCommandIndexes = false; }
      if (isCurveArray(pathInput)) {
          var cloned = clonePath(pathInput);
          if (needZCommandIndexes) {
              return [cloned, []];
          }
          else {
              return cloned;
          }
      }
      // fixPath will remove 'Z' command
      // const path = fixPath(normalizePath(pathInput));
      var path = normalizePath(pathInput);
      var params = __assign({}, paramsParser);
      var allPathCommands = [];
      var pathCommand = '';
      var ii = path.length;
      var segment;
      var seglen;
      var zCommandIndexes = [];
      for (var i = 0; i < ii; i += 1) {
          if (path[i])
              pathCommand = path[i][0];
          allPathCommands[i] = pathCommand;
          var curveSegment = segmentToCubic(path[i], params);
          path[i] = curveSegment;
          fixArc(path, allPathCommands, i);
          ii = path.length; // solves curveArrays ending in Z
          // keep Z command account for lineJoin
          // @see https://github.com/antvis/util/issues/68
          if (pathCommand === 'Z') {
              zCommandIndexes.push(i);
          }
          segment = path[i];
          seglen = segment.length;
          params.x1 = +segment[seglen - 2];
          params.y1 = +segment[seglen - 1];
          params.x2 = +segment[seglen - 4] || params.x1;
          params.y2 = +segment[seglen - 3] || params.y1;
      }
      // validate
      if (needZCommandIndexes) {
          return [path, zCommandIndexes];
      }
      else {
          return path;
      }
  }

  // reverse CURVE based pathArray segments only
  function reverseCurve(pathArray) {
      var rotatedCurve = pathArray
          .slice(1)
          .map(function (x, i, curveOnly) {
          // @ts-ignore
          return !i ? pathArray[0].slice(1).concat(x.slice(1)) : curveOnly[i - 1].slice(-2).concat(x.slice(1));
      })
          // @ts-ignore
          .map(function (x) { return x.map(function (y, i) { return x[x.length - i - 2 * (1 - (i % 2))]; }); })
          .reverse();
      return [['M'].concat(rotatedCurve[0].slice(0, 2))].concat(rotatedCurve.map(function (x) { return ['C'].concat(x.slice(2)); }));
  }

  function angleBetween(v0, v1) {
      var v0x = v0.x, v0y = v0.y;
      var v1x = v1.x, v1y = v1.y;
      var p = v0x * v1x + v0y * v1y;
      var n = Math.sqrt((Math.pow(v0x, 2) + Math.pow(v0y, 2)) * (Math.pow(v1x, 2) + Math.pow(v1y, 2)));
      var sign = v0x * v1y - v0y * v1x < 0 ? -1 : 1;
      var angle = sign * Math.acos(p / n);
      return angle;
  }
  /**
   * Returns a {x,y} point at a given length, the total length and
   * the minimum and maximum {x,y} coordinates of a C (cubic-bezier) segment.
   * @see https://github.com/MadLittleMods/svg-curve-lib/blob/master/src/js/svg-curve-lib.js
   */
  function getPointAtArcSegmentLength(x1, y1, RX, RY, angle, LAF, SF, x, y, t) {
      var abs = Math.abs, sin = Math.sin, cos = Math.cos, sqrt = Math.sqrt, PI = Math.PI;
      var rx = abs(RX);
      var ry = abs(RY);
      var xRot = ((angle % 360) + 360) % 360;
      var xRotRad = xRot * (PI / 180);
      if (x1 === x && y1 === y) {
          return { x: x1, y: y1 };
      }
      if (rx === 0 || ry === 0) {
          return segmentLineFactory(x1, y1, x, y, t).point;
      }
      var dx = (x1 - x) / 2;
      var dy = (y1 - y) / 2;
      var transformedPoint = {
          x: cos(xRotRad) * dx + sin(xRotRad) * dy,
          y: -sin(xRotRad) * dx + cos(xRotRad) * dy,
      };
      var radiiCheck = Math.pow(transformedPoint.x, 2) / Math.pow(rx, 2) + Math.pow(transformedPoint.y, 2) / Math.pow(ry, 2);
      if (radiiCheck > 1) {
          rx *= sqrt(radiiCheck);
          ry *= sqrt(radiiCheck);
      }
      var cSquareNumerator = Math.pow(rx, 2) * Math.pow(ry, 2) - Math.pow(rx, 2) * Math.pow(transformedPoint.y, 2) - Math.pow(ry, 2) * Math.pow(transformedPoint.x, 2);
      var cSquareRootDenom = Math.pow(rx, 2) * Math.pow(transformedPoint.y, 2) + Math.pow(ry, 2) * Math.pow(transformedPoint.x, 2);
      var cRadicand = cSquareNumerator / cSquareRootDenom;
      cRadicand = cRadicand < 0 ? 0 : cRadicand;
      var cCoef = (LAF !== SF ? 1 : -1) * sqrt(cRadicand);
      var transformedCenter = {
          x: cCoef * ((rx * transformedPoint.y) / ry),
          y: cCoef * (-(ry * transformedPoint.x) / rx),
      };
      var center = {
          x: cos(xRotRad) * transformedCenter.x - sin(xRotRad) * transformedCenter.y + (x1 + x) / 2,
          y: sin(xRotRad) * transformedCenter.x + cos(xRotRad) * transformedCenter.y + (y1 + y) / 2,
      };
      var startVector = {
          x: (transformedPoint.x - transformedCenter.x) / rx,
          y: (transformedPoint.y - transformedCenter.y) / ry,
      };
      var startAngle = angleBetween({ x: 1, y: 0 }, startVector);
      var endVector = {
          x: (-transformedPoint.x - transformedCenter.x) / rx,
          y: (-transformedPoint.y - transformedCenter.y) / ry,
      };
      var sweepAngle = angleBetween(startVector, endVector);
      if (!SF && sweepAngle > 0) {
          sweepAngle -= 2 * PI;
      }
      else if (SF && sweepAngle < 0) {
          sweepAngle += 2 * PI;
      }
      sweepAngle %= 2 * PI;
      var alpha = startAngle + sweepAngle * t;
      var ellipseComponentX = rx * cos(alpha);
      var ellipseComponentY = ry * sin(alpha);
      var point = {
          x: cos(xRotRad) * ellipseComponentX - sin(xRotRad) * ellipseComponentY + center.x,
          y: sin(xRotRad) * ellipseComponentX + cos(xRotRad) * ellipseComponentY + center.y,
      };
      // to be used later
      // point.ellipticalArcStartAngle = startAngle;
      // point.ellipticalArcEndAngle = startAngle + sweepAngle;
      // point.ellipticalArcAngle = alpha;
      // point.ellipticalArcCenter = center;
      // point.resultantRx = rx;
      // point.resultantRy = ry;
      return point;
  }
  /**
   * Returns a {x,y} point at a given length, the total length and
   * the shape minimum and maximum {x,y} coordinates of an A (arc-to) segment.
   *
   * For better performance, it can skip calculate bbox or length in some scenario.
   */
  function segmentArcFactory(X1, Y1, RX, RY, angle, LAF, SF, X2, Y2, distance, options) {
      var _a;
      var _b = options.bbox, bbox = _b === void 0 ? true : _b, _c = options.length, length = _c === void 0 ? true : _c, _d = options.sampleSize, sampleSize = _d === void 0 ? 30 : _d;
      var distanceIsNumber = typeof distance === 'number';
      var x = X1;
      var y = Y1;
      var LENGTH = 0;
      var prev = [x, y, LENGTH];
      var cur = [x, y];
      var t = 0;
      var POINT = { x: 0, y: 0 };
      var POINTS = [{ x: x, y: y }];
      if (distanceIsNumber && distance <= 0) {
          POINT = { x: x, y: y };
      }
      // bad perf when size > 100
      for (var j = 0; j <= sampleSize; j += 1) {
          t = j / sampleSize;
          (_a = getPointAtArcSegmentLength(X1, Y1, RX, RY, angle, LAF, SF, X2, Y2, t), x = _a.x, y = _a.y);
          if (bbox) {
              POINTS.push({ x: x, y: y });
          }
          if (length) {
              LENGTH += distanceSquareRoot(cur, [x, y]);
          }
          cur = [x, y];
          if (distanceIsNumber && LENGTH >= distance && distance > prev[2]) {
              var dv = (LENGTH - distance) / (LENGTH - prev[2]);
              POINT = {
                  x: cur[0] * (1 - dv) + prev[0] * dv,
                  y: cur[1] * (1 - dv) + prev[1] * dv,
              };
          }
          prev = [x, y, LENGTH];
      }
      if (distanceIsNumber && distance >= LENGTH) {
          POINT = { x: X2, y: Y2 };
      }
      return {
          length: LENGTH,
          point: POINT,
          min: {
              x: Math.min.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.min.apply(null, POINTS.map(function (n) { return n.y; })),
          },
          max: {
              x: Math.max.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.max.apply(null, POINTS.map(function (n) { return n.y; })),
          },
      };
  }

  /**
   * Returns a {x,y} point at a given length, the total length and
   * the minimum and maximum {x,y} coordinates of a C (cubic-bezier) segment.
   */
  function getPointAtCubicSegmentLength(x1, y1, c1x, c1y, c2x, c2y, x2, y2, t) {
      var t1 = 1 - t;
      return {
          x: Math.pow(t1, 3) * x1 + 3 * Math.pow(t1, 2) * t * c1x + 3 * t1 * Math.pow(t, 2) * c2x + Math.pow(t, 3) * x2,
          y: Math.pow(t1, 3) * y1 + 3 * Math.pow(t1, 2) * t * c1y + 3 * t1 * Math.pow(t, 2) * c2y + Math.pow(t, 3) * y2,
      };
  }
  /**
   * Returns the length of a C (cubic-bezier) segment
   * or an {x,y} point at a given length.
   */
  function segmentCubicFactory(x1, y1, c1x, c1y, c2x, c2y, x2, y2, distance, options) {
      var _a;
      var _b = options.bbox, bbox = _b === void 0 ? true : _b, _c = options.length, length = _c === void 0 ? true : _c, _d = options.sampleSize, sampleSize = _d === void 0 ? 10 : _d;
      var distanceIsNumber = typeof distance === 'number';
      var x = x1;
      var y = y1;
      var LENGTH = 0;
      var prev = [x, y, LENGTH];
      var cur = [x, y];
      var t = 0;
      var POINT = { x: 0, y: 0 };
      var POINTS = [{ x: x, y: y }];
      if (distanceIsNumber && distance <= 0) {
          POINT = { x: x, y: y };
      }
      // bad perf when size = 300
      for (var j = 0; j <= sampleSize; j += 1) {
          t = j / sampleSize;
          (_a = getPointAtCubicSegmentLength(x1, y1, c1x, c1y, c2x, c2y, x2, y2, t), x = _a.x, y = _a.y);
          if (bbox) {
              POINTS.push({ x: x, y: y });
          }
          if (length) {
              LENGTH += distanceSquareRoot(cur, [x, y]);
          }
          cur = [x, y];
          if (distanceIsNumber && LENGTH >= distance && distance > prev[2]) {
              var dv = (LENGTH - distance) / (LENGTH - prev[2]);
              POINT = {
                  x: cur[0] * (1 - dv) + prev[0] * dv,
                  y: cur[1] * (1 - dv) + prev[1] * dv,
              };
          }
          prev = [x, y, LENGTH];
      }
      if (distanceIsNumber && distance >= LENGTH) {
          POINT = { x: x2, y: y2 };
      }
      return {
          length: LENGTH,
          point: POINT,
          min: {
              x: Math.min.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.min.apply(null, POINTS.map(function (n) { return n.y; })),
          },
          max: {
              x: Math.max.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.max.apply(null, POINTS.map(function (n) { return n.y; })),
          },
      };
  }

  /**
   * Returns the {x,y} coordinates of a point at a
   * given length of a quadratic-bezier segment.
   *
   * @see https://github.com/substack/point-at-length
   */
  function getPointAtQuadSegmentLength(x1, y1, cx, cy, x2, y2, t) {
      var t1 = 1 - t;
      return {
          x: Math.pow(t1, 2) * x1 + 2 * t1 * t * cx + Math.pow(t, 2) * x2,
          y: Math.pow(t1, 2) * y1 + 2 * t1 * t * cy + Math.pow(t, 2) * y2,
      };
  }
  /**
   * Returns a {x,y} point at a given length, the total length and
   * the minimum and maximum {x,y} coordinates of a Q (quadratic-bezier) segment.
   */
  function segmentQuadFactory(x1, y1, qx, qy, x2, y2, distance, options) {
      var _a;
      var _b = options.bbox, bbox = _b === void 0 ? true : _b, _c = options.length, length = _c === void 0 ? true : _c, _d = options.sampleSize, sampleSize = _d === void 0 ? 10 : _d;
      var distanceIsNumber = typeof distance === 'number';
      var x = x1;
      var y = y1;
      var LENGTH = 0;
      var prev = [x, y, LENGTH];
      var cur = [x, y];
      var t = 0;
      var POINT = { x: 0, y: 0 };
      var POINTS = [{ x: x, y: y }];
      if (distanceIsNumber && distance <= 0) {
          POINT = { x: x, y: y };
      }
      for (var j = 0; j <= sampleSize; j += 1) {
          t = j / sampleSize;
          (_a = getPointAtQuadSegmentLength(x1, y1, qx, qy, x2, y2, t), x = _a.x, y = _a.y);
          if (bbox) {
              POINTS.push({ x: x, y: y });
          }
          if (length) {
              LENGTH += distanceSquareRoot(cur, [x, y]);
          }
          cur = [x, y];
          if (distanceIsNumber && LENGTH >= distance && distance > prev[2]) {
              var dv = (LENGTH - distance) / (LENGTH - prev[2]);
              POINT = {
                  x: cur[0] * (1 - dv) + prev[0] * dv,
                  y: cur[1] * (1 - dv) + prev[1] * dv,
              };
          }
          prev = [x, y, LENGTH];
      }
      /* istanbul ignore else */
      if (distanceIsNumber && distance >= LENGTH) {
          POINT = { x: x2, y: y2 };
      }
      return {
          length: LENGTH,
          point: POINT,
          min: {
              x: Math.min.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.min.apply(null, POINTS.map(function (n) { return n.y; })),
          },
          max: {
              x: Math.max.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.max.apply(null, POINTS.map(function (n) { return n.y; })),
          },
      };
  }

  /**
   * Returns a {x,y} point at a given length
   * of a shape, the shape total length and
   * the shape minimum and maximum {x,y} coordinates.
   */
  function pathLengthFactory(pathInput, distance, options) {
      var _a, _b, _c, _d, _e, _f;
      var path = normalizePath(pathInput);
      var distanceIsNumber = typeof distance === 'number';
      var isM;
      var data = [];
      var pathCommand;
      var x = 0;
      var y = 0;
      var mx = 0;
      var my = 0;
      var seg;
      var MIN = [];
      var MAX = [];
      var length = 0;
      var min = { x: 0, y: 0 };
      var max = min;
      var point = min;
      var POINT = min;
      var LENGTH = 0;
      for (var i = 0, ll = path.length; i < ll; i += 1) {
          seg = path[i];
          pathCommand = seg[0];
          isM = pathCommand === 'M';
          data = !isM ? [x, y].concat(seg.slice(1)) : data;
          // this segment is always ZERO
          /* istanbul ignore else */
          if (isM) {
              // remember mx, my for Z
              mx = seg[1], my = seg[2];
              min = { x: mx, y: my };
              max = min;
              length = 0;
              if (distanceIsNumber && distance < 0.001) {
                  POINT = min;
              }
          }
          else if (pathCommand === 'L') {
              (_a = segmentLineFactory(data[0], data[1], data[2], data[3], (distance || 0) - LENGTH), length = _a.length, min = _a.min, max = _a.max, point = _a.point);
          }
          else if (pathCommand === 'A') {
              (_b = segmentArcFactory(data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7], data[8], (distance || 0) - LENGTH, options || {}), length = _b.length, min = _b.min, max = _b.max, point = _b.point);
          }
          else if (pathCommand === 'C') {
              (_c = segmentCubicFactory(data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7], (distance || 0) - LENGTH, options || {}), length = _c.length, min = _c.min, max = _c.max, point = _c.point);
          }
          else if (pathCommand === 'Q') {
              (_d = segmentQuadFactory(data[0], data[1], data[2], data[3], data[4], data[5], (distance || 0) - LENGTH, options || {}), length = _d.length, min = _d.min, max = _d.max, point = _d.point);
          }
          else if (pathCommand === 'Z') {
              data = [x, y, mx, my];
              (_e = segmentLineFactory(data[0], data[1], data[2], data[3], (distance || 0) - LENGTH), length = _e.length, min = _e.min, max = _e.max, point = _e.point);
          }
          if (distanceIsNumber && LENGTH < distance && LENGTH + length >= distance) {
              POINT = point;
          }
          MAX.push(max);
          MIN.push(min);
          LENGTH += length;
          _f = pathCommand !== 'Z' ? seg.slice(-2) : [mx, my], x = _f[0], y = _f[1];
      }
      // native `getPointAtLength` behavior when the given distance
      // is higher than total length
      if (distanceIsNumber && distance >= LENGTH) {
          POINT = { x: x, y: y };
      }
      return {
          length: LENGTH,
          point: POINT,
          min: {
              x: Math.min.apply(null, MIN.map(function (n) { return n.x; })),
              y: Math.min.apply(null, MIN.map(function (n) { return n.y; })),
          },
          max: {
              x: Math.max.apply(null, MAX.map(function (n) { return n.x; })),
              y: Math.max.apply(null, MAX.map(function (n) { return n.y; })),
          },
      };
  }

  /**
   * Returns the shape total length, or the equivalent to `shape.getTotalLength()`.
   *
   * The `normalizePath` version is lighter, faster, more efficient and more accurate
   * with paths that are not `curveArray`.
   */
  function getTotalLength(pathInput, options) {
      return pathLengthFactory(pathInput, undefined, __assign(__assign({}, options), { bbox: false, length: true })).length;
  }

  function getRotations(a) {
      var segCount = a.length;
      var pointCount = segCount - 1;
      return a.map(function (f, idx) {
          return a.map(function (p, i) {
              var oldSegIdx = idx + i;
              var seg;
              if (i === 0 || (a[oldSegIdx] && a[oldSegIdx][0] === 'M')) {
                  seg = a[oldSegIdx];
                  return ['M'].concat(seg.slice(-2));
              }
              if (oldSegIdx >= segCount)
                  oldSegIdx -= pointCount;
              return a[oldSegIdx];
          });
      });
  }
  function getRotatedCurve(a, b) {
      var segCount = a.length - 1;
      var lineLengths = [];
      var computedIndex = 0;
      var sumLensSqrd = 0;
      var rotations = getRotations(a);
      rotations.forEach(function (r, i) {
          a.slice(1).forEach(function (s, j) {
              // @ts-ignore
              sumLensSqrd += distanceSquareRoot(a[(i + j) % segCount].slice(-2), b[j % segCount].slice(-2));
          });
          lineLengths[i] = sumLensSqrd;
          sumLensSqrd = 0;
      });
      computedIndex = lineLengths.indexOf(Math.min.apply(null, lineLengths));
      return rotations[computedIndex];
  }

  /**
   * Returns the area of a single cubic-bezier segment.
   *
   * http://objectmix.com/graphics/133553-area-closed-bezier-curve.html
   */
  function getCubicSegArea(x1, y1, c1x, c1y, c2x, c2y, x2, y2) {
      // https://stackoverflow.com/a/15845996
      return ((3 *
          ((y2 - y1) * (c1x + c2x) -
              (x2 - x1) * (c1y + c2y) +
              c1y * (x1 - c2x) -
              c1x * (y1 - c2y) +
              y2 * (c2x + x1 / 3) -
              x2 * (c2y + y1 / 3))) /
          20);
  }
  /**
   * Returns the area of a shape.
   * @author Jürg Lehni & Jonathan Puckey
   *
   * @see https://github.com/paperjs/paper.js/blob/develop/src/path/Path.js
   */
  function getPathArea(path) {
      var x = 0;
      var y = 0;
      var len = 0;
      return path2Curve(path)
          .map(function (seg) {
          var _a;
          switch (seg[0]) {
              case 'M':
                  x = seg[1], y = seg[2];
                  return 0;
              default:
                  // @ts-ignore
                  var _b = seg.slice(1), c1x = _b[0], c1y = _b[1], c2x = _b[2], c2y = _b[3], x2 = _b[4], y2 = _b[5];
                  len = getCubicSegArea(x, y, c1x, c1y, c2x, c2y, x2, y2);
                  _a = seg.slice(-2), x = _a[0], y = _a[1];
                  return len;
          }
      })
          .reduce(function (a, b) { return a + b; }, 0);
  }
  // export function getPathArea(pathArray: AbsoluteArray) {
  //   let x = 0;
  //   let y = 0;
  //   let mx = 0;
  //   let my = 0;
  //   let len = 0;
  //   return pathArray
  //     .map((seg) => {
  //       switch (seg[0]) {
  //         case 'M':
  //         case 'Z':
  //           mx = seg[0] === 'M' ? seg[1] : mx;
  //           my = seg[0] === 'M' ? seg[2] : my;
  //           x = mx;
  //           y = my;
  //           return 0;
  //         default:
  //           // @ts-ignore
  //           len = getCubicSegArea.apply(0, [x, y].concat(seg.slice(1)));
  //           [x, y] = seg.slice(-2) as [number, number];
  //           return len;
  //       }
  //     })
  //     .reduce((a, b) => a + b, 0);
  // }

  function getDrawDirection(pathArray) {
      return getPathArea(pathArray) >= 0;
  }

  /**
   * Returns [x,y] coordinates of a point at a given length of a shape.
   */
  function getPointAtLength(pathInput, distance, options) {
      return pathLengthFactory(pathInput, distance, __assign(__assign({}, options), { bbox: false, length: true })).point;
  }

  function splitCubic(pts, t) {
      if (t === void 0) { t = 0.5; }
      var p0 = pts.slice(0, 2);
      var p1 = pts.slice(2, 4);
      var p2 = pts.slice(4, 6);
      var p3 = pts.slice(6, 8);
      var p4 = midPoint(p0, p1, t);
      var p5 = midPoint(p1, p2, t);
      var p6 = midPoint(p2, p3, t);
      var p7 = midPoint(p4, p5, t);
      var p8 = midPoint(p5, p6, t);
      var p9 = midPoint(p7, p8, t);
      return [
          // @ts-ignore
          ['C'].concat(p4, p7, p9),
          // @ts-ignore
          ['C'].concat(p8, p6, p3),
      ];
  }
  function getCurveArray(segments) {
      return segments.map(function (segment, i, pathArray) {
          // @ts-ignore
          var segmentData = i && pathArray[i - 1].slice(-2).concat(segment.slice(1));
          // @ts-ignore
          var curveLength = i
              ? segmentCubicFactory(segmentData[0], segmentData[1], segmentData[2], segmentData[3], segmentData[4], segmentData[5], segmentData[6], segmentData[7], segmentData[8], { bbox: false }).length
              : 0;
          var subsegs;
          if (i) {
              // must be [segment,segment]
              subsegs = curveLength ? splitCubic(segmentData) : [segment, segment];
          }
          else {
              subsegs = [segment];
          }
          return {
              s: segment,
              ss: subsegs,
              l: curveLength,
          };
      });
  }
  function equalizeSegments(path1, path2, TL) {
      var c1 = getCurveArray(path1);
      var c2 = getCurveArray(path2);
      var L1 = c1.length;
      var L2 = c2.length;
      var l1 = c1.filter(function (x) { return x.l; }).length;
      var l2 = c2.filter(function (x) { return x.l; }).length;
      var m1 = c1.filter(function (x) { return x.l; }).reduce(function (a, _a) {
          var l = _a.l;
          return a + l;
      }, 0) / l1 || 0;
      var m2 = c2.filter(function (x) { return x.l; }).reduce(function (a, _a) {
          var l = _a.l;
          return a + l;
      }, 0) / l2 || 0;
      var tl = TL || Math.max(L1, L2);
      var mm = [m1, m2];
      var dif = [tl - L1, tl - L2];
      var canSplit = 0;
      var result = [c1, c2].map(function (x, i) {
          // @ts-ignore
          return x.l === tl
              ? x.map(function (y) { return y.s; })
              : x
                  .map(function (y, j) {
                  canSplit = j && dif[i] && y.l >= mm[i];
                  dif[i] -= canSplit ? 1 : 0;
                  return canSplit ? y.ss : [y.s];
              })
                  .flat();
      });
      return result[0].length === result[1].length ? result : equalizeSegments(result[0], result[1], tl);
  }

  /**
   * @see https://github.com/you-dont-need/You-Dont-Need-Lodash-Underscore#_isfunction
   */
  var isFunction = (function (value) {
      return typeof value === 'function';
  });

  // isFinite,
  var isNil = function (value) {
      /**
       * isNil(null) => true
       * isNil() => true
       */
      return value === null || value === undefined;
  };

  var toString = {}.toString;
  var isType = function (value, type) { return toString.call(value) === '[object ' + type + ']'; };

  var isArray = (function (value) {
      return Array.isArray ? Array.isArray(value) : isType(value, 'Array');
  });

  var isObject = (function (value) {
      /**
       * isObject({}) => true
       * isObject([1, 2, 3]) => true
       * isObject(Function) => true
       * isObject(null) => false
       */
      var type = typeof value;
      return (value !== null && type === 'object') || type === 'function';
  });

  function each(elements, func) {
      if (!elements) {
          return;
      }
      var rst;
      if (isArray(elements)) {
          for (var i = 0, len = elements.length; i < len; i++) {
              rst = func(elements[i], i);
              if (rst === false) {
                  break;
              }
          }
      }
      else if (isObject(elements)) {
          for (var k in elements) {
              if (elements.hasOwnProperty(k)) {
                  rst = func(elements[k], k);
                  if (rst === false) {
                      break;
                  }
              }
          }
      }
  }

  var isObjectLike = function (value) {
      /**
       * isObjectLike({}) => true
       * isObjectLike([1, 2, 3]) => true
       * isObjectLike(Function) => false
       * isObjectLike(null) => false
       */
      return typeof value === 'object' && value !== null;
  };

  var isPlainObject = function (value) {
      /**
       * isObjectLike(new Foo) => false
       * isObjectLike([1, 2, 3]) => false
       * isObjectLike({ x: 0, y: 0 }) => true
       * isObjectLike(Object.create(null)) => true
       */
      if (!isObjectLike(value) || !isType(value, 'Object')) {
          return false;
      }
      if (Object.getPrototypeOf(value) === null) {
          return true;
      }
      var proto = value;
      while (Object.getPrototypeOf(proto) !== null) {
          proto = Object.getPrototypeOf(proto);
      }
      return Object.getPrototypeOf(value) === proto;
  };

  /**
   * @param {Array} arr The array to iterate over.
   * @return {*} Returns the maximum value.
   * @example
   *
   * max([1, 2]);
   * // => 2
   *
   * max([]);
   * // => undefined
   *
   * const data = new Array(1250010).fill(1).map((d,idx) => idx);
   *
   * max(data);
   * // => 1250010
   * // Math.max(...data) will encounter "Maximum call stack size exceeded" error
   */
  var max$2 = (function (arr) {
      if (!isArray(arr)) {
          return undefined;
      }
      return arr.reduce(function (prev, curr) {
          return Math.max(prev, curr);
      }, arr[0]);
  });

  /**
   * @param {Array} arr The array to iterate over.
   * @return {*} Returns the minimum value.
   * @example
   *
   * min([1, 2]);
   * // => 1
   *
   * min([]);
   * // => undefined
   *
   * const data = new Array(1250010).fill(1).map((d,idx) => idx);
   *
   * min(data);
   * // => 1250010
   * // Math.min(...data) will encounter "Maximum call stack size exceeded" error
   */
  var min$2 = (function (arr) {
      if (!isArray(arr)) {
          return undefined;
      }
      return arr.reduce(function (prev, curr) {
          return Math.min(prev, curr);
      }, arr[0]);
  });

  var reduce = function (arr, fn, init) {
      if (!isArray(arr) && !isPlainObject(arr)) {
          return arr;
      }
      var result = init;
      each(arr, function (data, i) {
          result = fn(result, data, i);
      });
      return result;
  };

  var isString = (function (str) {
      return isType(str, 'String');
  });

  var clamp = function (a, min, max) {
      if (a < min) {
          return min;
      }
      else if (a > max) {
          return max;
      }
      return a;
  };

  /**
   * 判断是否数字
   * @return {Boolean} 是否数字
   */
  var isNumber = function (value) {
      return isType(value, 'Number');
  };

  var PRECISION = 0.00001; // numbers less than this is considered as 0
  function isNumberEqual(a, b, precision) {
      if (precision === void 0) { precision = PRECISION; }
      return Math.abs(a - b) < precision;
  }

  var mod = function (n, m) {
      return ((n % m) + m) % m;
  };

  /**
   * 是否是布尔类型
   *
   * @param {Object} value 测试的值
   * @return {Boolean}
   */
  var isBoolean = function (value) {
      return isType(value, 'Boolean');
  };

  var isUndefined = function (value) {
      return value === undefined;
  };

  /**
   * _.memoize(calColor);
   * _.memoize(calColor, (...args) => args[0]);
   * @param f
   * @param resolver
   */
  var memoize = (function (f, resolver) {
      if (!isFunction(f)) {
          throw new TypeError('Expected a function');
      }
      var memoized = function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          // 使用方法构造 key，如果不存在 resolver，则直接取第一个参数作为 key
          var key = resolver ? resolver.apply(this, args) : args[0];
          var cache = memoized.cache;
          if (cache.has(key)) {
              return cache.get(key);
          }
          var result = f.apply(this, args);
          // 缓存起来
          cache.set(key, result);
          return result;
      };
      memoized.cache = new Map();
      return memoized;
  });

  var hasOwnProperty = Object.prototype.hasOwnProperty;
  var pick = (function (object, keys) {
      if (object === null || !isPlainObject(object)) {
          return {};
      }
      var result = {};
      each(keys, function (key) {
          if (hasOwnProperty.call(object, key)) {
              result[key] = object[key];
          }
      });
      return result;
  });

  var omit = (function (obj, keys) {
      return reduce(obj, function (r, curr, key) {
          if (!keys.includes(key)) {
              r[key] = curr;
          }
          return r;
      }, {});
  });

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }
  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    Object.defineProperty(Constructor, "prototype", {
      writable: false
    });
    return Constructor;
  }

  var Component = /*#__PURE__*/function () {
    function Component(props, context, updater) {
      _classCallCheck(this, Component);
      // State 内部私有属性
      this.destroyed = false;
      this.props = props;
      this.state = {};
      this.context = context;
      this.updater = updater;
    }
    _createClass(Component, [{
      key: "willMount",
      value: function willMount() {}
    }, {
      key: "didMount",
      value: function didMount() {}
    }, {
      key: "shouldUpdate",
      value: function shouldUpdate(_nextProps) {
        return true;
      }
    }, {
      key: "willReceiveProps",
      value: function willReceiveProps(_props, _context) {}
    }, {
      key: "willUpdate",
      value: function willUpdate() {}
    }, {
      key: "didUpdate",
      value: function didUpdate() {}
    }, {
      key: "render",
      value: function render() {
        return null;
      }
    }, {
      key: "willUnmount",
      value: function willUnmount() {}
    }, {
      key: "didUnmount",
      value: function didUnmount() {}
    }, {
      key: "setState",
      value: function setState(partialState, callback) {
        if (this.destroyed) {
          return;
        }
        this.updater.enqueueSetState(this, partialState, callback);
      }
    }, {
      key: "forceUpdate",
      value: function forceUpdate(callback) {
        if (this.destroyed) {
          return;
        }
        this.updater.enqueueForceUpdate(this, {}, callback);
      }
    }, {
      key: "setAnimate",
      value: function setAnimate(animate) {
        this.animate = animate;
      }
    }, {
      key: "destroy",
      value: function destroy() {
        this.destroyed = true;
        this._vNode = null;
        this.animator = null;
      }
    }]);
    return Component;
  }(); // 标识是否是组件
  // @ts-ignore
  Component.prototype.isF2Component = true;

  function cloneElement(element, props) {
    if (!element) return element;
    return _objectSpread2(_objectSpread2({}, element), {}, {
      props: _objectSpread2(_objectSpread2({}, element.props), props)
    });
  }
  function map(children, fn) {
    if (!children) {
      return fn(children);
    }
    if (isArray(children)) {
      return children.map(function (child) {
        return map(child, fn);
      });
    }
    return fn(children);
  }
  function compareArray(nextElements, lastElements, callback) {
    var keyed = {};
    var nextLength = nextElements.length;
    var lastLength = lastElements.length;
    for (var i = 0, len = lastLength; i < len; i++) {
      var element = lastElements[i];
      if (element && !isNil(element.key)) {
        var key = element.key;
        keyed[key] = element;
      }
    }
    var result = [];
    // 比较元素
    for (var _i = 0, _len = nextLength; _i < _len; _i++) {
      var _lastElement;
      var _element = nextElements[_i];
      if (!_element) {
        continue;
      }
      var _key = _element.key;
      var lastElement = void 0;
      // 有key值定义
      if (!isNil(_element.key)) {
        lastElement = keyed[_key];
        if (lastElement) delete keyed[_key];
      } else {
        // 取相同位置的元素
        lastElement = lastElements[_i];
      }
      // 没有直接返回
      if (!lastElement) {
        result.push(compare(_element, null, callback));
        continue;
      }
      // 如果 lastElement 已经被处理过, next 处理成新增
      if ((_lastElement = lastElement) === null || _lastElement === void 0 ? void 0 : _lastElement.__processed) {
        result.push(compare(_element, null, callback));
        continue;
      }
      // 标记 element 已经被处理过
      lastElement.__processed = true;
      result.push(compare(_element, lastElement, callback));
    }
    // 处理 lastElements 里面还未被处理的元素
    for (var _i2 = 0, _len2 = lastLength; _i2 < _len2; _i2++) {
      var _lastElement2 = lastElements[_i2];
      if (!_lastElement2) {
        continue;
      }
      if (!(_lastElement2 === null || _lastElement2 === void 0 ? void 0 : _lastElement2.__processed)) {
        result.push(compare(null, _lastElement2, callback));
      } else {
        delete _lastElement2.__processed;
      }
    }
    return result;
  }
  // 比较2棵树
  function compare(nextElement, lastElement, callback) {
    // 有一个为空
    if (!nextElement || !lastElement) {
      return callback(nextElement, lastElement);
    }
    if (isArray(nextElement) || isArray(lastElement)) {
      var nextElementArray = isArray(nextElement) ? nextElement : [nextElement];
      var lastElementArray = isArray(lastElement) ? lastElement : [lastElement];
      return compareArray(nextElementArray, lastElementArray, callback);
    }
    return callback(nextElement, lastElement);
  }
  function toArray(element) {
    if (!element) {
      return element;
    }
    if (!isArray(element)) {
      return [element];
    }
    var newArray = [];
    for (var i = 0, len = element.length; i < len; i++) {
      var item = element[i];
      if (isArray(item)) {
        // @ts-ignore
        newArray = newArray.concat(toArray(item));
      } else {
        newArray.push(item);
      }
    }
    return newArray;
  }
  var Children = {
    cloneElement: cloneElement,
    map: map,
    toArray: toArray,
    compare: compare
  };

  function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
  }

  function _iterableToArrayLimit(arr, i) {
    var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];
    if (_i == null) return;
    var _arr = [];
    var _n = true;
    var _d = false;
    var _s, _e;
    try {
      for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);
        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"] != null) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }
    return _arr;
  }

  function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for (var i = 0, arr2 = new Array(len); i < len; i++) {
      arr2[i] = arr[i];
    }
    return arr2;
  }

  function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
  }

  function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
  }

  function quickselect(arr, k, left, right, compare) {
      quickselectStep(arr, k, left || 0, right || (arr.length - 1), compare || defaultCompare);
  }

  function quickselectStep(arr, k, left, right, compare) {

      while (right > left) {
          if (right - left > 600) {
              var n = right - left + 1;
              var m = k - left + 1;
              var z = Math.log(n);
              var s = 0.5 * Math.exp(2 * z / 3);
              var sd = 0.5 * Math.sqrt(z * s * (n - s) / n) * (m - n / 2 < 0 ? -1 : 1);
              var newLeft = Math.max(left, Math.floor(k - m * s / n + sd));
              var newRight = Math.min(right, Math.floor(k + (n - m) * s / n + sd));
              quickselectStep(arr, k, newLeft, newRight, compare);
          }

          var t = arr[k];
          var i = left;
          var j = right;

          swap(arr, left, k);
          if (compare(arr[right], t) > 0) swap(arr, left, right);

          while (i < j) {
              swap(arr, i, j);
              i++;
              j--;
              while (compare(arr[i], t) < 0) i++;
              while (compare(arr[j], t) > 0) j--;
          }

          if (compare(arr[left], t) === 0) swap(arr, left, j);
          else {
              j++;
              swap(arr, j, right);
          }

          if (j <= k) left = j + 1;
          if (k <= j) right = j - 1;
      }
  }

  function swap(arr, i, j) {
      var tmp = arr[i];
      arr[i] = arr[j];
      arr[j] = tmp;
  }

  function defaultCompare(a, b) {
      return a < b ? -1 : a > b ? 1 : 0;
  }

  class RBush {
      constructor(maxEntries = 9) {
          // max entries in a node is 9 by default; min node fill is 40% for best performance
          this._maxEntries = Math.max(4, maxEntries);
          this._minEntries = Math.max(2, Math.ceil(this._maxEntries * 0.4));
          this.clear();
      }

      all() {
          return this._all(this.data, []);
      }

      search(bbox) {
          let node = this.data;
          const result = [];

          if (!intersects(bbox, node)) return result;

          const toBBox = this.toBBox;
          const nodesToSearch = [];

          while (node) {
              for (let i = 0; i < node.children.length; i++) {
                  const child = node.children[i];
                  const childBBox = node.leaf ? toBBox(child) : child;

                  if (intersects(bbox, childBBox)) {
                      if (node.leaf) result.push(child);
                      else if (contains(bbox, childBBox)) this._all(child, result);
                      else nodesToSearch.push(child);
                  }
              }
              node = nodesToSearch.pop();
          }

          return result;
      }

      collides(bbox) {
          let node = this.data;

          if (!intersects(bbox, node)) return false;

          const nodesToSearch = [];
          while (node) {
              for (let i = 0; i < node.children.length; i++) {
                  const child = node.children[i];
                  const childBBox = node.leaf ? this.toBBox(child) : child;

                  if (intersects(bbox, childBBox)) {
                      if (node.leaf || contains(bbox, childBBox)) return true;
                      nodesToSearch.push(child);
                  }
              }
              node = nodesToSearch.pop();
          }

          return false;
      }

      load(data) {
          if (!(data && data.length)) return this;

          if (data.length < this._minEntries) {
              for (let i = 0; i < data.length; i++) {
                  this.insert(data[i]);
              }
              return this;
          }

          // recursively build the tree with the given data from scratch using OMT algorithm
          let node = this._build(data.slice(), 0, data.length - 1, 0);

          if (!this.data.children.length) {
              // save as is if tree is empty
              this.data = node;

          } else if (this.data.height === node.height) {
              // split root if trees have the same height
              this._splitRoot(this.data, node);

          } else {
              if (this.data.height < node.height) {
                  // swap trees if inserted one is bigger
                  const tmpNode = this.data;
                  this.data = node;
                  node = tmpNode;
              }

              // insert the small tree into the large tree at appropriate level
              this._insert(node, this.data.height - node.height - 1, true);
          }

          return this;
      }

      insert(item) {
          if (item) this._insert(item, this.data.height - 1);
          return this;
      }

      clear() {
          this.data = createNode([]);
          return this;
      }

      remove(item, equalsFn) {
          if (!item) return this;

          let node = this.data;
          const bbox = this.toBBox(item);
          const path = [];
          const indexes = [];
          let i, parent, goingUp;

          // depth-first iterative tree traversal
          while (node || path.length) {

              if (!node) { // go up
                  node = path.pop();
                  parent = path[path.length - 1];
                  i = indexes.pop();
                  goingUp = true;
              }

              if (node.leaf) { // check current node
                  const index = findItem(item, node.children, equalsFn);

                  if (index !== -1) {
                      // item found, remove the item and condense tree upwards
                      node.children.splice(index, 1);
                      path.push(node);
                      this._condense(path);
                      return this;
                  }
              }

              if (!goingUp && !node.leaf && contains(node, bbox)) { // go down
                  path.push(node);
                  indexes.push(i);
                  i = 0;
                  parent = node;
                  node = node.children[0];

              } else if (parent) { // go right
                  i++;
                  node = parent.children[i];
                  goingUp = false;

              } else node = null; // nothing found
          }

          return this;
      }

      toBBox(item) { return item; }

      compareMinX(a, b) { return a.minX - b.minX; }
      compareMinY(a, b) { return a.minY - b.minY; }

      toJSON() { return this.data; }

      fromJSON(data) {
          this.data = data;
          return this;
      }

      _all(node, result) {
          const nodesToSearch = [];
          while (node) {
              if (node.leaf) result.push(...node.children);
              else nodesToSearch.push(...node.children);

              node = nodesToSearch.pop();
          }
          return result;
      }

      _build(items, left, right, height) {

          const N = right - left + 1;
          let M = this._maxEntries;
          let node;

          if (N <= M) {
              // reached leaf level; return leaf
              node = createNode(items.slice(left, right + 1));
              calcBBox(node, this.toBBox);
              return node;
          }

          if (!height) {
              // target height of the bulk-loaded tree
              height = Math.ceil(Math.log(N) / Math.log(M));

              // target number of root entries to maximize storage utilization
              M = Math.ceil(N / Math.pow(M, height - 1));
          }

          node = createNode([]);
          node.leaf = false;
          node.height = height;

          // split the items into M mostly square tiles

          const N2 = Math.ceil(N / M);
          const N1 = N2 * Math.ceil(Math.sqrt(M));

          multiSelect(items, left, right, N1, this.compareMinX);

          for (let i = left; i <= right; i += N1) {

              const right2 = Math.min(i + N1 - 1, right);

              multiSelect(items, i, right2, N2, this.compareMinY);

              for (let j = i; j <= right2; j += N2) {

                  const right3 = Math.min(j + N2 - 1, right2);

                  // pack each entry recursively
                  node.children.push(this._build(items, j, right3, height - 1));
              }
          }

          calcBBox(node, this.toBBox);

          return node;
      }

      _chooseSubtree(bbox, node, level, path) {
          while (true) {
              path.push(node);

              if (node.leaf || path.length - 1 === level) break;

              let minArea = Infinity;
              let minEnlargement = Infinity;
              let targetNode;

              for (let i = 0; i < node.children.length; i++) {
                  const child = node.children[i];
                  const area = bboxArea(child);
                  const enlargement = enlargedArea(bbox, child) - area;

                  // choose entry with the least area enlargement
                  if (enlargement < minEnlargement) {
                      minEnlargement = enlargement;
                      minArea = area < minArea ? area : minArea;
                      targetNode = child;

                  } else if (enlargement === minEnlargement) {
                      // otherwise choose one with the smallest area
                      if (area < minArea) {
                          minArea = area;
                          targetNode = child;
                      }
                  }
              }

              node = targetNode || node.children[0];
          }

          return node;
      }

      _insert(item, level, isNode) {
          const bbox = isNode ? item : this.toBBox(item);
          const insertPath = [];

          // find the best node for accommodating the item, saving all nodes along the path too
          const node = this._chooseSubtree(bbox, this.data, level, insertPath);

          // put the item into the node
          node.children.push(item);
          extend(node, bbox);

          // split on node overflow; propagate upwards if necessary
          while (level >= 0) {
              if (insertPath[level].children.length > this._maxEntries) {
                  this._split(insertPath, level);
                  level--;
              } else break;
          }

          // adjust bboxes along the insertion path
          this._adjustParentBBoxes(bbox, insertPath, level);
      }

      // split overflowed node into two
      _split(insertPath, level) {
          const node = insertPath[level];
          const M = node.children.length;
          const m = this._minEntries;

          this._chooseSplitAxis(node, m, M);

          const splitIndex = this._chooseSplitIndex(node, m, M);

          const newNode = createNode(node.children.splice(splitIndex, node.children.length - splitIndex));
          newNode.height = node.height;
          newNode.leaf = node.leaf;

          calcBBox(node, this.toBBox);
          calcBBox(newNode, this.toBBox);

          if (level) insertPath[level - 1].children.push(newNode);
          else this._splitRoot(node, newNode);
      }

      _splitRoot(node, newNode) {
          // split root node
          this.data = createNode([node, newNode]);
          this.data.height = node.height + 1;
          this.data.leaf = false;
          calcBBox(this.data, this.toBBox);
      }

      _chooseSplitIndex(node, m, M) {
          let index;
          let minOverlap = Infinity;
          let minArea = Infinity;

          for (let i = m; i <= M - m; i++) {
              const bbox1 = distBBox(node, 0, i, this.toBBox);
              const bbox2 = distBBox(node, i, M, this.toBBox);

              const overlap = intersectionArea(bbox1, bbox2);
              const area = bboxArea(bbox1) + bboxArea(bbox2);

              // choose distribution with minimum overlap
              if (overlap < minOverlap) {
                  minOverlap = overlap;
                  index = i;

                  minArea = area < minArea ? area : minArea;

              } else if (overlap === minOverlap) {
                  // otherwise choose distribution with minimum area
                  if (area < minArea) {
                      minArea = area;
                      index = i;
                  }
              }
          }

          return index || M - m;
      }

      // sorts node children by the best axis for split
      _chooseSplitAxis(node, m, M) {
          const compareMinX = node.leaf ? this.compareMinX : compareNodeMinX;
          const compareMinY = node.leaf ? this.compareMinY : compareNodeMinY;
          const xMargin = this._allDistMargin(node, m, M, compareMinX);
          const yMargin = this._allDistMargin(node, m, M, compareMinY);

          // if total distributions margin value is minimal for x, sort by minX,
          // otherwise it's already sorted by minY
          if (xMargin < yMargin) node.children.sort(compareMinX);
      }

      // total margin of all possible split distributions where each node is at least m full
      _allDistMargin(node, m, M, compare) {
          node.children.sort(compare);

          const toBBox = this.toBBox;
          const leftBBox = distBBox(node, 0, m, toBBox);
          const rightBBox = distBBox(node, M - m, M, toBBox);
          let margin = bboxMargin(leftBBox) + bboxMargin(rightBBox);

          for (let i = m; i < M - m; i++) {
              const child = node.children[i];
              extend(leftBBox, node.leaf ? toBBox(child) : child);
              margin += bboxMargin(leftBBox);
          }

          for (let i = M - m - 1; i >= m; i--) {
              const child = node.children[i];
              extend(rightBBox, node.leaf ? toBBox(child) : child);
              margin += bboxMargin(rightBBox);
          }

          return margin;
      }

      _adjustParentBBoxes(bbox, path, level) {
          // adjust bboxes along the given tree path
          for (let i = level; i >= 0; i--) {
              extend(path[i], bbox);
          }
      }

      _condense(path) {
          // go through the path, removing empty nodes and updating bboxes
          for (let i = path.length - 1, siblings; i >= 0; i--) {
              if (path[i].children.length === 0) {
                  if (i > 0) {
                      siblings = path[i - 1].children;
                      siblings.splice(siblings.indexOf(path[i]), 1);

                  } else this.clear();

              } else calcBBox(path[i], this.toBBox);
          }
      }
  }

  function findItem(item, items, equalsFn) {
      if (!equalsFn) return items.indexOf(item);

      for (let i = 0; i < items.length; i++) {
          if (equalsFn(item, items[i])) return i;
      }
      return -1;
  }

  // calculate node's bbox from bboxes of its children
  function calcBBox(node, toBBox) {
      distBBox(node, 0, node.children.length, toBBox, node);
  }

  // min bounding rectangle of node children from k to p-1
  function distBBox(node, k, p, toBBox, destNode) {
      if (!destNode) destNode = createNode(null);
      destNode.minX = Infinity;
      destNode.minY = Infinity;
      destNode.maxX = -Infinity;
      destNode.maxY = -Infinity;

      for (let i = k; i < p; i++) {
          const child = node.children[i];
          extend(destNode, node.leaf ? toBBox(child) : child);
      }

      return destNode;
  }

  function extend(a, b) {
      a.minX = Math.min(a.minX, b.minX);
      a.minY = Math.min(a.minY, b.minY);
      a.maxX = Math.max(a.maxX, b.maxX);
      a.maxY = Math.max(a.maxY, b.maxY);
      return a;
  }

  function compareNodeMinX(a, b) { return a.minX - b.minX; }
  function compareNodeMinY(a, b) { return a.minY - b.minY; }

  function bboxArea(a)   { return (a.maxX - a.minX) * (a.maxY - a.minY); }
  function bboxMargin(a) { return (a.maxX - a.minX) + (a.maxY - a.minY); }

  function enlargedArea(a, b) {
      return (Math.max(b.maxX, a.maxX) - Math.min(b.minX, a.minX)) *
             (Math.max(b.maxY, a.maxY) - Math.min(b.minY, a.minY));
  }

  function intersectionArea(a, b) {
      const minX = Math.max(a.minX, b.minX);
      const minY = Math.max(a.minY, b.minY);
      const maxX = Math.min(a.maxX, b.maxX);
      const maxY = Math.min(a.maxY, b.maxY);

      return Math.max(0, maxX - minX) *
             Math.max(0, maxY - minY);
  }

  function contains(a, b) {
      return a.minX <= b.minX &&
             a.minY <= b.minY &&
             b.maxX <= a.maxX &&
             b.maxY <= a.maxY;
  }

  function intersects(a, b) {
      return b.minX <= a.maxX &&
             b.minY <= a.maxY &&
             b.maxX >= a.minX &&
             b.maxY >= a.minY;
  }

  function createNode(children) {
      return {
          children,
          height: 1,
          leaf: true,
          minX: Infinity,
          minY: Infinity,
          maxX: -Infinity,
          maxY: -Infinity
      };
  }

  // sort an array so that items come in groups of n unsorted items, with groups sorted between each other;
  // combines selection algorithm with binary divide & conquer approach

  function multiSelect(arr, left, right, n, compare) {
      const stack = [left, right];

      while (stack.length) {
          right = stack.pop();
          left = stack.pop();

          if (right - left <= n) continue;

          const mid = left + Math.ceil((right - left) / n / 2) * n;
          quickselect(arr, mid, left, right, compare);

          stack.push(left, mid, mid, right);
      }
  }

  function define(constructor, factory, prototype) {
    constructor.prototype = factory.prototype = prototype;
    prototype.constructor = constructor;
  }

  function extend$1(parent, definition) {
    var prototype = Object.create(parent.prototype);
    for (var key in definition) prototype[key] = definition[key];
    return prototype;
  }

  function Color() {}

  var darker = 0.7;
  var brighter = 1 / darker;

  var reI = "\\s*([+-]?\\d+)\\s*",
      reN = "\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*",
      reP = "\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*",
      reHex = /^#([0-9a-f]{3,8})$/,
      reRgbInteger = new RegExp("^rgb\\(" + [reI, reI, reI] + "\\)$"),
      reRgbPercent = new RegExp("^rgb\\(" + [reP, reP, reP] + "\\)$"),
      reRgbaInteger = new RegExp("^rgba\\(" + [reI, reI, reI, reN] + "\\)$"),
      reRgbaPercent = new RegExp("^rgba\\(" + [reP, reP, reP, reN] + "\\)$"),
      reHslPercent = new RegExp("^hsl\\(" + [reN, reP, reP] + "\\)$"),
      reHslaPercent = new RegExp("^hsla\\(" + [reN, reP, reP, reN] + "\\)$");

  var named = {
    aliceblue: 0xf0f8ff,
    antiquewhite: 0xfaebd7,
    aqua: 0x00ffff,
    aquamarine: 0x7fffd4,
    azure: 0xf0ffff,
    beige: 0xf5f5dc,
    bisque: 0xffe4c4,
    black: 0x000000,
    blanchedalmond: 0xffebcd,
    blue: 0x0000ff,
    blueviolet: 0x8a2be2,
    brown: 0xa52a2a,
    burlywood: 0xdeb887,
    cadetblue: 0x5f9ea0,
    chartreuse: 0x7fff00,
    chocolate: 0xd2691e,
    coral: 0xff7f50,
    cornflowerblue: 0x6495ed,
    cornsilk: 0xfff8dc,
    crimson: 0xdc143c,
    cyan: 0x00ffff,
    darkblue: 0x00008b,
    darkcyan: 0x008b8b,
    darkgoldenrod: 0xb8860b,
    darkgray: 0xa9a9a9,
    darkgreen: 0x006400,
    darkgrey: 0xa9a9a9,
    darkkhaki: 0xbdb76b,
    darkmagenta: 0x8b008b,
    darkolivegreen: 0x556b2f,
    darkorange: 0xff8c00,
    darkorchid: 0x9932cc,
    darkred: 0x8b0000,
    darksalmon: 0xe9967a,
    darkseagreen: 0x8fbc8f,
    darkslateblue: 0x483d8b,
    darkslategray: 0x2f4f4f,
    darkslategrey: 0x2f4f4f,
    darkturquoise: 0x00ced1,
    darkviolet: 0x9400d3,
    deeppink: 0xff1493,
    deepskyblue: 0x00bfff,
    dimgray: 0x696969,
    dimgrey: 0x696969,
    dodgerblue: 0x1e90ff,
    firebrick: 0xb22222,
    floralwhite: 0xfffaf0,
    forestgreen: 0x228b22,
    fuchsia: 0xff00ff,
    gainsboro: 0xdcdcdc,
    ghostwhite: 0xf8f8ff,
    gold: 0xffd700,
    goldenrod: 0xdaa520,
    gray: 0x808080,
    green: 0x008000,
    greenyellow: 0xadff2f,
    grey: 0x808080,
    honeydew: 0xf0fff0,
    hotpink: 0xff69b4,
    indianred: 0xcd5c5c,
    indigo: 0x4b0082,
    ivory: 0xfffff0,
    khaki: 0xf0e68c,
    lavender: 0xe6e6fa,
    lavenderblush: 0xfff0f5,
    lawngreen: 0x7cfc00,
    lemonchiffon: 0xfffacd,
    lightblue: 0xadd8e6,
    lightcoral: 0xf08080,
    lightcyan: 0xe0ffff,
    lightgoldenrodyellow: 0xfafad2,
    lightgray: 0xd3d3d3,
    lightgreen: 0x90ee90,
    lightgrey: 0xd3d3d3,
    lightpink: 0xffb6c1,
    lightsalmon: 0xffa07a,
    lightseagreen: 0x20b2aa,
    lightskyblue: 0x87cefa,
    lightslategray: 0x778899,
    lightslategrey: 0x778899,
    lightsteelblue: 0xb0c4de,
    lightyellow: 0xffffe0,
    lime: 0x00ff00,
    limegreen: 0x32cd32,
    linen: 0xfaf0e6,
    magenta: 0xff00ff,
    maroon: 0x800000,
    mediumaquamarine: 0x66cdaa,
    mediumblue: 0x0000cd,
    mediumorchid: 0xba55d3,
    mediumpurple: 0x9370db,
    mediumseagreen: 0x3cb371,
    mediumslateblue: 0x7b68ee,
    mediumspringgreen: 0x00fa9a,
    mediumturquoise: 0x48d1cc,
    mediumvioletred: 0xc71585,
    midnightblue: 0x191970,
    mintcream: 0xf5fffa,
    mistyrose: 0xffe4e1,
    moccasin: 0xffe4b5,
    navajowhite: 0xffdead,
    navy: 0x000080,
    oldlace: 0xfdf5e6,
    olive: 0x808000,
    olivedrab: 0x6b8e23,
    orange: 0xffa500,
    orangered: 0xff4500,
    orchid: 0xda70d6,
    palegoldenrod: 0xeee8aa,
    palegreen: 0x98fb98,
    paleturquoise: 0xafeeee,
    palevioletred: 0xdb7093,
    papayawhip: 0xffefd5,
    peachpuff: 0xffdab9,
    peru: 0xcd853f,
    pink: 0xffc0cb,
    plum: 0xdda0dd,
    powderblue: 0xb0e0e6,
    purple: 0x800080,
    rebeccapurple: 0x663399,
    red: 0xff0000,
    rosybrown: 0xbc8f8f,
    royalblue: 0x4169e1,
    saddlebrown: 0x8b4513,
    salmon: 0xfa8072,
    sandybrown: 0xf4a460,
    seagreen: 0x2e8b57,
    seashell: 0xfff5ee,
    sienna: 0xa0522d,
    silver: 0xc0c0c0,
    skyblue: 0x87ceeb,
    slateblue: 0x6a5acd,
    slategray: 0x708090,
    slategrey: 0x708090,
    snow: 0xfffafa,
    springgreen: 0x00ff7f,
    steelblue: 0x4682b4,
    tan: 0xd2b48c,
    teal: 0x008080,
    thistle: 0xd8bfd8,
    tomato: 0xff6347,
    turquoise: 0x40e0d0,
    violet: 0xee82ee,
    wheat: 0xf5deb3,
    white: 0xffffff,
    whitesmoke: 0xf5f5f5,
    yellow: 0xffff00,
    yellowgreen: 0x9acd32
  };

  define(Color, color, {
    copy: function(channels) {
      return Object.assign(new this.constructor, this, channels);
    },
    displayable: function() {
      return this.rgb().displayable();
    },
    hex: color_formatHex, // Deprecated! Use color.formatHex.
    formatHex: color_formatHex,
    formatHsl: color_formatHsl,
    formatRgb: color_formatRgb,
    toString: color_formatRgb
  });

  function color_formatHex() {
    return this.rgb().formatHex();
  }

  function color_formatHsl() {
    return hslConvert(this).formatHsl();
  }

  function color_formatRgb() {
    return this.rgb().formatRgb();
  }

  function color(format) {
    var m, l;
    format = (format + "").trim().toLowerCase();
    return (m = reHex.exec(format)) ? (l = m[1].length, m = parseInt(m[1], 16), l === 6 ? rgbn(m) // #ff0000
        : l === 3 ? new Rgb((m >> 8 & 0xf) | (m >> 4 & 0xf0), (m >> 4 & 0xf) | (m & 0xf0), ((m & 0xf) << 4) | (m & 0xf), 1) // #f00
        : l === 8 ? rgba(m >> 24 & 0xff, m >> 16 & 0xff, m >> 8 & 0xff, (m & 0xff) / 0xff) // #ff000000
        : l === 4 ? rgba((m >> 12 & 0xf) | (m >> 8 & 0xf0), (m >> 8 & 0xf) | (m >> 4 & 0xf0), (m >> 4 & 0xf) | (m & 0xf0), (((m & 0xf) << 4) | (m & 0xf)) / 0xff) // #f000
        : null) // invalid hex
        : (m = reRgbInteger.exec(format)) ? new Rgb(m[1], m[2], m[3], 1) // rgb(255, 0, 0)
        : (m = reRgbPercent.exec(format)) ? new Rgb(m[1] * 255 / 100, m[2] * 255 / 100, m[3] * 255 / 100, 1) // rgb(100%, 0%, 0%)
        : (m = reRgbaInteger.exec(format)) ? rgba(m[1], m[2], m[3], m[4]) // rgba(255, 0, 0, 1)
        : (m = reRgbaPercent.exec(format)) ? rgba(m[1] * 255 / 100, m[2] * 255 / 100, m[3] * 255 / 100, m[4]) // rgb(100%, 0%, 0%, 1)
        : (m = reHslPercent.exec(format)) ? hsla(m[1], m[2] / 100, m[3] / 100, 1) // hsl(120, 50%, 50%)
        : (m = reHslaPercent.exec(format)) ? hsla(m[1], m[2] / 100, m[3] / 100, m[4]) // hsla(120, 50%, 50%, 1)
        : named.hasOwnProperty(format) ? rgbn(named[format]) // eslint-disable-line no-prototype-builtins
        : format === "transparent" ? new Rgb(NaN, NaN, NaN, 0)
        : null;
  }

  function rgbn(n) {
    return new Rgb(n >> 16 & 0xff, n >> 8 & 0xff, n & 0xff, 1);
  }

  function rgba(r, g, b, a) {
    if (a <= 0) r = g = b = NaN;
    return new Rgb(r, g, b, a);
  }

  function rgbConvert(o) {
    if (!(o instanceof Color)) o = color(o);
    if (!o) return new Rgb;
    o = o.rgb();
    return new Rgb(o.r, o.g, o.b, o.opacity);
  }

  function rgb(r, g, b, opacity) {
    return arguments.length === 1 ? rgbConvert(r) : new Rgb(r, g, b, opacity == null ? 1 : opacity);
  }

  function Rgb(r, g, b, opacity) {
    this.r = +r;
    this.g = +g;
    this.b = +b;
    this.opacity = +opacity;
  }

  define(Rgb, rgb, extend$1(Color, {
    brighter: function(k) {
      k = k == null ? brighter : Math.pow(brighter, k);
      return new Rgb(this.r * k, this.g * k, this.b * k, this.opacity);
    },
    darker: function(k) {
      k = k == null ? darker : Math.pow(darker, k);
      return new Rgb(this.r * k, this.g * k, this.b * k, this.opacity);
    },
    rgb: function() {
      return this;
    },
    displayable: function() {
      return (-0.5 <= this.r && this.r < 255.5)
          && (-0.5 <= this.g && this.g < 255.5)
          && (-0.5 <= this.b && this.b < 255.5)
          && (0 <= this.opacity && this.opacity <= 1);
    },
    hex: rgb_formatHex, // Deprecated! Use color.formatHex.
    formatHex: rgb_formatHex,
    formatRgb: rgb_formatRgb,
    toString: rgb_formatRgb
  }));

  function rgb_formatHex() {
    return "#" + hex(this.r) + hex(this.g) + hex(this.b);
  }

  function rgb_formatRgb() {
    var a = this.opacity; a = isNaN(a) ? 1 : Math.max(0, Math.min(1, a));
    return (a === 1 ? "rgb(" : "rgba(")
        + Math.max(0, Math.min(255, Math.round(this.r) || 0)) + ", "
        + Math.max(0, Math.min(255, Math.round(this.g) || 0)) + ", "
        + Math.max(0, Math.min(255, Math.round(this.b) || 0))
        + (a === 1 ? ")" : ", " + a + ")");
  }

  function hex(value) {
    value = Math.max(0, Math.min(255, Math.round(value) || 0));
    return (value < 16 ? "0" : "") + value.toString(16);
  }

  function hsla(h, s, l, a) {
    if (a <= 0) h = s = l = NaN;
    else if (l <= 0 || l >= 1) h = s = NaN;
    else if (s <= 0) h = NaN;
    return new Hsl(h, s, l, a);
  }

  function hslConvert(o) {
    if (o instanceof Hsl) return new Hsl(o.h, o.s, o.l, o.opacity);
    if (!(o instanceof Color)) o = color(o);
    if (!o) return new Hsl;
    if (o instanceof Hsl) return o;
    o = o.rgb();
    var r = o.r / 255,
        g = o.g / 255,
        b = o.b / 255,
        min = Math.min(r, g, b),
        max = Math.max(r, g, b),
        h = NaN,
        s = max - min,
        l = (max + min) / 2;
    if (s) {
      if (r === max) h = (g - b) / s + (g < b) * 6;
      else if (g === max) h = (b - r) / s + 2;
      else h = (r - g) / s + 4;
      s /= l < 0.5 ? max + min : 2 - max - min;
      h *= 60;
    } else {
      s = l > 0 && l < 1 ? 0 : h;
    }
    return new Hsl(h, s, l, o.opacity);
  }

  function hsl(h, s, l, opacity) {
    return arguments.length === 1 ? hslConvert(h) : new Hsl(h, s, l, opacity == null ? 1 : opacity);
  }

  function Hsl(h, s, l, opacity) {
    this.h = +h;
    this.s = +s;
    this.l = +l;
    this.opacity = +opacity;
  }

  define(Hsl, hsl, extend$1(Color, {
    brighter: function(k) {
      k = k == null ? brighter : Math.pow(brighter, k);
      return new Hsl(this.h, this.s, this.l * k, this.opacity);
    },
    darker: function(k) {
      k = k == null ? darker : Math.pow(darker, k);
      return new Hsl(this.h, this.s, this.l * k, this.opacity);
    },
    rgb: function() {
      var h = this.h % 360 + (this.h < 0) * 360,
          s = isNaN(h) || isNaN(this.s) ? 0 : this.s,
          l = this.l,
          m2 = l + (l < 0.5 ? l : 1 - l) * s,
          m1 = 2 * l - m2;
      return new Rgb(
        hsl2rgb(h >= 240 ? h - 240 : h + 120, m1, m2),
        hsl2rgb(h, m1, m2),
        hsl2rgb(h < 120 ? h + 240 : h - 120, m1, m2),
        this.opacity
      );
    },
    displayable: function() {
      return (0 <= this.s && this.s <= 1 || isNaN(this.s))
          && (0 <= this.l && this.l <= 1)
          && (0 <= this.opacity && this.opacity <= 1);
    },
    formatHsl: function() {
      var a = this.opacity; a = isNaN(a) ? 1 : Math.max(0, Math.min(1, a));
      return (a === 1 ? "hsl(" : "hsla(")
          + (this.h || 0) + ", "
          + (this.s || 0) * 100 + "%, "
          + (this.l || 0) * 100 + "%"
          + (a === 1 ? ")" : ", " + a + ")");
    }
  }));

  /* From FvD 13.37, CSS Color Module Level 3 */
  function hsl2rgb(h, m1, m2) {
    return (h < 60 ? m1 + (m2 - m1) * h / 60
        : h < 180 ? m2
        : h < 240 ? m1 + (m2 - m1) * (240 - h) / 60
        : m1) * 255;
  }

  /**
   * 两点之间的距离
   * @param {number} x1 起始点 x
   * @param {number} y1 起始点 y
   * @param {number} x2 结束点 x
   * @param {number} y2 结束点 y
   * @return {number} 距离
   */
  function distance$1(x1, y1, x2, y2) {
    var dx = x1 - x2;
    var dy = y1 - y2;
    return Math.sqrt(dx * dx + dy * dy);
  }
  function isNumberEqual$1(v1, v2) {
    return Math.abs(v1 - v2) < 0.001;
  }
  function getBBoxByArray(xArr, yArr) {
    var minX = Math.min.apply(Math, xArr);
    var minY = Math.min.apply(Math, yArr);
    var maxX = Math.max.apply(Math, xArr);
    var maxY = Math.max.apply(Math, yArr);
    return {
      x: minX,
      y: minY,
      width: maxX - minX,
      height: maxY - minY
    };
  }
  function piMod(angle) {
    return (angle + Math.PI * 2) % (Math.PI * 2);
  }

  var line = {
    /**
     * 计算线段的包围盒
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {object} 包围盒对象
     */
    box: function box(x1, y1, x2, y2) {
      return getBBoxByArray([x1, x2], [y1, y2]);
    },
    /**
     * 线段的长度
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {number} 距离
     */
    length: function length(x1, y1, x2, y2) {
      return distance$1(x1, y1, x2, y2);
    },
    /**
     * 根据比例获取点
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} t 指定比例
     * @return {object} 包含 x, y 的点
     */
    pointAt: function pointAt(x1, y1, x2, y2, t) {
      return {
        x: (1 - t) * x1 + t * x2,
        y: (1 - t) * y1 + t * y2
      };
    },
    /**
     * 点到线段的距离
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} x  测试点 x
     * @param {number} y  测试点 y
     * @return {number} 距离
     */
    pointDistance: function pointDistance(x1, y1, x2, y2, x, y) {
      // 投影距离 x1, y1 的向量，假设 p, p1, p2 三个点，投影点为 a
      // p1a = p1p.p1p2/|p1p2| * (p1p 的单位向量)
      var cross = (x2 - x1) * (x - x1) + (y2 - y1) * (y - y1);
      if (cross < 0) {
        return distance$1(x1, y1, x, y);
      }
      var lengthSquare = (x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1);
      if (cross > lengthSquare) {
        return distance$1(x2, y2, x, y);
      }
      return this.pointToLine(x1, y1, x2, y2, x, y);
    },
    /**
     * 点到直线的距离，而不是点到线段的距离
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} x  测试点 x
     * @param {number} y  测试点 y
     * @return {number} 距离
     */
    pointToLine: function pointToLine(x1, y1, x2, y2, x, y) {
      var d = [x2 - x1, y2 - y1];
      // 如果端点相等，则判定点到点的距离
      if (exactEquals$1(d, [0, 0])) {
        return Math.sqrt((x - x1) * (x - x1) + (y - y1) * (y - y1));
      }
      var u = [-d[1], d[0]];
      normalize$3(u, u);
      var a = [x - x1, y - y1];
      return Math.abs(dot$1(a, u));
    },
    /**
     * 线段的角度
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {number} 导数
     */
    tangentAngle: function tangentAngle(x1, y1, x2, y2) {
      return Math.atan2(y2 - y1, x2 - x1);
    }
  };

  var EPSILON$1 = 0.0001;
  /**
   * 使用牛顿切割法求最近的点
   * @param {number[]} xArr      点的 x 数组
   * @param {number[]} yArr      点的 y 数组
   * @param {number}   x         指定的点 x
   * @param {number}   y         指定的点 y
   * @param {Function} tCallback 差值函数
   */
  function nearestPoint(xArr, yArr, x, y, tCallback, length) {
    var t = -1;
    var d = Infinity;
    var v0 = [x, y];
    var segNum = 20;
    if (length && length > 200) {
      segNum = length / 10;
    }
    var increaseRate = 1 / segNum;
    var interval = increaseRate / 10;
    for (var i = 0; i <= segNum; i++) {
      var _t = i * increaseRate;
      var v1 = [tCallback.apply(void 0, xArr.concat([_t])), tCallback.apply(void 0, yArr.concat([_t]))];
      var d1 = distance$1(v0[0], v0[1], v1[0], v1[1]);
      if (d1 < d) {
        t = _t;
        d = d1;
      }
    }
    // 提前终止
    if (t === 0) {
      return {
        x: xArr[0],
        y: yArr[0]
      };
    }
    if (t === 1) {
      var count = xArr.length;
      return {
        x: xArr[count - 1],
        y: yArr[count - 1]
      };
    }
    d = Infinity;
    for (var _i = 0; _i < 32; _i++) {
      if (interval < EPSILON$1) {
        break;
      }
      var prev = t - interval;
      var next = t + interval;
      var _v = [tCallback.apply(void 0, xArr.concat([prev])), tCallback.apply(void 0, yArr.concat([prev]))];
      var _d = distance$1(v0[0], v0[1], _v[0], _v[1]);
      if (prev >= 0 && _d < d) {
        t = prev;
        d = _d;
      } else {
        var v2 = [tCallback.apply(void 0, xArr.concat([next])), tCallback.apply(void 0, yArr.concat([next]))];
        var d2 = distance$1(v0[0], v0[1], v2[0], v2[1]);
        if (next <= 1 && d2 < d) {
          t = next;
          d = d2;
        } else {
          interval *= 0.5;
        }
      }
    }
    return {
      x: tCallback.apply(void 0, xArr.concat([t])),
      y: tCallback.apply(void 0, yArr.concat([t]))
    };
  }
  // 近似求解 https://community.khronos.org/t/3d-cubic-bezier-segment-length/62363/2
  function snapLength(xArr, yArr) {
    var totalLength = 0;
    var count = xArr.length;
    for (var i = 0; i < count; i++) {
      var x = xArr[i];
      var y = yArr[i];
      var nextX = xArr[(i + 1) % count];
      var nextY = yArr[(i + 1) % count];
      totalLength += distance$1(x, y, nextX, nextY);
    }
    return totalLength / 2;
  }

  // 差值公式
  function quadraticAt(p0, p1, p2, t) {
    var onet = 1 - t;
    return onet * onet * p0 + 2 * t * onet * p1 + t * t * p2;
  }
  // 求极值
  function extrema(p0, p1, p2) {
    var a = p0 + p2 - 2 * p1;
    if (isNumberEqual$1(a, 0)) {
      return [0.5];
    }
    var rst = (p0 - p1) / a;
    if (rst <= 1 && rst >= 0) {
      return [rst];
    }
    return [];
  }
  function derivativeAt(p0, p1, p2, t) {
    return 2 * (1 - t) * (p1 - p0) + 2 * t * (p2 - p1);
  }
  // 分割贝塞尔曲线
  function divideQuadratic(x1, y1, x2, y2, x3, y3, t) {
    // 划分点
    var xt = quadraticAt(x1, x2, x3, t);
    var yt = quadraticAt(y1, y2, y3, t);
    // 分割的第一条曲线的控制点
    var controlPoint1 = line.pointAt(x1, y1, x2, y2, t);
    // 分割的第二条曲线的控制点
    var controlPoint2 = line.pointAt(x2, y2, x3, y3, t);
    return [[x1, y1, controlPoint1.x, controlPoint1.y, xt, yt], [xt, yt, controlPoint2.x, controlPoint2.y, x3, y3]];
  }
  // 使用迭代法取贝塞尔曲线的长度
  function quadraticLength(x1, y1, x2, y2, x3, y3, iterationCount) {
    if (iterationCount === 0) {
      return (distance$1(x1, y1, x2, y2) + distance$1(x2, y2, x3, y3) + distance$1(x1, y1, x3, y3)) / 2;
    }
    var quadratics = divideQuadratic(x1, y1, x2, y2, x3, y3, 0.5);
    var left = quadratics[0];
    var right = quadratics[1];
    left.push(iterationCount - 1);
    right.push(iterationCount - 1);
    return quadraticLength.apply(void 0, left) + quadraticLength.apply(void 0, right);
  }
  var quadratic = {
    box: function box(x1, y1, x2, y2, x3, y3) {
      var xExtrema = extrema(x1, x2, x3)[0];
      var yExtrema = extrema(y1, y2, y3)[0];
      // 控制点不加入 box 的计算
      var xArr = [x1, x3];
      var yArr = [y1, y3];
      if (xExtrema !== undefined) {
        xArr.push(quadraticAt(x1, x2, x3, xExtrema));
      }
      if (yExtrema !== undefined) {
        yArr.push(quadraticAt(y1, y2, y3, yExtrema));
      }
      return getBBoxByArray(xArr, yArr);
    },
    length: function length(x1, y1, x2, y2, x3, y3) {
      return quadraticLength(x1, y1, x2, y2, x3, y3, 3);
    },
    nearestPoint: function nearestPoint$1(x1, y1, x2, y2, x3, y3, x0, y0) {
      return nearestPoint([x1, x2, x3], [y1, y2, y3], x0, y0, quadraticAt);
    },
    pointDistance: function pointDistance(x1, y1, x2, y2, x3, y3, x0, y0) {
      var point = this.nearestPoint(x1, y1, x2, y2, x3, y3, x0, y0);
      return distance$1(point.x, point.y, x0, y0);
    },
    interpolationAt: quadraticAt,
    pointAt: function pointAt(x1, y1, x2, y2, x3, y3, t) {
      return {
        x: quadraticAt(x1, x2, x3, t),
        y: quadraticAt(y1, y2, y3, t)
      };
    },
    divide: function divide(x1, y1, x2, y2, x3, y3, t) {
      return divideQuadratic(x1, y1, x2, y2, x3, y3, t);
    },
    tangentAngle: function tangentAngle(x1, y1, x2, y2, x3, y3, t) {
      var dx = derivativeAt(x1, x2, x3, t);
      var dy = derivativeAt(y1, y2, y3, t);
      var angle = Math.atan2(dy, dx);
      return piMod(angle);
    }
  };

  function cubicAt(p0, p1, p2, p3, t) {
    var onet = 1 - t; // t * t * t 的性能大概是 Math.pow(t, 3) 的三倍
    return onet * onet * onet * p0 + 3 * p1 * t * onet * onet + 3 * p2 * t * t * onet + p3 * t * t * t;
  }
  function derivativeAt$1(p0, p1, p2, p3, t) {
    var onet = 1 - t;
    return 3 * (onet * onet * (p1 - p0) + 2 * onet * t * (p2 - p1) + t * t * (p3 - p2));
  }
  function extrema$1(p0, p1, p2, p3) {
    var a = -3 * p0 + 9 * p1 - 9 * p2 + 3 * p3;
    var b = 6 * p0 - 12 * p1 + 6 * p2;
    var c = 3 * p1 - 3 * p0;
    var extremas = [];
    var t1;
    var t2;
    var discSqrt;
    if (isNumberEqual$1(a, 0)) {
      if (!isNumberEqual$1(b, 0)) {
        t1 = -c / b;
        if (t1 >= 0 && t1 <= 1) {
          extremas.push(t1);
        }
      }
    } else {
      var disc = b * b - 4 * a * c;
      if (isNumberEqual$1(disc, 0)) {
        extremas.push(-b / (2 * a));
      } else if (disc > 0) {
        discSqrt = Math.sqrt(disc);
        t1 = (-b + discSqrt) / (2 * a);
        t2 = (-b - discSqrt) / (2 * a);
        if (t1 >= 0 && t1 <= 1) {
          extremas.push(t1);
        }
        if (t2 >= 0 && t2 <= 1) {
          extremas.push(t2);
        }
      }
    }
    return extremas;
  }
  // 分割贝塞尔曲线
  function divideCubic(x1, y1, x2, y2, x3, y3, x4, y4, t) {
    // 划分点
    var xt = cubicAt(x1, x2, x3, x4, t);
    var yt = cubicAt(y1, y2, y3, y4, t);
    // 计算两点之间的差值点
    var c1 = line.pointAt(x1, y1, x2, y2, t);
    var c2 = line.pointAt(x2, y2, x3, y3, t);
    var c3 = line.pointAt(x3, y3, x4, y4, t);
    var c12 = line.pointAt(c1.x, c1.y, c2.x, c2.y, t);
    var c23 = line.pointAt(c2.x, c2.y, c3.x, c3.y, t);
    return [[x1, y1, c1.x, c1.y, c12.x, c12.y, xt, yt], [xt, yt, c23.x, c23.y, c3.x, c3.y, x4, y4]];
  }
  // 使用迭代法取贝塞尔曲线的长度，二阶和三阶分开写，更清晰和便于调试
  function cubicLength(x1, y1, x2, y2, x3, y3, x4, y4, iterationCount) {
    if (iterationCount === 0) {
      return snapLength([x1, x2, x3, x4], [y1, y2, y3, y4]);
    }
    var cubics = divideCubic(x1, y1, x2, y2, x3, y3, x4, y4, 0.5);
    var left = [].concat(cubics[0], [iterationCount - 1]);
    var right = [].concat(cubics[1], [iterationCount - 1]);
    return cubicLength.apply(void 0, left) + cubicLength.apply(void 0, right);
  }
  var cubic = {
    extrema: extrema$1,
    box: function box(x1, y1, x2, y2, x3, y3, x4, y4) {
      var xArr = [x1, x4];
      var yArr = [y1, y4];
      var xExtrema = extrema$1(x1, x2, x3, x4);
      var yExtrema = extrema$1(y1, y2, y3, y4);
      for (var i = 0; i < xExtrema.length; i++) {
        xArr.push(cubicAt(x1, x2, x3, x4, xExtrema[i]));
      }
      for (var _i = 0; _i < yExtrema.length; _i++) {
        yArr.push(cubicAt(y1, y2, y3, y4, yExtrema[_i]));
      }
      return getBBoxByArray(xArr, yArr);
    },
    length: function length(x1, y1, x2, y2, x3, y3, x4, y4) {
      // 迭代三次，划分成 8 段求长度
      return cubicLength(x1, y1, x2, y2, x3, y3, x4, y4, 3);
    },
    nearestPoint: function nearestPoint$1(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length) {
      return nearestPoint([x1, x2, x3, x4], [y1, y2, y3, y4], x0, y0, cubicAt, length);
    },
    pointDistance: function pointDistance(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length) {
      var point = this.nearestPoint(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length);
      return distance$1(point.x, point.y, x0, y0);
    },
    interpolationAt: cubicAt,
    pointAt: function pointAt(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      return {
        x: cubicAt(x1, x2, x3, x4, t),
        y: cubicAt(y1, y2, y3, y4, t)
      };
    },
    divide: function divide(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      return divideCubic(x1, y1, x2, y2, x3, y3, x4, y4, t);
    },
    tangentAngle: function tangentAngle(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      var dx = derivativeAt$1(x1, x2, x3, x4, t);
      var dy = derivativeAt$1(y1, y2, y3, y4, t);
      return piMod(Math.atan2(dy, dx));
    }
  };

  /**
   * @fileoverview 椭圆的一些计算，
   *  - 周长计算参考：https://www.mathsisfun.com/geometry/ellipse-perimeter.html
   *  - 距离计算参考：https://wet-robots.ghost.io/simple-method-for-distance-to-ellipse/
   * @author dxq613@gmail.com
   */
  function copysign(v1, v2) {
    var absv = Math.abs(v1);
    return v2 > 0 ? absv : absv * -1;
  }
  var ellipse = {
    /**
     * 包围盒计算
     * @param {number} x  椭圆中心 x
     * @param {number} y  椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @return {object} 包围盒
     */
    box: function box(x, y, rx, ry) {
      return {
        x: x - rx,
        y: y - ry,
        width: rx * 2,
        height: ry * 2
      };
    },
    /**
     * 计算周长，使用近似法
     * @param {number} x  椭圆中心 x
     * @param {number} y  椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @return {number} 椭圆周长
     */
    length: function length(x, y, rx, ry) {
      return Math.PI * (3 * (rx + ry) - Math.sqrt((3 * rx + ry) * (rx + 3 * ry)));
    },
    /**
     * 距离椭圆最近的点
     * @param {number} x  椭圆中心 x
     * @param {number} y  椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @param {number} x0  指定的点 x
     * @param {number} y0  指定的点 y
     * @return {object} 椭圆上距离指定点最近的点
     */
    nearestPoint: function nearestPoint(x, y, rx, ry, x0, y0) {
      var a = rx;
      var b = ry;
      // 假如椭圆半径为0则返回圆心
      if (a === 0 || b === 0) {
        return {
          x: x,
          y: y
        };
      }
      // 转换成 0， 0 为中心的椭圆计算
      var relativeX = x0 - x;
      var relativeY = y0 - y;
      var px = Math.abs(relativeX);
      var py = Math.abs(relativeY);
      var squareA = a * a;
      var squareB = b * b;
      // const angle0 = Math.atan2(relativeY, relativeX);
      var t = Math.PI / 4;
      var nearestX = 0; // 椭圆上的任一点
      var nearestY = 0;
      // 迭代 4 次
      for (var i = 0; i < 4; i++) {
        nearestX = a * Math.cos(t);
        nearestY = b * Math.sin(t);
        var ex = (squareA - squareB) * Math.pow(Math.cos(t), 3) / a;
        var ey = (squareB - squareA) * Math.pow(Math.sin(t), 3) / b;
        var rx1 = nearestX - ex;
        var ry1 = nearestY - ey;
        var qx = px - ex;
        var qy = py - ey;
        var r = Math.hypot(ry1, rx1);
        var q = Math.hypot(qy, qx);
        var delta_c = r * Math.asin((rx1 * qy - ry1 * qx) / (r * q));
        var delta_t = delta_c / Math.sqrt(squareA + squareB - nearestX * nearestX - nearestY * nearestY);
        t += delta_t;
        t = Math.min(Math.PI / 2, Math.max(0, t));
      }
      return {
        x: x + copysign(nearestX, relativeX),
        y: y + copysign(nearestY, relativeY)
      };
    },
    /**
     * 点到椭圆最近的距离
     * @param {number} x  椭圆中心 x
     * @param {number} y  椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @param {number} x0  指定的点 x
     * @param {number} y0  指定的点 y
     * @return {number} 点到椭圆的距离
     */
    pointDistance: function pointDistance(x, y, rx, ry, x0, y0) {
      var nearestPoint = this.nearestPoint(x, y, rx, ry, x0, y0);
      return distance$1(nearestPoint.x, nearestPoint.y, x0, y0);
    },
    /**
     * 根据比例获取点
     * @param {number} x 椭圆中心 x
     * @param {number} y 椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @param {number} t 指定比例，x轴方向为 0
     * @return {object} 点
     */
    pointAt: function pointAt(x, y, rx, ry, t) {
      var angle = 2 * Math.PI * t; // 按照角度进行计算，而不按照周长计算
      return {
        x: x + rx * Math.cos(angle),
        y: y + ry * Math.sin(angle)
      };
    },
    /**
     * 根据比例计算切线角度
     * @param {number} x 椭圆中心 x
     * @param {number} y 椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @param {number} t 指定比例 0 - 1 之间，x轴方向为 0。在 0-1 范围之外是循环还是返回 null，还需要调整
     * @return {number} 角度，在 0 - 2PI 之间
     */
    tangentAngle: function tangentAngle(x, y, rx, ry, t) {
      var angle = 2 * Math.PI * t; // 按照角度进行计算，而不按照周长计算
      // 直接使用 x,y 的导数计算， x' = -rx * sin(t); y' = ry * cos(t);
      var tangentAngle = Math.atan2(ry * Math.cos(angle), -rx * Math.sin(angle));
      // 也可以使用指定点的切线方程计算，成本有些高
      // const point = this.pointAt(0, 0, rx, ry, t); // 椭圆的切线同椭圆的中心不相关
      // let tangentAngle = -1 * Math.atan((ry * ry * point.x) / (rx * rx * point.y));
      // if (angle >= 0 && angle <= Math.PI) {
      //   tangentAngle += Math.PI;
      // }
      return piMod(tangentAngle);
    }
  };

  // 偏导数 x
  function derivativeXAt(cx, cy, rx, ry, xRotation, startAngle, endAngle, angle) {
    return -1 * rx * Math.cos(xRotation) * Math.sin(angle) - ry * Math.sin(xRotation) * Math.cos(angle);
  }
  // 偏导数 y
  function derivativeYAt(cx, cy, rx, ry, xRotation, startAngle, endAngle, angle) {
    return -1 * rx * Math.sin(xRotation) * Math.sin(angle) + ry * Math.cos(xRotation) * Math.cos(angle);
  }
  // x 的极值
  function xExtrema(rx, ry, xRotation) {
    return Math.atan(-ry / rx * Math.tan(xRotation));
  }
  // y 的极值
  function yExtrema(rx, ry, xRotation) {
    return Math.atan(ry / (rx * Math.tan(xRotation)));
  }
  // 根据角度求 x 坐标
  function xAt(cx, cy, rx, ry, xRotation, angle) {
    return rx * Math.cos(xRotation) * Math.cos(angle) - ry * Math.sin(xRotation) * Math.sin(angle) + cx;
  }
  // 根据角度求 y 坐标
  function yAt(cx, cy, rx, ry, xRotation, angle) {
    return rx * Math.sin(xRotation) * Math.cos(angle) + ry * Math.cos(xRotation) * Math.sin(angle) + cy;
  }
  // 获取点在椭圆上的角度
  function getAngle(rx, ry, x0, y0) {
    var angle = Math.atan2(y0 * rx, x0 * ry);
    // 转换到 0 - 2PI 内
    return (angle + Math.PI * 2) % (Math.PI * 2);
  }
  // 根据角度获取，x,y
  function getPoint(rx, ry, angle) {
    return {
      x: rx * Math.cos(angle),
      y: ry * Math.sin(angle)
    };
  }
  // 旋转
  function rotate$1(x, y, angle) {
    var cos = Math.cos(angle);
    var sin = Math.sin(angle);
    return [x * cos - y * sin, x * sin + y * cos];
  }
  var arc = {
    /**
     * 计算包围盒
     * @param {number} cx         圆心 x
     * @param {number} cy         圆心 y
     * @param {number} rx         x 轴方向的半径
     * @param {number} ry         y 轴方向的半径
     * @param {number} xRotation  旋转角度
     * @param {number} startAngle 起始角度
     * @param {number} endAngle   结束角度
     * @return {object} 包围盒对象
     */
    box: function box(cx, cy, rx, ry, xRotation, startAngle, endAngle) {
      var xDim = xExtrema(rx, ry, xRotation);
      var minX = Infinity;
      var maxX = -Infinity;
      var xs = [startAngle, endAngle];
      for (var i = -Math.PI * 2; i <= Math.PI * 2; i += Math.PI) {
        var xAngle = xDim + i;
        if (startAngle < endAngle) {
          if (startAngle < xAngle && xAngle < endAngle) {
            xs.push(xAngle);
          }
        } else {
          if (endAngle < xAngle && xAngle < startAngle) {
            xs.push(xAngle);
          }
        }
      }
      for (var _i = 0; _i < xs.length; _i++) {
        var x = xAt(cx, cy, rx, ry, xRotation, xs[_i]);
        if (x < minX) {
          minX = x;
        }
        if (x > maxX) {
          maxX = x;
        }
      }
      var yDim = yExtrema(rx, ry, xRotation);
      var minY = Infinity;
      var maxY = -Infinity;
      var ys = [startAngle, endAngle];
      for (var _i2 = -Math.PI * 2; _i2 <= Math.PI * 2; _i2 += Math.PI) {
        var yAngle = yDim + _i2;
        if (startAngle < endAngle) {
          if (startAngle < yAngle && yAngle < endAngle) {
            ys.push(yAngle);
          }
        } else {
          if (endAngle < yAngle && yAngle < startAngle) {
            ys.push(yAngle);
          }
        }
      }
      for (var _i3 = 0; _i3 < ys.length; _i3++) {
        var y = yAt(cx, cy, rx, ry, xRotation, ys[_i3]);
        if (y < minY) {
          minY = y;
        }
        if (y > maxY) {
          maxY = y;
        }
      }
      return {
        x: minX,
        y: minY,
        width: maxX - minX,
        height: maxY - minY
      };
    },
    /**
     * 获取圆弧的长度，计算圆弧长度时不考虑旋转角度，
     * 仅跟 rx, ry, startAngle, endAngle 相关
     * @param {number} cx         圆心 x
     * @param {number} cy         圆心 y
     * @param {number} rx         x 轴方向的半径
     * @param {number} ry         y 轴方向的半径
     * @param {number} xRotation  旋转角度
     * @param {number} startAngle 起始角度
     * @param {number} endAngle   结束角度
     */
    length: function length(cx, cy, rx, ry, xRotation, startAngle, endAngle) {},
    /**
     * 获取指定点到圆弧的最近距离的点
     * @param {number} cx         圆心 x
     * @param {number} cy         圆心 y
     * @param {number} rx         x 轴方向的半径
     * @param {number} ry         y 轴方向的半径
     * @param {number} xRotation  旋转角度
     * @param {number} startAngle 起始角度
     * @param {number} endAngle   结束角度
     * @param {number} x0         指定点的 x
     * @param {number} y0         指定点的 y
     * @return {object} 到指定点最近距离的点
     */
    nearestPoint: function nearestPoint(cx, cy, rx, ry, xRotation, startAngle, endAngle, x0, y0) {
      // 将最近距离问题转换成到椭圆中心 0,0 没有旋转的椭圆问题
      var relativeVector = rotate$1(x0 - cx, y0 - cy, -xRotation);
      var x1 = relativeVector[0],
        y1 = relativeVector[1];
      // 计算点到椭圆的最近的点
      var relativePoint = ellipse.nearestPoint(0, 0, rx, ry, x1, y1);
      // 获取点在椭圆上的角度
      var angle = getAngle(rx, ry, relativePoint.x, relativePoint.y);
      // 点没有在圆弧上
      if (angle < startAngle) {
        // 小于起始圆弧
        relativePoint = getPoint(rx, ry, startAngle);
      } else if (angle > endAngle) {
        // 大于结束圆弧
        relativePoint = getPoint(rx, ry, endAngle);
      }
      // 旋转到 xRotation 的角度
      var vector = rotate$1(relativePoint.x, relativePoint.y, xRotation);
      return {
        x: vector[0] + cx,
        y: vector[1] + cy
      };
    },
    pointDistance: function pointDistance(cx, cy, rx, ry, xRotation, startAngle, endAngle, x0, y0) {
      var nearestPoint = this.nearestPoint(cx, cy, rx, ry, xRotation, startAngle, endAngle, x0, y0);
      return distance$1(nearestPoint.x, nearestPoint.y, x0, y0);
    },
    pointAt: function pointAt(cx, cy, rx, ry, xRotation, startAngle, endAngle, t) {
      var angle = (endAngle - startAngle) * t + startAngle;
      return {
        x: xAt(cx, cy, rx, ry, xRotation, angle),
        y: yAt(cx, cy, rx, ry, xRotation, angle)
      };
    },
    tangentAngle: function tangentAngle(cx, cy, rx, ry, xRotation, startAngle, endAngle, t) {
      var angle = (endAngle - startAngle) * t + startAngle;
      var dx = derivativeXAt(cx, cy, rx, ry, xRotation, startAngle, endAngle, angle);
      var dy = derivativeYAt(cx, cy, rx, ry, xRotation, startAngle, endAngle, angle);
      return piMod(Math.atan2(dy, dx));
    }
  };

  function analyzePoints(points) {
    // 计算每段的长度和总的长度
    var totalLength = 0;
    var segments = [];
    for (var i = 0; i < points.length - 1; i++) {
      var from = points[i];
      var to = points[i + 1];
      var length = distance$1(from[0], from[1], to[0], to[1]);
      var seg = {
        from: from,
        to: to,
        length: length
      };
      segments.push(seg);
      totalLength += length;
    }
    return {
      segments: segments,
      totalLength: totalLength
    };
  }
  function lengthOfSegment(points) {
    if (points.length < 2) {
      return 0;
    }
    var totalLength = 0;
    for (var i = 0; i < points.length - 1; i++) {
      var from = points[i];
      var to = points[i + 1];
      totalLength += distance$1(from[0], from[1], to[0], to[1]);
    }
    return totalLength;
  }
  /**
   * 按照比例在数据片段中获取点
   * @param {array} points 点的集合
   * @param {number} t 百分比 0-1
   * @return {object} 点的坐标
   */
  function pointAtSegments(points, t) {
    // 边界判断
    if (t > 1 || t < 0 || points.length < 2) {
      return null;
    }
    var _analyzePoints = analyzePoints(points),
      segments = _analyzePoints.segments,
      totalLength = _analyzePoints.totalLength;
    // 多个点有可能重合
    if (totalLength === 0) {
      return {
        x: points[0][0],
        y: points[0][1]
      };
    }
    // 计算比例
    var startRatio = 0;
    var point = null;
    for (var i = 0; i < segments.length; i++) {
      var seg = segments[i];
      var from = seg.from,
        to = seg.to;
      var currentRatio = seg.length / totalLength;
      if (t >= startRatio && t <= startRatio + currentRatio) {
        var localRatio = (t - startRatio) / currentRatio;
        point = line.pointAt(from[0], from[1], to[0], to[1], localRatio);
        break;
      }
      startRatio += currentRatio;
    }
    return point;
  }
  /**
   * 按照比例在数据片段中获取切线的角度
   * @param {array} points 点的集合
   * @param {number} t 百分比 0-1
   */
  function angleAtSegments(points, t) {
    // 边界判断
    if (t > 1 || t < 0 || points.length < 2) {
      return 0;
    }
    var _analyzePoints2 = analyzePoints(points),
      segments = _analyzePoints2.segments,
      totalLength = _analyzePoints2.totalLength;
    // 计算比例
    var startRatio = 0;
    var angle = 0;
    for (var i = 0; i < segments.length; i++) {
      var seg = segments[i];
      var from = seg.from,
        to = seg.to;
      var currentRatio = seg.length / totalLength;
      if (t >= startRatio && t <= startRatio + currentRatio) {
        angle = Math.atan2(to[1] - from[1], to[0] - from[0]);
        break;
      }
      startRatio += currentRatio;
    }
    return angle;
  }
  function distanceAtSegment(points, x, y) {
    var minDistance = Infinity;
    for (var i = 0; i < points.length - 1; i++) {
      var point = points[i];
      var nextPoint = points[i + 1];
      var _distance = line.pointDistance(point[0], point[1], nextPoint[0], nextPoint[1], x, y);
      if (_distance < minDistance) {
        minDistance = _distance;
      }
    }
    return minDistance;
  }

  var polyline = {
    /**
     * 计算多折线的包围盒
     * @param {array} points 点的集合 [x,y] 的形式
     * @return {object} 包围盒
     */
    box: function box(points) {
      var xArr = [];
      var yArr = [];
      for (var i = 0; i < points.length; i++) {
        var point = points[i];
        xArr.push(point[0]);
        yArr.push(point[1]);
      }
      return getBBoxByArray(xArr, yArr);
    },
    /**
     * 计算多折线的长度
     * @param {array} points 点的集合 [x,y] 的形式
     * @return {object} 多条边的长度
     */
    length: function length(points) {
      return lengthOfSegment(points);
    },
    /**
     * 根据比例获取多折线的点
     * @param {array} points 点的集合 [x,y] 的形式
     * @param {number} t 在多折线的长度上的比例
     * @return {object} 根据比例值计算出来的点
     */
    pointAt: function pointAt(points, t) {
      return pointAtSegments(points, t);
    },
    /**
     * 指定点到多折线的距离
     * @param {array} points 点的集合 [x,y] 的形式
     * @param {number} x 指定点的 x
     * @param {number} y 指定点的 y
     * @return {number} 点到多折线的距离
     */
    pointDistance: function pointDistance(points, x, y) {
      return distanceAtSegment(points, x, y);
    },
    /**
     * 根据比例获取多折线的切线角度
     * @param {array} points 点的集合 [x,y] 的形式
     * @param {number} t 在多折线的长度上的比例
     * @return {object} 根据比例值计算出来的角度
     */
    tangentAngle: function tangentAngle(points, t) {
      return angleAtSegments(points, t);
    }
  };

  function getDefaultExportFromCjs (x) {
  	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
  }

  function createCommonjsModule(fn, basedir, module) {
  	return module = {
  		path: basedir,
  		exports: {},
  		require: function (path, base) {
  			return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
  		}
  	}, fn(module, module.exports), module.exports;
  }

  function commonjsRequire () {
  	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
  }

  var eventemitter3 = createCommonjsModule(function (module) {

  var has = Object.prototype.hasOwnProperty
    , prefix = '~';

  /**
   * Constructor to create a storage for our `EE` objects.
   * An `Events` instance is a plain object whose properties are event names.
   *
   * @constructor
   * @private
   */
  function Events() {}

  //
  // We try to not inherit from `Object.prototype`. In some engines creating an
  // instance in this way is faster than calling `Object.create(null)` directly.
  // If `Object.create(null)` is not supported we prefix the event names with a
  // character to make sure that the built-in object properties are not
  // overridden or used as an attack vector.
  //
  if (Object.create) {
    Events.prototype = Object.create(null);

    //
    // This hack is needed because the `__proto__` property is still inherited in
    // some old browsers like Android 4, iPhone 5.1, Opera 11 and Safari 5.
    //
    if (!new Events().__proto__) prefix = false;
  }

  /**
   * Representation of a single event listener.
   *
   * @param {Function} fn The listener function.
   * @param {*} context The context to invoke the listener with.
   * @param {Boolean} [once=false] Specify if the listener is a one-time listener.
   * @constructor
   * @private
   */
  function EE(fn, context, once) {
    this.fn = fn;
    this.context = context;
    this.once = once || false;
  }

  /**
   * Add a listener for a given event.
   *
   * @param {EventEmitter} emitter Reference to the `EventEmitter` instance.
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} context The context to invoke the listener with.
   * @param {Boolean} once Specify if the listener is a one-time listener.
   * @returns {EventEmitter}
   * @private
   */
  function addListener(emitter, event, fn, context, once) {
    if (typeof fn !== 'function') {
      throw new TypeError('The listener must be a function');
    }

    var listener = new EE(fn, context || emitter, once)
      , evt = prefix ? prefix + event : event;

    if (!emitter._events[evt]) emitter._events[evt] = listener, emitter._eventsCount++;
    else if (!emitter._events[evt].fn) emitter._events[evt].push(listener);
    else emitter._events[evt] = [emitter._events[evt], listener];

    return emitter;
  }

  /**
   * Clear event by name.
   *
   * @param {EventEmitter} emitter Reference to the `EventEmitter` instance.
   * @param {(String|Symbol)} evt The Event name.
   * @private
   */
  function clearEvent(emitter, evt) {
    if (--emitter._eventsCount === 0) emitter._events = new Events();
    else delete emitter._events[evt];
  }

  /**
   * Minimal `EventEmitter` interface that is molded against the Node.js
   * `EventEmitter` interface.
   *
   * @constructor
   * @public
   */
  function EventEmitter() {
    this._events = new Events();
    this._eventsCount = 0;
  }

  /**
   * Return an array listing the events for which the emitter has registered
   * listeners.
   *
   * @returns {Array}
   * @public
   */
  EventEmitter.prototype.eventNames = function eventNames() {
    var names = []
      , events
      , name;

    if (this._eventsCount === 0) return names;

    for (name in (events = this._events)) {
      if (has.call(events, name)) names.push(prefix ? name.slice(1) : name);
    }

    if (Object.getOwnPropertySymbols) {
      return names.concat(Object.getOwnPropertySymbols(events));
    }

    return names;
  };

  /**
   * Return the listeners registered for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Array} The registered listeners.
   * @public
   */
  EventEmitter.prototype.listeners = function listeners(event) {
    var evt = prefix ? prefix + event : event
      , handlers = this._events[evt];

    if (!handlers) return [];
    if (handlers.fn) return [handlers.fn];

    for (var i = 0, l = handlers.length, ee = new Array(l); i < l; i++) {
      ee[i] = handlers[i].fn;
    }

    return ee;
  };

  /**
   * Return the number of listeners listening to a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Number} The number of listeners.
   * @public
   */
  EventEmitter.prototype.listenerCount = function listenerCount(event) {
    var evt = prefix ? prefix + event : event
      , listeners = this._events[evt];

    if (!listeners) return 0;
    if (listeners.fn) return 1;
    return listeners.length;
  };

  /**
   * Calls each of the listeners registered for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Boolean} `true` if the event had listeners, else `false`.
   * @public
   */
  EventEmitter.prototype.emit = function emit(event, a1, a2, a3, a4, a5) {
    var evt = prefix ? prefix + event : event;

    if (!this._events[evt]) return false;

    var listeners = this._events[evt]
      , len = arguments.length
      , args
      , i;

    if (listeners.fn) {
      if (listeners.once) this.removeListener(event, listeners.fn, undefined, true);

      switch (len) {
        case 1: return listeners.fn.call(listeners.context), true;
        case 2: return listeners.fn.call(listeners.context, a1), true;
        case 3: return listeners.fn.call(listeners.context, a1, a2), true;
        case 4: return listeners.fn.call(listeners.context, a1, a2, a3), true;
        case 5: return listeners.fn.call(listeners.context, a1, a2, a3, a4), true;
        case 6: return listeners.fn.call(listeners.context, a1, a2, a3, a4, a5), true;
      }

      for (i = 1, args = new Array(len -1); i < len; i++) {
        args[i - 1] = arguments[i];
      }

      listeners.fn.apply(listeners.context, args);
    } else {
      var length = listeners.length
        , j;

      for (i = 0; i < length; i++) {
        if (listeners[i].once) this.removeListener(event, listeners[i].fn, undefined, true);

        switch (len) {
          case 1: listeners[i].fn.call(listeners[i].context); break;
          case 2: listeners[i].fn.call(listeners[i].context, a1); break;
          case 3: listeners[i].fn.call(listeners[i].context, a1, a2); break;
          case 4: listeners[i].fn.call(listeners[i].context, a1, a2, a3); break;
          default:
            if (!args) for (j = 1, args = new Array(len -1); j < len; j++) {
              args[j - 1] = arguments[j];
            }

            listeners[i].fn.apply(listeners[i].context, args);
        }
      }
    }

    return true;
  };

  /**
   * Add a listener for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} [context=this] The context to invoke the listener with.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.on = function on(event, fn, context) {
    return addListener(this, event, fn, context, false);
  };

  /**
   * Add a one-time listener for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} [context=this] The context to invoke the listener with.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.once = function once(event, fn, context) {
    return addListener(this, event, fn, context, true);
  };

  /**
   * Remove the listeners of a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn Only remove the listeners that match this function.
   * @param {*} context Only remove the listeners that have this context.
   * @param {Boolean} once Only remove one-time listeners.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.removeListener = function removeListener(event, fn, context, once) {
    var evt = prefix ? prefix + event : event;

    if (!this._events[evt]) return this;
    if (!fn) {
      clearEvent(this, evt);
      return this;
    }

    var listeners = this._events[evt];

    if (listeners.fn) {
      if (
        listeners.fn === fn &&
        (!once || listeners.once) &&
        (!context || listeners.context === context)
      ) {
        clearEvent(this, evt);
      }
    } else {
      for (var i = 0, events = [], length = listeners.length; i < length; i++) {
        if (
          listeners[i].fn !== fn ||
          (once && !listeners[i].once) ||
          (context && listeners[i].context !== context)
        ) {
          events.push(listeners[i]);
        }
      }

      //
      // Reset the array, or remove it completely if we have no more listeners.
      //
      if (events.length) this._events[evt] = events.length === 1 ? events[0] : events;
      else clearEvent(this, evt);
    }

    return this;
  };

  /**
   * Remove all listeners, or those of the specified event.
   *
   * @param {(String|Symbol)} [event] The event name.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.removeAllListeners = function removeAllListeners(event) {
    var evt;

    if (event) {
      evt = prefix ? prefix + event : event;
      if (this._events[evt]) clearEvent(this, evt);
    } else {
      this._events = new Events();
      this._eventsCount = 0;
    }

    return this;
  };

  //
  // Alias methods names because people roll like that.
  //
  EventEmitter.prototype.off = EventEmitter.prototype.removeListener;
  EventEmitter.prototype.addListener = EventEmitter.prototype.on;

  //
  // Expose the prefix.
  //
  EventEmitter.prefixed = prefix;

  //
  // Allow `EventEmitter` to be imported as module namespace.
  //
  EventEmitter.EventEmitter = EventEmitter;

  //
  // Expose the module.
  //
  {
    module.exports = EventEmitter;
  }
  });

  let uId=1;const root="object"==typeof self&&self.self==self?self:"object"==typeof global&&global.global==global?global:{},nowOffset=Date.now();let reservedCBs={},lastTime=Date.now();const polyfillRaf=e=>{if("function"!=typeof e)throw new TypeError(e+" is not a function");const o=Date.now(),t=o-lastTime,n=t>16?0:16-t,r=uId++;return reservedCBs[r]=e,Object.keys(reservedCBs).length>1||setTimeout((()=>{lastTime=o;const e=reservedCBs;reservedCBs={},Object.keys(e).forEach((o=>e[o](root.performance&&"function"==typeof root.performance.now?root.performance.now():Date.now()-nowOffset)));}),n),r},polyfillCaf=e=>{delete reservedCBs[e];},vendorPrefixes=["","webkit","moz","ms","o"],getRequestAnimationFrame=e=>"string"!=typeof e?polyfillRaf:""===e?root.requestAnimationFrame:root[e+"RequestAnimationFrame"],getCancelAnimationFrame=e=>"string"!=typeof e?polyfillCaf:""===e?root.cancelAnimationFrame:root[e+"CancelAnimationFrame"]||root[e+"CancelRequestAnimationFrame"],find=(e,o)=>{let t=0;for(;void 0!==e[t];){if(o(e[t]))return e[t];t+=1;}},vp=find(vendorPrefixes,(e=>!!getRequestAnimationFrame(e))),raf=getRequestAnimationFrame(vp),caf=getCancelAnimationFrame(vp);root.requestAnimationFrame=raf,root.cancelAnimationFrame=caf,"undefined"!=typeof module&&module.exports&&(module.exports={requestAnimationFrame:raf,cancelAnimationFrame:caf});

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type
   */
  var PropertySyntax;
  (function (PropertySyntax) {
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#coordinate
     */
    PropertySyntax["COORDINATE"] = "<coordinate>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#color
     */
    PropertySyntax["COLOR"] = "<color>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#paint
     */
    PropertySyntax["PAINT"] = "<paint>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#number
     */
    PropertySyntax["NUMBER"] = "<number>";
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/angle
     */
    PropertySyntax["ANGLE"] = "<angle>";
    /**
     * <number> with range 0..1
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#opacity_value
     */
    PropertySyntax["OPACITY_VALUE"] = "<opacity-value>";
    /**
     * <number> with range 0..Infinity
     */
    PropertySyntax["SHADOW_BLUR"] = "<shadow-blur>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#length
     */
    PropertySyntax["LENGTH"] = "<length>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#percentage
     */
    PropertySyntax["PERCENTAGE"] = "<percentage>";
    PropertySyntax["LENGTH_PERCENTAGE"] = "<length> | <percentage>";
    PropertySyntax["LENGTH_PERCENTAGE_12"] = "[<length> | <percentage>]{1,2}";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/CSS/margin#formal_syntax
     */
    PropertySyntax["LENGTH_PERCENTAGE_14"] = "[<length> | <percentage>]{1,4}";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#list-of-ts
     */
    PropertySyntax["LIST_OF_POINTS"] = "<list-of-points>";
    PropertySyntax["PATH"] = "<path>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/CSS/filter#formal_syntax
     */
    PropertySyntax["FILTER"] = "<filter>";
    PropertySyntax["Z_INDEX"] = "<z-index>";
    PropertySyntax["OFFSET_DISTANCE"] = "<offset-distance>";
    PropertySyntax["DEFINED_PATH"] = "<defined-path>";
    PropertySyntax["MARKER"] = "<marker>";
    PropertySyntax["TRANSFORM"] = "<transform>";
    PropertySyntax["TRANSFORM_ORIGIN"] = "<transform-origin>";
    PropertySyntax["TEXT"] = "<text>";
    PropertySyntax["TEXT_TRANSFORM"] = "<text-transform>";
  })(PropertySyntax || (PropertySyntax = {}));

  /**
   * Axis-Aligned Bounding Box
   * 为了便于后续 Frustum Culling，通过查找表定义 p-vertex 和 n-vertex
   * @see https://github.com/antvis/GWebGPUEngine/issues/3
   */
  var AABB = /*#__PURE__*/function () {
    AABB.isEmpty = function isEmpty(aabb) {
      return !aabb || aabb.halfExtents[0] === 0 && aabb.halfExtents[1] === 0 && aabb.halfExtents[2] === 0;
    };
    function AABB(center, halfExtents) {
      if (center === void 0) {
        center = [0, 0, 0];
      }
      if (halfExtents === void 0) {
        halfExtents = [0, 0, 0];
      }
      this.center = create$2();
      this.halfExtents = create$2();
      this.min = create$2();
      this.max = create$2();
      this.update(center, halfExtents);
    }
    var _proto = AABB.prototype;
    _proto.update = function update(center, halfExtents) {
      copy$1(this.center, center);
      copy$1(this.halfExtents, halfExtents);
      sub$1(this.min, this.center, this.halfExtents);
      add$1(this.max, this.center, this.halfExtents);
    };
    _proto.setMinMax = function setMinMax(min, max) {
      add$1(this.center, max, min);
      scale$1(this.center, this.center, 0.5);
      sub$1(this.halfExtents, max, min);
      scale$1(this.halfExtents, this.halfExtents, 0.5);
      copy$1(this.min, min);
      copy$1(this.max, max);
    };
    _proto.getMin = function getMin() {
      return this.min;
    };
    _proto.getMax = function getMax() {
      return this.max;
    };
    _proto.add = function add(aabb) {
      if (AABB.isEmpty(aabb)) {
        return;
      }
      if (AABB.isEmpty(this)) {
        this.setMinMax(aabb.getMin(), aabb.getMax());
        return;
      }
      var tc = this.center;
      var tcx = tc[0];
      var tcy = tc[1];
      var tcz = tc[2];
      var th = this.halfExtents;
      var thx = th[0];
      var thy = th[1];
      var thz = th[2];
      var tminx = tcx - thx;
      var tmaxx = tcx + thx;
      var tminy = tcy - thy;
      var tmaxy = tcy + thy;
      var tminz = tcz - thz;
      var tmaxz = tcz + thz;
      var oc = aabb.center;
      var ocx = oc[0];
      var ocy = oc[1];
      var ocz = oc[2];
      var oh = aabb.halfExtents;
      var ohx = oh[0];
      var ohy = oh[1];
      var ohz = oh[2];
      var ominx = ocx - ohx;
      var omaxx = ocx + ohx;
      var ominy = ocy - ohy;
      var omaxy = ocy + ohy;
      var ominz = ocz - ohz;
      var omaxz = ocz + ohz;
      if (ominx < tminx) {
        tminx = ominx;
      }
      if (omaxx > tmaxx) {
        tmaxx = omaxx;
      }
      if (ominy < tminy) {
        tminy = ominy;
      }
      if (omaxy > tmaxy) {
        tmaxy = omaxy;
      }
      if (ominz < tminz) {
        tminz = ominz;
      }
      if (omaxz > tmaxz) {
        tmaxz = omaxz;
      }
      tc[0] = (tminx + tmaxx) * 0.5;
      tc[1] = (tminy + tmaxy) * 0.5;
      tc[2] = (tminz + tmaxz) * 0.5;
      th[0] = (tmaxx - tminx) * 0.5;
      th[1] = (tmaxy - tminy) * 0.5;
      th[2] = (tmaxz - tminz) * 0.5;
      this.min[0] = tminx;
      this.min[1] = tminy;
      this.min[2] = tminz;
      this.max[0] = tmaxx;
      this.max[1] = tmaxy;
      this.max[2] = tmaxz;
    };
    _proto.setFromTransformedAABB = function setFromTransformedAABB(aabb, m) {
      var bc = this.center;
      var br = this.halfExtents;
      var ac = aabb.center;
      var ar = aabb.halfExtents;
      var mx0 = m[0];
      var mx1 = m[4];
      var mx2 = m[8];
      var my0 = m[1];
      var my1 = m[5];
      var my2 = m[9];
      var mz0 = m[2];
      var mz1 = m[6];
      var mz2 = m[10];
      var mx0a = Math.abs(mx0);
      var mx1a = Math.abs(mx1);
      var mx2a = Math.abs(mx2);
      var my0a = Math.abs(my0);
      var my1a = Math.abs(my1);
      var my2a = Math.abs(my2);
      var mz0a = Math.abs(mz0);
      var mz1a = Math.abs(mz1);
      var mz2a = Math.abs(mz2);
      set$1(bc, m[12] + mx0 * ac[0] + mx1 * ac[1] + mx2 * ac[2], m[13] + my0 * ac[0] + my1 * ac[1] + my2 * ac[2], m[14] + mz0 * ac[0] + mz1 * ac[1] + mz2 * ac[2]);
      set$1(br, mx0a * ar[0] + mx1a * ar[1] + mx2a * ar[2], my0a * ar[0] + my1a * ar[1] + my2a * ar[2], mz0a * ar[0] + mz1a * ar[1] + mz2a * ar[2]);
      this.min = sub$1(this.min, bc, br);
      this.max = add$1(this.max, bc, br);
    };
    _proto.intersects = function intersects(aabb) {
      var aMax = this.getMax();
      var aMin = this.getMin();
      var bMax = aabb.getMax();
      var bMin = aabb.getMin();
      return aMin[0] <= bMax[0] && aMax[0] >= bMin[0] && aMin[1] <= bMax[1] && aMax[1] >= bMin[1] && aMin[2] <= bMax[2] && aMax[2] >= bMin[2];
    };
    _proto.intersection = function intersection(aabb) {
      if (!this.intersects(aabb)) {
        return null;
      }
      var intersection = new AABB();
      var min$1 = max(create$2(), this.getMin(), aabb.getMin());
      var max$1 = min(create$2(), this.getMax(), aabb.getMax());
      intersection.setMinMax(min$1, max$1);
      return intersection;
    };
    _proto.containsPoint = function containsPoint(point) {
      var min = this.getMin();
      var max = this.getMax();
      return !(point[0] < min[0] || point[0] > max[0] || point[1] < min[1] || point[1] > max[1] || point[2] < min[2] || point[2] > max[2]);
    }
    /**
     * get n-vertex
     * @param plane plane of CullingVolume
     */;
    _proto.getNegativeFarPoint = function getNegativeFarPoint(plane) {
      if (plane.pnVertexFlag === 0x111) {
        return copy$1(create$2(), this.min);
      } else if (plane.pnVertexFlag === 0x110) {
        return fromValues$2(this.min[0], this.min[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x101) {
        return fromValues$2(this.min[0], this.max[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x100) {
        return fromValues$2(this.min[0], this.max[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x011) {
        return fromValues$2(this.max[0], this.min[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x010) {
        return fromValues$2(this.max[0], this.min[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x001) {
        return fromValues$2(this.max[0], this.max[1], this.min[2]);
      } else {
        return fromValues$2(this.max[0], this.max[1], this.max[2]);
      }
    }
    /**
     * get p-vertex
     * @param plane plane of CullingVolume
     */;
    _proto.getPositiveFarPoint = function getPositiveFarPoint(plane) {
      if (plane.pnVertexFlag === 0x111) {
        return copy$1(create$2(), this.max);
      } else if (plane.pnVertexFlag === 0x110) {
        return fromValues$2(this.max[0], this.max[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x101) {
        return fromValues$2(this.max[0], this.min[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x100) {
        return fromValues$2(this.max[0], this.min[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x011) {
        return fromValues$2(this.min[0], this.max[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x010) {
        return fromValues$2(this.min[0], this.max[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x001) {
        return fromValues$2(this.min[0], this.min[1], this.max[2]);
      } else {
        return fromValues$2(this.min[0], this.min[1], this.min[2]);
      }
    };
    return AABB;
  }();

  var Plane = /*#__PURE__*/function () {
    /**
     * lookup table for p-vertex & n-vertex when doing frustum culling
     */

    function Plane(distance, normal) {
      this.distance = void 0;
      this.normal = void 0;
      this.pnVertexFlag = void 0;
      this.distance = distance || 0;
      this.normal = normal || fromValues$2(0, 1, 0);
      this.updatePNVertexFlag();
    }
    var _proto = Plane.prototype;
    _proto.updatePNVertexFlag = function updatePNVertexFlag() {
      this.pnVertexFlag = (Number(this.normal[0] >= 0) << 8) + (Number(this.normal[1] >= 0) << 4) + Number(this.normal[2] >= 0);
    };
    _proto.distanceToPoint = function distanceToPoint(point) {
      return dot(point, this.normal) - this.distance;
    };
    _proto.normalize = function normalize() {
      var invLen = 1 / len(this.normal);
      scale$1(this.normal, this.normal, invLen);
      this.distance *= invLen;
    };
    _proto.intersectsLine = function intersectsLine(start, end, point) {
      var d0 = this.distanceToPoint(start);
      var d1 = this.distanceToPoint(end);
      var t = d0 / (d0 - d1);
      var intersects = t >= 0 && t <= 1;
      if (intersects && point) {
        lerp(point, start, end, t);
      }
      return intersects;
    };
    return Plane;
  }();

  var Mask;
  (function (Mask) {
    Mask[Mask["OUTSIDE"] = 4294967295] = "OUTSIDE";
    Mask[Mask["INSIDE"] = 0] = "INSIDE";
    Mask[Mask["INDETERMINATE"] = 2147483647] = "INDETERMINATE";
  })(Mask || (Mask = {}));
  var Frustum = /*#__PURE__*/function () {
    function Frustum(planes) {
      this.planes = [];
      if (planes) {
        this.planes = planes;
      } else {
        for (var i = 0; i < 6; i++) {
          this.planes.push(new Plane());
        }
      }
    }
    /**
     * extract 6 planes from projectionMatrix
     * @see http://www8.cs.umu.se/kurser/5DV051/HT12/lab/plane_extraction.pdf
     */
    var _proto = Frustum.prototype;
    _proto.extractFromVPMatrix = function extractFromVPMatrix(projectionMatrix) {
      // @ts-ignore
      var m0 = projectionMatrix[0],
        m1 = projectionMatrix[1],
        m2 = projectionMatrix[2],
        m3 = projectionMatrix[3],
        m4 = projectionMatrix[4],
        m5 = projectionMatrix[5],
        m6 = projectionMatrix[6],
        m7 = projectionMatrix[7],
        m8 = projectionMatrix[8],
        m9 = projectionMatrix[9],
        m10 = projectionMatrix[10],
        m11 = projectionMatrix[11],
        m12 = projectionMatrix[12],
        m13 = projectionMatrix[13],
        m14 = projectionMatrix[14],
        m15 = projectionMatrix[15];
      // right
      set$1(this.planes[0].normal, m3 - m0, m7 - m4, m11 - m8);
      this.planes[0].distance = m15 - m12;
      // left
      set$1(this.planes[1].normal, m3 + m0, m7 + m4, m11 + m8);
      this.planes[1].distance = m15 + m12;
      // bottom
      set$1(this.planes[2].normal, m3 + m1, m7 + m5, m11 + m9);
      this.planes[2].distance = m15 + m13;
      // top
      set$1(this.planes[3].normal, m3 - m1, m7 - m5, m11 - m9);
      this.planes[3].distance = m15 - m13;
      // far
      set$1(this.planes[4].normal, m3 - m2, m7 - m6, m11 - m10);
      this.planes[4].distance = m15 - m14;
      // near
      set$1(this.planes[5].normal, m3 + m2, m7 + m6, m11 + m10);
      this.planes[5].distance = m15 + m14;
      this.planes.forEach(function (plane) {
        plane.normalize();
        plane.updatePNVertexFlag();
      });
    };
    return Frustum;
  }();

  var Point = /*#__PURE__*/function () {
    function Point(x, y) {
      if (x === void 0) {
        x = 0;
      }
      if (y === void 0) {
        y = 0;
      }
      this.x = 0;
      this.y = 0;
      this.x = x;
      this.y = y;
    }
    var _proto = Point.prototype;
    _proto.clone = function clone() {
      return new Point(this.x, this.y);
    };
    _proto.copyFrom = function copyFrom(p) {
      this.x = p.x;
      this.y = p.y;
    };
    return Point;
  }();

  var Rectangle = /*#__PURE__*/function () {
    function Rectangle(x, y, width, height) {
      this.x = void 0;
      this.y = void 0;
      this.width = void 0;
      this.height = void 0;
      this.left = void 0;
      this.right = void 0;
      this.top = void 0;
      this.bottom = void 0;
      this.x = x;
      this.y = y;
      this.width = width;
      this.height = height;
      this.left = x;
      this.right = x + width;
      this.top = y;
      this.bottom = y + height;
    }
    var _proto = Rectangle.prototype;
    _proto.toJSON = function toJSON() {};
    return Rectangle;
  }();

  var Shape;
  (function (Shape) {
    Shape["GROUP"] = "g";
    Shape["CIRCLE"] = "circle";
    Shape["ELLIPSE"] = "ellipse";
    Shape["IMAGE"] = "image";
    Shape["RECT"] = "rect";
    Shape["LINE"] = "line";
    Shape["POLYLINE"] = "polyline";
    Shape["POLYGON"] = "polygon";
    Shape["TEXT"] = "text";
    Shape["PATH"] = "path";
    Shape["HTML"] = "html";
    Shape["MESH"] = "mesh";
  })(Shape || (Shape = {}));

  var camelCase = memoize(function (str) {
    if (str === void 0) {
      str = '';
    }
    return str.replace(/-([a-z])/g, function (g) {
      return g[1].toUpperCase();
    });
  });

  function DCHECK(bool) {
    if (!bool) {
      throw new Error();
    }
  }
  function DCHECK_EQ(a, b) {
    if (a !== b) {
      throw new Error();
    }
  }
  function DCHECK_NE(a, b) {
    if (a === b) {
      throw new Error();
    }
  }
  function isFunction$1(func) {
    return typeof func === 'function';
  }
  var FORMAT_ATTR_MAP = {
    d: {
      alias: 'path'
    },
    strokeDasharray: {
      alias: 'lineDash'
    },
    strokeWidth: {
      alias: 'lineWidth'
    },
    textAnchor: {
      alias: 'textAlign'
    },
    src: {
      alias: 'img'
    }
  };
  var formatAttributeName = memoize(function (name) {
    var attributeName = camelCase(name);
    var map = FORMAT_ATTR_MAP[attributeName];
    attributeName = (map === null || map === void 0 ? void 0 : map.alias) || attributeName;
    return attributeName;
  });

  var canvasMap = {};
  var defaultCanvasIdCounter = 0;
  /**
   * destroy existed canvas with the same id
   */
  function cleanExistedCanvas(container, canvas) {
    if (container) {
      var id = typeof container === 'string' ? container : container.id || defaultCanvasIdCounter++;
      if (canvasMap[id]) {
        canvasMap[id].destroy();
      }
      canvasMap[id] = canvas;
    }
  }
  var isBrowser = typeof window !== 'undefined' && typeof window.document !== 'undefined';

  function sortByZIndex(o1, o2) {
    var zIndex1 = Number(o1.style.zIndex);
    var zIndex2 = Number(o2.style.zIndex);
    if (zIndex1 === zIndex2) {
      // return o1.entity.getComponent(Sortable).lastSortedIndex - o2.entity.getComponent(Sortable).lastSortedIndex;
      var parent = o1.parentNode;
      if (parent) {
        var children = parent.childNodes || [];
        return children.indexOf(o1) - children.indexOf(o2);
      }
    }
    return zIndex1 - zIndex2;
  }
  function findClosestClipPathTarget(object) {
    var el = object;
    do {
      var _el$style;
      var clipPath = (_el$style = el.style) === null || _el$style === void 0 ? void 0 : _el$style.clipPath;
      if (clipPath) return el;
      el = el.parentElement;
    } while (el !== null);
    return null;
  }
  function getStyle($el, property) {
    if (isBrowser) {
      return document.defaultView.getComputedStyle($el, null).getPropertyValue(property);
    }
  }
  function getWidth($el) {
    var width = getStyle($el, 'width');
    if (width === 'auto') {
      return $el.offsetWidth;
    }
    return parseFloat(width);
  }
  function getHeight($el) {
    var height = getStyle($el, 'height');
    if (height === 'auto') {
      return $el.offsetHeight;
    }
    return parseFloat(height);
  }

  var ERROR_MSG_METHOD_NOT_IMPLEMENTED = 'Method not implemented.';
  var ERROR_MSG_USE_DOCUMENT_ELEMENT = 'Use document.documentElement instead.';
  var ERROR_MSG_APPEND_DESTROYED_ELEMENT = 'Cannot append a destroyed element.';

  // borrow from hammer.js
  var MOUSE_POINTER_ID = 1;
  var TOUCH_TO_POINTER = {
    touchstart: 'pointerdown',
    touchend: 'pointerup',
    touchendoutside: 'pointerupoutside',
    touchmove: 'pointermove',
    touchcancel: 'pointercancel'
  };

  function _regeneratorRuntime() {
    _regeneratorRuntime = function () {
      return exports;
    };
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function (obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function (method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) keys.push(key);
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function (skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
      },
      stop: function () {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function (exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function (type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function (record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function (finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      catch: function (tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function (iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  function _defineProperties$1(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }
  function _createClass$1(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties$1(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties$1(Constructor, staticProps);
    Object.defineProperty(Constructor, "prototype", {
      writable: false
    });
    return Constructor;
  }
  function _extends() {
    _extends = Object.assign ? Object.assign.bind() : function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];
        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }
      return target;
    };
    return _extends.apply(this, arguments);
  }
  function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass);
  }
  function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf(o, p);
  }
  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }
    return target;
  }
  function _assertThisInitialized(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
  }
  function _unsupportedIterableToArray$1(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray$1(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$1(o, minLen);
  }
  function _arrayLikeToArray$1(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
    return arr2;
  }
  function _createForOfIteratorHelperLoose(o, allowArrayLike) {
    var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
    if (it) return (it = it.call(o)).next.bind(it);
    if (Array.isArray(o) || (it = _unsupportedIterableToArray$1(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }
    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  // These units are iterated through, so be careful when adding or changing the
  // order.
  var UnitType;
  (function (UnitType) {
    UnitType[UnitType["kUnknown"] = 0] = "kUnknown";
    UnitType[UnitType["kNumber"] = 1] = "kNumber";
    UnitType[UnitType["kPercentage"] = 2] = "kPercentage";
    // Length units
    UnitType[UnitType["kEms"] = 3] = "kEms";
    // kExs,
    UnitType[UnitType["kPixels"] = 4] = "kPixels";
    // kCentimeters,
    // kMillimeters,
    // kInches,
    // kPoints,
    // kPicas,
    // kQuarterMillimeters,
    // https://drafts.csswg.org/css-values-4/#viewport-relative-lengths
    //
    // See also IsViewportPercentageLength.
    // kViewportWidth,
    // kViewportHeight,
    // kViewportInlineSize,
    // kViewportBlockSize,
    // kViewportMin,
    // kViewportMax,
    // kSmallViewportWidth,
    // kSmallViewportHeight,
    // kSmallViewportInlineSize,
    // kSmallViewportBlockSize,
    // kSmallViewportMin,
    // kSmallViewportMax,
    // kLargeViewportWidth,
    // kLargeViewportHeight,
    // kLargeViewportInlineSize,
    // kLargeViewportBlockSize,
    // kLargeViewportMin,
    // kLargeViewportMax,
    // kDynamicViewportWidth,
    // kDynamicViewportHeight,
    // kDynamicViewportInlineSize,
    // kDynamicViewportBlockSize,
    // kDynamicViewportMin,
    // kDynamicViewportMax,
    // https://drafts.csswg.org/css-contain-3/#container-lengths
    //
    // See also IsContainerPercentageLength.
    // kContainerWidth,
    // kContainerHeight,
    // kContainerInlineSize,
    // kContainerBlockSize,
    // kContainerMin,
    // kContainerMax,
    UnitType[UnitType["kRems"] = 5] = "kRems";
    // kChs,
    // kUserUnits, // The SVG term for unitless lengths
    // Angle units
    UnitType[UnitType["kDegrees"] = 6] = "kDegrees";
    UnitType[UnitType["kRadians"] = 7] = "kRadians";
    UnitType[UnitType["kGradians"] = 8] = "kGradians";
    UnitType[UnitType["kTurns"] = 9] = "kTurns";
    // Time units
    UnitType[UnitType["kMilliseconds"] = 10] = "kMilliseconds";
    UnitType[UnitType["kSeconds"] = 11] = "kSeconds";
    // kHertz,
    // kKilohertz,
    // Resolution
    // kDotsPerPixel,
    // kDotsPerInch,
    // kDotsPerCentimeter,
    // Other units
    // kFraction,
    UnitType[UnitType["kInteger"] = 12] = "kInteger";
    // This value is used to handle quirky margins in reflow roots (body, td,
    // and th) like WinIE. The basic idea is that a stylesheet can use the value
    // __qem (for quirky em) instead of em. When the quirky value is used, if
    // you're in quirks mode, the margin will collapse away inside a table cell.
    // This quirk is specified in the HTML spec but our impl is different.
    // TODO: Remove this. crbug.com/443952
    // kQuirkyEms,
  })(UnitType || (UnitType = {}));
  var UnitCategory;
  (function (UnitCategory) {
    UnitCategory[UnitCategory["kUNumber"] = 0] = "kUNumber";
    UnitCategory[UnitCategory["kUPercent"] = 1] = "kUPercent";
    UnitCategory[UnitCategory["kULength"] = 2] = "kULength";
    UnitCategory[UnitCategory["kUAngle"] = 3] = "kUAngle";
    UnitCategory[UnitCategory["kUTime"] = 4] = "kUTime";
    // kUFrequency,
    // kUResolution,
    UnitCategory[UnitCategory["kUOther"] = 5] = "kUOther";
  })(UnitCategory || (UnitCategory = {}));
  var ValueRange;
  (function (ValueRange) {
    ValueRange[ValueRange["kAll"] = 0] = "kAll";
    ValueRange[ValueRange["kNonNegative"] = 1] = "kNonNegative";
    ValueRange[ValueRange["kInteger"] = 2] = "kInteger";
    ValueRange[ValueRange["kNonNegativeInteger"] = 3] = "kNonNegativeInteger";
    ValueRange[ValueRange["kPositiveInteger"] = 4] = "kPositiveInteger";
  })(ValueRange || (ValueRange = {}));
  var Nested;
  (function (Nested) {
    Nested[Nested["kYes"] = 0] = "kYes";
    Nested[Nested["kNo"] = 1] = "kNo";
  })(Nested || (Nested = {}));
  var ParenLess;
  (function (ParenLess) {
    ParenLess[ParenLess["kYes"] = 0] = "kYes";
    ParenLess[ParenLess["kNo"] = 1] = "kNo";
  })(ParenLess || (ParenLess = {}));

  // This file specifies the unit strings used in CSSPrimitiveValues.
  var data = [{
    name: 'em',
    unit_type: UnitType.kEms
  },
  // {
  //   name: 'ex',
  //   unit_type: UnitType.kExs,
  // },
  {
    name: 'px',
    unit_type: UnitType.kPixels
  },
  // {
  //   name: "cm",
  //   unit_type: UnitType.kCentimeters,
  // },
  // {
  //   name: "mm",
  //   unit_type: UnitType.kMillimeters,
  // },
  // {
  //   name: "q",
  //   unit_type: UnitType.kQuarterMillimeters,
  // },
  // {
  //   name: "in",
  //   unit_type: UnitType.kInches,
  // },
  // {
  //   name: "pt",
  //   unit_type: UnitType.kPoints,
  // },
  // {
  //   name: "pc",
  //   unit_type: UnitType.kPicas,
  // },
  {
    name: 'deg',
    unit_type: UnitType.kDegrees
  }, {
    name: 'rad',
    unit_type: UnitType.kRadians
  }, {
    name: 'grad',
    unit_type: UnitType.kGradians
  }, {
    name: 'ms',
    unit_type: UnitType.kMilliseconds
  }, {
    name: 's',
    unit_type: UnitType.kSeconds
  },
  // {
  //   name: "hz",
  //   unit_type: UnitType.kHertz,
  // },
  // {
  //   name: "khz",
  //   unit_type: UnitType.kKilohertz,
  // },
  // {
  //   name: "dpi",
  //   unit_type: "kDotsPerInch",
  // },
  // {
  //   name: "dpcm",
  //   unit_type: "kDotsPerCentimeter",
  // },
  // {
  //   name: "dppx",
  //   unit_type: "kDotsPerPixel",
  // },
  // {
  //   name: "x",
  //   unit_type: "kDotsPerPixel",
  // },
  // {
  //   name: "vw",
  //   unit_type: "kViewportWidth",
  // },
  // {
  //   name: "vh",
  //   unit_type: "kViewportHeight",
  // },
  // {
  //   name: "vi",
  //   unit_type: "kViewportInlineSize",
  // },
  // {
  //   name: "vb",
  //   unit_type: "kViewportBlockSize",
  // },
  // {
  //   name: "vmin",
  //   unit_type: UnitType.kViewportMin,
  // },
  // {
  //   name: "vmax",
  //   unit_type: UnitType.kViewportMax,
  // },
  // {
  //   name: "svw",
  //   unit_type: "kSmallViewportWidth",
  // },
  // {
  //   name: "svh",
  //   unit_type: "kSmallViewportHeight",
  // },
  // {
  //   name: "svi",
  //   unit_type: "kSmallViewportInlineSize",
  // },
  // {
  //   name: "svb",
  //   unit_type: "kSmallViewportBlockSize",
  // },
  // {
  //   name: "svmin",
  //   unit_type: "kSmallViewportMin",
  // },
  // {
  //   name: "svmax",
  //   unit_type: "kSmallViewportMax",
  // },
  // {
  //   name: "lvw",
  //   unit_type: "kLargeViewportWidth",
  // },
  // {
  //   name: "lvh",
  //   unit_type: "kLargeViewportHeight",
  // },
  // {
  //   name: "lvi",
  //   unit_type: "kLargeViewportInlineSize",
  // },
  // {
  //   name: "lvb",
  //   unit_type: "kLargeViewportBlockSize",
  // },
  // {
  //   name: "lvmin",
  //   unit_type: UnitType.kLargeViewportMin,
  // },
  // {
  //   name: "lvmax",
  //   unit_type: UnitType.kLargeViewportMax,
  // },
  // {
  //   name: "dvw",
  //   unit_type: UnitType.kDynamicViewportWidth,
  // },
  // {
  //   name: "dvh",
  //   unit_type: UnitType.kDynamicViewportHeight,
  // },
  // {
  //   name: "dvi",
  //   unit_type: UnitType.kDynamicViewportInlineSize,
  // },
  // {
  //   name: "dvb",
  //   unit_type: UnitType.kDynamicViewportBlockSize,
  // },
  // {
  //   name: "dvmin",
  //   unit_type: UnitType.kDynamicViewportMin,
  // },
  // {
  //   name: "dvmax",
  //   unit_type: UnitType.kDynamicViewportMax,
  // },
  // {
  //   name: "cqw",
  //   unit_type: UnitType.kContainerWidth,
  // },
  // {
  //   name: "cqh",
  //   unit_type: UnitType.kContainerHeight,
  // },
  // {
  //   name: "cqi",
  //   unit_type: UnitType.kContainerInlineSize,
  // },
  // {
  //   name: "cqb",
  //   unit_type: UnitType.kContainerBlockSize,
  // },
  // {
  //   name: "cqmin",
  //   unit_type: UnitType.kContainerMin,
  // },
  // {
  //   name: "cqmax",
  //   unit_type: UnitType.kContainerMax,
  // },
  {
    name: 'rem',
    unit_type: UnitType.kRems
  },
  // {
  //   name: 'fr',
  //   unit_type: UnitType.kFraction,
  // },
  {
    name: 'turn',
    unit_type: UnitType.kTurns
  }
  // {
  //   name: 'ch',
  //   unit_type: UnitType.kChs,
  // },
  // {
  //   name: '__qem',
  //   unit_type: UnitType.kQuirkyEms,
  // },
  ];

  var CSSStyleValueType;
  (function (CSSStyleValueType) {
    CSSStyleValueType[CSSStyleValueType["kUnknownType"] = 0] = "kUnknownType";
    CSSStyleValueType[CSSStyleValueType["kUnparsedType"] = 1] = "kUnparsedType";
    CSSStyleValueType[CSSStyleValueType["kKeywordType"] = 2] = "kKeywordType";
    // Start of CSSNumericValue subclasses
    CSSStyleValueType[CSSStyleValueType["kUnitType"] = 3] = "kUnitType";
    CSSStyleValueType[CSSStyleValueType["kSumType"] = 4] = "kSumType";
    CSSStyleValueType[CSSStyleValueType["kProductType"] = 5] = "kProductType";
    CSSStyleValueType[CSSStyleValueType["kNegateType"] = 6] = "kNegateType";
    CSSStyleValueType[CSSStyleValueType["kInvertType"] = 7] = "kInvertType";
    CSSStyleValueType[CSSStyleValueType["kMinType"] = 8] = "kMinType";
    CSSStyleValueType[CSSStyleValueType["kMaxType"] = 9] = "kMaxType";
    CSSStyleValueType[CSSStyleValueType["kClampType"] = 10] = "kClampType";
    // End of CSSNumericValue subclasses
    CSSStyleValueType[CSSStyleValueType["kTransformType"] = 11] = "kTransformType";
    CSSStyleValueType[CSSStyleValueType["kPositionType"] = 12] = "kPositionType";
    CSSStyleValueType[CSSStyleValueType["kURLImageType"] = 13] = "kURLImageType";
    CSSStyleValueType[CSSStyleValueType["kColorType"] = 14] = "kColorType";
    CSSStyleValueType[CSSStyleValueType["kUnsupportedColorType"] = 15] = "kUnsupportedColorType";
  })(CSSStyleValueType || (CSSStyleValueType = {}));
  // function parseCSSStyleValue(propertyName: string, value: string): CSSStyleValue[] {
  //   // const propertyId = cssPropertyID(propertyName);
  //   // if (propertyId === CSSPropertyID.kInvalid) {
  //   //   return [];
  //   // }
  //   // const customPropertyName = propertyId === CSSPropertyID.kVariable ? propertyName : null;
  //   // return fromString(propertyId, customPropertyName, value);
  //   return [];
  // }
  var stringToUnitType = function stringToUnitType(name) {
    return data.find(function (item) {
      return item.name === name;
    }).unit_type;
  };
  var unitFromName = function unitFromName(name) {
    if (!name) {
      return UnitType.kUnknown;
    }
    if (name === 'number') {
      return UnitType.kNumber;
    }
    if (name === 'percent' || name === '%') {
      return UnitType.kPercentage;
    }
    return stringToUnitType(name);
  };
  var unitTypeToUnitCategory = function unitTypeToUnitCategory(type) {
    switch (type) {
      case UnitType.kNumber:
      case UnitType.kInteger:
        return UnitCategory.kUNumber;
      case UnitType.kPercentage:
        return UnitCategory.kUPercent;
      case UnitType.kPixels:
        // case UnitType.kCentimeters:
        // case UnitType.kMillimeters:
        // case UnitType.kQuarterMillimeters:
        // case UnitType.kInches:
        // case UnitType.kPoints:
        // case UnitType.kPicas:
        // case UnitType.kUserUnits:
        return UnitCategory.kULength;
      case UnitType.kMilliseconds:
      case UnitType.kSeconds:
        return UnitCategory.kUTime;
      case UnitType.kDegrees:
      case UnitType.kRadians:
      case UnitType.kGradians:
      case UnitType.kTurns:
        return UnitCategory.kUAngle;
      // case UnitType.kHertz:
      // case UnitType.kKilohertz:
      //   return UnitCategory.kUFrequency;
      // case UnitType.kDotsPerPixel:
      // case UnitType.kDotsPerInch:
      // case UnitType.kDotsPerCentimeter:
      //   return UnitCategory.kUResolution;
      default:
        return UnitCategory.kUOther;
    }
  };
  var canonicalUnitTypeForCategory = function canonicalUnitTypeForCategory(category) {
    // The canonical unit type is chosen according to the way
    // CSSPropertyParser.ValidUnit() chooses the default unit in each category
    // (based on unitflags).
    switch (category) {
      case UnitCategory.kUNumber:
        return UnitType.kNumber;
      case UnitCategory.kULength:
        return UnitType.kPixels;
      case UnitCategory.kUPercent:
        return UnitType.kPercentage;
      // return UnitType.kUnknown; // Cannot convert between numbers and percent.
      case UnitCategory.kUTime:
        return UnitType.kSeconds;
      case UnitCategory.kUAngle:
        return UnitType.kDegrees;
      // case UnitCategory.kUFrequency:
      //   return UnitType.kHertz;
      // case UnitCategory.kUResolution:
      //   return UnitType.kDotsPerPixel;
      default:
        return UnitType.kUnknown;
    }
  };
  /**
   * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/css_primitive_value.cc#353
   */
  var conversionToCanonicalUnitsScaleFactor = function conversionToCanonicalUnitsScaleFactor(unit_type) {
    var factor = 1.0;
    // FIXME: the switch can be replaced by an array of scale factors.
    switch (unit_type) {
      // These are "canonical" units in their respective categories.
      case UnitType.kPixels:
      // case UnitType.kUserUnits:
      case UnitType.kDegrees:
      case UnitType.kSeconds:
        // case UnitType.kHertz:
        break;
      case UnitType.kMilliseconds:
        factor = 0.001;
        break;
      // case UnitType.kCentimeters:
      //   // factor = kCssPixelsPerCentimeter;
      //   break;
      // case UnitType.kDotsPerCentimeter:
      //   // factor = 1 / kCssPixelsPerCentimeter;
      //   break;
      // case UnitType.kMillimeters:
      //   // factor = kCssPixelsPerMillimeter;
      //   break;
      // case UnitType.kQuarterMillimeters:
      //   // factor = kCssPixelsPerQuarterMillimeter;
      //   break;
      // case UnitType.kInches:
      //   // factor = kCssPixelsPerInch;
      //   break;
      // case UnitType.kDotsPerInch:
      //   // factor = 1 / kCssPixelsPerInch;
      //   break;
      // case UnitType.kPoints:
      //   // factor = kCssPixelsPerPoint;
      //   break;
      // case UnitType.kPicas:
      //   // factor = kCssPixelsPerPica;
      //   break;
      case UnitType.kRadians:
        factor = 180 / Math.PI;
        break;
      case UnitType.kGradians:
        factor = 0.9;
        break;
      case UnitType.kTurns:
        factor = 360;
        break;
    }
    return factor;
  };
  var unitTypeToString = function unitTypeToString(type) {
    switch (type) {
      case UnitType.kNumber:
      case UnitType.kInteger:
        // case UnitType.kUserUnits:
        return '';
      case UnitType.kPercentage:
        return '%';
      case UnitType.kEms:
        // case UnitType.kQuirkyEms:
        return 'em';
      // case UnitType.kExs:
      //   return 'ex';
      case UnitType.kRems:
        return 'rem';
      // case UnitType.kChs:
      //   return 'ch';
      case UnitType.kPixels:
        return 'px';
      // case UnitType.kCentimeters:
      //   return 'cm';
      // case UnitType.kDotsPerPixel:
      //   return 'dppx';
      // case UnitType.kDotsPerInch:
      //   return 'dpi';
      // case UnitType.kDotsPerCentimeter:
      //   return 'dpcm';
      // case UnitType.kMillimeters:
      //   return 'mm';
      // case UnitType.kQuarterMillimeters:
      //   return 'q';
      // case UnitType.kInches:
      //   return 'in';
      // case UnitType.kPoints:
      //   return 'pt';
      // case UnitType.kPicas:
      //   return 'pc';
      case UnitType.kDegrees:
        return 'deg';
      case UnitType.kRadians:
        return 'rad';
      case UnitType.kGradians:
        return 'grad';
      case UnitType.kMilliseconds:
        return 'ms';
      case UnitType.kSeconds:
        return 's';
      // case UnitType.kHertz:
      //   return 'hz';
      // case UnitType.kKilohertz:
      //   return 'khz';
      case UnitType.kTurns:
        return 'turn';
    }
    return '';
  };
  /**
   * CSSStyleValue is the base class for all CSS values accessible from Typed OM.
   * Values that are not yet supported as specific types are also returned as base CSSStyleValues.
   *
   * Spec @see https://drafts.css-houdini.org/css-typed-om/#stylevalue-objects
   * Docs @see https://developer.mozilla.org/en-US/docs/Web/API/CSSStyleValue
   */
  var CSSStyleValue = /*#__PURE__*/function () {
    function CSSStyleValue() {}
    // static parse(propertyName: string, value: string): CSSStyleValue {
    //   return parseCSSStyleValue(propertyName, value)[0];
    // }
    // static parseAll(propertyName: string, value: string): CSSStyleValue[] {
    //   return parseCSSStyleValue(propertyName, value);
    // }
    CSSStyleValue.isAngle = function isAngle(unit) {
      return unit === UnitType.kDegrees || unit === UnitType.kRadians || unit === UnitType.kGradians || unit === UnitType.kTurns;
    }
    // static isViewportPercentageLength(type: UnitType) {
    //   return type >= UnitType.kViewportWidth && type <= UnitType.kDynamicViewportMax;
    // }
    // static isContainerPercentageLength(type: UnitType) {
    //   return type >= UnitType.kContainerWidth && type <= UnitType.kContainerMax;
    // }
    ;
    CSSStyleValue.isLength = function isLength(type) {
      // return (type >= UnitType.kEms && type <= UnitType.kUserUnits) || type == UnitType.kQuirkyEms;
      return type >= UnitType.kEms && type < UnitType.kDegrees;
    };
    CSSStyleValue.isRelativeUnit = function isRelativeUnit(type) {
      return type === UnitType.kPercentage || type === UnitType.kEms ||
      // type === UnitType.kExs ||
      type === UnitType.kRems
      // type === UnitType.kChs ||
      // this.isViewportPercentageLength(type) ||
      // this.isContainerPercentageLength(type)
      ;
    };
    CSSStyleValue.isTime = function isTime(unit) {
      return unit === UnitType.kSeconds || unit === UnitType.kMilliseconds;
    }
    // protected abstract toCSSValue(): CSSValue;
    ;
    var _proto = CSSStyleValue.prototype;
    _proto.toString = function toString() {
      return this.buildCSSText(Nested.kNo, ParenLess.kNo, '');
    };
    _proto.isNumericValue = function isNumericValue() {
      return this.getType() >= CSSStyleValueType.kUnitType && this.getType() <= CSSStyleValueType.kClampType;
    };
    return CSSStyleValue;
  }();

  /**
   * CSSColorValue is the base class used for the various CSS color interfaces.
   *
   * @see https://drafts.css-houdini.org/css-typed-om-1/#colorvalue-objects
   */
  var CSSColorValue = /*#__PURE__*/function (_CSSStyleValue) {
    _inheritsLoose(CSSColorValue, _CSSStyleValue);
    function CSSColorValue(colorSpace) {
      var _this;
      _this = _CSSStyleValue.call(this) || this;
      _this.colorSpace = void 0;
      _this.colorSpace = colorSpace;
      return _this;
    }
    var _proto = CSSColorValue.prototype;
    _proto.getType = function getType() {
      return CSSStyleValueType.kColorType;
    }
    // buildCSSText(n: Nested, p: ParenLess, result: string): string {
    //   let text = '';
    //   if (this.colorSpace === 'rgb') {
    //     text = `rgba(${this.channels.join(',')},${this.alpha})`;
    //   }
    //   return (result += text);
    // }
    /**
     * @see https://drafts.css-houdini.org/css-typed-om-1/#dom-csscolorvalue-to
     */;
    _proto.to = function to(colorSpace) {
      return this;
    };
    return CSSColorValue;
  }(CSSStyleValue);

  var GradientType;
  (function (GradientType) {
    GradientType[GradientType["Constant"] = 0] = "Constant";
    GradientType[GradientType["LinearGradient"] = 1] = "LinearGradient";
    GradientType[GradientType["RadialGradient"] = 2] = "RadialGradient";
  })(GradientType || (GradientType = {}));
  var CSSGradientValue = /*#__PURE__*/function (_CSSStyleValue) {
    _inheritsLoose(CSSGradientValue, _CSSStyleValue);
    function CSSGradientValue(type, value) {
      var _this;
      _this = _CSSStyleValue.call(this) || this;
      _this.type = void 0;
      _this.value = void 0;
      _this.type = type;
      _this.value = value;
      return _this;
    }
    var _proto = CSSGradientValue.prototype;
    _proto.clone = function clone() {
      return new CSSGradientValue(this.type, this.value);
    };
    _proto.buildCSSText = function buildCSSText(n, p, result) {
      return result;
    };
    _proto.getType = function getType() {
      return CSSStyleValueType.kColorType;
    };
    return CSSGradientValue;
  }(CSSStyleValue);

  /**
   * CSSKeywordValue represents CSS Values that are specified as keywords
   * eg. 'initial'
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSKeywordValue
   * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_keyword_value.idl
   */
  var CSSKeywordValue = /*#__PURE__*/function (_CSSStyleValue) {
    _inheritsLoose(CSSKeywordValue, _CSSStyleValue);
    function CSSKeywordValue(value) {
      var _this;
      _this = _CSSStyleValue.call(this) || this;
      _this.value = void 0;
      _this.value = value;
      return _this;
    }
    var _proto = CSSKeywordValue.prototype;
    _proto.clone = function clone() {
      return new CSSKeywordValue(this.value);
    };
    _proto.getType = function getType() {
      return CSSStyleValueType.kKeywordType;
    };
    _proto.buildCSSText = function buildCSSText(n, p, result) {
      return result + this.value;
    };
    return CSSKeywordValue;
  }(CSSStyleValue);

  var CSSMathOperator;
  (function (CSSMathOperator) {
    CSSMathOperator["kAdd"] = "+";
    CSSMathOperator["kSubtract"] = "-";
    CSSMathOperator["kMultiply"] = "*";
    CSSMathOperator["kDivide"] = "/";
    CSSMathOperator["kMin"] = "min";
    CSSMathOperator["kMax"] = "max";
    CSSMathOperator["kClamp"] = "clamp";
    CSSMathOperator["kInvalid"] = "";
  })(CSSMathOperator || (CSSMathOperator = {}));

  // https://drafts.css-houdini.org/css-typed-om/#enumdef-cssnumericbasetype
  var BaseType;
  (function (BaseType) {
    BaseType[BaseType["kLength"] = 0] = "kLength";
    BaseType[BaseType["kAngle"] = 1] = "kAngle";
    BaseType[BaseType["kTime"] = 2] = "kTime";
    BaseType[BaseType["kFrequency"] = 3] = "kFrequency";
    BaseType[BaseType["kResolution"] = 4] = "kResolution";
    BaseType[BaseType["kFlex"] = 5] = "kFlex";
    BaseType[BaseType["kPercent"] = 6] = "kPercent";
    BaseType[BaseType["kNumBaseTypes"] = 7] = "kNumBaseTypes";
  })(BaseType || (BaseType = {}));
  function unitTypeToBaseType(unit) {
    DCHECK_NE(unit, UnitType.kNumber);
    switch (unit) {
      case UnitType.kEms:
      // case UnitType.kExs:
      case UnitType.kPixels:
      // case UnitType.kCentimeters:
      // case UnitType.kMillimeters:
      // case UnitType.kQuarterMillimeters:
      // case UnitType.kInches:
      // case UnitType.kPoints:
      // case UnitType.kPicas:
      // case UnitType.kUserUnits:
      // case UnitType.kViewportWidth:
      // case UnitType.kViewportHeight:
      // case UnitType.kViewportInlineSize:
      // case UnitType.kViewportBlockSize:
      // case UnitType.kViewportMin:
      // case UnitType.kViewportMax:
      // case UnitType.kSmallViewportWidth:
      // case UnitType.kSmallViewportHeight:
      // case UnitType.kSmallViewportInlineSize:
      // case UnitType.kSmallViewportBlockSize:
      // case UnitType.kSmallViewportMin:
      // case UnitType.kSmallViewportMax:
      // case UnitType.kLargeViewportWidth:
      // case UnitType.kLargeViewportHeight:
      // case UnitType.kLargeViewportInlineSize:
      // case UnitType.kLargeViewportBlockSize:
      // case UnitType.kLargeViewportMin:
      // case UnitType.kLargeViewportMax:
      // case UnitType.kDynamicViewportWidth:
      // case UnitType.kDynamicViewportHeight:
      // case UnitType.kDynamicViewportInlineSize:
      // case UnitType.kDynamicViewportBlockSize:
      // case UnitType.kDynamicViewportMin:
      // case UnitType.kDynamicViewportMax:
      // case UnitType.kContainerWidth:
      // case UnitType.kContainerHeight:
      // case UnitType.kContainerInlineSize:
      // case UnitType.kContainerBlockSize:
      // case UnitType.kContainerMin:
      // case UnitType.kContainerMax:
      case UnitType.kRems:
        // case UnitType.kChs:
        return BaseType.kLength;
      case UnitType.kMilliseconds:
      case UnitType.kSeconds:
        return BaseType.kTime;
      case UnitType.kDegrees:
      case UnitType.kRadians:
      case UnitType.kGradians:
      case UnitType.kTurns:
        return BaseType.kAngle;
      // case UnitType.kHertz:
      // case UnitType.kKilohertz:
      //   return BaseType.kFrequency;
      // case UnitType.kDotsPerPixel:
      // case UnitType.kDotsPerInch:
      // case UnitType.kDotsPerCentimeter:
      //   return BaseType.kResolution;
      // case UnitType.kFraction:
      //   return BaseType.kFlex;
      case UnitType.kPercentage:
        return BaseType.kPercent;
      default:
        return BaseType.kLength;
    }
  }
  function baseTypeToString(baseType) {
    switch (baseType) {
      case BaseType.kLength:
        return 'length';
      case BaseType.kAngle:
        return 'angle';
      case BaseType.kTime:
        return 'time';
      case BaseType.kFrequency:
        return 'frequency';
      case BaseType.kResolution:
        return 'resolution';
      case BaseType.kFlex:
        return 'flex';
      case BaseType.kPercent:
        return 'percent';
    }
    return '';
  }
  var CSSNumericValueType = /*#__PURE__*/function () {
    function CSSNumericValueType(unit, exponent) {
      if (unit === void 0) {
        unit = UnitType.kNumber;
      }
      if (exponent === void 0) {
        exponent = 1;
      }
      this.exponents = [];
      this.numNonZeroEntries = 0;
      this.percentHint = BaseType.kPercent;
      this.hasPercentHint = false;
      this.exponents = new Array(BaseType.kNumBaseTypes).fill(0);
      if (unit !== UnitType.kNumber) {
        this.setExponent(unitTypeToBaseType(unit), exponent);
      }
    }
    var _proto = CSSNumericValueType.prototype;
    _proto.applyPercentHint = function applyPercentHint(hint) {
      DCHECK_NE(hint, BaseType.kPercent);
      this.setExponent(hint, this.exponent(hint) + this.exponent(BaseType.kPercent));
      this.setExponent(BaseType.kPercent, 0);
      this.percentHint = hint;
      this.hasPercentHint = true;
    };
    _proto.hasNonZeroEntries = function hasNonZeroEntries() {
      return this.numNonZeroEntries > 0;
    };
    _proto.isOnlyNonZeroEntry = function isOnlyNonZeroEntry(baseType, value) {
      DCHECK_NE(value, 0);
      return this.numNonZeroEntries === 1 && this.exponent(baseType) === value;
    };
    _proto.exponent = function exponent(type) {
      return this.exponents[type];
    };
    _proto.setExponent = function setExponent(type, newValue) {
      var oldValue = this.exponents[type];
      if (oldValue == 0 && newValue !== 0) {
        this.numNonZeroEntries++;
      } else if (oldValue !== 0 && newValue == 0) {
        this.numNonZeroEntries--;
      }
      this.exponents[type] = newValue;
    };
    CSSNumericValueType.negateExponents = function negateExponents(type) {
      type.exponents.forEach(function (v) {
        return v *= -1;
      });
      return type;
    };
    _proto.matchesBaseType = function matchesBaseType(baseType) {
      DCHECK_NE(baseType, BaseType.kPercent);
      return this.isOnlyNonZeroEntry(baseType, 1) && !this.hasPercentHint;
    };
    _proto.matchesPercentage = function matchesPercentage() {
      return this.isOnlyNonZeroEntry(BaseType.kPercent, 1);
    };
    _proto.matchesBaseTypePercentage = function matchesBaseTypePercentage(baseType) {
      DCHECK_NE(baseType, BaseType.kPercent);
      return this.isOnlyNonZeroEntry(baseType, 1) || this.isOnlyNonZeroEntry(BaseType.kPercent, 1);
    };
    _proto.matchesNumber = function matchesNumber() {
      return !this.hasNonZeroEntries() && !this.hasPercentHint;
    };
    _proto.matchesNumberPercentage = function matchesNumberPercentage() {
      return !this.hasNonZeroEntries() || this.isOnlyNonZeroEntry(BaseType.kPercent, 1);
    };
    CSSNumericValueType.add = function add(type1, type2, error) {
      if (type1.hasPercentHint && type2.hasPercentHint && type1.percentHint != type2.percentHint) {
        return type1;
      }
      if (type1.hasPercentHint) type2.applyPercentHint(type1.percentHint);else if (type2.hasPercentHint) type1.applyPercentHint(type2.percentHint);
      DCHECK_EQ(type1.percentHint, type2.percentHint);
      // Match up base types. Try to use the percent hint to match up any
      // differences.
      for (var i = 0; i < BaseType.kNumBaseTypes; ++i) {
        var base_type = i;
        if (type1.exponents[i] !== type2.exponents[i]) {
          if (base_type !== BaseType.kPercent) {
            type1.applyPercentHint(base_type);
            type2.applyPercentHint(base_type);
          }
          if (type1.exponents[i] !== type2.exponents[i]) {
            return type1;
          }
        }
      }
      return type1;
    };
    CSSNumericValueType.multiply = function multiply(type1, type2, error) {
      if (type1.hasPercentHint && type2.hasPercentHint && type1.percentHint != type2.percentHint) {
        return type1;
      }
      if (type1.hasPercentHint) type2.applyPercentHint(type1.percentHint);else if (type2.hasPercentHint) type1.applyPercentHint(type2.percentHint);
      for (var i = 0; i < BaseType.kNumBaseTypes; ++i) {
        var base_type = i;
        type1.setExponent(base_type, type1.exponent(base_type) + type2.exponent(base_type));
      }
      return type1;
    };
    return CSSNumericValueType;
  }();
  var fromNumberish = function fromNumberish(value) {
    if (typeof value === 'number') {
      return new CSSUnitValue(value, UnitType.kNumber);
    }
    return value;
  };
  /**
   * CSSNumericValue is the base class for numeric and length typed CSS Values.
   * @see https://drafts.css-houdini.org/css-typed-om/#numeric-objects
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue
   * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.idl
   */
  var CSSNumericValue = /*#__PURE__*/function (_CSSStyleValue) {
    _inheritsLoose(CSSNumericValue, _CSSStyleValue);
    /**
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#296
     */
    // static fromPercentish(value: CSSNumberish): CSSNumericValue {
    //   if (typeof value === 'number') {
    //     return new CSSUnitValue(value * 100, UnitType.kPercentage);
    //   }
    //   return value;
    // }
    function CSSNumericValue(type_) {
      var _this;
      _this = _CSSStyleValue.call(this) || this;
      _this.type_ = void 0;
      _this.type_ = type_;
      return _this;
    }
    var _proto2 = CSSNumericValue.prototype;
    _proto2.getType = function getType() {
      return CSSStyleValueType.kUnknownType;
    }
    // toCSSValue() {
    //   return null;
    // }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/equals
     */;
    _proto2.equals = function equals() {
      var _this2 = this;
      for (var _len = arguments.length, numberishes = new Array(_len), _key = 0; _key < _len; _key++) {
        numberishes[_key] = arguments[_key];
      }
      var values = cssNumberishesToNumericValues(numberishes);
      return values.every(function (value) {
        return _this2.equals(value);
      });
    }
    /**
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#439
     */;
    _proto2.add = function add() {
      for (var _len2 = arguments.length, numberishes = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        numberishes[_key2] = arguments[_key2];
      }
      var values = cssNumberishesToNumericValues(numberishes);
      prependValueForArithmetic(CSSStyleValueType.kSumType, values, this);
      // eg. 1px + 2px = 3px
      var unitValue = maybeSimplifyAsUnitValue(values, CSSMathOperator.kAdd);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathSum.create(values);
    }
    /**
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#452
     */;
    _proto2.sub = function sub() {
      for (var _len3 = arguments.length, numberishes = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        numberishes[_key3] = arguments[_key3];
      }
      var values = cssNumberishesToNumericValues(numberishes);
      values = values.map(function (value) {
        return value.negate();
      });
      prependValueForArithmetic(CSSStyleValueType.kSumType, values, this);
      // eg. 3px - 2px = 1px
      var unitValue = maybeSimplifyAsUnitValue(values, CSSMathOperator.kAdd);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathSum.create(values);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/mul
     */;
    _proto2.mul = function mul() {
      for (var _len4 = arguments.length, numberishes = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        numberishes[_key4] = arguments[_key4];
      }
      var values = cssNumberishesToNumericValues(numberishes);
      prependValueForArithmetic(CSSStyleValueType.kProductType, values, this);
      var unitValue = maybeMultiplyAsUnitValue(values);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathProduct.create(values);
    }
    /**
     * eg. calc(24px / 4%)
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/div
     */;
    _proto2.div = function div() {
      for (var _len5 = arguments.length, numberishes = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        numberishes[_key5] = arguments[_key5];
      }
      var values = cssNumberishesToNumericValues(numberishes);
      values = values.map(function (value) {
        return value.invert();
      });
      prependValueForArithmetic(CSSStyleValueType.kProductType, values, this);
      var unitValue = maybeMultiplyAsUnitValue(values);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathProduct.create(values);
    };
    _proto2.min = function min() {
      for (var _len6 = arguments.length, numberishes = new Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        numberishes[_key6] = arguments[_key6];
      }
      var values = cssNumberishesToNumericValues(numberishes);
      prependValueForArithmetic(CSSStyleValueType.kMinType, values, this);
      var unitValue = maybeSimplifyAsUnitValue(values, CSSMathOperator.kMin);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathMin.create(values);
    };
    _proto2.max = function max() {
      for (var _len7 = arguments.length, numberishes = new Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
        numberishes[_key7] = arguments[_key7];
      }
      var values = cssNumberishesToNumericValues(numberishes);
      prependValueForArithmetic(CSSStyleValueType.kMaxType, values, this);
      var unitValue = maybeSimplifyAsUnitValue(values, CSSMathOperator.kMax);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathMax.create(values);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/to
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#331
     */;
    _proto2.to = function to(unitOrName) {
      var sum = this.sumValue();
      if (sum.length === 0 || sum.length !== 1) return null;
      var value = cssNumericSumValueEntryToUnitValue(sum[0]);
      if (!value) return null;
      var unit;
      if (typeof unitOrName === 'string') {
        unit = unitFromName(unitOrName);
      } else {
        unit = unitOrName;
      }
      return value.convertTo(unit);
    }
    /**
     * converts the object's value to a CSSMathSum object to values of the specified unit.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/toSum
     */;
    _proto2.toSum = function toSum() {
      for (var _len8 = arguments.length, unit_strings = new Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
        unit_strings[_key8] = arguments[_key8];
      }
      for (var _i = 0, _unit_strings = unit_strings; _i < _unit_strings.length; _i++) {
        var unit_string = _unit_strings[_i];
        if (!CSSNumericValue.isValidUnit(unitFromName(unit_string))) {
          return null;
        }
      }
      var sum = this.sumValue();
      if (!sum.length) {
        return null;
      }
      var values = [];
      for (var _iterator = _createForOfIteratorHelperLoose(sum), _step; !(_step = _iterator()).done;) {
        var term = _step.value;
        var value = cssNumericSumValueEntryToUnitValue(term);
        if (!value) {
          return null;
        }
        values.push(value);
      }
      if (unit_strings.length === 0) {
        values.sort(function (a, b) {
          return a.unit - b.unit;
        });
        // We got 'values' from a sum value, so it must be a valid CSSMathSum.
        var _result = CSSMathSum.create(values);
        DCHECK(!!_result);
        return _result;
      }
      var result = [];
      var _loop = function _loop() {
        var unit_string = _unit_strings2[_i2];
        var target_unit = unitFromName(unit_string);
        DCHECK(CSSNumericValue.isValidUnit(target_unit));
        // Collect all the terms that are compatible with this unit.
        // We mark used terms as null so we don't use them again.
        var total_value = values.reduce(function (cur_sum, value, i) {
          if (value) {
            var unit_value = value;
            var converted_value = unit_value.convertTo(target_unit);
            if (converted_value) {
              cur_sum += converted_value.value;
              values[i] = null;
            }
          }
          return cur_sum;
        }, 0);
        result.push(new CSSUnitValue(total_value, target_unit));
      };
      for (var _i2 = 0, _unit_strings2 = unit_strings; _i2 < _unit_strings2.length; _i2++) {
        _loop();
      }
      if (values.some(function (v) {
        return !!v;
      })) {
        throw new Error('There were leftover terms that were not converted');
      }
      return CSSMathSum.create(result);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/type
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#414
     */;
    _proto2.type = function type() {
      var type = {
        length: 0,
        angle: 0,
        time: 0,
        frequency: 0,
        resolution: 0,
        flex: 0,
        percent: 0,
        percentHint: 'length'
      };
      var exponent = this.type_.exponent(BaseType.kLength);
      if (exponent) {
        type.length = exponent;
      }
      exponent = this.type_.exponent(BaseType.kAngle);
      if (exponent) {
        type.angle = exponent;
      }
      exponent = this.type_.exponent(BaseType.kTime);
      if (exponent) {
        type.time = exponent;
      }
      exponent = this.type_.exponent(BaseType.kFrequency);
      if (exponent) {
        type.frequency = exponent;
      }
      exponent = this.type_.exponent(BaseType.kResolution);
      if (exponent) {
        type.resolution = exponent;
      }
      exponent = this.type_.exponent(BaseType.kFlex);
      if (exponent) {
        type.flex = exponent;
      }
      exponent = this.type_.exponent(BaseType.kPercent);
      if (exponent) {
        type.percent = exponent;
      }
      if (this.type_.hasPercentHint) {
        type.percentHint = baseTypeToString(this.type_.percentHint);
      }
      return type;
    };
    CSSNumericValue.isValidUnit = function isValidUnit(unit) {
      // if (unit === UnitType.kUserUnits) return false;
      if (unit === UnitType.kNumber || unit == UnitType.kPercentage || this.isLength(unit) || this.isAngle(unit) || this.isTime(unit)
      // this.isFrequency(unit) ||
      // this.isResolution(unit) ||
      // this.isFlex(unit)
      ) return true;
      return false;
    };
    _proto2.negate = function negate() {
      return CSSMathNegate.create(this);
    };
    _proto2.invert = function invert() {
      return CSSMathInvert.create(this);
    };
    return CSSNumericValue;
  }(CSSStyleValue);
  function cssNumberishesToNumericValues(values) {
    return values.map(fromNumberish);
  }
  function prependValueForArithmetic(type, values, value) {
    DCHECK(!!value);
    if (value.getType() === type) {
      values.unshift.apply(values, value.numericValues());
    } else {
      values.unshift(value);
    }
  }
  function cssNumericSumValueEntryToUnitValue(term) {
    if (Object.keys(term.units).length === 0) {
      return new CSSUnitValue(term.value);
    }
    if (Object.keys(term.units).length === 1 && term.units[Object.keys(term.units)[0]] === 1) {
      return new CSSUnitValue(term.value, Number(Object.keys(term.units)[0]));
    }
    return null;
  }
  function maybeSimplifyAsUnitValue(values, operator) {
    DCHECK(!!values.length);
    var first_unit_value = values[0] instanceof CSSUnitValue ? values[0] : null;
    if (!first_unit_value) return null;
    var final_value = first_unit_value.value;
    for (var i = 1; i < values.length; i++) {
      var unit_value = values[i] instanceof CSSUnitValue ? values[i] : null;
      if (!unit_value || unit_value.unit !== first_unit_value.unit) return null;
      if (operator === CSSMathOperator.kAdd) {
        final_value += unit_value.value;
      } else if (operator === CSSMathOperator.kMax) {
        final_value = Math.max(final_value, unit_value.value);
      } else if (operator === CSSMathOperator.kMin) {
        final_value = Math.min(final_value, unit_value.value);
      }
    }
    return new CSSUnitValue(final_value, first_unit_value.unit);
  }
  function maybeMultiplyAsUnitValue(values) {
    DCHECK(!!values.length);
    // We are allowed one unit value with type other than kNumber.
    var unit_other_than_number = UnitType.kNumber;
    var final_value = 1.0;
    for (var i = 0; i < values.length; i++) {
      var unit_value = values[i] instanceof CSSUnitValue ? values[i] : null;
      if (!unit_value) return null;
      if (unit_value.unit !== UnitType.kNumber) {
        if (unit_other_than_number !== UnitType.kNumber) return null;
        unit_other_than_number = unit_value.unit;
      }
      final_value *= unit_value.value;
    }
    return new CSSUnitValue(final_value, unit_other_than_number);
  }
  var toCanonicalUnit = function toCanonicalUnit(unit) {
    return canonicalUnitTypeForCategory(unitTypeToUnitCategory(unit));
  };
  var formatInfinityOrNaN = function formatInfinityOrNaN(number, suffix) {
    if (suffix === void 0) {
      suffix = '';
    }
    var result = '';
    if (!Number.isFinite(number)) {
      if (number > 0) result = 'infinity';else result = '-infinity';
    } else {
      DCHECK(Number.isNaN(number));
      result = 'NaN';
    }
    return result += suffix;
  };
  var toCanonicalUnitIfPossible = function toCanonicalUnitIfPossible(unit) {
    var canonical_unit = toCanonicalUnit(unit);
    if (canonical_unit === UnitType.kUnknown) return unit;
    return canonical_unit;
  };
  /**
   * Represents numeric values that can be expressed as a single number plus a
   * unit (or a naked number or percentage).
   * @see https://drafts.css-houdini.org/css-typed-om/#cssunitvalue
   */
  var CSSUnitValue = /*#__PURE__*/function (_CSSNumericValue) {
    _inheritsLoose(CSSUnitValue, _CSSNumericValue);
    // static fromCSSValue(value: CSSNumericLiteralValue) {
    //   let unit = value.type;
    //   if (unit === UnitType.kInteger) unit = UnitType.kNumber;
    //   if (!this.isValidUnit(unit)) return null;
    //   return new CSSUnitValue(value.getDoubleValue(), unit);
    // }

    function CSSUnitValue(value, unitOrName) {
      var _this3;
      if (unitOrName === void 0) {
        unitOrName = UnitType.kNumber;
      }
      var unit;
      if (typeof unitOrName === 'string') {
        unit = unitFromName(unitOrName);
      } else {
        unit = unitOrName;
      }
      // DCHECK(CSSUnitValue.isValidUnit(unit));
      _this3 = _CSSNumericValue.call(this, new CSSNumericValueType(unit)) || this;
      _this3.unit = void 0;
      _this3.value = void 0;
      _this3.unit = unit;
      _this3.value = value;
      return _this3;
    }
    var _proto3 = CSSUnitValue.prototype;
    _proto3.clone = function clone() {
      return new CSSUnitValue(this.value, this.unit);
    };
    _proto3.convertTo = function convertTo(target_unit) {
      if (this.unit === target_unit) {
        return new CSSUnitValue(this.value, this.unit);
      }
      // Instead of defining the scale factors for every unit to every other unit,
      // we simply convert to the canonical unit and back since we already have
      // the scale factors for canonical units.
      var canonical_unit = toCanonicalUnit(this.unit);
      if (canonical_unit !== toCanonicalUnit(target_unit) || canonical_unit === UnitType.kUnknown) {
        return null;
      }
      var scale_factor = conversionToCanonicalUnitsScaleFactor(this.unit) / conversionToCanonicalUnitsScaleFactor(target_unit);
      return new CSSUnitValue(this.value * scale_factor, target_unit);
    };
    _proto3.equals = function equals(other) {
      var other_unit_value = other;
      return this.value === other_unit_value.value && this.unit === other_unit_value.unit;
    };
    _proto3.getType = function getType() {
      return CSSStyleValueType.kUnitType;
    }
    // toCSSValue() {
    //   return new CSSNumericLiteralValue(this.value, this.unit);
    // }
    // // const CSSPrimitiveValue* ToCSSValueWithProperty(CSSPropertyID) const final;
    // toCalcExpressionNode() {
    //   return CSSMathExpressionNumericLiteral.create(
    //     new CSSNumericLiteralValue(this.value, this.unit),
    //   );
    // }
    ;
    _proto3.sumValue = function sumValue() {
      var sum = [];
      var unit_map = {};
      if (this.unit !== UnitType.kNumber) {
        unit_map[toCanonicalUnitIfPossible(this.unit)] = 1;
      }
      sum.push({
        value: this.value * conversionToCanonicalUnitsScaleFactor(this.unit),
        units: unit_map
      });
      return sum;
    };
    _proto3.negate = function negate() {
      return new CSSUnitValue(-this.value, this.unit);
    };
    _proto3.invert = function invert() {
      if (this.unit === UnitType.kNumber) {
        if (this.value === 0) return null;
        return new CSSUnitValue(1.0 / this.value, this.unit);
      }
      return CSSMathInvert.create(this);
    };
    _proto3.buildCSSText = function buildCSSText(n, p, result) {
      var text;
      switch (this.unit) {
        case UnitType.kUnknown:
          // FIXME
          break;
        case UnitType.kInteger:
          text = Number(this.value).toFixed(0);
          break;
        case UnitType.kNumber:
        case UnitType.kPercentage:
        case UnitType.kEms:
        // case UnitType.kQuirkyEms:
        // case UnitType.kExs:
        case UnitType.kRems:
        // case UnitType.kChs:
        case UnitType.kPixels:
        // case UnitType.kCentimeters:
        // case UnitType.kDotsPerPixel:
        // case UnitType.kDotsPerInch:
        // case UnitType.kDotsPerCentimeter:
        // case UnitType.kMillimeters:
        // case UnitType.kQuarterMillimeters:
        // case UnitType.kInches:
        // case UnitType.kPoints:
        // case UnitType.kPicas:
        // case UnitType.kUserUnits:
        case UnitType.kDegrees:
        case UnitType.kRadians:
        case UnitType.kGradians:
        case UnitType.kMilliseconds:
        case UnitType.kSeconds:
        // case UnitType.kHertz:
        // case UnitType.kKilohertz:
        case UnitType.kTurns:
          // case UnitType.kDynamicViewportBlockSize: // case UnitType.kDynamicViewportInlineSize: // case UnitType.kDynamicViewportHeight: // case UnitType.kDynamicViewportWidth: // case UnitType.kLargeViewportMax: // case UnitType.kLargeViewportMin: // case UnitType.kLargeViewportBlockSize: // case UnitType.kLargeViewportInlineSize: // case UnitType.kLargeViewportHeight: // case UnitType.kLargeViewportWidth: // case UnitType.kSmallViewportMax: // case UnitType.kSmallViewportMin: // case UnitType.kSmallViewportBlockSize: // case UnitType.kSmallViewportInlineSize: // case UnitType.kSmallViewportHeight: // case UnitType.kSmallViewportWidth: // case UnitType.kViewportMax: // case UnitType.kViewportMin: // case UnitType.kViewportBlockSize: // case UnitType.kViewportInlineSize: // case UnitType.kViewportHeight: // case UnitType.kViewportWidth: // case UnitType.kFraction:
          // case UnitType.kDynamicViewportMin:
          // case UnitType.kDynamicViewportMax:
          // case UnitType.kContainerWidth:
          // case UnitType.kContainerHeight:
          // case UnitType.kContainerInlineSize:
          // case UnitType.kContainerBlockSize:
          // case UnitType.kContainerMin:
          // case UnitType.kContainerMax: {
          {
            var kMinInteger = -999999;
            var kMaxInteger = 999999;
            var value = this.value;
            var unit = unitTypeToString(this.unit);
            if (value < kMinInteger || value > kMaxInteger) {
              var _unit = unitTypeToString(this.unit);
              if (!Number.isFinite(value) || Number.isNaN(value)) {
                text = formatInfinityOrNaN(value, _unit);
              } else {
                text = value + (_unit || '');
              }
            } else {
              text = "" + value + unit;
            }
          }
      }
      result += text;
      return result;
    };
    return CSSUnitValue;
  }(CSSNumericValue);
  /**
   * The CSSMathValue interface of the CSS_Object_Model a base class for classes representing complex numeric values.
   *
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSMathValue
   * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_math_value.idl
   */
  var CSSMathValue = /*#__PURE__*/function (_CSSNumericValue2) {
    _inheritsLoose(CSSMathValue, _CSSNumericValue2);
    function CSSMathValue() {
      return _CSSNumericValue2.apply(this, arguments) || this;
    }
    var _proto4 = CSSMathValue.prototype;
    // From CSSNumericValue
    _proto4.isUnitValue = function isUnitValue() {
      return false;
    };
    return CSSMathValue;
  }(CSSNumericValue);
  var CSSMathInvert = /*#__PURE__*/function (_CSSMathValue) {
    _inheritsLoose(CSSMathInvert, _CSSMathValue);
    CSSMathInvert.create = function create(value) {
      var type = CSSNumericValueType.negateExponents(value.type_);
      return new CSSMathInvert(value, type);
    };
    function CSSMathInvert(value, type) {
      var _this4;
      _this4 = _CSSMathValue.call(this, type) || this;
      _this4.value = void 0;
      _this4.operator = 'invert';
      _this4.value = value;
      return _this4;
    }
    var _proto5 = CSSMathInvert.prototype;
    _proto5.clone = function clone() {
      return new CSSMathInvert(this.value, this.type_);
    };
    _proto5.getType = function getType() {
      return CSSStyleValueType.kInvertType;
    };
    _proto5.equals = function equals(other) {
      if (other.getType() !== CSSStyleValueType.kInvertType) {
        return false;
      }
      // We can safely cast here as we know 'other' has the same type as us.
      var other_invert = other;
      return this.value.equals(other_invert.value);
    };
    _proto5.sumValue = function sumValue() {
      var sum = this.value.sumValue();
      if (sum.length === 0 || sum.length !== 1 || sum[0].value === 0) {
        return null;
      }
      Object.keys(sum[0].units).forEach(function (key) {
        sum[0].units[key] *= -1;
      });
      sum[0].value = 1.0 / sum[0].value;
      return sum;
    }
    // toCalcExpressionNode(): CSSMathExpressionNode {
    //   const right_side = this.value.toCalcExpressionNode();
    //   if (!right_side) {
    //     return null;
    //   }
    //   // return CSSMathExpressionOperation::CreateArithmeticOperation(
    //   //   CSSMathExpressionNumericLiteral::Create(
    //   //       1, UnitType.kNumber),
    //   //   right_side, CSSMathOperator.kDivide);
    //   return null;
    // }
    ;
    _proto5.buildCSSText = function buildCSSText(nested, paren_less, result) {
      if (paren_less == ParenLess.kNo) {
        result += nested === Nested.kYes ? '(' : 'calc(';
      }
      result += '1 / ';
      result = this.value.buildCSSText(Nested.kYes, ParenLess.kNo, result);
      if (paren_less === ParenLess.kNo) {
        result += ')';
      }
      return result;
    };
    return CSSMathInvert;
  }(CSSMathValue);
  var CSSMathNegate = /*#__PURE__*/function (_CSSMathValue2) {
    _inheritsLoose(CSSMathNegate, _CSSMathValue2);
    CSSMathNegate.create = function create(value) {
      return new CSSMathNegate(value, value.type_);
    };
    function CSSMathNegate(value, type) {
      var _this5;
      _this5 = _CSSMathValue2.call(this, type) || this;
      _this5.value = void 0;
      _this5.operator = 'negate';
      _this5.value = value;
      return _this5;
    }
    var _proto6 = CSSMathNegate.prototype;
    _proto6.clone = function clone() {
      return new CSSMathNegate(this.value, this.type_);
    };
    _proto6.getType = function getType() {
      return CSSStyleValueType.kNegateType;
    };
    _proto6.equals = function equals(other) {
      if (other.getType() !== CSSStyleValueType.kNegateType) {
        return false;
      }
      var other_invert = other;
      return this.value.equals(other_invert.value);
    };
    _proto6.sumValue = function sumValue() {
      var sum = this.value.sumValue();
      if (sum.length === 0) {
        return null;
      }
      sum.forEach(function (term) {
        term.value *= -1;
      });
      return sum;
    }
    // toCalcExpressionNode(): CSSMathExpressionNode {
    //   const right_side = this.value.toCalcExpressionNode();
    //   if (!right_side) {
    //     return null;
    //   }
    //   // return CSSMathExpressionOperation::CreateArithmeticOperationSimplified(
    //   //   CSSMathExpressionNumericLiteral::Create(
    //   //       -1, CSSPrimitiveValue::UnitType::kNumber),
    //   //   right_side, CSSMathOperator::kMultiply);
    // }
    ;
    _proto6.buildCSSText = function buildCSSText(nested, paren_less, result) {
      if (paren_less == ParenLess.kNo) {
        result += nested === Nested.kYes ? '(' : 'calc(';
      }
      result += '-';
      result = this.value.buildCSSText(Nested.kYes, ParenLess.kNo, result);
      if (paren_less === ParenLess.kNo) {
        result += ')';
      }
      return result;
    };
    return CSSMathNegate;
  }(CSSMathValue);
  function typeCheck(values, op, error) {
    error = false;
    var final_type = values[0].type_;
    for (var i = 1; i < values.length; i++) {
      final_type = op(final_type, values[i].type_, error);
      if (error) return final_type;
    }
    return final_type;
  }
  // Represents an arithmetic operation with one or more CSSNumericValues.
  var CSSMathVariadic = /*#__PURE__*/function (_CSSMathValue3) {
    _inheritsLoose(CSSMathVariadic, _CSSMathValue3);
    function CSSMathVariadic(values, type) {
      var _this6;
      _this6 = _CSSMathValue3.call(this, type) || this;
      _this6.values = void 0;
      _this6.values = values;
      return _this6;
    }
    var _proto7 = CSSMathVariadic.prototype;
    _proto7.numericValues = function numericValues() {
      return this.values;
    };
    _proto7.equals = function equals(other) {
      if (this.getType() !== other.getType()) {
        return false;
      }
      return this.values.every(function (value, i) {
        return value.equals(other.values[i]);
      });
    };
    return CSSMathVariadic;
  }(CSSMathValue);
  // Represents the minimum of one or more CSSNumericValues.
  // @see https://drafts.css-houdini.org/css-typed-om/#cssmathsum
  var CSSMathMin = /*#__PURE__*/function (_CSSMathVariadic) {
    _inheritsLoose(CSSMathMin, _CSSMathVariadic);
    function CSSMathMin() {
      var _this7;
      for (var _len9 = arguments.length, args = new Array(_len9), _key9 = 0; _key9 < _len9; _key9++) {
        args[_key9] = arguments[_key9];
      }
      _this7 = _CSSMathVariadic.call.apply(_CSSMathVariadic, [this].concat(args)) || this;
      _this7.operator = 'min';
      return _this7;
    }
    CSSMathMin.create = function create(values) {
      var error = false;
      var final_type = typeCheck(values, CSSNumericValueType.add, error);
      return  new CSSMathMin(values, final_type);
    };
    var _proto8 = CSSMathMin.prototype;
    _proto8.clone = function clone() {
      return new CSSMathMin(this.values, this.type_);
    };
    _proto8.getType = function getType() {
      return CSSStyleValueType.kMinType;
    };
    _proto8.sumValue = function sumValue() {
      var cur_min = this.numericValues()[0].sumValue();
      if (!cur_min.length) {
        return null;
      }
      for (var _iterator2 = _createForOfIteratorHelperLoose(this.numericValues()), _step2; !(_step2 = _iterator2()).done;) {
        var value = _step2.value;
        var child_sum = value.sumValue();
        if (!child_sum.length || JSON.stringify(child_sum[0].units) !== JSON.stringify(cur_min[0].units)) {
          return null;
        }
        if (child_sum[0].value < cur_min[0].value) {
          cur_min = child_sum;
        }
      }
      return cur_min;
    }
    // toCalcExpressionNode() {
    //   const operands: CSSMathExpressionOperation[] = [];
    //   for (const value of this.numericValues()) {
    //     const operand = value.toCalcExpressionNode();
    //     if (!operand) {
    //       continue;
    //     }
    //     operands.push(operand as CSSMathExpressionOperation);
    //   }
    //   return CSSMathExpressionOperation.createComparisonFunction(operands, CSSMathOperator.kMin);
    // }
    ;
    _proto8.buildCSSText = function buildCSSText(nested, paren_less, result) {
      result += 'min(';
      var first_iteration = true;
      for (var _iterator3 = _createForOfIteratorHelperLoose(this.numericValues()), _step3; !(_step3 = _iterator3()).done;) {
        var value = _step3.value;
        if (!first_iteration) result += ', ';
        first_iteration = false;
        result = value.buildCSSText(Nested.kYes, ParenLess.kYes, result);
      }
      result += ')';
      return result;
    };
    return CSSMathMin;
  }(CSSMathVariadic);
  // Represents the maximum of one or more CSSNumericValues.
  // @see https://drafts.css-houdini.org/css-typed-om/#cssmathsum
  var CSSMathMax = /*#__PURE__*/function (_CSSMathVariadic2) {
    _inheritsLoose(CSSMathMax, _CSSMathVariadic2);
    function CSSMathMax() {
      var _this8;
      for (var _len10 = arguments.length, args = new Array(_len10), _key10 = 0; _key10 < _len10; _key10++) {
        args[_key10] = arguments[_key10];
      }
      _this8 = _CSSMathVariadic2.call.apply(_CSSMathVariadic2, [this].concat(args)) || this;
      _this8.operator = 'max';
      return _this8;
    }
    CSSMathMax.create = function create(values) {
      var error = false;
      var final_type = typeCheck(values, CSSNumericValueType.add, error);
      return  new CSSMathMax(values, final_type);
    };
    var _proto9 = CSSMathMax.prototype;
    _proto9.clone = function clone() {
      return new CSSMathMax(this.values, this.type_);
    };
    _proto9.getType = function getType() {
      return CSSStyleValueType.kMaxType;
    };
    _proto9.sumValue = function sumValue() {
      var cur_max = this.numericValues()[0].sumValue();
      if (!cur_max.length) {
        return null;
      }
      for (var _iterator4 = _createForOfIteratorHelperLoose(this.numericValues()), _step4; !(_step4 = _iterator4()).done;) {
        var value = _step4.value;
        var child_sum = value.sumValue();
        if (!child_sum.length || JSON.stringify(child_sum[0].units) !== JSON.stringify(cur_max[0].units)) {
          return null;
        }
        if (child_sum[0].value < cur_max[0].value) {
          cur_max = child_sum;
        }
      }
      return cur_max;
    }
    // toCalcExpressionNode() {
    //   const operands: CSSMathExpressionOperation[] = [];
    //   for (const value of this.numericValues()) {
    //     const operand = value.toCalcExpressionNode();
    //     if (!operand) {
    //       continue;
    //     }
    //     operands.push(operand as CSSMathExpressionOperation);
    //   }
    //   return CSSMathExpressionOperation.createComparisonFunction(operands, CSSMathOperator.kMax);
    // }
    ;
    _proto9.buildCSSText = function buildCSSText(nested, paren_less, result) {
      result += 'max(';
      var first_iteration = true;
      for (var _iterator5 = _createForOfIteratorHelperLoose(this.numericValues()), _step5; !(_step5 = _iterator5()).done;) {
        var value = _step5.value;
        if (!first_iteration) result += ', ';
        first_iteration = false;
        result = value.buildCSSText(Nested.kYes, ParenLess.kYes, result);
      }
      result += ')';
      return result;
    };
    return CSSMathMax;
  }(CSSMathVariadic);
  /**
   * Represents the sum of one or more CSSNumericValues.
   * @see https://drafts.css-houdini.org/css-typed-om/#cssmathsum
   */
  var CSSMathSum = /*#__PURE__*/function (_CSSMathVariadic3) {
    _inheritsLoose(CSSMathSum, _CSSMathVariadic3);
    function CSSMathSum() {
      var _this9;
      for (var _len11 = arguments.length, args = new Array(_len11), _key11 = 0; _key11 < _len11; _key11++) {
        args[_key11] = arguments[_key11];
      }
      _this9 = _CSSMathVariadic3.call.apply(_CSSMathVariadic3, [this].concat(args)) || this;
      _this9.operator = 'sum';
      return _this9;
    }
    CSSMathSum.create = function create(values) {
      var error = false;
      var finalType = typeCheck(values, CSSNumericValueType.add, error);
      return  new CSSMathSum(values, finalType);
    };
    CSSMathSum.numericTypeFromUnitMap = function numericTypeFromUnitMap(units) {
      var type = new CSSNumericValueType();
      Object.keys(units).forEach(function (key) {
        var exp = units[key];
        var error = false;
        type = CSSNumericValueType.multiply(type, new CSSNumericValueType(Number(key), exp), error);
        DCHECK(!error);
      });
      return type;
    };
    CSSMathSum.canCreateNumericTypeFromSumValue = function canCreateNumericTypeFromSumValue(sum) {
      var _this10 = this;
      DCHECK(!!sum.length);
      var first_type = this.numericTypeFromUnitMap(sum[0].units);
      return sum.every(function (term) {
        var error = false;
        CSSNumericValueType.add(first_type, _this10.numericTypeFromUnitMap(term.units), error);
        return !error;
      });
    };
    var _proto10 = CSSMathSum.prototype;
    _proto10.clone = function clone() {
      return new CSSMathSum(this.values, this.type_);
    };
    _proto10.getType = function getType() {
      return CSSStyleValueType.kSumType;
    }
    // toCalcExpressionNode() {
    //   return this.toCalcExporessionNodeForVariadic(CSSMathOperator.kAdd);
    // }
    ;
    _proto10.sumValue = function sumValue() {
      var sum = [];
      this.numericValues().forEach(function (value) {
        var child_sum = value.sumValue();
        if (!child_sum.length) {
          return null;
        }
        child_sum.forEach(function (term) {
          var index = sum.findIndex(function (s) {
            return JSON.stringify(s.units) === JSON.stringify(term.units);
          });
          if (index === -1) {
            sum.push(_extends({}, term));
          } else {
            sum[index].value += term.value;
          }
        });
      });
      if (!CSSMathSum.canCreateNumericTypeFromSumValue(sum)) return null;
      return sum;
    };
    _proto10.buildCSSText = function buildCSSText(nested, paren_less, result) {
      if (paren_less == ParenLess.kNo) {
        result += nested === Nested.kYes ? '(' : 'calc(';
      }
      var values = this.numericValues();
      result = values[0].buildCSSText(Nested.kYes, ParenLess.kNo, result);
      for (var i = 1; i < values.length; i++) {
        var arg = values[i];
        if (arg.getType() === CSSStyleValueType.kNegateType) {
          result += ' - ';
          result = arg.value.buildCSSText(Nested.kYes, ParenLess.kNo, result);
        } else {
          result += ' + ';
          result = arg.buildCSSText(Nested.kYes, ParenLess.kNo, result);
        }
      }
      if (paren_less === ParenLess.kNo) {
        result += ')';
      }
      return result;
    };
    return CSSMathSum;
  }(CSSMathVariadic);
  /**
   * Represents the product of one or more CSSNumericValues.
   * @see https://drafts.css-houdini.org/css-typed-om/#cssmathproduct
   */
  var CSSMathProduct = /*#__PURE__*/function (_CSSMathVariadic4) {
    _inheritsLoose(CSSMathProduct, _CSSMathVariadic4);
    function CSSMathProduct() {
      var _this11;
      for (var _len12 = arguments.length, args = new Array(_len12), _key12 = 0; _key12 < _len12; _key12++) {
        args[_key12] = arguments[_key12];
      }
      _this11 = _CSSMathVariadic4.call.apply(_CSSMathVariadic4, [this].concat(args)) || this;
      _this11.operator = 'product';
      return _this11;
    }
    CSSMathProduct.create = function create(values) {
      var error = false;
      var finalType = typeCheck(values, CSSNumericValueType.multiply, error);
      return  new CSSMathProduct(values, finalType);
    };
    CSSMathProduct.multiplyUnitMaps = function multiplyUnitMaps(a, b) {
      Object.keys(b).forEach(function (key) {
        DCHECK_NE(b[key].value, 0);
        var old_value = key in a ? a[key].value : 0;
        if (old_value + b[key].value === 0) {
          delete a[key];
        } else {
          a[key] = old_value + b[key].value;
        }
      });
      return a;
    };
    var _proto11 = CSSMathProduct.prototype;
    _proto11.clone = function clone() {
      return new CSSMathProduct(this.values, this.type_);
    };
    _proto11.getType = function getType() {
      return CSSStyleValueType.kProductType;
    }
    // toCalcExpressionNode() {
    //   return this.toCalcExporessionNodeForVariadic(CSSMathOperator.kMultiply);
    // }
    ;
    _proto11.sumValue = function sumValue() {
      var sum = [{
        value: 1,
        units: {}
      }];
      this.numericValues().forEach(function (value) {
        var child_sum = value.sumValue();
        if (!child_sum.length) {
          return null;
        }
        var new_sum = [];
        sum.forEach(function (a) {
          child_sum.forEach(function (b) {
            new_sum.push({
              value: a.value * b.value,
              units: CSSMathProduct.multiplyUnitMaps(a.units, b.units)
            });
          });
        });
        sum = new_sum;
      });
      return sum;
    };
    _proto11.buildCSSText = function buildCSSText(nested, paren_less, result) {
      if (paren_less == ParenLess.kNo) {
        result += nested === Nested.kYes ? '(' : 'calc(';
      }
      var values = this.numericValues();
      result = values[0].buildCSSText(Nested.kYes, ParenLess.kNo, result);
      for (var i = 1; i < values.length; i++) {
        var arg = values[i];
        if (arg.getType() === CSSStyleValueType.kInvertType) {
          result += ' / ';
          result = arg.value.buildCSSText(Nested.kYes, ParenLess.kNo, result);
        } else {
          result += ' * ';
          result = arg.buildCSSText(Nested.kYes, ParenLess.kNo, result);
        }
      }
      if (paren_less === ParenLess.kNo) {
        result += ')';
      }
      return result;
    };
    return CSSMathProduct;
  }(CSSMathVariadic);
  var Opx = new CSSUnitValue(0, 'px');
  var Lpx = new CSSUnitValue(1, 'px');
  var Odeg = new CSSUnitValue(0, 'deg');

  /**
   * The CSSRGB class represents the CSS rgb()/rgba() functions.
   *
   * @see https://drafts.css-houdini.org/css-typed-om-1/#cssrgb
   */
  var CSSRGB = /*#__PURE__*/function (_CSSColorValue) {
    _inheritsLoose(CSSRGB, _CSSColorValue);
    function CSSRGB(r, g, b, alpha,
    /**
     * 'transparent' & 'none' has the same rgba data
     */
    isNone) {
      var _this;
      if (alpha === void 0) {
        alpha = 1;
      }
      if (isNone === void 0) {
        isNone = false;
      }
      _this = _CSSColorValue.call(this, 'rgb') || this;
      _this.r = void 0;
      _this.g = void 0;
      _this.b = void 0;
      _this.alpha = void 0;
      _this.isNone = void 0;
      _this.r = r;
      _this.g = g;
      _this.b = b;
      _this.alpha = alpha;
      _this.isNone = isNone;
      return _this;
    }
    var _proto = CSSRGB.prototype;
    _proto.clone = function clone() {
      return new CSSRGB(this.r, this.g, this.b, this.alpha);
    };
    _proto.buildCSSText = function buildCSSText(n, p, result) {
      return result + ("rgba(" + this.r + "," + this.g + "," + this.b + "," + this.alpha + ")");
    };
    return CSSRGB;
  }(CSSColorValue);

  /**
   * holds useful CSS-related methods.
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CSS
   *
   * * CSS Typed OM @see https://developer.mozilla.org/en-US/docs/Web/API/CSS/factory_functions
   * * register property @see https://developer.mozilla.org/en-US/docs/Web/API/CSS/RegisterProperty
   * * CSS Layout API
   */
  var CSS = {
    /**
     * <number>
     * @see https://drafts.csswg.org/css-values-4/#number-value
     */
    number: function number(n) {
      return new CSSUnitValue(n);
    },
    /**
     * <percentage>
     * @see https://drafts.csswg.org/css-values-4/#percentage-value
     */
    percent: function percent(n) {
      return new CSSUnitValue(n, '%');
    },
    /**
     * <length>
     */
    px: function px(n) {
      return new CSSUnitValue(n, 'px');
    },
    /**
     * <length>
     */
    em: function em(n) {
      return new CSSUnitValue(n, 'em');
    },
    rem: function rem(n) {
      return new CSSUnitValue(n, 'rem');
    },
    /**
     * <angle>
     */
    deg: function deg(n) {
      return new CSSUnitValue(n, 'deg');
    },
    /**
     * <angle>
     */
    grad: function grad(n) {
      return new CSSUnitValue(n, 'grad');
    },
    /**
     * <angle>
     */
    rad: function rad(n) {
      return new CSSUnitValue(n, 'rad');
    },
    /**
     * <angle>
     */
    turn: function turn(n) {
      return new CSSUnitValue(n, 'turn');
    },
    /**
     * <time>
     */
    s: function s(n) {
      return new CSSUnitValue(n, 's');
    },
    /**
     * <time>
     */
    ms: function ms(n) {
      return new CSSUnitValue(n, 'ms');
    },
    /**
     * CSS Properties & Values API
     *
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSS_Properties_and_Values_API
     * @see https://drafts.css-houdini.org/css-properties-values-api/#registering-custom-properties
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSS/RegisterProperty
     */
    registerProperty: function registerProperty(definition) {
      var name = definition.name,
        inherits = definition.inherits,
        interpolable = definition.interpolable,
        initialValue = definition.initialValue,
        syntax = definition.syntax;
      runtime.styleValueRegistry.registerMetadata({
        n: name,
        inh: inherits,
        int: interpolable,
        d: initialValue,
        syntax: syntax
      });
    },
    /**
     * CSS Layout API
     * register layout
     *
     * @see https://github.com/w3c/css-houdini-drafts/blob/main/css-layout-api/EXPLAINER.md
     * @see https://developer.mozilla.org/en-US/docs/Web/Guide/Houdini#css_layout_api
     */
    registerLayout: function registerLayout(name, clazz) {
      runtime.layoutRegistry.registerLayout(name, clazz);
    }
  };

  /**
   * CSSKeywordValue
   */
  var unsetKeywordValue = new CSSKeywordValue('unset');
  var initialKeywordValue = new CSSKeywordValue('initial');
  var inheritKeywordValue = new CSSKeywordValue('inherit');
  var keywordCache = {
    '': unsetKeywordValue,
    unset: unsetKeywordValue,
    initial: initialKeywordValue,
    inherit: inheritKeywordValue
  };
  var getOrCreateKeyword = function getOrCreateKeyword(name) {
    if (!keywordCache[name]) {
      keywordCache[name] = new CSSKeywordValue(name);
    }
    return keywordCache[name];
  };
  /**
   * CSSColor
   */
  var noneColor = new CSSRGB(0, 0, 0, 0, true);
  var transparentColor = new CSSRGB(0, 0, 0, 0);
  var getOrCreateRGBA = memoize(function (r, g, b, a) {
    return new CSSRGB(r, g, b, a);
  }, function (r, g, b, a) {
    return "rgba(" + r + "," + g + "," + b + "," + a + ")";
  });
  var getOrCreateUnitValue = memoize(function (value, unitOrName) {
    if (unitOrName === void 0) {
      unitOrName = UnitType.kNumber;
    }
    return new CSSUnitValue(value, unitOrName);
  }, function (value, unitOrName) {
    if (unitOrName === void 0) {
      unitOrName = UnitType.kNumber;
    }
    return "" + value + unitOrName;
  });

  var regexLG = /^l\s*\(\s*([\d.]+)\s*\)\s*(.*)/i;
  var regexRG = /^r\s*\(\s*([\d.]+)\s*,\s*([\d.]+)\s*,\s*([\d.]+)\s*\)\s*(.*)/i;
  var regexPR = /^p\s*\(\s*([axyn])\s*\)\s*(.*)/i;
  var regexColorStop = /[\d.]+:(#[^\s]+|[^\)]+\))/gi;
  function spaceColorStops(colorStops) {
    var _colorStops$length;
    var length = colorStops.length;
    colorStops[length - 1].length = (_colorStops$length = colorStops[length - 1].length) !== null && _colorStops$length !== void 0 ? _colorStops$length : {
      type: '%',
      value: '100'
    };
    if (length > 1) {
      var _colorStops$0$length;
      colorStops[0].length = (_colorStops$0$length = colorStops[0].length) !== null && _colorStops$0$length !== void 0 ? _colorStops$0$length : {
        type: '%',
        value: '0'
      };
    }
    var previousIndex = 0;
    var previousOffset = Number(colorStops[0].length.value);
    for (var i = 1; i < length; i++) {
      var _colorStops$i$length;
      // support '%' & 'px'
      var offset = (_colorStops$i$length = colorStops[i].length) === null || _colorStops$i$length === void 0 ? void 0 : _colorStops$i$length.value;
      if (!isNil(offset) && !isNil(previousOffset)) {
        for (var j = 1; j < i - previousIndex; j++) {
          colorStops[previousIndex + j].length = {
            type: '%',
            value: "" + (previousOffset + (Number(offset) - previousOffset) * j / (i - previousIndex))
          };
        }
        previousIndex = i;
        previousOffset = Number(offset);
      }
    }
  }
  // The position of the gradient line's starting point.
  // different from CSS side(to top) @see https://developer.mozilla.org/en-US/docs/Web/CSS/gradient/linear-gradient#values
  var SideOrCornerToDegMap = {
    left: 270 - 90,
    top: 0 - 90,
    bottom: 180 - 90,
    right: 90 - 90,
    'left top': 315 - 90,
    'top left': 315 - 90,
    'left bottom': 225 - 90,
    'bottom left': 225 - 90,
    'right top': 45 - 90,
    'top right': 45 - 90,
    'right bottom': 135 - 90,
    'bottom right': 135 - 90
  };
  var angleToDeg = memoize(function (orientation) {
    var angle;
    if (orientation.type === 'angular') {
      angle = Number(orientation.value);
    } else {
      angle = SideOrCornerToDegMap[orientation.value] || 0;
    }
    return getOrCreateUnitValue(angle, 'deg');
  });
  var positonToCSSUnitValue = memoize(function (position) {
    var cx = 50;
    var cy = 50;
    var unitX = '%';
    var unitY = '%';
    if ((position === null || position === void 0 ? void 0 : position.type) === 'position') {
      var _position$value = position.value,
        x = _position$value.x,
        y = _position$value.y;
      if ((x === null || x === void 0 ? void 0 : x.type) === 'position-keyword') {
        if (x.value === 'left') {
          cx = 0;
        } else if (x.value === 'center') {
          cx = 50;
        } else if (x.value === 'right') {
          cx = 100;
        } else if (x.value === 'top') {
          cy = 0;
        } else if (x.value === 'bottom') {
          cy = 100;
        }
      }
      if ((y === null || y === void 0 ? void 0 : y.type) === 'position-keyword') {
        if (y.value === 'left') {
          cx = 0;
        } else if (y.value === 'center') {
          cy = 50;
        } else if (y.value === 'right') {
          cx = 100;
        } else if (y.value === 'top') {
          cy = 0;
        } else if (y.value === 'bottom') {
          cy = 100;
        }
      }
      if ((x === null || x === void 0 ? void 0 : x.type) === 'px' || (x === null || x === void 0 ? void 0 : x.type) === '%' || (x === null || x === void 0 ? void 0 : x.type) === 'em') {
        unitX = x === null || x === void 0 ? void 0 : x.type;
        cx = Number(x.value);
      }
      if ((y === null || y === void 0 ? void 0 : y.type) === 'px' || (y === null || y === void 0 ? void 0 : y.type) === '%' || (y === null || y === void 0 ? void 0 : y.type) === 'em') {
        unitY = y === null || y === void 0 ? void 0 : y.type;
        cy = Number(y.value);
      }
    }
    return {
      cx: getOrCreateUnitValue(cx, unitX),
      cy: getOrCreateUnitValue(cy, unitY)
    };
  });
  var parseGradient = memoize(function (colorStr) {
    if (colorStr.indexOf('linear') > -1 || colorStr.indexOf('radial') > -1) {
      var ast = parseGradient$1(colorStr);
      return ast.map(function (_ref) {
        var type = _ref.type,
          orientation = _ref.orientation,
          colorStops = _ref.colorStops;
        spaceColorStops(colorStops);
        var steps = colorStops.map(function (colorStop) {
          // TODO: only support % for now, should calc percentage of axis length when using px/em
          return {
            offset: getOrCreateUnitValue(colorStop.length.value, '%'),
            color: colorStopToString(colorStop)
          };
        });
        if (type === 'linear-gradient') {
          return new CSSGradientValue(GradientType.LinearGradient, {
            angle: orientation ? angleToDeg(orientation) : Odeg,
            steps: steps
          });
        } else if (type === 'radial-gradient') {
          if (!orientation) {
            orientation = [{
              type: 'shape',
              value: 'circle'
            }];
          }
          if (orientation[0].type === 'shape' && orientation[0].value === 'circle') {
            var _positonToCSSUnitValu = positonToCSSUnitValue(orientation[0].at),
              cx = _positonToCSSUnitValu.cx,
              cy = _positonToCSSUnitValu.cy;
            var size;
            if (orientation[0].style) {
              var _orientation$0$style = orientation[0].style,
                _type = _orientation$0$style.type,
                value = _orientation$0$style.value;
              if (_type === 'extent-keyword') {
                size = getOrCreateKeyword(value);
              } else {
                size = getOrCreateUnitValue(value, _type);
              }
            }
            return new CSSGradientValue(GradientType.RadialGradient, {
              cx: cx,
              cy: cy,
              size: size,
              steps: steps
            });
          }
          // TODO: support ellipse shape
          // TODO: repeating-linear-gradient & repeating-radial-gradient
          // } else if (type === 'repeating-linear-gradient') {
          // } else if (type === 'repeating-radial-gradient') {
        }
      });
    }
    // legacy format, should be deprecated later
    var type = colorStr[0];
    if (colorStr[1] === '(' || colorStr[2] === '(') {
      if (type === 'l') {
        var arr = regexLG.exec(colorStr);
        if (arr) {
          var _arr$2$match;
          var steps = ((_arr$2$match = arr[2].match(regexColorStop)) === null || _arr$2$match === void 0 ? void 0 : _arr$2$match.map(function (stop) {
            return stop.split(':');
          })) || [];
          return [new CSSGradientValue(GradientType.LinearGradient, {
            angle: getOrCreateUnitValue(parseFloat(arr[1]), 'deg'),
            steps: steps.map(function (_ref2) {
              var offset = _ref2[0],
                color = _ref2[1];
              return {
                offset: getOrCreateUnitValue(Number(offset) * 100, '%'),
                color: color
              };
            })
          })];
        }
      } else if (type === 'r') {
        var parsedRadialGradient = parseRadialGradient(colorStr);
        if (parsedRadialGradient) {
          if (isString(parsedRadialGradient)) {
            colorStr = parsedRadialGradient;
          } else {
            return [new CSSGradientValue(GradientType.RadialGradient, parsedRadialGradient)];
          }
        }
      } else if (type === 'p') {
        return parsePattern(colorStr);
      }
    }
  });
  function parseRadialGradient(gradientStr) {
    var arr = regexRG.exec(gradientStr);
    if (arr) {
      var _arr$4$match;
      var steps = ((_arr$4$match = arr[4].match(regexColorStop)) === null || _arr$4$match === void 0 ? void 0 : _arr$4$match.map(function (stop) {
        return stop.split(':');
      })) || [];
      return {
        cx: getOrCreateUnitValue(50, '%'),
        cy: getOrCreateUnitValue(50, '%'),
        steps: steps.map(function (_ref3) {
          var offset = _ref3[0],
            color = _ref3[1];
          return {
            offset: getOrCreateUnitValue(Number(offset) * 100, '%'),
            color: color
          };
        })
      };
    }
    return null;
  }
  function parsePattern(patternStr) {
    var arr = regexPR.exec(patternStr);
    if (arr) {
      var repetition = arr[1];
      var src = arr[2];
      switch (repetition) {
        case 'a':
          repetition = 'repeat';
          break;
        case 'x':
          repetition = 'repeat-x';
          break;
        case 'y':
          repetition = 'repeat-y';
          break;
        case 'n':
          repetition = 'no-repeat';
          break;
        default:
          repetition = 'no-repeat';
      }
      return {
        image: src,
        // @ts-ignore
        repetition: repetition
      };
    }
    return null;
  }

  function isPattern(object) {
    return isObject(object) && !!object.image;
  }
  /**
   * @see https://github.com/WebKit/WebKit/blob/main/Source/WebCore/css/parser/CSSParser.cpp#L97
   */
  var parseColor = memoize(function (colorStr) {
    if (isPattern(colorStr)) {
      return _extends({
        repetition: 'repeat'
      }, colorStr);
    }
    if (isNil(colorStr)) {
      colorStr = '';
    }
    if (colorStr === 'transparent') {
      // transparent black
      return transparentColor;
    } else if (colorStr === 'currentColor') {
      // @see https://github.com/adobe-webplatform/Snap.svg/issues/526
      colorStr = 'black';
    }
    // support CSS gradient syntax
    var g = parseGradient(colorStr);
    if (g) {
      return g;
    }
    // constants
    var color$1 = color(colorStr);
    var rgba = [0, 0, 0, 0];
    if (color$1 !== null) {
      rgba[0] = color$1.r || 0;
      rgba[1] = color$1.g || 0;
      rgba[2] = color$1.b || 0;
      rgba[3] = color$1.opacity;
    }
    // return new CSSRGB(...rgba);
    return getOrCreateRGBA.apply(void 0, rgba);
  });
  function mergeColors(left, right) {
    // only support constant value, exclude gradient & pattern
    if (!(left instanceof CSSRGB) || !(right instanceof CSSRGB)) {
      return;
    }
    return [[Number(left.r), Number(left.g), Number(left.b), Number(left.alpha)], [Number(right.r), Number(right.g), Number(right.b), Number(right.alpha)], function (color) {
      var rgba = color.slice();
      if (rgba[3]) {
        for (var i = 0; i < 3; i++) {
          rgba[i] = Math.round(clamp(rgba[i], 0, 255));
        }
      }
      rgba[3] = clamp(rgba[3], 0, 1);
      return "rgba(" + rgba.join(',') + ")";
    }];
  }

  function parseDimension(unitRegExp, string) {
    if (isNil(string)) {
      return getOrCreateUnitValue(0, 'px');
    }
    string = ("" + string).trim().toLowerCase();
    if (isFinite(Number(string))) {
      if ('px'.search(unitRegExp) >= 0) {
        return getOrCreateUnitValue(Number(string), 'px');
      } else if ('deg'.search(unitRegExp) >= 0) {
        return getOrCreateUnitValue(Number(string), 'deg');
      }
    }
    var matchedUnits = [];
    string = string.replace(unitRegExp, function (match) {
      matchedUnits.push(match);
      return 'U' + match;
    });
    var taggedUnitRegExp = 'U(' + unitRegExp.source + ')';
    return matchedUnits.map(function (unit) {
      return getOrCreateUnitValue(Number(string.replace(new RegExp('U' + unit, 'g'), '').replace(new RegExp(taggedUnitRegExp, 'g'), '*0')), unit);
    })[0];
  }
  /**
   * <length>
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/length
   * length with only absolute unit, eg. 1px
   */
  var parseLength = memoize(function (css) {
    return parseDimension(new RegExp('px', 'g'), css);
  });
  /**
   * <percentage>
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/percentage
   */
  var parserPercentage = memoize(function (css) {
    return parseDimension(new RegExp('%', 'g'), css);
  });
  /**
   * length with absolute or relative unit,
   * eg. 1px, 0.7em, 50%, calc(100% - 200px);
   *
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/length-percentage
   */
  var parseLengthOrPercentage = memoize(function (css) {
    if (isNumber(css) || isFinite(Number(css))) {
      return getOrCreateUnitValue(Number(css), 'px');
    }
    return parseDimension(new RegExp('px|%|em|rem', 'g'), css);
  });
  var parseAngle = memoize(function (css) {
    return parseDimension(new RegExp('deg|rad|grad|turn', 'g'), css);
  });
  /**
   * merge CSSUnitValue
   *
   * @example
   * 10px + 20px = 30px
   * 10deg + 10rad
   * 10% + 20% = 30%
   */
  function mergeDimensions(left, right, target, nonNegative, index) {
    if (index === void 0) {
      index = 0;
    }
    var unit = '';
    var leftValue = left.value || 0;
    var rightValue = right.value || 0;
    var canonicalUnit = toCanonicalUnit(left.unit);
    var leftCanonicalUnitValue = left.convertTo(canonicalUnit);
    var rightCanonicalUnitValue = right.convertTo(canonicalUnit);
    if (leftCanonicalUnitValue && rightCanonicalUnitValue) {
      leftValue = leftCanonicalUnitValue.value;
      rightValue = rightCanonicalUnitValue.value;
      unit = unitTypeToString(left.unit);
    } else {
      // format '%' to 'px'
      if (CSSUnitValue.isLength(left.unit) || CSSUnitValue.isLength(right.unit)) {
        leftValue = convertPercentUnit(left, index, target);
        rightValue = convertPercentUnit(right, index, target);
        unit = 'px';
      }
    }
    // // format 'rad' 'turn' to 'deg'
    // if (CSSUnitValue.isAngle(left.unit) || CSSUnitValue.isAngle(right.unit)) {
    //   leftValue = convertAngleUnit(left);
    //   rightValue = convertAngleUnit(right);
    //   unit = 'deg';
    // }
    return [leftValue, rightValue, function (value) {
      if (nonNegative) {
        value = Math.max(value, 0);
      }
      return value + unit;
    }];
  }
  function convertAngleUnit(value) {
    var deg = 0;
    if (value.unit === UnitType.kDegrees) {
      deg = value.value;
    } else if (value.unit === UnitType.kRadians) {
      deg = rad2deg(Number(value.value));
    } else if (value.unit === UnitType.kTurns) {
      deg = turn2deg(Number(value.value));
    }
    return deg;
  }
  function parseDimensionArray(string) {
    if (isString(string)) {
      // "1px 2px 3px"
      return string.split(' ').map(function (segment) {
        return parseLengthOrPercentage(segment);
      });
    } else {
      // [1, '2px', 3]
      return string.map(function (segment) {
        return parseLengthOrPercentage(segment.toString());
      });
    }
  }
  // export function mergeDimensionList(
  //   left: CSSUnitValue[],
  //   right: CSSUnitValue[],
  //   target: IElement | null,
  // ): [number[], number[], (list: number[]) => string] | undefined {
  //   if (left.length !== right.length) {
  //     return;
  //   }
  //   const unit = left[0].unit;
  //   return [
  //     left.map((l) => l.value),
  //     right.map((l) => l.value),
  //     (values: number[]) => {
  //       return values.map((n) => new CSSUnitValue(n, unit)).join(' ');
  //     },
  //   ];
  // }
  function convertPercentUnit(valueWithUnit, vec3Index, target) {
    if (valueWithUnit.unit === UnitType.kPixels) {
      return Number(valueWithUnit.value);
    } else if (valueWithUnit.unit === UnitType.kPercentage && target) {
      var bounds = target.nodeName === Shape.GROUP ? target.getLocalBounds() : target.getGeometryBounds();
      var size = 0;
      if (!AABB.isEmpty(bounds)) {
        size = bounds.halfExtents[vec3Index] * 2;
      }
      return Number(valueWithUnit.value) / 100 * size;
    }
    return 0;
  }

  var parseParam = function parseParam(css) {
    return parseDimension(/deg|rad|grad|turn|px|%/g, css);
  };
  var supportedFilters = ['blur', 'brightness', 'drop-shadow', 'contrast', 'grayscale', 'sepia', 'saturate', 'hue-rotate', 'invert'];
  function parseFilter(filterStr) {
    if (filterStr === void 0) {
      filterStr = '';
    }
    filterStr = filterStr.toLowerCase().trim();
    if (filterStr === 'none') {
      return [];
    }
    var filterRegExp = /\s*([\w-]+)\(([^)]*)\)/g;
    var result = [];
    var match;
    var prevLastIndex = 0;
    while (match = filterRegExp.exec(filterStr)) {
      if (match.index !== prevLastIndex) {
        return [];
      }
      prevLastIndex = match.index + match[0].length;
      if (supportedFilters.indexOf(match[1]) > -1) {
        result.push({
          name: match[1],
          params: match[2].split(' ').map(function (p) {
            return parseParam(p) || parseColor(p);
          })
        });
      }
      if (filterRegExp.lastIndex === filterStr.length) {
        return result;
      }
    }
    return [];
  }

  function numberToString(x) {
    // scale(0.00000001) -> scale(0)
    // return x.toFixed(6).replace(/0+$/, '').replace(/\.$/, '');
    return x.toString();
  }
  /**
   * parse string or number to CSSUnitValue(numeric)
   *
   * eg.
   * * 0 -> CSSUnitValue(0)
   * * '2' -> CSSUnitValue(2)
   */
  var parseNumber = memoize(function (string) {
    if (typeof string === 'number') {
      return getOrCreateUnitValue(string);
    }
    if (/^\s*[-+]?(\d*\.)?\d+\s*$/.test(string)) {
      return getOrCreateUnitValue(Number(string));
    } else {
      return getOrCreateUnitValue(0);
    }
  });
  /**
   * separate string to array
   * eg.
   * * [0.5, 0.5] -> [CSSUnitValue, CSSUnitValue]
   */
  var parseNumberList = memoize(function (string) {
    if (isString(string)) {
      return string.split(' ').map(parseNumber);
    } else {
      return string.map(parseNumber);
    }
  });
  function mergeNumbers(left, right) {
    return [left, right, numberToString];
  }
  function clampedMergeNumbers(min, max) {
    return function (left, right) {
      return [left, right, function (x) {
        return numberToString(clamp(x, min, max));
      }];
    };
  }
  function mergeNumberLists(left, right) {
    if (left.length !== right.length) {
      return;
    }
    return [left, right, function (numberList) {
      return numberList;
    }];
  }

  var internalParsePath = function internalParsePath(path) {
    // empty path
    if (path === '' || Array.isArray(path) && path.length === 0) {
      return {
        absolutePath: [],
        hasArc: false,
        segments: [],
        polygons: [],
        polylines: [],
        curve: null,
        totalLength: 0,
        rect: {
          x: 0,
          y: 0,
          width: 0,
          height: 0
        }
      };
    }
    var absolutePath;
    try {
      absolutePath = normalizePath(path);
    } catch (e) {
      absolutePath = normalizePath('');
      console.error("[g]: Invalid SVG Path definition: " + path);
    }
    var hasArc = hasArcOrBezier(absolutePath);
    var _extractPolygons = extractPolygons(absolutePath),
      polygons = _extractPolygons.polygons,
      polylines = _extractPolygons.polylines;
    // for later use
    var segments = path2Segments(absolutePath);
    // Only calculate bbox here since we don't need length now.
    var _getPathBBox = getPathBBox(segments, 0),
      x = _getPathBBox.x,
      y = _getPathBBox.y,
      width = _getPathBBox.width,
      height = _getPathBBox.height;
    return {
      absolutePath: absolutePath,
      hasArc: hasArc,
      segments: segments,
      polygons: polygons,
      polylines: polylines,
      // curve,
      // Delay the calculation of length.
      totalLength: 0,
      rect: {
        x: Number.isFinite(x) ? x : 0,
        y: Number.isFinite(y) ? y : 0,
        width: Number.isFinite(width) ? width : 0,
        height: Number.isFinite(height) ? height : 0
      }
    };
  };
  var memoizedParsePath = memoize(internalParsePath);
  function parsePath(path, object) {
    var result = isString(path) ? memoizedParsePath(path) : internalParsePath(path);
    if (object) {
      object.parsedStyle.defX = result.rect.x;
      object.parsedStyle.defY = result.rect.y;
    }
    return result;
  }
  function mergePaths(left, right, object) {
    var curve1 = left.curve;
    var curve2 = right.curve;
    if (!curve1 || curve1.length === 0) {
      // convert to curves to do morphing & picking later
      // @see http://thednp.github.io/kute.js/svgCubicMorph.html
      curve1 = path2Curve(left.absolutePath, false);
      left.curve = curve1;
    }
    if (!curve2 || curve2.length === 0) {
      curve2 = path2Curve(right.absolutePath, false);
      right.curve = curve2;
    }
    var curves = [curve1, curve2];
    if (curve1.length !== curve2.length) {
      curves = equalizeSegments(curve1, curve2);
    }
    var curve0 = getDrawDirection(curves[0]) !== getDrawDirection(curves[1]) ? reverseCurve(curves[0]) : clonePath(curves[0]);
    return [curve0, getRotatedCurve(curves[1], curve0), function (pathArray) {
      // need converting to path string?
      return pathArray;
    }];
  }

  /**
   * @see https://developer.mozilla.org/zh-CN/docs/Web/SVG/Attribute/points
   *
   * @example
   * points="100,10 250,150 200,110"
   */
  function parsePoints(pointsOrStr, object) {
    var points;
    if (isString(pointsOrStr)) {
      points = pointsOrStr.split(' ').map(function (pointStr) {
        var _pointStr$split = pointStr.split(','),
          x = _pointStr$split[0],
          y = _pointStr$split[1];
        return [Number(x), Number(y)];
      });
    } else {
      points = pointsOrStr;
    }
    var segments = [];
    var tempLength = 0;
    var segmentT;
    var segmentL;
    var totalLength = polyline.length(points);
    points.forEach(function (p, i) {
      if (points[i + 1]) {
        segmentT = [0, 0];
        segmentT[0] = tempLength / totalLength;
        segmentL = line.length(p[0], p[1], points[i + 1][0], points[i + 1][1]);
        tempLength += segmentL;
        segmentT[1] = tempLength / totalLength;
        segments.push(segmentT);
      }
    });
    var minX = Math.min.apply(Math, points.map(function (point) {
      return point[0];
    }));
    var minY = Math.min.apply(Math, points.map(function (point) {
      return point[1];
    }));
    if (object) {
      object.parsedStyle.defX = minX;
      object.parsedStyle.defY = minY;
    }
    return {
      points: points,
      totalLength: totalLength,
      segments: segments
    };
  }

  var _ = null;
  function cast(pattern) {
    return function (contents) {
      var i = 0;
      return pattern.map(function (x) {
        return x === _ ? contents[i++] : x;
      });
    };
  }
  function id(x) {
    return x;
  }
  // type: [argTypes, convertTo3D, convertTo2D]
  // In the argument types string, lowercase characters represent optional arguments
  var transformFunctions = {
    // @ts-ignore
    matrix: ['NNNNNN', [_, _, 0, 0, _, _, 0, 0, 0, 0, 1, 0, _, _, 0, 1], id],
    matrix3d: ['NNNNNNNNNNNNNNNN', id],
    rotate: ['A'],
    rotatex: ['A'],
    rotatey: ['A'],
    rotatez: ['A'],
    rotate3d: ['NNNA'],
    perspective: ['L'],
    scale: ['Nn', cast([_, _, new CSSUnitValue(1)]), id],
    scalex: ['N', cast([_, new CSSUnitValue(1), new CSSUnitValue(1)]), cast([_, new CSSUnitValue(1)])],
    scaley: ['N', cast([new CSSUnitValue(1), _, new CSSUnitValue(1)]), cast([new CSSUnitValue(1), _])],
    scalez: ['N', cast([new CSSUnitValue(1), new CSSUnitValue(1), _])],
    scale3d: ['NNN', id],
    skew: ['Aa', null, id],
    skewx: ['A', null, cast([_, Odeg])],
    skewy: ['A', null, cast([Odeg, _])],
    translate: ['Tt', cast([_, _, Opx]), id],
    translatex: ['T', cast([_, Opx, Opx]), cast([_, Opx])],
    translatey: ['T', cast([Opx, _, Opx]), cast([Opx, _])],
    translatez: ['L', cast([Opx, Opx, _])],
    translate3d: ['TTL', id]
  };
  /**
   * none
   * scale(1) scale(1, 2)
   * scaleX(1)
   */
  function parseTransform(string) {
    string = (string || 'none').toLowerCase().trim();
    if (string === 'none') {
      return [];
    }
    var transformRegExp = /\s*(\w+)\(([^)]*)\)/g;
    var result = [];
    var match;
    var prevLastIndex = 0;
    while (match = transformRegExp.exec(string)) {
      if (match.index !== prevLastIndex) {
        return [];
      }
      prevLastIndex = match.index + match[0].length;
      var functionName = match[1]; // scale
      var functionData = transformFunctions[functionName]; // scale(1, 2)
      if (!functionData) {
        // invalid, eg. scale()
        return [];
      }
      var args = match[2].split(','); // 1,2
      var argTypes = functionData[0]; // Nn
      if (argTypes.length < args.length) {
        // scale(N, n)
        return [];
      }
      var parsedArgs = [];
      for (var i = 0; i < argTypes.length; i++) {
        var arg = args[i];
        var type = argTypes[i];
        var parsedArg = void 0;
        if (!arg) {
          // @ts-ignore
          parsedArg = {
            a: Odeg,
            n: parsedArgs[0],
            t: Opx
          }[type];
        } else {
          // @ts-ignore
          parsedArg = {
            A: function A(s) {
              return s.trim() === '0' ? Odeg : parseAngle(s);
            },
            N: parseNumber,
            T: parseLengthOrPercentage,
            L: parseLength
          }[type.toUpperCase()](arg);
        }
        if (parsedArg === undefined) {
          return [];
        }
        parsedArgs.push(parsedArg);
      }
      result.push({
        t: functionName,
        d: parsedArgs
      }); // { t: scale, d: [1, 2] }
      if (transformRegExp.lastIndex === string.length) {
        return result;
      }
    }
    return [];
  }
  function convertItemToMatrix(item) {
    var x;
    var y;
    var z;
    var angle;
    switch (item.t) {
      case 'rotatex':
        angle = deg2rad(convertAngleUnit(item.d[0]));
        return [1, 0, 0, 0, 0, Math.cos(angle), Math.sin(angle), 0, 0, -Math.sin(angle), Math.cos(angle), 0, 0, 0, 0, 1];
      case 'rotatey':
        angle = deg2rad(convertAngleUnit(item.d[0]));
        return [Math.cos(angle), 0, -Math.sin(angle), 0, 0, 1, 0, 0, Math.sin(angle), 0, Math.cos(angle), 0, 0, 0, 0, 1];
      case 'rotate':
      case 'rotatez':
        angle = deg2rad(convertAngleUnit(item.d[0]));
        return [Math.cos(angle), Math.sin(angle), 0, 0, -Math.sin(angle), Math.cos(angle), 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'rotate3d':
        x = item.d[0].value;
        y = item.d[1].value;
        z = item.d[2].value;
        angle = deg2rad(convertAngleUnit(item.d[3]));
        var sqrLength = x * x + y * y + z * z;
        if (sqrLength === 0) {
          x = 1;
          y = 0;
          z = 0;
        } else if (sqrLength !== 1) {
          var length = Math.sqrt(sqrLength);
          x /= length;
          y /= length;
          z /= length;
        }
        var s = Math.sin(angle / 2);
        var sc = s * Math.cos(angle / 2);
        var sq = s * s;
        return [1 - 2 * (y * y + z * z) * sq, 2 * (x * y * sq + z * sc), 2 * (x * z * sq - y * sc), 0, 2 * (x * y * sq - z * sc), 1 - 2 * (x * x + z * z) * sq, 2 * (y * z * sq + x * sc), 0, 2 * (x * z * sq + y * sc), 2 * (y * z * sq - x * sc), 1 - 2 * (x * x + y * y) * sq, 0, 0, 0, 0, 1];
      case 'scale':
        return [item.d[0].value, 0, 0, 0, 0, item.d[1].value, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'scalex':
        return [item.d[0].value, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'scaley':
        return [1, 0, 0, 0, 0, item.d[0].value, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'scalez':
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, item.d[0].value, 0, 0, 0, 0, 1];
      case 'scale3d':
        return [item.d[0].value, 0, 0, 0, 0, item.d[1].value, 0, 0, 0, 0, item.d[2].value, 0, 0, 0, 0, 1];
      case 'skew':
        var xAngle = deg2rad(convertAngleUnit(item.d[0]));
        var yAngle = deg2rad(convertAngleUnit(item.d[1]));
        return [1, Math.tan(yAngle), 0, 0, Math.tan(xAngle), 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'skewx':
        angle = deg2rad(convertAngleUnit(item.d[0]));
        return [1, 0, 0, 0, Math.tan(angle), 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'skewy':
        angle = deg2rad(convertAngleUnit(item.d[0]));
        return [1, Math.tan(angle), 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'translate':
        // TODO: pass target
        x = convertPercentUnit(item.d[0], 0, null) || 0;
        y = convertPercentUnit(item.d[1], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, x, y, 0, 1];
      case 'translatex':
        x = convertPercentUnit(item.d[0], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, x, 0, 0, 1];
      case 'translatey':
        y = convertPercentUnit(item.d[0], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, y, 0, 1];
      case 'translatez':
        z = convertPercentUnit(item.d[0], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, z, 1];
      case 'translate3d':
        x = convertPercentUnit(item.d[0], 0, null) || 0;
        y = convertPercentUnit(item.d[1], 0, null) || 0;
        z = convertPercentUnit(item.d[2], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, x, y, z, 1];
      case 'perspective':
        var t = convertPercentUnit(item.d[0], 0, null) || 0;
        var p = t ? -1 / t : 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, p, 0, 0, 0, 1];
      case 'matrix':
        return [item.d[0].value, item.d[1].value, 0, 0, item.d[2].value, item.d[3].value, 0, 0, 0, 0, 1, 0, item.d[4].value, item.d[5].value, 0, 1];
      case 'matrix3d':
        return item.d.map(function (d) {
          return d.value;
        });
    }
  }
  function multiplyMatrices(a, b) {
    return [a[0] * b[0] + a[4] * b[1] + a[8] * b[2] + a[12] * b[3], a[1] * b[0] + a[5] * b[1] + a[9] * b[2] + a[13] * b[3], a[2] * b[0] + a[6] * b[1] + a[10] * b[2] + a[14] * b[3], a[3] * b[0] + a[7] * b[1] + a[11] * b[2] + a[15] * b[3], a[0] * b[4] + a[4] * b[5] + a[8] * b[6] + a[12] * b[7], a[1] * b[4] + a[5] * b[5] + a[9] * b[6] + a[13] * b[7], a[2] * b[4] + a[6] * b[5] + a[10] * b[6] + a[14] * b[7], a[3] * b[4] + a[7] * b[5] + a[11] * b[6] + a[15] * b[7], a[0] * b[8] + a[4] * b[9] + a[8] * b[10] + a[12] * b[11], a[1] * b[8] + a[5] * b[9] + a[9] * b[10] + a[13] * b[11], a[2] * b[8] + a[6] * b[9] + a[10] * b[10] + a[14] * b[11], a[3] * b[8] + a[7] * b[9] + a[11] * b[10] + a[15] * b[11], a[0] * b[12] + a[4] * b[13] + a[8] * b[14] + a[12] * b[15], a[1] * b[12] + a[5] * b[13] + a[9] * b[14] + a[13] * b[15], a[2] * b[12] + a[6] * b[13] + a[10] * b[14] + a[14] * b[15], a[3] * b[12] + a[7] * b[13] + a[11] * b[14] + a[15] * b[15]];
  }
  function convertToMatrix(transformList) {
    if (transformList.length === 0) {
      return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
    }
    return transformList.map(convertItemToMatrix).reduce(multiplyMatrices);
  }
  function makeMatrixDecomposition(transformList) {
    var translate = [0, 0, 0];
    var scale = [1, 1, 1];
    var skew = [0, 0, 0];
    var perspective = [0, 0, 0, 1];
    var quaternion = [0, 0, 0, 1];
    // @ts-ignore
    decomposeMat4(convertToMatrix(transformList), translate, scale, skew, perspective, quaternion);
    return [[translate, scale, skew, quaternion, perspective]];
  }
  var composeMatrix = function () {
    function multiply(a, b) {
      var result = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0]];
      for (var i = 0; i < 4; i++) {
        for (var j = 0; j < 4; j++) {
          for (var k = 0; k < 4; k++) {
            result[i][j] += b[i][k] * a[k][j];
          }
        }
      }
      return result;
    }
    function is2D(m) {
      return m[0][2] == 0 && m[0][3] == 0 && m[1][2] == 0 && m[1][3] == 0 && m[2][0] == 0 && m[2][1] == 0 && m[2][2] == 1 && m[2][3] == 0 && m[3][2] == 0 && m[3][3] == 1;
    }
    function composeMatrix(translate, scale, skew, quat, perspective) {
      var matrix = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]];
      for (var i = 0; i < 4; i++) {
        matrix[i][3] = perspective[i];
      }
      for (var _i = 0; _i < 3; _i++) {
        for (var j = 0; j < 3; j++) {
          matrix[3][_i] += translate[j] * matrix[j][_i];
        }
      }
      var x = quat[0],
        y = quat[1],
        z = quat[2],
        w = quat[3];
      var rotMatrix = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]];
      rotMatrix[0][0] = 1 - 2 * (y * y + z * z);
      rotMatrix[0][1] = 2 * (x * y - z * w);
      rotMatrix[0][2] = 2 * (x * z + y * w);
      rotMatrix[1][0] = 2 * (x * y + z * w);
      rotMatrix[1][1] = 1 - 2 * (x * x + z * z);
      rotMatrix[1][2] = 2 * (y * z - x * w);
      rotMatrix[2][0] = 2 * (x * z - y * w);
      rotMatrix[2][1] = 2 * (y * z + x * w);
      rotMatrix[2][2] = 1 - 2 * (x * x + y * y);
      matrix = multiply(matrix, rotMatrix);
      var temp = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]];
      if (skew[2]) {
        temp[2][1] = skew[2];
        matrix = multiply(matrix, temp);
      }
      if (skew[1]) {
        temp[2][1] = 0;
        temp[2][0] = skew[0];
        matrix = multiply(matrix, temp);
      }
      if (skew[0]) {
        temp[2][0] = 0;
        temp[1][0] = skew[0];
        matrix = multiply(matrix, temp);
      }
      for (var _i2 = 0; _i2 < 3; _i2++) {
        for (var _j = 0; _j < 3; _j++) {
          matrix[_i2][_j] *= scale[_i2];
        }
      }
      if (is2D(matrix)) {
        return [matrix[0][0], matrix[0][1], matrix[1][0], matrix[1][1], matrix[3][0], matrix[3][1]];
      }
      return matrix[0].concat(matrix[1], matrix[2], matrix[3]);
    }
    return composeMatrix;
  }();
  function numberToLongString(x) {
    return x.toFixed(6).replace('.000000', '');
  }
  function mergeMatrices(left, right) {
    var leftArgs;
    var rightArgs;
    // @ts-ignore
    if (left.decompositionPair !== right) {
      // @ts-ignore
      left.decompositionPair = right;
      // @ts-ignore
      leftArgs = makeMatrixDecomposition(left);
    }
    // @ts-ignore
    if (right.decompositionPair !== left) {
      // @ts-ignore
      right.decompositionPair = left;
      // @ts-ignore
      rightArgs = makeMatrixDecomposition(right);
    }
    if (leftArgs[0] === null || rightArgs[0] === null) return [
    // @ts-ignore
    [false],
    // @ts-ignore
    [true],
    // @ts-ignore
    function (x) {
      return x ? right[0].d : left[0].d;
    }];
    leftArgs[0].push(0);
    rightArgs[0].push(1);
    return [leftArgs, rightArgs,
    // @ts-ignore
    function (list) {
      // @ts-ignore
      var q = quat(leftArgs[0][3], rightArgs[0][3], list[5]);
      var mat = composeMatrix(list[0], list[1], list[2], q, list[4]);
      var stringifiedArgs = mat.map(numberToLongString).join(',');
      return stringifiedArgs;
    }];
  }
  function dot$2(v1, v2) {
    var result = 0;
    for (var i = 0; i < v1.length; i++) {
      result += v1[i] * v2[i];
    }
    return result;
  }
  function quat(fromQ, toQ, f) {
    var product = dot$2(fromQ, toQ);
    product = clamp(product, -1.0, 1.0);
    var quat = [];
    if (product === 1.0) {
      quat = fromQ;
    } else {
      var theta = Math.acos(product);
      var w = Math.sin(f * theta) * 1 / Math.sqrt(1 - product * product);
      for (var i = 0; i < 4; i++) {
        quat.push(fromQ[i] * (Math.cos(f * theta) - product * w) + toQ[i] * w);
      }
    }
    return quat;
  }
  // scalex/y/z -> scale
  function typeTo2D(type) {
    return type.replace(/[xy]/, '');
  }
  // scalex/y/z -> scale3d
  function typeTo3D(type) {
    return type.replace(/(x|y|z|3d)?$/, '3d');
  }
  var isMatrixOrPerspective = function isMatrixOrPerspective(lt, rt) {
    return lt === 'perspective' && rt === 'perspective' || (lt === 'matrix' || lt === 'matrix3d') && (rt === 'matrix' || rt === 'matrix3d');
  };
  function mergeTransforms(left, right, target) {
    var flipResults = false;
    // padding empty transform, eg. merge 'scale(10)' with 'none' -> scale(1)
    if (!left.length || !right.length) {
      if (!left.length) {
        flipResults = true;
        left = right;
        right = [];
      }
      var _loop = function _loop(i) {
        var _left$i = left[i],
          type = _left$i.t,
          args = _left$i.d;
        // none -> scale(1)/translateX(0)
        var defaultValue = type.substring(0, 5) === 'scale' ? 1 : 0;
        right.push({
          t: type,
          d: args.map(function (arg) {
            if (typeof arg === 'number') {
              return getOrCreateUnitValue(defaultValue);
            }
            return getOrCreateUnitValue(defaultValue, arg.unit);
            //   {
            //     unit: arg.unit,
            //     value: defaultValue,
            //   };
          })
        });
      };
      for (var i = 0; i < left.length; i++) {
        _loop(i);
      }
    }
    var leftResult = [];
    var rightResult = [];
    var types = [];
    // merge matrix() with matrix3d()
    if (left.length !== right.length) {
      var merged = mergeMatrices(left, right);
      // @ts-ignore
      leftResult = [merged[0]];
      // @ts-ignore
      rightResult = [merged[1]];
      types = [['matrix', [merged[2]]]];
    } else {
      for (var _i3 = 0; _i3 < left.length; _i3++) {
        var leftType = left[_i3].t;
        var rightType = right[_i3].t;
        var leftArgs = left[_i3].d;
        var rightArgs = right[_i3].d;
        var leftFunctionData = transformFunctions[leftType];
        var rightFunctionData = transformFunctions[rightType];
        var type = void 0;
        if (isMatrixOrPerspective(leftType, rightType)) {
          var _merged = mergeMatrices([left[_i3]], [right[_i3]]);
          // @ts-ignore
          leftResult.push(_merged[0]);
          // @ts-ignore
          rightResult.push(_merged[1]);
          types.push(['matrix', [_merged[2]]]);
          continue;
        } else if (leftType === rightType) {
          type = leftType;
        } else if (leftFunctionData[2] && rightFunctionData[2] && typeTo2D(leftType) === typeTo2D(rightType)) {
          type = typeTo2D(leftType);
          // @ts-ignore
          leftArgs = leftFunctionData[2](leftArgs);
          // @ts-ignore
          rightArgs = rightFunctionData[2](rightArgs);
        } else if (leftFunctionData[1] && rightFunctionData[1] && typeTo3D(leftType) === typeTo3D(rightType)) {
          type = typeTo3D(leftType);
          // @ts-ignore
          leftArgs = leftFunctionData[1](leftArgs);
          // @ts-ignore
          rightArgs = rightFunctionData[1](rightArgs);
        } else {
          var _merged2 = mergeMatrices(left, right);
          // @ts-ignore
          leftResult = [_merged2[0]];
          // @ts-ignore
          rightResult = [_merged2[1]];
          types = [['matrix', [_merged2[2]]]];
          break;
        }
        var leftArgsCopy = [];
        var rightArgsCopy = [];
        var stringConversions = [];
        for (var j = 0; j < leftArgs.length; j++) {
          // const merge = leftArgs[j].unit === UnitType.kNumber ? mergeDimensions : mergeDimensions;
          var _merged3 = mergeDimensions(leftArgs[j], rightArgs[j], target, false, j);
          leftArgsCopy[j] = _merged3[0];
          rightArgsCopy[j] = _merged3[1];
          stringConversions.push(_merged3[2]);
        }
        leftResult.push(leftArgsCopy);
        rightResult.push(rightArgsCopy);
        types.push([type, stringConversions]);
      }
    }
    if (flipResults) {
      var tmp = leftResult;
      leftResult = rightResult;
      rightResult = tmp;
    }
    return [leftResult, rightResult, function (list) {
      return list.map(function (args, i) {
        var stringifiedArgs = args.map(function (arg, j) {
          return types[i][1][j](arg);
        }).join(',');
        if (types[i][0] === 'matrix' && stringifiedArgs.split(',').length === 16) {
          types[i][0] = 'matrix3d';
        }
        if (types[i][0] === 'matrix3d' && stringifiedArgs.split(',').length === 6) {
          types[i][0] = 'matrix';
        }
        return types[i][0] + '(' + stringifiedArgs + ')';
      }).join(' ');
    }];
  }

  /**
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/transform-origin
   * eg. 'center' 'top left' '50px 50px'
   */
  var parseTransformOrigin = memoize(function (value) {
    if (isString(value)) {
      if (value === 'text-anchor') {
        return [getOrCreateUnitValue(0, 'px'), getOrCreateUnitValue(0, 'px')];
      }
      var values = value.split(' ');
      if (values.length === 1) {
        if (values[0] === 'top' || values[0] === 'bottom') {
          // 'top' -> 'center top'
          values[1] = values[0];
          values[0] = 'center';
        } else {
          // '50px' -> '50px center'
          values[1] = 'center';
        }
      }
      if (values.length !== 2) {
        return null;
      }
      // eg. center bottom
      return [parseLengthOrPercentage(convertKeyword2Percent(values[0])), parseLengthOrPercentage(convertKeyword2Percent(values[1]))];
    } else {
      return [getOrCreateUnitValue(value[0] || 0, 'px'), getOrCreateUnitValue(value[1] || 0, 'px')];
    }
  });
  function convertKeyword2Percent(keyword) {
    if (keyword === 'center') {
      return '50%';
    } else if (keyword === 'left' || keyword === 'top') {
      return '0';
    } else if (keyword === 'right' || keyword === 'bottom') {
      return '100%';
    }
    return keyword;
  }

  var CSSPropertyAngle = /*#__PURE__*/function () {
    function CSSPropertyAngle() {
      this.parser = parseAngle;
      this.mixer = mergeNumbers;
    }
    var _proto = CSSPropertyAngle.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      return convertAngleUnit(parsed);
    };
    return CSSPropertyAngle;
  }();

  /**
   * clipPath / textPath / offsetPath
   */
  var CSSPropertyClipPath = /*#__PURE__*/function () {
    function CSSPropertyClipPath() {}
    var _proto = CSSPropertyClipPath.prototype;
    _proto.calculator = function calculator(name, oldPath, newPath, object) {
      // unset
      if (newPath instanceof CSSKeywordValue) {
        newPath = null;
      }
      runtime.sceneGraphService.updateDisplayObjectDependency(name, oldPath, newPath, object);
      if (name === 'clipPath') {
        // should affect children
        object.forEach(function (leaf) {
          if (leaf.childNodes.length === 0) {
            runtime.sceneGraphService.dirtifyToRoot(leaf);
          }
        });
      }
      return newPath;
    };
    return CSSPropertyClipPath;
  }();

  var CSSPropertyColor = /*#__PURE__*/function () {
    function CSSPropertyColor() {
      this.parser = parseColor;
      this.mixer = mergeColors;
    }
    var _proto = CSSPropertyColor.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      if (parsed instanceof CSSKeywordValue) {
        // 'unset' 'none'
        return parsed.value === 'none' ? noneColor : transparentColor;
      }
      return parsed;
    };
    return CSSPropertyColor;
  }();

  var CSSPropertyFilter = /*#__PURE__*/function () {
    function CSSPropertyFilter() {
      this.parser = parseFilter;
    }
    var _proto = CSSPropertyFilter.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed) {
      // unset or none
      if (parsed instanceof CSSKeywordValue) {
        return [];
      }
      return parsed;
    };
    return CSSPropertyFilter;
  }();

  function getFontSize(object) {
    var fontSize = object.parsedStyle.fontSize;
    return isNil(fontSize) ? null : fontSize;
  }
  /**
   * <length> & <percentage>
   */
  var CSSPropertyLengthOrPercentage = /*#__PURE__*/function () {
    function CSSPropertyLengthOrPercentage() {
      this.parser = parseLengthOrPercentage;
      this.mixer = mergeNumbers;
    }
    var _proto = CSSPropertyLengthOrPercentage.prototype;
    /**
     * according to parent's bounds
     *
     * @example
     * CSS.percent(50) -> CSS.px(0.5 * parent.width)
     */
    _proto.calculator = function calculator(name, oldParsed, computed, object, registry) {
      if (CSSUnitValue.isRelativeUnit(computed.unit)) {
        if (computed.unit === UnitType.kPercentage) {
          // TODO: merge dimensions
          return 0;
        } else if (computed.unit === UnitType.kEms) {
          if (object.parentNode) {
            var fontSize = getFontSize(object.parentNode);
            if (fontSize) {
              fontSize *= computed.value;
              return fontSize;
            } else {
              registry.addUnresolveProperty(object, name);
            }
          } else {
            registry.addUnresolveProperty(object, name);
          }
          return 0;
        } else if (computed.unit === UnitType.kRems) {
          var _object$ownerDocument;
          if (object === null || object === void 0 ? void 0 : (_object$ownerDocument = object.ownerDocument) === null || _object$ownerDocument === void 0 ? void 0 : _object$ownerDocument.documentElement) {
            var _fontSize = getFontSize(object.ownerDocument.documentElement);
            if (_fontSize) {
              _fontSize *= computed.value;
              return _fontSize;
            } else {
              registry.addUnresolveProperty(object, name);
            }
          } else {
            registry.addUnresolveProperty(object, name);
          }
          return 0;
        }
      } else {
        // remove listener if exists
        // registry.unregisterParentGeometryBoundsChangedHandler(object, name);
        // return absolute value
        return computed.value;
      }
    };
    return CSSPropertyLengthOrPercentage;
  }();

  /**
   * format to Tuple2<CSSUnitValue>
   *
   * @example
   * rect.style.lineDash = 10;
   * rect.style.lineDash = [10, 10];
   * rect.style.lineDash = '10 10';
   */
  var CSSPropertyLengthOrPercentage12 = /*#__PURE__*/function () {
    function CSSPropertyLengthOrPercentage12() {
      this.mixer = mergeNumberLists;
    }
    var _proto = CSSPropertyLengthOrPercentage12.prototype;
    _proto.parser = function parser(radius) {
      var parsed = parseDimensionArray(isNumber(radius) ? [radius] : radius);
      var formatted;
      if (parsed.length === 1) {
        formatted = [parsed[0], parsed[0]];
      } else {
        formatted = [parsed[0], parsed[1]];
      }
      return formatted;
    };
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.map(function (c) {
        return c.value;
      });
    };
    return CSSPropertyLengthOrPercentage12;
  }();

  /**
   * used in rounded rect
   *
   * @example
   * rect.style.radius = 10;
   * rect.style.radius = '10 10';
   * rect.style.radius = '10 10 10 10';
   */
  var CSSPropertyLengthOrPercentage14 = /*#__PURE__*/function () {
    function CSSPropertyLengthOrPercentage14() {
      this.mixer = mergeNumberLists;
    }
    var _proto = CSSPropertyLengthOrPercentage14.prototype;
    _proto.parser = function parser(radius) {
      var parsed = parseDimensionArray(isNumber(radius) ? [radius] : radius);
      var formatted;
      // format to Tuple<CSSUnitValue>
      if (parsed.length === 1) {
        formatted = [parsed[0], parsed[0], parsed[0], parsed[0]];
      } else if (parsed.length === 2) {
        formatted = [parsed[0], parsed[1], parsed[0], parsed[1]];
      } else if (parsed.length === 3) {
        formatted = [parsed[0], parsed[1], parsed[2], parsed[1]];
      } else {
        formatted = [parsed[0], parsed[1], parsed[2], parsed[3]];
      }
      return formatted;
    };
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.map(function (c) {
        return c.value;
      });
    };
    return CSSPropertyLengthOrPercentage14;
  }();

  /**
   * local position
   */
  var CSSPropertyLocalPosition = /*#__PURE__*/function (_CSSPropertyLengthOrP) {
    _inheritsLoose(CSSPropertyLocalPosition, _CSSPropertyLengthOrP);
    function CSSPropertyLocalPosition() {
      return _CSSPropertyLengthOrP.apply(this, arguments) || this;
    }
    var _proto = CSSPropertyLocalPosition.prototype;
    /**
     * update local position
     */
    _proto.postProcessor = function postProcessor(object) {
      var x;
      var y;
      var z;
      switch (object.nodeName) {
        case Shape.CIRCLE:
        case Shape.ELLIPSE:
          var _object$parsedStyle = object.parsedStyle,
            cx = _object$parsedStyle.cx,
            cy = _object$parsedStyle.cy;
          if (!isNil(cx)) {
            x = cx;
          }
          if (!isNil(cy)) {
            y = cy;
          }
          break;
        case Shape.LINE:
          var _object$parsedStyle2 = object.parsedStyle,
            x1 = _object$parsedStyle2.x1,
            x2 = _object$parsedStyle2.x2,
            y1 = _object$parsedStyle2.y1,
            y2 = _object$parsedStyle2.y2;
          var minX = Math.min(x1, x2);
          var minY = Math.min(y1, y2);
          x = minX;
          y = minY;
          z = 0;
          break;
        case Shape.RECT:
        case Shape.IMAGE:
        case Shape.GROUP:
        case Shape.HTML:
        case Shape.TEXT:
        case Shape.MESH:
          if (!isNil(object.parsedStyle.x)) {
            x = object.parsedStyle.x;
          }
          if (!isNil(object.parsedStyle.y)) {
            y = object.parsedStyle.y;
          }
          break;
      }
      if (object.nodeName !== Shape.PATH && object.nodeName !== Shape.POLYLINE && object.nodeName !== Shape.POLYGON) {
        object.parsedStyle.defX = x || 0;
        object.parsedStyle.defY = y || 0;
      }
      var needResetLocalPosition = !isNil(x) || !isNil(y) || !isNil(z);
      if (needResetLocalPosition) {
        var _object$getLocalPosit = object.getLocalPosition(),
          ox = _object$getLocalPosit[0],
          oy = _object$getLocalPosit[1],
          oz = _object$getLocalPosit[2];
        object.setLocalPosition(isNil(x) ? ox : x, isNil(y) ? oy : y, isNil(z) ? oz : z);
      }
    };
    return CSSPropertyLocalPosition;
  }(CSSPropertyLengthOrPercentage);

  var CSSPropertyMarker = /*#__PURE__*/function () {
    function CSSPropertyMarker() {}
    var _proto = CSSPropertyMarker.prototype;
    _proto.calculator = function calculator(name, oldMarker, newMarker, object) {
      var _newMarker;
      // unset
      if (newMarker instanceof CSSKeywordValue) {
        newMarker = null;
      }
      var cloned = (_newMarker = newMarker) === null || _newMarker === void 0 ? void 0 : _newMarker.cloneNode(true);
      if (cloned) {
        // FIXME: SVG should not inherit parent's style, add a flag here
        cloned.style.isMarker = true;
      }
      return cloned;
    };
    return CSSPropertyMarker;
  }();

  var CSSPropertyNumber = /*#__PURE__*/function () {
    function CSSPropertyNumber() {
      this.mixer = mergeNumbers;
      this.parser = parseNumber;
    }
    var _proto = CSSPropertyNumber.prototype;
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.value;
    };
    return CSSPropertyNumber;
  }();

  var CSSPropertyOffsetDistance = /*#__PURE__*/function () {
    function CSSPropertyOffsetDistance() {
      this.parser = parseNumber;
      this.mixer = clampedMergeNumbers(0, 1);
    }
    var _proto = CSSPropertyOffsetDistance.prototype;
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.value;
    };
    _proto.postProcessor = function postProcessor(object) {
      var _object$parsedStyle = object.parsedStyle,
        offsetPath = _object$parsedStyle.offsetPath,
        offsetDistance = _object$parsedStyle.offsetDistance;
      if (!offsetPath) {
        return;
      }
      var nodeName = offsetPath.nodeName;
      if (nodeName === Shape.LINE || nodeName === Shape.PATH || nodeName === Shape.POLYLINE) {
        // set position in world space
        var point = offsetPath.getPoint(offsetDistance);
        if (point) {
          object.parsedStyle.defX = point.x;
          object.parsedStyle.defY = point.y;
          object.setLocalPosition(point.x, point.y);
        }
      }
    };
    return CSSPropertyOffsetDistance;
  }();

  /**
   * opacity
   */
  var CSSPropertyOpacity = /*#__PURE__*/function () {
    function CSSPropertyOpacity() {
      this.parser = parseNumber;
      this.mixer = clampedMergeNumbers(0, 1);
    }
    var _proto = CSSPropertyOpacity.prototype;
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.value;
    };
    return CSSPropertyOpacity;
  }();

  var CSSPropertyPath = /*#__PURE__*/function () {
    function CSSPropertyPath() {
      this.parser = parsePath;
      this.mixer = mergePaths;
    }
    var _proto = CSSPropertyPath.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed) {
      // unset
      if (parsed instanceof CSSKeywordValue && parsed.value === 'unset') {
        return {
          absolutePath: [],
          hasArc: false,
          segments: [],
          polygons: [],
          polylines: [],
          curve: null,
          totalLength: 0,
          rect: new Rectangle(0, 0, 0, 0)
        };
      }
      return parsed;
    };
    /**
     * update local position
     */
    _proto.postProcessor = function postProcessor(object) {
      if (object.nodeName === Shape.PATH) {
        var _object$parsedStyle = object.parsedStyle,
          _object$parsedStyle$d = _object$parsedStyle.defX,
          defX = _object$parsedStyle$d === void 0 ? 0 : _object$parsedStyle$d,
          _object$parsedStyle$d2 = _object$parsedStyle.defY,
          defY = _object$parsedStyle$d2 === void 0 ? 0 : _object$parsedStyle$d2;
        object.setLocalPosition(defX, defY);
      }
    };
    return CSSPropertyPath;
  }();

  var CSSPropertyPoints = /*#__PURE__*/function () {
    function CSSPropertyPoints() {
      this.parser = parsePoints;
    }
    var _proto = CSSPropertyPoints.prototype;
    /**
     * update local position
     */
    _proto.postProcessor = function postProcessor(object) {
      if (object.nodeName === Shape.POLYGON || object.nodeName === Shape.POLYLINE) {
        var _object$parsedStyle = object.parsedStyle,
          defX = _object$parsedStyle.defX,
          defY = _object$parsedStyle.defY;
        object.setLocalPosition(defX, defY);
      }
    };
    return CSSPropertyPoints;
  }();

  var CSSPropertyShadowBlur = /*#__PURE__*/function (_CSSPropertyLengthOrP) {
    _inheritsLoose(CSSPropertyShadowBlur, _CSSPropertyLengthOrP);
    function CSSPropertyShadowBlur() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _CSSPropertyLengthOrP.call.apply(_CSSPropertyLengthOrP, [this].concat(args)) || this;
      _this.mixer = clampedMergeNumbers(0, Infinity);
      return _this;
    }
    return CSSPropertyShadowBlur;
  }(CSSPropertyLengthOrPercentage);

  var CSSPropertyText = /*#__PURE__*/function () {
    function CSSPropertyText() {}
    var _proto = CSSPropertyText.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      if (parsed instanceof CSSKeywordValue) {
        if (parsed.value === 'unset') {
          return '';
        } else {
          return parsed.value;
        }
      }
      // allow number as valid text content
      return "" + parsed;
    };
    _proto.postProcessor = function postProcessor(object) {
      object.nodeValue = "" + object.parsedStyle.text || '';
    };
    return CSSPropertyText;
  }();

  /**
   * it must transform after text get parsed
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/text-transform
   */
  var CSSPropertyTextTransform = /*#__PURE__*/function () {
    function CSSPropertyTextTransform() {}
    var _proto = CSSPropertyTextTransform.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      var rawText = object.getAttribute('text');
      if (rawText) {
        var transformedText = rawText;
        if (parsed.value === 'capitalize') {
          transformedText = rawText.charAt(0).toUpperCase() + rawText.slice(1);
        } else if (parsed.value === 'lowercase') {
          transformedText = rawText.toLowerCase();
        } else if (parsed.value === 'uppercase') {
          transformedText = rawText.toUpperCase();
        }
        object.parsedStyle.text = transformedText;
      }
      return parsed.value;
    };
    return CSSPropertyTextTransform;
  }();

  /**
   * @see /zh/docs/api/animation#支持变换的属性
   *
   * support the following formats like CSS Transform:
   *
   * scale
   * * scale(x, y)
   * * scaleX(x)
   * * scaleY(x)
   * * scaleZ(z)
   * * scale3d(x, y, z)
   *
   * translate (unit: none, px, %(relative to its bounds))
   * * translate(x, y) eg. translate(0, 0) translate(0, 30px) translate(100%, 100%)
   * * translateX(0)
   * * translateY(0)
   * * translateZ(0)
   * * translate3d(0, 0, 0)
   *
   * rotate (unit: deg rad turn)
   * * rotate(0.5turn) rotate(30deg) rotate(1rad)
   *
   * none
   *
   * unsupported for now:
   * * calc() eg. translate(calc(100% + 10px))
   * * matrix/matrix3d()
   * * skew/skewX/skewY
   * * perspective
   */
  var CSSPropertyTransform = /*#__PURE__*/function () {
    function CSSPropertyTransform() {
      this.tmpMat4 = create$1();
      this.parser = parseTransform;
      this.mixer = mergeTransforms;
    }
    var _proto = CSSPropertyTransform.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      // 'none'
      if (parsed instanceof CSSKeywordValue) {
        return [];
      }
      return parsed;
    };
    _proto.postProcessor = function postProcessor(object) {
      var _this = this;
      var _object$parsedStyle = object.parsedStyle,
        transform = _object$parsedStyle.transform,
        defX = _object$parsedStyle.defX,
        defY = _object$parsedStyle.defY;
      if (transform && transform.length) {
        // reset transform
        object.resetLocalTransform();
        object.setLocalPosition(defX, defY);
        transform.forEach(function (parsed) {
          var t = parsed.t,
            d = parsed.d;
          if (t === 'scale') {
            // scale(1) scale(1, 1)
            var newScale = (d === null || d === void 0 ? void 0 : d.map(function (s) {
              return s.value;
            })) || [1, 1];
            object.scaleLocal(newScale[0], newScale[1], 1);
          } else if (t === 'scalex') {
            var _newScale = (d === null || d === void 0 ? void 0 : d.map(function (s) {
              return s.value;
            })) || [1];
            object.scaleLocal(_newScale[0], 1, 1);
          } else if (t === 'scaley') {
            var _newScale2 = (d === null || d === void 0 ? void 0 : d.map(function (s) {
              return s.value;
            })) || [1];
            object.scaleLocal(1, _newScale2[0], 1);
          } else if (t === 'scalez') {
            var _newScale3 = (d === null || d === void 0 ? void 0 : d.map(function (s) {
              return s.value;
            })) || [1];
            object.scaleLocal(1, 1, _newScale3[0]);
          } else if (t === 'scale3d') {
            var _newScale4 = (d === null || d === void 0 ? void 0 : d.map(function (s) {
              return s.value;
            })) || [1, 1, 1];
            object.scaleLocal(_newScale4[0], _newScale4[1], _newScale4[2]);
          } else if (t === 'translate') {
            var newTranslation = d || [Opx, Opx];
            object.translateLocal(newTranslation[0].value, newTranslation[1].value, 0);
          } else if (t === 'translatex') {
            var _newTranslation = d || [Opx];
            object.translateLocal(_newTranslation[0].value, 0, 0);
          } else if (t === 'translatey') {
            var _newTranslation2 = d || [Opx];
            object.translateLocal(0, _newTranslation2[0].value, 0);
          } else if (t === 'translatez') {
            var _newTranslation3 = d || [Opx];
            object.translateLocal(0, 0, _newTranslation3[0].value);
          } else if (t === 'translate3d') {
            var _newTranslation4 = d || [Opx, Opx, Opx];
            object.translateLocal(_newTranslation4[0].value, _newTranslation4[1].value, _newTranslation4[2].value);
          } else if (t === 'rotate') {
            var newAngles = d || [Odeg];
            object.rotateLocal(0, 0, newAngles[0].value);
          } else if (t === 'rotatex') {
            var _newAngles = d || [Odeg];
            object.rotateLocal(_newAngles[0].value, 0, 0);
          } else if (t === 'rotatey') {
            var _newAngles2 = d || [Odeg];
            object.rotateLocal(0, _newAngles2[0].value, 0);
          } else if (t === 'rotatez') {
            var _newAngles3 = d || [Odeg];
            object.rotateLocal(0, 0, _newAngles3[0].value);
          } else if (t === 'rotate3d') ; else if (t === 'skew') {
            var newSkew = (d === null || d === void 0 ? void 0 : d.map(function (s) {
              return s.value;
            })) || [0, 0];
            object.setLocalSkew(deg2rad(newSkew[0]), deg2rad(newSkew[1]));
          } else if (t === 'skewx') {
            var _newSkew = (d === null || d === void 0 ? void 0 : d.map(function (s) {
              return s.value;
            })) || [0];
            object.setLocalSkew(deg2rad(_newSkew[0]), object.getLocalSkew()[1]);
          } else if (t === 'skewy') {
            var _newSkew2 = (d === null || d === void 0 ? void 0 : d.map(function (s) {
              return s.value;
            })) || [0];
            object.setLocalSkew(object.getLocalSkew()[0], deg2rad(_newSkew2[0]));
          } else if (t === 'matrix') {
            var _d$map = d.map(function (s) {
                return s.value;
              }),
              a = _d$map[0],
              b = _d$map[1],
              c = _d$map[2],
              dd = _d$map[3],
              tx = _d$map[4],
              ty = _d$map[5];
            object.setLocalTransform(set(_this.tmpMat4, a, b, 0, 0, c, dd, 0, 0, 0, 0, 1, 0, tx + defX, ty + defY, 0, 1));
          } else if (t === 'matrix3d') {
            // @ts-ignore
            set.apply(mat4, [_this.tmpMat4].concat(d.map(function (s) {
              return s.value;
            })));
            _this.tmpMat4[12] += defX;
            _this.tmpMat4[13] += defY;
            object.setLocalTransform(_this.tmpMat4);
          }
        });
      }
    };
    return CSSPropertyTransform;
  }();

  /**
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/transform-origin
   * @example
   * [10px, 10px] [10%, 10%]
   */
  var CSSPropertyTransformOrigin = function CSSPropertyTransformOrigin() {
    this.parser = parseTransformOrigin;
  };

  var CSSPropertyZIndex = /*#__PURE__*/function () {
    function CSSPropertyZIndex() {
      this.parser = parseNumber;
    }
    var _proto = CSSPropertyZIndex.prototype;
    _proto.calculator = function calculator(name, oldParsed, computed, object) {
      return computed.value;
    };
    _proto.postProcessor = function postProcessor(object) {
      if (object.parentNode) {
        var parentEntity = object.parentNode;
        var parentRenderable = parentEntity.renderable;
        var parentSortable = parentEntity.sortable;
        if (parentRenderable) {
          parentRenderable.dirty = true;
        }
        // need re-sort on parent
        if (parentSortable) {
          parentSortable.dirty = true;
        }
      }
    };
    return CSSPropertyZIndex;
  }();

  function getAngle$1(angle) {
    if (angle === undefined) {
      return 0;
    } else if (angle > 360 || angle < -360) {
      return angle % 360;
    }
    return angle;
  }
  function createVec3(x, y, z) {
    if (y === void 0) {
      y = 0;
    }
    if (z === void 0) {
      z = 0;
    }
    if (isNumber(x)) {
      return fromValues$2(x, y, z);
    }
    if (Array.isArray(x) && x.length === 3) {
      return clone$1(x);
    }
    return fromValues$2(x[0], x[1] || y, x[2] || z);
  }
  function deg2rad(deg) {
    return deg * (Math.PI / 180);
  }
  function rad2deg(rad) {
    return rad * (180 / Math.PI);
  }
  function turn2deg(turn) {
    return 360 * turn;
  }
  function getEulerFromQuat(out, quat) {
    var x = quat[0];
    var y = quat[1];
    var z = quat[2];
    var w = quat[3];
    var x2 = x * x;
    var y2 = y * y;
    var z2 = z * z;
    var w2 = w * w;
    var unit = x2 + y2 + z2 + w2;
    var test = x * w - y * z;
    if (test > 0.499995 * unit) {
      // TODO: Use glmatrix.EPSILON
      // singularity at the north pole
      out[0] = Math.PI / 2;
      out[1] = 2 * Math.atan2(y, x);
      out[2] = 0;
    } else if (test < -0.499995 * unit) {
      //TODO: Use glmatrix.EPSILON
      // singularity at the south pole
      out[0] = -Math.PI / 2;
      out[1] = 2 * Math.atan2(y, x);
      out[2] = 0;
    } else {
      out[0] = Math.asin(2 * (x * z - w * y));
      out[1] = Math.atan2(2 * (x * w + y * z), 1 - 2 * (z2 + w2));
      out[2] = Math.atan2(2 * (x * y + z * w), 1 - 2 * (y2 + z2));
    }
    // TODO: Return them as degrees and not as radians
    return out;
  }
  function getEulerFromMat4(out, m) {
    var x;
    var z;
    var halfPi = Math.PI * 0.5;
    var _mat4$getScaling = getScaling(create$2(), m),
      sx = _mat4$getScaling[0],
      sy = _mat4$getScaling[1],
      sz = _mat4$getScaling[2];
    var y = Math.asin(-m[2] / sx);
    if (y < halfPi) {
      if (y > -halfPi) {
        x = Math.atan2(m[6] / sy, m[10] / sz);
        z = Math.atan2(m[1] / sx, m[0] / sx);
      } else {
        // Not a unique solution
        z = 0;
        x = -Math.atan2(m[4] / sy, m[5] / sy);
      }
    } else {
      // Not a unique solution
      z = 0;
      x = Math.atan2(m[4] / sy, m[5] / sy);
    }
    out[0] = x;
    out[1] = y;
    out[2] = z;
    return out;
  }
  /**
   * @see https://github.com/toji/gl-matrix/issues/329
   * @see https://doc.babylonjs.com/divingDeeper/mesh/transforms/center_origin/rotation_conventions
   */
  function getEuler(out, quat) {
    if (quat.length === 16) {
      return getEulerFromMat4(out, quat);
    } else {
      return getEulerFromQuat(out, quat);
    }
  }
  function fromRotationTranslationScale$1(rotation, x, y, scaleX, scaleY) {
    var cos = Math.cos(rotation);
    var sin = Math.sin(rotation);
    return fromValues(scaleX * cos, scaleY * sin, 0, -scaleX * sin, scaleY * cos, 0, x, y, 1);
  }
  function makePerspective(out, left, right, top, bottom, near, far) {
    var x = 2 * near / (right - left);
    var y = 2 * near / (top - bottom);
    var a = (right + left) / (right - left);
    var b = (top + bottom) / (top - bottom);
    var c = -(far + near) / (far - near);
    var d = -2 * far * near / (far - near);
    out[0] = x;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = y;
    out[6] = 0;
    out[7] = 0;
    out[8] = a;
    out[9] = b;
    out[10] = c;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[14] = d;
    out[15] = 0;
    return out;
  }
  function decompose(mat) {
    var row0x = mat[0];
    var row0y = mat[1];
    var row1x = mat[3];
    var row1y = mat[4];
    // decompose 3x3 matrix
    // @see https://www.w3.org/TR/css-transforms-1/#decomposing-a-2d-matrix
    var scalingX = Math.sqrt(row0x * row0x + row0y * row0y);
    var scalingY = Math.sqrt(row1x * row1x + row1y * row1y);
    // If determinant is negative, one axis was flipped.
    var determinant = row0x * row1y - row0y * row1x;
    if (determinant < 0) {
      // Flip axis with minimum unit vector dot product.
      if (row0x < row1y) {
        scalingX = -scalingX;
      } else {
        scalingY = -scalingY;
      }
    }
    // Renormalize matrix to remove scale.
    if (scalingX) {
      row0x *= 1 / scalingX;
      row0y *= 1 / scalingX;
    }
    if (scalingY) {
      row1x *= 1 / scalingY;
      row1y *= 1 / scalingY;
    }
    // Compute rotation and renormalize matrix.
    var rotation = Math.atan2(row0y, row0x);
    var angle = rad2deg(rotation);
    return [mat[6], mat[7], scalingX, scalingY, angle];
  }
  var tmp = create$1();
  var perspectiveMatrix = create$1();
  var tmpVec4 = create$3();
  var row = [create$2(), create$2(), create$2()];
  var pdum3 = create$2();
  /*
  Input:  matrix      ; a 4x4 matrix
  Output: translation ; a 3 component vector
          scale       ; a 3 component vector
          skew        ; skew factors XY,XZ,YZ represented as a 3 component vector
          perspective ; a 4 component vector
          quaternion  ; a 4 component vector
  Returns false if the matrix cannot be decomposed, true if it can


  References:
  https://github.com/kamicane/matrix3d/blob/master/lib/Matrix3d.js
  https://github.com/ChromiumWebApps/chromium/blob/master/ui/gfx/transform_util.cc
  http://www.w3.org/TR/css3-transforms/#decomposing-a-3d-matrix
  */
  function decomposeMat4(matrix, translation, scale, skew, perspective, quaternion) {
    //normalize, if not possible then bail out early
    if (!normalize$4(tmp, matrix)) return false;
    // perspectiveMatrix is used to solve for perspective, but it also provides
    // an easy way to test for singularity of the upper 3x3 component.
    copy(perspectiveMatrix, tmp);
    perspectiveMatrix[3] = 0;
    perspectiveMatrix[7] = 0;
    perspectiveMatrix[11] = 0;
    perspectiveMatrix[15] = 1;
    // If the perspectiveMatrix is not invertible, we are also unable to
    // decompose, so we'll bail early. Constant taken from SkMatrix44::invert.
    if (Math.abs(determinant(perspectiveMatrix)) < 1e-8) return false;
    var a03 = tmp[3],
      a13 = tmp[7],
      a23 = tmp[11],
      a30 = tmp[12],
      a31 = tmp[13],
      a32 = tmp[14],
      a33 = tmp[15];
    // First, isolate perspective.
    if (a03 !== 0 || a13 !== 0 || a23 !== 0) {
      tmpVec4[0] = a03;
      tmpVec4[1] = a13;
      tmpVec4[2] = a23;
      tmpVec4[3] = a33;
      // Solve the equation by inverting perspectiveMatrix and multiplying
      // rightHandSide by the inverse.
      // resuing the perspectiveMatrix here since it's no longer needed
      var ret = invert(perspectiveMatrix, perspectiveMatrix);
      if (!ret) return false;
      transpose(perspectiveMatrix, perspectiveMatrix);
      //multiply by transposed inverse perspective matrix, into perspective vec4
      transformMat4$1(perspective, tmpVec4, perspectiveMatrix);
    } else {
      //no perspective
      perspective[0] = perspective[1] = perspective[2] = 0;
      perspective[3] = 1;
    }
    // Next take care of translation
    translation[0] = a30;
    translation[1] = a31;
    translation[2] = a32;
    // Now get scale and shear. 'row' is a 3 element array of 3 component vectors
    mat3from4(row, tmp);
    // Compute X scale factor and normalize first row.
    scale[0] = length(row[0]);
    normalize(row[0], row[0]);
    // Compute XY shear factor and make 2nd row orthogonal to 1st.
    skew[0] = dot(row[0], row[1]);
    combine(row[1], row[1], row[0], 1.0, -skew[0]);
    // Now, compute Y scale and normalize 2nd row.
    scale[1] = length(row[1]);
    normalize(row[1], row[1]);
    skew[0] /= scale[1];
    // Compute XZ and YZ shears, orthogonalize 3rd row
    skew[1] = dot(row[0], row[2]);
    combine(row[2], row[2], row[0], 1.0, -skew[1]);
    skew[2] = dot(row[1], row[2]);
    combine(row[2], row[2], row[1], 1.0, -skew[2]);
    // Next, get Z scale and normalize 3rd row.
    scale[2] = length(row[2]);
    normalize(row[2], row[2]);
    skew[1] /= scale[2];
    skew[2] /= scale[2];
    // At this point, the matrix (in rows) is orthonormal.
    // Check for a coordinate system flip.  If the determinant
    // is -1, then negate the matrix and the scaling factors.
    cross(pdum3, row[1], row[2]);
    if (dot(row[0], pdum3) < 0) {
      for (var i = 0; i < 3; i++) {
        scale[i] *= -1;
        row[i][0] *= -1;
        row[i][1] *= -1;
        row[i][2] *= -1;
      }
    }
    // Now, get the rotations out
    quaternion[0] = 0.5 * Math.sqrt(Math.max(1 + row[0][0] - row[1][1] - row[2][2], 0));
    quaternion[1] = 0.5 * Math.sqrt(Math.max(1 - row[0][0] + row[1][1] - row[2][2], 0));
    quaternion[2] = 0.5 * Math.sqrt(Math.max(1 - row[0][0] - row[1][1] + row[2][2], 0));
    quaternion[3] = 0.5 * Math.sqrt(Math.max(1 + row[0][0] + row[1][1] + row[2][2], 0));
    if (row[2][1] > row[1][2]) quaternion[0] = -quaternion[0];
    if (row[0][2] > row[2][0]) quaternion[1] = -quaternion[1];
    if (row[1][0] > row[0][1]) quaternion[2] = -quaternion[2];
    return true;
  }
  function normalize$4(out, mat) {
    var m44 = mat[15];
    // Cannot normalize.
    if (m44 === 0) return false;
    var scale = 1 / m44;
    for (var i = 0; i < 16; i++) {
      out[i] = mat[i] * scale;
    }
    return true;
  }
  //gets upper-left of a 4x4 matrix into a 3x3 of vectors
  function mat3from4(out, mat4x4) {
    out[0][0] = mat4x4[0];
    out[0][1] = mat4x4[1];
    out[0][2] = mat4x4[2];
    out[1][0] = mat4x4[4];
    out[1][1] = mat4x4[5];
    out[1][2] = mat4x4[6];
    out[2][0] = mat4x4[8];
    out[2][1] = mat4x4[9];
    out[2][2] = mat4x4[10];
  }
  function combine(out, a, b, scale1, scale2) {
    out[0] = a[0] * scale1 + b[0] * scale2;
    out[1] = a[1] * scale1 + b[1] * scale2;
    out[2] = a[2] * scale1 + b[2] * scale2;
  }

  /**
   * borrow from gradient-parser, but we delete some browser compatible prefix such as `-webkit-`
   * @see https://github.com/rafaelcaricio/gradient-parser
   */
  function colorStopToString(colorStop) {
    var type = colorStop.type,
      value = colorStop.value;
    if (type === 'hex') {
      return "#" + value;
    } else if (type === 'literal') {
      return value;
    } else if (type === 'rgb') {
      return "rgb(" + value.join(',') + ")";
    } else {
      return "rgba(" + value.join(',') + ")";
    }
  }
  var parseGradient$1 = function () {
    var tokens = {
      linearGradient: /^(linear\-gradient)/i,
      repeatingLinearGradient: /^(repeating\-linear\-gradient)/i,
      radialGradient: /^(radial\-gradient)/i,
      repeatingRadialGradient: /^(repeating\-radial\-gradient)/i,
      /**
       * @see https://projects.verou.me/conic-gradient/
       */
      conicGradient: /^(conic\-gradient)/i,
      sideOrCorner: /^to (left (top|bottom)|right (top|bottom)|top (left|right)|bottom (left|right)|left|right|top|bottom)/i,
      extentKeywords: /^(closest\-side|closest\-corner|farthest\-side|farthest\-corner|contain|cover)/,
      positionKeywords: /^(left|center|right|top|bottom)/i,
      pixelValue: /^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))px/,
      percentageValue: /^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))\%/,
      emValue: /^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))em/,
      angleValue: /^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))deg/,
      startCall: /^\(/,
      endCall: /^\)/,
      comma: /^,/,
      hexColor: /^\#([0-9a-fA-F]+)/,
      literalColor: /^([a-zA-Z]+)/,
      rgbColor: /^rgb/i,
      rgbaColor: /^rgba/i,
      number: /^(([0-9]*\.[0-9]+)|([0-9]+\.?))/
    };
    var input = '';
    function error(msg) {
      throw new Error(input + ': ' + msg);
    }
    function getAST() {
      var ast = matchListDefinitions();
      if (input.length > 0) {
        error('Invalid input not EOF');
      }
      return ast;
    }
    function matchListDefinitions() {
      return matchListing(matchDefinition);
    }
    function matchDefinition() {
      return matchGradient('linear-gradient', tokens.linearGradient, matchLinearOrientation) || matchGradient('repeating-linear-gradient', tokens.repeatingLinearGradient, matchLinearOrientation) || matchGradient('radial-gradient', tokens.radialGradient, matchListRadialOrientations) || matchGradient('repeating-radial-gradient', tokens.repeatingRadialGradient, matchListRadialOrientations) || matchGradient('conic-gradient', tokens.conicGradient, matchListRadialOrientations);
    }
    function matchGradient(gradientType, pattern, orientationMatcher) {
      return matchCall(pattern, function (captures) {
        var orientation = orientationMatcher();
        if (orientation) {
          if (!scan(tokens.comma)) {
            error('Missing comma before color stops');
          }
        }
        return {
          type: gradientType,
          orientation: orientation,
          colorStops: matchListing(matchColorStop)
        };
      });
    }
    function matchCall(pattern, callback) {
      var captures = scan(pattern);
      if (captures) {
        if (!scan(tokens.startCall)) {
          error('Missing (');
        }
        var result = callback(captures);
        if (!scan(tokens.endCall)) {
          error('Missing )');
        }
        return result;
      }
    }
    function matchLinearOrientation() {
      return matchSideOrCorner() || matchAngle();
    }
    function matchSideOrCorner() {
      return match('directional', tokens.sideOrCorner, 1);
    }
    function matchAngle() {
      return match('angular', tokens.angleValue, 1);
    }
    function matchListRadialOrientations() {
      var radialOrientations,
        radialOrientation = matchRadialOrientation(),
        lookaheadCache;
      if (radialOrientation) {
        radialOrientations = [];
        radialOrientations.push(radialOrientation);
        lookaheadCache = input;
        if (scan(tokens.comma)) {
          radialOrientation = matchRadialOrientation();
          if (radialOrientation) {
            radialOrientations.push(radialOrientation);
          } else {
            input = lookaheadCache;
          }
        }
      }
      return radialOrientations;
    }
    function matchRadialOrientation() {
      var radialType = matchCircle() || matchEllipse();
      if (radialType) {
        // @ts-ignore
        radialType.at = matchAtPosition();
      } else {
        var extent = matchExtentKeyword();
        if (extent) {
          radialType = extent;
          var positionAt = matchAtPosition();
          if (positionAt) {
            // @ts-ignore
            radialType.at = positionAt;
          }
        } else {
          var defaultPosition = matchPositioning();
          if (defaultPosition) {
            radialType = {
              type: 'default-radial',
              // @ts-ignore
              at: defaultPosition
            };
          }
        }
      }
      return radialType;
    }
    function matchCircle() {
      var circle = match('shape', /^(circle)/i, 0);
      if (circle) {
        // @ts-ignore
        circle.style = matchLength() || matchExtentKeyword();
      }
      return circle;
    }
    function matchEllipse() {
      var ellipse = match('shape', /^(ellipse)/i, 0);
      if (ellipse) {
        // @ts-ignore
        ellipse.style = matchDistance() || matchExtentKeyword();
      }
      return ellipse;
    }
    function matchExtentKeyword() {
      return match('extent-keyword', tokens.extentKeywords, 1);
    }
    function matchAtPosition() {
      if (match('position', /^at/, 0)) {
        var positioning = matchPositioning();
        if (!positioning) {
          error('Missing positioning value');
        }
        return positioning;
      }
    }
    function matchPositioning() {
      var location = matchCoordinates();
      if (location.x || location.y) {
        return {
          type: 'position',
          value: location
        };
      }
    }
    function matchCoordinates() {
      return {
        x: matchDistance(),
        y: matchDistance()
      };
    }
    function matchListing(matcher) {
      var captures = matcher();
      var result = [];
      if (captures) {
        result.push(captures);
        while (scan(tokens.comma)) {
          captures = matcher();
          if (captures) {
            result.push(captures);
          } else {
            error('One extra comma');
          }
        }
      }
      return result;
    }
    function matchColorStop() {
      var color = matchColor();
      if (!color) {
        error('Expected color definition');
      }
      color.length = matchDistance();
      return color;
    }
    function matchColor() {
      return matchHexColor() || matchRGBAColor() || matchRGBColor() || matchLiteralColor();
    }
    function matchLiteralColor() {
      return match('literal', tokens.literalColor, 0);
    }
    function matchHexColor() {
      return match('hex', tokens.hexColor, 1);
    }
    function matchRGBColor() {
      return matchCall(tokens.rgbColor, function () {
        return {
          type: 'rgb',
          value: matchListing(matchNumber)
        };
      });
    }
    function matchRGBAColor() {
      return matchCall(tokens.rgbaColor, function () {
        return {
          type: 'rgba',
          value: matchListing(matchNumber)
        };
      });
    }
    function matchNumber() {
      return scan(tokens.number)[1];
    }
    function matchDistance() {
      return match('%', tokens.percentageValue, 1) || matchPositionKeyword() || matchLength();
    }
    function matchPositionKeyword() {
      return match('position-keyword', tokens.positionKeywords, 1);
    }
    function matchLength() {
      return match('px', tokens.pixelValue, 1) || match('em', tokens.emValue, 1);
    }
    function match(type, pattern, captureIndex) {
      var captures = scan(pattern);
      if (captures) {
        return {
          type: type,
          value: captures[captureIndex]
        };
      }
    }
    function scan(regexp) {
      var blankCaptures = /^[\n\r\t\s]+/.exec(input);
      if (blankCaptures) {
        consume(blankCaptures[0].length);
      }
      var captures = regexp.exec(input);
      if (captures) {
        consume(captures[0].length);
      }
      return captures;
    }
    function consume(size) {
      input = input.substring(size);
    }
    return function (code) {
      input = code;
      return getAST();
    };
  }();
  function computeLinearGradient(width, height, angle) {
    var rad = deg2rad(angle.value);
    var rx = 0;
    var ry = 0;
    var rcx = rx + width / 2;
    var rcy = ry + height / 2;
    // get the length of gradient line
    // @see https://observablehq.com/@danburzo/css-gradient-line
    var length = Math.abs(width * Math.cos(rad)) + Math.abs(height * Math.sin(rad));
    var x1 = rcx - Math.cos(rad) * length / 2;
    var y1 = rcy - Math.sin(rad) * length / 2;
    var x2 = rcx + Math.cos(rad) * length / 2;
    var y2 = rcy + Math.sin(rad) * length / 2;
    return {
      x1: x1,
      y1: y1,
      x2: x2,
      y2: y2
    };
  }
  function computeRadialGradient(width, height, cx, cy, size) {
    // 'px'
    var x = cx.value;
    var y = cy.value;
    // TODO: 'em'
    // '%'
    if (cx.unit === UnitType.kPercentage) {
      x = cx.value / 100 * width;
    }
    if (cy.unit === UnitType.kPercentage) {
      y = cy.value / 100 * height;
    }
    // default to farthest-side
    var r = Math.max(distanceSquareRoot([0, 0], [x, y]), distanceSquareRoot([0, height], [x, y]), distanceSquareRoot([width, height], [x, y]), distanceSquareRoot([width, 0], [x, y]));
    if (size) {
      if (size instanceof CSSUnitValue) {
        r = size.value;
      } else if (size instanceof CSSKeywordValue) {
        // @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/CSS_Images/Using_CSS_gradients#example_closest-side_for_circles
        if (size.value === 'closest-side') {
          r = Math.min(x, width - x, y, height - y);
        } else if (size.value === 'farthest-side') {
          r = Math.max(x, width - x, y, height - y);
        } else if (size.value === 'closest-corner') {
          r = Math.min(distanceSquareRoot([0, 0], [x, y]), distanceSquareRoot([0, height], [x, y]), distanceSquareRoot([width, height], [x, y]), distanceSquareRoot([width, 0], [x, y]));
        }
      }
    }
    return {
      x: x,
      y: y,
      r: r
    };
  }

  function getOrCalculatePathTotalLength(path) {
    if (path.parsedStyle.path.totalLength === 0) {
      path.parsedStyle.path.totalLength = getTotalLength(path.parsedStyle.path.absolutePath);
    }
    return path.parsedStyle.path.totalLength;
  }
  function hasArcOrBezier(path) {
    var hasArc = false;
    var count = path.length;
    for (var i = 0; i < count; i++) {
      var params = path[i];
      var cmd = params[0];
      if (cmd === 'C' || cmd === 'A' || cmd === 'Q') {
        hasArc = true;
        break;
      }
    }
    return hasArc;
  }
  function extractPolygons(pathArray) {
    var polygons = [];
    var polylines = [];
    var points = []; // 防止第一个命令不是 'M'
    for (var i = 0; i < pathArray.length; i++) {
      var params = pathArray[i];
      var cmd = params[0];
      if (cmd === 'M') {
        // 遇到 'M' 判定是否是新数组，新数组中没有点
        if (points.length) {
          // 如果存在点，则说明没有遇到 'Z'，开始了一个新的多边形
          polylines.push(points);
          points = []; // 创建新的点
        }

        points.push([params[1], params[2]]);
      } else if (cmd === 'Z') {
        if (points.length) {
          // 存在点
          polygons.push(points);
          points = []; // 开始新的点集合
        }
        // 如果不存在点，同时 'Z'，则说明是错误，不处理
      } else {
        points.push([params[1], params[2]]);
      }
    }
    // 说明 points 未放入 polygons 或者 polyline
    // 仅当只有一个 M，没有 Z 时会发生这种情况
    if (points.length > 0) {
      polylines.push(points);
    }
    return {
      polygons: polygons,
      polylines: polylines
    };
  }
  function isSamePoint(point1, point2) {
    return point1[0] === point2[0] && point1[1] === point2[1];
  }
  function getPathBBox(segments, lineWidth) {
    var xArr = [];
    var yArr = [];
    var segmentsWithAngle = [];
    for (var i = 0; i < segments.length; i++) {
      var segment = segments[i];
      var currentPoint = segment.currentPoint,
        params = segment.params,
        prePoint = segment.prePoint;
      var box = void 0;
      switch (segment.command) {
        case 'Q':
          box = quadratic.box(prePoint[0], prePoint[1], params[1], params[2], params[3], params[4]);
          break;
        case 'C':
          box = cubic.box(prePoint[0], prePoint[1], params[1], params[2], params[3], params[4], params[5], params[6]);
          break;
        case 'A':
          var arcParams = segment.arcParams;
          box = arc.box(arcParams.cx, arcParams.cy, arcParams.rx, arcParams.ry, arcParams.xRotation, arcParams.startAngle, arcParams.endAngle);
          break;
        default:
          xArr.push(currentPoint[0]);
          yArr.push(currentPoint[1]);
          break;
      }
      if (box) {
        segment.box = box;
        xArr.push(box.x, box.x + box.width);
        yArr.push(box.y, box.y + box.height);
      }
      if (lineWidth && (segment.command === 'L' || segment.command === 'M') && segment.prePoint && segment.nextPoint) {
        segmentsWithAngle.push(segment);
      }
    }
    // bbox calculation should ignore NaN for path attribute
    // ref: https://github.com/antvis/g/issues/210
    // ref: https://github.com/antvis/G2/issues/3109
    xArr = xArr.filter(function (item) {
      return !Number.isNaN(item) && item !== Infinity && item !== -Infinity;
    });
    yArr = yArr.filter(function (item) {
      return !Number.isNaN(item) && item !== Infinity && item !== -Infinity;
    });
    var minX = min$2(xArr);
    var minY = min$2(yArr);
    var maxX = max$2(xArr);
    var maxY = max$2(yArr);
    if (segmentsWithAngle.length === 0) {
      return {
        x: minX,
        y: minY,
        width: maxX - minX,
        height: maxY - minY
      };
    }
    for (var _i = 0; _i < segmentsWithAngle.length; _i++) {
      var _segment = segmentsWithAngle[_i];
      var _currentPoint = _segment.currentPoint;
      var extra = void 0;
      if (_currentPoint[0] === minX) {
        extra = getExtraFromSegmentWithAngle(_segment, lineWidth);
        minX = minX - extra.xExtra;
      } else if (_currentPoint[0] === maxX) {
        extra = getExtraFromSegmentWithAngle(_segment, lineWidth);
        maxX = maxX + extra.xExtra;
      }
      if (_currentPoint[1] === minY) {
        extra = getExtraFromSegmentWithAngle(_segment, lineWidth);
        minY = minY - extra.yExtra;
      } else if (_currentPoint[1] === maxY) {
        extra = getExtraFromSegmentWithAngle(_segment, lineWidth);
        maxY = maxY + extra.yExtra;
      }
    }
    return {
      x: minX,
      y: minY,
      width: maxX - minX,
      height: maxY - minY
    };
  }
  function getExtraFromSegmentWithAngle(segment, lineWidth) {
    var prePoint = segment.prePoint,
      currentPoint = segment.currentPoint,
      nextPoint = segment.nextPoint;
    var currentAndPre = Math.pow(currentPoint[0] - prePoint[0], 2) + Math.pow(currentPoint[1] - prePoint[1], 2);
    var currentAndNext = Math.pow(currentPoint[0] - nextPoint[0], 2) + Math.pow(currentPoint[1] - nextPoint[1], 2);
    var preAndNext = Math.pow(prePoint[0] - nextPoint[0], 2) + Math.pow(prePoint[1] - nextPoint[1], 2);
    // 以 currentPoint 为顶点的夹角
    var currentAngle = Math.acos((currentAndPre + currentAndNext - preAndNext) / (2 * Math.sqrt(currentAndPre) * Math.sqrt(currentAndNext)));
    // 夹角为空、 0 或 PI 时，不需要计算夹角处的额外宽度
    // 注意: 由于计算精度问题，夹角为 0 的情况计算出来的角度可能是一个很小的值，还需要判断其与 0 是否近似相等
    if (!currentAngle || Math.sin(currentAngle) === 0 || isNumberEqual(currentAngle, 0)) {
      return {
        xExtra: 0,
        yExtra: 0
      };
    }
    var xAngle = Math.abs(Math.atan2(nextPoint[1] - currentPoint[1], nextPoint[0] - currentPoint[0]));
    var yAngle = Math.abs(Math.atan2(nextPoint[0] - currentPoint[0], nextPoint[1] - currentPoint[1]));
    // 将夹角转为锐角
    xAngle = xAngle > Math.PI / 2 ? Math.PI - xAngle : xAngle;
    yAngle = yAngle > Math.PI / 2 ? Math.PI - yAngle : yAngle;
    // 这里不考虑在水平和垂直方向的投影，直接使用最大差值
    // 由于上层统一加减了二分之一线宽，这里需要进行弥补
    var extra = {
      // 水平方向投影
      xExtra: Math.cos(currentAngle / 2 - xAngle) * (lineWidth / 2 * (1 / Math.sin(currentAngle / 2))) - lineWidth / 2 || 0,
      // 垂直方向投影
      yExtra: Math.cos(yAngle - currentAngle / 2) * (lineWidth / 2 * (1 / Math.sin(currentAngle / 2))) - lineWidth / 2 || 0
    };
    return extra;
  }
  // 点对称
  function toSymmetry(point, center) {
    return [center[0] + (center[0] - point[0]), center[1] + (center[1] - point[1])];
  }
  var angleBetween$1 = function angleBetween(v0, v1) {
    var p = v0.x * v1.x + v0.y * v1.y;
    var n = Math.sqrt((Math.pow(v0.x, 2) + Math.pow(v0.y, 2)) * (Math.pow(v1.x, 2) + Math.pow(v1.y, 2)));
    var sign = v0.x * v1.y - v0.y * v1.x < 0 ? -1 : 1;
    var angle = sign * Math.acos(p / n);
    return angle;
  };
  /**
   * @see https://github.com/rveciana/svg-path-properties/blob/b6bd9a322966f6ef7a311872d80c56e3718de861/src/arc.ts#L121
   */
  var pointOnEllipticalArc = function pointOnEllipticalArc(p0, rx, ry, xAxisRotation, largeArcFlag, sweepFlag, p1, t) {
    // In accordance to: http://www.w3.org/TR/SVG/implnote.html#ArcOutOfRangeParameters
    rx = Math.abs(rx);
    ry = Math.abs(ry);
    xAxisRotation = mod(xAxisRotation, 360);
    var xAxisRotationRadians = deg2rad(xAxisRotation);
    // If the endpoints are identical, then this is equivalent to omitting the elliptical arc segment entirely.
    if (p0.x === p1.x && p0.y === p1.y) {
      return {
        x: p0.x,
        y: p0.y,
        ellipticalArcAngle: 0
      }; // Check if angle is correct
    }
    // If rx = 0 or ry = 0 then this arc is treated as a straight line segment joining the endpoints.
    if (rx === 0 || ry === 0) {
      //return this.pointOnLine(p0, p1, t);
      return {
        x: 0,
        y: 0,
        ellipticalArcAngle: 0
      }; // Check if angle is correct
    }
    // Following "Conversion from endpoint to center parameterization"
    // http://www.w3.org/TR/SVG/implnote.html#ArcConversionEndpointToCenter
    // Step #1: Compute transformedPoint
    var dx = (p0.x - p1.x) / 2;
    var dy = (p0.y - p1.y) / 2;
    var transformedPoint = {
      x: Math.cos(xAxisRotationRadians) * dx + Math.sin(xAxisRotationRadians) * dy,
      y: -Math.sin(xAxisRotationRadians) * dx + Math.cos(xAxisRotationRadians) * dy
    };
    // Ensure radii are large enough
    var radiiCheck = Math.pow(transformedPoint.x, 2) / Math.pow(rx, 2) + Math.pow(transformedPoint.y, 2) / Math.pow(ry, 2);
    if (radiiCheck > 1) {
      rx = Math.sqrt(radiiCheck) * rx;
      ry = Math.sqrt(radiiCheck) * ry;
    }
    // Step #2: Compute transformedCenter
    var cSquareNumerator = Math.pow(rx, 2) * Math.pow(ry, 2) - Math.pow(rx, 2) * Math.pow(transformedPoint.y, 2) - Math.pow(ry, 2) * Math.pow(transformedPoint.x, 2);
    var cSquareRootDenom = Math.pow(rx, 2) * Math.pow(transformedPoint.y, 2) + Math.pow(ry, 2) * Math.pow(transformedPoint.x, 2);
    var cRadicand = cSquareNumerator / cSquareRootDenom;
    // Make sure this never drops below zero because of precision
    cRadicand = cRadicand < 0 ? 0 : cRadicand;
    var cCoef = (largeArcFlag !== sweepFlag ? 1 : -1) * Math.sqrt(cRadicand);
    var transformedCenter = {
      x: cCoef * (rx * transformedPoint.y / ry),
      y: cCoef * (-(ry * transformedPoint.x) / rx)
    };
    // Step #3: Compute center
    var center = {
      x: Math.cos(xAxisRotationRadians) * transformedCenter.x - Math.sin(xAxisRotationRadians) * transformedCenter.y + (p0.x + p1.x) / 2,
      y: Math.sin(xAxisRotationRadians) * transformedCenter.x + Math.cos(xAxisRotationRadians) * transformedCenter.y + (p0.y + p1.y) / 2
    };
    // Step #4: Compute start/sweep angles
    // Start angle of the elliptical arc prior to the stretch and rotate operations.
    // Difference between the start and end angles
    var startVector = {
      x: (transformedPoint.x - transformedCenter.x) / rx,
      y: (transformedPoint.y - transformedCenter.y) / ry
    };
    var startAngle = angleBetween$1({
      x: 1,
      y: 0
    }, startVector);
    var endVector = {
      x: (-transformedPoint.x - transformedCenter.x) / rx,
      y: (-transformedPoint.y - transformedCenter.y) / ry
    };
    var sweepAngle = angleBetween$1(startVector, endVector);
    if (!sweepFlag && sweepAngle > 0) {
      sweepAngle -= 2 * Math.PI;
    } else if (sweepFlag && sweepAngle < 0) {
      sweepAngle += 2 * Math.PI;
    }
    // We use % instead of `mod(..)` because we want it to be -360deg to 360deg(but actually in radians)
    sweepAngle %= 2 * Math.PI;
    // From http://www.w3.org/TR/SVG/implnote.html#ArcParameterizationAlternatives
    var angle = startAngle + sweepAngle * t;
    var ellipseComponentX = rx * Math.cos(angle);
    var ellipseComponentY = ry * Math.sin(angle);
    var point = {
      x: Math.cos(xAxisRotationRadians) * ellipseComponentX - Math.sin(xAxisRotationRadians) * ellipseComponentY + center.x,
      y: Math.sin(xAxisRotationRadians) * ellipseComponentX + Math.cos(xAxisRotationRadians) * ellipseComponentY + center.y,
      ellipticalArcStartAngle: startAngle,
      ellipticalArcEndAngle: startAngle + sweepAngle,
      ellipticalArcAngle: angle,
      ellipticalArcCenter: center,
      resultantRx: rx,
      resultantRy: ry
    };
    return point;
  };
  function path2Segments(path) {
    var segments = [];
    var currentPoint = null; // 当前图形
    var nextParams = null; // 下一节点的 path 参数
    var startMovePoint = null; // 开始 M 的点，可能会有多个
    var lastStartMovePointIndex = 0; // 最近一个开始点 M 的索引
    var count = path.length;
    for (var i = 0; i < count; i++) {
      var params = path[i];
      nextParams = path[i + 1];
      var command = params[0];
      // 数学定义上的参数，便于后面的计算
      var segment = {
        command: command,
        prePoint: currentPoint,
        params: params,
        startTangent: null,
        endTangent: null,
        currentPoint: null,
        nextPoint: null,
        arcParams: null,
        box: null,
        cubicParams: null
      };
      switch (command) {
        case 'M':
          startMovePoint = [params[1], params[2]];
          lastStartMovePointIndex = i;
          break;
        case 'A':
          var arcParams = getArcParams(currentPoint, params);
          segment.arcParams = arcParams;
          break;
      }
      if (command === 'Z') {
        // 有了 Z 后，当前节点从开始 M 的点开始
        currentPoint = startMovePoint;
        // 如果当前点的命令为 Z，相当于当前点为最近一个 M 点，则下一个点直接指向最近一个 M 点的下一个点
        nextParams = path[lastStartMovePointIndex + 1];
      } else {
        var len = params.length;
        currentPoint = [params[len - 2], params[len - 1]];
      }
      if (nextParams && nextParams[0] === 'Z') {
        // 如果下一个点的命令为 Z，则下一个点直接指向最近一个 M 点
        nextParams = path[lastStartMovePointIndex];
        if (segments[lastStartMovePointIndex]) {
          // 如果下一个点的命令为 Z，则最近一个 M 点的前一个点为当前点
          segments[lastStartMovePointIndex].prePoint = currentPoint;
        }
      }
      segment.currentPoint = currentPoint;
      // 如果当前点与最近一个 M 点相同，则最近一个 M 点的前一个点为当前点的前一个点
      if (segments[lastStartMovePointIndex] && isSamePoint(currentPoint, segments[lastStartMovePointIndex].currentPoint)) {
        segments[lastStartMovePointIndex].prePoint = segment.prePoint;
      }
      var nextPoint = nextParams ? [nextParams[nextParams.length - 2], nextParams[nextParams.length - 1]] : null;
      segment.nextPoint = nextPoint;
      // Add startTangent and endTangent
      var prePoint = segment.prePoint;
      if (['L', 'H', 'V'].includes(command)) {
        segment.startTangent = [prePoint[0] - currentPoint[0], prePoint[1] - currentPoint[1]];
        segment.endTangent = [currentPoint[0] - prePoint[0], currentPoint[1] - prePoint[1]];
      } else if (command === 'Q') {
        // 二次贝塞尔曲线只有一个控制点
        var cp = [params[1], params[2]];
        // 二次贝塞尔曲线的终点为 currentPoint
        segment.startTangent = [prePoint[0] - cp[0], prePoint[1] - cp[1]];
        segment.endTangent = [currentPoint[0] - cp[0], currentPoint[1] - cp[1]];
      } else if (command === 'T') {
        var preSegment = segments[i - 1];
        var _cp = toSymmetry(preSegment.currentPoint, prePoint);
        if (preSegment.command === 'Q') {
          segment.command = 'Q';
          segment.startTangent = [prePoint[0] - _cp[0], prePoint[1] - _cp[1]];
          segment.endTangent = [currentPoint[0] - _cp[0], currentPoint[1] - _cp[1]];
        } else {
          // @ts-ignore
          segment.command = 'TL';
          segment.startTangent = [prePoint[0] - currentPoint[0], prePoint[1] - currentPoint[1]];
          segment.endTangent = [currentPoint[0] - prePoint[0], currentPoint[1] - prePoint[1]];
        }
      } else if (command === 'C') {
        // 三次贝塞尔曲线有两个控制点
        var cp1 = [params[1], params[2]];
        var cp2 = [params[3], params[4]];
        segment.startTangent = [prePoint[0] - cp1[0], prePoint[1] - cp1[1]];
        segment.endTangent = [currentPoint[0] - cp2[0], currentPoint[1] - cp2[1]];
        // horizontal line, eg. ['C', 100, 100, 100, 100, 200, 200]
        if (segment.startTangent[0] === 0 && segment.startTangent[1] === 0) {
          segment.startTangent = [cp1[0] - cp2[0], cp1[1] - cp2[1]];
        }
        if (segment.endTangent[0] === 0 && segment.endTangent[1] === 0) {
          segment.endTangent = [cp2[0] - cp1[0], cp2[1] - cp1[1]];
        }
      } else if (command === 'S') {
        var _preSegment = segments[i - 1];
        var _cp2 = toSymmetry(_preSegment.currentPoint, prePoint);
        var _cp3 = [params[1], params[2]];
        if (_preSegment.command === 'C') {
          segment.command = 'C'; // 将 S 命令变换为 C 命令
          segment.startTangent = [prePoint[0] - _cp2[0], prePoint[1] - _cp2[1]];
          segment.endTangent = [currentPoint[0] - _cp3[0], currentPoint[1] - _cp3[1]];
        } else {
          // @ts-ignore
          segment.command = 'SQ'; // 将 S 命令变换为 SQ 命令
          segment.startTangent = [prePoint[0] - _cp3[0], prePoint[1] - _cp3[1]];
          segment.endTangent = [currentPoint[0] - _cp3[0], currentPoint[1] - _cp3[1]];
        }
      } else if (command === 'A') {
        var _getTangentAtRatio = getTangentAtRatio(segment, 0),
          dx1 = _getTangentAtRatio.x,
          dy1 = _getTangentAtRatio.y;
        var _getTangentAtRatio2 = getTangentAtRatio(segment, 1, false),
          dx2 = _getTangentAtRatio2.x,
          dy2 = _getTangentAtRatio2.y;
        segment.startTangent = [dx1, dy1];
        segment.endTangent = [dx2, dy2];
      }
      segments.push(segment);
    }
    return segments;
  }
  /**
   * Use length instead of ratio
   */
  function getTangentAtRatio(segment, ratio, sign) {
    if (sign === void 0) {
      sign = true;
    }
    var _segment$arcParams = segment.arcParams,
      _segment$arcParams$rx = _segment$arcParams.rx,
      rx = _segment$arcParams$rx === void 0 ? 0 : _segment$arcParams$rx,
      _segment$arcParams$ry = _segment$arcParams.ry,
      ry = _segment$arcParams$ry === void 0 ? 0 : _segment$arcParams$ry,
      xRotation = _segment$arcParams.xRotation,
      arcFlag = _segment$arcParams.arcFlag,
      sweepFlag = _segment$arcParams.sweepFlag;
    var p1 = pointOnEllipticalArc({
      x: segment.prePoint[0],
      y: segment.prePoint[1]
    }, rx, ry, xRotation, !!arcFlag, !!sweepFlag, {
      x: segment.currentPoint[0],
      y: segment.currentPoint[1]
    }, ratio);
    var p2 = pointOnEllipticalArc({
      x: segment.prePoint[0],
      y: segment.prePoint[1]
    }, rx, ry, xRotation, !!arcFlag, !!sweepFlag, {
      x: segment.currentPoint[0],
      y: segment.currentPoint[1]
    }, sign ? ratio + 0.005 : ratio - 0.005);
    var xDist = p2.x - p1.x;
    var yDist = p2.y - p1.y;
    var dist = Math.sqrt(xDist * xDist + yDist * yDist);
    return {
      x: -xDist / dist,
      y: -yDist / dist
    };
  }
  // 向量长度
  function vMag(v) {
    return Math.sqrt(v[0] * v[0] + v[1] * v[1]);
  }
  // u.v/|u||v|，计算夹角的余弦值
  function vRatio(u, v) {
    // 当存在一个向量的长度为 0 时，夹角也为 0，即夹角的余弦值为 1
    return vMag(u) * vMag(v) ? (u[0] * v[0] + u[1] * v[1]) / (vMag(u) * vMag(v)) : 1;
  }
  // 向量角度
  function vAngle(u, v) {
    return (u[0] * v[1] < u[1] * v[0] ? -1 : 1) * Math.acos(vRatio(u, v));
  }
  function getArcParams(startPoint, params) {
    var rx = params[1];
    var ry = params[2];
    var xRotation = mod(deg2rad(params[3]), Math.PI * 2);
    var arcFlag = params[4];
    var sweepFlag = params[5];
    // 弧形起点坐标
    var x1 = startPoint[0];
    var y1 = startPoint[1];
    // 弧形终点坐标
    var x2 = params[6];
    var y2 = params[7];
    var xp = Math.cos(xRotation) * (x1 - x2) / 2.0 + Math.sin(xRotation) * (y1 - y2) / 2.0;
    var yp = -1 * Math.sin(xRotation) * (x1 - x2) / 2.0 + Math.cos(xRotation) * (y1 - y2) / 2.0;
    var lambda = xp * xp / (rx * rx) + yp * yp / (ry * ry);
    if (lambda > 1) {
      rx *= Math.sqrt(lambda);
      ry *= Math.sqrt(lambda);
    }
    var diff = rx * rx * (yp * yp) + ry * ry * (xp * xp);
    var f = diff ? Math.sqrt((rx * rx * (ry * ry) - diff) / diff) : 1;
    if (arcFlag === sweepFlag) {
      f *= -1;
    }
    if (isNaN(f)) {
      f = 0;
    }
    // 旋转前的起点坐标，且当长半轴和短半轴的长度为 0 时，坐标按 (0, 0) 处理
    var cxp = ry ? f * rx * yp / ry : 0;
    var cyp = rx ? f * -ry * xp / rx : 0;
    // 椭圆圆心坐标
    var cx = (x1 + x2) / 2.0 + Math.cos(xRotation) * cxp - Math.sin(xRotation) * cyp;
    var cy = (y1 + y2) / 2.0 + Math.sin(xRotation) * cxp + Math.cos(xRotation) * cyp;
    // 起始点的单位向量
    var u = [(xp - cxp) / rx, (yp - cyp) / ry];
    // 终止点的单位向量
    var v = [(-1 * xp - cxp) / rx, (-1 * yp - cyp) / ry];
    // 计算起始点和圆心的连线，与 x 轴正方向的夹角
    var theta = vAngle([1, 0], u);
    // 计算圆弧起始点和终止点与椭圆圆心连线的夹角
    var dTheta = vAngle(u, v);
    if (vRatio(u, v) <= -1) {
      dTheta = Math.PI;
    }
    if (vRatio(u, v) >= 1) {
      dTheta = 0;
    }
    if (sweepFlag === 0 && dTheta > 0) {
      dTheta = dTheta - 2 * Math.PI;
    }
    if (sweepFlag === 1 && dTheta < 0) {
      dTheta = dTheta + 2 * Math.PI;
    }
    return {
      cx: cx,
      cy: cy,
      // 弧形的起点和终点相同时，长轴和短轴的长度按 0 处理
      rx: isSamePoint(startPoint, [x2, y2]) ? 0 : rx,
      ry: isSamePoint(startPoint, [x2, y2]) ? 0 : ry,
      startAngle: theta,
      endAngle: theta + dTheta,
      xRotation: xRotation,
      arcFlag: arcFlag,
      sweepFlag: sweepFlag
    };
  }
  function commandsToPathString(commands, object, transform) {
    var _object$parsedStyle = object.parsedStyle,
      _object$parsedStyle$d = _object$parsedStyle.defX,
      defX = _object$parsedStyle$d === void 0 ? 0 : _object$parsedStyle$d,
      _object$parsedStyle$d2 = _object$parsedStyle.defY,
      defY = _object$parsedStyle$d2 === void 0 ? 0 : _object$parsedStyle$d2;
    return commands.reduce(function (prev, cur) {
      var path = '';
      if (cur[0] === 'M' || cur[0] === 'L') {
        var p = fromValues$2(cur[1] - defX, cur[2] - defY, 0);
        if (transform) {
          transformMat4(p, p, transform);
        }
        path = "" + cur[0] + p[0] + "," + p[1];
      } else if (cur[0] === 'Z') {
        path = cur[0];
      } else if (cur[0] === 'C') {
        var p1 = fromValues$2(cur[1] - defX, cur[2] - defY, 0);
        var p2 = fromValues$2(cur[3] - defX, cur[4] - defY, 0);
        var p3 = fromValues$2(cur[5] - defX, cur[6] - defY, 0);
        if (transform) {
          transformMat4(p1, p1, transform);
          transformMat4(p2, p2, transform);
          transformMat4(p3, p3, transform);
        }
        path = "" + cur[0] + p1[0] + "," + p1[1] + "," + p2[0] + "," + p2[1] + "," + p3[0] + "," + p3[1];
      } else if (cur[0] === 'A') {
        var c = fromValues$2(cur[6] - defX, cur[7] - defY, 0);
        if (transform) {
          transformMat4(c, c, transform);
        }
        path = "" + cur[0] + cur[1] + "," + cur[2] + "," + cur[3] + "," + cur[4] + "," + cur[5] + "," + c[0] + "," + c[1];
      } else if (cur[0] === 'Q') {
        var _p = fromValues$2(cur[1] - defX, cur[2] - defY, 0);
        var _p2 = fromValues$2(cur[3] - defX, cur[4] - defY, 0);
        if (transform) {
          transformMat4(_p, _p, transform);
          transformMat4(_p2, _p2, transform);
        }
        path = "" + cur[0] + cur[1] + "," + cur[2] + "," + cur[3] + "," + cur[4] + "}";
      }
      return prev += path;
    }, '');
  }
  function lineToCommands(x1, y1, x2, y2) {
    return [['M', x1, y1], ['L', x2, y2]];
  }
  function ellipseToCommands(rx, ry, cx, cy) {
    var factor = (-1 + Math.sqrt(2)) / 3 * 4;
    var dx = rx * factor;
    var dy = ry * factor;
    var left = cx - rx;
    var right = cx + rx;
    var top = cy - ry;
    var bottom = cy + ry;
    return [['M', left, cy], ['C', left, cy - dy, cx - dx, top, cx, top], ['C', cx + dx, top, right, cy - dy, right, cy], ['C', right, cy + dy, cx + dx, bottom, cx, bottom], ['C', cx - dx, bottom, left, cy + dy, left, cy], ['Z']];
  }
  function polygonToCommands(points, closed) {
    var result = points.map(function (point, i) {
      return [i === 0 ? 'M' : 'L', point[0], point[1]];
    });
    if (closed) {
      result.push(['Z']);
    }
    return result;
  }
  function rectToCommands(width, height, x, y, radius) {
    // @see https://gist.github.com/danielpquinn/dd966af424030d47e476
    if (radius) {
      var tlr = radius[0],
        trr = radius[1],
        brr = radius[2],
        blr = radius[3];
      var signX = width > 0 ? 1 : -1;
      var signY = height > 0 ? 1 : -1;
      // sweep-flag @see https://developer.mozilla.org/zh-CN/docs/Web/SVG/Tutorial/Paths#arcs
      var sweepFlag = signX + signY !== 0 ? 1 : 0;
      return [['M', signX * tlr + x, y], ['L', width - signX * trr + x, y], trr ? ['A', trr, trr, 0, 0, sweepFlag, width + x, signY * trr + y] : null, ['L', width + x, height - signY * brr + y], brr ? ['A', brr, brr, 0, 0, sweepFlag, width + x - signX * brr, height + y] : null, ['L', x + signX * blr, height + y], blr ? ['A', blr, blr, 0, 0, sweepFlag, x, height + y - signY * blr] : null, ['L', x, signY * tlr + y], tlr ? ['A', tlr, tlr, 0, 0, sweepFlag, signX * tlr + x, y] : null, ['Z']].filter(function (command) {
        return command;
      });
    }
    return [['M', x, y], ['L', x + width, y], ['L', x + width, y + height], ['L', x, y + height], ['Z']];
  }
  /**
   * convert object to path, should account for:
   * * transform & origin
   * * anchor
   * * lineWidth
   */
  function convertToPath(object, transform) {
    if (transform === void 0) {
      transform = object.getLocalTransform();
    }
    var commands = [];
    switch (object.nodeName) {
      case Shape.LINE:
        var _object$parsedStyle2 = object.parsedStyle,
          x1 = _object$parsedStyle2.x1,
          y1 = _object$parsedStyle2.y1,
          x2 = _object$parsedStyle2.x2,
          y2 = _object$parsedStyle2.y2;
        commands = lineToCommands(x1, y1, x2, y2);
        break;
      case Shape.CIRCLE:
        {
          var _object$parsedStyle3 = object.parsedStyle,
            r = _object$parsedStyle3.r,
            cx = _object$parsedStyle3.cx,
            cy = _object$parsedStyle3.cy;
          commands = ellipseToCommands(r, r, cx, cy);
          break;
        }
      case Shape.ELLIPSE:
        {
          var _object$parsedStyle4 = object.parsedStyle,
            rx = _object$parsedStyle4.rx,
            ry = _object$parsedStyle4.ry,
            _cx = _object$parsedStyle4.cx,
            _cy = _object$parsedStyle4.cy;
          commands = ellipseToCommands(rx, ry, _cx, _cy);
          break;
        }
      case Shape.POLYLINE:
      case Shape.POLYGON:
        var points = object.parsedStyle.points;
        commands = polygonToCommands(points.points, object.nodeName === Shape.POLYGON);
        break;
      case Shape.RECT:
        var _object$parsedStyle5 = object.parsedStyle,
          width = _object$parsedStyle5.width,
          height = _object$parsedStyle5.height,
          x = _object$parsedStyle5.x,
          y = _object$parsedStyle5.y,
          radius = _object$parsedStyle5.radius;
        var hasRadius = radius && radius.some(function (r) {
          return r !== 0;
        });
        commands = rectToCommands(width, height, x, y, hasRadius && radius.map(function (r) {
          return clamp(r, 0, Math.min(Math.abs(width) / 2, Math.abs(height) / 2));
        }));
        break;
      case Shape.PATH:
        var absolutePath = object.parsedStyle.path.absolutePath;
        commands = [].concat(absolutePath);
        break;
    }
    if (commands.length) {
      return commandsToPathString(commands, object, transform);
    }
  }

  function isFillOrStrokeAffected(pointerEvents, fill, stroke) {
    // account for pointerEvents
    // @see https://developer.mozilla.org/en-US/docs/Web/CSS/pointer-events
    var hasFill = false;
    var hasStroke = false;
    var isFillOtherThanNone = !!fill && !fill.isNone;
    var isStrokeOtherThanNone = !!stroke && !stroke.isNone;
    if (pointerEvents === 'visiblepainted' || pointerEvents === 'painted' || pointerEvents === 'auto') {
      hasFill = isFillOtherThanNone;
      hasStroke = isStrokeOtherThanNone;
    } else if (pointerEvents === 'visiblefill' || pointerEvents === 'fill') {
      hasFill = true;
    } else if (pointerEvents === 'visiblestroke' || pointerEvents === 'stroke') {
      hasStroke = true;
    } else if (pointerEvents === 'visible' || pointerEvents === 'all') {
      // The values of the fill and stroke do not affect event processing.
      hasFill = true;
      hasStroke = true;
    }
    return [hasFill, hasStroke];
  }

  // @ts-nocheck
  // interface HookInterceptor<T, R, AdditionalOptions = UnsetAdditionalOptions> {
  //   name?: string;
  //   tap?: (tap: FullTap & IfSet<AdditionalOptions>) => void;
  //   call?: (...args: any[]) => void;
  //   loop?: (...args: any[]) => void;
  //   error?: (err: Error) => void;
  //   result?: (result: R) => void;
  //   done?: () => void;
  //   register?: (tap: FullTap & IfSet<AdditionalOptions>) => FullTap & IfSet<AdditionalOptions>;
  // }
  // type ArgumentNames<T extends any[]> = FixedSizeArray<T['length'], string>;
  // export declare class AsyncHook<T, R, AdditionalOptions = UnsetAdditionalOptions> extends Hook<
  //   T,
  //   R,
  //   AdditionalOptions
  // > {
  //   tapAsync(
  //     options: string | (Tap & IfSet<AdditionalOptions>),
  //     fn: (...args: Append<AsArray<T>, InnerCallback<Error, R>>) => void,
  //   ): void;
  //   tapPromise(
  //     options: string | (Tap & IfSet<AdditionalOptions>),
  //     fn: (...args: AsArray<T>) => Promise<R>,
  //   ): void;
  // }
  var CALL_DELEGATE = function CALL_DELEGATE() {
    this.call = this._createCall('sync');
    return this.call.apply(this, arguments);
  };
  var CALL_ASYNC_DELEGATE = function CALL_ASYNC_DELEGATE() {
    this.callAsync = this._createCall('async');
    return this.callAsync.apply(this, arguments);
  };
  var PROMISE_DELEGATE = function PROMISE_DELEGATE() {
    this.promise = this._createCall('promise');
    return this.promise.apply(this, arguments);
  };
  var Hook = /*#__PURE__*/function () {
    function Hook(args, name) {
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      this.name = void 0;
      this.taps = void 0;
      this.promise = void 0;
      this._promise = void 0;
      this._args = args;
      this.name = name;
      this.taps = [];
      this.interceptors = [];
      this._call = CALL_DELEGATE;
      this.call = CALL_DELEGATE;
      this._callAsync = CALL_ASYNC_DELEGATE;
      this.callAsync = CALL_ASYNC_DELEGATE;
      this._promise = PROMISE_DELEGATE;
      this.promise = PROMISE_DELEGATE;
      this._x = undefined;
      // this.compile = this.compile;
      // this.tap = this.tap;
      // this.tapAsync = this.tapAsync;
      // this.tapPromise = this.tapPromise;
    }
    var _proto = Hook.prototype;
    _proto.compile = function compile(options) {
      throw new Error('Abstract: should be overridden');
    };
    _proto._createCall = function _createCall(type) {
      return this.compile({
        taps: this.taps,
        interceptors: this.interceptors,
        args: this._args,
        type: type
      });
    };
    _proto._tap = function _tap(type, options, fn) {
      if (typeof options === 'string') {
        options = {
          name: options.trim()
        };
      } else if (typeof options !== 'object' || options === null) {
        throw new Error('Invalid tap options');
      }
      if (typeof options.name !== 'string' || options.name === '') {
        throw new Error('Missing name for tap');
      }
      // if (typeof options.context !== "undefined") {
      // 	deprecateContext();
      // }
      options = Object.assign({
        type: type,
        fn: fn
      }, options);
      options = this._runRegisterInterceptors(options);
      this._insert(options);
    };
    _proto.tap = function tap(options, fn) {
      this._tap('sync', options, fn);
    };
    _proto.tapAsync = function tapAsync(options, fn) {
      this._tap('async', options, fn);
    };
    _proto.tapPromise = function tapPromise(options, fn) {
      this._tap('promise', options, fn);
    };
    _proto._runRegisterInterceptors = function _runRegisterInterceptors(options) {
      for (var _iterator = _createForOfIteratorHelperLoose(this.interceptors), _step; !(_step = _iterator()).done;) {
        var interceptor = _step.value;
        if (interceptor.register) {
          var newOptions = interceptor.register(options);
          if (newOptions !== undefined) {
            options = newOptions;
          }
        }
      }
      return options;
    };
    _proto.withOptions = function withOptions(options) {
      var _this = this;
      var mergeOptions = function mergeOptions(opt) {
        return Object.assign({}, options, typeof opt === 'string' ? {
          name: opt
        } : opt);
      };
      return {
        name: this.name,
        tap: function tap(opt, fn) {
          return _this.tap(mergeOptions(opt), fn);
        },
        tapAsync: function tapAsync(opt, fn) {
          return _this.tapAsync(mergeOptions(opt), fn);
        },
        tapPromise: function tapPromise(opt, fn) {
          return _this.tapPromise(mergeOptions(opt), fn);
        },
        // intercept: (interceptor) => this.intercept(interceptor),
        isUsed: function isUsed() {
          return _this.isUsed();
        },
        withOptions: function withOptions(opt) {
          return _this.withOptions(mergeOptions(opt));
        }
      };
    };
    _proto.isUsed = function isUsed() {
      return this.taps.length > 0 || this.interceptors.length > 0;
    }
    // intercept(interceptor: HookInterceptor<T, R, AdditionalOptions>) {
    //   this._resetCompilation();
    //   this.interceptors.push(Object.assign({}, interceptor));
    //   if (interceptor.register) {
    //     for (let i = 0; i < this.taps.length; i++) {
    //       this.taps[i] = interceptor.register(this.taps[i]);
    //     }
    //   }
    // }
    ;
    _proto._resetCompilation = function _resetCompilation() {
      this.call = this._call;
      this.callAsync = this._callAsync;
      this.promise = this._promise;
    };
    _proto._insert = function _insert(item) {
      this._resetCompilation();
      var before;
      if (typeof item.before === 'string') {
        before = new Set([item.before]);
      } else if (Array.isArray(item.before)) {
        before = new Set(item.before);
      }
      var stage = 0;
      if (typeof item.stage === 'number') {
        stage = item.stage;
      }
      var i = this.taps.length;
      while (i > 0) {
        i--;
        var x = this.taps[i];
        this.taps[i + 1] = x;
        var xStage = x.stage || 0;
        if (before) {
          if (before.has(x.name)) {
            before.delete(x.name);
            continue;
          }
          if (before.size > 0) {
            continue;
          }
        }
        if (xStage > stage) {
          continue;
        }
        i++;
        break;
      }
      this.taps[i] = item;
    };
    return Hook;
  }();
  // Object.setPrototypeOf(Hook.prototype, null);

  // @ts-nocheck
  var HookCodeFactory = /*#__PURE__*/function () {
    function HookCodeFactory(config) {
      this.config = config;
      this.options = undefined;
      this._args = undefined;
    }
    var _proto = HookCodeFactory.prototype;
    _proto.create = function create(options) {
      this.init(options);
      var fn;
      switch (this.options.type) {
        case 'sync':
          fn = new Function(this.args(), '"use strict";\n' + this.header() + this.contentWithInterceptors({
            onError: function onError(err) {
              return "throw " + err + ";\n";
            },
            onResult: function onResult(result) {
              return "return " + result + ";\n";
            },
            resultReturns: true,
            onDone: function onDone() {
              return '';
            },
            rethrowIfPossible: true
          }));
          break;
        case 'async':
          fn = new Function(this.args({
            after: '_callback'
          }), '"use strict";\n' + this.header() + this.contentWithInterceptors({
            onError: function onError(err) {
              return "_callback(" + err + ");\n";
            },
            onResult: function onResult(result) {
              return "_callback(null, " + result + ");\n";
            },
            onDone: function onDone() {
              return '_callback();\n';
            }
          }));
          break;
        case 'promise':
          var errorHelperUsed = false;
          var content = this.contentWithInterceptors({
            onError: function onError(err) {
              errorHelperUsed = true;
              return "_error(" + err + ");\n";
            },
            onResult: function onResult(result) {
              return "_resolve(" + result + ");\n";
            },
            onDone: function onDone() {
              return '_resolve();\n';
            }
          });
          var code = '';
          code += '"use strict";\n';
          code += this.header();
          code += 'return new Promise((function(_resolve, _reject) {\n';
          if (errorHelperUsed) {
            code += 'var _sync = true;\n';
            code += 'function _error(_err) {\n';
            code += 'if(_sync)\n';
            code += '_resolve(Promise.resolve().then((function() { throw _err; })));\n';
            code += 'else\n';
            code += '_reject(_err);\n';
            code += '};\n';
          }
          code += content;
          if (errorHelperUsed) {
            code += '_sync = false;\n';
          }
          code += '}));\n';
          fn = new Function(this.args(), code);
          break;
      }
      this.deinit();
      return fn;
    };
    _proto.setup = function setup(instance, options) {
      instance._x = options.taps.map(function (t) {
        return t.fn;
      });
    }
    /**
     * @param {{ type: "sync" | "promise" | "async", taps: Array<Tap>, interceptors: Array<Interceptor> }} options
     */;
    _proto.init = function init(options) {
      this.options = options;
      this._args = options.args.slice();
    };
    _proto.deinit = function deinit() {
      this.options = undefined;
      this._args = undefined;
    };
    _proto.contentWithInterceptors = function contentWithInterceptors(options) {
      var _this = this;
      if (this.options.interceptors.length > 0) {
        var onError = options.onError;
        var onResult = options.onResult;
        var onDone = options.onDone;
        var code = '';
        for (var i = 0; i < this.options.interceptors.length; i++) {
          var interceptor = this.options.interceptors[i];
          if (interceptor.call) {
            code += this.getInterceptor(i) + ".call(" + this.args({
              before: interceptor.context ? '_context' : undefined
            }) + ");\n";
          }
        }
        code += this.content(Object.assign(options, {
          onError: onError && function (err) {
            var code = '';
            for (var _i = 0; _i < _this.options.interceptors.length; _i++) {
              var _interceptor = _this.options.interceptors[_i];
              if (_interceptor.error) {
                code += _this.getInterceptor(_i) + ".error(" + err + ");\n";
              }
            }
            code += onError(err);
            return code;
          },
          onResult: onResult && function (result) {
            var code = '';
            for (var _i2 = 0; _i2 < _this.options.interceptors.length; _i2++) {
              var _interceptor2 = _this.options.interceptors[_i2];
              if (_interceptor2.result) {
                code += _this.getInterceptor(_i2) + ".result(" + result + ");\n";
              }
            }
            code += onResult(result);
            return code;
          },
          onDone: onDone && function () {
            var code = '';
            for (var _i3 = 0; _i3 < _this.options.interceptors.length; _i3++) {
              var _interceptor3 = _this.options.interceptors[_i3];
              if (_interceptor3.done) {
                code += _this.getInterceptor(_i3) + ".done();\n";
              }
            }
            code += onDone();
            return code;
          }
        }));
        return code;
      } else {
        return this.content(options);
      }
    };
    _proto.header = function header() {
      var code = '';
      if (this.needContext()) {
        code += 'var _context = {};\n';
      } else {
        code += 'var _context;\n';
      }
      code += 'var _x = this._x;\n';
      if (this.options.interceptors.length > 0) {
        code += 'var _taps = this.taps;\n';
        code += 'var _interceptors = this.interceptors;\n';
      }
      return code;
    };
    _proto.needContext = function needContext() {
      for (var _iterator = _createForOfIteratorHelperLoose(this.options.taps), _step; !(_step = _iterator()).done;) {
        var tap = _step.value;
        if (tap.context) return true;
      }
      return false;
    };
    _proto.callTap = function callTap(tapIndex, _ref) {
      var onError = _ref.onError,
        onResult = _ref.onResult,
        onDone = _ref.onDone,
        rethrowIfPossible = _ref.rethrowIfPossible;
      var code = '';
      var hasTapCached = false;
      for (var i = 0; i < this.options.interceptors.length; i++) {
        var interceptor = this.options.interceptors[i];
        if (interceptor.tap) {
          if (!hasTapCached) {
            code += "var _tap" + tapIndex + " = " + this.getTap(tapIndex) + ";\n";
            hasTapCached = true;
          }
          code += this.getInterceptor(i) + ".tap(" + (interceptor.context ? '_context, ' : '') + "_tap" + tapIndex + ");\n";
        }
      }
      code += "var _fn" + tapIndex + " = " + this.getTapFn(tapIndex) + ";\n";
      var tap = this.options.taps[tapIndex];
      switch (tap.type) {
        case 'sync':
          if (!rethrowIfPossible) {
            code += "var _hasError" + tapIndex + " = false;\n";
            code += 'try {\n';
          }
          if (onResult) {
            code += "var _result" + tapIndex + " = _fn" + tapIndex + "(" + this.args({
              before: tap.context ? '_context' : undefined
            }) + ");\n";
          } else {
            code += "_fn" + tapIndex + "(" + this.args({
              before: tap.context ? '_context' : undefined
            }) + ");\n";
          }
          if (!rethrowIfPossible) {
            code += '} catch(_err) {\n';
            code += "_hasError" + tapIndex + " = true;\n";
            code += onError('_err');
            code += '}\n';
            code += "if(!_hasError" + tapIndex + ") {\n";
          }
          if (onResult) {
            code += onResult("_result" + tapIndex);
          }
          if (onDone) {
            code += onDone();
          }
          if (!rethrowIfPossible) {
            code += '}\n';
          }
          break;
        case 'async':
          var cbCode = '';
          if (onResult) cbCode += "(function(_err" + tapIndex + ", _result" + tapIndex + ") {\n";else cbCode += "(function(_err" + tapIndex + ") {\n";
          cbCode += "if(_err" + tapIndex + ") {\n";
          cbCode += onError("_err" + tapIndex);
          cbCode += '} else {\n';
          if (onResult) {
            cbCode += onResult("_result" + tapIndex);
          }
          if (onDone) {
            cbCode += onDone();
          }
          cbCode += '}\n';
          cbCode += '})';
          code += "_fn" + tapIndex + "(" + this.args({
            before: tap.context ? '_context' : undefined,
            after: cbCode
          }) + ");\n";
          break;
        case 'promise':
          code += "var _hasResult" + tapIndex + " = false;\n";
          code += "var _promise" + tapIndex + " = _fn" + tapIndex + "(" + this.args({
            before: tap.context ? '_context' : undefined
          }) + ");\n";
          code += "if (!_promise" + tapIndex + " || !_promise" + tapIndex + ".then)\n";
          code += "  throw new Error('Tap function (tapPromise) did not return promise (returned ' + _promise" + tapIndex + " + ')');\n";
          code += "_promise" + tapIndex + ".then((function(_result" + tapIndex + ") {\n";
          code += "_hasResult" + tapIndex + " = true;\n";
          if (onResult) {
            code += onResult("_result" + tapIndex);
          }
          if (onDone) {
            code += onDone();
          }
          code += "}), function(_err" + tapIndex + ") {\n";
          code += "if(_hasResult" + tapIndex + ") throw _err" + tapIndex + ";\n";
          code += onError("_err" + tapIndex);
          code += '});\n';
          break;
      }
      return code;
    };
    _proto.callTapsSeries = function callTapsSeries(_ref2) {
      var _this2 = this;
      var _onError = _ref2.onError,
        onResult = _ref2.onResult,
        resultReturns = _ref2.resultReturns,
        onDone = _ref2.onDone,
        doneReturns = _ref2.doneReturns,
        rethrowIfPossible = _ref2.rethrowIfPossible;
      if (this.options.taps.length === 0) return onDone();
      var firstAsync = this.options.taps.findIndex(function (t) {
        return t.type !== 'sync';
      });
      var somethingReturns = resultReturns || doneReturns;
      var code = '';
      var current = onDone;
      var unrollCounter = 0;
      var _loop = function _loop(j) {
        var i = j;
        var unroll = current !== onDone && (_this2.options.taps[i].type !== 'sync' || unrollCounter++ > 20);
        if (unroll) {
          unrollCounter = 0;
          code += "function _next" + i + "() {\n";
          code += current();
          code += "}\n";
          current = function current() {
            return (somethingReturns ? 'return ' : '') + "_next" + i + "();\n";
          };
        }
        var done = current;
        var doneBreak = function doneBreak(skipDone) {
          if (skipDone) return '';
          return onDone();
        };
        var content = _this2.callTap(i, {
          onError: function onError(error) {
            return _onError(i, error, done, doneBreak);
          },
          onResult: onResult && function (result) {
            return onResult(i, result, done, doneBreak);
          },
          onDone: !onResult && done,
          rethrowIfPossible: rethrowIfPossible && (firstAsync < 0 || i < firstAsync)
        });
        current = function current() {
          return content;
        };
      };
      for (var j = this.options.taps.length - 1; j >= 0; j--) {
        _loop(j);
      }
      code += current();
      return code;
    };
    _proto.callTapsLooping = function callTapsLooping(_ref3) {
      var onError = _ref3.onError,
        onDone = _ref3.onDone,
        rethrowIfPossible = _ref3.rethrowIfPossible;
      if (this.options.taps.length === 0) return onDone();
      var syncOnly = this.options.taps.every(function (t) {
        return t.type === 'sync';
      });
      var code = '';
      if (!syncOnly) {
        code += 'var _looper = (function() {\n';
        code += 'var _loopAsync = false;\n';
      }
      code += 'var _loop;\n';
      code += 'do {\n';
      code += '_loop = false;\n';
      for (var i = 0; i < this.options.interceptors.length; i++) {
        var interceptor = this.options.interceptors[i];
        if (interceptor.loop) {
          code += this.getInterceptor(i) + ".loop(" + this.args({
            before: interceptor.context ? '_context' : undefined
          }) + ");\n";
        }
      }
      code += this.callTapsSeries({
        onError: onError,
        onResult: function onResult(i, result, next, doneBreak) {
          var code = '';
          code += "if(" + result + " !== undefined) {\n";
          code += '_loop = true;\n';
          if (!syncOnly) code += 'if(_loopAsync) _looper();\n';
          code += doneBreak(true);
          code += "} else {\n";
          code += next();
          code += "}\n";
          return code;
        },
        onDone: onDone && function () {
          var code = '';
          code += 'if(!_loop) {\n';
          code += onDone();
          code += '}\n';
          return code;
        },
        rethrowIfPossible: rethrowIfPossible && syncOnly
      });
      code += '} while(_loop);\n';
      if (!syncOnly) {
        code += '_loopAsync = true;\n';
        code += '});\n';
        code += '_looper();\n';
      }
      return code;
    };
    _proto.callTapsParallel = function callTapsParallel(_ref4) {
      var _this3 = this;
      var _onError2 = _ref4.onError,
        onResult = _ref4.onResult,
        onDone = _ref4.onDone,
        rethrowIfPossible = _ref4.rethrowIfPossible,
        _ref4$onTap = _ref4.onTap,
        onTap = _ref4$onTap === void 0 ? function (i, run) {
          return run();
        } : _ref4$onTap;
      if (this.options.taps.length <= 1) {
        return this.callTapsSeries({
          onError: _onError2,
          onResult: onResult,
          onDone: onDone,
          rethrowIfPossible: rethrowIfPossible
        });
      }
      var code = '';
      code += 'do {\n';
      code += "var _counter = " + this.options.taps.length + ";\n";
      if (onDone) {
        code += 'var _done = (function() {\n';
        code += onDone();
        code += '});\n';
      }
      var _loop2 = function _loop2(i) {
        var done = function done() {
          if (onDone) return 'if(--_counter === 0) _done();\n';else return '--_counter;';
        };
        var doneBreak = function doneBreak(skipDone) {
          if (skipDone || !onDone) return '_counter = 0;\n';else return '_counter = 0;\n_done();\n';
        };
        code += 'if(_counter <= 0) break;\n';
        code += onTap(i, function () {
          return _this3.callTap(i, {
            onError: function onError(error) {
              var code = '';
              code += 'if(_counter > 0) {\n';
              code += _onError2(i, error, done, doneBreak);
              code += '}\n';
              return code;
            },
            onResult: onResult && function (result) {
              var code = '';
              code += 'if(_counter > 0) {\n';
              code += onResult(i, result, done, doneBreak);
              code += '}\n';
              return code;
            },
            onDone: !onResult && function () {
              return done();
            },
            rethrowIfPossible: rethrowIfPossible
          });
        }, done, doneBreak);
      };
      for (var i = 0; i < this.options.taps.length; i++) {
        _loop2(i);
      }
      code += '} while(false);\n';
      return code;
    };
    _proto.args = function args(_temp) {
      var _ref5 = _temp === void 0 ? {} : _temp,
        before = _ref5.before,
        after = _ref5.after;
      var allArgs = this._args;
      if (before) allArgs = [before].concat(allArgs);
      if (after) allArgs = allArgs.concat(after);
      if (allArgs.length === 0) {
        return '';
      } else {
        return allArgs.join(', ');
      }
    };
    _proto.getTapFn = function getTapFn(idx) {
      return "_x[" + idx + "]";
    };
    _proto.getTap = function getTap(idx) {
      return "_taps[" + idx + "]";
    };
    _proto.getInterceptor = function getInterceptor(idx) {
      return "_interceptors[" + idx + "]";
    };
    return HookCodeFactory;
  }();

  var AsyncParallelHookCodeFactory = /*#__PURE__*/function (_HookCodeFactory) {
    _inheritsLoose(AsyncParallelHookCodeFactory, _HookCodeFactory);
    function AsyncParallelHookCodeFactory() {
      return _HookCodeFactory.apply(this, arguments) || this;
    }
    var _proto = AsyncParallelHookCodeFactory.prototype;
    _proto.content = function content(_ref) {
      var _onError = _ref.onError,
        onDone = _ref.onDone;
      return this.callTapsParallel({
        onError: function onError(i, err, done, doneBreak) {
          return _onError(err) + doneBreak(true);
        },
        onDone: onDone
      });
    };
    return AsyncParallelHookCodeFactory;
  }(HookCodeFactory);
  var factory = new AsyncParallelHookCodeFactory();
  var COMPILE = function COMPILE(options) {
    factory.setup(this, options);
    return factory.create(options);
  };
  var AsyncParallelHook = /*#__PURE__*/function (_Hook) {
    _inheritsLoose(AsyncParallelHook, _Hook);
    function AsyncParallelHook(args, name) {
      var _this;
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      _this = _Hook.call(this, args, name) || this;
      _this.compile = COMPILE;
      _this._call = undefined;
      _this.call = undefined;
      return _this;
    }
    return AsyncParallelHook;
  }(Hook);
  // export function AsyncParallelHook(args = [], name = undefined) {
  //   const hook = new Hook(args, name);
  //   hook.constructor = AsyncParallelHook;
  //   hook.compile = COMPILE;
  //   hook._call = undefined;
  //   hook.call = undefined;
  //   return hook;
  // }
  // AsyncParallelHook.prototype = null;

  var AsyncSeriesWaterfallHookCodeFactory = /*#__PURE__*/function (_HookCodeFactory) {
    _inheritsLoose(AsyncSeriesWaterfallHookCodeFactory, _HookCodeFactory);
    function AsyncSeriesWaterfallHookCodeFactory() {
      return _HookCodeFactory.apply(this, arguments) || this;
    }
    var _proto = AsyncSeriesWaterfallHookCodeFactory.prototype;
    _proto.content = function content(_ref) {
      var _this = this;
      var _onError = _ref.onError,
        onResult = _ref.onResult,
        onDone = _ref.onDone;
      return this.callTapsSeries({
        onError: function onError(i, err, next, doneBreak) {
          return _onError(err) + doneBreak(true);
        },
        onResult: function onResult(i, result, next) {
          var code = '';
          code += "if(" + result + " !== undefined) {\n";
          code += _this._args[0] + " = " + result + ";\n";
          code += "}\n";
          code += next();
          return code;
        },
        onDone: function onDone() {
          return onResult(_this._args[0]);
        }
      });
    };
    return AsyncSeriesWaterfallHookCodeFactory;
  }(HookCodeFactory);
  var factory$1 = new AsyncSeriesWaterfallHookCodeFactory();
  var COMPILE$1 = function COMPILE(options) {
    factory$1.setup(this, options);
    return factory$1.create(options);
  };
  // export function AsyncSeriesWaterfallHook(args = [], name = undefined) {
  //   if (args.length < 1) throw new Error('Waterfall hooks must have at least one argument');
  //   const hook = new Hook(args, name);
  //   hook.constructor = AsyncSeriesWaterfallHook;
  //   hook.compile = COMPILE;
  //   hook._call = undefined;
  //   hook.call = undefined;
  //   return hook;
  // }
  // AsyncSeriesWaterfallHook.prototype = null;
  var AsyncSeriesWaterfallHook = /*#__PURE__*/function (_Hook) {
    _inheritsLoose(AsyncSeriesWaterfallHook, _Hook);
    function AsyncSeriesWaterfallHook(args, name) {
      var _this2;
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      _this2 = _Hook.call(this, args, name) || this;
      _this2.compile = COMPILE$1;
      _this2._call = undefined;
      _this2.call = undefined;
      return _this2;
    }
    return AsyncSeriesWaterfallHook;
  }(Hook);

  var SyncHookCodeFactory = /*#__PURE__*/function (_HookCodeFactory) {
    _inheritsLoose(SyncHookCodeFactory, _HookCodeFactory);
    function SyncHookCodeFactory() {
      return _HookCodeFactory.apply(this, arguments) || this;
    }
    var _proto = SyncHookCodeFactory.prototype;
    _proto.content = function content(_ref) {
      var _onError = _ref.onError,
        onDone = _ref.onDone,
        rethrowIfPossible = _ref.rethrowIfPossible;
      return this.callTapsSeries({
        onError: function onError(i, err) {
          return _onError(err);
        },
        onDone: onDone,
        rethrowIfPossible: rethrowIfPossible
      });
    };
    return SyncHookCodeFactory;
  }(HookCodeFactory);
  var factory$2 = new SyncHookCodeFactory();
  var TAP_ASYNC = function TAP_ASYNC() {
    throw new Error('tapAsync is not supported on a SyncHook');
  };
  var TAP_PROMISE = function TAP_PROMISE() {
    throw new Error('tapPromise is not supported on a SyncHook');
  };
  var COMPILE$2 = function COMPILE(options) {
    factory$2.setup(this, options);
    return factory$2.create(options);
  };
  var SyncHook = /*#__PURE__*/function (_Hook) {
    _inheritsLoose(SyncHook, _Hook);
    function SyncHook(args, name) {
      var _this;
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      _this = _Hook.call(this, args, name) || this;
      _this.tapAsync = TAP_ASYNC;
      _this.tapPromise = TAP_PROMISE;
      _this.compile = COMPILE$2;
      return _this;
    }
    return SyncHook;
  }(Hook);
  // export function SyncHook(args = [], name = undefined) {
  //   const hook = new Hook(args, name);
  //   hook.constructor = SyncHook;
  //   hook.tapAsync = TAP_ASYNC;
  //   hook.tapPromise = TAP_PROMISE;
  //   hook.compile = COMPILE;
  //   return hook;
  // }
  // SyncHook.prototype = null;

  var SyncWaterfallHookCodeFactory = /*#__PURE__*/function (_HookCodeFactory) {
    _inheritsLoose(SyncWaterfallHookCodeFactory, _HookCodeFactory);
    function SyncWaterfallHookCodeFactory() {
      return _HookCodeFactory.apply(this, arguments) || this;
    }
    var _proto = SyncWaterfallHookCodeFactory.prototype;
    _proto.content = function content(_ref) {
      var _this = this;
      var _onError = _ref.onError,
        onResult = _ref.onResult,
        resultReturns = _ref.resultReturns,
        rethrowIfPossible = _ref.rethrowIfPossible;
      return this.callTapsSeries({
        onError: function onError(i, err) {
          return _onError(err);
        },
        onResult: function onResult(i, result, next) {
          var code = '';
          code += "if(" + result + " !== undefined) {\n";
          code += _this._args[0] + " = " + result + ";\n";
          code += "}\n";
          code += next();
          return code;
        },
        onDone: function onDone() {
          return onResult(_this._args[0]);
        },
        doneReturns: resultReturns,
        rethrowIfPossible: rethrowIfPossible
      });
    };
    return SyncWaterfallHookCodeFactory;
  }(HookCodeFactory);
  var factory$3 = new SyncWaterfallHookCodeFactory();
  var TAP_ASYNC$1 = function TAP_ASYNC() {
    throw new Error('tapAsync is not supported on a SyncWaterfallHook');
  };
  var TAP_PROMISE$1 = function TAP_PROMISE() {
    throw new Error('tapPromise is not supported on a SyncWaterfallHook');
  };
  var COMPILE$3 = function COMPILE(options) {
    factory$3.setup(this, options);
    return factory$3.create(options);
  };
  var SyncWaterfallHook = /*#__PURE__*/function (_SyncHook) {
    _inheritsLoose(SyncWaterfallHook, _SyncHook);
    function SyncWaterfallHook(args, name) {
      var _this2;
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      _this2 = _SyncHook.call(this, args, name) || this;
      _this2.tapAsync = TAP_ASYNC$1;
      _this2.tapPromise = TAP_PROMISE$1;
      _this2.compile = COMPILE$3;
      return _this2;
    }
    return SyncWaterfallHook;
  }(SyncHook);
  // export function SyncWaterfallHook(args = [], name = undefined) {
  //   if (args.length < 1) throw new Error('Waterfall hooks must have at least one argument');
  //   const hook = new Hook(args, name);
  //   hook.constructor = SyncWaterfallHook;
  //   hook.tapAsync = TAP_ASYNC;
  //   hook.tapPromise = TAP_PROMISE;
  //   hook.compile = COMPILE;
  //   return hook;
  // }
  // SyncWaterfallHook.prototype = null;

  var genericFontFamilies = ['serif', 'sans-serif', 'monospace', 'cursive', 'fantasy', 'system-ui'];
  var stringRegExp = /([\"\'])[^\'\"]+\1/;
  function toFontString(attributes) {
    var fontSize = attributes.fontSize,
      fontFamily = attributes.fontFamily,
      fontStyle = attributes.fontStyle,
      fontVariant = attributes.fontVariant,
      fontWeight = attributes.fontWeight;
    // build canvas api font setting from individual components. Convert a numeric this.fontSize to px
    // const fontSizeString: string = isNumber(fontSize) ? `${fontSize}px` : fontSize.toString();
    var fontSizeString = isNumber(fontSize) && fontSize + "px" || '16px';
    // Clean-up fontFamily property by quoting each font name
    // this will support font names with spaces
    // @ts-ignore
    var fontFamilies = isString(fontFamily) ? fontFamily.split(',') : [fontFamily];
    for (var i = fontFamilies.length - 1; i >= 0; i--) {
      // Trim any extra white-space
      var _fontFamily = fontFamilies[i].trim();
      // Check if font already contains strings
      if (!stringRegExp.test(_fontFamily) && genericFontFamilies.indexOf(_fontFamily) < 0) {
        _fontFamily = "\"" + _fontFamily + "\"";
      }
      fontFamilies[i] = _fontFamily;
    }
    return fontStyle + " " + fontVariant + " " + fontWeight + " " + fontSizeString + " " + fontFamilies.join(',');
  }

  /**
   * Blink used them in code generation(css_properties.json5)
   */
  var BUILT_IN_PROPERTIES = [{
    /**
     * used in CSS Layout API
     * eg. `display: 'flex'`
     */
    n: 'display',
    k: ['none']
  }, {
    /**
     * range [0.0, 1.0]
     * @see https://developer.mozilla.org/en-US/docs/Web/CSS/opacity
     */
    n: 'opacity',
    int: true,
    inh: true,
    d: '1',
    syntax: PropertySyntax.OPACITY_VALUE
  }, {
    /**
     * inheritable, range [0.0, 1.0]
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/fill-opacity
     * @see https://svgwg.org/svg2-draft/painting.html#FillOpacity
     */
    n: 'fillOpacity',
    int: true,
    inh: true,
    d: '1',
    syntax: PropertySyntax.OPACITY_VALUE
  }, {
    /**
     * inheritable, range [0.0, 1.0]
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-opacity
     * @see https://svgwg.org/svg2-draft/painting.html#StrokeOpacity
     */
    n: 'strokeOpacity',
    int: true,
    inh: true,
    d: '1',
    syntax: PropertySyntax.OPACITY_VALUE
  }, {
    /**
     * background-color is not inheritable
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Tutorial/Fills_and_Strokes
     */
    n: 'fill',
    int: true,
    k: ['none'],
    d: 'none',
    syntax: PropertySyntax.PAINT
  }, {
    n: 'fillRule',
    k: ['nonzero', 'evenodd'],
    d: 'nonzero'
  },
  /**
   * default to none
   * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke#usage_notes
   */
  {
    n: 'stroke',
    int: true,
    k: ['none'],
    d: 'none',
    syntax: PropertySyntax.PAINT
  }, {
    n: 'shadowType',
    k: ['inner', 'outer', 'both'],
    d: 'outer',
    l: true
  }, {
    n: 'shadowColor',
    int: true,
    syntax: PropertySyntax.COLOR
  }, {
    n: 'shadowOffsetX',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'shadowOffsetY',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'shadowBlur',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.SHADOW_BLUR
  }, {
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-width
     */
    n: 'lineWidth',
    int: true,
    inh: true,
    d: '1',
    l: true,
    a: ['strokeWidth'],
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'increasedLineWidthForHitTesting',
    inh: true,
    d: '0',
    l: true,
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'lineJoin',
    inh: true,
    l: true,
    a: ['strokeLinejoin'],
    k: ['miter', 'bevel', 'round'],
    d: 'miter'
  }, {
    n: 'lineCap',
    inh: true,
    l: true,
    a: ['strokeLinecap'],
    k: ['butt', 'round', 'square'],
    d: 'butt'
  }, {
    n: 'lineDash',
    int: true,
    inh: true,
    k: ['none'],
    a: ['strokeDasharray'],
    syntax: PropertySyntax.LENGTH_PERCENTAGE_12
  }, {
    n: 'lineDashOffset',
    int: true,
    inh: true,
    d: '0',
    a: ['strokeDashoffset'],
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'offsetPath',
    syntax: PropertySyntax.DEFINED_PATH
  }, {
    n: 'offsetDistance',
    int: true,
    syntax: PropertySyntax.OFFSET_DISTANCE
  }, {
    n: 'dx',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'dy',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'zIndex',
    ind: true,
    int: true,
    d: '0',
    k: ['auto'],
    syntax: PropertySyntax.Z_INDEX
  }, {
    n: 'visibility',
    k: ['visible', 'hidden'],
    ind: true,
    inh: true,
    /**
     * TODO: support interpolation
     * @see https://developer.mozilla.org/en-US/docs/Web/CSS/visibility#interpolation
     */
    // int: true,
    d: 'visible'
  }, {
    n: 'pointerEvents',
    inh: true,
    k: ['none', 'auto', 'stroke', 'fill', 'painted', 'visible', 'visiblestroke', 'visiblefill', 'visiblepainted',
    // 'bounding-box',
    'all'],
    d: 'auto'
  }, {
    n: 'filter',
    ind: true,
    l: true,
    k: ['none'],
    d: 'none',
    syntax: PropertySyntax.FILTER
  }, {
    n: 'clipPath',
    syntax: PropertySyntax.DEFINED_PATH
  }, {
    n: 'textPath',
    syntax: PropertySyntax.DEFINED_PATH
  }, {
    n: 'textPathSide',
    k: ['left', 'right'],
    d: 'left'
  }, {
    n: 'textPathStartOffset',
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'transform',
    p: 100,
    int: true,
    k: ['none'],
    d: 'none',
    syntax: PropertySyntax.TRANSFORM
  }, {
    n: 'transformOrigin',
    p: 100,
    // int: true,
    d: function d(nodeName) {
      if (nodeName === Shape.CIRCLE || nodeName === Shape.ELLIPSE) {
        return 'center';
      }
      if (nodeName === Shape.TEXT) {
        return 'text-anchor';
      }
      return 'left top';
    },
    l: true,
    syntax: PropertySyntax.TRANSFORM_ORIGIN
  }, {
    n: 'anchor',
    p: 99,
    d: function d(nodeName) {
      if (nodeName === Shape.CIRCLE || nodeName === Shape.ELLIPSE) {
        return '0.5 0.5';
      }
      return '0 0';
    },
    l: true,
    syntax: PropertySyntax.LENGTH_PERCENTAGE_12
  },
  // <circle> & <ellipse>
  {
    n: 'cx',
    int: true,
    d: '0',
    syntax: PropertySyntax.COORDINATE
  }, {
    n: 'cy',
    int: true,
    d: '0',
    syntax: PropertySyntax.COORDINATE
  }, {
    n: 'r',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'rx',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'ry',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  },
  // Rect Image Group
  {
    // x in local space
    n: 'x',
    int: true,
    d: '0',
    syntax: PropertySyntax.COORDINATE
  }, {
    // y in local space
    n: 'y',
    int: true,
    d: '0',
    syntax: PropertySyntax.COORDINATE
  }, {
    // z in local space
    n: 'z',
    int: true,
    d: '0',
    syntax: PropertySyntax.COORDINATE
  }, {
    n: 'width',
    int: true,
    l: true,
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/width
     */
    k: ['auto', 'fit-content', 'min-content', 'max-content'],
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'height',
    int: true,
    l: true,
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/height
     */
    k: ['auto', 'fit-content', 'min-content', 'max-content'],
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'radius',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax.LENGTH_PERCENTAGE_14
  },
  // Line
  {
    n: 'x1',
    int: true,
    l: true,
    syntax: PropertySyntax.COORDINATE
  }, {
    n: 'y1',
    int: true,
    l: true,
    syntax: PropertySyntax.COORDINATE
  }, {
    n: 'z1',
    int: true,
    l: true,
    syntax: PropertySyntax.COORDINATE
  }, {
    n: 'x2',
    int: true,
    l: true,
    syntax: PropertySyntax.COORDINATE
  }, {
    n: 'y2',
    int: true,
    l: true,
    syntax: PropertySyntax.COORDINATE
  }, {
    n: 'z2',
    int: true,
    l: true,
    syntax: PropertySyntax.COORDINATE
  },
  // Path
  {
    n: 'path',
    int: true,
    l: true,
    d: '',
    a: ['d'],
    syntax: PropertySyntax.PATH,
    p: 50
  },
  // Polyline & Polygon
  {
    n: 'points',
    l: true,
    syntax: PropertySyntax.LIST_OF_POINTS,
    p: 50
  },
  // Text
  {
    n: 'text',
    l: true,
    d: '',
    syntax: PropertySyntax.TEXT,
    p: 50
  }, {
    n: 'textTransform',
    l: true,
    inh: true,
    k: ['capitalize', 'uppercase', 'lowercase', 'none'],
    d: 'none',
    syntax: PropertySyntax.TEXT_TRANSFORM,
    p: 51 // it must get parsed after text
  }, {
    n: 'font',
    l: true
  }, {
    n: 'fontSize',
    int: true,
    inh: true,
    /**
     * @see https://www.w3schools.com/css/css_font_size.asp
     */
    d: '16px',
    l: true,
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    n: 'fontFamily',
    l: true,
    inh: true,
    d: 'sans-serif'
  }, {
    n: 'fontStyle',
    l: true,
    inh: true,
    k: ['normal', 'italic', 'oblique'],
    d: 'normal'
  }, {
    n: 'fontWeight',
    l: true,
    inh: true,
    k: ['normal', 'bold', 'bolder', 'lighter'],
    d: 'normal'
  }, {
    n: 'fontVariant',
    l: true,
    inh: true,
    k: ['normal', 'small-caps'],
    d: 'normal'
  }, {
    n: 'lineHeight',
    l: true,
    syntax: PropertySyntax.LENGTH,
    int: true,
    d: '0'
  }, {
    n: 'letterSpacing',
    l: true,
    syntax: PropertySyntax.LENGTH,
    int: true,
    d: '0'
  }, {
    n: 'miterLimit',
    l: true,
    syntax: PropertySyntax.NUMBER,
    d: function d(nodeName) {
      if (nodeName === Shape.PATH || nodeName === Shape.POLYGON || nodeName === Shape.POLYLINE) {
        return '4';
      }
      return '10';
    }
  }, {
    n: 'wordWrap',
    l: true
  }, {
    n: 'wordWrapWidth',
    l: true
  }, {
    n: 'maxLines',
    l: true
  }, {
    n: 'textOverflow',
    l: true,
    d: 'clip'
  }, {
    n: 'leading',
    l: true
  }, {
    n: 'textBaseline',
    l: true,
    inh: true,
    k: ['top', 'hanging', 'middle', 'alphabetic', 'ideographic', 'bottom'],
    d: 'alphabetic'
  }, {
    n: 'textAlign',
    l: true,
    inh: true,
    k: ['start', 'center', 'middle', 'end', 'left', 'right'],
    d: 'start'
  },
  // {
  //   n: 'whiteSpace',
  //   l: true,
  // },
  {
    n: 'markerStart',
    syntax: PropertySyntax.MARKER
  }, {
    n: 'markerEnd',
    syntax: PropertySyntax.MARKER
  }, {
    n: 'markerMid',
    syntax: PropertySyntax.MARKER
  }, {
    n: 'markerStartOffset',
    syntax: PropertySyntax.LENGTH,
    l: true,
    int: true,
    d: '0'
  }, {
    n: 'markerEndOffset',
    syntax: PropertySyntax.LENGTH,
    l: true,
    int: true,
    d: '0'
  }];
  var propertyMetadataCache = {};
  var unresolvedProperties = new WeakMap();
  var priorityMap = {};
  var sortAttributeNames = memoize(function (attributeNames) {
    return attributeNames.sort(function (a, b) {
      return priorityMap[a] - priorityMap[b];
    });
  });
  var tmpVec3a = create$2();
  var tmpVec3b = create$2();
  var tmpVec3c = create$2();
  var isPropertyResolved = function isPropertyResolved(object, name) {
    var properties = unresolvedProperties.get(object);
    if (!properties || properties.length === 0) {
      return true;
    }
    return properties.includes(name);
  };
  var DefaultStyleValueRegistry = /*#__PURE__*/function () {
    /**
     * need recalc later
     */
    // dirty = false;
    function DefaultStyleValueRegistry() {
      var _this = this;
      BUILT_IN_PROPERTIES.forEach(function (property) {
        _this.registerMetadata(property);
      });
    }
    var _proto = DefaultStyleValueRegistry.prototype;
    _proto.registerMetadata = function registerMetadata(metadata) {
      [metadata.n].concat(metadata.a || []).forEach(function (name) {
        propertyMetadataCache[name] = metadata;
        priorityMap[name] = metadata.p || 0;
      });
    };
    _proto.unregisterMetadata = function unregisterMetadata(name) {
      delete propertyMetadataCache[name];
    };
    _proto.getPropertySyntax = function getPropertySyntax(syntax) {
      return runtime.CSSPropertySyntaxFactory[syntax];
    }
    /**
     * * parse value, eg.
     * fill: 'red' => CSSRGB
     * translateX: '10px' => CSSUnitValue { unit: 'px', value: 10 }
     * fontSize: '2em' => { unit: 'px', value: 32 }
     *
     * * calculate used value
     * * post process
     */;
    _proto.processProperties = function processProperties(object, attributes, options) {
      var _this2 = this;
      if (options === void 0) {
        options = {
          skipUpdateAttribute: false,
          skipParse: false,
          forceUpdateGeometry: false,
          usedAttributes: []
        };
      }
      var _options = options,
        skipUpdateAttribute = _options.skipUpdateAttribute,
        skipParse = _options.skipParse,
        forceUpdateGeometry = _options.forceUpdateGeometry,
        usedAttributes = _options.usedAttributes;
      var needUpdateGeometry = forceUpdateGeometry;
      var attributeNames = Object.keys(attributes);
      attributeNames.forEach(function (attributeName) {
        var _propertyMetadataCach;
        var name = formatAttributeName(attributeName);
        if (!skipUpdateAttribute) {
          object.attributes[name] = attributes[attributeName];
        }
        if (!needUpdateGeometry && ((_propertyMetadataCach = propertyMetadataCache[name]) === null || _propertyMetadataCach === void 0 ? void 0 : _propertyMetadataCach.l)) {
          needUpdateGeometry = true;
        }
      });
      if (!skipParse) {
        attributeNames.forEach(function (name) {
          object.computedStyle[name] = _this2.parseProperty(name, object.attributes[name], object);
        });
      }
      // let hasUnresolvedProperties = false;
      // parse according to priority
      attributeNames = sortAttributeNames((usedAttributes === null || usedAttributes === void 0 ? void 0 : usedAttributes.length) ? Array.from(new Set(attributeNames.concat(usedAttributes))) : attributeNames);
      // attributeNames.sort((a, b) => priorityMap[a] - priorityMap[b]);
      attributeNames.forEach(function (name) {
        // some style props maybe deleted after parsing such as `anchor` in Text
        if (name in object.computedStyle) {
          object.parsedStyle[name] = _this2.computeProperty(name, object.computedStyle[name], object);
        }
      });
      // if (hasUnresolvedProperties) {
      //   this.dirty = true;
      //   return;
      // }
      // update geometry
      if (needUpdateGeometry) {
        this.updateGeometry(object);
      }
      attributeNames.forEach(function (name) {
        if (name in object.parsedStyle) {
          _this2.postProcessProperty(name, object);
        }
      });
      if (object.children.length) {
        attributeNames.forEach(function (name) {
          if (name in object.parsedStyle && _this2.isPropertyInheritable(name)) {
            // update children's inheritable
            object.children.forEach(function (child) {
              child.internalSetAttribute(name, null, {
                skipUpdateAttribute: true,
                skipParse: true
              });
            });
          }
        });
      }
    }
    /**
     * string -> parsed value
     */;
    _proto.parseProperty = function parseProperty(name, value, object) {
      var metadata = propertyMetadataCache[name];
      var computed = value;
      if (value === '' || isNil(value)) {
        value = 'unset';
      }
      if (value === 'unset' || value === 'initial' || value === 'inherit') {
        // computed = new CSSKeywordValue(value);
        computed = getOrCreateKeyword(value);
      } else {
        if (metadata) {
          var keywords = metadata.k,
            syntax = metadata.syntax;
          var handler = syntax && this.getPropertySyntax(syntax);
          // use keywords
          if (keywords && keywords.indexOf(value) > -1) {
            // computed = new CSSKeywordValue(value);
            computed = getOrCreateKeyword(value);
          } else if (handler && handler.parser) {
            // try to parse it to CSSStyleValue, eg. '10px' -> CSS.px(10)
            computed = handler.parser(value, object);
          }
        }
      }
      return computed;
    }
    /**
     * computed value -> used value
     */;
    _proto.computeProperty = function computeProperty(name, computed, object) {
      var metadata = propertyMetadataCache[name];
      var isDocumentElement = object.id === 'g-root';
      // let used: CSSStyleValue = computed instanceof CSSStyleValue ? computed.clone() : computed;
      var used = computed;
      if (metadata) {
        var syntax = metadata.syntax,
          inherited = metadata.inh,
          defaultValue = metadata.d;
        if (computed instanceof CSSKeywordValue) {
          var value = computed.value;
          /**
           * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/unset
           */
          if (value === 'unset') {
            if (inherited && !isDocumentElement) {
              value = 'inherit';
            } else {
              value = 'initial';
            }
          }
          if (value === 'initial') {
            // @see https://developer.mozilla.org/en-US/docs/Web/CSS/initial
            if (!isNil(defaultValue)) {
              computed = this.parseProperty(name, isFunction$1(defaultValue) ? defaultValue(object.nodeName) : defaultValue, object);
            }
          } else if (value === 'inherit') {
            // @see https://developer.mozilla.org/en-US/docs/Web/CSS/inherit
            // behave like `inherit`
            var resolved = this.tryToResolveProperty(object, name, {
              inherited: true
            });
            if (!isNil(resolved)) {
              // object.parsedStyle[name] = resolved;
              // return false;
              return resolved;
            } else {
              this.addUnresolveProperty(object, name);
              return;
            }
          }
        }
        var handler = syntax && this.getPropertySyntax(syntax);
        if (handler && handler.calculator) {
          // convert computed value to used value
          var oldParsedValue = object.parsedStyle[name];
          used = handler.calculator(name, oldParsedValue, computed, object, this);
        } else if (computed instanceof CSSKeywordValue) {
          used = computed.value;
        } else {
          used = computed;
        }
      }
      // object.parsedStyle[name] = used;
      // return false;
      return used;
    };
    _proto.postProcessProperty = function postProcessProperty(name, object) {
      var metadata = propertyMetadataCache[name];
      if (metadata && metadata.syntax) {
        var handler = metadata.syntax && this.getPropertySyntax(metadata.syntax);
        var propertyHandler = handler;
        if (propertyHandler && propertyHandler.postProcessor) {
          propertyHandler.postProcessor(object);
        }
      }
    }
    /**
     * resolve later
     */;
    _proto.addUnresolveProperty = function addUnresolveProperty(object, name) {
      var properties = unresolvedProperties.get(object);
      if (!properties) {
        unresolvedProperties.set(object, []);
        properties = unresolvedProperties.get(object);
      }
      if (properties.indexOf(name) === -1) {
        properties.push(name);
      }
    };
    _proto.tryToResolveProperty = function tryToResolveProperty(object, name, options) {
      if (options === void 0) {
        options = {};
      }
      var _options2 = options,
        inherited = _options2.inherited;
      if (inherited) {
        if (object.parentElement && isPropertyResolved(object.parentElement, name)) {
          // const computedValue = object.parentElement.computedStyle[name];
          var usedValue = object.parentElement.parsedStyle[name];
          if (
          // usedValue instanceof CSSKeywordValue &&
          usedValue === 'unset' || usedValue === 'initial' || usedValue === 'inherit') {
            return;
          }
          // else if (
          //   usedValue instanceof CSSUnitValue &&
          //   CSSUnitValue.isRelativeUnit(usedValue.unit)
          // ) {
          //   return false;
          // }
          return usedValue;
        }
      }
      return;
    };
    _proto.recalc = function recalc(object) {
      var properties = unresolvedProperties.get(object);
      if (properties && properties.length) {
        var attributes = {};
        properties.forEach(function (property) {
          attributes[property] = object.attributes[property];
        });
        this.processProperties(object, attributes);
        unresolvedProperties.delete(object);
      }
    }
    /**
     * update geometry when relative props changed,
     * eg. r of Circle, width/height of Rect
     */;
    _proto.updateGeometry = function updateGeometry(object) {
      var geometryUpdater = runtime.geometryUpdaterFactory[object.nodeName];
      if (geometryUpdater) {
        var geometry = object.geometry;
        if (!geometry.contentBounds) {
          geometry.contentBounds = new AABB();
        }
        if (!geometry.renderBounds) {
          geometry.renderBounds = new AABB();
        }
        var parsedStyle = object.parsedStyle;
        var _geometryUpdater$upda = geometryUpdater.update(parsedStyle, object),
          width = _geometryUpdater$upda.width,
          height = _geometryUpdater$upda.height,
          _geometryUpdater$upda2 = _geometryUpdater$upda.depth,
          depth = _geometryUpdater$upda2 === void 0 ? 0 : _geometryUpdater$upda2,
          _geometryUpdater$upda3 = _geometryUpdater$upda.offsetX,
          offsetX = _geometryUpdater$upda3 === void 0 ? 0 : _geometryUpdater$upda3,
          _geometryUpdater$upda4 = _geometryUpdater$upda.offsetY,
          offsetY = _geometryUpdater$upda4 === void 0 ? 0 : _geometryUpdater$upda4,
          _geometryUpdater$upda5 = _geometryUpdater$upda.offsetZ,
          offsetZ = _geometryUpdater$upda5 === void 0 ? 0 : _geometryUpdater$upda5;
        // account for negative width / height of Rect
        // @see https://github.com/antvis/g/issues/957
        var flipY = width < 0;
        var flipX = height < 0;
        // init with content box
        var halfExtents = set$1(tmpVec3a, Math.abs(width) / 2, Math.abs(height) / 2, depth / 2);
        // anchor is center by default, don't account for lineWidth here
        var stroke = parsedStyle.stroke,
          lineWidth = parsedStyle.lineWidth,
          increasedLineWidthForHitTesting = parsedStyle.increasedLineWidthForHitTesting,
          shadowType = parsedStyle.shadowType,
          shadowColor = parsedStyle.shadowColor,
          _parsedStyle$filter = parsedStyle.filter,
          filter = _parsedStyle$filter === void 0 ? [] : _parsedStyle$filter,
          transformOrigin = parsedStyle.transformOrigin;
        var anchor = parsedStyle.anchor;
        // <Text> use textAlign & textBaseline instead of anchor
        if (object.nodeName === Shape.TEXT) {
          delete parsedStyle.anchor;
        }
        var center = set$1(tmpVec3b, (1 - (anchor && anchor[0] || 0) * 2) * width / 2 + offsetX, (1 - (anchor && anchor[1] || 0) * 2) * height / 2 + offsetY, (1 - (anchor && anchor[2] || 0) * 2) * halfExtents[2] + offsetZ);
        // update geometry's AABB
        geometry.contentBounds.update(center, halfExtents);
        // @see https://github.molgen.mpg.de/git-mirror/cairo/blob/master/src/cairo-stroke-style.c#L97..L128
        var expansion = object.nodeName === Shape.POLYLINE || object.nodeName === Shape.POLYGON || object.nodeName === Shape.PATH ? Math.SQRT2 : 0.5;
        // if (lineCap?.value === 'square') {
        //   expansion = Math.SQRT1_2;
        // }
        // if (lineJoin?.value === 'miter' && expansion < Math.SQRT2 * miterLimit) {
        //   expansion = Math.SQRT1_2 * miterLimit;
        // }
        // append border only if stroke existed
        var hasStroke = stroke && !stroke.isNone;
        if (hasStroke) {
          var halfLineWidth = ((lineWidth || 0) + (increasedLineWidthForHitTesting || 0)) * expansion;
          // halfExtents[0] += halfLineWidth[0];
          // halfExtents[1] += halfLineWidth[1];
          add$1(halfExtents, halfExtents, set$1(tmpVec3c, halfLineWidth, halfLineWidth, 0));
        }
        geometry.renderBounds.update(center, halfExtents);
        // account for shadow, only support constant value now
        if (shadowColor && shadowType && shadowType !== 'inner') {
          var _geometry$renderBound = geometry.renderBounds,
            min = _geometry$renderBound.min,
            max = _geometry$renderBound.max;
          var shadowBlur = parsedStyle.shadowBlur,
            shadowOffsetX = parsedStyle.shadowOffsetX,
            shadowOffsetY = parsedStyle.shadowOffsetY;
          var shadowBlurInPixels = shadowBlur || 0;
          var shadowOffsetXInPixels = shadowOffsetX || 0;
          var shadowOffsetYInPixels = shadowOffsetY || 0;
          var shadowLeft = min[0] - shadowBlurInPixels + shadowOffsetXInPixels;
          var shadowRight = max[0] + shadowBlurInPixels + shadowOffsetXInPixels;
          var shadowTop = min[1] - shadowBlurInPixels + shadowOffsetYInPixels;
          var shadowBottom = max[1] + shadowBlurInPixels + shadowOffsetYInPixels;
          min[0] = Math.min(min[0], shadowLeft);
          max[0] = Math.max(max[0], shadowRight);
          min[1] = Math.min(min[1], shadowTop);
          max[1] = Math.max(max[1], shadowBottom);
          geometry.renderBounds.setMinMax(min, max);
        }
        // account for filter, eg. blur(5px), drop-shadow()
        filter.forEach(function (_ref) {
          var name = _ref.name,
            params = _ref.params;
          if (name === 'blur') {
            var blurRadius = params[0].value;
            geometry.renderBounds.update(geometry.renderBounds.center, add$1(geometry.renderBounds.halfExtents, geometry.renderBounds.halfExtents, fromValues$2(blurRadius, blurRadius, 0)));
          } else if (name === 'drop-shadow') {
            var _shadowOffsetX = params[0].value;
            var _shadowOffsetY = params[1].value;
            var _shadowBlur = params[2].value;
            var _geometry$renderBound2 = geometry.renderBounds,
              _min = _geometry$renderBound2.min,
              _max = _geometry$renderBound2.max;
            var _shadowLeft = _min[0] - _shadowBlur + _shadowOffsetX;
            var _shadowRight = _max[0] + _shadowBlur + _shadowOffsetX;
            var _shadowTop = _min[1] - _shadowBlur + _shadowOffsetY;
            var _shadowBottom = _max[1] + _shadowBlur + _shadowOffsetY;
            _min[0] = Math.min(_min[0], _shadowLeft);
            _max[0] = Math.max(_max[0], _shadowRight);
            _min[1] = Math.min(_min[1], _shadowTop);
            _max[1] = Math.max(_max[1], _shadowBottom);
            geometry.renderBounds.setMinMax(_min, _max);
          }
        });
        anchor = parsedStyle.anchor;
        // set transform origin
        var usedOriginXValue = (flipY ? -1 : 1) * convertPercentUnit(transformOrigin[0], 0, object);
        var usedOriginYValue = (flipX ? -1 : 1) * convertPercentUnit(transformOrigin[1], 1, object);
        usedOriginXValue = usedOriginXValue - (flipY ? -1 : 1) * (anchor && anchor[0] || 0) * geometry.contentBounds.halfExtents[0] * 2;
        usedOriginYValue = usedOriginYValue - (flipX ? -1 : 1) * (anchor && anchor[1] || 0) * geometry.contentBounds.halfExtents[1] * 2;
        object.setOrigin(usedOriginXValue, usedOriginYValue);
        runtime.sceneGraphService.dirtifyToRoot(object);
      }
    };
    _proto.isPropertyInheritable = function isPropertyInheritable(name) {
      var metadata = propertyMetadataCache[name];
      if (!metadata) {
        return false;
      }
      return metadata.inh;
    };
    return DefaultStyleValueRegistry;
  }();

  /**
   * canvas.customElements
   *
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry
   */
  var CustomElementRegistry = /*#__PURE__*/function () {
    function CustomElementRegistry() {
      this.registry = {};
      this.define(Shape.CIRCLE, Circle);
      this.define(Shape.ELLIPSE, Ellipse);
      this.define(Shape.RECT, Rect);
      this.define(Shape.IMAGE, Image);
      this.define(Shape.LINE, Line);
      this.define(Shape.GROUP, Group);
      this.define(Shape.PATH, Path);
      this.define(Shape.POLYGON, Polygon);
      this.define(Shape.POLYLINE, Polyline);
      this.define(Shape.TEXT, Text);
      this.define(Shape.HTML, HTML);
    }
    var _proto = CustomElementRegistry.prototype;
    _proto.define = function define(name, constructor) {
      this.registry[name] = constructor;
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry/get
     */;
    _proto.get = function get(name) {
      return this.registry[name];
    };
    return CustomElementRegistry;
  }();

  /**
   * An DOM-compatible synthetic event implementation that is "forwarded" on behalf of an original
   * FederatedEvent or native {@link https://dom.spec.whatwg.org/#event Event}.
   */
  var FederatedEvent = /*#__PURE__*/function () {
    /**
     * The event boundary which manages this event. Propagation can only occur
     *  within the boundary's jurisdiction.
     */
    function FederatedEvent(manager) {
      this.type = void 0;
      this.eventPhase = FederatedEvent.prototype.NONE;
      this.target = void 0;
      this.bubbles = true;
      this.cancelBubble = true;
      this.cancelable = false;
      this.currentTarget = void 0;
      this.defaultPrevented = false;
      this.timeStamp = void 0;
      this.nativeEvent = void 0;
      this.originalEvent = void 0;
      this.propagationStopped = false;
      this.propagationImmediatelyStopped = false;
      this.manager = void 0;
      this.detail = void 0;
      this.layer = new Point();
      this.page = new Point();
      this.canvas = new Point();
      this.viewport = new Point();
      this.path = void 0;
      this.view = void 0;
      this.which = void 0;
      this.returnValue = void 0;
      this.srcElement = void 0;
      this.composed = false;
      this.isTrusted = void 0;
      this.NONE = 0;
      this.CAPTURING_PHASE = 1;
      this.AT_TARGET = 2;
      this.BUBBLING_PHASE = 3;
      this.manager = manager;
    }
    var _proto = FederatedEvent.prototype;
    /**
     * The propagation path for this event
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Event/composedPath
     *
     * So composedPath()[0] represents the original target.
     * @see https://polymer-library.polymer-project.org/3.0/docs/devguide/events#retargeting
     */
    _proto.composedPath = function composedPath() {
      if (this.manager && (!this.path || this.path[0] !== this.target)) {
        this.path = this.target ? this.manager.propagationPath(this.target) : [];
      }
      return this.path;
    }
    /**
     * @deprecated
     */;
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Event/preventDefault
     */
    _proto.preventDefault = function preventDefault() {
      if (this.nativeEvent instanceof Event && this.nativeEvent.cancelable) {
        this.nativeEvent.preventDefault();
      }
      this.defaultPrevented = true;
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Event/stopImmediatePropagation
     */;
    _proto.stopImmediatePropagation = function stopImmediatePropagation() {
      this.propagationImmediatelyStopped = true;
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Event/stopPropagation
     */;
    _proto.stopPropagation = function stopPropagation() {
      this.propagationStopped = true;
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/UIEvent/view
     */;
    /**
     * added for compatibility with DOM Event,
     * deprecated props and methods
     */
    _proto.initEvent = function initEvent() {};
    _proto.initUIEvent = function initUIEvent() {};
    _proto.clone = function clone() {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _createClass$1(FederatedEvent, [{
      key: "name",
      get:
      /**
       * The type of event, supports the following:
       * * pointerdown
       * * touchstart
       * * mousedown
       * * rightdown
       * * ...
       */

      /**
       * @deprecated
       */
      function get() {
        return this.type;
      }
      /**
       * The propagation phase.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Event/eventPhase
       */
    }, {
      key: "layerX",
      get: function get() {
        return this.layer.x;
      }
    }, {
      key: "layerY",
      get: function get() {
        return this.layer.y;
      }
      /**
       * The coordinates of the event relative to the DOM document.
       * This is a non-standard property.
       * relative to the DOM document.
       * @see https://developer.mozilla.org/zh-CN/docs/Web/API/MouseEvent/pageX
       */
    }, {
      key: "pageX",
      get: function get() {
        return this.page.x;
      }
    }, {
      key: "pageY",
      get: function get() {
        return this.page.y;
      }
      /**
       * relative to Canvas, origin is left-top
       */
    }, {
      key: "x",
      get: function get() {
        return this.canvas.x;
      }
    }, {
      key: "y",
      get: function get() {
        return this.canvas.y;
      }
    }, {
      key: "canvasX",
      get: function get() {
        return this.canvas.x;
      }
    }, {
      key: "canvasY",
      get: function get() {
        return this.canvas.y;
      }
      /**
       * relative to Viewport, account for Camera
       */
    }, {
      key: "viewportX",
      get: function get() {
        return this.viewport.x;
      }
    }, {
      key: "viewportY",
      get: function get() {
        return this.viewport.y;
      }
    }, {
      key: "propagationPath",
      get: function get() {
        return this.composedPath();
      }
    }]);
    return FederatedEvent;
  }();

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/Events/Creating_and_triggering_events
   *
   * @example
    const event = new CustomEvent('build', { detail: { prop1: 'xx' } });
    circle.addEventListener('build', (e) => {
      e.target; // circle
      e.detail; // { prop1: 'xx' }
    });

    circle.dispatchEvent(event);
   */
  var CustomEvent = /*#__PURE__*/function (_FederatedEvent) {
    _inheritsLoose(CustomEvent, _FederatedEvent);
    // eslint-disable-next-line @typescript-eslint/ban-types
    function CustomEvent(eventName, object) {
      var _this;
      _this = _FederatedEvent.call(this, null) || this;
      _this.type = eventName;
      _this.detail = object;
      // compatible with G 3.0
      Object.assign(_assertThisInitialized(_this), object);
      return _this;
    }
    return CustomEvent;
  }(FederatedEvent);

  var DELEGATION_SPLITTER = ':';
  /**
   * Objects that can receive events and may have listeners for them.
   * eg. Element, Canvas, DisplayObject
   * @see https://developer.mozilla.org/en-US/docs/Web/API/EventTarget
   */
  var EventTarget = /*#__PURE__*/function () {
    function EventTarget() {
      this.emitter = new eventemitter3();
    }
    var _proto = EventTarget.prototype;
    /**
     * @deprecated
     * @alias addEventListener
     */
    _proto.on = function on(type, listener, options) {
      this.addEventListener(type, listener, options);
      return this;
    }
    /**
     * support `capture` & `once` in options
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/EventTarget/addEventListener
     */;
    _proto.addEventListener = function addEventListener(type, listener, options) {
      var capture = isBoolean(options) && options || isObject(options) && options.capture;
      var once = isObject(options) && options.once;
      var context = isFunction$1(listener) ? undefined : listener;
      // compatible with G 3.0
      // support using delegate name in event type, eg. 'node:click'
      var useDelegatedName = false;
      var delegatedName = '';
      if (type.indexOf(DELEGATION_SPLITTER) > -1) {
        var _type$split = type.split(DELEGATION_SPLITTER),
          name = _type$split[0],
          eventType = _type$split[1];
        type = eventType;
        delegatedName = name;
        useDelegatedName = true;
      }
      type = capture ? type + "capture" : type;
      listener = isFunction$1(listener) ? listener : listener.handleEvent;
      // compatible with G 3.0
      if (useDelegatedName) {
        var originListener = listener;
        listener = function listener() {
          var _args$0$target;
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }
          if (((_args$0$target = args[0].target) === null || _args$0$target === void 0 ? void 0 : _args$0$target.name) !== delegatedName) {
            return;
          }
          // @ts-ignore
          originListener.apply(void 0, args);
        };
      }
      if (once) {
        this.emitter.once(type, listener, context);
      } else {
        this.emitter.on(type, listener, context);
      }
      return this;
    }
    /**
     * @deprecated
     * @alias removeEventListener
     */;
    _proto.off = function off(type, listener, options) {
      if (type) {
        this.removeEventListener(type, listener, options);
      } else {
        // remove all listeners
        this.removeAllEventListeners();
      }
      return this;
    };
    _proto.removeAllEventListeners = function removeAllEventListeners() {
      this.emitter.removeAllListeners();
    };
    _proto.removeEventListener = function removeEventListener(type, listener, options) {
      var _listener;
      var capture = isBoolean(options) && options || isObject(options) && options.capture;
      var context = isFunction$1(listener) ? undefined : listener;
      type = capture ? type + "capture" : type;
      listener = isFunction$1(listener) ? listener : (_listener = listener) === null || _listener === void 0 ? void 0 : _listener.handleEvent;
      this.emitter.off(type, listener, context);
      return this;
    }
    /**
     * @deprecated
     * @alias dispatchEvent
     */
    // eslint-disable-next-line @typescript-eslint/ban-types
    ;
    _proto.emit = function emit(eventName, object) {
      this.dispatchEvent(new CustomEvent(eventName, object));
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/dispatchEvent
     */;
    _proto.dispatchEvent = function dispatchEvent(e, skipPropagate) {
      if (skipPropagate === void 0) {
        skipPropagate = false;
      }
      if (!(e instanceof FederatedEvent)) {
        throw new Error('DisplayObject cannot propagate events outside of the Federated Events API');
      }
      // should account for Element / Document / Canvas
      var canvas;
      // @ts-ignore
      if (this.document) {
        canvas = this;
        // @ts-ignore
      } else if (this.defaultView) {
        canvas = this.defaultView;
      } else {
        var _this$ownerDocument;
        canvas = (_this$ownerDocument = this.ownerDocument) === null || _this$ownerDocument === void 0 ? void 0 : _this$ownerDocument.defaultView;
      }
      // assign event manager
      if (canvas) {
        var _e$manager;
        e.manager = canvas.getEventService() || null;
        if (!e.manager) {
          return false;
        }
        e.defaultPrevented = false;
        e.path = [];
        if (!skipPropagate) {
          e.target = this;
        }
        (_e$manager = e.manager) === null || _e$manager === void 0 ? void 0 : _e$manager.dispatchEvent(e, e.type, skipPropagate);
      }
      return !e.defaultPrevented;
    };
    return EventTarget;
  }();

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/API/Node
   */
  var Node$1 = /*#__PURE__*/function (_EventTarget) {
    _inheritsLoose(Node, _EventTarget);
    function Node() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _EventTarget.call.apply(_EventTarget, [this].concat(args)) || this;
      _this.shadow = false;
      _this.ownerDocument = null;
      _this.isConnected = false;
      _this.baseURI = '';
      _this.childNodes = [];
      _this.nodeType = 0;
      _this.nodeName = '';
      _this.nodeValue = null;
      _this.parentNode = null;
      return _this;
    }
    Node.isNode = function isNode(target) {
      return !!target.childNodes;
    };
    var _proto = Node.prototype;
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Node/getRootNode
     */
    _proto.getRootNode = function getRootNode(opts) {
      if (opts === void 0) {
        opts = {};
      }
      if (this.parentNode) {
        return this.parentNode.getRootNode(opts);
      }
      if (opts.composed && this.host) {
        return this.host.getRootNode(opts);
      }
      return this;
    };
    _proto.hasChildNodes = function hasChildNodes() {
      return this.childNodes.length > 0;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.isDefaultNamespace = function isDefaultNamespace(namespace) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.lookupNamespaceURI = function lookupNamespaceURI(prefix) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.lookupPrefix = function lookupPrefix(namespace) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.normalize = function normalize() {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Node/isEqualNode
     */;
    _proto.isEqualNode = function isEqualNode(otherNode) {
      // TODO: compare 2 nodes, not sameness
      return this === otherNode;
    };
    _proto.isSameNode = function isSameNode(otherNode) {
      return this.isEqualNode(otherNode);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/ParentNode
     */;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Node/compareDocumentPosition
     * @see https://github.com/b-fuze/deno-dom/blob/master/src/dom/node.ts#L338
     */
    _proto.compareDocumentPosition = function compareDocumentPosition(other) {
      if (other === this) {
        // same node
        return 0;
      }
      if (!(other instanceof Node)) {
        throw new TypeError('Node.compareDocumentPosition: Argument 1 does not implement interface Node.');
      }
      var node1Root = other;
      // eslint-disable-next-line @typescript-eslint/no-this-alias
      var node2Root = this;
      var node1Hierarchy = [node1Root];
      var node2Hierarchy = [node2Root];
      while ((_node1Root$parentNode = node1Root.parentNode) !== null && _node1Root$parentNode !== void 0 ? _node1Root$parentNode : node2Root.parentNode) {
        var _node1Root$parentNode;
        node1Root = node1Root.parentNode ? (node1Hierarchy.push(node1Root.parentNode), node1Root.parentNode) : node1Root;
        node2Root = node2Root.parentNode ? (node2Hierarchy.push(node2Root.parentNode), node2Root.parentNode) : node2Root;
      }
      // Check if they don't share the same root node
      if (node1Root !== node2Root) {
        return Node.DOCUMENT_POSITION_DISCONNECTED | Node.DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC | Node.DOCUMENT_POSITION_PRECEDING;
      }
      var longerHierarchy = node1Hierarchy.length > node2Hierarchy.length ? node1Hierarchy : node2Hierarchy;
      var shorterHierarchy = longerHierarchy === node1Hierarchy ? node2Hierarchy : node1Hierarchy;
      // Check if either is a container of the other
      if (longerHierarchy[longerHierarchy.length - shorterHierarchy.length] === shorterHierarchy[0]) {
        return longerHierarchy === node1Hierarchy ?
        // other is a child of this
        Node.DOCUMENT_POSITION_CONTAINED_BY | Node.DOCUMENT_POSITION_FOLLOWING :
        // this is a child of other
        Node.DOCUMENT_POSITION_CONTAINS | Node.DOCUMENT_POSITION_PRECEDING;
      }
      // Find their first common ancestor and see whether they
      // are preceding or following
      var longerStart = longerHierarchy.length - shorterHierarchy.length;
      for (var i = shorterHierarchy.length - 1; i >= 0; i--) {
        var shorterHierarchyNode = shorterHierarchy[i];
        var longerHierarchyNode = longerHierarchy[longerStart + i];
        // We found the first common ancestor
        if (longerHierarchyNode !== shorterHierarchyNode) {
          var siblings = shorterHierarchyNode.parentNode.childNodes;
          if (siblings.indexOf(shorterHierarchyNode) < siblings.indexOf(longerHierarchyNode)) {
            // Shorter is before longer
            if (shorterHierarchy === node1Hierarchy) {
              // Other is before this
              return Node.DOCUMENT_POSITION_PRECEDING;
            } else {
              // This is before other
              return Node.DOCUMENT_POSITION_FOLLOWING;
            }
          } else {
            // Longer is before shorter
            if (longerHierarchy === node1Hierarchy) {
              // Other is before this
              return Node.DOCUMENT_POSITION_PRECEDING;
            } else {
              // Other is after this
              return Node.DOCUMENT_POSITION_FOLLOWING;
            }
          }
        }
      }
      return Node.DOCUMENT_POSITION_FOLLOWING;
    }
    /**
     * @deprecated
     * @alias contains
     */;
    _proto.contain = function contain(other) {
      return this.contains(other);
    };
    _proto.contains = function contains(other) {
      // the node itself, one of its direct children
      var tmp = other;
      // @see https://developer.mozilla.org/en-US/docs/Web/API/Node/contains
      while (tmp && this !== tmp) {
        tmp = tmp.parentNode;
      }
      return !!tmp;
    };
    _proto.getAncestor = function getAncestor(n) {
      // eslint-disable-next-line @typescript-eslint/no-this-alias
      var temp = this;
      while (n > 0 && temp) {
        temp = temp.parentNode;
        n--;
      }
      return temp;
    };
    _proto.forEach = function forEach(callback) {
      if (!callback(this)) {
        this.childNodes.forEach(function (child) {
          child.forEach(callback);
        });
      }
    };
    _createClass$1(Node, [{
      key: "textContent",
      get:
      /**
       * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Node/textContent
       */
      function get() {
        var out = '';
        if (this.nodeName === Shape.TEXT) {
          // @ts-ignore
          out += this.style.text;
        }
        for (var _iterator = _createForOfIteratorHelperLoose(this.childNodes), _step; !(_step = _iterator()).done;) {
          var child = _step.value;
          if (child.nodeName === Shape.TEXT) {
            out += child.nodeValue;
          } else {
            out += child.textContent;
          }
        }
        return out;
      },
      set: function set(content) {
        var _this2 = this;
        // remove all children
        this.childNodes.slice().forEach(function (child) {
          _this2.removeChild(child);
        });
        if (this.nodeName === Shape.TEXT) {
          // @ts-ignore
          this.style.text = "" + content;
        }
      }
    }, {
      key: "parent",
      get:
      /**
       * @deprecated
       * @alias parentNode
       */
      function get() {
        return this.parentNode;
      }
    }, {
      key: "parentElement",
      get: function get() {
        return null;
      }
    }, {
      key: "nextSibling",
      get: function get() {
        return null;
      }
    }, {
      key: "previousSibling",
      get: function get() {
        return null;
      }
    }, {
      key: "firstChild",
      get: function get() {
        return this.childNodes.length > 0 ? this.childNodes[0] : null;
      }
    }, {
      key: "lastChild",
      get: function get() {
        return this.childNodes.length > 0 ? this.childNodes[this.childNodes.length - 1] : null;
      }
    }]);
    return Node;
  }(EventTarget);
  Node$1.DOCUMENT_POSITION_DISCONNECTED = 1;
  Node$1.DOCUMENT_POSITION_PRECEDING = 2;
  Node$1.DOCUMENT_POSITION_FOLLOWING = 4;
  Node$1.DOCUMENT_POSITION_CONTAINS = 8;
  Node$1.DOCUMENT_POSITION_CONTAINED_BY = 16;
  Node$1.DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC = 32;

  /**
   * the entry of DOM tree
   * Document -> Node -> EventTarget
   * @see https://developer.mozilla.org/en-US/docs/Web/API/Document
   */
  var Document = /*#__PURE__*/function (_Node) {
    _inheritsLoose(Document, _Node);
    function Document() {
      var _this;
      _this = _Node.call(this) || this;
      _this.defaultView = null;
      _this.documentElement = void 0;
      _this.timeline = void 0;
      _this.ownerDocument = null;
      _this.nodeName = 'document';
      // create timeline
      try {
        _this.timeline = new runtime.AnimationTimeline(_assertThisInitialized(_this));
      } catch (e) {}
      /**
       * for inherited properties, the initial value is used on the root element only,
       * as long as no specified value is supplied.
       * @see https://developer.mozilla.org/en-US/docs/Web/CSS/initial_value
       */
      var initialStyle = {};
      BUILT_IN_PROPERTIES.forEach(function (_ref) {
        var n = _ref.n,
          inh = _ref.inh,
          d = _ref.d;
        if (inh && d) {
          initialStyle[n] = isFunction$1(d) ? d(Shape.GROUP) : d;
        }
      });
      // like <html> in DOM tree
      _this.documentElement = new Group({
        id: 'g-root',
        style: initialStyle
      });
      _this.documentElement.ownerDocument = _assertThisInitialized(_this);
      _this.documentElement.parentNode = _assertThisInitialized(_this);
      _this.childNodes = [_this.documentElement];
      return _this;
    }
    var _proto = Document.prototype;
    /**
     * @example const circle = document.createElement('circle', { style: { r: 10 } });
     */
    _proto.createElement = function createElement(tagName, options) {
      // @observablehq/plot will create <svg>
      if (tagName === 'svg') {
        return this.documentElement;
      }
      // d3 will use <tspan>
      var clazz = this.defaultView.customElements.get(tagName);
      if (!clazz) {
        console.warn('Unsupported tagName: ', tagName);
        clazz = tagName === 'tspan' ? Text : Group;
      }
      var shape = new clazz(options);
      shape.ownerDocument = this;
      return shape;
    };
    _proto.createElementNS = function createElementNS(namespaceURI, tagName, options) {
      return this.createElement(tagName, options);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.cloneNode = function cloneNode(deep) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.destroy = function destroy() {
      try {
        this.documentElement.destroy();
        this.timeline.destroy();
      } catch (e) {}
    }
    /**
     * Picking 2D graphics with RBush based on BBox, fast but inaccurate.
     */;
    _proto.elementsFromBBox = function elementsFromBBox(minX, minY, maxX, maxY) {
      var rBush = this.defaultView.context.rBushRoot;
      var rBushNodes = rBush.search({
        minX: minX,
        minY: minY,
        maxX: maxX,
        maxY: maxY
      });
      var hitTestList = [];
      rBushNodes.forEach(function (_ref2) {
        var id = _ref2.id;
        var displayObject = runtime.displayObjectPool.getByEntity(id);
        var pointerEvents = displayObject.parsedStyle.pointerEvents;
        // account for `visibility`
        // @see https://developer.mozilla.org/en-US/docs/Web/CSS/pointer-events
        var isVisibilityAffected = ['auto', 'visiblepainted', 'visiblefill', 'visiblestroke', 'visible'].includes(pointerEvents);
        if ((!isVisibilityAffected || isVisibilityAffected && displayObject.isVisible()) && !displayObject.isCulled() && displayObject.isInteractive()) {
          hitTestList.push(displayObject);
        }
      });
      // find group with max z-index
      hitTestList.sort(function (a, b) {
        return b.sortable.renderOrder - a.sortable.renderOrder;
      });
      return hitTestList;
    };
    _proto.elementFromPointSync = function elementFromPointSync(x, y) {
      var _this$defaultView$can = this.defaultView.canvas2Viewport({
          x: x,
          y: y
        }),
        viewportX = _this$defaultView$can.x,
        viewportY = _this$defaultView$can.y;
      var _this$defaultView$get = this.defaultView.getConfig(),
        width = _this$defaultView$get.width,
        height = _this$defaultView$get.height;
      // outside canvas' viewport
      if (viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height) {
        return null;
      }
      var _this$defaultView$vie = this.defaultView.viewport2Client({
          x: viewportX,
          y: viewportY
        }),
        clientX = _this$defaultView$vie.x,
        clientY = _this$defaultView$vie.y;
      var _this$defaultView$get2 = this.defaultView.getRenderingService().hooks.pickSync.call({
          topmost: true,
          position: {
            x: x,
            y: y,
            viewportX: viewportX,
            viewportY: viewportY,
            clientX: clientX,
            clientY: clientY
          },
          picked: []
        }),
        picked = _this$defaultView$get2.picked;
      return picked && picked[0] || this.documentElement;
    }
    /**
     * Do picking with API instead of triggering interactive events.
     *
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/elementFromPoint
     */;
    _proto.elementFromPoint =
    /*#__PURE__*/
    function () {
      var _elementFromPoint = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(x, y) {
        var _this$defaultView$can2, viewportX, viewportY, _this$defaultView$get3, width, height, _this$defaultView$vie2, clientX, clientY, _yield$this$defaultVi, picked;
        return _regeneratorRuntime().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _this$defaultView$can2 = this.defaultView.canvas2Viewport({
                  x: x,
                  y: y
                }), viewportX = _this$defaultView$can2.x, viewportY = _this$defaultView$can2.y;
                _this$defaultView$get3 = this.defaultView.getConfig(), width = _this$defaultView$get3.width, height = _this$defaultView$get3.height; // outside canvas' viewport
                if (!(viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height)) {
                  _context.next = 4;
                  break;
                }
                return _context.abrupt("return", null);
              case 4:
                _this$defaultView$vie2 = this.defaultView.viewport2Client({
                  x: viewportX,
                  y: viewportY
                }), clientX = _this$defaultView$vie2.x, clientY = _this$defaultView$vie2.y;
                _context.next = 7;
                return this.defaultView.getRenderingService().hooks.pick.promise({
                  topmost: true,
                  position: {
                    x: x,
                    y: y,
                    viewportX: viewportX,
                    viewportY: viewportY,
                    clientX: clientX,
                    clientY: clientY
                  },
                  picked: []
                });
              case 7:
                _yield$this$defaultVi = _context.sent;
                picked = _yield$this$defaultVi.picked;
                return _context.abrupt("return", picked && picked[0] || this.documentElement);
              case 10:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
      function elementFromPoint(_x, _x2) {
        return _elementFromPoint.apply(this, arguments);
      }
      return elementFromPoint;
    }();
    _proto.elementsFromPointSync = function elementsFromPointSync(x, y) {
      var _this$defaultView$can3 = this.defaultView.canvas2Viewport({
          x: x,
          y: y
        }),
        viewportX = _this$defaultView$can3.x,
        viewportY = _this$defaultView$can3.y;
      var _this$defaultView$get4 = this.defaultView.getConfig(),
        width = _this$defaultView$get4.width,
        height = _this$defaultView$get4.height;
      // outside canvas' viewport
      if (viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height) {
        return [];
      }
      var _this$defaultView$vie3 = this.defaultView.viewport2Client({
          x: viewportX,
          y: viewportY
        }),
        clientX = _this$defaultView$vie3.x,
        clientY = _this$defaultView$vie3.y;
      var _this$defaultView$get5 = this.defaultView.getRenderingService().hooks.pickSync.call({
          topmost: false,
          position: {
            x: x,
            y: y,
            viewportX: viewportX,
            viewportY: viewportY,
            clientX: clientX,
            clientY: clientY
          },
          picked: []
        }),
        picked = _this$defaultView$get5.picked;
      if (picked[picked.length - 1] !== this.documentElement) {
        picked.push(this.documentElement);
      }
      return picked;
    }
    /**
     * Do picking with API instead of triggering interactive events.
     *
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/elementsFromPoint
     */;
    _proto.elementsFromPoint =
    /*#__PURE__*/
    function () {
      var _elementsFromPoint = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee2(x, y) {
        var _this$defaultView$can4, viewportX, viewportY, _this$defaultView$get6, width, height, _this$defaultView$vie4, clientX, clientY, _yield$this$defaultVi2, picked;
        return _regeneratorRuntime().wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _this$defaultView$can4 = this.defaultView.canvas2Viewport({
                  x: x,
                  y: y
                }), viewportX = _this$defaultView$can4.x, viewportY = _this$defaultView$can4.y;
                _this$defaultView$get6 = this.defaultView.getConfig(), width = _this$defaultView$get6.width, height = _this$defaultView$get6.height; // outside canvas' viewport
                if (!(viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height)) {
                  _context2.next = 4;
                  break;
                }
                return _context2.abrupt("return", []);
              case 4:
                _this$defaultView$vie4 = this.defaultView.viewport2Client({
                  x: viewportX,
                  y: viewportY
                }), clientX = _this$defaultView$vie4.x, clientY = _this$defaultView$vie4.y;
                _context2.next = 7;
                return this.defaultView.getRenderingService().hooks.pick.promise({
                  topmost: false,
                  position: {
                    x: x,
                    y: y,
                    viewportX: viewportX,
                    viewportY: viewportY,
                    clientX: clientX,
                    clientY: clientY
                  },
                  picked: []
                });
              case 7:
                _yield$this$defaultVi2 = _context2.sent;
                picked = _yield$this$defaultVi2.picked;
                if (picked[picked.length - 1] !== this.documentElement) {
                  picked.push(this.documentElement);
                }
                return _context2.abrupt("return", picked);
              case 11:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
      function elementsFromPoint(_x3, _x4) {
        return _elementsFromPoint.apply(this, arguments);
      }
      return elementsFromPoint;
    }()
    /**
     * eg. Uncaught DOMException: Failed to execute 'appendChild' on 'Node': Only one element on document allowed.
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.appendChild = function appendChild(newChild, index) {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.insertBefore = function insertBefore(newChild, refChild) {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.removeChild = function removeChild(oldChild, destroy) {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.replaceChild = function replaceChild(newChild, oldChild, destroy) {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.append = function append() {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.prepend = function prepend() {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT);
    }
    /**
     * Execute query on documentElement.
     */;
    _proto.getElementById = function getElementById(id) {
      return this.documentElement.getElementById(id);
    };
    _proto.getElementsByName = function getElementsByName(name) {
      return this.documentElement.getElementsByName(name);
    };
    _proto.getElementsByTagName = function getElementsByTagName(tagName) {
      return this.documentElement.getElementsByTagName(tagName);
    };
    _proto.getElementsByClassName = function getElementsByClassName(className) {
      return this.documentElement.getElementsByClassName(className);
    };
    _proto.querySelector = function querySelector(selectors) {
      return this.documentElement.querySelector(selectors);
    };
    _proto.querySelectorAll = function querySelectorAll(selectors) {
      return this.documentElement.querySelectorAll(selectors);
    };
    _proto.find = function find(filter) {
      return this.documentElement.find(filter);
    };
    _proto.findAll = function findAll(filter) {
      return this.documentElement.findAll(filter);
    };
    _createClass$1(Document, [{
      key: "children",
      get: function get() {
        return this.childNodes;
      }
    }, {
      key: "childElementCount",
      get: function get() {
        return this.childNodes.length;
      }
    }, {
      key: "firstElementChild",
      get: function get() {
        return this.firstChild;
      }
    }, {
      key: "lastElementChild",
      get: function get() {
        return this.lastChild;
      }
      /**
       * only document has defaultView, points to canvas,
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Document/defaultView
       */
    }]);
    return Document;
  }(Node$1);

  /**
   * @see https://doc.babylonjs.com/how_to/optimizing_your_scene#changing-mesh-culling-strategy
   */
  var Strategy;
  (function (Strategy) {
    Strategy[Strategy["Standard"] = 0] = "Standard";
  })(Strategy || (Strategy = {}));

  /**
   * built-in events for element
   * @see https://developer.mozilla.org/en-US/docs/Web/API/MutationEvent
   *
   * TODO: use MutationObserver instead
   * @see https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver
   */
  var ElementEvent;
  (function (ElementEvent) {
    ElementEvent["DESTROY"] = "destroy";
    /**
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#event-type-DOMAttrModified
     */
    ElementEvent["ATTR_MODIFIED"] = "DOMAttrModified";
    /**
     * it has been inserted
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#event-type-DOMNodeInserted
     */
    ElementEvent["INSERTED"] = "DOMNodeInserted";
    /**
     * it is being removed
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#event-type-DOMNodeRemoved
     */
    ElementEvent["REMOVED"] = "removed";
    /**
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#domnodeinsertedintodocument
     */
    ElementEvent["MOUNTED"] = "DOMNodeInsertedIntoDocument";
    /**
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#domnoderemovedfromdocument
     */
    ElementEvent["UNMOUNTED"] = "DOMNodeRemovedFromDocument";
    ElementEvent["BOUNDS_CHANGED"] = "bounds-changed";
    // GEOMETRY_BOUNDS_CHANGED = 'geometry-bounds-changed',
    /**
     * trigger when z-index changed
     */
    ElementEvent["RENDER_ORDER_CHANGED"] = "render-order-changed";
    ElementEvent["CULLED"] = "culled";
  })(ElementEvent || (ElementEvent = {}));

  var MutationEvent = /*#__PURE__*/function (_FederatedEvent) {
    _inheritsLoose(MutationEvent, _FederatedEvent);
    function MutationEvent(typeArg, relatedNode, prevValue, newValue, attrName, attrChange, prevParsedValue, newParsedValue) {
      var _this;
      _this = _FederatedEvent.call(this, null) || this;
      _this.relatedNode = void 0;
      _this.prevValue = void 0;
      _this.newValue = void 0;
      _this.attrName = void 0;
      _this.attrChange = void 0;
      _this.prevParsedValue = void 0;
      _this.newParsedValue = void 0;
      _this.relatedNode = relatedNode;
      _this.prevValue = prevValue;
      _this.newValue = newValue;
      _this.attrName = attrName;
      _this.attrChange = attrChange;
      _this.prevParsedValue = prevParsedValue;
      _this.newParsedValue = newParsedValue;
      _this.type = typeArg;
      return _this;
    }
    return MutationEvent;
  }(FederatedEvent);
  MutationEvent.ADDITION = 2;
  MutationEvent.MODIFICATION = 1;
  MutationEvent.REMOVAL = 3;

  var entityCounter = 0;
  var insertedEvent = new MutationEvent(ElementEvent.INSERTED, null, '', '', '', 0, '', '');
  var removedEvent = new MutationEvent(ElementEvent.REMOVED, null, '', '', '', 0, '', '');
  var destroyEvent = new CustomEvent(ElementEvent.DESTROY);
  /**
   * Has following capabilities:
   * * Node insert/remove, eg. appendChild, removeChild, remove...
   * * Query eg. querySelector getElementById...
   * * Animation
   */
  var Element = /*#__PURE__*/function (_Node) {
    _inheritsLoose(Element, _Node);
    function Element() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _Node.call.apply(_Node, [this].concat(args)) || this;
      _this.entity = entityCounter++;
      _this.renderable = {
        bounds: undefined,
        boundsDirty: true,
        renderBounds: undefined,
        renderBoundsDirty: true,
        dirtyRenderBounds: undefined,
        dirty: false,
        proxyNodeName: undefined
      };
      _this.cullable = {
        strategy: Strategy.Standard,
        visibilityPlaneMask: -1,
        visible: true,
        enable: true
      };
      _this.transformable = {
        dirtyFlag: false,
        localDirtyFlag: false,
        frozen: false,
        localPosition: fromValues$2(0, 0, 0),
        localRotation: fromValues$4(0, 0, 0, 1),
        localScale: fromValues$2(1, 1, 1),
        localTransform: create$1(),
        localSkew: fromValues$5(0, 0),
        position: fromValues$2(0, 0, 0),
        rotation: fromValues$4(0, 0, 0, 1),
        scaling: fromValues$2(1, 1, 1),
        worldTransform: create$1(),
        origin: fromValues$2(0, 0, 0)
      };
      _this.sortable = {
        dirty: false,
        sorted: undefined,
        lastSortedIndex: undefined,
        renderOrder: 0
      };
      _this.geometry = {
        contentBounds: undefined,
        renderBounds: undefined
      };
      _this.rBushNode = {
        aabb: undefined
      };
      _this.id = void 0;
      _this.name = void 0;
      _this.namespaceURI = 'g';
      _this.scrollLeft = 0;
      _this.scrollTop = 0;
      _this.clientTop = 0;
      _this.clientLeft = 0;
      _this.destroyed = false;
      _this.style = {};
      _this.computedStyle = {
        anchor: unsetKeywordValue,
        opacity: unsetKeywordValue,
        fillOpacity: unsetKeywordValue,
        strokeOpacity: unsetKeywordValue,
        fill: unsetKeywordValue,
        stroke: unsetKeywordValue,
        transform: unsetKeywordValue,
        transformOrigin: unsetKeywordValue,
        visibility: unsetKeywordValue,
        pointerEvents: unsetKeywordValue,
        lineWidth: unsetKeywordValue,
        lineCap: unsetKeywordValue,
        lineJoin: unsetKeywordValue,
        increasedLineWidthForHitTesting: unsetKeywordValue,
        fontSize: unsetKeywordValue,
        fontFamily: unsetKeywordValue,
        fontStyle: unsetKeywordValue,
        fontWeight: unsetKeywordValue,
        fontVariant: unsetKeywordValue,
        textAlign: unsetKeywordValue,
        textBaseline: unsetKeywordValue,
        textTransform: unsetKeywordValue,
        zIndex: unsetKeywordValue,
        filter: unsetKeywordValue,
        shadowType: unsetKeywordValue
      };
      _this.parsedStyle = {
        // opacity: '',
        // fillOpacity: '',
        // strokeOpacity: '',
        // transformOrigin: '',
        // visibility: '',
        // pointerEvents: '',
        // lineWidth: '',
        // lineCap: '',
        // lineJoin: '',
        // increasedLineWidthForHitTesting: '',
        // fontSize: '',
        // fontFamily: '',
        // fontStyle: '',
        // fontWeight: '',
        // fontVariant: '',
        // textAlign: '',
        // textBaseline: '',
        // textTransform: '',
      };
      _this.attributes = {};
      return _this;
    }
    Element.isElement = function isElement(target) {
      return !!target.getAttribute;
    }
    /**
     * Unique id.
     */;
    var _proto = Element.prototype;
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    _proto.cloneNode = function cloneNode(deep) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.appendChild = function appendChild(child, index) {
      var _this$ownerDocument;
      if (child.destroyed) {
        throw new Error(ERROR_MSG_APPEND_DESTROYED_ELEMENT);
      }
      runtime.sceneGraphService.attach(child, this, index);
      if ((_this$ownerDocument = this.ownerDocument) === null || _this$ownerDocument === void 0 ? void 0 : _this$ownerDocument.defaultView) {
        this.ownerDocument.defaultView.mountChildren(child);
      }
      insertedEvent.relatedNode = this;
      child.dispatchEvent(insertedEvent);
      return child;
    };
    _proto.insertBefore = function insertBefore(newChild, refChild) {
      if (!refChild) {
        this.appendChild(newChild);
      } else {
        var index = this.childNodes.indexOf(refChild);
        this.appendChild(newChild, index - 1);
      }
      return newChild;
    };
    _proto.replaceChild = function replaceChild(newChild, oldChild) {
      var index = this.childNodes.indexOf(oldChild);
      this.removeChild(oldChild);
      this.appendChild(newChild, index);
      return oldChild;
    };
    _proto.removeChild = function removeChild(child) {
      var _child$ownerDocument;
      // should emit on itself before detach
      removedEvent.relatedNode = this;
      child.dispatchEvent(removedEvent);
      if ((_child$ownerDocument = child.ownerDocument) === null || _child$ownerDocument === void 0 ? void 0 : _child$ownerDocument.defaultView) {
        child.ownerDocument.defaultView.unmountChildren(child);
      }
      // remove from scene graph
      runtime.sceneGraphService.detach(child);
      return child;
    }
    /**
     * Remove all children which can be appended to its original parent later again.
     */;
    _proto.removeChildren = function removeChildren() {
      var _this2 = this;
      this.childNodes.slice().forEach(function (child) {
        _this2.removeChild(child);
      });
    }
    /**
     * Recursively destroy all children which can not be appended to its original parent later again.
     */;
    _proto.destroyChildren = function destroyChildren() {
      this.childNodes.slice().forEach(function (child) {
        child.destroyChildren();
        child.destroy();
      });
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/matches
     */;
    _proto.matches = function matches(selector) {
      return runtime.sceneGraphService.matches(selector, this);
    };
    _proto.getElementById = function getElementById(id) {
      return runtime.sceneGraphService.querySelector("#" + id, this);
    };
    _proto.getElementsByName = function getElementsByName(name) {
      return runtime.sceneGraphService.querySelectorAll("[name=\"" + name + "\"]", this);
    };
    _proto.getElementsByClassName = function getElementsByClassName(className) {
      return runtime.sceneGraphService.querySelectorAll("." + className, this);
    };
    _proto.getElementsByTagName = function getElementsByTagName(tagName) {
      return runtime.sceneGraphService.querySelectorAll(tagName, this);
    };
    _proto.querySelector = function querySelector(selectors) {
      return runtime.sceneGraphService.querySelector(selectors, this);
    };
    _proto.querySelectorAll = function querySelectorAll(selectors) {
      return runtime.sceneGraphService.querySelectorAll(selectors, this);
    }
    /**
     * should traverses the element and its parents (heading toward the document root)
     * until it finds a node that matches the specified CSS selector.
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/closest
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/closest#polyfill
     */;
    _proto.closest = function closest(selectors) {
      var el = this;
      do {
        if (runtime.sceneGraphService.matches(selectors, el)) return el;
        el = el.parentElement;
      } while (el !== null);
      return null;
    }
    /**
     * search in scene group, but should not include itself
     */;
    _proto.find = function find(filter) {
      var _this3 = this;
      var target = null;
      this.forEach(function (object) {
        if (object !== _this3 && filter(object)) {
          target = object;
          return true;
        }
        return false;
      });
      return target;
    };
    _proto.findAll = function findAll(filter) {
      var _this4 = this;
      var objects = [];
      this.forEach(function (object) {
        if (object !== _this4 && filter(object)) {
          objects.push(object);
        }
      });
      return objects;
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/after
     */;
    _proto.after = function after() {
      var _this5 = this;
      if (this.parentNode) {
        var index = this.parentNode.childNodes.indexOf(this);
        for (var _len2 = arguments.length, nodes = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          nodes[_key2] = arguments[_key2];
        }
        nodes.forEach(function (node, i) {
          var _this5$parentNode;
          return (_this5$parentNode = _this5.parentNode) === null || _this5$parentNode === void 0 ? void 0 : _this5$parentNode.appendChild(node, index + i + 1);
        });
      }
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/before
     */;
    _proto.before = function before() {
      if (this.parentNode) {
        var index = this.parentNode.childNodes.indexOf(this);
        for (var _len3 = arguments.length, nodes = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
          nodes[_key3] = arguments[_key3];
        }
        var first = nodes[0],
          rest = nodes.slice(1);
        this.parentNode.appendChild(first, index);
        first.after.apply(first, rest);
      }
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/replaceWith
     */;
    _proto.replaceWith = function replaceWith() {
      this.after.apply(this, arguments);
      this.remove();
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/append
     */;
    _proto.append = function append() {
      var _this6 = this;
      for (var _len4 = arguments.length, nodes = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        nodes[_key4] = arguments[_key4];
      }
      nodes.forEach(function (node) {
        return _this6.appendChild(node);
      });
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/prepend
     */;
    _proto.prepend = function prepend() {
      var _this7 = this;
      for (var _len5 = arguments.length, nodes = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        nodes[_key5] = arguments[_key5];
      }
      nodes.forEach(function (node, i) {
        return _this7.appendChild(node, i);
      });
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/replaceChildren
     */;
    _proto.replaceChildren = function replaceChildren() {
      while (this.childNodes.length && this.firstChild) {
        this.removeChild(this.firstChild);
      }
      this.append.apply(this, arguments);
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/remove
     */;
    _proto.remove = function remove() {
      if (this.parentNode) {
        return this.parentNode.removeChild(this);
      }
      return this;
    }
    /**
     * is destroyed or not
     */;
    _proto.destroy = function destroy() {
      // destroy itself before remove
      this.dispatchEvent(destroyEvent);
      // remove from scenegraph first
      this.remove();
      // remove event listeners
      this.emitter.removeAllListeners();
      this.destroyed = true;
    };
    _proto.getGeometryBounds = function getGeometryBounds() {
      return runtime.sceneGraphService.getGeometryBounds(this);
    };
    _proto.getRenderBounds = function getRenderBounds() {
      return runtime.sceneGraphService.getBounds(this, true);
    }
    /**
     * get bounds in world space, account for children
     */;
    _proto.getBounds = function getBounds() {
      return runtime.sceneGraphService.getBounds(this);
    }
    /**
     * get bounds in local space, account for children
     */;
    _proto.getLocalBounds = function getLocalBounds() {
      return runtime.sceneGraphService.getLocalBounds(this);
    }
    /**
     * account for context's bounds in client space,
     * but not accounting for children
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/getBoundingClientRect
     */;
    _proto.getBoundingClientRect = function getBoundingClientRect() {
      return runtime.sceneGraphService.getBoundingClientRect(this);
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/getClientRects
     */;
    _proto.getClientRects = function getClientRects() {
      return [this.getBoundingClientRect()];
    }
    /**
     * compatible with `style`
     * @see https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement/style
     */;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/computedStyleMap
     * eg. circle.computedStyleMap().get('fill');
     */
    _proto.computedStyleMap = function computedStyleMap() {
      return new Map(Object.entries(this.computedStyle));
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/attributes
     */;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/getAttributeNames
     */
    _proto.getAttributeNames = function getAttributeNames() {
      return Object.keys(this.attributes);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/getAttribute
     */;
    _proto.getAttribute = function getAttribute(name) {
      var value = this.attributes[name];
      if (value === undefined) {
        var attributeName = formatAttributeName(name);
        value = this.attributes[attributeName];
        // if the given attribute does not exist, the value returned will either be null or ""
        return isNil(value) ? null : value;
      } else {
        return value;
      }
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/hasAttribute
     */;
    _proto.hasAttribute = function hasAttribute(qualifiedName) {
      return this.getAttributeNames().includes(qualifiedName);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/hasAttributes
     */;
    _proto.hasAttributes = function hasAttributes() {
      return !!this.getAttributeNames().length;
    }
    /**
     * should use removeAttribute() instead of setting the attribute value to null either directly or using setAttribute(). Many attributes will not behave as expected if you set them to null.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/removeAttribute
     */;
    _proto.removeAttribute = function removeAttribute(attributeName) {
      this.setAttribute(attributeName, null);
      delete this.attributes[attributeName];
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/setAttribute
     */;
    _proto.setAttribute = function setAttribute(attributeName, value,
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    force) {
      this.attributes[attributeName] = value;
    };
    _proto.getAttributeNS = function getAttributeNS(namespace, localName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.getAttributeNode = function getAttributeNode(qualifiedName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.getAttributeNodeNS = function getAttributeNodeNS(namespace, localName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.hasAttributeNS = function hasAttributeNS(namespace, localName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.removeAttributeNS = function removeAttributeNS(namespace, localName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.removeAttributeNode = function removeAttributeNode(attr) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.setAttributeNS = function setAttributeNS(namespace, qualifiedName, value) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.setAttributeNode = function setAttributeNode(attr) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.setAttributeNodeNS = function setAttributeNodeNS(attr) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.toggleAttribute = function toggleAttribute(qualifiedName, force) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _createClass$1(Element, [{
      key: "className",
      get:
      /**
       * used in `getElementsByClassName`
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Document/getElementsByClassName
       */
      function get() {
        // @ts-ignore
        return this.getAttribute('class') || '';
      },
      set: function set(className) {
        this.setAttribute('class', className);
      }
      /**
       * used in `getElementsByName`
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Document/getElementsByName
       */
    }, {
      key: "classList",
      get:
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/classList
       */
      function get() {
        return this.className.split(' ').filter(function (c) {
          return c !== '';
        });
      }
    }, {
      key: "tagName",
      get: function get() {
        return this.nodeName;
      }
    }, {
      key: "children",
      get: function get() {
        return this.childNodes;
      }
    }, {
      key: "childElementCount",
      get: function get() {
        return this.childNodes.length;
      }
    }, {
      key: "firstElementChild",
      get: function get() {
        return this.firstChild;
      }
    }, {
      key: "lastElementChild",
      get: function get() {
        return this.lastChild;
      }
    }, {
      key: "parentElement",
      get: function get() {
        return this.parentNode;
      }
    }, {
      key: "nextSibling",
      get: function get() {
        if (this.parentNode) {
          var index = this.parentNode.childNodes.indexOf(this);
          return this.parentNode.childNodes[index + 1] || null;
        }
        return null;
      }
    }, {
      key: "previousSibling",
      get: function get() {
        if (this.parentNode) {
          var index = this.parentNode.childNodes.indexOf(this);
          return this.parentNode.childNodes[index - 1] || null;
        }
        return null;
      }
    }]);
    return Element;
  }(Node$1);

  var FederatedMouseEvent = /*#__PURE__*/function (_FederatedEvent) {
    _inheritsLoose(FederatedMouseEvent, _FederatedEvent);
    function FederatedMouseEvent() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _FederatedEvent.call.apply(_FederatedEvent, [this].concat(args)) || this;
      _this.altKey = void 0;
      _this.button = void 0;
      _this.buttons = void 0;
      _this.ctrlKey = void 0;
      _this.metaKey = void 0;
      _this.relatedTarget = void 0;
      _this.shiftKey = void 0;
      _this.client = new Point();
      _this.movement = new Point();
      _this.offset = new Point();
      _this.global = new Point();
      _this.screen = new Point();
      return _this;
    }
    var _proto = FederatedMouseEvent.prototype;
    _proto.getModifierState = function getModifierState(key) {
      return 'getModifierState' in this.nativeEvent && this.nativeEvent.getModifierState(key);
    };
    _proto.initMouseEvent = function initMouseEvent() {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _createClass$1(FederatedMouseEvent, [{
      key: "clientX",
      get: function get() {
        return this.client.x;
      }
    }, {
      key: "clientY",
      get: function get() {
        return this.client.y;
      }
      /**
       * The movement in this pointer relative to the last `mousemove` event.
       */
    }, {
      key: "movementX",
      get: function get() {
        return this.movement.x;
      }
    }, {
      key: "movementY",
      get: function get() {
        return this.movement.y;
      }
      /**
       * The offset of the pointer coordinates w.r.t. target DisplayObject in world space. This is
       * not supported at the moment.
       */
    }, {
      key: "offsetX",
      get: function get() {
        return this.offset.x;
      }
    }, {
      key: "offsetY",
      get: function get() {
        return this.offset.y;
      }
      /**
       * The pointer coordinates in world space.
       */
    }, {
      key: "globalX",
      get: function get() {
        return this.global.x;
      }
    }, {
      key: "globalY",
      get: function get() {
        return this.global.y;
      }
      /**
       * The pointer coordinates in sceen space.
       */
    }, {
      key: "screenX",
      get: function get() {
        return this.screen.x;
      }
    }, {
      key: "screenY",
      get: function get() {
        return this.screen.y;
      }
    }]);
    return FederatedMouseEvent;
  }(FederatedEvent);

  var FederatedPointerEvent = /*#__PURE__*/function (_FederatedMouseEvent) {
    _inheritsLoose(FederatedPointerEvent, _FederatedMouseEvent);
    function FederatedPointerEvent() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _FederatedMouseEvent.call.apply(_FederatedMouseEvent, [this].concat(args)) || this;
      _this.pointerId = void 0;
      _this.width = 0;
      _this.height = 0;
      _this.isPrimary = false;
      _this.pointerType = void 0;
      _this.pressure = void 0;
      _this.tangentialPressure = void 0;
      _this.tiltX = void 0;
      _this.tiltY = void 0;
      _this.twist = void 0;
      return _this;
    }
    var _proto = FederatedPointerEvent.prototype;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/getCoalescedEvents
     */
    _proto.getCoalescedEvents = function getCoalescedEvents() {
      if (this.type === 'pointermove' || this.type === 'mousemove' || this.type === 'touchmove') {
        return [this];
      }
      return [];
    }
    /**
     * @see https://chromestatus.com/feature/5765569655603200
     */;
    _proto.getPredictedEvents = function getPredictedEvents() {
      throw new Error('getPredictedEvents is not supported!');
    }
    /**
     * @see https://github.com/antvis/G/issues/1115
     * We currently reuses event objects in the event system,
     * avoiding the creation of a large number of event objects.
     * Reused objects are only used to carry different data,
     * such as coordinate information, native event objects,
     * and therefore the lifecycle is limited to the event handler,
     * which can lead to unintended consequences if an attempt is made to cache the entire event object.
     *
     * Therefore, while keeping the above performance considerations in mind, it is possible to provide a clone method that creates a new object when the user really wants to cache it, e.g.
     */;
    _proto.clone = function clone() {
      return this.manager.clonePointerEvent(this);
    };
    return FederatedPointerEvent;
  }(FederatedMouseEvent);

  var FederatedWheelEvent = /*#__PURE__*/function (_FederatedMouseEvent) {
    _inheritsLoose(FederatedWheelEvent, _FederatedMouseEvent);
    function FederatedWheelEvent() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _FederatedMouseEvent.call.apply(_FederatedMouseEvent, [this].concat(args)) || this;
      _this.deltaMode = void 0;
      _this.deltaX = void 0;
      _this.deltaY = void 0;
      _this.deltaZ = void 0;
      _this.DOM_DELTA_LINE = 0;
      _this.DOM_DELTA_PAGE = 1;
      _this.DOM_DELTA_PIXEL = 2;
      return _this;
    }
    var _proto = FederatedWheelEvent.prototype;
    _proto.clone = function clone() {
      return this.manager.cloneWheelEvent(this);
    };
    return FederatedWheelEvent;
  }(FederatedMouseEvent);

  var mutationEvent = new MutationEvent(ElementEvent.ATTR_MODIFIED, null, null, null, null, MutationEvent.MODIFICATION, null, null);
  var DEFAULT_STYLE_PROPS = {
    anchor: '',
    opacity: '',
    fillOpacity: '',
    strokeOpacity: '',
    fill: '',
    stroke: '',
    transform: '',
    transformOrigin: '',
    visibility: '',
    pointerEvents: '',
    lineWidth: '',
    lineCap: '',
    lineJoin: '',
    increasedLineWidthForHitTesting: '',
    fontSize: '',
    fontFamily: '',
    fontStyle: '',
    fontWeight: '',
    fontVariant: '',
    textAlign: '',
    textBaseline: '',
    textTransform: '',
    zIndex: '',
    filter: '',
    shadowType: ''
  };
  var DEFAULT_PARSED_STYLE_PROPS = {
    anchor: [0, 0],
    fill: noneColor,
    stroke: noneColor,
    transform: [],
    zIndex: 0,
    filter: [],
    shadowType: 'outer'
  };
  var INHERITABLE_STYLE_PROPS = ['opacity', 'fillOpacity', 'strokeOpacity', 'transformOrigin', 'visibility', 'pointerEvents', 'lineWidth', 'lineCap', 'lineJoin', 'increasedLineWidthForHitTesting', 'fontSize', 'fontFamily', 'fontStyle', 'fontWeight', 'fontVariant', 'textAlign', 'textBaseline', 'textTransform'];
  /**
   * prototype chains: DisplayObject -> Element -> Node -> EventTarget
   *
   * mixins: Animatable, Transformable, Visible
   * @see https://github.com/tannerntannern/ts-mixer/blob/master/README.md#mixing-generic-classes
   *
   * Provide abilities in scene graph, such as:
   * * transform `translate/rotate/scale`
   * * add/remove child
   * * visibility and z-index
   *
   * Those abilities are implemented with those components: `Transform/Sortable/Visible`.
   *
   * Emit following events:
   * * init
   * * destroy
   * * attributeChanged
   */
  var DisplayObject = /*#__PURE__*/function (_Element) {
    _inheritsLoose(DisplayObject, _Element);
    /**
     * contains style props in constructor's params, eg. fill, stroke...
     */

    /**
     * push to active animations after calling `animate()`
     */

    function DisplayObject(config) {
      var _this$config$capture;
      var _this;
      _this = _Element.call(this) || this;
      // assign name, id to config
      // eg. group.get('name')
      _this.config = void 0;
      _this.isCustomElement = false;
      _this.isMutationObserved = false;
      _this.activeAnimations = [];
      _this.getClip = function () {
        return this.style.clipPath || null;
      };
      _this.config = config;
      // compatible with G 3.0
      _this.config.interactive = (_this$config$capture = _this.config.capture) !== null && _this$config$capture !== void 0 ? _this$config$capture : _this.config.interactive;
      // init scene graph node
      _this.id = _this.config.id || '';
      _this.name = _this.config.name || '';
      if (_this.config.className || _this.config.class) {
        _this.className = _this.config.className || _this.config.class;
      }
      _this.nodeName = _this.config.type || Shape.GROUP;
      // compatible with G 3.0
      _this.config.style = _extends({}, _this.config.style, _this.config.attrs);
      if (_this.config.visible != null) {
        _this.config.style.visibility = _this.config.visible === false ? 'hidden' : 'visible';
      }
      if (_this.config.interactive != null) {
        _this.config.style.pointerEvents = _this.config.interactive === false ? 'none' : 'auto';
      }
      // merge parsed value
      Object.assign(_this.parsedStyle, DEFAULT_PARSED_STYLE_PROPS, _this.config.initialParsedStyle);
      Object.assign(_this.attributes, DEFAULT_STYLE_PROPS);
      // start to process attributes
      _this.initAttributes(_this.config.style);
      _this.style = new Proxy(
      // @ts-ignore
      {
        // ...this.attributes,
        setProperty: function setProperty(propertyName, value) {
          _this.setAttribute(propertyName, value);
        },
        getPropertyValue: function getPropertyValue(propertyName) {
          return _this.getAttribute(propertyName);
        },
        removeProperty: function removeProperty(propertyName) {
          _this.removeAttribute(propertyName);
        },
        item: function item() {
          return '';
        }
      }, {
        get: function get(target, name) {
          if (target[name] !== undefined) {
            // if (name in target) {
            return target[name];
          }
          return _this.getAttribute(name);
        },
        set: function set(_, prop, value) {
          _this.setAttribute(prop, value);
          return true;
        }
      });
      // insert this group into pool
      runtime.displayObjectPool.add(_this.entity, _assertThisInitialized(_this));
      return _this;
    }
    var _proto = DisplayObject.prototype;
    _proto.destroy = function destroy() {
      _Element.prototype.destroy.call(this);
      // remove from into pool
      runtime.displayObjectPool.remove(this.entity);
      // stop all active animations
      this.getAnimations().forEach(function (animation) {
        animation.cancel();
      });
    };
    _proto.cloneNode = function cloneNode(deep, customCloneFunc) {
      var clonedStyle = _extends({}, this.attributes);
      for (var attributeName in clonedStyle) {
        var attribute = clonedStyle[attributeName];
        // @see https://github.com/antvis/G/issues/1095
        if (attribute instanceof DisplayObject &&
        // share the same clipPath if possible
        attributeName !== 'clipPath' && attributeName !== 'offsetPath' && attributeName !== 'textPath') {
          clonedStyle[attributeName] = attribute.cloneNode(deep);
        }
        // TODO: clone other type
        if (customCloneFunc) {
          clonedStyle[attributeName] = customCloneFunc(attributeName, attribute);
        }
      }
      var cloned = new this.constructor({
        // copy id & name
        // @see https://developer.mozilla.org/en-US/docs/Web/API/Node/cloneNode#notes
        id: this.id,
        name: this.name,
        className: this.name,
        interactive: this.interactive,
        style: clonedStyle
      });
      // apply transform
      cloned.setLocalTransform(this.getLocalTransform());
      if (deep) {
        this.children.forEach(function (child) {
          // skip marker
          if (!child.style.isMarker) {
            var clonedChild = child.cloneNode(deep);
            cloned.appendChild(clonedChild);
          }
        });
      }
      return cloned;
    };
    _proto.initAttributes = function initAttributes(attributes) {
      if (attributes === void 0) {
        attributes = {};
      }
      var renderable = this.renderable;
      // account for FCP, process properties as less as possible
      runtime.styleValueRegistry.processProperties(this, attributes, {
        forceUpdateGeometry: true,
        usedAttributes: INHERITABLE_STYLE_PROPS
      });
      // redraw at next frame
      renderable.dirty = true;
    };
    _proto.setAttribute = function setAttribute(name, value, force) {
      if (force === void 0) {
        force = false;
      }
      var attributeName = formatAttributeName(name);
      // ignore undefined value
      if (isUndefined(value)) {
        return;
      }
      if (force || value !== this.attributes[attributeName]) {
        this.internalSetAttribute(attributeName, value);
        _Element.prototype.setAttribute.call(this, attributeName, value);
      }
    }
    /**
     * called when attributes get changed or initialized
     */;
    _proto.internalSetAttribute = function internalSetAttribute(name, value, parseOptions) {
      var _runtime$styleValueRe;
      if (parseOptions === void 0) {
        parseOptions = {};
      }
      var renderable = this.renderable;
      var oldValue = this.attributes[name];
      var oldParsedValue = this.parsedStyle[name];
      runtime.styleValueRegistry.processProperties(this, (_runtime$styleValueRe = {}, _runtime$styleValueRe[name] = value, _runtime$styleValueRe), parseOptions);
      // redraw at next frame
      renderable.dirty = true;
      var newParsedValue = this.parsedStyle[name];
      if (this.isConnected) {
        mutationEvent.relatedNode = this;
        mutationEvent.prevValue = oldValue;
        mutationEvent.newValue = value;
        mutationEvent.attrName = name;
        mutationEvent.prevParsedValue = oldParsedValue;
        mutationEvent.newParsedValue = newParsedValue;
        if (this.isMutationObserved) {
          this.dispatchEvent(mutationEvent);
        } else {
          mutationEvent.target = this;
          this.ownerDocument.defaultView.dispatchEvent(mutationEvent, true);
        }
      }
      if ((this.isCustomElement && this.isConnected || !this.isCustomElement) && this.attributeChangedCallback) {
        this.attributeChangedCallback(name, oldValue, value, oldParsedValue, newParsedValue);
      }
    }
    // #region transformable
    /**
     * returns different values than getBoundingClientRect(), as the latter returns value relative to the viewport
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGGraphicsElement/getBBox
     *
     * FIXME: It is worth noting that getBBox responds to original untransformed values of a drawn object.
     * @see https://www.w3.org/Graphics/SVG/IG/resources/svgprimer.html#getBBox
     */;
    _proto.getBBox = function getBBox() {
      var aabb = this.getBounds();
      var _aabb$getMin = aabb.getMin(),
        left = _aabb$getMin[0],
        top = _aabb$getMin[1];
      var _aabb$getMax = aabb.getMax(),
        right = _aabb$getMax[0],
        bottom = _aabb$getMax[1];
      return new Rectangle(left, top, right - left, bottom - top);
    };
    _proto.setOrigin = function setOrigin(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime.sceneGraphService.setOrigin(this, createVec3(position, y, z));
      return this;
    };
    _proto.getOrigin = function getOrigin() {
      return runtime.sceneGraphService.getOrigin(this);
    }
    /**
     * set position in world space
     */;
    _proto.setPosition = function setPosition(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime.sceneGraphService.setPosition(this, createVec3(position, y, z));
      return this;
    }
    /**
     * set position in local space
     */;
    _proto.setLocalPosition = function setLocalPosition(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime.sceneGraphService.setLocalPosition(this, createVec3(position, y, z));
      return this;
    }
    /**
     * translate in world space
     */;
    _proto.translate = function translate(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime.sceneGraphService.translate(this, createVec3(position, y, z));
      return this;
    }
    /**
     * translate in local space
     */;
    _proto.translateLocal = function translateLocal(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime.sceneGraphService.translateLocal(this, createVec3(position, y, z));
      return this;
    };
    _proto.getPosition = function getPosition() {
      return runtime.sceneGraphService.getPosition(this);
    };
    _proto.getLocalPosition = function getLocalPosition() {
      return runtime.sceneGraphService.getLocalPosition(this);
    }
    /**
     * compatible with G 3.0
     *
     * scaling in local space
     * scale(10) = scale(10, 10, 10)
     *
     * we can't set scale in world space
     */;
    _proto.scale = function scale(scaling, y, z) {
      return this.scaleLocal(scaling, y, z);
    };
    _proto.scaleLocal = function scaleLocal(scaling, y, z) {
      if (typeof scaling === 'number') {
        y = y || scaling;
        z = z || scaling;
        scaling = createVec3(scaling, y, z);
      }
      runtime.sceneGraphService.scaleLocal(this, scaling);
      return this;
    }
    /**
     * set scaling in local space
     */;
    _proto.setLocalScale = function setLocalScale(scaling, y, z) {
      if (typeof scaling === 'number') {
        y = y || scaling;
        z = z || scaling;
        scaling = createVec3(scaling, y, z);
      }
      runtime.sceneGraphService.setLocalScale(this, scaling);
      return this;
    }
    /**
     * get scaling in local space
     */;
    _proto.getLocalScale = function getLocalScale() {
      return runtime.sceneGraphService.getLocalScale(this);
    }
    /**
     * get scaling in world space
     */;
    _proto.getScale = function getScale() {
      return runtime.sceneGraphService.getScale(this);
    }
    /**
     * only return degrees of Z axis in world space
     */;
    _proto.getEulerAngles = function getEulerAngles() {
      var _getEuler = getEuler(create$2(), runtime.sceneGraphService.getWorldTransform(this)),
        ez = _getEuler[2];
      return rad2deg(ez);
    }
    /**
     * only return degrees of Z axis in local space
     */;
    _proto.getLocalEulerAngles = function getLocalEulerAngles() {
      var _getEuler2 = getEuler(create$2(), runtime.sceneGraphService.getLocalRotation(this)),
        ez = _getEuler2[2];
      return rad2deg(ez);
    }
    /**
     * set euler angles(degrees) in world space
     */;
    _proto.setEulerAngles = function setEulerAngles(z) {
      runtime.sceneGraphService.setEulerAngles(this, 0, 0, z);
      return this;
    }
    /**
     * set euler angles(degrees) in local space
     */;
    _proto.setLocalEulerAngles = function setLocalEulerAngles(z) {
      runtime.sceneGraphService.setLocalEulerAngles(this, 0, 0, z);
      return this;
    };
    _proto.rotateLocal = function rotateLocal(x, y, z) {
      if (isNil(y) && isNil(z)) {
        runtime.sceneGraphService.rotateLocal(this, 0, 0, x);
      } else {
        runtime.sceneGraphService.rotateLocal(this, x, y, z);
      }
      return this;
    };
    _proto.rotate = function rotate(x, y, z) {
      if (isNil(y) && isNil(z)) {
        runtime.sceneGraphService.rotate(this, 0, 0, x);
      } else {
        runtime.sceneGraphService.rotate(this, x, y, z);
      }
      return this;
    };
    _proto.setRotation = function setRotation(rotation, y, z, w) {
      runtime.sceneGraphService.setRotation(this, rotation, y, z, w);
      return this;
    };
    _proto.setLocalRotation = function setLocalRotation(rotation, y, z, w) {
      runtime.sceneGraphService.setLocalRotation(this, rotation, y, z, w);
      return this;
    };
    _proto.setLocalSkew = function setLocalSkew(skew, y) {
      runtime.sceneGraphService.setLocalSkew(this, skew, y);
      return this;
    };
    _proto.getRotation = function getRotation() {
      return runtime.sceneGraphService.getRotation(this);
    };
    _proto.getLocalRotation = function getLocalRotation() {
      return runtime.sceneGraphService.getLocalRotation(this);
    };
    _proto.getLocalSkew = function getLocalSkew() {
      return runtime.sceneGraphService.getLocalSkew(this);
    };
    _proto.getLocalTransform = function getLocalTransform() {
      return runtime.sceneGraphService.getLocalTransform(this);
    };
    _proto.getWorldTransform = function getWorldTransform() {
      return runtime.sceneGraphService.getWorldTransform(this);
    };
    _proto.setLocalTransform = function setLocalTransform(transform) {
      runtime.sceneGraphService.setLocalTransform(this, transform);
      return this;
    };
    _proto.resetLocalTransform = function resetLocalTransform() {
      runtime.sceneGraphService.resetLocalTransform(this);
    }
    // #endregion transformable
    // #region animatable
    /**
     * returns an array of all Animation objects affecting this element
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/getAnimations
     */;
    _proto.getAnimations = function getAnimations() {
      return this.activeAnimations;
    }
    /**
     * create an animation with WAAPI
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/animate
     */;
    _proto.animate = function animate(keyframes, options) {
      var _this$ownerDocument;
      var timeline = (_this$ownerDocument = this.ownerDocument) === null || _this$ownerDocument === void 0 ? void 0 : _this$ownerDocument.timeline;
      if (timeline) {
        return timeline.play(this, keyframes, options);
      }
      return null;
    }
    // #endregion animatable
    // #region visible
    /**
     * shortcut for Used value of `visibility`
     */;
    _proto.isVisible = function isVisible() {
      var _this$parsedStyle;
      return ((_this$parsedStyle = this.parsedStyle) === null || _this$parsedStyle === void 0 ? void 0 : _this$parsedStyle.visibility) === 'visible';
    };
    _proto.isInteractive = function isInteractive() {
      var _this$parsedStyle2;
      return ((_this$parsedStyle2 = this.parsedStyle) === null || _this$parsedStyle2 === void 0 ? void 0 : _this$parsedStyle2.pointerEvents) !== 'none';
    };
    _proto.isCulled = function isCulled() {
      return !!(this.cullable && this.cullable.enable && !this.cullable.visible);
    }
    /**
     * bring to front in current group
     */;
    _proto.toFront = function toFront() {
      if (this.parentNode) {
        this.style.zIndex = Math.max.apply(Math, this.parentNode.children.map(function (child) {
          return Number(child.style.zIndex);
        })) + 1;
      }
      return this;
    }
    /**
     * send to back in current group
     */;
    _proto.toBack = function toBack() {
      if (this.parentNode) {
        this.style.zIndex = Math.min.apply(Math, this.parentNode.children.map(function (child) {
          return Number(child.style.zIndex);
        })) - 1;
      }
      return this;
    }
    // #endregion visible
    // #region deprecated
    /**
     * compatible with G 3.0
     * @alias object.config
     * @deprecated
     */;
    _proto.getConfig = function getConfig() {
      return this.config;
    };
    _proto.attr = function attr() {
      var _this2 = this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      var name = args[0],
        value = args[1];
      if (!name) {
        return this.attributes;
      }
      if (isObject(name)) {
        Object.keys(name).forEach(function (key) {
          _this2.setAttribute(key, name[key]);
        });
        return this;
      }
      if (args.length === 2) {
        this.setAttribute(name, value);
        return this;
      }
      return this.attributes[name];
    }
    /**
     * return 3x3 matrix in world space
     * @deprecated
     */;
    _proto.getMatrix = function getMatrix(transformMat4) {
      var transform = transformMat4 || this.getWorldTransform();
      var _mat4$getTranslation = getTranslation(create$2(), transform),
        tx = _mat4$getTranslation[0],
        ty = _mat4$getTranslation[1];
      var _mat4$getScaling = getScaling(create$2(), transform),
        sx = _mat4$getScaling[0],
        sy = _mat4$getScaling[1];
      var rotation = getRotation(create$4(), transform);
      var _getEuler3 = getEuler(create$2(), rotation),
        eux = _getEuler3[0],
        euz = _getEuler3[2];
      // gimbal lock at 90 degrees
      return fromRotationTranslationScale$1(eux || euz, tx, ty, sx, sy);
    }
    /**
     * return 3x3 matrix in local space
     * @deprecated
     */;
    _proto.getLocalMatrix = function getLocalMatrix() {
      return this.getMatrix(this.getLocalTransform());
    }
    /**
     * set 3x3 matrix in world space
     * @deprecated
     */;
    _proto.setMatrix = function setMatrix(mat) {
      var _decompose = decompose(mat),
        tx = _decompose[0],
        ty = _decompose[1],
        scalingX = _decompose[2],
        scalingY = _decompose[3],
        angle = _decompose[4];
      this.setEulerAngles(angle).setPosition(tx, ty).setLocalScale(scalingX, scalingY);
    }
    /**
     * set 3x3 matrix in local space
     * @deprecated
     */;
    _proto.setLocalMatrix = function setLocalMatrix(mat) {
      var _decompose2 = decompose(mat),
        tx = _decompose2[0],
        ty = _decompose2[1],
        scalingX = _decompose2[2],
        scalingY = _decompose2[3],
        angle = _decompose2[4];
      this.setLocalEulerAngles(angle).setLocalPosition(tx, ty).setLocalScale(scalingX, scalingY);
    }
    /**
     * Use `visibility: visible` instead.
     * @deprecated
     */;
    _proto.show = function show() {
      this.style.visibility = 'visible';
    }
    /**
     * Use `visibility: hidden` instead.
     * @deprecated
     */;
    _proto.hide = function hide() {
      this.style.visibility = 'hidden';
    }
    /**
     * Use `childElementCount` instead.
     * @deprecated
     */;
    _proto.getCount = function getCount() {
      return this.childElementCount;
    }
    /**
     * Use `parentElement` instead.
     * @deprecated
     */;
    _proto.getParent = function getParent() {
      return this.parentElement;
    }
    /**
     * Use `children` instead.
     * @deprecated
     */;
    _proto.getChildren = function getChildren() {
      return this.children;
    }
    /**
     * Use `firstElementChild` instead.
     * @deprecated
     */;
    _proto.getFirst = function getFirst() {
      return this.firstElementChild;
    }
    /**
     * Use `lastElementChild` instead.
     * @deprecated
     */;
    _proto.getLast = function getLast() {
      return this.lastElementChild;
    }
    /**
     * Use `this.children[index]` instead.
     * @deprecated
     */;
    _proto.getChildByIndex = function getChildByIndex(index) {
      return this.children[index] || null;
    }
    /**
     * Use `appendChild` instead.
     * @deprecated
     */;
    _proto.add = function add(child, index) {
      return this.appendChild(child, index);
    }
    /**
     * Use `this.style.clipPath` instead.
     * @deprecated
     */;
    _proto.setClip = function setClip(clipPath) {
      this.style.clipPath = clipPath;
    }
    /**
     * Use `this.style.clipPath` instead.
     * @deprecated
     */;
    /**
     * @deprecated
     */
    _proto.set = function set(name, value) {
      // @ts-ignore
      this.config[name] = value;
    }
    /**
     * @deprecated
     */;
    _proto.get = function get(name) {
      return this.config[name];
    }
    /**
     * Use `setPosition` instead.
     * @deprecated
     */;
    _proto.moveTo = function moveTo(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      this.setPosition(position, y, z);
      return this;
    }
    /**
     * Use `setPosition` instead.
     * @deprecated
     */;
    _proto.move = function move(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      this.setPosition(position, y, z);
      return this;
    }
    /**
     * Use `this.style.zIndex` instead.
     * @deprecated
     */;
    _proto.setZIndex = function setZIndex(zIndex) {
      this.style.zIndex = zIndex;
      return this;
    };
    _createClass$1(DisplayObject, [{
      key: "interactive",
      get: function get() {
        return this.isInteractive();
      },
      set: function set(b) {
        this.style.pointerEvents = b ? 'auto' : 'none';
      }
    }]);
    return DisplayObject;
  }(Element);

  var _excluded = ["style"];
  var Circle = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Circle, _DisplayObject);
    function Circle(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded);
      return _DisplayObject.call(this, _extends({
        type: Shape.CIRCLE,
        style: _extends({
          cx: '',
          cy: '',
          r: ''
        }, style),
        initialParsedStyle: {
          anchor: [0.5, 0.5]
        }
      }, rest)) || this;
    }
    return Circle;
  }(DisplayObject);

  var pool = {};
  var htmlPool = [];
  var DisplayObjectPool = /*#__PURE__*/function () {
    function DisplayObjectPool() {}
    var _proto = DisplayObjectPool.prototype;
    _proto.getByEntity = function getByEntity(entity) {
      return pool[entity];
    };
    _proto.getAll = function getAll() {
      return Object.keys(pool).map(function (entity) {
        return pool[entity];
      });
    };
    _proto.add = function add(entity, groupOrShape) {
      pool[entity] = groupOrShape;
      if (groupOrShape.nodeName === Shape.HTML) {
        htmlPool.push(groupOrShape);
      }
    };
    _proto.remove = function remove(entity) {
      var existed = pool[entity];
      if (existed) {
        delete pool[entity];
        if (existed.nodeName === Shape.HTML) {
          var index = htmlPool.indexOf(existed);
          htmlPool.splice(index, 1);
        }
      }
    };
    _proto.getHTMLs = function getHTMLs() {
      return htmlPool;
    };
    return DisplayObjectPool;
  }();

  var _excluded$2 = ["style"];
  var Ellipse = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Ellipse, _DisplayObject);
    function Ellipse(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$2);
      return _DisplayObject.call(this, _extends({
        type: Shape.ELLIPSE,
        style: _extends({
          cx: '',
          cy: '',
          rx: '',
          ry: ''
        }, style),
        initialParsedStyle: {
          anchor: [0.5, 0.5]
        }
      }, rest)) || this;
    }
    return Ellipse;
  }(DisplayObject);

  var _excluded$3 = ["style"];
  /**
   * its attributes are inherited by its children.
   * @see https://developer.mozilla.org/zh-CN/docs/Web/SVG/Element/g
   *
   * @example
   * <g fill="white" stroke="green" stroke-width="5">
      <circle cx="40" cy="40" r="25" />
      <circle cx="60" cy="60" r="25" />
    </g>
   */
  var Group = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Group, _DisplayObject);
    function Group(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$3);
      return _DisplayObject.call(this, _extends({
        type: Shape.GROUP,
        style: _extends({
          x: '',
          y: '',
          width: '',
          height: ''
        }, style)
      }, rest)) || this;
    }
    return Group;
  }(DisplayObject);

  var _excluded$4 = ["style"];
  /**
   * HTML container
   * @see https://github.com/pmndrs/drei#html
   */
  var HTML = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(HTML, _DisplayObject);
    function HTML(_temp) {
      var _this;
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$4);
      _this = _DisplayObject.call(this, _extends({
        type: Shape.HTML,
        style: _extends({
          x: '',
          y: '',
          width: 'auto',
          height: 'auto',
          innerHTML: ''
        }, style)
      }, rest)) || this;
      _this.cullable.enable = false;
      return _this;
    }
    /**
     * return wrapper HTMLElement
     * * <div> in g-webgl/canvas
     * * <foreignObject> in g-svg
     */
    var _proto = HTML.prototype;
    _proto.getDomElement = function getDomElement() {
      return this.parsedStyle.$el;
    }
    /**
     * override with $el.getBoundingClientRect
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/getBoundingClientRect
     */;
    _proto.getBoundingClientRect = function getBoundingClientRect() {
      return this.parsedStyle.$el.getBoundingClientRect();
    };
    _proto.getClientRects = function getClientRects() {
      return [this.getBoundingClientRect()];
    };
    _proto.getBounds = function getBounds() {
      var _this$ownerDocument, _this$ownerDocument$d;
      var clientRect = this.getBoundingClientRect();
      // calc context's offset
      // @ts-ignore
      var canvasRect = (_this$ownerDocument = this.ownerDocument) === null || _this$ownerDocument === void 0 ? void 0 : (_this$ownerDocument$d = _this$ownerDocument.defaultView) === null || _this$ownerDocument$d === void 0 ? void 0 : _this$ownerDocument$d.getContextService().getBoundingClientRect();
      if (canvasRect) {
        var minX = clientRect.left - canvasRect.left;
        var minY = clientRect.top - canvasRect.top;
        var aabb = new AABB();
        aabb.setMinMax(fromValues$2(minX, minY, 0), fromValues$2(minX + clientRect.width, minY + clientRect.height, 0));
        return aabb;
      }
      return null;
    };
    _proto.getLocalBounds = function getLocalBounds() {
      if (this.parentNode) {
        var parentInvert = invert(create$1(), this.parentNode.getWorldTransform());
        var bounds = this.getBounds();
        if (!AABB.isEmpty(bounds)) {
          var localBounds = new AABB();
          localBounds.setFromTransformedAABB(bounds, parentInvert);
          return localBounds;
        }
      }
      return this.getBounds();
    };
    return HTML;
  }(DisplayObject);

  var _excluded$5 = ["style"];
  var Image = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Image, _DisplayObject);
    function Image(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$5);
      return _DisplayObject.call(this, _extends({
        type: Shape.IMAGE,
        style: _extends({
          x: '',
          y: '',
          img: '',
          width: '',
          height: ''
        }, style)
      }, rest)) || this;
    }
    return Image;
  }(DisplayObject);

  var _excluded$6 = ["style"];
  /**
   * Create a line connecting two points.
   * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Element/line
   *
   * Also support for using marker.
   */
  var Line = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Line, _DisplayObject);
    function Line(_temp) {
      var _this;
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$6);
      _this = _DisplayObject.call(this, _extends({
        type: Shape.LINE,
        style: _extends({
          x1: 0,
          y1: 0,
          x2: 0,
          y2: 0,
          z1: 0,
          z2: 0,
          isBillboard: false
        }, style)
      }, rest)) || this;
      _this.markerStartAngle = 0;
      _this.markerEndAngle = 0;
      var _this$parsedStyle = _this.parsedStyle,
        markerStart = _this$parsedStyle.markerStart,
        markerEnd = _this$parsedStyle.markerEnd;
      if (markerStart && markerStart instanceof DisplayObject) {
        _this.markerStartAngle = markerStart.getLocalEulerAngles();
        _this.appendChild(markerStart);
      }
      if (markerEnd && markerEnd instanceof DisplayObject) {
        _this.markerEndAngle = markerEnd.getLocalEulerAngles();
        _this.appendChild(markerEnd);
      }
      _this.transformMarker(true);
      _this.transformMarker(false);
      return _this;
    }
    var _proto = Line.prototype;
    _proto.attributeChangedCallback = function attributeChangedCallback(attrName, oldValue, newValue, prevParsedValue, newParsedValue) {
      if (attrName === 'x1' || attrName === 'y1' || attrName === 'x2' || attrName === 'y2' || attrName === 'markerStartOffset' || attrName === 'markerEndOffset') {
        this.transformMarker(true);
        this.transformMarker(false);
      } else if (attrName === 'markerStart') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject) {
          this.markerStartAngle = 0;
          prevParsedValue.remove();
        }
        // CSSKeyword 'unset'
        if (newParsedValue && newParsedValue instanceof DisplayObject) {
          this.markerStartAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(true);
        }
      } else if (attrName === 'markerEnd') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject) {
          this.markerEndAngle = 0;
          prevParsedValue.remove();
        }
        if (newParsedValue && newParsedValue instanceof DisplayObject) {
          this.markerEndAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(false);
        }
      }
    };
    _proto.transformMarker = function transformMarker(isStart) {
      var _this$parsedStyle2 = this.parsedStyle,
        markerStart = _this$parsedStyle2.markerStart,
        markerEnd = _this$parsedStyle2.markerEnd,
        markerStartOffset = _this$parsedStyle2.markerStartOffset,
        markerEndOffset = _this$parsedStyle2.markerEndOffset,
        x1 = _this$parsedStyle2.x1,
        x2 = _this$parsedStyle2.x2,
        y1 = _this$parsedStyle2.y1,
        y2 = _this$parsedStyle2.y2,
        defX = _this$parsedStyle2.defX,
        defY = _this$parsedStyle2.defY;
      var marker = isStart ? markerStart : markerEnd;
      if (!marker || !(marker instanceof DisplayObject)) {
        return;
      }
      var rad = 0;
      var x;
      var y;
      var ox;
      var oy;
      var offset;
      var originalAngle;
      if (isStart) {
        ox = x1 - defX;
        oy = y1 - defY;
        x = x2 - x1;
        y = y2 - y1;
        offset = markerStartOffset || 0;
        originalAngle = this.markerStartAngle;
      } else {
        ox = x2 - defX;
        oy = y2 - defY;
        x = x1 - x2;
        y = y1 - y2;
        offset = markerEndOffset || 0;
        originalAngle = this.markerEndAngle;
      }
      rad = Math.atan2(y, x);
      // account for markerOffset
      marker.setLocalEulerAngles(rad * 180 / Math.PI + originalAngle);
      marker.setLocalPosition(ox + Math.cos(rad) * offset, oy + Math.sin(rad) * offset);
    };
    _proto.getPoint = function getPoint(ratio, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      // TODO: account for z1/z2 in 3D line
      var _this$parsedStyle3 = this.parsedStyle,
        x1 = _this$parsedStyle3.x1,
        y1 = _this$parsedStyle3.y1,
        x2 = _this$parsedStyle3.x2,
        y2 = _this$parsedStyle3.y2,
        defX = _this$parsedStyle3.defX,
        defY = _this$parsedStyle3.defY;
      var _LineUtil$pointAt = line.pointAt(x1, y1, x2, y2, ratio),
        x = _LineUtil$pointAt.x,
        y = _LineUtil$pointAt.y;
      var transformed = transformMat4(create$2(), fromValues$2(x - defX, y - defY, 0), inWorldSpace ? this.getWorldTransform() : this.getLocalTransform());
      // apply local transformation
      return new Point(transformed[0], transformed[1]);
    };
    _proto.getPointAtLength = function getPointAtLength(distance, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      return this.getPoint(distance / this.getTotalLength(), inWorldSpace);
    };
    _proto.getTotalLength = function getTotalLength() {
      // TODO: account for z1/z2 in 3D line
      var _this$parsedStyle4 = this.parsedStyle,
        x1 = _this$parsedStyle4.x1,
        y1 = _this$parsedStyle4.y1,
        x2 = _this$parsedStyle4.x2,
        y2 = _this$parsedStyle4.y2;
      return line.length(x1, y1, x2, y2);
    };
    return Line;
  }(DisplayObject);

  var _excluded$7 = ["style"];
  var Path = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Path, _DisplayObject);
    /**
     * markers placed at the mid
     */

    function Path(_temp) {
      var _this;
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$7);
      _this = _DisplayObject.call(this, _extends({
        type: Shape.PATH,
        style: _extends({
          path: '',
          miterLimit: ''
        }, style)
      }, rest)) || this;
      _this.markerStartAngle = 0;
      _this.markerEndAngle = 0;
      _this.markerMidList = [];
      var _this$parsedStyle = _this.parsedStyle,
        markerStart = _this$parsedStyle.markerStart,
        markerEnd = _this$parsedStyle.markerEnd,
        markerMid = _this$parsedStyle.markerMid;
      if (markerStart && markerStart instanceof DisplayObject) {
        _this.markerStartAngle = markerStart.getLocalEulerAngles();
        _this.appendChild(markerStart);
      }
      if (markerMid && markerMid instanceof DisplayObject) {
        _this.placeMarkerMid(markerMid);
      }
      if (markerEnd && markerEnd instanceof DisplayObject) {
        _this.markerEndAngle = markerEnd.getLocalEulerAngles();
        _this.appendChild(markerEnd);
      }
      _this.transformMarker(true);
      _this.transformMarker(false);
      return _this;
    }
    var _proto = Path.prototype;
    _proto.attributeChangedCallback = function attributeChangedCallback(attrName, oldValue, newValue, prevParsedValue, newParsedValue) {
      if (attrName === 'path') {
        // recalc markers
        this.transformMarker(true);
        this.transformMarker(false);
        this.placeMarkerMid(this.parsedStyle.markerMid);
      } else if (attrName === 'markerStartOffset' || attrName === 'markerEndOffset') {
        this.transformMarker(true);
        this.transformMarker(false);
      } else if (attrName === 'markerStart') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject) {
          this.markerStartAngle = 0;
          prevParsedValue.remove();
        }
        // CSSKeyword 'unset'
        if (newParsedValue && newParsedValue instanceof DisplayObject) {
          this.markerStartAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(true);
        }
      } else if (attrName === 'markerEnd') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject) {
          this.markerEndAngle = 0;
          prevParsedValue.remove();
        }
        if (newParsedValue && newParsedValue instanceof DisplayObject) {
          this.markerEndAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(false);
        }
      } else if (attrName === 'markerMid') {
        this.placeMarkerMid(newParsedValue);
      }
    };
    _proto.transformMarker = function transformMarker(isStart) {
      var _this$parsedStyle2 = this.parsedStyle,
        markerStart = _this$parsedStyle2.markerStart,
        markerEnd = _this$parsedStyle2.markerEnd,
        markerStartOffset = _this$parsedStyle2.markerStartOffset,
        markerEndOffset = _this$parsedStyle2.markerEndOffset,
        defX = _this$parsedStyle2.defX,
        defY = _this$parsedStyle2.defY;
      var marker = isStart ? markerStart : markerEnd;
      if (!marker || !(marker instanceof DisplayObject)) {
        return;
      }
      var rad = 0;
      var x;
      var y;
      var ox;
      var oy;
      var offset;
      var originalAngle;
      if (isStart) {
        var _this$getStartTangent = this.getStartTangent(),
          p1 = _this$getStartTangent[0],
          p2 = _this$getStartTangent[1];
        ox = p2[0] - defX;
        oy = p2[1] - defY;
        x = p1[0] - p2[0];
        y = p1[1] - p2[1];
        offset = markerStartOffset || 0;
        originalAngle = this.markerStartAngle;
      } else {
        var _this$getEndTangent = this.getEndTangent(),
          _p = _this$getEndTangent[0],
          _p2 = _this$getEndTangent[1];
        ox = _p2[0] - defX;
        oy = _p2[1] - defY;
        x = _p[0] - _p2[0];
        y = _p[1] - _p2[1];
        offset = markerEndOffset || 0;
        originalAngle = this.markerEndAngle;
      }
      rad = Math.atan2(y, x);
      // account for markerOffset
      marker.setLocalEulerAngles(rad * 180 / Math.PI + originalAngle);
      marker.setLocalPosition(ox + Math.cos(rad) * offset, oy + Math.sin(rad) * offset);
    };
    _proto.placeMarkerMid = function placeMarkerMid(marker) {
      var _this$parsedStyle3 = this.parsedStyle,
        segments = _this$parsedStyle3.path.segments,
        defX = _this$parsedStyle3.defX,
        defY = _this$parsedStyle3.defY;
      // clear all existed markers
      this.markerMidList.forEach(function (marker) {
        marker.remove();
      });
      if (marker && marker instanceof DisplayObject) {
        for (var i = 1; i < segments.length - 1; i++) {
          var _segments$i$currentPo = segments[i].currentPoint,
            ox = _segments$i$currentPo[0],
            oy = _segments$i$currentPo[1];
          var cloned = i === 1 ? marker : marker.cloneNode(true);
          this.markerMidList.push(cloned);
          this.appendChild(cloned);
          cloned.setLocalPosition(ox - defX, oy - defY);
          // TODO: orient of marker
        }
      }
    }
    /**
     * Returns the total length of the path.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGGeometryElement/getTotalLength
     */;
    _proto.getTotalLength = function getTotalLength() {
      return getOrCalculatePathTotalLength(this);
    }
    /**
     * Returns the point at a given distance along the path.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGGeometryElement/getPointAtLength
     */;
    _proto.getPointAtLength = function getPointAtLength$1(distance, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      var _this$parsedStyle4 = this.parsedStyle,
        defX = _this$parsedStyle4.defX,
        defY = _this$parsedStyle4.defY,
        absolutePath = _this$parsedStyle4.path.absolutePath;
      var _getPointAtLength2 = getPointAtLength(absolutePath, distance),
        x = _getPointAtLength2.x,
        y = _getPointAtLength2.y;
      var transformed = transformMat4(create$2(), fromValues$2(x - defX, y - defY, 0), inWorldSpace ? this.getWorldTransform() : this.getLocalTransform());
      // apply local transformation
      return new Point(transformed[0], transformed[1]);
    }
    /**
     * Returns the point at a given ratio of the total length in path.
     */;
    _proto.getPoint = function getPoint(ratio, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      return this.getPointAtLength(ratio * getOrCalculatePathTotalLength(this), inWorldSpace);
    }
    /**
     * Get start tangent vector
     */;
    _proto.getStartTangent = function getStartTangent() {
      var segments = this.parsedStyle.path.segments;
      var result = [];
      if (segments.length > 1) {
        var startPoint = segments[0].currentPoint;
        var endPoint = segments[1].currentPoint;
        var tangent = segments[1].startTangent;
        result = [];
        if (tangent) {
          result.push([startPoint[0] - tangent[0], startPoint[1] - tangent[1]]);
          result.push([startPoint[0], startPoint[1]]);
        } else {
          result.push([endPoint[0], endPoint[1]]);
          result.push([startPoint[0], startPoint[1]]);
        }
      }
      return result;
    }
    /**
     * Get end tangent vector
     */;
    _proto.getEndTangent = function getEndTangent() {
      var segments = this.parsedStyle.path.segments;
      var length = segments.length;
      var result = [];
      if (length > 1) {
        var startPoint = segments[length - 2].currentPoint;
        var endPoint = segments[length - 1].currentPoint;
        var tangent = segments[length - 1].endTangent;
        result = [];
        if (tangent) {
          result.push([endPoint[0] - tangent[0], endPoint[1] - tangent[1]]);
          result.push([endPoint[0], endPoint[1]]);
        } else {
          result.push([startPoint[0], startPoint[1]]);
          result.push([endPoint[0], endPoint[1]]);
        }
      }
      return result;
    };
    return Path;
  }(DisplayObject);

  var _excluded$8 = ["style"];
  var Polygon = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Polygon, _DisplayObject);
    /**
     * markers placed at the mid
     */

    function Polygon(_temp) {
      var _this;
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$8);
      _this = _DisplayObject.call(this, _extends({
        type: Shape.POLYGON,
        style: _extends({
          points: '',
          miterLimit: '',
          isClosed: true
        }, style)
      }, rest)) || this;
      _this.markerStartAngle = 0;
      _this.markerEndAngle = 0;
      _this.markerMidList = [];
      var _this$parsedStyle = _this.parsedStyle,
        markerStart = _this$parsedStyle.markerStart,
        markerEnd = _this$parsedStyle.markerEnd,
        markerMid = _this$parsedStyle.markerMid;
      if (markerStart && markerStart instanceof DisplayObject) {
        _this.markerStartAngle = markerStart.getLocalEulerAngles();
        _this.appendChild(markerStart);
      }
      if (markerMid && markerMid instanceof DisplayObject) {
        _this.placeMarkerMid(markerMid);
      }
      if (markerEnd && markerEnd instanceof DisplayObject) {
        _this.markerEndAngle = markerEnd.getLocalEulerAngles();
        _this.appendChild(markerEnd);
      }
      _this.transformMarker(true);
      _this.transformMarker(false);
      return _this;
    }
    var _proto = Polygon.prototype;
    _proto.attributeChangedCallback = function attributeChangedCallback(attrName, oldValue, newValue, prevParsedValue, newParsedValue) {
      if (attrName === 'points') {
        // recalc markers
        this.transformMarker(true);
        this.transformMarker(false);
        this.placeMarkerMid(this.parsedStyle.markerMid);
      } else if (attrName === 'markerStartOffset' || attrName === 'markerEndOffset') {
        this.transformMarker(true);
        this.transformMarker(false);
      } else if (attrName === 'markerStart') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject) {
          this.markerStartAngle = 0;
          prevParsedValue.remove();
        }
        // CSSKeyword 'unset'
        if (newParsedValue && newParsedValue instanceof DisplayObject) {
          this.markerStartAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(true);
        }
      } else if (attrName === 'markerEnd') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject) {
          this.markerEndAngle = 0;
          prevParsedValue.remove();
        }
        if (newParsedValue && newParsedValue instanceof DisplayObject) {
          this.markerEndAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(false);
        }
      } else if (attrName === 'markerMid') {
        this.placeMarkerMid(newParsedValue);
      }
    };
    _proto.transformMarker = function transformMarker(isStart) {
      var _this$parsedStyle2 = this.parsedStyle,
        markerStart = _this$parsedStyle2.markerStart,
        markerEnd = _this$parsedStyle2.markerEnd,
        markerStartOffset = _this$parsedStyle2.markerStartOffset,
        markerEndOffset = _this$parsedStyle2.markerEndOffset,
        points = _this$parsedStyle2.points.points,
        defX = _this$parsedStyle2.defX,
        defY = _this$parsedStyle2.defY;
      var marker = isStart ? markerStart : markerEnd;
      if (!marker || !(marker instanceof DisplayObject)) {
        return;
      }
      var rad = 0;
      var x;
      var y;
      var ox;
      var oy;
      var offset;
      var originalAngle;
      ox = points[0][0] - defX;
      oy = points[0][1] - defY;
      if (isStart) {
        x = points[1][0] - points[0][0];
        y = points[1][1] - points[0][1];
        offset = markerStartOffset || 0;
        originalAngle = this.markerStartAngle;
      } else {
        var length = points.length;
        if (!this.style.isClosed) {
          ox = points[length - 1][0] - defX;
          oy = points[length - 1][1] - defY;
          x = points[length - 2][0] - points[length - 1][0];
          y = points[length - 2][1] - points[length - 1][1];
        } else {
          x = points[length - 1][0] - points[0][0];
          y = points[length - 1][1] - points[0][1];
        }
        offset = markerEndOffset || 0;
        originalAngle = this.markerEndAngle;
      }
      rad = Math.atan2(y, x);
      // account for markerOffset
      marker.setLocalEulerAngles(rad * 180 / Math.PI + originalAngle);
      marker.setLocalPosition(ox + Math.cos(rad) * offset, oy + Math.sin(rad) * offset);
    };
    _proto.placeMarkerMid = function placeMarkerMid(marker) {
      var _this$parsedStyle3 = this.parsedStyle,
        points = _this$parsedStyle3.points.points,
        defX = _this$parsedStyle3.defX,
        defY = _this$parsedStyle3.defY;
      // clear all existed markers
      this.markerMidList.forEach(function (marker) {
        marker.remove();
      });
      this.markerMidList = [];
      if (marker && marker instanceof DisplayObject) {
        for (var i = 1; i < (this.style.isClosed ? points.length : points.length - 1); i++) {
          var ox = points[i][0] - defX;
          var oy = points[i][1] - defY;
          var cloned = i === 1 ? marker : marker.cloneNode(true);
          this.markerMidList.push(cloned);
          this.appendChild(cloned);
          cloned.setLocalPosition(ox, oy);
          // TODO: orient of marker
        }
      }
    };
    return Polygon;
  }(DisplayObject);

  var _excluded$9 = ["style"];
  /**
   * Polyline inherits the marker-related capabilities of Polygon.
   */
  var Polyline = /*#__PURE__*/function (_Polygon) {
    _inheritsLoose(Polyline, _Polygon);
    function Polyline(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$9);
      return _Polygon.call(this, _extends({
        type: Shape.POLYLINE,
        style: _extends({
          points: '',
          miterLimit: '',
          isClosed: false
        }, style)
      }, rest)) || this;
    }
    var _proto = Polyline.prototype;
    _proto.getTotalLength = function getTotalLength() {
      return this.parsedStyle.points.totalLength;
    };
    _proto.getPointAtLength = function getPointAtLength(distance, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      return this.getPoint(distance / this.getTotalLength(), inWorldSpace);
    };
    _proto.getPoint = function getPoint(ratio, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      var _this$parsedStyle = this.parsedStyle,
        defX = _this$parsedStyle.defX,
        defY = _this$parsedStyle.defY,
        _this$parsedStyle$poi = _this$parsedStyle.points,
        points = _this$parsedStyle$poi.points,
        segments = _this$parsedStyle$poi.segments;
      var subt = 0;
      var index = 0;
      segments.forEach(function (v, i) {
        if (ratio >= v[0] && ratio <= v[1]) {
          subt = (ratio - v[0]) / (v[1] - v[0]);
          index = i;
        }
      });
      var _LineUtil$pointAt = line.pointAt(points[index][0], points[index][1], points[index + 1][0], points[index + 1][1], subt),
        x = _LineUtil$pointAt.x,
        y = _LineUtil$pointAt.y;
      var transformed = transformMat4(create$2(), fromValues$2(x - defX, y - defY, 0), inWorldSpace ? this.getWorldTransform() : this.getLocalTransform());
      // apply local transformation
      return new Point(transformed[0], transformed[1]);
    };
    _proto.getStartTangent = function getStartTangent() {
      var points = this.parsedStyle.points.points;
      var result = [];
      result.push([points[1][0], points[1][1]]);
      result.push([points[0][0], points[0][1]]);
      return result;
    };
    _proto.getEndTangent = function getEndTangent() {
      var points = this.parsedStyle.points.points;
      var l = points.length - 1;
      var result = [];
      result.push([points[l - 1][0], points[l - 1][1]]);
      result.push([points[l][0], points[l][1]]);
      return result;
    };
    return Polyline;
  }(Polygon);

  var _excluded$a = ["style"];
  var Rect = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Rect, _DisplayObject);
    function Rect(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$a);
      return _DisplayObject.call(this, _extends({
        type: Shape.RECT,
        style: _extends({
          x: '',
          y: '',
          width: '',
          height: '',
          radius: ''
        }, style)
      }, rest)) || this;
    }
    return Rect;
  }(DisplayObject);

  var _excluded$b = ["style"];
  /**
   * <text> @see https://developer.mozilla.org/en-US/docs/Web/API/SVGTextElement
   */
  var Text = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose(Text, _DisplayObject);
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGTextContentElement#constants
     */
    // LENGTHADJUST_SPACING: number = 1;
    // LENGTHADJUST_SPACINGANDGLYPHS: number = 2;
    // LENGTHADJUST_UNKNOWN: number = 0;
    function Text(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose(_ref, _excluded$b);
      return _DisplayObject.call(this, _extends({
        type: Shape.TEXT,
        style: _extends({
          x: '',
          y: '',
          text: '',
          fontSize: '',
          fontFamily: '',
          fontStyle: '',
          fontWeight: '',
          fontVariant: '',
          textAlign: '',
          textBaseline: '',
          textTransform: '',
          fill: 'black',
          letterSpacing: '',
          lineHeight: '',
          miterLimit: '',
          // whiteSpace: 'pre',
          wordWrap: false,
          wordWrapWidth: 0,
          leading: 0,
          dx: '',
          dy: ''
        }, style)
      }, rest)) || this;
    }
    // lengthAdjust: SVGAnimatedEnumeration;
    // textLength: SVGAnimatedLength;
    // getCharNumAtPosition(point?: DOMPointInit): number {
    //   throw new Error('Method not implemented.');
    // }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGTextContentElement
     */
    var _proto = Text.prototype;
    _proto.getComputedTextLength = function getComputedTextLength() {
      var _this$parsedStyle$met;
      return ((_this$parsedStyle$met = this.parsedStyle.metrics) === null || _this$parsedStyle$met === void 0 ? void 0 : _this$parsedStyle$met.maxLineWidth) || 0;
    }
    // getEndPositionOfChar(charnum: number): DOMPoint {
    //   throw new Error('Method not implemented.');
    // }
    // getExtentOfChar(charnum: number): DOMRect {
    //   throw new Error('Method not implemented.');
    // }
    // getNumberOfChars(): number {
    //   throw new Error('Method not implemented.');
    // }
    // getRotationOfChar(charnum: number): number {
    //   throw new Error('Method not implemented.');
    // }
    // getStartPositionOfChar(charnum: number): DOMPoint {
    //   throw new Error('Method not implemented.');
    // }
    // getSubStringLength(charnum: number, nchars: number): number {
    //   throw new Error('Method not implemented.');
    // }
    // selectSubString(charnum: number, nchars: number): void {
    //   throw new Error('Method not implemented.');
    // }
    ;
    _proto.getLineBoundingRects = function getLineBoundingRects() {
      var _this$parsedStyle$met2;
      return ((_this$parsedStyle$met2 = this.parsedStyle.metrics) === null || _this$parsedStyle$met2 === void 0 ? void 0 : _this$parsedStyle$met2.lineMetrics) || [];
    };
    _proto.isOverflowing = function isOverflowing() {
      return !!this.parsedStyle.isOverflowing;
    };
    return Text;
  }(DisplayObject);

  /**
   * Different type of cameras, eg. simple camera used in 2D scene or
   * advanced camera which can do actions & switch between landmarks.
   */
  var CameraType;
  (function (CameraType) {
    /**
     * Performs all the rotational operations with the focal point instead of the camera position.
     * This type of camera is useful in applications(like CAD) where 3D objects are being designed or explored.
     * Camera cannot orbits over the north & south poles.
     * @see http://voxelent.com/tutorial-cameras/
     *
     * In Three.js it's used in OrbitControls.
     * @see https://threejs.org/docs/#examples/zh/controls/OrbitControls
     */
    CameraType[CameraType["ORBITING"] = 0] = "ORBITING";
    /**
     * It's similar to the ORBITING camera, but it allows the camera to orbit over the north or south poles.
     *
     * In Three.js it's used in OrbitControls.
     * @see https://threejs.org/docs/#examples/en/controls/TrackballControls
     */
    CameraType[CameraType["EXPLORING"] = 1] = "EXPLORING";
    /**
     * Performs all the rotational operations with the camera position.
     * It's useful in first person shooting games.
     * Camera cannot orbits over the north & south poles.
     *
     * In Three.js it's used in FirstPersonControls.
     * @see https://threejs.org/docs/#examples/en/controls/FirstPersonControls
     */
    CameraType[CameraType["TRACKING"] = 2] = "TRACKING";
  })(CameraType || (CameraType = {}));
  /**
   * CameraType must be TRACKING
   */
  var CameraTrackingMode;
  (function (CameraTrackingMode) {
    CameraTrackingMode[CameraTrackingMode["DEFAULT"] = 0] = "DEFAULT";
    CameraTrackingMode[CameraTrackingMode["ROTATIONAL"] = 1] = "ROTATIONAL";
    CameraTrackingMode[CameraTrackingMode["TRANSLATIONAL"] = 2] = "TRANSLATIONAL";
    CameraTrackingMode[CameraTrackingMode["CINEMATIC"] = 3] = "CINEMATIC";
  })(CameraTrackingMode || (CameraTrackingMode = {}));
  var CameraProjectionMode;
  (function (CameraProjectionMode) {
    CameraProjectionMode[CameraProjectionMode["ORTHOGRAPHIC"] = 0] = "ORTHOGRAPHIC";
    CameraProjectionMode[CameraProjectionMode["PERSPECTIVE"] = 1] = "PERSPECTIVE";
  })(CameraProjectionMode || (CameraProjectionMode = {}));
  var CameraEvent = {
    UPDATED: 'updated'
  };

  var MIN_DISTANCE = 0.0002;
  /**
   * 参考「WebGL Insights - 23.Designing Cameras for WebGL Applications」，基于 Responsible Camera 思路设计
   * @see https://github.com/d13g0/nucleo.js/blob/master/source/camera/Camera.js
   *
   * 保存相机参数，定义相机动作：
   * 1. dolly 沿 n 轴移动
   * 2. pan 沿 u v 轴移动
   * 3. rotate 以方位角旋转
   * 4. 移动到 Landmark，具有平滑的动画效果，其间禁止其他用户交互
   */
  var Camera = /*#__PURE__*/function () {
    function Camera() {
      this.canvas = void 0;
      this.eventEmitter = new eventemitter3();
      this.matrix = create$1();
      this.right = fromValues$2(1, 0, 0);
      this.up = fromValues$2(0, 1, 0);
      this.forward = fromValues$2(0, 0, 1);
      this.position = fromValues$2(0, 0, 1);
      this.focalPoint = fromValues$2(0, 0, 0);
      this.distanceVector = fromValues$2(0, 0, -1);
      this.distance = 1;
      this.azimuth = 0;
      this.elevation = 0;
      this.roll = 0;
      this.relAzimuth = 0;
      this.relElevation = 0;
      this.relRoll = 0;
      this.dollyingStep = 0;
      this.maxDistance = Infinity;
      this.minDistance = -Infinity;
      this.zoom = 1;
      this.rotateWorld = false;
      this.fov = 30;
      this.near = 0.1;
      this.far = 1000;
      this.aspect = 1;
      this.left = void 0;
      this.rright = void 0;
      this.top = void 0;
      this.bottom = void 0;
      this.projectionMatrix = create$1();
      this.projectionMatrixInverse = create$1();
      this.jitteredProjectionMatrix = undefined;
      this.view = void 0;
      this.enableUpdate = true;
      this.type = CameraType.EXPLORING;
      this.trackingMode = CameraTrackingMode.DEFAULT;
      this.projectionMode = CameraProjectionMode.PERSPECTIVE;
      this.frustum = new Frustum();
      this.orthoMatrix = create$1();
    }
    var _proto = Camera.prototype;
    // constructor(type = CameraType.EXPLORING, trackingMode = CameraTrackingMode.DEFAULT) {
    //   this.setType(type, trackingMode);
    // }
    _proto.isOrtho = function isOrtho() {
      return this.projectionMode === CameraProjectionMode.ORTHOGRAPHIC;
    };
    _proto.getProjectionMode = function getProjectionMode() {
      return this.projectionMode;
    };
    _proto.getPerspective = function getPerspective() {
      // account for TAA
      return this.jitteredProjectionMatrix || this.projectionMatrix;
    };
    _proto.getPerspectiveInverse = function getPerspectiveInverse() {
      return this.projectionMatrixInverse;
    };
    _proto.getFrustum = function getFrustum() {
      return this.frustum;
    };
    _proto.getPosition = function getPosition() {
      return this.position;
    };
    _proto.getFocalPoint = function getFocalPoint() {
      return this.focalPoint;
    };
    _proto.getDollyingStep = function getDollyingStep() {
      return this.dollyingStep;
    };
    _proto.getNear = function getNear() {
      return this.near;
    };
    _proto.getFar = function getFar() {
      return this.far;
    };
    _proto.getZoom = function getZoom() {
      return this.zoom;
    };
    _proto.getOrthoMatrix = function getOrthoMatrix() {
      return this.orthoMatrix;
    };
    _proto.getView = function getView() {
      return this.view;
    };
    _proto.setEnableUpdate = function setEnableUpdate(enabled) {
      this.enableUpdate = enabled;
    };
    _proto.setType = function setType(type, trackingMode) {
      this.type = type;
      if (this.type === CameraType.EXPLORING) {
        this.setWorldRotation(true);
      } else {
        this.setWorldRotation(false);
      }
      this._getAngles();
      if (this.type === CameraType.TRACKING && trackingMode !== undefined) {
        this.setTrackingMode(trackingMode);
      }
      return this;
    };
    _proto.setProjectionMode = function setProjectionMode(projectionMode) {
      this.projectionMode = projectionMode;
      return this;
    };
    _proto.setTrackingMode = function setTrackingMode(trackingMode) {
      if (this.type !== CameraType.TRACKING) {
        throw new Error('Impossible to set a tracking mode if the camera is not of tracking type');
      }
      this.trackingMode = trackingMode;
      return this;
    }
    /**
     * If flag is true, it reverses the azimuth and elevation angles.
     * Subsequent calls to rotate, setAzimuth, setElevation,
     * changeAzimuth or changeElevation will cause the inverted effect.
     * setRoll or changeRoll is not affected by this method.
     *
     * This inversion is useful when one wants to simulate that the world
     * is moving, instead of the camera.
     *
     * By default the camera angles are not reversed.
     * @param {Boolean} flag the boolean flag to reverse the angles.
     */;
    _proto.setWorldRotation = function setWorldRotation(flag) {
      this.rotateWorld = flag;
      this._getAngles();
      return this;
    }
    /**
     * 计算 MV 矩阵，为相机矩阵的逆矩阵
     */;
    _proto.getViewTransform = function getViewTransform() {
      return invert(create$1(), this.matrix);
    };
    _proto.getWorldTransform = function getWorldTransform() {
      return this.matrix;
    };
    _proto.jitterProjectionMatrix = function jitterProjectionMatrix(x, y) {
      var translation = fromTranslation(create$1(), [x, y, 0]);
      this.jitteredProjectionMatrix = multiply(create$1(), translation, this.projectionMatrix);
    };
    _proto.clearJitterProjectionMatrix = function clearJitterProjectionMatrix() {
      this.jitteredProjectionMatrix = undefined;
    }
    /**
     * 设置相机矩阵
     */;
    _proto.setMatrix = function setMatrix(matrix) {
      this.matrix = matrix;
      this._update();
      return this;
    };
    _proto.setFov = function setFov(fov) {
      this.setPerspective(this.near, this.far, fov, this.aspect);
      return this;
    };
    _proto.setAspect = function setAspect(aspect) {
      this.setPerspective(this.near, this.far, this.fov, aspect);
      return this;
    };
    _proto.setNear = function setNear(near) {
      if (this.projectionMode === CameraProjectionMode.PERSPECTIVE) {
        this.setPerspective(near, this.far, this.fov, this.aspect);
      } else {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, near, this.far);
      }
      return this;
    };
    _proto.setFar = function setFar(far) {
      if (this.projectionMode === CameraProjectionMode.PERSPECTIVE) {
        this.setPerspective(this.near, far, this.fov, this.aspect);
      } else {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, this.near, far);
      }
      return this;
    }
    /**
     * Sets an offset in a larger frustum, used in PixelPicking
     */;
    _proto.setViewOffset = function setViewOffset(fullWidth, fullHeight, x, y, width, height) {
      this.aspect = fullWidth / fullHeight;
      if (this.view === undefined) {
        this.view = {
          enabled: true,
          fullWidth: 1,
          fullHeight: 1,
          offsetX: 0,
          offsetY: 0,
          width: 1,
          height: 1
        };
      }
      this.view.enabled = true;
      this.view.fullWidth = fullWidth;
      this.view.fullHeight = fullHeight;
      this.view.offsetX = x;
      this.view.offsetY = y;
      this.view.width = width;
      this.view.height = height;
      if (this.projectionMode === CameraProjectionMode.PERSPECTIVE) {
        this.setPerspective(this.near, this.far, this.fov, this.aspect);
      } else {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, this.near, this.far);
      }
      return this;
    };
    _proto.clearViewOffset = function clearViewOffset() {
      if (this.view !== undefined) {
        this.view.enabled = false;
      }
      if (this.projectionMode === CameraProjectionMode.PERSPECTIVE) {
        this.setPerspective(this.near, this.far, this.fov, this.aspect);
      } else {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, this.near, this.far);
      }
      return this;
    };
    _proto.setZoom = function setZoom(zoom) {
      this.zoom = zoom;
      if (this.projectionMode === CameraProjectionMode.ORTHOGRAPHIC) {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, this.near, this.far);
      } else if (this.projectionMode === CameraProjectionMode.PERSPECTIVE) {
        this.setPerspective(this.near, this.far, this.fov, this.aspect);
      }
      return this;
    }
    /**
     * Zoom by specified point in viewport coordinates.
     */;
    _proto.setZoomByViewportPoint = function setZoomByViewportPoint(zoom, viewportPoint) {
      var _this$canvas$viewport = this.canvas.viewport2Canvas({
          x: viewportPoint[0],
          y: viewportPoint[1]
        }),
        ox = _this$canvas$viewport.x,
        oy = _this$canvas$viewport.y;
      var roll = this.roll;
      this.rotate(0, 0, -roll);
      this.setPosition(ox, oy);
      this.setFocalPoint(ox, oy);
      this.setZoom(zoom);
      this.rotate(0, 0, roll);
      var _this$canvas$viewport2 = this.canvas.viewport2Canvas({
          x: viewportPoint[0],
          y: viewportPoint[1]
        }),
        cx = _this$canvas$viewport2.x,
        cy = _this$canvas$viewport2.y;
      // project to rotated axis
      var dvec = fromValues$2(cx - ox, cy - oy, 0);
      var dx = dot(dvec, this.right) / length(this.right);
      var dy = dot(dvec, this.up) / length(this.up);
      this.pan(-dx, -dy);
      return this;
    };
    _proto.setPerspective = function setPerspective(near, far, fov, aspect) {
      var _this$view;
      this.projectionMode = CameraProjectionMode.PERSPECTIVE;
      this.fov = fov;
      this.near = near;
      this.far = far;
      this.aspect = aspect;
      var top = this.near * Math.tan(deg2rad(0.5 * this.fov)) / this.zoom;
      var height = 2 * top;
      var width = this.aspect * height;
      var left = -0.5 * width;
      if ((_this$view = this.view) === null || _this$view === void 0 ? void 0 : _this$view.enabled) {
        var fullWidth = this.view.fullWidth;
        var fullHeight = this.view.fullHeight;
        left += this.view.offsetX * width / fullWidth;
        top -= this.view.offsetY * height / fullHeight;
        width *= this.view.width / fullWidth;
        height *= this.view.height / fullHeight;
      }
      makePerspective(this.projectionMatrix, left, left + width, top, top - height, near, this.far);
      // flipY since the origin of OpenGL/WebGL is bottom-left compared with top-left in Canvas2D
      scale(this.projectionMatrix, this.projectionMatrix, fromValues$2(1, -1, 1));
      invert(this.projectionMatrixInverse, this.projectionMatrix);
      this.triggerUpdate();
      return this;
    };
    _proto.setOrthographic = function setOrthographic(l, r, t, b, near, far) {
      var _this$view2;
      this.projectionMode = CameraProjectionMode.ORTHOGRAPHIC;
      this.rright = r;
      this.left = l;
      this.top = t;
      this.bottom = b;
      this.near = near;
      this.far = far;
      var dx = (this.rright - this.left) / (2 * this.zoom);
      var dy = (this.top - this.bottom) / (2 * this.zoom);
      var cx = (this.rright + this.left) / 2;
      var cy = (this.top + this.bottom) / 2;
      var left = cx - dx;
      var right = cx + dx;
      var top = cy + dy;
      var bottom = cy - dy;
      if ((_this$view2 = this.view) === null || _this$view2 === void 0 ? void 0 : _this$view2.enabled) {
        var scaleW = (this.rright - this.left) / this.view.fullWidth / this.zoom;
        var scaleH = (this.top - this.bottom) / this.view.fullHeight / this.zoom;
        left += scaleW * this.view.offsetX;
        right = left + scaleW * this.view.width;
        top -= scaleH * this.view.offsetY;
        bottom = top - scaleH * this.view.height;
      }
      ortho(this.projectionMatrix, left, right, bottom, top, near, far);
      // flipY since the origin of OpenGL/WebGL is bottom-left compared with top-left in Canvas2D
      scale(this.projectionMatrix, this.projectionMatrix, fromValues$2(1, -1, 1));
      invert(this.projectionMatrixInverse, this.projectionMatrix);
      this._getOrthoMatrix();
      this.triggerUpdate();
      return this;
    }
    /**
     * Move the camera in world coordinates.
     * It will keep looking at the current focal point.
     *
     * support scalars or vectors.
     * @example
     * setPosition(1, 2, 3);
     * setPosition([1, 2, 3]);
     */;
    _proto.setPosition = function setPosition(x, y, z) {
      if (y === void 0) {
        y = this.position[1];
      }
      if (z === void 0) {
        z = this.position[2];
      }
      var position = createVec3(x, y, z);
      this._setPosition(position);
      this.setFocalPoint(this.focalPoint);
      this.triggerUpdate();
      return this;
    }
    /**
     * Sets the focal point of this camera in world coordinates.
     *
     * support scalars or vectors.
     * @example
     * setFocalPoint(1, 2, 3);
     * setFocalPoint([1, 2, 3]);
     */;
    _proto.setFocalPoint = function setFocalPoint(x, y, z) {
      if (y === void 0) {
        y = this.focalPoint[1];
      }
      if (z === void 0) {
        z = this.focalPoint[2];
      }
      var up = fromValues$2(0, 1, 0);
      this.focalPoint = createVec3(x, y, z);
      if (this.trackingMode === CameraTrackingMode.CINEMATIC) {
        var d = subtract$1(create$2(), this.focalPoint, this.position);
        x = d[0];
        y = d[1];
        z = d[2];
        var r = length(d);
        var el = rad2deg(Math.asin(y / r));
        var az = 90 + rad2deg(Math.atan2(z, x));
        var m = create$1();
        rotateY(m, m, deg2rad(az));
        rotateX(m, m, deg2rad(el));
        up = transformMat4(create$2(), [0, 1, 0], m);
      }
      invert(this.matrix, lookAt(create$1(), this.position, this.focalPoint, up));
      this._getAxes();
      this._getDistance();
      this._getAngles();
      this.triggerUpdate();
      return this;
    };
    _proto.getDistance = function getDistance() {
      return this.distance;
    };
    _proto.getDistanceVector = function getDistanceVector() {
      return this.distanceVector;
    }
    /**
     * Moves the camera towards/from the focal point.
     */;
    _proto.setDistance = function setDistance(d) {
      if (this.distance === d || d < 0) {
        return this;
      }
      this.distance = d;
      if (this.distance < MIN_DISTANCE) {
        this.distance = MIN_DISTANCE;
      }
      this.dollyingStep = this.distance / 100;
      var pos = create$2();
      d = this.distance;
      var n = this.forward;
      var f = this.focalPoint;
      pos[0] = d * n[0] + f[0];
      pos[1] = d * n[1] + f[1];
      pos[2] = d * n[2] + f[2];
      this._setPosition(pos);
      this.triggerUpdate();
      return this;
    };
    _proto.setMaxDistance = function setMaxDistance(d) {
      this.maxDistance = d;
      return this;
    };
    _proto.setMinDistance = function setMinDistance(d) {
      this.minDistance = d;
      return this;
    }
    /**
     * 设置相机方位角，不同相机模式下需要重新计算相机位置或者是视点位置
     * the azimuth in degrees
     */;
    _proto.setAzimuth = function setAzimuth(az) {
      this.azimuth = getAngle$1(az);
      this.computeMatrix();
      this._getAxes();
      if (this.type === CameraType.ORBITING || this.type === CameraType.EXPLORING) {
        this._getPosition();
      } else if (this.type === CameraType.TRACKING) {
        this._getFocalPoint();
      }
      this.triggerUpdate();
      return this;
    };
    _proto.getAzimuth = function getAzimuth() {
      return this.azimuth;
    }
    /**
     * 设置相机方位角，不同相机模式下需要重新计算相机位置或者是视点位置
     */;
    _proto.setElevation = function setElevation(el) {
      this.elevation = getAngle$1(el);
      this.computeMatrix();
      this._getAxes();
      if (this.type === CameraType.ORBITING || this.type === CameraType.EXPLORING) {
        this._getPosition();
      } else if (this.type === CameraType.TRACKING) {
        this._getFocalPoint();
      }
      this.triggerUpdate();
      return this;
    };
    _proto.getElevation = function getElevation() {
      return this.elevation;
    }
    /**
     * 设置相机方位角，不同相机模式下需要重新计算相机位置或者是视点位置
     */;
    _proto.setRoll = function setRoll(angle) {
      this.roll = getAngle$1(angle);
      this.computeMatrix();
      this._getAxes();
      if (this.type === CameraType.ORBITING || this.type === CameraType.EXPLORING) {
        this._getPosition();
      } else if (this.type === CameraType.TRACKING) {
        this._getFocalPoint();
      }
      this.triggerUpdate();
      return this;
    };
    _proto.getRoll = function getRoll() {
      return this.roll;
    }
    /**
     * 根据相机矩阵重新计算各种相机参数
     */;
    _proto._update = function _update() {
      this._getAxes();
      this._getPosition();
      this._getDistance();
      this._getAngles();
      this._getOrthoMatrix();
      this.triggerUpdate();
    }
    /**
     * 计算相机矩阵
     */;
    _proto.computeMatrix = function computeMatrix() {
      // 使用四元数描述 3D 旋转
      // @see https://xiaoiver.github.io/coding/2018/12/28/Camera-%E8%AE%BE%E8%AE%A1-%E4%B8%80.html
      var rotZ = setAxisAngle(create$4(), [0, 0, 1], deg2rad(this.roll));
      identity(this.matrix);
      // only consider HCS for EXPLORING and ORBITING cameras
      var rotX = setAxisAngle(create$4(), [1, 0, 0], deg2rad((this.rotateWorld && this.type !== CameraType.TRACKING || this.type === CameraType.TRACKING ? 1 : -1) * this.elevation));
      var rotY = setAxisAngle(create$4(), [0, 1, 0], deg2rad((this.rotateWorld && this.type !== CameraType.TRACKING || this.type === CameraType.TRACKING ? 1 : -1) * this.azimuth));
      var rotQ = multiply$2(create$4(), rotY, rotX);
      rotQ = multiply$2(create$4(), rotQ, rotZ);
      var rotMatrix = fromQuat(create$1(), rotQ);
      if (this.type === CameraType.ORBITING || this.type === CameraType.EXPLORING) {
        translate(this.matrix, this.matrix, this.focalPoint);
        multiply(this.matrix, this.matrix, rotMatrix);
        translate(this.matrix, this.matrix, [0, 0, this.distance]);
      } else if (this.type === CameraType.TRACKING) {
        translate(this.matrix, this.matrix, this.position);
        multiply(this.matrix, this.matrix, rotMatrix);
      }
    }
    /**
     * Sets the camera position in the camera matrix
     */;
    _proto._setPosition = function _setPosition(x, y, z) {
      this.position = createVec3(x, y, z);
      var m = this.matrix;
      m[12] = this.position[0];
      m[13] = this.position[1];
      m[14] = this.position[2];
      m[15] = 1;
      this._getOrthoMatrix();
    }
    /**
     * Recalculates axes based on the current matrix
     */;
    _proto._getAxes = function _getAxes() {
      copy$1(this.right, createVec3(transformMat4$1(create$3(), [1, 0, 0, 0], this.matrix)));
      copy$1(this.up, createVec3(transformMat4$1(create$3(), [0, 1, 0, 0], this.matrix)));
      copy$1(this.forward, createVec3(transformMat4$1(create$3(), [0, 0, 1, 0], this.matrix)));
      normalize(this.right, this.right);
      normalize(this.up, this.up);
      normalize(this.forward, this.forward);
    }
    /**
     * Recalculates euler angles based on the current state
     */;
    _proto._getAngles = function _getAngles() {
      // Recalculates angles
      var x = this.distanceVector[0];
      var y = this.distanceVector[1];
      var z = this.distanceVector[2];
      var r = length(this.distanceVector);
      // FAST FAIL: If there is no distance we cannot compute angles
      if (r === 0) {
        this.elevation = 0;
        this.azimuth = 0;
        return;
      }
      if (this.type === CameraType.TRACKING) {
        this.elevation = rad2deg(Math.asin(y / r));
        this.azimuth = rad2deg(Math.atan2(-x, -z));
      } else {
        if (this.rotateWorld) {
          this.elevation = rad2deg(Math.asin(y / r));
          this.azimuth = rad2deg(Math.atan2(-x, -z));
        } else {
          this.elevation = -rad2deg(Math.asin(y / r));
          this.azimuth = -rad2deg(Math.atan2(-x, -z));
        }
      }
    }
    /**
     * 重新计算相机位置，只有 ORBITING 模式相机位置才会发生变化
     */;
    _proto._getPosition = function _getPosition() {
      copy$1(this.position, createVec3(transformMat4$1(create$3(), [0, 0, 0, 1], this.matrix)));
      // 相机位置变化，需要重新计算视距
      this._getDistance();
    }
    /**
     * 重新计算视点，只有 TRACKING 模式视点才会发生变化
     */;
    _proto._getFocalPoint = function _getFocalPoint() {
      transformMat3(this.distanceVector, [0, 0, -this.distance], fromMat4(create(), this.matrix));
      add$1(this.focalPoint, this.position, this.distanceVector);
      // 视点变化，需要重新计算视距
      this._getDistance();
    }
    /**
     * 重新计算视距
     */;
    _proto._getDistance = function _getDistance() {
      this.distanceVector = subtract$1(create$2(), this.focalPoint, this.position);
      this.distance = length(this.distanceVector);
      this.dollyingStep = this.distance / 100;
    };
    _proto._getOrthoMatrix = function _getOrthoMatrix() {
      if (this.projectionMode !== CameraProjectionMode.ORTHOGRAPHIC) {
        return;
      }
      var position = this.position;
      var rotZ = setAxisAngle(create$4(), [0, 0, 1], -this.roll * Math.PI / 180);
      fromRotationTranslationScaleOrigin(this.orthoMatrix, rotZ, fromValues$2((this.rright - this.left) / 2 - position[0], (this.top - this.bottom) / 2 - position[1], 0), fromValues$2(this.zoom, this.zoom, 1), position);
    };
    _proto.triggerUpdate = function triggerUpdate() {
      if (this.enableUpdate) {
        // update frustum
        var viewMatrix = this.getViewTransform();
        var vpMatrix = multiply(create$1(), this.getPerspective(), viewMatrix);
        this.getFrustum().extractFromVPMatrix(vpMatrix);
        this.eventEmitter.emit(CameraEvent.UPDATED);
      }
    };
    _proto.rotate = function rotate(azimuth, elevation, roll) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.pan = function pan(tx, ty) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.dolly = function dolly(value) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.createLandmark = function createLandmark(name, params) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.gotoLandmark = function gotoLandmark(name, options) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    return Camera;
  }();

  var CircleUpdater = /*#__PURE__*/function () {
    function CircleUpdater() {}
    var _proto = CircleUpdater.prototype;
    _proto.update = function update(parsedStyle, object) {
      var r = parsedStyle.r;
      var width = r * 2;
      var height = r * 2;
      return {
        width: width,
        height: height
      };
    };
    return CircleUpdater;
  }();

  var EllipseUpdater = /*#__PURE__*/function () {
    function EllipseUpdater() {}
    var _proto = EllipseUpdater.prototype;
    _proto.update = function update(parsedStyle, object) {
      var rx = parsedStyle.rx,
        ry = parsedStyle.ry;
      var width = rx * 2;
      var height = ry * 2;
      return {
        width: width,
        height: height
      };
    };
    return EllipseUpdater;
  }();

  var LineUpdater = /*#__PURE__*/function () {
    function LineUpdater() {}
    var _proto = LineUpdater.prototype;
    _proto.update = function update(parsedStyle) {
      var x1 = parsedStyle.x1,
        y1 = parsedStyle.y1,
        x2 = parsedStyle.x2,
        y2 = parsedStyle.y2;
      var minX = Math.min(x1, x2);
      var maxX = Math.max(x1, x2);
      var minY = Math.min(y1, y2);
      var maxY = Math.max(y1, y2);
      var width = maxX - minX;
      var height = maxY - minY;
      return {
        width: width,
        height: height
      };
    };
    return LineUpdater;
  }();

  var PathUpdater = /*#__PURE__*/function () {
    function PathUpdater() {}
    var _proto = PathUpdater.prototype;
    _proto.update = function update(parsedStyle) {
      var path = parsedStyle.path;
      var _path$rect = path.rect,
        width = _path$rect.width,
        height = _path$rect.height;
      return {
        width: width,
        height: height
      };
    };
    return PathUpdater;
  }();

  var PolylineUpdater = /*#__PURE__*/function () {
    function PolylineUpdater() {}
    var _proto = PolylineUpdater.prototype;
    _proto.update = function update(parsedStyle) {
      var points = parsedStyle.points.points;
      // FIXME: account for miter lineJoin
      var minX = Math.min.apply(Math, points.map(function (point) {
        return point[0];
      }));
      var maxX = Math.max.apply(Math, points.map(function (point) {
        return point[0];
      }));
      var minY = Math.min.apply(Math, points.map(function (point) {
        return point[1];
      }));
      var maxY = Math.max.apply(Math, points.map(function (point) {
        return point[1];
      }));
      var width = maxX - minX;
      var height = maxY - minY;
      return {
        width: width,
        height: height
      };
    };
    return PolylineUpdater;
  }();

  var RectUpdater = /*#__PURE__*/function () {
    function RectUpdater() {}
    var _proto = RectUpdater.prototype;
    _proto.update = function update(parsedStyle, object) {
      var img = parsedStyle.img,
        _parsedStyle$width = parsedStyle.width,
        width = _parsedStyle$width === void 0 ? 0 : _parsedStyle$width,
        _parsedStyle$height = parsedStyle.height,
        height = _parsedStyle$height === void 0 ? 0 : _parsedStyle$height;
      var contentWidth = width;
      var contentHeight = height;
      // resize with HTMLImageElement's size
      if (img && !isString(img)) {
        if (!contentWidth) {
          contentWidth = img.width;
          parsedStyle.width = contentWidth;
        }
        if (!contentHeight) {
          contentHeight = img.height;
          parsedStyle.height = contentHeight;
        }
      }
      return {
        width: contentWidth,
        height: contentHeight
      };
    };
    return RectUpdater;
  }();

  var TextUpdater = /*#__PURE__*/function () {
    function TextUpdater(globalRuntime) {
      this.globalRuntime = void 0;
      this.globalRuntime = globalRuntime;
    }
    var _proto = TextUpdater.prototype;
    _proto.isReadyToMeasure = function isReadyToMeasure(parsedStyle, object) {
      var text = parsedStyle.text,
        textAlign = parsedStyle.textAlign,
        textBaseline = parsedStyle.textBaseline,
        fontSize = parsedStyle.fontSize,
        fontStyle = parsedStyle.fontStyle,
        fontWeight = parsedStyle.fontWeight,
        fontVariant = parsedStyle.fontVariant,
        lineWidth = parsedStyle.lineWidth;
      return text && fontSize && fontStyle && fontWeight && fontVariant && textAlign && textBaseline && !isNil(lineWidth);
    };
    _proto.update = function update(parsedStyle, object) {
      var _object$ownerDocument, _object$ownerDocument2;
      var text = parsedStyle.text,
        textAlign = parsedStyle.textAlign,
        lineWidth = parsedStyle.lineWidth,
        textBaseline = parsedStyle.textBaseline,
        dx = parsedStyle.dx,
        dy = parsedStyle.dy;
      var _ref = (object === null || object === void 0 ? void 0 : (_object$ownerDocument = object.ownerDocument) === null || _object$ownerDocument === void 0 ? void 0 : (_object$ownerDocument2 = _object$ownerDocument.defaultView) === null || _object$ownerDocument2 === void 0 ? void 0 : _object$ownerDocument2.getConfig()) || {},
        offscreenCanvas = _ref.offscreenCanvas;
      if (!this.isReadyToMeasure(parsedStyle, object)) {
        parsedStyle.metrics = {
          font: '',
          width: 0,
          height: 0,
          lines: [],
          lineWidths: [],
          lineHeight: 0,
          maxLineWidth: 0,
          fontProperties: {
            ascent: 0,
            descent: 0,
            fontSize: 0
          },
          lineMetrics: []
        };
        return {
          width: 0,
          height: 0,
          x: 0,
          y: 0,
          offsetX: 0,
          offsetY: 0
        };
      }
      var metrics = this.globalRuntime.textService.measureText(text, parsedStyle, offscreenCanvas);
      parsedStyle.metrics = metrics;
      var width = metrics.width,
        height = metrics.height,
        lineHeight = metrics.lineHeight,
        fontProperties = metrics.fontProperties;
      // anchor is left-top by default
      var halfExtents = [width / 2, height / 2, 0];
      // default 'left'
      var anchor = [0, 1];
      var lineXOffset = 0;
      if (textAlign === 'center' || textAlign === 'middle') {
        lineXOffset = lineWidth / 2;
        anchor = [0.5, 1];
      } else if (textAlign === 'right' || textAlign === 'end') {
        lineXOffset = lineWidth;
        anchor = [1, 1];
      }
      var lineYOffset = 0;
      if (textBaseline === 'middle') {
        // eslint-disable-next-line prefer-destructuring
        lineYOffset = halfExtents[1];
      } else if (textBaseline === 'top' || textBaseline === 'hanging') {
        lineYOffset = halfExtents[1] * 2;
      } else if (textBaseline === 'alphabetic') {
        lineYOffset = lineHeight - fontProperties.ascent;
      } else if (textBaseline === 'bottom' || textBaseline === 'ideographic') {
        lineYOffset = 0;
      }
      // TODO: ideographic & bottom
      if (dx) {
        lineXOffset += dx;
      }
      if (dy) {
        lineYOffset += dy;
      }
      // update anchor
      parsedStyle.anchor = [anchor[0], anchor[1], 0];
      return {
        width: halfExtents[0] * 2,
        height: halfExtents[1] * 2,
        offsetX: lineXOffset,
        offsetY: lineYOffset
      };
    };
    return TextUpdater;
  }();

  var PROPAGATION_LIMIT = 2048;
  var EventService = /*#__PURE__*/function () {
    function EventService(globalRuntime, context) {
      var _this = this;
      this.globalRuntime = void 0;
      this.context = void 0;
      this.rootTarget = void 0;
      this.emitter = new eventemitter3();
      this.cursor = 'default';
      this.mappingTable = {};
      this.mappingState = {
        trackingData: {}
      };
      this.eventPool = new Map();
      this.pickHandler = void 0;
      this.tmpMatrix = create$1();
      this.tmpVec3 = create$2();
      this.onPointerDown = /*#__PURE__*/function () {
        var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(from) {
          var e, isRightButton, trackingData;
          return _regeneratorRuntime().wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent) {
                    _context.next = 2;
                    break;
                  }
                  return _context.abrupt("return");
                case 2:
                  _context.next = 4;
                  return _this.createPointerEvent(from);
                case 4:
                  e = _context.sent;
                  _this.dispatchEvent(e, 'pointerdown');
                  if (e.pointerType === 'touch') {
                    _this.dispatchEvent(e, 'touchstart');
                  } else if (e.pointerType === 'mouse' || e.pointerType === 'pen') {
                    isRightButton = e.button === 2;
                    _this.dispatchEvent(e, isRightButton ? 'rightdown' : 'mousedown');
                  }
                  trackingData = _this.trackingData(from.pointerId);
                  trackingData.pressTargetsByButton[from.button] = e.composedPath();
                  _this.freeEvent(e);
                case 10:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));
        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }();
      this.onPointerUp = /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee2(from) {
          var now, e, isRightButton, trackingData, pressTarget, clickTarget, currentTarget, _isRightButton, _e$detail, clickEvent, clickHistory;
          return _regeneratorRuntime().wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent) {
                    _context2.next = 2;
                    break;
                  }
                  return _context2.abrupt("return");
                case 2:
                  now = performance.now();
                  _context2.next = 5;
                  return _this.createPointerEvent(from);
                case 5:
                  e = _context2.sent;
                  _this.dispatchEvent(e, 'pointerup');
                  if (e.pointerType === 'touch') {
                    _this.dispatchEvent(e, 'touchend');
                  } else if (e.pointerType === 'mouse' || e.pointerType === 'pen') {
                    isRightButton = e.button === 2;
                    _this.dispatchEvent(e, isRightButton ? 'rightup' : 'mouseup');
                  }
                  trackingData = _this.trackingData(from.pointerId);
                  pressTarget = _this.findMountedTarget(trackingData.pressTargetsByButton[from.button]);
                  clickTarget = pressTarget; // pointerupoutside only bubbles. It only bubbles upto the parent that doesn't contain
                  // the pointerup location.
                  if (pressTarget && !e.composedPath().includes(pressTarget)) {
                    currentTarget = pressTarget;
                    while (currentTarget && !e.composedPath().includes(currentTarget)) {
                      e.currentTarget = currentTarget;
                      _this.notifyTarget(e, 'pointerupoutside');
                      if (e.pointerType === 'touch') {
                        _this.notifyTarget(e, 'touchendoutside');
                      } else if (e.pointerType === 'mouse' || e.pointerType === 'pen') {
                        _isRightButton = e.button === 2;
                        _this.notifyTarget(e, _isRightButton ? 'rightupoutside' : 'mouseupoutside');
                      }
                      if (Node$1.isNode(currentTarget)) {
                        currentTarget = currentTarget.parentNode;
                      }
                    }
                    delete trackingData.pressTargetsByButton[from.button];
                    // currentTarget is the most specific ancestor holding both the pointerdown and pointerup
                    // targets. That is - it's our click target!
                    clickTarget = currentTarget;
                  }
                  if (clickTarget) {
                    clickEvent = _this.clonePointerEvent(e, 'click');
                    clickEvent.target = clickTarget;
                    clickEvent.path = [];
                    if (!trackingData.clicksByButton[from.button]) {
                      trackingData.clicksByButton[from.button] = {
                        clickCount: 0,
                        target: clickEvent.target,
                        timeStamp: now
                      };
                    }
                    clickHistory = trackingData.clicksByButton[from.button];
                    if (clickHistory.target === clickEvent.target && now - clickHistory.timeStamp < 200) {
                      ++clickHistory.clickCount;
                    } else {
                      clickHistory.clickCount = 1;
                    }
                    clickHistory.target = clickEvent.target;
                    clickHistory.timeStamp = now;
                    clickEvent.detail = clickHistory.clickCount;
                    // @see https://github.com/antvis/G/issues/1091
                    if (!((_e$detail = e.detail) === null || _e$detail === void 0 ? void 0 : _e$detail.preventClick)) {
                      if (clickEvent.pointerType === 'mouse' || clickEvent.pointerType === 'touch') {
                        _this.dispatchEvent(clickEvent, 'click');
                      }
                      _this.dispatchEvent(clickEvent, 'pointertap');
                    }
                    _this.freeEvent(clickEvent);
                  }
                  _this.freeEvent(e);
                case 14:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2);
        }));
        return function (_x2) {
          return _ref2.apply(this, arguments);
        };
      }();
      this.onPointerMove = /*#__PURE__*/function () {
        var _ref3 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee3(from) {
          var e, isMouse, trackingData, outTarget, outType, outEvent, leaveEvent, overType, overEvent, overTargetAncestor, didPointerEnter, enterEvent;
          return _regeneratorRuntime().wrap(function _callee3$(_context3) {
            while (1) {
              switch (_context3.prev = _context3.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent) {
                    _context3.next = 2;
                    break;
                  }
                  return _context3.abrupt("return");
                case 2:
                  _context3.next = 4;
                  return _this.createPointerEvent(from);
                case 4:
                  e = _context3.sent;
                  isMouse = e.pointerType === 'mouse' || e.pointerType === 'pen';
                  trackingData = _this.trackingData(from.pointerId);
                  outTarget = _this.findMountedTarget(trackingData.overTargets); // First pointerout/pointerleave
                  if (!(trackingData.overTargets && outTarget !== e.target)) {
                    _context3.next = 23;
                    break;
                  }
                  // pointerout always occurs on the overTarget when the pointer hovers over another element.
                  outType = from.type === 'mousemove' ? 'mouseout' : 'pointerout';
                  _context3.next = 12;
                  return _this.createPointerEvent(from, outType, outTarget || undefined);
                case 12:
                  outEvent = _context3.sent;
                  _this.dispatchEvent(outEvent, 'pointerout');
                  if (isMouse) _this.dispatchEvent(outEvent, 'mouseout');
                  // If the pointer exits overTarget and its descendants, then a pointerleave event is also fired. This event
                  // is dispatched to all ancestors that no longer capture the pointer.
                  if (e.composedPath().includes(outTarget)) {
                    _context3.next = 22;
                    break;
                  }
                  _context3.next = 18;
                  return _this.createPointerEvent(from, 'pointerleave', outTarget || undefined);
                case 18:
                  leaveEvent = _context3.sent;
                  leaveEvent.eventPhase = leaveEvent.AT_TARGET;
                  while (leaveEvent.target && !e.composedPath().includes(leaveEvent.target)) {
                    leaveEvent.currentTarget = leaveEvent.target;
                    _this.notifyTarget(leaveEvent);
                    if (isMouse) {
                      _this.notifyTarget(leaveEvent, 'mouseleave');
                    }
                    if (Node$1.isNode(leaveEvent.target)) {
                      leaveEvent.target = leaveEvent.target.parentNode;
                    }
                  }
                  _this.freeEvent(leaveEvent);
                case 22:
                  _this.freeEvent(outEvent);
                case 23:
                  if (!(outTarget !== e.target)) {
                    _context3.next = 38;
                    break;
                  }
                  // pointerover always occurs on the new overTarget
                  overType = from.type === 'mousemove' ? 'mouseover' : 'pointerover';
                  overEvent = _this.clonePointerEvent(e, overType); // clone faster
                  _this.dispatchEvent(overEvent, 'pointerover');
                  if (isMouse) _this.dispatchEvent(overEvent, 'mouseover');
                  // Probe whether the newly hovered Node is an ancestor of the original overTarget.
                  overTargetAncestor = outTarget && Node$1.isNode(outTarget) && outTarget.parentNode;
                case 29:
                  if (!(overTargetAncestor && overTargetAncestor !== (Node$1.isNode(_this.rootTarget) && _this.rootTarget.parentNode))) {
                    _context3.next = 35;
                    break;
                  }
                  if (!(overTargetAncestor === e.target)) {
                    _context3.next = 32;
                    break;
                  }
                  return _context3.abrupt("break", 35);
                case 32:
                  overTargetAncestor = overTargetAncestor.parentNode;
                  _context3.next = 29;
                  break;
                case 35:
                  // The pointer has entered a non-ancestor of the original overTarget. This means we need a pointerentered
                  // event.
                  didPointerEnter = !overTargetAncestor || overTargetAncestor === (Node$1.isNode(_this.rootTarget) && _this.rootTarget.parentNode);
                  if (didPointerEnter) {
                    enterEvent = _this.clonePointerEvent(e, 'pointerenter');
                    enterEvent.eventPhase = enterEvent.AT_TARGET;
                    while (enterEvent.target && enterEvent.target !== outTarget && enterEvent.target !== (Node$1.isNode(_this.rootTarget) && _this.rootTarget.parentNode)) {
                      enterEvent.currentTarget = enterEvent.target;
                      _this.notifyTarget(enterEvent);
                      if (isMouse) _this.notifyTarget(enterEvent, 'mouseenter');
                      if (Node$1.isNode(enterEvent.target)) {
                        enterEvent.target = enterEvent.target.parentNode;
                      }
                    }
                    _this.freeEvent(enterEvent);
                  }
                  _this.freeEvent(overEvent);
                case 38:
                  // Then pointermove
                  _this.dispatchEvent(e, 'pointermove');
                  if (e.pointerType === 'touch') _this.dispatchEvent(e, 'touchmove');
                  if (isMouse) {
                    _this.dispatchEvent(e, 'mousemove');
                    _this.cursor = _this.getCursor(e.target);
                  }
                  trackingData.overTargets = e.composedPath();
                  _this.freeEvent(e);
                case 43:
                case "end":
                  return _context3.stop();
              }
            }
          }, _callee3);
        }));
        return function (_x3) {
          return _ref3.apply(this, arguments);
        };
      }();
      this.onPointerOut = /*#__PURE__*/function () {
        var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee4(from) {
          var trackingData, isMouse, outTarget, outEvent, leaveEvent;
          return _regeneratorRuntime().wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent) {
                    _context4.next = 2;
                    break;
                  }
                  return _context4.abrupt("return");
                case 2:
                  trackingData = _this.trackingData(from.pointerId);
                  if (!trackingData.overTargets) {
                    _context4.next = 19;
                    break;
                  }
                  isMouse = from.pointerType === 'mouse' || from.pointerType === 'pen';
                  outTarget = _this.findMountedTarget(trackingData.overTargets); // pointerout first
                  _context4.next = 8;
                  return _this.createPointerEvent(from, 'pointerout', outTarget || undefined);
                case 8:
                  outEvent = _context4.sent;
                  _this.dispatchEvent(outEvent);
                  if (isMouse) _this.dispatchEvent(outEvent, 'mouseout');
                  // pointerleave(s) are also dispatched b/c the pointer must've left rootTarget and its descendants to
                  // get an upstream pointerout event (upstream events do not know rootTarget has descendants).
                  _context4.next = 13;
                  return _this.createPointerEvent(from, 'pointerleave', outTarget || undefined);
                case 13:
                  leaveEvent = _context4.sent;
                  leaveEvent.eventPhase = leaveEvent.AT_TARGET;
                  while (leaveEvent.target && leaveEvent.target !== (Node$1.isNode(_this.rootTarget) && _this.rootTarget.parentNode)) {
                    leaveEvent.currentTarget = leaveEvent.target;
                    _this.notifyTarget(leaveEvent);
                    if (isMouse) {
                      _this.notifyTarget(leaveEvent, 'mouseleave');
                    }
                    if (Node$1.isNode(leaveEvent.target)) {
                      leaveEvent.target = leaveEvent.target.parentNode;
                    }
                  }
                  trackingData.overTargets = null;
                  _this.freeEvent(outEvent);
                  _this.freeEvent(leaveEvent);
                case 19:
                  _this.cursor = null;
                case 20:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4);
        }));
        return function (_x4) {
          return _ref4.apply(this, arguments);
        };
      }();
      this.onPointerOver = /*#__PURE__*/function () {
        var _ref5 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee5(from) {
          var trackingData, e, isMouse, enterEvent;
          return _regeneratorRuntime().wrap(function _callee5$(_context5) {
            while (1) {
              switch (_context5.prev = _context5.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent) {
                    _context5.next = 2;
                    break;
                  }
                  return _context5.abrupt("return");
                case 2:
                  trackingData = _this.trackingData(from.pointerId);
                  _context5.next = 5;
                  return _this.createPointerEvent(from);
                case 5:
                  e = _context5.sent;
                  isMouse = e.pointerType === 'mouse' || e.pointerType === 'pen';
                  _this.dispatchEvent(e, 'pointerover');
                  if (isMouse) _this.dispatchEvent(e, 'mouseover');
                  if (e.pointerType === 'mouse') _this.cursor = _this.getCursor(e.target);
                  // pointerenter events must be fired since the pointer entered from upstream.
                  enterEvent = _this.clonePointerEvent(e, 'pointerenter');
                  enterEvent.eventPhase = enterEvent.AT_TARGET;
                  while (enterEvent.target && enterEvent.target !== (Node$1.isNode(_this.rootTarget) && _this.rootTarget.parentNode)) {
                    enterEvent.currentTarget = enterEvent.target;
                    _this.notifyTarget(enterEvent);
                    if (isMouse) {
                      // mouseenter should not bubble
                      // @see https://developer.mozilla.org/en-US/docs/Web/API/Element/mouseenter_event#usage_notes
                      _this.notifyTarget(enterEvent, 'mouseenter');
                    }
                    if (Node$1.isNode(enterEvent.target)) {
                      enterEvent.target = enterEvent.target.parentNode;
                    }
                  }
                  trackingData.overTargets = e.composedPath();
                  _this.freeEvent(e);
                  _this.freeEvent(enterEvent);
                case 16:
                case "end":
                  return _context5.stop();
              }
            }
          }, _callee5);
        }));
        return function (_x5) {
          return _ref5.apply(this, arguments);
        };
      }();
      this.onPointerUpOutside = /*#__PURE__*/function () {
        var _ref6 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee6(from) {
          var trackingData, pressTarget, e, currentTarget;
          return _regeneratorRuntime().wrap(function _callee6$(_context6) {
            while (1) {
              switch (_context6.prev = _context6.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent) {
                    _context6.next = 2;
                    break;
                  }
                  return _context6.abrupt("return");
                case 2:
                  trackingData = _this.trackingData(from.pointerId);
                  pressTarget = _this.findMountedTarget(trackingData.pressTargetsByButton[from.button]);
                  _context6.next = 6;
                  return _this.createPointerEvent(from);
                case 6:
                  e = _context6.sent;
                  if (pressTarget) {
                    currentTarget = pressTarget;
                    while (currentTarget) {
                      e.currentTarget = currentTarget;
                      _this.notifyTarget(e, 'pointerupoutside');
                      if (e.pointerType === 'touch') ; else if (e.pointerType === 'mouse' || e.pointerType === 'pen') {
                        _this.notifyTarget(e, e.button === 2 ? 'rightupoutside' : 'mouseupoutside');
                      }
                      if (Node$1.isNode(currentTarget)) {
                        currentTarget = currentTarget.parentNode;
                      }
                    }
                    delete trackingData.pressTargetsByButton[from.button];
                  }
                  _this.freeEvent(e);
                case 9:
                case "end":
                  return _context6.stop();
              }
            }
          }, _callee6);
        }));
        return function (_x6) {
          return _ref6.apply(this, arguments);
        };
      }();
      this.onWheel = /*#__PURE__*/function () {
        var _ref7 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee7(from) {
          var wheelEvent;
          return _regeneratorRuntime().wrap(function _callee7$(_context7) {
            while (1) {
              switch (_context7.prev = _context7.next) {
                case 0:
                  if (from instanceof FederatedWheelEvent) {
                    _context7.next = 2;
                    break;
                  }
                  return _context7.abrupt("return");
                case 2:
                  _context7.next = 4;
                  return _this.createWheelEvent(from);
                case 4:
                  wheelEvent = _context7.sent;
                  _this.dispatchEvent(wheelEvent);
                  _this.freeEvent(wheelEvent);
                case 7:
                case "end":
                  return _context7.stop();
              }
            }
          }, _callee7);
        }));
        return function (_x7) {
          return _ref7.apply(this, arguments);
        };
      }();
      this.globalRuntime = globalRuntime;
      this.context = context;
    }
    var _proto = EventService.prototype;
    _proto.init = function init() {
      this.rootTarget = this.context.renderingContext.root.parentNode; // document
      this.addEventMapping('pointerdown', this.onPointerDown);
      this.addEventMapping('pointerup', this.onPointerUp);
      this.addEventMapping('pointermove', this.onPointerMove);
      this.addEventMapping('pointerout', this.onPointerOut);
      this.addEventMapping('pointerleave', this.onPointerOut);
      this.addEventMapping('pointerover', this.onPointerOver);
      this.addEventMapping('pointerupoutside', this.onPointerUpOutside);
      this.addEventMapping('wheel', this.onWheel);
    };
    _proto.client2Viewport = function client2Viewport(client) {
      var bbox = this.context.contextService.getBoundingClientRect();
      return new Point(client.x - ((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0), client.y - ((bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0));
    };
    _proto.viewport2Client = function viewport2Client(canvas) {
      var bbox = this.context.contextService.getBoundingClientRect();
      return new Point(canvas.x + ((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0), canvas.y + ((bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0));
    };
    _proto.viewport2Canvas = function viewport2Canvas(_ref8) {
      var x = _ref8.x,
        y = _ref8.y;
      var canvas = this.rootTarget.defaultView;
      var camera = canvas.getCamera();
      var _this$context$config = this.context.config,
        width = _this$context$config.width,
        height = _this$context$config.height;
      var projectionMatrixInverse = camera.getPerspectiveInverse();
      var worldMatrix = camera.getWorldTransform();
      var vpMatrix = multiply(this.tmpMatrix, worldMatrix, projectionMatrixInverse);
      var viewport = set$1(this.tmpVec3, x / width * 2 - 1, (1 - y / height) * 2 - 1, 0);
      transformMat4(viewport, viewport, vpMatrix);
      return new Point(viewport[0], viewport[1]);
    };
    _proto.canvas2Viewport = function canvas2Viewport(canvasP) {
      var canvas = this.rootTarget.defaultView;
      var camera = canvas.getCamera();
      // World -> Clip
      var projectionMatrix = camera.getPerspective();
      var viewMatrix = camera.getViewTransform();
      var vpMatrix = multiply(this.tmpMatrix, projectionMatrix, viewMatrix);
      var clip = set$1(this.tmpVec3, canvasP.x, canvasP.y, 0);
      transformMat4(this.tmpVec3, this.tmpVec3, vpMatrix);
      // Clip -> NDC -> Viewport, flip Y
      var _this$context$config2 = this.context.config,
        width = _this$context$config2.width,
        height = _this$context$config2.height;
      return new Point((clip[0] + 1) / 2 * width, (1 - (clip[1] + 1) / 2) * height);
    };
    _proto.setPickHandler = function setPickHandler(pickHandler) {
      this.pickHandler = pickHandler;
    };
    _proto.addEventMapping = function addEventMapping(type, fn) {
      if (!this.mappingTable[type]) {
        this.mappingTable[type] = [];
      }
      this.mappingTable[type].push({
        fn: fn,
        priority: 0
      });
      this.mappingTable[type].sort(function (a, b) {
        return a.priority - b.priority;
      });
    };
    _proto.mapEvent = function mapEvent(e) {
      if (!this.rootTarget) {
        return;
      }
      var mappers = this.mappingTable[e.type];
      if (mappers) {
        for (var i = 0, j = mappers.length; i < j; i++) {
          mappers[i].fn(e);
        }
      } else {
        console.warn("[EventService]: Event mapping not defined for " + e.type);
      }
    };
    _proto.dispatchEvent = function dispatchEvent(e, type, skipPropagate) {
      // Canvas should skip
      if (!skipPropagate) {
        e.propagationStopped = false;
        e.propagationImmediatelyStopped = false;
        this.propagate(e, type);
      } else {
        // target phase
        e.eventPhase = e.AT_TARGET;
        var canvas = this.rootTarget.defaultView || null;
        e.currentTarget = canvas;
        this.notifyListeners(e, type);
      }
      this.emitter.emit(type || e.type, e);
    };
    _proto.propagate = function propagate(e, type) {
      if (!e.target) {
        return;
      }
      // [target, parent, root, Canvas]
      var composedPath = e.composedPath();
      // event flow: capture -> target -> bubbling
      // capture phase
      e.eventPhase = e.CAPTURING_PHASE;
      for (var i = composedPath.length - 1; i >= 1; i--) {
        e.currentTarget = composedPath[i];
        this.notifyTarget(e, type);
        if (e.propagationStopped || e.propagationImmediatelyStopped) return;
      }
      // target phase
      e.eventPhase = e.AT_TARGET;
      e.currentTarget = e.target;
      this.notifyTarget(e, type);
      if (e.propagationStopped || e.propagationImmediatelyStopped) return;
      // find current target in composed path
      var index = composedPath.indexOf(e.currentTarget);
      // bubbling phase
      e.eventPhase = e.BUBBLING_PHASE;
      for (var _i = index + 1; _i < composedPath.length; _i++) {
        e.currentTarget = composedPath[_i];
        this.notifyTarget(e, type);
        if (e.propagationStopped || e.propagationImmediatelyStopped) return;
      }
    };
    _proto.propagationPath = function propagationPath(target) {
      var propagationPath = [target];
      var canvas = this.rootTarget.defaultView || null;
      if (canvas && canvas === target) {
        propagationPath.unshift(canvas.document);
        return propagationPath;
      }
      for (var i = 0; i < PROPAGATION_LIMIT && target !== this.rootTarget; i++) {
        // if (Node.isNode(target) && !target.parentNode) {
        //   throw new Error('Cannot find propagation path to disconnected target');
        // }
        if (Node$1.isNode(target) && target.parentNode) {
          // [target, parent, parent, root]
          propagationPath.push(target.parentNode);
          target = target.parentNode;
        }
      }
      if (canvas) {
        // @ts-ignore
        propagationPath.push(canvas);
      }
      return propagationPath;
    };
    _proto.hitTest = /*#__PURE__*/function () {
      var _hitTest = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee8(position) {
        var viewportX, viewportY, _this$context$config3, width, height;
        return _regeneratorRuntime().wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                viewportX = position.viewportX, viewportY = position.viewportY;
                _this$context$config3 = this.context.config, width = _this$context$config3.width, height = _this$context$config3.height; // outside canvas
                if (!(viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height)) {
                  _context8.next = 4;
                  break;
                }
                return _context8.abrupt("return", null);
              case 4:
                _context8.next = 6;
                return this.pickHandler(position);
              case 6:
                _context8.t1 = _context8.sent;
                if (_context8.t1) {
                  _context8.next = 9;
                  break;
                }
                _context8.t1 = this.rootTarget;
              case 9:
                _context8.t0 = _context8.t1;
                if (_context8.t0) {
                  _context8.next = 12;
                  break;
                }
                _context8.t0 =
                // return Document
                null;
              case 12:
                return _context8.abrupt("return", _context8.t0);
              case 13:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));
      function hitTest(_x8) {
        return _hitTest.apply(this, arguments);
      }
      return hitTest;
    }()
    /**
     * whether the native event trigger came from Canvas,
     * should account for HTML shape
     */
    ;
    _proto.isNativeEventFromCanvas = function isNativeEventFromCanvas(event) {
      var _event$nativeEvent;
      var $el = this.context.contextService.getDomElement();
      var target = (_event$nativeEvent = event.nativeEvent) === null || _event$nativeEvent === void 0 ? void 0 : _event$nativeEvent.target;
      if (target) {
        // from <canvas>
        if (target === $el) {
          return true;
        }
        // from <svg>
        if ($el && $el.contains) {
          return $el.contains(target);
        }
      }
      if (event.nativeEvent.composedPath) {
        return event.nativeEvent.composedPath().indexOf($el) > -1;
      }
      // account for Touch
      return false;
    };
    _proto.getExistedHTML = function getExistedHTML(event) {
      if (event.nativeEvent.composedPath) {
        var htmls = this.globalRuntime.displayObjectPool.getHTMLs();
        for (var _iterator = _createForOfIteratorHelperLoose(htmls), _step; !(_step = _iterator()).done;) {
          var html = _step.value;
          if (event.nativeEvent.composedPath().indexOf(html) > -1) {
            return html;
          }
        }
      }
      return null;
    };
    _proto.pickTarget = /*#__PURE__*/function () {
      var _pickTarget = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee9(event) {
        return _regeneratorRuntime().wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                return _context9.abrupt("return", this.hitTest({
                  clientX: event.clientX,
                  clientY: event.clientY,
                  viewportX: event.viewportX,
                  viewportY: event.viewportY,
                  x: event.canvasX,
                  y: event.canvasY
                }));
              case 1:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));
      function pickTarget(_x9) {
        return _pickTarget.apply(this, arguments);
      }
      return pickTarget;
    }();
    _proto.createPointerEvent = /*#__PURE__*/function () {
      var _createPointerEvent = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee10(from, type, target) {
        var event, existedHTML;
        return _regeneratorRuntime().wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                event = this.allocateEvent(FederatedPointerEvent);
                this.copyPointerData(from, event);
                this.copyMouseData(from, event);
                this.copyData(from, event);
                event.nativeEvent = from.nativeEvent;
                event.originalEvent = from;
                existedHTML = this.getExistedHTML(event);
                if (!(target !== null && target !== void 0)) {
                  _context10.next = 11;
                  break;
                }
                _context10.t0 = target;
                _context10.next = 20;
                break;
              case 11:
                _context10.t1 = existedHTML;
                if (_context10.t1) {
                  _context10.next = 19;
                  break;
                }
                _context10.t2 = this.isNativeEventFromCanvas(event);
                if (!_context10.t2) {
                  _context10.next = 18;
                  break;
                }
                _context10.next = 17;
                return this.pickTarget(event);
              case 17:
                _context10.t2 = _context10.sent;
              case 18:
                _context10.t1 = _context10.t2;
              case 19:
                _context10.t0 = _context10.t1;
              case 20:
                event.target = _context10.t0;
                if (typeof type === 'string') {
                  event.type = type;
                }
                return _context10.abrupt("return", event);
              case 23:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));
      function createPointerEvent(_x10, _x11, _x12) {
        return _createPointerEvent.apply(this, arguments);
      }
      return createPointerEvent;
    }();
    _proto.createWheelEvent = /*#__PURE__*/function () {
      var _createWheelEvent = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee11(from) {
        var event, existedHTML;
        return _regeneratorRuntime().wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                event = this.allocateEvent(FederatedWheelEvent);
                this.copyWheelData(from, event);
                this.copyMouseData(from, event);
                this.copyData(from, event);
                event.nativeEvent = from.nativeEvent;
                event.originalEvent = from;
                existedHTML = this.getExistedHTML(event);
                _context11.t0 = existedHTML;
                if (_context11.t0) {
                  _context11.next = 15;
                  break;
                }
                _context11.t1 = this.isNativeEventFromCanvas(event);
                if (!_context11.t1) {
                  _context11.next = 14;
                  break;
                }
                _context11.next = 13;
                return this.pickTarget(event);
              case 13:
                _context11.t1 = _context11.sent;
              case 14:
                _context11.t0 = _context11.t1;
              case 15:
                event.target = _context11.t0;
                return _context11.abrupt("return", event);
              case 17:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));
      function createWheelEvent(_x13) {
        return _createWheelEvent.apply(this, arguments);
      }
      return createWheelEvent;
    }();
    _proto.trackingData = function trackingData(id) {
      if (!this.mappingState.trackingData[id]) {
        this.mappingState.trackingData[id] = {
          pressTargetsByButton: {},
          clicksByButton: {},
          overTarget: null
        };
      }
      return this.mappingState.trackingData[id];
    };
    _proto.cloneWheelEvent = function cloneWheelEvent(from) {
      var event = this.allocateEvent(FederatedWheelEvent);
      event.nativeEvent = from.nativeEvent;
      event.originalEvent = from.originalEvent;
      this.copyWheelData(from, event);
      this.copyMouseData(from, event);
      this.copyData(from, event);
      event.target = from.target;
      event.path = from.composedPath().slice();
      event.type = event.type;
      return event;
    };
    _proto.clonePointerEvent = function clonePointerEvent(from, type) {
      var event = this.allocateEvent(FederatedPointerEvent);
      event.nativeEvent = from.nativeEvent;
      event.originalEvent = from.originalEvent;
      this.copyPointerData(from, event);
      this.copyMouseData(from, event);
      this.copyData(from, event);
      event.target = from.target;
      event.path = from.composedPath().slice();
      event.type = type !== null && type !== void 0 ? type : event.type;
      return event;
    };
    _proto.copyPointerData = function copyPointerData(from, to) {
      if (!(from instanceof FederatedPointerEvent && to instanceof FederatedPointerEvent)) return;
      to.pointerId = from.pointerId;
      to.width = from.width;
      to.height = from.height;
      to.isPrimary = from.isPrimary;
      to.pointerType = from.pointerType;
      to.pressure = from.pressure;
      to.tangentialPressure = from.tangentialPressure;
      to.tiltX = from.tiltX;
      to.tiltY = from.tiltY;
      to.twist = from.twist;
    };
    _proto.copyMouseData = function copyMouseData(from, to) {
      if (!(from instanceof FederatedMouseEvent && to instanceof FederatedMouseEvent)) return;
      to.altKey = from.altKey;
      to.button = from.button;
      to.buttons = from.buttons;
      to.ctrlKey = from.ctrlKey;
      to.metaKey = from.metaKey;
      to.shiftKey = from.shiftKey;
      to.client.copyFrom(from.client);
      to.movement.copyFrom(from.movement);
      to.canvas.copyFrom(from.canvas);
      to.screen.copyFrom(from.screen);
      to.global.copyFrom(from.global);
      to.offset.copyFrom(from.offset);
    };
    _proto.copyWheelData = function copyWheelData(from, to) {
      to.deltaMode = from.deltaMode;
      to.deltaX = from.deltaX;
      to.deltaY = from.deltaY;
      to.deltaZ = from.deltaZ;
    };
    _proto.copyData = function copyData(from, to) {
      to.isTrusted = from.isTrusted;
      to.timeStamp = performance.now();
      to.type = from.type;
      to.detail = from.detail;
      to.view = from.view;
      to.page.copyFrom(from.page);
      to.viewport.copyFrom(from.viewport);
    };
    _proto.allocateEvent = function allocateEvent(constructor) {
      if (!this.eventPool.has(constructor)) {
        this.eventPool.set(constructor, []);
      }
      // @ts-ignore
      var event = this.eventPool.get(constructor).pop() || new constructor(this);
      event.eventPhase = event.NONE;
      event.currentTarget = null;
      event.path = [];
      event.target = null;
      return event;
    };
    _proto.freeEvent = function freeEvent(event) {
      if (event.manager !== this) throw new Error('It is illegal to free an event not managed by this EventBoundary!');
      var constructor = event.constructor;
      if (!this.eventPool.has(constructor)) {
        this.eventPool.set(constructor, []);
      }
      // @ts-ignore
      this.eventPool.get(constructor).push(event);
    };
    _proto.notifyTarget = function notifyTarget(e, type) {
      var _type;
      type = (_type = type) !== null && _type !== void 0 ? _type : e.type;
      var key = e.eventPhase === e.CAPTURING_PHASE || e.eventPhase === e.AT_TARGET ? type + "capture" : type;
      this.notifyListeners(e, key);
      if (e.eventPhase === e.AT_TARGET) {
        this.notifyListeners(e, type);
      }
    };
    _proto.notifyListeners = function notifyListeners(e, type) {
      // hack EventEmitter, stops if the `propagationImmediatelyStopped` flag is set
      // @ts-ignore
      var emitter = e.currentTarget.emitter;
      // @ts-ignore
      var listeners = emitter._events[type];
      if (!listeners) return;
      if ('fn' in listeners) {
        if (listeners.once) {
          emitter.removeListener(type, listeners.fn, undefined, true);
        }
        listeners.fn.call(e.currentTarget || listeners.context, e);
        // listeners.fn.call(listeners.context, e);
      } else {
        for (var i = 0; i < listeners.length && !e.propagationImmediatelyStopped; i++) {
          if (listeners[i].once) {
            emitter.removeListener(type, listeners[i].fn, undefined, true);
          }
          listeners[i].fn.call(e.currentTarget || listeners[i].context, e);
          // listeners[i].fn.call(listeners[i].context, e);
        }
      }
    }
    /**
     * some detached nodes may exist in propagation path, need to skip them
     */;
    _proto.findMountedTarget = function findMountedTarget(propagationPath) {
      if (!propagationPath) {
        return null;
      }
      var currentTarget = propagationPath[propagationPath.length - 1];
      for (var i = propagationPath.length - 2; i >= 0; i--) {
        var target = propagationPath[i];
        if (target === this.rootTarget || Node$1.isNode(target) && target.parentNode === currentTarget) {
          currentTarget = propagationPath[i];
        } else {
          break;
        }
      }
      return currentTarget;
    };
    _proto.getCursor = function getCursor(target) {
      var tmp = target;
      while (tmp) {
        var cursor = Element.isElement(tmp) && tmp.getAttribute('cursor');
        if (cursor) {
          return cursor;
        }
        tmp = Node$1.isNode(tmp) && tmp.parentNode;
      }
    };
    return EventService;
  }();

  /**
   * used in following scenes:
   * - g `ctx.measureText`
   * - g-plugin-canvas-picker `ctx.isPointInPath`
   * - g-plugin-device-renderer `ctx.createLinearGradient` and generate texture
   */
  var OffscreenCanvasCreator = /*#__PURE__*/function () {
    function OffscreenCanvasCreator() {
      this.canvas = void 0;
      this.context = void 0;
    }
    var _proto = OffscreenCanvasCreator.prototype;
    _proto.getOrCreateCanvas = function getOrCreateCanvas(offscreenCanvas) {
      if (this.canvas) {
        return this.canvas;
      }
      // user-defined offscreen canvas
      if (offscreenCanvas) {
        this.canvas = offscreenCanvas;
        this.context = this.canvas.getContext('2d');
      } else {
        try {
          // OffscreenCanvas2D measureText can be up to 40% faster.
          // @ts-ignore
          this.canvas = new window.OffscreenCanvas(0, 0);
          this.context = this.canvas.getContext('2d');
          if (!this.context || !this.context.measureText) {
            this.canvas = document.createElement('canvas');
            this.context = this.canvas.getContext('2d');
          }
        } catch (ex) {
          this.canvas = document.createElement('canvas');
          this.context = this.canvas.getContext('2d');
        }
      }
      this.canvas.width = 10;
      this.canvas.height = 10;
      return this.canvas;
    };
    _proto.getOrCreateContext = function getOrCreateContext(offscreenCanvas) {
      if (this.context) {
        return this.context;
      }
      this.getOrCreateCanvas(offscreenCanvas);
      return this.context;
    };
    return OffscreenCanvasCreator;
  }();

  /**
   * why we need re-render
   */
  var RenderReason;
  (function (RenderReason) {
    RenderReason[RenderReason["CAMERA_CHANGED"] = 0] = "CAMERA_CHANGED";
    RenderReason[RenderReason["DISPLAY_OBJECT_CHANGED"] = 1] = "DISPLAY_OBJECT_CHANGED";
    RenderReason[RenderReason["NONE"] = 2] = "NONE";
  })(RenderReason || (RenderReason = {}));

  /**
   * Use frame renderer implemented by `g-canvas/svg/webgl`, in every frame we do followings:
   * * update & merge dirty rectangles
   * * begin frame
   * * filter by visible
   * * sort by z-index in scene graph
   * * culling with strategies registered in `g-canvas/webgl`
   * * end frame
   */
  var RenderingService = /*#__PURE__*/function () {
    function RenderingService(globalRuntime, context) {
      this.globalRuntime = void 0;
      this.context = void 0;
      this.inited = false;
      this.stats = {
        /**
         * total display objects in scenegraph
         */
        total: 0,
        /**
         * number of display objects need to render in current frame
         */
        rendered: 0
      };
      this.zIndexCounter = 0;
      this.renderOrderChangedEvent = new CustomEvent(ElementEvent.RENDER_ORDER_CHANGED);
      this.hooks = {
        /**
         * called before any frame rendered
         */
        init: new AsyncParallelHook(),
        /**
         * only dirty object which has sth changed will be rendered
         */
        dirtycheck: new SyncWaterfallHook(['object']),
        /**
         * do culling
         */
        cull: new SyncWaterfallHook(['object', 'camera']),
        /**
         * called at beginning of each frame, won't get called if nothing to re-render
         */
        beginFrame: new SyncHook([]),
        /**
         * called before every dirty object get rendered
         */
        beforeRender: new SyncHook(['objectToRender']),
        /**
         * called when every dirty object rendering even it's culled
         */
        render: new SyncHook(['objectToRender']),
        /**
         * called after every dirty object get rendered
         */
        afterRender: new SyncHook(['objectToRender']),
        endFrame: new SyncHook([]),
        destroy: new SyncHook([]),
        /**
         * use async but faster method such as GPU-based picking in `g-plugin-device-renderer`
         */
        pick: new AsyncSeriesWaterfallHook(['result']),
        /**
         * Unsafe but sync version of pick.
         */
        pickSync: new SyncWaterfallHook(['result']),
        /**
         * used in event system
         */
        pointerDown: new SyncHook(['event']),
        pointerUp: new SyncHook(['event']),
        pointerMove: new SyncHook(['event']),
        pointerOut: new SyncHook(['event']),
        pointerOver: new SyncHook(['event']),
        pointerWheel: new SyncHook(['event']),
        pointerCancel: new SyncHook(['event'])
      };
      this.globalRuntime = globalRuntime;
      this.context = context;
    }
    var _proto = RenderingService.prototype;
    _proto.init = /*#__PURE__*/function () {
      var _init = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee() {
        var context;
        return _regeneratorRuntime().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                context = _extends({}, this.globalRuntime, this.context); // register rendering plugins
                this.context.renderingPlugins.forEach(function (plugin) {
                  plugin.apply(context);
                });
                // await this.hooks.init.callPromise();
                _context.next = 4;
                return this.hooks.init.promise();
              case 4:
                this.inited = true;
              case 5:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
      function init() {
        return _init.apply(this, arguments);
      }
      return init;
    }();
    _proto.getStats = function getStats() {
      return this.stats;
    }
    /**
     * Meet the following conditions:
     * * disable DirtyRectangleRendering
     * * camera changed
     */;
    _proto.disableDirtyRectangleRendering = function disableDirtyRectangleRendering() {
      var renderer = this.context.config.renderer;
      var _renderer$getConfig = renderer.getConfig(),
        enableDirtyRectangleRendering = _renderer$getConfig.enableDirtyRectangleRendering;
      return !enableDirtyRectangleRendering || this.context.renderingContext.renderReasons.has(RenderReason.CAMERA_CHANGED);
    };
    _proto.render = function render(canvasConfig) {
      var _this = this;
      this.stats.total = 0;
      this.stats.rendered = 0;
      this.zIndexCounter = 0;
      var renderingContext = this.context.renderingContext;
      this.globalRuntime.sceneGraphService.syncHierarchy(renderingContext.root);
      this.globalRuntime.sceneGraphService.triggerPendingEvents();
      if (renderingContext.renderReasons.size && this.inited) {
        this.renderDisplayObject(renderingContext.root, canvasConfig, renderingContext);
        this.hooks.beginFrame.call();
        renderingContext.renderListCurrentFrame.forEach(function (object) {
          _this.hooks.beforeRender.call(object);
          _this.hooks.render.call(object);
          _this.hooks.afterRender.call(object);
        });
        this.hooks.endFrame.call();
        renderingContext.renderListCurrentFrame = [];
        renderingContext.renderReasons.clear();
      }
      // console.log('stats', this.stats);
    };
    _proto.renderDisplayObject = function renderDisplayObject(displayObject, canvasConfig, renderingContext) {
      var _this2 = this;
      var _canvasConfig$rendere = canvasConfig.renderer.getConfig(),
        enableDirtyCheck = _canvasConfig$rendere.enableDirtyCheck,
        enableCulling = _canvasConfig$rendere.enableCulling;
      // recalc style values
      this.globalRuntime.styleValueRegistry.recalc(displayObject);
      // TODO: relayout
      // dirtycheck first
      var objectChanged = enableDirtyCheck ? this.hooks.dirtycheck.call(displayObject) : displayObject;
      if (objectChanged) {
        var objectToRender = enableCulling ? this.hooks.cull.call(objectChanged, this.context.camera) : objectChanged;
        if (objectToRender) {
          this.stats.rendered++;
          renderingContext.renderListCurrentFrame.push(objectToRender);
        }
      }
      displayObject.renderable.dirty = false;
      displayObject.sortable.renderOrder = this.zIndexCounter++;
      this.stats.total++;
      // sort is very expensive, use cached result if posible
      var sortable = displayObject.sortable;
      var renderOrderChanged = false;
      if (sortable.dirty) {
        sortable.sorted = displayObject.childNodes.slice().sort(sortByZIndex);
        renderOrderChanged = true;
        sortable.dirty = false;
      }
      // recursive rendering its children
      (sortable.sorted || displayObject.childNodes).forEach(function (child) {
        _this2.renderDisplayObject(child, canvasConfig, renderingContext);
      });
      if (renderOrderChanged) {
        displayObject.forEach(function (child) {
          _this2.renderOrderChangedEvent.target = child;
          _this2.renderOrderChangedEvent.detail = {
            renderOrder: child.sortable.renderOrder
          };
          child.ownerDocument.defaultView.dispatchEvent(_this2.renderOrderChangedEvent, true);
        });
      }
    };
    _proto.destroy = function destroy() {
      this.inited = false;
      this.hooks.destroy.call();
    };
    _proto.dirtify = function dirtify() {
      // need re-render
      this.context.renderingContext.renderReasons.add(RenderReason.DISPLAY_OBJECT_CHANGED);
    };
    return RenderingService;
  }();

  var ATTRIBUTE_REGEXP = /\[\s*(.*)=(.*)\s*\]/;
  /**
   * support the following DOM API:
   * * getElementById
   * * getElementsByClassName
   * * getElementsByName
   * * getElementsByTag
   * * querySelector
   * * querySelectorAll
   */
  var DefaultSceneGraphSelector = /*#__PURE__*/function () {
    function DefaultSceneGraphSelector() {}
    var _proto = DefaultSceneGraphSelector.prototype;
    _proto.selectOne = function selectOne(query, root) {
      var _this = this;
      if (query.startsWith('.')) {
        return root.find(function (node) {
          // return !node.shadow && node.id === query.substring(1);
          return ((node === null || node === void 0 ? void 0 : node.classList) || []).indexOf(_this.getIdOrClassname(query)) > -1;
        });
      } else if (query.startsWith('#')) {
        // getElementById('id')
        return root.find(function (node) {
          // return !node.shadow && node.id === query.substring(1);
          return node.id === _this.getIdOrClassname(query);
        });
      } else if (query.startsWith('[')) {
        var _this$getAttribute = this.getAttribute(query),
          name = _this$getAttribute.name,
          value = _this$getAttribute.value;
        if (name) {
          // getElementByName();
          return root.find(function (node) {
            return root !== node && (name === 'name' ? node.name === value : _this.attributeToString(node, name) === value);
          });
        } else {
          return null;
        }
      } else {
        // getElementsByTag('circle');
        return root.find(function (node) {
          return root !== node && node.nodeName === query;
        });
      }
    };
    _proto.selectAll = function selectAll(query, root) {
      var _this2 = this;
      // only support `[name="${name}"]` `.className` `#id`
      if (query.startsWith('.')) {
        // getElementsByClassName('className');
        // should not include itself
        return root.findAll(function (node) {
          return root !== node && ((node === null || node === void 0 ? void 0 : node.classList) || []).indexOf(_this2.getIdOrClassname(query)) > -1;
        });
      } else if (query.startsWith('#')) {
        return root.findAll(function (node) {
          return root !== node && node.id === _this2.getIdOrClassname(query);
        });
      } else if (query.startsWith('[')) {
        var _this$getAttribute2 = this.getAttribute(query),
          name = _this$getAttribute2.name,
          value = _this$getAttribute2.value;
        if (name) {
          // getElementsByName();
          return root.findAll(function (node) {
            return root !== node && (name === 'name' ? node.name === value : _this2.attributeToString(node, name) === value);
          });
        } else {
          return [];
        }
      } else {
        // getElementsByTag('circle');
        return root.findAll(function (node) {
          return root !== node && node.nodeName === query;
        });
      }
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.is = function is(query, node) {
      // a simple `matches` implementation
      if (query.startsWith('.')) {
        return node.className === this.getIdOrClassname(query);
      } else if (query.startsWith('#')) {
        return node.id === this.getIdOrClassname(query);
      } else if (query.startsWith('[')) {
        var _this$getAttribute3 = this.getAttribute(query),
          name = _this$getAttribute3.name,
          value = _this$getAttribute3.value;
        return name === 'name' ? node.name === value : this.attributeToString(node, name) === value;
      } else {
        return node.nodeName === query;
      }
    };
    _proto.getIdOrClassname = function getIdOrClassname(query) {
      return query.substring(1);
    };
    _proto.getAttribute = function getAttribute(query) {
      var matches = query.match(ATTRIBUTE_REGEXP);
      var name = '';
      var value = '';
      if (matches && matches.length > 2) {
        name = matches[1].replace(/"/g, '');
        value = matches[2].replace(/"/g, '');
      }
      return {
        name: name,
        value: value
      };
    };
    _proto.attributeToString = function attributeToString(node, name) {
      if (!node.getAttribute) {
        return '';
      }
      var value = node.getAttribute(name);
      if (isNil(value)) {
        return '';
      }
      if (value.toString) {
        return value.toString();
      }
      return '';
    };
    return DefaultSceneGraphSelector;
  }();

  /**
   * update transform in scene graph
   *
   * @see https://community.khronos.org/t/scene-graphs/50542/7
   */
  var DefaultSceneGraphService = /*#__PURE__*/function () {
    function DefaultSceneGraphService(runtime) {
      var _this = this;
      this.runtime = void 0;
      this.pendingEvents = [];
      this.boundsChangedEvent = new CustomEvent(ElementEvent.BOUNDS_CHANGED);
      this.rotate = function () {
        var parentInvertRotation = create$4();
        return function (element, degrees, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof degrees === 'number') {
            degrees = fromValues$2(degrees, y, z);
          }
          var transform = element.transformable;
          if (element.parentNode === null || !element.parentNode.transformable) {
            _this.rotateLocal(element, degrees);
          } else {
            var rotation = create$4();
            fromEuler(rotation, degrees[0], degrees[1], degrees[2]);
            var rot = _this.getRotation(element);
            var parentRot = _this.getRotation(element.parentNode);
            copy$3(parentInvertRotation, parentRot);
            invert$1(parentInvertRotation, parentInvertRotation);
            multiply$2(rotation, parentInvertRotation, rotation);
            multiply$2(transform.localRotation, rotation, rot);
            normalize$2(transform.localRotation, transform.localRotation);
            _this.dirtifyLocal(element, transform);
          }
        };
      }();
      this.rotateLocal = function () {
        var rotation = create$4();
        return function (element, degrees, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof degrees === 'number') {
            degrees = fromValues$2(degrees, y, z);
          }
          var transform = element.transformable;
          fromEuler(rotation, degrees[0], degrees[1], degrees[2]);
          mul$1(transform.localRotation, transform.localRotation, rotation);
          _this.dirtifyLocal(element, transform);
        };
      }();
      this.setEulerAngles = function () {
        var invParentRot = create$4();
        return function (element, degrees, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof degrees === 'number') {
            degrees = fromValues$2(degrees, y, z);
          }
          var transform = element.transformable;
          if (element.parentNode === null || !element.parentNode.transformable) {
            _this.setLocalEulerAngles(element, degrees);
          } else {
            fromEuler(transform.localRotation, degrees[0], degrees[1], degrees[2]);
            var parentRotation = _this.getRotation(element.parentNode);
            copy$3(invParentRot, invert$1(create$4(), parentRotation));
            mul$1(transform.localRotation, transform.localRotation, invParentRot);
            _this.dirtifyLocal(element, transform);
          }
        };
      }();
      this.translateLocal = function () {
        return function (element, translation, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof translation === 'number') {
            translation = fromValues$2(translation, y, z);
          }
          var transform = element.transformable;
          if (equals$1(translation, create$2())) {
            return;
          }
          transformQuat(translation, translation, transform.localRotation);
          add$1(transform.localPosition, transform.localPosition, translation);
          _this.dirtifyLocal(element, transform);
        };
      }();
      this.setPosition = function () {
        var parentInvertMatrix = create$1();
        var tmpPosition = create$2();
        return function (element, position) {
          var transform = element.transformable;
          tmpPosition[0] = position[0];
          tmpPosition[1] = position[1];
          tmpPosition[2] = position[2] || 0;
          if (equals$1(_this.getPosition(element), tmpPosition)) {
            return;
          }
          copy$1(transform.position, tmpPosition);
          if (element.parentNode === null || !element.parentNode.transformable) {
            copy$1(transform.localPosition, tmpPosition);
          } else {
            var parentTransform = element.parentNode.transformable;
            copy(parentInvertMatrix, parentTransform.worldTransform);
            invert(parentInvertMatrix, parentInvertMatrix);
            transformMat4(transform.localPosition, tmpPosition, parentInvertMatrix);
          }
          _this.dirtifyLocal(element, transform);
        };
      }();
      this.setLocalPosition = function () {
        var tmpPosition = create$2();
        return function (element, position) {
          var transform = element.transformable;
          tmpPosition[0] = position[0];
          tmpPosition[1] = position[1];
          tmpPosition[2] = position[2] || 0;
          if (equals$1(transform.localPosition, tmpPosition)) {
            return;
          }
          copy$1(transform.localPosition, tmpPosition);
          _this.dirtifyLocal(element, transform);
        };
      }();
      this.translate = function () {
        var zeroVec3 = create$2();
        var tmpVec3 = create$2();
        var tr = create$2();
        return function (element, translation, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof translation === 'number') {
            translation = set$1(tmpVec3, translation, y, z);
          }
          if (equals$1(translation, zeroVec3)) {
            return;
          }
          add$1(tr, _this.getPosition(element), translation);
          _this.setPosition(element, tr);
        };
      }();
      this.setRotation = function () {
        var parentInvertRotation = create$4();
        return function (element, rotation, y, z, w) {
          var transform = element.transformable;
          if (typeof rotation === 'number') {
            rotation = fromValues$4(rotation, y, z, w);
          }
          if (element.parentNode === null || !element.parentNode.transformable) {
            _this.setLocalRotation(element, rotation);
          } else {
            var parentRot = _this.getRotation(element.parentNode);
            copy$3(parentInvertRotation, parentRot);
            invert$1(parentInvertRotation, parentInvertRotation);
            multiply$2(transform.localRotation, parentInvertRotation, rotation);
            normalize$2(transform.localRotation, transform.localRotation);
            _this.dirtifyLocal(element, transform);
          }
        };
      };
      this.displayObjectDependencyMap = new WeakMap();
      this.calcLocalTransform = function () {
        var tmpMat = create$1();
        var tmpPosition = create$2();
        var tmpQuat = fromValues$4(0, 0, 0, 1);
        return function (transform) {
          var hasSkew = transform.localSkew[0] !== 0 || transform.localSkew[1] !== 0;
          if (hasSkew) {
            fromRotationTranslationScaleOrigin(transform.localTransform, transform.localRotation, transform.localPosition, fromValues$2(1, 1, 1), transform.origin);
            // apply skew2D
            if (transform.localSkew[0] !== 0 || transform.localSkew[1] !== 0) {
              var tmpMat4 = identity(tmpMat);
              tmpMat4[4] = Math.tan(transform.localSkew[0]);
              tmpMat4[1] = Math.tan(transform.localSkew[1]);
              multiply(transform.localTransform, transform.localTransform, tmpMat4);
            }
            var scaling = fromRotationTranslationScaleOrigin(tmpMat, tmpQuat, tmpPosition, transform.localScale, transform.origin);
            multiply(transform.localTransform, transform.localTransform, scaling);
          } else {
            // @see https://github.com/mattdesl/css-mat4/blob/master/index.js
            fromRotationTranslationScaleOrigin(transform.localTransform, transform.localRotation, transform.localPosition, transform.localScale, transform.origin);
          }
        };
      }();
      this.runtime = runtime;
    }
    var _proto = DefaultSceneGraphService.prototype;
    _proto.matches = function matches(query, root) {
      return this.runtime.sceneGraphSelector.is(query, root);
    };
    _proto.querySelector = function querySelector(query, root) {
      return this.runtime.sceneGraphSelector.selectOne(query, root);
    };
    _proto.querySelectorAll = function querySelectorAll(query, root) {
      return this.runtime.sceneGraphSelector.selectAll(query, root);
      // .filter((node) => !node.shadow);
    };
    _proto.attach = function attach(child, parent, index) {
      var _sortable$sorted, _child$style;
      if (child.parentNode) {
        this.detach(child);
      }
      child.parentNode = parent;
      if (!isNil(index)) {
        child.parentNode.childNodes.splice(index, 0, child);
      } else {
        child.parentNode.childNodes.push(child);
      }
      // parent needs re-sort
      var sortable = parent.sortable;
      if ((sortable === null || sortable === void 0 ? void 0 : (_sortable$sorted = sortable.sorted) === null || _sortable$sorted === void 0 ? void 0 : _sortable$sorted.length) || ((_child$style = child.style) === null || _child$style === void 0 ? void 0 : _child$style.zIndex)) {
        // if (sortable) {
        // only child has z-Index
        sortable.dirty = true;
      }
      // this.updateGraphDepth(child);
      var transform = child.transformable;
      if (transform) {
        this.dirtifyWorld(child, transform);
      }
      if (transform.frozen) {
        this.unfreezeParentToRoot(child);
      }
    };
    _proto.detach = function detach(child) {
      if (child.parentNode) {
        var _sortable$sorted2, _child$style2;
        var transform = child.transformable;
        // if (transform) {
        //   const worldTransform = this.getWorldTransform(child, transform);
        //   mat4.getScaling(transform.localScale, worldTransform);
        //   mat4.getTranslation(transform.localPosition, worldTransform);
        //   mat4.getRotation(transform.localRotation, worldTransform);
        //   transform.localDirtyFlag = true;
        // }
        // parent needs re-sort
        var sortable = child.parentNode.sortable;
        // if (sortable) {
        if ((sortable === null || sortable === void 0 ? void 0 : (_sortable$sorted2 = sortable.sorted) === null || _sortable$sorted2 === void 0 ? void 0 : _sortable$sorted2.length) || ((_child$style2 = child.style) === null || _child$style2 === void 0 ? void 0 : _child$style2.zIndex)) {
          sortable.dirty = true;
        }
        var index = child.parentNode.childNodes.indexOf(child);
        if (index > -1) {
          child.parentNode.childNodes.splice(index, 1);
        }
        if (transform) {
          this.dirtifyWorld(child, transform);
        }
        child.parentNode = null;
      }
    };
    _proto.getOrigin = function getOrigin(element) {
      return element.transformable.origin;
    }
    /**
     * same as pivot in Pixi.js
     *
     * @see https://stackoverflow.com/questions/40748452/how-to-change-css-transform-origin-but-preserve-transformation
     */;
    _proto.setOrigin = function setOrigin(element, origin, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      if (typeof origin === 'number') {
        origin = fromValues$2(origin, y, z);
      }
      var transform = element.transformable;
      if (equals$1(origin, transform.origin)) {
        return;
      }
      var originVec = transform.origin;
      // const delta = vec3.subtract(vec3.create(), origin, originVec);
      // vec3.add(transform.localPosition, transform.localPosition, delta);
      // update origin
      originVec[0] = origin[0];
      originVec[1] = origin[1];
      originVec[2] = origin[2] || 0;
      this.dirtifyLocal(element, transform);
    }
    /**
     * rotate in world space
     */;
    /**
     * set euler angles(degrees) in local space
     */
    _proto.setLocalEulerAngles = function setLocalEulerAngles(element, degrees, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      if (typeof degrees === 'number') {
        degrees = fromValues$2(degrees, y, z);
      }
      var transform = element.transformable;
      fromEuler(transform.localRotation, degrees[0], degrees[1], degrees[2]);
      this.dirtifyLocal(element, transform);
    }
    /**
     * translate in local space
     *
     * @example
     * ```
     * translateLocal(x, y, z)
     * translateLocal(vec3(x, y, z))
     * ```
     */;
    /**
     * scale in local space
     */
    _proto.scaleLocal = function scaleLocal(element, scaling) {
      var transform = element.transformable;
      multiply$1(transform.localScale, transform.localScale, fromValues$2(scaling[0], scaling[1], scaling[2] || 1));
      this.dirtifyLocal(element, transform);
    };
    _proto.setLocalScale = function setLocalScale(element, scaling) {
      var transform = element.transformable;
      var updatedScaling = fromValues$2(scaling[0], scaling[1], scaling[2] || transform.localScale[2]);
      if (equals$1(updatedScaling, transform.localScale)) {
        return;
      }
      copy$1(transform.localScale, updatedScaling);
      this.dirtifyLocal(element, transform);
    }
    /**
     * translate in world space
     *
     * @example
     * ```
     * translate(x, y, z)
     * translate(vec3(x, y, z))
     * ```
     *
     * 对应 g 原版的 translate 2D
     * @see https://github.com/antvis/g/blob/master/packages/g-base/src/abstract/element.ts#L665-L676
     */;
    _proto.setLocalRotation = function setLocalRotation(element, rotation, y, z, w) {
      if (typeof rotation === 'number') {
        rotation = fromValues$4(rotation, y, z, w);
      }
      var transform = element.transformable;
      copy$3(transform.localRotation, rotation);
      this.dirtifyLocal(element, transform);
    };
    _proto.setLocalSkew = function setLocalSkew(element, skew, y) {
      if (typeof skew === 'number') {
        skew = fromValues$5(skew, y);
      }
      var transform = element.transformable;
      copy$4(transform.localSkew, skew);
      this.dirtifyLocal(element, transform);
    };
    _proto.dirtifyLocal = function dirtifyLocal(element, transform) {
      if (!transform.localDirtyFlag) {
        transform.localDirtyFlag = true;
        if (!transform.dirtyFlag) {
          this.dirtifyWorld(element, transform);
        }
      }
    };
    _proto.dirtifyWorld = function dirtifyWorld(element, transform) {
      if (!transform.dirtyFlag) {
        this.unfreezeParentToRoot(element);
      }
      this.dirtifyWorldInternal(element, transform);
      this.dirtifyToRoot(element, true);
    };
    _proto.triggerPendingEvents = function triggerPendingEvents() {
      var _this2 = this;
      var set = new Set();
      var trigger = function trigger(element, detail) {
        if (element.isConnected && !set.has(element.entity)) {
          _this2.boundsChangedEvent.detail = detail;
          _this2.boundsChangedEvent.target = element;
          element.ownerDocument.defaultView.dispatchEvent(_this2.boundsChangedEvent, true);
          set.add(element.entity);
        }
      };
      this.pendingEvents.forEach(function (_ref) {
        var element = _ref[0],
          detail = _ref[1];
        if (detail.affectChildren) {
          element.forEach(function (e) {
            trigger(e, detail);
          });
        } else {
          trigger(element, detail);
        }
      });
      this.pendingEvents = [];
      set.clear();
    };
    _proto.dirtifyToRoot = function dirtifyToRoot(element, affectChildren) {
      if (affectChildren === void 0) {
        affectChildren = false;
      }
      var p = element;
      // only need to re-render itself
      if (p.renderable) {
        p.renderable.dirty = true;
      }
      while (p) {
        var renderable = p.renderable;
        if (renderable) {
          renderable.renderBoundsDirty = true;
          renderable.boundsDirty = true;
        }
        p = p.parentNode;
      }
      // inform dependencies
      this.informDependentDisplayObjects(element);
      // reuse the same custom event
      this.pendingEvents.push([element, {
        affectChildren: affectChildren
      }]);
    };
    _proto.updateDisplayObjectDependency = function updateDisplayObjectDependency(name, oldPath, newPath, object) {
      // clear ref to old clip path
      if (oldPath && oldPath !== newPath) {
        var oldDependencyMap = this.displayObjectDependencyMap.get(oldPath);
        if (oldDependencyMap && oldDependencyMap[name]) {
          var index = oldDependencyMap[name].indexOf(object);
          oldDependencyMap[name].splice(index, 1);
        }
      }
      if (newPath) {
        var newDependencyMap = this.displayObjectDependencyMap.get(newPath);
        if (!newDependencyMap) {
          this.displayObjectDependencyMap.set(newPath, {});
          newDependencyMap = this.displayObjectDependencyMap.get(newPath);
        }
        if (!newDependencyMap[name]) {
          newDependencyMap[name] = [];
        }
        newDependencyMap[name].push(object);
      }
    };
    _proto.informDependentDisplayObjects = function informDependentDisplayObjects(object) {
      var _this3 = this;
      var dependencyMap = this.displayObjectDependencyMap.get(object);
      if (dependencyMap) {
        Object.keys(dependencyMap).forEach(function (name) {
          dependencyMap[name].forEach(function (target) {
            _this3.dirtifyToRoot(target);
            target.dispatchEvent(new MutationEvent(ElementEvent.ATTR_MODIFIED, target, _this3, _this3, name, MutationEvent.MODIFICATION, _this3, _this3));
            if (target.isCustomElement && target.isConnected) {
              if (target.attributeChangedCallback) {
                target.attributeChangedCallback(name, _this3, _this3);
              }
            }
          });
        });
      }
    };
    _proto.getPosition = function getPosition(element) {
      var transform = element.transformable;
      return getTranslation(transform.position, this.getWorldTransform(element, transform));
    };
    _proto.getRotation = function getRotation$1(element) {
      var transform = element.transformable;
      return getRotation(transform.rotation, this.getWorldTransform(element, transform));
    };
    _proto.getScale = function getScale(element) {
      var transform = element.transformable;
      return getScaling(transform.scaling, this.getWorldTransform(element, transform));
    };
    _proto.getWorldTransform = function getWorldTransform(element, transform) {
      if (transform === void 0) {
        transform = element.transformable;
      }
      if (!transform.localDirtyFlag && !transform.dirtyFlag) {
        return transform.worldTransform;
      }
      if (element.parentNode && element.parentNode.transformable) {
        this.getWorldTransform(element.parentNode);
      }
      this.sync(element, transform);
      return transform.worldTransform;
    };
    _proto.getLocalPosition = function getLocalPosition(element) {
      return element.transformable.localPosition;
    };
    _proto.getLocalRotation = function getLocalRotation(element) {
      return element.transformable.localRotation;
    };
    _proto.getLocalScale = function getLocalScale(element) {
      return element.transformable.localScale;
    };
    _proto.getLocalSkew = function getLocalSkew(element) {
      return element.transformable.localSkew;
    };
    _proto.getLocalTransform = function getLocalTransform(element) {
      var transform = element.transformable;
      if (transform.localDirtyFlag) {
        this.calcLocalTransform(transform);
        transform.localDirtyFlag = false;
      }
      return transform.localTransform;
    };
    _proto.setLocalTransform = function setLocalTransform(element, transform) {
      var t = getTranslation(create$2(), transform);
      var r = getRotation(create$4(), transform);
      var s = getScaling(create$2(), transform);
      this.setLocalScale(element, s);
      this.setLocalPosition(element, t);
      this.setLocalRotation(element, r);
    };
    _proto.resetLocalTransform = function resetLocalTransform(element) {
      this.setLocalScale(element, [1, 1, 1]);
      this.setLocalPosition(element, [0, 0, 0]);
      this.setLocalEulerAngles(element, [0, 0, 0]);
      this.setLocalSkew(element, [0, 0]);
    };
    _proto.getTransformedGeometryBounds = function getTransformedGeometryBounds(element, render, existedAABB) {
      if (render === void 0) {
        render = false;
      }
      var bounds = this.getGeometryBounds(element, render);
      if (!AABB.isEmpty(bounds)) {
        var aabb = existedAABB || new AABB();
        aabb.setFromTransformedAABB(bounds, this.getWorldTransform(element));
        return aabb;
      } else {
        return null;
      }
    }
    /**
     * won't account for children
     */;
    _proto.getGeometryBounds = function getGeometryBounds(element, render) {
      if (render === void 0) {
        render = false;
      }
      var geometry = element.geometry;
      var bounds = render ? geometry.renderBounds : geometry.contentBounds || null;
      // return (bounds && new AABB(bounds.center, bounds.halfExtents)) || new AABB();
      return bounds || new AABB();
    }
    /**
     * account for children in world space
     */;
    _proto.getBounds = function getBounds(element, render) {
      var _this4 = this;
      if (render === void 0) {
        render = false;
      }
      var renderable = element.renderable;
      if (!renderable.boundsDirty && !render && renderable.bounds) {
        return renderable.bounds;
      }
      if (!renderable.renderBoundsDirty && render && renderable.renderBounds) {
        return renderable.renderBounds;
      }
      // reuse existed if possible
      var existedAABB = render ? renderable.renderBounds : renderable.bounds;
      // reset with geometry's aabb
      var aabb = this.getTransformedGeometryBounds(element, render, existedAABB);
      // merge children's aabbs
      var children = element.childNodes;
      children.forEach(function (child) {
        var childBounds = _this4.getBounds(child, render);
        if (childBounds) {
          if (!aabb) {
            aabb = existedAABB || new AABB();
            aabb.update(childBounds.center, childBounds.halfExtents);
          } else {
            aabb.add(childBounds);
          }
        }
      });
      if (render) {
        // FIXME: account for clip path
        var clipped = findClosestClipPathTarget(element);
        if (clipped) {
          // use bounds under world space
          var clipPathBounds = clipped.style.clipPath.getBounds(render);
          if (!aabb) {
            aabb = clipPathBounds;
          } else if (clipPathBounds) {
            aabb = clipPathBounds.intersection(aabb);
          }
        }
      }
      if (!aabb) {
        aabb = new AABB();
      }
      if (aabb) {
        if (render) {
          renderable.renderBounds = aabb;
        } else {
          renderable.bounds = aabb;
        }
      }
      if (render) {
        renderable.renderBoundsDirty = false;
      } else {
        renderable.boundsDirty = false;
      }
      return aabb;
    }
    /**
     * account for children in local space
     */;
    _proto.getLocalBounds = function getLocalBounds(element) {
      if (element.parentNode) {
        var parentInvert = create$1();
        if (element.parentNode.transformable) {
          parentInvert = invert(create$1(), this.getWorldTransform(element.parentNode));
        }
        var bounds = this.getBounds(element);
        if (!AABB.isEmpty(bounds)) {
          var localBounds = new AABB();
          localBounds.setFromTransformedAABB(bounds, parentInvert);
          return localBounds;
        }
      }
      return this.getBounds(element);
    };
    _proto.getBoundingClientRect = function getBoundingClientRect(element) {
      var _element$ownerDocumen, _element$ownerDocumen2;
      var aabb;
      var bounds = this.getGeometryBounds(element);
      if (!AABB.isEmpty(bounds)) {
        aabb = new AABB();
        // apply transformation to aabb
        aabb.setFromTransformedAABB(bounds, this.getWorldTransform(element));
      }
      // calc context's offset
      var bbox = (_element$ownerDocumen = element.ownerDocument) === null || _element$ownerDocumen === void 0 ? void 0 : (_element$ownerDocumen2 = _element$ownerDocumen.defaultView) === null || _element$ownerDocumen2 === void 0 ? void 0 : _element$ownerDocumen2.getContextService().getBoundingClientRect();
      if (aabb) {
        var _aabb$getMin = aabb.getMin(),
          left = _aabb$getMin[0],
          top = _aabb$getMin[1];
        var _aabb$getMax = aabb.getMax(),
          right = _aabb$getMax[0],
          bottom = _aabb$getMax[1];
        return new Rectangle(left + ((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0), top + ((bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0), right - left, bottom - top);
      }
      return new Rectangle((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0, (bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0, 0, 0);
    };
    _proto.dirtifyWorldInternal = function dirtifyWorldInternal(element, transform) {
      var _this5 = this;
      if (!transform.dirtyFlag) {
        transform.dirtyFlag = true;
        transform.frozen = false;
        element.childNodes.forEach(function (child) {
          var childTransform = child.transformable;
          if (!childTransform.dirtyFlag) {
            _this5.dirtifyWorldInternal(child, childTransform);
          }
        });
        var renderable = element.renderable;
        if (renderable) {
          renderable.renderBoundsDirty = true;
          renderable.boundsDirty = true;
          renderable.dirty = true;
        }
      }
    };
    _proto.syncHierarchy = function syncHierarchy(element) {
      var transform = element.transformable;
      if (transform.frozen) {
        return;
      }
      transform.frozen = true;
      if (transform.localDirtyFlag || transform.dirtyFlag) {
        this.sync(element, transform);
      }
      var children = element.childNodes;
      for (var i = 0; i < children.length; i++) {
        this.syncHierarchy(children[i]);
      }
    };
    _proto.sync = function sync(element, transform) {
      if (transform.localDirtyFlag) {
        this.calcLocalTransform(transform);
        transform.localDirtyFlag = false;
      }
      if (transform.dirtyFlag) {
        var parent = element.parentNode;
        var parentTransform = parent && parent.transformable;
        if (parent === null || !parentTransform) {
          copy(transform.worldTransform, transform.localTransform);
        } else {
          // TODO: should we support scale compensation?
          // @see https://github.com/playcanvas/engine/issues/1077#issuecomment-359765557
          multiply(transform.worldTransform, parentTransform.worldTransform, transform.localTransform);
        }
        transform.dirtyFlag = false;
      }
    };
    _proto.unfreezeParentToRoot = function unfreezeParentToRoot(child) {
      var p = child.parentNode;
      while (p) {
        var transform = p.transformable;
        if (transform) {
          transform.frozen = false;
        }
        p = p.parentNode;
      }
    };
    return DefaultSceneGraphService;
  }();

  var TEXT_METRICS = {
    MetricsString: '|ÉqÅ',
    BaselineSymbol: 'M',
    BaselineMultiplier: 1.4,
    HeightMultiplier: 2,
    Newlines: [0x000a, 0x000d // carriage return
    ],

    BreakingSpaces: [0x0009, 0x0020, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005, 0x2006, 0x2008, 0x2009, 0x200a, 0x205f, 0x3000 // ideographic space
    ]
  };

  var LATIN_REGEX = /[a-zA-Z0-9\u00C0-\u00D6\u00D8-\u00f6\u00f8-\u00ff!"#$%&'()*+,-./:;]/;
  // Line breaking rules in CJK (Kinsoku Shori)
  // Refer from https://en.wikipedia.org/wiki/Line_breaking_rules_in_East_Asian_languages
  var regexCannotStartZhCn = /[!%),.:;?\]}¢°·'""†‡›℃∶、。〃〆〕〗〞﹚﹜！＂％＇），．：；？！］｝～]/;
  var regexCannotEndZhCn = /[$(£¥·'"〈《「『【〔〖〝﹙﹛＄（．［｛￡￥]/;
  var regexCannotStartZhTw = /[!),.:;?\]}¢·–—'"•"、。〆〞〕〉》」︰︱︲︳﹐﹑﹒﹓﹔﹕﹖﹘﹚﹜！），．：；？︶︸︺︼︾﹀﹂﹗］｜｝､]/;
  var regexCannotEndZhTw = /[([{£¥'"‵〈《「『〔〝︴﹙﹛（｛︵︷︹︻︽︿﹁﹃﹏]/;
  var regexCannotStartJaJp = /[)\]｝〕〉》」』】〙〗〟'"｠»ヽヾーァィゥェォッャュョヮヵヶぁぃぅぇぉっゃゅょゎゕゖㇰㇱㇲㇳㇴㇵㇶㇷㇸㇹㇺㇻㇼㇽㇾㇿ々〻‐゠–〜?!‼⁇⁈⁉・、:;,。.]/;
  var regexCannotEndJaJp = /[([｛〔〈《「『【〘〖〝'"｟«—...‥〳〴〵]/;
  var regexCannotStartKoKr = /[!%),.:;?\]}¢°'"†‡℃〆〈《「『〕！％），．：；？］｝]/;
  var regexCannotEndKoKr = /[$([{£¥'"々〇〉》」〔＄（［｛｠￥￦#]/;
  var regexCannotStart = new RegExp(regexCannotStartZhCn.source + "|" + regexCannotStartZhTw.source + "|" + regexCannotStartJaJp.source + "|" + regexCannotStartKoKr.source);
  var regexCannotEnd = new RegExp(regexCannotEndZhCn.source + "|" + regexCannotEndZhTw.source + "|" + regexCannotEndJaJp.source + "|" + regexCannotEndKoKr.source);
  /**
   * Borrow from pixi/packages/text/src/TextMetrics.ts
   */
  var TextService = /*#__PURE__*/function () {
    function TextService(runtime) {
      var _this = this;
      this.runtime = void 0;
      this.fontMetricsCache = {};
      this.shouldBreakByKinsokuShorui = function (char, nextChar) {
        if (_this.isBreakingSpace(nextChar)) return false;
        if (char) {
          // Line breaking rules in CJK (Kinsoku Shori)
          if (regexCannotEnd.exec(nextChar) || regexCannotStart.exec(char)) {
            return true;
          }
        }
        return false;
      };
      this.trimByKinsokuShorui = function (prev) {
        var next = [].concat(prev);
        var prevLine = next[next.length - 2];
        if (!prevLine) {
          return prev;
        }
        var lastChar = prevLine[prevLine.length - 1];
        next[next.length - 2] = prevLine.slice(0, -1);
        next[next.length - 1] = lastChar + next[next.length - 1];
        return next;
      };
      this.runtime = runtime;
    }
    /**
     * font metrics cache
     */
    var _proto = TextService.prototype;
    /**
     * Calculates the ascent, descent and fontSize of a given font-style.
     */
    _proto.measureFont = function measureFont(font, offscreenCanvas) {
      // as this method is used for preparing assets, don't recalculate things if we don't need to
      if (this.fontMetricsCache[font]) {
        return this.fontMetricsCache[font];
      }
      var properties = {
        ascent: 0,
        descent: 0,
        fontSize: 0
      };
      var canvas = this.runtime.offscreenCanvas.getOrCreateCanvas(offscreenCanvas);
      var context = this.runtime.offscreenCanvas.getOrCreateContext(offscreenCanvas);
      context.font = font;
      var metricsString = TEXT_METRICS.MetricsString + TEXT_METRICS.BaselineSymbol;
      var width = Math.ceil(context.measureText(metricsString).width);
      var baseline = Math.ceil(context.measureText(TEXT_METRICS.BaselineSymbol).width);
      var height = TEXT_METRICS.HeightMultiplier * baseline;
      baseline = baseline * TEXT_METRICS.BaselineMultiplier | 0;
      // @ts-ignore
      canvas.width = width;
      // @ts-ignore
      canvas.height = height;
      context.fillStyle = '#f00';
      context.fillRect(0, 0, width, height);
      context.font = font;
      context.textBaseline = 'alphabetic';
      context.fillStyle = '#000';
      context.fillText(metricsString, 0, baseline);
      var imagedata = context.getImageData(0, 0, width || 1, height || 1).data;
      var pixels = imagedata.length;
      var line = width * 4;
      var i = 0;
      var idx = 0;
      var stop = false;
      // ascent. scan from top to bottom until we find a non red pixel
      for (i = 0; i < baseline; ++i) {
        for (var j = 0; j < line; j += 4) {
          if (imagedata[idx + j] !== 255) {
            stop = true;
            break;
          }
        }
        if (!stop) {
          idx += line;
        } else {
          break;
        }
      }
      properties.ascent = baseline - i;
      idx = pixels - line;
      stop = false;
      // descent. scan from bottom to top until we find a non red pixel
      for (i = height; i > baseline; --i) {
        for (var _j = 0; _j < line; _j += 4) {
          if (imagedata[idx + _j] !== 255) {
            stop = true;
            break;
          }
        }
        if (!stop) {
          idx -= line;
        } else {
          break;
        }
      }
      properties.descent = i - baseline;
      properties.fontSize = properties.ascent + properties.descent;
      this.fontMetricsCache[font] = properties;
      return properties;
    };
    _proto.measureText = function measureText(text, parsedStyle, offscreenCanvas) {
      var fontSize = parsedStyle.fontSize,
        wordWrap = parsedStyle.wordWrap,
        strokeHeight = parsedStyle.lineHeight,
        lineWidth = parsedStyle.lineWidth,
        textBaseline = parsedStyle.textBaseline,
        textAlign = parsedStyle.textAlign,
        letterSpacing = parsedStyle.letterSpacing,
        textPath = parsedStyle.textPath,
        textPathSide = parsedStyle.textPathSide,
        textPathStartOffset = parsedStyle.textPathStartOffset,
        _parsedStyle$leading = parsedStyle.leading,
        leading = _parsedStyle$leading === void 0 ? 0 : _parsedStyle$leading;
      var font = toFontString(parsedStyle);
      var fontProperties = this.measureFont(font, offscreenCanvas);
      // fallback in case UA disallow canvas data extraction
      // (toDataURI, getImageData functions)
      if (fontProperties.fontSize === 0) {
        fontProperties.fontSize = fontSize;
        fontProperties.ascent = fontSize;
      }
      var context = this.runtime.offscreenCanvas.getOrCreateContext(offscreenCanvas);
      context.font = font;
      // no overflowing by default
      parsedStyle.isOverflowing = false;
      var outputText = wordWrap ? this.wordWrap(text, parsedStyle, offscreenCanvas) : text;
      var lines = outputText.split(/(?:\r\n|\r|\n)/);
      var lineWidths = new Array(lines.length);
      var maxLineWidth = 0;
      // account for textPath
      if (textPath) {
        var totalPathLength = textPath.getTotalLength();
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        var startingPoint = textPath.getPoint(0);
        for (var i = 0; i < lines.length; i++) {
          var width = context.measureText(lines[i]).width + (lines[i].length - 1) * letterSpacing;
          // for (
          //   let i = reverse ? lines[0].length - 1 : 0;
          //   reverse ? i >= 0 : i < lines[0].length;
          //   reverse ? i-- : i++
          // ) {
          //   graphemeInfo = lineBounds[i];
          //   if (positionInPath > totalPathLength) {
          //     positionInPath %= totalPathLength;
          //   } else if (positionInPath < 0) {
          //     positionInPath += totalPathLength;
          //   }
          //   // it would probably much faster to send all the grapheme position for a line
          //   // and calculate path position/angle at once.
          //   this.setGraphemeOnPath(
          //     positionInPath,
          //     graphemeInfo,
          //     startingPoint
          //   );
          //   positionInPath += graphemeInfo.kernedWidth;
          // }
        }
      } else {
        for (var _i = 0; _i < lines.length; _i++) {
          // char width + letterSpacing
          var _lineWidth = context.measureText(lines[_i]).width + (lines[_i].length - 1) * letterSpacing;
          lineWidths[_i] = _lineWidth;
          maxLineWidth = Math.max(maxLineWidth, _lineWidth);
        }
        var _width = maxLineWidth + lineWidth;
        // if (dropShadow) {
        //   width += dropShadowDistance;
        // }
        var lineHeight = strokeHeight || fontProperties.fontSize + lineWidth;
        var height = Math.max(lineHeight, fontProperties.fontSize + lineWidth) + (lines.length - 1) * (lineHeight + leading);
        // if (dropShadow) {
        //   height += dropShadowDistance;
        // }
        lineHeight += leading;
        // handle vertical text baseline
        var offsetY = 0;
        if (textBaseline === 'middle') {
          offsetY = -height / 2;
        } else if (textBaseline === 'bottom' || textBaseline === 'alphabetic' || textBaseline === 'ideographic') {
          offsetY = -height;
        } else if (textBaseline === 'top' || textBaseline === 'hanging') {
          offsetY = 0;
        }
        return {
          font: font,
          width: _width,
          height: height,
          lines: lines,
          lineWidths: lineWidths,
          lineHeight: lineHeight,
          maxLineWidth: maxLineWidth,
          fontProperties: fontProperties,
          lineMetrics: lineWidths.map(function (width, i) {
            var offsetX = 0;
            // handle horizontal text align
            if (textAlign === 'center' || textAlign === 'middle') {
              offsetX -= width / 2;
            } else if (textAlign === 'right' || textAlign === 'end') {
              offsetX -= width;
            }
            return new Rectangle(offsetX - lineWidth / 2, offsetY + i * lineHeight, width + lineWidth, lineHeight);
          })
        };
      }
    };
    _proto.setGraphemeOnPath = function setGraphemeOnPath() {};
    _proto.wordWrap = function wordWrap(text, parsedStyle, offscreenCanvas) {
      var _this2 = this;
      var _parsedStyle$wordWrap = parsedStyle.wordWrapWidth,
        wordWrapWidth = _parsedStyle$wordWrap === void 0 ? 0 : _parsedStyle$wordWrap,
        letterSpacing = parsedStyle.letterSpacing,
        _parsedStyle$maxLines = parsedStyle.maxLines,
        maxLines = _parsedStyle$maxLines === void 0 ? Infinity : _parsedStyle$maxLines,
        textOverflow = parsedStyle.textOverflow;
      var context = this.runtime.offscreenCanvas.getOrCreateContext(offscreenCanvas);
      var maxWidth = wordWrapWidth + letterSpacing;
      var ellipsis = '';
      if (textOverflow === 'ellipsis') {
        ellipsis = '...';
      } else if (textOverflow && textOverflow !== 'clip') {
        ellipsis = textOverflow;
      }
      var lines = [];
      var currentIndex = 0;
      var currentWidth = 0;
      var cache = {};
      var calcWidth = function calcWidth(char) {
        return _this2.getFromCache(char, letterSpacing, cache, context);
      };
      var ellipsisWidth = Array.from(ellipsis).reduce(function (prev, cur) {
        return prev + calcWidth(cur);
      }, 0);
      var chars = Array.from(text);
      for (var i = 0; i < chars.length; i++) {
        var char = chars[i];
        var prevChar = text[i - 1];
        var nextChar = text[i + 1];
        var width = calcWidth(char);
        if (this.isNewline(char)) {
          currentIndex++;
          currentWidth = 0;
          lines[currentIndex] = '';
          continue;
        }
        if (currentWidth > 0 && currentWidth + width > maxWidth) {
          if (currentIndex + 1 >= maxLines) {
            parsedStyle.isOverflowing = true;
            // If there is not enough space to display the string itself, it is clipped.
            // @see https://developer.mozilla.org/en-US/docs/Web/CSS/text-overflow#values
            if (ellipsisWidth > 0 && ellipsisWidth <= maxWidth) {
              // Backspace from line's end.
              var lastLineWidth = 0;
              var lastLineIndex = 0;
              for (var _i2 = 0; _i2 < lines[currentIndex].length; _i2++) {
                var _width2 = calcWidth(lines[currentIndex][_i2]);
                if (lastLineWidth + _width2 + ellipsisWidth > maxWidth) {
                  lastLineIndex = _i2;
                  break;
                }
                lastLineWidth += _width2;
              }
              lines[currentIndex] = (lines[currentIndex] || '').slice(0, lastLineIndex) + ellipsis;
            }
            break;
          }
          currentIndex++;
          currentWidth = 0;
          lines[currentIndex] = '';
          if (this.isBreakingSpace(char)) {
            continue;
          }
          if (!this.canBreakInLastChar(char)) {
            lines = this.trimToBreakable(lines);
            currentWidth = this.sumTextWidthByCache(lines[currentIndex] || '', cache);
          }
          if (this.shouldBreakByKinsokuShorui(char, nextChar)) {
            lines = this.trimByKinsokuShorui(lines);
            currentWidth += calcWidth(prevChar || '');
          }
        }
        currentWidth += width;
        lines[currentIndex] = (lines[currentIndex] || '') + char;
      }
      return lines.join('\n');
    };
    _proto.isBreakingSpace = function isBreakingSpace(char) {
      if (typeof char !== 'string') {
        return false;
      }
      return TEXT_METRICS.BreakingSpaces.indexOf(char.charCodeAt(0)) >= 0;
    };
    _proto.isNewline = function isNewline(char) {
      if (typeof char !== 'string') {
        return false;
      }
      return TEXT_METRICS.Newlines.indexOf(char.charCodeAt(0)) >= 0;
    };
    _proto.trimToBreakable = function trimToBreakable(prev) {
      var next = [].concat(prev);
      var prevLine = next[next.length - 2];
      var index = this.findBreakableIndex(prevLine);
      if (index === -1 || !prevLine) return next;
      var trimmedChar = prevLine.slice(index, index + 1);
      var isTrimmedWithSpace = this.isBreakingSpace(trimmedChar);
      var trimFrom = index + 1;
      var trimTo = index + (isTrimmedWithSpace ? 0 : 1);
      next[next.length - 1] += prevLine.slice(trimFrom, prevLine.length);
      next[next.length - 2] = prevLine.slice(0, trimTo);
      return next;
    };
    _proto.canBreakInLastChar = function canBreakInLastChar(char) {
      if (char && LATIN_REGEX.test(char)) return false;
      return true;
    };
    _proto.sumTextWidthByCache = function sumTextWidthByCache(text, cache) {
      return text.split('').reduce(function (sum, c) {
        if (!cache[c]) throw Error('cannot count the word without cache');
        return sum + cache[c];
      }, 0);
    };
    _proto.findBreakableIndex = function findBreakableIndex(line) {
      for (var i = line.length - 1; i >= 0; i--) {
        if (!LATIN_REGEX.test(line[i])) return i;
      }
      return -1;
    };
    _proto.getFromCache = function getFromCache(key, letterSpacing, cache, context) {
      var width = cache[key];
      if (typeof width !== 'number') {
        var spacing = key.length * letterSpacing;
        width = context.measureText(key).width + spacing;
        cache[key] = width;
      }
      return width;
    };
    return TextService;
  }();

  var runtime = {};
  /**
   * Replace with IoC container
   */
  var geometryUpdaterFactory = function () {
    var _ref;
    var rectUpdater = new RectUpdater();
    var polylineUpdater = new PolylineUpdater();
    return _ref = {}, _ref[Shape.CIRCLE] = new CircleUpdater(), _ref[Shape.ELLIPSE] = new EllipseUpdater(), _ref[Shape.RECT] = rectUpdater, _ref[Shape.IMAGE] = rectUpdater, _ref[Shape.GROUP] = rectUpdater, _ref[Shape.LINE] = new LineUpdater(), _ref[Shape.TEXT] = new TextUpdater(runtime), _ref[Shape.POLYLINE] = polylineUpdater, _ref[Shape.POLYGON] = polylineUpdater, _ref[Shape.PATH] = new PathUpdater(), _ref[Shape.HTML] = null, _ref[Shape.MESH] = null, _ref;
  }();
  var CSSPropertySyntaxFactory = function () {
    var _ref2;
    var color = new CSSPropertyColor();
    var length = new CSSPropertyLengthOrPercentage();
    return _ref2 = {}, _ref2[PropertySyntax.PERCENTAGE] = null, _ref2[PropertySyntax.NUMBER] = new CSSPropertyNumber(), _ref2[PropertySyntax.ANGLE] = new CSSPropertyAngle(), _ref2[PropertySyntax.DEFINED_PATH] = new CSSPropertyClipPath(), _ref2[PropertySyntax.PAINT] = color, _ref2[PropertySyntax.COLOR] = color, _ref2[PropertySyntax.FILTER] = new CSSPropertyFilter(), _ref2[PropertySyntax.LENGTH] = length, _ref2[PropertySyntax.LENGTH_PERCENTAGE] = length, _ref2[PropertySyntax.LENGTH_PERCENTAGE_12] = new CSSPropertyLengthOrPercentage12(), _ref2[PropertySyntax.LENGTH_PERCENTAGE_14] = new CSSPropertyLengthOrPercentage14(), _ref2[PropertySyntax.COORDINATE] = new CSSPropertyLocalPosition(), _ref2[PropertySyntax.OFFSET_DISTANCE] = new CSSPropertyOffsetDistance(), _ref2[PropertySyntax.OPACITY_VALUE] = new CSSPropertyOpacity(), _ref2[PropertySyntax.PATH] = new CSSPropertyPath(), _ref2[PropertySyntax.LIST_OF_POINTS] = new CSSPropertyPoints(), _ref2[PropertySyntax.SHADOW_BLUR] = new CSSPropertyShadowBlur(), _ref2[PropertySyntax.TEXT] = new CSSPropertyText(), _ref2[PropertySyntax.TEXT_TRANSFORM] = new CSSPropertyTextTransform(), _ref2[PropertySyntax.TRANSFORM] = new CSSPropertyTransform(), _ref2[PropertySyntax.TRANSFORM_ORIGIN] = new CSSPropertyTransformOrigin(), _ref2[PropertySyntax.Z_INDEX] = new CSSPropertyZIndex(), _ref2[PropertySyntax.MARKER] = new CSSPropertyMarker(), _ref2;
  }();
  /**
   * Camera
   * `g-camera-api` will provide an advanced implementation
   */
  runtime.CameraContribution = Camera;
  /**
   * `g-web-animations-api` will provide an AnimationTimeline
   */
  runtime.AnimationTimeline = null;
  runtime.EasingFunction = null;
  runtime.offscreenCanvas = new OffscreenCanvasCreator();
  runtime.displayObjectPool = new DisplayObjectPool();
  runtime.sceneGraphSelector = new DefaultSceneGraphSelector();
  runtime.sceneGraphService = new DefaultSceneGraphService(runtime);
  runtime.textService = new TextService(runtime);
  runtime.geometryUpdaterFactory = geometryUpdaterFactory;
  runtime.CSSPropertySyntaxFactory = CSSPropertySyntaxFactory;
  runtime.styleValueRegistry = new DefaultStyleValueRegistry();
  runtime.layoutRegistry = null;

  var AbstractRendererPlugin = /*#__PURE__*/function () {
    function AbstractRendererPlugin() {
      this.context = void 0;
      this.plugins = [];
    }
    var _proto = AbstractRendererPlugin.prototype;
    _proto.addRenderingPlugin = function addRenderingPlugin(plugin) {
      this.plugins.push(plugin);
      this.context.renderingPlugins.push(plugin);
    };
    _proto.removeAllRenderingPlugins = function removeAllRenderingPlugins() {
      var _this = this;
      this.plugins.forEach(function (plugin) {
        var index = _this.context.renderingPlugins.indexOf(plugin);
        if (index >= 0) {
          _this.context.renderingPlugins.splice(index, 1);
        }
      });
    };
    return AbstractRendererPlugin;
  }();
  var AbstractRenderer = /*#__PURE__*/function () {
    function AbstractRenderer(config) {
      this.plugins = [];
      this.config = void 0;
      this.config = _extends({
        /**
         * only dirty object will cause re-render
         */
        enableDirtyCheck: true,
        enableCulling: false,
        /**
         * enable auto rendering by default
         */
        enableAutoRendering: true,
        /**
         * enable dirty rectangle rendering by default
         */
        enableDirtyRectangleRendering: true,
        enableDirtyRectangleRenderingDebug: false
      }, config);
    }
    var _proto2 = AbstractRenderer.prototype;
    _proto2.registerPlugin = function registerPlugin(plugin) {
      var index = this.plugins.findIndex(function (p) {
        return p === plugin;
      });
      if (index === -1) {
        this.plugins.push(plugin);
      }
    };
    _proto2.unregisterPlugin = function unregisterPlugin(plugin) {
      var index = this.plugins.findIndex(function (p) {
        return p === plugin;
      });
      if (index > -1) {
        this.plugins.splice(index, 1);
      }
    };
    _proto2.getPlugins = function getPlugins() {
      return this.plugins;
    };
    _proto2.getPlugin = function getPlugin(name) {
      return this.plugins.find(function (plugin) {
        return plugin.name === name;
      });
    };
    _proto2.getConfig = function getConfig() {
      return this.config;
    };
    _proto2.setConfig = function setConfig(config) {
      Object.assign(this.config, config);
    };
    return AbstractRenderer;
  }();

  /**
   * apply following rules:
   * 1. `visibility` in scenegraph node
   * 2. other custom culling strategies, eg. frustum culling
   */
  var CullingPlugin = /*#__PURE__*/function () {
    function CullingPlugin(strategies) {
      this.strategies = void 0;
      this.strategies = strategies;
    }
    var _proto = CullingPlugin.prototype;
    _proto.apply = function apply(context) {
      var camera = context.camera,
        renderingService = context.renderingService,
        renderingContext = context.renderingContext;
      var strategies = this.strategies;
      renderingService.hooks.cull.tap(CullingPlugin.tag, function (object) {
        if (object) {
          var cullable = object.cullable;
          // cullable.visible = true;
          // const renderBounds = object.getRenderBounds();
          // if (AABB.isEmpty(renderBounds)) {
          //   cullable.visible = false;
          // } else {
          //   const isShape2D = shape2D.indexOf(object.nodeName as Shape) > -1;
          //   const [p0, p1, p2, p3] = camera.getFrustum().planes;
          //   tmpAABB.setMinMax([-p1.distance, -p3.distance, 0], [p0.distance, p2.distance, 0]);
          //   cullable.visible = isShape2D ? renderBounds.intersects(tmpAABB) : true;
          // }
          if (strategies.length === 0) {
            cullable.visible = renderingContext.unculledEntities.indexOf(object.entity) > -1;
          } else {
            // eg. implemented by g-webgl(frustum culling)
            cullable.visible = strategies.every(function (strategy) {
              return strategy.isVisible(camera, object);
            });
          }
          if (!object.isCulled() && object.isVisible()) {
            return object;
          } else {
            // if (this.renderingContext.renderListLastFrame.indexOf(object) > -1) {
            object.dispatchEvent(new CustomEvent(ElementEvent.CULLED));
            // }
          }

          return null;
        }
        return object;
      });
      renderingService.hooks.afterRender.tap(CullingPlugin.tag, function (object) {
        object.cullable.visibilityPlaneMask = -1;
      });
    };
    return CullingPlugin;
  }();
  CullingPlugin.tag = 'Culling';

  /**
   * Filter dirty renderables and calculate the "dirty rectangle" which will be clear when frame began
   */
  var DirtyCheckPlugin = /*#__PURE__*/function () {
    function DirtyCheckPlugin() {}
    var _proto = DirtyCheckPlugin.prototype;
    _proto.apply = function apply(context) {
      var renderingService = context.renderingService;
      renderingService.hooks.dirtycheck.tap(DirtyCheckPlugin.tag, function (object) {
        if (object) {
          var renderable = object.renderable;
          var isDirty = renderable.dirty || renderingService.disableDirtyRectangleRendering();
          if (isDirty) {
            return object;
          } else {
            return null;
          }
        }
        return object;
      });
    };
    return DirtyCheckPlugin;
  }();
  DirtyCheckPlugin.tag = 'DirtyCheck';

  /**
   * support mouse & touch events
   * @see https://github.com/pixijs/pixi.js/blob/dev/packages/interaction/README.md
   *
   * also provide some extra events such as `drag`
   */
  var EventPlugin = /*#__PURE__*/function () {
    function EventPlugin() {
      var _this = this;
      this.autoPreventDefault = false;
      this.rootPointerEvent = new FederatedPointerEvent(null);
      this.rootWheelEvent = new FederatedWheelEvent(null);
      this.context = void 0;
      this.onPointerMove = function (nativeEvent) {
        var _this$context$renderi, _this$context$renderi2;
        var canvas = (_this$context$renderi = _this.context.renderingContext.root) === null || _this$context$renderi === void 0 ? void 0 : (_this$context$renderi2 = _this$context$renderi.ownerDocument) === null || _this$context$renderi2 === void 0 ? void 0 : _this$context$renderi2.defaultView;
        if (canvas.supportsTouchEvents && nativeEvent.pointerType === 'touch') return;
        var normalizedEvents = _this.normalizeToPointerEvent(nativeEvent, canvas);
        for (var _iterator = _createForOfIteratorHelperLoose(normalizedEvents), _step; !(_step = _iterator()).done;) {
          var normalizedEvent = _step.value;
          var event = _this.bootstrapEvent(_this.rootPointerEvent, normalizedEvent, canvas);
          _this.context.eventService.mapEvent(event);
        }
        _this.setCursor(_this.context.eventService.cursor);
      };
    }
    var _proto = EventPlugin.prototype;
    _proto.apply = function apply(context) {
      var _this2 = this;
      this.context = context;
      var renderingService = context.renderingService;
      var canvas = this.context.renderingContext.root.ownerDocument.defaultView;
      this.context.eventService.setPickHandler( /*#__PURE__*/function () {
        var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(position) {
          var _yield$_this2$context, picked;
          return _regeneratorRuntime().wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  _context.next = 2;
                  return _this2.context.renderingService.hooks.pick.promise({
                    position: position,
                    picked: [],
                    topmost: true // we only concern the topmost element
                  });
                case 2:
                  _yield$_this2$context = _context.sent;
                  picked = _yield$_this2$context.picked;
                  return _context.abrupt("return", picked[0] || null);
                case 5:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));
        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }());
      renderingService.hooks.pointerWheel.tap(EventPlugin.tag, function (nativeEvent) {
        var wheelEvent = _this2.normalizeWheelEvent(nativeEvent);
        _this2.context.eventService.mapEvent(wheelEvent);
      });
      renderingService.hooks.pointerDown.tap(EventPlugin.tag, function (nativeEvent) {
        if (canvas.supportsTouchEvents && nativeEvent.pointerType === 'touch') return;
        var events = _this2.normalizeToPointerEvent(nativeEvent, canvas);
        if (_this2.autoPreventDefault && events[0].isNormalized) {
          var cancelable = nativeEvent.cancelable || !('cancelable' in nativeEvent);
          if (cancelable) {
            nativeEvent.preventDefault();
          }
        }
        for (var _iterator2 = _createForOfIteratorHelperLoose(events), _step2; !(_step2 = _iterator2()).done;) {
          var event = _step2.value;
          var federatedEvent = _this2.bootstrapEvent(_this2.rootPointerEvent, event, canvas);
          _this2.context.eventService.mapEvent(federatedEvent);
        }
        _this2.setCursor(_this2.context.eventService.cursor);
      });
      renderingService.hooks.pointerUp.tap(EventPlugin.tag, function (nativeEvent) {
        if (canvas.supportsTouchEvents && nativeEvent.pointerType === 'touch') return;
        // account for element in SVG
        var $element = _this2.context.contextService.getDomElement();
        var outside = $element && nativeEvent.target && nativeEvent.target !== $element &&
        // @ts-ignore
        $element.contains &&
        // @ts-ignore
        !$element.contains(nativeEvent.target) ? 'outside' : '';
        var normalizedEvents = _this2.normalizeToPointerEvent(nativeEvent, canvas);
        for (var _iterator3 = _createForOfIteratorHelperLoose(normalizedEvents), _step3; !(_step3 = _iterator3()).done;) {
          var normalizedEvent = _step3.value;
          var event = _this2.bootstrapEvent(_this2.rootPointerEvent, normalizedEvent, canvas);
          event.type += outside;
          _this2.context.eventService.mapEvent(event);
        }
        _this2.setCursor(_this2.context.eventService.cursor);
      });
      renderingService.hooks.pointerMove.tap(EventPlugin.tag, this.onPointerMove);
      renderingService.hooks.pointerOver.tap(EventPlugin.tag, this.onPointerMove);
      renderingService.hooks.pointerOut.tap(EventPlugin.tag, this.onPointerMove);
    };
    _proto.getViewportXY = function getViewportXY(nativeEvent) {
      var x;
      var y;
      /**
       * Should account for CSS Transform applied on container.
       * @see https://github.com/antvis/G/issues/1161
       * @see https://developer.mozilla.org/zh-CN/docs/Web/API/MouseEvent/offsetX
       */
      var offsetX = nativeEvent.offsetX,
        offsetY = nativeEvent.offsetY,
        clientX = nativeEvent.clientX,
        clientY = nativeEvent.clientY;
      if (this.context.config.supportsCSSTransform && !isNil(offsetX) && !isNil(offsetY)) {
        x = offsetX;
        y = offsetY;
      } else {
        var point = this.context.eventService.client2Viewport(new Point(clientX, clientY));
        x = point.x;
        y = point.y;
      }
      return {
        x: x,
        y: y
      };
    };
    _proto.bootstrapEvent = function bootstrapEvent(event, nativeEvent, view) {
      event.view = view;
      event.originalEvent = null;
      event.nativeEvent = nativeEvent;
      event.pointerId = nativeEvent.pointerId;
      event.width = nativeEvent.width;
      event.height = nativeEvent.height;
      event.isPrimary = nativeEvent.isPrimary;
      event.pointerType = nativeEvent.pointerType;
      event.pressure = nativeEvent.pressure;
      event.tangentialPressure = nativeEvent.tangentialPressure;
      event.tiltX = nativeEvent.tiltX;
      event.tiltY = nativeEvent.tiltY;
      event.twist = nativeEvent.twist;
      this.transferMouseData(event, nativeEvent);
      var _this$getViewportXY = this.getViewportXY(nativeEvent),
        x = _this$getViewportXY.x,
        y = _this$getViewportXY.y;
      event.viewport.x = x;
      event.viewport.y = y;
      var _this$context$eventSe = this.context.eventService.viewport2Canvas(event.viewport),
        canvasX = _this$context$eventSe.x,
        canvasY = _this$context$eventSe.y;
      event.canvas.x = canvasX;
      event.canvas.y = canvasY;
      event.global.copyFrom(event.canvas);
      event.offset.copyFrom(event.canvas);
      event.isTrusted = nativeEvent.isTrusted;
      if (event.type === 'pointerleave') {
        event.type = 'pointerout';
      }
      if (event.type.startsWith('mouse')) {
        event.type = event.type.replace('mouse', 'pointer');
      }
      if (event.type.startsWith('touch')) {
        event.type = TOUCH_TO_POINTER[event.type] || event.type;
      }
      return event;
    };
    _proto.normalizeWheelEvent = function normalizeWheelEvent(nativeEvent) {
      var event = this.rootWheelEvent;
      this.transferMouseData(event, nativeEvent);
      event.deltaMode = nativeEvent.deltaMode;
      event.deltaX = nativeEvent.deltaX;
      event.deltaY = nativeEvent.deltaY;
      event.deltaZ = nativeEvent.deltaZ;
      var _this$getViewportXY2 = this.getViewportXY(nativeEvent),
        x = _this$getViewportXY2.x,
        y = _this$getViewportXY2.y;
      event.viewport.x = x;
      event.viewport.y = y;
      var _this$context$eventSe2 = this.context.eventService.viewport2Canvas(event.viewport),
        canvasX = _this$context$eventSe2.x,
        canvasY = _this$context$eventSe2.y;
      event.canvas.x = canvasX;
      event.canvas.y = canvasY;
      event.global.copyFrom(event.canvas);
      event.offset.copyFrom(event.canvas);
      event.nativeEvent = nativeEvent;
      event.type = nativeEvent.type;
      return event;
    }
    /**
     * Transfers base & mouse event data from the nativeEvent to the federated event.
     */;
    _proto.transferMouseData = function transferMouseData(event, nativeEvent) {
      event.isTrusted = nativeEvent.isTrusted;
      event.srcElement = nativeEvent.srcElement;
      event.timeStamp = performance.now();
      event.type = nativeEvent.type;
      event.altKey = nativeEvent.altKey;
      event.metaKey = nativeEvent.metaKey;
      event.shiftKey = nativeEvent.shiftKey;
      event.ctrlKey = nativeEvent.ctrlKey;
      event.button = nativeEvent.button;
      event.buttons = nativeEvent.buttons;
      event.client.x = nativeEvent.clientX;
      event.client.y = nativeEvent.clientY;
      event.movement.x = nativeEvent.movementX;
      event.movement.y = nativeEvent.movementY;
      event.page.x = nativeEvent.pageX;
      event.page.y = nativeEvent.pageY;
      event.screen.x = nativeEvent.screenX;
      event.screen.y = nativeEvent.screenY;
      event.relatedTarget = null;
    };
    _proto.setCursor = function setCursor(cursor) {
      this.context.contextService.applyCursorStyle(cursor || this.context.config.cursor || 'default');
    };
    _proto.normalizeToPointerEvent = function normalizeToPointerEvent(event, canvas) {
      var normalizedEvents = [];
      if (canvas.isTouchEvent(event)) {
        for (var i = 0; i < event.changedTouches.length; i++) {
          var touch = event.changedTouches[i];
          // use changedTouches instead of touches since touchend has no touches
          // @see https://stackoverflow.com/a/10079076
          if (isUndefined(touch.button)) touch.button = 0;
          if (isUndefined(touch.buttons)) touch.buttons = 1;
          if (isUndefined(touch.isPrimary)) {
            touch.isPrimary = event.touches.length === 1 && event.type === 'touchstart';
          }
          if (isUndefined(touch.width)) touch.width = touch.radiusX || 1;
          if (isUndefined(touch.height)) touch.height = touch.radiusY || 1;
          if (isUndefined(touch.tiltX)) touch.tiltX = 0;
          if (isUndefined(touch.tiltY)) touch.tiltY = 0;
          if (isUndefined(touch.pointerType)) touch.pointerType = 'touch';
          // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Touch/identifier
          if (isUndefined(touch.pointerId)) touch.pointerId = touch.identifier || 0;
          if (isUndefined(touch.pressure)) touch.pressure = touch.force || 0.5;
          if (isUndefined(touch.twist)) touch.twist = 0;
          if (isUndefined(touch.tangentialPressure)) touch.tangentialPressure = 0;
          touch.isNormalized = true;
          touch.type = event.type;
          normalizedEvents.push(touch);
        }
      } else if (canvas.isMouseEvent(event)) {
        var tempEvent = event;
        if (isUndefined(tempEvent.isPrimary)) tempEvent.isPrimary = true;
        if (isUndefined(tempEvent.width)) tempEvent.width = 1;
        if (isUndefined(tempEvent.height)) tempEvent.height = 1;
        if (isUndefined(tempEvent.tiltX)) tempEvent.tiltX = 0;
        if (isUndefined(tempEvent.tiltY)) tempEvent.tiltY = 0;
        if (isUndefined(tempEvent.pointerType)) tempEvent.pointerType = 'mouse';
        if (isUndefined(tempEvent.pointerId)) tempEvent.pointerId = MOUSE_POINTER_ID;
        if (isUndefined(tempEvent.pressure)) tempEvent.pressure = 0.5;
        if (isUndefined(tempEvent.twist)) tempEvent.twist = 0;
        if (isUndefined(tempEvent.tangentialPressure)) tempEvent.tangentialPressure = 0;
        tempEvent.isNormalized = true;
        normalizedEvents.push(tempEvent);
      } else {
        normalizedEvents.push(event);
      }
      return normalizedEvents;
    };
    return EventPlugin;
  }();
  EventPlugin.tag = 'Event';

  // group is not a 2d shape
  var shape2D = [Shape.CIRCLE, Shape.ELLIPSE, Shape.IMAGE, Shape.RECT, Shape.LINE, Shape.POLYLINE, Shape.POLYGON, Shape.TEXT, Shape.PATH, Shape.HTML];
  var FrustumCullingStrategy = /*#__PURE__*/function () {
    function FrustumCullingStrategy() {}
    var _proto = FrustumCullingStrategy.prototype;
    _proto.isVisible = function isVisible(camera, object) {
      var _object$parentNode, _object$parentNode$cu;
      // return true;
      var cullable = object.cullable;
      if (!cullable.enable) {
        return true;
      }
      var renderBounds = object.getRenderBounds();
      if (AABB.isEmpty(renderBounds)) {
        return false;
      }
      // get VP matrix from camera
      var frustum = camera.getFrustum();
      var parentVisibilityPlaneMask = (_object$parentNode = object.parentNode) === null || _object$parentNode === void 0 ? void 0 : (_object$parentNode$cu = _object$parentNode.cullable) === null || _object$parentNode$cu === void 0 ? void 0 : _object$parentNode$cu.visibilityPlaneMask;
      cullable.visibilityPlaneMask = this.computeVisibilityWithPlaneMask(object, renderBounds, parentVisibilityPlaneMask || Mask.INDETERMINATE, frustum.planes);
      cullable.visible = cullable.visibilityPlaneMask !== Mask.OUTSIDE;
      return cullable.visible;
    }
    /**
     *
     * @see「Optimized View Frustum Culling Algorithms for Bounding Boxes」
     * @see https://github.com/antvis/GWebGPUEngine/issues/3
     *
     * * 基础相交测试 the basic intersection test
     * * 标记 masking @see https://cesium.com/blog/2015/08/04/fast-hierarchical-culling/
     * * TODO: 平面一致性测试 the plane-coherency test
     * * TODO: 支持 mesh 指定自身的剔除策略，参考 Babylon.js @see https://doc.babylonjs.com/how_to/optimizing_your_scene#changing-mesh-culling-strategy
     *
     * @param aabb aabb
     * @param parentPlaneMask mask of parent
     * @param planes planes of frustum
     */;
    _proto.computeVisibilityWithPlaneMask = function computeVisibilityWithPlaneMask(object, aabb, parentPlaneMask, planes) {
      if (parentPlaneMask === Mask.OUTSIDE || parentPlaneMask === Mask.INSIDE) {
        // 父节点完全位于视锥内或者外部，直接返回
        return parentPlaneMask;
      }
      // Start with MASK_INSIDE (all zeros) so that after the loop, the return value can be compared with MASK_INSIDE.
      // (Because if there are fewer than 31 planes, the upper bits wont be changed.)
      var mask = Mask.INSIDE;
      var isShape2D = shape2D.indexOf(object.nodeName) > -1;
      // Use viewport culling for 2D shapes
      // @see https://github.com/antvis/g/issues/914
      for (var k = 0, len = planes.length; k < len; ++k) {
        // For k greater than 31 (since 31 is the maximum number of INSIDE/INTERSECTING bits we can store), skip the optimization.
        var flag = 1 << k;
        if ((parentPlaneMask & flag) === 0) {
          // 父节点处于当前面内部，可以跳过
          continue;
        }
        // skip near & far planes when testing 2D shapes
        if (isShape2D && (k === 4 || k === 5)) {
          continue;
        }
        // p-vertex n-vertex <-|plane p-vertex n-vertex
        // 使用 p-vertex 和 n-vertex 加速，避免进行平面和 aabb 全部顶点的相交检测
        var _planes$k = planes[k],
          normal = _planes$k.normal,
          distance = _planes$k.distance;
        if (dot(normal, aabb.getPositiveFarPoint(planes[k])) + distance < 0) {
          return Mask.OUTSIDE;
        }
        if (dot(normal, aabb.getNegativeFarPoint(planes[k])) + distance < 0) {
          // 和当前面相交，对应位置为1，继续检测下一个面
          mask |= flag;
        }
      }
      return mask;
    };
    return FrustumCullingStrategy;
  }();

  var PrepareRendererPlugin = /*#__PURE__*/function () {
    function PrepareRendererPlugin() {
      this.rBush = void 0;
      this.toSync = new Set();
    }
    var _proto = PrepareRendererPlugin.prototype;
    // private isFirstTimeRendering = true;
    _proto.apply = function apply(context) {
      var _this = this;
      var renderingService = context.renderingService,
        renderingContext = context.renderingContext,
        rBushRoot = context.rBushRoot;
      var canvas = renderingContext.root.ownerDocument.defaultView;
      this.rBush = rBushRoot;
      var handleAttributeChanged = function handleAttributeChanged(e) {
        var object = e.target;
        object.renderable.dirty = true;
        renderingService.dirtify();
      };
      var handleBoundsChanged = function handleBoundsChanged(e) {
        var affectChildren = e.detail.affectChildren;
        var object = e.target;
        if (affectChildren) {
          object.forEach(function (node) {
            _this.toSync.add(node);
          });
        }
        var p = object;
        while (p) {
          if (p.renderable) {
            _this.toSync.add(p);
          }
          p = p.parentElement;
        }
        // this.pushToSync(e.composedPath().slice(0, -2) as DisplayObject[]);
        renderingService.dirtify();
      };
      var handleMounted = function handleMounted(e) {
        var object = e.target;
        // recalc style values
        runtime.styleValueRegistry.recalc(object);
        runtime.sceneGraphService.dirtifyToRoot(object);
        renderingService.dirtify();
      };
      var handleUnmounted = function handleUnmounted(e) {
        var object = e.target;
        var rBushNode = object.rBushNode;
        if (rBushNode.aabb) {
          _this.rBush.remove(rBushNode.aabb);
        }
        _this.toSync.delete(object);
        runtime.sceneGraphService.dirtifyToRoot(object);
        renderingService.dirtify();
      };
      renderingService.hooks.init.tapPromise(PrepareRendererPlugin.tag, /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee() {
        return _regeneratorRuntime().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                canvas.addEventListener(ElementEvent.MOUNTED, handleMounted);
                canvas.addEventListener(ElementEvent.UNMOUNTED, handleUnmounted);
                canvas.addEventListener(ElementEvent.ATTR_MODIFIED, handleAttributeChanged);
                canvas.addEventListener(ElementEvent.BOUNDS_CHANGED, handleBoundsChanged);
              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      })));
      renderingService.hooks.destroy.tap(PrepareRendererPlugin.tag, function () {
        canvas.removeEventListener(ElementEvent.MOUNTED, handleMounted);
        canvas.removeEventListener(ElementEvent.UNMOUNTED, handleUnmounted);
        canvas.removeEventListener(ElementEvent.ATTR_MODIFIED, handleAttributeChanged);
        canvas.removeEventListener(ElementEvent.BOUNDS_CHANGED, handleBoundsChanged);
      });
      renderingService.hooks.endFrame.tap(PrepareRendererPlugin.tag, function () {
        // if (this.isFirstTimeRendering) {
        //   // @see https://github.com/antvis/G/issues/1117
        //   setTimeout(() => this.syncRTree());
        //   this.isFirstTimeRendering = false;
        // } else {
        //   this.syncRTree();
        // }
        _this.syncRTree();
      });
    };
    _proto.syncRTree = function syncRTree() {
      var _this2 = this;
      // bounds changed, need re-inserting its children
      var bulk = [];
      Array.from(this.toSync)
      // some objects may be removed since last frame
      .filter(function (object) {
        return object.isConnected;
      }).forEach(function (node) {
        var rBushNode = node.rBushNode;
        // clear dirty node
        if (rBushNode && rBushNode.aabb) {
          _this2.rBush.remove(rBushNode.aabb);
        }
        var renderBounds = node.getRenderBounds();
        if (renderBounds) {
          var _renderBounds$getMin = renderBounds.getMin(),
            minX = _renderBounds$getMin[0],
            minY = _renderBounds$getMin[1];
          var _renderBounds$getMax = renderBounds.getMax(),
            maxX = _renderBounds$getMax[0],
            maxY = _renderBounds$getMax[1];
          if (!rBushNode.aabb) {
            // @ts-ignore
            rBushNode.aabb = {};
          }
          rBushNode.aabb.id = node.entity;
          rBushNode.aabb.minX = minX;
          rBushNode.aabb.minY = minY;
          rBushNode.aabb.maxX = maxX;
          rBushNode.aabb.maxY = maxY;
        }
        if (rBushNode.aabb) {
          // TODO: NaN occurs when width/height of Rect is 0
          if (!isNaN(rBushNode.aabb.maxX) && !isNaN(rBushNode.aabb.maxX) && !isNaN(rBushNode.aabb.minX) && !isNaN(rBushNode.aabb.minY)) {
            bulk.push(rBushNode.aabb);
          }
        }
      });
      // use bulk inserting, which is ~2-3 times faster
      // @see https://github.com/mourner/rbush#bulk-inserting-data
      this.rBush.load(bulk);
      bulk.length = 0;
      this.toSync.clear();
    };
    return PrepareRendererPlugin;
  }();
  PrepareRendererPlugin.tag = 'Prepare';

  var CanvasEvent;
  (function (CanvasEvent) {
    CanvasEvent["READY"] = "ready";
    CanvasEvent["BEFORE_RENDER"] = "beforerender";
    CanvasEvent["AFTER_RENDER"] = "afterrender";
    CanvasEvent["BEFORE_DESTROY"] = "beforedestroy";
    CanvasEvent["AFTER_DESTROY"] = "afterdestroy";
    CanvasEvent["RESIZE"] = "resize";
    CanvasEvent["DIRTY_RECTANGLE"] = "dirtyrectangle";
  })(CanvasEvent || (CanvasEvent = {}));
  var DEFAULT_CAMERA_Z = 500;
  var DEFAULT_CAMERA_NEAR = 0.1;
  var DEFAULT_CAMERA_FAR = 1000;
  /**
   * reuse custom event preventing from re-create them in every frame
   */
  var mountedEvent = new CustomEvent(ElementEvent.MOUNTED);
  var unmountedEvent = new CustomEvent(ElementEvent.UNMOUNTED);
  var beforeRenderEvent = new CustomEvent(CanvasEvent.BEFORE_RENDER);
  var afterRenderEvent = new CustomEvent(CanvasEvent.AFTER_RENDER);
  /**
   * can be treated like Window in DOM
   * provide some extra methods like `window`, such as:
   * * `window.requestAnimationFrame`
   * * `window.devicePixelRatio`
   *
   * prototype chains: Canvas(Window) -> EventTarget
   */
  var Canvas = /*#__PURE__*/function (_EventTarget) {
    _inheritsLoose(Canvas, _EventTarget);
    /**
     * window.document
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/window/requestAnimationFrame
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/cancelAnimationFrame
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio
     */

    /**
     * whether the runtime supports PointerEvent?
     * if not, the event system won't trigger pointer events like `pointerdown`
     */

    /**
     * whether the runtime supports TouchEvent?
     * if not, the event system won't trigger touch events like `touchstart`
     */

    /**
     * is this native event a TouchEvent?
     */

    /**
     * is this native event a MouseEvent?
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element
     */

    /**
     * rAF in auto rendering
     */

    function Canvas(config) {
      var _this;
      _this = _EventTarget.call(this) || this;
      // create document
      _this.document = void 0;
      _this.customElements = void 0;
      _this.requestAnimationFrame = void 0;
      _this.cancelAnimationFrame = void 0;
      _this.devicePixelRatio = void 0;
      _this.supportsPointerEvents = void 0;
      _this.supportsTouchEvents = void 0;
      _this.isTouchEvent = void 0;
      _this.isMouseEvent = void 0;
      _this.Element = DisplayObject;
      _this.frameId = void 0;
      _this.inited = false;
      _this.readyPromise = void 0;
      _this.resolveReadyPromise = void 0;
      _this.context = {};
      _this.document = new Document();
      _this.document.defaultView = _assertThisInitialized(_this);
      // create registry of custom elements
      _this.customElements = new CustomElementRegistry();
      var container = config.container,
        canvas = config.canvas,
        offscreenCanvas = config.offscreenCanvas,
        width = config.width,
        height = config.height,
        devicePixelRatio = config.devicePixelRatio,
        renderer = config.renderer,
        background = config.background,
        cursor = config.cursor,
        document = config.document,
        requestAnimationFrame = config.requestAnimationFrame,
        cancelAnimationFrame = config.cancelAnimationFrame,
        createImage = config.createImage,
        supportsPointerEvents = config.supportsPointerEvents,
        supportsTouchEvents = config.supportsTouchEvents,
        supportsCSSTransform = config.supportsCSSTransform,
        isTouchEvent = config.isTouchEvent,
        isMouseEvent = config.isMouseEvent;
      cleanExistedCanvas(container, _assertThisInitialized(_this));
      var canvasWidth = width;
      var canvasHeight = height;
      var dpr = devicePixelRatio;
      // use user-defined <canvas> or OffscreenCanvas
      if (canvas) {
        // infer width & height with dpr
        dpr = devicePixelRatio || isBrowser && window.devicePixelRatio || 1;
        dpr = dpr >= 1 ? Math.ceil(dpr) : 1;
        canvasWidth = width || getWidth(canvas) || canvas.width / dpr;
        canvasHeight = height || getHeight(canvas) || canvas.height / dpr;
      }
      /**
       * implements `Window` interface
       */
      _this.devicePixelRatio = dpr;
      _this.requestAnimationFrame = requestAnimationFrame !== null && requestAnimationFrame !== void 0 ? requestAnimationFrame : raf.bind(globalThis);
      _this.cancelAnimationFrame = cancelAnimationFrame !== null && cancelAnimationFrame !== void 0 ? cancelAnimationFrame : caf.bind(globalThis);
      /**
       * limits query
       */
      // the following feature-detect from hammer.js
      // @see https://github.com/hammerjs/hammer.js/blob/master/src/inputjs/input-consts.js#L5
      _this.supportsTouchEvents = supportsTouchEvents !== null && supportsTouchEvents !== void 0 ? supportsTouchEvents : 'ontouchstart' in globalThis;
      _this.supportsPointerEvents = supportsPointerEvents !== null && supportsPointerEvents !== void 0 ? supportsPointerEvents : !!globalThis.PointerEvent;
      _this.isTouchEvent = isTouchEvent !== null && isTouchEvent !== void 0 ? isTouchEvent : function (event) {
        return _this.supportsTouchEvents && event instanceof globalThis.TouchEvent;
      };
      _this.isMouseEvent = isMouseEvent !== null && isMouseEvent !== void 0 ? isMouseEvent : function (event) {
        return !globalThis.MouseEvent || event instanceof globalThis.MouseEvent && (!_this.supportsPointerEvents || !(event instanceof globalThis.PointerEvent));
      };
      _this.initRenderingContext({
        container: container,
        canvas: canvas,
        width: canvasWidth,
        height: canvasHeight,
        renderer: renderer,
        offscreenCanvas: offscreenCanvas,
        devicePixelRatio: dpr,
        cursor: cursor || 'default',
        background: background || 'transparent',
        createImage: createImage,
        document: document,
        supportsCSSTransform: supportsCSSTransform
      });
      _this.initDefaultCamera(canvasWidth, canvasHeight);
      _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee() {
        return _regeneratorRuntime().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return _this.initRenderer(renderer);
              case 2:
                _this.dispatchEvent(new CustomEvent(CanvasEvent.READY));
                if (_this.readyPromise) {
                  _this.resolveReadyPromise();
                }
              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }))();
      return _this;
    }
    var _proto = Canvas.prototype;
    _proto.initRenderingContext = function initRenderingContext(mergedConfig) {
      this.context.config = mergedConfig;
      // bind rendering context, shared by all renderers
      this.context.renderingContext = {
        /**
         * the root node in scene graph
         */
        root: this.document.documentElement,
        renderListCurrentFrame: [],
        unculledEntities: [],
        renderReasons: new Set(),
        force: false,
        dirty: false
      };
    };
    _proto.initDefaultCamera = function initDefaultCamera(width, height) {
      var _this2 = this;
      // set a default ortho camera
      var camera = new runtime.CameraContribution();
      camera.setType(CameraType.EXPLORING, CameraTrackingMode.DEFAULT).setPosition(width / 2, height / 2, DEFAULT_CAMERA_Z).setFocalPoint(width / 2, height / 2, 0).setOrthographic(width / -2, width / 2, height / 2, height / -2, DEFAULT_CAMERA_NEAR, DEFAULT_CAMERA_FAR);
      // keep ref since it will use raf in camera animation
      camera.canvas = this;
      // redraw when camera changed
      camera.eventEmitter.on(CameraEvent.UPDATED, function () {
        _this2.context.renderingContext.renderReasons.add(RenderReason.CAMERA_CHANGED);
      });
      // bind camera
      this.context.camera = camera;
    };
    _proto.getConfig = function getConfig() {
      return this.context.config;
    }
    /**
     * get the root displayObject in scenegraph
     * @alias this.document.documentElement
     */;
    _proto.getRoot = function getRoot() {
      return this.document.documentElement;
    }
    /**
     * get the camera of canvas
     */;
    _proto.getCamera = function getCamera() {
      return this.context.camera;
    };
    _proto.getContextService = function getContextService() {
      return this.context.contextService;
    };
    _proto.getEventService = function getEventService() {
      return this.context.eventService;
    };
    _proto.getRenderingService = function getRenderingService() {
      return this.context.renderingService;
    };
    _proto.getRenderingContext = function getRenderingContext() {
      return this.context.renderingContext;
    };
    _proto.getStats = function getStats() {
      return this.getRenderingService().getStats();
    }
    // /**
    //  * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Window/getComputedStyle
    //  */
    // getComputedStyle(node: DisplayObject) {
    //   return node.computedStyle;
    // }
    ;
    _proto.destroy = function destroy(destroyScenegraph, skipTriggerEvent) {
      if (destroyScenegraph === void 0) {
        destroyScenegraph = true;
      }
      if (skipTriggerEvent === void 0) {
        skipTriggerEvent = false;
      }
      if (!skipTriggerEvent) {
        this.dispatchEvent(new CustomEvent(CanvasEvent.BEFORE_DESTROY));
      }
      if (this.frameId) {
        var _cancelRAF = this.getConfig().cancelAnimationFrame || cancelAnimationFrame;
        _cancelRAF(this.frameId);
      }
      // unmount all children
      var root = this.getRoot();
      this.unmountChildren(root);
      // root.children.forEach((child: DisplayObject) => {
      //   this.unmountChildren(child);
      // });
      if (destroyScenegraph) {
        // destroy Document
        this.document.destroy();
      }
      // destroy services
      this.getRenderingService().destroy();
      this.getContextService().destroy();
      if (!skipTriggerEvent) {
        this.dispatchEvent(new CustomEvent(CanvasEvent.AFTER_DESTROY));
      }
    }
    /**
     * compatible with G 3.0
     * @deprecated
     * @alias resize
     */;
    _proto.changeSize = function changeSize(width, height) {
      this.resize(width, height);
    };
    _proto.resize = function resize(width, height) {
      // update canvas' config
      var canvasConfig = this.context.config;
      canvasConfig.width = width;
      canvasConfig.height = height;
      // resize context
      this.getContextService().resize(width, height);
      // resize camera
      var camera = this.context.camera;
      var projectionMode = camera.getProjectionMode();
      camera.setPosition(width / 2, height / 2, DEFAULT_CAMERA_Z).setFocalPoint(width / 2, height / 2, 0);
      if (projectionMode === CameraProjectionMode.ORTHOGRAPHIC) {
        camera.setOrthographic(width / -2, width / 2, height / 2, height / -2, camera.getNear(), camera.getFar());
      } else {
        camera.setAspect(width / height);
      }
      this.dispatchEvent(new CustomEvent(CanvasEvent.RESIZE, {
        width: width,
        height: height
      }));
    }
    // proxy to document.documentElement
    ;
    _proto.appendChild = function appendChild(child, index) {
      return this.document.documentElement.appendChild(child, index);
    };
    _proto.insertBefore = function insertBefore(newChild, refChild) {
      return this.document.documentElement.insertBefore(newChild, refChild);
    };
    _proto.removeChild = function removeChild(child) {
      return this.document.documentElement.removeChild(child);
    }
    /**
     * Remove all children which can be appended to its original parent later again.
     */;
    _proto.removeChildren = function removeChildren() {
      this.document.documentElement.removeChildren();
    }
    /**
     * Recursively destroy all children which can not be appended to its original parent later again.
     */;
    _proto.destroyChildren = function destroyChildren() {
      this.document.documentElement.destroyChildren();
    };
    _proto.render = function render() {
      this.dispatchEvent(beforeRenderEvent);
      var renderingService = this.getRenderingService();
      renderingService.render(this.getConfig());
      this.dispatchEvent(afterRenderEvent);
    };
    _proto.run = function run() {
      var _this3 = this;
      var tick = function tick() {
        _this3.render();
        _this3.frameId = requestAnimationFrame(tick);
      };
      tick();
    };
    _proto.initRenderer = /*#__PURE__*/function () {
      var _initRenderer = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee2(renderer) {
        return _regeneratorRuntime().wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (renderer) {
                  _context2.next = 2;
                  break;
                }
                throw new Error('Renderer is required.');
              case 2:
                // reset
                this.inited = false;
                this.readyPromise = undefined;
                // FIXME: should re-create here?
                this.context.rBushRoot = new RBush();
                // reset rendering plugins
                this.context.renderingPlugins = [];
                this.context.renderingPlugins.push(new EventPlugin(), new PrepareRendererPlugin(), new DirtyCheckPlugin(), new CullingPlugin([new FrustumCullingStrategy()]));
                //
                this.loadRendererContainerModule(renderer);
                // init context service
                this.context.contextService = new this.context.ContextService(_extends({}, runtime, this.context));
                // init rendering service
                this.context.renderingService = new RenderingService(runtime, this.context);
                // init event service
                this.context.eventService = new EventService(runtime, this.context);
                this.context.eventService.init();
                _context2.next = 14;
                return this.context.contextService.init();
              case 14:
                _context2.next = 16;
                return this.context.renderingService.init();
              case 16:
                this.inited = true;
                this.getRoot().forEach(function (node) {
                  var renderable = node.renderable;
                  if (renderable) {
                    renderable.renderBoundsDirty = true;
                    renderable.boundsDirty = true;
                    renderable.dirty = true;
                  }
                });
                // keep current scenegraph unchanged, just trigger mounted event
                this.mountChildren(this.getRoot());
                if (renderer.getConfig().enableAutoRendering) {
                  this.run();
                }
              case 20:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
      function initRenderer(_x) {
        return _initRenderer.apply(this, arguments);
      }
      return initRenderer;
    }();
    _proto.loadRendererContainerModule = function loadRendererContainerModule(renderer) {
      var _this4 = this;
      // load other container modules provided by g-canvas/g-svg/g-webgl
      var plugins = renderer.getPlugins();
      plugins.forEach(function (plugin) {
        plugin.context = _this4.context;
        plugin.init();
      });
    };
    _proto.setRenderer = /*#__PURE__*/function () {
      var _setRenderer = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee3(renderer) {
        var canvasConfig, oldRenderer;
        return _regeneratorRuntime().wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                // update canvas' config
                canvasConfig = this.getConfig();
                if (!(canvasConfig.renderer === renderer)) {
                  _context3.next = 3;
                  break;
                }
                return _context3.abrupt("return");
              case 3:
                oldRenderer = canvasConfig.renderer;
                canvasConfig.renderer = renderer;
                // keep all children undestroyed
                this.destroy(false, true);
                // destroy all plugins, reverse will mutate origin array
                [].concat(oldRenderer === null || oldRenderer === void 0 ? void 0 : oldRenderer.getPlugins()).reverse().forEach(function (plugin) {
                  plugin.destroy();
                });
                _context3.next = 9;
                return this.initRenderer(renderer);
              case 9:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));
      function setRenderer(_x2) {
        return _setRenderer.apply(this, arguments);
      }
      return setRenderer;
    }();
    _proto.setCursor = function setCursor(cursor) {
      var canvasConfig = this.getConfig();
      canvasConfig.cursor = cursor;
      this.getContextService().applyCursorStyle(cursor);
    };
    _proto.unmountChildren = function unmountChildren(parent) {
      var _this5 = this;
      // unmountChildren recursively
      parent.childNodes.forEach(function (child) {
        _this5.unmountChildren(child);
      });
      if (this.inited) {
        if (parent.isMutationObserved) {
          parent.dispatchEvent(unmountedEvent);
        } else {
          unmountedEvent.target = parent;
          this.dispatchEvent(unmountedEvent, true);
        }
        // skip document.documentElement
        if (parent !== this.document.documentElement) {
          parent.ownerDocument = null;
        }
        parent.isConnected = false;
      }
      // trigger after unmounted
      if (parent.isCustomElement) {
        if (parent.disconnectedCallback) {
          parent.disconnectedCallback();
        }
      }
    };
    _proto.mountChildren = function mountChildren(parent) {
      var _this6 = this;
      if (this.inited) {
        if (!parent.isConnected) {
          parent.ownerDocument = this.document;
          parent.isConnected = true;
          if (parent.isMutationObserved) {
            parent.dispatchEvent(mountedEvent);
          } else {
            mountedEvent.target = parent;
            this.dispatchEvent(mountedEvent, true);
          }
        }
      } else {
        console.warn("[g]: You are trying to call `canvas.appendChild` before canvas' initialization finished. You can either await `canvas.ready` or listen to `CanvasEvent.READY` manually.", 'appended child: ', parent.nodeName);
      }
      // recursively mount children
      parent.childNodes.forEach(function (child) {
        _this6.mountChildren(child);
      });
      // trigger after mounted
      if (parent.isCustomElement) {
        if (parent.connectedCallback) {
          parent.connectedCallback();
        }
      }
    };
    _proto.client2Viewport = function client2Viewport(client) {
      return this.getEventService().client2Viewport(client);
    };
    _proto.viewport2Client = function viewport2Client(canvas) {
      return this.getEventService().viewport2Client(canvas);
    };
    _proto.viewport2Canvas = function viewport2Canvas(viewport) {
      return this.getEventService().viewport2Canvas(viewport);
    };
    _proto.canvas2Viewport = function canvas2Viewport(canvas) {
      return this.getEventService().canvas2Viewport(canvas);
    }
    /**
     * @deprecated
     * @alias client2Viewport
     */;
    _proto.getPointByClient = function getPointByClient(clientX, clientY) {
      return this.client2Viewport({
        x: clientX,
        y: clientY
      });
    }
    /**
     * @deprecated
     * @alias viewport2Client
     */;
    _proto.getClientByPoint = function getClientByPoint(x, y) {
      return this.viewport2Client({
        x: x,
        y: y
      });
    };
    _createClass$1(Canvas, [{
      key: "ready",
      get: function get() {
        var _this7 = this;
        if (!this.readyPromise) {
          this.readyPromise = new Promise(function (resolve) {
            _this7.resolveReadyPromise = function () {
              resolve(_this7);
            };
          });
          if (this.inited) {
            this.resolveReadyPromise();
          }
        }
        return this.readyPromise;
      }
    }]);
    return Canvas;
  }(EventTarget);

  function _getPrototypeOf(o) {
    _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) {
      return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf(o);
  }

  function _superPropBase(object, property) {
    while (!Object.prototype.hasOwnProperty.call(object, property)) {
      object = _getPrototypeOf(object);
      if (object === null) break;
    }
    return object;
  }

  function _get() {
    if (typeof Reflect !== "undefined" && Reflect.get) {
      _get = Reflect.get.bind();
    } else {
      _get = function _get(target, property, receiver) {
        var base = _superPropBase(target, property);
        if (!base) return;
        var desc = Object.getOwnPropertyDescriptor(base, property);
        if (desc.get) {
          return desc.get.call(arguments.length < 3 ? target : receiver);
        }
        return desc.value;
      };
    }
    return _get.apply(this, arguments);
  }

  function _setPrototypeOf$1(o, p) {
    _setPrototypeOf$1 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$1(o, p);
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        writable: true,
        configurable: true
      }
    });
    Object.defineProperty(subClass, "prototype", {
      writable: false
    });
    if (superClass) _setPrototypeOf$1(subClass, superClass);
  }

  function _isNativeReflectConstruct() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
      Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
      return true;
    } catch (e) {
      return false;
    }
  }

  function _typeof(obj) {
    "@babel/helpers - typeof";

    return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) {
      return typeof obj;
    } : function (obj) {
      return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    }, _typeof(obj);
  }

  function _assertThisInitialized$1(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
  }

  function _possibleConstructorReturn(self, call) {
    if (call && (_typeof(call) === "object" || typeof call === "function")) {
      return call;
    } else if (call !== void 0) {
      throw new TypeError("Derived constructors may only return object or undefined");
    }
    return _assertThisInitialized$1(self);
  }

  function _createSuper(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct();
    return function _createSuperInternal() {
      var Super = _getPrototypeOf(Derived),
        result;
      if (hasNativeReflectConstruct) {
        var NewTarget = _getPrototypeOf(this).constructor;
        result = Reflect.construct(Super, arguments, NewTarget);
      } else {
        result = Super.apply(this, arguments);
      }
      return _possibleConstructorReturn(this, result);
    };
  }

  var polarToCartesian = function polarToCartesian(centerX, centerY, radius, angleInRadian) {
    return {
      x: centerX + radius * Math.cos(angleInRadian),
      y: centerY + radius * Math.sin(angleInRadian)
    };
  };

  var Arc = /*#__PURE__*/function (_Path) {
    _inherits(Arc, _Path);
    var _super = _createSuper(Arc);
    function Arc(config) {
      var _this;
      _classCallCheck(this, Arc);
      _this = _super.call(this, config);
      _this.updatePath();
      return _this;
    }
    _createClass(Arc, [{
      key: "setAttribute",
      value: function setAttribute(name, value, force) {
        _get(_getPrototypeOf(Arc.prototype), "setAttribute", this).call(this, name, value, force);
        if (['startAngle', 'endAngle', 'r'].indexOf(name) > -1) {
          this.updatePath();
        }
      }
    }, {
      key: "updatePath",
      value: function updatePath() {
        var _this$parsedStyle = this.parsedStyle,
          x = _this$parsedStyle.x,
          y = _this$parsedStyle.y,
          startAngle = _this$parsedStyle.startAngle,
          endAngle = _this$parsedStyle.endAngle,
          r = _this$parsedStyle.r,
          anticlockwise = _this$parsedStyle.anticlockwise;
        var path = this.createPath(x, y, startAngle ? deg2rad(startAngle) : 0, endAngle ? deg2rad(endAngle) : Math.PI * 2, r ? r : 0, anticlockwise);
        _get(_getPrototypeOf(Arc.prototype), "setAttribute", this).call(this, 'path', path);
      }
    }, {
      key: "createPath",
      value: function createPath(x, y, startAngle, endAngle, r, anticlockwise) {
        if (endAngle < startAngle) {
          endAngle = endAngle + Math.PI * 2;
        }
        if (r <= 0) {
          return null;
        }
        var start = polarToCartesian(x, y, r, startAngle);
        var end = polarToCartesian(x, y, r, endAngle);
        if (isNumberEqual(endAngle - startAngle, Math.PI * 2)) {
          var middlePoint = polarToCartesian(x, y, r, startAngle + Math.PI);
          return [['M', start.x, start.y], ['A', r, r, 0, 1, 1, middlePoint.x, middlePoint.y], ['A', r, r, 0, 1, 1, start.x, start.y], ['A', r, r, 0, 1, 0, middlePoint.x, middlePoint.y], ['A', r, r, 0, 1, 0, start.x, start.y], ['Z']];
        }
        var arcSweep = endAngle - startAngle <= Math.PI ? 0 : 1;
        return [['M', start.x, start.y], ['A', r, r, 0, arcSweep, anticlockwise ? 0 : 1, end.x, end.y]];
      }
    }]);
    return Arc;
  }(Path);

  var SYMBOLS = {
    circle: function circle(x, y, r) {
      return [['M', x - r, y], ['A', r, r, 0, 1, 0, x + r, y], ['A', r, r, 0, 1, 0, x - r, y]];
    },
    square: function square(x, y, r) {
      return [['M', x - r, y - r], ['L', x + r, y - r], ['L', x + r, y + r], ['L', x - r, y + r], ['Z']];
    }
  };
  var Marker = /*#__PURE__*/function (_Path) {
    _inherits(Marker, _Path);
    var _super = _createSuper(Marker);
    function Marker(config) {
      var _this;
      _classCallCheck(this, Marker);
      _this = _super.call(this, config);
      _this.updatePath();
      return _this;
    }
    _createClass(Marker, [{
      key: "setAttribute",
      value: function setAttribute(name, value, force) {
        _get(_getPrototypeOf(Marker.prototype), "setAttribute", this).call(this, name, value, force);
        if (['x', 'y', 'symbol', 'radius'].indexOf(name) > -1) {
          this.updatePath();
        }
      }
    }, {
      key: "updatePath",
      value: function updatePath() {
        var _this$parsedStyle = this.parsedStyle,
          _this$parsedStyle$rad = _this$parsedStyle.radius,
          radius = _this$parsedStyle$rad === void 0 ? [0, 0, 0, 0] : _this$parsedStyle$rad,
          _this$parsedStyle$sym = _this$parsedStyle.symbol,
          symbol = _this$parsedStyle$sym === void 0 ? 'circle' : _this$parsedStyle$sym,
          _this$parsedStyle$x = _this$parsedStyle.x,
          x = _this$parsedStyle$x === void 0 ? 0 : _this$parsedStyle$x,
          _this$parsedStyle$y = _this$parsedStyle.y,
          y = _this$parsedStyle$y === void 0 ? 0 : _this$parsedStyle$y;
        var method = SYMBOLS[symbol];
        // radius 表示半径，内部自动格式化成了[]
        var path = method(x, y, radius[0]);
        _get(_getPrototypeOf(Marker.prototype), "setAttribute", this).call(this, 'path', path);
      }
    }]);
    return Marker;
  }(Path);

  var PI = Math.PI;
  var PI2 = PI * 2;
  var mathSin = Math.sin;
  var mathCos = Math.cos;
  var mathACos = Math.acos;
  var mathATan2 = Math.atan2;
  // const mathAbs = Math.abs;
  var mathSqrt = Math.sqrt;
  var mathMax = Math.max;
  var mathMin = Math.min;
  var e = 1e-4;
  function intersect(x0, y0, x1, y1, x2, y2, x3, y3) {
    var dx10 = x1 - x0;
    var dy10 = y1 - y0;
    var dx32 = x3 - x2;
    var dy32 = y3 - y2;
    var t = dy32 * dx10 - dx32 * dy10;
    if (t * t < e) {
      return;
    }
    t = (dx32 * (y0 - y2) - dy32 * (x0 - x2)) / t;
    return [x0 + t * dx10, y0 + t * dy10];
  }
  // Compute perpendicular offset line of length rc.
  function computeCornerTangents(x0, y0, x1, y1, radius, cr, clockwise) {
    var x01 = x0 - x1;
    var y01 = y0 - y1;
    var lo = (clockwise ? cr : -cr) / mathSqrt(x01 * x01 + y01 * y01);
    var ox = lo * y01;
    var oy = -lo * x01;
    var x11 = x0 + ox;
    var y11 = y0 + oy;
    var x10 = x1 + ox;
    var y10 = y1 + oy;
    var x00 = (x11 + x10) / 2;
    var y00 = (y11 + y10) / 2;
    var dx = x10 - x11;
    var dy = y10 - y11;
    var d2 = dx * dx + dy * dy;
    var r = radius - cr;
    var s = x11 * y10 - x10 * y11;
    var d = (dy < 0 ? -1 : 1) * mathSqrt(mathMax(0, r * r * d2 - s * s));
    var cx0 = (s * dy - dx * d) / d2;
    var cy0 = (-s * dx - dy * d) / d2;
    var cx1 = (s * dy + dx * d) / d2;
    var cy1 = (-s * dx + dy * d) / d2;
    var dx0 = cx0 - x00;
    var dy0 = cy0 - y00;
    var dx1 = cx1 - x00;
    var dy1 = cy1 - y00;
    // Pick the closer of the two intersection points
    // TODO: Is there a faster way to determine which intersection to use?
    if (dx0 * dx0 + dy0 * dy0 > dx1 * dx1 + dy1 * dy1) {
      cx0 = cx1;
      cy0 = cy1;
    }
    return {
      cx: cx0,
      cy: cy0,
      x0: -ox,
      y0: -oy,
      x1: cx0 * (radius / r - 1),
      y1: cy0 * (radius / r - 1)
    };
  }
  function computeArcSweep(startAngle, endAngle) {
    endAngle = endAngle < 0 && startAngle >= 0 ? endAngle + PI2 : endAngle;
    return endAngle - startAngle <= PI ? 0 : 1;
  }
  var Sector = /*#__PURE__*/function (_Path) {
    _inherits(Sector, _Path);
    var _super = _createSuper(Sector);
    function Sector(config) {
      var _this;
      _classCallCheck(this, Sector);
      _this = _super.call(this, config);
      _this.updatePath();
      return _this;
    }
    _createClass(Sector, [{
      key: "setAttribute",
      value: function setAttribute(name, value, force) {
        _get(_getPrototypeOf(Sector.prototype), "setAttribute", this).call(this, name, value, force);
        if (['startAngle', 'endAngle', 'r', 'r0', 'radius'].indexOf(name) > -1) {
          this.updatePath();
        }
      }
    }, {
      key: "updatePath",
      value: function updatePath() {
        var _this$parsedStyle = this.parsedStyle,
          x = _this$parsedStyle.x,
          y = _this$parsedStyle.y,
          startAngle = _this$parsedStyle.startAngle,
          endAngle = _this$parsedStyle.endAngle,
          r = _this$parsedStyle.r,
          r0 = _this$parsedStyle.r0,
          radius = _this$parsedStyle.radius;
        var path = this.createPath(x, y, startAngle ? deg2rad(startAngle) : 0, endAngle ? deg2rad(endAngle) : Math.PI * 2, r ? r : 0, r0 ? r0 : 0, radius ? radius : [0, 0, 0, 0]);
        _get(_getPrototypeOf(Sector.prototype), "setAttribute", this).call(this, 'path', path);
      }
    }, {
      key: "createPath",
      value: function createPath(x, y, startAngle, endAngle, r, r0, borderRadius) {
        if (r <= 0) {
          return null;
        }
        var start = polarToCartesian(x, y, r, startAngle);
        var end = polarToCartesian(x, y, r, endAngle);
        var innerStart = polarToCartesian(x, y, r0, startAngle);
        var innerEnd = polarToCartesian(x, y, r0, endAngle);
        // 整圆
        if (isNumberEqual(endAngle - startAngle, Math.PI * 2)) {
          // 整个圆是分割成两个圆
          var middlePoint = polarToCartesian(x, y, r, startAngle + Math.PI);
          var innerMiddlePoint = polarToCartesian(x, y, r0, startAngle + Math.PI);
          var circlePathCommands = [['M', start.x, start.y], ['A', r, r, 0, 1, 1, middlePoint.x, middlePoint.y], ['A', r, r, 0, 1, 1, end.x, end.y], ['M', innerStart.x, innerStart.y]];
          if (r0) {
            circlePathCommands.push(['A', r0, r0, 0, 1, 0, innerMiddlePoint.x, innerMiddlePoint.y]);
            circlePathCommands.push(['A', r0, r0, 0, 1, 0, innerEnd.x, innerEnd.y]);
          }
          circlePathCommands.push(['M', start.x, start.y]);
          circlePathCommands.push(['Z']);
          return circlePathCommands;
        }
        var angle = endAngle - startAngle;
        var xrs = r * mathCos(startAngle);
        var yrs = r * mathSin(startAngle);
        var xire = r0 * mathCos(endAngle);
        var yire = r0 * mathSin(endAngle);
        var xre = r * mathCos(endAngle);
        var yre = r * mathSin(endAngle);
        var xirs = r0 * mathCos(startAngle);
        var yirs = r0 * mathSin(startAngle);
        // 顺时针反向，同 radius
        var _borderRadius = _slicedToArray(borderRadius, 4),
          outStartRadius = _borderRadius[0],
          outEndRadius = _borderRadius[1],
          innerEndRadius = _borderRadius[2],
          innerStartRadius = _borderRadius[3];
        var halfRadius = (r - r0) / 2;
        var outStartBorderRadius = mathMin(halfRadius, outStartRadius);
        var outEndBorderRadius = mathMin(halfRadius, outEndRadius);
        var innerEndBorderRadius = mathMin(halfRadius, innerEndRadius);
        var innerStartBorderRadius = mathMin(halfRadius, innerStartRadius);
        var outBorderRadiusMax = mathMax(outStartBorderRadius, outEndBorderRadius);
        var innerBorderRadiusMax = mathMax(innerEndBorderRadius, innerStartBorderRadius);
        var limitedOutBorderRadiusMax = outBorderRadiusMax;
        var limitedInnerBorderRadiusMax = innerBorderRadiusMax;
        // draw corner radius
        if (outBorderRadiusMax > e || innerBorderRadiusMax > e) {
          // restrict the max value of corner radius
          if (angle < PI) {
            var it = intersect(xrs, yrs, xirs, yirs, xre, yre, xire, yire);
            if (it) {
              var x0 = xrs - it[0];
              var y0 = yrs - it[1];
              var x1 = xre - it[0];
              var y1 = yre - it[1];
              var a = 1 / mathSin(mathACos((x0 * x1 + y0 * y1) / (mathSqrt(x0 * x0 + y0 * y0) * mathSqrt(x1 * x1 + y1 * y1))) / 2);
              var b = mathSqrt(it[0] * it[0] + it[1] * it[1]);
              limitedOutBorderRadiusMax = mathMin(outBorderRadiusMax, (r - b) / (a + 1));
              limitedInnerBorderRadiusMax = mathMin(innerBorderRadiusMax, (r0 - b) / (a - 1));
            }
          }
        }
        var arcSweep = computeArcSweep(startAngle, endAngle);
        var clockwise = true;
        var sectorPathCommands = [];
        if (limitedOutBorderRadiusMax > e) {
          var crStart = mathMin(outStartRadius, limitedOutBorderRadiusMax);
          var crEnd = mathMin(outEndRadius, limitedOutBorderRadiusMax);
          var ct0 = computeCornerTangents(xirs, yirs, xrs, yrs, r, crStart, clockwise);
          var ct1 = computeCornerTangents(xre, yre, xire, yire, r, crEnd, clockwise);
          sectorPathCommands.push(['M', x + ct0.cx + ct0.x0, y + ct0.cy + ct0.y0]);
          // Have the corners merged?
          if (limitedOutBorderRadiusMax < outBorderRadiusMax && crStart === crEnd) {
            var outStartBorderRadiusStartAngle = mathATan2(ct0.y0, ct0.x0);
            var outStartBorderRadiusEndAngle = mathATan2(ct1.y0, ct1.x0);
            // const outStartBorderRadiusEndPoint = polarToCartesian(
            //   x,
            //   y,
            //   r,
            //   outStartBorderRadiusEndAngle
            // );
            sectorPathCommands.push(['A', limitedOutBorderRadiusMax, limitedOutBorderRadiusMax, 0, computeArcSweep(outStartBorderRadiusStartAngle, outStartBorderRadiusEndAngle), 1, x + ct1.cx + ct1.x0, y + ct1.cy + ct1.y0]);
          } else {
            // draw the two corners and the ring
            if (crStart > 0) {
              var _outStartBorderRadiusStartAngle = mathATan2(ct0.y0, ct0.x0);
              var _outStartBorderRadiusEndAngle = mathATan2(ct0.y1, ct0.x1);
              var outStartBorderRadiusEndPoint = polarToCartesian(x, y, r, _outStartBorderRadiusEndAngle);
              sectorPathCommands.push(['A', crStart, crStart, 0, computeArcSweep(_outStartBorderRadiusStartAngle, _outStartBorderRadiusEndAngle), 1, outStartBorderRadiusEndPoint.x, outStartBorderRadiusEndPoint.y]);
            }
            var outRadiusStartAngle = mathATan2(ct0.cy + ct0.y1, ct0.cx + ct0.x1);
            var outRadiusEndAngle = mathATan2(ct1.cy + ct1.y1, ct1.cx + ct1.x1);
            var outRadiusEndPoint = polarToCartesian(x, y, r, outRadiusEndAngle);
            sectorPathCommands.push(['A', r, r, 0, computeArcSweep(outRadiusStartAngle, outRadiusEndAngle), 1, outRadiusEndPoint.x, outRadiusEndPoint.y]);
            if (crEnd > 0) {
              var outEndBorderRadiusStartAngle = mathATan2(ct1.y1, ct1.x1);
              var outEndBorderRadiusEndAngle = mathATan2(ct1.y0, ct1.x0);
              sectorPathCommands.push(['A', crEnd, crEnd, 0, computeArcSweep(outEndBorderRadiusStartAngle, outEndBorderRadiusEndAngle), 1, x + ct1.cx + ct1.x0, y + ct1.cy + ct1.y0]);
            }
          }
        } else {
          sectorPathCommands.push(['M', start.x, start.y]);
          sectorPathCommands.push(['A', r, r, 0, arcSweep, 1, end.x, end.y]);
        }
        // no inner ring, is a circular sector
        if (r0 < e) {
          sectorPathCommands.push(['L', innerEnd.x, innerEnd.y]);
        }
        // the inner ring has corners
        else if (limitedInnerBorderRadiusMax > e) {
          var _crStart = mathMin(innerStartRadius, limitedInnerBorderRadiusMax);
          var _crEnd = mathMin(innerEndRadius, limitedInnerBorderRadiusMax);
          var _ct = computeCornerTangents(xire, yire, 0, 0, r0 - r, _crEnd, clockwise);
          var _ct2 = computeCornerTangents(0, 0, xirs, yirs, r0 - r, _crStart, clockwise);
          sectorPathCommands.push(['L', x + _ct.cx + _ct.x0, y + _ct.cy + _ct.y0]);
          // Have the corners merged?
          if (limitedInnerBorderRadiusMax < innerBorderRadiusMax && _crStart === _crEnd) {
            var innerStartBorderRadiusStartAngle = mathATan2(_ct.y0, _ct.x0);
            var innerStartBorderRadiusEndAngle = mathATan2(_ct2.y0, _ct2.x0);
            var innerStartBorderRadiusEndPoint = polarToCartesian(x, y, r0, innerStartBorderRadiusEndAngle);
            sectorPathCommands.push(['A', limitedOutBorderRadiusMax, limitedOutBorderRadiusMax, 0, computeArcSweep(innerStartBorderRadiusStartAngle, innerStartBorderRadiusEndAngle), 1, innerStartBorderRadiusEndPoint.x, innerStartBorderRadiusEndPoint.y]);
          }
          // draw the two corners and the ring
          else {
            if (_crEnd > 0) {
              var _innerStartBorderRadiusStartAngle = mathATan2(_ct.y0, _ct.x0);
              var _innerStartBorderRadiusEndAngle = mathATan2(_ct.y1, _ct.x1);
              var _innerStartBorderRadiusEndPoint = polarToCartesian(x, y, r0 - r, _innerStartBorderRadiusEndAngle);
              sectorPathCommands.push(['A', _crEnd, _crEnd, 0, computeArcSweep(_innerStartBorderRadiusStartAngle, _innerStartBorderRadiusEndAngle), 1, _innerStartBorderRadiusEndPoint.x, _innerStartBorderRadiusEndPoint.y]);
            }
            var innerRadiusStartAngle = mathATan2(_ct.cy + _ct.y1, _ct.cx + _ct.x1);
            var innerRadiusEndAngle = mathATan2(_ct2.cy + _ct2.y1, _ct2.cx + _ct2.x1);
            var innerRadiusEndPoint = polarToCartesian(x, y, r0, innerRadiusEndAngle);
            sectorPathCommands.push(['A', r0, r0, 0, computeArcSweep(innerRadiusEndAngle, innerRadiusStartAngle), 0, innerRadiusEndPoint.x, innerRadiusEndPoint.y]);
            sectorPathCommands.push(['L', innerRadiusEndPoint.x, innerRadiusEndPoint.y]);
            if (_crStart > 0) {
              var innerEndBorderRadiusStartAngle = mathATan2(_ct2.y1, _ct2.x1);
              var innerEndBorderRadiusEndAngle = mathATan2(_ct2.y0, _ct2.x0);
              sectorPathCommands.push(['A', _crStart, _crStart, 0, computeArcSweep(innerEndBorderRadiusStartAngle, innerEndBorderRadiusEndAngle), 1, x + _ct2.cx + _ct2.x0, y + _ct2.cy + _ct2.y0]);
            }
          }
        }
        // the inner ring is just a circular arc
        else {
          sectorPathCommands.push(['L', innerEnd.x, innerEnd.y]);
          sectorPathCommands.push(['A', r0, r0, 0, arcSweep, 0, innerStart.x, innerStart.y]);
        }
        sectorPathCommands.push(['Z']);
        return sectorPathCommands;
      }
    }]);
    return Sector;
  }(Path);

  /**
   * @fileOverview convert the line to curve
   * @author dxq613@gmail.com
   */
  function getPoint$1(v) {
    return [v.x, v.y];
  }
  function smoothBezier(points, smooth, isLoop, constraint) {
    var cps = [];
    var prevPoint;
    var nextPoint;
    var hasConstraint = !!constraint;
    var min;
    var max;
    var point;
    var len;
    var l;
    var i;
    if (hasConstraint) {
      min = [Infinity, Infinity];
      max = [-Infinity, -Infinity];
      for (i = 0, l = points.length; i < l; i++) {
        point = getPoint$1(points[i]);
        min$1(min, min, point);
        max$1(max, max, point);
      }
      min$1(min, min, constraint[0]);
      max$1(max, max, constraint[1]);
    }
    for (i = 0, len = points.length; i < len; i++) {
      point = getPoint$1(points[i]);
      if (isLoop) {
        prevPoint = getPoint$1(points[i ? i - 1 : len - 1]);
        nextPoint = getPoint$1(points[(i + 1) % len]);
      } else {
        if (i === 0 || i === len - 1) {
          cps.push([point[0], point[1]]);
          continue;
        } else {
          prevPoint = getPoint$1(points[i - 1]);
          nextPoint = getPoint$1(points[i + 1]);
        }
      }
      var v = sub$2([], nextPoint, prevPoint);
      scale$2(v, v, smooth);
      var d0 = distance(point, prevPoint);
      var d1 = distance(point, nextPoint);
      var sum = d0 + d1;
      if (sum !== 0) {
        d0 /= sum;
        d1 /= sum;
      }
      var v1 = scale$2([], v, -d0);
      var v2 = scale$2([], v, d1);
      var cp0 = add$2([], point, v1);
      var cp1 = add$2([], point, v2);
      if (hasConstraint) {
        max$1(cp0, cp0, min);
        min$1(cp0, cp0, max);
        max$1(cp1, cp1, min);
        min$1(cp1, cp1, max);
      }
      cps.push([cp0[0], cp0[1]]);
      cps.push([cp1[0], cp1[1]]);
    }
    if (isLoop) {
      cps.push(cps.shift());
    }
    return cps;
  }
  function catmullRom2bezier(pointList, z, constraint) {
    var isLoop = !!z;
    var controlPointList = smoothBezier(pointList, 0.4, isLoop, constraint);
    var len = pointList.length;
    var d1 = [];
    var cp1;
    var cp2;
    var p;
    for (var i = 0; i < len - 1; i++) {
      cp1 = controlPointList[i * 2];
      cp2 = controlPointList[i * 2 + 1];
      p = pointList[i + 1];
      d1.push(['C', cp1[0], cp1[1], cp2[0], cp2[1], p.x, p.y]);
    }
    if (isLoop) {
      cp1 = controlPointList[len];
      cp2 = controlPointList[len + 1];
      p = pointList[0];
      d1.push(['C', cp1[0], cp1[1], cp2[0], cp2[1], p.x, p.y]);
    }
    return d1;
  }

  var SmoothPolyline = /*#__PURE__*/function (_Path) {
    _inherits(SmoothPolyline, _Path);
    var _super = _createSuper(SmoothPolyline);
    function SmoothPolyline(config) {
      var _this;
      _classCallCheck(this, SmoothPolyline);
      _this = _super.call(this, config);
      _this.updatePath();
      return _this;
    }
    _createClass(SmoothPolyline, [{
      key: "setAttribute",
      value: function setAttribute(name, value, force) {
        _get(_getPrototypeOf(SmoothPolyline.prototype), "setAttribute", this).call(this, name, value, force);
        if (['smooth', 'points'].indexOf(name) > -1) {
          this.updatePath();
        }
      }
    }, {
      key: "updatePath",
      value: function updatePath() {
        var _this$parsedStyle = this.parsedStyle,
          smooth = _this$parsedStyle.smooth,
          points = _this$parsedStyle.points;
        var pos = points.points;
        var d = [['M', pos[0][0], pos[0][1]]];
        if (smooth) {
          var constaint = [[0, 0], [1, 1]];
          var sps = catmullRom2bezier(pos.map(function (d) {
            return {
              x: d[0],
              y: d[1]
            };
          }), false, constaint);
          for (var i = 0, n = sps.length; i < n; i++) {
            var sp = sps[i];
            d.push(['C', sp[1], sp[2], sp[3], sp[4], sp[5], sp[6]]);
          }
        } else {
          var _i;
          var l;
          for (_i = 1, l = pos.length - 1; _i < l; _i++) {
            d.push(['L', pos[_i][0], pos[_i][1]]);
          }
          d.push(['L', pos[l][0], pos[l][1]]);
        }
        _get(_getPrototypeOf(SmoothPolyline.prototype), "setAttribute", this).call(this, 'path', d);
      }
    }]);
    return SmoothPolyline;
  }(Path);
  SmoothPolyline.tag = 'smooth-polyline';

  // 注册 css 属性
  var SECTOR_CSS_PROPERTY = [{
    name: 'r',
    inherits: false,
    interpolable: true,
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    name: 'r0',
    inherits: false,
    interpolable: true,
    syntax: PropertySyntax.LENGTH_PERCENTAGE
  }, {
    name: 'startAngle',
    inherits: false,
    interpolable: true,
    syntax: PropertySyntax.ANGLE
  }, {
    name: 'endAngle',
    inherits: false,
    interpolable: true,
    syntax: PropertySyntax.ANGLE
  }];
  SECTOR_CSS_PROPERTY.forEach(function (property) {
    CSS.registerProperty(property);
  });

  function _inheritsLoose$1(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$2(subClass, superClass);
  }
  function _setPrototypeOf$2(o, p) {
    _setPrototypeOf$2 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$2(o, p);
  }
  function _assertThisInitialized$2(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
  }

  var clock = typeof performance === 'object' && performance.now ? performance : Date;
  var PRESS_DELAY = 250;
  // 计算滑动的方向
  var calcDirection = function calcDirection(start, end) {
    var xDistance = end.x - start.x;
    var yDistance = end.y - start.y;
    // x 的距离大于y 说明是横向，否则就是纵向
    if (Math.abs(xDistance) > Math.abs(yDistance)) {
      return xDistance > 0 ? 'right' : 'left';
    }
    return yDistance > 0 ? 'down' : 'up';
  };
  // 计算2点之间的距离
  var calcDistance = function calcDistance(point1, point2) {
    var xDistance = Math.abs(point2.x - point1.x);
    var yDistance = Math.abs(point2.y - point1.y);
    return Math.sqrt(xDistance * xDistance + yDistance * yDistance);
  };
  var getCenter = function getCenter(point1, point2) {
    var x = point1.x + (point2.x - point1.x) / 2;
    var y = point1.y + (point2.y - point1.y) / 2;
    return {
      x: x,
      y: y
    };
  };
  var Gesture = /*#__PURE__*/function (_EE) {
    _inheritsLoose$1(Gesture, _EE);
    // 用来记录当前触发的事件

    function Gesture(el) {
      var _this;
      _this = _EE.call(this) || this;
      _this.el = void 0;
      _this.evCache = [];
      _this.startTime = void 0;
      _this.pressTimeout = void 0;
      _this.startPoints = [];
      _this.processEvent = {};
      _this.startDistance = void 0;
      _this.center = void 0;
      _this.eventType = void 0;
      _this.direction = void 0;
      _this.lastMoveTime = void 0;
      _this.prevMovePoint = void 0;
      _this.prevMoveTime = void 0;
      _this.lastMovePoint = void 0;
      _this.throttleTimer = 0;
      _this.emitThrottles = [];
      _this._start = function (ev) {
        // 每次触点开始都重置事件
        _this.reset();
        // 记录touch start 的时间
        _this.startTime = clock.now();
        var _assertThisInitialize = _assertThisInitialized$2(_this),
          evCache = _assertThisInitialize.evCache,
          startPoints = _assertThisInitialize.startPoints;
        if (ev) {
          evCache.push(ev);
        }
        // 重置 startPoints
        startPoints.length = evCache.length;
        for (var i = 0; i < evCache.length; i++) {
          var _evCache$i = evCache[i],
            x = _evCache$i.x,
            y = _evCache$i.y;
          var point = {
            x: x,
            y: y
          };
          startPoints[i] = point;
        }
        // 单指事件
        if (startPoints.length === 1) {
          var event = evCache[0];
          // 如果touchstart后停顿250ms, 则也触发press事件
          _this.pressTimeout = setTimeout(function () {
            // 这里固定触发press事件
            var eventType = 'press';
            var direction = 'none';
            event.direction = direction;
            event.deltaX = 0;
            event.deltaY = 0;
            event.points = startPoints;
            _this.emitStart(eventType, event);
            _this._emit(eventType, event);
            _this.eventType = eventType;
            _this.direction = direction;
          }, PRESS_DELAY);
          return;
        }
        // 目前只处理双指
        _this.startDistance = calcDistance(startPoints[0], startPoints[1]);
        _this.center = getCenter(startPoints[0], startPoints[1]);
      };
      _this._move = function (ev) {
        _this.clearPressTimeout();
        var _assertThisInitialize2 = _assertThisInitialized$2(_this),
          startPoints = _assertThisInitialize2.startPoints,
          evCache = _assertThisInitialize2.evCache;
        if (!startPoints.length) return;
        var x = ev.x,
          y = ev.y,
          pointerId = ev.pointerId;
        // Find this event in the cache and update its record with this event
        for (var i = 0, len = evCache.length; i < len; i++) {
          if (pointerId === evCache[i].pointerId) {
            evCache[i] = ev;
            break;
          }
        }
        var point = {
          x: x,
          y: y
        };
        var points = evCache.map(function (ev) {
          return {
            x: ev.x,
            y: ev.y
          };
        });
        // 记录最后2次move的时间和坐标，为了给swipe事件用
        var now = clock.now();
        _this.prevMoveTime = _this.lastMoveTime;
        _this.prevMovePoint = _this.lastMovePoint;
        _this.lastMoveTime = now;
        _this.lastMovePoint = point;
        if (startPoints.length === 1) {
          var startPoint = startPoints[0];
          var deltaX = x - startPoint.x;
          var deltaY = y - startPoint.y;
          var direction = _this.direction || calcDirection(startPoint, point);
          _this.direction = direction;
          // 获取press或者pan的事件类型
          // press 按住滑动, pan表示平移
          // 如果start后立刻move，则触发pan, 如果有停顿，则触发press
          var eventType = _this.getEventType(point);
          ev.direction = direction;
          ev.deltaX = deltaX;
          ev.deltaY = deltaY;
          ev.points = points;
          _this.emitStart(eventType, ev);
          _this._emit(eventType, ev);
          return;
        }
        // 多指触控
        var _assertThisInitialize3 = _assertThisInitialized$2(_this),
          startDistance = _assertThisInitialize3.startDistance;
        var currentDistance = calcDistance(points[0], points[1]);
        // 缩放比例
        ev.zoom = currentDistance / startDistance;
        ev.center = _this.center;
        ev.points = points;
        // 触发缩放事件
        _this.emitStart('pinch', ev);
        _this._emit('pinch', ev);
      };
      _this._end = function (ev) {
        var _assertThisInitialize4 = _assertThisInitialized$2(_this),
          evCache = _assertThisInitialize4.evCache,
          startPoints = _assertThisInitialize4.startPoints;
        var points = evCache.map(function (ev) {
          return {
            x: ev.x,
            y: ev.y
          };
        });
        ev.points = points;
        _this.emitEnd(ev);
        // 单指
        if (evCache.length === 1) {
          // swipe事件处理, 在end之后触发
          var now = clock.now();
          var lastMoveTime = _this.lastMoveTime;
          // 做这个判断是为了最后一次touchmove后到end前，是否还有一个停顿的过程
          // 100 是拍的一个值，理论这个值会很短，一般不卡顿的话在10ms以内
          if (now - lastMoveTime < 100) {
            var prevMoveTime = _this.prevMoveTime || _this.startTime;
            var intervalTime = lastMoveTime - prevMoveTime;
            // 时间间隔一定要大于0, 否则计算没意义
            if (intervalTime > 0) {
              var prevMovePoint = _this.prevMovePoint || startPoints[0];
              var lastMovePoint = _this.lastMovePoint || startPoints[0];
              // move速率
              var velocity = calcDistance(prevMovePoint, lastMovePoint) / intervalTime;
              // 0.3 是参考hammerjs的设置
              if (velocity > 0.3) {
                ev.velocity = velocity;
                ev.direction = calcDirection(prevMovePoint, lastMovePoint);
                _this._emit('swipe', ev);
              }
            }
          }
        }
        // remove event from cache
        for (var i = 0, len = evCache.length; i < len; i++) {
          if (evCache[i].pointerId === ev.pointerId) {
            evCache.splice(i, 1);
            startPoints.splice(i, 1);
            break;
          }
        }
        _this.reset();
        // 多指离开 1 指后，重新触发一次start
        if (evCache.length > 0) {
          _this._start();
        }
      };
      _this.el = el;
      _this._initEvent();
      return _this;
    }
    var _proto = Gesture.prototype;
    _proto._initEvent = function _initEvent() {
      var el = this.el;
      el.addEventListener('pointerdown', this._start);
      el.addEventListener('pointermove', this._move);
      el.addEventListener('pointerup', this._end);
      el.addEventListener('pointerupoutside', this._end);
    };
    _proto.getEventType = function getEventType(point) {
      var eventType = this.eventType,
        startTime = this.startTime,
        startPoints = this.startPoints;
      if (eventType) {
        return eventType;
      }
      var type;
      // @ts-ignore
      var panEventListeners = this._events.pan;
      // 如果没有pan事件的监听，默认都是press
      if (!panEventListeners) {
        type = 'press';
      } else {
        // 如果有pan事件的处理，press则需要停顿250ms, 且移动距离小于10
        var now = clock.now();
        if (now - startTime > PRESS_DELAY && calcDistance(startPoints[0], point) < 10) {
          type = 'press';
        } else {
          type = 'pan';
        }
      }
      this.eventType = type;
      return type;
    };
    _proto.enable = function enable(eventType) {
      this.processEvent[eventType] = true;
    }
    // 是否进行中的事件
    ;
    _proto.isProcess = function isProcess(eventType) {
      return this.processEvent[eventType];
    }
    // 触发start事件
    ;
    _proto.emitStart = function emitStart(type, ev) {
      if (this.isProcess(type)) {
        return;
      }
      this.enable(type);
      this._emit(type + "start", ev);
    }
    // 触发事件
    ;
    _proto._emit = function _emit(type, ev) {
      var _el$ownerDocument,
        _el$document,
        _this2 = this;
      // 主要是节流处理
      this.pushEvent(type, ev);
      var el = this.el,
        throttleTimer = this.throttleTimer,
        emitThrottles = this.emitThrottles;
      if (throttleTimer) {
        return;
      }
      // @ts-ignore
      var global = ((_el$ownerDocument = el.ownerDocument) === null || _el$ownerDocument === void 0 ? void 0 : _el$ownerDocument.defaultView) || ((_el$document = el.document) === null || _el$document === void 0 ? void 0 : _el$document.defaultView);
      this.throttleTimer = global.requestAnimationFrame(function () {
        for (var i = 0, len = emitThrottles.length; i < len; i++) {
          var _emitThrottles$i = emitThrottles[i],
            _type = _emitThrottles$i.type,
            _ev = _emitThrottles$i.ev;
          _this2.emit(_type, _ev);
        }
        // 清空
        _this2.throttleTimer = 0;
        _this2.emitThrottles.length = 0;
      });
    }
    // 触发end事件
    ;
    _proto.emitEnd = function emitEnd(ev) {
      var _this3 = this;
      var processEvent = this.processEvent;
      Object.keys(processEvent).forEach(function (type) {
        _this3._emit(type + "end", ev);
        delete processEvent[type];
      });
    };
    _proto.pushEvent = function pushEvent(type, ev) {
      var emitThrottles = this.emitThrottles;
      var newEvent = {
        type: type,
        ev: ev
      };
      for (var i = 0, len = emitThrottles.length; i < len; i++) {
        if (emitThrottles[i].type === type) {
          emitThrottles.splice(i, 1, newEvent);
          return;
        }
      }
      emitThrottles.push(newEvent);
    };
    _proto.clearPressTimeout = function clearPressTimeout() {
      if (this.pressTimeout) {
        clearTimeout(this.pressTimeout);
        this.pressTimeout = null;
      }
    };
    _proto.reset = function reset() {
      this.clearPressTimeout();
      this.startTime = 0;
      this.startDistance = 0;
      this.direction = null;
      this.eventType = null;
      this.prevMoveTime = 0;
      this.prevMovePoint = null;
      this.lastMoveTime = 0;
      this.lastMovePoint = null;
    };
    return Gesture;
  }(eventemitter3);

  var G_SHAPE_EVENT = {
    click: true,
    touchstart: true,
    touchmove: true,
    touchend: true,
    touchendoutside: true,
    dragenter: true,
    dragleave: true,
    dragover: true,
    drop: true,
    dragstart: true,
    drag: true,
    dragend: true
  };
  var GESTURE_EVENT = {
    panstart: true,
    pan: true,
    panend: true,
    pressstart: true,
    press: true,
    pressend: true,
    swipe: true,
    pinchstart: true,
    pinch: true,
    pinchend: true
  };
  var Gesture$1 = /*#__PURE__*/function () {
    function Gesture$1(element) {
      _classCallCheck(this, Gesture$1);
      this.el = element;
      this.gesture = new Gesture(element);
    }
    _createClass(Gesture$1, [{
      key: "on",
      value: function on(eventName, listener) {
        var el = this.el,
          gesture = this.gesture;
        if (G_SHAPE_EVENT[eventName]) {
          el.addEventListener(eventName, listener);
          return;
        }
        if (GESTURE_EVENT[eventName]) {
          gesture.on(eventName, listener);
        }
      }
    }, {
      key: "off",
      value: function off(eventName, listener) {
        var el = this.el,
          gesture = this.gesture;
        if (G_SHAPE_EVENT[eventName]) {
          el.removeEventListener(eventName, listener);
          return;
        }
        if (GESTURE_EVENT[eventName]) {
          gesture.off(eventName, listener);
        }
      }
    }]);
    return Gesture$1;
  }();

  var SHAPE_TAG = {};
  /**
   * 注册新的标签
   */
  var registerTag = function registerTag(name, ShapeConstructor) {
    SHAPE_TAG[name] = ShapeConstructor;
  };
  var getTag = function getTag(type) {
    return SHAPE_TAG[type];
  };

  var EVENT_LIST = [['click', 'onClick'], ['touchstart', 'onTouchStart'], ['touchmove', 'onTouchMove'], ['touchend', 'onTouchEnd'], ['touchendoutside', 'onTouchEndOutside'],
  // drage 相关
  ['dragenter', 'onDragEnter'], ['dragleave', 'onDragLeave'], ['dragover', 'onDragOver'], ['drop', 'onDrop'], ['dragstart', 'onDragStart'], ['drag', 'onDrag'], ['dragend', 'onDragEnd'],
  // pan
  ['panstart', 'onPanStart'], ['pan', 'onPan'], ['panend', 'onPanEnd'],
  // press
  ['pressstart', 'onPressStart'], ['press', 'onPress'], ['pressend', 'onPressEnd'],
  // swipe
  ['swipe', 'onSwipe'],
  // pinch
  ['pinchstart', 'onPinchStart'], ['pinch', 'onPinch'], ['pinchend', 'onPinchEnd']];
  // 默认标签
  var TagElements = [['group', Group], ['text', Text], ['circle', Circle], ['path', Path], ['ellipse', Ellipse], ['rect', Rect], ['image', Image], ['line', Line], ['polyline', SmoothPolyline], ['polygon', Polygon], ['arc', Arc], ['marker', Marker], ['sector', Sector]];
  TagElements.map(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
      type = _ref2[0],
      ShapeClass = _ref2[1];
    registerTag(type, ShapeClass);
  });
  function createShape(type, props) {
    if (!type) return null;
    var ShapeClass = getTag(type);
    if (!ShapeClass) return null;
    // const result = checkCSSRule(type, originStyle);
    var shape = new ShapeClass(props);
    addEvent(shape, props);
    return shape;
  }
  function addEvent(shape, props) {
    var gesture = new Gesture$1(shape);
    EVENT_LIST.forEach(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
        eventName = _ref4[0],
        handlerName = _ref4[1];
      if (!props[handlerName]) return;
      gesture.on(eventName, props[handlerName]);
    });
  }

  function objToString(obj) {
    return Object.prototype.toString.call(obj);
  }
  function objectKeys(obj) {
    return Object.keys(obj);
  }
  function equal(a, b) {
    if (a === b) return true;
    if (_typeof(a) !== _typeof(b)) {
      return false;
    }
    // null 和 undefined
    if (a == null || b == null) {
      return false;
    }
    // 特殊处理NaN
    if (Number.isNaN(a) && Number.isNaN(b)) {
      return true;
    }
    if (objToString(a) !== objToString(b)) {
      return false;
    }
    // 如果是function， 则认为是相对
    if (isFunction(a)) {
      return true;
    }
    // 值类型，Number String Boolean
    if (_typeof(a) !== 'object') {
      return false;
    }
    if (isArray(a)) {
      if (a.length !== b.length) {
        return false;
      }
      for (var i = a.length - 1; i >= 0; i--) {
        if (!equal(a[i], b[i])) {
          return false;
        }
      }
      return true;
    }
    if (!isPlainObject(a)) {
      return false;
    }
    var ka = objectKeys(a);
    var kb = objectKeys(b);
    // having the same number of owned properties (keys incorporates hasOwnProperty)
    if (ka.length !== kb.length) {
      return false;
    }
    // the same set of keys (although not necessarily the same order),
    ka.sort();
    kb.sort();
    // ~~~cheap key test
    for (var _i = ka.length - 1; _i >= 0; _i--) {
      if (ka[_i] != kb[_i]) {
        return false;
      }
    }
    // equivalent values for every corresponding key, and ~~~possibly expensive deep test
    for (var _i2 = ka.length - 1; _i2 >= 0; _i2--) {
      var key = ka[_i2];
      if (!equal(a[key], b[key])) {
        return false;
      }
    }
    return true;
  }

  function applyStyle(shape, style) {
    if (!style) return;
    Object.keys(style).forEach(function (key) {
      // 特殊处理 clip
      if (key === 'clip') {
        var clip = style.clip;
        var clipConfig = isFunction(clip) ? clip(style) : clip;
        if (clipConfig) {
          var type = clipConfig.type,
            _style = clipConfig.style;
          var clipShape = createShape(type, {
            style: _style
          });
          shape.setAttribute('clipPath', clipShape);
        }
      } else {
        shape.setAttribute(key, style[key]);
      }
    });
  }
  function appearAnimation(vNode) {
    return Children.map(vNode, function (node) {
      if (!node) return;
      var shape = node.shape,
        style = node.style,
        children = node.children,
        animate = node.animate,
        props = node.props,
        animator = node.animator;
      // 有叶子节点，先执行叶子节点
      var childrenAnimation = children ? createAnimation(node, children, null) : null;
      // 不需要执行动画
      if (animate === false) {
        applyStyle(shape, style);
        return null;
      }
      var animation = props.animation;
      var animationEffect = animation ? animation.appear : null;
      animator.reset();
      if (!animationEffect) {
        // 没有动画直接应用样式
        applyStyle(shape, style);
      }
      // 如果当前和子元素都没有动画，直接返回
      if (!animationEffect && !(childrenAnimation && childrenAnimation.length)) {
        return null;
      }
      // 子元素动画
      if (childrenAnimation && childrenAnimation.length) {
        animator.children = childrenAnimation;
      }
      // 图形有动画
      if (animationEffect) {
        var _animationEffect$star = animationEffect.start,
          start = _animationEffect$star === void 0 ? {} : _animationEffect$star,
          end = animationEffect.end;
        var endStyle = _objectSpread2(_objectSpread2({}, style), end);
        animator.animate(shape, start, endStyle, animationEffect);
      }
      return animator;
    });
  }
  function updateAnimation(nextNode, lastNode) {
    var nextType = nextNode.type,
      nextStyle = nextNode.style,
      nextChildren = nextNode.children,
      nextProps = nextNode.props,
      nextShape = nextNode.shape,
      animator = nextNode.animator,
      animate = nextNode.animate;
    var lastType = lastNode.type,
      lastStyle = lastNode.style,
      lastChildren = lastNode.children,
      lastShape = lastNode.shape;
    // 先处理叶子节点
    var childrenAnimation = createAnimation(nextNode, nextChildren, lastChildren);
    // 清除之前的样式
    var resetStyle = lastStyle ? Object.keys(lastStyle).reduce(function (prev, cur) {
      prev[cur] = '';
      return prev;
    }, {}) : null;
    // 需要更新的样式
    var style = _objectSpread2(_objectSpread2({}, resetStyle), nextStyle);
    // 没有动画直接应用样式
    if (animate === false) {
      applyStyle(nextShape, style);
      return null;
    }
    var animation = nextProps.animation;
    animator.reset();
    // 子元素动画
    if (childrenAnimation && childrenAnimation.length) {
      animator.children = childrenAnimation;
    }
    var animationEffect = animation ? animation.update : null;
    // 没有动画直接应用样式
    if (!animationEffect) {
      applyStyle(nextShape, style);
      // 如果也没有子元素动画，直接返回
      if (!childrenAnimation || !childrenAnimation.length) {
        return null;
      }
    }
    // 自身动画
    if (isString(nextType) && isString(lastType)) {
      if (animationEffect) {
        // 图形属性动画
        if (nextType === lastType) {
          var start = animationEffect.start,
            end = animationEffect.end;
          var startStyle = _objectSpread2(_objectSpread2({}, lastStyle), start);
          var endStyle = _objectSpread2(_objectSpread2({}, style), end);
          animator.animate(nextShape, startStyle, endStyle, animationEffect);
        } else {
          // 形变动画
          var _start = animationEffect.start,
            _end = animationEffect.end,
            _animationEffect$prop = animationEffect.property,
            property = _animationEffect$prop === void 0 ? [] : _animationEffect$prop;
          var lastPath = convertToPath(lastShape);
          var nextPath = convertToPath(nextShape);
          var _startStyle = _objectSpread2(_objectSpread2(_objectSpread2({}, lastStyle), _start), {}, {
            path: lastPath
          });
          var _endStyle = _objectSpread2(_objectSpread2(_objectSpread2({}, nextStyle), _end), {}, {
            path: nextPath
          });
          var pathShape = createShape('path', {
            style: _objectSpread2(_objectSpread2({}, _startStyle), {}, {
              path: ''
            })
          });
          lastShape.replaceWith(pathShape);
          animator.animate(pathShape, _startStyle, _endStyle, _objectSpread2(_objectSpread2({}, animationEffect), {}, {
            property: ['path'].concat(property)
          }));
          animator.once('end', function () {
            applyStyle(nextShape, _endStyle);
            pathShape.replaceWith(nextShape);
          });
        }
      }
    }
    return animator;
  }
  function destroyAnimation(vNode) {
    if (!vNode) return null;
    var shape = vNode.shape,
      children = vNode.children,
      animate = vNode.animate,
      style = vNode.style,
      props = vNode.props,
      animator = vNode.animator;
    // 先处理叶子节点
    var childrenAnimation = children ?
    // @ts-ignore
    Children.toArray(children).map(function (child) {
      return destroyAnimation(child);
    }).filter(Boolean) : null;
    if (animate === false) {
      shape.remove();
      return null;
    }
    var animation = props.animation;
    var animationEffect = animation ? animation.leave : null;
    // 重置
    animator.reset();
    // 没有叶子节点的动画， 直接删除
    if (!(childrenAnimation && childrenAnimation.length) && !animationEffect) {
      shape.remove();
      return null;
    }
    if (childrenAnimation && childrenAnimation.length) {
      animator.children = childrenAnimation;
    }
    // 图形有动画
    if (animationEffect) {
      var start = animationEffect.start,
        _animationEffect$end = animationEffect.end,
        end = _animationEffect$end === void 0 ? {} : _animationEffect$end;
      var startStyle = _objectSpread2(_objectSpread2({}, style), start);
      var endStyle = end;
      animator.animate(shape, startStyle, endStyle, animationEffect);
    }
    animator.once('end', function () {
      shape.remove();
    });
    return animator;
  }
  function createAnimator(nextNode, lastNode) {
    if (!nextNode && !lastNode) {
      return;
    }
    // delete 动画
    if (!nextNode && lastNode) {
      return destroyAnimation(lastNode);
    }
    // appear 动画
    if (nextNode && !lastNode) {
      return appearAnimation(nextNode);
    }
    // update 动画
    return updateAnimation(nextNode, lastNode);
  }
  function createAnimation(parent, nextChildren, lastChildren) {
    var parentShape = parent.shape;
    var nextSibling = parentShape.firstChild;
    var childrenAnimation = [];
    Children.compare(nextChildren, lastChildren, function (nextNode, lastNode) {
      var animator = createAnimator(nextNode, lastNode);
      // 更新文档流
      if (nextNode && !lastNode) {
        Children.map(nextNode, function (node) {
          if (!node) return;
          var shape = node.shape;
          if (nextSibling) {
            parentShape.insertBefore(shape, nextSibling);
          } else {
            parentShape.appendChild(shape);
          }
        });
      } else if (nextNode && lastNode) {
        var shape = nextNode.shape;
        nextSibling = shape.nextSibling;
      }
      if (animator) {
        Children.map(animator, function (item) {
          if (!item) return;
          childrenAnimation.push(item);
        });
      }
      return animator;
    });
    return childrenAnimation;
  }

  var WILDCARD = '*';
  /* event-emitter */
  var EventEmitter = /** @class */ (function () {
      function EventEmitter() {
          this._events = {};
      }
      /**
       * 监听一个事件
       * @param evt
       * @param callback
       * @param once
       */
      EventEmitter.prototype.on = function (evt, callback, once) {
          if (!this._events[evt]) {
              this._events[evt] = [];
          }
          this._events[evt].push({
              callback: callback,
              once: !!once,
          });
          return this;
      };
      /**
       * 监听一个事件一次
       * @param evt
       * @param callback
       */
      EventEmitter.prototype.once = function (evt, callback) {
          return this.on(evt, callback, true);
      };
      /**
       * 触发一个事件
       * @param evt
       * @param args
       */
      EventEmitter.prototype.emit = function (evt) {
          var _this = this;
          var args = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              args[_i - 1] = arguments[_i];
          }
          var events = this._events[evt] || [];
          var wildcardEvents = this._events[WILDCARD] || [];
          // 实际的处理 emit 方法
          var doEmit = function (es) {
              var length = es.length;
              for (var i = 0; i < length; i++) {
                  if (!es[i]) {
                      continue;
                  }
                  var _a = es[i], callback = _a.callback, once = _a.once;
                  if (once) {
                      es.splice(i, 1);
                      if (es.length === 0) {
                          delete _this._events[evt];
                      }
                      length--;
                      i--;
                  }
                  callback.apply(_this, args);
              }
          };
          doEmit(events);
          doEmit(wildcardEvents);
      };
      /**
       * 取消监听一个事件，或者一个channel
       * @param evt
       * @param callback
       */
      EventEmitter.prototype.off = function (evt, callback) {
          if (!evt) {
              // evt 为空全部清除
              this._events = {};
          }
          else {
              if (!callback) {
                  // evt 存在，callback 为空，清除事件所有方法
                  delete this._events[evt];
              }
              else {
                  // evt 存在，callback 存在，清除匹配的
                  var events = this._events[evt] || [];
                  var length_1 = events.length;
                  for (var i = 0; i < length_1; i++) {
                      if (events[i].callback === callback) {
                          events.splice(i, 1);
                          length_1--;
                          i--;
                      }
                  }
                  if (events.length === 0) {
                      delete this._events[evt];
                  }
              }
          }
          return this;
      };
      /* 当前所有的事件 */
      EventEmitter.prototype.getEvents = function () {
          return this._events;
      };
      return EventEmitter;
  }());

  var Animator = /*#__PURE__*/function (_EE) {
    _inherits(Animator, _EE);
    var _super = _createSuper(Animator);
    function Animator() {
      _classCallCheck(this, Animator);
      return _super.apply(this, arguments);
    }
    _createClass(Animator, [{
      key: "animate",
      value: function animate(shape, start, end, effect) {
        this.shape = shape;
        this.start = start;
        this.end = end;
        this.effect = effect;
      }
    }, {
      key: "play",
      value: function play() {
        var _this = this;
        var shape = this.shape,
          start = this.start,
          end = this.end,
          effect = this.effect,
          children = this.children;
        var animations = [];
        if (effect) {
          var _effect$property = effect.property,
            property = _effect$property === void 0 ? [] : _effect$property,
            easing = effect.easing,
            duration = effect.duration,
            delay = effect.delay,
            iterations = effect.iterations,
            clip = effect.clip,
            _effect$onFrame = effect.onFrame,
            onFrame = _effect$onFrame === void 0 ? function () {} : _effect$onFrame,
            _effect$onEnd = effect.onEnd,
            onEnd = _effect$onEnd === void 0 ? function () {} : _effect$onEnd;
          // 应用样式
          var style = _objectSpread2(_objectSpread2({}, omit(end, property)), omit(start, property));
          Object.keys(style).forEach(function (key) {
            shape.setAttribute(key, style[key]);
          });
          // 开始帧
          var keyframeStart = property.reduce(function (prev, cur) {
            prev[cur] = start[cur];
            return prev;
          }, {});
          // 结束帧
          var keyframeEnd = pick(end, property);
          var animation = shape.animate([keyframeStart, keyframeEnd], {
            fill: 'both',
            easing: easing,
            duration: duration,
            delay: delay,
            iterations: iterations
          });
          if (animation) {
            animation.onframe = onFrame;
            animation.onfinish = onEnd;
          }
          // 过滤无限循环的动画
          if (animation && iterations !== Infinity) {
            animations.push(animation.finished);
          }
          // clip 动画
          if (clip) {
            var clipConfig = isFunction(clip) ? clip(style) : clip;
            if (clipConfig) {
              var clipType = clipConfig.type,
                clipStyle = clipConfig.style,
                _clipConfig$property = clipConfig.property,
                clipProperty = _clipConfig$property === void 0 ? [] : _clipConfig$property,
                clipEasing = clipConfig.easing,
                clipDuration = clipConfig.duration,
                clipDelay = clipConfig.delay,
                clipIterations = clipConfig.iterations,
                clipStart = clipConfig.start,
                clipEnd = clipConfig.end;
              var clipStartStyle = _objectSpread2(_objectSpread2({}, clipStyle), clipStart);
              var clipEndStyle = _objectSpread2(_objectSpread2({}, clipStyle), clipEnd);
              // 开始帧
              var clipKeyframeStart = clipProperty.reduce(function (prev, cur) {
                prev[cur] = clipStartStyle[cur];
                return prev;
              }, {});
              // 结束帧
              var clipKeyframeEnd = pick(clipEndStyle, clipProperty);
              var clipShape = createShape(clipType, {
                style: clipStartStyle
              });
              shape.setAttribute('clipPath', clipShape);
              // g 中 clip 为全局，且如果要在 clip上加动画，需要手动加到canvas上
              shape.ownerDocument.documentElement.appendChild(clipShape);
              var clipAnimation = clipShape.animate([clipKeyframeStart, clipKeyframeEnd], {
                fill: 'both',
                easing: clipEasing || easing,
                duration: clipDuration || duration,
                delay: clipDelay || delay,
                iterations: clipIterations || iterations
              });
              // 过滤无限循环的动画
              if (clipAnimation && (clipIterations || iterations) !== Infinity) {
                var clipFinished = clipAnimation.finished;
                animations.push(clipFinished);
                clipFinished.then(function () {
                  // 删掉 clip
                  shape.setAttribute('clipPath', null);
                });
              }
            }
          }
        }
        if (children && children.length) {
          children.forEach(function (child) {
            if (!child) return;
            var childAnimator = child.play();
            if (childAnimator) {
              animations.push(childAnimator);
            }
          });
        }
        if (!animations.length) {
          this.emit('end');
          return null;
        }
        var finished = Promise.all(animations);
        finished.then(function () {
          _this.emit('end');
        });
        return finished;
      }
    }, {
      key: "reset",
      value: function reset() {
        this.shape = null;
        this.start = null;
        this.end = null;
        this.effect = null;
        this.children = null;
      }
    }]);
    return Animator;
  }(EventEmitter);

  var FunctionComponent = 0;
  var ClassComponent = 1;
  var Shape$1 = 2;
  function getWorkTag(type) {
    if (isString(type)) {
      return Shape$1;
    }
    if (type.prototype && type.prototype.isF2Component) {
      return ClassComponent;
    }
    return FunctionComponent;
  }

  function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
  }

  function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
  }

  function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
  }

  var rect = (function (layout) {
    var left = layout.left,
      top = layout.top,
      width = layout.width,
      height = layout.height;
    return {
      x: left,
      y: top,
      width: width,
      height: height
    };
  });

  var line$1 = (function (layout) {
    var left = layout.left,
      top = layout.top,
      width = layout.width,
      height = layout.height;
    return {
      x1: left,
      y1: top,
      x2: left + width,
      y2: top + height
    };
  });

  var text = (function (layout) {
    var height = layout.height,
      left = layout.left,
      top = layout.top;
    return {
      x: left,
      y: top + height / 2,
      // 通过middle + top 才能比较好的实现文本对齐
      textBaseline: 'middle'
    };
  });

  var circle = (function (layout) {
    var left = layout.left,
      top = layout.top,
      width = layout.width;
    var r = width / 2;
    return {
      cx: left + r,
      cy: top + r,
      r: r
    };
  });

  var marker = (function (layout) {
    var left = layout.left,
      top = layout.top,
      width = layout.width;
    var r = width / 2;
    return {
      x: left + r,
      y: top,
      radius: r
    };
  });

  var map$1 = {
    rect: rect,
    line: line$1,
    text: text,
    circle: circle,
    marker: marker,
    group: rect
  };
  var getShapeAttrs = (function (type, layout) {
    if (!layout) return null;
    var fn = map$1[type] || rect;
    return fn(layout);
  });

  /* eslint-disable */
  // @ts-nocheck
  // from css-layout
  var CSS_UNDEFINED;
  var CSS_DIRECTION_INHERIT = 'inherit';
  var CSS_DIRECTION_LTR = 'ltr';
  var CSS_DIRECTION_RTL = 'rtl';
  var CSS_FLEX_DIRECTION_ROW = 'row';
  var CSS_FLEX_DIRECTION_ROW_REVERSE = 'row-reverse';
  var CSS_FLEX_DIRECTION_COLUMN = 'column';
  var CSS_FLEX_DIRECTION_COLUMN_REVERSE = 'column-reverse';
  var CSS_JUSTIFY_FLEX_START = 'flex-start';
  var CSS_JUSTIFY_CENTER = 'center';
  var CSS_JUSTIFY_FLEX_END = 'flex-end';
  var CSS_JUSTIFY_SPACE_BETWEEN = 'space-between';
  var CSS_JUSTIFY_SPACE_AROUND = 'space-around';
  var CSS_ALIGN_FLEX_START = 'flex-start';
  var CSS_ALIGN_CENTER = 'center';
  var CSS_ALIGN_FLEX_END = 'flex-end';
  var CSS_ALIGN_STRETCH = 'stretch';
  var CSS_POSITION_RELATIVE = 'relative';
  var CSS_POSITION_ABSOLUTE = 'absolute';
  var leading = {
    row: 'left',
    'row-reverse': 'right',
    column: 'top',
    'column-reverse': 'bottom'
  };
  var trailing = {
    row: 'right',
    'row-reverse': 'left',
    column: 'bottom',
    'column-reverse': 'top'
  };
  var pos = {
    row: 'left',
    'row-reverse': 'right',
    column: 'top',
    'column-reverse': 'bottom'
  };
  var dim = {
    row: 'width',
    'row-reverse': 'width',
    column: 'height',
    'column-reverse': 'height'
  };
  // When transpiled to Java / C the node type has layout, children and style
  // properties. For the JavaScript version this function adds these properties
  // if they don't already exist.
  function fillNodes(node) {
    if (!node.layout || node.isDirty) {
      node.layout = {
        width: undefined,
        height: undefined,
        top: 0,
        left: 0,
        right: 0,
        bottom: 0
      };
    }
    if (!node.style) {
      node.style = {};
    }
    if (!node.children) {
      node.children = [];
    }
    node.children.forEach(fillNodes);
    return node;
  }
  function isUndefined$1(value) {
    return value === undefined;
  }
  function isRowDirection(flexDirection) {
    return flexDirection === CSS_FLEX_DIRECTION_ROW || flexDirection === CSS_FLEX_DIRECTION_ROW_REVERSE;
  }
  function isColumnDirection(flexDirection) {
    return flexDirection === CSS_FLEX_DIRECTION_COLUMN || flexDirection === CSS_FLEX_DIRECTION_COLUMN_REVERSE;
  }
  function getLeadingMargin(node, axis) {
    if (node.style.marginStart !== undefined && isRowDirection(axis)) {
      return node.style.marginStart;
    }
    var value = null;
    switch (axis) {
      case 'row':
        value = node.style.marginLeft;
        break;
      case 'row-reverse':
        value = node.style.marginRight;
        break;
      case 'column':
        value = node.style.marginTop;
        break;
      case 'column-reverse':
        value = node.style.marginBottom;
        break;
    }
    if (value !== undefined) {
      return value;
    }
    if (node.style.margin !== undefined) {
      return node.style.margin;
    }
    return 0;
  }
  function getTrailingMargin(node, axis) {
    if (node.style.marginEnd !== undefined && isRowDirection(axis)) {
      return node.style.marginEnd;
    }
    var value = null;
    switch (axis) {
      case 'row':
        value = node.style.marginRight;
        break;
      case 'row-reverse':
        value = node.style.marginLeft;
        break;
      case 'column':
        value = node.style.marginBottom;
        break;
      case 'column-reverse':
        value = node.style.marginTop;
        break;
    }
    if (value != null) {
      return value;
    }
    if (node.style.margin !== undefined) {
      return node.style.margin;
    }
    return 0;
  }
  function getLeadingPadding(node, axis) {
    if (node.style.paddingStart !== undefined && node.style.paddingStart >= 0 && isRowDirection(axis)) {
      return node.style.paddingStart;
    }
    var value = null;
    switch (axis) {
      case 'row':
        value = node.style.paddingLeft;
        break;
      case 'row-reverse':
        value = node.style.paddingRight;
        break;
      case 'column':
        value = node.style.paddingTop;
        break;
      case 'column-reverse':
        value = node.style.paddingBottom;
        break;
    }
    if (value != null && value >= 0) {
      return value;
    }
    if (node.style.padding !== undefined && node.style.padding >= 0) {
      return node.style.padding;
    }
    return 0;
  }
  function getTrailingPadding(node, axis) {
    if (node.style.paddingEnd !== undefined && node.style.paddingEnd >= 0 && isRowDirection(axis)) {
      return node.style.paddingEnd;
    }
    var value = null;
    switch (axis) {
      case 'row':
        value = node.style.paddingRight;
        break;
      case 'row-reverse':
        value = node.style.paddingLeft;
        break;
      case 'column':
        value = node.style.paddingBottom;
        break;
      case 'column-reverse':
        value = node.style.paddingTop;
        break;
    }
    if (value != null && value >= 0) {
      return value;
    }
    if (node.style.padding !== undefined && node.style.padding >= 0) {
      return node.style.padding;
    }
    return 0;
  }
  function getLeadingBorder(node, axis) {
    if (node.style.borderStartWidth !== undefined && node.style.borderStartWidth >= 0 && isRowDirection(axis)) {
      return node.style.borderStartWidth;
    }
    var value = null;
    switch (axis) {
      case 'row':
        value = node.style.borderLeftWidth;
        break;
      case 'row-reverse':
        value = node.style.borderRightWidth;
        break;
      case 'column':
        value = node.style.borderTopWidth;
        break;
      case 'column-reverse':
        value = node.style.borderBottomWidth;
        break;
    }
    if (value != null && value >= 0) {
      return value;
    }
    if (node.style.borderWidth !== undefined && node.style.borderWidth >= 0) {
      return node.style.borderWidth;
    }
    return 0;
  }
  function getTrailingBorder(node, axis) {
    if (node.style.borderEndWidth !== undefined && node.style.borderEndWidth >= 0 && isRowDirection(axis)) {
      return node.style.borderEndWidth;
    }
    var value = null;
    switch (axis) {
      case 'row':
        value = node.style.borderRightWidth;
        break;
      case 'row-reverse':
        value = node.style.borderLeftWidth;
        break;
      case 'column':
        value = node.style.borderBottomWidth;
        break;
      case 'column-reverse':
        value = node.style.borderTopWidth;
        break;
    }
    if (value != null && value >= 0) {
      return value;
    }
    if (node.style.borderWidth !== undefined && node.style.borderWidth >= 0) {
      return node.style.borderWidth;
    }
    return 0;
  }
  function getLeadingPaddingAndBorder(node, axis) {
    return getLeadingPadding(node, axis) + getLeadingBorder(node, axis);
  }
  function getTrailingPaddingAndBorder(node, axis) {
    return getTrailingPadding(node, axis) + getTrailingBorder(node, axis);
  }
  function getBorderAxis(node, axis) {
    return getLeadingBorder(node, axis) + getTrailingBorder(node, axis);
  }
  function getMarginAxis(node, axis) {
    return getLeadingMargin(node, axis) + getTrailingMargin(node, axis);
  }
  function getPaddingAndBorderAxis(node, axis) {
    return getLeadingPaddingAndBorder(node, axis) + getTrailingPaddingAndBorder(node, axis);
  }
  function getJustifyContent(node) {
    if (node.style.justifyContent) {
      return node.style.justifyContent;
    }
    return 'flex-start';
  }
  function getAlignContent(node) {
    if (node.style.alignContent) {
      return node.style.alignContent;
    }
    return 'flex-start';
  }
  function getAlignItem(node, child) {
    if (child.style.alignSelf) {
      return child.style.alignSelf;
    }
    if (node.style.alignItems) {
      return node.style.alignItems;
    }
    return 'stretch';
  }
  function resolveAxis(axis, direction) {
    if (direction === CSS_DIRECTION_RTL) {
      if (axis === CSS_FLEX_DIRECTION_ROW) {
        return CSS_FLEX_DIRECTION_ROW_REVERSE;
      } else if (axis === CSS_FLEX_DIRECTION_ROW_REVERSE) {
        return CSS_FLEX_DIRECTION_ROW;
      }
    }
    return axis;
  }
  function resolveDirection(node, parentDirection) {
    var direction;
    if (node.style.direction) {
      direction = node.style.direction;
    } else {
      direction = CSS_DIRECTION_INHERIT;
    }
    if (direction === CSS_DIRECTION_INHERIT) {
      direction = parentDirection === undefined ? CSS_DIRECTION_LTR : parentDirection;
    }
    return direction;
  }
  function getFlexDirection(node) {
    if (node.style.flexDirection) {
      return node.style.flexDirection;
    }
    return CSS_FLEX_DIRECTION_COLUMN;
  }
  function getCrossFlexDirection(flexDirection, direction) {
    if (isColumnDirection(flexDirection)) {
      return resolveAxis(CSS_FLEX_DIRECTION_ROW, direction);
    } else {
      return CSS_FLEX_DIRECTION_COLUMN;
    }
  }
  function getPositionType(node) {
    if (node.style.position) {
      return node.style.position;
    }
    return 'relative';
  }
  function isFlex(node) {
    return getPositionType(node) === CSS_POSITION_RELATIVE && node.style.flex > 0;
  }
  function isFlexWrap(node) {
    return node.style.flexWrap === 'wrap';
  }
  function getDimWithMargin(node, axis) {
    return node.layout[dim[axis]] + getMarginAxis(node, axis);
  }
  function isDimDefined(node, axis) {
    return node.style[dim[axis]] !== undefined && node.style[dim[axis]] >= 0;
  }
  function isPosDefined(node, pos) {
    return node.style[pos] !== undefined;
  }
  function isMeasureDefined(node) {
    return node.style.measure !== undefined;
  }
  function getPosition(node, pos) {
    if (node.style[pos] !== undefined) {
      return node.style[pos];
    }
    return 0;
  }
  function boundAxis(node, axis, value) {
    var min = {
      row: node.style.minWidth,
      'row-reverse': node.style.minWidth,
      column: node.style.minHeight,
      'column-reverse': node.style.minHeight
    }[axis];
    var max = {
      row: node.style.maxWidth,
      'row-reverse': node.style.maxWidth,
      column: node.style.maxHeight,
      'column-reverse': node.style.maxHeight
    }[axis];
    var boundValue = value;
    if (max !== undefined && max >= 0 && boundValue > max) {
      boundValue = max;
    }
    if (min !== undefined && min >= 0 && boundValue < min) {
      boundValue = min;
    }
    return boundValue;
  }
  function fmaxf(a, b) {
    if (a > b) {
      return a;
    }
    return b;
  }
  // When the user specifically sets a value for width or height
  function setDimensionFromStyle(node, axis) {
    // The parent already computed us a width or height. We just skip it
    if (node.layout[dim[axis]] !== undefined) {
      return;
    }
    // We only run if there's a width or height defined
    if (!isDimDefined(node, axis)) {
      return;
    }
    // The dimensions can never be smaller than the padding and border
    node.layout[dim[axis]] = fmaxf(boundAxis(node, axis, node.style[dim[axis]]), getPaddingAndBorderAxis(node, axis));
  }
  function setTrailingPosition(node, child, axis) {
    child.layout[trailing[axis]] = node.layout[dim[axis]] - child.layout[dim[axis]] - child.layout[pos[axis]];
  }
  // If both left and right are defined, then use left. Otherwise return
  // +left or -right depending on which is defined.
  function getRelativePosition(node, axis) {
    if (node.style[leading[axis]] !== undefined) {
      return getPosition(node, leading[axis]);
    }
    return -getPosition(node, trailing[axis]);
  }
  function layoutNodeImpl(node, parentMaxWidth, /*css_direction_t*/parentDirection) {
    var /*css_direction_t*/direction = resolveDirection(node, parentDirection);
    var /*(c)!css_flex_direction_t*/ /*(java)!int*/mainAxis = resolveAxis(getFlexDirection(node), direction);
    var /*(c)!css_flex_direction_t*/ /*(java)!int*/crossAxis = getCrossFlexDirection(mainAxis, direction);
    var /*(c)!css_flex_direction_t*/ /*(java)!int*/resolvedRowAxis = resolveAxis(CSS_FLEX_DIRECTION_ROW, direction);
    // Handle width and height style attributes
    setDimensionFromStyle(node, mainAxis);
    setDimensionFromStyle(node, crossAxis);
    // Set the resolved resolution in the node's layout
    node.layout.direction = direction;
    // The position is set by the parent, but we need to complete it with a
    // delta composed of the margin and left/top/right/bottom
    node.layout[leading[mainAxis]] += getLeadingMargin(node, mainAxis) + getRelativePosition(node, mainAxis);
    node.layout[trailing[mainAxis]] += getTrailingMargin(node, mainAxis) + getRelativePosition(node, mainAxis);
    node.layout[leading[crossAxis]] += getLeadingMargin(node, crossAxis) + getRelativePosition(node, crossAxis);
    node.layout[trailing[crossAxis]] += getTrailingMargin(node, crossAxis) + getRelativePosition(node, crossAxis);
    // Inline immutable values from the target node to avoid excessive method
    // invocations during the layout calculation.
    var /*int*/childCount = node.children.length;
    var /*float*/paddingAndBorderAxisResolvedRow = getPaddingAndBorderAxis(node, resolvedRowAxis);
    if (isMeasureDefined(node)) {
      var /*bool*/isResolvedRowDimDefined = !isUndefined$1(node.layout[dim[resolvedRowAxis]]);
      var /*float*/width = CSS_UNDEFINED;
      if (isDimDefined(node, resolvedRowAxis)) {
        width = node.style.width;
      } else if (isResolvedRowDimDefined) {
        width = node.layout[dim[resolvedRowAxis]];
      } else {
        width = parentMaxWidth - getMarginAxis(node, resolvedRowAxis);
      }
      width -= paddingAndBorderAxisResolvedRow;
      // We only need to give a dimension for the text if we haven't got any
      // for it computed yet. It can either be from the style attribute or because
      // the element is flexible.
      var /*bool*/isRowUndefined = !isDimDefined(node, resolvedRowAxis) && !isResolvedRowDimDefined;
      var /*bool*/isColumnUndefined = !isDimDefined(node, CSS_FLEX_DIRECTION_COLUMN) && isUndefined$1(node.layout[dim[CSS_FLEX_DIRECTION_COLUMN]]);
      // Let's not measure the text if we already know both dimensions
      if (isRowUndefined || isColumnUndefined) {
        var /*css_dim_t*/measureDim = node.style.measure( /*(c)!node->context,*/
        /*(java)!layoutContext.measureOutput,*/
        width);
        if (isRowUndefined) {
          node.layout.width = measureDim.width + paddingAndBorderAxisResolvedRow;
        }
        if (isColumnUndefined) {
          node.layout.height = measureDim.height + getPaddingAndBorderAxis(node, CSS_FLEX_DIRECTION_COLUMN);
        }
      }
      if (childCount === 0) {
        return;
      }
    }
    var /*bool*/isNodeFlexWrap = isFlexWrap(node);
    var /*css_justify_t*/justifyContent = getJustifyContent(node);
    var /*float*/leadingPaddingAndBorderMain = getLeadingPaddingAndBorder(node, mainAxis);
    var /*float*/leadingPaddingAndBorderCross = getLeadingPaddingAndBorder(node, crossAxis);
    var /*float*/paddingAndBorderAxisMain = getPaddingAndBorderAxis(node, mainAxis);
    var /*float*/paddingAndBorderAxisCross = getPaddingAndBorderAxis(node, crossAxis);
    var /*bool*/isMainDimDefined = !isUndefined$1(node.layout[dim[mainAxis]]);
    var /*bool*/isCrossDimDefined = !isUndefined$1(node.layout[dim[crossAxis]]);
    var /*bool*/isMainRowDirection = isRowDirection(mainAxis);
    var /*int*/i;
    var /*int*/ii;
    var /*css_node_t**/child;
    var /*(c)!css_flex_direction_t*/ /*(java)!int*/axis;
    var /*css_node_t**/firstAbsoluteChild = null;
    var /*css_node_t**/currentAbsoluteChild = null;
    var /*float*/definedMainDim = CSS_UNDEFINED;
    if (isMainDimDefined) {
      definedMainDim = node.layout[dim[mainAxis]] - paddingAndBorderAxisMain;
    }
    // We want to execute the next two loops one per line with flex-wrap
    var /*int*/startLine = 0;
    var /*int*/endLine = 0;
    // var/*int*/ nextOffset = 0;
    var /*int*/alreadyComputedNextLayout = 0;
    // We aggregate the total dimensions of the container in those two variables
    var /*float*/linesCrossDim = 0;
    var /*float*/linesMainDim = 0;
    var /*int*/linesCount = 0;
    while (endLine < childCount) {
      // <Loop A> Layout non flexible children and count children by type
      // mainContentDim is accumulation of the dimensions and margin of all the
      // non flexible children. This will be used in order to either set the
      // dimensions of the node if none already exist, or to compute the
      // remaining space left for the flexible children.
      var /*float*/mainContentDim = 0;
      // There are three kind of children, non flexible, flexible and absolute.
      // We need to know how many there are in order to distribute the space.
      var /*int*/flexibleChildrenCount = 0;
      var /*float*/totalFlexible = 0;
      var /*int*/nonFlexibleChildrenCount = 0;
      // Use the line loop to position children in the main axis for as long
      // as they are using a simple stacking behaviour. Children that are
      // immediately stacked in the initial loop will not be touched again
      // in <Loop C>.
      var /*bool*/isSimpleStackMain = isMainDimDefined && justifyContent === CSS_JUSTIFY_FLEX_START || !isMainDimDefined && justifyContent !== CSS_JUSTIFY_CENTER;
      var /*int*/firstComplexMain = isSimpleStackMain ? childCount : startLine;
      // Use the initial line loop to position children in the cross axis for
      // as long as they are relatively positioned with alignment STRETCH or
      // FLEX_START. Children that are immediately stacked in the initial loop
      // will not be touched again in <Loop D>.
      var /*bool*/isSimpleStackCross = true;
      var /*int*/firstComplexCross = childCount;
      var /*css_node_t**/firstFlexChild = null;
      var /*css_node_t**/currentFlexChild = null;
      var /*float*/mainDim = leadingPaddingAndBorderMain;
      var /*float*/crossDim = 0;
      var /*float*/maxWidth;
      for (i = startLine; i < childCount; ++i) {
        child = node.children[i];
        child.lineIndex = linesCount;
        child.nextAbsoluteChild = null;
        child.nextFlexChild = null;
        var /*css_align_t*/alignItem = getAlignItem(node, child);
        // Pre-fill cross axis dimensions when the child is using stretch before
        // we call the recursive layout pass
        if (alignItem === CSS_ALIGN_STRETCH && getPositionType(child) === CSS_POSITION_RELATIVE && isCrossDimDefined && !isDimDefined(child, crossAxis)) {
          child.layout[dim[crossAxis]] = fmaxf(boundAxis(child, crossAxis, node.layout[dim[crossAxis]] - paddingAndBorderAxisCross - getMarginAxis(child, crossAxis)),
          // You never want to go smaller than padding
          getPaddingAndBorderAxis(child, crossAxis));
        } else if (getPositionType(child) === CSS_POSITION_ABSOLUTE) {
          // Store a private linked list of absolutely positioned children
          // so that we can efficiently traverse them later.
          if (firstAbsoluteChild === null) {
            firstAbsoluteChild = child;
          }
          if (currentAbsoluteChild !== null) {
            currentAbsoluteChild.nextAbsoluteChild = child;
          }
          currentAbsoluteChild = child;
          // Pre-fill dimensions when using absolute position and both offsets for the axis are defined (either both
          // left and right or top and bottom).
          for (ii = 0; ii < 2; ii++) {
            axis = ii !== 0 ? CSS_FLEX_DIRECTION_ROW : CSS_FLEX_DIRECTION_COLUMN;
            if (!isUndefined$1(node.layout[dim[axis]]) && !isDimDefined(child, axis) && isPosDefined(child, leading[axis]) && isPosDefined(child, trailing[axis])) {
              child.layout[dim[axis]] = fmaxf(boundAxis(child, axis, node.layout[dim[axis]] - getPaddingAndBorderAxis(node, axis) - getMarginAxis(child, axis) - getPosition(child, leading[axis]) - getPosition(child, trailing[axis])),
              // You never want to go smaller than padding
              getPaddingAndBorderAxis(child, axis));
            }
          }
        }
        var /*float*/nextContentDim = 0;
        // It only makes sense to consider a child flexible if we have a computed
        // dimension for the node.
        if (isMainDimDefined && isFlex(child)) {
          flexibleChildrenCount++;
          totalFlexible += child.style.flex;
          // Store a private linked list of flexible children so that we can
          // efficiently traverse them later.
          if (firstFlexChild === null) {
            firstFlexChild = child;
          }
          if (currentFlexChild !== null) {
            currentFlexChild.nextFlexChild = child;
          }
          currentFlexChild = child;
          // Even if we don't know its exact size yet, we already know the padding,
          // border and margin. We'll use this partial information, which represents
          // the smallest possible size for the child, to compute the remaining
          // available space.
          nextContentDim = getPaddingAndBorderAxis(child, mainAxis) + getMarginAxis(child, mainAxis);
        } else {
          maxWidth = CSS_UNDEFINED;
          if (!isMainRowDirection) {
            if (isDimDefined(node, resolvedRowAxis)) {
              maxWidth = node.layout[dim[resolvedRowAxis]] - paddingAndBorderAxisResolvedRow;
            } else {
              maxWidth = parentMaxWidth - getMarginAxis(node, resolvedRowAxis) - paddingAndBorderAxisResolvedRow;
            }
          }
          // This is the main recursive call. We layout non flexible children.
          if (alreadyComputedNextLayout === 0) {
            layoutNode( /*(java)!layoutContext, */child, maxWidth, direction);
          }
          // Absolute positioned elements do not take part of the layout, so we
          // don't use them to compute mainContentDim
          if (getPositionType(child) === CSS_POSITION_RELATIVE) {
            nonFlexibleChildrenCount++;
            // At this point we know the final size and margin of the element.
            nextContentDim = getDimWithMargin(child, mainAxis);
          }
        }
        // The element we are about to add would make us go to the next line
        if (isNodeFlexWrap && isMainDimDefined && mainContentDim + nextContentDim > definedMainDim &&
        // If there's only one element, then it's bigger than the content
        // and needs its own line
        i !== startLine) {
          nonFlexibleChildrenCount--;
          alreadyComputedNextLayout = 1;
          break;
        }
        // Disable simple stacking in the main axis for the current line as
        // we found a non-trivial child. The remaining children will be laid out
        // in <Loop C>.
        if (isSimpleStackMain && (getPositionType(child) !== CSS_POSITION_RELATIVE || isFlex(child))) {
          isSimpleStackMain = false;
          firstComplexMain = i;
        }
        // Disable simple stacking in the cross axis for the current line as
        // we found a non-trivial child. The remaining children will be laid out
        // in <Loop D>.
        if (isSimpleStackCross && (getPositionType(child) !== CSS_POSITION_RELATIVE || alignItem !== CSS_ALIGN_STRETCH && alignItem !== CSS_ALIGN_FLEX_START || isUndefined$1(child.layout[dim[crossAxis]]))) {
          isSimpleStackCross = false;
          firstComplexCross = i;
        }
        if (isSimpleStackMain) {
          child.layout[pos[mainAxis]] += mainDim;
          if (isMainDimDefined) {
            setTrailingPosition(node, child, mainAxis);
          }
          mainDim += getDimWithMargin(child, mainAxis);
          crossDim = fmaxf(crossDim, boundAxis(child, crossAxis, getDimWithMargin(child, crossAxis)));
        }
        if (isSimpleStackCross) {
          child.layout[pos[crossAxis]] += linesCrossDim + leadingPaddingAndBorderCross;
          if (isCrossDimDefined) {
            setTrailingPosition(node, child, crossAxis);
          }
        }
        alreadyComputedNextLayout = 0;
        mainContentDim += nextContentDim;
        endLine = i + 1;
      }
      // <Loop B> Layout flexible children and allocate empty space
      // In order to position the elements in the main axis, we have two
      // controls. The space between the beginning and the first element
      // and the space between each two elements.
      var /*float*/leadingMainDim = 0;
      var /*float*/betweenMainDim = 0;
      // The remaining available space that needs to be allocated
      var /*float*/remainingMainDim = 0;
      if (isMainDimDefined) {
        remainingMainDim = definedMainDim - mainContentDim;
      } else {
        remainingMainDim = fmaxf(mainContentDim, 0) - mainContentDim;
      }
      // If there are flexible children in the mix, they are going to fill the
      // remaining space
      if (flexibleChildrenCount !== 0) {
        var /*float*/flexibleMainDim = remainingMainDim / totalFlexible;
        var /*float*/baseMainDim;
        var /*float*/boundMainDim;
        // If the flex share of remaining space doesn't meet min/max bounds,
        // remove this child from flex calculations.
        currentFlexChild = firstFlexChild;
        while (currentFlexChild !== null) {
          baseMainDim = flexibleMainDim * currentFlexChild.style.flex + getPaddingAndBorderAxis(currentFlexChild, mainAxis);
          boundMainDim = boundAxis(currentFlexChild, mainAxis, baseMainDim);
          if (baseMainDim !== boundMainDim) {
            remainingMainDim -= boundMainDim;
            totalFlexible -= currentFlexChild.style.flex;
          }
          currentFlexChild = currentFlexChild.nextFlexChild;
        }
        flexibleMainDim = remainingMainDim / totalFlexible;
        // The non flexible children can overflow the container, in this case
        // we should just assume that there is no space available.
        if (flexibleMainDim < 0) {
          flexibleMainDim = 0;
        }
        currentFlexChild = firstFlexChild;
        while (currentFlexChild !== null) {
          // At this point we know the final size of the element in the main
          // dimension
          currentFlexChild.layout[dim[mainAxis]] = boundAxis(currentFlexChild, mainAxis, flexibleMainDim * currentFlexChild.style.flex + getPaddingAndBorderAxis(currentFlexChild, mainAxis));
          maxWidth = CSS_UNDEFINED;
          if (isDimDefined(node, resolvedRowAxis)) {
            maxWidth = node.layout[dim[resolvedRowAxis]] - paddingAndBorderAxisResolvedRow;
          } else if (!isMainRowDirection) {
            maxWidth = parentMaxWidth - getMarginAxis(node, resolvedRowAxis) - paddingAndBorderAxisResolvedRow;
          }
          // And we recursively call the layout algorithm for this child
          layoutNode( /*(java)!layoutContext, */currentFlexChild, maxWidth, direction);
          child = currentFlexChild;
          currentFlexChild = currentFlexChild.nextFlexChild;
          child.nextFlexChild = null;
        }
        // We use justifyContent to figure out how to allocate the remaining
        // space available
      } else if (justifyContent !== CSS_JUSTIFY_FLEX_START) {
        if (justifyContent === CSS_JUSTIFY_CENTER) {
          leadingMainDim = remainingMainDim / 2;
        } else if (justifyContent === CSS_JUSTIFY_FLEX_END) {
          leadingMainDim = remainingMainDim;
        } else if (justifyContent === CSS_JUSTIFY_SPACE_BETWEEN) {
          remainingMainDim = fmaxf(remainingMainDim, 0);
          if (flexibleChildrenCount + nonFlexibleChildrenCount - 1 !== 0) {
            betweenMainDim = remainingMainDim / (flexibleChildrenCount + nonFlexibleChildrenCount - 1);
          } else {
            betweenMainDim = 0;
          }
        } else if (justifyContent === CSS_JUSTIFY_SPACE_AROUND) {
          // Space on the edges is half of the space between elements
          betweenMainDim = remainingMainDim / (flexibleChildrenCount + nonFlexibleChildrenCount);
          leadingMainDim = betweenMainDim / 2;
        }
      }
      // <Loop C> Position elements in the main axis and compute dimensions
      // At this point, all the children have their dimensions set. We need to
      // find their position. In order to do that, we accumulate data in
      // variables that are also useful to compute the total dimensions of the
      // container!
      mainDim += leadingMainDim;
      for (i = firstComplexMain; i < endLine; ++i) {
        child = node.children[i];
        if (getPositionType(child) === CSS_POSITION_ABSOLUTE && isPosDefined(child, leading[mainAxis])) {
          // In case the child is position absolute and has left/top being
          // defined, we override the position to whatever the user said
          // (and margin/border).
          child.layout[pos[mainAxis]] = getPosition(child, leading[mainAxis]) + getLeadingBorder(node, mainAxis) + getLeadingMargin(child, mainAxis);
        } else {
          // If the child is position absolute (without top/left) or relative,
          // we put it at the current accumulated offset.
          child.layout[pos[mainAxis]] += mainDim;
          // Define the trailing position accordingly.
          if (isMainDimDefined) {
            setTrailingPosition(node, child, mainAxis);
          }
          // Now that we placed the element, we need to update the variables
          // We only need to do that for relative elements. Absolute elements
          // do not take part in that phase.
          if (getPositionType(child) === CSS_POSITION_RELATIVE) {
            // The main dimension is the sum of all the elements dimension plus
            // the spacing.
            mainDim += betweenMainDim + getDimWithMargin(child, mainAxis);
            // The cross dimension is the max of the elements dimension since there
            // can only be one element in that cross dimension.
            crossDim = fmaxf(crossDim, boundAxis(child, crossAxis, getDimWithMargin(child, crossAxis)));
          }
        }
      }
      var /*float*/containerCrossAxis = node.layout[dim[crossAxis]];
      if (!isCrossDimDefined) {
        containerCrossAxis = fmaxf(
        // For the cross dim, we add both sides at the end because the value
        // is aggregate via a max function. Intermediate negative values
        // can mess this computation otherwise
        boundAxis(node, crossAxis, crossDim + paddingAndBorderAxisCross), paddingAndBorderAxisCross);
      }
      // <Loop D> Position elements in the cross axis
      for (i = firstComplexCross; i < endLine; ++i) {
        child = node.children[i];
        if (getPositionType(child) === CSS_POSITION_ABSOLUTE && isPosDefined(child, leading[crossAxis])) {
          // In case the child is absolutely positionned and has a
          // top/left/bottom/right being set, we override all the previously
          // computed positions to set it correctly.
          child.layout[pos[crossAxis]] = getPosition(child, leading[crossAxis]) + getLeadingBorder(node, crossAxis) + getLeadingMargin(child, crossAxis);
        } else {
          var /*float*/leadingCrossDim = leadingPaddingAndBorderCross;
          // For a relative children, we're either using alignItems (parent) or
          // alignSelf (child) in order to determine the position in the cross axis
          if (getPositionType(child) === CSS_POSITION_RELATIVE) {
            // This variable is intentionally re-defined as the code is transpiled to a block scope language
            var /*css_align_t*/alignItem = getAlignItem(node, child);
            if (alignItem === CSS_ALIGN_STRETCH) {
              // You can only stretch if the dimension has not already been set
              // previously.
              if (isUndefined$1(child.layout[dim[crossAxis]])) {
                child.layout[dim[crossAxis]] = fmaxf(boundAxis(child, crossAxis, containerCrossAxis - paddingAndBorderAxisCross - getMarginAxis(child, crossAxis)),
                // You never want to go smaller than padding
                getPaddingAndBorderAxis(child, crossAxis));
              }
            } else if (alignItem !== CSS_ALIGN_FLEX_START) {
              // The remaining space between the parent dimensions+padding and child
              // dimensions+margin.
              var /*float*/remainingCrossDim = containerCrossAxis - paddingAndBorderAxisCross - getDimWithMargin(child, crossAxis);
              if (alignItem === CSS_ALIGN_CENTER) {
                leadingCrossDim += remainingCrossDim / 2;
              } else {
                // CSS_ALIGN_FLEX_END
                leadingCrossDim += remainingCrossDim;
              }
            }
          }
          // And we apply the position
          child.layout[pos[crossAxis]] += linesCrossDim + leadingCrossDim;
          // Define the trailing position accordingly.
          if (isCrossDimDefined) {
            setTrailingPosition(node, child, crossAxis);
          }
        }
      }
      linesCrossDim += crossDim;
      linesMainDim = fmaxf(linesMainDim, mainDim);
      linesCount += 1;
      startLine = endLine;
    }
    // <Loop E>
    //
    // Note(prenaux): More than one line, we need to layout the crossAxis
    // according to alignContent.
    //
    // Note that we could probably remove <Loop D> and handle the one line case
    // here too, but for the moment this is safer since it won't interfere with
    // previously working code.
    //
    // See specs:
    // http://www.w3.org/TR/2012/CR-css3-flexbox-20120918/#layout-algorithm
    // section 9.4
    //
    if (linesCount > 1 && isCrossDimDefined) {
      var /*float*/nodeCrossAxisInnerSize = node.layout[dim[crossAxis]] - paddingAndBorderAxisCross;
      var /*float*/remainingAlignContentDim = nodeCrossAxisInnerSize - linesCrossDim;
      var /*float*/crossDimLead = 0;
      var /*float*/currentLead = leadingPaddingAndBorderCross;
      var /*css_align_t*/alignContent = getAlignContent(node);
      if (alignContent === CSS_ALIGN_FLEX_END) {
        currentLead += remainingAlignContentDim;
      } else if (alignContent === CSS_ALIGN_CENTER) {
        currentLead += remainingAlignContentDim / 2;
      } else if (alignContent === CSS_ALIGN_STRETCH) {
        if (nodeCrossAxisInnerSize > linesCrossDim) {
          crossDimLead = remainingAlignContentDim / linesCount;
        }
      }
      var /*int*/endIndex = 0;
      for (i = 0; i < linesCount; ++i) {
        var /*int*/startIndex = endIndex;
        // compute the line's height and find the endIndex
        var /*float*/lineHeight = 0;
        for (ii = startIndex; ii < childCount; ++ii) {
          child = node.children[ii];
          if (getPositionType(child) !== CSS_POSITION_RELATIVE) {
            continue;
          }
          if (child.lineIndex !== i) {
            break;
          }
          if (!isUndefined$1(child.layout[dim[crossAxis]])) {
            lineHeight = fmaxf(lineHeight, child.layout[dim[crossAxis]] + getMarginAxis(child, crossAxis));
          }
        }
        endIndex = ii;
        lineHeight += crossDimLead;
        for (ii = startIndex; ii < endIndex; ++ii) {
          child = node.children[ii];
          if (getPositionType(child) !== CSS_POSITION_RELATIVE) {
            continue;
          }
          var /*css_align_t*/alignContentAlignItem = getAlignItem(node, child);
          if (alignContentAlignItem === CSS_ALIGN_FLEX_START) {
            child.layout[pos[crossAxis]] = currentLead + getLeadingMargin(child, crossAxis);
          } else if (alignContentAlignItem === CSS_ALIGN_FLEX_END) {
            child.layout[pos[crossAxis]] = currentLead + lineHeight - getTrailingMargin(child, crossAxis) - child.layout[dim[crossAxis]];
          } else if (alignContentAlignItem === CSS_ALIGN_CENTER) {
            var /*float*/childHeight = child.layout[dim[crossAxis]];
            child.layout[pos[crossAxis]] = currentLead + (lineHeight - childHeight) / 2;
          } else if (alignContentAlignItem === CSS_ALIGN_STRETCH) {
            child.layout[pos[crossAxis]] = currentLead + getLeadingMargin(child, crossAxis);
            // TODO(prenaux): Correctly set the height of items with undefined
            //                (auto) crossAxis dimension.
          }
        }

        currentLead += lineHeight;
      }
    }
    var /*bool*/needsMainTrailingPos = false;
    var /*bool*/needsCrossTrailingPos = false;
    // If the user didn't specify a width or height, and it has not been set
    // by the container, then we set it via the children.
    if (!isMainDimDefined) {
      node.layout[dim[mainAxis]] = fmaxf(
      // We're missing the last padding at this point to get the final
      // dimension
      boundAxis(node, mainAxis, linesMainDim + getTrailingPaddingAndBorder(node, mainAxis)),
      // We can never assign a width smaller than the padding and borders
      paddingAndBorderAxisMain);
      if (mainAxis === CSS_FLEX_DIRECTION_ROW_REVERSE || mainAxis === CSS_FLEX_DIRECTION_COLUMN_REVERSE) {
        needsMainTrailingPos = true;
      }
    }
    if (!isCrossDimDefined) {
      node.layout[dim[crossAxis]] = fmaxf(
      // For the cross dim, we add both sides at the end because the value
      // is aggregate via a max function. Intermediate negative values
      // can mess this computation otherwise
      boundAxis(node, crossAxis, linesCrossDim + paddingAndBorderAxisCross), paddingAndBorderAxisCross);
      if (crossAxis === CSS_FLEX_DIRECTION_ROW_REVERSE || crossAxis === CSS_FLEX_DIRECTION_COLUMN_REVERSE) {
        needsCrossTrailingPos = true;
      }
    }
    // <Loop F> Set trailing position if necessary
    if (needsMainTrailingPos || needsCrossTrailingPos) {
      for (i = 0; i < childCount; ++i) {
        child = node.children[i];
        if (needsMainTrailingPos) {
          setTrailingPosition(node, child, mainAxis);
        }
        if (needsCrossTrailingPos) {
          setTrailingPosition(node, child, crossAxis);
        }
      }
    }
    // <Loop G> Calculate dimensions for absolutely positioned elements
    currentAbsoluteChild = firstAbsoluteChild;
    while (currentAbsoluteChild !== null) {
      // Pre-fill dimensions when using absolute position and both offsets for
      // the axis are defined (either both left and right or top and bottom).
      for (ii = 0; ii < 2; ii++) {
        axis = ii !== 0 ? CSS_FLEX_DIRECTION_ROW : CSS_FLEX_DIRECTION_COLUMN;
        if (!isUndefined$1(node.layout[dim[axis]]) && !isDimDefined(currentAbsoluteChild, axis) && isPosDefined(currentAbsoluteChild, leading[axis]) && isPosDefined(currentAbsoluteChild, trailing[axis])) {
          currentAbsoluteChild.layout[dim[axis]] = fmaxf(boundAxis(currentAbsoluteChild, axis, node.layout[dim[axis]] - getBorderAxis(node, axis) - getMarginAxis(currentAbsoluteChild, axis) - getPosition(currentAbsoluteChild, leading[axis]) - getPosition(currentAbsoluteChild, trailing[axis])),
          // You never want to go smaller than padding
          getPaddingAndBorderAxis(currentAbsoluteChild, axis));
        }
        if (isPosDefined(currentAbsoluteChild, trailing[axis]) && !isPosDefined(currentAbsoluteChild, leading[axis])) {
          currentAbsoluteChild.layout[leading[axis]] = node.layout[dim[axis]] - currentAbsoluteChild.layout[dim[axis]] - getPosition(currentAbsoluteChild, trailing[axis]);
        }
      }
      child = currentAbsoluteChild;
      currentAbsoluteChild = currentAbsoluteChild.nextAbsoluteChild;
      child.nextAbsoluteChild = null;
    }
  }
  // 在外层做的margin补丁
  function saveMargin(node) {
    var style = node.style;
    var margin = {};
    ['marginTop', 'marginRight', 'marginBottom', 'marginLeft' // 只支持marginLeft
    ].forEach(function (key) {
      // 只处理百分号
      var value = style[key];
      if (value && /^-?\d+%$/.test(value)) {
        margin[key] = value;
        style[key] = 0;
      }
    });
    node.margin = margin;
  }
  function percent2Num(value) {
    var percent = Number(value.substr(0, value.length - 1));
    return percent / 100;
  }
  function layoutMargin(node) {
    var margin = node.margin,
      layout = node.layout;
    Object.keys(margin).forEach(function (key) {
      var percent = percent2Num(margin[key]);
      if ((key === 'marginLeft' || key === 'marginRight') && layout.width) {
        layout.left += layout.width * percent;
      } else if ((key === 'marginTop' || key === 'marginBottom') && layout.height) {
        layout.top += layout.height * percent;
      }
    });
  }
  function layoutNode(node, parentMaxWidth, parentDirection) {
    node.shouldUpdate = true;
    // hack
    saveMargin(node);
    var direction = node.style.direction || CSS_DIRECTION_LTR;
    var skipLayout = !node.isDirty && node.lastLayout && node.lastLayout.requestedHeight === node.layout.height && node.lastLayout.requestedWidth === node.layout.width && node.lastLayout.parentMaxWidth === parentMaxWidth && node.lastLayout.direction === direction;
    if (skipLayout) {
      node.layout.width = node.lastLayout.width;
      node.layout.height = node.lastLayout.height;
      node.layout.top = node.lastLayout.top;
      node.layout.left = node.lastLayout.left;
    } else {
      if (!node.lastLayout) {
        node.lastLayout = {};
      }
      node.lastLayout.requestedWidth = node.layout.width;
      node.lastLayout.requestedHeight = node.layout.height;
      node.lastLayout.parentMaxWidth = parentMaxWidth;
      node.lastLayout.direction = direction;
      // Reset child layouts
      node.children.forEach(function (child) {
        child.layout.width = undefined;
        child.layout.height = undefined;
        child.layout.top = 0;
        child.layout.left = 0;
      });
      layoutNodeImpl(node, parentMaxWidth, parentDirection);
      node.lastLayout.width = node.layout.width;
      node.lastLayout.height = node.layout.height;
      node.lastLayout.top = node.layout.top;
      node.lastLayout.left = node.layout.left;
    }
    // hack
    layoutMargin(node);
  }
  /* eslint-enable */
  function computeLayout(node) {
    if (!node) return node;
    fillNodes(node);
    layoutNode(node, null, null);
    return node;
  }

  function createMeasure(style, measureText) {
    return function /* width */
    () {
      var text = style.text,
        width = style.width,
        height = style.height;
      var outputWidth = width;
      var outputHeight = height;
      if (!isNumber(width) || !isNumber(height)) {
        var _measureText = measureText(text, style),
          measureWidth = _measureText.width,
          measureHeight = _measureText.height;
        if (!isNumber(width)) {
          outputWidth = measureWidth;
        }
        if (!isNumber(height)) {
          outputHeight = measureHeight;
        }
      }
      return {
        width: outputWidth,
        height: outputHeight
      };
    };
  }
  function getChildrenLayout(nodeTree) {
    if (!nodeTree) return;
    var left = 0;
    var top = 0;
    var right = 0;
    var bottom = 0;
    var width = 0;
    var height = 0;
    nodeTree.forEach(function (node) {
      var layout = node.layout;
      if (!layout) return;
      left = Math.min(left, layout.left);
      top = Math.min(top, layout.top);
      right = Math.min(right, layout.left + layout.width);
      bottom = Math.min(bottom, layout.top + layout.height);
      width = Math.max(width, layout.width);
      height = Math.max(height, layout.height);
    });
    return {
      left: left,
      top: top,
      right: right,
      bottom: bottom,
      width: width,
      height: height
    };
  }
  var NodeTree = /*#__PURE__*/function () {
    function NodeTree(node) {
      _classCallCheck(this, NodeTree);
      var className = node.className,
        children = node.children,
        layout = node.layout;
      var nodeChildren = children && children.length ? children.map(function (child) {
        return new NodeTree(child);
      }) : undefined;
      var nodeLayout = layout ? layout : getChildrenLayout(nodeChildren);
      this.children = nodeChildren;
      this.layout = nodeLayout;
      this.className = className;
    }
    _createClass(NodeTree, [{
      key: "getElementsByClassName",
      value: function getElementsByClassName(targetClassName) {
        var result = [];
        var className = this.className,
          children = this.children;
        if (className === targetClassName) {
          result.push(this);
        }
        if (children) {
          children.forEach(function (child) {
            result.push.apply(result, _toConsumableArray(child.getElementsByClassName(targetClassName)));
          });
        }
        return result;
      }
    }]);
    return NodeTree;
  }(); // 展开数组
  function extendMap(arr, fn) {
    if (!arr) {
      return arr;
    }
    var newArray = [];
    if (!isArray(arr)) {
      var rst = fn(arr);
      if (!rst) {
        return newArray;
      }
      if (isArray(rst)) {
        newArray = newArray.concat(rst);
      } else {
        newArray.push(rst);
      }
      return newArray;
    }
    for (var i = 0; i < arr.length; i++) {
      var element = arr[i];
      if (isArray(element)) {
        newArray = newArray.concat(extendMap(element, fn));
      } else if (element) {
        var _rst = fn(element);
        if (!_rst) {
          continue;
        }
        if (isArray(_rst)) {
          newArray = newArray.concat(_rst);
        } else {
          newArray.push(_rst);
        }
      }
    }
    return newArray;
  }
  // 主要是把function节点，全部转换成string标签节点
  function renderJSXElement(element, context, updater) {
    if (!element) return element;
    var px2hd = context.px2hd,
      measureText = context.measureText;
    var type = element.type,
      props = element.props;
    var tag = getWorkTag(type);
    // 只处理 function 组件
    if (tag === FunctionComponent) {
      // @ts-ignore
      var newElement = type(element.props, context, updater);
      // return element if type is string
      return renderJSXElement(newElement, context, updater);
    }
    var className = props.className,
      _props$style = props.style,
      customStyle = _props$style === void 0 ? {} : _props$style,
      attrs = props.attrs,
      newChildren = props.children;
    var style = px2hd(_objectSpread2(_objectSpread2({}, customStyle), attrs));
    // 文本需要计算文本的宽高来进行flex布局
    if (type === 'text') {
      style.measure = createMeasure(style, measureText);
    }
    // render children first
    var nextChildren = newChildren ? Children.toArray(newChildren).map(function (child) {
      return renderJSXElement(child, context, updater);
    }) : [];
    return {
      type: type,
      className: className,
      children: nextChildren.filter(Boolean),
      style: style
    };
  }
  // 计算布局
  function computeLayout$1(component, newChildren) {
    var context = component.context,
      updater = component.updater;
    var nodeTree = renderJSXElement(newChildren, context, updater);
    computeLayout(nodeTree);
    // 构造一个 NodeTree, 方便外部使用
    return new NodeTree(nodeTree);
  }
  function createChildNodeTree(parent, vNodeChildren) {
    var tag = parent.tag;
    var children = extendMap(vNodeChildren, function (child) {
      var childTag = child.tag,
        childStyle = child.style,
        childChildren = child.children;
      // 如果组件的根节点不是 flex, 则该组件不需要计算 flex 布局
      if (tag !== Shape$1 && childTag === Shape$1 && childStyle.display !== 'flex') {
        return null;
      }
      // 如果子组件不是 shape，则布局计算时，忽略当前节点
      if (childTag !== Shape$1) {
        return createChildNodeTree(child, childChildren);
      }
      return createNodeTree(child);
    });
    return children;
  }
  // 创建组件的布局树
  function createNodeTree(vNode) {
    var tag = vNode.tag,
      type = vNode.type,
      style = vNode.style,
      context = vNode.context,
      vNodeChildren = vNode.children;
    var measureText = context.measureText;
    var children = createChildNodeTree(vNode, vNodeChildren);
    // 文本需要计算文本的宽高来进行flex布局
    if (type === 'text') {
      style.measure = createMeasure(style, measureText);
    }
    return {
      tag: tag,
      type: type,
      style: style,
      children: children,
      // 保留对 vNode 的引用，用于把布局结果回填
      vNode: vNode
    };
  }
  function fillElementLayout(node) {
    var type = node.type,
      style = node.style,
      vNode = node.vNode,
      children = node.children,
      layout = node.layout;
    var attrs = getShapeAttrs(type, layout);
    if (style.measure) {
      delete style.measure;
    }
    // 更新布局和样式
    vNode.layout = layout;
    vNode.style = _objectSpread2(_objectSpread2({}, attrs), style);
    if (!children || !children.length) {
      return;
    }
    for (var i = 0, len = children.length; i < len; i++) {
      var child = children[i];
      fillElementLayout(child);
    }
  }
  function fillComponentLayout(vNode) {
    var layout = vNode.layout,
      vNodeChildren = vNode.children;
    Children.map(vNodeChildren, function (child) {
      if (!child) {
        return;
      }
      var childTag = child.tag,
        childLayout = child.layout,
        style = child.style;
      if (childTag !== Shape$1 && layout && !childLayout) {
        child.layout = layout;
        child.style = _objectSpread2({
          width: layout.width,
          height: layout.height
        }, style);
      }
      fillComponentLayout(child);
    });
  }

  function getStyle$1(tagType, props, context) {
    var _props$style = props.style,
      customStyle = _props$style === void 0 ? {} : _props$style,
      attrs = props.attrs,
      zIndex = props.zIndex;
    if (tagType === Shape$1) {
      return context.px2hd(_objectSpread2(_objectSpread2({}, customStyle), attrs));
    }
    // const style = {
    //   position: 'absolute',
    //   left: 0,
    //   top: 0,
    //   right: 0,
    //   bottom: 0,
    // } as Record<string, any>;
    if (isNumber(zIndex)) {
      return {
        zIndex: zIndex
      };
    }
    return {};
  }
  function createVNode(parent, vNode) {
    var canvas = parent.canvas,
      context = parent.context,
      updater = parent.updater,
      parentAnimate = parent.animate;
    var ref = vNode.ref,
      type = vNode.type,
      props = vNode.props;
    var animate = props.animate,
      transformFrom = props.transformFrom;
    var tag = getWorkTag(type);
    var animator = new Animator();
    var style = getStyle$1(tag, props, context);
    vNode.tag = tag;
    vNode.style = style;
    vNode.context = context;
    vNode.updater = updater;
    vNode.canvas = canvas;
    vNode.animate = isBoolean(animate) ? animate : parentAnimate;
    vNode.animator = animator;
    // shape 标签
    if (tag === Shape$1) {
      var shape = createShape(type, _objectSpread2(_objectSpread2({}, props), {}, {
        style: style
      }));
      if (ref) {
        ref.current = shape;
      }
      // @ts-ignore
      shape._vNode = vNode; // shape 保留 vNode 的引用
      vNode.shape = shape;
    } else {
      // 组件
      var component;
      if (tag === ClassComponent) {
        // @ts-ignore
        component = new type(props, context, updater);
      } else {
        component = new Component(props, context, updater);
        component.render = function () {
          // @ts-ignore
          return type(this.props, context, updater);
        };
      }
      var group = new Group();
      component.container = group;
      // 设置ref
      if (ref) {
        ref.current = component;
      }
      component.context = context;
      component.updater = updater;
      component.animator = animator;
      component._vNode = vNode;
      vNode.shape = group;
      vNode.component = component;
    }
    if (transformFrom && transformFrom.current) {
      var transformVNode = transformFrom.current._vNode;
      vNode.children = transformVNode.children;
      transformVNode.children = null;
    }
    return vNode;
  }
  function updateVNode(parent, nextNode, lastNode) {
    var canvas = parent.canvas,
      context = parent.context,
      updater = parent.updater,
      parentAnimate = parent.animate;
    var tag = lastNode.tag,
      animator = lastNode.animator,
      component = lastNode.component,
      shape = lastNode.shape,
      children = lastNode.children;
    var props = nextNode.props;
    var animate = props.animate;
    nextNode.tag = tag;
    nextNode.canvas = canvas;
    nextNode.context = context;
    nextNode.updater = updater;
    nextNode.component = component;
    nextNode.shape = shape;
    nextNode.parent = parent;
    nextNode.children = children;
    nextNode.animate = isBoolean(animate) ? animate : parentAnimate;
    nextNode.animator = animator;
    nextNode.style = getStyle$1(tag, props, context);
    return nextNode;
  }
  function createElement(parent, element) {
    return Children.map(element, function (el) {
      if (!el) return el;
      return createVNode(parent, el);
    });
  }
  function destroyElement(vNode) {
    Children.map(vNode, function (node) {
      if (!node) return;
      var component = node.component,
        children = node.children;
      if (component) {
        component.willUnmount();
        destroyElement(children);
        component.didUnmount();
        component.destroy();
      }
    });
  }
  function updateElement(parent, nextElement, lastElement) {
    var nextType = nextElement.type,
      nextProps = nextElement.props;
    var lastType = lastElement.type,
      lastProps = lastElement.props;
    if (nextType === lastType) {
      var _nextVNode = updateVNode(parent, nextElement, lastElement);
      // props 无变化 和 context 都无变化
      if (equal(nextProps, lastProps) && parent.context === lastElement.context) {
        return null;
      }
      return _nextVNode;
    }
    var nextVNode = createVNode(parent, nextElement);
    destroyElement(lastElement);
    return nextVNode;
  }
  function diffElement(parent, nextElement, lastElement) {
    if (!nextElement && !lastElement) {
      return null;
    }
    // 删除
    if (!nextElement && lastElement) {
      destroyElement(lastElement);
      return null;
    }
    // 新建
    if (nextElement && !lastElement) {
      return createElement(parent, nextElement);
    }
    // 更新
    return updateElement(parent, nextElement, lastElement);
  }
  function renderComponentNodes(componentNodes) {
    if (!componentNodes || !componentNodes.length) {
      return;
    }
    // 1. shouldUpdate & willReceiveProps
    var shouldProcessChildren = componentNodes.filter(function (node) {
      var component = node.component,
        props = node.props,
        context = node.context,
        layout = node.layout;
      // 更新组件 layout
      component.layout = layout;
      // 新创建的 component
      if (!component.children) return true;
      // 不需要更新
      if (component.shouldUpdate(props) === false) {
        return false;
      }
      component.willReceiveProps(props, context);
      component.props = props;
      component.context = context;
      return true;
    });
    if (!shouldProcessChildren.length) {
      return;
    }
    // 2. willMount / willUpdate
    shouldProcessChildren.forEach(function (child) {
      var component = child.component;
      if (!component.children) {
        component.willMount();
      } else {
        component.willUpdate();
      }
    });
    // 3. render
    shouldProcessChildren.forEach(function (child) {
      var canvas = child.canvas,
        component = child.component,
        children = child.children;
      var mount = !component.children;
      var newChildren = canvas.toRawChildren(component.render());
      renderChildren(child, newChildren, children);
      if (mount) {
        component.didMount();
      } else {
        component.didUpdate();
      }
    });
  }
  function renderVNode(node, nextChildren, lastChildren) {
    var component = node.component;
    // 设置新的 children
    node.children = nextChildren;
    // 如果是组件，需要同时更新组件的 children
    // 等同于 node.tag === ClassComponent || node.tag === FunctionComponent
    if (component) {
      component.children = nextChildren;
    }
    var componentNodeChildren = [];
    Children.compare(
    // @ts-ignore
    nextChildren,
    // @ts-ignore
    lastChildren, function (next, last) {
      var element = diffElement(node, next, last);
      Children.map(element, function (child) {
        if (!child) return;
        var canvas = child.canvas,
          tag = child.tag,
          childProps = child.props,
          component = child.component,
          childLastChildren = child.children;
        var childrenNode = [];
        if (tag === Shape$1) {
          childrenNode = renderVNode(child, childProps.children, childLastChildren);
        } else if (tag === FunctionComponent) {
          // FunctionComponent 直接更新 props
          component.props = childProps;
          var newChildren = canvas.toRawChildren(component.render());
          childrenNode = renderVNode(child, newChildren, childLastChildren);
        } else {
          childrenNode = [child];
        }
        componentNodeChildren = componentNodeChildren.concat(childrenNode);
      });
    });
    return componentNodeChildren;
  }
  function renderChildren(parent, nextChildren, lastChildren) {
    // 返回的都是 classComponent 的节点
    var componentNodeChildren = renderVNode(parent, nextChildren, lastChildren);
    // 计算 flex 布局
    var nodeTree = createNodeTree(parent);
    computeLayout(nodeTree);
    fillElementLayout(nodeTree);
    fillComponentLayout(parent);
    var newChildren = parent.children;
    if (!componentNodeChildren.length) {
      return newChildren;
    }
    renderComponentNodes(componentNodeChildren);
    return newChildren;
  }
  function render(vNode) {
    var lastChildren = vNode.children,
      props = vNode.props;
    var nextChildren = props.children;
    // render 节点
    var children = renderChildren(vNode, nextChildren, lastChildren);
    // vNode.children = children;
    // 创建动画
    var childrenAnimation = createAnimation(vNode, children, lastChildren);
    // 执行动画
    if (childrenAnimation.length) {
      childrenAnimation.forEach(function (animator) {
        animator.play();
      });
    }
  }
  // setState 触发的更新
  function updateComponents(components) {
    if (!components.length) return;
    components.forEach(function (component) {
      var vNode = component._vNode,
        lastChildren = component.children,
        props = component.props,
        animator = component.animator;
      // 是否需要更新
      if (component.shouldUpdate(props) === false) {
        return false;
      }
      component.willUpdate();
      var canvas = vNode.canvas;
      var newChildren = canvas.toRawChildren(component.render());
      var nextChildren = renderChildren(vNode, newChildren, lastChildren);
      // 更新 children
      component.children = nextChildren;
      vNode.children = nextChildren;
      component.didUpdate();
      // 创建动画
      var childrenAnimation = createAnimation(vNode, nextChildren, lastChildren);
      if (childrenAnimation.length) {
        animator.children = childrenAnimation;
      }
      // 执行动画
      animator.play();
    });
  }

  function _inheritsLoose$2(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$3(subClass, superClass);
  }
  function _setPrototypeOf$3(o, p) {
    _setPrototypeOf$3 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$3(o, p);
  }

  function generatePath(context, parsedStyle) {
    var r = parsedStyle.r;
    context.arc(r, r, r, 0, Math.PI * 2, false);
  }

  function generatePath$1(context, parsedStyle) {
    var rxInPixels = parsedStyle.rx,
      ryInPixels = parsedStyle.ry;
    var rx = rxInPixels;
    var ry = ryInPixels;
    // @see https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/ellipse
    if (context.ellipse) {
      context.ellipse(rx, ry, rx, ry, 0, 0, Math.PI * 2, false);
    } else {
      // 如果不支持，则使用圆来绘制，进行变形
      var r = rx > ry ? rx : ry;
      var scaleX = rx > ry ? 1 : rx / ry;
      var scaleY = rx > ry ? ry / rx : 1;
      context.save();
      context.scale(scaleX, scaleY);
      context.arc(0, 0, r, 0, Math.PI * 2);
    }
  }

  function generatePath$2(context, parsedStyle) {
    var x1 = parsedStyle.x1,
      y1 = parsedStyle.y1,
      x2 = parsedStyle.x2,
      y2 = parsedStyle.y2,
      _parsedStyle$defX = parsedStyle.defX,
      defX = _parsedStyle$defX === void 0 ? 0 : _parsedStyle$defX,
      _parsedStyle$defY = parsedStyle.defY,
      defY = _parsedStyle$defY === void 0 ? 0 : _parsedStyle$defY,
      markerStart = parsedStyle.markerStart,
      markerEnd = parsedStyle.markerEnd,
      markerStartOffset = parsedStyle.markerStartOffset,
      markerEndOffset = parsedStyle.markerEndOffset;
    var startOffsetX = 0;
    var startOffsetY = 0;
    var endOffsetX = 0;
    var endOffsetY = 0;
    var rad = 0;
    var x;
    var y;
    if (markerStart && markerStart instanceof DisplayObject && markerStartOffset) {
      x = x2 - x1;
      y = y2 - y1;
      rad = Math.atan2(y, x);
      startOffsetX = Math.cos(rad) * (markerStartOffset || 0);
      startOffsetY = Math.sin(rad) * (markerStartOffset || 0);
    }
    if (markerEnd && markerEnd instanceof DisplayObject && markerEndOffset) {
      x = x1 - x2;
      y = y1 - y2;
      rad = Math.atan2(y, x);
      endOffsetX = Math.cos(rad) * (markerEndOffset || 0);
      endOffsetY = Math.sin(rad) * (markerEndOffset || 0);
    }
    context.moveTo(x1 - defX + startOffsetX, y1 - defY + startOffsetY);
    context.lineTo(x2 - defX + endOffsetX, y2 - defY + endOffsetY);
  }

  function generatePath$3(context, parsedStyle) {
    var _parsedStyle$defX = parsedStyle.defX,
      defX = _parsedStyle$defX === void 0 ? 0 : _parsedStyle$defX,
      _parsedStyle$defY = parsedStyle.defY,
      defY = _parsedStyle$defY === void 0 ? 0 : _parsedStyle$defY,
      markerStart = parsedStyle.markerStart,
      markerEnd = parsedStyle.markerEnd,
      markerStartOffset = parsedStyle.markerStartOffset,
      markerEndOffset = parsedStyle.markerEndOffset;
    var _parsedStyle$path = parsedStyle.path,
      absolutePath = _parsedStyle$path.absolutePath,
      segments = _parsedStyle$path.segments;
    var startOffsetX = 0;
    var startOffsetY = 0;
    var endOffsetX = 0;
    var endOffsetY = 0;
    var rad = 0;
    var x;
    var y;
    if (markerStart && markerStart instanceof DisplayObject && markerStartOffset) {
      var _markerStart$parentNo = markerStart.parentNode.getStartTangent(),
        p1 = _markerStart$parentNo[0],
        p2 = _markerStart$parentNo[1];
      x = p1[0] - p2[0];
      y = p1[1] - p2[1];
      rad = Math.atan2(y, x);
      startOffsetX = Math.cos(rad) * (markerStartOffset || 0);
      startOffsetY = Math.sin(rad) * (markerStartOffset || 0);
    }
    if (markerEnd && markerEnd instanceof DisplayObject && markerEndOffset) {
      var _markerEnd$parentNode = markerEnd.parentNode.getEndTangent(),
        _p = _markerEnd$parentNode[0],
        _p2 = _markerEnd$parentNode[1];
      x = _p[0] - _p2[0];
      y = _p[1] - _p2[1];
      rad = Math.atan2(y, x);
      endOffsetX = Math.cos(rad) * (markerEndOffset || 0);
      endOffsetY = Math.sin(rad) * (markerEndOffset || 0);
    }
    for (var i = 0; i < absolutePath.length; i++) {
      var params = absolutePath[i];
      var command = params[0];
      var nextSegment = absolutePath[i + 1];
      var useStartOffset = i === 0 && (startOffsetX !== 0 || startOffsetY !== 0);
      var useEndOffset = (i === absolutePath.length - 1 || nextSegment && (nextSegment[0] === 'M' || nextSegment[0] === 'Z')) && endOffsetX !== 0 && endOffsetY !== 0;
      switch (command) {
        case 'M':
          // Use start marker offset
          if (useStartOffset) {
            context.moveTo(params[1] - defX + startOffsetX, params[2] - defY + startOffsetY);
            context.lineTo(params[1] - defX, params[2] - defY);
          } else {
            context.moveTo(params[1] - defX, params[2] - defY);
          }
          break;
        case 'L':
          if (useEndOffset) {
            context.lineTo(params[1] - defX + endOffsetX, params[2] - defY + endOffsetY);
          } else {
            context.lineTo(params[1] - defX, params[2] - defY);
          }
          break;
        case 'Q':
          context.quadraticCurveTo(params[1] - defX, params[2] - defY, params[3] - defX, params[4] - defY);
          if (useEndOffset) {
            context.lineTo(params[3] - defX + endOffsetX, params[4] - defY + endOffsetY);
          }
          break;
        case 'C':
          context.bezierCurveTo(params[1] - defX, params[2] - defY, params[3] - defX, params[4] - defY, params[5] - defX, params[6] - defY);
          if (useEndOffset) {
            context.lineTo(params[5] - defX + endOffsetX, params[6] - defY + endOffsetY);
          }
          break;
        case 'A':
          {
            var arcParams = segments[i].arcParams;
            var cx = arcParams.cx,
              cy = arcParams.cy,
              rx = arcParams.rx,
              ry = arcParams.ry,
              startAngle = arcParams.startAngle,
              endAngle = arcParams.endAngle,
              xRotation = arcParams.xRotation,
              sweepFlag = arcParams.sweepFlag;
            // @see https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/ellipse
            if (context.ellipse) {
              context.ellipse(cx - defX, cy - defY, rx, ry, xRotation, startAngle, endAngle, !!(1 - sweepFlag));
            } else {
              // @see https://stackoverflow.com/a/47494351
              var r = rx > ry ? rx : ry;
              var scaleX = rx > ry ? 1 : rx / ry;
              var scaleY = rx > ry ? ry / rx : 1;
              context.translate(cx - defX, cy - defY);
              context.rotate(xRotation);
              context.scale(scaleX, scaleY);
              context.arc(0, 0, r, startAngle, endAngle, !!(1 - sweepFlag));
              context.scale(1 / scaleX, 1 / scaleY);
              context.rotate(-xRotation);
              context.translate(-(cx - defX), -(cy - defY));
            }
            if (useEndOffset) {
              context.lineTo(params[6] - defX + endOffsetX, params[7] - defY + endOffsetY);
            }
            break;
          }
        case 'Z':
          context.closePath();
          break;
      }
    }
  }

  function generatePath$4(context, parsedStyle) {
    var _parsedStyle$defX = parsedStyle.defX,
      defX = _parsedStyle$defX === void 0 ? 0 : _parsedStyle$defX,
      _parsedStyle$defY = parsedStyle.defY,
      defY = _parsedStyle$defY === void 0 ? 0 : _parsedStyle$defY,
      markerStart = parsedStyle.markerStart,
      markerEnd = parsedStyle.markerEnd,
      markerStartOffset = parsedStyle.markerStartOffset,
      markerEndOffset = parsedStyle.markerEndOffset;
    var points = parsedStyle.points.points;
    var length = points.length;
    var x1 = points[0][0] - defX;
    var y1 = points[0][1] - defY;
    var x2 = points[length - 1][0] - defX;
    var y2 = points[length - 1][1] - defY;
    var startOffsetX = 0;
    var startOffsetY = 0;
    var endOffsetX = 0;
    var endOffsetY = 0;
    var rad = 0;
    var x;
    var y;
    if (markerStart && markerStart instanceof DisplayObject && markerStartOffset) {
      x = points[1][0] - points[0][0];
      y = points[1][1] - points[0][1];
      rad = Math.atan2(y, x);
      startOffsetX = Math.cos(rad) * (markerStartOffset || 0);
      startOffsetY = Math.sin(rad) * (markerStartOffset || 0);
    }
    if (markerEnd && markerEnd instanceof DisplayObject && markerEndOffset) {
      x = points[length - 1][0] - points[0][0];
      y = points[length - 1][1] - points[0][1];
      rad = Math.atan2(y, x);
      endOffsetX = Math.cos(rad) * (markerEndOffset || 0);
      endOffsetY = Math.sin(rad) * (markerEndOffset || 0);
    }
    context.moveTo(x1 + (startOffsetX || endOffsetX), y1 + (startOffsetY || endOffsetY));
    for (var i = 1; i < length - 1; i++) {
      var point = points[i];
      context.lineTo(point[0] - defX, point[1] - defY);
    }
    context.lineTo(x2, y2);
  }

  function generatePath$5(context, parsedStyle) {
    var _parsedStyle$defX = parsedStyle.defX,
      defX = _parsedStyle$defX === void 0 ? 0 : _parsedStyle$defX,
      _parsedStyle$defY = parsedStyle.defY,
      defY = _parsedStyle$defY === void 0 ? 0 : _parsedStyle$defY,
      markerStart = parsedStyle.markerStart,
      markerEnd = parsedStyle.markerEnd,
      markerStartOffset = parsedStyle.markerStartOffset,
      markerEndOffset = parsedStyle.markerEndOffset;
    var points = parsedStyle.points.points;
    var length = points.length;
    var x1 = points[0][0] - defX;
    var y1 = points[0][1] - defY;
    var x2 = points[length - 1][0] - defX;
    var y2 = points[length - 1][1] - defY;
    var startOffsetX = 0;
    var startOffsetY = 0;
    var endOffsetX = 0;
    var endOffsetY = 0;
    var rad = 0;
    var x;
    var y;
    if (markerStart && markerStart instanceof DisplayObject && markerStartOffset) {
      x = points[1][0] - points[0][0];
      y = points[1][1] - points[0][1];
      rad = Math.atan2(y, x);
      startOffsetX = Math.cos(rad) * (markerStartOffset || 0);
      startOffsetY = Math.sin(rad) * (markerStartOffset || 0);
    }
    if (markerEnd && markerEnd instanceof DisplayObject && markerEndOffset) {
      x = points[length - 2][0] - points[length - 1][0];
      y = points[length - 2][1] - points[length - 1][1];
      rad = Math.atan2(y, x);
      endOffsetX = Math.cos(rad) * (markerEndOffset || 0);
      endOffsetY = Math.sin(rad) * (markerEndOffset || 0);
    }
    context.moveTo(x1 + startOffsetX, y1 + startOffsetY);
    for (var i = 1; i < length - 1; i++) {
      var point = points[i];
      context.lineTo(point[0] - defX, point[1] - defY);
    }
    context.lineTo(x2 + endOffsetX, y2 + endOffsetY);
  }

  function generatePath$6(context, parsedStyle) {
    var radius = parsedStyle.radius,
      width = parsedStyle.width,
      height = parsedStyle.height;
    var w = width;
    var h = height;
    var hasRadius = radius && radius.some(function (r) {
      return r !== 0;
    });
    if (!hasRadius) {
      // Canvas support negative width/height of rect
      context.rect(0, 0, w, h);
    } else {
      var signX = width > 0 ? 1 : -1;
      var signY = height > 0 ? 1 : -1;
      var sweepFlag = signX + signY === 0;
      var _radius$map = radius.map(function (r) {
          return clamp(r, 0, Math.min(Math.abs(w) / 2, Math.abs(h) / 2));
        }),
        tlr = _radius$map[0],
        trr = _radius$map[1],
        brr = _radius$map[2],
        blr = _radius$map[3];
      context.moveTo(signX * tlr, 0);
      context.lineTo(w - signX * trr, 0);
      if (trr !== 0) {
        context.arc(w - signX * trr, signY * trr, trr, -signY * Math.PI / 2, signX > 0 ? 0 : Math.PI, sweepFlag);
      }
      context.lineTo(w, h - signY * brr);
      if (brr !== 0) {
        context.arc(w - signX * brr, h - signY * brr, brr, signX > 0 ? 0 : Math.PI, signY > 0 ? Math.PI / 2 : 1.5 * Math.PI, sweepFlag);
      }
      context.lineTo(signX * blr, h);
      if (blr !== 0) {
        context.arc(signX * blr, h - signY * blr, blr, signY > 0 ? Math.PI / 2 : -Math.PI / 2, signX > 0 ? Math.PI : 0, sweepFlag);
      }
      context.lineTo(0, signY * tlr);
      if (tlr !== 0) {
        context.arc(signX * tlr, signY * tlr, tlr, signX > 0 ? Math.PI : 0, signY > 0 ? Math.PI * 1.5 : Math.PI / 2, sweepFlag);
      }
    }
  }

  var Plugin = /*#__PURE__*/function (_AbstractRendererPlug) {
    _inheritsLoose$2(Plugin, _AbstractRendererPlug);
    function Plugin() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _AbstractRendererPlug.call.apply(_AbstractRendererPlug, [this].concat(args)) || this;
      _this.name = 'canvas-path-generator';
      return _this;
    }
    var _proto = Plugin.prototype;
    _proto.init = function init() {
      var _pathGeneratorFactory;
      var pathGeneratorFactory = (_pathGeneratorFactory = {}, _pathGeneratorFactory[Shape.CIRCLE] = generatePath, _pathGeneratorFactory[Shape.ELLIPSE] = generatePath$1, _pathGeneratorFactory[Shape.RECT] = generatePath$6, _pathGeneratorFactory[Shape.LINE] = generatePath$2, _pathGeneratorFactory[Shape.POLYLINE] = generatePath$5, _pathGeneratorFactory[Shape.POLYGON] = generatePath$4, _pathGeneratorFactory[Shape.PATH] = generatePath$3, _pathGeneratorFactory[Shape.TEXT] = undefined, _pathGeneratorFactory[Shape.GROUP] = undefined, _pathGeneratorFactory[Shape.IMAGE] = undefined, _pathGeneratorFactory[Shape.HTML] = undefined, _pathGeneratorFactory[Shape.MESH] = undefined, _pathGeneratorFactory);
      // @ts-ignore
      this.context.pathGeneratorFactory = pathGeneratorFactory;
    };
    _proto.destroy = function destroy() {
      // @ts-ignore
      delete this.context.pathGeneratorFactory;
    };
    return Plugin;
  }(AbstractRendererPlugin);

  function _regeneratorRuntime$1() {
    _regeneratorRuntime$1 = function () {
      return exports;
    };
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function (obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function (method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) keys.push(key);
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function (skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
      },
      stop: function () {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function (exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function (type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function (record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function (finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      catch: function (tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function (iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  function asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator$1(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  function _inheritsLoose$3(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$4(subClass, superClass);
  }
  function _setPrototypeOf$4(o, p) {
    _setPrototypeOf$4 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$4(o, p);
  }
  function _unsupportedIterableToArray$2(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray$2(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$2(o, minLen);
  }
  function _arrayLikeToArray$2(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
    return arr2;
  }
  function _createForOfIteratorHelperLoose$1(o, allowArrayLike) {
    var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
    if (it) return (it = it.call(o)).next.bind(it);
    if (Array.isArray(o) || (it = _unsupportedIterableToArray$2(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }
    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var tmpVec3a$1 = create$2();
  var tmpVec3b$1 = create$2();
  var tmpVec3c$1 = create$2();
  var tmpMat4 = create$1();
  /**
   * pick shape(s) with Mouse/Touch event
   *
   * 1. find AABB with r-tree
   * 2. do math calculation with geometry in an accurate way
   */
  var CanvasPickerPlugin = /*#__PURE__*/function () {
    function CanvasPickerPlugin() {
      var _this = this;
      this.canvasConfig = void 0;
      this.pathGeneratorFactory = void 0;
      this.pointInPathPickerFactory = void 0;
      this.isHit = function (displayObject, position, worldTransform, isClipPath) {
        // use picker for current shape's type
        var pick = _this.pointInPathPickerFactory[displayObject.nodeName];
        if (pick) {
          // invert with world matrix
          var invertWorldMat = invert(tmpMat4, worldTransform);
          // transform client position to local space, do picking in local space
          var localPosition = transformMat4(tmpVec3b$1, set$1(tmpVec3c$1, position[0], position[1], 0), invertWorldMat);
          // account for anchor
          var _displayObject$getGeo = displayObject.getGeometryBounds(),
            halfExtents = _displayObject$getGeo.halfExtents;
          var anchor = displayObject.parsedStyle.anchor;
          localPosition[0] += (anchor && anchor[0] || 0) * halfExtents[0] * 2;
          localPosition[1] += (anchor && anchor[1] || 0) * halfExtents[1] * 2;
          if (pick(displayObject, new Point(localPosition[0], localPosition[1]), isClipPath, _this.isPointInPath)) {
            return true;
          }
        }
        return false;
      };
      this.isPointInPath = function (displayObject, position) {
        var context = runtime.offscreenCanvas.getOrCreateContext(_this.canvasConfig.offscreenCanvas);
        var generatePath = _this.pathGeneratorFactory[displayObject.nodeName];
        if (generatePath) {
          context.beginPath();
          generatePath(context, displayObject.parsedStyle);
          context.closePath();
        }
        return context.isPointInPath(position.x, position.y);
      };
    }
    var _proto = CanvasPickerPlugin.prototype;
    _proto.apply = function apply(context) {
      var _renderingContext$roo,
        _this2 = this;
      var config = context.config,
        renderingService = context.renderingService,
        renderingContext = context.renderingContext,
        pathGeneratorFactory = context.pathGeneratorFactory,
        pointInPathPickerFactory = context.pointInPathPickerFactory;
      this.canvasConfig = config;
      this.pathGeneratorFactory = pathGeneratorFactory;
      this.pointInPathPickerFactory = pointInPathPickerFactory;
      var document = (_renderingContext$roo = renderingContext.root) === null || _renderingContext$roo === void 0 ? void 0 : _renderingContext$roo.ownerDocument;
      renderingService.hooks.pick.tapPromise(CanvasPickerPlugin.tag, /*#__PURE__*/function () {
        var _ref = _asyncToGenerator$1( /*#__PURE__*/_regeneratorRuntime$1().mark(function _callee(result) {
          return _regeneratorRuntime$1().wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  return _context.abrupt("return", _this2.pick(document, result));
                case 1:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));
        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }());
      renderingService.hooks.pickSync.tap(CanvasPickerPlugin.tag, function (result) {
        return _this2.pick(document, result);
      });
    };
    _proto.pick = function pick(document, result) {
      var topmost = result.topmost,
        _result$position = result.position,
        x = _result$position.x,
        y = _result$position.y;
      // position in world space
      var position = set$1(tmpVec3a$1, x, y, 0);
      // query by AABB first with spatial index(r-tree)
      var hitTestList = document.elementsFromBBox(position[0], position[1], position[0], position[1]);
      // test with clip path & origin shape
      // @see https://github.com/antvis/g/issues/1064
      var pickedDisplayObjects = [];
      for (var _iterator = _createForOfIteratorHelperLoose$1(hitTestList), _step; !(_step = _iterator()).done;) {
        var displayObject = _step.value;
        var worldTransform = displayObject.getWorldTransform();
        var isHitOriginShape = this.isHit(displayObject, position, worldTransform, false);
        if (isHitOriginShape) {
          // should look up in the ancestor node
          var clipped = findClosestClipPathTarget(displayObject);
          if (clipped) {
            var clipPath = clipped.parsedStyle.clipPath;
            var isHitClipPath = this.isHit(clipPath, position, clipPath.getWorldTransform(), true);
            if (isHitClipPath) {
              if (topmost) {
                result.picked = [displayObject];
                return result;
              } else {
                pickedDisplayObjects.push(displayObject);
              }
            }
          } else {
            if (topmost) {
              result.picked = [displayObject];
              return result;
            } else {
              pickedDisplayObjects.push(displayObject);
            }
          }
        }
      }
      result.picked = pickedDisplayObjects;
      return result;
    };
    return CanvasPickerPlugin;
  }();
  CanvasPickerPlugin.tag = 'CanvasPicker';

  /**
   * 两点之间的距离
   * @param {number} x1 起始点 x
   * @param {number} y1 起始点 y
   * @param {number} x2 结束点 x
   * @param {number} y2 结束点 y
   * @return {number} 距离
   */
  function distance$2(x1, y1, x2, y2) {
    var dx = x1 - x2;
    var dy = y1 - y2;
    return Math.sqrt(dx * dx + dy * dy);
  }
  function isNumberEqual$2(v1, v2) {
    return Math.abs(v1 - v2) < 0.001;
  }
  function getBBoxByArray$1(xArr, yArr) {
    var minX = Math.min.apply(Math, xArr);
    var minY = Math.min.apply(Math, yArr);
    var maxX = Math.max.apply(Math, xArr);
    var maxY = Math.max.apply(Math, yArr);
    return {
      x: minX,
      y: minY,
      width: maxX - minX,
      height: maxY - minY
    };
  }
  function piMod$1(angle) {
    return (angle + Math.PI * 2) % (Math.PI * 2);
  }
  var line$2 = {
    /**
     * 计算线段的包围盒
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {object} 包围盒对象
     */
    box: function box(x1, y1, x2, y2) {
      return getBBoxByArray$1([x1, x2], [y1, y2]);
    },
    /**
     * 线段的长度
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {number} 距离
     */
    length: function length(x1, y1, x2, y2) {
      return distance$2(x1, y1, x2, y2);
    },
    /**
     * 根据比例获取点
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} t 指定比例
     * @return {object} 包含 x, y 的点
     */
    pointAt: function pointAt(x1, y1, x2, y2, t) {
      return {
        x: (1 - t) * x1 + t * x2,
        y: (1 - t) * y1 + t * y2
      };
    },
    /**
     * 点到线段的距离
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} x  测试点 x
     * @param {number} y  测试点 y
     * @return {number} 距离
     */
    pointDistance: function pointDistance(x1, y1, x2, y2, x, y) {
      // 投影距离 x1, y1 的向量，假设 p, p1, p2 三个点，投影点为 a
      // p1a = p1p.p1p2/|p1p2| * (p1p 的单位向量)
      var cross = (x2 - x1) * (x - x1) + (y2 - y1) * (y - y1);
      if (cross < 0) {
        return distance$2(x1, y1, x, y);
      }
      var lengthSquare = (x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1);
      if (cross > lengthSquare) {
        return distance$2(x2, y2, x, y);
      }
      return this.pointToLine(x1, y1, x2, y2, x, y);
    },
    /**
     * 点到直线的距离，而不是点到线段的距离
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} x  测试点 x
     * @param {number} y  测试点 y
     * @return {number} 距离
     */
    pointToLine: function pointToLine(x1, y1, x2, y2, x, y) {
      var d = [x2 - x1, y2 - y1];
      // 如果端点相等，则判定点到点的距离
      if (exactEquals$1(d, [0, 0])) {
        return Math.sqrt((x - x1) * (x - x1) + (y - y1) * (y - y1));
      }
      var u = [-d[1], d[0]];
      normalize$3(u, u);
      var a = [x - x1, y - y1];
      return Math.abs(dot$1(a, u));
    },
    /**
     * 线段的角度
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {number} 导数
     */
    tangentAngle: function tangentAngle(x1, y1, x2, y2) {
      return Math.atan2(y2 - y1, x2 - x1);
    }
  };
  var EPSILON$2 = 0.0001;
  /**
   * 使用牛顿切割法求最近的点
   * @param {number[]} xArr      点的 x 数组
   * @param {number[]} yArr      点的 y 数组
   * @param {number}   x         指定的点 x
   * @param {number}   y         指定的点 y
   * @param {Function} tCallback 差值函数
   */
  function nearestPoint$1(xArr, yArr, x, y, tCallback, length) {
    var t = -1;
    var d = Infinity;
    var v0 = [x, y];
    var segNum = 20;
    if (length && length > 200) {
      segNum = length / 10;
    }
    var increaseRate = 1 / segNum;
    var interval = increaseRate / 10;
    for (var i = 0; i <= segNum; i++) {
      var _t = i * increaseRate;
      var v1 = [tCallback.apply(void 0, xArr.concat([_t])), tCallback.apply(void 0, yArr.concat([_t]))];
      var d1 = distance$2(v0[0], v0[1], v1[0], v1[1]);
      if (d1 < d) {
        t = _t;
        d = d1;
      }
    }
    // 提前终止
    if (t === 0) {
      return {
        x: xArr[0],
        y: yArr[0]
      };
    }
    if (t === 1) {
      var count = xArr.length;
      return {
        x: xArr[count - 1],
        y: yArr[count - 1]
      };
    }
    d = Infinity;
    for (var _i = 0; _i < 32; _i++) {
      if (interval < EPSILON$2) {
        break;
      }
      var prev = t - interval;
      var next = t + interval;
      var _v = [tCallback.apply(void 0, xArr.concat([prev])), tCallback.apply(void 0, yArr.concat([prev]))];
      var _d = distance$2(v0[0], v0[1], _v[0], _v[1]);
      if (prev >= 0 && _d < d) {
        t = prev;
        d = _d;
      } else {
        var v2 = [tCallback.apply(void 0, xArr.concat([next])), tCallback.apply(void 0, yArr.concat([next]))];
        var d2 = distance$2(v0[0], v0[1], v2[0], v2[1]);
        if (next <= 1 && d2 < d) {
          t = next;
          d = d2;
        } else {
          interval *= 0.5;
        }
      }
    }
    return {
      x: tCallback.apply(void 0, xArr.concat([t])),
      y: tCallback.apply(void 0, yArr.concat([t]))
    };
  }
  // 近似求解 https://community.khronos.org/t/3d-cubic-bezier-segment-length/62363/2
  function snapLength$1(xArr, yArr) {
    var totalLength = 0;
    var count = xArr.length;
    for (var i = 0; i < count; i++) {
      var x = xArr[i];
      var y = yArr[i];
      var nextX = xArr[(i + 1) % count];
      var nextY = yArr[(i + 1) % count];
      totalLength += distance$2(x, y, nextX, nextY);
    }
    return totalLength / 2;
  }

  // 差值公式
  function quadraticAt$1(p0, p1, p2, t) {
    var onet = 1 - t;
    return onet * onet * p0 + 2 * t * onet * p1 + t * t * p2;
  }
  // 求极值
  function extrema$2(p0, p1, p2) {
    var a = p0 + p2 - 2 * p1;
    if (isNumberEqual$2(a, 0)) {
      return [0.5];
    }
    var rst = (p0 - p1) / a;
    if (rst <= 1 && rst >= 0) {
      return [rst];
    }
    return [];
  }
  function derivativeAt$2(p0, p1, p2, t) {
    return 2 * (1 - t) * (p1 - p0) + 2 * t * (p2 - p1);
  }
  // 分割贝塞尔曲线
  function divideQuadratic$1(x1, y1, x2, y2, x3, y3, t) {
    // 划分点
    var xt = quadraticAt$1(x1, x2, x3, t);
    var yt = quadraticAt$1(y1, y2, y3, t);
    // 分割的第一条曲线的控制点
    var controlPoint1 = line$2.pointAt(x1, y1, x2, y2, t);
    // 分割的第二条曲线的控制点
    var controlPoint2 = line$2.pointAt(x2, y2, x3, y3, t);
    return [[x1, y1, controlPoint1.x, controlPoint1.y, xt, yt], [xt, yt, controlPoint2.x, controlPoint2.y, x3, y3]];
  }
  // 使用迭代法取贝塞尔曲线的长度
  function quadraticLength$1(x1, y1, x2, y2, x3, y3, iterationCount) {
    if (iterationCount === 0) {
      return (distance$2(x1, y1, x2, y2) + distance$2(x2, y2, x3, y3) + distance$2(x1, y1, x3, y3)) / 2;
    }
    var quadratics = divideQuadratic$1(x1, y1, x2, y2, x3, y3, 0.5);
    var left = quadratics[0];
    var right = quadratics[1];
    left.push(iterationCount - 1);
    right.push(iterationCount - 1);
    return quadraticLength$1.apply(void 0, left) + quadraticLength$1.apply(void 0, right);
  }
  var quadratic$1 = {
    box: function box(x1, y1, x2, y2, x3, y3) {
      var xExtrema = extrema$2(x1, x2, x3)[0];
      var yExtrema = extrema$2(y1, y2, y3)[0];
      // 控制点不加入 box 的计算
      var xArr = [x1, x3];
      var yArr = [y1, y3];
      if (xExtrema !== undefined) {
        xArr.push(quadraticAt$1(x1, x2, x3, xExtrema));
      }
      if (yExtrema !== undefined) {
        yArr.push(quadraticAt$1(y1, y2, y3, yExtrema));
      }
      return getBBoxByArray$1(xArr, yArr);
    },
    length: function length(x1, y1, x2, y2, x3, y3) {
      return quadraticLength$1(x1, y1, x2, y2, x3, y3, 3);
    },
    nearestPoint: function nearestPoint$1$1(x1, y1, x2, y2, x3, y3, x0, y0) {
      return nearestPoint$1([x1, x2, x3], [y1, y2, y3], x0, y0, quadraticAt$1);
    },
    pointDistance: function pointDistance(x1, y1, x2, y2, x3, y3, x0, y0) {
      var point = this.nearestPoint(x1, y1, x2, y2, x3, y3, x0, y0);
      return distance$2(point.x, point.y, x0, y0);
    },
    interpolationAt: quadraticAt$1,
    pointAt: function pointAt(x1, y1, x2, y2, x3, y3, t) {
      return {
        x: quadraticAt$1(x1, x2, x3, t),
        y: quadraticAt$1(y1, y2, y3, t)
      };
    },
    divide: function divide(x1, y1, x2, y2, x3, y3, t) {
      return divideQuadratic$1(x1, y1, x2, y2, x3, y3, t);
    },
    tangentAngle: function tangentAngle(x1, y1, x2, y2, x3, y3, t) {
      var dx = derivativeAt$2(x1, x2, x3, t);
      var dy = derivativeAt$2(y1, y2, y3, t);
      var angle = Math.atan2(dy, dx);
      return piMod$1(angle);
    }
  };
  function cubicAt$1(p0, p1, p2, p3, t) {
    var onet = 1 - t; // t * t * t 的性能大概是 Math.pow(t, 3) 的三倍
    return onet * onet * onet * p0 + 3 * p1 * t * onet * onet + 3 * p2 * t * t * onet + p3 * t * t * t;
  }
  function derivativeAt$1$1(p0, p1, p2, p3, t) {
    var onet = 1 - t;
    return 3 * (onet * onet * (p1 - p0) + 2 * onet * t * (p2 - p1) + t * t * (p3 - p2));
  }
  function extrema$1$1(p0, p1, p2, p3) {
    var a = -3 * p0 + 9 * p1 - 9 * p2 + 3 * p3;
    var b = 6 * p0 - 12 * p1 + 6 * p2;
    var c = 3 * p1 - 3 * p0;
    var extremas = [];
    var t1;
    var t2;
    var discSqrt;
    if (isNumberEqual$2(a, 0)) {
      if (!isNumberEqual$2(b, 0)) {
        t1 = -c / b;
        if (t1 >= 0 && t1 <= 1) {
          extremas.push(t1);
        }
      }
    } else {
      var disc = b * b - 4 * a * c;
      if (isNumberEqual$2(disc, 0)) {
        extremas.push(-b / (2 * a));
      } else if (disc > 0) {
        discSqrt = Math.sqrt(disc);
        t1 = (-b + discSqrt) / (2 * a);
        t2 = (-b - discSqrt) / (2 * a);
        if (t1 >= 0 && t1 <= 1) {
          extremas.push(t1);
        }
        if (t2 >= 0 && t2 <= 1) {
          extremas.push(t2);
        }
      }
    }
    return extremas;
  }
  // 分割贝塞尔曲线
  function divideCubic$1(x1, y1, x2, y2, x3, y3, x4, y4, t) {
    // 划分点
    var xt = cubicAt$1(x1, x2, x3, x4, t);
    var yt = cubicAt$1(y1, y2, y3, y4, t);
    // 计算两点之间的差值点
    var c1 = line$2.pointAt(x1, y1, x2, y2, t);
    var c2 = line$2.pointAt(x2, y2, x3, y3, t);
    var c3 = line$2.pointAt(x3, y3, x4, y4, t);
    var c12 = line$2.pointAt(c1.x, c1.y, c2.x, c2.y, t);
    var c23 = line$2.pointAt(c2.x, c2.y, c3.x, c3.y, t);
    return [[x1, y1, c1.x, c1.y, c12.x, c12.y, xt, yt], [xt, yt, c23.x, c23.y, c3.x, c3.y, x4, y4]];
  }
  // 使用迭代法取贝塞尔曲线的长度，二阶和三阶分开写，更清晰和便于调试
  function cubicLength$1(x1, y1, x2, y2, x3, y3, x4, y4, iterationCount) {
    if (iterationCount === 0) {
      return snapLength$1([x1, x2, x3, x4], [y1, y2, y3, y4]);
    }
    var cubics = divideCubic$1(x1, y1, x2, y2, x3, y3, x4, y4, 0.5);
    var left = [].concat(cubics[0], [iterationCount - 1]);
    var right = [].concat(cubics[1], [iterationCount - 1]);
    return cubicLength$1.apply(void 0, left) + cubicLength$1.apply(void 0, right);
  }
  var cubic$1 = {
    extrema: extrema$1$1,
    box: function box(x1, y1, x2, y2, x3, y3, x4, y4) {
      var xArr = [x1, x4];
      var yArr = [y1, y4];
      var xExtrema = extrema$1$1(x1, x2, x3, x4);
      var yExtrema = extrema$1$1(y1, y2, y3, y4);
      for (var i = 0; i < xExtrema.length; i++) {
        xArr.push(cubicAt$1(x1, x2, x3, x4, xExtrema[i]));
      }
      for (var _i = 0; _i < yExtrema.length; _i++) {
        yArr.push(cubicAt$1(y1, y2, y3, y4, yExtrema[_i]));
      }
      return getBBoxByArray$1(xArr, yArr);
    },
    length: function length(x1, y1, x2, y2, x3, y3, x4, y4) {
      // 迭代三次，划分成 8 段求长度
      return cubicLength$1(x1, y1, x2, y2, x3, y3, x4, y4, 3);
    },
    nearestPoint: function nearestPoint$1$1(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length) {
      return nearestPoint$1([x1, x2, x3, x4], [y1, y2, y3, y4], x0, y0, cubicAt$1, length);
    },
    pointDistance: function pointDistance(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length) {
      var point = this.nearestPoint(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length);
      return distance$2(point.x, point.y, x0, y0);
    },
    interpolationAt: cubicAt$1,
    pointAt: function pointAt(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      return {
        x: cubicAt$1(x1, x2, x3, x4, t),
        y: cubicAt$1(y1, y2, y3, y4, t)
      };
    },
    divide: function divide(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      return divideCubic$1(x1, y1, x2, y2, x3, y3, x4, y4, t);
    },
    tangentAngle: function tangentAngle(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      var dx = derivativeAt$1$1(x1, x2, x3, x4, t);
      var dy = derivativeAt$1$1(y1, y2, y3, y4, t);
      return piMod$1(Math.atan2(dy, dx));
    }
  };

  function distance$1$1(x1, y1, x2, y2) {
    var dx = x1 - x2;
    var dy = y1 - y2;
    return Math.sqrt(dx * dx + dy * dy);
  }
  function inBox(minX, minY, width, height, x, y) {
    return x >= minX && x <= minX + width && y >= minY && y <= minY + height;
  }
  function inRect(minX, minY, width, height, lineWidth, x, y) {
    var halfWidth = lineWidth / 2;
    // 将四个边看做矩形来检测，比边的检测算法要快
    return inBox(minX - halfWidth, minY - halfWidth, width, lineWidth, x, y) ||
    // 上边
    inBox(minX + width - halfWidth, minY - halfWidth, lineWidth, height, x, y) ||
    // 右边
    inBox(minX + halfWidth, minY + height - halfWidth, width, lineWidth, x, y) ||
    // 下边
    inBox(minX - halfWidth, minY + halfWidth, lineWidth, height, x, y); // 左边
  }

  function inArc(cx, cy, r, startAngle, endAngle, lineWidth, x, y) {
    var angle = (Math.atan2(y - cy, x - cx) + Math.PI * 2) % (Math.PI * 2); // 转换到 0 - 2 * Math.PI 之间
    // if (angle < startAngle || angle > endAngle) {
    //   return false;
    // }
    var point = {
      x: cx + r * Math.cos(angle),
      y: cy + r * Math.sin(angle)
    };
    return distance$1$1(point.x, point.y, x, y) <= lineWidth / 2;
  }
  function inLine(x1, y1, x2, y2, lineWidth, x, y) {
    var minX = Math.min(x1, x2);
    var maxX = Math.max(x1, x2);
    var minY = Math.min(y1, y2);
    var maxY = Math.max(y1, y2);
    var halfWidth = lineWidth / 2;
    // 因为目前的方案是计算点到直线的距离，而有可能会在延长线上，所以要先判断是否在包围盒内
    // 这种方案会在水平或者竖直的情况下载线的延长线上有半 lineWidth 的误差
    if (!(x >= minX - halfWidth && x <= maxX + halfWidth && y >= minY - halfWidth && y <= maxY + halfWidth)) {
      return false;
    }
    // 因为已经计算了包围盒，所以仅需要计算到直线的距离即可，可以显著提升性能
    return line$2.pointToLine(x1, y1, x2, y2, x, y) <= lineWidth / 2;
  }
  function inPolyline(points, lineWidth, x, y, isClose) {
    var count = points.length;
    if (count < 2) {
      return false;
    }
    for (var i = 0; i < count - 1; i++) {
      var x1 = points[i][0];
      var y1 = points[i][1];
      var x2 = points[i + 1][0];
      var y2 = points[i + 1][1];
      if (inLine(x1, y1, x2, y2, lineWidth, x, y)) {
        return true;
      }
    }
    // 如果封闭，则计算起始点和结束点的边
    if (isClose) {
      var first = points[0];
      var last = points[count - 1];
      if (inLine(first[0], first[1], last[0], last[1], lineWidth, x, y)) {
        return true;
      }
    }
    return false;
  }
  // 多边形的射线检测，参考：https://blog.csdn.net/WilliamSun0122/article/details/77994526
  var tolerance = 1e-6;
  // 三态函数，判断两个double在eps精度下的大小关系
  function dcmp(x) {
    if (Math.abs(x) < tolerance) {
      return 0;
    }
    return x < 0 ? -1 : 1;
  }
  // 判断点Q是否在p1和p2的线段上
  function onSegment(p1, p2, q) {
    if ((q[0] - p1[0]) * (p2[1] - p1[1]) === (p2[0] - p1[0]) * (q[1] - p1[1]) && Math.min(p1[0], p2[0]) <= q[0] && q[0] <= Math.max(p1[0], p2[0]) && Math.min(p1[1], p2[1]) <= q[1] && q[1] <= Math.max(p1[1], p2[1])) {
      return true;
    }
    return false;
  }
  // 判断点P在多边形内-射线法
  function inPolygon(points, x, y) {
    var isHit = false;
    var n = points.length;
    if (n <= 2) {
      // svg 中点小于 3 个时，不显示，也无法被拾取
      return false;
    }
    for (var i = 0; i < n; i++) {
      var p1 = points[i];
      var p2 = points[(i + 1) % n];
      if (onSegment(p1, p2, [x, y])) {
        // 点在多边形一条边上
        return true;
      }
      // 前一个判断min(p1[1],p2[1])<P.y<=max(p1[1],p2[1])
      // 后一个判断被测点 在 射线与边交点 的左边
      if (dcmp(p1[1] - y) > 0 !== dcmp(p2[1] - y) > 0 && dcmp(x - (y - p1[1]) * (p1[0] - p2[0]) / (p1[1] - p2[1]) - p1[0]) < 0) {
        isHit = !isHit;
      }
    }
    return isHit;
  }
  function inPolygons(polygons, x, y) {
    var isHit = false;
    for (var i = 0; i < polygons.length; i++) {
      var points = polygons[i];
      isHit = inPolygon(points, x, y);
      if (isHit) {
        break;
      }
    }
    return isHit;
  }

  function isPointInPath(displayObject, position, isClipPath) {
    var _displayObject$parsed = displayObject.parsedStyle,
      r = _displayObject$parsed.r,
      fill = _displayObject$parsed.fill,
      stroke = _displayObject$parsed.stroke,
      lineWidth = _displayObject$parsed.lineWidth,
      increasedLineWidthForHitTesting = _displayObject$parsed.increasedLineWidthForHitTesting,
      pointerEvents = _displayObject$parsed.pointerEvents;
    var halfLineWidth = ((lineWidth || 0) + (increasedLineWidthForHitTesting || 0)) / 2;
    var absDistance = distance$1$1(r, r, position.x, position.y);
    var _isFillOrStrokeAffect = isFillOrStrokeAffected(pointerEvents, fill, stroke),
      hasFill = _isFillOrStrokeAffect[0],
      hasStroke = _isFillOrStrokeAffect[1];
    if (hasFill && hasStroke || isClipPath) {
      return absDistance <= r + halfLineWidth;
    }
    if (hasFill) {
      return absDistance <= r;
    }
    if (hasStroke) {
      return absDistance >= r - halfLineWidth && absDistance <= r + halfLineWidth;
    }
    return false;
  }

  function ellipseDistance(squareX, squareY, rx, ry) {
    return squareX / (rx * rx) + squareY / (ry * ry);
  }
  function isPointInPath$1(displayObject, position, isClipPath) {
    var _displayObject$parsed = displayObject.parsedStyle,
      rx = _displayObject$parsed.rx,
      ry = _displayObject$parsed.ry,
      fill = _displayObject$parsed.fill,
      stroke = _displayObject$parsed.stroke,
      lineWidth = _displayObject$parsed.lineWidth,
      increasedLineWidthForHitTesting = _displayObject$parsed.increasedLineWidthForHitTesting,
      pointerEvents = _displayObject$parsed.pointerEvents;
    var x = position.x,
      y = position.y;
    var _isFillOrStrokeAffect = isFillOrStrokeAffected(pointerEvents, fill, stroke),
      hasFill = _isFillOrStrokeAffect[0],
      hasStroke = _isFillOrStrokeAffect[1];
    var halfLineWith = ((lineWidth || 0) + (increasedLineWidthForHitTesting || 0)) / 2;
    var squareX = (x - rx) * (x - rx);
    var squareY = (y - ry) * (y - ry);
    // 使用椭圆的公式： x*x/rx*rx + y*y/ry*ry = 1;
    if (hasFill && hasStroke || isClipPath) {
      return ellipseDistance(squareX, squareY, rx + halfLineWith, ry + halfLineWith) <= 1;
    }
    if (hasFill) {
      return ellipseDistance(squareX, squareY, rx, ry) <= 1;
    }
    if (hasStroke) {
      return ellipseDistance(squareX, squareY, rx - halfLineWith, ry - halfLineWith) >= 1 && ellipseDistance(squareX, squareY, rx + halfLineWith, ry + halfLineWith) <= 1;
    }
    return false;
  }

  function isPointInPath$2(displayObject, position, isClipPath) {
    var _displayObject$parsed = displayObject.parsedStyle,
      x1 = _displayObject$parsed.x1,
      y1 = _displayObject$parsed.y1,
      x2 = _displayObject$parsed.x2,
      y2 = _displayObject$parsed.y2,
      lineWidth = _displayObject$parsed.lineWidth,
      increasedLineWidthForHitTesting = _displayObject$parsed.increasedLineWidthForHitTesting,
      _displayObject$parsed2 = _displayObject$parsed.defX,
      x = _displayObject$parsed2 === void 0 ? 0 : _displayObject$parsed2,
      _displayObject$parsed3 = _displayObject$parsed.defY,
      y = _displayObject$parsed3 === void 0 ? 0 : _displayObject$parsed3,
      pointerEvents = _displayObject$parsed.pointerEvents,
      fill = _displayObject$parsed.fill,
      stroke = _displayObject$parsed.stroke;
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    var _isFillOrStrokeAffect = isFillOrStrokeAffected(pointerEvents, fill, stroke),
      hasStroke = _isFillOrStrokeAffect[1];
    if (!hasStroke && !isClipPath || !lineWidth) {
      return false;
    }
    return inLine(x1, y1, x2, y2, (lineWidth || 0) + (increasedLineWidthForHitTesting || 0), position.x + x, position.y + y);
  }

  // TODO: replace it with method in @antv/util
  function isPointInStroke(segments, lineWidth, px, py, length) {
    var isHit = false;
    var halfWidth = lineWidth / 2;
    for (var i = 0; i < segments.length; i++) {
      var segment = segments[i];
      var currentPoint = segment.currentPoint,
        params = segment.params,
        prePoint = segment.prePoint,
        box = segment.box;
      // 如果在前面已经生成过包围盒，直接按照包围盒计算
      if (box && !inBox(box.x - halfWidth, box.y - halfWidth, box.width + lineWidth, box.height + lineWidth, px, py)) {
        continue;
      }
      switch (segment.command) {
        // L 和 Z 都是直线， M 不进行拾取
        case 'L':
        case 'Z':
          isHit = inLine(prePoint[0], prePoint[1], currentPoint[0], currentPoint[1], lineWidth, px, py);
          if (isHit) {
            return true;
          }
          break;
        case 'Q':
          var qDistance = quadratic$1.pointDistance(prePoint[0], prePoint[1], params[1], params[2], params[3], params[4], px, py);
          isHit = qDistance <= lineWidth / 2;
          if (isHit) {
            return true;
          }
          break;
        case 'C':
          var cDistance = cubic$1.pointDistance(prePoint[0],
          // 上一段结束位置, 即 C 的起始点
          prePoint[1], params[1],
          // 'C' 的参数，1、2 为第一个控制点，3、4 为第二个控制点，5、6 为结束点
          params[2], params[3], params[4], params[5], params[6], px, py, length);
          isHit = cDistance <= lineWidth / 2;
          if (isHit) {
            return true;
          }
          break;
        case 'A':
          // cache conversion result
          if (!segment.cubicParams) {
            segment.cubicParams = arcToCubic(prePoint[0], prePoint[1], params[1], params[2], params[3], params[4], params[5], params[6], params[7], undefined);
          }
          var args = segment.cubicParams;
          // fixArc
          var prePointInCubic = prePoint;
          for (var _i = 0; _i < args.length; _i += 6) {
            var _cDistance = cubic$1.pointDistance(prePointInCubic[0],
            // 上一段结束位置, 即 C 的起始点
            prePointInCubic[1], args[_i], args[_i + 1], args[_i + 2], args[_i + 3], args[_i + 4], args[_i + 5], px, py, length);
            prePointInCubic = [args[_i + 4], args[_i + 5]];
            isHit = _cDistance <= lineWidth / 2;
            if (isHit) {
              return true;
            }
          }
          break;
      }
    }
    return isHit;
  }
  function isPointInPath$3(displayObject, position, isClipPath, isPointInPath) {
    var _displayObject$parsed = displayObject.parsedStyle,
      lineWidth = _displayObject$parsed.lineWidth,
      increasedLineWidthForHitTesting = _displayObject$parsed.increasedLineWidthForHitTesting,
      stroke = _displayObject$parsed.stroke,
      fill = _displayObject$parsed.fill,
      _displayObject$parsed2 = _displayObject$parsed.defX,
      x = _displayObject$parsed2 === void 0 ? 0 : _displayObject$parsed2,
      _displayObject$parsed3 = _displayObject$parsed.defY,
      y = _displayObject$parsed3 === void 0 ? 0 : _displayObject$parsed3,
      path = _displayObject$parsed.path,
      pointerEvents = _displayObject$parsed.pointerEvents;
    var _isFillOrStrokeAffect = isFillOrStrokeAffected(pointerEvents, fill, stroke),
      hasFill = _isFillOrStrokeAffect[0],
      hasStroke = _isFillOrStrokeAffect[1];
    var segments = path.segments,
      hasArc = path.hasArc,
      polylines = path.polylines,
      polygons = path.polygons;
    var totalLength = getOrCalculatePathTotalLength(displayObject);
    var isHit = false;
    if (hasFill || isClipPath) {
      if (hasArc) {
        // 存在曲线时，暂时使用 canvas 的 api 计算，后续可以进行多边形切割
        isHit = isPointInPath(displayObject, position);
      } else {
        // 提取出来的多边形包含闭合的和非闭合的，在这里统一按照多边形处理
        isHit = inPolygons(polygons, position.x + x, position.y + y) || inPolygons(polylines, position.x + x, position.y + y);
      }
      return isHit;
    } else if (hasStroke || isClipPath) {
      isHit = isPointInStroke(segments, (lineWidth || 0) + (increasedLineWidthForHitTesting || 0), position.x + x, position.y + y, totalLength);
    }
    return isHit;
  }

  function isPointInPath$4(displayObject, position, isClipPath) {
    var _displayObject$parsed = displayObject.parsedStyle,
      stroke = _displayObject$parsed.stroke,
      fill = _displayObject$parsed.fill,
      lineWidth = _displayObject$parsed.lineWidth,
      increasedLineWidthForHitTesting = _displayObject$parsed.increasedLineWidthForHitTesting,
      points = _displayObject$parsed.points,
      _displayObject$parsed2 = _displayObject$parsed.defX,
      x = _displayObject$parsed2 === void 0 ? 0 : _displayObject$parsed2,
      _displayObject$parsed3 = _displayObject$parsed.defY,
      y = _displayObject$parsed3 === void 0 ? 0 : _displayObject$parsed3,
      pointerEvents = _displayObject$parsed.pointerEvents;
    var _isFillOrStrokeAffect = isFillOrStrokeAffected(pointerEvents, fill, stroke),
      hasFill = _isFillOrStrokeAffect[0],
      hasStroke = _isFillOrStrokeAffect[1];
    var isHit = false;
    if (hasStroke || isClipPath) {
      isHit = inPolyline(points.points, (lineWidth || 0) + (increasedLineWidthForHitTesting || 0), position.x + x, position.y + y, true);
    }
    if (!isHit && (hasFill || isClipPath)) {
      isHit = inPolygon(points.points, position.x + x, position.y + y);
    }
    return isHit;
  }

  function isPointInPath$5(displayObject, position, isClipPath) {
    var _displayObject$parsed = displayObject.parsedStyle,
      lineWidth = _displayObject$parsed.lineWidth,
      increasedLineWidthForHitTesting = _displayObject$parsed.increasedLineWidthForHitTesting,
      points = _displayObject$parsed.points,
      _displayObject$parsed2 = _displayObject$parsed.defX,
      x = _displayObject$parsed2 === void 0 ? 0 : _displayObject$parsed2,
      _displayObject$parsed3 = _displayObject$parsed.defY,
      y = _displayObject$parsed3 === void 0 ? 0 : _displayObject$parsed3,
      pointerEvents = _displayObject$parsed.pointerEvents,
      fill = _displayObject$parsed.fill,
      stroke = _displayObject$parsed.stroke;
    var _isFillOrStrokeAffect = isFillOrStrokeAffected(pointerEvents, fill, stroke),
      hasStroke = _isFillOrStrokeAffect[1];
    if (!hasStroke && !isClipPath || !lineWidth) {
      return false;
    }
    return inPolyline(points.points, (lineWidth || 0) + (increasedLineWidthForHitTesting || 0), position.x + x, position.y + y, false);
  }

  function isPointInPath$6(displayObject, position, isClipPath, isPointInPath) {
    var _displayObject$parsed = displayObject.parsedStyle,
      radius = _displayObject$parsed.radius,
      fill = _displayObject$parsed.fill,
      stroke = _displayObject$parsed.stroke,
      lineWidth = _displayObject$parsed.lineWidth,
      increasedLineWidthForHitTesting = _displayObject$parsed.increasedLineWidthForHitTesting,
      width = _displayObject$parsed.width,
      height = _displayObject$parsed.height,
      pointerEvents = _displayObject$parsed.pointerEvents;
    var _isFillOrStrokeAffect = isFillOrStrokeAffected(pointerEvents, fill, stroke),
      hasFill = _isFillOrStrokeAffect[0],
      hasStroke = _isFillOrStrokeAffect[1];
    var hasRadius = radius && radius.some(function (r) {
      return r !== 0;
    });
    var lineWidthForHitTesting = (lineWidth || 0) + (increasedLineWidthForHitTesting || 0);
    // 无圆角时的策略
    if (!hasRadius) {
      var halfWidth = lineWidthForHitTesting / 2;
      // 同时填充和带有边框
      if (hasFill && hasStroke || isClipPath) {
        return inBox(0 - halfWidth, 0 - halfWidth, width + halfWidth, height + halfWidth, position.x, position.y);
      }
      // 仅填充
      if (hasFill) {
        return inBox(0, 0, width, height, position.x, position.y);
      }
      if (hasStroke) {
        return inRect(0, 0, width, height, lineWidthForHitTesting, position.x, position.y);
      }
    } else {
      var isHit = false;
      if (hasStroke || isClipPath) {
        isHit = inRectWithRadius(0, 0, width, height, radius.map(function (r) {
          return clamp(r, 0, Math.min(Math.abs(width) / 2, Math.abs(height) / 2));
        }), lineWidthForHitTesting, position.x, position.y);
      }
      // 仅填充时带有圆角的矩形直接通过图形拾取
      // 以后可以改成纯数学的近似拾取，将圆弧切割成多边形
      if (!isHit && (hasFill || isClipPath)) {
        isHit = isPointInPath(displayObject, position);
      }
      return isHit;
    }
    return false;
  }
  function inRectWithRadius(minX, minY, width, height, radiusArray, lineWidth, x, y) {
    var tlr = radiusArray[0],
      trr = radiusArray[1],
      brr = radiusArray[2],
      blr = radiusArray[3];
    return inLine(minX + tlr, minY, minX + width - trr, minY, lineWidth, x, y) || inLine(minX + width, minY + trr, minX + width, minY + height - brr, lineWidth, x, y) || inLine(minX + width - brr, minY + height, minX + blr, minY + height, lineWidth, x, y) || inLine(minX, minY + height - blr, minX, minY + tlr, lineWidth, x, y) || inArc(minX + width - trr, minY + trr, trr, 1.5 * Math.PI, 2 * Math.PI, lineWidth, x, y) || inArc(minX + width - brr, minY + height - brr, brr, 0, 0.5 * Math.PI, lineWidth, x, y) || inArc(minX + blr, minY + height - blr, blr, 0.5 * Math.PI, Math.PI, lineWidth, x, y) || inArc(minX + tlr, minY + tlr, tlr, Math.PI, 1.5 * Math.PI, lineWidth, x, y);
  }

  var Plugin$1 = /*#__PURE__*/function (_AbstractRendererPlug) {
    _inheritsLoose$3(Plugin, _AbstractRendererPlug);
    function Plugin() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _AbstractRendererPlug.call.apply(_AbstractRendererPlug, [this].concat(args)) || this;
      _this.name = 'canvas-picker';
      return _this;
    }
    var _proto = Plugin.prototype;
    _proto.init = function init() {
      var _pointInPathPickerFac;
      var trueFunc = function trueFunc() {
        return true;
      };
      var pointInPathPickerFactory = (_pointInPathPickerFac = {}, _pointInPathPickerFac[Shape.CIRCLE] = isPointInPath, _pointInPathPickerFac[Shape.ELLIPSE] = isPointInPath$1, _pointInPathPickerFac[Shape.RECT] = isPointInPath$6, _pointInPathPickerFac[Shape.LINE] = isPointInPath$2, _pointInPathPickerFac[Shape.POLYLINE] = isPointInPath$5, _pointInPathPickerFac[Shape.POLYGON] = isPointInPath$4, _pointInPathPickerFac[Shape.PATH] = isPointInPath$3, _pointInPathPickerFac[Shape.TEXT] = trueFunc, _pointInPathPickerFac[Shape.GROUP] = null, _pointInPathPickerFac[Shape.IMAGE] = trueFunc, _pointInPathPickerFac[Shape.HTML] = null, _pointInPathPickerFac[Shape.MESH] = null, _pointInPathPickerFac);
      // @ts-ignore
      this.context.pointInPathPickerFactory = pointInPathPickerFactory;
      this.addRenderingPlugin(new CanvasPickerPlugin());
    };
    _proto.destroy = function destroy() {
      // @ts-ignore
      delete this.context.pointInPathPickerFactory;
      this.removeAllRenderingPlugins();
    };
    return Plugin;
  }(AbstractRendererPlugin);

  function _regeneratorRuntime$2() {
    _regeneratorRuntime$2 = function () {
      return exports;
    };
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function (obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function (method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) keys.push(key);
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function (skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
      },
      stop: function () {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function (exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function (type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function (record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function (finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      catch: function (tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function (iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  function asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator$2(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  function _extends$1() {
    _extends$1 = Object.assign ? Object.assign.bind() : function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];
        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }
      return target;
    };
    return _extends$1.apply(this, arguments);
  }
  function _inheritsLoose$4(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$5(subClass, superClass);
  }
  function _setPrototypeOf$5(o, p) {
    _setPrototypeOf$5 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$5(o, p);
  }

  /**
   * support 2 modes in rendering:
   * * immediate
   * * delayed: render at the end of frame with dirty-rectangle
   */
  var CanvasRendererPlugin = /*#__PURE__*/function () {
    /**
     * RBush used in dirty rectangle rendering
     */

    function CanvasRendererPlugin(canvasRendererPluginOptions) {
      this.canvasRendererPluginOptions = void 0;
      this.context = void 0;
      this.pathGeneratorFactory = void 0;
      this.rBush = void 0;
      this.removedRBushNodeAABBs = [];
      this.renderQueue = [];
      this.restoreStack = [];
      this.clearFullScreen = false;
      this.vpMatrix = create$1();
      this.dprMatrix = create$1();
      this.tmpMat4 = create$1();
      this.vec3a = create$2();
      this.vec3b = create$2();
      this.vec3c = create$2();
      this.vec3d = create$2();
      this.canvasRendererPluginOptions = canvasRendererPluginOptions;
    }
    var _proto = CanvasRendererPlugin.prototype;
    _proto.apply = function apply(context) {
      var _this = this;
      this.context = context;
      // @ts-ignore
      var config = context.config,
        camera = context.camera,
        renderingService = context.renderingService,
        renderingContext = context.renderingContext,
        rBushRoot = context.rBushRoot,
        pathGeneratorFactory = context.pathGeneratorFactory;
      this.rBush = rBushRoot;
      this.pathGeneratorFactory = pathGeneratorFactory;
      var contextService = context.contextService;
      var canvas = renderingContext.root.ownerDocument.defaultView;
      var handleUnmounted = function handleUnmounted(e) {
        var object = e.target;
        // remove r-bush node
        // @ts-ignore
        var rBushNode = object.rBushNode;
        if (rBushNode.aabb) {
          // save removed aabbs for dirty-rectangle rendering later
          _this.removedRBushNodeAABBs.push(rBushNode.aabb);
        }
      };
      var handleCulled = function handleCulled(e) {
        var object = e.target;
        // @ts-ignore
        var rBushNode = object.rBushNode;
        if (rBushNode.aabb) {
          // save removed aabbs for dirty-rectangle rendering later
          _this.removedRBushNodeAABBs.push(rBushNode.aabb);
        }
      };
      renderingService.hooks.init.tapPromise(CanvasRendererPlugin.tag, /*#__PURE__*/_asyncToGenerator$2( /*#__PURE__*/_regeneratorRuntime$2().mark(function _callee() {
        var dpr, width, height, context;
        return _regeneratorRuntime$2().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                canvas.addEventListener(ElementEvent.UNMOUNTED, handleUnmounted);
                canvas.addEventListener(ElementEvent.CULLED, handleCulled);
                // clear fullscreen
                dpr = contextService.getDPR();
                width = config.width, height = config.height;
                context = contextService.getContext();
                _this.clearRect(context, 0, 0, width * dpr, height * dpr, config.background);
              case 6:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      })));
      renderingService.hooks.destroy.tap(CanvasRendererPlugin.tag, function () {
        canvas.removeEventListener(ElementEvent.UNMOUNTED, handleUnmounted);
        canvas.removeEventListener(ElementEvent.CULLED, handleCulled);
      });
      renderingService.hooks.beginFrame.tap(CanvasRendererPlugin.tag, function () {
        var context = contextService.getContext();
        var dpr = contextService.getDPR();
        var width = config.width,
          height = config.height;
        var _this$canvasRendererP = _this.canvasRendererPluginOptions,
          dirtyObjectNumThreshold = _this$canvasRendererP.dirtyObjectNumThreshold,
          dirtyObjectRatioThreshold = _this$canvasRendererP.dirtyObjectRatioThreshold;
        // some heuristic conditions such as 80% object changed
        var _renderingService$get = renderingService.getStats(),
          total = _renderingService$get.total,
          rendered = _renderingService$get.rendered;
        var ratio = rendered / total;
        _this.clearFullScreen = renderingService.disableDirtyRectangleRendering() || rendered > dirtyObjectNumThreshold && ratio > dirtyObjectRatioThreshold;
        if (context) {
          context.resetTransform();
          if (_this.clearFullScreen) {
            _this.clearRect(context, 0, 0, width * dpr, height * dpr, config.background);
          }
        }
      });
      var renderByZIndex = function renderByZIndex(object) {
        if (object.isVisible() && !object.isCulled()) {
          _this.renderDisplayObject(object, contextService, renderingService);
          // if we did a full screen rendering last frame
          _this.saveDirtyAABB(object);
        }
        var sorted = object.sortable.sorted || object.childNodes;
        // should account for z-index
        sorted.forEach(function (child) {
          renderByZIndex(child);
        });
      };
      // render at the end of frame
      renderingService.hooks.endFrame.tap(CanvasRendererPlugin.tag, function () {
        var context = contextService.getContext();
        // clear & clip dirty rectangle
        var dpr = contextService.getDPR();
        fromScaling(_this.dprMatrix, fromValues$2(dpr, dpr, 1));
        multiply(_this.vpMatrix, _this.dprMatrix, camera.getOrthoMatrix());
        if (_this.clearFullScreen) {
          renderByZIndex(renderingContext.root);
        } else {
          // merge removed AABB
          var dirtyRenderBounds = _this.safeMergeAABB.apply(_this, [_this.mergeDirtyAABBs(_this.renderQueue)].concat(_this.removedRBushNodeAABBs.map(function (_ref2) {
            var minX = _ref2.minX,
              minY = _ref2.minY,
              maxX = _ref2.maxX,
              maxY = _ref2.maxY;
            var aabb = new AABB();
            aabb.setMinMax(fromValues$2(minX, minY, 0), fromValues$2(maxX, maxY, 0));
            return aabb;
          })));
          _this.removedRBushNodeAABBs = [];
          if (AABB.isEmpty(dirtyRenderBounds)) {
            _this.renderQueue = [];
            return;
          }
          var dirtyRect = _this.convertAABB2Rect(dirtyRenderBounds);
          var x = dirtyRect.x,
            y = dirtyRect.y,
            width = dirtyRect.width,
            height = dirtyRect.height;
          var tl = transformMat4(_this.vec3a, fromValues$2(x, y, 0), _this.vpMatrix);
          var tr = transformMat4(_this.vec3b, fromValues$2(x + width, y, 0), _this.vpMatrix);
          var bl = transformMat4(_this.vec3c, fromValues$2(x, y + height, 0), _this.vpMatrix);
          var br = transformMat4(_this.vec3d, fromValues$2(x + width, y + height, 0), _this.vpMatrix);
          var minx = Math.min(tl[0], tr[0], br[0], bl[0]);
          var miny = Math.min(tl[1], tr[1], br[1], bl[1]);
          var maxx = Math.max(tl[0], tr[0], br[0], bl[0]);
          var maxy = Math.max(tl[1], tr[1], br[1], bl[1]);
          var ix = Math.floor(minx);
          var iy = Math.floor(miny);
          var iwidth = Math.ceil(maxx - minx);
          var iheight = Math.ceil(maxy - miny);
          context.save();
          _this.clearRect(context, ix, iy, iwidth, iheight, config.background);
          context.beginPath();
          context.rect(ix, iy, iwidth, iheight);
          context.clip();
          // @see https://developer.mozilla.org/en-US/docs/Web/API/Canvas_API/Tutorial/Transformations
          context.setTransform(_this.vpMatrix[0], _this.vpMatrix[1], _this.vpMatrix[4], _this.vpMatrix[5], _this.vpMatrix[12], _this.vpMatrix[13]);
          // draw dirty rectangle
          var _config$renderer$getC = config.renderer.getConfig(),
            enableDirtyRectangleRenderingDebug = _config$renderer$getC.enableDirtyRectangleRenderingDebug;
          if (enableDirtyRectangleRenderingDebug) {
            canvas.dispatchEvent(new CustomEvent(CanvasEvent.DIRTY_RECTANGLE, {
              dirtyRect: {
                x: ix,
                y: iy,
                width: iwidth,
                height: iheight
              }
            }));
          }
          // search objects intersect with dirty rectangle
          var dirtyObjects = _this.searchDirtyObjects(dirtyRenderBounds);
          // do rendering
          dirtyObjects
          // sort by z-index
          .sort(function (a, b) {
            return a.sortable.renderOrder - b.sortable.renderOrder;
          }).forEach(function (object) {
            // culled object should not be rendered
            if (object && object.isVisible() && !object.isCulled()) {
              _this.renderDisplayObject(object, contextService, renderingService);
            }
          });
          context.restore();
          // save dirty AABBs in last frame
          _this.renderQueue.forEach(function (object) {
            _this.saveDirtyAABB(object);
          });
          // clear queue
          _this.renderQueue = [];
        }
        // pop restore stack, eg. root -> parent -> child
        _this.restoreStack.forEach(function () {
          context.restore();
        });
        // clear restore stack
        _this.restoreStack = [];
      });
      renderingService.hooks.render.tap(CanvasRendererPlugin.tag, function (object) {
        if (!_this.clearFullScreen) {
          // render at the end of frame
          _this.renderQueue.push(object);
        }
      });
    };
    _proto.clearRect = function clearRect(context, x, y, width, height, background) {
      // clearRect is faster than fillRect @see https://stackoverflow.com/a/30830253
      context.clearRect(x, y, width, height);
      if (background) {
        context.fillStyle = background;
        context.fillRect(x, y, width, height);
      }
    };
    _proto.renderDisplayObject = function renderDisplayObject(object, contextService, renderingService) {
      var context = contextService.getContext();
      var nodeName = object.nodeName;
      // restore to its ancestor
      var parent = this.restoreStack[this.restoreStack.length - 1];
      if (parent && !(object.compareDocumentPosition(parent) & Node.DOCUMENT_POSITION_CONTAINS)) {
        context.restore();
        this.restoreStack.pop();
      }
      // @ts-ignore
      var styleRenderer = this.context.styleRendererFactory[nodeName];
      var generatePath = this.pathGeneratorFactory[nodeName];
      // clip path
      var clipPath = object.parsedStyle.clipPath;
      if (clipPath) {
        this.applyWorldTransform(context, clipPath);
        // generate path in local space
        var _generatePath = this.pathGeneratorFactory[clipPath.nodeName];
        if (_generatePath) {
          context.save();
          // save clip
          this.restoreStack.push(object);
          context.beginPath();
          _generatePath(context, clipPath.parsedStyle);
          context.closePath();
          context.clip();
        }
      }
      // fill & stroke
      if (styleRenderer) {
        this.applyWorldTransform(context, object);
        context.save();
        // apply attributes to context
        this.applyAttributesToContext(context, object);
      }
      if (generatePath) {
        context.beginPath();
        generatePath(context, object.parsedStyle);
        if (object.nodeName !== Shape.LINE && object.nodeName !== Shape.PATH && object.nodeName !== Shape.POLYLINE) {
          context.closePath();
        }
      }
      // fill & stroke
      if (styleRenderer) {
        styleRenderer.render(context, object.parsedStyle, object, renderingService);
        // restore applied attributes, eg. shadowBlur shadowColor...
        context.restore();
      }
      // finish rendering, clear dirty flag
      object.renderable.dirty = false;
    };
    _proto.convertAABB2Rect = function convertAABB2Rect(aabb) {
      var min = aabb.getMin();
      var max = aabb.getMax();
      // expand the rectangle a bit to avoid artifacts
      // @see https://www.yuque.com/antv/ou292n/bi8nix#ExvCu
      var minX = Math.floor(min[0]);
      var minY = Math.floor(min[1]);
      var maxX = Math.ceil(max[0]);
      var maxY = Math.ceil(max[1]);
      var width = maxX - minX;
      var height = maxY - minY;
      return {
        x: minX,
        y: minY,
        width: width,
        height: height
      };
    }
    /**
     * TODO: merge dirty rectangles with some strategies.
     * For now, we just simply merge all the rectangles into one.
     * @see https://idom.me/articles/841.html
     */;
    _proto.mergeDirtyAABBs = function mergeDirtyAABBs(dirtyObjects) {
      // merge into a big AABB
      // TODO: skip descendant if ancestor is caculated, but compareNodePosition is really slow
      var aabb = new AABB();
      dirtyObjects.forEach(function (object) {
        var renderBounds = object.getRenderBounds();
        aabb.add(renderBounds);
        var dirtyRenderBounds = object.renderable.dirtyRenderBounds;
        if (dirtyRenderBounds) {
          aabb.add(dirtyRenderBounds);
        }
      });
      return aabb;
    };
    _proto.searchDirtyObjects = function searchDirtyObjects(dirtyRectangle) {
      // search in r-tree, get all affected nodes
      var _dirtyRectangle$getMi = dirtyRectangle.getMin(),
        minX = _dirtyRectangle$getMi[0],
        minY = _dirtyRectangle$getMi[1];
      var _dirtyRectangle$getMa = dirtyRectangle.getMax(),
        maxX = _dirtyRectangle$getMa[0],
        maxY = _dirtyRectangle$getMa[1];
      var rBushNodes = this.rBush.search({
        minX: minX,
        minY: minY,
        maxX: maxX,
        maxY: maxY
      });
      return rBushNodes.map(function (_ref3) {
        var id = _ref3.id;
        return runtime.displayObjectPool.getByEntity(id);
      });
    };
    _proto.saveDirtyAABB = function saveDirtyAABB(object) {
      var renderable = object.renderable;
      if (!renderable.dirtyRenderBounds) {
        renderable.dirtyRenderBounds = new AABB();
      }
      var renderBounds = object.getRenderBounds();
      if (renderBounds) {
        // save last dirty aabb
        renderable.dirtyRenderBounds.update(renderBounds.center, renderBounds.halfExtents);
      }
    }
    /**
     * TODO: batch the same global attributes
     */;
    _proto.applyAttributesToContext = function applyAttributesToContext(context, object) {
      var _object$parsedStyle = object.parsedStyle,
        stroke = _object$parsedStyle.stroke,
        fill = _object$parsedStyle.fill,
        opacity = _object$parsedStyle.opacity,
        lineDash = _object$parsedStyle.lineDash,
        lineDashOffset = _object$parsedStyle.lineDashOffset;
      // @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/setLineDash
      if (lineDash) {
        context.setLineDash(lineDash);
      }
      // @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/lineDashOffset
      if (!isNil(lineDashOffset)) {
        context.lineDashOffset = lineDashOffset;
      }
      if (!isNil(opacity)) {
        context.globalAlpha *= opacity;
      }
      if (!isNil(stroke) && !Array.isArray(stroke) && !stroke.isNone) {
        context.strokeStyle = object.attributes.stroke;
      }
      if (!isNil(fill) && !Array.isArray(fill) && !fill.isNone) {
        context.fillStyle = object.attributes.fill;
      }
    };
    _proto.applyWorldTransform = function applyWorldTransform(context, object, matrix) {
      var tx = 0;
      var ty = 0;
      var _ref4 = object.parsedStyle || {},
        anchor = _ref4.anchor;
      var anchorX = anchor && anchor[0] || 0;
      var anchorY = anchor && anchor[1] || 0;
      if (anchorX !== 0 || anchorY !== 0) {
        var bounds = object.getGeometryBounds();
        var width = bounds && bounds.halfExtents[0] * 2 || 0;
        var height = bounds && bounds.halfExtents[1] * 2 || 0;
        tx = -(anchorX * width);
        ty = -(anchorY * height);
      }
      // apply clip shape's RTS
      if (matrix) {
        copy(this.tmpMat4, object.getLocalTransform());
        this.vec3a[0] = tx;
        this.vec3a[1] = ty;
        this.vec3a[2] = 0;
        translate(this.tmpMat4, this.tmpMat4, this.vec3a);
        multiply(this.tmpMat4, matrix, this.tmpMat4);
        multiply(this.tmpMat4, this.vpMatrix, this.tmpMat4);
      } else {
        // apply RTS transformation in world space
        copy(this.tmpMat4, object.getWorldTransform());
        this.vec3a[0] = tx;
        this.vec3a[1] = ty;
        this.vec3a[2] = 0;
        translate(this.tmpMat4, this.tmpMat4, this.vec3a);
        multiply(this.tmpMat4, this.vpMatrix, this.tmpMat4);
      }
      // @see https://developer.mozilla.org/en-US/docs/Web/API/Canvas_API/Tutorial/Transformations
      context.setTransform(this.tmpMat4[0], this.tmpMat4[1], this.tmpMat4[4], this.tmpMat4[5], this.tmpMat4[12], this.tmpMat4[13]);
    };
    _proto.safeMergeAABB = function safeMergeAABB() {
      var merged = new AABB();
      for (var _len = arguments.length, aabbs = new Array(_len), _key = 0; _key < _len; _key++) {
        aabbs[_key] = arguments[_key];
      }
      aabbs.forEach(function (aabb) {
        merged.add(aabb);
      });
      return merged;
    };
    return CanvasRendererPlugin;
  }();
  CanvasRendererPlugin.tag = 'CanvasRenderer';

  var DefaultRenderer = /*#__PURE__*/function () {
    function DefaultRenderer(imagePool) {
      this.imagePool = void 0;
      this.imagePool = imagePool;
    }
    var _proto = DefaultRenderer.prototype;
    _proto.render = function render(context, parsedStyle, object, renderingService) {
      var fill = parsedStyle.fill,
        fillRule = parsedStyle.fillRule,
        opacity = parsedStyle.opacity,
        fillOpacity = parsedStyle.fillOpacity,
        stroke = parsedStyle.stroke,
        strokeOpacity = parsedStyle.strokeOpacity,
        lineWidth = parsedStyle.lineWidth,
        lineCap = parsedStyle.lineCap,
        lineJoin = parsedStyle.lineJoin,
        shadowType = parsedStyle.shadowType,
        shadowColor = parsedStyle.shadowColor,
        shadowBlur = parsedStyle.shadowBlur,
        filter = parsedStyle.filter,
        miterLimit = parsedStyle.miterLimit;
      var hasFill = !isNil(fill) && !fill.isNone;
      var hasStroke = !isNil(stroke) && !stroke.isNone && lineWidth > 0;
      var isFillTransparent = fill.alpha === 0;
      var hasFilter = !!(filter && filter.length);
      var hasShadow = !isNil(shadowColor) && shadowBlur > 0;
      var nodeName = object.nodeName;
      var isInnerShadow = shadowType === 'inner';
      var shouldDrawShadowWithStroke = hasStroke && hasShadow && (nodeName === Shape.PATH || nodeName === Shape.LINE || nodeName === Shape.POLYLINE || isFillTransparent || isInnerShadow);
      if (hasFill) {
        context.globalAlpha = opacity * fillOpacity;
        if (!shouldDrawShadowWithStroke) {
          setShadowAndFilter(object, context, hasShadow);
        }
        this.fill(context, object, fill, fillRule, renderingService);
        if (!shouldDrawShadowWithStroke) {
          this.clearShadowAndFilter(context, hasFilter, hasShadow);
        }
      }
      if (hasStroke) {
        context.globalAlpha = opacity * strokeOpacity;
        context.lineWidth = lineWidth;
        if (!isNil(miterLimit)) {
          context.miterLimit = miterLimit;
        }
        if (!isNil(lineCap)) {
          context.lineCap = lineCap;
        }
        if (!isNil(lineJoin)) {
          context.lineJoin = lineJoin;
        }
        if (shouldDrawShadowWithStroke) {
          if (isInnerShadow) {
            context.globalCompositeOperation = 'source-atop';
          }
          setShadowAndFilter(object, context, true);
          if (isInnerShadow) {
            this.stroke(context, object, stroke, renderingService);
            context.globalCompositeOperation = 'source-over';
            this.clearShadowAndFilter(context, hasFilter, true);
          }
        }
        this.stroke(context, object, stroke, renderingService);
      }
    };
    _proto.clearShadowAndFilter = function clearShadowAndFilter(context, hasFilter, hasShadow) {
      if (hasShadow) {
        context.shadowColor = 'transparent';
        context.shadowBlur = 0;
      }
      if (hasFilter) {
        // save drop-shadow filter
        var oldFilter = context.filter;
        if (!isNil(oldFilter) && oldFilter.indexOf('drop-shadow') > -1) {
          context.filter = oldFilter.replace(/drop-shadow\([^)]*\)/, '').trim() || 'none';
        }
      }
    };
    _proto.fill = function fill(context, object, _fill, fillRule, renderingService) {
      var _this = this;
      if (Array.isArray(_fill)) {
        _fill.forEach(function (gradient) {
          context.fillStyle = _this.getColor(gradient, object, context);
          context.fill(fillRule);
        });
      } else {
        if (isPattern(_fill)) {
          context.fillStyle = this.getPattern(_fill, object, context, renderingService);
        }
        context.fill(fillRule);
      }
    };
    _proto.stroke = function stroke(context, object, _stroke, renderingService) {
      var _this2 = this;
      if (Array.isArray(_stroke)) {
        _stroke.forEach(function (gradient) {
          context.strokeStyle = _this2.getColor(gradient, object, context);
          context.stroke();
        });
      } else {
        if (isPattern(_stroke)) {
          context.strokeStyle = this.getPattern(_stroke, object, context, renderingService);
        }
        context.stroke();
      }
    };
    _proto.getPattern = function getPattern(pattern, object, context, renderingService) {
      var canvasPattern = this.imagePool.getOrCreatePatternSync(pattern, context, function () {
        // set dirty rectangle flag
        object.renderable.dirty = true;
        renderingService.dirtify();
      });
      return canvasPattern;
    };
    _proto.getColor = function getColor(parsedColor, object, context) {
      var color;
      if (parsedColor.type === GradientType.LinearGradient || parsedColor.type === GradientType.RadialGradient) {
        var bounds = object.getGeometryBounds();
        var width = bounds && bounds.halfExtents[0] * 2 || 1;
        var height = bounds && bounds.halfExtents[1] * 2 || 1;
        color = this.imagePool.getOrCreateGradient(_extends$1({
          type: parsedColor.type
        }, parsedColor.value, {
          width: width,
          height: height
        }), context);
      }
      return color;
    };
    return DefaultRenderer;
  }();
  /**
   * apply before fill and stroke but only once
   */
  function setShadowAndFilter(object, context, hasShadow) {
    var _object$parsedStyle = object.parsedStyle,
      filter = _object$parsedStyle.filter,
      shadowColor = _object$parsedStyle.shadowColor,
      shadowBlur = _object$parsedStyle.shadowBlur,
      shadowOffsetX = _object$parsedStyle.shadowOffsetX,
      shadowOffsetY = _object$parsedStyle.shadowOffsetY;
    if (filter && filter.length) {
      // use raw filter string
      context.filter = object.style.filter;
    }
    if (hasShadow) {
      context.shadowColor = shadowColor.toString();
      context.shadowBlur = shadowBlur || 0;
      context.shadowOffsetX = shadowOffsetX || 0;
      context.shadowOffsetY = shadowOffsetY || 0;
    }
  }

  var ImageRenderer = /*#__PURE__*/function () {
    function ImageRenderer(imagePool) {
      this.imagePool = void 0;
      this.imagePool = imagePool;
    }
    var _proto = ImageRenderer.prototype;
    _proto.render = function render(context, parsedStyle, object) {
      var width = parsedStyle.width,
        height = parsedStyle.height,
        img = parsedStyle.img,
        shadowColor = parsedStyle.shadowColor,
        shadowBlur = parsedStyle.shadowBlur;
      var image;
      var iw = width;
      var ih = height;
      if (isString(img)) {
        // image has been loaded in `mounted` hook
        image = this.imagePool.getImageSync(img);
      } else {
        iw || (iw = img.width);
        ih || (ih = img.height);
        image = img;
      }
      if (image) {
        var hasShadow = !isNil(shadowColor) && shadowBlur > 0;
        setShadowAndFilter(object, context, hasShadow);
        // node-canvas will throw the following err:
        // Error: Image given has not completed loading
        try {
          context.drawImage(image, 0, 0, iw, ih);
        } catch (e) {}
      }
    };
    return ImageRenderer;
  }();

  var TextRenderer = /*#__PURE__*/function () {
    function TextRenderer() {}
    var _proto = TextRenderer.prototype;
    _proto.render = function render(context, parsedStyle, object) {
      var lineWidth = parsedStyle.lineWidth,
        textAlign = parsedStyle.textAlign,
        textBaseline = parsedStyle.textBaseline,
        lineJoin = parsedStyle.lineJoin,
        miterLimit = parsedStyle.miterLimit,
        letterSpacing = parsedStyle.letterSpacing,
        stroke = parsedStyle.stroke,
        fill = parsedStyle.fill,
        fillOpacity = parsedStyle.fillOpacity,
        strokeOpacity = parsedStyle.strokeOpacity,
        opacity = parsedStyle.opacity,
        metrics = parsedStyle.metrics,
        dx = parsedStyle.dx,
        dy = parsedStyle.dy,
        shadowColor = parsedStyle.shadowColor,
        shadowBlur = parsedStyle.shadowBlur;
      var font = metrics.font,
        lines = metrics.lines,
        height = metrics.height,
        lineHeight = metrics.lineHeight,
        lineMetrics = metrics.lineMetrics;
      context.font = font;
      context.lineWidth = lineWidth;
      context.textAlign = textAlign === 'middle' ? 'center' : textAlign;
      context.textBaseline = textBaseline;
      context.lineJoin = lineJoin;
      if (!isNil(miterLimit)) {
        context.miterLimit = miterLimit;
      }
      var linePositionY = 0;
      // handle vertical text baseline
      if (textBaseline === 'middle') {
        linePositionY = -height / 2 - lineHeight / 2;
      } else if (textBaseline === 'bottom' || textBaseline === 'alphabetic' || textBaseline === 'ideographic') {
        linePositionY = -height;
      } else if (textBaseline === 'top' || textBaseline === 'hanging') {
        linePositionY = -lineHeight;
      }
      // account for dx & dy
      var offsetX = dx || 0;
      linePositionY += dy || 0;
      var hasShadow = !isNil(shadowColor) && shadowBlur > 0;
      setShadowAndFilter(object, context, hasShadow);
      // draw lines line by line
      for (var i = 0; i < lines.length; i++) {
        var linePositionX = lineWidth / 2 + offsetX;
        linePositionY += lineHeight;
        // no need to re-position X, cause we already set text align
        // @see https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/textAlign
        if (!isNil(stroke) && !stroke.isNone && lineWidth) {
          this.drawLetterSpacing(context, lines[i], lineMetrics[i], textAlign, linePositionX, linePositionY, letterSpacing, fillOpacity, strokeOpacity, opacity, true);
        }
        if (!isNil(fill)) {
          this.drawLetterSpacing(context, lines[i], lineMetrics[i], textAlign, linePositionX, linePositionY, letterSpacing, fillOpacity, strokeOpacity, opacity);
        }
      }
    };
    _proto.drawLetterSpacing = function drawLetterSpacing(context, text, lineMetrics, textAlign, x, y, letterSpacing, fillOpacity, strokeOpacity, opacity, isStroke) {
      if (isStroke === void 0) {
        isStroke = false;
      }
      // letterSpacing of 0 means normal, render all texts directly
      if (letterSpacing === 0) {
        if (isStroke) {
          this.strokeText(context, text, x, y, strokeOpacity);
        } else {
          this.fillText(context, text, x, y, fillOpacity, opacity);
        }
        return;
      }
      // draw text using left align
      var currentTextAlign = context.textAlign;
      context.textAlign = 'left';
      var currentPosition = x;
      if (textAlign === 'center' || textAlign === 'middle') {
        currentPosition = x - lineMetrics.width / 2;
      } else if (textAlign === 'right' || textAlign === 'end') {
        currentPosition = x - lineMetrics.width;
      }
      var stringArray = Array.from(text);
      var previousWidth = context.measureText(text).width;
      var currentWidth = 0;
      for (var i = 0; i < stringArray.length; ++i) {
        var currentChar = stringArray[i];
        if (isStroke) {
          this.strokeText(context, currentChar, currentPosition, y, strokeOpacity);
        } else {
          this.fillText(context, currentChar, currentPosition, y, fillOpacity, opacity);
        }
        currentWidth = context.measureText(text.substring(i + 1)).width;
        currentPosition += previousWidth - currentWidth + letterSpacing;
        previousWidth = currentWidth;
      }
      context.textAlign = currentTextAlign;
    };
    _proto.fillText = function fillText(context, text, x, y, fillOpacity, opacity) {
      var currentGlobalAlpha;
      var applyOpacity = !isNil(fillOpacity) && fillOpacity !== 1;
      if (applyOpacity) {
        currentGlobalAlpha = context.globalAlpha;
        context.globalAlpha = fillOpacity * opacity;
      }
      context.fillText(text, x, y);
      if (applyOpacity) {
        context.globalAlpha = currentGlobalAlpha;
      }
    };
    _proto.strokeText = function strokeText(context, text, x, y, strokeOpacity) {
      var currentGlobalAlpha;
      var applyOpacity = !isNil(strokeOpacity) && strokeOpacity !== 1;
      if (applyOpacity) {
        currentGlobalAlpha = context.globalAlpha;
        context.globalAlpha = strokeOpacity;
      }
      context.strokeText(text, x, y);
      if (applyOpacity) {
        context.globalAlpha = currentGlobalAlpha;
      }
    };
    return TextRenderer;
  }();

  var Plugin$2 = /*#__PURE__*/function (_AbstractRendererPlug) {
    _inheritsLoose$4(Plugin, _AbstractRendererPlug);
    function Plugin(options) {
      var _this;
      if (options === void 0) {
        options = {};
      }
      _this = _AbstractRendererPlug.call(this) || this;
      _this.options = void 0;
      _this.name = 'canvas-renderer';
      _this.options = options;
      return _this;
    }
    var _proto = Plugin.prototype;
    _proto.init = function init() {
      var _defaultStyleRenderer;
      var canvasRendererPluginOptions = _extends$1({
        dirtyObjectNumThreshold: 500,
        dirtyObjectRatioThreshold: 0.8
      }, this.options);
      // @ts-ignore
      var imagePool = this.context.imagePool;
      var defaultRenderer = new DefaultRenderer(imagePool);
      var defaultStyleRendererFactory = (_defaultStyleRenderer = {}, _defaultStyleRenderer[Shape.CIRCLE] = defaultRenderer, _defaultStyleRenderer[Shape.ELLIPSE] = defaultRenderer, _defaultStyleRenderer[Shape.RECT] = defaultRenderer, _defaultStyleRenderer[Shape.IMAGE] = new ImageRenderer(imagePool), _defaultStyleRenderer[Shape.TEXT] = new TextRenderer(), _defaultStyleRenderer[Shape.LINE] = defaultRenderer, _defaultStyleRenderer[Shape.POLYLINE] = defaultRenderer, _defaultStyleRenderer[Shape.POLYGON] = defaultRenderer, _defaultStyleRenderer[Shape.PATH] = defaultRenderer, _defaultStyleRenderer[Shape.GROUP] = undefined, _defaultStyleRenderer[Shape.HTML] = undefined, _defaultStyleRenderer[Shape.MESH] = undefined, _defaultStyleRenderer);
      this.context.defaultStyleRendererFactory = defaultStyleRendererFactory;
      this.context.styleRendererFactory = defaultStyleRendererFactory;
      this.addRenderingPlugin(new CanvasRendererPlugin(canvasRendererPluginOptions));
    };
    _proto.destroy = function destroy() {
      this.removeAllRenderingPlugins();
      delete this.context.defaultStyleRendererFactory;
      delete this.context.styleRendererFactory;
    };
    return Plugin;
  }(AbstractRendererPlugin);

  function _regeneratorRuntime$3() {
    _regeneratorRuntime$3 = function () {
      return exports;
    };
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function (obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function (method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) keys.push(key);
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function (skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
      },
      stop: function () {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function (exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function (type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function (record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function (finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      catch: function (tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function (iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  function asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator$3(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  function _inheritsLoose$5(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$6(subClass, superClass);
  }
  function _setPrototypeOf$6(o, p) {
    _setPrototypeOf$6 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$6(o, p);
  }

  var ImagePool = /*#__PURE__*/function () {
    function ImagePool(canvasConfig) {
      this.canvasConfig = void 0;
      this.imageCache = {};
      this.gradientCache = {};
      this.patternCache = {};
      this.canvasConfig = canvasConfig;
    }
    var _proto = ImagePool.prototype;
    _proto.getImageSync = function getImageSync(src, callback) {
      if (!this.imageCache[src]) {
        this.getOrCreateImage(src).then(function () {
          if (callback) {
            callback();
          }
        });
      } else {
        if (callback) {
          callback();
        }
      }
      return this.imageCache[src];
    };
    _proto.getOrCreateImage = function getOrCreateImage(src) {
      var _this = this;
      if (this.imageCache[src]) {
        return Promise.resolve(this.imageCache[src]);
      }
      // @see https://github.com/antvis/g/issues/938
      var createImage = this.canvasConfig.createImage;
      return new Promise(function (resolve, reject) {
        var image;
        if (createImage) {
          image = createImage(src);
        } else if (isBrowser) {
          image = new window.Image();
        }
        if (image) {
          image.onload = function () {
            resolve(image);
          };
          image.onerror = function (ev) {
            reject(ev);
          };
          image.crossOrigin = 'Anonymous';
          image.src = src;
          _this.imageCache[src] = image;
        }
      });
    };
    _proto.getOrCreatePatternSync = function getOrCreatePatternSync(pattern, context, callback) {
      var patternKey = this.generatePatternKey(pattern);
      if (patternKey && this.patternCache[patternKey]) {
        return this.patternCache[patternKey];
      }
      var image = pattern.image,
        repetition = pattern.repetition;
      var src;
      // Image URL
      if (isString(image)) {
        src = this.getImageSync(image, callback);
      } else {
        src = image;
      }
      // @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/createPattern
      var canvasPattern = src && context.createPattern(src, repetition);
      if (patternKey && canvasPattern) {
        this.patternCache[patternKey] = canvasPattern;
      }
      return canvasPattern;
    };
    _proto.getOrCreateGradient = function getOrCreateGradient(params, context) {
      var key = this.generateGradientKey(params);
      var type = params.type,
        steps = params.steps,
        width = params.width,
        height = params.height,
        angle = params.angle,
        cx = params.cx,
        cy = params.cy,
        size = params.size;
      if (this.gradientCache[key]) {
        return this.gradientCache[key];
      }
      var gradient = null;
      if (type === GradientType.LinearGradient) {
        var _computeLinearGradien = computeLinearGradient(width, height, angle),
          x1 = _computeLinearGradien.x1,
          y1 = _computeLinearGradien.y1,
          x2 = _computeLinearGradien.x2,
          y2 = _computeLinearGradien.y2;
        // @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/createLinearGradient
        gradient = context.createLinearGradient(x1, y1, x2, y2);
      } else if (type === GradientType.RadialGradient) {
        var _computeRadialGradien = computeRadialGradient(width, height, cx, cy, size),
          x = _computeRadialGradien.x,
          y = _computeRadialGradien.y,
          r = _computeRadialGradien.r;
        // @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/createRadialGradient
        gradient = context.createRadialGradient(x, y, 0, x, y, r);
      }
      if (gradient) {
        steps.forEach(function (_ref) {
          var offset = _ref.offset,
            color = _ref.color;
          if (offset.unit === UnitType.kPercentage) {
            var _gradient;
            (_gradient = gradient) === null || _gradient === void 0 ? void 0 : _gradient.addColorStop(offset.value / 100, color.toString());
          }
        });
        this.gradientCache[key] = gradient;
      }
      return this.gradientCache[key];
    };
    _proto.generateGradientKey = function generateGradientKey(params) {
      var type = params.type,
        width = params.width,
        height = params.height,
        steps = params.steps,
        angle = params.angle,
        cx = params.cx,
        cy = params.cy,
        size = params.size;
      return "gradient-" + type + "-" + ((angle === null || angle === void 0 ? void 0 : angle.toString()) || 0) + "-" + ((cx === null || cx === void 0 ? void 0 : cx.toString()) || 0) + "-" + ((cy === null || cy === void 0 ? void 0 : cy.toString()) || 0) + "-" + ((size === null || size === void 0 ? void 0 : size.toString()) || 0) + "-" + width + "-" + height + "-" + steps.map(function (_ref2) {
        var offset = _ref2.offset,
          color = _ref2.color;
        return "" + offset + color;
      }).join('-');
    };
    _proto.generatePatternKey = function generatePatternKey(pattern) {
      var image = pattern.image,
        repetition = pattern.repetition;
      // only generate cache for Image
      if (isString(image)) {
        return "pattern-" + image + "-" + repetition;
      }
    };
    return ImagePool;
  }();

  var LoadImagePlugin = /*#__PURE__*/function () {
    function LoadImagePlugin() {}
    var _proto = LoadImagePlugin.prototype;
    _proto.apply = function apply(context) {
      // @ts-ignore
      var renderingService = context.renderingService,
        renderingContext = context.renderingContext,
        imagePool = context.imagePool;
      var canvas = renderingContext.root.ownerDocument.defaultView;
      var handleMounted = function handleMounted(e) {
        var object = e.target;
        var nodeName = object.nodeName,
          attributes = object.attributes;
        if (nodeName === Shape.IMAGE) {
          var img = attributes.img;
          if (isString(img)) {
            imagePool.getImageSync(img, function () {
              // set dirty rectangle flag
              object.renderable.dirty = true;
              renderingService.dirtify();
            });
          }
        }
      };
      var handleAttributeChanged = function handleAttributeChanged(e) {
        var object = e.target;
        var attrName = e.attrName,
          newValue = e.newValue;
        if (object.nodeName === Shape.IMAGE) {
          if (attrName === 'img') {
            if (isString(newValue)) {
              imagePool.getOrCreateImage(newValue).then(function () {
                // set dirty rectangle flag
                object.renderable.dirty = true;
                renderingService.dirtify();
              });
            }
          }
        }
      };
      renderingService.hooks.init.tapPromise(LoadImagePlugin.tag, /*#__PURE__*/_asyncToGenerator$3( /*#__PURE__*/_regeneratorRuntime$3().mark(function _callee() {
        return _regeneratorRuntime$3().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                canvas.addEventListener(ElementEvent.MOUNTED, handleMounted);
                canvas.addEventListener(ElementEvent.ATTR_MODIFIED, handleAttributeChanged);
              case 2:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      })));
      renderingService.hooks.destroy.tap(LoadImagePlugin.tag, function () {
        canvas.removeEventListener(ElementEvent.MOUNTED, handleMounted);
        canvas.removeEventListener(ElementEvent.ATTR_MODIFIED, handleAttributeChanged);
      });
    };
    return LoadImagePlugin;
  }();
  LoadImagePlugin.tag = 'LoadImage';

  var Plugin$3 = /*#__PURE__*/function (_AbstractRendererPlug) {
    _inheritsLoose$5(Plugin, _AbstractRendererPlug);
    function Plugin() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _AbstractRendererPlug.call.apply(_AbstractRendererPlug, [this].concat(args)) || this;
      _this.name = 'image-loader';
      return _this;
    }
    var _proto = Plugin.prototype;
    _proto.init = function init() {
      // @ts-ignore
      this.context.imagePool = new ImagePool(this.context.config);
      this.addRenderingPlugin(new LoadImagePlugin());
    };
    _proto.destroy = function destroy() {
      this.removeAllRenderingPlugins();
    };
    return Plugin;
  }(AbstractRendererPlugin);

  function _regeneratorRuntime$4() {
    _regeneratorRuntime$4 = function () {
      return exports;
    };
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function (obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function (method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) keys.push(key);
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function (skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
      },
      stop: function () {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function (exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function (type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function (record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function (finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      catch: function (tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function (iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  function asyncGeneratorStep$4(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator$4(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep$4(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep$4(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  function _inheritsLoose$6(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$7(subClass, superClass);
  }
  function _setPrototypeOf$7(o, p) {
    _setPrototypeOf$7 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$7(o, p);
  }

  /**
   * listen to mouse/touch/pointer events on DOM wrapper, trigger pointer events
   */
  var MobileInteractionPlugin = /*#__PURE__*/function () {
    function MobileInteractionPlugin() {}
    var _proto = MobileInteractionPlugin.prototype;
    _proto.apply = function apply(context) {
      var renderingService = context.renderingService,
        contextService = context.contextService;
      // 获取小程序上下文
      var canvasEl = contextService.getDomElement();
      var onPointerDown = function onPointerDown(ev) {
        renderingService.hooks.pointerDown.call(ev);
      };
      var onPointerUp = function onPointerUp(ev) {
        renderingService.hooks.pointerUp.call(ev);
      };
      var onPointerMove = function onPointerMove(ev) {
        // 触发 G 定义的标准 pointerMove 事件
        renderingService.hooks.pointerMove.call(ev);
      };
      var onPointerOver = function onPointerOver(ev) {
        renderingService.hooks.pointerOver.call(ev);
      };
      var onPointerOut = function onPointerOut(ev) {
        renderingService.hooks.pointerOut.call(ev);
      };
      var onPointerCancel = function onPointerCancel(ev) {
        renderingService.hooks.pointerCancel.call(ev);
      };
      renderingService.hooks.init.tapPromise(MobileInteractionPlugin.tag, /*#__PURE__*/_asyncToGenerator$4( /*#__PURE__*/_regeneratorRuntime$4().mark(function _callee() {
        return _regeneratorRuntime$4().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                // 基于小程序上下文的事件监听方式，绑定事件监听，可以参考下面基于 DOM 的方式
                canvasEl.addEventListener('touchstart', onPointerDown, true);
                canvasEl.addEventListener('touchend', onPointerUp, true);
                canvasEl.addEventListener('touchmove', onPointerMove, true);
                canvasEl.addEventListener('touchcancel', onPointerCancel, true);
                // FIXME: 这里不应该只在 canvasEl 上监听 mousemove 和 mouseup，而应该在更高层级的节点上例如 document 监听。
                // 否则无法判断是否移出了 canvasEl
                canvasEl.addEventListener('mousemove', onPointerMove, true);
                canvasEl.addEventListener('mousedown', onPointerDown, true);
                canvasEl.addEventListener('mouseout', onPointerOut, true);
                canvasEl.addEventListener('mouseover', onPointerOver, true);
                canvasEl.addEventListener('mouseup', onPointerUp, true);
              case 9:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      })));
      renderingService.hooks.destroy.tap(MobileInteractionPlugin.tag, function () {
        // 基于小程序上下文的事件监听方式，移除事件监听
        canvasEl.removeEventListener('touchstart', onPointerDown, true);
        canvasEl.removeEventListener('touchend', onPointerUp, true);
        canvasEl.removeEventListener('touchmove', onPointerMove, true);
        canvasEl.removeEventListener('touchcancel', onPointerCancel, true);
        canvasEl.removeEventListener('mousemove', onPointerMove, true);
        canvasEl.removeEventListener('mousedown', onPointerDown, true);
        canvasEl.removeEventListener('mouseout', onPointerOut, true);
        canvasEl.removeEventListener('mouseover', onPointerOver, true);
        canvasEl.removeEventListener('mouseup', onPointerUp, true);
      });
    };
    return MobileInteractionPlugin;
  }();
  MobileInteractionPlugin.tag = 'MobileInteraction';

  var Plugin$4 = /*#__PURE__*/function (_AbstractRendererPlug) {
    _inheritsLoose$6(Plugin, _AbstractRendererPlug);
    function Plugin() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _AbstractRendererPlug.call.apply(_AbstractRendererPlug, [this].concat(args)) || this;
      _this.name = 'mobile-interaction';
      return _this;
    }
    var _proto = Plugin.prototype;
    _proto.init = function init() {
      this.addRenderingPlugin(new MobileInteractionPlugin());
    };
    _proto.destroy = function destroy() {
      this.removeAllRenderingPlugins();
    };
    return Plugin;
  }(AbstractRendererPlugin);

  function _regeneratorRuntime$5() {
    _regeneratorRuntime$5 = function () {
      return exports;
    };
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function (obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function (method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) keys.push(key);
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function (skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
      },
      stop: function () {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function (exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function (type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function (record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function (finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      catch: function (tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function (iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  function asyncGeneratorStep$5(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator$5(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep$5(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep$5(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  function _inheritsLoose$7(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$8(subClass, superClass);
  }
  function _setPrototypeOf$8(o, p) {
    _setPrototypeOf$8 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$8(o, p);
  }

  function _regeneratorRuntime$1$1() {
    _regeneratorRuntime$1$1 = function _regeneratorRuntime() {
      return exports;
    };
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function define(obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function value(method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) {
                if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              }
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) {
        keys.push(key);
      }
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function reset(skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) {
          "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
        }
      },
      stop: function stop() {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function dispatchException(exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function abrupt(type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function complete(record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function finish(finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      catch: function _catch(tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function delegateYield(iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  function asyncGeneratorStep$1$1(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator$1$1(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep$1$1(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep$1$1(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  function _extends$2() {
    _extends$2 = Object.assign ? Object.assign.bind() : function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];
        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }
      return target;
    };
    return _extends$2.apply(this, arguments);
  }
  function _inheritsLoose$1$1(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$1$1(subClass, superClass);
  }
  function _setPrototypeOf$1$1(o, p) {
    _setPrototypeOf$1$1 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$1$1(o, p);
  }
  var DragndropPlugin = /*#__PURE__*/function () {
    function DragndropPlugin(dragndropPluginOptions) {
      this.dragndropPluginOptions = void 0;
      this.dragndropPluginOptions = dragndropPluginOptions;
    }
    var _proto = DragndropPlugin.prototype;
    _proto.apply = function apply(context) {
      var renderingService = context.renderingService,
        renderingContext = context.renderingContext;
      var document = renderingContext.root.ownerDocument;
      // TODO: should we add an option like `draggable` to Canvas
      var canvas = document.defaultView;
      var _this$dragndropPlugin = this.dragndropPluginOptions,
        overlap = _this$dragndropPlugin.overlap,
        isDocumentDraggable = _this$dragndropPlugin.isDocumentDraggable,
        isDocumentDroppable = _this$dragndropPlugin.isDocumentDroppable,
        dragstartDistanceThreshold = _this$dragndropPlugin.dragstartDistanceThreshold,
        dragstartTimeThreshold = _this$dragndropPlugin.dragstartTimeThreshold;
      var handlePointerdown = function handlePointerdown(event) {
        var target = event.target;
        var isDocument = target === document;
        var draggableEventTarget = isDocument && isDocumentDraggable ? document : target.closest && target.closest('[draggable=true]');
        // `draggable` may be set on ancestor nodes:
        // @see https://github.com/antvis/G/issues/1088
        if (!!draggableEventTarget) {
          // @ts-ignore
          var handlePointermove = /*#__PURE__*/function () {
            var _ref = _asyncToGenerator$1$1( /*#__PURE__*/_regeneratorRuntime$1$1().mark(function _callee(event) {
              var timeElapsed, distanceMoved, point, elementsBelow, elementBelow, droppableBelow;
              return _regeneratorRuntime$1$1().wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      if (dragstartTriggered) {
                        _context.next = 8;
                        break;
                      }
                      timeElapsed = event.timeStamp - dragstartTimeStamp;
                      distanceMoved = distanceSquareRoot([event.clientX, event.clientY], dragstartClientCoordinates); // check thresholds
                      if (!(timeElapsed <= dragstartTimeThreshold || distanceMoved <= dragstartDistanceThreshold)) {
                        _context.next = 5;
                        break;
                      }
                      return _context.abrupt("return");
                    case 5:
                      // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/dragstart_event
                      event.type = 'dragstart';
                      draggableEventTarget.dispatchEvent(event);
                      dragstartTriggered = true;
                    case 8:
                      // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/drag_event
                      event.type = 'drag';
                      // @ts-ignore
                      event.dx = event.clientX - lastDragClientCoordinates[0];
                      // @ts-ignore
                      event.dy = event.clientY - lastDragClientCoordinates[1];
                      draggableEventTarget.dispatchEvent(event);
                      lastDragClientCoordinates = [event.clientX, event.clientY];
                      if (isDocument) {
                        _context.next = 21;
                        break;
                      }
                      point = overlap === 'pointer' ? [event.canvasX, event.canvasY] : target.getBounds().center;
                      _context.next = 17;
                      return document.elementsFromPoint(point[0], point[1]);
                    case 17:
                      elementsBelow = _context.sent;
                      // prevent from picking the dragging element
                      elementBelow = elementsBelow[elementsBelow.indexOf(target) + 1];
                      droppableBelow = (elementBelow === null || elementBelow === void 0 ? void 0 : elementBelow.closest('[droppable=true]')) || (isDocumentDroppable ? document : null);
                      if (currentDroppable !== droppableBelow) {
                        if (currentDroppable) {
                          // null when we were not over a droppable before this event
                          // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/dragleave_event
                          event.type = 'dragleave';
                          event.target = currentDroppable;
                          currentDroppable.dispatchEvent(event);
                        }
                        if (droppableBelow) {
                          // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/dragleave_event
                          event.type = 'dragenter';
                          event.target = droppableBelow;
                          droppableBelow.dispatchEvent(event);
                        }
                        currentDroppable = droppableBelow;
                        if (currentDroppable) {
                          // null if we're not coming over a droppable now
                          // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/dragover_event
                          event.type = 'dragover';
                          event.target = currentDroppable;
                          currentDroppable.dispatchEvent(event);
                        }
                      }
                    case 21:
                    case "end":
                      return _context.stop();
                  }
                }
              }, _callee);
            }));
            return function handlePointermove(_x) {
              return _ref.apply(this, arguments);
            };
          }();
          // delay triggering dragstart event
          var dragstartTriggered = false;
          var dragstartTimeStamp = event.timeStamp;
          var dragstartClientCoordinates = [event.clientX, event.clientY];
          var currentDroppable = null;
          var lastDragClientCoordinates = [event.clientX, event.clientY];
          canvas.addEventListener('pointermove', handlePointermove);
          var stopDragging = function stopDragging(originalPointerUpEvent) {
            if (dragstartTriggered) {
              // prevent click event being triggerd
              // @see https://github.com/antvis/G/issues/1091
              originalPointerUpEvent.detail = {
                preventClick: true
              };
              // clone event first
              var _event = originalPointerUpEvent.clone();
              // drop should fire before dragend
              // @see https://javascript.tutorialink.com/is-there-a-defined-ordering-between-dragend-and-drop-events/
              if (currentDroppable) {
                // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/drop_event
                _event.type = 'drop';
                _event.target = currentDroppable;
                currentDroppable.dispatchEvent(_event);
              }
              // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/dragend_event
              _event.type = 'dragend';
              draggableEventTarget.dispatchEvent(_event);
              dragstartTriggered = false;
            }
            canvas.removeEventListener('pointermove', handlePointermove);
          };
          target.addEventListener('pointerup', stopDragging, {
            once: true
          });
          target.addEventListener('pointerupoutside', stopDragging, {
            once: true
          });
        }
      };
      renderingService.hooks.init.tapPromise(DragndropPlugin.tag, /*#__PURE__*/_asyncToGenerator$1$1( /*#__PURE__*/_regeneratorRuntime$1$1().mark(function _callee2() {
        return _regeneratorRuntime$1$1().wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                canvas.addEventListener('pointerdown', handlePointerdown);
              case 1:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })));
      renderingService.hooks.destroy.tap(DragndropPlugin.tag, function () {
        canvas.removeEventListener('pointerdown', handlePointerdown);
      });
    };
    return DragndropPlugin;
  }();
  DragndropPlugin.tag = 'Dragndrop';
  var Plugin$5 = /*#__PURE__*/function (_AbstractRendererPlug) {
    _inheritsLoose$1$1(Plugin, _AbstractRendererPlug);
    function Plugin(options) {
      var _this;
      if (options === void 0) {
        options = {};
      }
      _this = _AbstractRendererPlug.call(this) || this;
      _this.options = void 0;
      _this.name = 'dragndrop';
      _this.options = options;
      return _this;
    }
    var _proto = Plugin.prototype;
    _proto.init = function init() {
      this.addRenderingPlugin(new DragndropPlugin(_extends$2({
        overlap: 'pointer',
        isDocumentDraggable: false,
        isDocumentDroppable: false,
        dragstartDistanceThreshold: 0,
        dragstartTimeThreshold: 0
      }, this.options)));
    };
    _proto.destroy = function destroy() {
      this.removeAllRenderingPlugins();
    };
    return Plugin;
  }(AbstractRendererPlugin);

  function isCanvasElement(el) {
    if (!el || typeof el !== 'object') return false;
    if (el.nodeType === 1 && el.nodeName) {
      // HTMLCanvasElement
      return true;
    }
    // CanvasElement
    return !!el.isCanvasElement;
  }

  var Canvas2DContextService = /*#__PURE__*/function () {
    function Canvas2DContextService(context) {
      this.$canvas = void 0;
      this.dpr = void 0;
      this.context = void 0;
      this.canvasConfig = void 0;
      this.canvasConfig = context.config;
    }
    var _proto = Canvas2DContextService.prototype;
    _proto.init = /*#__PURE__*/function () {
      var _init = _asyncToGenerator$5( /*#__PURE__*/_regeneratorRuntime$5().mark(function _callee() {
        var _this$canvasConfig, canvas, devicePixelRatio, dpr;
        return _regeneratorRuntime$5().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _this$canvasConfig = this.canvasConfig, canvas = _this$canvasConfig.canvas, devicePixelRatio = _this$canvasConfig.devicePixelRatio;
                this.$canvas = canvas;
                // 实际获取到小程序环境的上下文
                this.context = this.$canvas.getContext('2d');
                // use user-defined dpr first
                dpr = devicePixelRatio || 1;
                dpr = dpr >= 1 ? Math.ceil(dpr) : 1;
                this.dpr = dpr;
                this.resize(this.canvasConfig.width, this.canvasConfig.height);
              case 7:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
      function init() {
        return _init.apply(this, arguments);
      }
      return init;
    }();
    _proto.getContext = function getContext() {
      return this.context;
    };
    _proto.getDomElement = function getDomElement() {
      return this.$canvas;
    };
    _proto.getDPR = function getDPR() {
      return this.dpr;
    };
    _proto.getBoundingClientRect = function getBoundingClientRect() {
      if (this.$canvas.getBoundingClientRect) {
        return this.$canvas.getBoundingClientRect();
      }
    };
    _proto.destroy = function destroy() {
      // TODO: 小程序环境销毁 context
      this.context = null;
      this.$canvas = null;
    };
    _proto.resize = function resize(width, height) {
      var devicePixelRatio = this.canvasConfig.devicePixelRatio;
      var pixelRatio = devicePixelRatio;
      var canvasDOM = this.$canvas; // HTMLCanvasElement or canvasElement
      // 浏览器环境设置style样式
      if (canvasDOM.style) {
        canvasDOM.style.width = width + 'px';
        canvasDOM.style.height = height + 'px';
      }
      if (isCanvasElement(canvasDOM)) {
        canvasDOM.width = width * pixelRatio;
        canvasDOM.height = height * pixelRatio;
        if (pixelRatio !== 1) {
          this.context.scale(pixelRatio, pixelRatio);
        }
      }
    };
    _proto.applyCursorStyle = function applyCursorStyle(cursor) {
      // 小程序环境无需设置鼠标样式
    };
    _proto.toDataURL = /*#__PURE__*/function () {
      var _toDataURL = _asyncToGenerator$5( /*#__PURE__*/_regeneratorRuntime$5().mark(function _callee2(options) {
        var type, encoderOptions;
        return _regeneratorRuntime$5().wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                type = options.type, encoderOptions = options.encoderOptions;
                return _context2.abrupt("return", this.context.canvas.toDataURL(type, encoderOptions));
              case 2:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
      function toDataURL(_x) {
        return _toDataURL.apply(this, arguments);
      }
      return toDataURL;
    }();
    return Canvas2DContextService;
  }();

  var ContextRegisterPlugin = /*#__PURE__*/function (_AbstractRendererPlug) {
    _inheritsLoose$7(ContextRegisterPlugin, _AbstractRendererPlug);
    function ContextRegisterPlugin() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _AbstractRendererPlug.call.apply(_AbstractRendererPlug, [this].concat(args)) || this;
      _this.name = 'mobile-canvas-context-register';
      return _this;
    }
    var _proto = ContextRegisterPlugin.prototype;
    _proto.init = function init() {
      this.context.ContextService = Canvas2DContextService;
    };
    _proto.destroy = function destroy() {
      delete this.context.ContextService;
    };
    return ContextRegisterPlugin;
  }(AbstractRendererPlugin);

  var Renderer = /*#__PURE__*/function (_AbstractRenderer) {
    _inheritsLoose$7(Renderer, _AbstractRenderer);
    function Renderer(config) {
      var _this;
      _this = _AbstractRenderer.call(this, config) || this;
      // register Canvas2DContext
      _this.registerPlugin(new ContextRegisterPlugin());
      _this.registerPlugin(new Plugin$3());
      _this.registerPlugin(new Plugin());
      // enable rendering with Canvas2D API
      _this.registerPlugin(new Plugin$2());
      _this.registerPlugin(new Plugin$4());
      // enable picking with Canvas2D API
      _this.registerPlugin(new Plugin$1());
      _this.registerPlugin(new Plugin$5({
        isDocumentDraggable: isNil(config === null || config === void 0 ? void 0 : config.isDocumentDraggable) ? true : config.isDocumentDraggable,
        isDocumentDroppable: isNil(config === null || config === void 0 ? void 0 : config.isDocumentDroppable) ? true : config.isDocumentDroppable,
        dragstartDistanceThreshold: isNil(config === null || config === void 0 ? void 0 : config.dragstartDistanceThreshold) ? 10 : config.dragstartDistanceThreshold,
        dragstartTimeThreshold: isNil(config === null || config === void 0 ? void 0 : config.dragstartTimeThreshold) ? 50 : config.dragstartTimeThreshold
      }));
      return _this;
    }
    return Renderer;
  }(AbstractRenderer);

  function _defineProperties$2(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }
  function _createClass$2(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties$2(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties$2(Constructor, staticProps);
    Object.defineProperty(Constructor, "prototype", {
      writable: false
    });
    return Constructor;
  }
  function _inheritsLoose$8(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$9(subClass, superClass);
  }
  function _setPrototypeOf$9(o, p) {
    _setPrototypeOf$9 = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$9(o, p);
  }

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/API/AnimationPlaybackEvent
   */
  var AnimationEvent = /*#__PURE__*/function (_FederatedEvent) {
    _inheritsLoose$8(AnimationEvent, _FederatedEvent);
    function AnimationEvent(manager, target, currentTime, timelineTime) {
      var _this;
      _this = _FederatedEvent.call(this, manager) || this;
      _this.currentTime = void 0;
      _this.timelineTime = void 0;
      _this.currentTime = currentTime;
      _this.timelineTime = timelineTime;
      // @ts-ignore
      _this.target = target;
      _this.type = 'finish';
      _this.bubbles = false;
      // @ts-ignore
      _this.currentTarget = target;
      _this.defaultPrevented = false;
      _this.eventPhase = _this.AT_TARGET;
      _this.timeStamp = Date.now();
      _this.currentTime = currentTime;
      _this.timelineTime = timelineTime;
      return _this;
    }
    return AnimationEvent;
  }(FederatedEvent);

  var sequenceNumber = 0;
  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/Animation
   */
  var Animation = /*#__PURE__*/function () {
    function Animation(effect, timeline) {
      var _this$effect;
      this.effect = void 0;
      this.timeline = void 0;
      this.id = void 0;
      this.currentTimePending = false;
      this._idle = true;
      this._paused = false;
      this._finishedFlag = true;
      this.oldPlayState = void 0;
      this._holdTime = void 0;
      this.readyPromise = void 0;
      this.finishedPromise = void 0;
      this.resolveReadyPromise = void 0;
      this.rejectReadyPromise = void 0;
      this.resolveFinishedPromise = void 0;
      this.rejectFinishedPromise = void 0;
      this.onfinish = void 0;
      this.oncancel = void 0;
      this.onframe = void 0;
      this._currentTime = 0;
      this._startTime = void 0;
      this._playbackRate = 1;
      this._totalDuration = void 0;
      this._inEffect = void 0;
      this._inTimeline = true;
      this.onremove = void 0;
      this.effect = effect;
      effect.animation = this;
      this.timeline = timeline;
      this.id = "" + sequenceNumber++;
      this._inEffect = !!this.effect.update(0);
      this._totalDuration = Number((_this$effect = this.effect) === null || _this$effect === void 0 ? void 0 : _this$effect.getComputedTiming().endTime);
      this._holdTime = 0;
      this._paused = false;
      this.oldPlayState = 'idle';
      this.updatePromises();
    }
    /**
     * state machine,
     * resolve/reject ready/finished Promise according to current state
     */
    var _proto = Animation.prototype;
    _proto.updatePromises = function updatePromises() {
      var oldPlayState = this.oldPlayState;
      var newPlayState = this.pending ? 'pending' : this.playState;
      if (this.readyPromise && newPlayState !== oldPlayState) {
        if (newPlayState === 'idle') {
          this.rejectReadyPromise();
          this.readyPromise = undefined;
        } else if (oldPlayState === 'pending') {
          this.resolveReadyPromise();
        } else if (newPlayState === 'pending') {
          this.readyPromise = undefined;
        }
      }
      if (this.finishedPromise && newPlayState !== oldPlayState) {
        if (newPlayState === 'idle') {
          this.rejectFinishedPromise();
          this.finishedPromise = undefined;
        } else if (newPlayState === 'finished') {
          this.resolveFinishedPromise();
        } else if (oldPlayState === 'finished') {
          this.finishedPromise = undefined;
        }
      }
      this.oldPlayState = newPlayState;
      return this.readyPromise || this.finishedPromise;
    };
    _proto.play = function play() {
      this.updatePromises();
      this._paused = false;
      if (this._isFinished || this._idle) {
        this.rewind();
        this._startTime = null;
      }
      this._finishedFlag = false;
      this._idle = false;
      this.ensureAlive();
      this.timeline.applyDirtiedAnimation(this);
      if (this.timeline.animations.indexOf(this) === -1) {
        this.timeline.animations.push(this);
      }
      this.updatePromises();
    };
    _proto.pause = function pause() {
      this.updatePromises();
      if (this.currentTime) {
        this._holdTime = this.currentTime;
      }
      if (!this._isFinished && !this._paused && !this._idle) {
        this.currentTimePending = true;
      } else if (this._idle) {
        this.rewind();
        this._idle = false;
      }
      this._startTime = null;
      this._paused = true;
      this.updatePromises();
    };
    _proto.finish = function finish() {
      this.updatePromises();
      if (this._idle) return;
      this.currentTime = this._playbackRate > 0 ? this._totalDuration : 0;
      this._startTime = this._totalDuration - this.currentTime;
      this.currentTimePending = false;
      this.timeline.applyDirtiedAnimation(this);
      this.updatePromises();
    };
    _proto.cancel = function cancel() {
      this.updatePromises();
      if (!this._inEffect) return;
      this._inEffect = false;
      this._idle = true;
      this._paused = false;
      this._finishedFlag = true;
      this._currentTime = 0;
      this._startTime = null;
      this.effect.update(null);
      // effects are invalid after cancellation as the animation state
      // needs to un-apply.
      this.timeline.applyDirtiedAnimation(this);
      this.updatePromises();
    };
    _proto.reverse = function reverse() {
      this.updatePromises();
      var oldCurrentTime = this.currentTime;
      this.playbackRate *= -1;
      this.play();
      if (oldCurrentTime !== null) {
        this.currentTime = oldCurrentTime;
      }
      this.updatePromises();
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/updatePlaybackRate
     */;
    _proto.updatePlaybackRate = function updatePlaybackRate(playbackRate) {
      this.playbackRate = playbackRate;
    };
    _proto.targetAnimations = function targetAnimations() {
      var _this$effect2;
      var target = (_this$effect2 = this.effect) === null || _this$effect2 === void 0 ? void 0 : _this$effect2.target;
      return target.getAnimations();
    };
    _proto.markTarget = function markTarget() {
      var animations = this.targetAnimations();
      if (animations.indexOf(this) === -1) {
        animations.push(this);
      }
    };
    _proto.unmarkTarget = function unmarkTarget() {
      var animations = this.targetAnimations();
      var index = animations.indexOf(this);
      if (index !== -1) {
        animations.splice(index, 1);
      }
    };
    _proto.tick = function tick(timelineTime, isAnimationFrame) {
      if (!this._idle && !this._paused) {
        if (this._startTime === null) {
          if (isAnimationFrame) {
            this.startTime = timelineTime - this._currentTime / this.playbackRate;
          }
        } else if (!this._isFinished) {
          this.tickCurrentTime((timelineTime - this._startTime) * this.playbackRate);
        }
      }
      if (isAnimationFrame) {
        this.currentTimePending = false;
        this.fireEvents(timelineTime);
      }
    };
    _proto.rewind = function rewind() {
      if (this.playbackRate >= 0) {
        this.currentTime = 0;
      } else if (this._totalDuration < Infinity) {
        this.currentTime = this._totalDuration;
      } else {
        throw new Error('Unable to rewind negative playback rate animation with infinite duration');
      }
    };
    _proto.persist = function persist() {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.addEventListener = function addEventListener(type, listener, options) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.removeEventListener = function removeEventListener(type, listener, options) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.dispatchEvent = function dispatchEvent(event) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    // replaceState: AnimationReplaceState;
    _proto.commitStyles = function commitStyles() {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED);
    };
    _proto.ensureAlive = function ensureAlive() {
      // If an animation is playing backwards and is not fill backwards/both
      // then it should go out of effect when it reaches the start of its
      // active interval (currentTime === 0).
      if (this.playbackRate < 0 && this.currentTime === 0) {
        var _this$effect3;
        this._inEffect = !!((_this$effect3 = this.effect) === null || _this$effect3 === void 0 ? void 0 : _this$effect3.update(-1));
      } else {
        var _this$effect4;
        this._inEffect = !!((_this$effect4 = this.effect) === null || _this$effect4 === void 0 ? void 0 : _this$effect4.update(this.currentTime));
      }
      if (!this._inTimeline && (this._inEffect || !this._finishedFlag)) {
        this._inTimeline = true;
        this.timeline.animations.push(this);
      }
    };
    _proto.tickCurrentTime = function tickCurrentTime(newTime, ignoreLimit) {
      if (newTime !== this._currentTime) {
        this._currentTime = newTime;
        if (this._isFinished && !ignoreLimit) {
          this._currentTime = this._playbackRate > 0 ? this._totalDuration : 0;
        }
        this.ensureAlive();
      }
    };
    _proto.fireEvents = function fireEvents(baseTime) {
      var _this = this;
      if (this._isFinished) {
        if (!this._finishedFlag) {
          if (this.onfinish) {
            var event = new AnimationEvent(null, this, this.currentTime, baseTime);
            setTimeout(function () {
              if (_this.onfinish) {
                _this.onfinish(event);
              }
            });
          }
          this._finishedFlag = true;
        }
      } else {
        if (this.onframe && this.playState === 'running') {
          var _event = new AnimationEvent(null, this, this.currentTime, baseTime);
          this.onframe(_event);
        }
        this._finishedFlag = false;
      }
    };
    _createClass$2(Animation, [{
      key: "pending",
      get:
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/effect
       */

      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/timeline
       */

      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/id
       */

      // animation: InternalAnimation | null;
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/pending
       */
      function get() {
        return this._startTime === null && !this._paused && this.playbackRate !== 0 || this.currentTimePending;
      }
    }, {
      key: "playState",
      get: function get() {
        if (this._idle) return 'idle';
        if (this._isFinished) return 'finished';
        if (this._paused) return 'paused';
        return 'running';
      }
      /**
       * record previos state
       */
    }, {
      key: "ready",
      get:
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/ready
       * @example
        animation.pause();
        animation.ready.then(function() {
          // Displays 'running'
          alert(animation.playState);
        });
        animation.play();
       */
      function get() {
        var _this2 = this;
        if (!this.readyPromise) {
          if (this.timeline.animationsWithPromises.indexOf(this) === -1) {
            this.timeline.animationsWithPromises.push(this);
          }
          this.readyPromise = new Promise(function (resolve, reject) {
            _this2.resolveReadyPromise = function () {
              resolve(_this2);
            };
            _this2.rejectReadyPromise = function () {
              reject(new Error());
            };
          });
          if (!this.pending) {
            this.resolveReadyPromise();
          }
        }
        return this.readyPromise;
      }
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/finished
       * @example
        Promise.all(
          elem.getAnimations().map(
            function(animation) {
              return animation.finished
            }
          )
        ).then(
          function() {
            return elem.remove();
          }
        );
       */
    }, {
      key: "finished",
      get: function get() {
        var _this3 = this;
        if (!this.finishedPromise) {
          if (this.timeline.animationsWithPromises.indexOf(this) === -1) {
            this.timeline.animationsWithPromises.push(this);
          }
          this.finishedPromise = new Promise(function (resolve, reject) {
            _this3.resolveFinishedPromise = function () {
              resolve(_this3);
            };
            _this3.rejectFinishedPromise = function () {
              reject(new Error());
            };
          });
          if (this.playState === 'finished') {
            this.resolveFinishedPromise();
          }
        }
        return this.finishedPromise;
      }
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/onfinish
       */
    }, {
      key: "currentTime",
      get:
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/currentTime
       */
      function get() {
        this.updatePromises();
        return this._idle || this.currentTimePending ? null : this._currentTime;
      },
      set: function set(newTime) {
        newTime = Number(newTime);
        if (isNaN(newTime)) return;
        this.timeline.restart();
        if (!this._paused && this._startTime !== null) {
          var _this$timeline;
          this._startTime = Number((_this$timeline = this.timeline) === null || _this$timeline === void 0 ? void 0 : _this$timeline.currentTime) - newTime / this.playbackRate;
        }
        this.currentTimePending = false;
        if (this._currentTime === newTime) {
          return;
        }
        if (this._idle) {
          this._idle = false;
          this._paused = true;
        }
        this.tickCurrentTime(newTime, true);
        this.timeline.applyDirtiedAnimation(this);
      }
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Animation/startTime
       */
    }, {
      key: "startTime",
      get: function get() {
        return this._startTime;
      },
      set: function set(newTime) {
        if (newTime !== null) {
          this.updatePromises();
          newTime = Number(newTime);
          if (isNaN(newTime)) return;
          if (this._paused || this._idle) return;
          this._startTime = newTime;
          this.tickCurrentTime((Number(this.timeline.currentTime) - this._startTime) * this.playbackRate);
          this.timeline.applyDirtiedAnimation(this);
          this.updatePromises();
        }
      }
    }, {
      key: "playbackRate",
      get: function get() {
        return this._playbackRate;
      },
      set: function set(value) {
        if (value === this._playbackRate) {
          return;
        }
        this.updatePromises();
        var oldCurrentTime = this.currentTime;
        this._playbackRate = value;
        this.startTime = null;
        if (this.playState !== 'paused' && this.playState !== 'idle') {
          this._finishedFlag = false;
          this._idle = false;
          this.ensureAlive();
          this.timeline.applyDirtiedAnimation(this);
        }
        if (oldCurrentTime !== null) {
          this.currentTime = oldCurrentTime;
        }
        this.updatePromises();
      }
    }, {
      key: "_isFinished",
      get: function get() {
        return !this._idle && (this._playbackRate > 0 && Number(this._currentTime) >= this._totalDuration || this._playbackRate < 0 && Number(this._currentTime) <= 0);
      }
    }, {
      key: "totalDuration",
      get: function get() {
        return this._totalDuration;
      }
    }, {
      key: "_needsTick",
      get: function get() {
        return this.pending || this.playState === 'running' || !this._finishedFlag;
      }
    }]);
    return Animation;
  }();

  /**
   * https://github.com/gre/bezier-easing
   * BezierEasing - use bezier curve for transition easing function
   * by Gaëtan Renaudeau 2014 - 2015 – MIT License
   */
  // These values are established by empiricism with tests (tradeoff: performance VS precision)
  var NEWTON_ITERATIONS = 4;
  var NEWTON_MIN_SLOPE = 0.001;
  var SUBDIVISION_PRECISION = 0.0000001;
  var SUBDIVISION_MAX_ITERATIONS = 10;
  var kSplineTableSize = 11;
  var kSampleStepSize = 1.0 / (kSplineTableSize - 1.0);
  var float32ArraySupported = typeof Float32Array === 'function';
  var A = function A(aA1, aA2) {
    return 1.0 - 3.0 * aA2 + 3.0 * aA1;
  };
  var B = function B(aA1, aA2) {
    return 3.0 * aA2 - 6.0 * aA1;
  };
  var C = function C(aA1) {
    return 3.0 * aA1;
  };
  // Returns x(t) given t, x1, and x2, or y(t) given t, y1, and y2.
  var calcBezier = function calcBezier(aT, aA1, aA2) {
    return ((A(aA1, aA2) * aT + B(aA1, aA2)) * aT + C(aA1)) * aT;
  };
  // Returns dx/dt given t, x1, and x2, or dy/dt given t, y1, and y2.
  var getSlope = function getSlope(aT, aA1, aA2) {
    return 3.0 * A(aA1, aA2) * aT * aT + 2.0 * B(aA1, aA2) * aT + C(aA1);
  };
  var binarySubdivide = function binarySubdivide(aX, aA, aB, mX1, mX2) {
    var currentX,
      currentT,
      i = 0;
    do {
      currentT = aA + (aB - aA) / 2.0;
      currentX = calcBezier(currentT, mX1, mX2) - aX;
      if (currentX > 0.0) aB = currentT;else aA = currentT;
    } while (Math.abs(currentX) > SUBDIVISION_PRECISION && ++i < SUBDIVISION_MAX_ITERATIONS);
    return currentT;
  };
  var newtonRaphsonIterate = function newtonRaphsonIterate(aX, aGuessT, mX1, mX2) {
    for (var i = 0; i < NEWTON_ITERATIONS; ++i) {
      var currentSlope = getSlope(aGuessT, mX1, mX2);
      if (currentSlope === 0.0) return aGuessT;
      var currentX = calcBezier(aGuessT, mX1, mX2) - aX;
      aGuessT -= currentX / currentSlope;
    }
    return aGuessT;
  };
  var bezier = function bezier(mX1, mY1, mX2, mY2) {
    if (!(0 <= mX1 && mX1 <= 1 && 0 <= mX2 && mX2 <= 1)) throw new Error('bezier x values must be in [0, 1] range');
    if (mX1 === mY1 && mX2 === mY2) return function (t) {
      return t;
    };
    // Precompute samples table
    var sampleValues = float32ArraySupported ? new Float32Array(kSplineTableSize) : new Array(kSplineTableSize);
    for (var i = 0; i < kSplineTableSize; ++i) {
      sampleValues[i] = calcBezier(i * kSampleStepSize, mX1, mX2);
    }
    var getTForX = function getTForX(aX) {
      var intervalStart = 0.0;
      var currentSample = 1;
      var lastSample = kSplineTableSize - 1;
      for (; currentSample !== lastSample && sampleValues[currentSample] <= aX; ++currentSample) {
        intervalStart += kSampleStepSize;
      }
      --currentSample;
      // Interpolate to provide an initial guess for t
      var dist = (aX - sampleValues[currentSample]) / (sampleValues[currentSample + 1] - sampleValues[currentSample]);
      var guessForT = intervalStart + dist * kSampleStepSize;
      var initialSlope = getSlope(guessForT, mX1, mX2);
      if (initialSlope >= NEWTON_MIN_SLOPE) return newtonRaphsonIterate(aX, guessForT, mX1, mX2);else if (initialSlope === 0.0) return guessForT;else {
        return binarySubdivide(aX, intervalStart, intervalStart + kSampleStepSize, mX1, mX2);
      }
    };
    return function (t) {
      // Because JavaScript number are imprecise, we should guarantee the extremes are right.
      if (t === 0 || t === 1) return t;
      return calcBezier(getTForX(t), mY1, mY2);
    };
  };

  var convertToDash = function convertToDash(str) {
    str = str.replace(/([A-Z])/g, function (letter) {
      return "-" + letter.toLowerCase();
    });
    // Remove first dash
    return str.charAt(0) === '-' ? str.substring(1) : str;
  };
  /**
    Easing Functions from anime.js, they are tried and true, so, its better to use them instead of other alternatives
  */
  var Quad = function Quad(t) {
    return Math.pow(t, 2);
  };
  var Cubic = function Cubic(t) {
    return Math.pow(t, 3);
  };
  var Quart = function Quart(t) {
    return Math.pow(t, 4);
  };
  var Quint = function Quint(t) {
    return Math.pow(t, 5);
  };
  var Expo = function Expo(t) {
    return Math.pow(t, 6);
  };
  var Sine = function Sine(t) {
    return 1 - Math.cos(t * Math.PI / 2);
  };
  var Circ = function Circ(t) {
    return 1 - Math.sqrt(1 - t * t);
  };
  var Back = function Back(t) {
    return t * t * (3 * t - 2);
  };
  var Bounce = function Bounce(t) {
    var pow2,
      b = 4;
    while (t < ((pow2 = Math.pow(2, --b)) - 1) / 11) {}
    return 1 / Math.pow(4, 3 - b) - 7.5625 * Math.pow((pow2 * 3 - 2) / 22 - t, 2);
  };
  var Elastic = function Elastic(t, params) {
    if (params === void 0) {
      params = [];
    }
    var _params = params,
      _params$ = _params[0],
      amplitude = _params$ === void 0 ? 1 : _params$,
      _params$2 = _params[1],
      period = _params$2 === void 0 ? 0.5 : _params$2;
    var a = clamp(Number(amplitude), 1, 10);
    var p = clamp(Number(period), 0.1, 2);
    if (t === 0 || t === 1) return t;
    return -a * Math.pow(2, 10 * (t - 1)) * Math.sin((t - 1 - p / (Math.PI * 2) * Math.asin(1 / a)) * (Math.PI * 2) / p);
  };
  var Spring = function Spring(t, params, duration) {
    if (params === void 0) {
      params = [];
    }
    var _params2 = params,
      _params2$ = _params2[0],
      mass = _params2$ === void 0 ? 1 : _params2$,
      _params2$2 = _params2[1],
      stiffness = _params2$2 === void 0 ? 100 : _params2$2,
      _params2$3 = _params2[2],
      damping = _params2$3 === void 0 ? 10 : _params2$3,
      _params2$4 = _params2[3],
      velocity = _params2$4 === void 0 ? 0 : _params2$4;
    mass = clamp(mass, 0.1, 1000);
    stiffness = clamp(stiffness, 0.1, 1000);
    damping = clamp(damping, 0.1, 1000);
    velocity = clamp(velocity, 0.1, 1000);
    var w0 = Math.sqrt(stiffness / mass);
    var zeta = damping / (2 * Math.sqrt(stiffness * mass));
    var wd = zeta < 1 ? w0 * Math.sqrt(1 - zeta * zeta) : 0;
    var a = 1;
    var b = zeta < 1 ? (zeta * w0 + -velocity) / wd : -velocity + w0;
    var progress = duration ? duration * t / 1000 : t;
    if (zeta < 1) {
      progress = Math.exp(-progress * zeta * w0) * (a * Math.cos(wd * progress) + b * Math.sin(wd * progress));
    } else {
      progress = (a + b * progress) * Math.exp(-progress * w0);
    }
    if (t === 0 || t === 1) return t;
    return 1 - progress;
  };
  /**
   * Cache the durations at set easing parameters
   */
  // export const EasingDurationCache: Map<string | TypeEasingFunction, number> = new Map();
  /**
   * The threshold for an infinite loop
   */
  // const INTINITE_LOOP_LIMIT = 10000;
  /** Convert easing parameters to Array of numbers, e.g. "spring(2, 500)" to [2, 500] */
  // export const parseEasingParameters = (str: string) => {
  //   const match = /(\(|\s)([^)]+)\)?/.exec(str);
  //   return match
  //     ? match[2].split(',').map((value) => {
  //         const num = parseFloat(value);
  //         return !Number.isNaN(num) ? num : value.trim();
  //       })
  //     : [];
  // };
  /**
   * The spring easing function will only look smooth at certain durations, with certain parameters.
   * This functions returns the optimal duration to create a smooth springy animation based on physics
   *
   * Note: it can also be used to determine the optimal duration of other types of easing function, but be careful of 'in-'
   * easing functions, because of the nature of the function it can sometimes create an infinite loop, I suggest only using
   * `getEasingDuration` for `spring`, specifically 'out-spring' and 'spring'
   */
  // export const getEasingDuration = (easing: string | TypeEasingFunction = 'spring') => {
  //   if (EasingDurationCache.has(easing)) return EasingDurationCache.get(easing);
  //   // eslint-disable-next-line @typescript-eslint/no-use-before-define
  //   const easingFunction = typeof easing == 'function' ? easing : getEasingFunction(easing as string);
  //   const params = typeof easing == 'function' ? [] : parseEasingParameters(easing);
  //   const frame = 1 / 6;
  //   let elapsed = 0;
  //   let rest = 0;
  //   let count = 0;
  //   while (++count < INTINITE_LOOP_LIMIT) {
  //     elapsed += frame;
  //     if (easingFunction(elapsed, params, undefined) === 1) {
  //       rest++;
  //       if (rest >= 16) break;
  //     } else {
  //       rest = 0;
  //     }
  //   }
  //   const duration = elapsed * frame * 1000;
  //   EasingDurationCache.set(easing, duration);
  //   return duration;
  // };
  /**
    These Easing Functions are based off of the Sozi Project's easing functions
    https://github.com/sozi-projects/Sozi/blob/d72e44ebd580dc7579d1e177406ad41e632f961d/src/js/player/Timing.js
  */
  var Steps = function Steps(t, params) {
    if (params === void 0) {
      params = [];
    }
    var _params3 = params,
      _params3$ = _params3[0],
      steps = _params3$ === void 0 ? 10 : _params3$,
      type = _params3[1];
    var trunc = type == 'start' ? Math.ceil : Math.floor;
    return trunc(clamp(t, 0, 1) * steps) / steps;
  };
  // @ts-ignore
  var Bezier = function Bezier(t, params) {
    if (params === void 0) {
      params = [];
    }
    var _params4 = params,
      mX1 = _params4[0],
      mY1 = _params4[1],
      mX2 = _params4[2],
      mY2 = _params4[3];
    return bezier(mX1, mY1, mX2, mY2)(t);
  };
  /** The default `ease-in` easing function */
  var easein = bezier(0.42, 0.0, 1.0, 1.0);
  /** Converts easing functions to their `out`counter parts */
  var EaseOut = function EaseOut(ease) {
    return function (t, params, duration) {
      if (params === void 0) {
        params = [];
      }
      return 1 - ease(1 - t, params, duration);
    };
  };
  /** Converts easing functions to their `in-out` counter parts */
  var EaseInOut = function EaseInOut(ease) {
    return function (t, params, duration) {
      if (params === void 0) {
        params = [];
      }
      return t < 0.5 ? ease(t * 2, params, duration) / 2 : 1 - ease(t * -2 + 2, params, duration) / 2;
    };
  };
  /** Converts easing functions to their `out-in` counter parts */
  var EaseOutIn = function EaseOutIn(ease) {
    return function (t, params, duration) {
      if (params === void 0) {
        params = [];
      }
      return t < 0.5 ? (1 - ease(1 - t * 2, params, duration)) / 2 : (ease(t * 2 - 1, params, duration) + 1) / 2;
    };
  };
  var EasingFunctions = {
    steps: Steps,
    'step-start': function stepStart(t) {
      return Steps(t, [1, 'start']);
    },
    'step-end': function stepEnd(t) {
      return Steps(t, [1, 'end']);
    },
    linear: function linear(t) {
      return t;
    },
    'cubic-bezier': Bezier,
    ease: function ease(t) {
      return Bezier(t, [0.25, 0.1, 0.25, 1.0]);
    },
    in: easein,
    out: EaseOut(easein),
    'in-out': EaseInOut(easein),
    'out-in': EaseOutIn(easein),
    'in-quad': Quad,
    'out-quad': EaseOut(Quad),
    'in-out-quad': EaseInOut(Quad),
    'out-in-quad': EaseOutIn(Quad),
    'in-cubic': Cubic,
    'out-cubic': EaseOut(Cubic),
    'in-out-cubic': EaseInOut(Cubic),
    'out-in-cubic': EaseOutIn(Cubic),
    'in-quart': Quart,
    'out-quart': EaseOut(Quart),
    'in-out-quart': EaseInOut(Quart),
    'out-in-quart': EaseOutIn(Quart),
    'in-quint': Quint,
    'out-quint': EaseOut(Quint),
    'in-out-quint': EaseInOut(Quint),
    'out-in-quint': EaseOutIn(Quint),
    'in-expo': Expo,
    'out-expo': EaseOut(Expo),
    'in-out-expo': EaseInOut(Expo),
    'out-in-expo': EaseOutIn(Expo),
    'in-sine': Sine,
    'out-sine': EaseOut(Sine),
    'in-out-sine': EaseInOut(Sine),
    'out-in-sine': EaseOutIn(Sine),
    'in-circ': Circ,
    'out-circ': EaseOut(Circ),
    'in-out-circ': EaseInOut(Circ),
    'out-in-circ': EaseOutIn(Circ),
    'in-back': Back,
    'out-back': EaseOut(Back),
    'in-out-back': EaseInOut(Back),
    'out-in-back': EaseOutIn(Back),
    'in-bounce': Bounce,
    'out-bounce': EaseOut(Bounce),
    'in-out-bounce': EaseInOut(Bounce),
    'out-in-bounce': EaseOutIn(Bounce),
    'in-elastic': Elastic,
    'out-elastic': EaseOut(Elastic),
    'in-out-elastic': EaseInOut(Elastic),
    'out-in-elastic': EaseOutIn(Elastic),
    spring: Spring,
    'spring-in': Spring,
    'spring-out': EaseOut(Spring),
    'spring-in-out': EaseInOut(Spring),
    'spring-out-in': EaseOutIn(Spring)
  };
  /**
   * Convert string easing to their proper form
   */
  var complexEasingSyntax = function complexEasingSyntax(ease) {
    return convertToDash(ease).replace(/^ease-/, '') // Remove the "ease-" keyword
    .replace(/(\(|\s).+/, '') // Remove the function brackets and parameters
    .toLowerCase().trim();
  };
  /** Re-maps a number from one range to another. Numbers outside the range are not clamped to 0 and 1, because out-of-range values are often intentional and useful. */
  var getEasingFunction = function getEasingFunction(ease) {
    return EasingFunctions[complexEasingSyntax(ease)] || EasingFunctions.linear;
  };
  // /**
  //  * Allows you to register new easing functions
  //  */
  // export const registerEasingFunction = (key: string, fn: TypeEasingFunction) => {
  //   Object.assign(EasingFunctions, {
  //     [key]: fn,
  //   });
  // };
  // /**
  //  * Allows you to register multiple new easing functions
  //  */
  // export const registerEasingFunctions = (...obj: typeof EasingFunctions[]) => {
  //   Object.assign(EasingFunctions, ...obj);
  // };

  var linear = function linear(x) {
    return x;
  };
  var Start = 1;
  var Middle = 0.5;
  var End = 0;
  function step(count, pos) {
    return function (x) {
      if (x >= 1) {
        return 1;
      }
      var stepSize = 1 / count;
      x += pos * stepSize;
      return x - x % stepSize;
    };
  }
  var numberString = '\\s*(-?\\d+\\.?\\d*|-?\\.\\d+)\\s*';
  var cubicBezierRe = new RegExp('cubic-bezier\\(' + numberString + ',' + numberString + ',' + numberString + ',' + numberString + '\\)');
  var step1Re = /steps\(\s*(\d+)\s*\)/;
  var step2Re = /steps\(\s*(\d+)\s*,\s*(start|middle|end)\s*\)/;
  function parseEasingFunction(normalizedEasing) {
    var cubicData = cubicBezierRe.exec(normalizedEasing);
    if (cubicData) {
      // @ts-ignore
      return bezier.apply(void 0, cubicData.slice(1).map(Number));
    }
    var step1Data = step1Re.exec(normalizedEasing);
    if (step1Data) {
      return step(Number(step1Data[1]), End);
    }
    var step2Data = step2Re.exec(normalizedEasing);
    if (step2Data) {
      // @ts-ignore
      return step(Number(step2Data[1]), {
        start: Start,
        middle: Middle,
        end: End
      }[step2Data[2]]);
    }
    return getEasingFunction(normalizedEasing);
  }
  function calculateActiveDuration(timing) {
    // @ts-ignore
    return Math.abs(repeatedDuration(timing) / (timing.playbackRate || 1));
  }
  function repeatedDuration(timing) {
    var _timing$iterations;
    // https://drafts.csswg.org/web-animations/#calculating-the-active-duration
    if (timing.duration === 0 || timing.iterations === 0) {
      return 0;
    }
    // @see https://developer.mozilla.org/en-US/docs/Web/API/EffectTiming/duration#value
    // if (timing.duration === 'auto') {
    //   timing.duration = 0;
    // }
    return (timing.duration === 'auto' ? 0 : Number(timing.duration)) * ((_timing$iterations = timing.iterations) !== null && _timing$iterations !== void 0 ? _timing$iterations : 1);
  }
  var PhaseNone = 0;
  var PhaseBefore = 1;
  var PhaseAfter = 2;
  var PhaseActive = 3;
  function calculatePhase(activeDuration, localTime, timing) {
    // https://drafts.csswg.org/web-animations/#animation-effect-phases-and-states
    if (localTime === null) {
      return PhaseNone;
    }
    var endTime = timing.endTime;
    if (localTime < Math.min(timing.delay, endTime)) {
      return PhaseBefore;
    }
    if (localTime >= Math.min(timing.delay + activeDuration + timing.endDelay, endTime)) {
      return PhaseAfter;
    }
    return PhaseActive;
  }
  function calculateActiveTime(activeDuration, fillMode, localTime, phase, delay) {
    // https://drafts.csswg.org/web-animations/#calculating-the-active-time
    switch (phase) {
      case PhaseBefore:
        if (fillMode === 'backwards' || fillMode === 'both') return 0;
        return null;
      case PhaseActive:
        return localTime - delay;
      case PhaseAfter:
        if (fillMode === 'forwards' || fillMode === 'both') return activeDuration;
        return null;
      case PhaseNone:
        return null;
    }
  }
  function calculateOverallProgress(iterationDuration, phase, iterations, activeTime, iterationStart) {
    // https://drafts.csswg.org/web-animations/#calculating-the-overall-progress
    var overallProgress = iterationStart;
    if (iterationDuration === 0) {
      if (phase !== PhaseBefore) {
        overallProgress += iterations;
      }
    } else {
      overallProgress += activeTime / iterationDuration;
    }
    return overallProgress;
  }
  function calculateSimpleIterationProgress(overallProgress, iterationStart, phase, iterations, activeTime, iterationDuration) {
    // https://drafts.csswg.org/web-animations/#calculating-the-simple-iteration-progress
    var simpleIterationProgress = overallProgress === Infinity ? iterationStart % 1 : overallProgress % 1;
    if (simpleIterationProgress === 0 && phase === PhaseAfter && iterations !== 0 && (activeTime !== 0 || iterationDuration === 0)) {
      simpleIterationProgress = 1;
    }
    return simpleIterationProgress;
  }
  function calculateCurrentIteration(phase, iterations, simpleIterationProgress, overallProgress) {
    // https://drafts.csswg.org/web-animations/#calculating-the-current-iteration
    if (phase === PhaseAfter && iterations === Infinity) {
      return Infinity;
    }
    if (simpleIterationProgress === 1) {
      return Math.floor(overallProgress) - 1;
    }
    return Math.floor(overallProgress);
  }
  function calculateDirectedProgress(playbackDirection, currentIteration, simpleIterationProgress) {
    // https://drafts.csswg.org/web-animations/#calculating-the-directed-progress
    var currentDirection = playbackDirection;
    if (playbackDirection !== 'normal' && playbackDirection !== 'reverse') {
      var d = currentIteration;
      if (playbackDirection === 'alternate-reverse') {
        d += 1;
      }
      currentDirection = 'normal';
      if (d !== Infinity && d % 2 !== 0) {
        currentDirection = 'reverse';
      }
    }
    if (currentDirection === 'normal') {
      return simpleIterationProgress;
    }
    return 1 - simpleIterationProgress;
  }
  function calculateIterationProgress(activeDuration, localTime, timing) {
    var phase = calculatePhase(activeDuration, localTime, timing);
    var activeTime = calculateActiveTime(activeDuration, timing.fill, localTime, phase, timing.delay);
    if (activeTime === null) return null;
    var duration = timing.duration === 'auto' ? 0 : timing.duration;
    var overallProgress = calculateOverallProgress(duration, phase, timing.iterations, activeTime, timing.iterationStart);
    var simpleIterationProgress = calculateSimpleIterationProgress(overallProgress, timing.iterationStart, phase, timing.iterations, activeTime, duration);
    var currentIteration = calculateCurrentIteration(phase, timing.iterations, simpleIterationProgress, overallProgress);
    var directedProgress = calculateDirectedProgress(timing.direction, currentIteration, simpleIterationProgress);
    timing.currentIteration = currentIteration;
    timing.progress = directedProgress;
    // https://drafts.csswg.org/web-animations/#calculating-the-transformed-progress
    // https://drafts.csswg.org/web-animations/#calculating-the-iteration-progress
    return timing.easingFunction(directedProgress);
  }

  function convertEffectInput(keyframes, timing, target) {
    var propertySpecificKeyframeGroups = makePropertySpecificKeyframeGroups(keyframes, timing);
    var interpolations = makeInterpolations(propertySpecificKeyframeGroups, target);
    return function (target, fraction) {
      if (fraction !== null) {
        interpolations.filter(function (interpolation) {
          return fraction >= interpolation.applyFrom && fraction < interpolation.applyTo;
        }).forEach(function (interpolation) {
          var offsetFraction = fraction - interpolation.startOffset;
          var localDuration = interpolation.endOffset - interpolation.startOffset;
          var scaledLocalTime = localDuration === 0 ? 0 : interpolation.easingFunction(offsetFraction / localDuration);
          // apply updated attribute
          target.style[interpolation.property] = interpolation.interpolation(scaledLocalTime);
        });
      } else {
        for (var property in propertySpecificKeyframeGroups) {
          if (isNotReservedWord(property)) {
            // clear attribute
            target.style[property] = null;
          }
        }
      }
    };
  }
  function isNotReservedWord(member) {
    return member !== 'offset' && member !== 'easing' && member !== 'composite' && member !== 'computedOffset';
  }
  function makePropertySpecificKeyframeGroups(keyframes, timing) {
    var propertySpecificKeyframeGroups = {};
    for (var i = 0; i < keyframes.length; i++) {
      for (var member in keyframes[i]) {
        if (isNotReservedWord(member)) {
          var propertySpecificKeyframe = {
            offset: keyframes[i].offset,
            computedOffset: keyframes[i].computedOffset,
            easing: keyframes[i].easing,
            easingFunction: parseEasingFunction(keyframes[i].easing) || timing.easingFunction,
            value: keyframes[i][member]
          };
          propertySpecificKeyframeGroups[member] = propertySpecificKeyframeGroups[member] || [];
          propertySpecificKeyframeGroups[member].push(propertySpecificKeyframe);
        }
      }
    }
    return propertySpecificKeyframeGroups;
  }
  function makeInterpolations(propertySpecificKeyframeGroups, target) {
    var interpolations = [];
    for (var groupName in propertySpecificKeyframeGroups) {
      var keyframes = propertySpecificKeyframeGroups[groupName];
      for (var i = 0; i < keyframes.length - 1; i++) {
        var startIndex = i;
        var endIndex = i + 1;
        var startOffset = keyframes[startIndex].computedOffset;
        var endOffset = keyframes[endIndex].computedOffset;
        var applyFrom = startOffset;
        var applyTo = endOffset;
        if (i === 0) {
          applyFrom = -Infinity;
          if (endOffset === 0) {
            endIndex = startIndex;
          }
        }
        if (i === keyframes.length - 2) {
          applyTo = Infinity;
          if (startOffset === 1) {
            startIndex = endIndex;
          }
        }
        interpolations.push({
          applyFrom: applyFrom,
          applyTo: applyTo,
          startOffset: keyframes[startIndex].computedOffset,
          endOffset: keyframes[endIndex].computedOffset,
          easingFunction: keyframes[startIndex].easingFunction,
          property: groupName,
          interpolation: propertyInterpolation(groupName, keyframes[startIndex].value, keyframes[endIndex].value, target)
        });
      }
    }
    interpolations.sort(function (leftInterpolation, rightInterpolation) {
      return leftInterpolation.startOffset - rightInterpolation.startOffset;
    });
    return interpolations;
  }
  var InterpolationFactory = function InterpolationFactory(from, to,
  // eslint-disable-next-line @typescript-eslint/ban-types
  convertToString) {
    return function (f) {
      return convertToString(interpolate(from, to, f));
    };
  };
  function propertyInterpolation(property, left, right, target) {
    var metadata = propertyMetadataCache[property];
    if (metadata && metadata.syntax && metadata.int) {
      var propertyHandler = runtime.styleValueRegistry.getPropertySyntax(metadata.syntax);
      if (propertyHandler) {
        var computedLeft = runtime.styleValueRegistry.parseProperty(property, left, target);
        var computedRight = runtime.styleValueRegistry.parseProperty(property, right, target);
        var usedLeft = runtime.styleValueRegistry.computeProperty(property, computedLeft, target);
        var usedRight = runtime.styleValueRegistry.computeProperty(property, computedRight, target);
        // use defaultValue
        // const parsedDefaultLeft = parseDefaultValue(metadata, target, property, left);
        // const parsedDefaultRight = parseDefaultValue(metadata, target, property, right);
        // if (!isNil(parsedDefaultLeft)) {
        //   // @ts-ignore
        //   parsedLeft = parsedDefaultLeft;
        // } else if (propertyHandler.parser) {
        //   parsedLeft = propertyHandler.parser(left, target as DisplayObject);
        // }
        // if (!isNil(parsedDefaultRight)) {
        //   // @ts-ignore
        //   parsedRight = parsedDefaultRight;
        // } else if (propertyHandler.parser) {
        //   parsedRight = propertyHandler.parser(right, target as DisplayObject);
        // }
        // if (propertyHandler.parser) {
        //   parsedLeft = propertyHandler.parser(left, target as DisplayObject);
        //   parsedRight = propertyHandler.parser(right, target as DisplayObject);
        // }
        // // no need to calculate
        // if (propertyHandler.calculator) {
        //   parsedLeft = propertyHandler.calculator(parsedLeft);
        //   parsedRight = propertyHandler.calculator(parsedRight);
        //   // parsedLeft = handler.calculator
        // }
        // merger [left, right, n2string()]
        var interpolationArgs = propertyHandler.mixer(usedLeft, usedRight, target);
        if (interpolationArgs) {
          // @ts-ignore
          var interp = InterpolationFactory.apply(void 0, interpolationArgs);
          return function (t) {
            if (t === 0) return left;
            if (t === 1) return right;
            return interp(t);
          };
        }
      }
    }
    // eslint-disable-next-line @typescript-eslint/no-use-before-define
    return InterpolationFactory(false, true, function (bool) {
      return bool ? right : left;
    });
  }
  /**
   * interpolate with number, boolean, number[], boolean[]
   */
  function interpolate(from, to, f) {
    if (typeof from === 'number' && typeof to === 'number') {
      return from * (1 - f) + to * f;
    }
    if (typeof from === 'boolean' && typeof to === 'boolean' || typeof from === 'string' && typeof to === 'string' // skip string, eg. path ['M', 10, 10]
    ) {
      return f < 0.5 ? from : to;
    }
    if (Array.isArray(from) && Array.isArray(to)) {
      // interpolate arrays/matrix
      if (from.length === to.length) {
        var r = [];
        for (var i = 0; i < from.length; i++) {
          r.push(interpolate(from[i], to[i], f));
        }
        return r;
      }
    }
    throw new Error('Mismatched interpolation arguments ' + from + ':' + to);
  }

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/API/EffectTiming
   */
  var AnimationEffectTiming = /*#__PURE__*/function () {
    function AnimationEffectTiming() {
      this.delay = 0;
      this.direction = 'normal';
      this.duration = 'auto';
      this._easing = 'linear';
      this.easingFunction = linear;
      this.endDelay = 0;
      this.fill = 'auto';
      this.iterationStart = 0;
      this.iterations = 1;
      this.playbackRate = void 0;
      this.effect = void 0;
      this.composite = void 0;
      this.activeDuration = void 0;
      this.endTime = void 0;
      this.currentIteration = null;
      this.progress = null;
    }
    _createClass$2(AnimationEffectTiming, [{
      key: "easing",
      get: function get() {
        return this._easing;
      },
      set: function set(value) {
        this.easingFunction = parseEasingFunction(value);
        this._easing = value;
      }
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/EffectTiming/endDelay
       */
    }]);
    return AnimationEffectTiming;
  }();

  /**
   * @example
    {
      translateY: [200, 300],
      scale: [1, 10],
    }

   * groups' length can be different, the following config should generate 3 frames:
    @example
    {
      translateY: [200, 300, 400],
      scale: [1, 10],
    }
   */
  function convertToArrayForm(effectInput) {
    var normalizedEffectInput = [];
    for (var property in effectInput) {
      // skip reserved props
      if (property in ['easing', 'offset', 'composite']) {
        continue;
      }
      // @ts-ignore
      var values = effectInput[property];
      if (!Array.isArray(values)) {
        values = [values];
      }
      var numKeyframes = values.length;
      for (var i = 0; i < numKeyframes; i++) {
        if (!normalizedEffectInput[i]) {
          var keyframe = {};
          if ('offset' in effectInput) {
            keyframe.offset = Number(effectInput.offset);
          }
          if ('easing' in effectInput) {
            // @ts-ignore
            keyframe.easing = effectInput.easing;
          }
          if ('composite' in effectInput) {
            // @ts-ignore
            keyframe.composite = effectInput.composite;
          }
          normalizedEffectInput[i] = keyframe;
        }
        if (values[i] !== undefined && values[i] !== null) {
          normalizedEffectInput[i][property] = values[i];
        }
      }
    }
    normalizedEffectInput.sort(function (a, b) {
      return (a.computedOffset || 0) - (b.computedOffset || 0);
    });
    return normalizedEffectInput;
  }
  function normalizeKeyframes(effectInput, timing) {
    if (effectInput === null) {
      return [];
    }
    if (!Array.isArray(effectInput)) {
      effectInput = convertToArrayForm(effectInput);
    }
    var keyframes = effectInput.map(function (originalKeyframe) {
      var keyframe = {};
      if (timing === null || timing === void 0 ? void 0 : timing.composite) {
        // This will be auto if the composite operation specified on the effect is being used.
        // @see https://developer.mozilla.org/en-US/docs/Web/API/Web_Animations_API/Keyframe_Formats
        keyframe.composite = 'auto';
      }
      for (var member in originalKeyframe) {
        var memberValue = originalKeyframe[member];
        if (member === 'offset') {
          if (memberValue !== null) {
            memberValue = Number(memberValue);
            if (!isFinite(memberValue)) throw new Error('Keyframe offsets must be numbers.');
            if (memberValue < 0 || memberValue > 1) throw new Error('Keyframe offsets must be between 0 and 1.');
            keyframe.computedOffset = memberValue;
          }
        } else if (member === 'composite') {
          // TODO: Support add & accumulate in KeyframeEffect.composite
          // @see https://developer.mozilla.org/en-US/docs/Web/API/KeyframeEffect/composite
          if (['replace', 'add', 'accumulate', 'auto'].indexOf(memberValue) === -1) {
            throw new Error(memberValue + " compositing is not supported");
          }
        } else ;
        // assign to keyframe, no need to parse shorthand value
        keyframe[member] = memberValue;
      }
      if (keyframe.offset === undefined) {
        keyframe.offset = null;
      }
      if (keyframe.easing === undefined) {
        // override with timing.easing
        keyframe.easing = (timing === null || timing === void 0 ? void 0 : timing.easing) || 'linear';
      }
      if (keyframe.composite === undefined) {
        keyframe.composite = 'auto';
      }
      return keyframe;
    });
    var everyFrameHasOffset = true;
    var previousOffset = -Infinity;
    for (var i = 0; i < keyframes.length; i++) {
      var offset = keyframes[i].offset;
      if (!isNil(offset)) {
        if (offset < previousOffset) {
          throw new TypeError('Keyframes are not loosely sorted by offset. Sort or specify offsets.');
        }
        previousOffset = offset;
      } else {
        everyFrameHasOffset = false;
      }
    }
    keyframes = keyframes.filter(function (keyframe) {
      return Number(keyframe.offset) >= 0 && Number(keyframe.offset) <= 1;
    });
    function spaceKeyframes() {
      var _keyframes$offset;
      var length = keyframes.length;
      keyframes[length - 1].computedOffset = Number((_keyframes$offset = keyframes[length - 1].offset) !== null && _keyframes$offset !== void 0 ? _keyframes$offset : 1);
      if (length > 1) {
        var _keyframes$0$offset;
        keyframes[0].computedOffset = Number((_keyframes$0$offset = keyframes[0].offset) !== null && _keyframes$0$offset !== void 0 ? _keyframes$0$offset : 0);
      }
      var previousIndex = 0;
      var previousOffset = Number(keyframes[0].computedOffset);
      for (var _i = 1; _i < length; _i++) {
        var _offset = keyframes[_i].computedOffset;
        if (!isNil(_offset) && !isNil(previousOffset)) {
          for (var j = 1; j < _i - previousIndex; j++) {
            keyframes[previousIndex + j].computedOffset = previousOffset + (Number(_offset) - previousOffset) * j / (_i - previousIndex);
          }
          previousIndex = _i;
          previousOffset = Number(_offset);
        }
      }
    }
    if (!everyFrameHasOffset) spaceKeyframes();
    return keyframes;
  }

  var fills = 'backwards|forwards|both|none'.split('|');
  var directions = 'reverse|alternate|alternate-reverse'.split('|');
  function makeTiming(timingInput, forGroup) {
    var timing = new AnimationEffectTiming();
    if (forGroup) {
      timing.fill = 'both';
      timing.duration = 'auto';
    }
    if (typeof timingInput === 'number' && !isNaN(timingInput)) {
      timing.duration = timingInput;
    } else if (timingInput !== undefined) {
      Object.keys(timingInput).forEach(function (property) {
        if (timingInput[property] !== undefined && timingInput[property] !== null && timingInput[property] !== 'auto') {
          if (typeof timing[property] === 'number' || property === 'duration') {
            if (typeof timingInput[property] !== 'number' || isNaN(timingInput[property])) {
              return;
            }
          }
          if (property === 'fill' && fills.indexOf(timingInput[property]) === -1) {
            return;
          }
          if (property === 'direction' && directions.indexOf(timingInput[property]) === -1) {
            return;
          }
          // @ts-ignore
          timing[property] = timingInput[property];
        }
      });
    }
    return timing;
  }
  function normalizeTimingInput(timingInput, forGroup) {
    var _timingInput;
    timingInput = numericTimingToObject((_timingInput = timingInput) !== null && _timingInput !== void 0 ? _timingInput : {
      duration: 'auto'
    });
    return makeTiming(timingInput, forGroup);
  }
  function numericTimingToObject(timingInput) {
    if (typeof timingInput === 'number') {
      if (isNaN(timingInput)) {
        timingInput = {
          duration: 'auto'
        };
      } else {
        timingInput = {
          duration: timingInput
        };
      }
    }
    return timingInput;
  }
  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/API/KeyframeEffect
   * @example
    const circleDownKeyframes = new KeyframeEffect(
      circle, // element to animate
      [
        { transform: 'translateY(0)' }, // keyframe
        { transform: 'translateY(100)' } // keyframe
      ],
      { duration: 3000, fill: 'forwards' } // keyframe options
    );
   *
   */
  var KeyframeEffect = /*#__PURE__*/function () {
    // pseudoElement: string | null;

    function KeyframeEffect(target, effectInput, timingInput) {
      var _this = this;
      this.composite = 'replace';
      this.iterationComposite = 'replace';
      this.target = void 0;
      this.animation = void 0;
      this.timing = void 0;
      this.computedTiming = void 0;
      this.normalizedKeyframes = void 0;
      this.timeFraction = void 0;
      this.interpolations = void 0;
      this.target = target;
      this.timing = normalizeTimingInput(timingInput, false);
      this.timing.effect = this;
      this.timing.activeDuration = calculateActiveDuration(this.timing);
      this.timing.endTime = Math.max(0, this.timing.delay + this.timing.activeDuration + this.timing.endDelay);
      this.normalizedKeyframes = normalizeKeyframes(effectInput, this.timing);
      this.interpolations = convertEffectInput(this.normalizedKeyframes, this.timing, this.target);
      this.computedTiming = new Proxy(this.timing, {
        get: function get(target, prop) {
          if (prop === 'duration') {
            return target.duration === 'auto' ? 0 : target.duration;
          } else if (prop === 'fill') {
            return target.fill === 'auto' ? 'none' : target.fill;
          } else if (prop === 'localTime') {
            return _this.animation && _this.animation.currentTime || null;
          } else if (prop === 'currentIteration') {
            if (!_this.animation || _this.animation.playState !== 'running') {
              return null;
            }
            return target.currentIteration || 0;
          } else if (prop === 'progress') {
            if (!_this.animation || _this.animation.playState !== 'running') {
              return null;
            }
            return target.progress || 0;
          }
          return target[prop];
        },
        set: function set() {
          return true;
        }
      });
    }
    var _proto = KeyframeEffect.prototype;
    _proto.applyInterpolations = function applyInterpolations() {
      this.interpolations(this.target, Number(this.timeFraction));
    };
    _proto.update = function update(localTime) {
      if (localTime === null) {
        return false;
      }
      this.timeFraction = calculateIterationProgress(this.timing.activeDuration, localTime, this.timing);
      return this.timeFraction !== null;
    };
    _proto.getKeyframes = function getKeyframes() {
      return this.normalizedKeyframes;
    };
    _proto.setKeyframes = function setKeyframes(keyframes) {
      this.normalizedKeyframes = normalizeKeyframes(keyframes);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/AnimationEffect/getComputedTiming
     */;
    _proto.getComputedTiming = function getComputedTiming() {
      return this.computedTiming;
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/AnimationEffect/getTiming
     */;
    _proto.getTiming = function getTiming() {
      return this.timing;
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/AnimationEffect/updateTiming
     */;
    _proto.updateTiming = function updateTiming(timing) {
      var _this2 = this;
      Object.keys(timing || {}).forEach(function (name) {
        _this2.timing = timing[name];
      });
    };
    return KeyframeEffect;
  }();

  function compareAnimations(leftAnimation, rightAnimation) {
    return Number(leftAnimation.id) - Number(rightAnimation.id);
  }
  /**
   * @see https://github.com/DefinitelyTyped/DefinitelyTyped/blob/master/types/web-animations-js/index.d.ts
   */
  var AnimationTimeline = /*#__PURE__*/function () {
    /**
     * all active animations
     */

    function AnimationTimeline(document) {
      var _this = this;
      this.document = void 0;
      this.animations = [];
      this.ticking = false;
      this.timelineTicking = false;
      this.hasRestartedThisFrame = false;
      this.animationsWithPromises = [];
      this.inTick = false;
      this.pendingEffects = [];
      this.currentTime = null;
      this.rafId = 0;
      this.rafCallbacks = [];
      this.frameId = void 0;
      this.webAnimationsNextTick = function (t) {
        _this.currentTime = t;
        _this.discardAnimations();
        if (_this.animations.length === 0) {
          _this.timelineTicking = false;
        } else {
          _this.requestAnimationFrame(_this.webAnimationsNextTick);
        }
      };
      this.processRafCallbacks = function (t) {
        var processing = _this.rafCallbacks;
        _this.rafCallbacks = [];
        if (t < Number(_this.currentTime)) t = Number(_this.currentTime);
        _this.animations.sort(compareAnimations);
        _this.animations = _this.tick(t, true, _this.animations)[0];
        processing.forEach(function (entry) {
          entry[1](t);
        });
        _this.applyPendingEffects();
      };
      this.document = document;
    }
    var _proto = AnimationTimeline.prototype;
    _proto.getAnimations = function getAnimations() {
      this.discardAnimations();
      return this.animations.slice();
    };
    _proto.isTicking = function isTicking() {
      return this.inTick;
    };
    _proto.play = function play(target, keyframes, options) {
      var effect = new KeyframeEffect(target, keyframes, options);
      var animation = new Animation(effect, this);
      this.animations.push(animation);
      this.restartWebAnimationsNextTick();
      animation.updatePromises();
      animation.play();
      animation.updatePromises();
      return animation;
    }
    // RAF is supposed to be the last script to occur before frame rendering but not
    // all browsers behave like this. This function is for synchonously updating an
    // animation's effects whenever its state is mutated by script to work around
    // incorrect script execution ordering by the browser.
    ;
    _proto.applyDirtiedAnimation = function applyDirtiedAnimation(animation) {
      var _this2 = this;
      if (this.inTick) {
        return;
      }
      // update active animations in displayobject
      animation.markTarget();
      var animations = animation.targetAnimations();
      animations.sort(compareAnimations);
      // clear inactive animations
      var inactiveAnimations = this.tick(Number(this.currentTime), false, animations.slice())[1];
      inactiveAnimations.forEach(function (animation) {
        var index = _this2.animations.indexOf(animation);
        if (index !== -1) {
          _this2.animations.splice(index, 1);
        }
      });
      this.applyPendingEffects();
    };
    _proto.restart = function restart() {
      if (!this.ticking) {
        this.ticking = true;
        this.requestAnimationFrame(function () {});
        this.hasRestartedThisFrame = true;
      }
      return this.hasRestartedThisFrame;
    };
    _proto.destroy = function destroy() {
      this.document.defaultView.cancelAnimationFrame(this.frameId);
    };
    _proto.applyPendingEffects = function applyPendingEffects() {
      this.pendingEffects.forEach(function (effect) {
        effect === null || effect === void 0 ? void 0 : effect.applyInterpolations();
      });
      this.pendingEffects = [];
    };
    _proto.updateAnimationsPromises = function updateAnimationsPromises() {
      this.animationsWithPromises = this.animationsWithPromises.filter(function (animation) {
        return animation.updatePromises();
      });
    };
    _proto.discardAnimations = function discardAnimations() {
      this.updateAnimationsPromises();
      this.animations = this.animations.filter(function (animation) {
        return animation.playState !== 'finished' && animation.playState !== 'idle';
      });
    };
    _proto.restartWebAnimationsNextTick = function restartWebAnimationsNextTick() {
      if (!this.timelineTicking) {
        this.timelineTicking = true;
        this.requestAnimationFrame(this.webAnimationsNextTick);
      }
    };
    _proto.rAF = function rAF(f) {
      var id = this.rafId++;
      if (this.rafCallbacks.length === 0) {
        this.frameId = this.document.defaultView.requestAnimationFrame(this.processRafCallbacks);
      }
      this.rafCallbacks.push([id, f]);
      return id;
    };
    _proto.requestAnimationFrame = function requestAnimationFrame(f) {
      var _this3 = this;
      return this.rAF(function (x) {
        _this3.updateAnimationsPromises();
        f(x);
        _this3.updateAnimationsPromises();
      });
    };
    _proto.tick = function tick(t, isAnimationFrame, updatingAnimations) {
      var _this4 = this,
        _this$pendingEffects,
        _this$pendingEffects2;
      this.inTick = true;
      this.hasRestartedThisFrame = false;
      this.currentTime = t;
      this.ticking = false;
      var newPendingClears = [];
      var newPendingEffects = [];
      var activeAnimations = [];
      var inactiveAnimations = [];
      updatingAnimations.forEach(function (animation) {
        animation.tick(t, isAnimationFrame);
        if (!animation._inEffect) {
          newPendingClears.push(animation.effect);
          animation.unmarkTarget();
        } else {
          newPendingEffects.push(animation.effect);
          animation.markTarget();
        }
        if (animation._needsTick) _this4.ticking = true;
        var alive = animation._inEffect || animation._needsTick;
        animation._inTimeline = alive;
        if (alive) {
          activeAnimations.push(animation);
        } else {
          inactiveAnimations.push(animation);
        }
      });
      (_this$pendingEffects = this.pendingEffects).push.apply(_this$pendingEffects, newPendingClears);
      (_this$pendingEffects2 = this.pendingEffects).push.apply(_this$pendingEffects2, newPendingEffects);
      if (this.ticking) this.requestAnimationFrame(function () {});
      this.inTick = false;
      return [activeAnimations, inactiveAnimations];
    };
    return AnimationTimeline;
  }();

  runtime.EasingFunction = parseEasingFunction;
  runtime.AnimationTimeline = AnimationTimeline;

  function _objectWithoutPropertiesLoose$1(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }
    return target;
  }

  function _objectWithoutProperties(source, excluded) {
    if (source == null) return {};
    var target = _objectWithoutPropertiesLoose$1(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
      var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
      for (i = 0; i < sourceSymbolKeys.length; i++) {
        key = sourceSymbolKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
        target[key] = source[key];
      }
    }
    return target;
  }

  var _excluded$1 = ["key", "ref"];
  // 实现jsx-classic 入口
  function jsx(type, config) {
    var _ref = config || {},
      key = _ref.key,
      ref = _ref.ref,
      props = _objectWithoutProperties(_ref, _excluded$1);
    // 保持和automatic模式一致
    for (var _len = arguments.length, children = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
      children[_key - 2] = arguments[_key];
    }
    if (children.length) {
      props.children = children.length === 1 ? children[0] : children;
    }
    return {
      key: key,
      ref: ref,
      type: type,
      props: props
    };
  }

  var fragment = (function (props) {
    return props.children;
  });

  function asyncGeneratorStep$6(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator$6(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep$6(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep$6(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }

  var _typeof_1 = createCommonjsModule(function (module) {
  function _typeof(obj) {
    "@babel/helpers - typeof";

    return (module.exports = _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) {
      return typeof obj;
    } : function (obj) {
      return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    }, module.exports.__esModule = true, module.exports["default"] = module.exports), _typeof(obj);
  }
  module.exports = _typeof, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var regeneratorRuntime$1 = createCommonjsModule(function (module) {
  var _typeof = _typeof_1["default"];
  function _regeneratorRuntime() {
    module.exports = _regeneratorRuntime = function _regeneratorRuntime() {
      return exports;
    }, module.exports.__esModule = true, module.exports["default"] = module.exports;
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function define(obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == _typeof(value) && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function value(method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator["return"] && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) {
                if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              }
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) {
        keys.push(key);
      }
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function reset(skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) {
          "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
        }
      },
      stop: function stop() {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function dispatchException(exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function abrupt(type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function complete(record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function finish(finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      "catch": function _catch(tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function delegateYield(iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  module.exports = _regeneratorRuntime, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  // TODO(Babel 8): Remove this file.

  var runtime$1 = regeneratorRuntime$1();
  var regenerator = runtime$1;

  // Copied from https://github.com/facebook/regenerator/blob/main/packages/runtime/runtime.js#L736=
  try {
    regeneratorRuntime = runtime$1;
  } catch (accidentalStrictMode) {
    if (typeof globalThis === "object") {
      globalThis.regeneratorRuntime = runtime$1;
    } else {
      Function("r", "regeneratorRuntime = r")(runtime$1);
    }
  }

  var CanvasElement = /*#__PURE__*/function () {
    function CanvasElement(ctx) {
      this.width = void 0;
      this.height = void 0;
      this.isCanvasElement = true;
      this.context = void 0;
      this.emitter = new eventemitter3();
      this.context = ctx;
      // 有可能是 node canvas 创建的 context 对象
      var canvas = ctx.canvas || {};
      this.width = canvas.width || 0;
      this.height = canvas.height || 0;
    }
    var _proto = CanvasElement.prototype;
    _proto.getContext = function getContext(contextId, contextAttributes) {
      return this.context;
    };
    _proto.getBoundingClientRect = function getBoundingClientRect() {
      var width = this.width;
      var height = this.height;
      // 默认都处理成可视窗口的顶部位置
      return {
        top: 0,
        right: width,
        bottom: height,
        left: 0,
        width: width,
        height: height,
        x: 0,
        y: 0
      };
    };
    _proto.addEventListener = function addEventListener(type, listener, options) {
      // TODO: implement options
      this.emitter.on(type, listener);
    };
    _proto.removeEventListener = function removeEventListener(type, listener, options) {
      this.emitter.off(type, listener);
    }
    /**
     * @see https://www.w3.org/TR/DOM-Level-2-Events/events.html#Events-EventTarget-dispatchEvent
     */;
    _proto.dispatchEvent = function dispatchEvent(e) {
      this.emitter.emit(e.type, e);
      return true;
    };
    return CanvasElement;
  }();
  function supportEventListener(canvas) {
    if (!canvas) {
      return false;
    }
    // 非 HTMLCanvasElement
    if (canvas.nodeType !== 1 || !canvas.nodeName || canvas.nodeName.toLowerCase() !== 'canvas') {
      return false;
    }
    // 微信小程序canvas.getContext('2d')时也是CanvasRenderingContext2D
    // 也会有ctx.canvas, 而且nodeType也是1，所以还要在看下是否支持addEventListener
    var support = false;
    try {
      canvas.addEventListener('eventTest', function () {
        support = true;
      });
      canvas.dispatchEvent(new Event('eventTest'));
    } catch (error) {
      support = false;
    }
    return support;
  }
  function createMobileCanvasElement(ctx) {
    if (!ctx) {
      return null;
    }
    if (supportEventListener(ctx.canvas)) {
      return ctx.canvas;
    }
    return new CanvasElement(ctx);
  }

  function createUpdater(canvas) {
    var setStateQueue = [];
    function process() {
      var item;
      var renderComponents = [];
      var renderCallbackQueue = [];
      while (item = setStateQueue.shift()) {
        var _item = item,
          state = _item.state,
          component = _item.component,
          callback = _item.callback;
        // 组件已销毁，不再触发 setState
        if (component.destroyed) {
          continue;
        }
        // 如果没有prevState，则将当前的state作为初始的prevState
        if (!component.prevState) {
          component.prevState = Object.assign({}, component.state);
        }
        // 如果stateChange是一个方法，也就是setState的第二种形式
        if (typeof state === 'function') {
          Object.assign(component.state, state(component.prevState, component.props));
        } else {
          // 如果stateChange是一个对象，则直接合并到setState中
          Object.assign(component.state, state);
        }
        component.prevState = component.state;
        if (typeof callback === 'function') {
          renderCallbackQueue.push({
            callback: callback,
            component: component
          });
        }
        if (renderComponents.indexOf(component) < 0) {
          renderComponents.push(component);
        }
      }
      canvas.updateComponents(renderComponents);
      // callback queue
      commitRenderQueue(renderCallbackQueue);
    }
    function enqueueSetState(component, state, callback) {
      if (setStateQueue.length === 0) {
        setTimeout(process, 0);
      }
      setStateQueue.push({
        component: component,
        state: state,
        callback: callback
      });
    }
    function commitRenderQueue(callbackQueue) {
      for (var i = 0; i < callbackQueue.length; i++) {
        var _callbackQueue$i = callbackQueue[i],
          callback = _callbackQueue$i.callback,
          component = _callbackQueue$i.component;
        callback.call(component);
      }
    }
    var updater = {
      // isMounted: function(publicInstance) {
      //   return false;
      // },
      enqueueForceUpdate: enqueueSetState,
      // enqueueReplaceState: function(publicInstance, completeState) {
      // },
      enqueueSetState: enqueueSetState
    };
    return updater;
  }

  var THEME = {
    fontSize: '24px',
    fontFamily: '"Helvetica Neue", Helvetica, "PingFang SC", "Hiragino Sans GB", "Microsoft YaHei", Arial, sans-serif',
    pixelRatio: 1,
    padding: [0, 0, 0, 0]
  };

  var elementStyle = {
    fillStyle: 'String',
    font: 'String',
    globalAlpha: 'Number',
    lineCap: 'String',
    lineWidth: ['Number', 'String'],
    lineJoin: 'String',
    miterLimit: 'Number',
    shadowBlur: 'Number',
    shadowColor: 'String',
    shadowOffsetX: 'Number',
    shadowOffsetY: 'Number',
    strokeStyle: 'String',
    textAlign: 'String',
    textBaseline: 'String',
    lineDash: ['Array', 'Number'],
    shadow: 'String',
    matrix: 'Array',
    stroke: 'String',
    fill: ['String', 'Object'],
    opacity: 'Number',
    fillOpacity: 'Number',
    strokeOpacity: 'Number'
  };
  // css规则表 TODO：补充 / 多种类型
  var DEFAULT_CSS_RULE = {
    group: _objectSpread2({}, elementStyle),
    text: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      x: 'Number',
      y: 'Number',
      text: 'String',
      width: 'Number',
      height: 'Number',
      fontSize: ['Number', 'String'],
      fontFamily: 'String',
      fontStyle: 'String',
      fontWeight: ['Number', 'String'],
      fontVariant: 'String'
    }),
    circle: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      cx: 'Number',
      cy: 'Number',
      r: 'Number'
    }),
    path: _objectSpread2({}, elementStyle),
    ellipse: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      cy: 'Number',
      cx: 'Number',
      ry: 'Number',
      rx: 'Number'
    }),
    rect: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      width: 'Number',
      height: 'Number',
      x: 'Number',
      y: 'Number',
      radius: ['Array', 'Number']
    }),
    image: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      width: 'Number',
      height: 'Number',
      x: 'Number',
      y: 'Number',
      img: 'String',
      src: 'String'
    }),
    line: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      x1: 'Number',
      x2: 'Number',
      y1: 'Number',
      y2: 'Number'
    }),
    polyline: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      points: 'Array',
      smooth: 'Boolean'
    }),
    polygon: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      points: 'Array'
    }),
    arc: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      x: 'Number',
      y: 'Number',
      r: 'Number',
      startAngle: ['Number', 'String'],
      endAngle: ['Number', 'String'],
      anticlockwise: 'Boolean'
    }),
    marker: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      x: 'Number',
      y: 'Number',
      radius: 'Number',
      symbol: 'String'
    }),
    sector: _objectSpread2(_objectSpread2({}, elementStyle), {}, {
      x: 'Number',
      y: 'Number',
      startAngle: ['Number', 'String'],
      endAngle: ['Number', 'String'],
      r: 'Number',
      r0: 'Number',
      anticlockwise: 'Boolean'
    })
  };
  function checkCSSRule(type, style) {
    if (!style) {
      return style;
    }
    var cssStyle = {};
    Object.keys(style).forEach(function (key) {
      var value = style[key];
      if (isNil(value)) {
        return;
      }
      var rule = DEFAULT_CSS_RULE[type] && DEFAULT_CSS_RULE[type][key];
      if (!rule) {
        cssStyle[key] = value;
        return;
      }
      var valueType = Object.prototype.toString.call(value);
      if (isArray(rule)) {
        for (var i = 0, len = rule.length; i < len; i++) {
          if (valueType === "[object ".concat(rule[i], "]")) {
            cssStyle[key] = value;
            return;
          }
        }
        // 没有匹配的类型
        return;
      }
      // string
      if (valueType === "[object ".concat(rule, "]")) {
        cssStyle[key] = value;
      }
    });
    return cssStyle;
  }

  // 默认设置50
  var ONE_REM;
  try {
    // xgraph下这段会抛错
    ONE_REM = parseInt(document.documentElement.style.fontSize, 10) || 50;
  } catch (e) {
    ONE_REM = 50;
  }
  var SCALE = ONE_REM / 100;
  /**
   * 像素转换
   * @param {Number} px - 750视觉稿像素
   * @return {Number} 屏幕上实际像素
   */
  function defaultPx2hd(px) {
    if (!px) {
      return 0;
    }
    return Number((px * SCALE).toFixed(1));
  }
  function parsePadding(padding) {
    if (isNumber(padding)) {
      return [padding, padding, padding, padding];
    }
    var top = padding[0];
    var right = isNumber(padding[1]) ? padding[1] : padding[0];
    var bottom = isNumber(padding[2]) ? padding[2] : top;
    var left = isNumber(padding[3]) ? padding[3] : right;
    return [top, right, bottom, left];
  }
  function batch2hd(px2hd) {
    var batchPx2hd = function batchPx2hd(value) {
      // 处理带px的数据
      if (isString(value) && /^-?\d+px$/.test(value)) {
        var num = value.substr(0, value.length - 2);
        return px2hd(Number(num));
      }
      if (isArray(value)) {
        return value.map(function (v) {
          return batchPx2hd(v);
        });
      }
      if (isPlainObject(value)) {
        var result = {};
        for (var key in value) {
          if (value.hasOwnProperty(key)) {
            var rst = batchPx2hd(value[key]);
            if (!rst) {
              result[key] = rst;
              continue;
            }
            if (key === 'padding' || key === 'margin') {
              var paddingArray = parsePadding(rst);
              result[key] = paddingArray;
              result["".concat(key, "Top")] = paddingArray[0];
              result["".concat(key, "Right")] = paddingArray[1];
              result["".concat(key, "Bottom")] = paddingArray[2];
              result["".concat(key, "Left")] = paddingArray[3];
              continue;
            }
            result[key] = rst;
          }
        }
        return result;
      }
      // 默认直接返回
      return value;
    };
    return batchPx2hd;
  }
  var px2hd = batch2hd(defaultPx2hd);

  function measureText(container, px2hd, theme) {
    return function (text, font) {
      var _ref = font || {},
        _ref$fontSize = _ref.fontSize,
        fontSize = _ref$fontSize === void 0 ? theme.fontSize : _ref$fontSize,
        _ref$fontFamily = _ref.fontFamily,
        fontFamily = _ref$fontFamily === void 0 ? theme.fontFamily : _ref$fontFamily,
        _ref$fontWeight = _ref.fontWeight,
        fontWeight = _ref$fontWeight === void 0 ? theme.fontWeight : _ref$fontWeight,
        _ref$fontVariant = _ref.fontVariant,
        fontVariant = _ref$fontVariant === void 0 ? theme.fontVariant : _ref$fontVariant,
        _ref$fontStyle = _ref.fontStyle,
        fontStyle = _ref$fontStyle === void 0 ? theme.fontStyle : _ref$fontStyle,
        _ref$textAlign = _ref.textAlign,
        textAlign = _ref$textAlign === void 0 ? theme.textAlign : _ref$textAlign,
        _ref$textBaseline = _ref.textBaseline,
        textBaseline = _ref$textBaseline === void 0 ? theme.textBaseline : _ref$textBaseline,
        _ref$lineWidth = _ref.lineWidth,
        lineWidth = _ref$lineWidth === void 0 ? 1 : _ref$lineWidth;
      var style = {
        x: 0,
        y: 0,
        fontSize: px2hd(fontSize),
        fontFamily: fontFamily,
        fontStyle: fontStyle,
        fontWeight: fontWeight,
        fontVariant: fontVariant,
        text: text,
        textAlign: textAlign,
        textBaseline: textBaseline,
        lineWidth: lineWidth,
        visibility: 'hidden'
      };
      var result = checkCSSRule('text', style);
      var shape = new Text({
        style: result
      });
      container.appendChild(shape);
      var _shape$getBBox = shape.getBBox(),
        width = _shape$getBBox.width,
        height = _shape$getBBox.height;
      shape.remove();
      return {
        width: width,
        height: height
      };
    };
  }
  function computeLayout$2(style) {
    var left = style.left,
      top = style.top,
      width = style.width,
      height = style.height,
      padding = style.padding;
    var _padding = _slicedToArray(padding, 4),
      paddingTop = _padding[0],
      paddingRight = _padding[1],
      paddingBottom = _padding[2],
      paddingLeft = _padding[3];
    return {
      left: left + paddingLeft,
      top: top + paddingTop,
      width: width - paddingLeft - paddingRight,
      height: height - paddingTop - paddingBottom
    };
  }
  // 顶层Canvas标签
  var Canvas$1 = /*#__PURE__*/function () {
    function Canvas$1(props) {
      _classCallCheck(this, Canvas$1);
      var context = props.context,
        _props$renderer = props.renderer,
        renderer = _props$renderer === void 0 ? new Renderer() : _props$renderer,
        width = props.width,
        height = props.height,
        customTheme = props.theme,
        customPx2hd = props.px2hd,
        customPixelRatio = props.pixelRatio,
        createImage = props.createImage,
        landscape = props.landscape,
        rendererContainer = props.container,
        customStyle = props.style,
        _props$animate = props.animate,
        animate = _props$animate === void 0 ? true : _props$animate;
      var px2hd$1 = isFunction(customPx2hd) ? batch2hd(customPx2hd) : px2hd;
      // 初始化主题
      var theme = px2hd$1(_objectSpread2(_objectSpread2({}, THEME), customTheme));
      var pixelRatio = theme.pixelRatio,
        fontSize = theme.fontSize,
        fontFamily = theme.fontFamily,
        padding = theme.padding;
      var devicePixelRatio = customPixelRatio ? customPixelRatio : pixelRatio;
      // 组件更新器
      var updater = createUpdater(this);
      var canvasElement = createMobileCanvasElement(context);
      var canvas = new Canvas({
        container: rendererContainer,
        canvas: canvasElement,
        devicePixelRatio: devicePixelRatio,
        renderer: renderer,
        width: width,
        height: height,
        supportsTouchEvents: true,
        supportsPointerEvents: true,
        createImage: createImage
      });
      var container = canvas.getRoot();
      var _canvas$getConfig = canvas.getConfig(),
        canvasWidth = _canvas$getConfig.width,
        canvasHeight = _canvas$getConfig.height;
      var style = px2hd$1(_objectSpread2({
        left: 0,
        top: 0,
        width: canvasWidth,
        height: canvasHeight,
        padding: padding
      }, customStyle));
      var layout = computeLayout$2(style);
      // 设置默认的全局样式
      var documentElement = canvas.document.documentElement;
      documentElement.setAttribute('fontSize', fontSize);
      documentElement.setAttribute('fontFamily', fontFamily);
      // 设置 container 的位置
      container.setAttribute('x', layout.left);
      container.setAttribute('y', layout.top);
      var gesture = new Gesture$1(canvas);
      // 供全局使用的一些变量
      var componentContext = {
        root: this,
        canvas: canvas,
        left: layout.left,
        top: layout.top,
        width: layout.width,
        height: layout.height,
        px2hd: px2hd$1,
        theme: theme,
        gesture: gesture,
        measureText: measureText(container, px2hd$1, theme)
      };
      var vNode = {
        key: undefined,
        tag: ClassComponent,
        style: layout,
        // @ts-ignore
        type: Canvas$1,
        props: props,
        shape: container,
        animate: animate,
        // @ts-ignore
        component: this,
        canvas: this,
        context: componentContext,
        updater: updater
      };
      this._ee = new EventEmitter();
      this.props = props;
      this.context = componentContext;
      this.updater = updater;
      this.gesture = gesture;
      this.theme = theme;
      this.canvas = canvas;
      this.container = container;
      this.canvasElement = canvasElement;
      this.vNode = vNode;
      // todo: 横屏事件逻辑
      this.landscape = landscape;
    }
    _createClass(Canvas$1, [{
      key: "updateComponents",
      value: function updateComponents$1(components) {
        updateComponents(components);
      }
    }, {
      key: "update",
      value: function () {
        var _update = _asyncToGenerator$6( /*#__PURE__*/regenerator.mark(function _callee(nextProps) {
          var props, vNode, _props$animate2, animate;
          return regenerator.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  props = this.props, vNode = this.vNode;
                  if (!equal(nextProps, props)) {
                    _context.next = 3;
                    break;
                  }
                  return _context.abrupt("return");
                case 3:
                  _props$animate2 = props.animate, animate = _props$animate2 === void 0 ? true : _props$animate2;
                  this.props = nextProps;
                  vNode.props = nextProps;
                  vNode.animate = animate;
                  _context.next = 9;
                  return this.render();
                case 9:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, this);
        }));
        function update(_x) {
          return _update.apply(this, arguments);
        }
        return update;
      }()
    }, {
      key: "render",
      value: function () {
        var _render2 = _asyncToGenerator$6( /*#__PURE__*/regenerator.mark(function _callee2() {
          var canvas, vNode;
          return regenerator.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  canvas = this.canvas, vNode = this.vNode;
                  _context2.next = 3;
                  return canvas.ready;
                case 3:
                  render(vNode);
                case 4:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2, this);
        }));
        function render$1() {
          return _render2.apply(this, arguments);
        }
        return render$1;
      }()
    }, {
      key: "emit",
      value: function emit(type, event) {
        this._ee.emit(type, event);
      }
    }, {
      key: "on",
      value: function on(type, listener) {
        this._ee.on(type, listener);
      }
    }, {
      key: "off",
      value: function off(type, listener) {
        this._ee.off(type, listener);
      }
    }, {
      key: "getCanvasEl",
      value: function getCanvasEl() {
        return this.canvasElement;
      }
    }, {
      key: "resize",
      value: function resize(width, height) {
        var canvas = this.canvas;
        canvas.resize(width, height);
      }
    }, {
      key: "toRawChildren",
      value: function toRawChildren(children) {
        return children;
      }
    }, {
      key: "destroy",
      value: function destroy() {
        var canvas = this.canvas;
        // 销毁也需要等 ready
        canvas.ready.then(function () {
          canvas.destroy();
        });
        this.props = null;
        this.context = null;
        this.updater = null;
        this.theme = null;
        this.canvas = null;
        this.container = null;
        this.canvasElement = null;
        this.vNode = null;
      }
    }]);
    return Canvas$1;
  }();

  function createRef() {
    var ref = {
      current: null
    };
    return ref;
  }

  var Timeline = /*#__PURE__*/function (_Component) {
    _inherits(Timeline, _Component);
    var _super = _createSuper(Timeline);
    function Timeline(props) {
      var _this;
      _classCallCheck(this, Timeline);
      _this = _super.call(this, props);
      _this.next = function () {
        var _assertThisInitialize = _assertThisInitialized$1(_this),
          state = _assertThisInitialize.state,
          props = _assertThisInitialize.props;
        var index = state.index,
          count = state.count,
          delay = state.delay;
        var loop = props.loop;
        var next = loop ? (index + 1) % count : index + 1;
        if (next < count) {
          setTimeout(function () {
            _this.setState({
              index: next
            });
          }, delay || 0);
        }
      };
      var delay = props.delay,
        _props$start = props.start,
        start = _props$start === void 0 ? 0 : _props$start,
        children = props.children;
      var count = Children.toArray(children).length;
      _this.state = {
        delay: delay,
        count: count,
        index: start
      };
      return _this;
    }
    _createClass(Timeline, [{
      key: "didMount",
      value: function didMount() {
        this.animator.on('end', this.next);
      }
    }, {
      key: "didUnmount",
      value: function didUnmount() {
        this.animator.off('end', this.next);
      }
    }, {
      key: "render",
      value: function render() {
        var state = this.state,
          props = this.props;
        var children = props.children;
        var index = state.index;
        var childrenArray = Children.toArray(children);
        return childrenArray[index];
      }
    }]);
    return Timeline;
  }(Component);

  var defineProperty = createCommonjsModule(function (module) {
  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }
    return obj;
  }
  module.exports = _defineProperty, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _defineProperty$1 = /*@__PURE__*/getDefaultExportFromCjs(defineProperty);

  var objectSpread2 = createCommonjsModule(function (module) {
  function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);
    if (Object.getOwnPropertySymbols) {
      var symbols = Object.getOwnPropertySymbols(object);
      enumerableOnly && (symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      })), keys.push.apply(keys, symbols);
    }
    return keys;
  }
  function _objectSpread2(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = null != arguments[i] ? arguments[i] : {};
      i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
        defineProperty(target, key, source[key]);
      }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
    return target;
  }
  module.exports = _objectSpread2, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _objectSpread = /*@__PURE__*/getDefaultExportFromCjs(objectSpread2);

  var classCallCheck = createCommonjsModule(function (module) {
  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }
  module.exports = _classCallCheck, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _classCallCheck$1 = /*@__PURE__*/getDefaultExportFromCjs(classCallCheck);

  var createClass = createCommonjsModule(function (module) {
  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }
  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    Object.defineProperty(Constructor, "prototype", {
      writable: false
    });
    return Constructor;
  }
  module.exports = _createClass, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _createClass$3 = /*@__PURE__*/getDefaultExportFromCjs(createClass);

  var setPrototypeOf = createCommonjsModule(function (module) {
  function _setPrototypeOf(o, p) {
    module.exports = _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    }, module.exports.__esModule = true, module.exports["default"] = module.exports;
    return _setPrototypeOf(o, p);
  }
  module.exports = _setPrototypeOf, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var inherits = createCommonjsModule(function (module) {
  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        writable: true,
        configurable: true
      }
    });
    Object.defineProperty(subClass, "prototype", {
      writable: false
    });
    if (superClass) setPrototypeOf(subClass, superClass);
  }
  module.exports = _inherits, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _inherits$1 = /*@__PURE__*/getDefaultExportFromCjs(inherits);

  var getPrototypeOf = createCommonjsModule(function (module) {
  function _getPrototypeOf(o) {
    module.exports = _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) {
      return o.__proto__ || Object.getPrototypeOf(o);
    }, module.exports.__esModule = true, module.exports["default"] = module.exports;
    return _getPrototypeOf(o);
  }
  module.exports = _getPrototypeOf, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _getPrototypeOf$1 = /*@__PURE__*/getDefaultExportFromCjs(getPrototypeOf);

  var isNativeReflectConstruct = createCommonjsModule(function (module) {
  function _isNativeReflectConstruct() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
      Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
      return true;
    } catch (e) {
      return false;
    }
  }
  module.exports = _isNativeReflectConstruct, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _typeof_1$1 = createCommonjsModule(function (module) {
  function _typeof(obj) {
    "@babel/helpers - typeof";

    return (module.exports = _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) {
      return typeof obj;
    } : function (obj) {
      return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    }, module.exports.__esModule = true, module.exports["default"] = module.exports), _typeof(obj);
  }
  module.exports = _typeof, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _typeof$1 = /*@__PURE__*/getDefaultExportFromCjs(_typeof_1$1);

  var assertThisInitialized = createCommonjsModule(function (module) {
  function _assertThisInitialized(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
  }
  module.exports = _assertThisInitialized, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _assertThisInitialized$3 = /*@__PURE__*/getDefaultExportFromCjs(assertThisInitialized);

  var possibleConstructorReturn = createCommonjsModule(function (module) {
  var _typeof = _typeof_1$1["default"];

  function _possibleConstructorReturn(self, call) {
    if (call && (_typeof(call) === "object" || typeof call === "function")) {
      return call;
    } else if (call !== void 0) {
      throw new TypeError("Derived constructors may only return object or undefined");
    }
    return assertThisInitialized(self);
  }
  module.exports = _possibleConstructorReturn, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _possibleConstructorReturn$1 = /*@__PURE__*/getDefaultExportFromCjs(possibleConstructorReturn);

  var createSuper = createCommonjsModule(function (module) {
  function _createSuper(Derived) {
    var hasNativeReflectConstruct = isNativeReflectConstruct();
    return function _createSuperInternal() {
      var Super = getPrototypeOf(Derived),
        result;
      if (hasNativeReflectConstruct) {
        var NewTarget = getPrototypeOf(this).constructor;
        result = Reflect.construct(Super, arguments, NewTarget);
      } else {
        result = Super.apply(this, arguments);
      }
      return possibleConstructorReturn(this, result);
    };
  }
  module.exports = _createSuper, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _createSuper$1 = /*@__PURE__*/getDefaultExportFromCjs(createSuper);

  /**
   * Common utilities
   * @module glMatrix
   */
  // Configuration Constants
  var EPSILON$3 = 0.000001;
  var ARRAY_TYPE$1 = typeof Float32Array !== 'undefined' ? Float32Array : Array;
  if (!Math.hypot) Math.hypot = function () {
    var y = 0,
        i = arguments.length;

    while (i--) {
      y += arguments[i] * arguments[i];
    }

    return Math.sqrt(y);
  };

  /**
   * Rotates a mat2d by the given angle
   *
   * @param {mat2d} out the receiving matrix
   * @param {ReadonlyMat2d} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat2d} out
   */

  function rotate$2(out, a, rad) {
    var a0 = a[0],
        a1 = a[1],
        a2 = a[2],
        a3 = a[3],
        a4 = a[4],
        a5 = a[5];
    var s = Math.sin(rad);
    var c = Math.cos(rad);
    out[0] = a0 * c + a2 * s;
    out[1] = a1 * c + a3 * s;
    out[2] = a0 * -s + a2 * c;
    out[3] = a1 * -s + a3 * c;
    out[4] = a4;
    out[5] = a5;
    return out;
  }

  /**
   * 3x3 Matrix
   * @module mat3
   */

  /**
   * Creates a new identity mat3
   *
   * @returns {mat3} a new 3x3 matrix
   */

  function create$6() {
    var out = new ARRAY_TYPE$1(9);

    if (ARRAY_TYPE$1 != Float32Array) {
      out[1] = 0;
      out[2] = 0;
      out[3] = 0;
      out[5] = 0;
      out[6] = 0;
      out[7] = 0;
    }

    out[0] = 1;
    out[4] = 1;
    out[8] = 1;
    return out;
  }
  /**
   * Copies the upper-left 3x3 values into the given mat3.
   *
   * @param {mat3} out the receiving 3x3 matrix
   * @param {ReadonlyMat4} a   the source 4x4 matrix
   * @returns {mat3} out
   */

  function fromMat4$1(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[4];
    out[4] = a[5];
    out[5] = a[6];
    out[6] = a[8];
    out[7] = a[9];
    out[8] = a[10];
    return out;
  }
  /**
   * Create a new mat3 with the given values
   *
   * @param {Number} m00 Component in column 0, row 0 position (index 0)
   * @param {Number} m01 Component in column 0, row 1 position (index 1)
   * @param {Number} m02 Component in column 0, row 2 position (index 2)
   * @param {Number} m10 Component in column 1, row 0 position (index 3)
   * @param {Number} m11 Component in column 1, row 1 position (index 4)
   * @param {Number} m12 Component in column 1, row 2 position (index 5)
   * @param {Number} m20 Component in column 2, row 0 position (index 6)
   * @param {Number} m21 Component in column 2, row 1 position (index 7)
   * @param {Number} m22 Component in column 2, row 2 position (index 8)
   * @returns {mat3} A new mat3
   */

  function fromValues$6(m00, m01, m02, m10, m11, m12, m20, m21, m22) {
    var out = new ARRAY_TYPE$1(9);
    out[0] = m00;
    out[1] = m01;
    out[2] = m02;
    out[3] = m10;
    out[4] = m11;
    out[5] = m12;
    out[6] = m20;
    out[7] = m21;
    out[8] = m22;
    return out;
  }

  /**
   * 4x4 Matrix<br>Format: column-major, when typed out it looks like row-major<br>The matrices are being post multiplied.
   * @module mat4
   */

  /**
   * Creates a new identity mat4
   *
   * @returns {mat4} a new 4x4 matrix
   */

  function create$7() {
    var out = new ARRAY_TYPE$1(16);

    if (ARRAY_TYPE$1 != Float32Array) {
      out[1] = 0;
      out[2] = 0;
      out[3] = 0;
      out[4] = 0;
      out[6] = 0;
      out[7] = 0;
      out[8] = 0;
      out[9] = 0;
      out[11] = 0;
      out[12] = 0;
      out[13] = 0;
      out[14] = 0;
    }

    out[0] = 1;
    out[5] = 1;
    out[10] = 1;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a new mat4 initialized with values from an existing matrix
   *
   * @param {ReadonlyMat4} a matrix to clone
   * @returns {mat4} a new 4x4 matrix
   */

  function clone$2(a) {
    var out = new ARRAY_TYPE$1(16);
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[3];
    out[4] = a[4];
    out[5] = a[5];
    out[6] = a[6];
    out[7] = a[7];
    out[8] = a[8];
    out[9] = a[9];
    out[10] = a[10];
    out[11] = a[11];
    out[12] = a[12];
    out[13] = a[13];
    out[14] = a[14];
    out[15] = a[15];
    return out;
  }
  /**
   * Copy the values from one mat4 to another
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the source matrix
   * @returns {mat4} out
   */

  function copy$5(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[3];
    out[4] = a[4];
    out[5] = a[5];
    out[6] = a[6];
    out[7] = a[7];
    out[8] = a[8];
    out[9] = a[9];
    out[10] = a[10];
    out[11] = a[11];
    out[12] = a[12];
    out[13] = a[13];
    out[14] = a[14];
    out[15] = a[15];
    return out;
  }
  /**
   * Create a new mat4 with the given values
   *
   * @param {Number} m00 Component in column 0, row 0 position (index 0)
   * @param {Number} m01 Component in column 0, row 1 position (index 1)
   * @param {Number} m02 Component in column 0, row 2 position (index 2)
   * @param {Number} m03 Component in column 0, row 3 position (index 3)
   * @param {Number} m10 Component in column 1, row 0 position (index 4)
   * @param {Number} m11 Component in column 1, row 1 position (index 5)
   * @param {Number} m12 Component in column 1, row 2 position (index 6)
   * @param {Number} m13 Component in column 1, row 3 position (index 7)
   * @param {Number} m20 Component in column 2, row 0 position (index 8)
   * @param {Number} m21 Component in column 2, row 1 position (index 9)
   * @param {Number} m22 Component in column 2, row 2 position (index 10)
   * @param {Number} m23 Component in column 2, row 3 position (index 11)
   * @param {Number} m30 Component in column 3, row 0 position (index 12)
   * @param {Number} m31 Component in column 3, row 1 position (index 13)
   * @param {Number} m32 Component in column 3, row 2 position (index 14)
   * @param {Number} m33 Component in column 3, row 3 position (index 15)
   * @returns {mat4} A new mat4
   */

  function fromValues$7(m00, m01, m02, m03, m10, m11, m12, m13, m20, m21, m22, m23, m30, m31, m32, m33) {
    var out = new ARRAY_TYPE$1(16);
    out[0] = m00;
    out[1] = m01;
    out[2] = m02;
    out[3] = m03;
    out[4] = m10;
    out[5] = m11;
    out[6] = m12;
    out[7] = m13;
    out[8] = m20;
    out[9] = m21;
    out[10] = m22;
    out[11] = m23;
    out[12] = m30;
    out[13] = m31;
    out[14] = m32;
    out[15] = m33;
    return out;
  }
  /**
   * Set the components of a mat4 to the given values
   *
   * @param {mat4} out the receiving matrix
   * @param {Number} m00 Component in column 0, row 0 position (index 0)
   * @param {Number} m01 Component in column 0, row 1 position (index 1)
   * @param {Number} m02 Component in column 0, row 2 position (index 2)
   * @param {Number} m03 Component in column 0, row 3 position (index 3)
   * @param {Number} m10 Component in column 1, row 0 position (index 4)
   * @param {Number} m11 Component in column 1, row 1 position (index 5)
   * @param {Number} m12 Component in column 1, row 2 position (index 6)
   * @param {Number} m13 Component in column 1, row 3 position (index 7)
   * @param {Number} m20 Component in column 2, row 0 position (index 8)
   * @param {Number} m21 Component in column 2, row 1 position (index 9)
   * @param {Number} m22 Component in column 2, row 2 position (index 10)
   * @param {Number} m23 Component in column 2, row 3 position (index 11)
   * @param {Number} m30 Component in column 3, row 0 position (index 12)
   * @param {Number} m31 Component in column 3, row 1 position (index 13)
   * @param {Number} m32 Component in column 3, row 2 position (index 14)
   * @param {Number} m33 Component in column 3, row 3 position (index 15)
   * @returns {mat4} out
   */

  function set$2(out, m00, m01, m02, m03, m10, m11, m12, m13, m20, m21, m22, m23, m30, m31, m32, m33) {
    out[0] = m00;
    out[1] = m01;
    out[2] = m02;
    out[3] = m03;
    out[4] = m10;
    out[5] = m11;
    out[6] = m12;
    out[7] = m13;
    out[8] = m20;
    out[9] = m21;
    out[10] = m22;
    out[11] = m23;
    out[12] = m30;
    out[13] = m31;
    out[14] = m32;
    out[15] = m33;
    return out;
  }
  /**
   * Set a mat4 to the identity matrix
   *
   * @param {mat4} out the receiving matrix
   * @returns {mat4} out
   */

  function identity$1(out) {
    out[0] = 1;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = 1;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 1;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Transpose the values of a mat4
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the source matrix
   * @returns {mat4} out
   */

  function transpose$1(out, a) {
    // If we are transposing ourselves we can skip a few steps but have to cache some values
    if (out === a) {
      var a01 = a[1],
          a02 = a[2],
          a03 = a[3];
      var a12 = a[6],
          a13 = a[7];
      var a23 = a[11];
      out[1] = a[4];
      out[2] = a[8];
      out[3] = a[12];
      out[4] = a01;
      out[6] = a[9];
      out[7] = a[13];
      out[8] = a02;
      out[9] = a12;
      out[11] = a[14];
      out[12] = a03;
      out[13] = a13;
      out[14] = a23;
    } else {
      out[0] = a[0];
      out[1] = a[4];
      out[2] = a[8];
      out[3] = a[12];
      out[4] = a[1];
      out[5] = a[5];
      out[6] = a[9];
      out[7] = a[13];
      out[8] = a[2];
      out[9] = a[6];
      out[10] = a[10];
      out[11] = a[14];
      out[12] = a[3];
      out[13] = a[7];
      out[14] = a[11];
      out[15] = a[15];
    }

    return out;
  }
  /**
   * Inverts a mat4
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the source matrix
   * @returns {mat4} out
   */

  function invert$2(out, a) {
    var a00 = a[0],
        a01 = a[1],
        a02 = a[2],
        a03 = a[3];
    var a10 = a[4],
        a11 = a[5],
        a12 = a[6],
        a13 = a[7];
    var a20 = a[8],
        a21 = a[9],
        a22 = a[10],
        a23 = a[11];
    var a30 = a[12],
        a31 = a[13],
        a32 = a[14],
        a33 = a[15];
    var b00 = a00 * a11 - a01 * a10;
    var b01 = a00 * a12 - a02 * a10;
    var b02 = a00 * a13 - a03 * a10;
    var b03 = a01 * a12 - a02 * a11;
    var b04 = a01 * a13 - a03 * a11;
    var b05 = a02 * a13 - a03 * a12;
    var b06 = a20 * a31 - a21 * a30;
    var b07 = a20 * a32 - a22 * a30;
    var b08 = a20 * a33 - a23 * a30;
    var b09 = a21 * a32 - a22 * a31;
    var b10 = a21 * a33 - a23 * a31;
    var b11 = a22 * a33 - a23 * a32; // Calculate the determinant

    var det = b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;

    if (!det) {
      return null;
    }

    det = 1.0 / det;
    out[0] = (a11 * b11 - a12 * b10 + a13 * b09) * det;
    out[1] = (a02 * b10 - a01 * b11 - a03 * b09) * det;
    out[2] = (a31 * b05 - a32 * b04 + a33 * b03) * det;
    out[3] = (a22 * b04 - a21 * b05 - a23 * b03) * det;
    out[4] = (a12 * b08 - a10 * b11 - a13 * b07) * det;
    out[5] = (a00 * b11 - a02 * b08 + a03 * b07) * det;
    out[6] = (a32 * b02 - a30 * b05 - a33 * b01) * det;
    out[7] = (a20 * b05 - a22 * b02 + a23 * b01) * det;
    out[8] = (a10 * b10 - a11 * b08 + a13 * b06) * det;
    out[9] = (a01 * b08 - a00 * b10 - a03 * b06) * det;
    out[10] = (a30 * b04 - a31 * b02 + a33 * b00) * det;
    out[11] = (a21 * b02 - a20 * b04 - a23 * b00) * det;
    out[12] = (a11 * b07 - a10 * b09 - a12 * b06) * det;
    out[13] = (a00 * b09 - a01 * b07 + a02 * b06) * det;
    out[14] = (a31 * b01 - a30 * b03 - a32 * b00) * det;
    out[15] = (a20 * b03 - a21 * b01 + a22 * b00) * det;
    return out;
  }
  /**
   * Calculates the adjugate of a mat4
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the source matrix
   * @returns {mat4} out
   */

  function adjoint$1(out, a) {
    var a00 = a[0],
        a01 = a[1],
        a02 = a[2],
        a03 = a[3];
    var a10 = a[4],
        a11 = a[5],
        a12 = a[6],
        a13 = a[7];
    var a20 = a[8],
        a21 = a[9],
        a22 = a[10],
        a23 = a[11];
    var a30 = a[12],
        a31 = a[13],
        a32 = a[14],
        a33 = a[15];
    out[0] = a11 * (a22 * a33 - a23 * a32) - a21 * (a12 * a33 - a13 * a32) + a31 * (a12 * a23 - a13 * a22);
    out[1] = -(a01 * (a22 * a33 - a23 * a32) - a21 * (a02 * a33 - a03 * a32) + a31 * (a02 * a23 - a03 * a22));
    out[2] = a01 * (a12 * a33 - a13 * a32) - a11 * (a02 * a33 - a03 * a32) + a31 * (a02 * a13 - a03 * a12);
    out[3] = -(a01 * (a12 * a23 - a13 * a22) - a11 * (a02 * a23 - a03 * a22) + a21 * (a02 * a13 - a03 * a12));
    out[4] = -(a10 * (a22 * a33 - a23 * a32) - a20 * (a12 * a33 - a13 * a32) + a30 * (a12 * a23 - a13 * a22));
    out[5] = a00 * (a22 * a33 - a23 * a32) - a20 * (a02 * a33 - a03 * a32) + a30 * (a02 * a23 - a03 * a22);
    out[6] = -(a00 * (a12 * a33 - a13 * a32) - a10 * (a02 * a33 - a03 * a32) + a30 * (a02 * a13 - a03 * a12));
    out[7] = a00 * (a12 * a23 - a13 * a22) - a10 * (a02 * a23 - a03 * a22) + a20 * (a02 * a13 - a03 * a12);
    out[8] = a10 * (a21 * a33 - a23 * a31) - a20 * (a11 * a33 - a13 * a31) + a30 * (a11 * a23 - a13 * a21);
    out[9] = -(a00 * (a21 * a33 - a23 * a31) - a20 * (a01 * a33 - a03 * a31) + a30 * (a01 * a23 - a03 * a21));
    out[10] = a00 * (a11 * a33 - a13 * a31) - a10 * (a01 * a33 - a03 * a31) + a30 * (a01 * a13 - a03 * a11);
    out[11] = -(a00 * (a11 * a23 - a13 * a21) - a10 * (a01 * a23 - a03 * a21) + a20 * (a01 * a13 - a03 * a11));
    out[12] = -(a10 * (a21 * a32 - a22 * a31) - a20 * (a11 * a32 - a12 * a31) + a30 * (a11 * a22 - a12 * a21));
    out[13] = a00 * (a21 * a32 - a22 * a31) - a20 * (a01 * a32 - a02 * a31) + a30 * (a01 * a22 - a02 * a21);
    out[14] = -(a00 * (a11 * a32 - a12 * a31) - a10 * (a01 * a32 - a02 * a31) + a30 * (a01 * a12 - a02 * a11));
    out[15] = a00 * (a11 * a22 - a12 * a21) - a10 * (a01 * a22 - a02 * a21) + a20 * (a01 * a12 - a02 * a11);
    return out;
  }
  /**
   * Calculates the determinant of a mat4
   *
   * @param {ReadonlyMat4} a the source matrix
   * @returns {Number} determinant of a
   */

  function determinant$1(a) {
    var a00 = a[0],
        a01 = a[1],
        a02 = a[2],
        a03 = a[3];
    var a10 = a[4],
        a11 = a[5],
        a12 = a[6],
        a13 = a[7];
    var a20 = a[8],
        a21 = a[9],
        a22 = a[10],
        a23 = a[11];
    var a30 = a[12],
        a31 = a[13],
        a32 = a[14],
        a33 = a[15];
    var b00 = a00 * a11 - a01 * a10;
    var b01 = a00 * a12 - a02 * a10;
    var b02 = a00 * a13 - a03 * a10;
    var b03 = a01 * a12 - a02 * a11;
    var b04 = a01 * a13 - a03 * a11;
    var b05 = a02 * a13 - a03 * a12;
    var b06 = a20 * a31 - a21 * a30;
    var b07 = a20 * a32 - a22 * a30;
    var b08 = a20 * a33 - a23 * a30;
    var b09 = a21 * a32 - a22 * a31;
    var b10 = a21 * a33 - a23 * a31;
    var b11 = a22 * a33 - a23 * a32; // Calculate the determinant

    return b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;
  }
  /**
   * Multiplies two mat4s
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the first operand
   * @param {ReadonlyMat4} b the second operand
   * @returns {mat4} out
   */

  function multiply$3(out, a, b) {
    var a00 = a[0],
        a01 = a[1],
        a02 = a[2],
        a03 = a[3];
    var a10 = a[4],
        a11 = a[5],
        a12 = a[6],
        a13 = a[7];
    var a20 = a[8],
        a21 = a[9],
        a22 = a[10],
        a23 = a[11];
    var a30 = a[12],
        a31 = a[13],
        a32 = a[14],
        a33 = a[15]; // Cache only the current line of the second matrix

    var b0 = b[0],
        b1 = b[1],
        b2 = b[2],
        b3 = b[3];
    out[0] = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
    out[1] = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
    out[2] = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
    out[3] = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
    b0 = b[4];
    b1 = b[5];
    b2 = b[6];
    b3 = b[7];
    out[4] = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
    out[5] = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
    out[6] = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
    out[7] = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
    b0 = b[8];
    b1 = b[9];
    b2 = b[10];
    b3 = b[11];
    out[8] = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
    out[9] = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
    out[10] = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
    out[11] = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
    b0 = b[12];
    b1 = b[13];
    b2 = b[14];
    b3 = b[15];
    out[12] = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
    out[13] = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
    out[14] = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
    out[15] = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
    return out;
  }
  /**
   * Translate a mat4 by the given vector
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to translate
   * @param {ReadonlyVec3} v vector to translate by
   * @returns {mat4} out
   */

  function translate$1(out, a, v) {
    var x = v[0],
        y = v[1],
        z = v[2];
    var a00, a01, a02, a03;
    var a10, a11, a12, a13;
    var a20, a21, a22, a23;

    if (a === out) {
      out[12] = a[0] * x + a[4] * y + a[8] * z + a[12];
      out[13] = a[1] * x + a[5] * y + a[9] * z + a[13];
      out[14] = a[2] * x + a[6] * y + a[10] * z + a[14];
      out[15] = a[3] * x + a[7] * y + a[11] * z + a[15];
    } else {
      a00 = a[0];
      a01 = a[1];
      a02 = a[2];
      a03 = a[3];
      a10 = a[4];
      a11 = a[5];
      a12 = a[6];
      a13 = a[7];
      a20 = a[8];
      a21 = a[9];
      a22 = a[10];
      a23 = a[11];
      out[0] = a00;
      out[1] = a01;
      out[2] = a02;
      out[3] = a03;
      out[4] = a10;
      out[5] = a11;
      out[6] = a12;
      out[7] = a13;
      out[8] = a20;
      out[9] = a21;
      out[10] = a22;
      out[11] = a23;
      out[12] = a00 * x + a10 * y + a20 * z + a[12];
      out[13] = a01 * x + a11 * y + a21 * z + a[13];
      out[14] = a02 * x + a12 * y + a22 * z + a[14];
      out[15] = a03 * x + a13 * y + a23 * z + a[15];
    }

    return out;
  }
  /**
   * Scales the mat4 by the dimensions in the given vec3 not using vectorization
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to scale
   * @param {ReadonlyVec3} v the vec3 to scale the matrix by
   * @returns {mat4} out
   **/

  function scale$3(out, a, v) {
    var x = v[0],
        y = v[1],
        z = v[2];
    out[0] = a[0] * x;
    out[1] = a[1] * x;
    out[2] = a[2] * x;
    out[3] = a[3] * x;
    out[4] = a[4] * y;
    out[5] = a[5] * y;
    out[6] = a[6] * y;
    out[7] = a[7] * y;
    out[8] = a[8] * z;
    out[9] = a[9] * z;
    out[10] = a[10] * z;
    out[11] = a[11] * z;
    out[12] = a[12];
    out[13] = a[13];
    out[14] = a[14];
    out[15] = a[15];
    return out;
  }
  /**
   * Rotates a mat4 by the given angle around the given axis
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @param {ReadonlyVec3} axis the axis to rotate around
   * @returns {mat4} out
   */

  function rotate$3(out, a, rad, axis) {
    var x = axis[0],
        y = axis[1],
        z = axis[2];
    var len = Math.hypot(x, y, z);
    var s, c, t;
    var a00, a01, a02, a03;
    var a10, a11, a12, a13;
    var a20, a21, a22, a23;
    var b00, b01, b02;
    var b10, b11, b12;
    var b20, b21, b22;

    if (len < EPSILON$3) {
      return null;
    }

    len = 1 / len;
    x *= len;
    y *= len;
    z *= len;
    s = Math.sin(rad);
    c = Math.cos(rad);
    t = 1 - c;
    a00 = a[0];
    a01 = a[1];
    a02 = a[2];
    a03 = a[3];
    a10 = a[4];
    a11 = a[5];
    a12 = a[6];
    a13 = a[7];
    a20 = a[8];
    a21 = a[9];
    a22 = a[10];
    a23 = a[11]; // Construct the elements of the rotation matrix

    b00 = x * x * t + c;
    b01 = y * x * t + z * s;
    b02 = z * x * t - y * s;
    b10 = x * y * t - z * s;
    b11 = y * y * t + c;
    b12 = z * y * t + x * s;
    b20 = x * z * t + y * s;
    b21 = y * z * t - x * s;
    b22 = z * z * t + c; // Perform rotation-specific matrix multiplication

    out[0] = a00 * b00 + a10 * b01 + a20 * b02;
    out[1] = a01 * b00 + a11 * b01 + a21 * b02;
    out[2] = a02 * b00 + a12 * b01 + a22 * b02;
    out[3] = a03 * b00 + a13 * b01 + a23 * b02;
    out[4] = a00 * b10 + a10 * b11 + a20 * b12;
    out[5] = a01 * b10 + a11 * b11 + a21 * b12;
    out[6] = a02 * b10 + a12 * b11 + a22 * b12;
    out[7] = a03 * b10 + a13 * b11 + a23 * b12;
    out[8] = a00 * b20 + a10 * b21 + a20 * b22;
    out[9] = a01 * b20 + a11 * b21 + a21 * b22;
    out[10] = a02 * b20 + a12 * b21 + a22 * b22;
    out[11] = a03 * b20 + a13 * b21 + a23 * b22;

    if (a !== out) {
      // If the source and destination differ, copy the unchanged last row
      out[12] = a[12];
      out[13] = a[13];
      out[14] = a[14];
      out[15] = a[15];
    }

    return out;
  }
  /**
   * Rotates a matrix by the given angle around the X axis
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function rotateX$1(out, a, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad);
    var a10 = a[4];
    var a11 = a[5];
    var a12 = a[6];
    var a13 = a[7];
    var a20 = a[8];
    var a21 = a[9];
    var a22 = a[10];
    var a23 = a[11];

    if (a !== out) {
      // If the source and destination differ, copy the unchanged rows
      out[0] = a[0];
      out[1] = a[1];
      out[2] = a[2];
      out[3] = a[3];
      out[12] = a[12];
      out[13] = a[13];
      out[14] = a[14];
      out[15] = a[15];
    } // Perform axis-specific matrix multiplication


    out[4] = a10 * c + a20 * s;
    out[5] = a11 * c + a21 * s;
    out[6] = a12 * c + a22 * s;
    out[7] = a13 * c + a23 * s;
    out[8] = a20 * c - a10 * s;
    out[9] = a21 * c - a11 * s;
    out[10] = a22 * c - a12 * s;
    out[11] = a23 * c - a13 * s;
    return out;
  }
  /**
   * Rotates a matrix by the given angle around the Y axis
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function rotateY$1(out, a, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad);
    var a00 = a[0];
    var a01 = a[1];
    var a02 = a[2];
    var a03 = a[3];
    var a20 = a[8];
    var a21 = a[9];
    var a22 = a[10];
    var a23 = a[11];

    if (a !== out) {
      // If the source and destination differ, copy the unchanged rows
      out[4] = a[4];
      out[5] = a[5];
      out[6] = a[6];
      out[7] = a[7];
      out[12] = a[12];
      out[13] = a[13];
      out[14] = a[14];
      out[15] = a[15];
    } // Perform axis-specific matrix multiplication


    out[0] = a00 * c - a20 * s;
    out[1] = a01 * c - a21 * s;
    out[2] = a02 * c - a22 * s;
    out[3] = a03 * c - a23 * s;
    out[8] = a00 * s + a20 * c;
    out[9] = a01 * s + a21 * c;
    out[10] = a02 * s + a22 * c;
    out[11] = a03 * s + a23 * c;
    return out;
  }
  /**
   * Rotates a matrix by the given angle around the Z axis
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to rotate
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function rotateZ$1(out, a, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad);
    var a00 = a[0];
    var a01 = a[1];
    var a02 = a[2];
    var a03 = a[3];
    var a10 = a[4];
    var a11 = a[5];
    var a12 = a[6];
    var a13 = a[7];

    if (a !== out) {
      // If the source and destination differ, copy the unchanged last row
      out[8] = a[8];
      out[9] = a[9];
      out[10] = a[10];
      out[11] = a[11];
      out[12] = a[12];
      out[13] = a[13];
      out[14] = a[14];
      out[15] = a[15];
    } // Perform axis-specific matrix multiplication


    out[0] = a00 * c + a10 * s;
    out[1] = a01 * c + a11 * s;
    out[2] = a02 * c + a12 * s;
    out[3] = a03 * c + a13 * s;
    out[4] = a10 * c - a00 * s;
    out[5] = a11 * c - a01 * s;
    out[6] = a12 * c - a02 * s;
    out[7] = a13 * c - a03 * s;
    return out;
  }
  /**
   * Creates a matrix from a vector translation
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.translate(dest, dest, vec);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {ReadonlyVec3} v Translation vector
   * @returns {mat4} out
   */

  function fromTranslation$1(out, v) {
    out[0] = 1;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = 1;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 1;
    out[11] = 0;
    out[12] = v[0];
    out[13] = v[1];
    out[14] = v[2];
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from a vector scaling
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.scale(dest, dest, vec);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {ReadonlyVec3} v Scaling vector
   * @returns {mat4} out
   */

  function fromScaling$1(out, v) {
    out[0] = v[0];
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = v[1];
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = v[2];
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from a given angle around a given axis
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.rotate(dest, dest, rad, axis);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {Number} rad the angle to rotate the matrix by
   * @param {ReadonlyVec3} axis the axis to rotate around
   * @returns {mat4} out
   */

  function fromRotation$1(out, rad, axis) {
    var x = axis[0],
        y = axis[1],
        z = axis[2];
    var len = Math.hypot(x, y, z);
    var s, c, t;

    if (len < EPSILON$3) {
      return null;
    }

    len = 1 / len;
    x *= len;
    y *= len;
    z *= len;
    s = Math.sin(rad);
    c = Math.cos(rad);
    t = 1 - c; // Perform rotation-specific matrix multiplication

    out[0] = x * x * t + c;
    out[1] = y * x * t + z * s;
    out[2] = z * x * t - y * s;
    out[3] = 0;
    out[4] = x * y * t - z * s;
    out[5] = y * y * t + c;
    out[6] = z * y * t + x * s;
    out[7] = 0;
    out[8] = x * z * t + y * s;
    out[9] = y * z * t - x * s;
    out[10] = z * z * t + c;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from the given angle around the X axis
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.rotateX(dest, dest, rad);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function fromXRotation$1(out, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad); // Perform axis-specific matrix multiplication

    out[0] = 1;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = c;
    out[6] = s;
    out[7] = 0;
    out[8] = 0;
    out[9] = -s;
    out[10] = c;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from the given angle around the Y axis
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.rotateY(dest, dest, rad);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function fromYRotation$1(out, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad); // Perform axis-specific matrix multiplication

    out[0] = c;
    out[1] = 0;
    out[2] = -s;
    out[3] = 0;
    out[4] = 0;
    out[5] = 1;
    out[6] = 0;
    out[7] = 0;
    out[8] = s;
    out[9] = 0;
    out[10] = c;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from the given angle around the Z axis
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.rotateZ(dest, dest, rad);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {Number} rad the angle to rotate the matrix by
   * @returns {mat4} out
   */

  function fromZRotation$1(out, rad) {
    var s = Math.sin(rad);
    var c = Math.cos(rad); // Perform axis-specific matrix multiplication

    out[0] = c;
    out[1] = s;
    out[2] = 0;
    out[3] = 0;
    out[4] = -s;
    out[5] = c;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 1;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from a quaternion rotation and vector translation
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.translate(dest, vec);
   *     let quatMat = mat4.create();
   *     quat4.toMat4(quat, quatMat);
   *     mat4.multiply(dest, quatMat);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {quat4} q Rotation quaternion
   * @param {ReadonlyVec3} v Translation vector
   * @returns {mat4} out
   */

  function fromRotationTranslation$1(out, q, v) {
    // Quaternion math
    var x = q[0],
        y = q[1],
        z = q[2],
        w = q[3];
    var x2 = x + x;
    var y2 = y + y;
    var z2 = z + z;
    var xx = x * x2;
    var xy = x * y2;
    var xz = x * z2;
    var yy = y * y2;
    var yz = y * z2;
    var zz = z * z2;
    var wx = w * x2;
    var wy = w * y2;
    var wz = w * z2;
    out[0] = 1 - (yy + zz);
    out[1] = xy + wz;
    out[2] = xz - wy;
    out[3] = 0;
    out[4] = xy - wz;
    out[5] = 1 - (xx + zz);
    out[6] = yz + wx;
    out[7] = 0;
    out[8] = xz + wy;
    out[9] = yz - wx;
    out[10] = 1 - (xx + yy);
    out[11] = 0;
    out[12] = v[0];
    out[13] = v[1];
    out[14] = v[2];
    out[15] = 1;
    return out;
  }
  /**
   * Creates a new mat4 from a dual quat.
   *
   * @param {mat4} out Matrix
   * @param {ReadonlyQuat2} a Dual Quaternion
   * @returns {mat4} mat4 receiving operation result
   */

  function fromQuat2$1(out, a) {
    var translation = new ARRAY_TYPE$1(3);
    var bx = -a[0],
        by = -a[1],
        bz = -a[2],
        bw = a[3],
        ax = a[4],
        ay = a[5],
        az = a[6],
        aw = a[7];
    var magnitude = bx * bx + by * by + bz * bz + bw * bw; //Only scale if it makes sense

    if (magnitude > 0) {
      translation[0] = (ax * bw + aw * bx + ay * bz - az * by) * 2 / magnitude;
      translation[1] = (ay * bw + aw * by + az * bx - ax * bz) * 2 / magnitude;
      translation[2] = (az * bw + aw * bz + ax * by - ay * bx) * 2 / magnitude;
    } else {
      translation[0] = (ax * bw + aw * bx + ay * bz - az * by) * 2;
      translation[1] = (ay * bw + aw * by + az * bx - ax * bz) * 2;
      translation[2] = (az * bw + aw * bz + ax * by - ay * bx) * 2;
    }

    fromRotationTranslation$1(out, a, translation);
    return out;
  }
  /**
   * Returns the translation vector component of a transformation
   *  matrix. If a matrix is built with fromRotationTranslation,
   *  the returned vector will be the same as the translation vector
   *  originally supplied.
   * @param  {vec3} out Vector to receive translation component
   * @param  {ReadonlyMat4} mat Matrix to be decomposed (input)
   * @return {vec3} out
   */

  function getTranslation$1(out, mat) {
    out[0] = mat[12];
    out[1] = mat[13];
    out[2] = mat[14];
    return out;
  }
  /**
   * Returns the scaling factor component of a transformation
   *  matrix. If a matrix is built with fromRotationTranslationScale
   *  with a normalized Quaternion paramter, the returned vector will be
   *  the same as the scaling vector
   *  originally supplied.
   * @param  {vec3} out Vector to receive scaling factor component
   * @param  {ReadonlyMat4} mat Matrix to be decomposed (input)
   * @return {vec3} out
   */

  function getScaling$1(out, mat) {
    var m11 = mat[0];
    var m12 = mat[1];
    var m13 = mat[2];
    var m21 = mat[4];
    var m22 = mat[5];
    var m23 = mat[6];
    var m31 = mat[8];
    var m32 = mat[9];
    var m33 = mat[10];
    out[0] = Math.hypot(m11, m12, m13);
    out[1] = Math.hypot(m21, m22, m23);
    out[2] = Math.hypot(m31, m32, m33);
    return out;
  }
  /**
   * Returns a quaternion representing the rotational component
   *  of a transformation matrix. If a matrix is built with
   *  fromRotationTranslation, the returned quaternion will be the
   *  same as the quaternion originally supplied.
   * @param {quat} out Quaternion to receive the rotation component
   * @param {ReadonlyMat4} mat Matrix to be decomposed (input)
   * @return {quat} out
   */

  function getRotation$1(out, mat) {
    var scaling = new ARRAY_TYPE$1(3);
    getScaling$1(scaling, mat);
    var is1 = 1 / scaling[0];
    var is2 = 1 / scaling[1];
    var is3 = 1 / scaling[2];
    var sm11 = mat[0] * is1;
    var sm12 = mat[1] * is2;
    var sm13 = mat[2] * is3;
    var sm21 = mat[4] * is1;
    var sm22 = mat[5] * is2;
    var sm23 = mat[6] * is3;
    var sm31 = mat[8] * is1;
    var sm32 = mat[9] * is2;
    var sm33 = mat[10] * is3;
    var trace = sm11 + sm22 + sm33;
    var S = 0;

    if (trace > 0) {
      S = Math.sqrt(trace + 1.0) * 2;
      out[3] = 0.25 * S;
      out[0] = (sm23 - sm32) / S;
      out[1] = (sm31 - sm13) / S;
      out[2] = (sm12 - sm21) / S;
    } else if (sm11 > sm22 && sm11 > sm33) {
      S = Math.sqrt(1.0 + sm11 - sm22 - sm33) * 2;
      out[3] = (sm23 - sm32) / S;
      out[0] = 0.25 * S;
      out[1] = (sm12 + sm21) / S;
      out[2] = (sm31 + sm13) / S;
    } else if (sm22 > sm33) {
      S = Math.sqrt(1.0 + sm22 - sm11 - sm33) * 2;
      out[3] = (sm31 - sm13) / S;
      out[0] = (sm12 + sm21) / S;
      out[1] = 0.25 * S;
      out[2] = (sm23 + sm32) / S;
    } else {
      S = Math.sqrt(1.0 + sm33 - sm11 - sm22) * 2;
      out[3] = (sm12 - sm21) / S;
      out[0] = (sm31 + sm13) / S;
      out[1] = (sm23 + sm32) / S;
      out[2] = 0.25 * S;
    }

    return out;
  }
  /**
   * Creates a matrix from a quaternion rotation, vector translation and vector scale
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.translate(dest, vec);
   *     let quatMat = mat4.create();
   *     quat4.toMat4(quat, quatMat);
   *     mat4.multiply(dest, quatMat);
   *     mat4.scale(dest, scale)
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {quat4} q Rotation quaternion
   * @param {ReadonlyVec3} v Translation vector
   * @param {ReadonlyVec3} s Scaling vector
   * @returns {mat4} out
   */

  function fromRotationTranslationScale$2(out, q, v, s) {
    // Quaternion math
    var x = q[0],
        y = q[1],
        z = q[2],
        w = q[3];
    var x2 = x + x;
    var y2 = y + y;
    var z2 = z + z;
    var xx = x * x2;
    var xy = x * y2;
    var xz = x * z2;
    var yy = y * y2;
    var yz = y * z2;
    var zz = z * z2;
    var wx = w * x2;
    var wy = w * y2;
    var wz = w * z2;
    var sx = s[0];
    var sy = s[1];
    var sz = s[2];
    out[0] = (1 - (yy + zz)) * sx;
    out[1] = (xy + wz) * sx;
    out[2] = (xz - wy) * sx;
    out[3] = 0;
    out[4] = (xy - wz) * sy;
    out[5] = (1 - (xx + zz)) * sy;
    out[6] = (yz + wx) * sy;
    out[7] = 0;
    out[8] = (xz + wy) * sz;
    out[9] = (yz - wx) * sz;
    out[10] = (1 - (xx + yy)) * sz;
    out[11] = 0;
    out[12] = v[0];
    out[13] = v[1];
    out[14] = v[2];
    out[15] = 1;
    return out;
  }
  /**
   * Creates a matrix from a quaternion rotation, vector translation and vector scale, rotating and scaling around the given origin
   * This is equivalent to (but much faster than):
   *
   *     mat4.identity(dest);
   *     mat4.translate(dest, vec);
   *     mat4.translate(dest, origin);
   *     let quatMat = mat4.create();
   *     quat4.toMat4(quat, quatMat);
   *     mat4.multiply(dest, quatMat);
   *     mat4.scale(dest, scale)
   *     mat4.translate(dest, negativeOrigin);
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {quat4} q Rotation quaternion
   * @param {ReadonlyVec3} v Translation vector
   * @param {ReadonlyVec3} s Scaling vector
   * @param {ReadonlyVec3} o The origin vector around which to scale and rotate
   * @returns {mat4} out
   */

  function fromRotationTranslationScaleOrigin$1(out, q, v, s, o) {
    // Quaternion math
    var x = q[0],
        y = q[1],
        z = q[2],
        w = q[3];
    var x2 = x + x;
    var y2 = y + y;
    var z2 = z + z;
    var xx = x * x2;
    var xy = x * y2;
    var xz = x * z2;
    var yy = y * y2;
    var yz = y * z2;
    var zz = z * z2;
    var wx = w * x2;
    var wy = w * y2;
    var wz = w * z2;
    var sx = s[0];
    var sy = s[1];
    var sz = s[2];
    var ox = o[0];
    var oy = o[1];
    var oz = o[2];
    var out0 = (1 - (yy + zz)) * sx;
    var out1 = (xy + wz) * sx;
    var out2 = (xz - wy) * sx;
    var out4 = (xy - wz) * sy;
    var out5 = (1 - (xx + zz)) * sy;
    var out6 = (yz + wx) * sy;
    var out8 = (xz + wy) * sz;
    var out9 = (yz - wx) * sz;
    var out10 = (1 - (xx + yy)) * sz;
    out[0] = out0;
    out[1] = out1;
    out[2] = out2;
    out[3] = 0;
    out[4] = out4;
    out[5] = out5;
    out[6] = out6;
    out[7] = 0;
    out[8] = out8;
    out[9] = out9;
    out[10] = out10;
    out[11] = 0;
    out[12] = v[0] + ox - (out0 * ox + out4 * oy + out8 * oz);
    out[13] = v[1] + oy - (out1 * ox + out5 * oy + out9 * oz);
    out[14] = v[2] + oz - (out2 * ox + out6 * oy + out10 * oz);
    out[15] = 1;
    return out;
  }
  /**
   * Calculates a 4x4 matrix from the given quaternion
   *
   * @param {mat4} out mat4 receiving operation result
   * @param {ReadonlyQuat} q Quaternion to create matrix from
   *
   * @returns {mat4} out
   */

  function fromQuat$1(out, q) {
    var x = q[0],
        y = q[1],
        z = q[2],
        w = q[3];
    var x2 = x + x;
    var y2 = y + y;
    var z2 = z + z;
    var xx = x * x2;
    var yx = y * x2;
    var yy = y * y2;
    var zx = z * x2;
    var zy = z * y2;
    var zz = z * z2;
    var wx = w * x2;
    var wy = w * y2;
    var wz = w * z2;
    out[0] = 1 - yy - zz;
    out[1] = yx + wz;
    out[2] = zx - wy;
    out[3] = 0;
    out[4] = yx - wz;
    out[5] = 1 - xx - zz;
    out[6] = zy + wx;
    out[7] = 0;
    out[8] = zx + wy;
    out[9] = zy - wx;
    out[10] = 1 - xx - yy;
    out[11] = 0;
    out[12] = 0;
    out[13] = 0;
    out[14] = 0;
    out[15] = 1;
    return out;
  }
  /**
   * Generates a frustum matrix with the given bounds
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {Number} left Left bound of the frustum
   * @param {Number} right Right bound of the frustum
   * @param {Number} bottom Bottom bound of the frustum
   * @param {Number} top Top bound of the frustum
   * @param {Number} near Near bound of the frustum
   * @param {Number} far Far bound of the frustum
   * @returns {mat4} out
   */

  function frustum$1(out, left, right, bottom, top, near, far) {
    var rl = 1 / (right - left);
    var tb = 1 / (top - bottom);
    var nf = 1 / (near - far);
    out[0] = near * 2 * rl;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = near * 2 * tb;
    out[6] = 0;
    out[7] = 0;
    out[8] = (right + left) * rl;
    out[9] = (top + bottom) * tb;
    out[10] = (far + near) * nf;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[14] = far * near * 2 * nf;
    out[15] = 0;
    return out;
  }
  /**
   * Generates a perspective projection matrix with the given bounds.
   * The near/far clip planes correspond to a normalized device coordinate Z range of [-1, 1],
   * which matches WebGL/OpenGL's clip volume.
   * Passing null/undefined/no value for far will generate infinite projection matrix.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {number} fovy Vertical field of view in radians
   * @param {number} aspect Aspect ratio. typically viewport width/height
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum, can be null or Infinity
   * @returns {mat4} out
   */

  function perspectiveNO$1(out, fovy, aspect, near, far) {
    var f = 1.0 / Math.tan(fovy / 2),
        nf;
    out[0] = f / aspect;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = f;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[15] = 0;

    if (far != null && far !== Infinity) {
      nf = 1 / (near - far);
      out[10] = (far + near) * nf;
      out[14] = 2 * far * near * nf;
    } else {
      out[10] = -1;
      out[14] = -2 * near;
    }

    return out;
  }
  /**
   * Alias for {@link mat4.perspectiveNO}
   * @function
   */

  var perspective$1 = perspectiveNO$1;
  /**
   * Generates a perspective projection matrix suitable for WebGPU with the given bounds.
   * The near/far clip planes correspond to a normalized device coordinate Z range of [0, 1],
   * which matches WebGPU/Vulkan/DirectX/Metal's clip volume.
   * Passing null/undefined/no value for far will generate infinite projection matrix.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {number} fovy Vertical field of view in radians
   * @param {number} aspect Aspect ratio. typically viewport width/height
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum, can be null or Infinity
   * @returns {mat4} out
   */

  function perspectiveZO$1(out, fovy, aspect, near, far) {
    var f = 1.0 / Math.tan(fovy / 2),
        nf;
    out[0] = f / aspect;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = f;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[15] = 0;

    if (far != null && far !== Infinity) {
      nf = 1 / (near - far);
      out[10] = far * nf;
      out[14] = far * near * nf;
    } else {
      out[10] = -1;
      out[14] = -near;
    }

    return out;
  }
  /**
   * Generates a perspective projection matrix with the given field of view.
   * This is primarily useful for generating projection matrices to be used
   * with the still experiemental WebVR API.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {Object} fov Object containing the following values: upDegrees, downDegrees, leftDegrees, rightDegrees
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum
   * @returns {mat4} out
   */

  function perspectiveFromFieldOfView$1(out, fov, near, far) {
    var upTan = Math.tan(fov.upDegrees * Math.PI / 180.0);
    var downTan = Math.tan(fov.downDegrees * Math.PI / 180.0);
    var leftTan = Math.tan(fov.leftDegrees * Math.PI / 180.0);
    var rightTan = Math.tan(fov.rightDegrees * Math.PI / 180.0);
    var xScale = 2.0 / (leftTan + rightTan);
    var yScale = 2.0 / (upTan + downTan);
    out[0] = xScale;
    out[1] = 0.0;
    out[2] = 0.0;
    out[3] = 0.0;
    out[4] = 0.0;
    out[5] = yScale;
    out[6] = 0.0;
    out[7] = 0.0;
    out[8] = -((leftTan - rightTan) * xScale * 0.5);
    out[9] = (upTan - downTan) * yScale * 0.5;
    out[10] = far / (near - far);
    out[11] = -1.0;
    out[12] = 0.0;
    out[13] = 0.0;
    out[14] = far * near / (near - far);
    out[15] = 0.0;
    return out;
  }
  /**
   * Generates a orthogonal projection matrix with the given bounds.
   * The near/far clip planes correspond to a normalized device coordinate Z range of [-1, 1],
   * which matches WebGL/OpenGL's clip volume.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {number} left Left bound of the frustum
   * @param {number} right Right bound of the frustum
   * @param {number} bottom Bottom bound of the frustum
   * @param {number} top Top bound of the frustum
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum
   * @returns {mat4} out
   */

  function orthoNO$1(out, left, right, bottom, top, near, far) {
    var lr = 1 / (left - right);
    var bt = 1 / (bottom - top);
    var nf = 1 / (near - far);
    out[0] = -2 * lr;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = -2 * bt;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = 2 * nf;
    out[11] = 0;
    out[12] = (left + right) * lr;
    out[13] = (top + bottom) * bt;
    out[14] = (far + near) * nf;
    out[15] = 1;
    return out;
  }
  /**
   * Alias for {@link mat4.orthoNO}
   * @function
   */

  var ortho$1 = orthoNO$1;
  /**
   * Generates a orthogonal projection matrix with the given bounds.
   * The near/far clip planes correspond to a normalized device coordinate Z range of [0, 1],
   * which matches WebGPU/Vulkan/DirectX/Metal's clip volume.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {number} left Left bound of the frustum
   * @param {number} right Right bound of the frustum
   * @param {number} bottom Bottom bound of the frustum
   * @param {number} top Top bound of the frustum
   * @param {number} near Near bound of the frustum
   * @param {number} far Far bound of the frustum
   * @returns {mat4} out
   */

  function orthoZO$1(out, left, right, bottom, top, near, far) {
    var lr = 1 / (left - right);
    var bt = 1 / (bottom - top);
    var nf = 1 / (near - far);
    out[0] = -2 * lr;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = -2 * bt;
    out[6] = 0;
    out[7] = 0;
    out[8] = 0;
    out[9] = 0;
    out[10] = nf;
    out[11] = 0;
    out[12] = (left + right) * lr;
    out[13] = (top + bottom) * bt;
    out[14] = near * nf;
    out[15] = 1;
    return out;
  }
  /**
   * Generates a look-at matrix with the given eye position, focal point, and up axis.
   * If you want a matrix that actually makes an object look at another object, you should use targetTo instead.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {ReadonlyVec3} eye Position of the viewer
   * @param {ReadonlyVec3} center Point the viewer is looking at
   * @param {ReadonlyVec3} up vec3 pointing up
   * @returns {mat4} out
   */

  function lookAt$1(out, eye, center, up) {
    var x0, x1, x2, y0, y1, y2, z0, z1, z2, len;
    var eyex = eye[0];
    var eyey = eye[1];
    var eyez = eye[2];
    var upx = up[0];
    var upy = up[1];
    var upz = up[2];
    var centerx = center[0];
    var centery = center[1];
    var centerz = center[2];

    if (Math.abs(eyex - centerx) < EPSILON$3 && Math.abs(eyey - centery) < EPSILON$3 && Math.abs(eyez - centerz) < EPSILON$3) {
      return identity$1(out);
    }

    z0 = eyex - centerx;
    z1 = eyey - centery;
    z2 = eyez - centerz;
    len = 1 / Math.hypot(z0, z1, z2);
    z0 *= len;
    z1 *= len;
    z2 *= len;
    x0 = upy * z2 - upz * z1;
    x1 = upz * z0 - upx * z2;
    x2 = upx * z1 - upy * z0;
    len = Math.hypot(x0, x1, x2);

    if (!len) {
      x0 = 0;
      x1 = 0;
      x2 = 0;
    } else {
      len = 1 / len;
      x0 *= len;
      x1 *= len;
      x2 *= len;
    }

    y0 = z1 * x2 - z2 * x1;
    y1 = z2 * x0 - z0 * x2;
    y2 = z0 * x1 - z1 * x0;
    len = Math.hypot(y0, y1, y2);

    if (!len) {
      y0 = 0;
      y1 = 0;
      y2 = 0;
    } else {
      len = 1 / len;
      y0 *= len;
      y1 *= len;
      y2 *= len;
    }

    out[0] = x0;
    out[1] = y0;
    out[2] = z0;
    out[3] = 0;
    out[4] = x1;
    out[5] = y1;
    out[6] = z1;
    out[7] = 0;
    out[8] = x2;
    out[9] = y2;
    out[10] = z2;
    out[11] = 0;
    out[12] = -(x0 * eyex + x1 * eyey + x2 * eyez);
    out[13] = -(y0 * eyex + y1 * eyey + y2 * eyez);
    out[14] = -(z0 * eyex + z1 * eyey + z2 * eyez);
    out[15] = 1;
    return out;
  }
  /**
   * Generates a matrix that makes something look at something else.
   *
   * @param {mat4} out mat4 frustum matrix will be written into
   * @param {ReadonlyVec3} eye Position of the viewer
   * @param {ReadonlyVec3} center Point the viewer is looking at
   * @param {ReadonlyVec3} up vec3 pointing up
   * @returns {mat4} out
   */

  function targetTo$1(out, eye, target, up) {
    var eyex = eye[0],
        eyey = eye[1],
        eyez = eye[2],
        upx = up[0],
        upy = up[1],
        upz = up[2];
    var z0 = eyex - target[0],
        z1 = eyey - target[1],
        z2 = eyez - target[2];
    var len = z0 * z0 + z1 * z1 + z2 * z2;

    if (len > 0) {
      len = 1 / Math.sqrt(len);
      z0 *= len;
      z1 *= len;
      z2 *= len;
    }

    var x0 = upy * z2 - upz * z1,
        x1 = upz * z0 - upx * z2,
        x2 = upx * z1 - upy * z0;
    len = x0 * x0 + x1 * x1 + x2 * x2;

    if (len > 0) {
      len = 1 / Math.sqrt(len);
      x0 *= len;
      x1 *= len;
      x2 *= len;
    }

    out[0] = x0;
    out[1] = x1;
    out[2] = x2;
    out[3] = 0;
    out[4] = z1 * x2 - z2 * x1;
    out[5] = z2 * x0 - z0 * x2;
    out[6] = z0 * x1 - z1 * x0;
    out[7] = 0;
    out[8] = z0;
    out[9] = z1;
    out[10] = z2;
    out[11] = 0;
    out[12] = eyex;
    out[13] = eyey;
    out[14] = eyez;
    out[15] = 1;
    return out;
  }
  /**
   * Returns a string representation of a mat4
   *
   * @param {ReadonlyMat4} a matrix to represent as a string
   * @returns {String} string representation of the matrix
   */

  function str$1(a) {
    return "mat4(" + a[0] + ", " + a[1] + ", " + a[2] + ", " + a[3] + ", " + a[4] + ", " + a[5] + ", " + a[6] + ", " + a[7] + ", " + a[8] + ", " + a[9] + ", " + a[10] + ", " + a[11] + ", " + a[12] + ", " + a[13] + ", " + a[14] + ", " + a[15] + ")";
  }
  /**
   * Returns Frobenius norm of a mat4
   *
   * @param {ReadonlyMat4} a the matrix to calculate Frobenius norm of
   * @returns {Number} Frobenius norm
   */

  function frob$1(a) {
    return Math.hypot(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12], a[13], a[14], a[15]);
  }
  /**
   * Adds two mat4's
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the first operand
   * @param {ReadonlyMat4} b the second operand
   * @returns {mat4} out
   */

  function add$3(out, a, b) {
    out[0] = a[0] + b[0];
    out[1] = a[1] + b[1];
    out[2] = a[2] + b[2];
    out[3] = a[3] + b[3];
    out[4] = a[4] + b[4];
    out[5] = a[5] + b[5];
    out[6] = a[6] + b[6];
    out[7] = a[7] + b[7];
    out[8] = a[8] + b[8];
    out[9] = a[9] + b[9];
    out[10] = a[10] + b[10];
    out[11] = a[11] + b[11];
    out[12] = a[12] + b[12];
    out[13] = a[13] + b[13];
    out[14] = a[14] + b[14];
    out[15] = a[15] + b[15];
    return out;
  }
  /**
   * Subtracts matrix b from matrix a
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the first operand
   * @param {ReadonlyMat4} b the second operand
   * @returns {mat4} out
   */

  function subtract$3(out, a, b) {
    out[0] = a[0] - b[0];
    out[1] = a[1] - b[1];
    out[2] = a[2] - b[2];
    out[3] = a[3] - b[3];
    out[4] = a[4] - b[4];
    out[5] = a[5] - b[5];
    out[6] = a[6] - b[6];
    out[7] = a[7] - b[7];
    out[8] = a[8] - b[8];
    out[9] = a[9] - b[9];
    out[10] = a[10] - b[10];
    out[11] = a[11] - b[11];
    out[12] = a[12] - b[12];
    out[13] = a[13] - b[13];
    out[14] = a[14] - b[14];
    out[15] = a[15] - b[15];
    return out;
  }
  /**
   * Multiply each element of the matrix by a scalar.
   *
   * @param {mat4} out the receiving matrix
   * @param {ReadonlyMat4} a the matrix to scale
   * @param {Number} b amount to scale the matrix's elements by
   * @returns {mat4} out
   */

  function multiplyScalar$1(out, a, b) {
    out[0] = a[0] * b;
    out[1] = a[1] * b;
    out[2] = a[2] * b;
    out[3] = a[3] * b;
    out[4] = a[4] * b;
    out[5] = a[5] * b;
    out[6] = a[6] * b;
    out[7] = a[7] * b;
    out[8] = a[8] * b;
    out[9] = a[9] * b;
    out[10] = a[10] * b;
    out[11] = a[11] * b;
    out[12] = a[12] * b;
    out[13] = a[13] * b;
    out[14] = a[14] * b;
    out[15] = a[15] * b;
    return out;
  }
  /**
   * Adds two mat4's after multiplying each element of the second operand by a scalar value.
   *
   * @param {mat4} out the receiving vector
   * @param {ReadonlyMat4} a the first operand
   * @param {ReadonlyMat4} b the second operand
   * @param {Number} scale the amount to scale b's elements by before adding
   * @returns {mat4} out
   */

  function multiplyScalarAndAdd$1(out, a, b, scale) {
    out[0] = a[0] + b[0] * scale;
    out[1] = a[1] + b[1] * scale;
    out[2] = a[2] + b[2] * scale;
    out[3] = a[3] + b[3] * scale;
    out[4] = a[4] + b[4] * scale;
    out[5] = a[5] + b[5] * scale;
    out[6] = a[6] + b[6] * scale;
    out[7] = a[7] + b[7] * scale;
    out[8] = a[8] + b[8] * scale;
    out[9] = a[9] + b[9] * scale;
    out[10] = a[10] + b[10] * scale;
    out[11] = a[11] + b[11] * scale;
    out[12] = a[12] + b[12] * scale;
    out[13] = a[13] + b[13] * scale;
    out[14] = a[14] + b[14] * scale;
    out[15] = a[15] + b[15] * scale;
    return out;
  }
  /**
   * Returns whether or not the matrices have exactly the same elements in the same position (when compared with ===)
   *
   * @param {ReadonlyMat4} a The first matrix.
   * @param {ReadonlyMat4} b The second matrix.
   * @returns {Boolean} True if the matrices are equal, false otherwise.
   */

  function exactEquals$2(a, b) {
    return a[0] === b[0] && a[1] === b[1] && a[2] === b[2] && a[3] === b[3] && a[4] === b[4] && a[5] === b[5] && a[6] === b[6] && a[7] === b[7] && a[8] === b[8] && a[9] === b[9] && a[10] === b[10] && a[11] === b[11] && a[12] === b[12] && a[13] === b[13] && a[14] === b[14] && a[15] === b[15];
  }
  /**
   * Returns whether or not the matrices have approximately the same elements in the same position.
   *
   * @param {ReadonlyMat4} a The first matrix.
   * @param {ReadonlyMat4} b The second matrix.
   * @returns {Boolean} True if the matrices are equal, false otherwise.
   */

  function equals$2(a, b) {
    var a0 = a[0],
        a1 = a[1],
        a2 = a[2],
        a3 = a[3];
    var a4 = a[4],
        a5 = a[5],
        a6 = a[6],
        a7 = a[7];
    var a8 = a[8],
        a9 = a[9],
        a10 = a[10],
        a11 = a[11];
    var a12 = a[12],
        a13 = a[13],
        a14 = a[14],
        a15 = a[15];
    var b0 = b[0],
        b1 = b[1],
        b2 = b[2],
        b3 = b[3];
    var b4 = b[4],
        b5 = b[5],
        b6 = b[6],
        b7 = b[7];
    var b8 = b[8],
        b9 = b[9],
        b10 = b[10],
        b11 = b[11];
    var b12 = b[12],
        b13 = b[13],
        b14 = b[14],
        b15 = b[15];
    return Math.abs(a0 - b0) <= EPSILON$3 * Math.max(1.0, Math.abs(a0), Math.abs(b0)) && Math.abs(a1 - b1) <= EPSILON$3 * Math.max(1.0, Math.abs(a1), Math.abs(b1)) && Math.abs(a2 - b2) <= EPSILON$3 * Math.max(1.0, Math.abs(a2), Math.abs(b2)) && Math.abs(a3 - b3) <= EPSILON$3 * Math.max(1.0, Math.abs(a3), Math.abs(b3)) && Math.abs(a4 - b4) <= EPSILON$3 * Math.max(1.0, Math.abs(a4), Math.abs(b4)) && Math.abs(a5 - b5) <= EPSILON$3 * Math.max(1.0, Math.abs(a5), Math.abs(b5)) && Math.abs(a6 - b6) <= EPSILON$3 * Math.max(1.0, Math.abs(a6), Math.abs(b6)) && Math.abs(a7 - b7) <= EPSILON$3 * Math.max(1.0, Math.abs(a7), Math.abs(b7)) && Math.abs(a8 - b8) <= EPSILON$3 * Math.max(1.0, Math.abs(a8), Math.abs(b8)) && Math.abs(a9 - b9) <= EPSILON$3 * Math.max(1.0, Math.abs(a9), Math.abs(b9)) && Math.abs(a10 - b10) <= EPSILON$3 * Math.max(1.0, Math.abs(a10), Math.abs(b10)) && Math.abs(a11 - b11) <= EPSILON$3 * Math.max(1.0, Math.abs(a11), Math.abs(b11)) && Math.abs(a12 - b12) <= EPSILON$3 * Math.max(1.0, Math.abs(a12), Math.abs(b12)) && Math.abs(a13 - b13) <= EPSILON$3 * Math.max(1.0, Math.abs(a13), Math.abs(b13)) && Math.abs(a14 - b14) <= EPSILON$3 * Math.max(1.0, Math.abs(a14), Math.abs(b14)) && Math.abs(a15 - b15) <= EPSILON$3 * Math.max(1.0, Math.abs(a15), Math.abs(b15));
  }
  /**
   * Alias for {@link mat4.multiply}
   * @function
   */

  var mul$2 = multiply$3;
  /**
   * Alias for {@link mat4.subtract}
   * @function
   */

  var sub$3 = subtract$3;

  var mat4$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    create: create$7,
    clone: clone$2,
    copy: copy$5,
    fromValues: fromValues$7,
    set: set$2,
    identity: identity$1,
    transpose: transpose$1,
    invert: invert$2,
    adjoint: adjoint$1,
    determinant: determinant$1,
    multiply: multiply$3,
    translate: translate$1,
    scale: scale$3,
    rotate: rotate$3,
    rotateX: rotateX$1,
    rotateY: rotateY$1,
    rotateZ: rotateZ$1,
    fromTranslation: fromTranslation$1,
    fromScaling: fromScaling$1,
    fromRotation: fromRotation$1,
    fromXRotation: fromXRotation$1,
    fromYRotation: fromYRotation$1,
    fromZRotation: fromZRotation$1,
    fromRotationTranslation: fromRotationTranslation$1,
    fromQuat2: fromQuat2$1,
    getTranslation: getTranslation$1,
    getScaling: getScaling$1,
    getRotation: getRotation$1,
    fromRotationTranslationScale: fromRotationTranslationScale$2,
    fromRotationTranslationScaleOrigin: fromRotationTranslationScaleOrigin$1,
    fromQuat: fromQuat$1,
    frustum: frustum$1,
    perspectiveNO: perspectiveNO$1,
    perspective: perspective$1,
    perspectiveZO: perspectiveZO$1,
    perspectiveFromFieldOfView: perspectiveFromFieldOfView$1,
    orthoNO: orthoNO$1,
    ortho: ortho$1,
    orthoZO: orthoZO$1,
    lookAt: lookAt$1,
    targetTo: targetTo$1,
    str: str$1,
    frob: frob$1,
    add: add$3,
    subtract: subtract$3,
    multiplyScalar: multiplyScalar$1,
    multiplyScalarAndAdd: multiplyScalarAndAdd$1,
    exactEquals: exactEquals$2,
    equals: equals$2,
    mul: mul$2,
    sub: sub$3
  });

  /**
   * 3 Dimensional Vector
   * @module vec3
   */

  /**
   * Creates a new, empty vec3
   *
   * @returns {vec3} a new 3D vector
   */

  function create$8() {
    var out = new ARRAY_TYPE$1(3);

    if (ARRAY_TYPE$1 != Float32Array) {
      out[0] = 0;
      out[1] = 0;
      out[2] = 0;
    }

    return out;
  }
  /**
   * Creates a new vec3 initialized with values from an existing vector
   *
   * @param {ReadonlyVec3} a vector to clone
   * @returns {vec3} a new 3D vector
   */

  function clone$3(a) {
    var out = new ARRAY_TYPE$1(3);
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    return out;
  }
  /**
   * Calculates the length of a vec3
   *
   * @param {ReadonlyVec3} a vector to calculate length of
   * @returns {Number} length of a
   */

  function length$1(a) {
    var x = a[0];
    var y = a[1];
    var z = a[2];
    return Math.hypot(x, y, z);
  }
  /**
   * Creates a new vec3 initialized with the given values
   *
   * @param {Number} x X component
   * @param {Number} y Y component
   * @param {Number} z Z component
   * @returns {vec3} a new 3D vector
   */

  function fromValues$8(x, y, z) {
    var out = new ARRAY_TYPE$1(3);
    out[0] = x;
    out[1] = y;
    out[2] = z;
    return out;
  }
  /**
   * Copy the values from one vec3 to another
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the source vector
   * @returns {vec3} out
   */

  function copy$6(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    return out;
  }
  /**
   * Set the components of a vec3 to the given values
   *
   * @param {vec3} out the receiving vector
   * @param {Number} x X component
   * @param {Number} y Y component
   * @param {Number} z Z component
   * @returns {vec3} out
   */

  function set$3(out, x, y, z) {
    out[0] = x;
    out[1] = y;
    out[2] = z;
    return out;
  }
  /**
   * Adds two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function add$4(out, a, b) {
    out[0] = a[0] + b[0];
    out[1] = a[1] + b[1];
    out[2] = a[2] + b[2];
    return out;
  }
  /**
   * Subtracts vector b from vector a
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function subtract$4(out, a, b) {
    out[0] = a[0] - b[0];
    out[1] = a[1] - b[1];
    out[2] = a[2] - b[2];
    return out;
  }
  /**
   * Multiplies two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function multiply$4(out, a, b) {
    out[0] = a[0] * b[0];
    out[1] = a[1] * b[1];
    out[2] = a[2] * b[2];
    return out;
  }
  /**
   * Returns the minimum of two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function min$3(out, a, b) {
    out[0] = Math.min(a[0], b[0]);
    out[1] = Math.min(a[1], b[1]);
    out[2] = Math.min(a[2], b[2]);
    return out;
  }
  /**
   * Returns the maximum of two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function max$3(out, a, b) {
    out[0] = Math.max(a[0], b[0]);
    out[1] = Math.max(a[1], b[1]);
    out[2] = Math.max(a[2], b[2]);
    return out;
  }
  /**
   * Scales a vec3 by a scalar number
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the vector to scale
   * @param {Number} b amount to scale the vector by
   * @returns {vec3} out
   */

  function scale$4(out, a, b) {
    out[0] = a[0] * b;
    out[1] = a[1] * b;
    out[2] = a[2] * b;
    return out;
  }
  /**
   * Normalize a vec3
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a vector to normalize
   * @returns {vec3} out
   */

  function normalize$5(out, a) {
    var x = a[0];
    var y = a[1];
    var z = a[2];
    var len = x * x + y * y + z * z;

    if (len > 0) {
      //TODO: evaluate use of glm_invsqrt here?
      len = 1 / Math.sqrt(len);
    }

    out[0] = a[0] * len;
    out[1] = a[1] * len;
    out[2] = a[2] * len;
    return out;
  }
  /**
   * Calculates the dot product of two vec3's
   *
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {Number} dot product of a and b
   */

  function dot$3(a, b) {
    return a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
  }
  /**
   * Computes the cross product of two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @returns {vec3} out
   */

  function cross$1(out, a, b) {
    var ax = a[0],
        ay = a[1],
        az = a[2];
    var bx = b[0],
        by = b[1],
        bz = b[2];
    out[0] = ay * bz - az * by;
    out[1] = az * bx - ax * bz;
    out[2] = ax * by - ay * bx;
    return out;
  }
  /**
   * Performs a linear interpolation between two vec3's
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the first operand
   * @param {ReadonlyVec3} b the second operand
   * @param {Number} t interpolation amount, in the range [0-1], between the two inputs
   * @returns {vec3} out
   */

  function lerp$1(out, a, b, t) {
    var ax = a[0];
    var ay = a[1];
    var az = a[2];
    out[0] = ax + t * (b[0] - ax);
    out[1] = ay + t * (b[1] - ay);
    out[2] = az + t * (b[2] - az);
    return out;
  }
  /**
   * Transforms the vec3 with a mat4.
   * 4th vector component is implicitly '1'
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the vector to transform
   * @param {ReadonlyMat4} m matrix to transform with
   * @returns {vec3} out
   */

  function transformMat4$2(out, a, m) {
    var x = a[0],
        y = a[1],
        z = a[2];
    var w = m[3] * x + m[7] * y + m[11] * z + m[15];
    w = w || 1.0;
    out[0] = (m[0] * x + m[4] * y + m[8] * z + m[12]) / w;
    out[1] = (m[1] * x + m[5] * y + m[9] * z + m[13]) / w;
    out[2] = (m[2] * x + m[6] * y + m[10] * z + m[14]) / w;
    return out;
  }
  /**
   * Transforms the vec3 with a mat3.
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the vector to transform
   * @param {ReadonlyMat3} m the 3x3 matrix to transform with
   * @returns {vec3} out
   */

  function transformMat3$1(out, a, m) {
    var x = a[0],
        y = a[1],
        z = a[2];
    out[0] = x * m[0] + y * m[3] + z * m[6];
    out[1] = x * m[1] + y * m[4] + z * m[7];
    out[2] = x * m[2] + y * m[5] + z * m[8];
    return out;
  }
  /**
   * Transforms the vec3 with a quat
   * Can also be used for dual quaternions. (Multiply it with the real part)
   *
   * @param {vec3} out the receiving vector
   * @param {ReadonlyVec3} a the vector to transform
   * @param {ReadonlyQuat} q quaternion to transform with
   * @returns {vec3} out
   */

  function transformQuat$1(out, a, q) {
    // benchmarks: https://jsperf.com/quaternion-transform-vec3-implementations-fixed
    var qx = q[0],
        qy = q[1],
        qz = q[2],
        qw = q[3];
    var x = a[0],
        y = a[1],
        z = a[2]; // var qvec = [qx, qy, qz];
    // var uv = vec3.cross([], qvec, a);

    var uvx = qy * z - qz * y,
        uvy = qz * x - qx * z,
        uvz = qx * y - qy * x; // var uuv = vec3.cross([], qvec, uv);

    var uuvx = qy * uvz - qz * uvy,
        uuvy = qz * uvx - qx * uvz,
        uuvz = qx * uvy - qy * uvx; // vec3.scale(uv, uv, 2 * w);

    var w2 = qw * 2;
    uvx *= w2;
    uvy *= w2;
    uvz *= w2; // vec3.scale(uuv, uuv, 2);

    uuvx *= 2;
    uuvy *= 2;
    uuvz *= 2; // return vec3.add(out, a, vec3.add(out, uv, uuv));

    out[0] = x + uvx + uuvx;
    out[1] = y + uvy + uuvy;
    out[2] = z + uvz + uuvz;
    return out;
  }
  /**
   * Returns whether or not the vectors have approximately the same elements in the same position.
   *
   * @param {ReadonlyVec3} a The first vector.
   * @param {ReadonlyVec3} b The second vector.
   * @returns {Boolean} True if the vectors are equal, false otherwise.
   */

  function equals$3(a, b) {
    var a0 = a[0],
        a1 = a[1],
        a2 = a[2];
    var b0 = b[0],
        b1 = b[1],
        b2 = b[2];
    return Math.abs(a0 - b0) <= EPSILON$3 * Math.max(1.0, Math.abs(a0), Math.abs(b0)) && Math.abs(a1 - b1) <= EPSILON$3 * Math.max(1.0, Math.abs(a1), Math.abs(b1)) && Math.abs(a2 - b2) <= EPSILON$3 * Math.max(1.0, Math.abs(a2), Math.abs(b2));
  }
  /**
   * Alias for {@link vec3.subtract}
   * @function
   */

  var sub$4 = subtract$4;
  /**
   * Alias for {@link vec3.length}
   * @function
   */

  var len$1 = length$1;
  /**
   * Perform some operation over an array of vec3s.
   *
   * @param {Array} a the array of vectors to iterate over
   * @param {Number} stride Number of elements between the start of each vec3. If 0 assumes tightly packed
   * @param {Number} offset Number of elements to skip at the beginning of the array
   * @param {Number} count Number of vec3s to iterate over. If 0 iterates over entire array
   * @param {Function} fn Function to call for each vector in the array
   * @param {Object} [arg] additional argument to pass to fn
   * @returns {Array} a
   * @function
   */

  var forEach$3 = function () {
    var vec = create$8();
    return function (a, stride, offset, count, fn, arg) {
      var i, l;

      if (!stride) {
        stride = 3;
      }

      if (!offset) {
        offset = 0;
      }

      if (count) {
        l = Math.min(count * stride + offset, a.length);
      } else {
        l = a.length;
      }

      for (i = offset; i < l; i += stride) {
        vec[0] = a[i];
        vec[1] = a[i + 1];
        vec[2] = a[i + 2];
        fn(vec, vec, arg);
        a[i] = vec[0];
        a[i + 1] = vec[1];
        a[i + 2] = vec[2];
      }

      return a;
    };
  }();

  /**
   * 4 Dimensional Vector
   * @module vec4
   */

  /**
   * Creates a new, empty vec4
   *
   * @returns {vec4} a new 4D vector
   */

  function create$9() {
    var out = new ARRAY_TYPE$1(4);

    if (ARRAY_TYPE$1 != Float32Array) {
      out[0] = 0;
      out[1] = 0;
      out[2] = 0;
      out[3] = 0;
    }

    return out;
  }
  /**
   * Creates a new vec4 initialized with the given values
   *
   * @param {Number} x X component
   * @param {Number} y Y component
   * @param {Number} z Z component
   * @param {Number} w W component
   * @returns {vec4} a new 4D vector
   */

  function fromValues$9(x, y, z, w) {
    var out = new ARRAY_TYPE$1(4);
    out[0] = x;
    out[1] = y;
    out[2] = z;
    out[3] = w;
    return out;
  }
  /**
   * Copy the values from one vec4 to another
   *
   * @param {vec4} out the receiving vector
   * @param {ReadonlyVec4} a the source vector
   * @returns {vec4} out
   */

  function copy$7(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[3];
    return out;
  }
  /**
   * Normalize a vec4
   *
   * @param {vec4} out the receiving vector
   * @param {ReadonlyVec4} a vector to normalize
   * @returns {vec4} out
   */

  function normalize$6(out, a) {
    var x = a[0];
    var y = a[1];
    var z = a[2];
    var w = a[3];
    var len = x * x + y * y + z * z + w * w;

    if (len > 0) {
      len = 1 / Math.sqrt(len);
    }

    out[0] = x * len;
    out[1] = y * len;
    out[2] = z * len;
    out[3] = w * len;
    return out;
  }
  /**
   * Transforms the vec4 with a mat4.
   *
   * @param {vec4} out the receiving vector
   * @param {ReadonlyVec4} a the vector to transform
   * @param {ReadonlyMat4} m matrix to transform with
   * @returns {vec4} out
   */

  function transformMat4$3(out, a, m) {
    var x = a[0],
        y = a[1],
        z = a[2],
        w = a[3];
    out[0] = m[0] * x + m[4] * y + m[8] * z + m[12] * w;
    out[1] = m[1] * x + m[5] * y + m[9] * z + m[13] * w;
    out[2] = m[2] * x + m[6] * y + m[10] * z + m[14] * w;
    out[3] = m[3] * x + m[7] * y + m[11] * z + m[15] * w;
    return out;
  }
  /**
   * Perform some operation over an array of vec4s.
   *
   * @param {Array} a the array of vectors to iterate over
   * @param {Number} stride Number of elements between the start of each vec4. If 0 assumes tightly packed
   * @param {Number} offset Number of elements to skip at the beginning of the array
   * @param {Number} count Number of vec4s to iterate over. If 0 iterates over entire array
   * @param {Function} fn Function to call for each vector in the array
   * @param {Object} [arg] additional argument to pass to fn
   * @returns {Array} a
   * @function
   */

  var forEach$4 = function () {
    var vec = create$9();
    return function (a, stride, offset, count, fn, arg) {
      var i, l;

      if (!stride) {
        stride = 4;
      }

      if (!offset) {
        offset = 0;
      }

      if (count) {
        l = Math.min(count * stride + offset, a.length);
      } else {
        l = a.length;
      }

      for (i = offset; i < l; i += stride) {
        vec[0] = a[i];
        vec[1] = a[i + 1];
        vec[2] = a[i + 2];
        vec[3] = a[i + 3];
        fn(vec, vec, arg);
        a[i] = vec[0];
        a[i + 1] = vec[1];
        a[i + 2] = vec[2];
        a[i + 3] = vec[3];
      }

      return a;
    };
  }();

  /**
   * Quaternion
   * @module quat
   */

  /**
   * Creates a new identity quat
   *
   * @returns {quat} a new quaternion
   */

  function create$a() {
    var out = new ARRAY_TYPE$1(4);

    if (ARRAY_TYPE$1 != Float32Array) {
      out[0] = 0;
      out[1] = 0;
      out[2] = 0;
    }

    out[3] = 1;
    return out;
  }
  /**
   * Sets a quat from the given angle and rotation axis,
   * then returns it.
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyVec3} axis the axis around which to rotate
   * @param {Number} rad the angle in radians
   * @returns {quat} out
   **/

  function setAxisAngle$1(out, axis, rad) {
    rad = rad * 0.5;
    var s = Math.sin(rad);
    out[0] = s * axis[0];
    out[1] = s * axis[1];
    out[2] = s * axis[2];
    out[3] = Math.cos(rad);
    return out;
  }
  /**
   * Multiplies two quat's
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a the first operand
   * @param {ReadonlyQuat} b the second operand
   * @returns {quat} out
   */

  function multiply$5(out, a, b) {
    var ax = a[0],
        ay = a[1],
        az = a[2],
        aw = a[3];
    var bx = b[0],
        by = b[1],
        bz = b[2],
        bw = b[3];
    out[0] = ax * bw + aw * bx + ay * bz - az * by;
    out[1] = ay * bw + aw * by + az * bx - ax * bz;
    out[2] = az * bw + aw * bz + ax * by - ay * bx;
    out[3] = aw * bw - ax * bx - ay * by - az * bz;
    return out;
  }
  /**
   * Performs a spherical linear interpolation between two quat
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a the first operand
   * @param {ReadonlyQuat} b the second operand
   * @param {Number} t interpolation amount, in the range [0-1], between the two inputs
   * @returns {quat} out
   */

  function slerp$1(out, a, b, t) {
    // benchmarks:
    //    http://jsperf.com/quaternion-slerp-implementations
    var ax = a[0],
        ay = a[1],
        az = a[2],
        aw = a[3];
    var bx = b[0],
        by = b[1],
        bz = b[2],
        bw = b[3];
    var omega, cosom, sinom, scale0, scale1; // calc cosine

    cosom = ax * bx + ay * by + az * bz + aw * bw; // adjust signs (if necessary)

    if (cosom < 0.0) {
      cosom = -cosom;
      bx = -bx;
      by = -by;
      bz = -bz;
      bw = -bw;
    } // calculate coefficients


    if (1.0 - cosom > EPSILON$3) {
      // standard case (slerp)
      omega = Math.acos(cosom);
      sinom = Math.sin(omega);
      scale0 = Math.sin((1.0 - t) * omega) / sinom;
      scale1 = Math.sin(t * omega) / sinom;
    } else {
      // "from" and "to" quaternions are very close
      //  ... so we can do a linear interpolation
      scale0 = 1.0 - t;
      scale1 = t;
    } // calculate final values


    out[0] = scale0 * ax + scale1 * bx;
    out[1] = scale0 * ay + scale1 * by;
    out[2] = scale0 * az + scale1 * bz;
    out[3] = scale0 * aw + scale1 * bw;
    return out;
  }
  /**
   * Calculates the inverse of a quat
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a quat to calculate inverse of
   * @returns {quat} out
   */

  function invert$3(out, a) {
    var a0 = a[0],
        a1 = a[1],
        a2 = a[2],
        a3 = a[3];
    var dot = a0 * a0 + a1 * a1 + a2 * a2 + a3 * a3;
    var invDot = dot ? 1.0 / dot : 0; // TODO: Would be faster to return [0,0,0,0] immediately if dot == 0

    out[0] = -a0 * invDot;
    out[1] = -a1 * invDot;
    out[2] = -a2 * invDot;
    out[3] = a3 * invDot;
    return out;
  }
  /**
   * Creates a quaternion from the given 3x3 rotation matrix.
   *
   * NOTE: The resultant quaternion is not normalized, so you should be sure
   * to renormalize the quaternion yourself where necessary.
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyMat3} m rotation matrix
   * @returns {quat} out
   * @function
   */

  function fromMat3$1(out, m) {
    // Algorithm in Ken Shoemake's article in 1987 SIGGRAPH course notes
    // article "Quaternion Calculus and Fast Animation".
    var fTrace = m[0] + m[4] + m[8];
    var fRoot;

    if (fTrace > 0.0) {
      // |w| > 1/2, may as well choose w > 1/2
      fRoot = Math.sqrt(fTrace + 1.0); // 2w

      out[3] = 0.5 * fRoot;
      fRoot = 0.5 / fRoot; // 1/(4w)

      out[0] = (m[5] - m[7]) * fRoot;
      out[1] = (m[6] - m[2]) * fRoot;
      out[2] = (m[1] - m[3]) * fRoot;
    } else {
      // |w| <= 1/2
      var i = 0;
      if (m[4] > m[0]) i = 1;
      if (m[8] > m[i * 3 + i]) i = 2;
      var j = (i + 1) % 3;
      var k = (i + 2) % 3;
      fRoot = Math.sqrt(m[i * 3 + i] - m[j * 3 + j] - m[k * 3 + k] + 1.0);
      out[i] = 0.5 * fRoot;
      fRoot = 0.5 / fRoot;
      out[3] = (m[j * 3 + k] - m[k * 3 + j]) * fRoot;
      out[j] = (m[j * 3 + i] + m[i * 3 + j]) * fRoot;
      out[k] = (m[k * 3 + i] + m[i * 3 + k]) * fRoot;
    }

    return out;
  }
  /**
   * Creates a quaternion from the given euler angle x, y, z.
   *
   * @param {quat} out the receiving quaternion
   * @param {x} Angle to rotate around X axis in degrees.
   * @param {y} Angle to rotate around Y axis in degrees.
   * @param {z} Angle to rotate around Z axis in degrees.
   * @returns {quat} out
   * @function
   */

  function fromEuler$1(out, x, y, z) {
    var halfToRad = 0.5 * Math.PI / 180.0;
    x *= halfToRad;
    y *= halfToRad;
    z *= halfToRad;
    var sx = Math.sin(x);
    var cx = Math.cos(x);
    var sy = Math.sin(y);
    var cy = Math.cos(y);
    var sz = Math.sin(z);
    var cz = Math.cos(z);
    out[0] = sx * cy * cz - cx * sy * sz;
    out[1] = cx * sy * cz + sx * cy * sz;
    out[2] = cx * cy * sz - sx * sy * cz;
    out[3] = cx * cy * cz + sx * sy * sz;
    return out;
  }
  /**
   * Creates a new quat initialized with the given values
   *
   * @param {Number} x X component
   * @param {Number} y Y component
   * @param {Number} z Z component
   * @param {Number} w W component
   * @returns {quat} a new quaternion
   * @function
   */

  var fromValues$a = fromValues$9;
  /**
   * Copy the values from one quat to another
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a the source quaternion
   * @returns {quat} out
   * @function
   */

  var copy$8 = copy$7;
  /**
   * Alias for {@link quat.multiply}
   * @function
   */

  var mul$3 = multiply$5;
  /**
   * Normalize a quat
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a quaternion to normalize
   * @returns {quat} out
   * @function
   */

  var normalize$7 = normalize$6;
  /**
   * Sets a quaternion to represent the shortest rotation from one
   * vector to another.
   *
   * Both vectors are assumed to be unit length.
   *
   * @param {quat} out the receiving quaternion.
   * @param {ReadonlyVec3} a the initial vector
   * @param {ReadonlyVec3} b the destination vector
   * @returns {quat} out
   */

  var rotationTo$1 = function () {
    var tmpvec3 = create$8();
    var xUnitVec3 = fromValues$8(1, 0, 0);
    var yUnitVec3 = fromValues$8(0, 1, 0);
    return function (out, a, b) {
      var dot = dot$3(a, b);

      if (dot < -0.999999) {
        cross$1(tmpvec3, xUnitVec3, a);
        if (len$1(tmpvec3) < 0.000001) cross$1(tmpvec3, yUnitVec3, a);
        normalize$5(tmpvec3, tmpvec3);
        setAxisAngle$1(out, tmpvec3, Math.PI);
        return out;
      } else if (dot > 0.999999) {
        out[0] = 0;
        out[1] = 0;
        out[2] = 0;
        out[3] = 1;
        return out;
      } else {
        cross$1(tmpvec3, a, b);
        out[0] = tmpvec3[0];
        out[1] = tmpvec3[1];
        out[2] = tmpvec3[2];
        out[3] = 1 + dot;
        return normalize$7(out, out);
      }
    };
  }();
  /**
   * Performs a spherical linear interpolation with two control points
   *
   * @param {quat} out the receiving quaternion
   * @param {ReadonlyQuat} a the first operand
   * @param {ReadonlyQuat} b the second operand
   * @param {ReadonlyQuat} c the third operand
   * @param {ReadonlyQuat} d the fourth operand
   * @param {Number} t interpolation amount, in the range [0-1], between the two inputs
   * @returns {quat} out
   */

  var sqlerp$1 = function () {
    var temp1 = create$a();
    var temp2 = create$a();
    return function (out, a, b, c, d, t) {
      slerp$1(temp1, a, d, t);
      slerp$1(temp2, b, c, t);
      slerp$1(out, temp1, temp2, 2 * t * (1 - t));
      return out;
    };
  }();
  /**
   * Sets the specified quaternion with values corresponding to the given
   * axes. Each axis is a vec3 and is expected to be unit length and
   * perpendicular to all other specified axes.
   *
   * @param {ReadonlyVec3} view  the vector representing the viewing direction
   * @param {ReadonlyVec3} right the vector representing the local "right" direction
   * @param {ReadonlyVec3} up    the vector representing the local "up" direction
   * @returns {quat} out
   */

  var setAxes$1 = function () {
    var matr = create$6();
    return function (out, view, right, up) {
      matr[0] = right[0];
      matr[3] = right[1];
      matr[6] = right[2];
      matr[1] = up[0];
      matr[4] = up[1];
      matr[7] = up[2];
      matr[2] = -view[0];
      matr[5] = -view[1];
      matr[8] = -view[2];
      return normalize$7(out, fromMat3$1(out, matr));
    };
  }();

  /**
   * 2 Dimensional Vector
   * @module vec2
   */

  /**
   * Creates a new, empty vec2
   *
   * @returns {vec2} a new 2D vector
   */

  function create$b() {
    var out = new ARRAY_TYPE$1(2);

    if (ARRAY_TYPE$1 != Float32Array) {
      out[0] = 0;
      out[1] = 0;
    }

    return out;
  }
  /**
   * Creates a new vec2 initialized with the given values
   *
   * @param {Number} x X component
   * @param {Number} y Y component
   * @returns {vec2} a new 2D vector
   */

  function fromValues$b(x, y) {
    var out = new ARRAY_TYPE$1(2);
    out[0] = x;
    out[1] = y;
    return out;
  }
  /**
   * Copy the values from one vec2 to another
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a the source vector
   * @returns {vec2} out
   */

  function copy$9(out, a) {
    out[0] = a[0];
    out[1] = a[1];
    return out;
  }
  /**
   * Calculates the length of a vec2
   *
   * @param {ReadonlyVec2} a vector to calculate length of
   * @returns {Number} length of a
   */

  function length$2(a) {
    var x = a[0],
        y = a[1];
    return Math.hypot(x, y);
  }
  /**
   * Normalize a vec2
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a vector to normalize
   * @returns {vec2} out
   */

  function normalize$8(out, a) {
    var x = a[0],
        y = a[1];
    var len = x * x + y * y;

    if (len > 0) {
      //TODO: evaluate use of glm_invsqrt here?
      len = 1 / Math.sqrt(len);
    }

    out[0] = a[0] * len;
    out[1] = a[1] * len;
    return out;
  }
  /**
   * Calculates the dot product of two vec2's
   *
   * @param {ReadonlyVec2} a the first operand
   * @param {ReadonlyVec2} b the second operand
   * @returns {Number} dot product of a and b
   */

  function dot$4(a, b) {
    return a[0] * b[0] + a[1] * b[1];
  }
  /**
   * Transforms the vec2 with a mat2d
   *
   * @param {vec2} out the receiving vector
   * @param {ReadonlyVec2} a the vector to transform
   * @param {ReadonlyMat2d} m matrix to transform with
   * @returns {vec2} out
   */

  function transformMat2d(out, a, m) {
    var x = a[0],
        y = a[1];
    out[0] = m[0] * x + m[2] * y + m[4];
    out[1] = m[1] * x + m[3] * y + m[5];
    return out;
  }
  /**
   * Get the angle between two 2D vectors
   * @param {ReadonlyVec2} a The first operand
   * @param {ReadonlyVec2} b The second operand
   * @returns {Number} The angle in radians
   */

  function angle(a, b) {
    var x1 = a[0],
        y1 = a[1],
        x2 = b[0],
        y2 = b[1],
        // mag is the product of the magnitudes of a and b
    mag = Math.sqrt(x1 * x1 + y1 * y1) * Math.sqrt(x2 * x2 + y2 * y2),
        // mag &&.. short circuits if mag == 0
    cosine = mag && (x1 * x2 + y1 * y2) / mag; // Math.min(Math.max(cosine, -1), 1) clamps the cosine between -1 and 1

    return Math.acos(Math.min(Math.max(cosine, -1), 1));
  }
  /**
   * Returns whether or not the vectors exactly have the same elements in the same position (when compared with ===)
   *
   * @param {ReadonlyVec2} a The first vector.
   * @param {ReadonlyVec2} b The second vector.
   * @returns {Boolean} True if the vectors are equal, false otherwise.
   */

  function exactEquals$3(a, b) {
    return a[0] === b[0] && a[1] === b[1];
  }
  /**
   * Perform some operation over an array of vec2s.
   *
   * @param {Array} a the array of vectors to iterate over
   * @param {Number} stride Number of elements between the start of each vec2. If 0 assumes tightly packed
   * @param {Number} offset Number of elements to skip at the beginning of the array
   * @param {Number} count Number of vec2s to iterate over. If 0 iterates over entire array
   * @param {Function} fn Function to call for each vector in the array
   * @param {Object} [arg] additional argument to pass to fn
   * @returns {Array} a
   * @function
   */

  var forEach$5 = function () {
    var vec = create$b();
    return function (a, stride, offset, count, fn, arg) {
      var i, l;

      if (!stride) {
        stride = 2;
      }

      if (!offset) {
        offset = 0;
      }

      if (count) {
        l = Math.min(count * stride + offset, a.length);
      } else {
        l = a.length;
      }

      for (i = offset; i < l; i += stride) {
        vec[0] = a[i];
        vec[1] = a[i + 1];
        fn(vec, vec, arg);
        a[i] = vec[0];
        a[i + 1] = vec[1];
      }

      return a;
    };
  }();

  function clonePath$1(path) {
      return path.map(function (x) { return (Array.isArray(x) ? [].concat(x) : x); });
  }

  /**
   * Rounds the values of a `PathArray` instance to
   * a specified amount of decimals and returns it.
   */
  function roundPath(path, round) {
      if (round === 'off')
          return clonePath$1(path);
      // to round values to the power
      // the `round` value must be integer
      var pow = typeof round === 'number' && round >= 1 ? Math.pow(10, round) : 1;
      return path.map(function (pi) {
          var values = pi
              .slice(1)
              .map(Number)
              .map(function (n) { return (round ? Math.round(n * pow) / pow : Math.round(n)); });
          // @ts-ignore
          return [pi[0]].concat(values);
      });
  }

  /**
   * Returns a valid `d` attribute string value created
   * by rounding values and concatenating the `pathArray` segments.
   */
  function path2String(path, round) {
      if (round === void 0) { round = 'off'; }
      return roundPath(path, round)
          .map(function (x) { return x[0] + x.slice(1).join(' '); })
          .join('');
  }

  /******************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  var __assign$1 = function() {
      __assign$1 = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign$1.apply(this, arguments);
  };

  var paramsParser$1 = {
      x1: 0,
      y1: 0,
      x2: 0,
      y2: 0,
      x: 0,
      y: 0,
      qx: null,
      qy: null,
  };

  function fixArc$1(pathArray, allPathCommands, i) {
      if (pathArray[i].length > 7) {
          pathArray[i].shift();
          var pi = pathArray[i];
          // const ni = i + 1;
          var ni = i;
          while (pi.length) {
              // if created multiple C:s, their original seg is saved
              allPathCommands[i] = 'A';
              // @ts-ignore
              pathArray.splice((ni += 1), 0, ['C'].concat(pi.splice(0, 6)));
          }
          pathArray.splice(i, 1);
      }
  }

  var paramsCount$1 = {
      a: 7,
      c: 6,
      h: 1,
      l: 2,
      m: 2,
      r: 4,
      q: 4,
      s: 4,
      t: 2,
      v: 1,
      z: 0,
  };

  /**
   * Iterates an array to check if it's an actual `PathArray`.
   */
  function isPathArray$1(path) {
      return (Array.isArray(path) &&
          path.every(function (seg) {
              var lk = seg[0].toLowerCase();
              return paramsCount$1[lk] === seg.length - 1 && 'achlmqstvz'.includes(lk);
          }));
  }

  /**
   * Iterates an array to check if it's a `PathArray`
   * with all absolute values.
   */
  function isAbsoluteArray$1(path) {
      return (isPathArray$1(path) &&
          // @ts-ignore -- `isPathArray` also checks if it's `Array`
          path.every(function (_a) {
              var x = _a[0];
              return x === x.toUpperCase();
          }));
  }

  /**
   * Iterates an array to check if it's a `PathArray`
   * with all segments are in non-shorthand notation
   * with absolute values.
   */
  function isNormalizedArray$1(path) {
      return isAbsoluteArray$1(path) && path.every(function (_a) {
          var pc = _a[0];
          return 'ACLMQZ'.includes(pc);
      });
  }

  /**
   * Breaks the parsing of a pathString once a segment is finalized.
   */
  function finalizeSegment$1(path) {
      var pathCommand = path.pathValue[path.segmentStart];
      var LK = pathCommand.toLowerCase();
      var data = path.data;
      while (data.length >= paramsCount$1[LK]) {
          // overloaded `moveTo`
          // https://github.com/rveciana/svg-path-properties/blob/master/src/parse.ts
          if (LK === 'm' && data.length > 2) {
              // @ts-ignore
              path.segments.push([pathCommand].concat(data.splice(0, 2)));
              LK = 'l';
              pathCommand = pathCommand === 'm' ? 'l' : 'L';
          }
          else {
              // @ts-ignore
              path.segments.push([pathCommand].concat(data.splice(0, paramsCount$1[LK])));
          }
          if (!paramsCount$1[LK]) {
              break;
          }
      }
  }

  /**
   * Validates an A (arc-to) specific path command value.
   * Usually a `large-arc-flag` or `sweep-flag`.
   */
  function scanFlag$1(path) {
      var index = path.index, pathValue = path.pathValue;
      var code = pathValue.charCodeAt(index);
      if (code === 0x30 /* 0 */) {
          path.param = 0;
          path.index += 1;
          return;
      }
      if (code === 0x31 /* 1 */) {
          path.param = 1;
          path.index += 1;
          return;
      }
      path.err = "[path-util]: invalid Arc flag \"" + pathValue[index] + "\", expecting 0 or 1 at index " + index;
  }

  /**
   * Checks if the character is or belongs to a number.
   * [0-9]|+|-|.
   */
  function isDigitStart$1(code) {
      return ((code >= 48 && code <= 57) /* 0..9 */ || code === 0x2b /* + */ || code === 0x2d /* - */ || code === 0x2e); /* . */
  }
  function isDigit$1(code) {
      return code >= 48 && code <= 57; // 0..9
  }

  /**
   * Validates every character of the path string,
   * every path command, negative numbers or floating point numbers.
   */
  function scanParam$1(path) {
      var max = path.max, pathValue = path.pathValue, start = path.index;
      var index = start;
      var zeroFirst = false;
      var hasCeiling = false;
      var hasDecimal = false;
      var hasDot = false;
      var ch;
      if (index >= max) {
          // path.err = 'SvgPath: missed param (at pos ' + index + ')';
          path.err = "[path-util]: Invalid path value at index " + index + ", \"pathValue\" is missing param";
          return;
      }
      ch = pathValue.charCodeAt(index);
      if (ch === 0x2b /* + */ || ch === 0x2d /* - */) {
          index += 1;
          // ch = (index < max) ? pathValue.charCodeAt(index) : 0;
          ch = pathValue.charCodeAt(index);
      }
      // This logic is shamelessly borrowed from Esprima
      // https://github.com/ariya/esprimas
      if (!isDigit$1(ch) && ch !== 0x2e /* . */) {
          // path.err = 'SvgPath: param should start with 0..9 or `.` (at pos ' + index + ')';
          path.err = "[path-util]: Invalid path value at index " + index + ", \"" + pathValue[index] + "\" is not a number";
          return;
      }
      if (ch !== 0x2e /* . */) {
          zeroFirst = ch === 0x30 /* 0 */;
          index += 1;
          ch = pathValue.charCodeAt(index);
          if (zeroFirst && index < max) {
              // decimal number starts with '0' such as '09' is illegal.
              if (ch && isDigit$1(ch)) {
                  // path.err = 'SvgPath: numbers started with `0` such as `09`
                  // are illegal (at pos ' + start + ')';
                  path.err = "[path-util]: Invalid path value at index " + start + ", \"" + pathValue[start] + "\" illegal number";
                  return;
              }
          }
          while (index < max && isDigit$1(pathValue.charCodeAt(index))) {
              index += 1;
              hasCeiling = true;
          }
          ch = pathValue.charCodeAt(index);
      }
      if (ch === 0x2e /* . */) {
          hasDot = true;
          index += 1;
          while (isDigit$1(pathValue.charCodeAt(index))) {
              index += 1;
              hasDecimal = true;
          }
          ch = pathValue.charCodeAt(index);
      }
      if (ch === 0x65 /* e */ || ch === 0x45 /* E */) {
          if (hasDot && !hasCeiling && !hasDecimal) {
              path.err = "[path-util]: Invalid path value at index " + index + ", \"" + pathValue[index] + "\" invalid float exponent";
              return;
          }
          index += 1;
          ch = pathValue.charCodeAt(index);
          if (ch === 0x2b /* + */ || ch === 0x2d /* - */) {
              index += 1;
          }
          if (index < max && isDigit$1(pathValue.charCodeAt(index))) {
              while (index < max && isDigit$1(pathValue.charCodeAt(index))) {
                  index += 1;
              }
          }
          else {
              path.err = "[path-util]: Invalid path value at index " + index + ", \"" + pathValue[index] + "\" invalid integer exponent";
              return;
          }
      }
      path.index = index;
      path.param = +path.pathValue.slice(start, index);
  }

  /**
   * Checks if the character is a space.
   */
  function isSpace$1(ch) {
      var specialSpaces = [
          0x1680, 0x180e, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005, 0x2006, 0x2007, 0x2008, 0x2009, 0x200a, 0x202f,
          0x205f, 0x3000, 0xfeff,
      ];
      /* istanbul ignore next */
      return (ch === 0x0a ||
          ch === 0x0d ||
          ch === 0x2028 ||
          ch === 0x2029 || // Line terminators
          // White spaces
          ch === 0x20 ||
          ch === 0x09 ||
          ch === 0x0b ||
          ch === 0x0c ||
          ch === 0xa0 ||
          (ch >= 0x1680 && specialSpaces.includes(ch)));
  }

  /**
   * Points the parser to the next character in the
   * path string every time it encounters any kind of
   * space character.
   */
  function skipSpaces$1(path) {
      var pathValue = path.pathValue, max = path.max;
      while (path.index < max && isSpace$1(pathValue.charCodeAt(path.index))) {
          path.index += 1;
      }
  }

  /**
   * Checks if the character is a path command.
   */
  function isPathCommand$1(code) {
      // eslint-disable-next-line no-bitwise -- Impossible to satisfy
      switch (code | 0x20) {
          case 0x6d /* m */:
          case 0x7a /* z */:
          case 0x6c /* l */:
          case 0x68 /* h */:
          case 0x76 /* v */:
          case 0x63 /* c */:
          case 0x73 /* s */:
          case 0x71 /* q */:
          case 0x74 /* t */:
          case 0x61 /* a */:
              // case 0x72/* r */:
              return true;
          default:
              return false;
      }
  }

  /**
   * Checks if the character is an A (arc-to) path command.
   */
  function isArcCommand$1(code) {
      return (code | 0x20) === 0x61;
  }

  /**
   * Scans every character in the path string to determine
   * where a segment starts and where it ends.
   */
  function scanSegment$1(path) {
      var max = path.max, pathValue = path.pathValue, index = path.index;
      var cmdCode = pathValue.charCodeAt(index);
      var reqParams = paramsCount$1[pathValue[index].toLowerCase()];
      path.segmentStart = index;
      if (!isPathCommand$1(cmdCode)) {
          path.err = "[path-util]: Invalid path value \"" + pathValue[index] + "\" is not a path command";
          return;
      }
      path.index += 1;
      skipSpaces$1(path);
      path.data = [];
      if (!reqParams) {
          // Z
          finalizeSegment$1(path);
          return;
      }
      for (;;) {
          for (var i = reqParams; i > 0; i -= 1) {
              if (isArcCommand$1(cmdCode) && (i === 3 || i === 4))
                  scanFlag$1(path);
              else
                  scanParam$1(path);
              if (path.err.length) {
                  return;
              }
              path.data.push(path.param);
              skipSpaces$1(path);
              // after ',' param is mandatory
              if (path.index < max && pathValue.charCodeAt(path.index) === 0x2c /* , */) {
                  path.index += 1;
                  skipSpaces$1(path);
              }
          }
          if (path.index >= path.max) {
              break;
          }
          // Stop on next segment
          if (!isDigitStart$1(pathValue.charCodeAt(path.index))) {
              break;
          }
      }
      finalizeSegment$1(path);
  }

  /**
   * The `PathParser` is used by the `parsePathString` static method
   * to generate a `pathArray`.
   */
  var PathParser$1 = /** @class */ (function () {
      function PathParser(pathString) {
          this.pathValue = pathString;
          // @ts-ignore
          this.segments = [];
          this.max = pathString.length;
          this.index = 0;
          this.param = 0.0;
          this.segmentStart = 0;
          this.data = [];
          this.err = '';
      }
      return PathParser;
  }());

  /**
   * Parses a path string value and returns an array
   * of segments we like to call `pathArray`.
   */
  function parsePathString$1(pathInput) {
      if (isPathArray$1(pathInput)) {
          return clonePath$1(pathInput);
      }
      var path = new PathParser$1(pathInput);
      skipSpaces$1(path);
      while (path.index < path.max && !path.err.length) {
          scanSegment$1(path);
      }
      return path.err ? path.err : path.segments;
  }

  function path2Absolute$1(pathInput) {
      if (isAbsoluteArray$1(pathInput)) {
          return clonePath$1(pathInput);
      }
      var path = parsePathString$1(pathInput);
      // if (!path || !path.length) {
      //   return [['M', 0, 0]];
      // }
      var x = 0;
      var y = 0;
      var mx = 0;
      var my = 0;
      // @ts-ignore
      return path.map(function (segment) {
          var values = segment.slice(1).map(Number);
          var pathCommand = segment[0];
          var absCommand = pathCommand.toUpperCase();
          if (pathCommand === 'M') {
              x = values[0], y = values[1];
              mx = x;
              my = y;
              return ['M', x, y];
          }
          var absoluteSegment;
          if (pathCommand !== absCommand) {
              switch (absCommand) {
                  case 'A':
                      absoluteSegment = [
                          absCommand,
                          values[0],
                          values[1],
                          values[2],
                          values[3],
                          values[4],
                          values[5] + x,
                          values[6] + y,
                      ];
                      break;
                  case 'V':
                      absoluteSegment = [absCommand, values[0] + y];
                      break;
                  case 'H':
                      absoluteSegment = [absCommand, values[0] + x];
                      break;
                  default: {
                      // use brakets for `eslint: no-case-declaration`
                      // https://stackoverflow.com/a/50753272/803358
                      var absValues = values.map(function (n, j) { return n + (j % 2 ? y : x); });
                      // for n, l, c, s, q, t
                      // @ts-ignore
                      absoluteSegment = [absCommand].concat(absValues);
                  }
              }
          }
          else {
              // @ts-ignore
              absoluteSegment = [absCommand].concat(values);
          }
          var segLength = absoluteSegment.length;
          switch (absCommand) {
              case 'Z':
                  x = mx;
                  y = my;
                  break;
              case 'H':
                  x = absoluteSegment[1];
                  break;
              case 'V':
                  y = absoluteSegment[1];
                  break;
              default:
                  x = absoluteSegment[segLength - 2];
                  y = absoluteSegment[segLength - 1];
                  if (absCommand === 'M') {
                      mx = x;
                      my = y;
                  }
          }
          return absoluteSegment;
      });
  }

  /**
   * Normalizes a single segment of a `PathArray` object.
   * eg. H/V -> L, T -> Q
   */
  function normalizeSegment$1(segment, params) {
      var pathCommand = segment[0];
      var px1 = params.x1, py1 = params.y1, px2 = params.x2, py2 = params.y2;
      var values = segment.slice(1).map(Number);
      var result = segment;
      if (!'TQ'.includes(pathCommand)) {
          // optional but good to be cautious
          params.qx = null;
          params.qy = null;
      }
      if (pathCommand === 'H') {
          result = ['L', segment[1], py1];
      }
      else if (pathCommand === 'V') {
          result = ['L', px1, segment[1]];
      }
      else if (pathCommand === 'S') {
          var x1 = px1 * 2 - px2;
          var y1 = py1 * 2 - py2;
          params.x1 = x1;
          params.y1 = y1;
          result = ['C', x1, y1].concat(values);
      }
      else if (pathCommand === 'T') {
          var qx = px1 * 2 - params.qx;
          var qy = py1 * 2 - params.qy;
          params.qx = qx;
          params.qy = qy;
          result = ['Q', qx, qy].concat(values);
      }
      else if (pathCommand === 'Q') {
          var nqx = values[0], nqy = values[1];
          params.qx = nqx;
          params.qy = nqy;
      }
      return result;
  }

  /**
   * @example
   * const path = 'M0 0 H50';
   * const normalizedPath = SVGPathCommander.normalizePath(path);
   * // result => [['M', 0, 0], ['L', 50, 0]]
   */
  function normalizePath$1(pathInput) {
      if (isNormalizedArray$1(pathInput)) {
          return clonePath$1(pathInput);
      }
      var path = path2Absolute$1(pathInput);
      var params = __assign$1({}, paramsParser$1);
      for (var i = 0; i < path.length; i += 1) {
          // Save current path command
          path[i] = normalizeSegment$1(path[i], params);
          var segment = path[i];
          var seglen = segment.length;
          params.x1 = +segment[seglen - 2];
          params.y1 = +segment[seglen - 1];
          params.x2 = +segment[seglen - 4] || params.x1;
          params.y2 = +segment[seglen - 3] || params.y1;
      }
      return path;
  }

  /**
   * Iterates an array to check if it's a `PathArray`
   * with all C (cubic bezier) segments.
   *
   * @param {string | PathArray} path the `Array` to be checked
   * @returns {boolean} iteration result
   */
  function isCurveArray$1(path) {
      return isNormalizedArray$1(path) && path.every(function (_a) {
          var pc = _a[0];
          return 'MC'.includes(pc);
      });
  }

  function rotateVector$1(x, y, rad) {
      var X = x * Math.cos(rad) - y * Math.sin(rad);
      var Y = x * Math.sin(rad) + y * Math.cos(rad);
      return { x: X, y: Y };
  }

  /**
   * Converts A (arc-to) segments to C (cubic-bezier-to).
   *
   * For more information of where this math came from visit:
   * http://www.w3.org/TR/SVG11/implnote.html#ArcImplementationNotes
   */
  function arcToCubic$1(X1, Y1, RX, RY, angle, LAF, SF, X2, Y2, recursive) {
      var x1 = X1;
      var y1 = Y1;
      var rx = RX;
      var ry = RY;
      var x2 = X2;
      var y2 = Y2;
      // for more information of where this Math came from visit:
      // http://www.w3.org/TR/SVG11/implnote.html#ArcImplementationNotes
      var d120 = (Math.PI * 120) / 180;
      var rad = (Math.PI / 180) * (+angle || 0);
      /** @type {number[]} */
      var res = [];
      var xy;
      var f1;
      var f2;
      var cx;
      var cy;
      if (!recursive) {
          xy = rotateVector$1(x1, y1, -rad);
          x1 = xy.x;
          y1 = xy.y;
          xy = rotateVector$1(x2, y2, -rad);
          x2 = xy.x;
          y2 = xy.y;
          var x = (x1 - x2) / 2;
          var y = (y1 - y2) / 2;
          var h = (x * x) / (rx * rx) + (y * y) / (ry * ry);
          if (h > 1) {
              h = Math.sqrt(h);
              rx *= h;
              ry *= h;
          }
          var rx2 = rx * rx;
          var ry2 = ry * ry;
          var k = (LAF === SF ? -1 : 1) *
              Math.sqrt(Math.abs((rx2 * ry2 - rx2 * y * y - ry2 * x * x) / (rx2 * y * y + ry2 * x * x)));
          cx = (k * rx * y) / ry + (x1 + x2) / 2;
          cy = (k * -ry * x) / rx + (y1 + y2) / 2;
          // eslint-disable-next-line no-bitwise -- Impossible to satisfy no-bitwise
          f1 = Math.asin(((((y1 - cy) / ry) * Math.pow(10, 9)) >> 0) / Math.pow(10, 9));
          // eslint-disable-next-line no-bitwise -- Impossible to satisfy no-bitwise
          f2 = Math.asin(((((y2 - cy) / ry) * Math.pow(10, 9)) >> 0) / Math.pow(10, 9));
          f1 = x1 < cx ? Math.PI - f1 : f1;
          f2 = x2 < cx ? Math.PI - f2 : f2;
          if (f1 < 0)
              f1 = Math.PI * 2 + f1;
          if (f2 < 0)
              f2 = Math.PI * 2 + f2;
          if (SF && f1 > f2) {
              f1 -= Math.PI * 2;
          }
          if (!SF && f2 > f1) {
              f2 -= Math.PI * 2;
          }
      }
      else {
          f1 = recursive[0], f2 = recursive[1], cx = recursive[2], cy = recursive[3];
      }
      var df = f2 - f1;
      if (Math.abs(df) > d120) {
          var f2old = f2;
          var x2old = x2;
          var y2old = y2;
          f2 = f1 + d120 * (SF && f2 > f1 ? 1 : -1);
          x2 = cx + rx * Math.cos(f2);
          y2 = cy + ry * Math.sin(f2);
          res = arcToCubic$1(x2, y2, rx, ry, angle, 0, SF, x2old, y2old, [f2, f2old, cx, cy]);
      }
      df = f2 - f1;
      var c1 = Math.cos(f1);
      var s1 = Math.sin(f1);
      var c2 = Math.cos(f2);
      var s2 = Math.sin(f2);
      var t = Math.tan(df / 4);
      var hx = (4 / 3) * rx * t;
      var hy = (4 / 3) * ry * t;
      var m1 = [x1, y1];
      var m2 = [x1 + hx * s1, y1 - hy * c1];
      var m3 = [x2 + hx * s2, y2 - hy * c2];
      var m4 = [x2, y2];
      m2[0] = 2 * m1[0] - m2[0];
      m2[1] = 2 * m1[1] - m2[1];
      if (recursive) {
          return m2.concat(m3, m4, res);
          // return [...m2, ...m3, ...m4, ...res];
      }
      res = m2.concat(m3, m4, res);
      // res = [...m2, ...m3, ...m4, ...res];
      var newres = [];
      for (var i = 0, ii = res.length; i < ii; i += 1) {
          newres[i] = i % 2 ? rotateVector$1(res[i - 1], res[i], rad).y : rotateVector$1(res[i], res[i + 1], rad).x;
      }
      return newres;
  }
  // const TAU = Math.PI * 2;
  // const mapToEllipse = (
  //   { x, y }: { x: number; y: number },
  //   rx: number,
  //   ry: number,
  //   cosphi: number,
  //   sinphi: number,
  //   centerx: number,
  //   centery: number,
  // ) => {
  //   x *= rx;
  //   y *= ry;
  //   const xp = cosphi * x - sinphi * y;
  //   const yp = sinphi * x + cosphi * y;
  //   return {
  //     x: xp + centerx,
  //     y: yp + centery,
  //   };
  // };
  // const approxUnitArc = (ang1: number, ang2: number) => {
  //   // If 90 degree circular arc, use a constant
  //   // as derived from http://spencermortensen.com/articles/bezier-circle
  //   const a =
  //     ang2 === 1.5707963267948966
  //       ? 0.551915024494
  //       : ang2 === -1.5707963267948966
  //       ? -0.551915024494
  //       : (4 / 3) * Math.tan(ang2 / 4);
  //   const x1 = Math.cos(ang1);
  //   const y1 = Math.sin(ang1);
  //   const x2 = Math.cos(ang1 + ang2);
  //   const y2 = Math.sin(ang1 + ang2);
  //   return [
  //     {
  //       x: x1 - y1 * a,
  //       y: y1 + x1 * a,
  //     },
  //     {
  //       x: x2 + y2 * a,
  //       y: y2 - x2 * a,
  //     },
  //     {
  //       x: x2,
  //       y: y2,
  //     },
  //   ];
  // };
  // const vectorAngle = (ux: number, uy: number, vx: number, vy: number) => {
  //   const sign = ux * vy - uy * vx < 0 ? -1 : 1;
  //   let dot = ux * vx + uy * vy;
  //   if (dot > 1) {
  //     dot = 1;
  //   }
  //   if (dot < -1) {
  //     dot = -1;
  //   }
  //   return sign * Math.acos(dot);
  // };
  // const getArcCenter = (
  //   px: any,
  //   py: any,
  //   cx: any,
  //   cy: any,
  //   rx: number,
  //   ry: number,
  //   largeArcFlag: number,
  //   sweepFlag: number,
  //   sinphi: number,
  //   cosphi: number,
  //   pxp: number,
  //   pyp: number,
  // ) => {
  //   const rxsq = Math.pow(rx, 2);
  //   const rysq = Math.pow(ry, 2);
  //   const pxpsq = Math.pow(pxp, 2);
  //   const pypsq = Math.pow(pyp, 2);
  //   let radicant = rxsq * rysq - rxsq * pypsq - rysq * pxpsq;
  //   if (radicant < 0) {
  //     radicant = 0;
  //   }
  //   radicant /= rxsq * pypsq + rysq * pxpsq;
  //   radicant = Math.sqrt(radicant) * (largeArcFlag === sweepFlag ? -1 : 1);
  //   const centerxp = ((radicant * rx) / ry) * pyp;
  //   const centeryp = ((radicant * -ry) / rx) * pxp;
  //   const centerx = cosphi * centerxp - sinphi * centeryp + (px + cx) / 2;
  //   const centery = sinphi * centerxp + cosphi * centeryp + (py + cy) / 2;
  //   const vx1 = (pxp - centerxp) / rx;
  //   const vy1 = (pyp - centeryp) / ry;
  //   const vx2 = (-pxp - centerxp) / rx;
  //   const vy2 = (-pyp - centeryp) / ry;
  //   const ang1 = vectorAngle(1, 0, vx1, vy1);
  //   let ang2 = vectorAngle(vx1, vy1, vx2, vy2);
  //   if (sweepFlag === 0 && ang2 > 0) {
  //     ang2 -= TAU;
  //   }
  //   if (sweepFlag === 1 && ang2 < 0) {
  //     ang2 += TAU;
  //   }
  //   return [centerx, centery, ang1, ang2];
  // };
  // const arcToBezier = ({ px, py, cx, cy, rx, ry, xAxisRotation = 0, largeArcFlag = 0, sweepFlag = 0 }) => {
  //   const curves = [];
  //   if (rx === 0 || ry === 0) {
  //     return [{ x1: 0, y1: 0, x2: 0, y2: 0, x: cx, y: cy }];
  //   }
  //   const sinphi = Math.sin((xAxisRotation * TAU) / 360);
  //   const cosphi = Math.cos((xAxisRotation * TAU) / 360);
  //   const pxp = (cosphi * (px - cx)) / 2 + (sinphi * (py - cy)) / 2;
  //   const pyp = (-sinphi * (px - cx)) / 2 + (cosphi * (py - cy)) / 2;
  //   if (pxp === 0 && pyp === 0) {
  //     return [{ x1: 0, y1: 0, x2: 0, y2: 0, x: cx, y: cy }];
  //   }
  //   rx = Math.abs(rx);
  //   ry = Math.abs(ry);
  //   const lambda = Math.pow(pxp, 2) / Math.pow(rx, 2) + Math.pow(pyp, 2) / Math.pow(ry, 2);
  //   if (lambda > 1) {
  //     rx *= Math.sqrt(lambda);
  //     ry *= Math.sqrt(lambda);
  //   }
  //   let [centerx, centery, ang1, ang2] = getArcCenter(
  //     px,
  //     py,
  //     cx,
  //     cy,
  //     rx,
  //     ry,
  //     largeArcFlag,
  //     sweepFlag,
  //     sinphi,
  //     cosphi,
  //     pxp,
  //     pyp,
  //   );
  //   // If 'ang2' == 90.0000000001, then `ratio` will evaluate to
  //   // 1.0000000001. This causes `segments` to be greater than one, which is an
  //   // unecessary split, and adds extra points to the bezier curve. To alleviate
  //   // this issue, we round to 1.0 when the ratio is close to 1.0.
  //   let ratio = Math.abs(ang2) / (TAU / 4);
  //   if (Math.abs(1.0 - ratio) < 0.0000001) {
  //     ratio = 1.0;
  //   }
  //   const segments = Math.max(Math.ceil(ratio), 1);
  //   ang2 /= segments;
  //   for (let i = 0; i < segments; i++) {
  //     curves.push(approxUnitArc(ang1, ang2));
  //     ang1 += ang2;
  //   }
  //   return curves.map((curve) => {
  //     const { x: x1, y: y1 } = mapToEllipse(curve[0], rx, ry, cosphi, sinphi, centerx, centery);
  //     const { x: x2, y: y2 } = mapToEllipse(curve[1], rx, ry, cosphi, sinphi, centerx, centery);
  //     const { x, y } = mapToEllipse(curve[2], rx, ry, cosphi, sinphi, centerx, centery);
  //     return { x1, y1, x2, y2, x, y };
  //   });
  // };
  // export function arcToCubic(
  //   x1: number,
  //   y1: number,
  //   rx: number,
  //   ry: number,
  //   angle: number,
  //   LAF: number,
  //   SF: number,
  //   x2: number,
  //   y2: number,
  // ) {
  //   const curves = arcToBezier({
  //     px: x1,
  //     py: y1,
  //     cx: x2,
  //     cy: y2,
  //     rx,
  //     ry,
  //     xAxisRotation: angle,
  //     largeArcFlag: LAF,
  //     sweepFlag: SF,
  //   });
  //   return curves.reduce((prev, cur) => {
  //     const { x1, y1, x2, y2, x, y } = cur;
  //     prev.push(x1, y1, x2, y2, x, y);
  //     return prev;
  //   }, [] as number[]);
  // }

  function quadToCubic$1(x1, y1, qx, qy, x2, y2) {
      var r13 = 1 / 3;
      var r23 = 2 / 3;
      return [
          r13 * x1 + r23 * qx,
          r13 * y1 + r23 * qy,
          r13 * x2 + r23 * qx,
          r13 * y2 + r23 * qy,
          x2,
          y2, // x,y
      ];
  }

  function midPoint$1(a, b, t) {
      var ax = a[0];
      var ay = a[1];
      var bx = b[0];
      var by = b[1];
      return [ax + (bx - ax) * t, ay + (by - ay) * t];
  }

  function distanceSquareRoot$1(a, b) {
      return Math.sqrt((a[0] - b[0]) * (a[0] - b[0]) + (a[1] - b[1]) * (a[1] - b[1]));
  }

  /**
   * Returns a {x,y} point at a given length, the total length and
   * the minimum and maximum {x,y} coordinates of a line (L,V,H,Z) segment.
   */
  function segmentLineFactory$1(x1, y1, x2, y2, distance) {
      var length = distanceSquareRoot$1([x1, y1], [x2, y2]);
      var point = { x: 0, y: 0 };
      if (typeof distance === 'number') {
          if (distance <= 0) {
              point = { x: x1, y: y1 };
          }
          else if (distance >= length) {
              point = { x: x2, y: y2 };
          }
          else {
              var _a = midPoint$1([x1, y1], [x2, y2], distance / length), x = _a[0], y = _a[1];
              point = { x: x, y: y };
          }
      }
      return {
          length: length,
          point: point,
          min: {
              x: Math.min(x1, x2),
              y: Math.min(y1, y2),
          },
          max: {
              x: Math.max(x1, x2),
              y: Math.max(y1, y2),
          },
      };
  }

  function lineToCubic$1(x1, y1, x2, y2) {
      var t = 0.5;
      var p0 = [x1, y1];
      var p1 = [x2, y2];
      var p2 = midPoint$1(p0, p1, t);
      var p3 = midPoint$1(p1, p2, t);
      var p4 = midPoint$1(p2, p3, t);
      var p5 = midPoint$1(p3, p4, t);
      var p6 = midPoint$1(p4, p5, t);
      // const seg1 = [...p0, ...p2, ...p4, ...p6, t];
      // @ts-ignore
      var cp1 = segmentLineFactory$1(p0[0], p0[1], p2[0], p2[1], p4[0]).point;
      // const seg2 = [...p6, ...p5, ...p3, ...p1, 0];
      // @ts-ignore
      var cp2 = segmentLineFactory$1(p6[0], p6[1], p5[0], p5[1], p3[0]).point;
      return [cp1.x, cp1.y, cp2.x, cp2.y, x2, y2];
  }

  function segmentToCubic$1(segment, params) {
      var pathCommand = segment[0];
      var values = segment.slice(1).map(Number);
      var x = values[0], y = values[1];
      var args;
      var px1 = params.x1, py1 = params.y1, px = params.x, py = params.y;
      if (!'TQ'.includes(pathCommand)) {
          params.qx = null;
          params.qy = null;
      }
      switch (pathCommand) {
          case 'M':
              params.x = x;
              params.y = y;
              return segment;
          case 'A':
              args = [px1, py1].concat(values);
              // @ts-ignore
              return ['C'].concat(arcToCubic$1(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9]));
          case 'Q':
              params.qx = x;
              params.qy = y;
              args = [px1, py1].concat(values);
              // @ts-ignore
              return ['C'].concat(quadToCubic$1(args[0], args[1], args[2], args[3], args[4], args[5]));
          case 'L':
              // @ts-ignore
              return ['C'].concat(lineToCubic$1(px1, py1, x, y));
          case 'Z':
              // prevent NaN from divide 0
              if (px1 === px && py1 === py) {
                  return ['C', px1, py1, px, py, px, py];
              }
              // @ts-ignore
              return ['C'].concat(lineToCubic$1(px1, py1, px, py));
      }
      return segment;
  }

  // import { fixPath } from '../process/fix-path';
  function path2Curve$1(pathInput, needZCommandIndexes) {
      if (needZCommandIndexes === void 0) { needZCommandIndexes = false; }
      if (isCurveArray$1(pathInput)) {
          var cloned = clonePath$1(pathInput);
          if (needZCommandIndexes) {
              return [cloned, []];
          }
          else {
              return cloned;
          }
      }
      // fixPath will remove 'Z' command
      // const path = fixPath(normalizePath(pathInput));
      var path = normalizePath$1(pathInput);
      var params = __assign$1({}, paramsParser$1);
      var allPathCommands = [];
      var pathCommand = '';
      var ii = path.length;
      var segment;
      var seglen;
      var zCommandIndexes = [];
      for (var i = 0; i < ii; i += 1) {
          if (path[i])
              pathCommand = path[i][0];
          allPathCommands[i] = pathCommand;
          var curveSegment = segmentToCubic$1(path[i], params);
          path[i] = curveSegment;
          fixArc$1(path, allPathCommands, i);
          ii = path.length; // solves curveArrays ending in Z
          // keep Z command account for lineJoin
          // @see https://github.com/antvis/util/issues/68
          if (pathCommand === 'Z') {
              zCommandIndexes.push(i);
          }
          segment = path[i];
          seglen = segment.length;
          params.x1 = +segment[seglen - 2];
          params.y1 = +segment[seglen - 1];
          params.x2 = +segment[seglen - 4] || params.x1;
          params.y2 = +segment[seglen - 3] || params.y1;
      }
      // validate
      if (needZCommandIndexes) {
          return [path, zCommandIndexes];
      }
      else {
          return path;
      }
  }

  // reverse CURVE based pathArray segments only
  function reverseCurve$1(pathArray) {
      var rotatedCurve = pathArray
          .slice(1)
          .map(function (x, i, curveOnly) {
          // @ts-ignore
          return !i ? pathArray[0].slice(1).concat(x.slice(1)) : curveOnly[i - 1].slice(-2).concat(x.slice(1));
      })
          // @ts-ignore
          .map(function (x) { return x.map(function (y, i) { return x[x.length - i - 2 * (1 - (i % 2))]; }); })
          .reverse();
      return [['M'].concat(rotatedCurve[0].slice(0, 2))].concat(rotatedCurve.map(function (x) { return ['C'].concat(x.slice(2)); }));
  }

  function angleBetween$2(v0, v1) {
      var v0x = v0.x, v0y = v0.y;
      var v1x = v1.x, v1y = v1.y;
      var p = v0x * v1x + v0y * v1y;
      var n = Math.sqrt((Math.pow(v0x, 2) + Math.pow(v0y, 2)) * (Math.pow(v1x, 2) + Math.pow(v1y, 2)));
      var sign = v0x * v1y - v0y * v1x < 0 ? -1 : 1;
      var angle = sign * Math.acos(p / n);
      return angle;
  }
  /**
   * Returns a {x,y} point at a given length, the total length and
   * the minimum and maximum {x,y} coordinates of a C (cubic-bezier) segment.
   * @see https://github.com/MadLittleMods/svg-curve-lib/blob/master/src/js/svg-curve-lib.js
   */
  function getPointAtArcSegmentLength$1(x1, y1, RX, RY, angle, LAF, SF, x, y, t) {
      var abs = Math.abs, sin = Math.sin, cos = Math.cos, sqrt = Math.sqrt, PI = Math.PI;
      var rx = abs(RX);
      var ry = abs(RY);
      var xRot = ((angle % 360) + 360) % 360;
      var xRotRad = xRot * (PI / 180);
      if (x1 === x && y1 === y) {
          return { x: x1, y: y1 };
      }
      if (rx === 0 || ry === 0) {
          return segmentLineFactory$1(x1, y1, x, y, t).point;
      }
      var dx = (x1 - x) / 2;
      var dy = (y1 - y) / 2;
      var transformedPoint = {
          x: cos(xRotRad) * dx + sin(xRotRad) * dy,
          y: -sin(xRotRad) * dx + cos(xRotRad) * dy,
      };
      var radiiCheck = Math.pow(transformedPoint.x, 2) / Math.pow(rx, 2) + Math.pow(transformedPoint.y, 2) / Math.pow(ry, 2);
      if (radiiCheck > 1) {
          rx *= sqrt(radiiCheck);
          ry *= sqrt(radiiCheck);
      }
      var cSquareNumerator = Math.pow(rx, 2) * Math.pow(ry, 2) - Math.pow(rx, 2) * Math.pow(transformedPoint.y, 2) - Math.pow(ry, 2) * Math.pow(transformedPoint.x, 2);
      var cSquareRootDenom = Math.pow(rx, 2) * Math.pow(transformedPoint.y, 2) + Math.pow(ry, 2) * Math.pow(transformedPoint.x, 2);
      var cRadicand = cSquareNumerator / cSquareRootDenom;
      cRadicand = cRadicand < 0 ? 0 : cRadicand;
      var cCoef = (LAF !== SF ? 1 : -1) * sqrt(cRadicand);
      var transformedCenter = {
          x: cCoef * ((rx * transformedPoint.y) / ry),
          y: cCoef * (-(ry * transformedPoint.x) / rx),
      };
      var center = {
          x: cos(xRotRad) * transformedCenter.x - sin(xRotRad) * transformedCenter.y + (x1 + x) / 2,
          y: sin(xRotRad) * transformedCenter.x + cos(xRotRad) * transformedCenter.y + (y1 + y) / 2,
      };
      var startVector = {
          x: (transformedPoint.x - transformedCenter.x) / rx,
          y: (transformedPoint.y - transformedCenter.y) / ry,
      };
      var startAngle = angleBetween$2({ x: 1, y: 0 }, startVector);
      var endVector = {
          x: (-transformedPoint.x - transformedCenter.x) / rx,
          y: (-transformedPoint.y - transformedCenter.y) / ry,
      };
      var sweepAngle = angleBetween$2(startVector, endVector);
      if (!SF && sweepAngle > 0) {
          sweepAngle -= 2 * PI;
      }
      else if (SF && sweepAngle < 0) {
          sweepAngle += 2 * PI;
      }
      sweepAngle %= 2 * PI;
      var alpha = startAngle + sweepAngle * t;
      var ellipseComponentX = rx * cos(alpha);
      var ellipseComponentY = ry * sin(alpha);
      var point = {
          x: cos(xRotRad) * ellipseComponentX - sin(xRotRad) * ellipseComponentY + center.x,
          y: sin(xRotRad) * ellipseComponentX + cos(xRotRad) * ellipseComponentY + center.y,
      };
      // to be used later
      // point.ellipticalArcStartAngle = startAngle;
      // point.ellipticalArcEndAngle = startAngle + sweepAngle;
      // point.ellipticalArcAngle = alpha;
      // point.ellipticalArcCenter = center;
      // point.resultantRx = rx;
      // point.resultantRy = ry;
      return point;
  }
  /**
   * Returns a {x,y} point at a given length, the total length and
   * the shape minimum and maximum {x,y} coordinates of an A (arc-to) segment.
   *
   * For better performance, it can skip calculate bbox or length in some scenario.
   */
  function segmentArcFactory$1(X1, Y1, RX, RY, angle, LAF, SF, X2, Y2, distance, options) {
      var _a;
      var _b = options.bbox, bbox = _b === void 0 ? true : _b, _c = options.length, length = _c === void 0 ? true : _c, _d = options.sampleSize, sampleSize = _d === void 0 ? 30 : _d;
      var distanceIsNumber = typeof distance === 'number';
      var x = X1;
      var y = Y1;
      var LENGTH = 0;
      var prev = [x, y, LENGTH];
      var cur = [x, y];
      var t = 0;
      var POINT = { x: 0, y: 0 };
      var POINTS = [{ x: x, y: y }];
      if (distanceIsNumber && distance <= 0) {
          POINT = { x: x, y: y };
      }
      // bad perf when size > 100
      for (var j = 0; j <= sampleSize; j += 1) {
          t = j / sampleSize;
          (_a = getPointAtArcSegmentLength$1(X1, Y1, RX, RY, angle, LAF, SF, X2, Y2, t), x = _a.x, y = _a.y);
          if (bbox) {
              POINTS.push({ x: x, y: y });
          }
          if (length) {
              LENGTH += distanceSquareRoot$1(cur, [x, y]);
          }
          cur = [x, y];
          if (distanceIsNumber && LENGTH >= distance && distance > prev[2]) {
              var dv = (LENGTH - distance) / (LENGTH - prev[2]);
              POINT = {
                  x: cur[0] * (1 - dv) + prev[0] * dv,
                  y: cur[1] * (1 - dv) + prev[1] * dv,
              };
          }
          prev = [x, y, LENGTH];
      }
      if (distanceIsNumber && distance >= LENGTH) {
          POINT = { x: X2, y: Y2 };
      }
      return {
          length: LENGTH,
          point: POINT,
          min: {
              x: Math.min.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.min.apply(null, POINTS.map(function (n) { return n.y; })),
          },
          max: {
              x: Math.max.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.max.apply(null, POINTS.map(function (n) { return n.y; })),
          },
      };
  }

  /**
   * Returns a {x,y} point at a given length, the total length and
   * the minimum and maximum {x,y} coordinates of a C (cubic-bezier) segment.
   */
  function getPointAtCubicSegmentLength$1(x1, y1, c1x, c1y, c2x, c2y, x2, y2, t) {
      var t1 = 1 - t;
      return {
          x: Math.pow(t1, 3) * x1 + 3 * Math.pow(t1, 2) * t * c1x + 3 * t1 * Math.pow(t, 2) * c2x + Math.pow(t, 3) * x2,
          y: Math.pow(t1, 3) * y1 + 3 * Math.pow(t1, 2) * t * c1y + 3 * t1 * Math.pow(t, 2) * c2y + Math.pow(t, 3) * y2,
      };
  }
  /**
   * Returns the length of a C (cubic-bezier) segment
   * or an {x,y} point at a given length.
   */
  function segmentCubicFactory$1(x1, y1, c1x, c1y, c2x, c2y, x2, y2, distance, options) {
      var _a;
      var _b = options.bbox, bbox = _b === void 0 ? true : _b, _c = options.length, length = _c === void 0 ? true : _c, _d = options.sampleSize, sampleSize = _d === void 0 ? 10 : _d;
      var distanceIsNumber = typeof distance === 'number';
      var x = x1;
      var y = y1;
      var LENGTH = 0;
      var prev = [x, y, LENGTH];
      var cur = [x, y];
      var t = 0;
      var POINT = { x: 0, y: 0 };
      var POINTS = [{ x: x, y: y }];
      if (distanceIsNumber && distance <= 0) {
          POINT = { x: x, y: y };
      }
      // bad perf when size = 300
      for (var j = 0; j <= sampleSize; j += 1) {
          t = j / sampleSize;
          (_a = getPointAtCubicSegmentLength$1(x1, y1, c1x, c1y, c2x, c2y, x2, y2, t), x = _a.x, y = _a.y);
          if (bbox) {
              POINTS.push({ x: x, y: y });
          }
          if (length) {
              LENGTH += distanceSquareRoot$1(cur, [x, y]);
          }
          cur = [x, y];
          if (distanceIsNumber && LENGTH >= distance && distance > prev[2]) {
              var dv = (LENGTH - distance) / (LENGTH - prev[2]);
              POINT = {
                  x: cur[0] * (1 - dv) + prev[0] * dv,
                  y: cur[1] * (1 - dv) + prev[1] * dv,
              };
          }
          prev = [x, y, LENGTH];
      }
      if (distanceIsNumber && distance >= LENGTH) {
          POINT = { x: x2, y: y2 };
      }
      return {
          length: LENGTH,
          point: POINT,
          min: {
              x: Math.min.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.min.apply(null, POINTS.map(function (n) { return n.y; })),
          },
          max: {
              x: Math.max.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.max.apply(null, POINTS.map(function (n) { return n.y; })),
          },
      };
  }

  /**
   * Returns the {x,y} coordinates of a point at a
   * given length of a quadratic-bezier segment.
   *
   * @see https://github.com/substack/point-at-length
   */
  function getPointAtQuadSegmentLength$1(x1, y1, cx, cy, x2, y2, t) {
      var t1 = 1 - t;
      return {
          x: Math.pow(t1, 2) * x1 + 2 * t1 * t * cx + Math.pow(t, 2) * x2,
          y: Math.pow(t1, 2) * y1 + 2 * t1 * t * cy + Math.pow(t, 2) * y2,
      };
  }
  /**
   * Returns a {x,y} point at a given length, the total length and
   * the minimum and maximum {x,y} coordinates of a Q (quadratic-bezier) segment.
   */
  function segmentQuadFactory$1(x1, y1, qx, qy, x2, y2, distance, options) {
      var _a;
      var _b = options.bbox, bbox = _b === void 0 ? true : _b, _c = options.length, length = _c === void 0 ? true : _c, _d = options.sampleSize, sampleSize = _d === void 0 ? 10 : _d;
      var distanceIsNumber = typeof distance === 'number';
      var x = x1;
      var y = y1;
      var LENGTH = 0;
      var prev = [x, y, LENGTH];
      var cur = [x, y];
      var t = 0;
      var POINT = { x: 0, y: 0 };
      var POINTS = [{ x: x, y: y }];
      if (distanceIsNumber && distance <= 0) {
          POINT = { x: x, y: y };
      }
      for (var j = 0; j <= sampleSize; j += 1) {
          t = j / sampleSize;
          (_a = getPointAtQuadSegmentLength$1(x1, y1, qx, qy, x2, y2, t), x = _a.x, y = _a.y);
          if (bbox) {
              POINTS.push({ x: x, y: y });
          }
          if (length) {
              LENGTH += distanceSquareRoot$1(cur, [x, y]);
          }
          cur = [x, y];
          if (distanceIsNumber && LENGTH >= distance && distance > prev[2]) {
              var dv = (LENGTH - distance) / (LENGTH - prev[2]);
              POINT = {
                  x: cur[0] * (1 - dv) + prev[0] * dv,
                  y: cur[1] * (1 - dv) + prev[1] * dv,
              };
          }
          prev = [x, y, LENGTH];
      }
      /* istanbul ignore else */
      if (distanceIsNumber && distance >= LENGTH) {
          POINT = { x: x2, y: y2 };
      }
      return {
          length: LENGTH,
          point: POINT,
          min: {
              x: Math.min.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.min.apply(null, POINTS.map(function (n) { return n.y; })),
          },
          max: {
              x: Math.max.apply(null, POINTS.map(function (n) { return n.x; })),
              y: Math.max.apply(null, POINTS.map(function (n) { return n.y; })),
          },
      };
  }

  /**
   * Returns a {x,y} point at a given length
   * of a shape, the shape total length and
   * the shape minimum and maximum {x,y} coordinates.
   */
  function pathLengthFactory$1(pathInput, distance, options) {
      var _a, _b, _c, _d, _e, _f;
      var path = normalizePath$1(pathInput);
      var distanceIsNumber = typeof distance === 'number';
      var isM;
      var data = [];
      var pathCommand;
      var x = 0;
      var y = 0;
      var mx = 0;
      var my = 0;
      var seg;
      var MIN = [];
      var MAX = [];
      var length = 0;
      var min = { x: 0, y: 0 };
      var max = min;
      var point = min;
      var POINT = min;
      var LENGTH = 0;
      for (var i = 0, ll = path.length; i < ll; i += 1) {
          seg = path[i];
          pathCommand = seg[0];
          isM = pathCommand === 'M';
          data = !isM ? [x, y].concat(seg.slice(1)) : data;
          // this segment is always ZERO
          /* istanbul ignore else */
          if (isM) {
              // remember mx, my for Z
              mx = seg[1], my = seg[2];
              min = { x: mx, y: my };
              max = min;
              length = 0;
              if (distanceIsNumber && distance < 0.001) {
                  POINT = min;
              }
          }
          else if (pathCommand === 'L') {
              (_a = segmentLineFactory$1(data[0], data[1], data[2], data[3], (distance || 0) - LENGTH), length = _a.length, min = _a.min, max = _a.max, point = _a.point);
          }
          else if (pathCommand === 'A') {
              (_b = segmentArcFactory$1(data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7], data[8], (distance || 0) - LENGTH, options || {}), length = _b.length, min = _b.min, max = _b.max, point = _b.point);
          }
          else if (pathCommand === 'C') {
              (_c = segmentCubicFactory$1(data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7], (distance || 0) - LENGTH, options || {}), length = _c.length, min = _c.min, max = _c.max, point = _c.point);
          }
          else if (pathCommand === 'Q') {
              (_d = segmentQuadFactory$1(data[0], data[1], data[2], data[3], data[4], data[5], (distance || 0) - LENGTH, options || {}), length = _d.length, min = _d.min, max = _d.max, point = _d.point);
          }
          else if (pathCommand === 'Z') {
              data = [x, y, mx, my];
              (_e = segmentLineFactory$1(data[0], data[1], data[2], data[3], (distance || 0) - LENGTH), length = _e.length, min = _e.min, max = _e.max, point = _e.point);
          }
          if (distanceIsNumber && LENGTH < distance && LENGTH + length >= distance) {
              POINT = point;
          }
          MAX.push(max);
          MIN.push(min);
          LENGTH += length;
          _f = pathCommand !== 'Z' ? seg.slice(-2) : [mx, my], x = _f[0], y = _f[1];
      }
      // native `getPointAtLength` behavior when the given distance
      // is higher than total length
      if (distanceIsNumber && distance >= LENGTH) {
          POINT = { x: x, y: y };
      }
      return {
          length: LENGTH,
          point: POINT,
          min: {
              x: Math.min.apply(null, MIN.map(function (n) { return n.x; })),
              y: Math.min.apply(null, MIN.map(function (n) { return n.y; })),
          },
          max: {
              x: Math.max.apply(null, MAX.map(function (n) { return n.x; })),
              y: Math.max.apply(null, MAX.map(function (n) { return n.y; })),
          },
      };
  }

  /**
   * Returns the shape total length, or the equivalent to `shape.getTotalLength()`.
   *
   * The `normalizePath` version is lighter, faster, more efficient and more accurate
   * with paths that are not `curveArray`.
   */
  function getTotalLength$1(pathInput, options) {
      return pathLengthFactory$1(pathInput, undefined, __assign$1(__assign$1({}, options), { bbox: false, length: true })).length;
  }

  function getRotations$1(a) {
      var segCount = a.length;
      var pointCount = segCount - 1;
      return a.map(function (f, idx) {
          return a.map(function (p, i) {
              var oldSegIdx = idx + i;
              var seg;
              if (i === 0 || (a[oldSegIdx] && a[oldSegIdx][0] === 'M')) {
                  seg = a[oldSegIdx];
                  return ['M'].concat(seg.slice(-2));
              }
              if (oldSegIdx >= segCount)
                  oldSegIdx -= pointCount;
              return a[oldSegIdx];
          });
      });
  }
  function getRotatedCurve$1(a, b) {
      var segCount = a.length - 1;
      var lineLengths = [];
      var computedIndex = 0;
      var sumLensSqrd = 0;
      var rotations = getRotations$1(a);
      rotations.forEach(function (r, i) {
          a.slice(1).forEach(function (s, j) {
              // @ts-ignore
              sumLensSqrd += distanceSquareRoot$1(a[(i + j) % segCount].slice(-2), b[j % segCount].slice(-2));
          });
          lineLengths[i] = sumLensSqrd;
          sumLensSqrd = 0;
      });
      computedIndex = lineLengths.indexOf(Math.min.apply(null, lineLengths));
      return rotations[computedIndex];
  }

  /**
   * Returns the area of a single cubic-bezier segment.
   *
   * http://objectmix.com/graphics/133553-area-closed-bezier-curve.html
   */
  function getCubicSegArea$1(x1, y1, c1x, c1y, c2x, c2y, x2, y2) {
      // https://stackoverflow.com/a/15845996
      return ((3 *
          ((y2 - y1) * (c1x + c2x) -
              (x2 - x1) * (c1y + c2y) +
              c1y * (x1 - c2x) -
              c1x * (y1 - c2y) +
              y2 * (c2x + x1 / 3) -
              x2 * (c2y + y1 / 3))) /
          20);
  }
  /**
   * Returns the area of a shape.
   * @author Jürg Lehni & Jonathan Puckey
   *
   * @see https://github.com/paperjs/paper.js/blob/develop/src/path/Path.js
   */
  function getPathArea$1(path) {
      var x = 0;
      var y = 0;
      var len = 0;
      return path2Curve$1(path)
          .map(function (seg) {
          var _a;
          switch (seg[0]) {
              case 'M':
                  x = seg[1], y = seg[2];
                  return 0;
              default:
                  // @ts-ignore
                  var _b = seg.slice(1), c1x = _b[0], c1y = _b[1], c2x = _b[2], c2y = _b[3], x2 = _b[4], y2 = _b[5];
                  len = getCubicSegArea$1(x, y, c1x, c1y, c2x, c2y, x2, y2);
                  _a = seg.slice(-2), x = _a[0], y = _a[1];
                  return len;
          }
      })
          .reduce(function (a, b) { return a + b; }, 0);
  }
  // export function getPathArea(pathArray: AbsoluteArray) {
  //   let x = 0;
  //   let y = 0;
  //   let mx = 0;
  //   let my = 0;
  //   let len = 0;
  //   return pathArray
  //     .map((seg) => {
  //       switch (seg[0]) {
  //         case 'M':
  //         case 'Z':
  //           mx = seg[0] === 'M' ? seg[1] : mx;
  //           my = seg[0] === 'M' ? seg[2] : my;
  //           x = mx;
  //           y = my;
  //           return 0;
  //         default:
  //           // @ts-ignore
  //           len = getCubicSegArea.apply(0, [x, y].concat(seg.slice(1)));
  //           [x, y] = seg.slice(-2) as [number, number];
  //           return len;
  //       }
  //     })
  //     .reduce((a, b) => a + b, 0);
  // }

  function getDrawDirection$1(pathArray) {
      return getPathArea$1(pathArray) >= 0;
  }

  /**
   * Returns [x,y] coordinates of a point at a given length of a shape.
   */
  function getPointAtLength$1(pathInput, distance, options) {
      return pathLengthFactory$1(pathInput, distance, __assign$1(__assign$1({}, options), { bbox: false, length: true })).point;
  }

  function splitCubic$1(pts, t) {
      if (t === void 0) { t = 0.5; }
      var p0 = pts.slice(0, 2);
      var p1 = pts.slice(2, 4);
      var p2 = pts.slice(4, 6);
      var p3 = pts.slice(6, 8);
      var p4 = midPoint$1(p0, p1, t);
      var p5 = midPoint$1(p1, p2, t);
      var p6 = midPoint$1(p2, p3, t);
      var p7 = midPoint$1(p4, p5, t);
      var p8 = midPoint$1(p5, p6, t);
      var p9 = midPoint$1(p7, p8, t);
      return [
          // @ts-ignore
          ['C'].concat(p4, p7, p9),
          // @ts-ignore
          ['C'].concat(p8, p6, p3),
      ];
  }
  function getCurveArray$1(segments) {
      return segments.map(function (segment, i, pathArray) {
          // @ts-ignore
          var segmentData = i && pathArray[i - 1].slice(-2).concat(segment.slice(1));
          // @ts-ignore
          var curveLength = i
              ? segmentCubicFactory$1(segmentData[0], segmentData[1], segmentData[2], segmentData[3], segmentData[4], segmentData[5], segmentData[6], segmentData[7], segmentData[8], { bbox: false }).length
              : 0;
          var subsegs;
          if (i) {
              // must be [segment,segment]
              subsegs = curveLength ? splitCubic$1(segmentData) : [segment, segment];
          }
          else {
              subsegs = [segment];
          }
          return {
              s: segment,
              ss: subsegs,
              l: curveLength,
          };
      });
  }
  function equalizeSegments$1(path1, path2, TL) {
      var c1 = getCurveArray$1(path1);
      var c2 = getCurveArray$1(path2);
      var L1 = c1.length;
      var L2 = c2.length;
      var l1 = c1.filter(function (x) { return x.l; }).length;
      var l2 = c2.filter(function (x) { return x.l; }).length;
      var m1 = c1.filter(function (x) { return x.l; }).reduce(function (a, _a) {
          var l = _a.l;
          return a + l;
      }, 0) / l1 || 0;
      var m2 = c2.filter(function (x) { return x.l; }).reduce(function (a, _a) {
          var l = _a.l;
          return a + l;
      }, 0) / l2 || 0;
      var tl = TL || Math.max(L1, L2);
      var mm = [m1, m2];
      var dif = [tl - L1, tl - L2];
      var canSplit = 0;
      var result = [c1, c2].map(function (x, i) {
          // @ts-ignore
          return x.l === tl
              ? x.map(function (y) { return y.s; })
              : x
                  .map(function (y, j) {
                  canSplit = j && dif[i] && y.l >= mm[i];
                  dif[i] -= canSplit ? 1 : 0;
                  return canSplit ? y.ss : [y.s];
              })
                  .flat();
      });
      return result[0].length === result[1].length ? result : equalizeSegments$1(result[0], result[1], tl);
  }

  var isArrayLike = function (value) {
      /**
       * isArrayLike([1, 2, 3]) => true
       * isArrayLike(document.body.children) => true
       * isArrayLike('abc') => true
       * isArrayLike(Function) => false
       */
      return value !== null && typeof value !== 'function' && isFinite(value.length);
  };

  var filter = function (arr, func) {
      if (!isArrayLike(arr)) {
          return arr;
      }
      var result = [];
      for (var index = 0; index < arr.length; index++) {
          var value = arr[index];
          if (func(value, index)) {
              result.push(value);
          }
      }
      return result;
  };

  /**
   * @see https://github.com/you-dont-need/You-Dont-Need-Lodash-Underscore#_isfunction
   */
  var isFunction$2 = (function (value) {
      return typeof value === 'function';
  });

  // isFinite,
  var isNil$1 = function (value) {
      /**
       * isNil(null) => true
       * isNil() => true
       */
      return value === null || value === undefined;
  };

  var toString$1 = {}.toString;
  var isType$1 = function (value, type) { return toString$1.call(value) === '[object ' + type + ']'; };

  var isArray$1 = (function (value) {
      return Array.isArray ? Array.isArray(value) : isType$1(value, 'Array');
  });

  var isObject$1 = (function (value) {
      /**
       * isObject({}) => true
       * isObject([1, 2, 3]) => true
       * isObject(Function) => true
       * isObject(null) => false
       */
      var type = typeof value;
      return (value !== null && type === 'object') || type === 'function';
  });

  function each$1(elements, func) {
      if (!elements) {
          return;
      }
      var rst;
      if (isArray$1(elements)) {
          for (var i = 0, len = elements.length; i < len; i++) {
              rst = func(elements[i], i);
              if (rst === false) {
                  break;
              }
          }
      }
      else if (isObject$1(elements)) {
          for (var k in elements) {
              if (elements.hasOwnProperty(k)) {
                  rst = func(elements[k], k);
                  if (rst === false) {
                      break;
                  }
              }
          }
      }
  }

  var keys = Object.keys
      ? function (obj) { return Object.keys(obj); }
      : function (obj) {
          var result = [];
          each$1(obj, function (value, key) {
              if (!(isFunction$2(obj) && key === 'prototype')) {
                  result.push(key);
              }
          });
          return result;
      };

  function isMatch(obj, attrs) {
      var _keys = keys(attrs);
      var length = _keys.length;
      if (isNil$1(obj))
          return !length;
      for (var i = 0; i < length; i += 1) {
          var key = _keys[i];
          if (attrs[key] !== obj[key] || !(key in obj)) {
              return false;
          }
      }
      return true;
  }

  var isObjectLike$1 = function (value) {
      /**
       * isObjectLike({}) => true
       * isObjectLike([1, 2, 3]) => true
       * isObjectLike(Function) => false
       * isObjectLike(null) => false
       */
      return typeof value === 'object' && value !== null;
  };

  var isPlainObject$1 = function (value) {
      /**
       * isObjectLike(new Foo) => false
       * isObjectLike([1, 2, 3]) => false
       * isObjectLike({ x: 0, y: 0 }) => true
       * isObjectLike(Object.create(null)) => true
       */
      if (!isObjectLike$1(value) || !isType$1(value, 'Object')) {
          return false;
      }
      if (Object.getPrototypeOf(value) === null) {
          return true;
      }
      var proto = value;
      while (Object.getPrototypeOf(proto) !== null) {
          proto = Object.getPrototypeOf(proto);
      }
      return Object.getPrototypeOf(value) === proto;
  };

  function find$1(arr, predicate) {
      if (!isArray$1(arr))
          return null;
      var _predicate;
      if (isFunction$2(predicate)) {
          _predicate = predicate;
      }
      if (isPlainObject$1(predicate)) {
          _predicate = function (a) { return isMatch(a, predicate); };
      }
      if (_predicate) {
          for (var i = 0; i < arr.length; i += 1) {
              if (_predicate(arr[i])) {
                  return arr[i];
              }
          }
      }
      return null;
  }

  function findIndex(arr, predicate, fromIndex) {
      if (fromIndex === void 0) { fromIndex = 0; }
      for (var i = fromIndex; i < arr.length; i++) {
          if (predicate(arr[i], i)) {
              // 找到终止循环
              return i;
          }
      }
      return -1;
  }

  /**
   * Flattens `array` a single level deep.
   *
   * @param {Array} arr The array to flatten.
   * @return {Array} Returns the new flattened array.
   * @example
   *
   * flatten([1, [2, [3, [4]], 5]]);  // => [1, 2, [3, [4]], 5]
   */
  var flatten = function (arr) {
      if (!isArray$1(arr)) {
          return [];
      }
      var rst = [];
      for (var i = 0; i < arr.length; i++) {
          rst = rst.concat(arr[i]);
      }
      return rst;
  };

  /**
   * @param {Array} arr The array to iterate over.
   * @return {*} Returns the maximum value.
   * @example
   *
   * max([1, 2]);
   * // => 2
   *
   * max([]);
   * // => undefined
   *
   * const data = new Array(1250010).fill(1).map((d,idx) => idx);
   *
   * max(data);
   * // => 1250010
   * // Math.max(...data) will encounter "Maximum call stack size exceeded" error
   */
  var max$4 = (function (arr) {
      if (!isArray$1(arr)) {
          return undefined;
      }
      return arr.reduce(function (prev, curr) {
          return Math.max(prev, curr);
      }, arr[0]);
  });

  /**
   * @param {Array} arr The array to iterate over.
   * @return {*} Returns the minimum value.
   * @example
   *
   * min([1, 2]);
   * // => 1
   *
   * min([]);
   * // => undefined
   *
   * const data = new Array(1250010).fill(1).map((d,idx) => idx);
   *
   * min(data);
   * // => 1250010
   * // Math.min(...data) will encounter "Maximum call stack size exceeded" error
   */
  var min$4 = (function (arr) {
      if (!isArray$1(arr)) {
          return undefined;
      }
      return arr.reduce(function (prev, curr) {
          return Math.min(prev, curr);
      }, arr[0]);
  });

  var getRange = function (values) {
      // 存在 NaN 时，min,max 判定会出问题
      var filterValues = values.filter(function (v) { return !isNaN(v); });
      if (!filterValues.length) {
          // 如果没有数值则直接返回0
          return {
              min: 0,
              max: 0,
          };
      }
      if (isArray$1(values[0])) {
          var tmp = [];
          for (var i = 0; i < values.length; i++) {
              tmp = tmp.concat(values[i]);
          }
          filterValues = tmp;
      }
      var max = max$4(filterValues);
      var min = min$4(filterValues);
      return {
          min: min,
          max: max,
      };
  };

  var isString$1 = (function (str) {
      return isType$1(str, 'String');
  });

  var valuesOfKey = (function (data, name) {
      var rst = [];
      var tmpMap = {};
      for (var i = 0; i < data.length; i++) {
          var obj = data[i];
          var value = obj[name];
          if (!isNil$1(value)) {
              // flatten
              if (!isArray$1(value)) {
                  value = [value];
              }
              for (var j = 0; j < value.length; j++) {
                  var val = value[j];
                  // unique
                  if (!tmpMap[val]) {
                      rst.push(val);
                      tmpMap[val] = true;
                  }
              }
          }
      }
      return rst;
  });

  function head(o) {
      if (isArrayLike(o)) {
          return o[0];
      }
      return undefined;
  }

  function last(o) {
      if (isArrayLike(o)) {
          var arr = o;
          return arr[arr.length - 1];
      }
      return undefined;
  }

  var hasOwnProperty$1 = Object.prototype.hasOwnProperty;
  function groupBy(data, condition) {
      if (!condition || !isArray$1(data)) {
          return {};
      }
      var result = {};
      // 兼容方法和 字符串的写法
      var predicate = isFunction$2(condition) ? condition : function (item) { return item[condition]; };
      var key;
      for (var i = 0; i < data.length; i++) {
          var item = data[i];
          key = predicate(item);
          if (hasOwnProperty$1.call(result, key)) {
              result[key].push(item);
          }
          else {
              result[key] = [item];
          }
      }
      return result;
  }

  /**
   * 将数据分组成 map
   * @param data
   * @param condition
   */
  function groupToMap(data, condition) {
      if (!condition) {
          return {
              0: data,
          };
      }
      if (!isFunction$2(condition)) {
          // 如果是字符串，则按照 a*b 风格成数组
          var paramscondition_1 = isArray$1(condition) ? condition : condition.replace(/\s+/g, '').split('*');
          condition = function (row) {
              var unique = '_'; // 避免出现数字作为Key的情况，会进行按照数字的排序
              // 根据字段列表的值，拼接成 key
              for (var i = 0, l = paramscondition_1.length; i < l; i++) {
                  unique += row[paramscondition_1[i]] && row[paramscondition_1[i]].toString();
              }
              return unique;
          };
      }
      return groupBy(data, condition);
  }

  var group = (function (data, condition) {
      if (!condition) {
          // 没有条件，则自身改成数组
          return [data];
      }
      var groups = groupToMap(data, condition);
      var array = [];
      for (var i in groups) {
          array.push(groups[i]);
      }
      return array;
  });

  var clamp$1 = function (a, min, max) {
      if (a < min) {
          return min;
      }
      else if (a > max) {
          return max;
      }
      return a;
  };

  /**
   * 判断是否数字
   * @return {Boolean} 是否数字
   */
  var isNumber$1 = function (value) {
      return isType$1(value, 'Number');
  };

  var PRECISION$1 = 0.00001; // numbers less than this is considered as 0
  function isNumberEqual$3(a, b, precision) {
      if (precision === void 0) { precision = PRECISION$1; }
      return Math.abs(a - b) < precision;
  }

  var mod$1 = function (n, m) {
      return ((n % m) + m) % m;
  };

  var toString$2 = (function (value) {
      if (isNil$1(value))
          return '';
      return value.toString();
  });

  var upperFirst = function (value) {
      var str = toString$2(value);
      return str.charAt(0).toUpperCase() + str.substring(1);
  };

  var toString$3 = {}.toString;
  var getType = function (value) {
      return toString$3
          .call(value)
          .replace(/^\[object /, '')
          .replace(/]$/, '');
  };

  /**
   * 是否是布尔类型
   *
   * @param {Object} value 测试的值
   * @return {Boolean}
   */
  var isBoolean$1 = function (value) {
      return isType$1(value, 'Boolean');
  };

  var isDate = function (value) {
      return isType$1(value, 'Date');
  };

  var objectProto = Object.prototype;
  var isPrototype = function (value) {
      var Ctor = value && value.constructor;
      var proto = (typeof Ctor === 'function' && Ctor.prototype) || objectProto;
      return value === proto;
  };

  var isUndefined$2 = function (value) {
      return value === undefined;
  };

  // FIXME: Mutable param should be forbidden in static lang.
  function _mix(dist, obj) {
      for (var key in obj) {
          if (obj.hasOwnProperty(key) && key !== 'constructor' && obj[key] !== undefined) {
              dist[key] = obj[key];
          }
      }
  }
  function mix(dist, src1, src2, src3) {
      if (src1)
          _mix(dist, src1);
      if (src2)
          _mix(dist, src2);
      if (src3)
          _mix(dist, src3);
      return dist;
  }

  var clone$4 = function (obj) {
      if (typeof obj !== 'object' || obj === null) {
          return obj;
      }
      var rst;
      if (isArray$1(obj)) {
          rst = [];
          for (var i = 0, l = obj.length; i < l; i++) {
              if (typeof obj[i] === 'object' && obj[i] != null) {
                  rst[i] = clone$4(obj[i]);
              }
              else {
                  rst[i] = obj[i];
              }
          }
      }
      else {
          rst = {};
          for (var k in obj) {
              if (typeof obj[k] === 'object' && obj[k] != null) {
                  rst[k] = clone$4(obj[k]);
              }
              else {
                  rst[k] = obj[k];
              }
          }
      }
      return rst;
  };

  /**
   * _.memoize(calColor);
   * _.memoize(calColor, (...args) => args[0]);
   * @param f
   * @param resolver
   */
  var memoize$1 = (function (f, resolver) {
      if (!isFunction$2(f)) {
          throw new TypeError('Expected a function');
      }
      var memoized = function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          // 使用方法构造 key，如果不存在 resolver，则直接取第一个参数作为 key
          var key = resolver ? resolver.apply(this, args) : args[0];
          var cache = memoized.cache;
          if (cache.has(key)) {
              return cache.get(key);
          }
          var result = f.apply(this, args);
          // 缓存起来
          cache.set(key, result);
          return result;
      };
      memoized.cache = new Map();
      return memoized;
  });

  var MAX_MIX_LEVEL = 5;
  function _deepMix(dist, src, level, maxLevel) {
      level = level || 0;
      maxLevel = maxLevel || MAX_MIX_LEVEL;
      for (var key in src) {
          if (src.hasOwnProperty(key)) {
              var value = src[key];
              if (value !== null && isPlainObject$1(value)) {
                  if (!isPlainObject$1(dist[key])) {
                      dist[key] = {};
                  }
                  if (level < maxLevel) {
                      _deepMix(dist[key], value, level + 1, maxLevel);
                  }
                  else {
                      dist[key] = src[key];
                  }
              }
              else if (isArray$1(value)) {
                  dist[key] = [];
                  dist[key] = dist[key].concat(value);
              }
              else if (value !== undefined) {
                  dist[key] = value;
              }
          }
      }
  }
  // todo 重写
  var deepMix = function (rst) {
      var args = [];
      for (var _i = 1; _i < arguments.length; _i++) {
          args[_i - 1] = arguments[_i];
      }
      for (var i = 0; i < args.length; i += 1) {
          _deepMix(rst, args[i]);
      }
      return rst;
  };

  var hasOwnProperty$2 = Object.prototype.hasOwnProperty;
  function isEmpty(value) {
      /**
       * isEmpty(null) => true
       * isEmpty() => true
       * isEmpty(true) => true
       * isEmpty(1) => true
       * isEmpty([1, 2, 3]) => false
       * isEmpty('abc') => false
       * isEmpty({ a: 1 }) => false
       */
      if (isNil$1(value)) {
          return true;
      }
      if (isArrayLike(value)) {
          return !value.length;
      }
      var type = getType(value);
      if (type === 'Map' || type === 'Set') {
          return !value.size;
      }
      if (isPrototype(value)) {
          return !Object.keys(value).length;
      }
      for (var key in value) {
          if (hasOwnProperty$2.call(value, key)) {
              return false;
          }
      }
      return true;
  }

  var map$2 = function (arr, func) {
      if (!isArrayLike(arr)) {
          // @ts-ignore
          return arr;
      }
      var result = [];
      for (var index = 0; index < arr.length; index++) {
          var value = arr[index];
          result.push(func(value, index));
      }
      return result;
  };

  var identity$2 = function (v) { return v; };
  var mapValues = (function (object, func) {
      if (func === void 0) { func = identity$2; }
      var r = {};
      if (isObject$1(object) && !isNil$1(object)) {
          Object.keys(object).forEach(function (key) {
              // @ts-ignore
              r[key] = func(object[key], key);
          });
      }
      return r;
  });

  /**
   * https://github.com/developit/dlv/blob/master/index.js
   * @param obj
   * @param key
   * @param defaultValue
   */
  var get = (function (obj, key, defaultValue) {
      var p = 0;
      var keyArr = isString$1(key) ? key.split('.') : key;
      while (obj && p < keyArr.length) {
          obj = obj[keyArr[p++]];
      }
      return obj === undefined || p < keyArr.length ? defaultValue : obj;
  });

  /**
   * k-v 存储
   */
  var default_1 = /** @class */ (function () {
      function default_1() {
          this.map = {};
      }
      default_1.prototype.has = function (key) {
          return this.map[key] !== undefined;
      };
      default_1.prototype.get = function (key, def) {
          var v = this.map[key];
          return v === undefined ? def : v;
      };
      default_1.prototype.set = function (key, value) {
          this.map[key] = value;
      };
      default_1.prototype.clear = function () {
          this.map = {};
      };
      default_1.prototype.delete = function (key) {
          delete this.map[key];
      };
      default_1.prototype.size = function () {
          return Object.keys(this.map).length;
      };
      return default_1;
  }());

  var arrayWithHoles = createCommonjsModule(function (module) {
  function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
  }
  module.exports = _arrayWithHoles, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var iterableToArrayLimit = createCommonjsModule(function (module) {
  function _iterableToArrayLimit(arr, i) {
    var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];
    if (_i == null) return;
    var _arr = [];
    var _n = true;
    var _d = false;
    var _s, _e;
    try {
      for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);
        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"] != null) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }
    return _arr;
  }
  module.exports = _iterableToArrayLimit, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var arrayLikeToArray = createCommonjsModule(function (module) {
  function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for (var i = 0, arr2 = new Array(len); i < len; i++) {
      arr2[i] = arr[i];
    }
    return arr2;
  }
  module.exports = _arrayLikeToArray, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var unsupportedIterableToArray = createCommonjsModule(function (module) {
  function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return arrayLikeToArray(o, minLen);
  }
  module.exports = _unsupportedIterableToArray, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var nonIterableRest = createCommonjsModule(function (module) {
  function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }
  module.exports = _nonIterableRest, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var slicedToArray = createCommonjsModule(function (module) {
  function _slicedToArray(arr, i) {
    return arrayWithHoles(arr) || iterableToArrayLimit(arr, i) || unsupportedIterableToArray(arr, i) || nonIterableRest();
  }
  module.exports = _slicedToArray, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _slicedToArray$1 = /*@__PURE__*/getDefaultExportFromCjs(slicedToArray);

  function transposedRect(_ref) {
    var xMin = _ref.xMin,
      xMax = _ref.xMax,
      yMin = _ref.yMin,
      yMax = _ref.yMax;
    return {
      xMin: yMin,
      xMax: yMax,
      yMin: xMin,
      yMax: xMax
    };
  }
  function _convertRect(_ref2) {
    var x = _ref2.x,
      y = _ref2.y,
      size = _ref2.size,
      y0 = _ref2.y0;
    var xMin;
    var xMax;
    if (isArray$1(x)) {
      xMin = x[0];
      xMax = x[1];
    } else {
      xMin = x - size / 2;
      xMax = x + size / 2;
    }
    var yMin;
    var yMax;
    if (isArray$1(y)) {
      yMin = y[0];
      yMax = y[1];
    } else {
      yMin = Math.min(y0, y);
      yMax = Math.max(y0, y);
    }
    return {
      xMin: xMin,
      xMax: xMax,
      yMin: yMin,
      yMax: yMax
    };
  }
  /**
   * 直角坐标系
   * convert相关的方法，涉及将标准坐标系映射到实际坐标系内
   * transform相关的方法，是仅将某一种关键点转换成另一种关键点 (比如将x/y/size/y0转换成yMin/yMax/..)
   */
  var Base = /*#__PURE__*/function () {
    function Base(option) {
      _classCallCheck$1(this, Base);
      this.left = 0;
      this.top = 0;
      this.width = 0;
      this.height = 0;
      // x y 调换
      this.transposed = false;
      // x，y 的值域，在极坐标中对应的就是弧度和半径
      this.x = [0, 1];
      this.y = [0, 1];
      this.update(option);
    }
    _createClass$3(Base, [{
      key: "update",
      value: function update(option) {
        mix(this, option);
        var left = this.left,
          top = this.top,
          width = this.width,
          height = this.height;
        this.right = left + width;
        this.bottom = top + height;
        this.center = {
          x: left + width / 2,
          y: top + height / 2
        };
        return this;
      }
      // 是循环， 比如极坐标是以 2π 循环的
    }, {
      key: "isCyclic",
      value: function isCyclic() {
        return false;
      }
    }, {
      key: "_zoomVal",
      value: function _zoomVal(val, func) {
        return isArray$1(val) ? val.map(function (v) {
          return func(v);
        }) : func(val);
      }
      /**
       * 把归一后的值映射到对应的定义域
       * @param point
       */
    }, {
      key: "convert",
      value: function convert(point) {
        var transposed = this.transposed,
          x = this.x,
          y = this.y;
        var xDim = transposed ? 'y' : 'x';
        var yDim = transposed ? 'x' : 'y';
        var pointX = point[xDim];
        var pointY = point[yDim];
        // 超出边界不绘制
        if (pointX < 0 || pointX > 1 || pointY < 0 || pointY > 1) {
          return {
            x: NaN,
            y: NaN
          };
        }
        return {
          x: this._zoomVal(point[xDim], function (v) {
            return x[0] + (x[1] - x[0]) * v;
          }),
          y: this._zoomVal(point[yDim], function (v) {
            return y[0] + (y[1] - y[0]) * v;
          })
        };
      }
      /**
       * convert 的反处理，把定义域的值，反处理到归一的值
       */
    }, {
      key: "invert",
      value: function invert(point) {
        var _ref3;
        var transposed = this.transposed,
          x = this.x,
          y = this.y;
        var xDim = transposed ? 'y' : 'x';
        var yDim = transposed ? 'x' : 'y';
        return _ref3 = {}, _defineProperty$1(_ref3, xDim, this._zoomVal(point.x, function (v) {
          return (v - x[0]) / (x[1] - x[0]);
        })), _defineProperty$1(_ref3, yDim, this._zoomVal(point.y, function (v) {
          return (v - y[0]) / (y[1] - y[0]);
        })), _ref3;
      }
      /**
       * 把归一化的值映射到 canvas 的坐标点
       * @param point
       * @returns
       */
    }, {
      key: "convertPoint",
      value: function convertPoint(point) {
        return this.convert(point);
      }
      /**
       * 把canvas坐标的点位映射回归一的值
       */
    }, {
      key: "invertPoint",
      value: function invertPoint(point) {
        return this.invert(point);
      }
      // 将标准坐标系下的矩形绘制关键点映射成实际绘制的坐标点
    }, {
      key: "convertRect",
      value: function convertRect(rectPoint) {
        var xRange = this.x,
          yRange = this.y,
          transposed = this.transposed;
        var _xRange = _slicedToArray$1(xRange, 2),
          xStart = _xRange[0],
          xEnd = _xRange[1];
        var _yRange = _slicedToArray$1(yRange, 2),
          yStart = _yRange[0],
          yEnd = _yRange[1];
        var rect = _convertRect(rectPoint);
        var _ref4 = transposed ? transposedRect(rect) : rect,
          xMin = _ref4.xMin,
          xMax = _ref4.xMax,
          yMin = _ref4.yMin,
          yMax = _ref4.yMax;
        var x0 = xStart + (xEnd - xStart) * xMin;
        var x1 = xStart + (xEnd - xStart) * xMax;
        var y0 = yStart + (yEnd - yStart) * yMin;
        var y1 = yStart + (yEnd - yStart) * yMax;
        return {
          xMin: Math.min(x0, x1),
          xMax: Math.max(x0, x1),
          yMin: Math.min(y0, y1),
          yMax: Math.max(y0, y1)
        };
      }
      // 将已经映射好的矩形绘制关键点转换成实际绘制的坐标点
    }, {
      key: "transformToRect",
      value: function transformToRect(rectPoint) {
        var x = rectPoint.x,
          y = rectPoint.y,
          y0 = rectPoint.y0,
          size = rectPoint.size;
        var coordOrigin = this.convertPoint({
          x: 0,
          y: y0
        });
        var transposed = this.transposed;
        var _rectPoint = {
          size: size,
          x: transposed ? y : x,
          y: transposed ? x : y,
          y0: transposed ? coordOrigin.x : coordOrigin.y
        };
        var rect = _convertRect(_rectPoint);
        var _ref5 = transposed ? transposedRect(rect) : rect,
          xMin = _ref5.xMin,
          xMax = _ref5.xMax,
          yMin = _ref5.yMin,
          yMax = _ref5.yMax;
        return {
          xMin: xMin,
          xMax: xMax,
          yMin: yMin,
          yMax: yMax
        };
      }
    }]);
    return Base;
  }();

  var superPropBase = createCommonjsModule(function (module) {
  function _superPropBase(object, property) {
    while (!Object.prototype.hasOwnProperty.call(object, property)) {
      object = getPrototypeOf(object);
      if (object === null) break;
    }
    return object;
  }
  module.exports = _superPropBase, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var get$1 = createCommonjsModule(function (module) {
  function _get() {
    if (typeof Reflect !== "undefined" && Reflect.get) {
      module.exports = _get = Reflect.get.bind(), module.exports.__esModule = true, module.exports["default"] = module.exports;
    } else {
      module.exports = _get = function _get(target, property, receiver) {
        var base = superPropBase(target, property);
        if (!base) return;
        var desc = Object.getOwnPropertyDescriptor(base, property);
        if (desc.get) {
          return desc.get.call(arguments.length < 3 ? target : receiver);
        }
        return desc.value;
      }, module.exports.__esModule = true, module.exports["default"] = module.exports;
    }
    return _get.apply(this, arguments);
  }
  module.exports = _get, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _get$1 = /*@__PURE__*/getDefaultExportFromCjs(get$1);

  var Rect$1 = /*#__PURE__*/function (_Base) {
    _inherits$1(Rect, _Base);
    var _super = _createSuper$1(Rect);
    function Rect() {
      var _this;
      _classCallCheck$1(this, Rect);
      _this = _super.apply(this, arguments);
      _this.type = 'rect';
      return _this;
    }
    _createClass$3(Rect, [{
      key: "update",
      value: function update(option) {
        _get$1(_getPrototypeOf$1(Rect.prototype), "update", this).call(this, option);
        var left = this.left,
          top = this.top,
          width = this.width,
          height = this.height;
        var x = [left, left + width];
        var y = [top + height, top];
        this.x = x;
        this.y = y;
        return this;
      }
    }]);
    return Rect;
  }(Base);

  /**
   *  expand Vec2
   */
  var vec2Direction = function vec2Direction(v1, v2) {
    return v1[0] * v2[1] - v2[0] * v1[1];
  };
  var vec2Zero = function vec2Zero(v) {
    return v[0] === 0 && v[1] === 0;
  };
  var vec2AngleTo = function vec2AngleTo(v1, v2, direction) {
    var angle$1 = angle(v1, v2);
    var angleLargeThanPI = vec2Direction(v1, v2) >= 0;
    if (direction) {
      if (angleLargeThanPI) {
        return Math.PI * 2 - angle$1;
      }
      return angle$1;
    }
    if (angleLargeThanPI) {
      return angle$1;
    }
    return Math.PI * 2 - angle$1;
  };

  var Polar = /*#__PURE__*/function (_Base) {
    _inherits$1(Polar, _Base);
    var _super = _createSuper$1(Polar);
    function Polar() {
      var _this;
      _classCallCheck$1(this, Polar);
      _this = _super.apply(this, arguments);
      _this.type = 'polar';
      _this.isPolar = true;
      return _this;
    }
    _createClass$3(Polar, [{
      key: "update",
      value: function update(option) {
        _get$1(_getPrototypeOf$1(Polar.prototype), "update", this).call(this, option);
        if (!this.option) {
          this.option = option;
        }
        var _this$option = this.option,
          _this$option$radius = _this$option.radius,
          radiusRatio = _this$option$radius === void 0 ? 1 : _this$option$radius,
          _this$option$innerRad = _this$option.innerRadius,
          innerRadiusRatio = _this$option$innerRad === void 0 ? 0 : _this$option$innerRad;
        var width = this.width,
          height = this.height,
          _this$startAngle = this.startAngle,
          startAngle = _this$startAngle === void 0 ? -Math.PI / 2 : _this$startAngle,
          _this$endAngle = this.endAngle,
          endAngle = _this$endAngle === void 0 ? Math.PI * 3 / 2 : _this$endAngle;
        // 半径取宽高的最小值
        var radius = radiusRatio * (Math.min(width, height) / 2);
        // 极坐标下 x 表示弧度， y 代表 半径
        var x = [startAngle, endAngle];
        var y = [innerRadiusRatio * radius, radius];
        this.x = x;
        this.y = y;
        this.startAngle = startAngle;
        this.endAngle = endAngle;
        this.radius = radius;
        this.innnerRadius = innerRadiusRatio * radius;
        return this;
      }
    }, {
      key: "isCyclic",
      value: function isCyclic() {
        var startAngle = this.startAngle,
          endAngle = this.endAngle;
        if (endAngle - startAngle < Math.PI * 2) {
          return false;
        }
        return true;
      }
    }, {
      key: "convertPoint",
      value: function convertPoint(point) {
        var center = this.center,
          transposed = this.transposed,
          x = this.x,
          y = this.y;
        var xDim = transposed ? 'y' : 'x';
        var yDim = transposed ? 'x' : 'y';
        var _x = _slicedToArray$1(x, 2),
          xStart = _x[0],
          xEnd = _x[1];
        var _y = _slicedToArray$1(y, 2),
          yStart = _y[0],
          yEnd = _y[1];
        var angle = xStart + (xEnd - xStart) * point[xDim];
        var radius = yStart + (yEnd - yStart) * point[yDim];
        return {
          x: center.x + Math.cos(angle) * radius,
          y: center.y + Math.sin(angle) * radius
        };
      }
    }, {
      key: "invertPoint",
      value: function invertPoint(point) {
        var center = this.center,
          transposed = this.transposed,
          x = this.x,
          y = this.y;
        var xDim = transposed ? 'y' : 'x';
        var yDim = transposed ? 'x' : 'y';
        var _x2 = _slicedToArray$1(x, 2),
          xStart = _x2[0],
          xEnd = _x2[1];
        var _y2 = _slicedToArray$1(y, 2),
          yStart = _y2[0],
          yEnd = _y2[1];
        var m = [1, 0, 0, 1, 0, 0];
        rotate$2(m, m, xStart);
        var startV = [1, 0];
        transformMat2d(startV, startV, m);
        startV = [startV[0], startV[1]];
        var pointV = [point.x - center.x, point.y - center.y];
        if (vec2Zero(pointV)) {
          return {
            x: 0,
            y: 0
          };
        }
        var theta = vec2AngleTo(startV, pointV, xEnd < xStart);
        if (Math.abs(theta - Math.PI * 2) < 0.001) {
          theta = 0;
        }
        var l = length$2(pointV);
        var percentX = theta / (xEnd - xStart);
        percentX = xEnd - xStart > 0 ? percentX : -percentX;
        var percentY = (l - yStart) / (yEnd - yStart);
        var rst = {};
        rst[xDim] = percentX;
        rst[yDim] = percentY;
        return rst;
      }
    }]);
    return Polar;
  }(Base);

  var coordMap = {
    rect: Rect$1,
    polar: Polar
  };
  var coordController = /*#__PURE__*/function () {
    function coordController() {
      _classCallCheck$1(this, coordController);
    }
    _createClass$3(coordController, [{
      key: "getOption",
      value: function getOption(cfg) {
        if (isString$1(cfg)) {
          return {
            type: coordMap[cfg] || Rect$1
          };
        }
        if (isFunction$2(cfg)) {
          return {
            type: cfg
          };
        }
        var _ref = cfg || {},
          type = _ref.type;
        return _objectSpread(_objectSpread({}, cfg), {}, {
          // 默认直角坐标系
          type: isFunction$2(type) ? type : coordMap[type] || Rect$1
        });
      }
    }, {
      key: "create",
      value: function create(cfg) {
        var layout = this.layout;
        var option = this.getOption(cfg);
        var type = option.type;
        var coord = new type(_objectSpread(_objectSpread({}, option), layout));
        this.coord = coord;
        return coord;
      }
    }, {
      key: "updateLayout",
      value: function updateLayout(style) {
        var coord = this.coord;
        var left = style.left,
          top = style.top,
          width = style.width,
          height = style.height,
          padding = style.padding;
        var _ref2 = padding || [0, 0, 0, 0],
          _ref3 = _slicedToArray$1(_ref2, 4),
          paddingTop = _ref3[0],
          paddingRight = _ref3[1],
          paddingBottom = _ref3[2],
          paddingLeft = _ref3[3];
        this.layout = {
          left: left + paddingLeft,
          top: top + paddingTop,
          width: width - paddingLeft - paddingRight,
          height: height - paddingTop - paddingBottom
        };
        if (coord) {
          coord.update(this.layout);
        }
      }
    }, {
      key: "useLayout",
      value: function useLayout(positionLayout) {
        var coord = this.coord;
        var position = positionLayout.position,
          boxWidth = positionLayout.width,
          boxHeight = positionLayout.height;
        var left = coord.left,
          top = coord.top,
          width = coord.width,
          height = coord.height;
        switch (position) {
          case 'left':
            left += boxWidth;
            width = Math.max(0, width - boxWidth);
            break;
          case 'right':
            width = Math.max(0, width - boxWidth);
            break;
          case 'top':
            top += boxHeight;
            height = Math.max(0, height - boxHeight);
            break;
          case 'bottom':
            height = Math.max(0, height - boxHeight);
            break;
        }
        coord.update({
          left: left,
          top: top,
          width: width,
          height: height
        });
      }
    }, {
      key: "update",
      value: function update() {}
    }, {
      key: "getCoord",
      value: function getCoord() {
        return this.coord;
      }
    }]);
    return coordController;
  }();

  var methodCache = {};
  /**
   * 获取计算 ticks 的方法
   * @param key 键值
   * @returns 计算 ticks 的方法
   */
  function getTickMethod(key) {
    return methodCache[key];
  }
  /**
   * 注册计算 ticks 的方法
   * @param key 键值
   * @param method 方法
   */
  function registerTickMethod(key, method) {
    methodCache[key] = method;
  }

  var Scale = /*#__PURE__*/function () {
    function Scale(cfg) {
      _classCallCheck$1(this, Scale);
      /**
       * 度量的类型
       */
      this.type = 'base';
      /**
       * 是否分类类型的度量
       */
      this.isCategory = false;
      /**
       * 是否线性度量，有linear, time 度量
       */
      this.isLinear = false;
      /**
       * 是否连续类型的度量，linear,time,log, pow, quantile, quantize 都支持
       */
      this.isContinuous = false;
      /**
       * 是否是常量的度量，传入和传出一致
       */
      this.isIdentity = false;
      this.values = [];
      this.range = [0, 1];
      this.ticks = [];
      this.__cfg__ = cfg;
      this.initCfg();
      this.init();
    }
    // 对于原始值的必要转换，如分类、时间字段需转换成数值，用transform/map命名可能更好
    _createClass$3(Scale, [{
      key: "translate",
      value: function translate(v) {
        return v;
      }
      /** 重新初始化 */
    }, {
      key: "change",
      value: function change(cfg) {
        // 覆盖配置项，而不替代
        mix(this.__cfg__, cfg);
        this.init();
      }
    }, {
      key: "clone",
      value: function clone() {
        return this.constructor(this.__cfg__);
      }
      /** 获取坐标轴需要的ticks */
    }, {
      key: "getTicks",
      value: function getTicks() {
        var _this = this;
        return map$2(this.ticks, function (tick, idx) {
          if (isObject$1(tick)) {
            // 仅当符合Tick类型时才有意义
            return tick;
          }
          return {
            text: _this.getText(tick, idx),
            tickValue: tick,
            value: _this.scale(tick) // scaled
          };
        });
      }
      /** 获取Tick的格式化结果 */
    }, {
      key: "getText",
      value: function getText(value, key) {
        var formatter = this.formatter;
        var res = formatter ? formatter(value, key) : value;
        if (isNil$1(res) || !isFunction$2(res.toString)) {
          return '';
        }
        return res.toString();
      }
      // 获取配置项中的值，当前 scale 上的值可能会被修改
    }, {
      key: "getConfig",
      value: function getConfig(key) {
        return this.__cfg__[key];
      }
      // scale初始化
    }, {
      key: "init",
      value: function init() {
        mix(this, this.__cfg__);
        this.setDomain();
        if (isEmpty(this.getConfig('ticks'))) {
          this.ticks = this.calculateTicks();
        }
      }
      // 子类上覆盖某些属性，不能直接在类上声明，否则会被覆盖
    }, {
      key: "initCfg",
      value: function initCfg() {}
    }, {
      key: "setDomain",
      value: function setDomain() {}
    }, {
      key: "calculateTicks",
      value: function calculateTicks() {
        var tickMethod = this.tickMethod;
        var ticks = [];
        if (isString$1(tickMethod)) {
          var method = getTickMethod(tickMethod);
          if (!method) {
            throw new Error('There is no method to to calculate ticks!');
          }
          ticks = method(this);
        } else if (isFunction$2(tickMethod)) {
          ticks = tickMethod(this);
        }
        return ticks;
      }
      // range 的最小值
    }, {
      key: "rangeMin",
      value: function rangeMin() {
        return this.range[0];
      }
      // range 的最大值
    }, {
      key: "rangeMax",
      value: function rangeMax() {
        return this.range[1];
      }
      /** 定义域转 0~1 */
    }, {
      key: "calcPercent",
      value: function calcPercent(value, min, max) {
        if (isNumber$1(value)) {
          return (value - min) / (max - min);
        }
        return NaN;
      }
      /** 0~1转定义域 */
    }, {
      key: "calcValue",
      value: function calcValue(percent, min, max) {
        return min + percent * (max - min);
      }
    }]);
    return Scale;
  }();

  /**
   * 分类度量
   * @class
   */
  var Category = /*#__PURE__*/function (_Base) {
    _inherits$1(Category, _Base);
    var _super = _createSuper$1(Category);
    function Category() {
      var _this;
      _classCallCheck$1(this, Category);
      _this = _super.apply(this, arguments);
      _this.type = 'cat';
      _this.isCategory = true;
      return _this;
    }
    _createClass$3(Category, [{
      key: "buildIndexMap",
      value: function buildIndexMap() {
        if (!this.translateIndexMap) {
          this.translateIndexMap = new Map();
          // 重新构建缓存
          for (var i = 0; i < this.values.length; i++) {
            this.translateIndexMap.set(this.values[i], i);
          }
        }
      }
    }, {
      key: "translate",
      value: function translate(value) {
        // 按需构建 map
        this.buildIndexMap();
        // 找得到
        var idx = this.translateIndexMap.get(value);
        if (idx === undefined) {
          idx = isNumber$1(value) ? value : NaN;
        }
        return idx;
      }
    }, {
      key: "scale",
      value: function scale(value) {
        var order = this.translate(value);
        // 分类数据允许 0.5 范围内调整
        // if (order < this.min - 0.5 || order > this.max + 0.5) {
        //   return NaN;
        // }
        var percent = this.calcPercent(order, this.min, this.max);
        return this.calcValue(percent, this.rangeMin(), this.rangeMax());
      }
    }, {
      key: "invert",
      value: function invert(scaledValue) {
        var domainRange = this.max - this.min;
        var percent = this.calcPercent(scaledValue, this.rangeMin(), this.rangeMax());
        var idx = Math.round(domainRange * percent) + this.min;
        if (idx < this.min || idx > this.max) {
          return NaN;
        }
        return this.values[idx];
      }
    }, {
      key: "getText",
      value: function getText(value) {
        var _get2;
        var v = value;
        // value为index
        if (isNumber$1(value) && !this.values.includes(value)) {
          v = this.values[v];
        }
        for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }
        return (_get2 = _get$1(_getPrototypeOf$1(Category.prototype), "getText", this)).call.apply(_get2, [this, v].concat(args));
      }
      // 复写属性
    }, {
      key: "initCfg",
      value: function initCfg() {
        this.tickMethod = 'cat';
      }
      // 设置 min, max
    }, {
      key: "setDomain",
      value: function setDomain() {
        // 用户有可能设置 min
        if (isNil$1(this.getConfig('min'))) {
          this.min = 0;
        }
        if (isNil$1(this.getConfig('max'))) {
          var size = this.values.length;
          this.max = size > 1 ? size - 1 : size;
        }
        // scale.init 的时候清除缓存
        if (this.translateIndexMap) {
          this.translateIndexMap = undefined;
        }
      }
    }]);
    return Category;
  }(Scale);

  var token = /d{1,4}|M{1,4}|YY(?:YY)?|S{1,3}|Do|ZZ|Z|([HhMsDm])\1?|[aA]|"[^"]*"|'[^']*'/g;
  var literal = /\[([^]*?)\]/gm;
  function shorten(arr, sLen) {
      var newArr = [];
      for (var i = 0, len = arr.length; i < len; i++) {
          newArr.push(arr[i].substr(0, sLen));
      }
      return newArr;
  }
  function assign(origObj) {
      var args = [];
      for (var _i = 1; _i < arguments.length; _i++) {
          args[_i - 1] = arguments[_i];
      }
      for (var _a = 0, args_1 = args; _a < args_1.length; _a++) {
          var obj = args_1[_a];
          for (var key in obj) {
              // @ts-ignore ex
              origObj[key] = obj[key];
          }
      }
      return origObj;
  }
  var dayNames = [
      "Sunday",
      "Monday",
      "Tuesday",
      "Wednesday",
      "Thursday",
      "Friday",
      "Saturday"
  ];
  var monthNames = [
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December"
  ];
  var monthNamesShort = shorten(monthNames, 3);
  var dayNamesShort = shorten(dayNames, 3);
  var defaultI18n = {
      dayNamesShort: dayNamesShort,
      dayNames: dayNames,
      monthNamesShort: monthNamesShort,
      monthNames: monthNames,
      amPm: ["am", "pm"],
      DoFn: function (dayOfMonth) {
          return (dayOfMonth +
              ["th", "st", "nd", "rd"][dayOfMonth % 10 > 3
                  ? 0
                  : ((dayOfMonth - (dayOfMonth % 10) !== 10 ? 1 : 0) * dayOfMonth) % 10]);
      }
  };
  var globalI18n = assign({}, defaultI18n);
  var pad = function (val, len) {
      if (len === void 0) { len = 2; }
      val = String(val);
      while (val.length < len) {
          val = "0" + val;
      }
      return val;
  };
  var formatFlags = {
      D: function (dateObj) { return String(dateObj.getDate()); },
      DD: function (dateObj) { return pad(dateObj.getDate()); },
      Do: function (dateObj, i18n) {
          return i18n.DoFn(dateObj.getDate());
      },
      d: function (dateObj) { return String(dateObj.getDay()); },
      dd: function (dateObj) { return pad(dateObj.getDay()); },
      ddd: function (dateObj, i18n) {
          return i18n.dayNamesShort[dateObj.getDay()];
      },
      dddd: function (dateObj, i18n) {
          return i18n.dayNames[dateObj.getDay()];
      },
      M: function (dateObj) { return String(dateObj.getMonth() + 1); },
      MM: function (dateObj) { return pad(dateObj.getMonth() + 1); },
      MMM: function (dateObj, i18n) {
          return i18n.monthNamesShort[dateObj.getMonth()];
      },
      MMMM: function (dateObj, i18n) {
          return i18n.monthNames[dateObj.getMonth()];
      },
      YY: function (dateObj) {
          return pad(String(dateObj.getFullYear()), 4).substr(2);
      },
      YYYY: function (dateObj) { return pad(dateObj.getFullYear(), 4); },
      h: function (dateObj) { return String(dateObj.getHours() % 12 || 12); },
      hh: function (dateObj) { return pad(dateObj.getHours() % 12 || 12); },
      H: function (dateObj) { return String(dateObj.getHours()); },
      HH: function (dateObj) { return pad(dateObj.getHours()); },
      m: function (dateObj) { return String(dateObj.getMinutes()); },
      mm: function (dateObj) { return pad(dateObj.getMinutes()); },
      s: function (dateObj) { return String(dateObj.getSeconds()); },
      ss: function (dateObj) { return pad(dateObj.getSeconds()); },
      S: function (dateObj) {
          return String(Math.round(dateObj.getMilliseconds() / 100));
      },
      SS: function (dateObj) {
          return pad(Math.round(dateObj.getMilliseconds() / 10), 2);
      },
      SSS: function (dateObj) { return pad(dateObj.getMilliseconds(), 3); },
      a: function (dateObj, i18n) {
          return dateObj.getHours() < 12 ? i18n.amPm[0] : i18n.amPm[1];
      },
      A: function (dateObj, i18n) {
          return dateObj.getHours() < 12
              ? i18n.amPm[0].toUpperCase()
              : i18n.amPm[1].toUpperCase();
      },
      ZZ: function (dateObj) {
          var offset = dateObj.getTimezoneOffset();
          return ((offset > 0 ? "-" : "+") +
              pad(Math.floor(Math.abs(offset) / 60) * 100 + (Math.abs(offset) % 60), 4));
      },
      Z: function (dateObj) {
          var offset = dateObj.getTimezoneOffset();
          return ((offset > 0 ? "-" : "+") +
              pad(Math.floor(Math.abs(offset) / 60), 2) +
              ":" +
              pad(Math.abs(offset) % 60, 2));
      }
  };
  // Some common format strings
  var globalMasks = {
      default: "ddd MMM DD YYYY HH:mm:ss",
      shortDate: "M/D/YY",
      mediumDate: "MMM D, YYYY",
      longDate: "MMMM D, YYYY",
      fullDate: "dddd, MMMM D, YYYY",
      isoDate: "YYYY-MM-DD",
      isoDateTime: "YYYY-MM-DDTHH:mm:ssZ",
      shortTime: "HH:mm",
      mediumTime: "HH:mm:ss",
      longTime: "HH:mm:ss.SSS"
  };
  /***
   * Format a date
   * @method format
   * @param {Date|number} dateObj
   * @param {string} mask Format of the date, i.e. 'mm-dd-yy' or 'shortDate'
   * @returns {string} Formatted date string
   */
  var format = function (dateObj, mask, i18n) {
      if (mask === void 0) { mask = globalMasks["default"]; }
      if (i18n === void 0) { i18n = {}; }
      if (typeof dateObj === "number") {
          dateObj = new Date(dateObj);
      }
      if (Object.prototype.toString.call(dateObj) !== "[object Date]" ||
          isNaN(dateObj.getTime())) {
          throw new Error("Invalid Date pass to format");
      }
      mask = globalMasks[mask] || mask;
      var literals = [];
      // Make literals inactive by replacing them with @@@
      mask = mask.replace(literal, function ($0, $1) {
          literals.push($1);
          return "@@@";
      });
      var combinedI18nSettings = assign(assign({}, globalI18n), i18n);
      // Apply formatting rules
      mask = mask.replace(token, function ($0) {
          return formatFlags[$0](dateObj, combinedI18nSettings);
      });
      // Inline literal values back into the formatted value
      return mask.replace(/@@@/g, function () { return literals.shift(); });
  };

  function timeFormat(time, mask) {
    return format(time, mask);
  }
  /**
   * 转换成时间戳
   * @param value 时间值
   */
  function toTimeStamp(value) {
    if (isString$1(value)) {
      if (value.indexOf('T') > 0) {
        value = new Date(value).getTime();
      } else {
        // new Date('2010/01/10') 和 new Date('2010-01-10') 的差别在于:
        // 如果仅有年月日时，前者是带有时区的: Fri Jan 10 2020 02:40:13 GMT+0800 (中国标准时间)
        // 后者会格式化成 Sun Jan 10 2010 08:00:00 GMT+0800 (中国标准时间)
        value = new Date(value.replace(/-/gi, '/')).getTime();
      }
    }
    if (isDate(value)) {
      value = value.getTime();
    }
    return value;
  }

  /**
   * 时间分类度量
   * @class
   */
  var TimeCat = /*#__PURE__*/function (_Category) {
    _inherits$1(TimeCat, _Category);
    var _super = _createSuper$1(TimeCat);
    function TimeCat() {
      var _this;
      _classCallCheck$1(this, TimeCat);
      _this = _super.apply(this, arguments);
      _this.type = 'timeCat';
      return _this;
    }
    /**
     * @override
     */
    _createClass$3(TimeCat, [{
      key: "translate",
      value: function translate(value) {
        value = toTimeStamp(value);
        var index = this.values.indexOf(value);
        if (index === -1) {
          if (isNumber$1(value) && value < this.values.length) {
            index = value;
          } else {
            index = NaN;
          }
        }
        return index;
      }
      /**
       * 由于时间类型数据需要转换一下，所以复写 getText
       * @override
       */
    }, {
      key: "getText",
      value: function getText(value, tickIndex) {
        var index = this.translate(value);
        if (index > -1) {
          var result = this.values[index];
          var formatter = this.formatter;
          result = formatter ? formatter(result, tickIndex) : timeFormat(result, this.mask);
          return result;
        }
        return value;
      }
    }, {
      key: "initCfg",
      value: function initCfg() {
        this.tickMethod = 'time-cat';
        this.mask = 'YYYY-MM-DD';
        this.tickCount = 7; // 一般时间数据会显示 7， 14， 30 天的数字
      }
    }, {
      key: "setDomain",
      value: function setDomain() {
        var values = this.values;
        // 针对时间分类类型，会将时间统一转换为时间戳
        each$1(values, function (v, i) {
          values[i] = toTimeStamp(v);
        });
        values.sort(function (v1, v2) {
          return v1 - v2;
        });
        _get$1(_getPrototypeOf$1(TimeCat.prototype), "setDomain", this).call(this);
      }
    }]);
    return TimeCat;
  }(Category);

  /**
   * 连续度量的基类
   * @class
   */
  var Continuous = /*#__PURE__*/function (_Base) {
    _inherits$1(Continuous, _Base);
    var _super = _createSuper$1(Continuous);
    function Continuous() {
      var _this;
      _classCallCheck$1(this, Continuous);
      _this = _super.apply(this, arguments);
      _this.isContinuous = true;
      return _this;
    }
    _createClass$3(Continuous, [{
      key: "scale",
      value: function scale(value) {
        if (isNil$1(value)) {
          return NaN;
        }
        var rangeMin = this.rangeMin();
        var rangeMax = this.rangeMax();
        var max = this.max;
        var min = this.min;
        if (max === min) {
          return rangeMin;
        }
        var percent = this.getScalePercent(value);
        return rangeMin + percent * (rangeMax - rangeMin);
      }
    }, {
      key: "init",
      value: function init() {
        _get$1(_getPrototypeOf$1(Continuous.prototype), "init", this).call(this);
        // init 完成后保证 min, max 包含 ticks 的范围
        var ticks = this.ticks;
        var firstTick = head(ticks);
        var lastTick = last(ticks);
        if (firstTick < this.min) {
          this.min = firstTick;
        }
        if (lastTick > this.max) {
          this.max = lastTick;
        }
        // strict-limit 方式
        if (!isNil$1(this.minLimit)) {
          this.min = firstTick;
        }
        if (!isNil$1(this.maxLimit)) {
          this.max = lastTick;
        }
      }
    }, {
      key: "setDomain",
      value: function setDomain() {
        var _getRange = getRange(this.values),
          min = _getRange.min,
          max = _getRange.max;
        if (isNil$1(this.min)) {
          this.min = min;
        }
        if (isNil$1(this.max)) {
          this.max = max;
        }
        if (this.min > this.max) {
          this.min = min;
          this.max = max;
        }
      }
    }, {
      key: "calculateTicks",
      value: function calculateTicks() {
        var _this2 = this;
        var ticks = _get$1(_getPrototypeOf$1(Continuous.prototype), "calculateTicks", this).call(this);
        if (!this.nice) {
          ticks = filter(ticks, function (tick) {
            return tick >= _this2.min && tick <= _this2.max;
          });
        }
        return ticks;
      }
      // 计算原始值值占的百分比
    }, {
      key: "getScalePercent",
      value: function getScalePercent(value) {
        var max = this.max;
        var min = this.min;
        return (value - min) / (max - min);
      }
    }, {
      key: "getInvertPercent",
      value: function getInvertPercent(value) {
        return (value - this.rangeMin()) / (this.rangeMax() - this.rangeMin());
      }
    }]);
    return Continuous;
  }(Scale);

  /**
   * 线性度量
   * @class
   */
  var Linear = /*#__PURE__*/function (_Continuous) {
    _inherits$1(Linear, _Continuous);
    var _super = _createSuper$1(Linear);
    function Linear() {
      var _this;
      _classCallCheck$1(this, Linear);
      _this = _super.apply(this, arguments);
      _this.type = 'linear';
      _this.isLinear = true;
      return _this;
    }
    _createClass$3(Linear, [{
      key: "invert",
      value: function invert(value) {
        var percent = this.getInvertPercent(value);
        return this.min + percent * (this.max - this.min);
      }
    }, {
      key: "initCfg",
      value: function initCfg() {
        this.tickMethod = 'wilkinson-extended';
        this.nice = false;
      }
    }]);
    return Linear;
  }(Continuous);

  // 求以a为次幂，结果为b的基数，如 x^^a = b;求x
  // 虽然数学上 b 不支持负数，但是这里需要支持 负数
  function calBase(a, b) {
    var e = Math.E;
    var value;
    if (b >= 0) {
      value = Math.pow(e, Math.log(b) / a); // 使用换底公式求底
    } else {
      value = Math.pow(e, Math.log(-b) / a) * -1; // 使用换底公式求底
    }

    return value;
  }
  function log(a, b) {
    if (a === 1) {
      return 1;
    }
    return Math.log(b) / Math.log(a);
  }
  function getLogPositiveMin(values, base, max) {
    if (isNil$1(max)) {
      max = Math.max.apply(null, values);
    }
    var positiveMin = max;
    each$1(values, function (value) {
      if (value > 0 && value < positiveMin) {
        positiveMin = value;
      }
    });
    if (positiveMin === max) {
      positiveMin = max / base;
    }
    if (positiveMin > 1) {
      positiveMin = 1;
    }
    return positiveMin;
  }

  /**
   * Log 度量，处理非均匀分布
   */
  var Log = /*#__PURE__*/function (_Continuous) {
    _inherits$1(Log, _Continuous);
    var _super = _createSuper$1(Log);
    function Log() {
      var _this;
      _classCallCheck$1(this, Log);
      _this = _super.apply(this, arguments);
      _this.type = 'log';
      return _this;
    }
    /**
     * @override
     */
    _createClass$3(Log, [{
      key: "invert",
      value: function invert(value) {
        var base = this.base;
        var max = log(base, this.max);
        var rangeMin = this.rangeMin();
        var range = this.rangeMax() - rangeMin;
        var min;
        var positiveMin = this.positiveMin;
        if (positiveMin) {
          if (value === 0) {
            return 0;
          }
          min = log(base, positiveMin / base);
          var appendPercent = 1 / (max - min) * range; // 0 到 positiveMin的占比
          if (value < appendPercent) {
            // 落到 0 - positiveMin 之间
            return value / appendPercent * positiveMin;
          }
        } else {
          min = log(base, this.min);
        }
        var percent = (value - rangeMin) / range;
        var tmp = percent * (max - min) + min;
        return Math.pow(base, tmp);
      }
    }, {
      key: "initCfg",
      value: function initCfg() {
        this.tickMethod = 'log';
        this.base = 10;
        this.tickCount = 6;
        this.nice = true;
      }
      // 设置
    }, {
      key: "setDomain",
      value: function setDomain() {
        _get$1(_getPrototypeOf$1(Log.prototype), "setDomain", this).call(this);
        var min = this.min;
        if (min < 0) {
          throw new Error('When you use log scale, the minimum value must be greater than zero!');
        }
        if (min === 0) {
          this.positiveMin = getLogPositiveMin(this.values, this.base, this.max);
        }
      }
      // 根据当前值获取占比
    }, {
      key: "getScalePercent",
      value: function getScalePercent(value) {
        var max = this.max;
        var min = this.min;
        if (max === min) {
          return 0;
        }
        // 如果值小于等于0，则按照0处理
        if (value <= 0) {
          return 0;
        }
        var base = this.base;
        var positiveMin = this.positiveMin;
        // 如果min == 0, 则根据比0大的最小值，计算比例关系。这个最小值作为坐标轴上的第二个tick，第一个是0但是不显示
        if (positiveMin) {
          min = positiveMin * 1 / base;
        }
        var percent;
        // 如果数值小于次小值，那么就计算 value / 次小值 占整体的比例
        if (value < positiveMin) {
          percent = value / positiveMin / (log(base, max) - log(base, min));
        } else {
          percent = (log(base, value) - log(base, min)) / (log(base, max) - log(base, min));
        }
        return percent;
      }
    }]);
    return Log;
  }(Continuous);

  /**
   * Pow 度量，处理非均匀分布
   */
  var Pow = /*#__PURE__*/function (_Continuous) {
    _inherits$1(Pow, _Continuous);
    var _super = _createSuper$1(Pow);
    function Pow() {
      var _this;
      _classCallCheck$1(this, Pow);
      _this = _super.apply(this, arguments);
      _this.type = 'pow';
      return _this;
    }
    /**
     * @override
     */
    _createClass$3(Pow, [{
      key: "invert",
      value: function invert(value) {
        var percent = this.getInvertPercent(value);
        var exponent = this.exponent;
        var max = calBase(exponent, this.max);
        var min = calBase(exponent, this.min);
        var tmp = percent * (max - min) + min;
        var factor = tmp >= 0 ? 1 : -1;
        return Math.pow(tmp, exponent) * factor;
      }
    }, {
      key: "initCfg",
      value: function initCfg() {
        this.tickMethod = 'pow';
        this.exponent = 2;
        this.tickCount = 5;
        this.nice = true;
      }
      // 获取度量计算时，value占的定义域百分比
    }, {
      key: "getScalePercent",
      value: function getScalePercent(value) {
        var max = this.max;
        var min = this.min;
        if (max === min) {
          return 0;
        }
        var exponent = this.exponent;
        var percent = (calBase(exponent, value) - calBase(exponent, min)) / (calBase(exponent, max) - calBase(exponent, min));
        return percent;
      }
    }]);
    return Pow;
  }(Continuous);

  /**
   * 时间度量
   * @class
   */
  var Time = /*#__PURE__*/function (_Linear) {
    _inherits$1(Time, _Linear);
    var _super = _createSuper$1(Time);
    function Time() {
      var _this;
      _classCallCheck$1(this, Time);
      _this = _super.apply(this, arguments);
      _this.type = 'time';
      return _this;
    }
    /**
     * @override
     */
    _createClass$3(Time, [{
      key: "getText",
      value: function getText(value, index) {
        var numberValue = this.translate(value);
        var formatter = this.formatter;
        return formatter ? formatter(numberValue, index) : timeFormat(numberValue, this.mask);
      }
      /**
       * @override
       */
    }, {
      key: "scale",
      value: function scale(value) {
        var v = value;
        if (isString$1(v) || isDate(v)) {
          v = this.translate(v);
        }
        return _get$1(_getPrototypeOf$1(Time.prototype), "scale", this).call(this, v);
      }
      /**
       * 将时间转换成数字
       * @override
       */
    }, {
      key: "translate",
      value: function translate(v) {
        return toTimeStamp(v);
      }
    }, {
      key: "initCfg",
      value: function initCfg() {
        this.tickMethod = 'time-pretty';
        this.mask = 'YYYY-MM-DD';
        this.tickCount = 7;
        this.nice = false;
      }
    }, {
      key: "setDomain",
      value: function setDomain() {
        var values = this.values;
        // 是否设置了 min, max，而不是直接取 this.min, this.max
        var minConfig = this.getConfig('min');
        var maxConfig = this.getConfig('max');
        // 如果设置了 min,max 则转换成时间戳
        if (!isNil$1(minConfig) || !isNumber$1(minConfig)) {
          this.min = this.translate(this.min);
        }
        if (!isNil$1(maxConfig) || !isNumber$1(maxConfig)) {
          this.max = this.translate(this.max);
        }
        // 没有设置 min, max 时
        if (values && values.length) {
          // 重新计算最大最小值
          var timeStamps = [];
          var min = Infinity; // 最小值
          var secondMin = min; // 次小值
          var max = 0;
          // 使用一个循环，计算min,max,secondMin
          each$1(values, function (v) {
            var timeStamp = toTimeStamp(v);
            if (isNaN(timeStamp)) {
              throw new TypeError("Invalid Time: ".concat(v, " in time scale!"));
            }
            if (min > timeStamp) {
              secondMin = min;
              min = timeStamp;
            } else if (secondMin > timeStamp) {
              secondMin = timeStamp;
            }
            if (max < timeStamp) {
              max = timeStamp;
            }
            timeStamps.push(timeStamp);
          });
          // 存在多个值时，设置最小间距
          if (values.length > 1) {
            this.minTickInterval = secondMin - min;
          }
          if (isNil$1(minConfig)) {
            this.min = min;
          }
          if (isNil$1(maxConfig)) {
            this.max = max;
          }
        }
      }
    }]);
    return Time;
  }(Linear);

  /**
   * 分段度量
   */
  var Quantize = /*#__PURE__*/function (_Continuous) {
    _inherits$1(Quantize, _Continuous);
    var _super = _createSuper$1(Quantize);
    function Quantize() {
      var _this;
      _classCallCheck$1(this, Quantize);
      _this = _super.apply(this, arguments);
      _this.type = 'quantize';
      return _this;
    }
    _createClass$3(Quantize, [{
      key: "invert",
      value: function invert(value) {
        var ticks = this.ticks;
        var length = ticks.length;
        var percent = this.getInvertPercent(value);
        var minIndex = Math.floor(percent * (length - 1));
        // 最后一个
        if (minIndex >= length - 1) {
          return last(ticks);
        }
        // 超出左边界， 则取第一个
        if (minIndex < 0) {
          return head(ticks);
        }
        var minTick = ticks[minIndex];
        var nextTick = ticks[minIndex + 1];
        // 比当前值小的 tick 在度量上的占比
        var minIndexPercent = minIndex / (length - 1);
        var maxIndexPercent = (minIndex + 1) / (length - 1);
        return minTick + (percent - minIndexPercent) / (maxIndexPercent - minIndexPercent) * (nextTick - minTick);
      }
    }, {
      key: "initCfg",
      value: function initCfg() {
        this.tickMethod = 'r-pretty';
        this.tickCount = 5;
        this.nice = true;
      }
    }, {
      key: "calculateTicks",
      value: function calculateTicks() {
        var ticks = _get$1(_getPrototypeOf$1(Quantize.prototype), "calculateTicks", this).call(this);
        if (!this.nice) {
          // 如果 nice = false ,补充 min, max
          if (last(ticks) !== this.max) {
            ticks.push(this.max);
          }
          if (head(ticks) !== this.min) {
            ticks.unshift(this.min);
          }
        }
        return ticks;
      }
      // 计算当前值在刻度中的占比
    }, {
      key: "getScalePercent",
      value: function getScalePercent(value) {
        var ticks = this.ticks;
        // 超出左边界
        if (value < head(ticks)) {
          return 0;
        }
        // 超出右边界
        if (value > last(ticks)) {
          return 1;
        }
        var minIndex = 0;
        each$1(ticks, function (tick, index) {
          if (value >= tick) {
            minIndex = index;
          } else {
            return false;
          }
        });
        return minIndex / (ticks.length - 1);
      }
    }]);
    return Quantize;
  }(Continuous);

  var Quantile = /*#__PURE__*/function (_Quantize) {
    _inherits$1(Quantile, _Quantize);
    var _super = _createSuper$1(Quantile);
    function Quantile() {
      var _this;
      _classCallCheck$1(this, Quantile);
      _this = _super.apply(this, arguments);
      _this.type = 'quantile';
      return _this;
    }
    _createClass$3(Quantile, [{
      key: "initCfg",
      value: function initCfg() {
        this.tickMethod = 'quantile';
        this.tickCount = 5;
        this.nice = true;
      }
    }]);
    return Quantile;
  }(Quantize);

  var map$3 = {};
  function getClass(key) {
    return map$3[key];
  }
  function registerClass(key, cls) {
    if (getClass(key)) {
      throw new Error("type '".concat(key, "' existed."));
    }
    map$3[key] = cls;
  }

  /**
   * identity scale原则上是定义域和值域一致，scale/invert方法也是一致的
   * 参考R的实现：https://github.com/r-lib/scales/blob/master/R/pal-identity.r
   * 参考d3的实现（做了下转型）：https://github.com/d3/d3-scale/blob/master/src/identity.js
   */
  var Identity = /*#__PURE__*/function (_Base) {
    _inherits$1(Identity, _Base);
    var _super = _createSuper$1(Identity);
    function Identity() {
      var _this;
      _classCallCheck$1(this, Identity);
      _this = _super.apply(this, arguments);
      _this.type = 'identity';
      _this.isIdentity = true;
      return _this;
    }
    _createClass$3(Identity, [{
      key: "calculateTicks",
      value: function calculateTicks() {
        return this.values;
      }
    }, {
      key: "scale",
      value: function scale(value) {
        // 如果传入的值不等于 identity 的值，则直接返回，用于一维图时的 dodge
        if (this.values[0] !== value && isNumber$1(value)) {
          return value;
        }
        return this.range[0];
      }
    }, {
      key: "invert",
      value: function invert(value) {
        var range = this.range;
        if (value < range[0] || value > range[1]) {
          return NaN;
        }
        return this.values[0];
      }
    }]);
    return Identity;
  }(Scale);

  // cat平均算法，保头保尾
  var CatTick = (function (cfg) {
    var values = cfg.values,
      tickCount = cfg.tickCount;
    if (!tickCount) {
      return values;
    }
    if (values.length <= 1) {
      return values;
    }
    // 获取间隔步长, 最小是1
    var step = Math.floor(values.length / (tickCount - 1)) || 1;
    var ticks = [];
    // 按间隔数取对应节点
    for (var index = 0; index < values.length; index = index + step) {
      ticks.push(values[index]);
    }
    var last = values[values.length - 1];
    // 如果最后一个tick不等于原数据的最后一个
    if (ticks[ticks.length - 1] !== last) {
      if (ticks.length >= tickCount) {
        // 如果当前的tick个数满足要求
        ticks[ticks.length - 1] = last;
      } else {
        // 不满足tickCount则直接加入最后一个
        ticks.push(last);
      }
    }
    return ticks;
  });

  // 认为是nice的刻度
  var SNAP_COUNT_ARRAY = [1, 1.2, 1.5, 2, 2.2, 2.4, 2.5, 3, 4, 5, 6, 7.5, 8, 10];
  var DEFAULT_COUNT = 5; // 默认刻度值
  var LinearTick = (function (cfg) {
    var _ref = cfg || {},
      tickCount = _ref.tickCount,
      tickInterval = _ref.tickInterval;
    var _ref2 = cfg || {},
      min = _ref2.min,
      max = _ref2.max;
    min = isNaN(min) ? 0 : min;
    max = isNaN(max) ? 0 : max;
    var count = tickCount && tickCount >= 2 ? tickCount : DEFAULT_COUNT;
    // 计算interval， 优先取tickInterval
    var interval = tickInterval || getBestInterval({
      tickCount: count,
      max: max,
      min: min
    });
    // 通过interval计算最小tick
    var minTick = Math.floor(min / interval) * interval;
    // 如果指定了tickInterval, count 需要根据指定的tickInterval来算计
    if (tickInterval) {
      var intervalCount = Math.abs(Math.ceil((max - minTick) / tickInterval)) + 1;
      // tickCount 作为最小 count 处理
      count = Math.max(count, intervalCount);
    }
    var ticks = [];
    var tickLength = 0;
    var fixedLength = getFixedLength(interval);
    while (tickLength < count) {
      ticks.push(toFixed(minTick + tickLength * interval, fixedLength));
      tickLength++;
    }
    return ticks;
  });
  var DECIMAL_LENGTH = 12;
  function getFactor(number) {
    // 取正数
    number = Math.abs(number);
    var factor = 1;
    if (number === 0) {
      return factor;
    }
    // 小于1,逐渐放大
    if (number < 1) {
      var count = 0;
      while (number < 1) {
        factor = factor / 10;
        number = number * 10;
        count++;
      }
      // 浮点数计算出现问题
      if (factor.toString().length > DECIMAL_LENGTH) {
        factor = parseFloat(factor.toFixed(count));
      }
      return factor;
    }
    // 大于10逐渐缩小
    while (number > 10) {
      factor = factor * 10;
      number = number / 10;
    }
    return factor;
  }
  // 获取最佳匹配刻度
  function getBestInterval(_ref3) {
    var tickCount = _ref3.tickCount,
      min = _ref3.min,
      max = _ref3.max;
    // 如果最大最小相等，则直接按1处理
    if (min === max) {
      return 1 * getFactor(max);
    }
    // 1.计算平均刻度间隔
    var avgInterval = (max - min) / (tickCount - 1);
    // 2.数据标准归一化 映射到[1-10]区间
    var factor = getFactor(avgInterval);
    var calInterval = avgInterval / factor;
    var calMax = max / factor;
    var calMin = min / factor;
    // 根据平均值推算最逼近刻度值
    var similarityIndex = 0;
    for (var index = 0; index < SNAP_COUNT_ARRAY.length; index++) {
      var item = SNAP_COUNT_ARRAY[index];
      if (calInterval <= item) {
        similarityIndex = index;
        break;
      }
    }
    var similarityInterval = getInterval(similarityIndex, tickCount, calMin, calMax);
    // 小数点位数还原到数据的位数, 因为similarityIndex有可能是小数，所以需要保留similarityIndex自己的小数位数
    var fixedLength = getFixedLength(similarityInterval) + getFixedLength(factor);
    return toFixed(similarityInterval * factor, fixedLength);
  }
  function getInterval(startIndex, tickCount, min, max) {
    var verify = false;
    var interval = SNAP_COUNT_ARRAY[startIndex];
    // 刻度值校验，如果不满足，循环下去
    for (var i = startIndex; i < SNAP_COUNT_ARRAY.length; i++) {
      if (intervalIsVerify({
        interval: SNAP_COUNT_ARRAY[i],
        tickCount: tickCount,
        max: max,
        min: min
      })) {
        // 有符合条件的interval
        interval = SNAP_COUNT_ARRAY[i];
        verify = true;
        break;
      }
    }
    // 如果不满足, 依次缩小10倍，再计算
    if (!verify) {
      return 10 * getInterval(0, tickCount, min / 10, max / 10);
    }
    return interval;
  }
  // 刻度是否满足展示需求
  function intervalIsVerify(_ref4) {
    var interval = _ref4.interval,
      tickCount = _ref4.tickCount,
      max = _ref4.max,
      min = _ref4.min;
    var minTick = Math.floor(min / interval) * interval;
    if (minTick + (tickCount - 1) * interval >= max) {
      return true;
    }
    return false;
  }
  // 计算小数点应该保留的位数
  function getFixedLength(num) {
    var str = num.toString();
    var index = str.indexOf('.');
    var indexOfExp = str.indexOf('e-');
    var length = indexOfExp >= 0 ? parseInt(str.substr(indexOfExp + 2), 10) : str.substr(index + 1).length;
    if (length > 20) {
      // 最多保留20位小数
      length = 20;
    }
    return length;
  }
  // @antv/util fixedbase不支持科学计数法的判断，需要提mr
  function toFixed(v, length) {
    return parseFloat(v.toFixed(length));
  }

  registerClass('cat', Category);
  registerClass('category', Category);
  registerClass('identity', Identity);
  registerClass('linear', Linear);
  registerClass('log', Log);
  registerClass('pow', Pow);
  registerClass('time', Time);
  registerClass('timeCat', TimeCat);
  registerClass('quantize', Quantize);
  registerClass('quantile', Quantile);
  // 覆盖0.3.x的 cat 方法
  registerTickMethod('cat', CatTick);
  registerTickMethod('time-cat', CatTick);
  // 覆盖linear 度量的tick算法
  registerTickMethod('wilkinson-extended', LinearTick);
  var ScaleController = /*#__PURE__*/function () {
    function ScaleController(data) {
      _classCallCheck$1(this, ScaleController);
      this.data = data;
      this.options = {};
      this.scales = {};
    }
    _createClass$3(ScaleController, [{
      key: "_getType",
      value: function _getType(option) {
        var type = option.type,
          values = option.values,
          field = option.field;
        if (type) {
          return type;
        }
        if (isNumber$1(field) || isNil$1(values[0]) && field) {
          return 'identity';
        }
        if (typeof values[0] === 'number') {
          return 'linear';
        }
        return 'cat';
      }
    }, {
      key: "_getOption",
      value: function _getOption(option) {
        var values = option.values,
          field = option.field,
          justifyContent = option.justifyContent;
        var type = this._getType(option);
        option.type = type;
        // identity
        if (type === 'identity') {
          option.field = field.toString();
          option.values = [field];
          return option;
        }
        // linear 类型
        if (type === 'linear') {
          // 设置默认nice
          if (typeof option.nice !== 'boolean') {
            option.nice = true;
          }
          // 重置最大最小
          var _getRange = getRange(values),
            min = _getRange.min,
            max = _getRange.max;
          if (isNil$1(option.min)) {
            option.min = min;
          }
          if (isNil$1(option.max)) {
            option.max = max;
          }
          option.values = values.sort(function (a, b) {
            return a - b;
          });
          return option;
        }
        // 分类类型和 timeCat 类型，调整 range
        if (type === 'cat' || type === 'timeCat') {
          if (option.range) {
            return option;
          }
          var count = values.length;
          var range = [0, 1];
          // 如果只有一项，显示在中间
          if (count === 1) {
            range = [0.5, 1];
          } else if (justifyContent) {
            // 居中
            var offset = 1 / count * 0.5;
            range = [offset, 1 - offset];
          } else {
            // 最后留 1 / count
            var _offset = 1 / count;
            range = [0, 1 - _offset];
          }
          option.range = range;
        }
        return option;
      }
    }, {
      key: "createScale",
      value: function createScale(option) {
        var type = option.type;
        if (isFunction$2(type)) {
          return new type(option);
        }
        var ScaleClass = getClass(type);
        return new ScaleClass(option);
      }
      // 更新或创建scale
    }, {
      key: "setScale",
      value: function setScale(field, option) {
        var options = this.options,
          scales = this.scales;
        options[field] = mix({}, options[field], option);
        // 如果scale有更新，scale 也需要重新创建
        if (scales[field]) {
          delete scales[field];
        }
      }
    }, {
      key: "create",
      value: function create(options) {
        this.update(options);
      }
    }, {
      key: "update",
      value: function update(options) {
        var _this = this;
        if (!options) return;
        each$1(options, function (option, field) {
          _this.setScale(field, option);
        });
        // 为了让外部感知到scale有变化
        this.scales = _objectSpread({}, this.scales);
      }
    }, {
      key: "changeData",
      value: function changeData(data) {
        this.data = data;
        this.scales = {};
      }
    }, {
      key: "getData",
      value: function getData() {
        return this.data;
      }
    }, {
      key: "getScale",
      value: function getScale(field) {
        var scales = this.scales,
          options = this.options,
          data = this.data;
        var scale = scales[field];
        if (scale) {
          return scale;
        }
        var option = options[field];
        if (!option) {
          return null;
        }
        var values = option.values ? option.values : data ? valuesOfKey(data, field) : [];
        var scaleOption = this._getOption(_objectSpread(_objectSpread({}, option), {}, {
          field: field,
          values: values
        }));
        var newScale = this.createScale(scaleOption);
        scales[field] = newScale;
        return newScale;
      }
    }, {
      key: "getScales",
      value: function getScales() {
        var _this2 = this;
        var options = this.options,
          scales = this.scales;
        each$1(options, function (option, field) {
          _this2.getScale(field);
        });
        return scales;
      }
    }, {
      key: "adjustStartZero",
      value: function adjustStartZero(scale) {
        var options = this.options;
        var field = scale.field,
          min = scale.min,
          max = scale.max;
        var option = options[field];
        // 如果有定义，则不处理
        if (option && option.min) {
          return;
        }
        if (min > 0) {
          scale.change({
            min: 0
          });
        } else if (max < 0) {
          scale.change({
            max: 0
          });
        }
      }
      // 饼图下的scale调整
    }, {
      key: "adjustPieScale",
      value: function adjustPieScale(scale) {
        var options = this.options;
        var field = scale.field;
        var option = options[field];
        if (option && !isNil$1(option.nice)) {
          return null;
        }
        scale.change({
          nice: false
        });
      }
      // 获取scale 在 0点对位置的值
    }, {
      key: "getZeroValue",
      value: function getZeroValue(scale) {
        var min = scale.min,
          max = scale.max;
        var value;
        if (min >= 0) {
          value = min;
        } else if (max <= 0) {
          value = max;
        } else {
          value = 0;
        }
        return scale.scale(value);
      }
    }]);
    return ScaleController;
  }();

  var axis = {
    labelOffset: '15px',
    line: {
      stroke: '#E8E8E8',
      lineWidth: '1px'
    },
    label: {
      fill: '#808080',
      fontSize: '20px'
    },
    grid: {
      stroke: '#E8E8E8',
      lineWidth: '1px',
      lineDash: ['4px']
    }
  };
  var guide = {
    line: {
      style: {
        stroke: '#a3a3a3',
        lineWidth: 1
      },
      offsetX: 0,
      offsetY: 0
    },
    text: {
      style: {
        fill: '#787878',
        // textAlign: 'center',
        textBaseline: 'middle'
      },
      offsetX: 0,
      offsetY: 0
    },
    rect: {
      style: {
        fill: '#fafafa'
      }
    },
    arc: {
      style: {
        stroke: '#a3a3a3'
      }
    },
    html: {
      offsetX: 0,
      offsetY: 0,
      alignX: 'center',
      alignY: 'middle'
    },
    tag: {
      offsetX: 0,
      offsetY: 0,
      side: 4,
      background: {
        padding: 5,
        radius: 2,
        fill: '#1890FF'
      },
      textStyle: {
        fontSize: 12,
        fill: '#fff',
        textAlign: 'center',
        textBaseline: 'middle'
      }
    },
    point: {
      offsetX: 0,
      offsetY: 0,
      style: {
        fill: '#fff',
        r: 3,
        lineWidth: 2,
        stroke: '#1890ff'
      }
    }
  };
  var chart = {
    padding: ['30px', '30px', '30px', '30px']
  };
  var Theme = {
    chart: chart,
    colors: ['#1890FF', '#2FC25B', '#FACC14', '#223273', '#8543E0', '#13C2C2', '#3436C7', '#F04864'],
    shapes: {
      line: ['line', 'dash', 'smooth'],
      point: ['circle', 'hollowCircle', 'rect'],
      area: ['area', 'smooth'],
      interval: ['rect', 'pyramid', 'funnel']
    },
    sizes: ['4px', '6px', '8px', '10px', '12px'],
    shape: {
      line: {
        default: {
          lineWidth: '4px',
          lineJoin: 'round',
          lineCap: 'round'
        },
        smooth: {
          smooth: true
        },
        dash: {
          lineDash: ['8px', '8px']
        }
      },
      point: {
        default: {
          size: '6px'
        },
        hollowCircle: {
          lineWidth: '2px'
        }
      },
      area: {
        default: {
          fillOpacity: 0.1
        }
      },
      interval: {
        default: {}
      }
    },
    axis: axis,
    guide: guide
  };

  // 统计图表
  var Chart = /*#__PURE__*/function (_Component) {
    _inherits$1(Chart, _Component);
    var _super = _createSuper$1(Chart);
    function Chart(props, context) {
      var _this;
      _classCallCheck$1(this, Chart);
      _this = _super.call(this, props);
      // 坐标系
      _this.componentsPosition = [];
      var theme = context.theme,
        px2hd = context.px2hd;
      // hack 处理，设置默认的主题样式
      // 目前没想到其他更合适的方式，只能先这样处理
      deepMix(theme, px2hd(Theme));
      var data = props.data;
      _this.scale = new ScaleController(data);
      _this.coord = new coordController();
      // state
      _this.state = {
        filters: {}
      };
      return _this;
    }
    _createClass$3(Chart, [{
      key: "getStyle",
      value: function getStyle(props) {
        var context = this.context,
          layout = this.layout;
        var theme = context.theme,
          px2hd = context.px2hd;
        var left = layout.left,
          top = layout.top,
          width = layout.width,
          height = layout.height;
        var customStyle = props.style;
        return px2hd(_objectSpread(_objectSpread({
          left: left,
          top: top,
          width: width,
          height: height
        }, theme.chart), customStyle));
      }
    }, {
      key: "willMount",
      value: function willMount() {
        var props = this.props,
          coord = this.coord,
          scale = this.scale;
        var scaleOptions = props.scale,
          coordOption = props.coord;
        var style = this.getStyle(props);
        coord.updateLayout(style);
        // 初始化 scale
        scale.create(scaleOptions);
        // 初始化 coord
        coord.create(coordOption);
      }
      // props 更新
    }, {
      key: "willReceiveProps",
      value: function willReceiveProps(nextProps) {
        var scale = this.scale,
          coord = this.coord,
          lastProps = this.props;
        var nextStyle = nextProps.style,
          nextData = nextProps.data,
          nextScale = nextProps.scale;
        var lastStyle = lastProps.style,
          lastData = lastProps.data,
          lastScale = lastProps.scale;
        // style 更新
        if (!equal(nextStyle, lastStyle)) {
          var style = this.getStyle(nextProps);
          coord.updateLayout(style);
        }
        if (nextData !== lastData) {
          scale.changeData(nextData);
        }
        // scale
        if (!equal(nextScale, lastScale)) {
          scale.update(nextScale);
        }
      }
      // 给需要显示的组件留空
    }, {
      key: "layoutCoord",
      value: function layoutCoord(layout) {
        this.coord.useLayout(layout);
      }
    }, {
      key: "resetCoordLayout",
      value: function resetCoordLayout() {
        var coord = this.coord,
          props = this.props;
        var style = this.getStyle(props);
        coord.updateLayout(style);
      }
    }, {
      key: "updateCoordLayout",
      value: function updateCoordLayout(layout) {
        var _this2 = this;
        if (isArray$1(layout)) {
          layout.forEach(function (item) {
            _this2.layoutCoord(item);
          });
          return;
        }
        this.layoutCoord(layout);
      }
    }, {
      key: "updateCoordFor",
      value: function updateCoordFor(component, layout) {
        var _this3 = this;
        if (!layout) return;
        var componentsPosition = this.componentsPosition;
        var componentPosition = {
          component: component,
          layout: layout
        };
        var existIndex = findIndex(componentsPosition, function (item) {
          return item.component === component;
        });
        // 说明是已经存在的组件
        if (existIndex > -1) {
          componentsPosition.splice(existIndex, 1, componentPosition);
          // 先重置，然后整体重新算一次
          this.resetCoordLayout();
          componentsPosition.forEach(function (componentPosition) {
            var layout = componentPosition.layout;
            _this3.updateCoordLayout(layout);
          });
          return;
        }
        // 是新组件，直接添加
        componentsPosition.push(componentPosition);
        this.updateCoordLayout(layout);
      }
    }, {
      key: "getGeometrys",
      value: function getGeometrys() {
        var children = this.children;
        var geometrys = [];
        // @ts-ignore
        Children.toArray(children).forEach(function (element) {
          if (!element) return false;
          var component = element.component;
          if (component && component.isGeometry) {
            geometrys.push(component);
          }
        });
        return geometrys;
      }
      /**
       * calculate dataset's position on canvas
       * @param  {Object} record the dataset
       * @return {Object} return the position
       */
    }, {
      key: "getPosition",
      value: function getPosition(record) {
        var coord = this.getCoord();
        var xScale = this.getXScales()[0];
        var xField = xScale.field;
        var yScales = this.getYScales();
        // default first
        var yScale = yScales[0];
        var yField = yScale.field;
        for (var i = 0, len = yScales.length; i < len; i++) {
          var scale = yScales[i];
          var field = scale.field;
          if (record[field]) {
            yScale = scale;
            yField = field;
            break;
          }
        }
        var x = xScale.scale(record[xField]);
        var y = yScale.scale(record[yField]);
        return coord.convertPoint({
          x: x,
          y: y
        });
      }
    }, {
      key: "getSnapRecords",
      value: function getSnapRecords(point, inCoordRange) {
        var geometrys = this.getGeometrys();
        if (!geometrys.length) return;
        // @ts-ignore
        return geometrys[0].getSnapRecords(point, inCoordRange);
      }
    }, {
      key: "getLegendItems",
      value: function getLegendItems(point) {
        var geometrys = this.getGeometrys();
        if (!geometrys.length) return;
        // @ts-ignore
        return geometrys[0].getLegendItems(point);
      }
    }, {
      key: "setScale",
      value: function setScale(field, option) {
        this.scale.setScale(field, option);
      }
    }, {
      key: "getScale",
      value: function getScale(field) {
        return this.scale.getScale(field);
      }
    }, {
      key: "getScales",
      value: function getScales() {
        return this.scale.getScales();
      }
    }, {
      key: "getXScales",
      value: function getXScales() {
        var geometrys = this.getGeometrys();
        return geometrys.map(function (component) {
          // @ts-ignore
          return component.getXScale();
        });
      }
    }, {
      key: "getYScales",
      value: function getYScales() {
        var geometrys = this.getGeometrys();
        return geometrys.map(function (component) {
          // @ts-ignore
          return component.getYScale();
        });
      }
    }, {
      key: "getLayout",
      value: function getLayout() {
        return this.coord.layout;
      }
    }, {
      key: "getCoord",
      value: function getCoord() {
        return this.coord.coord;
      }
    }, {
      key: "filter",
      value: function filter(field, condition) {
        var filters = this.state.filters;
        this.setState({
          filters: _objectSpread(_objectSpread({}, filters), {}, _defineProperty$1({}, field, condition))
        });
      }
    }, {
      key: "_getRenderData",
      value: function _getRenderData() {
        var props = this.props,
          state = this.state;
        var data = props.data;
        var filters = state.filters;
        if (!filters || !Object.keys(filters).length) {
          return data;
        }
        var filteredData = data;
        each$1(filters, function (condition, field) {
          if (!condition) return;
          filteredData = filteredData.filter(function (record) {
            return condition(record[field], record);
          });
        });
        return filteredData;
      }
    }, {
      key: "render",
      value: function render() {
        var _this4 = this;
        var props = this.props;
        var children = props.children,
          originData = props.data;
        if (!originData) return null;
        var data = this._getRenderData();
        var layout = this.getLayout();
        var coord = this.getCoord();
        return Children.map(children, function (child) {
          return Children.cloneElement(child, {
            chart: _this4,
            layout: layout,
            coord: coord,
            data: data
          });
        });
      }
    }]);
    return Chart;
  }(Component);

  var objectWithoutPropertiesLoose = createCommonjsModule(function (module) {
  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }
    return target;
  }
  module.exports = _objectWithoutPropertiesLoose, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var objectWithoutProperties = createCommonjsModule(function (module) {
  function _objectWithoutProperties(source, excluded) {
    if (source == null) return {};
    var target = objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
      var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
      for (i = 0; i < sourceSymbolKeys.length; i++) {
        key = sourceSymbolKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
        target[key] = source[key];
      }
    }
    return target;
  }
  module.exports = _objectWithoutProperties, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _objectWithoutProperties$1 = /*@__PURE__*/getDefaultExportFromCjs(objectWithoutProperties);

  function isEqual(origin1, origin2, fields) {
    if (origin1 === origin2) {
      return true;
    }
    for (var i = 0, len = fields.length; i < len; i++) {
      var field = fields[i];
      if (origin1[field] !== origin2[field]) {
        return false;
      }
    }
    return true;
  }
  var Selection = /*#__PURE__*/function (_Component) {
    _inherits$1(Selection, _Component);
    var _super = _createSuper$1(Selection);
    function Selection(props, context) {
      var _this;
      _classCallCheck$1(this, Selection);
      _this = _super.call(this, props, context);
      var selection = props.selection;
      if (!selection) return _possibleConstructorReturn$1(_this);
      var defaultSelected = selection.defaultSelected;
      _this.state.selected = defaultSelected;
      return _this;
    }
    _createClass$3(Selection, [{
      key: "didMount",
      value: function didMount() {
        var _this2 = this;
        var props = this.props,
          state = this.state,
          context = this.context;
        var selection = props.selection,
          chart = props.chart;
        if (!selection) return;
        // 默认为 click
        var _selection$triggerOn = selection.triggerOn,
          triggerOn = _selection$triggerOn === void 0 ? 'click' : _selection$triggerOn;
        context.gesture.on(triggerOn, function (ev) {
          var points = ev.points,
            x = ev.canvasX,
            y = ev.canvasY;
          var point = triggerOn === 'click' ? {
            x: x,
            y: y
          } : points[0];
          var records = _this2.getSnapRecords(point);
          var _selection$type = selection.type,
            type = _selection$type === void 0 ? 'single' : _selection$type,
            _selection$cancelable = selection.cancelable,
            cancelable = _selection$cancelable === void 0 ? true : _selection$cancelable;
          if (!records || !records.length) {
            if (cancelable) {
              _this2.setState({
                selected: null
              });
            }
            return;
          }
          var selected = state.selected;
          var origins = records.map(function (record) {
            return record.origin;
          });
          if (!selected || !selected.length) {
            _this2.setState({
              selected: origins
            });
          }
          if (type === 'single') {
            if (!cancelable) {
              _this2.setState({
                selected: origins
              });
              return;
            }
            var _newSelected = [];
            records.forEach(function (record) {
              if (!_this2.isSelected(record)) {
                _newSelected.push(record.origin);
              }
            });
            _this2.setState({
              selected: _newSelected
            });
            return;
          }
          // 多选
          var scales = chart.getScales();
          var fields = Object.keys(scales);
          var selectedMap = {};
          selected.forEach(function (item) {
            var key = fields.map(function (field) {
              return item[field];
            }).join('-');
            selectedMap[key] = item;
          });
          records.forEach(function (record) {
            var origin = record.origin;
            var key = fields.map(function (field) {
              return origin[field];
            }).join('-');
            selectedMap[key] = selectedMap[key] ? null : origin;
          });
          var newSelected = Object.keys(selectedMap).map(function (key) {
            return selectedMap[key];
          }).filter(Boolean);
          _this2.setState({
            selected: newSelected
          });
        });
      }
    }, {
      key: "willReceiveProps",
      value: function willReceiveProps(nextProps) {
        var nextSelection = nextProps.selection;
        var lastSelection = this.props.selection;
        if (!nextSelection || !lastSelection) {
          return;
        }
        var nextDefaultSelected = nextSelection.defaultSelected;
        var lastDefaultSelected = lastSelection.defaultSelected;
        if (!equal(nextDefaultSelected, lastDefaultSelected)) {
          this.state.selected = nextDefaultSelected;
        }
      }
    }, {
      key: "getSnapRecords",
      value: function getSnapRecords(_point) {
        return null;
      }
    }, {
      key: "isSelected",
      value: function isSelected(record) {
        var state = this.state,
          props = this.props;
        var selected = state.selected;
        if (!selected || !selected.length) {
          return false;
        }
        var chart = props.chart;
        var scales = chart.getScales();
        var fields = Object.keys(scales);
        for (var i = 0, len = selected.length; i < len; i++) {
          var item = selected[i];
          if (isEqual(record.origin, item, fields)) {
            return true;
          }
        }
        return false;
      }
    }, {
      key: "getSelectionStyle",
      value: function getSelectionStyle(record) {
        var state = this.state,
          props = this.props;
        var selected = state.selected;
        if (!selected || !selected.length) {
          return null;
        }
        var selection = props.selection;
        var selectedStyle = selection.selectedStyle,
          unSelectedStyle = selection.unSelectedStyle;
        var isSelected = this.isSelected(record);
        if (isSelected) {
          return isFunction$2(selectedStyle) ? selectedStyle(record) : selectedStyle;
        }
        return isFunction$2(unSelectedStyle) ? unSelectedStyle(record) : unSelectedStyle;
      }
    }]);
    return Selection;
  }(Component);

  var DEFAULT_Y = 0; // 默认的 y 的值
  // 偏移之后，间距
  var MARGIN_RATIO = 1 / 2;
  var DODGE_RATIO = 1 / 2;
  // 散点分开之后，距离边界的距离
  var GAP = 0.05;

  var Adjust = /*#__PURE__*/function () {
    function Adjust(cfg) {
      _classCallCheck$1(this, Adjust);
      var xField = cfg.xField,
        yField = cfg.yField,
        _cfg$adjustNames = cfg.adjustNames,
        adjustNames = _cfg$adjustNames === void 0 ? ['x', 'y'] : _cfg$adjustNames,
        dimValuesMap = cfg.dimValuesMap;
      this.adjustNames = adjustNames;
      this.xField = xField;
      this.yField = yField;
      this.dimValuesMap = dimValuesMap;
    }
    /**
     * 查看维度是否是 adjust 字段
     * @param dim
     */
    _createClass$3(Adjust, [{
      key: "isAdjust",
      value: function isAdjust(dim) {
        return this.adjustNames.indexOf(dim) >= 0;
      }
    }, {
      key: "getAdjustRange",
      value: function getAdjustRange(dim, dimValue, values) {
        var yField = this.yField;
        var index = values.indexOf(dimValue);
        var length = values.length;
        var pre;
        var next;
        // 没有 y 字段，但是需要根据 y 调整
        if (!yField && this.isAdjust('y')) {
          pre = 0;
          next = 1;
        } else if (length > 1) {
          // 如果以其开头，则取之，否则取他前面一个
          pre = values[index === 0 ? 0 : index - 1];
          // 如果以其结尾，则取之，否则取他后面一个
          next = values[index === length - 1 ? length - 1 : index + 1];
          if (index !== 0) {
            pre += (dimValue - pre) / 2;
          } else {
            pre -= (next - dimValue) / 2;
          }
          if (index !== length - 1) {
            next -= (next - dimValue) / 2;
          } else {
            next += (dimValue - values[length - 2]) / 2;
          }
        } else {
          pre = dimValue === 0 ? 0 : dimValue - 0.5;
          next = dimValue === 0 ? 1 : dimValue + 0.5;
        }
        return {
          pre: pre,
          next: next
        };
      }
    }, {
      key: "adjustData",
      value: function adjustData(groupedDataArray, mergedData) {
        var _this = this;
        // 所有调整维度的值数组
        var dimValuesMap = this.getDimValues(mergedData);
        // 按照每一个分组来进行调整
        each$1(groupedDataArray, function (dataArray, index) {
          // 遍历所有数据集合
          // 每个分组中，分别按照不同的 dim 进行调整
          each$1(dimValuesMap, function (values, dim) {
            // 根据不同的度量分别调整位置
            _this.adjustDim(dim, values, dataArray, index);
          });
        });
      }
      /**
       * 对数据进行分组adjustData
       * @param data 数据
       * @param dim 分组的字段
       * @return 分组结果
       */
    }, {
      key: "groupData",
      value: function groupData(data, dim) {
        // 补齐数据空数据为默认值
        each$1(data, function (record) {
          if (record[dim] === undefined) {
            record[dim] = DEFAULT_Y;
          }
        });
        // 按照 dim 维度分组
        return groupBy(data, dim);
      }
      /** @override */
    }, {
      key: "adjustDim",
      value: function adjustDim(dim, values, data, index) {}
      /**
       * 获取可调整度量对应的值
       * @param mergedData 数据
       * @return 值的映射
       */
    }, {
      key: "getDimValues",
      value: function getDimValues(mergedData) {
        var xField = this.xField,
          yField = this.yField;
        var dimValuesMap = mix({}, this.dimValuesMap);
        // 所有的维度
        var dims = [];
        if (xField && this.isAdjust('x')) {
          dims.push(xField);
        }
        if (yField && this.isAdjust('y')) {
          dims.push(yField);
        }
        dims.forEach(function (dim) {
          if (dimValuesMap && dimValuesMap[dim]) {
            return;
          }
          // 在每个维度上，所有的值
          dimValuesMap[dim] = valuesOfKey(mergedData, dim).sort(function (v1, v2) {
            return v1 - v2;
          });
        });
        // 只有一维的情况下，同时调整 y，赋予默认值
        if (!yField && this.isAdjust('y')) {
          var dim = 'y';
          dimValuesMap[dim] = [DEFAULT_Y, 1]; // 默认分布在 y 轴的 0 与 1 之间
        }

        return dimValuesMap;
      }
    }]);
    return Adjust;
  }();

  var Dodge = /*#__PURE__*/function (_Adjust) {
    _inherits$1(Dodge, _Adjust);
    var _super = _createSuper$1(Dodge);
    function Dodge(cfg) {
      var _this;
      _classCallCheck$1(this, Dodge);
      _this = _super.call(this, cfg);
      _this.cacheMap = {};
      _this.adjustDataArray = [];
      _this.mergeData = [];
      var _cfg$marginRatio = cfg.marginRatio,
        marginRatio = _cfg$marginRatio === void 0 ? MARGIN_RATIO : _cfg$marginRatio,
        _cfg$dodgeRatio = cfg.dodgeRatio,
        dodgeRatio = _cfg$dodgeRatio === void 0 ? DODGE_RATIO : _cfg$dodgeRatio,
        dodgeBy = cfg.dodgeBy,
        intervalPadding = cfg.intervalPadding,
        dodgePadding = cfg.dodgePadding,
        xDimensionLength = cfg.xDimensionLength,
        groupNum = cfg.groupNum,
        defaultSize = cfg.defaultSize,
        maxColumnWidth = cfg.maxColumnWidth,
        minColumnWidth = cfg.minColumnWidth,
        columnWidthRatio = cfg.columnWidthRatio,
        customOffset = cfg.customOffset;
      _this.marginRatio = marginRatio;
      _this.dodgeRatio = dodgeRatio;
      _this.dodgeBy = dodgeBy;
      _this.intervalPadding = intervalPadding;
      _this.dodgePadding = dodgePadding;
      _this.xDimensionLegenth = xDimensionLength;
      _this.groupNum = groupNum;
      _this.defaultSize = defaultSize;
      _this.maxColumnWidth = maxColumnWidth;
      _this.minColumnWidth = minColumnWidth;
      _this.columnWidthRatio = columnWidthRatio;
      _this.customOffset = customOffset;
      return _this;
    }
    _createClass$3(Dodge, [{
      key: "process",
      value: function process(groupDataArray) {
        var groupedDataArray = clone$4(groupDataArray);
        // 将数据数组展开一层
        var mergeData = flatten(groupedDataArray);
        var dodgeBy = this.dodgeBy;
        // 如果指定了分组 dim 的字段
        var adjustDataArray = dodgeBy ? group(mergeData, dodgeBy) : groupedDataArray;
        this.cacheMap = {};
        this.adjustDataArray = adjustDataArray;
        this.mergeData = mergeData;
        this.adjustData(adjustDataArray, mergeData);
        this.adjustDataArray = [];
        this.mergeData = [];
        return groupedDataArray;
      }
    }, {
      key: "adjustDim",
      value: function adjustDim(dim, values, data, frameIndex) {
        var _this2 = this;
        var customOffset = this.customOffset;
        var map = this.getDistribution(dim);
        var groupData = this.groupData(data, dim); // 根据值分组
        each$1(groupData, function (group, key) {
          var range;
          // xField 中只有一个值，不需要做 dodge
          if (values.length === 1) {
            range = {
              pre: values[0] - 1,
              next: values[0] + 1
            };
          } else {
            // 如果有多个，则需要获取调整的范围
            range = _this2.getAdjustRange(dim, parseFloat(key), values);
          }
          each$1(group, function (d) {
            var value = d[dim];
            var valueArr = map[value];
            var valIndex = valueArr.indexOf(frameIndex);
            if (!isNil$1(customOffset)) {
              var _range = range,
                pre = _range.pre,
                next = _range.next;
              d[dim] = isFunction$2(customOffset) ? customOffset(d, range) : (pre + next) / 2 + customOffset;
            } else {
              d[dim] = _this2.getDodgeOffset(range, valIndex, valueArr.length);
            }
          });
        });
        return [];
      }
    }, {
      key: "getDodgeOffset",
      value: function getDodgeOffset(range, idx, len) {
        var dodgeRatio = this.dodgeRatio,
          marginRatio = this.marginRatio,
          intervalPadding = this.intervalPadding,
          dodgePadding = this.dodgePadding;
        var pre = range.pre,
          next = range.next;
        var tickLength = next - pre;
        var position;
        // 分多种输入情况
        if (!isNil$1(intervalPadding) && isNil$1(dodgePadding) && intervalPadding >= 0) {
          // 仅配置intervalPadding
          var offset = this.getIntervalOnlyOffset(len, idx);
          position = pre + offset;
        } else if (!isNil$1(dodgePadding) && isNil$1(intervalPadding) && dodgePadding >= 0) {
          // 仅配置dodgePadding
          var _offset = this.getDodgeOnlyOffset(len, idx);
          position = pre + _offset;
        } else if (!isNil$1(intervalPadding) && !isNil$1(dodgePadding) && intervalPadding >= 0 && dodgePadding >= 0) {
          // 同时配置intervalPadding和dodgePadding
          var _offset2 = this.getIntervalAndDodgeOffset(len, idx);
          position = pre + _offset2;
        } else {
          // 默认情况
          var width = tickLength * dodgeRatio / len;
          var margin = marginRatio * width;
          var _offset3 = 1 / 2 * (tickLength - len * width - (len - 1) * margin) + ((idx + 1) * width + idx * margin) - 1 / 2 * width - 1 / 2 * tickLength;
          position = (pre + next) / 2 + _offset3;
        }
        return position;
      }
    }, {
      key: "getIntervalOnlyOffset",
      value: function getIntervalOnlyOffset(len, idx) {
        var defaultSize = this.defaultSize,
          intervalPadding = this.intervalPadding,
          xDimensionLegenth = this.xDimensionLegenth,
          groupNum = this.groupNum,
          dodgeRatio = this.dodgeRatio,
          maxColumnWidth = this.maxColumnWidth,
          minColumnWidth = this.minColumnWidth,
          columnWidthRatio = this.columnWidthRatio;
        var normalizedIntervalPadding = intervalPadding / xDimensionLegenth;
        var normalizedDodgePadding = (1 - (groupNum - 1) * normalizedIntervalPadding) / groupNum * dodgeRatio / (len - 1);
        var geomWidth = ((1 - normalizedIntervalPadding * (groupNum - 1)) / groupNum - normalizedDodgePadding * (len - 1)) / len;
        // 根据columnWidthRatio/defaultSize/maxColumnWidth/minColumnWidth调整宽度
        geomWidth = !isNil$1(columnWidthRatio) ? 1 / groupNum / len * columnWidthRatio : geomWidth;
        if (!isNil$1(maxColumnWidth)) {
          var normalizedMaxWidht = maxColumnWidth / xDimensionLegenth;
          geomWidth = Math.min(geomWidth, normalizedMaxWidht);
        }
        if (!isNil$1(minColumnWidth)) {
          var normalizedMinWidht = minColumnWidth / xDimensionLegenth;
          geomWidth = Math.max(geomWidth, normalizedMinWidht);
        }
        geomWidth = defaultSize ? defaultSize / xDimensionLegenth : geomWidth;
        // 调整组内间隔
        normalizedDodgePadding = ((1 - (groupNum - 1) * normalizedIntervalPadding) / groupNum - len * geomWidth) / (len - 1);
        var offset = ((1 / 2 + idx) * geomWidth + idx * normalizedDodgePadding + 1 / 2 * normalizedIntervalPadding) * groupNum - normalizedIntervalPadding / 2;
        return offset;
      }
    }, {
      key: "getDodgeOnlyOffset",
      value: function getDodgeOnlyOffset(len, idx) {
        var defaultSize = this.defaultSize,
          dodgePadding = this.dodgePadding,
          xDimensionLegenth = this.xDimensionLegenth,
          groupNum = this.groupNum,
          marginRatio = this.marginRatio,
          maxColumnWidth = this.maxColumnWidth,
          minColumnWidth = this.minColumnWidth,
          columnWidthRatio = this.columnWidthRatio;
        var normalizedDodgePadding = dodgePadding / xDimensionLegenth;
        var normalizedIntervalPadding = 1 * marginRatio / (groupNum - 1);
        var geomWidth = ((1 - normalizedIntervalPadding * (groupNum - 1)) / groupNum - normalizedDodgePadding * (len - 1)) / len;
        // 根据columnWidthRatio/defaultSize/maxColumnWidth/minColumnWidth调整宽度
        geomWidth = columnWidthRatio ? 1 / groupNum / len * columnWidthRatio : geomWidth;
        if (!isNil$1(maxColumnWidth)) {
          var normalizedMaxWidht = maxColumnWidth / xDimensionLegenth;
          geomWidth = Math.min(geomWidth, normalizedMaxWidht);
        }
        if (!isNil$1(minColumnWidth)) {
          var normalizedMinWidht = minColumnWidth / xDimensionLegenth;
          geomWidth = Math.max(geomWidth, normalizedMinWidht);
        }
        geomWidth = defaultSize ? defaultSize / xDimensionLegenth : geomWidth;
        // 调整组间距
        normalizedIntervalPadding = (1 - (geomWidth * len + normalizedDodgePadding * (len - 1)) * groupNum) / (groupNum - 1);
        var offset = ((1 / 2 + idx) * geomWidth + idx * normalizedDodgePadding + 1 / 2 * normalizedIntervalPadding) * groupNum - normalizedIntervalPadding / 2;
        return offset;
      }
    }, {
      key: "getIntervalAndDodgeOffset",
      value: function getIntervalAndDodgeOffset(len, idx) {
        var intervalPadding = this.intervalPadding,
          dodgePadding = this.dodgePadding,
          xDimensionLegenth = this.xDimensionLegenth,
          groupNum = this.groupNum;
        var normalizedIntervalPadding = intervalPadding / xDimensionLegenth;
        var normalizedDodgePadding = dodgePadding / xDimensionLegenth;
        var geomWidth = ((1 - normalizedIntervalPadding * (groupNum - 1)) / groupNum - normalizedDodgePadding * (len - 1)) / len;
        var offset = ((1 / 2 + idx) * geomWidth + idx * normalizedDodgePadding + 1 / 2 * normalizedIntervalPadding) * groupNum - normalizedIntervalPadding / 2;
        return offset;
      }
    }, {
      key: "getDistribution",
      value: function getDistribution(dim) {
        var groupedDataArray = this.adjustDataArray;
        var cacheMap = this.cacheMap;
        var map = cacheMap[dim];
        if (!map) {
          map = {};
          each$1(groupedDataArray, function (data, index) {
            var values = valuesOfKey(data, dim);
            if (!values.length) {
              values.push(0);
            }
            each$1(values, function (val) {
              if (!map[val]) {
                map[val] = [];
              }
              map[val].push(index);
            });
          });
          cacheMap[dim] = map;
        }
        return map;
      }
    }]);
    return Dodge;
  }(Adjust);

  function randomNumber(min, max) {
    return (max - min) * Math.random() + min;
  }
  var Jitter = /*#__PURE__*/function (_Adjust) {
    _inherits$1(Jitter, _Adjust);
    var _super = _createSuper$1(Jitter);
    function Jitter() {
      _classCallCheck$1(this, Jitter);
      return _super.apply(this, arguments);
    }
    _createClass$3(Jitter, [{
      key: "process",
      value: function process(groupDataArray) {
        var groupedDataArray = clone$4(groupDataArray);
        // 之前分组之后的数据，然后有合并回去（和分组前可以理解成是一样的）
        var mergeData = flatten(groupedDataArray);
        // 返回值
        this.adjustData(groupedDataArray, mergeData);
        return groupedDataArray;
      }
      /**
       * 当前数据分组（index）中，按照维度 dim 进行 jitter 调整
       * @param dim
       * @param values
       * @param dataArray
       */
    }, {
      key: "adjustDim",
      value: function adjustDim(dim, values, dataArray) {
        var _this = this;
        // 在每一个分组中，将数据再按照 dim 分组，用于散列
        var groupDataArray = this.groupData(dataArray, dim);
        return each$1(groupDataArray, function (data, dimValue) {
          return _this.adjustGroup(data, dim, parseFloat(dimValue), values);
        });
      }
      // 随机出来的字段值
    }, {
      key: "getAdjustOffset",
      value: function getAdjustOffset(range) {
        var pre = range.pre,
          next = range.next;
        // 随机的范围
        var margin = (next - pre) * GAP;
        return randomNumber(pre + margin, next - margin);
      }
      // adjust group data
    }, {
      key: "adjustGroup",
      value: function adjustGroup(group, dim, dimValue, values) {
        var _this2 = this;
        // 调整范围
        var range = this.getAdjustRange(dim, dimValue, values);
        each$1(group, function (data) {
          data[dim] = _this2.getAdjustOffset(range); // 获取调整的位置
        });

        return group;
      }
    }]);
    return Jitter;
  }(Adjust);

  var Cache = default_1;
  var Stack = /*#__PURE__*/function (_Adjust) {
    _inherits$1(Stack, _Adjust);
    var _super = _createSuper$1(Stack);
    function Stack(cfg) {
      var _this;
      _classCallCheck$1(this, Stack);
      _this = _super.call(this, cfg);
      var _cfg$adjustNames = cfg.adjustNames,
        adjustNames = _cfg$adjustNames === void 0 ? ['y'] : _cfg$adjustNames,
        _cfg$height = cfg.height,
        height = _cfg$height === void 0 ? NaN : _cfg$height,
        _cfg$size = cfg.size,
        size = _cfg$size === void 0 ? 10 : _cfg$size,
        _cfg$reverseOrder = cfg.reverseOrder,
        reverseOrder = _cfg$reverseOrder === void 0 ? false : _cfg$reverseOrder;
      _this.adjustNames = adjustNames;
      _this.height = height;
      _this.size = size;
      _this.reverseOrder = reverseOrder;
      return _this;
    }
    /**
     * 方法入参是经过数据分组、数据数字化之后的二维数组
     * @param groupDataArray 分组之后的数据
     */
    _createClass$3(Stack, [{
      key: "process",
      value: function process(groupDataArray) {
        var yField = this.yField,
          reverseOrder = this.reverseOrder;
        // 如果有指定 y 字段，那么按照 y 字段来 stack
        // 否则，按照高度均分
        var d = yField ? this.processStack(groupDataArray) : this.processOneDimStack(groupDataArray);
        return reverseOrder ? this.reverse(d) : d;
      }
    }, {
      key: "reverse",
      value: function reverse(groupedDataArray) {
        return groupedDataArray.slice(0).reverse();
      }
    }, {
      key: "processStack",
      value: function processStack(groupDataArray) {
        var xField = this.xField,
          yField = this.yField,
          reverseOrder = this.reverseOrder;
        // 层叠顺序翻转
        var groupedDataArray = reverseOrder ? this.reverse(groupDataArray) : groupDataArray;
        // 用来缓存，正数和负数的堆叠问题
        var positive = new Cache();
        var negative = new Cache();
        return groupedDataArray.map(function (dataArray) {
          return dataArray.map(function (data) {
            var x = get(data, xField, 0);
            var y = get(data, [yField]);
            var xKey = x.toString();
            // todo 是否应该取 _origin？因为 y 可能取到的值不正确，比如先 symmetric，再 stack！
            y = isArray$1(y) ? y[1] : y;
            if (!isNil$1(y)) {
              var cache = y >= 0 ? positive : negative;
              if (!cache.has(xKey)) {
                cache.set(xKey, 0);
              }
              var xValue = cache.get(xKey);
              var newXValue = y + xValue;
              // 存起来
              cache.set(xKey, newXValue);
              return _objectSpread(_objectSpread({}, data), {}, _defineProperty$1({}, yField, [xValue, newXValue]));
            }
            // 没有修改，则直接返回
            return data;
          });
        });
      }
    }, {
      key: "processOneDimStack",
      value: function processOneDimStack(groupDataArray) {
        var _this2 = this;
        var xField = this.xField,
          height = this.height,
          reverseOrder = this.reverseOrder;
        var yField = 'y';
        // 如果层叠的顺序翻转
        var groupedDataArray = reverseOrder ? this.reverse(groupDataArray) : groupDataArray;
        // 缓存累加数据
        var cache = new Cache();
        return groupedDataArray.map(function (dataArray) {
          return dataArray.map(function (data) {
            var size = _this2.size;
            var xValue = data[xField];
            // todo 没有看到这个 stack 计算原理
            var stackHeight = size * 2 / height;
            if (!cache.has(xValue)) {
              cache.set(xValue, stackHeight / 2); // 初始值大小
            }

            var stackValue = cache.get(xValue);
            // 增加一层 stackHeight
            cache.set(xValue, stackValue + stackHeight);
            return _objectSpread(_objectSpread({}, data), {}, _defineProperty$1({}, yField, stackValue));
          });
        });
      }
    }]);
    return Stack;
  }(Adjust);

  var arrayWithoutHoles = createCommonjsModule(function (module) {
  function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return arrayLikeToArray(arr);
  }
  module.exports = _arrayWithoutHoles, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var iterableToArray = createCommonjsModule(function (module) {
  function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
  }
  module.exports = _iterableToArray, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var nonIterableSpread = createCommonjsModule(function (module) {
  function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }
  module.exports = _nonIterableSpread, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var toConsumableArray = createCommonjsModule(function (module) {
  function _toConsumableArray(arr) {
    return arrayWithoutHoles(arr) || iterableToArray(arr) || unsupportedIterableToArray(arr) || nonIterableSpread();
  }
  module.exports = _toConsumableArray, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _toConsumableArray$1 = /*@__PURE__*/getDefaultExportFromCjs(toConsumableArray);

  var Symmetric = /*#__PURE__*/function (_Adjust) {
    _inherits$1(Symmetric, _Adjust);
    var _super = _createSuper$1(Symmetric);
    function Symmetric() {
      _classCallCheck$1(this, Symmetric);
      return _super.apply(this, arguments);
    }
    _createClass$3(Symmetric, [{
      key: "process",
      value: function process(groupDataArray) {
        var mergeData = flatten(groupDataArray);
        var xField = this.xField,
          yField = this.yField;
        // 每个 x 值对应的 最大值
        var cache = this.getXValuesMaxMap(mergeData);
        // 所有数据的最大的值
        var max = Math.max.apply(Math, _toConsumableArray$1(Object.keys(cache).map(function (key) {
          return cache[key];
        })));
        return map$2(groupDataArray, function (dataArray) {
          return map$2(dataArray, function (data) {
            var yValue = data[yField];
            var xValue = data[xField];
            // 数组处理逻辑
            if (isArray$1(yValue)) {
              var off = (max - cache[xValue]) / 2;
              return _objectSpread(_objectSpread({}, data), {}, _defineProperty$1({}, yField, map$2(yValue, function (y) {
                return off + y;
              })));
            }
            // 非数组处理逻辑
            var offset = (max - yValue) / 2;
            return _objectSpread(_objectSpread({}, data), {}, _defineProperty$1({}, yField, [offset, yValue + offset]));
          });
        });
      }
      // 获取每个 x 对应的最大的值
    }, {
      key: "getXValuesMaxMap",
      value: function getXValuesMaxMap(mergeData) {
        var _this = this;
        var xField = this.xField,
          yField = this.yField;
        // 根据 xField 的值进行分组
        var groupDataArray = groupBy(mergeData, function (data) {
          return data[xField];
        });
        // 获取每个 xField 值中的最大值
        return mapValues(groupDataArray, function (dataArray) {
          return _this.getDimMaxValue(dataArray, yField);
        });
      }
    }, {
      key: "getDimMaxValue",
      value: function getDimMaxValue(mergeData, dim) {
        // 所有的 value 值
        var dimValues = map$2(mergeData, function (data) {
          return get(data, dim, []);
        });
        // 将数组打平（dim value 有可能是数组，比如 stack 之后的）
        var flattenValues = flatten(dimValues);
        // 求出数组的最大值
        return Math.max.apply(Math, _toConsumableArray$1(flattenValues));
      }
    }]);
    return Symmetric;
  }(Adjust);

  function toTimeStamp$1(value) {
    if (isString$1(value)) {
      if (value.indexOf('T') > 0) {
        value = new Date(value).getTime();
      } else {
        // new Date('2010/01/10') 和 new Date('2010-01-10') 的差别在于:
        // 如果仅有年月日时，前者是带有时区的: Fri Jan 10 2020 02:40:13 GMT+0800 (中国标准时间)
        // 后者会格式化成 Sun Jan 10 2010 08:00:00 GMT+0800 (中国标准时间)
        value = new Date(value.replace(/-/gi, '/')).getTime();
      }
    }
    if (isDate(value)) {
      value = value.getTime();
    }
    return value;
  }
  function isInBBox(bbox, point) {
    // const { minX, maxX, minY, maxY } = bbox;
    var left = bbox.left,
      top = bbox.top,
      width = bbox.width,
      height = bbox.height;
    var minX = left;
    var maxX = left + width;
    var minY = top;
    var maxY = top + height;
    var x = point.x,
      y = point.y;
    return minX <= x && maxX >= x && minY <= y && maxY >= y;
  }

  var Base$1 = /*#__PURE__*/function () {
    function Base(options) {
      _classCallCheck$1(this, Base);
      mix(this, options);
      var scale = this.scale,
        field = this.field,
        data = this.data;
      if (!scale && data) {
        var values = valuesOfKey(data, field);
        this.scale = this.createScale({
          values: values,
          field: field
        });
      }
    }
    _createClass$3(Base, [{
      key: "createScale",
      value: function createScale(_scaleConfig) {
        return null;
      }
      // 数据映射方法
    }, {
      key: "_mapping",
      value: function _mapping(value) {
        return value;
      }
    }, {
      key: "update",
      value: function update(options) {
        mix(this, options);
      }
    }, {
      key: "setRange",
      value: function setRange(range) {
        this.range = range;
      }
      // 归一化，参数是原始数据，返回是归一化的数据
    }, {
      key: "normalize",
      value: function normalize(value) {
        var scale = this.scale;
        if (isArray$1(value)) {
          return value.map(function (v) {
            return scale.scale(v);
          });
        }
        return scale.scale(value);
      }
      // convert 参数是归一化的数据，返回定义域的值
    }, {
      key: "convert",
      value: function convert(value) {
        return value;
      }
      // 等于 normalize + convert， 参数是原始数据，返回是定义域的值
    }, {
      key: "mapping",
      value: function mapping(value) {
        var child = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
        var rst = isFunction$2(this.callback) ? this.callback(value, child) : null;
        if (!isNil$1(rst)) {
          return rst;
        }
        return this._mapping(value);
      }
    }]);
    return Base;
  }();

  function define$1 (constructor, factory, prototype) {
    constructor.prototype = factory.prototype = prototype;
    prototype.constructor = constructor;
  }
  function extend$2(parent, definition) {
    var prototype = Object.create(parent.prototype);
    for (var key in definition) {
      prototype[key] = definition[key];
    }
    return prototype;
  }

  function Color$1() {}
  var _darker = 0.7;
  var _brighter = 1 / _darker;
  var reI$1 = "\\s*([+-]?\\d+)\\s*",
    reN$1 = "\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*",
    reP$1 = "\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*",
    reHex$1 = /^#([0-9a-f]{3,8})$/,
    reRgbInteger$1 = new RegExp("^rgb\\(" + [reI$1, reI$1, reI$1] + "\\)$"),
    reRgbPercent$1 = new RegExp("^rgb\\(" + [reP$1, reP$1, reP$1] + "\\)$"),
    reRgbaInteger$1 = new RegExp("^rgba\\(" + [reI$1, reI$1, reI$1, reN$1] + "\\)$"),
    reRgbaPercent$1 = new RegExp("^rgba\\(" + [reP$1, reP$1, reP$1, reN$1] + "\\)$"),
    reHslPercent$1 = new RegExp("^hsl\\(" + [reN$1, reP$1, reP$1] + "\\)$"),
    reHslaPercent$1 = new RegExp("^hsla\\(" + [reN$1, reP$1, reP$1, reN$1] + "\\)$");
  var named$1 = {
    aliceblue: 0xf0f8ff,
    antiquewhite: 0xfaebd7,
    aqua: 0x00ffff,
    aquamarine: 0x7fffd4,
    azure: 0xf0ffff,
    beige: 0xf5f5dc,
    bisque: 0xffe4c4,
    black: 0x000000,
    blanchedalmond: 0xffebcd,
    blue: 0x0000ff,
    blueviolet: 0x8a2be2,
    brown: 0xa52a2a,
    burlywood: 0xdeb887,
    cadetblue: 0x5f9ea0,
    chartreuse: 0x7fff00,
    chocolate: 0xd2691e,
    coral: 0xff7f50,
    cornflowerblue: 0x6495ed,
    cornsilk: 0xfff8dc,
    crimson: 0xdc143c,
    cyan: 0x00ffff,
    darkblue: 0x00008b,
    darkcyan: 0x008b8b,
    darkgoldenrod: 0xb8860b,
    darkgray: 0xa9a9a9,
    darkgreen: 0x006400,
    darkgrey: 0xa9a9a9,
    darkkhaki: 0xbdb76b,
    darkmagenta: 0x8b008b,
    darkolivegreen: 0x556b2f,
    darkorange: 0xff8c00,
    darkorchid: 0x9932cc,
    darkred: 0x8b0000,
    darksalmon: 0xe9967a,
    darkseagreen: 0x8fbc8f,
    darkslateblue: 0x483d8b,
    darkslategray: 0x2f4f4f,
    darkslategrey: 0x2f4f4f,
    darkturquoise: 0x00ced1,
    darkviolet: 0x9400d3,
    deeppink: 0xff1493,
    deepskyblue: 0x00bfff,
    dimgray: 0x696969,
    dimgrey: 0x696969,
    dodgerblue: 0x1e90ff,
    firebrick: 0xb22222,
    floralwhite: 0xfffaf0,
    forestgreen: 0x228b22,
    fuchsia: 0xff00ff,
    gainsboro: 0xdcdcdc,
    ghostwhite: 0xf8f8ff,
    gold: 0xffd700,
    goldenrod: 0xdaa520,
    gray: 0x808080,
    green: 0x008000,
    greenyellow: 0xadff2f,
    grey: 0x808080,
    honeydew: 0xf0fff0,
    hotpink: 0xff69b4,
    indianred: 0xcd5c5c,
    indigo: 0x4b0082,
    ivory: 0xfffff0,
    khaki: 0xf0e68c,
    lavender: 0xe6e6fa,
    lavenderblush: 0xfff0f5,
    lawngreen: 0x7cfc00,
    lemonchiffon: 0xfffacd,
    lightblue: 0xadd8e6,
    lightcoral: 0xf08080,
    lightcyan: 0xe0ffff,
    lightgoldenrodyellow: 0xfafad2,
    lightgray: 0xd3d3d3,
    lightgreen: 0x90ee90,
    lightgrey: 0xd3d3d3,
    lightpink: 0xffb6c1,
    lightsalmon: 0xffa07a,
    lightseagreen: 0x20b2aa,
    lightskyblue: 0x87cefa,
    lightslategray: 0x778899,
    lightslategrey: 0x778899,
    lightsteelblue: 0xb0c4de,
    lightyellow: 0xffffe0,
    lime: 0x00ff00,
    limegreen: 0x32cd32,
    linen: 0xfaf0e6,
    magenta: 0xff00ff,
    maroon: 0x800000,
    mediumaquamarine: 0x66cdaa,
    mediumblue: 0x0000cd,
    mediumorchid: 0xba55d3,
    mediumpurple: 0x9370db,
    mediumseagreen: 0x3cb371,
    mediumslateblue: 0x7b68ee,
    mediumspringgreen: 0x00fa9a,
    mediumturquoise: 0x48d1cc,
    mediumvioletred: 0xc71585,
    midnightblue: 0x191970,
    mintcream: 0xf5fffa,
    mistyrose: 0xffe4e1,
    moccasin: 0xffe4b5,
    navajowhite: 0xffdead,
    navy: 0x000080,
    oldlace: 0xfdf5e6,
    olive: 0x808000,
    olivedrab: 0x6b8e23,
    orange: 0xffa500,
    orangered: 0xff4500,
    orchid: 0xda70d6,
    palegoldenrod: 0xeee8aa,
    palegreen: 0x98fb98,
    paleturquoise: 0xafeeee,
    palevioletred: 0xdb7093,
    papayawhip: 0xffefd5,
    peachpuff: 0xffdab9,
    peru: 0xcd853f,
    pink: 0xffc0cb,
    plum: 0xdda0dd,
    powderblue: 0xb0e0e6,
    purple: 0x800080,
    rebeccapurple: 0x663399,
    red: 0xff0000,
    rosybrown: 0xbc8f8f,
    royalblue: 0x4169e1,
    saddlebrown: 0x8b4513,
    salmon: 0xfa8072,
    sandybrown: 0xf4a460,
    seagreen: 0x2e8b57,
    seashell: 0xfff5ee,
    sienna: 0xa0522d,
    silver: 0xc0c0c0,
    skyblue: 0x87ceeb,
    slateblue: 0x6a5acd,
    slategray: 0x708090,
    slategrey: 0x708090,
    snow: 0xfffafa,
    springgreen: 0x00ff7f,
    steelblue: 0x4682b4,
    tan: 0xd2b48c,
    teal: 0x008080,
    thistle: 0xd8bfd8,
    tomato: 0xff6347,
    turquoise: 0x40e0d0,
    violet: 0xee82ee,
    wheat: 0xf5deb3,
    white: 0xffffff,
    whitesmoke: 0xf5f5f5,
    yellow: 0xffff00,
    yellowgreen: 0x9acd32
  };
  define$1(Color$1, color$1, {
    copy: function copy(channels) {
      return Object.assign(new this.constructor(), this, channels);
    },
    displayable: function displayable() {
      return this.rgb().displayable();
    },
    hex: color_formatHex$1,
    // Deprecated! Use color.formatHex.
    formatHex: color_formatHex$1,
    formatHsl: color_formatHsl$1,
    formatRgb: color_formatRgb$1,
    toString: color_formatRgb$1
  });
  function color_formatHex$1() {
    return this.rgb().formatHex();
  }
  function color_formatHsl$1() {
    return hslConvert$1(this).formatHsl();
  }
  function color_formatRgb$1() {
    return this.rgb().formatRgb();
  }
  function color$1(format) {
    var m, l;
    format = (format + "").trim().toLowerCase();
    return (m = reHex$1.exec(format)) ? (l = m[1].length, m = parseInt(m[1], 16), l === 6 ? rgbn$1(m) // #ff0000
    : l === 3 ? new Rgb$1(m >> 8 & 0xf | m >> 4 & 0xf0, m >> 4 & 0xf | m & 0xf0, (m & 0xf) << 4 | m & 0xf, 1) // #f00
    : l === 8 ? rgba$1(m >> 24 & 0xff, m >> 16 & 0xff, m >> 8 & 0xff, (m & 0xff) / 0xff) // #ff000000
    : l === 4 ? rgba$1(m >> 12 & 0xf | m >> 8 & 0xf0, m >> 8 & 0xf | m >> 4 & 0xf0, m >> 4 & 0xf | m & 0xf0, ((m & 0xf) << 4 | m & 0xf) / 0xff) // #f000
    : null // invalid hex
    ) : (m = reRgbInteger$1.exec(format)) ? new Rgb$1(m[1], m[2], m[3], 1) // rgb(255, 0, 0)
    : (m = reRgbPercent$1.exec(format)) ? new Rgb$1(m[1] * 255 / 100, m[2] * 255 / 100, m[3] * 255 / 100, 1) // rgb(100%, 0%, 0%)
    : (m = reRgbaInteger$1.exec(format)) ? rgba$1(m[1], m[2], m[3], m[4]) // rgba(255, 0, 0, 1)
    : (m = reRgbaPercent$1.exec(format)) ? rgba$1(m[1] * 255 / 100, m[2] * 255 / 100, m[3] * 255 / 100, m[4]) // rgb(100%, 0%, 0%, 1)
    : (m = reHslPercent$1.exec(format)) ? hsla$1(m[1], m[2] / 100, m[3] / 100, 1) // hsl(120, 50%, 50%)
    : (m = reHslaPercent$1.exec(format)) ? hsla$1(m[1], m[2] / 100, m[3] / 100, m[4]) // hsla(120, 50%, 50%, 1)
    : named$1.hasOwnProperty(format) ? rgbn$1(named$1[format]) // eslint-disable-line no-prototype-builtins
    : format === "transparent" ? new Rgb$1(NaN, NaN, NaN, 0) : null;
  }
  function rgbn$1(n) {
    return new Rgb$1(n >> 16 & 0xff, n >> 8 & 0xff, n & 0xff, 1);
  }
  function rgba$1(r, g, b, a) {
    if (a <= 0) r = g = b = NaN;
    return new Rgb$1(r, g, b, a);
  }
  function rgbConvert$1(o) {
    if (!(o instanceof Color$1)) o = color$1(o);
    if (!o) return new Rgb$1();
    o = o.rgb();
    return new Rgb$1(o.r, o.g, o.b, o.opacity);
  }
  function rgb$1(r, g, b, opacity) {
    return arguments.length === 1 ? rgbConvert$1(r) : new Rgb$1(r, g, b, opacity == null ? 1 : opacity);
  }
  function Rgb$1(r, g, b, opacity) {
    this.r = +r;
    this.g = +g;
    this.b = +b;
    this.opacity = +opacity;
  }
  define$1(Rgb$1, rgb$1, extend$2(Color$1, {
    brighter: function brighter(k) {
      k = k == null ? _brighter : Math.pow(_brighter, k);
      return new Rgb$1(this.r * k, this.g * k, this.b * k, this.opacity);
    },
    darker: function darker(k) {
      k = k == null ? _darker : Math.pow(_darker, k);
      return new Rgb$1(this.r * k, this.g * k, this.b * k, this.opacity);
    },
    rgb: function rgb() {
      return this;
    },
    displayable: function displayable() {
      return -0.5 <= this.r && this.r < 255.5 && -0.5 <= this.g && this.g < 255.5 && -0.5 <= this.b && this.b < 255.5 && 0 <= this.opacity && this.opacity <= 1;
    },
    hex: rgb_formatHex$1,
    // Deprecated! Use color.formatHex.
    formatHex: rgb_formatHex$1,
    formatRgb: rgb_formatRgb$1,
    toString: rgb_formatRgb$1
  }));
  function rgb_formatHex$1() {
    return "#" + hex$1(this.r) + hex$1(this.g) + hex$1(this.b);
  }
  function rgb_formatRgb$1() {
    var a = this.opacity;
    a = isNaN(a) ? 1 : Math.max(0, Math.min(1, a));
    return (a === 1 ? "rgb(" : "rgba(") + Math.max(0, Math.min(255, Math.round(this.r) || 0)) + ", " + Math.max(0, Math.min(255, Math.round(this.g) || 0)) + ", " + Math.max(0, Math.min(255, Math.round(this.b) || 0)) + (a === 1 ? ")" : ", " + a + ")");
  }
  function hex$1(value) {
    value = Math.max(0, Math.min(255, Math.round(value) || 0));
    return (value < 16 ? "0" : "") + value.toString(16);
  }
  function hsla$1(h, s, l, a) {
    if (a <= 0) h = s = l = NaN;else if (l <= 0 || l >= 1) h = s = NaN;else if (s <= 0) h = NaN;
    return new Hsl$1(h, s, l, a);
  }
  function hslConvert$1(o) {
    if (o instanceof Hsl$1) return new Hsl$1(o.h, o.s, o.l, o.opacity);
    if (!(o instanceof Color$1)) o = color$1(o);
    if (!o) return new Hsl$1();
    if (o instanceof Hsl$1) return o;
    o = o.rgb();
    var r = o.r / 255,
      g = o.g / 255,
      b = o.b / 255,
      min = Math.min(r, g, b),
      max = Math.max(r, g, b),
      h = NaN,
      s = max - min,
      l = (max + min) / 2;
    if (s) {
      if (r === max) h = (g - b) / s + (g < b) * 6;else if (g === max) h = (b - r) / s + 2;else h = (r - g) / s + 4;
      s /= l < 0.5 ? max + min : 2 - max - min;
      h *= 60;
    } else {
      s = l > 0 && l < 1 ? 0 : h;
    }
    return new Hsl$1(h, s, l, o.opacity);
  }
  function hsl$1(h, s, l, opacity) {
    return arguments.length === 1 ? hslConvert$1(h) : new Hsl$1(h, s, l, opacity == null ? 1 : opacity);
  }
  function Hsl$1(h, s, l, opacity) {
    this.h = +h;
    this.s = +s;
    this.l = +l;
    this.opacity = +opacity;
  }
  define$1(Hsl$1, hsl$1, extend$2(Color$1, {
    brighter: function brighter(k) {
      k = k == null ? _brighter : Math.pow(_brighter, k);
      return new Hsl$1(this.h, this.s, this.l * k, this.opacity);
    },
    darker: function darker(k) {
      k = k == null ? _darker : Math.pow(_darker, k);
      return new Hsl$1(this.h, this.s, this.l * k, this.opacity);
    },
    rgb: function rgb() {
      var h = this.h % 360 + (this.h < 0) * 360,
        s = isNaN(h) || isNaN(this.s) ? 0 : this.s,
        l = this.l,
        m2 = l + (l < 0.5 ? l : 1 - l) * s,
        m1 = 2 * l - m2;
      return new Rgb$1(hsl2rgb$1(h >= 240 ? h - 240 : h + 120, m1, m2), hsl2rgb$1(h, m1, m2), hsl2rgb$1(h < 120 ? h + 240 : h - 120, m1, m2), this.opacity);
    },
    displayable: function displayable() {
      return (0 <= this.s && this.s <= 1 || isNaN(this.s)) && 0 <= this.l && this.l <= 1 && 0 <= this.opacity && this.opacity <= 1;
    },
    formatHsl: function formatHsl() {
      var a = this.opacity;
      a = isNaN(a) ? 1 : Math.max(0, Math.min(1, a));
      return (a === 1 ? "hsl(" : "hsla(") + (this.h || 0) + ", " + (this.s || 0) * 100 + "%, " + (this.l || 0) * 100 + "%" + (a === 1 ? ")" : ", " + a + ")");
    }
  }));

  /* From FvD 13.37, CSS Color Module Level 3 */
  function hsl2rgb$1(h, m1, m2) {
    return (h < 60 ? m1 + (m2 - m1) * h / 60 : h < 180 ? m2 : h < 240 ? m1 + (m2 - m1) * (240 - h) / 60 : m1) * 255;
  }

  var constant = (function (x) {
    return function () {
      return x;
    };
  });

  function linear$1(a, d) {
    return function (t) {
      return a + t * d;
    };
  }
  function exponential(a, b, y) {
    return a = Math.pow(a, y), b = Math.pow(b, y) - a, y = 1 / y, function (t) {
      return Math.pow(a + t * b, y);
    };
  }
  function gamma(y) {
    return (y = +y) === 1 ? nogamma : function (a, b) {
      return b - a ? exponential(a, b, y) : constant(isNaN(a) ? b : a);
    };
  }
  function nogamma(a, b) {
    var d = b - a;
    return d ? linear$1(a, d) : constant(isNaN(a) ? b : a);
  }

  var rgb$2 = (function rgbGamma(y) {
    var color = gamma(y);
    function rgb(start, end) {
      var r = color((start = rgb$1(start)).r, (end = rgb$1(end)).r),
        g = color(start.g, end.g),
        b = color(start.b, end.b),
        opacity = nogamma(start.opacity, end.opacity);
      return function (t) {
        start.r = r(t);
        start.g = g(t);
        start.b = b(t);
        start.opacity = opacity(t);
        return start + '';
      };
    }
    rgb.gamma = rgbGamma;
    return rgb;
  })(1);

  function numberArray (a, b) {
    if (!b) b = [];
    var n = a ? Math.min(b.length, a.length) : 0,
      c = b.slice(),
      i;
    return function (t) {
      for (i = 0; i < n; ++i) {
        c[i] = a[i] * (1 - t) + b[i] * t;
      }
      return c;
    };
  }
  function isNumberArray(x) {
    return ArrayBuffer.isView(x) && !(x instanceof DataView);
  }

  function genericArray(a, b) {
    var nb = b ? b.length : 0,
      na = a ? Math.min(nb, a.length) : 0,
      x = new Array(na),
      c = new Array(nb),
      i;
    for (i = 0; i < na; ++i) {
      x[i] = interpolate$1(a[i], b[i]);
    }
    for (; i < nb; ++i) {
      c[i] = b[i];
    }
    return function (t) {
      for (i = 0; i < na; ++i) {
        c[i] = x[i](t);
      }
      return c;
    };
  }

  function date (a, b) {
    var d = new Date();
    return a = +a, b = +b, function (t) {
      return d.setTime(a * (1 - t) + b * t), d;
    };
  }

  function number (a, b) {
    return a = +a, b = +b, function (t) {
      return a * (1 - t) + b * t;
    };
  }

  function object (a, b) {
    var i = {},
      c = {},
      k;
    if (a === null || _typeof$1(a) !== "object") a = {};
    if (b === null || _typeof$1(b) !== "object") b = {};
    for (k in b) {
      if (k in a) {
        i[k] = interpolate$1(a[k], b[k]);
      } else {
        c[k] = b[k];
      }
    }
    return function (t) {
      for (k in i) {
        c[k] = i[k](t);
      }
      return c;
    };
  }

  var reA = /[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,
    reB = new RegExp(reA.source, "g");
  function zero(b) {
    return function () {
      return b;
    };
  }
  function one(b) {
    return function (t) {
      return b(t) + "";
    };
  }
  function string (a, b) {
    var bi = reA.lastIndex = reB.lastIndex = 0,
      // scan index for next number in b
      am,
      // current match in a
      bm,
      // current match in b
      bs,
      // string preceding current number in b, if any
      i = -1,
      // index in s
      s = [],
      // string constants and placeholders
      q = []; // number interpolators

    // Coerce inputs to strings.
    a = a + "", b = b + "";

    // Interpolate pairs of numbers in a & b.
    while ((am = reA.exec(a)) && (bm = reB.exec(b))) {
      if ((bs = bm.index) > bi) {
        // a string precedes the next number in b
        bs = b.slice(bi, bs);
        if (s[i]) s[i] += bs; // coalesce with previous string
        else s[++i] = bs;
      }
      if ((am = am[0]) === (bm = bm[0])) {
        // numbers in a & b match
        if (s[i]) s[i] += bm; // coalesce with previous string
        else s[++i] = bm;
      } else {
        // interpolate non-matching numbers
        s[++i] = null;
        q.push({
          i: i,
          x: number(am, bm)
        });
      }
      bi = reB.lastIndex;
    }

    // Add remains of b.
    if (bi < b.length) {
      bs = b.slice(bi);
      if (s[i]) s[i] += bs; // coalesce with previous string
      else s[++i] = bs;
    }

    // Special optimization for only a single match.
    // Otherwise, interpolate each of the numbers and rejoin the string.
    return s.length < 2 ? q[0] ? one(q[0].x) : zero(b) : (b = q.length, function (t) {
      for (var i = 0, o; i < b; ++i) {
        s[(o = q[i]).i] = o.x(t);
      }
      return s.join("");
    });
  }

  function interpolate$1 (a, b) {
    var t = _typeof$1(b),
      c;
    return b == null || t === 'boolean' ? constant(b) : (t === 'number' ? number : t === 'string' ? (c = color$1(b)) ? (b = c, rgb$2) : string : b instanceof color$1 ? rgb$2 : b instanceof Date ? date : isNumberArray(b) ? numberArray : Array.isArray(b) ? genericArray : typeof b.valueOf !== 'function' && typeof b.toString !== 'function' || isNaN(b) ? object : number)(a, b);
  }

  var Linear$1 = /*#__PURE__*/function (_Base) {
    _inherits$1(Linear$1, _Base);
    var _super = _createSuper$1(Linear$1);
    function Linear$1(options) {
      var _this;
      _classCallCheck$1(this, Linear$1);
      _this = _super.call(this, options);
      _this._updateInterpolate();
      return _this;
    }
    _createClass$3(Linear$1, [{
      key: "createScale",
      value: function createScale(scaleConfig) {
        return new Linear(scaleConfig);
      }
    }, {
      key: "_updateInterpolate",
      value: function _updateInterpolate() {
        var _this$range = _slicedToArray$1(this.range, 2),
          min = _this$range[0],
          max = _this$range[1];
        this.interpolate = interpolate$1(min, max);
      }
    }, {
      key: "update",
      value: function update(options) {
        _get$1(_getPrototypeOf$1(Linear$1.prototype), "update", this).call(this, options);
        this._updateInterpolate();
      }
    }, {
      key: "_mapping",
      value: function _mapping(value) {
        var scale = this.scale,
          interpolate = this.interpolate;
        if (isArray$1(value)) {
          return value.map(function (v) {
            return interpolate(scale.scale(v));
          });
        }
        return interpolate(scale.scale(value));
      }
    }, {
      key: "normalize",
      value: function normalize(value) {
        var scale = this.scale;
        if (isArray$1(value)) {
          return value.map(function (v) {
            return scale.scale(v);
          });
        }
        return scale.scale(value);
      }
    }, {
      key: "convert",
      value: function convert(value) {
        var range = this.range;
        var _range = _slicedToArray$1(range, 2),
          min = _range[0],
          max = _range[1];
        if (isArray$1(value)) {
          return value.map(function (v) {
            return min + (max - min) * v;
          });
        }
        return min + (max - min) * value;
      }
    }]);
    return Linear$1;
  }(Base$1);

  var Category$1 = /*#__PURE__*/function (_Base) {
    _inherits$1(Category$1, _Base);
    var _super = _createSuper$1(Category$1);
    function Category$1() {
      _classCallCheck$1(this, Category$1);
      return _super.apply(this, arguments);
    }
    _createClass$3(Category$1, [{
      key: "createScale",
      value: function createScale(scaleConfig) {
        return new Category(scaleConfig);
      }
    }, {
      key: "_mapping",
      value: function _mapping(value) {
        var scale = this.scale,
          range = this.range;
        if (scale.type === 'cat') {
          var _index = scale.translate(value);
          return range[_index % range.length];
        }
        var normalizeValue = scale.scale(value);
        var index = Math.round(normalizeValue * (range.length - 1));
        return range[index];
      }
    }]);
    return Category$1;
  }(Base$1);

  var Identity$1 = /*#__PURE__*/function (_Base) {
    _inherits$1(Identity$1, _Base);
    var _super = _createSuper$1(Identity$1);
    function Identity$1() {
      _classCallCheck$1(this, Identity$1);
      return _super.apply(this, arguments);
    }
    _createClass$3(Identity$1, [{
      key: "createScale",
      value: function createScale(scaleConfig) {
        return new Identity(scaleConfig);
      }
    }, {
      key: "_mapping",
      value: function _mapping() {
        var field = this.field,
          range = this.range;
        return field || range && range[0];
      }
    }]);
    return Identity$1;
  }(Base$1);

  var Attrs = /*#__PURE__*/Object.freeze({
    __proto__: null,
    Attr: Base$1,
    Linear: Linear$1,
    Category: Category$1,
    Identity: Identity$1
  });

  var Identity$2 = Identity$1,
    Linear$2 = Linear$1,
    Category$2 = Category$1;
  // 需要映射的属性名
  var ATTRS = ['x', 'y', 'color', 'size', 'shape'];
  // 分组处理的属性
  var GROUP_ATTRS = ['color', 'size', 'shape'];
  function cloneScale(scale, scaleConfig) {
    // @ts-ignore
    return new scale.constructor(_objectSpread(_objectSpread({}, scale.__cfg__), scaleConfig));
  }
  var AttrController = /*#__PURE__*/function () {
    function AttrController(scaleController, attrsRange) {
      _classCallCheck$1(this, AttrController);
      this.scaleController = scaleController;
      this.attrsRange = attrsRange;
      this.options = {};
      this.attrs = {};
    }
    _createClass$3(AttrController, [{
      key: "parseOption",
      value: function parseOption(option, attrName) {
        if (!option) {
          return {
            type: 'identity'
          };
        }
        if (isString$1(option)) {
          return {
            field: option,
            type: 'category'
          };
        }
        if (isNumber$1(option)) {
          if (attrName === 'size') {
            return {
              type: 'identity',
              field: option
            };
          }
        }
        if (isArray$1(option)) {
          return {
            field: option[0],
            range: option[1]
          };
        }
        return option;
      }
    }, {
      key: "getAttrOptions",
      value: function getAttrOptions(props, justifyContentCenter) {
        var _this = this;
        if (!props.x || !props.y) {
          throw new Error('x, y are required !');
        }
        var options = {};
        var ranges = this.attrsRange;
        ATTRS.forEach(function (attrName) {
          if (!props[attrName]) return;
          var option = _this.parseOption(props[attrName], attrName);
          if (!option.range) {
            option.range = ranges[attrName];
          }
          options[attrName] = option;
        });
        // @ts-ignore
        var x = options.x,
          y = options.y;
        x.justifyContent = justifyContentCenter;
        // x, y 都是固定Linear 映射
        x.type = Linear$2;
        y.type = Linear$2;
        return options;
      }
    }, {
      key: "getDefaultAttrValues",
      value: function getDefaultAttrValues() {
        var _this$attrsRange = this.attrsRange,
          color = _this$attrsRange.color,
          shape = _this$attrsRange.shape;
        return {
          color: color[0],
          shape: shape && shape[0]
        };
      }
    }, {
      key: "getGroupScales",
      value: function getGroupScales() {
        var attrs = this.attrs;
        var scales = [];
        each$1(GROUP_ATTRS, function (attrName) {
          var attr = attrs[attrName];
          if (!attr) {
            return;
          }
          var scale = attr.scale;
          if (scale && scale.isCategory && scales.indexOf(scale) === -1) {
            scales.push(scale);
          }
        });
        return scales;
      }
    }, {
      key: "createAttr",
      value: function createAttr(option) {
        var type = option.type,
          field = option.field,
          scaleConfig = option.scale;
        if (isNil$1(field) || type === Identity$2) {
          return new Identity$2(option);
        }
        var scale = this.scaleController.getScale(field);
        var attrOption = _objectSpread(_objectSpread({}, option), {}, {
          data: this.scaleController.getData(),
          // scaleConfig 只在属性映射中生效
          scale: scaleConfig ? cloneScale(scale, scaleConfig) : scale
        });
        // identity
        if (scale && scale.type === 'identity') {
          return new Identity$2(attrOption);
        }
        // Attr的默认类型和scale类型保持一致
        var AttrConstructor = scale.isLinear ? Linear$2 : Category$2;
        // custom Attr Constructor
        if (isFunction$2(type)) {
          AttrConstructor = type;
        }
        if (isString$1(type) && Attrs[upperFirst(type)]) {
          AttrConstructor = Attrs[upperFirst(type)];
        }
        return new AttrConstructor(attrOption);
      }
    }, {
      key: "create",
      value: function create(options) {
        this.update(options);
      }
    }, {
      key: "update",
      value: function update(nextOptions) {
        var scaleController = this.scaleController,
          lastOptions = this.options,
          lastAttrs = this.attrs;
        var nextAttrs = {};
        each$1(nextOptions, function (nextOption, attrName) {
          var lastOption = lastOptions[attrName];
          if (equal(nextOption, lastOption)) {
            nextAttrs[attrName] = lastAttrs[attrName];
          }
          var field = nextOption.field,
            justifyContent = nextOption.justifyContent;
          if (field) {
            scaleController.setScale(field, {
              justifyContent: justifyContent
            });
          }
        });
        this.options = nextOptions;
        this.attrs = nextAttrs;
      }
    }, {
      key: "getAttr",
      value: function getAttr(attrName) {
        var attrs = this.attrs,
          options = this.options;
        var attr = attrs[attrName];
        if (attr) {
          return attr;
        }
        var option = options[attrName];
        if (!option) {
          return null;
        }
        var newAttr = this.createAttr(option);
        attrs[attrName] = newAttr;
        return newAttr;
      }
    }, {
      key: "getAttrs",
      value: function getAttrs() {
        var _this2 = this;
        var options = this.options,
          attrs = this.attrs;
        each$1(options, function (option, attrName) {
          _this2.getAttr(attrName);
        });
        return attrs;
      }
    }, {
      key: "isGroupAttr",
      value: function isGroupAttr(attrName) {
        return GROUP_ATTRS.indexOf(attrName) !== -1;
      }
    }, {
      key: "getAttrsByLinear",
      value: function getAttrsByLinear() {
        var attrs = this.attrs;
        var attrNames = Object.keys(attrs);
        var linearAttrs = [];
        var nonlinearAttrs = [];
        attrNames.forEach(function (attrName) {
          if (attrName === 'x' || attrName === 'y') {
            linearAttrs.push(attrName);
            return;
          }
          var scale = attrs[attrName].scale;
          if (scale && scale.type === 'linear') {
            linearAttrs.push(attrName);
          } else {
            nonlinearAttrs.push(attrName);
          }
        });
        return {
          linearAttrs: linearAttrs,
          nonlinearAttrs: nonlinearAttrs
        };
      }
    }]);
    return AttrController;
  }();

  var _excluded$c = ["field"];
  var AdjustMap = {
    Stack: Stack,
    Dodge: Dodge,
    Jitter: Jitter,
    Symmetric: Symmetric
  };
  // 保留原始数据的字段
  var FIELD_ORIGIN = 'origin';
  var Geometry = /*#__PURE__*/function (_Selection) {
    _inherits$1(Geometry, _Selection);
    var _super = _createSuper$1(Geometry);
    function Geometry(props, context) {
      var _this;
      _classCallCheck$1(this, Geometry);
      _this = _super.call(this, props, context);
      _this.isGeometry = true;
      // x 轴居中
      _this.justifyContent = false;
      // y 轴是否从0开始
      _this.startOnZero = false;
      // 是否连接空值
      _this.connectNulls = false;
      // 是否需要排序
      _this.sortable = false;
      mix(_assertThisInitialized$3(_this), _this.getDefaultCfg());
      var chart = props.chart,
        coord = props.coord;
      var attrsRange = _this._getThemeAttrsRange();
      _this.attrController = new AttrController(chart.scale, attrsRange);
      var _assertThisInitialize = _assertThisInitialized$3(_this),
        attrController = _assertThisInitialize.attrController,
        justifyContent = _assertThisInitialize.justifyContent;
      var attrOptions = attrController.getAttrOptions(props, !coord.isCyclic() || justifyContent);
      attrController.create(attrOptions);
      return _this;
    }
    _createClass$3(Geometry, [{
      key: "getDefaultCfg",
      value: function getDefaultCfg() {
        return {};
      }
    }, {
      key: "willReceiveProps",
      value: function willReceiveProps(nextProps) {
        var lastProps = this.props,
          attrController = this.attrController,
          justifyContent = this.justifyContent;
        var nextData = nextProps.data,
          nextAdjust = nextProps.adjust,
          coord = nextProps.coord,
          selection = nextProps.selection;
        var lastData = lastProps.data,
          lastAdjust = lastProps.adjust,
          lastSelection = lastProps.selection;
        var justifyContentCenter = !coord.isCyclic() || justifyContent;
        var lastAttrOptions = attrController.getAttrOptions(lastProps, justifyContentCenter);
        attrController.attrsRange = this._getThemeAttrsRange();
        var nextAttrOptions = attrController.getAttrOptions(nextProps, justifyContentCenter);
        if (!equal(nextAttrOptions, lastAttrOptions)) {
          attrController.update(nextAttrOptions);
          this.records = null;
        }
        // 重新处理数据
        if (nextData !== lastData) {
          this.records = null;
        }
        // 重新处理数据
        if (nextAdjust !== lastAdjust) {
          this.records = null;
        }
        // selection 发生变化
        if (!equal(selection, lastSelection)) {
          _get$1(_getPrototypeOf$1(Geometry.prototype), "willReceiveProps", this).call(this, nextProps);
        }
      }
    }, {
      key: "willMount",
      value: function willMount() {
        this._createAttrs();
        if (!this.records) {
          this._processData();
        }
      }
    }, {
      key: "willUpdate",
      value: function willUpdate() {
        this._createAttrs();
        if (!this.records) {
          this._processData();
        }
      }
    }, {
      key: "didMount",
      value: function didMount() {
        _get$1(_getPrototypeOf$1(Geometry.prototype), "didMount", this).call(this);
        this._initEvent();
      }
    }, {
      key: "_createAttrs",
      value: function _createAttrs() {
        var attrController = this.attrController;
        attrController.attrs = {};
        this.attrs = attrController.getAttrs();
      }
    }, {
      key: "_getThemeAttrsRange",
      value: function _getThemeAttrsRange() {
        var context = this.context,
          props = this.props,
          geomType = this.geomType;
        var coord = props.coord;
        var theme = context.theme;
        var colors = theme.colors,
          sizes = theme.sizes,
          shapes = theme.shapes;
        return {
          x: coord.x,
          y: coord.y,
          color: colors,
          size: sizes,
          shape: shapes[geomType]
        };
      }
    }, {
      key: "_adjustScales",
      value: function _adjustScales() {
        var attrs = this.attrs,
          props = this.props,
          defaultStartOnZero = this.startOnZero;
        var chart = props.chart,
          _props$startOnZero = props.startOnZero,
          startOnZero = _props$startOnZero === void 0 ? defaultStartOnZero : _props$startOnZero,
          coord = props.coord,
          adjust = props.adjust;
        var isPolar = coord.isPolar,
          transposed = coord.transposed;
        var y = attrs.y;
        var yField = y.field;
        // 如果从 0 开始，只调整 y 轴 scale
        if (startOnZero) {
          var _y = attrs.y;
          chart.scale.adjustStartZero(_y.scale);
        }
        // 饼图的scale调整，关闭nice
        if (isPolar && transposed && (adjust === 'stack' || (adjust === null || adjust === void 0 ? void 0 : adjust.type) === 'stack')) {
          var _y2 = attrs.y;
          chart.scale.adjustPieScale(_y2.scale);
        }
        if (adjust === 'stack' || (adjust === null || adjust === void 0 ? void 0 : adjust.type) === 'stack') {
          this._updateStackRange(yField, y.scale, this.dataArray);
        }
      }
    }, {
      key: "_groupData",
      value: function _groupData(data) {
        var attrController = this.attrController;
        var groupScales = attrController.getGroupScales();
        if (!groupScales.length) {
          return [{
            children: data
          }];
        }
        var names = [];
        groupScales.forEach(function (scale) {
          var field = scale.field;
          names.push(field);
        });
        var groups = groupToMap(data, names);
        var records = [];
        for (var key in groups) {
          records.push({
            key: key.replace(/^_/, ''),
            children: groups[key]
          });
        }
        return records;
      }
    }, {
      key: "_saveOrigin",
      value: function _saveOrigin(originData) {
        var len = originData.length;
        var data = new Array(len);
        for (var i = 0; i < len; i++) {
          var record = originData[i];
          data[i] = _objectSpread(_objectSpread({}, record), {}, _defineProperty$1({}, FIELD_ORIGIN, record));
        }
        return data;
      }
    }, {
      key: "_numberic",
      value: function _numberic(data) {
        var attrs = this.attrs;
        var scales = [attrs.x.scale, attrs.y.scale];
        for (var j = 0, len = data.length; j < len; j++) {
          var obj = data[j];
          var count = scales.length;
          for (var i = 0; i < count; i++) {
            var scale = scales[i];
            if (scale.isCategory) {
              var field = scale.field;
              obj[field] = scale.translate(obj[field]);
            }
          }
        }
      }
    }, {
      key: "_adjustData",
      value: function _adjustData(records) {
        var attrs = this.attrs,
          props = this.props;
        var adjust = props.adjust;
        // groupedArray 是二维数组
        var groupedArray = records.map(function (record) {
          return record.children;
        });
        if (!adjust) {
          return groupedArray;
        }
        var adjustCfg = typeof adjust === 'string' ? {
          type: adjust
        } : adjust;
        var adjustType = upperFirst(adjustCfg.type);
        var AdjustConstructor = AdjustMap[adjustType];
        if (!AdjustConstructor) {
          throw new Error('not support such adjust : ' + adjust);
        }
        if (adjustType === 'Dodge') {
          for (var i = 0, len = groupedArray.length; i < len; i++) {
            // 如果是dodge, 需要处理数字再处理
            this._numberic(groupedArray[i]);
          }
          // @ts-ignore
          adjustCfg.adjustNames = ['x'];
        }
        var x = attrs.x,
          y = attrs.y;
        // @ts-ignore
        adjustCfg.xField = x.field;
        // @ts-ignore
        adjustCfg.yField = y.field;
        var adjustInstance = new AdjustConstructor(adjustCfg);
        var adjustData = adjustInstance.process(groupedArray);
        this.adjust = {
          type: adjustCfg.type,
          adjust: adjustInstance
        };
        // process 返回的是新数组，所以要修改 records
        records.forEach(function (record, index) {
          record.children = adjustData[index];
        });
        return adjustData;
      }
    }, {
      key: "_updateStackRange",
      value: function _updateStackRange(field, scale, dataArray) {
        var flattenArray = flatten(dataArray);
        var min = Infinity;
        var max = -Infinity;
        for (var i = 0, len = flattenArray.length; i < len; i++) {
          var obj = flattenArray[i];
          var tmpMin = Math.min.apply(null, obj[field]);
          var tmpMax = Math.max.apply(null, obj[field]);
          if (tmpMin < min) {
            min = tmpMin;
          }
          if (tmpMax > max) {
            max = tmpMax;
          }
        }
        if (min !== scale.min || max !== scale.max) {
          scale.change({
            min: min,
            max: max
          });
        }
      }
    }, {
      key: "_processData",
      value: function _processData() {
        var props = this.props;
        var originData = props.data;
        var data = this._saveOrigin(originData);
        // 根据分类度量进行数据分组
        var records = this._groupData(data);
        // 根据adjust分组
        var dataArray = this._adjustData(records);
        this.dataArray = dataArray;
        // scale适配调整，主要是调整 y 轴是否从 0 开始 以及 饼图
        this._adjustScales();
        // 数据排序（非必须）
        if (this.sortable) {
          this._sortData(records);
        }
        this.records = records;
      }
    }, {
      key: "_sortData",
      value: function _sortData(records) {
        var xScale = this.getXScale();
        var field = xScale.field,
          type = xScale.type;
        if (type !== 'identity' && xScale.values.length > 1) {
          each$1(records, function (_ref) {
            var children = _ref.children;
            children.sort(function (record1, record2) {
              if (type === 'timeCat') {
                return toTimeStamp$1(record1[FIELD_ORIGIN][field]) - toTimeStamp$1(record2[FIELD_ORIGIN][field]);
              }
              return xScale.translate(record1[FIELD_ORIGIN][field]) - xScale.translate(record2[FIELD_ORIGIN][field]);
            });
          });
        }
      }
    }, {
      key: "_initEvent",
      value: function _initEvent() {
        var _this2 = this;
        var context = this.context,
          props = this.props;
        ['onPressStart', 'onPress', 'onPressEnd', 'onPan', 'onPanStart', 'onPanEnd'].forEach(function (eventName) {
          if (props[eventName]) {
            context.gesture.on(eventName.substr(2).toLowerCase(), function (ev) {
              ev.geometry = _this2;
              props[eventName](ev);
            });
          }
        });
      }
    }, {
      key: "getY0Value",
      value: function getY0Value() {
        var attrs = this.attrs,
          props = this.props;
        var chart = props.chart;
        var field = attrs.y.field;
        var scale = chart.getScale(field);
        return chart.scale.getZeroValue(scale);
      }
      // 根据各属性映射的值域来获取真正的绘图属性
    }, {
      key: "_getShapeStyle",
      value: function _getShapeStyle(shape, origin) {
        var context = this.context,
          props = this.props,
          geomType = this.geomType;
        var theme = context.theme;
        var shapeTheme = theme.shape[geomType] || {};
        var defaultShapeStyle = shapeTheme.default;
        var shapeThemeStyle = shapeTheme[shape];
        var style = props.style;
        var shapeStyle = _objectSpread(_objectSpread({}, defaultShapeStyle), shapeThemeStyle);
        if (!style || !isObject$1(style)) {
          return shapeStyle;
        }
        // @ts-ignore
        var field = style.field,
          styles = _objectWithoutProperties$1(style, _excluded$c);
        var value = field ? origin[field] : origin;
        each$1(styles, function (attr, key) {
          if (isFunction$2(attr)) {
            shapeStyle[key] = attr(value);
          } else {
            shapeStyle[key] = attr;
          }
        });
        return shapeStyle;
      }
      /**
       * 数据映射到视图属性核心逻辑
       * x、y 每个元素走 normalize 然后 convertPoint
       * color、size、shape
       *  如果是Linear，则每个元素 走 mapping
       *  如果是Category/Identity 则第一个元素走 mapping
       */
    }, {
      key: "_mapping",
      value: function _mapping(records) {
        var attrs = this.attrs,
          props = this.props,
          attrController = this.attrController;
        var coord = props.coord;
        var _attrController$getAt = attrController.getAttrsByLinear(),
          linearAttrs = _attrController$getAt.linearAttrs,
          nonlinearAttrs = _attrController$getAt.nonlinearAttrs;
        var defaultAttrValues = attrController.getDefaultAttrValues();
        for (var i = 0, len = records.length; i < len; i++) {
          var record = records[i];
          var children = record.children;
          var attrValues = _objectSpread({}, defaultAttrValues);
          var firstChild = children[0];
          if (children.length === 0) {
            continue;
          }
          // 非线性映射
          for (var k = 0, _len = nonlinearAttrs.length; k < _len; k++) {
            var attrName = nonlinearAttrs[k];
            var attr = attrs[attrName];
            // 非线性映射只用映射第一项就可以了
            attrValues[attrName] = attr.mapping(firstChild[attr.field]);
          }
          // 线性属性映射
          for (var j = 0, childrenLen = children.length; j < childrenLen; j++) {
            var child = children[j];
            var normalized = {};
            for (var _k = 0; _k < linearAttrs.length; _k++) {
              var _attrName = linearAttrs[_k];
              var _attr = attrs[_attrName];
              // 分类属性的线性映射
              if (attrController.isGroupAttr(_attrName)) {
                attrValues[_attrName] = _attr.mapping(child[_attr.field], child);
              } else {
                normalized[_attrName] = _attr.normalize(child[_attr.field]);
              }
            }
            var _coord$convertPoint = coord.convertPoint({
                x: normalized.x,
                y: normalized.y
              }),
              x = _coord$convertPoint.x,
              y = _coord$convertPoint.y;
            // 获取 shape 的 style
            var shapeName = attrValues.shape;
            var shape = this._getShapeStyle(shapeName, child.origin);
            var selected = this.isSelected(child);
            mix(child, attrValues, {
              normalized: normalized,
              x: x,
              y: y,
              shapeName: shapeName,
              shape: shape,
              selected: selected
            });
          }
        }
        return records;
      }
      // 数据映射
    }, {
      key: "mapping",
      value: function mapping() {
        var records = this.records;
        // 数据映射
        this._mapping(records);
        return records;
      }
    }, {
      key: "getClip",
      value: function getClip() {
        var _this$props = this.props,
          coord = _this$props.coord,
          viewClip = _this$props.viewClip;
        var contentWidth = coord.width,
          contentHeight = coord.height,
          left = coord.left,
          top = coord.top;
        if (viewClip) {
          return {
            type: 'rect',
            attrs: {
              x: left,
              y: top,
              width: contentWidth,
              height: contentHeight
            }
          };
        }
        return null;
      }
    }, {
      key: "getAttr",
      value: function getAttr(attrName) {
        return this.attrController.getAttr(attrName);
      }
    }, {
      key: "getXScale",
      value: function getXScale() {
        return this.getAttr('x').scale;
      }
    }, {
      key: "getYScale",
      value: function getYScale() {
        return this.getAttr('y').scale;
      }
    }, {
      key: "_getXSnap",
      value: function _getXSnap(invertPointX) {
        var xScale = this.getXScale();
        if (xScale.isCategory) {
          return xScale.invert(invertPointX);
        }
        // linear 类型
        var invertValue = xScale.invert(invertPointX);
        var values = xScale.values;
        var len = values.length;
        // 如果只有1个点直接返回第1个点
        if (len === 1) {
          return values[0];
        }
        // 第1个点和第2个点之间
        if ((values[0] + values[1]) / 2 > invertValue) {
          return values[0];
        }
        // 最后2个点
        if ((values[len - 2] + values[len - 1]) / 2 <= invertValue) {
          return values[len - 1];
        }
        for (var i = 1; i < len; i++) {
          // 中间的点
          if ((values[i - 1] + values[i]) / 2 <= invertValue && (values[i + 1] + values[i]) / 2 > invertValue) {
            return values[i];
          }
        }
        return null;
      }
    }, {
      key: "_getYSnapRecords",
      value: function _getYSnapRecords(invertPointY, records) {
        var yScale = this.getYScale();
        var yField = yScale.field;
        var yValue = yScale.invert(invertPointY);
        // category
        if (yScale.isCategory) {
          return records.filter(function (record) {
            return record[FIELD_ORIGIN][yField] === yValue;
          });
        }
        // linear
        return records.filter(function (record) {
          var rangeY = record[yField];
          if (rangeY[0] <= yValue && rangeY[1] >= yValue) {
            return true;
          }
          return false;
        });
      }
      // 把 records 拍平
    }, {
      key: "flatRecords",
      value: function flatRecords() {
        var records = this.records;
        return records.reduce(function (prevRecords, record) {
          return prevRecords.concat(record.children);
        }, []);
      }
    }, {
      key: "getSnapRecords",
      value: function getSnapRecords(point, inCoordRange) {
        var props = this.props;
        var coord = props.coord,
          adjust = props.adjust;
        var invertPoint = coord.invertPoint(point);
        var xScale = this.getXScale();
        var yScale = this.getYScale();
        // 如果不在coord坐标范围内，直接返回空
        // if (invertPoint.x < 0 || invertPoint.y < 0) {
        //   return [];
        // }
        // 是否调整 point，默认为不调整
        if (inCoordRange) {
          var xRange = xScale.range;
          var yRange = yScale.range;
          // 如果 inCoordRange=true，当 point 不在 coord 坐标范围内时，调整到 range 内
          invertPoint.x = Math.min(Math.max(invertPoint.x, xRange[0]), xRange[1]);
          invertPoint.y = Math.min(Math.max(invertPoint.y, yRange[0]), yRange[1]);
        }
        var records = this.flatRecords();
        // 处理饼图
        if (adjust === 'stack' && coord.isPolar && coord.transposed) {
          // 弧度在半径范围内
          if (invertPoint.x >= 0 && invertPoint.x <= 1) {
            var snapRecords = this._getYSnapRecords(invertPoint.y, records);
            return snapRecords;
          }
        }
        var rst = [];
        var value = this._getXSnap(invertPoint.x);
        if (!value) {
          return rst;
        }
        var xField = xScale.field;
        var yField = yScale.field;
        for (var i = 0, len = records.length; i < len; i++) {
          var record = _objectSpread(_objectSpread({}, records[i]), {}, {
            xField: xField,
            yField: yField
          });
          var originValue = record[FIELD_ORIGIN][xField];
          if (xScale.type === 'timeCat' && toTimeStamp$1(originValue) === value) {
            rst.push(record);
          } else if (originValue === value) {
            rst.push(record);
          }
        }
        return rst;
      }
    }, {
      key: "getLegendItems",
      value: function getLegendItems() {
        var attrController = this.attrController;
        var colorAttr = attrController.getAttr('color');
        if (!colorAttr) return null;
        var scale = colorAttr.scale;
        if (!scale.isCategory) return null;
        var ticks = scale.getTicks();
        var items = ticks.map(function (tick) {
          var text = tick.text,
            tickValue = tick.tickValue;
          var color = colorAttr.mapping(tickValue);
          return {
            field: scale.field,
            color: color,
            name: text,
            tickValue: tickValue
          };
        });
        return items;
      }
    }]);
    return Geometry;
  }(Selection);

  var withLine = (function (View) {
    return /*#__PURE__*/function (_Geometry) {
      _inherits$1(Line, _Geometry);
      var _super = _createSuper$1(Line);
      function Line() {
        _classCallCheck$1(this, Line);
        return _super.apply(this, arguments);
      }
      _createClass$3(Line, [{
        key: "getDefaultCfg",
        value: function getDefaultCfg() {
          return {
            geomType: 'line',
            sortable: true
          };
        }
      }, {
        key: "splitPoints",
        value: function splitPoints(points) {
          var topPoints = [];
          var bottomPoints = [];
          for (var i = 0, len = points.length; i < len; i++) {
            var point = points[i];
            var x = point.x,
              y = point.y;
            topPoints.push(_objectSpread(_objectSpread({}, point), {}, {
              x: x,
              y: y[1]
            }));
            bottomPoints.push(_objectSpread(_objectSpread({}, point), {}, {
              x: x,
              y: y[0]
            }));
          }
          return [topPoints, bottomPoints];
        }
      }, {
        key: "splitNulls",
        value: function splitNulls(points, connectNulls) {
          if (connectNulls) {
            var _tmpPoints = [];
            for (var i = 0, len = points.length; i < len; i++) {
              var point = points[i];
              var y = point.y;
              if (isArray$1(y)) {
                if (isNaN(y[0])) {
                  continue;
                }
                _tmpPoints.push(point);
                continue;
              }
              if (isNaN(y)) {
                continue;
              }
              _tmpPoints.push(point);
            }
            if (_tmpPoints.length) {
              return [_tmpPoints];
            }
            return [];
          }
          var result = [];
          var tmpPoints = [];
          for (var _i = 0, _len = points.length; _i < _len; _i++) {
            var _point = points[_i];
            var _y = _point.y;
            if (isArray$1(_y)) {
              if (isNaN(_y[0])) {
                if (tmpPoints.length) {
                  result.push(tmpPoints);
                  tmpPoints = [];
                }
                continue;
              }
              tmpPoints.push(_point);
              continue;
            }
            if (isNaN(_y)) {
              if (tmpPoints.length) {
                result.push(tmpPoints);
                tmpPoints = [];
              }
              continue;
            }
            tmpPoints.push(_point);
          }
          if (tmpPoints.length) {
            result.push(tmpPoints);
          }
          return result;
        }
      }, {
        key: "mapping",
        value: function mapping() {
          var _this = this;
          var records = _get$1(_getPrototypeOf$1(Line.prototype), "mapping", this).call(this);
          var props = this.props,
            defaultConnectNulls = this.connectNulls;
          var coord = props.coord,
            _props$connectNulls = props.connectNulls,
            connectNulls = _props$connectNulls === void 0 ? defaultConnectNulls : _props$connectNulls;
          return records.map(function (record) {
            var children = record.children;
            // children 有可能为空
            var _ref = children[0] || {},
              size = _ref.size,
              color = _ref.color,
              shape = _ref.shape,
              y = _ref.y;
            // 极坐标时，需加入起点，从而闭合所绘图形
            var points = coord.isPolar ? [].concat(_toConsumableArray$1(children), [children[0]]) : children;
            var splitPoints = _this.splitNulls(points, connectNulls);
            var newChildren = splitPoints.map(function (points) {
              var _ref2 = isArray$1(y) ? _this.splitPoints(points) : [points, undefined],
                _ref3 = _slicedToArray$1(_ref2, 2),
                topPoints = _ref3[0],
                bottomPoints = _ref3[1];
              return {
                size: size,
                color: color,
                shape: shape,
                points: topPoints,
                bottomPoints: bottomPoints
              };
            });
            return _objectSpread(_objectSpread({}, record), {}, {
              children: newChildren
            });
          });
        }
      }, {
        key: "render",
        value: function render() {
          var props = this.props;
          var coord = props.coord;
          var records = this.mapping();
          var clip = this.getClip();
          return jsx(View, _objectSpread(_objectSpread({}, props), {}, {
            coord: coord,
            records: records,
            clip: clip
          }));
        }
      }]);
      return Line;
    }(Geometry);
  });

  var LineView = (function (props) {
    var records = props.records,
      coord = props.coord,
      animation = props.animation,
      EndView = props.endView,
      clip = props.clip;
    var left = coord.left,
      top = coord.top,
      width = coord.width,
      height = coord.height,
      center = coord.center,
      startAngle = coord.startAngle,
      endAngle = coord.endAngle,
      radius = coord.radius;
    var appear = coord.isPolar ? {
      easing: 'quadraticOut',
      duration: 450,
      clip: {
        type: 'sector',
        property: ['endAngle'],
        style: {
          x: center.x,
          y: center.y,
          startAngle: startAngle,
          r: radius
        },
        start: {
          endAngle: startAngle
        },
        end: {
          endAngle: endAngle
        }
      }
    } : {
      easing: 'quadraticOut',
      duration: 450,
      clip: {
        type: 'rect',
        property: ['width'],
        style: {
          x: left,
          y: top,
          height: height
        },
        start: {
          width: 0
        },
        end: {
          width: width
        }
      }
    };
    return jsx("group", {
      attrs: {
        clip: clip
      }
    }, records.map(function (record) {
      var key = record.key,
        children = record.children;
      return jsx("group", {
        key: key
      }, children.map(function (child) {
        var points = child.points,
          color = child.color,
          size = child.size,
          shape = child.shape;
        var fliterPoints = points.filter(function (point) {
          return !isNaN(point.x) && !isNaN(point.y);
        });
        if (fliterPoints.length === 0) return;
        return jsx("polyline", {
          key: key,
          style: _objectSpread(_objectSpread({
            points: fliterPoints.map(function (point) {
              return [point.x, point.y];
            }),
            stroke: color
          }, shape), {}, {
            lineWidth: size || shape.lineWidth
          }),
          animation: deepMix({
            update: {
              easing: 'linear',
              duration: 450,
              property: ['points']
            },
            appear: appear
          }, animation)
        });
      }));
    }));
  });

  var index = withLine(LineView);

  var withArea = (function (View) {
    return /*#__PURE__*/function (_withLine) {
      _inherits$1(Area, _withLine);
      var _super = _createSuper$1(Area);
      function Area() {
        _classCallCheck$1(this, Area);
        return _super.apply(this, arguments);
      }
      _createClass$3(Area, [{
        key: "getDefaultCfg",
        value: function getDefaultCfg() {
          return {
            geomType: 'area',
            // 面积图默认设为从0开始
            startOnZero: true,
            // 点需要排序
            sortable: true
          };
        }
      }, {
        key: "mapping",
        value: function mapping() {
          var records = _get$1(_getPrototypeOf$1(Area.prototype), "mapping", this).call(this);
          // 坐标轴 y0
          var y0 = this.getY0Value();
          var props = this.props,
            defaultStartOnZero = this.startOnZero;
          var coord = props.coord,
            _props$startOnZero = props.startOnZero,
            startOnZero = _props$startOnZero === void 0 ? defaultStartOnZero : _props$startOnZero;
          var baseY = coord.y[0];
          if (startOnZero) {
            // 零点映射到绝对坐标
            var originCoord = coord.convertPoint({
              x: 0,
              y: y0
            });
            baseY = originCoord.y;
          }
          for (var i = 0, len = records.length; i < len; i++) {
            var record = records[i];
            var children = record.children;
            for (var j = 0, _len = children.length; j < _len; j++) {
              var child = children[j];
              var points = child.points,
                bottomPoints = child.bottomPoints;
              if (bottomPoints && bottomPoints.length) {
                bottomPoints.reverse();
                child.points = points.concat(bottomPoints);
              } else {
                points.push({
                  x: points[points.length - 1].x,
                  y: baseY
                });
                points.push({
                  x: points[0].x,
                  y: baseY
                });
              }
            }
          }
          return records;
        }
      }]);
      return Area;
    }(withLine(View));
  });

  var AreaView = (function (props) {
    var coord = props.coord,
      records = props.records,
      shape = props.shape,
      animation = props.animation;
    var isSmooth = shape === 'smooth';
    var left = coord.left,
      top = coord.top,
      width = coord.width,
      height = coord.height,
      center = coord.center,
      startAngle = coord.startAngle,
      endAngle = coord.endAngle,
      radius = coord.radius;
    var appear = coord.isPolar ? {
      easing: 'quadraticOut',
      duration: 450,
      clip: {
        type: 'sector',
        property: ['endAngle'],
        style: {
          x: center.x,
          y: center.y,
          startAngle: startAngle,
          r: radius
        },
        start: {
          endAngle: startAngle
        },
        end: {
          endAngle: endAngle
        }
      }
    } : {
      easing: 'quadraticOut',
      duration: 450,
      clip: {
        type: 'rect',
        property: ['width'],
        style: {
          x: left,
          y: top,
          height: height
        },
        start: {
          width: 0
        },
        end: {
          width: width
        }
      }
    };
    return jsx("group", null, records.map(function (record) {
      var key = record.key,
        children = record.children;
      return jsx("group", {
        key: key
      }, children.map(function (child) {
        var points = child.points,
          bottomPoints = child.bottomPoints,
          color = child.color,
          shape = child.shape;
        if (isSmooth) {
          var generatePath = function generatePath() {
            var d = [];
            var constaint = [[0, 0], [1, 1]];
            var bottomPointsLen = (bottomPoints === null || bottomPoints === void 0 ? void 0 : bottomPoints.length) || 0;
            var topPoints = points.slice(0, points.length - bottomPointsLen);
            var topSps = catmullRom2bezier(topPoints, false, constaint);
            d.push(['M', topPoints[0].x, topPoints[0].y]);
            for (var i = 0, n = topSps.length; i < n; i++) {
              var sp = topSps[i];
              d.push(['C', sp[1], sp[2], sp[3], sp[4], sp[5], sp[6]]);
            }
            if (bottomPointsLen) {
              var bottomSps = catmullRom2bezier(bottomPoints, false, constaint);
              d.push(['L', bottomPoints[0].x, bottomPoints[0].y]);
              for (var _i = 0, _n = bottomSps.length; _i < _n; _i++) {
                var _sp = bottomSps[_i];
                d.push(['C', _sp[1], _sp[2], _sp[3], _sp[4], _sp[5], _sp[6]]);
              }
            }
            return d;
          };
          return jsx("path", {
            style: _objectSpread({
              path: generatePath(),
              lineWidth: '2px',
              fill: color
            }, shape)
          });
        }
        return jsx("polygon", {
          attrs: _objectSpread({
            points: points.map(function (point) {
              return [point.x, point.y];
            }),
            lineWidth: '2px',
            fill: color
          }, shape),
          animation: deepMix({
            appear: appear,
            update: {
              easing: 'linear',
              duration: 450,
              property: ['points']
            }
          }, animation)
        });
      }));
    }));
  });

  var index$1 = withArea(AreaView);

  /**
   * 计算两个坐标的中点坐标
   * @param start 起始点{x:number, y:number}
   * @param end 结束点{x:number, y:number}
   * @returns 中点坐标{x:number, y:number}
   */
  function getMiddlePoint(start, end) {
    var x = (end.x - start.x) / 2 + start.x;
    var y = (end.y - start.y) / 2 + start.y;
    return {
      x: x,
      y: y
    };
  }

  var DEFAULT_LABEL_CFG = {
    textBaseline: 'middle',
    fill: '#808080'
  };
  function LabelView(props) {
    var record = props.record,
      offsetX = props.offsetX,
      offsetY = props.offsetY,
      points = props.points,
      label = props.label,
      guide = props.guide;
    var origin = record.origin,
      color = record.color;
    var labelAttrs, guideAttrs;
    if (isFunction$2(label)) {
      var point = points.length === 4 // 如果是金字塔图，顶部只有 3 个点
      ? getMiddlePoint(points[1], points[2]) : getMiddlePoint(points[0], points[1]);
      labelAttrs = mix({
        x: point.x + offsetX,
        y: point.y + offsetY
      }, DEFAULT_LABEL_CFG, label(origin, color));
    }
    if (isFunction$2(guide)) {
      var _points$;
      var _point = getMiddlePoint(points.length === 4 ? getMiddlePoint(points[0], points[1]) : points[0], getMiddlePoint(points[2], (_points$ = points[3]) !== null && _points$ !== void 0 ? _points$ : points[1]));
      guideAttrs = mix({
        x: _point.x,
        y: _point.y,
        textBaseline: 'middle',
        textAlign: 'center'
      }, DEFAULT_LABEL_CFG, guide(origin, color));
    }
    return jsx("group", null, labelAttrs && jsx("text", {
      attrs: labelAttrs
    }), guideAttrs && jsx("text", {
      attrs: guideAttrs
    }));
  }

  var LabelViews = /*#__PURE__*/Object.freeze({
    __proto__: null,
    pyramid: LabelView,
    funnel: LabelView
  });

  var withInterval = (function (Views) {
    return /*#__PURE__*/function (_Geometry) {
      _inherits$1(Interval, _Geometry);
      var _super = _createSuper$1(Interval);
      function Interval() {
        _classCallCheck$1(this, Interval);
        return _super.apply(this, arguments);
      }
      _createClass$3(Interval, [{
        key: "getDefaultCfg",
        value: function getDefaultCfg() {
          return {
            geomType: 'interval',
            justifyContent: true,
            startOnZero: true
          };
        }
      }, {
        key: "getDefaultSize",
        value: function getDefaultSize() {
          var attrs = this.attrs,
            props = this.props,
            adjust = this.adjust,
            records = this.records;
          var coord = props.coord,
            sizeRatio = props.sizeRatio;
          var x = attrs.x;
          var scale = x.scale;
          var values = scale.values;
          if (sizeRatio) {
            return 1 / values.length * sizeRatio;
          }
          var defaultWithRatio = {
            column: 1 / 2,
            rose: 0.999999,
            multiplePie: 3 / 4 // 多饼图
          };

          var count = values.length;
          var ratio;
          if (coord.isPolar) {
            if (coord.transposed && count > 1) {
              ratio = defaultWithRatio.multiplePie;
            } else {
              ratio = defaultWithRatio.rose;
            }
          } else {
            ratio = defaultWithRatio.column;
          }
          var size = 1 / values.length * ratio;
          // 分组时size要除以类别个数
          if (adjust && adjust.type === 'dodge') {
            return size / records.length;
          }
          return size;
        }
      }, {
        key: "mapping",
        value: function mapping() {
          var records = _get$1(_getPrototypeOf$1(Interval.prototype), "mapping", this).call(this);
          var props = this.props;
          var coord = props.coord;
          var y0 = this.getY0Value();
          var defaultSize = this.getDefaultSize();
          for (var i = 0, len = records.length; i < len; i++) {
            var record = records[i];
            var children = record.children;
            for (var j = 0, _len = children.length; j < _len; j++) {
              var child = children[j];
              var normalized = child.normalized,
                mappedSize = child.size;
              // 没有指定size，则根据数据来计算默认size
              if (isNil$1(mappedSize)) {
                var x = normalized.x,
                  y = normalized.y,
                  _normalized$size = normalized.size,
                  size = _normalized$size === void 0 ? defaultSize : _normalized$size;
                mix(child, coord.convertRect({
                  x: x,
                  y: y,
                  y0: y0,
                  size: size
                }));
              } else {
                var _child = child,
                  _x = _child.x,
                  _y = _child.y;
                var rect = {
                  size: mappedSize,
                  x: _x,
                  y: _y,
                  y0: y0
                };
                mix(child, coord.transformToRect(rect));
              }
              mix(child.shape, this.getSelectionStyle(child));
            }
          }
          return records;
        }
        // 获取Y轴坐标零点的画布位置
      }, {
        key: "getPointY0",
        value: function getPointY0() {
          var props = this.props;
          var coord = props.coord;
          var y0 = this.getY0Value();
          var y0Point = coord.convertPoint({
            y: y0,
            x: 0
          });
          return y0Point === null || y0Point === void 0 ? void 0 : y0Point.y;
        }
      }, {
        key: "render",
        value: function render() {
          var props = this.props,
            state = this.state;
          var coord = props.coord,
            _props$shape = props.shape,
            shape = _props$shape === void 0 ? 'rect' : _props$shape,
            animation = props.animation,
            showLabel = props.showLabel,
            customLabelCfg = props.labelCfg;
          var View = isFunction$2(Views) ? Views : Views[shape];
          var LabelView = LabelViews[shape];
          var labelCfg = deepMix({
            label: null,
            offsetX: 0,
            offsetY: 0
          }, customLabelCfg);
          if (!View) return null;
          var selected = state.selected;
          var records = this.mapping();
          var pointY0 = this.getPointY0();
          var clip = this.getClip();
          return jsx(View, {
            coord: coord,
            records: records,
            selected: selected,
            shape: shape,
            animation: animation,
            showLabel: showLabel,
            labelCfg: labelCfg,
            LabelView: LabelView,
            y0: pointY0,
            clip: clip
          });
        }
      }]);
      return Interval;
    }(Geometry);
  });

  var Rect$2 = (function (props) {
    var records = props.records,
      animation = props.animation,
      y0 = props.y0,
      clip = props.clip;
    return jsx("group", {
      attrs: {
        clip: clip
      }
    }, records.map(function (record) {
      var key = record.key,
        children = record.children;
      return jsx("group", {
        key: key
      }, children.map(function (item) {
        var key = item.key,
          xMin = item.xMin,
          xMax = item.xMax,
          yMin = item.yMin,
          yMax = item.yMax,
          color = item.color,
          shape = item.shape;
        if (isNaN(xMin) || isNaN(xMax) || isNaN(yMin) || isNaN(yMax)) {
          return null;
        }
        return jsx("rect", {
          key: key,
          attrs: _objectSpread({
            x: xMin,
            y: yMin,
            width: xMax - xMin,
            height: yMax - yMin,
            fill: color
          }, shape),
          animation: deepMix({
            appear: {
              easing: 'linear',
              duration: 450,
              property: ['y', 'height'],
              start: {
                y: y0,
                height: 0
              }
            },
            update: {
              easing: 'linear',
              duration: 450,
              property: ['x', 'y', 'width', 'height']
            }
          }, animation)
        });
      }));
    }));
  });

  var Polar$1 = (function (props) {
    var coord = props.coord,
      records = props.records,
      animation = props.animation;
    var center = coord.center,
      startAngle = coord.startAngle,
      endAngle = coord.endAngle,
      radius = coord.radius;
    return jsx("group", {
      animation: {
        appear: _objectSpread(_objectSpread({
          easing: 'quadraticOut',
          duration: 450
        }, animation && animation.appear), {}, {
          clip: {
            type: 'sector',
            property: ['endAngle'],
            style: {
              x: center.x,
              y: center.y,
              startAngle: startAngle,
              r: radius
            },
            start: {
              endAngle: startAngle
            },
            end: {
              endAngle: endAngle
            }
          }
        })
      }
    }, records.map(function (record) {
      var key = record.key,
        children = record.children;
      return jsx("group", {
        key: key
      }, children.map(function (item) {
        var key = item.key,
          xMin = item.xMin,
          xMax = item.xMax,
          yMin = item.yMin,
          yMax = item.yMax,
          color = item.color,
          shape = item.shape;
        return jsx("sector", {
          key: key,
          attrs: _objectSpread({
            x: center.x,
            y: center.y,
            fill: color,
            lineWidth: 1,
            startAngle: "".concat(xMin, "rad"),
            endAngle: "".concat(xMax, "rad"),
            r0: yMin,
            r: yMax
          }, shape),
          animation: deepMix({
            update: {
              easing: 'linear',
              duration: 450,
              property: ['x', 'y', 'startAngle', 'endAngle', 'r0', 'r']
            }
          }, animation)
        });
      }));
    }));
  });

  var intervalView = (function (props) {
    var coord = props.coord;
    var coordType = coord.type;
    // 直角坐标系
    if (coordType === 'rect') {
      return jsx(Rect$2, _objectSpread({}, props));
    }
    // 极坐标系
    return jsx(Polar$1, _objectSpread({}, props));
  });

  function convertToPoints(_ref) {
    var xMin = _ref.xMin,
      xMax = _ref.xMax,
      yMin = _ref.yMin,
      yMax = _ref.yMax;
    return [{
      x: xMin,
      y: yMin
    }, {
      x: xMax,
      y: yMin
    }, {
      x: xMax,
      y: yMax
    }, {
      x: xMin,
      y: yMax
    } // bl
    ];
  }

  // 金字塔图和漏斗图的View
  var polygonView = (function (props) {
    var records = props.records,
      shape = props.shape,
      showLabel = props.showLabel,
      labelCfg = props.labelCfg,
      LabelView = props.LabelView;
    // 是否倒置
    var overturn = false;
    return jsx("group", null, records.map(function (record, index) {
      var key = record.key,
        children = record.children;
      var isLastRecord = index === records.length - 1;
      var nextRecord = isLastRecord ? record : records[index + 1];
      var nextChildren = nextRecord.children;
      var nextFirstPoint = convertToPoints(nextChildren[0]);
      var nextLastPoints = convertToPoints(nextChildren[nextChildren.length - 1]);
      if (!overturn) {
        overturn = nextChildren[0].yMax > children[0].yMax;
      }
      if (overturn) {
        nextFirstPoint.reverse();
        nextLastPoints.reverse();
      }
      var polygonPoints = children.map(function (child, childIndex) {
        var points = convertToPoints(child);
        if (overturn) {
          points.reverse();
        }
        if (isLastRecord) {
          if (shape === 'pyramid') {
            points = [getMiddlePoint(points[0], points[1]), points[2], points[3]];
          }
        } else {
          if (childIndex === 0) {
            points[0] = nextFirstPoint[3];
          }
          if (childIndex === children.length - 1) {
            points[1] = nextLastPoints[2];
          }
        }
        return _objectSpread(_objectSpread({}, child), {}, {
          points: points
        });
      });
      return jsx("group", {
        key: key
      }, polygonPoints.map(function (child) {
        var points = child.points,
          color = child.color,
          shape = child.shape;
        return jsx("group", null, jsx("polygon", {
          attrs: _objectSpread({
            points: points.map(function (d) {
              return [d.x, d.y];
            }),
            fill: color
          }, shape)
        }), showLabel && LabelView ? jsx(LabelView, _objectSpread({
          record: child,
          points: points
        }, labelCfg)) : null);
      }));
    }));
  });

  // 柱图/条图

  var Views = /*#__PURE__*/Object.freeze({
    __proto__: null,
    rect: intervalView,
    pyramid: polygonView,
    funnel: polygonView
  });

  var index$2 = withInterval(Views);

  var withPoint = (function (View) {
    return /*#__PURE__*/function (_Geometry) {
      _inherits$1(Point, _Geometry);
      var _super = _createSuper$1(Point);
      function Point() {
        _classCallCheck$1(this, Point);
        return _super.apply(this, arguments);
      }
      _createClass$3(Point, [{
        key: "getDefaultCfg",
        value: function getDefaultCfg() {
          return {
            geomType: 'point'
          };
        }
      }, {
        key: "render",
        value: function render() {
          var props = this.props;
          var coord = props.coord;
          var records = this.mapping();
          var clip = this.getClip();
          return jsx(View, _objectSpread(_objectSpread({}, props), {}, {
            coord: coord,
            records: records,
            clip: clip
          }));
        }
      }]);
      return Point;
    }(Geometry);
  });

  var PointView = (function (props) {
    var records = props.records,
      animation = props.animation,
      clip = props.clip;
    return jsx("group", {
      attrs: {
        clip: clip
      }
    }, records.map(function (record) {
      var key = record.key,
        children = record.children;
      return jsx("group", {
        key: key
      }, children.map(function (item) {
        var x = item.x,
          y = item.y,
          size = item.size,
          color = item.color,
          shapeName = item.shapeName,
          shape = item.shape;
        if (isNaN(x) || isNaN(y)) {
          return null;
        }
        if (shapeName === 'rect') {
          var rectSize = isNil$1(size) ? shape.size : size;
          return jsx("rect", {
            key: key,
            attrs: _objectSpread(_objectSpread({
              x: x - rectSize,
              y: y - rectSize,
              fill: color,
              stroke: color
            }, shape), {}, {
              width: rectSize * 2,
              height: rectSize * 2
            }),
            animation: deepMix({
              appear: {
                easing: 'linear',
                duration: 450
              },
              update: {
                easing: 'linear',
                duration: 450,
                property: ['x', 'y', 'width', 'height', 'fill']
              }
            }, animation)
          });
        }
        return jsx("circle", {
          key: key,
          style: _objectSpread(_objectSpread({
            cx: x,
            cy: y,
            fill: shapeName === 'circle' ? color : null,
            stroke: shapeName === 'hollowCircle' ? color : null
          }, shape), {}, {
            r: isNil$1(size) ? shape.size : size
          }),
          animation: deepMix({
            appear: {
              easing: 'linear',
              duration: 450
            },
            update: {
              easing: 'linear',
              duration: 450,
              property: ['cx', 'cy', 'r', 'fill']
            }
          }, animation)
        });
      }));
    }));
  });

  var index$3 = withPoint(PointView);

  var withAxis = (function (View) {
    return /*#__PURE__*/function (_Component) {
      _inherits$1(Axis, _Component);
      var _super = _createSuper$1(Axis);
      function Axis(props) {
        var _this;
        _classCallCheck$1(this, Axis);
        _this = _super.call(this, props);
        _this.axisStyle = {};
        var chart = props.chart,
          field = props.field;
        var scaleOption = _this.getScaleOption(props);
        chart.setScale(field, scaleOption);
        return _this;
      }
      _createClass$3(Axis, [{
        key: "willReceiveProps",
        value: function willReceiveProps(nextProps) {
          var lastProps = this.props;
          var chart = nextProps.chart,
            field = nextProps.field;
          var nextScaleOption = this.getScaleOption(nextProps);
          var lastScaleOption = this.getScaleOption(lastProps);
          if (!equal(nextScaleOption, lastScaleOption)) {
            chart.setScale(field, nextScaleOption);
          }
        }
      }, {
        key: "willMount",
        value: function willMount() {
          this.updateCoord();
        }
      }, {
        key: "willUpdate",
        value: function willUpdate() {
          this.updateCoord();
        }
      }, {
        key: "getScaleOption",
        value: function getScaleOption(props) {
          var type = props.type,
            tickCount = props.tickCount,
            range = props.range,
            mask = props.mask,
            formatter = props.formatter,
            min = props.min,
            max = props.max,
            nice = props.nice;
          return {
            type: type,
            tickCount: tickCount,
            range: range,
            mask: mask,
            formatter: formatter,
            min: min,
            max: max,
            nice: nice
          };
        }
      }, {
        key: "_getDimType",
        value: function _getDimType() {
          var props = this.props;
          var field = props.field,
            chart = props.chart;
          var xScales = chart.getXScales();
          var scales = xScales.filter(function (scale) {
            return scale.field === field;
          });
          return scales.length > 0 ? 'x' : 'y';
        }
        // 获取ticks最大的宽高
      }, {
        key: "getMaxBBox",
        value: function getMaxBBox(ticks, style) {
          var context = this.context;
          var measureText = context.measureText;
          var label = style.label,
            labelOffset = style.labelOffset;
          var width = 0;
          var height = 0;
          ticks.forEach(function (tick) {
            if (!label) return;
            var labelStyle = tick.labelStyle,
              text = tick.text;
            var bbox = measureText(text, _objectSpread(_objectSpread({}, label), labelStyle));
            width = Math.max(width, bbox.width);
            height = Math.max(height, bbox.height);
          });
          if (!width && !height) {
            return {
              width: width,
              height: height
            };
          }
          var bbox = {
            width: width + labelOffset,
            height: height + labelOffset
          };
          return bbox;
        }
      }, {
        key: "_getPosition",
        value: function _getPosition() {
          var props = this.props;
          var position = props.position,
            coord = props.coord;
          if (position) {
            return position;
          }
          var dimType = this._getDimType();
          if (coord.transposed) {
            return dimType === 'x' ? 'left' : 'bottom';
          }
          return dimType === 'x' ? 'bottom' : 'left';
        }
      }, {
        key: "getTicks",
        value: function getTicks() {
          var props = this.props;
          var field = props.field,
            chart = props.chart;
          var scale = chart.getScale(field);
          var ticks = scale.getTicks();
          // 设置tick的样式
          ticks = this._setTicksStyle(ticks);
          ticks = this._generateGridPoints(ticks);
          return ticks;
        }
        /**
         * 生成极坐标下网格线的交叉点
         * @param ticks
         * @returns
         */
      }, {
        key: "_generateGridPoints",
        value: function _generateGridPoints(ticks) {
          var props = this.props;
          var chart = props.chart,
            coord = props.coord;
          if (!coord.isPolar) {
            return ticks;
          }
          var dimType = this._getDimType();
          // 只需要在 y 的时候生成
          if (dimType !== 'y') {
            return ticks;
          }
          var xScale = chart.getXScales()[0];
          var xTicks = xScale.getTicks();
          ticks.forEach(function (tick) {
            var gridPoints = xTicks.map(function (xTick) {
              return coord.convertPoint({
                x: xTick.value,
                y: tick.value
              });
            });
            // 添加第 1 个点，形成环状
            gridPoints.push(gridPoints[0]);
            tick.gridPoints = gridPoints;
          });
          return ticks;
        }
      }, {
        key: "_setTicksStyle",
        value: function _setTicksStyle(ticks) {
          var _this2 = this;
          var props = this.props,
            context = this.context;
          var theme = context.theme,
            px2hd = context.px2hd;
          var _props$style = props.style,
            style = _props$style === void 0 ? {} : _props$style;
          var themeAxis = theme.axis;
          each$1(themeAxis, function (value, key) {
            // 关闭tick的样式
            if (style[key] === null) {
              return;
            }
            var styleValue = isFunction$2(style[key]) ? undefined : style[key];
            if (isString$1(value) || isNumber$1(value)) {
              _this2.axisStyle[key] = px2hd(styleValue) || value;
            } else {
              _this2.axisStyle[key] = px2hd(deepMix(clone$4(value), styleValue));
            }
          });
          return ticks.map(function (tick, index) {
            var label = style.label,
              grid = style.grid;
            var defaultLabelStyle = themeAxis.label,
              defaultGridStyle = themeAxis.grid;
            if (isFunction$2(label)) {
              tick.labelStyle = px2hd(mix({}, defaultLabelStyle, label(tick.text, index, ticks)));
            }
            if (isFunction$2(grid)) {
              tick.gridStyle = px2hd(mix({}, defaultGridStyle, grid(tick.text, index, ticks.length)));
            }
            return tick;
          });
        }
      }, {
        key: "convertTicks",
        value: function convertTicks(ticks) {
          var props = this.props;
          var coord = props.coord;
          var dimType = this._getDimType();
          var otherDim = dimType === 'x' ? 'y' : 'x';
          return ticks.map(function (tick) {
            var _coord$convertPoint, _coord$convertPoint2;
            var start = coord.convertPoint((_coord$convertPoint = {}, _defineProperty$1(_coord$convertPoint, dimType, tick.value), _defineProperty$1(_coord$convertPoint, otherDim, 0), _coord$convertPoint));
            var end = coord.convertPoint((_coord$convertPoint2 = {}, _defineProperty$1(_coord$convertPoint2, dimType, tick.value), _defineProperty$1(_coord$convertPoint2, otherDim, 1), _coord$convertPoint2));
            return _objectSpread(_objectSpread({}, tick), {}, {
              points: [start, end]
            });
          });
        }
      }, {
        key: "measureLayout",
        value: function measureLayout() {
          var props = this.props;
          var visible = props.visible,
            coord = props.coord;
          if (visible === false) {
            return null;
          }
          var ticks = this.getTicks();
          var bbox = this.getMaxBBox(ticks, this.axisStyle);
          var isPolar = coord.isPolar;
          var dimType = this._getDimType();
          var width = bbox.width,
            height = bbox.height;
          if (isPolar) {
            // 机坐标系的 y 不占位置
            if (dimType === 'y') {
              return null;
            }
            // 4 个方向都需要留空
            return ['top', 'right', 'bottom', 'left'].map(function (position) {
              return {
                position: position,
                width: width,
                height: height
              };
            });
          }
          // 直角坐标系下
          var position = this._getPosition();
          return {
            position: position,
            width: width,
            height: height
          };
        }
        // 主要是计算coord的布局
      }, {
        key: "updateCoord",
        value: function updateCoord() {
          var props = this.props;
          var chart = props.chart;
          var layout = this.measureLayout();
          chart.updateCoordFor(this, layout);
        }
      }, {
        key: "render",
        value: function render() {
          var props = this.props,
            axisStyle = this.axisStyle;
          var visible = props.visible,
            coord = props.coord;
          if (visible === false) {
            return null;
          }
          var ticks = this.getTicks();
          var position = this._getPosition();
          var dimType = this._getDimType();
          return jsx(View, _objectSpread(_objectSpread({}, props), {}, {
            style: axisStyle,
            ticks: this.convertTicks(ticks),
            coord: coord,
            position: position,
            dimType: dimType
          }));
        }
      }]);
      return Axis;
    }(Component);
  });

  // 相对圆心偏移量的点
  function getOffsetPoint(center, point, offset) {
    var vectorX = point.x - center.x;
    var vectorY = point.y - center.y;
    var vector = [vectorX, vectorY];
    var vectorLength = length$2(vector);
    var offsetLength = vectorLength + offset;
    var x = vectorX / vectorLength * offsetLength;
    var y = vectorY / vectorLength * offsetLength;
    return {
      x: center.x + x,
      y: center.y + y
    };
  }
  // 获取文本的对齐方式
  function getTextAlignInfo(center, point) {
    // 文本点向量
    var vector = [point.x - center.x, point.y - center.y];
    var align;
    var baseLine;
    // 水平对齐
    if (vector[0] > 0) {
      align = 'left';
    } else if (vector[0] < 0) {
      align = 'right';
    } else {
      align = 'center';
    }
    // 垂直对齐
    if (vector[1] > 0) {
      baseLine = 'top';
    } else if (vector[1] < 0) {
      baseLine = 'bottom';
    } else {
      baseLine = 'middle';
    }
    return {
      textAlign: align,
      textBaseline: baseLine
    };
  }
  var Line$1 = function Line(props) {
    var line = props.line,
      gridType = props.gridType,
      center = props.center,
      radius = props.radius,
      ticks = props.ticks;
    if (!line) return null;
    if (gridType !== 'line') {
      return jsx("arc", {
        attrs: _objectSpread({
          x: center.x,
          y: center.y,
          r: radius
        }, line)
      });
    }
    var points = ticks.map(function (tick) {
      var points = tick.points;
      return points[points.length - 1];
    });
    // 头尾相连
    points.push(points[0]);
    return jsx("polyline", {
      attrs: _objectSpread({
        points: points.map(function (d) {
          return [d.x, d.y];
        })
      }, line)
    });
  };
  var PolarX = (function (props) {
    var originTicks = props.ticks,
      coord = props.coord,
      style = props.style,
      gridType = props.grid;
    var center = coord.center;
    var grid = style.grid,
      tickLine = style.tickLine,
      line = style.line,
      labelOffset = style.labelOffset,
      label = style.label;
    var ticks = originTicks.filter(function (d) {
      return !isNaN(d.value);
    });
    var firstTicks = ticks[0];
    var points = firstTicks.points;
    var end = points[points.length - 1];
    var radius = length$2([end.x - center.x, end.y - center.y]);
    return jsx("group", null, grid ? ticks.map(function (tick) {
      var points = tick.points,
        gridStyle = tick.gridStyle;
      var end = points[points.length - 1];
      return jsx("line", {
        attrs: _objectSpread(_objectSpread({
          x1: center.x,
          y1: center.y,
          x2: end.x,
          y2: end.y
        }, grid), gridStyle)
      });
    }) : null, tickLine && tickLine.length ? ticks.map(function (tick) {
      var points = tick.points;
      var end = points[points.length - 1];
      var offsetPoint = getOffsetPoint(center, end, tickLine.length);
      return jsx("line", {
        attrs: _objectSpread({
          x1: end.x,
          y1: end.y,
          x2: offsetPoint.x,
          y2: offsetPoint.y
        }, tickLine)
      });
    }) : null, jsx(Line$1, {
      line: line,
      gridType: gridType,
      center: center,
      radius: radius,
      ticks: ticks
    }), label ? ticks.map(function (tick) {
      var points = tick.points,
        text = tick.text,
        labelStyle = tick.labelStyle;
      var end = points[points.length - 1];
      var offsetPoint = getOffsetPoint(center, end, labelOffset);
      return jsx("text", {
        attrs: _objectSpread(_objectSpread(_objectSpread({
          x: offsetPoint.x,
          y: offsetPoint.y,
          text: text
        }, getTextAlignInfo(center, end)), label), labelStyle)
      });
    }) : null);
  });

  var PolarY = (function (props) {
    var originTicks = props.ticks,
      coord = props.coord,
      style = props.style,
      gridType = props.grid;
    var center = coord.center;
    var grid = style.grid,
      tickLine = style.tickLine,
      line = style.line,
      labelOffset = style.labelOffset,
      label = style.label;
    var ticks = originTicks.filter(function (d) {
      return !isNaN(d.value);
    });
    return jsx("group", null, grid ? ticks.map(function (tick) {
      var points = tick.points,
        gridStyle = tick.gridStyle,
        gridPoints = tick.gridPoints;
      var end = points[points.length - 1];
      if (gridType !== 'line') {
        return jsx("arc", {
          attrs: _objectSpread(_objectSpread({
            x: center.x,
            y: center.y,
            r: length$2([end.x - center.x, end.y - center.y])
          }, grid), gridStyle)
        });
      }
      return jsx("polyline", {
        attrs: _objectSpread(_objectSpread({
          points: gridPoints.map(function (d) {
            return [d.x, d.y];
          })
        }, grid), gridStyle)
      });
    }) : null, tickLine && tickLine.length ? ticks.map(function (tick) {
      var points = tick.points;
      var end = points[points.length - 1];
      return jsx("line", {
        attrs: _objectSpread({
          x1: end.x,
          y1: end.y,
          x2: end.x - tickLine.length,
          y2: end.y
        }, tickLine)
      });
    }) : null, line ? jsx("line", {
      attrs: _objectSpread({
        x1: ticks[0].points[0].x,
        y1: ticks[0].points[0].y,
        x2: ticks[ticks.length - 1].points[0].x,
        y2: ticks[ticks.length - 1].points[0].y
      }, line)
    }) : null, label ? ticks.map(function (tick) {
      var points = tick.points,
        text = tick.text,
        labelStyle = tick.labelStyle;
      var end = points[points.length - 1];
      return jsx("text", {
        attrs: _objectSpread(_objectSpread({
          x: end.x - labelOffset,
          y: end.y,
          text: text,
          textAlign: 'right',
          textBaseline: 'middle'
        }, label), labelStyle)
      });
    }) : null);
  });

  var Top = (function (props) {
    var originTicks = props.ticks,
      coord = props.coord,
      style = props.style;
    var left = coord.left,
      top = coord.top,
      right = coord.right;
    var grid = style.grid,
      tickLine = style.tickLine,
      line = style.line,
      labelOffset = style.labelOffset,
      label = style.label;
    var ticks = originTicks.filter(function (d) {
      return !isNaN(d.value);
    });
    return jsx("group", null, grid ? ticks.map(function (tick) {
      var points = tick.points,
        gridStyle = tick.gridStyle;
      var start = points[0];
      var end = points[points.length - 1];
      return jsx("line", {
        attrs: _objectSpread(_objectSpread({
          x1: start.x,
          y1: start.y,
          x2: end.x,
          y2: end.y
        }, grid), gridStyle)
      });
    }) : null, tickLine && tickLine.length ? ticks.map(function (tick) {
      var points = tick.points;
      var end = points[points.length - 1];
      return jsx("line", {
        attrs: _objectSpread({
          x1: end.x,
          y1: end.y,
          x2: end.x,
          y2: end.y - tickLine.length
        }, tickLine)
      });
    }) : null, line ? jsx("line", {
      attrs: _objectSpread({
        x1: left,
        y1: top,
        x2: right,
        y2: top
      }, line)
    }) : null, label ? ticks.map(function (tick, _index) {
      var points = tick.points,
        text = tick.text,
        labelStyle = tick.labelStyle;
      var end = points[points.length - 1];
      return jsx("text", {
        attrs: _objectSpread(_objectSpread({
          x: end.x,
          y: end.y - labelOffset,
          textAlign: 'center',
          textBaseline: 'bottom',
          text: text
        }, label), labelStyle)
      });
    }) : null);
  });

  var Bottom = (function (props, context) {
    var ticks = props.ticks,
      coord = props.coord,
      style = props.style,
      animation = props.animation;
    var px2hd = context.px2hd;
    var left = coord.left,
      right = coord.right,
      bottom = coord.bottom;
    var grid = style.grid,
      tickLine = style.tickLine,
      line = style.line,
      labelOffset = style.labelOffset,
      label = style.label;
    var filterTicks = ticks.filter(function (d) {
      return !isNaN(d.value);
    });
    return jsx("group", null, grid ? filterTicks.map(function (tick) {
      var points = tick.points,
        tickValue = tick.tickValue,
        gridStyle = tick.gridStyle;
      var start = points[0];
      var end = points[points.length - 1];
      return jsx("line", {
        key: tickValue,
        attrs: _objectSpread(_objectSpread({
          x1: start.x,
          y1: start.y,
          x2: end.x,
          y2: end.y
        }, grid), gridStyle)
      });
    }) : null, tickLine && tickLine.length ? filterTicks.map(function (tick) {
      var points = tick.points,
        tickValue = tick.tickValue;
      var start = points[0];
      return jsx("line", {
        key: tickValue,
        attrs: _objectSpread({
          x1: start.x,
          y1: start.y,
          x2: start.x,
          y2: start.y + px2hd(tickLine.length)
        }, tickLine)
      });
    }) : null, line ? jsx("line", {
      attrs: _objectSpread({
        x1: left,
        y1: bottom,
        x2: right,
        y2: bottom
      }, line)
    }) : null, label ? filterTicks.map(function (tick, index) {
      var points = tick.points,
        text = tick.text,
        tickValue = tick.tickValue,
        labelStyle = tick.labelStyle;
      var start = points[0];
      var _ref = labelStyle || label || {},
        _ref$align = _ref.align,
        align = _ref$align === void 0 ? 'center' : _ref$align;
      var textAttrs = _objectSpread(_objectSpread({
        x: start.x,
        y: start.y + labelOffset,
        textBaseline: 'top',
        text: text
      }, label), labelStyle);
      if (align === 'between') {
        if (index === 0) {
          textAttrs.textAlign = 'start';
        } else if (index === ticks.length - 1) {
          textAttrs.textAlign = 'end';
        } else {
          textAttrs.textAlign = 'center';
        }
      } else {
        textAttrs.textAlign = align;
      }
      return jsx("text", {
        key: tickValue,
        attrs: textAttrs,
        animation: animation || {
          appear: {
            easing: 'linear',
            duration: 300,
            delay: 0,
            property: ['fillOpacity'],
            start: {
              fillOpacity: 0
            },
            end: {
              fillOpacity: 1
            }
          },
          update: {
            easing: 'linear',
            duration: 450,
            delay: 0,
            property: ['x', 'y']
          },
          leave: {
            easing: 'linear',
            duration: 450,
            delay: 0,
            property: ['fillOpacity'],
            start: {
              fillOpacity: 1
            },
            end: {
              fillOpacity: 0
            }
          }
        }
      });
    }) : null);
  });

  var Right = (function (props) {
    var originTicks = props.ticks,
      coord = props.coord,
      style = props.style;
    var top = coord.top,
      right = coord.right,
      bottom = coord.bottom;
    var grid = style.grid,
      tickLine = style.tickLine,
      line = style.line,
      labelOffset = style.labelOffset,
      label = style.label;
    var ticks = originTicks.filter(function (d) {
      return !isNaN(d.value);
    });
    return jsx("group", null, grid ? ticks.map(function (tick) {
      var points = tick.points,
        gridStyle = tick.gridStyle;
      var start = points[0];
      var end = points[points.length - 1];
      return jsx("line", {
        attrs: _objectSpread(_objectSpread({
          x1: start.x,
          y1: start.y,
          x2: end.x,
          y2: end.y
        }, grid), gridStyle)
      });
    }) : null, tickLine && tickLine.length ? ticks.map(function (tick) {
      var points = tick.points;
      var end = points[points.length - 1];
      return jsx("line", {
        attrs: _objectSpread({
          x1: end.x,
          y1: end.y,
          x2: end.x + tickLine.length,
          y2: end.y
        }, tickLine)
      });
    }) : null, line ? jsx("line", {
      attrs: _objectSpread({
        x1: right,
        y1: top,
        x2: right,
        y2: bottom
      }, line)
    }) : null, label ? ticks.map(function (tick, _index) {
      var points = tick.points,
        text = tick.text,
        labelStyle = tick.labelStyle;
      var end = points[points.length - 1];
      return jsx("text", {
        attrs: _objectSpread(_objectSpread({
          x: end.x + labelOffset,
          y: end.y,
          textAlign: 'left',
          textBaseline: 'middle',
          text: text
        }, label), labelStyle)
      });
    }) : null);
  });

  var Left = (function (props) {
    var originTicks = props.ticks,
      coord = props.coord,
      style = props.style,
      animation = props.animation;
    var left = coord.left,
      top = coord.top,
      bottom = coord.bottom;
    var grid = style.grid,
      tickLine = style.tickLine,
      line = style.line,
      labelOffset = style.labelOffset,
      label = style.label;
    var ticks = originTicks.filter(function (d) {
      return !isNaN(d.value);
    });
    return jsx("group", null, grid ? ticks.map(function (tick) {
      var points = tick.points,
        tickValue = tick.tickValue,
        gridStyle = tick.gridStyle;
      var start = points[0];
      var end = points[points.length - 1];
      return jsx("line", {
        key: tickValue,
        attrs: _objectSpread(_objectSpread({
          x1: start.x,
          y1: start.y,
          x2: end.x,
          y2: end.y
        }, grid), gridStyle)
      });
    }) : null, tickLine && tickLine.length ? ticks.map(function (tick) {
      var points = tick.points,
        tickValue = tick.tickValue;
      var start = points[0];
      return jsx("line", {
        key: tickValue,
        attrs: _objectSpread({
          x1: start.x,
          y1: start.y,
          x2: start.x - tickLine.length,
          y2: start.y
        }, tickLine)
      });
    }) : null, line ? jsx("line", {
      attrs: _objectSpread({
        x1: left,
        y1: top,
        x2: left,
        y2: bottom
      }, line)
    }) : null, label ? ticks.map(function (tick, _index) {
      var tickValue = tick.tickValue,
        points = tick.points,
        text = tick.text,
        labelStyle = tick.labelStyle;
      var start = points[0];
      return jsx("text", {
        key: tickValue,
        attrs: _objectSpread(_objectSpread({
          x: start.x - labelOffset,
          y: start.y,
          textAlign: 'right',
          textBaseline: 'middle',
          text: text
        }, label), labelStyle),
        animation: animation || {
          appear: {
            easing: 'linear',
            duration: 300,
            delay: 0,
            property: ['fillOpacity'],
            start: {
              fillOpacity: 0
            },
            end: {
              fillOpacity: 1
            }
          },
          update: {
            easing: 'linear',
            duration: 450,
            delay: 0,
            property: ['x', 'y']
          },
          leave: {
            easing: 'linear',
            duration: 450,
            delay: 0,
            property: ['fillOpacity'],
            start: {
              fillOpacity: 1
            },
            end: {
              fillOpacity: 0
            }
          }
        }
      });
    }) : null);
  });

  function isPolar(props) {
    return props.coord.isPolar;
  }
  var AxisView = (function (props) {
    // 极坐标
    if (isPolar(props)) {
      var dimType = props.dimType;
      if (dimType === 'x') {
        return jsx(PolarX, _objectSpread({}, props));
      }
      return jsx(PolarY, _objectSpread({}, props));
    }
    var position = props.position;
    // 直角坐标
    if (position === 'right') {
      return jsx(Right, _objectSpread({}, props));
    }
    if (position === 'left') {
      return jsx(Left, _objectSpread({}, props));
    }
    if (position === 'top') {
      return jsx(Top, _objectSpread({}, props));
    }
    return jsx(Bottom, _objectSpread({}, props));
  });

  var index$4 = withAxis(AxisView);

  var withLegend = (function (View) {
    return /*#__PURE__*/function (_Component) {
      _inherits$1(Legend, _Component);
      var _super = _createSuper$1(Legend);
      function Legend(props) {
        var _this;
        _classCallCheck$1(this, Legend);
        _this = _super.call(this, props);
        _this._onclick = function (item) {
          var _assertThisInitialize = _assertThisInitialized$3(_this),
            props = _assertThisInitialize.props;
          var chart = props.chart,
            _props$clickable = props.clickable,
            clickable = _props$clickable === void 0 ? true : _props$clickable,
            onClick = props.onClick;
          if (!clickable) return;
          var clickItem = item.currentTarget;
          if (!clickItem) {
            return;
          }
          // @ts-ignore
          var dataItem = clickItem.config['data-item'];
          if (!dataItem) {
            return;
          }
          if (isFunction$2(onClick)) {
            onClick(dataItem);
          }
          var field = dataItem.field,
            tickValue = dataItem.tickValue;
          var prevFiltered = _this.state.filtered;
          var filtered = _objectSpread(_objectSpread({}, prevFiltered), {}, _defineProperty$1({}, tickValue, !prevFiltered[tickValue]));
          _this.setState({
            filtered: filtered
          });
          chart.filter(field, function (value) {
            return !filtered[value];
          });
        };
        _this.state = {
          filtered: {},
          items: []
        };
        return _this;
      }
      _createClass$3(Legend, [{
        key: "getOriginItems",
        value: function getOriginItems() {
          var chart = this.props.chart;
          return chart.getLegendItems();
        }
      }, {
        key: "getItems",
        value: function getItems() {
          var _props$items;
          var props = this.props,
            state = this.state;
          var filtered = state.filtered;
          var renderItems = ((_props$items = props.items) === null || _props$items === void 0 ? void 0 : _props$items.length) ? props.items : this.getOriginItems();
          if (!renderItems) return null;
          return renderItems.map(function (item) {
            var tickValue = item.tickValue;
            return _objectSpread(_objectSpread({}, item), {}, {
              filtered: filtered[tickValue]
            });
          });
        }
      }, {
        key: "setItems",
        value: function setItems(items) {
          this.setState({
            items: items
          });
        }
      }, {
        key: "getMaxItemBox",
        value: function getMaxItemBox(node) {
          var maxItemWidth = 0;
          var maxItemHeight = 0;
          (node.children || []).forEach(function (child) {
            var layout = child.layout;
            var width = layout.width,
              height = layout.height;
            maxItemWidth = Math.max(maxItemWidth, width);
            maxItemHeight = Math.max(maxItemHeight, height);
          });
          return {
            width: maxItemWidth,
            height: maxItemHeight
          };
        }
        // 计算 legend 的位置
      }, {
        key: "_init",
        value: function _init() {
          var props = this.props,
            context = this.context;
          var parentLayout = props.layout,
            customWidth = props.width,
            customHeight = props.height,
            _props$position = props.position,
            position = _props$position === void 0 ? 'top' : _props$position;
          var items = this.getItems();
          if (!items || !items.length) return;
          var left = parentLayout.left,
            top = parentLayout.top,
            layoutWidth = parentLayout.width,
            layoutHeight = parentLayout.height;
          var width = context.px2hd(customWidth) || layoutWidth;
          var node = computeLayout$1(this, this.render());
          var _this$getMaxItemBox = this.getMaxItemBox(node),
            itemMaxWidth = _this$getMaxItemBox.width,
            itemMaxHeight = _this$getMaxItemBox.height;
          // 每行最多的个数
          var lineMaxCount = Math.max(1, Math.floor(width / itemMaxWidth));
          var itemCount = items.length;
          // legend item 的行数
          var lineCount = Math.ceil(itemCount / lineMaxCount);
          var itemWidth = width / lineMaxCount;
          var autoHeight = itemMaxHeight * lineCount;
          var style = {
            left: left,
            top: top,
            width: width,
            // height 默认自适应
            height: undefined,
            flexDirection: 'row',
            flexWrap: 'wrap',
            alignItems: 'center',
            justifyContent: 'flex-start'
          };
          // 如果只有一行，2端对齐
          if (lineCount === 1) {
            style.justifyContent = 'space-between';
          }
          if (position === 'top') {
            style.height = customHeight ? customHeight : autoHeight;
          }
          if (position === 'left') {
            style.flexDirection = 'column';
            style.justifyContent = 'center';
            style.width = itemMaxWidth;
            style.height = customHeight ? customHeight : layoutHeight;
          }
          if (position === 'right') {
            style.flexDirection = 'column';
            style.alignItems = 'flex-start';
            style.justifyContent = 'center';
            style.width = itemMaxWidth;
            style.height = customHeight ? customHeight : layoutHeight;
            style.left = left + (width - itemMaxWidth);
          }
          if (position === 'bottom') {
            style.top = top + (layoutHeight - autoHeight);
            style.height = customHeight ? customHeight : autoHeight;
          }
          this.itemWidth = itemWidth;
          this.legendStyle = style;
        }
      }, {
        key: "updateCoord",
        value: function updateCoord() {
          var context = this.context,
            props = this.props,
            legendStyle = this.legendStyle;
          var _props$position2 = props.position,
            position = _props$position2 === void 0 ? 'top' : _props$position2,
            _props$margin = props.margin,
            margin = _props$margin === void 0 ? '30px' : _props$margin,
            chart = props.chart;
          var width = legendStyle.width,
            height = legendStyle.height;
          var marginNumber = context.px2hd(margin);
          chart.updateCoordFor(this, {
            position: position,
            width: width + marginNumber,
            height: height + marginNumber
          });
        }
      }, {
        key: "willMount",
        value: function willMount() {
          var items = this.getItems();
          if (!items || !items.length) return;
          this._init();
          this.updateCoord();
        }
      }, {
        key: "didMount",
        value: function didMount() {
          // this._initEvent();
        }
      }, {
        key: "willUpdate",
        value: function willUpdate() {
          var items = this.getItems();
          if (!items || !items.length) return;
          this.updateCoord();
        }
      }, {
        key: "render",
        value: function render() {
          var props = this.props,
            itemWidth = this.itemWidth,
            legendStyle = this.legendStyle;
          var items = this.getItems();
          if (!items || !items.length) {
            return null;
          }
          return jsx(View, _objectSpread(_objectSpread({}, props), {}, {
            items: items,
            itemWidth: itemWidth,
            style: _objectSpread(_objectSpread({}, legendStyle), props.style),
            onClick: this._onclick
          }));
        }
      }]);
      return Legend;
    }(Component);
  });

  var Marker$1 = function Marker(_ref) {
    var type = _ref.type,
      color = _ref.color;
    if (type === 'square') {
      return jsx("rect", {
        style: {
          display: 'flex',
          width: '12px',
          height: '12px',
          marginRight: '10px'
        },
        attrs: {
          fill: color
        }
      });
    }
    return jsx("circle", {
      style: {
        display: 'flex',
        width: '12px',
        height: '12px',
        marginRight: '10px',
        fill: color
      }
    });
  };
  var LegendView = (function (props) {
    var items = props.items,
      itemWidth = props.itemWidth,
      itemFormatter = props.itemFormatter,
      style = props.style,
      _props$marker = props.marker,
      marker = _props$marker === void 0 ? 'circle' : _props$marker,
      nameStyle = props.nameStyle,
      valueStyle = props.valueStyle,
      valuePrefix = props.valuePrefix,
      onClick = props.onClick;
    var formatValue = function formatValue(value) {
      var valuePrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ': ';
      return "".concat(valuePrefix).concat(value);
    };
    return jsx("group", {
      style: _objectSpread({
        display: 'flex'
      }, style)
    }, items.map(function (item) {
      var color = item.color,
        name = item.name,
        value = item.value,
        filtered = item.filtered,
        tickValue = item.tickValue;
      var valueText = isFunction$2(itemFormatter) ? itemFormatter(value, tickValue) : value;
      return jsx("group", {
        className: "legend-item",
        style: {
          width: itemWidth,
          display: 'flex',
          flexDirection: 'row',
          alignItems: 'center',
          justifyContent: 'flex-start',
          //TODO: padding改为’12px‘ 就和原来一致了
          padding: ['6px', '6px', '6px', 0]
        },
        "data-item": item,
        onClick: onClick
      }, jsx(Marker$1, {
        color: filtered ? '#bfbfbf' : color,
        type: marker
      }), jsx("text", {
        attrs: _objectSpread({
          fill: filtered ? '#bfbfbf' : '#808080',
          text: name
        }, nameStyle)
      }), valueText ? jsx("text", {
        attrs: _objectSpread({
          fill: '#808080',
          text: formatValue(valueText, valuePrefix)
        }, valueStyle)
      }) : null);
    }));
  });

  var index$5 = withLegend(LegendView);

  function isInBBox$1(bbox, point) {
    var minX = bbox.minX,
      maxX = bbox.maxX,
      minY = bbox.minY,
      maxY = bbox.maxY;
    var x = point.x,
      y = point.y;
    return minX <= x && maxX >= x && minY <= y && maxY >= y;
  }
  var withGuide = (function (View) {
    return /*#__PURE__*/function (_Component) {
      _inherits$1(Guide, _Component);
      var _super = _createSuper$1(Guide);
      function Guide(props) {
        var _this;
        _classCallCheck$1(this, Guide);
        _this = _super.call(this, props);
        // 创建ref
        _this.triggerRef = {};
        _this.state = {};
        return _this;
      }
      _createClass$3(Guide, [{
        key: "willMount",
        value: function willMount() {
          this.guideBBox = this.getGuideBBox();
        }
      }, {
        key: "didMount",
        value: function didMount() {
          var _this2 = this;
          var context = this.context,
            props = this.props;
          var onClick = props.onClick;
          context.gesture.on('click', function (ev) {
            var x = ev.x,
              y = ev.y;
            var shape = _this2.triggerRef.current;
            if (!shape || shape.isDestroyed()) return;
            var bbox = shape.getBBox();
            if (isInBBox$1(bbox, {
              x: x,
              y: y
            })) {
              ev.shape = shape;
              onClick && onClick(ev);
            }
          });
        }
      }, {
        key: "didUpdate",
        value: function didUpdate() {
          this.guideBBox = this.getGuideBBox();
        }
      }, {
        key: "getGuideBBox",
        value: function getGuideBBox() {
          var node = computeLayout$1(this, this.render());
          var layout = node.layout;
          if (!layout) return;
          var width = layout.width,
            height = layout.height;
          // getBBox 没有包含 padding 所以这里手动计算 bbox
          var box = {
            width: width,
            height: height
          };
          return box;
        }
        // 解析record里的模板字符串，如min、max、50%...
      }, {
        key: "parseReplaceStr",
        value: function parseReplaceStr(value, scale) {
          var replaceMap = {
            min: 0,
            max: 1,
            median: 0.5
          };
          // 传入的是 min、max、median 的
          if (!isNil$1(replaceMap[value])) {
            return replaceMap[value];
          }
          // 传入的是 xx%
          if (isString$1(value) && value.indexOf('%') != -1 && !isNaN(Number(value.slice(0, -1)))) {
            var rateValue = Number(value.slice(0, -1));
            var percent = rateValue / 100;
            return percent;
          }
          return scale.scale(value);
        }
      }, {
        key: "parsePoint",
        value: function parsePoint(record) {
          var props = this.props;
          var chart = props.chart,
            coord = props.coord;
          var xScale = chart.getXScales()[0];
          // 只取第一个yScale
          var yScale = chart.getYScales()[0];
          // 解析 record 为归一化后的坐标
          var x = this.parseReplaceStr(record[xScale.field], xScale);
          var y = this.parseReplaceStr(record[yScale.field], yScale);
          return coord.convertPoint({
            x: x,
            y: y
          });
        }
      }, {
        key: "convertPoints",
        value: function convertPoints(records) {
          var _this3 = this;
          return records.map(function (record) {
            return _this3.parsePoint(record);
          });
        }
      }, {
        key: "getGuideTheme",
        value: function getGuideTheme() {
          var context = this.context;
          var theme = context.theme;
          return theme.guide;
        }
      }, {
        key: "render",
        value: function render() {
          var props = this.props,
            context = this.context,
            guideBBox = this.guideBBox;
          var coord = props.coord,
            _props$records = props.records,
            records = _props$records === void 0 ? [] : _props$records,
            animation = props.animation,
            chart = props.chart;
          var width = context.width,
            height = context.height;
          var points = this.convertPoints(records);
          var theme = this.getGuideTheme();
          return jsx(View, _objectSpread(_objectSpread({
            triggerRef: this.triggerRef,
            points: points,
            theme: theme,
            coord: coord
          }, props), {}, {
            canvasWidth: width,
            canvasHeight: height,
            guideBBox: guideBBox,
            animation: isFunction$2(animation) ? animation(points, chart) : animation
          }));
        }
      }]);
      return Guide;
    }(Component);
  });

  var TextGuideView = (function (props, context) {
    var _props$theme = props.theme,
      theme = _props$theme === void 0 ? {} : _props$theme;
    var _deepMix = deepMix(_objectSpread({}, theme.text), props),
      points = _deepMix.points,
      style = _deepMix.style,
      offsetX = _deepMix.offsetX,
      offsetY = _deepMix.offsetY,
      content = _deepMix.content,
      animation = _deepMix.animation;
    var _ref = points[0] || {},
      x = _ref.x,
      y = _ref.y;
    var offsetXNum = context.px2hd(offsetX);
    var offsetYNum = context.px2hd(offsetY);
    var posX = x + (offsetXNum || 0);
    var posY = y + (offsetYNum || 0);
    return jsx("text", {
      attrs: _objectSpread({
        text: "".concat(content),
        x: posX,
        y: posY
      }, style),
      animation: deepMix({
        update: {
          easing: 'linear',
          duration: 450,
          property: ['x', 'y']
        }
      }, animation)
    });
  });

  var PointGuideView = (function (props, context) {
    var theme = props.theme;
    var _deepMix = deepMix(_objectSpread({}, theme.point), props),
      points = _deepMix.points,
      style = _deepMix.style,
      offsetX = _deepMix.offsetX,
      offsetY = _deepMix.offsetY,
      animation = _deepMix.animation;
    var _ref = points[0] || {},
      x = _ref.x,
      y = _ref.y;
    var offsetXNum = context.px2hd(offsetX);
    var offsetYNum = context.px2hd(offsetY);
    var posX = x + (offsetXNum || 0);
    var posY = y + (offsetYNum || 0);
    return jsx("group", null, jsx("circle", {
      attrs: _objectSpread({
        cx: posX,
        cy: posY
      }, style),
      animation: animation
    }));
  });

  var LineGuideView = (function (props, context) {
    var _props$theme = props.theme,
      theme = _props$theme === void 0 ? {} : _props$theme;
    var _deepMix = deepMix(_objectSpread({}, theme.line), props),
      points = _deepMix.points,
      style = _deepMix.style,
      offsetX = _deepMix.offsetX,
      offsetY = _deepMix.offsetY,
      animation = _deepMix.animation;
    var _ref = points[0] || {},
      x1 = _ref.x,
      y1 = _ref.y;
    var _ref2 = points[1] || {},
      x2 = _ref2.x,
      y2 = _ref2.y;
    var offsetXNum = context.px2hd(offsetX);
    var offsetYNum = context.px2hd(offsetY);
    var posX1 = x1 + (isArray$1(offsetXNum) ? offsetXNum[0] || 0 : offsetXNum || 0);
    var posY1 = y1 + (isArray$1(offsetYNum) ? offsetYNum[0] || 0 : offsetYNum || 0);
    var posX2 = x2 + (isArray$1(offsetXNum) ? offsetXNum[1] || 0 : offsetXNum || 0);
    var posY2 = y2 + (isArray$1(offsetYNum) ? offsetYNum[1] || 0 : offsetYNum || 0);
    return jsx("group", null, jsx("line", {
      attrs: _objectSpread({
        x1: posX1,
        y1: posY1,
        x2: posX2,
        y2: posY2
      }, style),
      animation: animation
    }));
  });

  var ArcGuideView = (function (props) {
    var _props$theme = props.theme,
      theme = _props$theme === void 0 ? {} : _props$theme;
    var _deepMix = deepMix(_objectSpread({}, theme.line), props),
      coord = _deepMix.coord,
      points = _deepMix.points,
      style = _deepMix.style,
      animation = _deepMix.animation;
    var start = points[0] || {};
    var end = points[1] || {};
    var coordCenter = coord.center;
    var radius = Math.sqrt((start.x - coordCenter.x) * (start.x - coordCenter.x) + (start.y - coordCenter.y) * (start.y - coordCenter.y));
    var startAngle = Math.atan2(start.y - coordCenter.y, start.x - coordCenter.x);
    var endAngle = Math.atan2(end.y - coordCenter.y, end.x - coordCenter.x);
    return jsx("group", null, jsx("arc", {
      attrs: _objectSpread({
        x: coordCenter.x,
        y: coordCenter.y,
        r: radius,
        startAngle: startAngle,
        endAngle: endAngle
      }, style),
      animation: animation
    }));
  });

  var RectGuideView = (function (props) {
    var _props$theme = props.theme,
      theme = _props$theme === void 0 ? {} : _props$theme;
    var _deepMix = deepMix(_objectSpread({}, theme.rect), props),
      points = _deepMix.points,
      style = _deepMix.style,
      animation = _deepMix.animation;
    var start = points[0] || {};
    var end = points[1] || {};
    return jsx("group", null, jsx("rect", {
      attrs: _objectSpread({
        x: Math.min(start.x, end.x),
        y: Math.min(start.y, end.y),
        width: Math.abs(end.x - start.x),
        height: Math.abs(start.y - end.y)
      }, style),
      animation: animation
    }));
  });

  var defaultProps = {
    offsetX: 0,
    offsetY: 0,
    points: [],
    src: ''
  };
  var baseAttrs = {
    height: '20px',
    width: '20px'
  };
  var ImageGuideView = (function (props, context) {
    var cfg = deepMix({}, defaultProps, props);
    var points = cfg.points,
      style = cfg.style,
      attrs = cfg.attrs,
      offsetX = cfg.offsetX,
      offsetY = cfg.offsetY,
      src = cfg.src,
      animation = cfg.animation;
    var _ref = points[0] || {},
      x = _ref.x,
      y = _ref.y;
    var _attrs$height = attrs.height,
      height = _attrs$height === void 0 ? 0 : _attrs$height,
      _attrs$width = attrs.width,
      width = _attrs$width === void 0 ? 0 : _attrs$width;
    var offsetXNum = context.px2hd(offsetX);
    var offsetYNum = context.px2hd(offsetY);
    var posX = x + (offsetXNum || 0) - height / 2;
    var posY = y + (offsetYNum || 0) - width / 2;
    return jsx("group", {
      style: style
    }, jsx("image", {
      attrs: _objectSpread(_objectSpread(_objectSpread({}, baseAttrs), attrs), {}, {
        height: height + 'px',
        width: width + 'px',
        x: posX,
        y: posY,
        src: src
      }),
      animation: deepMix({
        update: {
          easing: 'linear',
          duration: 450,
          property: ['x', 'y']
        }
      }, animation)
    }));
  });

  var defaultProps$1 = {
    offsetX: 0,
    offsetY: 0,
    points: [],
    direct: 'tl',
    side: '8px',
    autoAdjust: true
  };
  var defaultStyle = {
    container: {
      fill: '#1677FF',
      radius: '4px',
      padding: ['4px', '8px']
    },
    text: {
      fontSize: '22px',
      fill: '#fff'
    },
    arrow: {
      fill: '#1677FF'
    }
  };
  var TagGuideView = (function (props, context) {
    var px2hd = context.px2hd;
    var cfg = _objectSpread(_objectSpread({}, defaultProps$1), props);
    var _px2hd = px2hd(cfg),
      points = _px2hd.points,
      content = _px2hd.content,
      offsetX = _px2hd.offsetX,
      offsetY = _px2hd.offsetY,
      direct = _px2hd.direct,
      side = _px2hd.side,
      autoAdjust = _px2hd.autoAdjust,
      canvasWidth = _px2hd.canvasWidth,
      canvasHeight = _px2hd.canvasHeight,
      guideBBox = _px2hd.guideBBox,
      background = _px2hd.background,
      textStyle = _px2hd.textStyle,
      triggerRef = _px2hd.triggerRef;
    var _ref = points[0] || {},
      x = _ref.x,
      y = _ref.y;
    var _ref2 = guideBBox || {},
      guideWidth = _ref2.width,
      guideHeight = _ref2.height;
    var offsetXNum = context.px2hd(offsetX);
    var offsetYNum = context.px2hd(offsetY);
    var posX = x + (offsetXNum || 0);
    var posY = y + (offsetYNum || 0);
    var _getDirect = function _getDirect(point) {
      var newDirect = direct;
      var x = point.x,
        y = point.y;
      var vertical = newDirect[0];
      var horizontal = newDirect[1];
      // adjust for vertical direction
      if (vertical === 't' && y - side - guideHeight < 0) {
        vertical = 'b';
      } else if (vertical === 'b' && y + side + guideHeight > canvasHeight) {
        vertical = 't';
      }
      // adjust for horizontal direction
      var diff = vertical === 'c' ? side : 0;
      if (horizontal === 'l' && x - diff - guideWidth < 0) {
        horizontal = 'r';
      } else if (horizontal === 'r' && x + diff + guideWidth > canvasWidth) {
        horizontal = 'l';
      } else if (horizontal === 'c') {
        if (guideWidth / 2 + x + diff > canvasWidth) {
          horizontal = 'l';
        } else if (x - guideWidth / 2 - diff < 0) {
          horizontal = 'r';
        }
      }
      newDirect = vertical + horizontal;
      return newDirect;
    };
    var _getArrowPoints = function _getArrowPoints(direct) {
      var arrowPoints = [];
      if (direct === 'tl') {
        arrowPoints = [{
          x: posX,
          y: posY - side - 1
        }, {
          x: posX,
          y: posY
        }, {
          x: posX - side,
          y: posY - side - 1
        }];
        posX -= guideWidth / 2 || 0;
        posY = posY - side;
      } else if (direct === 'cl') {
        arrowPoints = [{
          x: posX - side - 1,
          y: posY - side
        }, {
          x: posX - side - 1,
          y: posY + side
        }, {
          x: posX,
          y: posY
        }];
        posX = posX - (guideWidth / 2 || 0) - side;
        posY += (guideHeight / 2 || 0) - 1;
      } else if (direct === 'bl') {
        arrowPoints = [{
          x: posX,
          y: posY
        }, {
          x: posX,
          y: posY + side + 1
        }, {
          x: posX - side,
          y: posY + side + 1
        }];
        posX = posX - (guideWidth / 2 || 0);
        posY += (guideHeight || 0) + side - 1;
      } else if (direct === 'bc') {
        // 有问题
        arrowPoints = [{
          x: posX,
          y: posY
        }, {
          x: posX - side,
          y: posY + side + 1
        }, {
          x: posX + side,
          y: posY + side + 1
        }];
        posY += (guideHeight || 0) + side - 1;
      } else if (direct === 'br') {
        arrowPoints = [{
          x: posX,
          y: posY
        }, {
          x: posX,
          y: posY + side + 1
        }, {
          x: posX + side,
          y: posY + side + 1
        }];
        posX += guideWidth / 2 || 0;
        posY += (guideHeight || 0) + side - 1;
      } else if (direct === 'cr') {
        arrowPoints = [{
          x: posX,
          y: posY
        }, {
          x: posX + side,
          y: posY - side
        }, {
          x: posX + side,
          y: posY + side
        }];
        posX += (guideWidth / 2 || 0) + side;
        posY += guideHeight / 2 || 0;
      } else if (direct === 'tr') {
        arrowPoints = [{
          x: posX,
          y: posY
        }, {
          x: posX,
          y: posY - side - 1
        }, {
          x: posX + side,
          y: posY - side - 1
        }];
        posX += guideWidth / 2 || 0;
        posY = posY - side;
      } else if (direct === 'tc') {
        arrowPoints = [{
          x: guideWidth / 2,
          y: guideHeight + side
        }, {
          x: guideWidth / 2 - side,
          y: guideHeight - 1
        }, {
          x: guideWidth / 2 + side,
          y: guideHeight - 1
        }];
        posY = posY - guideHeight - side;
      }
      return arrowPoints;
    };
    var dr = autoAdjust ? _getDirect(points[0]) : direct;
    var arrowPoints = _getArrowPoints(dr);
    return jsx("group", {
      style: _objectSpread({
        x: posX,
        y: posY,
        fill: defaultStyle.container.fill,
        radius: defaultStyle.container.radius,
        padding: defaultStyle.container.padding
      }, background),
      ref: triggerRef
    }, jsx("rect", {
      style: _objectSpread({
        display: 'flex',
        fill: defaultStyle.container.fill,
        padding: defaultStyle.container.padding,
        radius: defaultStyle.container.radius
      }, background)
    }, jsx("text", {
      style: _objectSpread({
        text: content,
        fontSize: defaultStyle.text.fontSize,
        fill: defaultStyle.text.fill
      }, textStyle)
    })), guideBBox && jsx("polygon", {
      style: {
        points: arrowPoints.map(function (d) {
          return [d.x, d.y];
        }),
        fill: (background === null || background === void 0 ? void 0 : background.fill) || defaultStyle.arrow.fill
      }
    }));
  });

  function _classCallCheck$2(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  function _defineProperties$3(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }
  function _createClass$4(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties$3(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties$3(Constructor, staticProps);
    Object.defineProperty(Constructor, "prototype", {
      writable: false
    });
    return Constructor;
  }

  function _setPrototypeOf$a(o, p) {
    _setPrototypeOf$a = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$a(o, p);
  }

  function _inherits$2(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        writable: true,
        configurable: true
      }
    });
    Object.defineProperty(subClass, "prototype", {
      writable: false
    });
    if (superClass) _setPrototypeOf$a(subClass, superClass);
  }

  function _getPrototypeOf$2(o) {
    _getPrototypeOf$2 = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) {
      return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf$2(o);
  }

  function _isNativeReflectConstruct$1() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
      Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
      return true;
    } catch (e) {
      return false;
    }
  }

  function _typeof$2(obj) {
    "@babel/helpers - typeof";

    return _typeof$2 = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) {
      return typeof obj;
    } : function (obj) {
      return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    }, _typeof$2(obj);
  }

  function _assertThisInitialized$4(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
  }

  function _possibleConstructorReturn$2(self, call) {
    if (call && (_typeof$2(call) === "object" || typeof call === "function")) {
      return call;
    } else if (call !== void 0) {
      throw new TypeError("Derived constructors may only return object or undefined");
    }
    return _assertThisInitialized$4(self);
  }

  function _createSuper$2(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct$1();
    return function _createSuperInternal() {
      var Super = _getPrototypeOf$2(Derived),
        result;
      if (hasNativeReflectConstruct) {
        var NewTarget = _getPrototypeOf$2(this).constructor;
        result = Reflect.construct(Super, arguments, NewTarget);
      } else {
        result = Super.apply(this, arguments);
      }
      return _possibleConstructorReturn$2(this, result);
    };
  }

  function quickselect$1(arr, k, left, right, compare) {
      quickselectStep$1(arr, k, left || 0, right || (arr.length - 1), compare || defaultCompare$1);
  }

  function quickselectStep$1(arr, k, left, right, compare) {

      while (right > left) {
          if (right - left > 600) {
              var n = right - left + 1;
              var m = k - left + 1;
              var z = Math.log(n);
              var s = 0.5 * Math.exp(2 * z / 3);
              var sd = 0.5 * Math.sqrt(z * s * (n - s) / n) * (m - n / 2 < 0 ? -1 : 1);
              var newLeft = Math.max(left, Math.floor(k - m * s / n + sd));
              var newRight = Math.min(right, Math.floor(k + (n - m) * s / n + sd));
              quickselectStep$1(arr, k, newLeft, newRight, compare);
          }

          var t = arr[k];
          var i = left;
          var j = right;

          swap$1(arr, left, k);
          if (compare(arr[right], t) > 0) swap$1(arr, left, right);

          while (i < j) {
              swap$1(arr, i, j);
              i++;
              j--;
              while (compare(arr[i], t) < 0) i++;
              while (compare(arr[j], t) > 0) j--;
          }

          if (compare(arr[left], t) === 0) swap$1(arr, left, j);
          else {
              j++;
              swap$1(arr, j, right);
          }

          if (j <= k) left = j + 1;
          if (k <= j) right = j - 1;
      }
  }

  function swap$1(arr, i, j) {
      var tmp = arr[i];
      arr[i] = arr[j];
      arr[j] = tmp;
  }

  function defaultCompare$1(a, b) {
      return a < b ? -1 : a > b ? 1 : 0;
  }

  class RBush$1 {
      constructor(maxEntries = 9) {
          // max entries in a node is 9 by default; min node fill is 40% for best performance
          this._maxEntries = Math.max(4, maxEntries);
          this._minEntries = Math.max(2, Math.ceil(this._maxEntries * 0.4));
          this.clear();
      }

      all() {
          return this._all(this.data, []);
      }

      search(bbox) {
          let node = this.data;
          const result = [];

          if (!intersects$1(bbox, node)) return result;

          const toBBox = this.toBBox;
          const nodesToSearch = [];

          while (node) {
              for (let i = 0; i < node.children.length; i++) {
                  const child = node.children[i];
                  const childBBox = node.leaf ? toBBox(child) : child;

                  if (intersects$1(bbox, childBBox)) {
                      if (node.leaf) result.push(child);
                      else if (contains$1(bbox, childBBox)) this._all(child, result);
                      else nodesToSearch.push(child);
                  }
              }
              node = nodesToSearch.pop();
          }

          return result;
      }

      collides(bbox) {
          let node = this.data;

          if (!intersects$1(bbox, node)) return false;

          const nodesToSearch = [];
          while (node) {
              for (let i = 0; i < node.children.length; i++) {
                  const child = node.children[i];
                  const childBBox = node.leaf ? this.toBBox(child) : child;

                  if (intersects$1(bbox, childBBox)) {
                      if (node.leaf || contains$1(bbox, childBBox)) return true;
                      nodesToSearch.push(child);
                  }
              }
              node = nodesToSearch.pop();
          }

          return false;
      }

      load(data) {
          if (!(data && data.length)) return this;

          if (data.length < this._minEntries) {
              for (let i = 0; i < data.length; i++) {
                  this.insert(data[i]);
              }
              return this;
          }

          // recursively build the tree with the given data from scratch using OMT algorithm
          let node = this._build(data.slice(), 0, data.length - 1, 0);

          if (!this.data.children.length) {
              // save as is if tree is empty
              this.data = node;

          } else if (this.data.height === node.height) {
              // split root if trees have the same height
              this._splitRoot(this.data, node);

          } else {
              if (this.data.height < node.height) {
                  // swap trees if inserted one is bigger
                  const tmpNode = this.data;
                  this.data = node;
                  node = tmpNode;
              }

              // insert the small tree into the large tree at appropriate level
              this._insert(node, this.data.height - node.height - 1, true);
          }

          return this;
      }

      insert(item) {
          if (item) this._insert(item, this.data.height - 1);
          return this;
      }

      clear() {
          this.data = createNode$1([]);
          return this;
      }

      remove(item, equalsFn) {
          if (!item) return this;

          let node = this.data;
          const bbox = this.toBBox(item);
          const path = [];
          const indexes = [];
          let i, parent, goingUp;

          // depth-first iterative tree traversal
          while (node || path.length) {

              if (!node) { // go up
                  node = path.pop();
                  parent = path[path.length - 1];
                  i = indexes.pop();
                  goingUp = true;
              }

              if (node.leaf) { // check current node
                  const index = findItem$1(item, node.children, equalsFn);

                  if (index !== -1) {
                      // item found, remove the item and condense tree upwards
                      node.children.splice(index, 1);
                      path.push(node);
                      this._condense(path);
                      return this;
                  }
              }

              if (!goingUp && !node.leaf && contains$1(node, bbox)) { // go down
                  path.push(node);
                  indexes.push(i);
                  i = 0;
                  parent = node;
                  node = node.children[0];

              } else if (parent) { // go right
                  i++;
                  node = parent.children[i];
                  goingUp = false;

              } else node = null; // nothing found
          }

          return this;
      }

      toBBox(item) { return item; }

      compareMinX(a, b) { return a.minX - b.minX; }
      compareMinY(a, b) { return a.minY - b.minY; }

      toJSON() { return this.data; }

      fromJSON(data) {
          this.data = data;
          return this;
      }

      _all(node, result) {
          const nodesToSearch = [];
          while (node) {
              if (node.leaf) result.push(...node.children);
              else nodesToSearch.push(...node.children);

              node = nodesToSearch.pop();
          }
          return result;
      }

      _build(items, left, right, height) {

          const N = right - left + 1;
          let M = this._maxEntries;
          let node;

          if (N <= M) {
              // reached leaf level; return leaf
              node = createNode$1(items.slice(left, right + 1));
              calcBBox$1(node, this.toBBox);
              return node;
          }

          if (!height) {
              // target height of the bulk-loaded tree
              height = Math.ceil(Math.log(N) / Math.log(M));

              // target number of root entries to maximize storage utilization
              M = Math.ceil(N / Math.pow(M, height - 1));
          }

          node = createNode$1([]);
          node.leaf = false;
          node.height = height;

          // split the items into M mostly square tiles

          const N2 = Math.ceil(N / M);
          const N1 = N2 * Math.ceil(Math.sqrt(M));

          multiSelect$1(items, left, right, N1, this.compareMinX);

          for (let i = left; i <= right; i += N1) {

              const right2 = Math.min(i + N1 - 1, right);

              multiSelect$1(items, i, right2, N2, this.compareMinY);

              for (let j = i; j <= right2; j += N2) {

                  const right3 = Math.min(j + N2 - 1, right2);

                  // pack each entry recursively
                  node.children.push(this._build(items, j, right3, height - 1));
              }
          }

          calcBBox$1(node, this.toBBox);

          return node;
      }

      _chooseSubtree(bbox, node, level, path) {
          while (true) {
              path.push(node);

              if (node.leaf || path.length - 1 === level) break;

              let minArea = Infinity;
              let minEnlargement = Infinity;
              let targetNode;

              for (let i = 0; i < node.children.length; i++) {
                  const child = node.children[i];
                  const area = bboxArea$1(child);
                  const enlargement = enlargedArea$1(bbox, child) - area;

                  // choose entry with the least area enlargement
                  if (enlargement < minEnlargement) {
                      minEnlargement = enlargement;
                      minArea = area < minArea ? area : minArea;
                      targetNode = child;

                  } else if (enlargement === minEnlargement) {
                      // otherwise choose one with the smallest area
                      if (area < minArea) {
                          minArea = area;
                          targetNode = child;
                      }
                  }
              }

              node = targetNode || node.children[0];
          }

          return node;
      }

      _insert(item, level, isNode) {
          const bbox = isNode ? item : this.toBBox(item);
          const insertPath = [];

          // find the best node for accommodating the item, saving all nodes along the path too
          const node = this._chooseSubtree(bbox, this.data, level, insertPath);

          // put the item into the node
          node.children.push(item);
          extend$3(node, bbox);

          // split on node overflow; propagate upwards if necessary
          while (level >= 0) {
              if (insertPath[level].children.length > this._maxEntries) {
                  this._split(insertPath, level);
                  level--;
              } else break;
          }

          // adjust bboxes along the insertion path
          this._adjustParentBBoxes(bbox, insertPath, level);
      }

      // split overflowed node into two
      _split(insertPath, level) {
          const node = insertPath[level];
          const M = node.children.length;
          const m = this._minEntries;

          this._chooseSplitAxis(node, m, M);

          const splitIndex = this._chooseSplitIndex(node, m, M);

          const newNode = createNode$1(node.children.splice(splitIndex, node.children.length - splitIndex));
          newNode.height = node.height;
          newNode.leaf = node.leaf;

          calcBBox$1(node, this.toBBox);
          calcBBox$1(newNode, this.toBBox);

          if (level) insertPath[level - 1].children.push(newNode);
          else this._splitRoot(node, newNode);
      }

      _splitRoot(node, newNode) {
          // split root node
          this.data = createNode$1([node, newNode]);
          this.data.height = node.height + 1;
          this.data.leaf = false;
          calcBBox$1(this.data, this.toBBox);
      }

      _chooseSplitIndex(node, m, M) {
          let index;
          let minOverlap = Infinity;
          let minArea = Infinity;

          for (let i = m; i <= M - m; i++) {
              const bbox1 = distBBox$1(node, 0, i, this.toBBox);
              const bbox2 = distBBox$1(node, i, M, this.toBBox);

              const overlap = intersectionArea$1(bbox1, bbox2);
              const area = bboxArea$1(bbox1) + bboxArea$1(bbox2);

              // choose distribution with minimum overlap
              if (overlap < minOverlap) {
                  minOverlap = overlap;
                  index = i;

                  minArea = area < minArea ? area : minArea;

              } else if (overlap === minOverlap) {
                  // otherwise choose distribution with minimum area
                  if (area < minArea) {
                      minArea = area;
                      index = i;
                  }
              }
          }

          return index || M - m;
      }

      // sorts node children by the best axis for split
      _chooseSplitAxis(node, m, M) {
          const compareMinX = node.leaf ? this.compareMinX : compareNodeMinX$1;
          const compareMinY = node.leaf ? this.compareMinY : compareNodeMinY$1;
          const xMargin = this._allDistMargin(node, m, M, compareMinX);
          const yMargin = this._allDistMargin(node, m, M, compareMinY);

          // if total distributions margin value is minimal for x, sort by minX,
          // otherwise it's already sorted by minY
          if (xMargin < yMargin) node.children.sort(compareMinX);
      }

      // total margin of all possible split distributions where each node is at least m full
      _allDistMargin(node, m, M, compare) {
          node.children.sort(compare);

          const toBBox = this.toBBox;
          const leftBBox = distBBox$1(node, 0, m, toBBox);
          const rightBBox = distBBox$1(node, M - m, M, toBBox);
          let margin = bboxMargin$1(leftBBox) + bboxMargin$1(rightBBox);

          for (let i = m; i < M - m; i++) {
              const child = node.children[i];
              extend$3(leftBBox, node.leaf ? toBBox(child) : child);
              margin += bboxMargin$1(leftBBox);
          }

          for (let i = M - m - 1; i >= m; i--) {
              const child = node.children[i];
              extend$3(rightBBox, node.leaf ? toBBox(child) : child);
              margin += bboxMargin$1(rightBBox);
          }

          return margin;
      }

      _adjustParentBBoxes(bbox, path, level) {
          // adjust bboxes along the given tree path
          for (let i = level; i >= 0; i--) {
              extend$3(path[i], bbox);
          }
      }

      _condense(path) {
          // go through the path, removing empty nodes and updating bboxes
          for (let i = path.length - 1, siblings; i >= 0; i--) {
              if (path[i].children.length === 0) {
                  if (i > 0) {
                      siblings = path[i - 1].children;
                      siblings.splice(siblings.indexOf(path[i]), 1);

                  } else this.clear();

              } else calcBBox$1(path[i], this.toBBox);
          }
      }
  }

  function findItem$1(item, items, equalsFn) {
      if (!equalsFn) return items.indexOf(item);

      for (let i = 0; i < items.length; i++) {
          if (equalsFn(item, items[i])) return i;
      }
      return -1;
  }

  // calculate node's bbox from bboxes of its children
  function calcBBox$1(node, toBBox) {
      distBBox$1(node, 0, node.children.length, toBBox, node);
  }

  // min bounding rectangle of node children from k to p-1
  function distBBox$1(node, k, p, toBBox, destNode) {
      if (!destNode) destNode = createNode$1(null);
      destNode.minX = Infinity;
      destNode.minY = Infinity;
      destNode.maxX = -Infinity;
      destNode.maxY = -Infinity;

      for (let i = k; i < p; i++) {
          const child = node.children[i];
          extend$3(destNode, node.leaf ? toBBox(child) : child);
      }

      return destNode;
  }

  function extend$3(a, b) {
      a.minX = Math.min(a.minX, b.minX);
      a.minY = Math.min(a.minY, b.minY);
      a.maxX = Math.max(a.maxX, b.maxX);
      a.maxY = Math.max(a.maxY, b.maxY);
      return a;
  }

  function compareNodeMinX$1(a, b) { return a.minX - b.minX; }
  function compareNodeMinY$1(a, b) { return a.minY - b.minY; }

  function bboxArea$1(a)   { return (a.maxX - a.minX) * (a.maxY - a.minY); }
  function bboxMargin$1(a) { return (a.maxX - a.minX) + (a.maxY - a.minY); }

  function enlargedArea$1(a, b) {
      return (Math.max(b.maxX, a.maxX) - Math.min(b.minX, a.minX)) *
             (Math.max(b.maxY, a.maxY) - Math.min(b.minY, a.minY));
  }

  function intersectionArea$1(a, b) {
      const minX = Math.max(a.minX, b.minX);
      const minY = Math.max(a.minY, b.minY);
      const maxX = Math.min(a.maxX, b.maxX);
      const maxY = Math.min(a.maxY, b.maxY);

      return Math.max(0, maxX - minX) *
             Math.max(0, maxY - minY);
  }

  function contains$1(a, b) {
      return a.minX <= b.minX &&
             a.minY <= b.minY &&
             b.maxX <= a.maxX &&
             b.maxY <= a.maxY;
  }

  function intersects$1(a, b) {
      return b.minX <= a.maxX &&
             b.minY <= a.maxY &&
             b.maxX >= a.minX &&
             b.maxY >= a.minY;
  }

  function createNode$1(children) {
      return {
          children,
          height: 1,
          leaf: true,
          minX: Infinity,
          minY: Infinity,
          maxX: -Infinity,
          maxY: -Infinity
      };
  }

  // sort an array so that items come in groups of n unsorted items, with groups sorted between each other;
  // combines selection algorithm with binary divide & conquer approach

  function multiSelect$1(arr, left, right, n, compare) {
      const stack = [left, right];

      while (stack.length) {
          right = stack.pop();
          left = stack.pop();

          if (right - left <= n) continue;

          const mid = left + Math.ceil((right - left) / n / 2) * n;
          quickselect$1(arr, mid, left, right, compare);

          stack.push(left, mid, mid, right);
      }
  }

  function define$2(constructor, factory, prototype) {
    constructor.prototype = factory.prototype = prototype;
    prototype.constructor = constructor;
  }

  function extend$4(parent, definition) {
    var prototype = Object.create(parent.prototype);
    for (var key in definition) prototype[key] = definition[key];
    return prototype;
  }

  function Color$2() {}

  var darker$1 = 0.7;
  var brighter$1 = 1 / darker$1;

  var reI$2 = "\\s*([+-]?\\d+)\\s*",
      reN$2 = "\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*",
      reP$2 = "\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*",
      reHex$2 = /^#([0-9a-f]{3,8})$/,
      reRgbInteger$2 = new RegExp("^rgb\\(" + [reI$2, reI$2, reI$2] + "\\)$"),
      reRgbPercent$2 = new RegExp("^rgb\\(" + [reP$2, reP$2, reP$2] + "\\)$"),
      reRgbaInteger$2 = new RegExp("^rgba\\(" + [reI$2, reI$2, reI$2, reN$2] + "\\)$"),
      reRgbaPercent$2 = new RegExp("^rgba\\(" + [reP$2, reP$2, reP$2, reN$2] + "\\)$"),
      reHslPercent$2 = new RegExp("^hsl\\(" + [reN$2, reP$2, reP$2] + "\\)$"),
      reHslaPercent$2 = new RegExp("^hsla\\(" + [reN$2, reP$2, reP$2, reN$2] + "\\)$");

  var named$2 = {
    aliceblue: 0xf0f8ff,
    antiquewhite: 0xfaebd7,
    aqua: 0x00ffff,
    aquamarine: 0x7fffd4,
    azure: 0xf0ffff,
    beige: 0xf5f5dc,
    bisque: 0xffe4c4,
    black: 0x000000,
    blanchedalmond: 0xffebcd,
    blue: 0x0000ff,
    blueviolet: 0x8a2be2,
    brown: 0xa52a2a,
    burlywood: 0xdeb887,
    cadetblue: 0x5f9ea0,
    chartreuse: 0x7fff00,
    chocolate: 0xd2691e,
    coral: 0xff7f50,
    cornflowerblue: 0x6495ed,
    cornsilk: 0xfff8dc,
    crimson: 0xdc143c,
    cyan: 0x00ffff,
    darkblue: 0x00008b,
    darkcyan: 0x008b8b,
    darkgoldenrod: 0xb8860b,
    darkgray: 0xa9a9a9,
    darkgreen: 0x006400,
    darkgrey: 0xa9a9a9,
    darkkhaki: 0xbdb76b,
    darkmagenta: 0x8b008b,
    darkolivegreen: 0x556b2f,
    darkorange: 0xff8c00,
    darkorchid: 0x9932cc,
    darkred: 0x8b0000,
    darksalmon: 0xe9967a,
    darkseagreen: 0x8fbc8f,
    darkslateblue: 0x483d8b,
    darkslategray: 0x2f4f4f,
    darkslategrey: 0x2f4f4f,
    darkturquoise: 0x00ced1,
    darkviolet: 0x9400d3,
    deeppink: 0xff1493,
    deepskyblue: 0x00bfff,
    dimgray: 0x696969,
    dimgrey: 0x696969,
    dodgerblue: 0x1e90ff,
    firebrick: 0xb22222,
    floralwhite: 0xfffaf0,
    forestgreen: 0x228b22,
    fuchsia: 0xff00ff,
    gainsboro: 0xdcdcdc,
    ghostwhite: 0xf8f8ff,
    gold: 0xffd700,
    goldenrod: 0xdaa520,
    gray: 0x808080,
    green: 0x008000,
    greenyellow: 0xadff2f,
    grey: 0x808080,
    honeydew: 0xf0fff0,
    hotpink: 0xff69b4,
    indianred: 0xcd5c5c,
    indigo: 0x4b0082,
    ivory: 0xfffff0,
    khaki: 0xf0e68c,
    lavender: 0xe6e6fa,
    lavenderblush: 0xfff0f5,
    lawngreen: 0x7cfc00,
    lemonchiffon: 0xfffacd,
    lightblue: 0xadd8e6,
    lightcoral: 0xf08080,
    lightcyan: 0xe0ffff,
    lightgoldenrodyellow: 0xfafad2,
    lightgray: 0xd3d3d3,
    lightgreen: 0x90ee90,
    lightgrey: 0xd3d3d3,
    lightpink: 0xffb6c1,
    lightsalmon: 0xffa07a,
    lightseagreen: 0x20b2aa,
    lightskyblue: 0x87cefa,
    lightslategray: 0x778899,
    lightslategrey: 0x778899,
    lightsteelblue: 0xb0c4de,
    lightyellow: 0xffffe0,
    lime: 0x00ff00,
    limegreen: 0x32cd32,
    linen: 0xfaf0e6,
    magenta: 0xff00ff,
    maroon: 0x800000,
    mediumaquamarine: 0x66cdaa,
    mediumblue: 0x0000cd,
    mediumorchid: 0xba55d3,
    mediumpurple: 0x9370db,
    mediumseagreen: 0x3cb371,
    mediumslateblue: 0x7b68ee,
    mediumspringgreen: 0x00fa9a,
    mediumturquoise: 0x48d1cc,
    mediumvioletred: 0xc71585,
    midnightblue: 0x191970,
    mintcream: 0xf5fffa,
    mistyrose: 0xffe4e1,
    moccasin: 0xffe4b5,
    navajowhite: 0xffdead,
    navy: 0x000080,
    oldlace: 0xfdf5e6,
    olive: 0x808000,
    olivedrab: 0x6b8e23,
    orange: 0xffa500,
    orangered: 0xff4500,
    orchid: 0xda70d6,
    palegoldenrod: 0xeee8aa,
    palegreen: 0x98fb98,
    paleturquoise: 0xafeeee,
    palevioletred: 0xdb7093,
    papayawhip: 0xffefd5,
    peachpuff: 0xffdab9,
    peru: 0xcd853f,
    pink: 0xffc0cb,
    plum: 0xdda0dd,
    powderblue: 0xb0e0e6,
    purple: 0x800080,
    rebeccapurple: 0x663399,
    red: 0xff0000,
    rosybrown: 0xbc8f8f,
    royalblue: 0x4169e1,
    saddlebrown: 0x8b4513,
    salmon: 0xfa8072,
    sandybrown: 0xf4a460,
    seagreen: 0x2e8b57,
    seashell: 0xfff5ee,
    sienna: 0xa0522d,
    silver: 0xc0c0c0,
    skyblue: 0x87ceeb,
    slateblue: 0x6a5acd,
    slategray: 0x708090,
    slategrey: 0x708090,
    snow: 0xfffafa,
    springgreen: 0x00ff7f,
    steelblue: 0x4682b4,
    tan: 0xd2b48c,
    teal: 0x008080,
    thistle: 0xd8bfd8,
    tomato: 0xff6347,
    turquoise: 0x40e0d0,
    violet: 0xee82ee,
    wheat: 0xf5deb3,
    white: 0xffffff,
    whitesmoke: 0xf5f5f5,
    yellow: 0xffff00,
    yellowgreen: 0x9acd32
  };

  define$2(Color$2, color$2, {
    copy: function(channels) {
      return Object.assign(new this.constructor, this, channels);
    },
    displayable: function() {
      return this.rgb().displayable();
    },
    hex: color_formatHex$2, // Deprecated! Use color.formatHex.
    formatHex: color_formatHex$2,
    formatHsl: color_formatHsl$2,
    formatRgb: color_formatRgb$2,
    toString: color_formatRgb$2
  });

  function color_formatHex$2() {
    return this.rgb().formatHex();
  }

  function color_formatHsl$2() {
    return hslConvert$2(this).formatHsl();
  }

  function color_formatRgb$2() {
    return this.rgb().formatRgb();
  }

  function color$2(format) {
    var m, l;
    format = (format + "").trim().toLowerCase();
    return (m = reHex$2.exec(format)) ? (l = m[1].length, m = parseInt(m[1], 16), l === 6 ? rgbn$2(m) // #ff0000
        : l === 3 ? new Rgb$2((m >> 8 & 0xf) | (m >> 4 & 0xf0), (m >> 4 & 0xf) | (m & 0xf0), ((m & 0xf) << 4) | (m & 0xf), 1) // #f00
        : l === 8 ? rgba$2(m >> 24 & 0xff, m >> 16 & 0xff, m >> 8 & 0xff, (m & 0xff) / 0xff) // #ff000000
        : l === 4 ? rgba$2((m >> 12 & 0xf) | (m >> 8 & 0xf0), (m >> 8 & 0xf) | (m >> 4 & 0xf0), (m >> 4 & 0xf) | (m & 0xf0), (((m & 0xf) << 4) | (m & 0xf)) / 0xff) // #f000
        : null) // invalid hex
        : (m = reRgbInteger$2.exec(format)) ? new Rgb$2(m[1], m[2], m[3], 1) // rgb(255, 0, 0)
        : (m = reRgbPercent$2.exec(format)) ? new Rgb$2(m[1] * 255 / 100, m[2] * 255 / 100, m[3] * 255 / 100, 1) // rgb(100%, 0%, 0%)
        : (m = reRgbaInteger$2.exec(format)) ? rgba$2(m[1], m[2], m[3], m[4]) // rgba(255, 0, 0, 1)
        : (m = reRgbaPercent$2.exec(format)) ? rgba$2(m[1] * 255 / 100, m[2] * 255 / 100, m[3] * 255 / 100, m[4]) // rgb(100%, 0%, 0%, 1)
        : (m = reHslPercent$2.exec(format)) ? hsla$2(m[1], m[2] / 100, m[3] / 100, 1) // hsl(120, 50%, 50%)
        : (m = reHslaPercent$2.exec(format)) ? hsla$2(m[1], m[2] / 100, m[3] / 100, m[4]) // hsla(120, 50%, 50%, 1)
        : named$2.hasOwnProperty(format) ? rgbn$2(named$2[format]) // eslint-disable-line no-prototype-builtins
        : format === "transparent" ? new Rgb$2(NaN, NaN, NaN, 0)
        : null;
  }

  function rgbn$2(n) {
    return new Rgb$2(n >> 16 & 0xff, n >> 8 & 0xff, n & 0xff, 1);
  }

  function rgba$2(r, g, b, a) {
    if (a <= 0) r = g = b = NaN;
    return new Rgb$2(r, g, b, a);
  }

  function rgbConvert$2(o) {
    if (!(o instanceof Color$2)) o = color$2(o);
    if (!o) return new Rgb$2;
    o = o.rgb();
    return new Rgb$2(o.r, o.g, o.b, o.opacity);
  }

  function rgb$3(r, g, b, opacity) {
    return arguments.length === 1 ? rgbConvert$2(r) : new Rgb$2(r, g, b, opacity == null ? 1 : opacity);
  }

  function Rgb$2(r, g, b, opacity) {
    this.r = +r;
    this.g = +g;
    this.b = +b;
    this.opacity = +opacity;
  }

  define$2(Rgb$2, rgb$3, extend$4(Color$2, {
    brighter: function(k) {
      k = k == null ? brighter$1 : Math.pow(brighter$1, k);
      return new Rgb$2(this.r * k, this.g * k, this.b * k, this.opacity);
    },
    darker: function(k) {
      k = k == null ? darker$1 : Math.pow(darker$1, k);
      return new Rgb$2(this.r * k, this.g * k, this.b * k, this.opacity);
    },
    rgb: function() {
      return this;
    },
    displayable: function() {
      return (-0.5 <= this.r && this.r < 255.5)
          && (-0.5 <= this.g && this.g < 255.5)
          && (-0.5 <= this.b && this.b < 255.5)
          && (0 <= this.opacity && this.opacity <= 1);
    },
    hex: rgb_formatHex$2, // Deprecated! Use color.formatHex.
    formatHex: rgb_formatHex$2,
    formatRgb: rgb_formatRgb$2,
    toString: rgb_formatRgb$2
  }));

  function rgb_formatHex$2() {
    return "#" + hex$2(this.r) + hex$2(this.g) + hex$2(this.b);
  }

  function rgb_formatRgb$2() {
    var a = this.opacity; a = isNaN(a) ? 1 : Math.max(0, Math.min(1, a));
    return (a === 1 ? "rgb(" : "rgba(")
        + Math.max(0, Math.min(255, Math.round(this.r) || 0)) + ", "
        + Math.max(0, Math.min(255, Math.round(this.g) || 0)) + ", "
        + Math.max(0, Math.min(255, Math.round(this.b) || 0))
        + (a === 1 ? ")" : ", " + a + ")");
  }

  function hex$2(value) {
    value = Math.max(0, Math.min(255, Math.round(value) || 0));
    return (value < 16 ? "0" : "") + value.toString(16);
  }

  function hsla$2(h, s, l, a) {
    if (a <= 0) h = s = l = NaN;
    else if (l <= 0 || l >= 1) h = s = NaN;
    else if (s <= 0) h = NaN;
    return new Hsl$2(h, s, l, a);
  }

  function hslConvert$2(o) {
    if (o instanceof Hsl$2) return new Hsl$2(o.h, o.s, o.l, o.opacity);
    if (!(o instanceof Color$2)) o = color$2(o);
    if (!o) return new Hsl$2;
    if (o instanceof Hsl$2) return o;
    o = o.rgb();
    var r = o.r / 255,
        g = o.g / 255,
        b = o.b / 255,
        min = Math.min(r, g, b),
        max = Math.max(r, g, b),
        h = NaN,
        s = max - min,
        l = (max + min) / 2;
    if (s) {
      if (r === max) h = (g - b) / s + (g < b) * 6;
      else if (g === max) h = (b - r) / s + 2;
      else h = (r - g) / s + 4;
      s /= l < 0.5 ? max + min : 2 - max - min;
      h *= 60;
    } else {
      s = l > 0 && l < 1 ? 0 : h;
    }
    return new Hsl$2(h, s, l, o.opacity);
  }

  function hsl$2(h, s, l, opacity) {
    return arguments.length === 1 ? hslConvert$2(h) : new Hsl$2(h, s, l, opacity == null ? 1 : opacity);
  }

  function Hsl$2(h, s, l, opacity) {
    this.h = +h;
    this.s = +s;
    this.l = +l;
    this.opacity = +opacity;
  }

  define$2(Hsl$2, hsl$2, extend$4(Color$2, {
    brighter: function(k) {
      k = k == null ? brighter$1 : Math.pow(brighter$1, k);
      return new Hsl$2(this.h, this.s, this.l * k, this.opacity);
    },
    darker: function(k) {
      k = k == null ? darker$1 : Math.pow(darker$1, k);
      return new Hsl$2(this.h, this.s, this.l * k, this.opacity);
    },
    rgb: function() {
      var h = this.h % 360 + (this.h < 0) * 360,
          s = isNaN(h) || isNaN(this.s) ? 0 : this.s,
          l = this.l,
          m2 = l + (l < 0.5 ? l : 1 - l) * s,
          m1 = 2 * l - m2;
      return new Rgb$2(
        hsl2rgb$2(h >= 240 ? h - 240 : h + 120, m1, m2),
        hsl2rgb$2(h, m1, m2),
        hsl2rgb$2(h < 120 ? h + 240 : h - 120, m1, m2),
        this.opacity
      );
    },
    displayable: function() {
      return (0 <= this.s && this.s <= 1 || isNaN(this.s))
          && (0 <= this.l && this.l <= 1)
          && (0 <= this.opacity && this.opacity <= 1);
    },
    formatHsl: function() {
      var a = this.opacity; a = isNaN(a) ? 1 : Math.max(0, Math.min(1, a));
      return (a === 1 ? "hsl(" : "hsla(")
          + (this.h || 0) + ", "
          + (this.s || 0) * 100 + "%, "
          + (this.l || 0) * 100 + "%"
          + (a === 1 ? ")" : ", " + a + ")");
    }
  }));

  /* From FvD 13.37, CSS Color Module Level 3 */
  function hsl2rgb$2(h, m1, m2) {
    return (h < 60 ? m1 + (m2 - m1) * h / 60
        : h < 180 ? m2
        : h < 240 ? m1 + (m2 - m1) * (240 - h) / 60
        : m1) * 255;
  }

  /**
   * 两点之间的距离
   * @param {number} x1 起始点 x
   * @param {number} y1 起始点 y
   * @param {number} x2 结束点 x
   * @param {number} y2 结束点 y
   * @return {number} 距离
   */
  function distance$3(x1, y1, x2, y2) {
    var dx = x1 - x2;
    var dy = y1 - y2;
    return Math.sqrt(dx * dx + dy * dy);
  }
  function isNumberEqual$4(v1, v2) {
    return Math.abs(v1 - v2) < 0.001;
  }
  function getBBoxByArray$2(xArr, yArr) {
    var minX = Math.min.apply(Math, xArr);
    var minY = Math.min.apply(Math, yArr);
    var maxX = Math.max.apply(Math, xArr);
    var maxY = Math.max.apply(Math, yArr);
    return {
      x: minX,
      y: minY,
      width: maxX - minX,
      height: maxY - minY
    };
  }
  function piMod$2(angle) {
    return (angle + Math.PI * 2) % (Math.PI * 2);
  }

  var line$3 = {
    /**
     * 计算线段的包围盒
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {object} 包围盒对象
     */
    box: function box(x1, y1, x2, y2) {
      return getBBoxByArray$2([x1, x2], [y1, y2]);
    },
    /**
     * 线段的长度
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {number} 距离
     */
    length: function length(x1, y1, x2, y2) {
      return distance$3(x1, y1, x2, y2);
    },
    /**
     * 根据比例获取点
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} t 指定比例
     * @return {object} 包含 x, y 的点
     */
    pointAt: function pointAt(x1, y1, x2, y2, t) {
      return {
        x: (1 - t) * x1 + t * x2,
        y: (1 - t) * y1 + t * y2
      };
    },
    /**
     * 点到线段的距离
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} x  测试点 x
     * @param {number} y  测试点 y
     * @return {number} 距离
     */
    pointDistance: function pointDistance(x1, y1, x2, y2, x, y) {
      // 投影距离 x1, y1 的向量，假设 p, p1, p2 三个点，投影点为 a
      // p1a = p1p.p1p2/|p1p2| * (p1p 的单位向量)
      var cross = (x2 - x1) * (x - x1) + (y2 - y1) * (y - y1);
      if (cross < 0) {
        return distance$3(x1, y1, x, y);
      }
      var lengthSquare = (x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1);
      if (cross > lengthSquare) {
        return distance$3(x2, y2, x, y);
      }
      return this.pointToLine(x1, y1, x2, y2, x, y);
    },
    /**
     * 点到直线的距离，而不是点到线段的距离
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @param {number} x  测试点 x
     * @param {number} y  测试点 y
     * @return {number} 距离
     */
    pointToLine: function pointToLine(x1, y1, x2, y2, x, y) {
      var d = [x2 - x1, y2 - y1];
      // 如果端点相等，则判定点到点的距离
      if (exactEquals$3(d, [0, 0])) {
        return Math.sqrt((x - x1) * (x - x1) + (y - y1) * (y - y1));
      }
      var u = [-d[1], d[0]];
      normalize$8(u, u);
      var a = [x - x1, y - y1];
      return Math.abs(dot$4(a, u));
    },
    /**
     * 线段的角度
     * @param {number} x1 起始点 x
     * @param {number} y1 起始点 y
     * @param {number} x2 结束点 x
     * @param {number} y2 结束点 y
     * @return {number} 导数
     */
    tangentAngle: function tangentAngle(x1, y1, x2, y2) {
      return Math.atan2(y2 - y1, x2 - x1);
    }
  };

  var EPSILON$4 = 0.0001;
  /**
   * 使用牛顿切割法求最近的点
   * @param {number[]} xArr      点的 x 数组
   * @param {number[]} yArr      点的 y 数组
   * @param {number}   x         指定的点 x
   * @param {number}   y         指定的点 y
   * @param {Function} tCallback 差值函数
   */
  function nearestPoint$2(xArr, yArr, x, y, tCallback, length) {
    var t = -1;
    var d = Infinity;
    var v0 = [x, y];
    var segNum = 20;
    if (length && length > 200) {
      segNum = length / 10;
    }
    var increaseRate = 1 / segNum;
    var interval = increaseRate / 10;
    for (var i = 0; i <= segNum; i++) {
      var _t = i * increaseRate;
      var v1 = [tCallback.apply(void 0, xArr.concat([_t])), tCallback.apply(void 0, yArr.concat([_t]))];
      var d1 = distance$3(v0[0], v0[1], v1[0], v1[1]);
      if (d1 < d) {
        t = _t;
        d = d1;
      }
    }
    // 提前终止
    if (t === 0) {
      return {
        x: xArr[0],
        y: yArr[0]
      };
    }
    if (t === 1) {
      var count = xArr.length;
      return {
        x: xArr[count - 1],
        y: yArr[count - 1]
      };
    }
    d = Infinity;
    for (var _i = 0; _i < 32; _i++) {
      if (interval < EPSILON$4) {
        break;
      }
      var prev = t - interval;
      var next = t + interval;
      var _v = [tCallback.apply(void 0, xArr.concat([prev])), tCallback.apply(void 0, yArr.concat([prev]))];
      var _d = distance$3(v0[0], v0[1], _v[0], _v[1]);
      if (prev >= 0 && _d < d) {
        t = prev;
        d = _d;
      } else {
        var v2 = [tCallback.apply(void 0, xArr.concat([next])), tCallback.apply(void 0, yArr.concat([next]))];
        var d2 = distance$3(v0[0], v0[1], v2[0], v2[1]);
        if (next <= 1 && d2 < d) {
          t = next;
          d = d2;
        } else {
          interval *= 0.5;
        }
      }
    }
    return {
      x: tCallback.apply(void 0, xArr.concat([t])),
      y: tCallback.apply(void 0, yArr.concat([t]))
    };
  }
  // 近似求解 https://community.khronos.org/t/3d-cubic-bezier-segment-length/62363/2
  function snapLength$2(xArr, yArr) {
    var totalLength = 0;
    var count = xArr.length;
    for (var i = 0; i < count; i++) {
      var x = xArr[i];
      var y = yArr[i];
      var nextX = xArr[(i + 1) % count];
      var nextY = yArr[(i + 1) % count];
      totalLength += distance$3(x, y, nextX, nextY);
    }
    return totalLength / 2;
  }

  // 差值公式
  function quadraticAt$2(p0, p1, p2, t) {
    var onet = 1 - t;
    return onet * onet * p0 + 2 * t * onet * p1 + t * t * p2;
  }
  // 求极值
  function extrema$3(p0, p1, p2) {
    var a = p0 + p2 - 2 * p1;
    if (isNumberEqual$4(a, 0)) {
      return [0.5];
    }
    var rst = (p0 - p1) / a;
    if (rst <= 1 && rst >= 0) {
      return [rst];
    }
    return [];
  }
  function derivativeAt$3(p0, p1, p2, t) {
    return 2 * (1 - t) * (p1 - p0) + 2 * t * (p2 - p1);
  }
  // 分割贝塞尔曲线
  function divideQuadratic$2(x1, y1, x2, y2, x3, y3, t) {
    // 划分点
    var xt = quadraticAt$2(x1, x2, x3, t);
    var yt = quadraticAt$2(y1, y2, y3, t);
    // 分割的第一条曲线的控制点
    var controlPoint1 = line$3.pointAt(x1, y1, x2, y2, t);
    // 分割的第二条曲线的控制点
    var controlPoint2 = line$3.pointAt(x2, y2, x3, y3, t);
    return [[x1, y1, controlPoint1.x, controlPoint1.y, xt, yt], [xt, yt, controlPoint2.x, controlPoint2.y, x3, y3]];
  }
  // 使用迭代法取贝塞尔曲线的长度
  function quadraticLength$2(x1, y1, x2, y2, x3, y3, iterationCount) {
    if (iterationCount === 0) {
      return (distance$3(x1, y1, x2, y2) + distance$3(x2, y2, x3, y3) + distance$3(x1, y1, x3, y3)) / 2;
    }
    var quadratics = divideQuadratic$2(x1, y1, x2, y2, x3, y3, 0.5);
    var left = quadratics[0];
    var right = quadratics[1];
    left.push(iterationCount - 1);
    right.push(iterationCount - 1);
    return quadraticLength$2.apply(void 0, left) + quadraticLength$2.apply(void 0, right);
  }
  var quadratic$2 = {
    box: function box(x1, y1, x2, y2, x3, y3) {
      var xExtrema = extrema$3(x1, x2, x3)[0];
      var yExtrema = extrema$3(y1, y2, y3)[0];
      // 控制点不加入 box 的计算
      var xArr = [x1, x3];
      var yArr = [y1, y3];
      if (xExtrema !== undefined) {
        xArr.push(quadraticAt$2(x1, x2, x3, xExtrema));
      }
      if (yExtrema !== undefined) {
        yArr.push(quadraticAt$2(y1, y2, y3, yExtrema));
      }
      return getBBoxByArray$2(xArr, yArr);
    },
    length: function length(x1, y1, x2, y2, x3, y3) {
      return quadraticLength$2(x1, y1, x2, y2, x3, y3, 3);
    },
    nearestPoint: function nearestPoint$1(x1, y1, x2, y2, x3, y3, x0, y0) {
      return nearestPoint$2([x1, x2, x3], [y1, y2, y3], x0, y0, quadraticAt$2);
    },
    pointDistance: function pointDistance(x1, y1, x2, y2, x3, y3, x0, y0) {
      var point = this.nearestPoint(x1, y1, x2, y2, x3, y3, x0, y0);
      return distance$3(point.x, point.y, x0, y0);
    },
    interpolationAt: quadraticAt$2,
    pointAt: function pointAt(x1, y1, x2, y2, x3, y3, t) {
      return {
        x: quadraticAt$2(x1, x2, x3, t),
        y: quadraticAt$2(y1, y2, y3, t)
      };
    },
    divide: function divide(x1, y1, x2, y2, x3, y3, t) {
      return divideQuadratic$2(x1, y1, x2, y2, x3, y3, t);
    },
    tangentAngle: function tangentAngle(x1, y1, x2, y2, x3, y3, t) {
      var dx = derivativeAt$3(x1, x2, x3, t);
      var dy = derivativeAt$3(y1, y2, y3, t);
      var angle = Math.atan2(dy, dx);
      return piMod$2(angle);
    }
  };

  function cubicAt$2(p0, p1, p2, p3, t) {
    var onet = 1 - t; // t * t * t 的性能大概是 Math.pow(t, 3) 的三倍
    return onet * onet * onet * p0 + 3 * p1 * t * onet * onet + 3 * p2 * t * t * onet + p3 * t * t * t;
  }
  function derivativeAt$1$2(p0, p1, p2, p3, t) {
    var onet = 1 - t;
    return 3 * (onet * onet * (p1 - p0) + 2 * onet * t * (p2 - p1) + t * t * (p3 - p2));
  }
  function extrema$1$2(p0, p1, p2, p3) {
    var a = -3 * p0 + 9 * p1 - 9 * p2 + 3 * p3;
    var b = 6 * p0 - 12 * p1 + 6 * p2;
    var c = 3 * p1 - 3 * p0;
    var extremas = [];
    var t1;
    var t2;
    var discSqrt;
    if (isNumberEqual$4(a, 0)) {
      if (!isNumberEqual$4(b, 0)) {
        t1 = -c / b;
        if (t1 >= 0 && t1 <= 1) {
          extremas.push(t1);
        }
      }
    } else {
      var disc = b * b - 4 * a * c;
      if (isNumberEqual$4(disc, 0)) {
        extremas.push(-b / (2 * a));
      } else if (disc > 0) {
        discSqrt = Math.sqrt(disc);
        t1 = (-b + discSqrt) / (2 * a);
        t2 = (-b - discSqrt) / (2 * a);
        if (t1 >= 0 && t1 <= 1) {
          extremas.push(t1);
        }
        if (t2 >= 0 && t2 <= 1) {
          extremas.push(t2);
        }
      }
    }
    return extremas;
  }
  // 分割贝塞尔曲线
  function divideCubic$2(x1, y1, x2, y2, x3, y3, x4, y4, t) {
    // 划分点
    var xt = cubicAt$2(x1, x2, x3, x4, t);
    var yt = cubicAt$2(y1, y2, y3, y4, t);
    // 计算两点之间的差值点
    var c1 = line$3.pointAt(x1, y1, x2, y2, t);
    var c2 = line$3.pointAt(x2, y2, x3, y3, t);
    var c3 = line$3.pointAt(x3, y3, x4, y4, t);
    var c12 = line$3.pointAt(c1.x, c1.y, c2.x, c2.y, t);
    var c23 = line$3.pointAt(c2.x, c2.y, c3.x, c3.y, t);
    return [[x1, y1, c1.x, c1.y, c12.x, c12.y, xt, yt], [xt, yt, c23.x, c23.y, c3.x, c3.y, x4, y4]];
  }
  // 使用迭代法取贝塞尔曲线的长度，二阶和三阶分开写，更清晰和便于调试
  function cubicLength$2(x1, y1, x2, y2, x3, y3, x4, y4, iterationCount) {
    if (iterationCount === 0) {
      return snapLength$2([x1, x2, x3, x4], [y1, y2, y3, y4]);
    }
    var cubics = divideCubic$2(x1, y1, x2, y2, x3, y3, x4, y4, 0.5);
    var left = [].concat(cubics[0], [iterationCount - 1]);
    var right = [].concat(cubics[1], [iterationCount - 1]);
    return cubicLength$2.apply(void 0, left) + cubicLength$2.apply(void 0, right);
  }
  var cubic$2 = {
    extrema: extrema$1$2,
    box: function box(x1, y1, x2, y2, x3, y3, x4, y4) {
      var xArr = [x1, x4];
      var yArr = [y1, y4];
      var xExtrema = extrema$1$2(x1, x2, x3, x4);
      var yExtrema = extrema$1$2(y1, y2, y3, y4);
      for (var i = 0; i < xExtrema.length; i++) {
        xArr.push(cubicAt$2(x1, x2, x3, x4, xExtrema[i]));
      }
      for (var _i = 0; _i < yExtrema.length; _i++) {
        yArr.push(cubicAt$2(y1, y2, y3, y4, yExtrema[_i]));
      }
      return getBBoxByArray$2(xArr, yArr);
    },
    length: function length(x1, y1, x2, y2, x3, y3, x4, y4) {
      // 迭代三次，划分成 8 段求长度
      return cubicLength$2(x1, y1, x2, y2, x3, y3, x4, y4, 3);
    },
    nearestPoint: function nearestPoint$1(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length) {
      return nearestPoint$2([x1, x2, x3, x4], [y1, y2, y3, y4], x0, y0, cubicAt$2, length);
    },
    pointDistance: function pointDistance(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length) {
      var point = this.nearestPoint(x1, y1, x2, y2, x3, y3, x4, y4, x0, y0, length);
      return distance$3(point.x, point.y, x0, y0);
    },
    interpolationAt: cubicAt$2,
    pointAt: function pointAt(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      return {
        x: cubicAt$2(x1, x2, x3, x4, t),
        y: cubicAt$2(y1, y2, y3, y4, t)
      };
    },
    divide: function divide(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      return divideCubic$2(x1, y1, x2, y2, x3, y3, x4, y4, t);
    },
    tangentAngle: function tangentAngle(x1, y1, x2, y2, x3, y3, x4, y4, t) {
      var dx = derivativeAt$1$2(x1, x2, x3, x4, t);
      var dy = derivativeAt$1$2(y1, y2, y3, y4, t);
      return piMod$2(Math.atan2(dy, dx));
    }
  };

  /**
   * @fileoverview 椭圆的一些计算，
   *  - 周长计算参考：https://www.mathsisfun.com/geometry/ellipse-perimeter.html
   *  - 距离计算参考：https://wet-robots.ghost.io/simple-method-for-distance-to-ellipse/
   * @author dxq613@gmail.com
   */
  function copysign$1(v1, v2) {
    var absv = Math.abs(v1);
    return v2 > 0 ? absv : absv * -1;
  }
  var ellipse$1 = {
    /**
     * 包围盒计算
     * @param {number} x  椭圆中心 x
     * @param {number} y  椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @return {object} 包围盒
     */
    box: function box(x, y, rx, ry) {
      return {
        x: x - rx,
        y: y - ry,
        width: rx * 2,
        height: ry * 2
      };
    },
    /**
     * 计算周长，使用近似法
     * @param {number} x  椭圆中心 x
     * @param {number} y  椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @return {number} 椭圆周长
     */
    length: function length(x, y, rx, ry) {
      return Math.PI * (3 * (rx + ry) - Math.sqrt((3 * rx + ry) * (rx + 3 * ry)));
    },
    /**
     * 距离椭圆最近的点
     * @param {number} x  椭圆中心 x
     * @param {number} y  椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @param {number} x0  指定的点 x
     * @param {number} y0  指定的点 y
     * @return {object} 椭圆上距离指定点最近的点
     */
    nearestPoint: function nearestPoint(x, y, rx, ry, x0, y0) {
      var a = rx;
      var b = ry;
      // 假如椭圆半径为0则返回圆心
      if (a === 0 || b === 0) {
        return {
          x: x,
          y: y
        };
      }
      // 转换成 0， 0 为中心的椭圆计算
      var relativeX = x0 - x;
      var relativeY = y0 - y;
      var px = Math.abs(relativeX);
      var py = Math.abs(relativeY);
      var squareA = a * a;
      var squareB = b * b;
      // const angle0 = Math.atan2(relativeY, relativeX);
      var t = Math.PI / 4;
      var nearestX = 0; // 椭圆上的任一点
      var nearestY = 0;
      // 迭代 4 次
      for (var i = 0; i < 4; i++) {
        nearestX = a * Math.cos(t);
        nearestY = b * Math.sin(t);
        var ex = (squareA - squareB) * Math.pow(Math.cos(t), 3) / a;
        var ey = (squareB - squareA) * Math.pow(Math.sin(t), 3) / b;
        var rx1 = nearestX - ex;
        var ry1 = nearestY - ey;
        var qx = px - ex;
        var qy = py - ey;
        var r = Math.hypot(ry1, rx1);
        var q = Math.hypot(qy, qx);
        var delta_c = r * Math.asin((rx1 * qy - ry1 * qx) / (r * q));
        var delta_t = delta_c / Math.sqrt(squareA + squareB - nearestX * nearestX - nearestY * nearestY);
        t += delta_t;
        t = Math.min(Math.PI / 2, Math.max(0, t));
      }
      return {
        x: x + copysign$1(nearestX, relativeX),
        y: y + copysign$1(nearestY, relativeY)
      };
    },
    /**
     * 点到椭圆最近的距离
     * @param {number} x  椭圆中心 x
     * @param {number} y  椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @param {number} x0  指定的点 x
     * @param {number} y0  指定的点 y
     * @return {number} 点到椭圆的距离
     */
    pointDistance: function pointDistance(x, y, rx, ry, x0, y0) {
      var nearestPoint = this.nearestPoint(x, y, rx, ry, x0, y0);
      return distance$3(nearestPoint.x, nearestPoint.y, x0, y0);
    },
    /**
     * 根据比例获取点
     * @param {number} x 椭圆中心 x
     * @param {number} y 椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @param {number} t 指定比例，x轴方向为 0
     * @return {object} 点
     */
    pointAt: function pointAt(x, y, rx, ry, t) {
      var angle = 2 * Math.PI * t; // 按照角度进行计算，而不按照周长计算
      return {
        x: x + rx * Math.cos(angle),
        y: y + ry * Math.sin(angle)
      };
    },
    /**
     * 根据比例计算切线角度
     * @param {number} x 椭圆中心 x
     * @param {number} y 椭圆中心 y
     * @param {number} rx 椭圆 x 方向半径
     * @param {number} ry 椭圆 y 方向半径
     * @param {number} t 指定比例 0 - 1 之间，x轴方向为 0。在 0-1 范围之外是循环还是返回 null，还需要调整
     * @return {number} 角度，在 0 - 2PI 之间
     */
    tangentAngle: function tangentAngle(x, y, rx, ry, t) {
      var angle = 2 * Math.PI * t; // 按照角度进行计算，而不按照周长计算
      // 直接使用 x,y 的导数计算， x' = -rx * sin(t); y' = ry * cos(t);
      var tangentAngle = Math.atan2(ry * Math.cos(angle), -rx * Math.sin(angle));
      // 也可以使用指定点的切线方程计算，成本有些高
      // const point = this.pointAt(0, 0, rx, ry, t); // 椭圆的切线同椭圆的中心不相关
      // let tangentAngle = -1 * Math.atan((ry * ry * point.x) / (rx * rx * point.y));
      // if (angle >= 0 && angle <= Math.PI) {
      //   tangentAngle += Math.PI;
      // }
      return piMod$2(tangentAngle);
    }
  };

  // 偏导数 x
  function derivativeXAt$1(cx, cy, rx, ry, xRotation, startAngle, endAngle, angle) {
    return -1 * rx * Math.cos(xRotation) * Math.sin(angle) - ry * Math.sin(xRotation) * Math.cos(angle);
  }
  // 偏导数 y
  function derivativeYAt$1(cx, cy, rx, ry, xRotation, startAngle, endAngle, angle) {
    return -1 * rx * Math.sin(xRotation) * Math.sin(angle) + ry * Math.cos(xRotation) * Math.cos(angle);
  }
  // x 的极值
  function xExtrema$1(rx, ry, xRotation) {
    return Math.atan(-ry / rx * Math.tan(xRotation));
  }
  // y 的极值
  function yExtrema$1(rx, ry, xRotation) {
    return Math.atan(ry / (rx * Math.tan(xRotation)));
  }
  // 根据角度求 x 坐标
  function xAt$1(cx, cy, rx, ry, xRotation, angle) {
    return rx * Math.cos(xRotation) * Math.cos(angle) - ry * Math.sin(xRotation) * Math.sin(angle) + cx;
  }
  // 根据角度求 y 坐标
  function yAt$1(cx, cy, rx, ry, xRotation, angle) {
    return rx * Math.sin(xRotation) * Math.cos(angle) + ry * Math.cos(xRotation) * Math.sin(angle) + cy;
  }
  // 获取点在椭圆上的角度
  function getAngle$2(rx, ry, x0, y0) {
    var angle = Math.atan2(y0 * rx, x0 * ry);
    // 转换到 0 - 2PI 内
    return (angle + Math.PI * 2) % (Math.PI * 2);
  }
  // 根据角度获取，x,y
  function getPoint$2(rx, ry, angle) {
    return {
      x: rx * Math.cos(angle),
      y: ry * Math.sin(angle)
    };
  }
  // 旋转
  function rotate$4(x, y, angle) {
    var cos = Math.cos(angle);
    var sin = Math.sin(angle);
    return [x * cos - y * sin, x * sin + y * cos];
  }
  var arc$1 = {
    /**
     * 计算包围盒
     * @param {number} cx         圆心 x
     * @param {number} cy         圆心 y
     * @param {number} rx         x 轴方向的半径
     * @param {number} ry         y 轴方向的半径
     * @param {number} xRotation  旋转角度
     * @param {number} startAngle 起始角度
     * @param {number} endAngle   结束角度
     * @return {object} 包围盒对象
     */
    box: function box(cx, cy, rx, ry, xRotation, startAngle, endAngle) {
      var xDim = xExtrema$1(rx, ry, xRotation);
      var minX = Infinity;
      var maxX = -Infinity;
      var xs = [startAngle, endAngle];
      for (var i = -Math.PI * 2; i <= Math.PI * 2; i += Math.PI) {
        var xAngle = xDim + i;
        if (startAngle < endAngle) {
          if (startAngle < xAngle && xAngle < endAngle) {
            xs.push(xAngle);
          }
        } else {
          if (endAngle < xAngle && xAngle < startAngle) {
            xs.push(xAngle);
          }
        }
      }
      for (var _i = 0; _i < xs.length; _i++) {
        var x = xAt$1(cx, cy, rx, ry, xRotation, xs[_i]);
        if (x < minX) {
          minX = x;
        }
        if (x > maxX) {
          maxX = x;
        }
      }
      var yDim = yExtrema$1(rx, ry, xRotation);
      var minY = Infinity;
      var maxY = -Infinity;
      var ys = [startAngle, endAngle];
      for (var _i2 = -Math.PI * 2; _i2 <= Math.PI * 2; _i2 += Math.PI) {
        var yAngle = yDim + _i2;
        if (startAngle < endAngle) {
          if (startAngle < yAngle && yAngle < endAngle) {
            ys.push(yAngle);
          }
        } else {
          if (endAngle < yAngle && yAngle < startAngle) {
            ys.push(yAngle);
          }
        }
      }
      for (var _i3 = 0; _i3 < ys.length; _i3++) {
        var y = yAt$1(cx, cy, rx, ry, xRotation, ys[_i3]);
        if (y < minY) {
          minY = y;
        }
        if (y > maxY) {
          maxY = y;
        }
      }
      return {
        x: minX,
        y: minY,
        width: maxX - minX,
        height: maxY - minY
      };
    },
    /**
     * 获取圆弧的长度，计算圆弧长度时不考虑旋转角度，
     * 仅跟 rx, ry, startAngle, endAngle 相关
     * @param {number} cx         圆心 x
     * @param {number} cy         圆心 y
     * @param {number} rx         x 轴方向的半径
     * @param {number} ry         y 轴方向的半径
     * @param {number} xRotation  旋转角度
     * @param {number} startAngle 起始角度
     * @param {number} endAngle   结束角度
     */
    length: function length(cx, cy, rx, ry, xRotation, startAngle, endAngle) {},
    /**
     * 获取指定点到圆弧的最近距离的点
     * @param {number} cx         圆心 x
     * @param {number} cy         圆心 y
     * @param {number} rx         x 轴方向的半径
     * @param {number} ry         y 轴方向的半径
     * @param {number} xRotation  旋转角度
     * @param {number} startAngle 起始角度
     * @param {number} endAngle   结束角度
     * @param {number} x0         指定点的 x
     * @param {number} y0         指定点的 y
     * @return {object} 到指定点最近距离的点
     */
    nearestPoint: function nearestPoint(cx, cy, rx, ry, xRotation, startAngle, endAngle, x0, y0) {
      // 将最近距离问题转换成到椭圆中心 0,0 没有旋转的椭圆问题
      var relativeVector = rotate$4(x0 - cx, y0 - cy, -xRotation);
      var x1 = relativeVector[0],
        y1 = relativeVector[1];
      // 计算点到椭圆的最近的点
      var relativePoint = ellipse$1.nearestPoint(0, 0, rx, ry, x1, y1);
      // 获取点在椭圆上的角度
      var angle = getAngle$2(rx, ry, relativePoint.x, relativePoint.y);
      // 点没有在圆弧上
      if (angle < startAngle) {
        // 小于起始圆弧
        relativePoint = getPoint$2(rx, ry, startAngle);
      } else if (angle > endAngle) {
        // 大于结束圆弧
        relativePoint = getPoint$2(rx, ry, endAngle);
      }
      // 旋转到 xRotation 的角度
      var vector = rotate$4(relativePoint.x, relativePoint.y, xRotation);
      return {
        x: vector[0] + cx,
        y: vector[1] + cy
      };
    },
    pointDistance: function pointDistance(cx, cy, rx, ry, xRotation, startAngle, endAngle, x0, y0) {
      var nearestPoint = this.nearestPoint(cx, cy, rx, ry, xRotation, startAngle, endAngle, x0, y0);
      return distance$3(nearestPoint.x, nearestPoint.y, x0, y0);
    },
    pointAt: function pointAt(cx, cy, rx, ry, xRotation, startAngle, endAngle, t) {
      var angle = (endAngle - startAngle) * t + startAngle;
      return {
        x: xAt$1(cx, cy, rx, ry, xRotation, angle),
        y: yAt$1(cx, cy, rx, ry, xRotation, angle)
      };
    },
    tangentAngle: function tangentAngle(cx, cy, rx, ry, xRotation, startAngle, endAngle, t) {
      var angle = (endAngle - startAngle) * t + startAngle;
      var dx = derivativeXAt$1(cx, cy, rx, ry, xRotation, startAngle, endAngle, angle);
      var dy = derivativeYAt$1(cx, cy, rx, ry, xRotation, startAngle, endAngle, angle);
      return piMod$2(Math.atan2(dy, dx));
    }
  };

  function analyzePoints$1(points) {
    // 计算每段的长度和总的长度
    var totalLength = 0;
    var segments = [];
    for (var i = 0; i < points.length - 1; i++) {
      var from = points[i];
      var to = points[i + 1];
      var length = distance$3(from[0], from[1], to[0], to[1]);
      var seg = {
        from: from,
        to: to,
        length: length
      };
      segments.push(seg);
      totalLength += length;
    }
    return {
      segments: segments,
      totalLength: totalLength
    };
  }
  function lengthOfSegment$1(points) {
    if (points.length < 2) {
      return 0;
    }
    var totalLength = 0;
    for (var i = 0; i < points.length - 1; i++) {
      var from = points[i];
      var to = points[i + 1];
      totalLength += distance$3(from[0], from[1], to[0], to[1]);
    }
    return totalLength;
  }
  /**
   * 按照比例在数据片段中获取点
   * @param {array} points 点的集合
   * @param {number} t 百分比 0-1
   * @return {object} 点的坐标
   */
  function pointAtSegments$1(points, t) {
    // 边界判断
    if (t > 1 || t < 0 || points.length < 2) {
      return null;
    }
    var _analyzePoints = analyzePoints$1(points),
      segments = _analyzePoints.segments,
      totalLength = _analyzePoints.totalLength;
    // 多个点有可能重合
    if (totalLength === 0) {
      return {
        x: points[0][0],
        y: points[0][1]
      };
    }
    // 计算比例
    var startRatio = 0;
    var point = null;
    for (var i = 0; i < segments.length; i++) {
      var seg = segments[i];
      var from = seg.from,
        to = seg.to;
      var currentRatio = seg.length / totalLength;
      if (t >= startRatio && t <= startRatio + currentRatio) {
        var localRatio = (t - startRatio) / currentRatio;
        point = line$3.pointAt(from[0], from[1], to[0], to[1], localRatio);
        break;
      }
      startRatio += currentRatio;
    }
    return point;
  }
  /**
   * 按照比例在数据片段中获取切线的角度
   * @param {array} points 点的集合
   * @param {number} t 百分比 0-1
   */
  function angleAtSegments$1(points, t) {
    // 边界判断
    if (t > 1 || t < 0 || points.length < 2) {
      return 0;
    }
    var _analyzePoints2 = analyzePoints$1(points),
      segments = _analyzePoints2.segments,
      totalLength = _analyzePoints2.totalLength;
    // 计算比例
    var startRatio = 0;
    var angle = 0;
    for (var i = 0; i < segments.length; i++) {
      var seg = segments[i];
      var from = seg.from,
        to = seg.to;
      var currentRatio = seg.length / totalLength;
      if (t >= startRatio && t <= startRatio + currentRatio) {
        angle = Math.atan2(to[1] - from[1], to[0] - from[0]);
        break;
      }
      startRatio += currentRatio;
    }
    return angle;
  }
  function distanceAtSegment$1(points, x, y) {
    var minDistance = Infinity;
    for (var i = 0; i < points.length - 1; i++) {
      var point = points[i];
      var nextPoint = points[i + 1];
      var _distance = line$3.pointDistance(point[0], point[1], nextPoint[0], nextPoint[1], x, y);
      if (_distance < minDistance) {
        minDistance = _distance;
      }
    }
    return minDistance;
  }

  var polyline$1 = {
    /**
     * 计算多折线的包围盒
     * @param {array} points 点的集合 [x,y] 的形式
     * @return {object} 包围盒
     */
    box: function box(points) {
      var xArr = [];
      var yArr = [];
      for (var i = 0; i < points.length; i++) {
        var point = points[i];
        xArr.push(point[0]);
        yArr.push(point[1]);
      }
      return getBBoxByArray$2(xArr, yArr);
    },
    /**
     * 计算多折线的长度
     * @param {array} points 点的集合 [x,y] 的形式
     * @return {object} 多条边的长度
     */
    length: function length(points) {
      return lengthOfSegment$1(points);
    },
    /**
     * 根据比例获取多折线的点
     * @param {array} points 点的集合 [x,y] 的形式
     * @param {number} t 在多折线的长度上的比例
     * @return {object} 根据比例值计算出来的点
     */
    pointAt: function pointAt(points, t) {
      return pointAtSegments$1(points, t);
    },
    /**
     * 指定点到多折线的距离
     * @param {array} points 点的集合 [x,y] 的形式
     * @param {number} x 指定点的 x
     * @param {number} y 指定点的 y
     * @return {number} 点到多折线的距离
     */
    pointDistance: function pointDistance(points, x, y) {
      return distanceAtSegment$1(points, x, y);
    },
    /**
     * 根据比例获取多折线的切线角度
     * @param {array} points 点的集合 [x,y] 的形式
     * @param {number} t 在多折线的长度上的比例
     * @return {object} 根据比例值计算出来的角度
     */
    tangentAngle: function tangentAngle(points, t) {
      return angleAtSegments$1(points, t);
    }
  };

  var eventemitter3$1 = createCommonjsModule(function (module) {

  var has = Object.prototype.hasOwnProperty
    , prefix = '~';

  /**
   * Constructor to create a storage for our `EE` objects.
   * An `Events` instance is a plain object whose properties are event names.
   *
   * @constructor
   * @private
   */
  function Events() {}

  //
  // We try to not inherit from `Object.prototype`. In some engines creating an
  // instance in this way is faster than calling `Object.create(null)` directly.
  // If `Object.create(null)` is not supported we prefix the event names with a
  // character to make sure that the built-in object properties are not
  // overridden or used as an attack vector.
  //
  if (Object.create) {
    Events.prototype = Object.create(null);

    //
    // This hack is needed because the `__proto__` property is still inherited in
    // some old browsers like Android 4, iPhone 5.1, Opera 11 and Safari 5.
    //
    if (!new Events().__proto__) prefix = false;
  }

  /**
   * Representation of a single event listener.
   *
   * @param {Function} fn The listener function.
   * @param {*} context The context to invoke the listener with.
   * @param {Boolean} [once=false] Specify if the listener is a one-time listener.
   * @constructor
   * @private
   */
  function EE(fn, context, once) {
    this.fn = fn;
    this.context = context;
    this.once = once || false;
  }

  /**
   * Add a listener for a given event.
   *
   * @param {EventEmitter} emitter Reference to the `EventEmitter` instance.
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} context The context to invoke the listener with.
   * @param {Boolean} once Specify if the listener is a one-time listener.
   * @returns {EventEmitter}
   * @private
   */
  function addListener(emitter, event, fn, context, once) {
    if (typeof fn !== 'function') {
      throw new TypeError('The listener must be a function');
    }

    var listener = new EE(fn, context || emitter, once)
      , evt = prefix ? prefix + event : event;

    if (!emitter._events[evt]) emitter._events[evt] = listener, emitter._eventsCount++;
    else if (!emitter._events[evt].fn) emitter._events[evt].push(listener);
    else emitter._events[evt] = [emitter._events[evt], listener];

    return emitter;
  }

  /**
   * Clear event by name.
   *
   * @param {EventEmitter} emitter Reference to the `EventEmitter` instance.
   * @param {(String|Symbol)} evt The Event name.
   * @private
   */
  function clearEvent(emitter, evt) {
    if (--emitter._eventsCount === 0) emitter._events = new Events();
    else delete emitter._events[evt];
  }

  /**
   * Minimal `EventEmitter` interface that is molded against the Node.js
   * `EventEmitter` interface.
   *
   * @constructor
   * @public
   */
  function EventEmitter() {
    this._events = new Events();
    this._eventsCount = 0;
  }

  /**
   * Return an array listing the events for which the emitter has registered
   * listeners.
   *
   * @returns {Array}
   * @public
   */
  EventEmitter.prototype.eventNames = function eventNames() {
    var names = []
      , events
      , name;

    if (this._eventsCount === 0) return names;

    for (name in (events = this._events)) {
      if (has.call(events, name)) names.push(prefix ? name.slice(1) : name);
    }

    if (Object.getOwnPropertySymbols) {
      return names.concat(Object.getOwnPropertySymbols(events));
    }

    return names;
  };

  /**
   * Return the listeners registered for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Array} The registered listeners.
   * @public
   */
  EventEmitter.prototype.listeners = function listeners(event) {
    var evt = prefix ? prefix + event : event
      , handlers = this._events[evt];

    if (!handlers) return [];
    if (handlers.fn) return [handlers.fn];

    for (var i = 0, l = handlers.length, ee = new Array(l); i < l; i++) {
      ee[i] = handlers[i].fn;
    }

    return ee;
  };

  /**
   * Return the number of listeners listening to a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Number} The number of listeners.
   * @public
   */
  EventEmitter.prototype.listenerCount = function listenerCount(event) {
    var evt = prefix ? prefix + event : event
      , listeners = this._events[evt];

    if (!listeners) return 0;
    if (listeners.fn) return 1;
    return listeners.length;
  };

  /**
   * Calls each of the listeners registered for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @returns {Boolean} `true` if the event had listeners, else `false`.
   * @public
   */
  EventEmitter.prototype.emit = function emit(event, a1, a2, a3, a4, a5) {
    var evt = prefix ? prefix + event : event;

    if (!this._events[evt]) return false;

    var listeners = this._events[evt]
      , len = arguments.length
      , args
      , i;

    if (listeners.fn) {
      if (listeners.once) this.removeListener(event, listeners.fn, undefined, true);

      switch (len) {
        case 1: return listeners.fn.call(listeners.context), true;
        case 2: return listeners.fn.call(listeners.context, a1), true;
        case 3: return listeners.fn.call(listeners.context, a1, a2), true;
        case 4: return listeners.fn.call(listeners.context, a1, a2, a3), true;
        case 5: return listeners.fn.call(listeners.context, a1, a2, a3, a4), true;
        case 6: return listeners.fn.call(listeners.context, a1, a2, a3, a4, a5), true;
      }

      for (i = 1, args = new Array(len -1); i < len; i++) {
        args[i - 1] = arguments[i];
      }

      listeners.fn.apply(listeners.context, args);
    } else {
      var length = listeners.length
        , j;

      for (i = 0; i < length; i++) {
        if (listeners[i].once) this.removeListener(event, listeners[i].fn, undefined, true);

        switch (len) {
          case 1: listeners[i].fn.call(listeners[i].context); break;
          case 2: listeners[i].fn.call(listeners[i].context, a1); break;
          case 3: listeners[i].fn.call(listeners[i].context, a1, a2); break;
          case 4: listeners[i].fn.call(listeners[i].context, a1, a2, a3); break;
          default:
            if (!args) for (j = 1, args = new Array(len -1); j < len; j++) {
              args[j - 1] = arguments[j];
            }

            listeners[i].fn.apply(listeners[i].context, args);
        }
      }
    }

    return true;
  };

  /**
   * Add a listener for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} [context=this] The context to invoke the listener with.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.on = function on(event, fn, context) {
    return addListener(this, event, fn, context, false);
  };

  /**
   * Add a one-time listener for a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn The listener function.
   * @param {*} [context=this] The context to invoke the listener with.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.once = function once(event, fn, context) {
    return addListener(this, event, fn, context, true);
  };

  /**
   * Remove the listeners of a given event.
   *
   * @param {(String|Symbol)} event The event name.
   * @param {Function} fn Only remove the listeners that match this function.
   * @param {*} context Only remove the listeners that have this context.
   * @param {Boolean} once Only remove one-time listeners.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.removeListener = function removeListener(event, fn, context, once) {
    var evt = prefix ? prefix + event : event;

    if (!this._events[evt]) return this;
    if (!fn) {
      clearEvent(this, evt);
      return this;
    }

    var listeners = this._events[evt];

    if (listeners.fn) {
      if (
        listeners.fn === fn &&
        (!once || listeners.once) &&
        (!context || listeners.context === context)
      ) {
        clearEvent(this, evt);
      }
    } else {
      for (var i = 0, events = [], length = listeners.length; i < length; i++) {
        if (
          listeners[i].fn !== fn ||
          (once && !listeners[i].once) ||
          (context && listeners[i].context !== context)
        ) {
          events.push(listeners[i]);
        }
      }

      //
      // Reset the array, or remove it completely if we have no more listeners.
      //
      if (events.length) this._events[evt] = events.length === 1 ? events[0] : events;
      else clearEvent(this, evt);
    }

    return this;
  };

  /**
   * Remove all listeners, or those of the specified event.
   *
   * @param {(String|Symbol)} [event] The event name.
   * @returns {EventEmitter} `this`.
   * @public
   */
  EventEmitter.prototype.removeAllListeners = function removeAllListeners(event) {
    var evt;

    if (event) {
      evt = prefix ? prefix + event : event;
      if (this._events[evt]) clearEvent(this, evt);
    } else {
      this._events = new Events();
      this._eventsCount = 0;
    }

    return this;
  };

  //
  // Alias methods names because people roll like that.
  //
  EventEmitter.prototype.off = EventEmitter.prototype.removeListener;
  EventEmitter.prototype.addListener = EventEmitter.prototype.on;

  //
  // Expose the prefix.
  //
  EventEmitter.prefixed = prefix;

  //
  // Allow `EventEmitter` to be imported as module namespace.
  //
  EventEmitter.EventEmitter = EventEmitter;

  //
  // Expose the module.
  //
  {
    module.exports = EventEmitter;
  }
  });

  let uId$1=1;const root$1="object"==typeof self&&self.self==self?self:"object"==typeof global&&global.global==global?global:{},nowOffset$1=Date.now();let reservedCBs$1={},lastTime$1=Date.now();const polyfillRaf$1=e=>{if("function"!=typeof e)throw new TypeError(e+" is not a function");const o=Date.now(),t=o-lastTime$1,n=t>16?0:16-t,r=uId$1++;return reservedCBs$1[r]=e,Object.keys(reservedCBs$1).length>1||setTimeout((()=>{lastTime$1=o;const e=reservedCBs$1;reservedCBs$1={},Object.keys(e).forEach((o=>e[o](root$1.performance&&"function"==typeof root$1.performance.now?root$1.performance.now():Date.now()-nowOffset$1)));}),n),r},polyfillCaf$1=e=>{delete reservedCBs$1[e];},vendorPrefixes$1=["","webkit","moz","ms","o"],getRequestAnimationFrame$1=e=>"string"!=typeof e?polyfillRaf$1:""===e?root$1.requestAnimationFrame:root$1[e+"RequestAnimationFrame"],getCancelAnimationFrame$1=e=>"string"!=typeof e?polyfillCaf$1:""===e?root$1.cancelAnimationFrame:root$1[e+"CancelAnimationFrame"]||root$1[e+"CancelRequestAnimationFrame"],find$2=(e,o)=>{let t=0;for(;void 0!==e[t];){if(o(e[t]))return e[t];t+=1;}},vp$1=find$2(vendorPrefixes$1,(e=>!!getRequestAnimationFrame$1(e))),raf$1=getRequestAnimationFrame$1(vp$1),caf$1=getCancelAnimationFrame$1(vp$1);root$1.requestAnimationFrame=raf$1,root$1.cancelAnimationFrame=caf$1,"undefined"!=typeof module&&module.exports&&(module.exports={requestAnimationFrame:raf$1,cancelAnimationFrame:caf$1});

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type
   */
  var PropertySyntax$1;
  (function (PropertySyntax) {
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#coordinate
     */
    PropertySyntax["COORDINATE"] = "<coordinate>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#color
     */
    PropertySyntax["COLOR"] = "<color>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#paint
     */
    PropertySyntax["PAINT"] = "<paint>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#number
     */
    PropertySyntax["NUMBER"] = "<number>";
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/angle
     */
    PropertySyntax["ANGLE"] = "<angle>";
    /**
     * <number> with range 0..1
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#opacity_value
     */
    PropertySyntax["OPACITY_VALUE"] = "<opacity-value>";
    /**
     * <number> with range 0..Infinity
     */
    PropertySyntax["SHADOW_BLUR"] = "<shadow-blur>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#length
     */
    PropertySyntax["LENGTH"] = "<length>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#percentage
     */
    PropertySyntax["PERCENTAGE"] = "<percentage>";
    PropertySyntax["LENGTH_PERCENTAGE"] = "<length> | <percentage>";
    PropertySyntax["LENGTH_PERCENTAGE_12"] = "[<length> | <percentage>]{1,2}";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/CSS/margin#formal_syntax
     */
    PropertySyntax["LENGTH_PERCENTAGE_14"] = "[<length> | <percentage>]{1,4}";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#list-of-ts
     */
    PropertySyntax["LIST_OF_POINTS"] = "<list-of-points>";
    PropertySyntax["PATH"] = "<path>";
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/CSS/filter#formal_syntax
     */
    PropertySyntax["FILTER"] = "<filter>";
    PropertySyntax["Z_INDEX"] = "<z-index>";
    PropertySyntax["OFFSET_DISTANCE"] = "<offset-distance>";
    PropertySyntax["DEFINED_PATH"] = "<defined-path>";
    PropertySyntax["MARKER"] = "<marker>";
    PropertySyntax["TRANSFORM"] = "<transform>";
    PropertySyntax["TRANSFORM_ORIGIN"] = "<transform-origin>";
    PropertySyntax["TEXT"] = "<text>";
    PropertySyntax["TEXT_TRANSFORM"] = "<text-transform>";
  })(PropertySyntax$1 || (PropertySyntax$1 = {}));

  /**
   * Axis-Aligned Bounding Box
   * 为了便于后续 Frustum Culling，通过查找表定义 p-vertex 和 n-vertex
   * @see https://github.com/antvis/GWebGPUEngine/issues/3
   */
  var AABB$1 = /*#__PURE__*/function () {
    AABB.isEmpty = function isEmpty(aabb) {
      return !aabb || aabb.halfExtents[0] === 0 && aabb.halfExtents[1] === 0 && aabb.halfExtents[2] === 0;
    };
    function AABB(center, halfExtents) {
      if (center === void 0) {
        center = [0, 0, 0];
      }
      if (halfExtents === void 0) {
        halfExtents = [0, 0, 0];
      }
      this.center = create$8();
      this.halfExtents = create$8();
      this.min = create$8();
      this.max = create$8();
      this.update(center, halfExtents);
    }
    var _proto = AABB.prototype;
    _proto.update = function update(center, halfExtents) {
      copy$6(this.center, center);
      copy$6(this.halfExtents, halfExtents);
      sub$4(this.min, this.center, this.halfExtents);
      add$4(this.max, this.center, this.halfExtents);
    };
    _proto.setMinMax = function setMinMax(min, max) {
      add$4(this.center, max, min);
      scale$4(this.center, this.center, 0.5);
      sub$4(this.halfExtents, max, min);
      scale$4(this.halfExtents, this.halfExtents, 0.5);
      copy$6(this.min, min);
      copy$6(this.max, max);
    };
    _proto.getMin = function getMin() {
      return this.min;
    };
    _proto.getMax = function getMax() {
      return this.max;
    };
    _proto.add = function add(aabb) {
      if (AABB.isEmpty(aabb)) {
        return;
      }
      if (AABB.isEmpty(this)) {
        this.setMinMax(aabb.getMin(), aabb.getMax());
        return;
      }
      var tc = this.center;
      var tcx = tc[0];
      var tcy = tc[1];
      var tcz = tc[2];
      var th = this.halfExtents;
      var thx = th[0];
      var thy = th[1];
      var thz = th[2];
      var tminx = tcx - thx;
      var tmaxx = tcx + thx;
      var tminy = tcy - thy;
      var tmaxy = tcy + thy;
      var tminz = tcz - thz;
      var tmaxz = tcz + thz;
      var oc = aabb.center;
      var ocx = oc[0];
      var ocy = oc[1];
      var ocz = oc[2];
      var oh = aabb.halfExtents;
      var ohx = oh[0];
      var ohy = oh[1];
      var ohz = oh[2];
      var ominx = ocx - ohx;
      var omaxx = ocx + ohx;
      var ominy = ocy - ohy;
      var omaxy = ocy + ohy;
      var ominz = ocz - ohz;
      var omaxz = ocz + ohz;
      if (ominx < tminx) {
        tminx = ominx;
      }
      if (omaxx > tmaxx) {
        tmaxx = omaxx;
      }
      if (ominy < tminy) {
        tminy = ominy;
      }
      if (omaxy > tmaxy) {
        tmaxy = omaxy;
      }
      if (ominz < tminz) {
        tminz = ominz;
      }
      if (omaxz > tmaxz) {
        tmaxz = omaxz;
      }
      tc[0] = (tminx + tmaxx) * 0.5;
      tc[1] = (tminy + tmaxy) * 0.5;
      tc[2] = (tminz + tmaxz) * 0.5;
      th[0] = (tmaxx - tminx) * 0.5;
      th[1] = (tmaxy - tminy) * 0.5;
      th[2] = (tmaxz - tminz) * 0.5;
      this.min[0] = tminx;
      this.min[1] = tminy;
      this.min[2] = tminz;
      this.max[0] = tmaxx;
      this.max[1] = tmaxy;
      this.max[2] = tmaxz;
    };
    _proto.setFromTransformedAABB = function setFromTransformedAABB(aabb, m) {
      var bc = this.center;
      var br = this.halfExtents;
      var ac = aabb.center;
      var ar = aabb.halfExtents;
      var mx0 = m[0];
      var mx1 = m[4];
      var mx2 = m[8];
      var my0 = m[1];
      var my1 = m[5];
      var my2 = m[9];
      var mz0 = m[2];
      var mz1 = m[6];
      var mz2 = m[10];
      var mx0a = Math.abs(mx0);
      var mx1a = Math.abs(mx1);
      var mx2a = Math.abs(mx2);
      var my0a = Math.abs(my0);
      var my1a = Math.abs(my1);
      var my2a = Math.abs(my2);
      var mz0a = Math.abs(mz0);
      var mz1a = Math.abs(mz1);
      var mz2a = Math.abs(mz2);
      set$3(bc, m[12] + mx0 * ac[0] + mx1 * ac[1] + mx2 * ac[2], m[13] + my0 * ac[0] + my1 * ac[1] + my2 * ac[2], m[14] + mz0 * ac[0] + mz1 * ac[1] + mz2 * ac[2]);
      set$3(br, mx0a * ar[0] + mx1a * ar[1] + mx2a * ar[2], my0a * ar[0] + my1a * ar[1] + my2a * ar[2], mz0a * ar[0] + mz1a * ar[1] + mz2a * ar[2]);
      this.min = sub$4(this.min, bc, br);
      this.max = add$4(this.max, bc, br);
    };
    _proto.intersects = function intersects(aabb) {
      var aMax = this.getMax();
      var aMin = this.getMin();
      var bMax = aabb.getMax();
      var bMin = aabb.getMin();
      return aMin[0] <= bMax[0] && aMax[0] >= bMin[0] && aMin[1] <= bMax[1] && aMax[1] >= bMin[1] && aMin[2] <= bMax[2] && aMax[2] >= bMin[2];
    };
    _proto.intersection = function intersection(aabb) {
      if (!this.intersects(aabb)) {
        return null;
      }
      var intersection = new AABB();
      var min = max$3(create$8(), this.getMin(), aabb.getMin());
      var max = min$3(create$8(), this.getMax(), aabb.getMax());
      intersection.setMinMax(min, max);
      return intersection;
    };
    _proto.containsPoint = function containsPoint(point) {
      var min = this.getMin();
      var max = this.getMax();
      return !(point[0] < min[0] || point[0] > max[0] || point[1] < min[1] || point[1] > max[1] || point[2] < min[2] || point[2] > max[2]);
    }
    /**
     * get n-vertex
     * @param plane plane of CullingVolume
     */;
    _proto.getNegativeFarPoint = function getNegativeFarPoint(plane) {
      if (plane.pnVertexFlag === 0x111) {
        return copy$6(create$8(), this.min);
      } else if (plane.pnVertexFlag === 0x110) {
        return fromValues$8(this.min[0], this.min[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x101) {
        return fromValues$8(this.min[0], this.max[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x100) {
        return fromValues$8(this.min[0], this.max[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x011) {
        return fromValues$8(this.max[0], this.min[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x010) {
        return fromValues$8(this.max[0], this.min[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x001) {
        return fromValues$8(this.max[0], this.max[1], this.min[2]);
      } else {
        return fromValues$8(this.max[0], this.max[1], this.max[2]);
      }
    }
    /**
     * get p-vertex
     * @param plane plane of CullingVolume
     */;
    _proto.getPositiveFarPoint = function getPositiveFarPoint(plane) {
      if (plane.pnVertexFlag === 0x111) {
        return copy$6(create$8(), this.max);
      } else if (plane.pnVertexFlag === 0x110) {
        return fromValues$8(this.max[0], this.max[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x101) {
        return fromValues$8(this.max[0], this.min[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x100) {
        return fromValues$8(this.max[0], this.min[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x011) {
        return fromValues$8(this.min[0], this.max[1], this.max[2]);
      } else if (plane.pnVertexFlag === 0x010) {
        return fromValues$8(this.min[0], this.max[1], this.min[2]);
      } else if (plane.pnVertexFlag === 0x001) {
        return fromValues$8(this.min[0], this.min[1], this.max[2]);
      } else {
        return fromValues$8(this.min[0], this.min[1], this.min[2]);
      }
    };
    return AABB;
  }();

  var Plane$1 = /*#__PURE__*/function () {
    /**
     * lookup table for p-vertex & n-vertex when doing frustum culling
     */

    function Plane(distance, normal) {
      this.distance = void 0;
      this.normal = void 0;
      this.pnVertexFlag = void 0;
      this.distance = distance || 0;
      this.normal = normal || fromValues$8(0, 1, 0);
      this.updatePNVertexFlag();
    }
    var _proto = Plane.prototype;
    _proto.updatePNVertexFlag = function updatePNVertexFlag() {
      this.pnVertexFlag = (Number(this.normal[0] >= 0) << 8) + (Number(this.normal[1] >= 0) << 4) + Number(this.normal[2] >= 0);
    };
    _proto.distanceToPoint = function distanceToPoint(point) {
      return dot$3(point, this.normal) - this.distance;
    };
    _proto.normalize = function normalize() {
      var invLen = 1 / len$1(this.normal);
      scale$4(this.normal, this.normal, invLen);
      this.distance *= invLen;
    };
    _proto.intersectsLine = function intersectsLine(start, end, point) {
      var d0 = this.distanceToPoint(start);
      var d1 = this.distanceToPoint(end);
      var t = d0 / (d0 - d1);
      var intersects = t >= 0 && t <= 1;
      if (intersects && point) {
        lerp$1(point, start, end, t);
      }
      return intersects;
    };
    return Plane;
  }();

  var Mask$1;
  (function (Mask) {
    Mask[Mask["OUTSIDE"] = 4294967295] = "OUTSIDE";
    Mask[Mask["INSIDE"] = 0] = "INSIDE";
    Mask[Mask["INDETERMINATE"] = 2147483647] = "INDETERMINATE";
  })(Mask$1 || (Mask$1 = {}));
  var Frustum$1 = /*#__PURE__*/function () {
    function Frustum(planes) {
      this.planes = [];
      if (planes) {
        this.planes = planes;
      } else {
        for (var i = 0; i < 6; i++) {
          this.planes.push(new Plane$1());
        }
      }
    }
    /**
     * extract 6 planes from projectionMatrix
     * @see http://www8.cs.umu.se/kurser/5DV051/HT12/lab/plane_extraction.pdf
     */
    var _proto = Frustum.prototype;
    _proto.extractFromVPMatrix = function extractFromVPMatrix(projectionMatrix) {
      // @ts-ignore
      var m0 = projectionMatrix[0],
        m1 = projectionMatrix[1],
        m2 = projectionMatrix[2],
        m3 = projectionMatrix[3],
        m4 = projectionMatrix[4],
        m5 = projectionMatrix[5],
        m6 = projectionMatrix[6],
        m7 = projectionMatrix[7],
        m8 = projectionMatrix[8],
        m9 = projectionMatrix[9],
        m10 = projectionMatrix[10],
        m11 = projectionMatrix[11],
        m12 = projectionMatrix[12],
        m13 = projectionMatrix[13],
        m14 = projectionMatrix[14],
        m15 = projectionMatrix[15];
      // right
      set$3(this.planes[0].normal, m3 - m0, m7 - m4, m11 - m8);
      this.planes[0].distance = m15 - m12;
      // left
      set$3(this.planes[1].normal, m3 + m0, m7 + m4, m11 + m8);
      this.planes[1].distance = m15 + m12;
      // bottom
      set$3(this.planes[2].normal, m3 + m1, m7 + m5, m11 + m9);
      this.planes[2].distance = m15 + m13;
      // top
      set$3(this.planes[3].normal, m3 - m1, m7 - m5, m11 - m9);
      this.planes[3].distance = m15 - m13;
      // far
      set$3(this.planes[4].normal, m3 - m2, m7 - m6, m11 - m10);
      this.planes[4].distance = m15 - m14;
      // near
      set$3(this.planes[5].normal, m3 + m2, m7 + m6, m11 + m10);
      this.planes[5].distance = m15 + m14;
      this.planes.forEach(function (plane) {
        plane.normalize();
        plane.updatePNVertexFlag();
      });
    };
    return Frustum;
  }();

  var Point$1 = /*#__PURE__*/function () {
    function Point(x, y) {
      if (x === void 0) {
        x = 0;
      }
      if (y === void 0) {
        y = 0;
      }
      this.x = 0;
      this.y = 0;
      this.x = x;
      this.y = y;
    }
    var _proto = Point.prototype;
    _proto.clone = function clone() {
      return new Point(this.x, this.y);
    };
    _proto.copyFrom = function copyFrom(p) {
      this.x = p.x;
      this.y = p.y;
    };
    return Point;
  }();

  var Rectangle$1 = /*#__PURE__*/function () {
    function Rectangle(x, y, width, height) {
      this.x = void 0;
      this.y = void 0;
      this.width = void 0;
      this.height = void 0;
      this.left = void 0;
      this.right = void 0;
      this.top = void 0;
      this.bottom = void 0;
      this.x = x;
      this.y = y;
      this.width = width;
      this.height = height;
      this.left = x;
      this.right = x + width;
      this.top = y;
      this.bottom = y + height;
    }
    var _proto = Rectangle.prototype;
    _proto.toJSON = function toJSON() {};
    return Rectangle;
  }();

  var Shape$2;
  (function (Shape) {
    Shape["GROUP"] = "g";
    Shape["CIRCLE"] = "circle";
    Shape["ELLIPSE"] = "ellipse";
    Shape["IMAGE"] = "image";
    Shape["RECT"] = "rect";
    Shape["LINE"] = "line";
    Shape["POLYLINE"] = "polyline";
    Shape["POLYGON"] = "polygon";
    Shape["TEXT"] = "text";
    Shape["PATH"] = "path";
    Shape["HTML"] = "html";
    Shape["MESH"] = "mesh";
  })(Shape$2 || (Shape$2 = {}));

  var camelCase$1 = memoize$1(function (str) {
    if (str === void 0) {
      str = '';
    }
    return str.replace(/-([a-z])/g, function (g) {
      return g[1].toUpperCase();
    });
  });

  function DCHECK$1(bool) {
    if (!bool) {
      throw new Error();
    }
  }
  function DCHECK_EQ$1(a, b) {
    if (a !== b) {
      throw new Error();
    }
  }
  function DCHECK_NE$1(a, b) {
    if (a === b) {
      throw new Error();
    }
  }
  function isFunction$3(func) {
    return typeof func === 'function';
  }
  var definedProps = function definedProps(obj) {
    return Object.fromEntries(Object.entries(obj).filter(function (_ref) {
      var v = _ref[1];
      return v !== undefined;
    }));
  };
  var FORMAT_ATTR_MAP$1 = {
    d: {
      alias: 'path'
    },
    strokeDasharray: {
      alias: 'lineDash'
    },
    strokeWidth: {
      alias: 'lineWidth'
    },
    textAnchor: {
      alias: 'textAlign'
    },
    src: {
      alias: 'img'
    }
  };
  var formatAttributeName$1 = memoize$1(function (name) {
    var attributeName = camelCase$1(name);
    var map = FORMAT_ATTR_MAP$1[attributeName];
    attributeName = (map === null || map === void 0 ? void 0 : map.alias) || attributeName;
    return attributeName;
  });

  var canvasMap$1 = {};
  var defaultCanvasIdCounter$1 = 0;
  /**
   * destroy existed canvas with the same id
   */
  function cleanExistedCanvas$1(container, canvas) {
    if (container) {
      var id = typeof container === 'string' ? container : container.id || defaultCanvasIdCounter$1++;
      if (canvasMap$1[id]) {
        canvasMap$1[id].destroy();
      }
      canvasMap$1[id] = canvas;
    }
  }
  var isBrowser$1 = typeof window !== 'undefined' && typeof window.document !== 'undefined';

  function sortByZIndex$1(o1, o2) {
    var zIndex1 = Number(o1.style.zIndex);
    var zIndex2 = Number(o2.style.zIndex);
    if (zIndex1 === zIndex2) {
      // return o1.entity.getComponent(Sortable).lastSortedIndex - o2.entity.getComponent(Sortable).lastSortedIndex;
      var parent = o1.parentNode;
      if (parent) {
        var children = parent.childNodes || [];
        return children.indexOf(o1) - children.indexOf(o2);
      }
    }
    return zIndex1 - zIndex2;
  }
  function findClosestClipPathTarget$1(object) {
    var el = object;
    do {
      var _el$style;
      var clipPath = (_el$style = el.style) === null || _el$style === void 0 ? void 0 : _el$style.clipPath;
      if (clipPath) return el;
      el = el.parentElement;
    } while (el !== null);
    return null;
  }
  function getStyle$2($el, property) {
    if (isBrowser$1) {
      return document.defaultView.getComputedStyle($el, null).getPropertyValue(property);
    }
  }
  function getWidth$1($el) {
    var width = getStyle$2($el, 'width');
    if (width === 'auto') {
      return $el.offsetWidth;
    }
    return parseFloat(width);
  }
  function getHeight$1($el) {
    var height = getStyle$2($el, 'height');
    if (height === 'auto') {
      return $el.offsetHeight;
    }
    return parseFloat(height);
  }

  var ERROR_MSG_METHOD_NOT_IMPLEMENTED$1 = 'Method not implemented.';
  var ERROR_MSG_USE_DOCUMENT_ELEMENT$1 = 'Use document.documentElement instead.';
  var ERROR_MSG_APPEND_DESTROYED_ELEMENT$1 = 'Cannot append a destroyed element.';

  // borrow from hammer.js
  var MOUSE_POINTER_ID$1 = 1;
  var TOUCH_TO_POINTER$1 = {
    touchstart: 'pointerdown',
    touchend: 'pointerup',
    touchendoutside: 'pointerupoutside',
    touchmove: 'pointermove',
    touchcancel: 'pointercancel'
  };

  function _regeneratorRuntime$6() {
    _regeneratorRuntime$6 = function () {
      return exports;
    };
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function (obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function (method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) keys.push(key);
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function (skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
      },
      stop: function () {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function (exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function (type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function (record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function (finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      catch: function (tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function (iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  function asyncGeneratorStep$7(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator$7(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep$7(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep$7(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  function _defineProperties$4(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }
  function _createClass$5(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties$4(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties$4(Constructor, staticProps);
    Object.defineProperty(Constructor, "prototype", {
      writable: false
    });
    return Constructor;
  }
  function _extends$3() {
    _extends$3 = Object.assign ? Object.assign.bind() : function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];
        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }
      return target;
    };
    return _extends$3.apply(this, arguments);
  }
  function _inheritsLoose$9(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf$b(subClass, superClass);
  }
  function _setPrototypeOf$b(o, p) {
    _setPrototypeOf$b = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };
    return _setPrototypeOf$b(o, p);
  }
  function _objectWithoutPropertiesLoose$2(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }
    return target;
  }
  function _assertThisInitialized$5(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
  }
  function _unsupportedIterableToArray$3(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray$3(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$3(o, minLen);
  }
  function _arrayLikeToArray$3(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
    return arr2;
  }
  function _createForOfIteratorHelperLoose$2(o, allowArrayLike) {
    var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
    if (it) return (it = it.call(o)).next.bind(it);
    if (Array.isArray(o) || (it = _unsupportedIterableToArray$3(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }
    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  // These units are iterated through, so be careful when adding or changing the
  // order.
  var UnitType$1;
  (function (UnitType) {
    UnitType[UnitType["kUnknown"] = 0] = "kUnknown";
    UnitType[UnitType["kNumber"] = 1] = "kNumber";
    UnitType[UnitType["kPercentage"] = 2] = "kPercentage";
    // Length units
    UnitType[UnitType["kEms"] = 3] = "kEms";
    // kExs,
    UnitType[UnitType["kPixels"] = 4] = "kPixels";
    // kCentimeters,
    // kMillimeters,
    // kInches,
    // kPoints,
    // kPicas,
    // kQuarterMillimeters,
    // https://drafts.csswg.org/css-values-4/#viewport-relative-lengths
    //
    // See also IsViewportPercentageLength.
    // kViewportWidth,
    // kViewportHeight,
    // kViewportInlineSize,
    // kViewportBlockSize,
    // kViewportMin,
    // kViewportMax,
    // kSmallViewportWidth,
    // kSmallViewportHeight,
    // kSmallViewportInlineSize,
    // kSmallViewportBlockSize,
    // kSmallViewportMin,
    // kSmallViewportMax,
    // kLargeViewportWidth,
    // kLargeViewportHeight,
    // kLargeViewportInlineSize,
    // kLargeViewportBlockSize,
    // kLargeViewportMin,
    // kLargeViewportMax,
    // kDynamicViewportWidth,
    // kDynamicViewportHeight,
    // kDynamicViewportInlineSize,
    // kDynamicViewportBlockSize,
    // kDynamicViewportMin,
    // kDynamicViewportMax,
    // https://drafts.csswg.org/css-contain-3/#container-lengths
    //
    // See also IsContainerPercentageLength.
    // kContainerWidth,
    // kContainerHeight,
    // kContainerInlineSize,
    // kContainerBlockSize,
    // kContainerMin,
    // kContainerMax,
    UnitType[UnitType["kRems"] = 5] = "kRems";
    // kChs,
    // kUserUnits, // The SVG term for unitless lengths
    // Angle units
    UnitType[UnitType["kDegrees"] = 6] = "kDegrees";
    UnitType[UnitType["kRadians"] = 7] = "kRadians";
    UnitType[UnitType["kGradians"] = 8] = "kGradians";
    UnitType[UnitType["kTurns"] = 9] = "kTurns";
    // Time units
    UnitType[UnitType["kMilliseconds"] = 10] = "kMilliseconds";
    UnitType[UnitType["kSeconds"] = 11] = "kSeconds";
    // kHertz,
    // kKilohertz,
    // Resolution
    // kDotsPerPixel,
    // kDotsPerInch,
    // kDotsPerCentimeter,
    // Other units
    // kFraction,
    UnitType[UnitType["kInteger"] = 12] = "kInteger";
    // This value is used to handle quirky margins in reflow roots (body, td,
    // and th) like WinIE. The basic idea is that a stylesheet can use the value
    // __qem (for quirky em) instead of em. When the quirky value is used, if
    // you're in quirks mode, the margin will collapse away inside a table cell.
    // This quirk is specified in the HTML spec but our impl is different.
    // TODO: Remove this. crbug.com/443952
    // kQuirkyEms,
  })(UnitType$1 || (UnitType$1 = {}));
  var UnitCategory$1;
  (function (UnitCategory) {
    UnitCategory[UnitCategory["kUNumber"] = 0] = "kUNumber";
    UnitCategory[UnitCategory["kUPercent"] = 1] = "kUPercent";
    UnitCategory[UnitCategory["kULength"] = 2] = "kULength";
    UnitCategory[UnitCategory["kUAngle"] = 3] = "kUAngle";
    UnitCategory[UnitCategory["kUTime"] = 4] = "kUTime";
    // kUFrequency,
    // kUResolution,
    UnitCategory[UnitCategory["kUOther"] = 5] = "kUOther";
  })(UnitCategory$1 || (UnitCategory$1 = {}));
  var ValueRange$1;
  (function (ValueRange) {
    ValueRange[ValueRange["kAll"] = 0] = "kAll";
    ValueRange[ValueRange["kNonNegative"] = 1] = "kNonNegative";
    ValueRange[ValueRange["kInteger"] = 2] = "kInteger";
    ValueRange[ValueRange["kNonNegativeInteger"] = 3] = "kNonNegativeInteger";
    ValueRange[ValueRange["kPositiveInteger"] = 4] = "kPositiveInteger";
  })(ValueRange$1 || (ValueRange$1 = {}));
  var Nested$1;
  (function (Nested) {
    Nested[Nested["kYes"] = 0] = "kYes";
    Nested[Nested["kNo"] = 1] = "kNo";
  })(Nested$1 || (Nested$1 = {}));
  var ParenLess$1;
  (function (ParenLess) {
    ParenLess[ParenLess["kYes"] = 0] = "kYes";
    ParenLess[ParenLess["kNo"] = 1] = "kNo";
  })(ParenLess$1 || (ParenLess$1 = {}));

  // This file specifies the unit strings used in CSSPrimitiveValues.
  var data$1 = [{
    name: 'em',
    unit_type: UnitType$1.kEms
  },
  // {
  //   name: 'ex',
  //   unit_type: UnitType.kExs,
  // },
  {
    name: 'px',
    unit_type: UnitType$1.kPixels
  },
  // {
  //   name: "cm",
  //   unit_type: UnitType.kCentimeters,
  // },
  // {
  //   name: "mm",
  //   unit_type: UnitType.kMillimeters,
  // },
  // {
  //   name: "q",
  //   unit_type: UnitType.kQuarterMillimeters,
  // },
  // {
  //   name: "in",
  //   unit_type: UnitType.kInches,
  // },
  // {
  //   name: "pt",
  //   unit_type: UnitType.kPoints,
  // },
  // {
  //   name: "pc",
  //   unit_type: UnitType.kPicas,
  // },
  {
    name: 'deg',
    unit_type: UnitType$1.kDegrees
  }, {
    name: 'rad',
    unit_type: UnitType$1.kRadians
  }, {
    name: 'grad',
    unit_type: UnitType$1.kGradians
  }, {
    name: 'ms',
    unit_type: UnitType$1.kMilliseconds
  }, {
    name: 's',
    unit_type: UnitType$1.kSeconds
  },
  // {
  //   name: "hz",
  //   unit_type: UnitType.kHertz,
  // },
  // {
  //   name: "khz",
  //   unit_type: UnitType.kKilohertz,
  // },
  // {
  //   name: "dpi",
  //   unit_type: "kDotsPerInch",
  // },
  // {
  //   name: "dpcm",
  //   unit_type: "kDotsPerCentimeter",
  // },
  // {
  //   name: "dppx",
  //   unit_type: "kDotsPerPixel",
  // },
  // {
  //   name: "x",
  //   unit_type: "kDotsPerPixel",
  // },
  // {
  //   name: "vw",
  //   unit_type: "kViewportWidth",
  // },
  // {
  //   name: "vh",
  //   unit_type: "kViewportHeight",
  // },
  // {
  //   name: "vi",
  //   unit_type: "kViewportInlineSize",
  // },
  // {
  //   name: "vb",
  //   unit_type: "kViewportBlockSize",
  // },
  // {
  //   name: "vmin",
  //   unit_type: UnitType.kViewportMin,
  // },
  // {
  //   name: "vmax",
  //   unit_type: UnitType.kViewportMax,
  // },
  // {
  //   name: "svw",
  //   unit_type: "kSmallViewportWidth",
  // },
  // {
  //   name: "svh",
  //   unit_type: "kSmallViewportHeight",
  // },
  // {
  //   name: "svi",
  //   unit_type: "kSmallViewportInlineSize",
  // },
  // {
  //   name: "svb",
  //   unit_type: "kSmallViewportBlockSize",
  // },
  // {
  //   name: "svmin",
  //   unit_type: "kSmallViewportMin",
  // },
  // {
  //   name: "svmax",
  //   unit_type: "kSmallViewportMax",
  // },
  // {
  //   name: "lvw",
  //   unit_type: "kLargeViewportWidth",
  // },
  // {
  //   name: "lvh",
  //   unit_type: "kLargeViewportHeight",
  // },
  // {
  //   name: "lvi",
  //   unit_type: "kLargeViewportInlineSize",
  // },
  // {
  //   name: "lvb",
  //   unit_type: "kLargeViewportBlockSize",
  // },
  // {
  //   name: "lvmin",
  //   unit_type: UnitType.kLargeViewportMin,
  // },
  // {
  //   name: "lvmax",
  //   unit_type: UnitType.kLargeViewportMax,
  // },
  // {
  //   name: "dvw",
  //   unit_type: UnitType.kDynamicViewportWidth,
  // },
  // {
  //   name: "dvh",
  //   unit_type: UnitType.kDynamicViewportHeight,
  // },
  // {
  //   name: "dvi",
  //   unit_type: UnitType.kDynamicViewportInlineSize,
  // },
  // {
  //   name: "dvb",
  //   unit_type: UnitType.kDynamicViewportBlockSize,
  // },
  // {
  //   name: "dvmin",
  //   unit_type: UnitType.kDynamicViewportMin,
  // },
  // {
  //   name: "dvmax",
  //   unit_type: UnitType.kDynamicViewportMax,
  // },
  // {
  //   name: "cqw",
  //   unit_type: UnitType.kContainerWidth,
  // },
  // {
  //   name: "cqh",
  //   unit_type: UnitType.kContainerHeight,
  // },
  // {
  //   name: "cqi",
  //   unit_type: UnitType.kContainerInlineSize,
  // },
  // {
  //   name: "cqb",
  //   unit_type: UnitType.kContainerBlockSize,
  // },
  // {
  //   name: "cqmin",
  //   unit_type: UnitType.kContainerMin,
  // },
  // {
  //   name: "cqmax",
  //   unit_type: UnitType.kContainerMax,
  // },
  {
    name: 'rem',
    unit_type: UnitType$1.kRems
  },
  // {
  //   name: 'fr',
  //   unit_type: UnitType.kFraction,
  // },
  {
    name: 'turn',
    unit_type: UnitType$1.kTurns
  }
  // {
  //   name: 'ch',
  //   unit_type: UnitType.kChs,
  // },
  // {
  //   name: '__qem',
  //   unit_type: UnitType.kQuirkyEms,
  // },
  ];

  var CSSStyleValueType$1;
  (function (CSSStyleValueType) {
    CSSStyleValueType[CSSStyleValueType["kUnknownType"] = 0] = "kUnknownType";
    CSSStyleValueType[CSSStyleValueType["kUnparsedType"] = 1] = "kUnparsedType";
    CSSStyleValueType[CSSStyleValueType["kKeywordType"] = 2] = "kKeywordType";
    // Start of CSSNumericValue subclasses
    CSSStyleValueType[CSSStyleValueType["kUnitType"] = 3] = "kUnitType";
    CSSStyleValueType[CSSStyleValueType["kSumType"] = 4] = "kSumType";
    CSSStyleValueType[CSSStyleValueType["kProductType"] = 5] = "kProductType";
    CSSStyleValueType[CSSStyleValueType["kNegateType"] = 6] = "kNegateType";
    CSSStyleValueType[CSSStyleValueType["kInvertType"] = 7] = "kInvertType";
    CSSStyleValueType[CSSStyleValueType["kMinType"] = 8] = "kMinType";
    CSSStyleValueType[CSSStyleValueType["kMaxType"] = 9] = "kMaxType";
    CSSStyleValueType[CSSStyleValueType["kClampType"] = 10] = "kClampType";
    // End of CSSNumericValue subclasses
    CSSStyleValueType[CSSStyleValueType["kTransformType"] = 11] = "kTransformType";
    CSSStyleValueType[CSSStyleValueType["kPositionType"] = 12] = "kPositionType";
    CSSStyleValueType[CSSStyleValueType["kURLImageType"] = 13] = "kURLImageType";
    CSSStyleValueType[CSSStyleValueType["kColorType"] = 14] = "kColorType";
    CSSStyleValueType[CSSStyleValueType["kUnsupportedColorType"] = 15] = "kUnsupportedColorType";
  })(CSSStyleValueType$1 || (CSSStyleValueType$1 = {}));
  // function parseCSSStyleValue(propertyName: string, value: string): CSSStyleValue[] {
  //   // const propertyId = cssPropertyID(propertyName);
  //   // if (propertyId === CSSPropertyID.kInvalid) {
  //   //   return [];
  //   // }
  //   // const customPropertyName = propertyId === CSSPropertyID.kVariable ? propertyName : null;
  //   // return fromString(propertyId, customPropertyName, value);
  //   return [];
  // }
  var stringToUnitType$1 = function stringToUnitType(name) {
    return data$1.find(function (item) {
      return item.name === name;
    }).unit_type;
  };
  var unitFromName$1 = function unitFromName(name) {
    if (!name) {
      return UnitType$1.kUnknown;
    }
    if (name === 'number') {
      return UnitType$1.kNumber;
    }
    if (name === 'percent' || name === '%') {
      return UnitType$1.kPercentage;
    }
    return stringToUnitType$1(name);
  };
  var unitTypeToUnitCategory$1 = function unitTypeToUnitCategory(type) {
    switch (type) {
      case UnitType$1.kNumber:
      case UnitType$1.kInteger:
        return UnitCategory$1.kUNumber;
      case UnitType$1.kPercentage:
        return UnitCategory$1.kUPercent;
      case UnitType$1.kPixels:
        // case UnitType.kCentimeters:
        // case UnitType.kMillimeters:
        // case UnitType.kQuarterMillimeters:
        // case UnitType.kInches:
        // case UnitType.kPoints:
        // case UnitType.kPicas:
        // case UnitType.kUserUnits:
        return UnitCategory$1.kULength;
      case UnitType$1.kMilliseconds:
      case UnitType$1.kSeconds:
        return UnitCategory$1.kUTime;
      case UnitType$1.kDegrees:
      case UnitType$1.kRadians:
      case UnitType$1.kGradians:
      case UnitType$1.kTurns:
        return UnitCategory$1.kUAngle;
      // case UnitType.kHertz:
      // case UnitType.kKilohertz:
      //   return UnitCategory.kUFrequency;
      // case UnitType.kDotsPerPixel:
      // case UnitType.kDotsPerInch:
      // case UnitType.kDotsPerCentimeter:
      //   return UnitCategory.kUResolution;
      default:
        return UnitCategory$1.kUOther;
    }
  };
  var canonicalUnitTypeForCategory$1 = function canonicalUnitTypeForCategory(category) {
    // The canonical unit type is chosen according to the way
    // CSSPropertyParser.ValidUnit() chooses the default unit in each category
    // (based on unitflags).
    switch (category) {
      case UnitCategory$1.kUNumber:
        return UnitType$1.kNumber;
      case UnitCategory$1.kULength:
        return UnitType$1.kPixels;
      case UnitCategory$1.kUPercent:
        return UnitType$1.kPercentage;
      // return UnitType.kUnknown; // Cannot convert between numbers and percent.
      case UnitCategory$1.kUTime:
        return UnitType$1.kSeconds;
      case UnitCategory$1.kUAngle:
        return UnitType$1.kDegrees;
      // case UnitCategory.kUFrequency:
      //   return UnitType.kHertz;
      // case UnitCategory.kUResolution:
      //   return UnitType.kDotsPerPixel;
      default:
        return UnitType$1.kUnknown;
    }
  };
  /**
   * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/css_primitive_value.cc#353
   */
  var conversionToCanonicalUnitsScaleFactor$1 = function conversionToCanonicalUnitsScaleFactor(unit_type) {
    var factor = 1.0;
    // FIXME: the switch can be replaced by an array of scale factors.
    switch (unit_type) {
      // These are "canonical" units in their respective categories.
      case UnitType$1.kPixels:
      // case UnitType.kUserUnits:
      case UnitType$1.kDegrees:
      case UnitType$1.kSeconds:
        // case UnitType.kHertz:
        break;
      case UnitType$1.kMilliseconds:
        factor = 0.001;
        break;
      // case UnitType.kCentimeters:
      //   // factor = kCssPixelsPerCentimeter;
      //   break;
      // case UnitType.kDotsPerCentimeter:
      //   // factor = 1 / kCssPixelsPerCentimeter;
      //   break;
      // case UnitType.kMillimeters:
      //   // factor = kCssPixelsPerMillimeter;
      //   break;
      // case UnitType.kQuarterMillimeters:
      //   // factor = kCssPixelsPerQuarterMillimeter;
      //   break;
      // case UnitType.kInches:
      //   // factor = kCssPixelsPerInch;
      //   break;
      // case UnitType.kDotsPerInch:
      //   // factor = 1 / kCssPixelsPerInch;
      //   break;
      // case UnitType.kPoints:
      //   // factor = kCssPixelsPerPoint;
      //   break;
      // case UnitType.kPicas:
      //   // factor = kCssPixelsPerPica;
      //   break;
      case UnitType$1.kRadians:
        factor = 180 / Math.PI;
        break;
      case UnitType$1.kGradians:
        factor = 0.9;
        break;
      case UnitType$1.kTurns:
        factor = 360;
        break;
    }
    return factor;
  };
  var unitTypeToString$1 = function unitTypeToString(type) {
    switch (type) {
      case UnitType$1.kNumber:
      case UnitType$1.kInteger:
        // case UnitType.kUserUnits:
        return '';
      case UnitType$1.kPercentage:
        return '%';
      case UnitType$1.kEms:
        // case UnitType.kQuirkyEms:
        return 'em';
      // case UnitType.kExs:
      //   return 'ex';
      case UnitType$1.kRems:
        return 'rem';
      // case UnitType.kChs:
      //   return 'ch';
      case UnitType$1.kPixels:
        return 'px';
      // case UnitType.kCentimeters:
      //   return 'cm';
      // case UnitType.kDotsPerPixel:
      //   return 'dppx';
      // case UnitType.kDotsPerInch:
      //   return 'dpi';
      // case UnitType.kDotsPerCentimeter:
      //   return 'dpcm';
      // case UnitType.kMillimeters:
      //   return 'mm';
      // case UnitType.kQuarterMillimeters:
      //   return 'q';
      // case UnitType.kInches:
      //   return 'in';
      // case UnitType.kPoints:
      //   return 'pt';
      // case UnitType.kPicas:
      //   return 'pc';
      case UnitType$1.kDegrees:
        return 'deg';
      case UnitType$1.kRadians:
        return 'rad';
      case UnitType$1.kGradians:
        return 'grad';
      case UnitType$1.kMilliseconds:
        return 'ms';
      case UnitType$1.kSeconds:
        return 's';
      // case UnitType.kHertz:
      //   return 'hz';
      // case UnitType.kKilohertz:
      //   return 'khz';
      case UnitType$1.kTurns:
        return 'turn';
    }
    return '';
  };
  /**
   * CSSStyleValue is the base class for all CSS values accessible from Typed OM.
   * Values that are not yet supported as specific types are also returned as base CSSStyleValues.
   *
   * Spec @see https://drafts.css-houdini.org/css-typed-om/#stylevalue-objects
   * Docs @see https://developer.mozilla.org/en-US/docs/Web/API/CSSStyleValue
   */
  var CSSStyleValue$1 = /*#__PURE__*/function () {
    function CSSStyleValue() {}
    // static parse(propertyName: string, value: string): CSSStyleValue {
    //   return parseCSSStyleValue(propertyName, value)[0];
    // }
    // static parseAll(propertyName: string, value: string): CSSStyleValue[] {
    //   return parseCSSStyleValue(propertyName, value);
    // }
    CSSStyleValue.isAngle = function isAngle(unit) {
      return unit === UnitType$1.kDegrees || unit === UnitType$1.kRadians || unit === UnitType$1.kGradians || unit === UnitType$1.kTurns;
    }
    // static isViewportPercentageLength(type: UnitType) {
    //   return type >= UnitType.kViewportWidth && type <= UnitType.kDynamicViewportMax;
    // }
    // static isContainerPercentageLength(type: UnitType) {
    //   return type >= UnitType.kContainerWidth && type <= UnitType.kContainerMax;
    // }
    ;
    CSSStyleValue.isLength = function isLength(type) {
      // return (type >= UnitType.kEms && type <= UnitType.kUserUnits) || type == UnitType.kQuirkyEms;
      return type >= UnitType$1.kEms && type < UnitType$1.kDegrees;
    };
    CSSStyleValue.isRelativeUnit = function isRelativeUnit(type) {
      return type === UnitType$1.kPercentage || type === UnitType$1.kEms ||
      // type === UnitType.kExs ||
      type === UnitType$1.kRems
      // type === UnitType.kChs ||
      // this.isViewportPercentageLength(type) ||
      // this.isContainerPercentageLength(type)
      ;
    };
    CSSStyleValue.isTime = function isTime(unit) {
      return unit === UnitType$1.kSeconds || unit === UnitType$1.kMilliseconds;
    }
    // protected abstract toCSSValue(): CSSValue;
    ;
    var _proto = CSSStyleValue.prototype;
    _proto.toString = function toString() {
      return this.buildCSSText(Nested$1.kNo, ParenLess$1.kNo, '');
    };
    _proto.isNumericValue = function isNumericValue() {
      return this.getType() >= CSSStyleValueType$1.kUnitType && this.getType() <= CSSStyleValueType$1.kClampType;
    };
    return CSSStyleValue;
  }();

  /**
   * CSSColorValue is the base class used for the various CSS color interfaces.
   *
   * @see https://drafts.css-houdini.org/css-typed-om-1/#colorvalue-objects
   */
  var CSSColorValue$1 = /*#__PURE__*/function (_CSSStyleValue) {
    _inheritsLoose$9(CSSColorValue, _CSSStyleValue);
    function CSSColorValue(colorSpace) {
      var _this;
      _this = _CSSStyleValue.call(this) || this;
      _this.colorSpace = void 0;
      _this.colorSpace = colorSpace;
      return _this;
    }
    var _proto = CSSColorValue.prototype;
    _proto.getType = function getType() {
      return CSSStyleValueType$1.kColorType;
    }
    // buildCSSText(n: Nested, p: ParenLess, result: string): string {
    //   let text = '';
    //   if (this.colorSpace === 'rgb') {
    //     text = `rgba(${this.channels.join(',')},${this.alpha})`;
    //   }
    //   return (result += text);
    // }
    /**
     * @see https://drafts.css-houdini.org/css-typed-om-1/#dom-csscolorvalue-to
     */;
    _proto.to = function to(colorSpace) {
      return this;
    };
    return CSSColorValue;
  }(CSSStyleValue$1);

  var GradientType$1;
  (function (GradientType) {
    GradientType[GradientType["Constant"] = 0] = "Constant";
    GradientType[GradientType["LinearGradient"] = 1] = "LinearGradient";
    GradientType[GradientType["RadialGradient"] = 2] = "RadialGradient";
  })(GradientType$1 || (GradientType$1 = {}));
  var CSSGradientValue$1 = /*#__PURE__*/function (_CSSStyleValue) {
    _inheritsLoose$9(CSSGradientValue, _CSSStyleValue);
    function CSSGradientValue(type, value) {
      var _this;
      _this = _CSSStyleValue.call(this) || this;
      _this.type = void 0;
      _this.value = void 0;
      _this.type = type;
      _this.value = value;
      return _this;
    }
    var _proto = CSSGradientValue.prototype;
    _proto.clone = function clone() {
      return new CSSGradientValue(this.type, this.value);
    };
    _proto.buildCSSText = function buildCSSText(n, p, result) {
      return result;
    };
    _proto.getType = function getType() {
      return CSSStyleValueType$1.kColorType;
    };
    return CSSGradientValue;
  }(CSSStyleValue$1);

  /**
   * CSSKeywordValue represents CSS Values that are specified as keywords
   * eg. 'initial'
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSKeywordValue
   * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_keyword_value.idl
   */
  var CSSKeywordValue$1 = /*#__PURE__*/function (_CSSStyleValue) {
    _inheritsLoose$9(CSSKeywordValue, _CSSStyleValue);
    function CSSKeywordValue(value) {
      var _this;
      _this = _CSSStyleValue.call(this) || this;
      _this.value = void 0;
      _this.value = value;
      return _this;
    }
    var _proto = CSSKeywordValue.prototype;
    _proto.clone = function clone() {
      return new CSSKeywordValue(this.value);
    };
    _proto.getType = function getType() {
      return CSSStyleValueType$1.kKeywordType;
    };
    _proto.buildCSSText = function buildCSSText(n, p, result) {
      return result + this.value;
    };
    return CSSKeywordValue;
  }(CSSStyleValue$1);

  var CSSMathOperator$1;
  (function (CSSMathOperator) {
    CSSMathOperator["kAdd"] = "+";
    CSSMathOperator["kSubtract"] = "-";
    CSSMathOperator["kMultiply"] = "*";
    CSSMathOperator["kDivide"] = "/";
    CSSMathOperator["kMin"] = "min";
    CSSMathOperator["kMax"] = "max";
    CSSMathOperator["kClamp"] = "clamp";
    CSSMathOperator["kInvalid"] = "";
  })(CSSMathOperator$1 || (CSSMathOperator$1 = {}));

  // https://drafts.css-houdini.org/css-typed-om/#enumdef-cssnumericbasetype
  var BaseType$1;
  (function (BaseType) {
    BaseType[BaseType["kLength"] = 0] = "kLength";
    BaseType[BaseType["kAngle"] = 1] = "kAngle";
    BaseType[BaseType["kTime"] = 2] = "kTime";
    BaseType[BaseType["kFrequency"] = 3] = "kFrequency";
    BaseType[BaseType["kResolution"] = 4] = "kResolution";
    BaseType[BaseType["kFlex"] = 5] = "kFlex";
    BaseType[BaseType["kPercent"] = 6] = "kPercent";
    BaseType[BaseType["kNumBaseTypes"] = 7] = "kNumBaseTypes";
  })(BaseType$1 || (BaseType$1 = {}));
  function unitTypeToBaseType$1(unit) {
    DCHECK_NE$1(unit, UnitType$1.kNumber);
    switch (unit) {
      case UnitType$1.kEms:
      // case UnitType.kExs:
      case UnitType$1.kPixels:
      // case UnitType.kCentimeters:
      // case UnitType.kMillimeters:
      // case UnitType.kQuarterMillimeters:
      // case UnitType.kInches:
      // case UnitType.kPoints:
      // case UnitType.kPicas:
      // case UnitType.kUserUnits:
      // case UnitType.kViewportWidth:
      // case UnitType.kViewportHeight:
      // case UnitType.kViewportInlineSize:
      // case UnitType.kViewportBlockSize:
      // case UnitType.kViewportMin:
      // case UnitType.kViewportMax:
      // case UnitType.kSmallViewportWidth:
      // case UnitType.kSmallViewportHeight:
      // case UnitType.kSmallViewportInlineSize:
      // case UnitType.kSmallViewportBlockSize:
      // case UnitType.kSmallViewportMin:
      // case UnitType.kSmallViewportMax:
      // case UnitType.kLargeViewportWidth:
      // case UnitType.kLargeViewportHeight:
      // case UnitType.kLargeViewportInlineSize:
      // case UnitType.kLargeViewportBlockSize:
      // case UnitType.kLargeViewportMin:
      // case UnitType.kLargeViewportMax:
      // case UnitType.kDynamicViewportWidth:
      // case UnitType.kDynamicViewportHeight:
      // case UnitType.kDynamicViewportInlineSize:
      // case UnitType.kDynamicViewportBlockSize:
      // case UnitType.kDynamicViewportMin:
      // case UnitType.kDynamicViewportMax:
      // case UnitType.kContainerWidth:
      // case UnitType.kContainerHeight:
      // case UnitType.kContainerInlineSize:
      // case UnitType.kContainerBlockSize:
      // case UnitType.kContainerMin:
      // case UnitType.kContainerMax:
      case UnitType$1.kRems:
        // case UnitType.kChs:
        return BaseType$1.kLength;
      case UnitType$1.kMilliseconds:
      case UnitType$1.kSeconds:
        return BaseType$1.kTime;
      case UnitType$1.kDegrees:
      case UnitType$1.kRadians:
      case UnitType$1.kGradians:
      case UnitType$1.kTurns:
        return BaseType$1.kAngle;
      // case UnitType.kHertz:
      // case UnitType.kKilohertz:
      //   return BaseType.kFrequency;
      // case UnitType.kDotsPerPixel:
      // case UnitType.kDotsPerInch:
      // case UnitType.kDotsPerCentimeter:
      //   return BaseType.kResolution;
      // case UnitType.kFraction:
      //   return BaseType.kFlex;
      case UnitType$1.kPercentage:
        return BaseType$1.kPercent;
      default:
        return BaseType$1.kLength;
    }
  }
  function baseTypeToString$1(baseType) {
    switch (baseType) {
      case BaseType$1.kLength:
        return 'length';
      case BaseType$1.kAngle:
        return 'angle';
      case BaseType$1.kTime:
        return 'time';
      case BaseType$1.kFrequency:
        return 'frequency';
      case BaseType$1.kResolution:
        return 'resolution';
      case BaseType$1.kFlex:
        return 'flex';
      case BaseType$1.kPercent:
        return 'percent';
    }
    return '';
  }
  var CSSNumericValueType$1 = /*#__PURE__*/function () {
    function CSSNumericValueType(unit, exponent) {
      if (unit === void 0) {
        unit = UnitType$1.kNumber;
      }
      if (exponent === void 0) {
        exponent = 1;
      }
      this.exponents = [];
      this.numNonZeroEntries = 0;
      this.percentHint = BaseType$1.kPercent;
      this.hasPercentHint = false;
      this.exponents = new Array(BaseType$1.kNumBaseTypes).fill(0);
      if (unit !== UnitType$1.kNumber) {
        this.setExponent(unitTypeToBaseType$1(unit), exponent);
      }
    }
    var _proto = CSSNumericValueType.prototype;
    _proto.applyPercentHint = function applyPercentHint(hint) {
      DCHECK_NE$1(hint, BaseType$1.kPercent);
      this.setExponent(hint, this.exponent(hint) + this.exponent(BaseType$1.kPercent));
      this.setExponent(BaseType$1.kPercent, 0);
      this.percentHint = hint;
      this.hasPercentHint = true;
    };
    _proto.hasNonZeroEntries = function hasNonZeroEntries() {
      return this.numNonZeroEntries > 0;
    };
    _proto.isOnlyNonZeroEntry = function isOnlyNonZeroEntry(baseType, value) {
      DCHECK_NE$1(value, 0);
      return this.numNonZeroEntries === 1 && this.exponent(baseType) === value;
    };
    _proto.exponent = function exponent(type) {
      return this.exponents[type];
    };
    _proto.setExponent = function setExponent(type, newValue) {
      var oldValue = this.exponents[type];
      if (oldValue == 0 && newValue !== 0) {
        this.numNonZeroEntries++;
      } else if (oldValue !== 0 && newValue == 0) {
        this.numNonZeroEntries--;
      }
      this.exponents[type] = newValue;
    };
    CSSNumericValueType.negateExponents = function negateExponents(type) {
      type.exponents.forEach(function (v) {
        return v *= -1;
      });
      return type;
    };
    _proto.matchesBaseType = function matchesBaseType(baseType) {
      DCHECK_NE$1(baseType, BaseType$1.kPercent);
      return this.isOnlyNonZeroEntry(baseType, 1) && !this.hasPercentHint;
    };
    _proto.matchesPercentage = function matchesPercentage() {
      return this.isOnlyNonZeroEntry(BaseType$1.kPercent, 1);
    };
    _proto.matchesBaseTypePercentage = function matchesBaseTypePercentage(baseType) {
      DCHECK_NE$1(baseType, BaseType$1.kPercent);
      return this.isOnlyNonZeroEntry(baseType, 1) || this.isOnlyNonZeroEntry(BaseType$1.kPercent, 1);
    };
    _proto.matchesNumber = function matchesNumber() {
      return !this.hasNonZeroEntries() && !this.hasPercentHint;
    };
    _proto.matchesNumberPercentage = function matchesNumberPercentage() {
      return !this.hasNonZeroEntries() || this.isOnlyNonZeroEntry(BaseType$1.kPercent, 1);
    };
    CSSNumericValueType.add = function add(type1, type2, error) {
      if (type1.hasPercentHint && type2.hasPercentHint && type1.percentHint != type2.percentHint) {
        return type1;
      }
      if (type1.hasPercentHint) type2.applyPercentHint(type1.percentHint);else if (type2.hasPercentHint) type1.applyPercentHint(type2.percentHint);
      DCHECK_EQ$1(type1.percentHint, type2.percentHint);
      // Match up base types. Try to use the percent hint to match up any
      // differences.
      for (var i = 0; i < BaseType$1.kNumBaseTypes; ++i) {
        var base_type = i;
        if (type1.exponents[i] !== type2.exponents[i]) {
          if (base_type !== BaseType$1.kPercent) {
            type1.applyPercentHint(base_type);
            type2.applyPercentHint(base_type);
          }
          if (type1.exponents[i] !== type2.exponents[i]) {
            return type1;
          }
        }
      }
      return type1;
    };
    CSSNumericValueType.multiply = function multiply(type1, type2, error) {
      if (type1.hasPercentHint && type2.hasPercentHint && type1.percentHint != type2.percentHint) {
        return type1;
      }
      if (type1.hasPercentHint) type2.applyPercentHint(type1.percentHint);else if (type2.hasPercentHint) type1.applyPercentHint(type2.percentHint);
      for (var i = 0; i < BaseType$1.kNumBaseTypes; ++i) {
        var base_type = i;
        type1.setExponent(base_type, type1.exponent(base_type) + type2.exponent(base_type));
      }
      return type1;
    };
    return CSSNumericValueType;
  }();
  var fromNumberish$1 = function fromNumberish(value) {
    if (typeof value === 'number') {
      return new CSSUnitValue$1(value, UnitType$1.kNumber);
    }
    return value;
  };
  /**
   * CSSNumericValue is the base class for numeric and length typed CSS Values.
   * @see https://drafts.css-houdini.org/css-typed-om/#numeric-objects
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue
   * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.idl
   */
  var CSSNumericValue$1 = /*#__PURE__*/function (_CSSStyleValue) {
    _inheritsLoose$9(CSSNumericValue, _CSSStyleValue);
    /**
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#296
     */
    // static fromPercentish(value: CSSNumberish): CSSNumericValue {
    //   if (typeof value === 'number') {
    //     return new CSSUnitValue(value * 100, UnitType.kPercentage);
    //   }
    //   return value;
    // }
    function CSSNumericValue(type_) {
      var _this;
      _this = _CSSStyleValue.call(this) || this;
      _this.type_ = void 0;
      _this.type_ = type_;
      return _this;
    }
    var _proto2 = CSSNumericValue.prototype;
    _proto2.getType = function getType() {
      return CSSStyleValueType$1.kUnknownType;
    }
    // toCSSValue() {
    //   return null;
    // }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/equals
     */;
    _proto2.equals = function equals() {
      var _this2 = this;
      for (var _len = arguments.length, numberishes = new Array(_len), _key = 0; _key < _len; _key++) {
        numberishes[_key] = arguments[_key];
      }
      var values = cssNumberishesToNumericValues$1(numberishes);
      return values.every(function (value) {
        return _this2.equals(value);
      });
    }
    /**
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#439
     */;
    _proto2.add = function add() {
      for (var _len2 = arguments.length, numberishes = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        numberishes[_key2] = arguments[_key2];
      }
      var values = cssNumberishesToNumericValues$1(numberishes);
      prependValueForArithmetic$1(CSSStyleValueType$1.kSumType, values, this);
      // eg. 1px + 2px = 3px
      var unitValue = maybeSimplifyAsUnitValue$1(values, CSSMathOperator$1.kAdd);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathSum$1.create(values);
    }
    /**
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#452
     */;
    _proto2.sub = function sub() {
      for (var _len3 = arguments.length, numberishes = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        numberishes[_key3] = arguments[_key3];
      }
      var values = cssNumberishesToNumericValues$1(numberishes);
      values = values.map(function (value) {
        return value.negate();
      });
      prependValueForArithmetic$1(CSSStyleValueType$1.kSumType, values, this);
      // eg. 3px - 2px = 1px
      var unitValue = maybeSimplifyAsUnitValue$1(values, CSSMathOperator$1.kAdd);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathSum$1.create(values);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/mul
     */;
    _proto2.mul = function mul() {
      for (var _len4 = arguments.length, numberishes = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        numberishes[_key4] = arguments[_key4];
      }
      var values = cssNumberishesToNumericValues$1(numberishes);
      prependValueForArithmetic$1(CSSStyleValueType$1.kProductType, values, this);
      var unitValue = maybeMultiplyAsUnitValue$1(values);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathProduct$1.create(values);
    }
    /**
     * eg. calc(24px / 4%)
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/div
     */;
    _proto2.div = function div() {
      for (var _len5 = arguments.length, numberishes = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        numberishes[_key5] = arguments[_key5];
      }
      var values = cssNumberishesToNumericValues$1(numberishes);
      values = values.map(function (value) {
        return value.invert();
      });
      prependValueForArithmetic$1(CSSStyleValueType$1.kProductType, values, this);
      var unitValue = maybeMultiplyAsUnitValue$1(values);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathProduct$1.create(values);
    };
    _proto2.min = function min() {
      for (var _len6 = arguments.length, numberishes = new Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        numberishes[_key6] = arguments[_key6];
      }
      var values = cssNumberishesToNumericValues$1(numberishes);
      prependValueForArithmetic$1(CSSStyleValueType$1.kMinType, values, this);
      var unitValue = maybeSimplifyAsUnitValue$1(values, CSSMathOperator$1.kMin);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathMin$1.create(values);
    };
    _proto2.max = function max() {
      for (var _len7 = arguments.length, numberishes = new Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
        numberishes[_key7] = arguments[_key7];
      }
      var values = cssNumberishesToNumericValues$1(numberishes);
      prependValueForArithmetic$1(CSSStyleValueType$1.kMaxType, values, this);
      var unitValue = maybeSimplifyAsUnitValue$1(values, CSSMathOperator$1.kMax);
      if (unitValue) {
        return unitValue;
      }
      return CSSMathMax$1.create(values);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/to
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#331
     */;
    _proto2.to = function to(unitOrName) {
      var sum = this.sumValue();
      if (sum.length === 0 || sum.length !== 1) return null;
      var value = cssNumericSumValueEntryToUnitValue$1(sum[0]);
      if (!value) return null;
      var unit;
      if (typeof unitOrName === 'string') {
        unit = unitFromName$1(unitOrName);
      } else {
        unit = unitOrName;
      }
      return value.convertTo(unit);
    }
    /**
     * converts the object's value to a CSSMathSum object to values of the specified unit.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/toSum
     */;
    _proto2.toSum = function toSum() {
      for (var _len8 = arguments.length, unit_strings = new Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
        unit_strings[_key8] = arguments[_key8];
      }
      for (var _i = 0, _unit_strings = unit_strings; _i < _unit_strings.length; _i++) {
        var unit_string = _unit_strings[_i];
        if (!CSSNumericValue.isValidUnit(unitFromName$1(unit_string))) {
          return null;
        }
      }
      var sum = this.sumValue();
      if (!sum.length) {
        return null;
      }
      var values = [];
      for (var _iterator = _createForOfIteratorHelperLoose$2(sum), _step; !(_step = _iterator()).done;) {
        var term = _step.value;
        var value = cssNumericSumValueEntryToUnitValue$1(term);
        if (!value) {
          return null;
        }
        values.push(value);
      }
      if (unit_strings.length === 0) {
        values.sort(function (a, b) {
          return a.unit - b.unit;
        });
        // We got 'values' from a sum value, so it must be a valid CSSMathSum.
        var _result = CSSMathSum$1.create(values);
        DCHECK$1(!!_result);
        return _result;
      }
      var result = [];
      var _loop = function _loop() {
        var unit_string = _unit_strings2[_i2];
        var target_unit = unitFromName$1(unit_string);
        DCHECK$1(CSSNumericValue.isValidUnit(target_unit));
        // Collect all the terms that are compatible with this unit.
        // We mark used terms as null so we don't use them again.
        var total_value = values.reduce(function (cur_sum, value, i) {
          if (value) {
            var unit_value = value;
            var converted_value = unit_value.convertTo(target_unit);
            if (converted_value) {
              cur_sum += converted_value.value;
              values[i] = null;
            }
          }
          return cur_sum;
        }, 0);
        result.push(new CSSUnitValue$1(total_value, target_unit));
      };
      for (var _i2 = 0, _unit_strings2 = unit_strings; _i2 < _unit_strings2.length; _i2++) {
        _loop();
      }
      if (values.some(function (v) {
        return !!v;
      })) {
        throw new Error('There were leftover terms that were not converted');
      }
      return CSSMathSum$1.create(result);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSNumericValue/type
     * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_numeric_value.cc#414
     */;
    _proto2.type = function type() {
      var type = {
        length: 0,
        angle: 0,
        time: 0,
        frequency: 0,
        resolution: 0,
        flex: 0,
        percent: 0,
        percentHint: 'length'
      };
      var exponent = this.type_.exponent(BaseType$1.kLength);
      if (exponent) {
        type.length = exponent;
      }
      exponent = this.type_.exponent(BaseType$1.kAngle);
      if (exponent) {
        type.angle = exponent;
      }
      exponent = this.type_.exponent(BaseType$1.kTime);
      if (exponent) {
        type.time = exponent;
      }
      exponent = this.type_.exponent(BaseType$1.kFrequency);
      if (exponent) {
        type.frequency = exponent;
      }
      exponent = this.type_.exponent(BaseType$1.kResolution);
      if (exponent) {
        type.resolution = exponent;
      }
      exponent = this.type_.exponent(BaseType$1.kFlex);
      if (exponent) {
        type.flex = exponent;
      }
      exponent = this.type_.exponent(BaseType$1.kPercent);
      if (exponent) {
        type.percent = exponent;
      }
      if (this.type_.hasPercentHint) {
        type.percentHint = baseTypeToString$1(this.type_.percentHint);
      }
      return type;
    };
    CSSNumericValue.isValidUnit = function isValidUnit(unit) {
      // if (unit === UnitType.kUserUnits) return false;
      if (unit === UnitType$1.kNumber || unit == UnitType$1.kPercentage || this.isLength(unit) || this.isAngle(unit) || this.isTime(unit)
      // this.isFrequency(unit) ||
      // this.isResolution(unit) ||
      // this.isFlex(unit)
      ) return true;
      return false;
    };
    _proto2.negate = function negate() {
      return CSSMathNegate$1.create(this);
    };
    _proto2.invert = function invert() {
      return CSSMathInvert$1.create(this);
    };
    return CSSNumericValue;
  }(CSSStyleValue$1);
  function cssNumberishesToNumericValues$1(values) {
    return values.map(fromNumberish$1);
  }
  function prependValueForArithmetic$1(type, values, value) {
    DCHECK$1(!!value);
    if (value.getType() === type) {
      values.unshift.apply(values, value.numericValues());
    } else {
      values.unshift(value);
    }
  }
  function cssNumericSumValueEntryToUnitValue$1(term) {
    if (Object.keys(term.units).length === 0) {
      return new CSSUnitValue$1(term.value);
    }
    if (Object.keys(term.units).length === 1 && term.units[Object.keys(term.units)[0]] === 1) {
      return new CSSUnitValue$1(term.value, Number(Object.keys(term.units)[0]));
    }
    return null;
  }
  function maybeSimplifyAsUnitValue$1(values, operator) {
    DCHECK$1(!!values.length);
    var first_unit_value = values[0] instanceof CSSUnitValue$1 ? values[0] : null;
    if (!first_unit_value) return null;
    var final_value = first_unit_value.value;
    for (var i = 1; i < values.length; i++) {
      var unit_value = values[i] instanceof CSSUnitValue$1 ? values[i] : null;
      if (!unit_value || unit_value.unit !== first_unit_value.unit) return null;
      if (operator === CSSMathOperator$1.kAdd) {
        final_value += unit_value.value;
      } else if (operator === CSSMathOperator$1.kMax) {
        final_value = Math.max(final_value, unit_value.value);
      } else if (operator === CSSMathOperator$1.kMin) {
        final_value = Math.min(final_value, unit_value.value);
      }
    }
    return new CSSUnitValue$1(final_value, first_unit_value.unit);
  }
  function maybeMultiplyAsUnitValue$1(values) {
    DCHECK$1(!!values.length);
    // We are allowed one unit value with type other than kNumber.
    var unit_other_than_number = UnitType$1.kNumber;
    var final_value = 1.0;
    for (var i = 0; i < values.length; i++) {
      var unit_value = values[i] instanceof CSSUnitValue$1 ? values[i] : null;
      if (!unit_value) return null;
      if (unit_value.unit !== UnitType$1.kNumber) {
        if (unit_other_than_number !== UnitType$1.kNumber) return null;
        unit_other_than_number = unit_value.unit;
      }
      final_value *= unit_value.value;
    }
    return new CSSUnitValue$1(final_value, unit_other_than_number);
  }
  var toCanonicalUnit$1 = function toCanonicalUnit(unit) {
    return canonicalUnitTypeForCategory$1(unitTypeToUnitCategory$1(unit));
  };
  var formatInfinityOrNaN$1 = function formatInfinityOrNaN(number, suffix) {
    if (suffix === void 0) {
      suffix = '';
    }
    var result = '';
    if (!Number.isFinite(number)) {
      if (number > 0) result = 'infinity';else result = '-infinity';
    } else {
      DCHECK$1(Number.isNaN(number));
      result = 'NaN';
    }
    return result += suffix;
  };
  var toCanonicalUnitIfPossible$1 = function toCanonicalUnitIfPossible(unit) {
    var canonical_unit = toCanonicalUnit$1(unit);
    if (canonical_unit === UnitType$1.kUnknown) return unit;
    return canonical_unit;
  };
  /**
   * Represents numeric values that can be expressed as a single number plus a
   * unit (or a naked number or percentage).
   * @see https://drafts.css-houdini.org/css-typed-om/#cssunitvalue
   */
  var CSSUnitValue$1 = /*#__PURE__*/function (_CSSNumericValue) {
    _inheritsLoose$9(CSSUnitValue, _CSSNumericValue);
    // static fromCSSValue(value: CSSNumericLiteralValue) {
    //   let unit = value.type;
    //   if (unit === UnitType.kInteger) unit = UnitType.kNumber;
    //   if (!this.isValidUnit(unit)) return null;
    //   return new CSSUnitValue(value.getDoubleValue(), unit);
    // }

    function CSSUnitValue(value, unitOrName) {
      var _this3;
      if (unitOrName === void 0) {
        unitOrName = UnitType$1.kNumber;
      }
      var unit;
      if (typeof unitOrName === 'string') {
        unit = unitFromName$1(unitOrName);
      } else {
        unit = unitOrName;
      }
      // DCHECK(CSSUnitValue.isValidUnit(unit));
      _this3 = _CSSNumericValue.call(this, new CSSNumericValueType$1(unit)) || this;
      _this3.unit = void 0;
      _this3.value = void 0;
      _this3.unit = unit;
      _this3.value = value;
      return _this3;
    }
    var _proto3 = CSSUnitValue.prototype;
    _proto3.clone = function clone() {
      return new CSSUnitValue(this.value, this.unit);
    };
    _proto3.convertTo = function convertTo(target_unit) {
      if (this.unit === target_unit) {
        return new CSSUnitValue(this.value, this.unit);
      }
      // Instead of defining the scale factors for every unit to every other unit,
      // we simply convert to the canonical unit and back since we already have
      // the scale factors for canonical units.
      var canonical_unit = toCanonicalUnit$1(this.unit);
      if (canonical_unit !== toCanonicalUnit$1(target_unit) || canonical_unit === UnitType$1.kUnknown) {
        return null;
      }
      var scale_factor = conversionToCanonicalUnitsScaleFactor$1(this.unit) / conversionToCanonicalUnitsScaleFactor$1(target_unit);
      return new CSSUnitValue(this.value * scale_factor, target_unit);
    };
    _proto3.equals = function equals(other) {
      var other_unit_value = other;
      return this.value === other_unit_value.value && this.unit === other_unit_value.unit;
    };
    _proto3.getType = function getType() {
      return CSSStyleValueType$1.kUnitType;
    }
    // toCSSValue() {
    //   return new CSSNumericLiteralValue(this.value, this.unit);
    // }
    // // const CSSPrimitiveValue* ToCSSValueWithProperty(CSSPropertyID) const final;
    // toCalcExpressionNode() {
    //   return CSSMathExpressionNumericLiteral.create(
    //     new CSSNumericLiteralValue(this.value, this.unit),
    //   );
    // }
    ;
    _proto3.sumValue = function sumValue() {
      var sum = [];
      var unit_map = {};
      if (this.unit !== UnitType$1.kNumber) {
        unit_map[toCanonicalUnitIfPossible$1(this.unit)] = 1;
      }
      sum.push({
        value: this.value * conversionToCanonicalUnitsScaleFactor$1(this.unit),
        units: unit_map
      });
      return sum;
    };
    _proto3.negate = function negate() {
      return new CSSUnitValue(-this.value, this.unit);
    };
    _proto3.invert = function invert() {
      if (this.unit === UnitType$1.kNumber) {
        if (this.value === 0) return null;
        return new CSSUnitValue(1.0 / this.value, this.unit);
      }
      return CSSMathInvert$1.create(this);
    };
    _proto3.buildCSSText = function buildCSSText(n, p, result) {
      var text;
      switch (this.unit) {
        case UnitType$1.kUnknown:
          // FIXME
          break;
        case UnitType$1.kInteger:
          text = Number(this.value).toFixed(0);
          break;
        case UnitType$1.kNumber:
        case UnitType$1.kPercentage:
        case UnitType$1.kEms:
        // case UnitType.kQuirkyEms:
        // case UnitType.kExs:
        case UnitType$1.kRems:
        // case UnitType.kChs:
        case UnitType$1.kPixels:
        // case UnitType.kCentimeters:
        // case UnitType.kDotsPerPixel:
        // case UnitType.kDotsPerInch:
        // case UnitType.kDotsPerCentimeter:
        // case UnitType.kMillimeters:
        // case UnitType.kQuarterMillimeters:
        // case UnitType.kInches:
        // case UnitType.kPoints:
        // case UnitType.kPicas:
        // case UnitType.kUserUnits:
        case UnitType$1.kDegrees:
        case UnitType$1.kRadians:
        case UnitType$1.kGradians:
        case UnitType$1.kMilliseconds:
        case UnitType$1.kSeconds:
        // case UnitType.kHertz:
        // case UnitType.kKilohertz:
        case UnitType$1.kTurns:
          // case UnitType.kDynamicViewportMax: // case UnitType.kDynamicViewportMin: // case UnitType.kDynamicViewportBlockSize: // case UnitType.kDynamicViewportInlineSize: // case UnitType.kDynamicViewportHeight: // case UnitType.kDynamicViewportWidth: // case UnitType.kLargeViewportMax: // case UnitType.kLargeViewportMin: // case UnitType.kLargeViewportBlockSize: // case UnitType.kLargeViewportInlineSize: // case UnitType.kLargeViewportHeight: // case UnitType.kLargeViewportWidth: // case UnitType.kSmallViewportMax: // case UnitType.kSmallViewportMin: // case UnitType.kSmallViewportBlockSize: // case UnitType.kSmallViewportInlineSize: // case UnitType.kSmallViewportHeight: // case UnitType.kSmallViewportWidth: // case UnitType.kViewportMax: // case UnitType.kViewportMin: // case UnitType.kViewportBlockSize: // case UnitType.kViewportInlineSize: // case UnitType.kViewportHeight: // case UnitType.kViewportWidth: // case UnitType.kFraction:
          // case UnitType.kContainerWidth:
          // case UnitType.kContainerHeight:
          // case UnitType.kContainerInlineSize:
          // case UnitType.kContainerBlockSize:
          // case UnitType.kContainerMin:
          // case UnitType.kContainerMax: {
          {
            var kMinInteger = -999999;
            var kMaxInteger = 999999;
            var value = this.value;
            var unit = unitTypeToString$1(this.unit);
            if (value < kMinInteger || value > kMaxInteger) {
              var _unit = unitTypeToString$1(this.unit);
              if (!Number.isFinite(value) || Number.isNaN(value)) {
                text = formatInfinityOrNaN$1(value, _unit);
              } else {
                text = value + (_unit || '');
              }
            } else {
              text = "" + value + unit;
            }
          }
      }
      result += text;
      return result;
    };
    return CSSUnitValue;
  }(CSSNumericValue$1);
  /**
   * The CSSMathValue interface of the CSS_Object_Model a base class for classes representing complex numeric values.
   *
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSMathValue
   * @see https://chromium.googlesource.com/chromium/src/+/refs/heads/main/third_party/blink/renderer/core/css/cssom/css_math_value.idl
   */
  var CSSMathValue$1 = /*#__PURE__*/function (_CSSNumericValue2) {
    _inheritsLoose$9(CSSMathValue, _CSSNumericValue2);
    function CSSMathValue() {
      return _CSSNumericValue2.apply(this, arguments) || this;
    }
    var _proto4 = CSSMathValue.prototype;
    // From CSSNumericValue
    _proto4.isUnitValue = function isUnitValue() {
      return false;
    };
    return CSSMathValue;
  }(CSSNumericValue$1);
  var CSSMathInvert$1 = /*#__PURE__*/function (_CSSMathValue) {
    _inheritsLoose$9(CSSMathInvert, _CSSMathValue);
    CSSMathInvert.create = function create(value) {
      var type = CSSNumericValueType$1.negateExponents(value.type_);
      return new CSSMathInvert(value, type);
    };
    function CSSMathInvert(value, type) {
      var _this4;
      _this4 = _CSSMathValue.call(this, type) || this;
      _this4.value = void 0;
      _this4.operator = 'invert';
      _this4.value = value;
      return _this4;
    }
    var _proto5 = CSSMathInvert.prototype;
    _proto5.clone = function clone() {
      return new CSSMathInvert(this.value, this.type_);
    };
    _proto5.getType = function getType() {
      return CSSStyleValueType$1.kInvertType;
    };
    _proto5.equals = function equals(other) {
      if (other.getType() !== CSSStyleValueType$1.kInvertType) {
        return false;
      }
      // We can safely cast here as we know 'other' has the same type as us.
      var other_invert = other;
      return this.value.equals(other_invert.value);
    };
    _proto5.sumValue = function sumValue() {
      var sum = this.value.sumValue();
      if (sum.length === 0 || sum.length !== 1 || sum[0].value === 0) {
        return null;
      }
      Object.keys(sum[0].units).forEach(function (key) {
        sum[0].units[key] *= -1;
      });
      sum[0].value = 1.0 / sum[0].value;
      return sum;
    }
    // toCalcExpressionNode(): CSSMathExpressionNode {
    //   const right_side = this.value.toCalcExpressionNode();
    //   if (!right_side) {
    //     return null;
    //   }
    //   // return CSSMathExpressionOperation::CreateArithmeticOperation(
    //   //   CSSMathExpressionNumericLiteral::Create(
    //   //       1, UnitType.kNumber),
    //   //   right_side, CSSMathOperator.kDivide);
    //   return null;
    // }
    ;
    _proto5.buildCSSText = function buildCSSText(nested, paren_less, result) {
      if (paren_less == ParenLess$1.kNo) {
        result += nested === Nested$1.kYes ? '(' : 'calc(';
      }
      result += '1 / ';
      result = this.value.buildCSSText(Nested$1.kYes, ParenLess$1.kNo, result);
      if (paren_less === ParenLess$1.kNo) {
        result += ')';
      }
      return result;
    };
    return CSSMathInvert;
  }(CSSMathValue$1);
  var CSSMathNegate$1 = /*#__PURE__*/function (_CSSMathValue2) {
    _inheritsLoose$9(CSSMathNegate, _CSSMathValue2);
    CSSMathNegate.create = function create(value) {
      return new CSSMathNegate(value, value.type_);
    };
    function CSSMathNegate(value, type) {
      var _this5;
      _this5 = _CSSMathValue2.call(this, type) || this;
      _this5.value = void 0;
      _this5.operator = 'negate';
      _this5.value = value;
      return _this5;
    }
    var _proto6 = CSSMathNegate.prototype;
    _proto6.clone = function clone() {
      return new CSSMathNegate(this.value, this.type_);
    };
    _proto6.getType = function getType() {
      return CSSStyleValueType$1.kNegateType;
    };
    _proto6.equals = function equals(other) {
      if (other.getType() !== CSSStyleValueType$1.kNegateType) {
        return false;
      }
      var other_invert = other;
      return this.value.equals(other_invert.value);
    };
    _proto6.sumValue = function sumValue() {
      var sum = this.value.sumValue();
      if (sum.length === 0) {
        return null;
      }
      sum.forEach(function (term) {
        term.value *= -1;
      });
      return sum;
    }
    // toCalcExpressionNode(): CSSMathExpressionNode {
    //   const right_side = this.value.toCalcExpressionNode();
    //   if (!right_side) {
    //     return null;
    //   }
    //   // return CSSMathExpressionOperation::CreateArithmeticOperationSimplified(
    //   //   CSSMathExpressionNumericLiteral::Create(
    //   //       -1, CSSPrimitiveValue::UnitType::kNumber),
    //   //   right_side, CSSMathOperator::kMultiply);
    // }
    ;
    _proto6.buildCSSText = function buildCSSText(nested, paren_less, result) {
      if (paren_less == ParenLess$1.kNo) {
        result += nested === Nested$1.kYes ? '(' : 'calc(';
      }
      result += '-';
      result = this.value.buildCSSText(Nested$1.kYes, ParenLess$1.kNo, result);
      if (paren_less === ParenLess$1.kNo) {
        result += ')';
      }
      return result;
    };
    return CSSMathNegate;
  }(CSSMathValue$1);
  function typeCheck$1(values, op, error) {
    error = false;
    var final_type = values[0].type_;
    for (var i = 1; i < values.length; i++) {
      final_type = op(final_type, values[i].type_, error);
      if (error) return final_type;
    }
    return final_type;
  }
  // Represents an arithmetic operation with one or more CSSNumericValues.
  var CSSMathVariadic$1 = /*#__PURE__*/function (_CSSMathValue3) {
    _inheritsLoose$9(CSSMathVariadic, _CSSMathValue3);
    function CSSMathVariadic(values, type) {
      var _this6;
      _this6 = _CSSMathValue3.call(this, type) || this;
      _this6.values = void 0;
      _this6.values = values;
      return _this6;
    }
    var _proto7 = CSSMathVariadic.prototype;
    _proto7.numericValues = function numericValues() {
      return this.values;
    };
    _proto7.equals = function equals(other) {
      if (this.getType() !== other.getType()) {
        return false;
      }
      return this.values.every(function (value, i) {
        return value.equals(other.values[i]);
      });
    };
    return CSSMathVariadic;
  }(CSSMathValue$1);
  // Represents the minimum of one or more CSSNumericValues.
  // @see https://drafts.css-houdini.org/css-typed-om/#cssmathsum
  var CSSMathMin$1 = /*#__PURE__*/function (_CSSMathVariadic) {
    _inheritsLoose$9(CSSMathMin, _CSSMathVariadic);
    function CSSMathMin() {
      var _this7;
      for (var _len9 = arguments.length, args = new Array(_len9), _key9 = 0; _key9 < _len9; _key9++) {
        args[_key9] = arguments[_key9];
      }
      _this7 = _CSSMathVariadic.call.apply(_CSSMathVariadic, [this].concat(args)) || this;
      _this7.operator = 'min';
      return _this7;
    }
    CSSMathMin.create = function create(values) {
      var error = false;
      var final_type = typeCheck$1(values, CSSNumericValueType$1.add, error);
      return  new CSSMathMin(values, final_type);
    };
    var _proto8 = CSSMathMin.prototype;
    _proto8.clone = function clone() {
      return new CSSMathMin(this.values, this.type_);
    };
    _proto8.getType = function getType() {
      return CSSStyleValueType$1.kMinType;
    };
    _proto8.sumValue = function sumValue() {
      var cur_min = this.numericValues()[0].sumValue();
      if (!cur_min.length) {
        return null;
      }
      for (var _iterator2 = _createForOfIteratorHelperLoose$2(this.numericValues()), _step2; !(_step2 = _iterator2()).done;) {
        var value = _step2.value;
        var child_sum = value.sumValue();
        if (!child_sum.length || JSON.stringify(child_sum[0].units) !== JSON.stringify(cur_min[0].units)) {
          return null;
        }
        if (child_sum[0].value < cur_min[0].value) {
          cur_min = child_sum;
        }
      }
      return cur_min;
    }
    // toCalcExpressionNode() {
    //   const operands: CSSMathExpressionOperation[] = [];
    //   for (const value of this.numericValues()) {
    //     const operand = value.toCalcExpressionNode();
    //     if (!operand) {
    //       continue;
    //     }
    //     operands.push(operand as CSSMathExpressionOperation);
    //   }
    //   return CSSMathExpressionOperation.createComparisonFunction(operands, CSSMathOperator.kMin);
    // }
    ;
    _proto8.buildCSSText = function buildCSSText(nested, paren_less, result) {
      result += 'min(';
      var first_iteration = true;
      for (var _iterator3 = _createForOfIteratorHelperLoose$2(this.numericValues()), _step3; !(_step3 = _iterator3()).done;) {
        var value = _step3.value;
        if (!first_iteration) result += ', ';
        first_iteration = false;
        result = value.buildCSSText(Nested$1.kYes, ParenLess$1.kYes, result);
      }
      result += ')';
      return result;
    };
    return CSSMathMin;
  }(CSSMathVariadic$1);
  // Represents the maximum of one or more CSSNumericValues.
  // @see https://drafts.css-houdini.org/css-typed-om/#cssmathsum
  var CSSMathMax$1 = /*#__PURE__*/function (_CSSMathVariadic2) {
    _inheritsLoose$9(CSSMathMax, _CSSMathVariadic2);
    function CSSMathMax() {
      var _this8;
      for (var _len10 = arguments.length, args = new Array(_len10), _key10 = 0; _key10 < _len10; _key10++) {
        args[_key10] = arguments[_key10];
      }
      _this8 = _CSSMathVariadic2.call.apply(_CSSMathVariadic2, [this].concat(args)) || this;
      _this8.operator = 'max';
      return _this8;
    }
    CSSMathMax.create = function create(values) {
      var error = false;
      var final_type = typeCheck$1(values, CSSNumericValueType$1.add, error);
      return  new CSSMathMax(values, final_type);
    };
    var _proto9 = CSSMathMax.prototype;
    _proto9.clone = function clone() {
      return new CSSMathMax(this.values, this.type_);
    };
    _proto9.getType = function getType() {
      return CSSStyleValueType$1.kMaxType;
    };
    _proto9.sumValue = function sumValue() {
      var cur_max = this.numericValues()[0].sumValue();
      if (!cur_max.length) {
        return null;
      }
      for (var _iterator4 = _createForOfIteratorHelperLoose$2(this.numericValues()), _step4; !(_step4 = _iterator4()).done;) {
        var value = _step4.value;
        var child_sum = value.sumValue();
        if (!child_sum.length || JSON.stringify(child_sum[0].units) !== JSON.stringify(cur_max[0].units)) {
          return null;
        }
        if (child_sum[0].value < cur_max[0].value) {
          cur_max = child_sum;
        }
      }
      return cur_max;
    }
    // toCalcExpressionNode() {
    //   const operands: CSSMathExpressionOperation[] = [];
    //   for (const value of this.numericValues()) {
    //     const operand = value.toCalcExpressionNode();
    //     if (!operand) {
    //       continue;
    //     }
    //     operands.push(operand as CSSMathExpressionOperation);
    //   }
    //   return CSSMathExpressionOperation.createComparisonFunction(operands, CSSMathOperator.kMax);
    // }
    ;
    _proto9.buildCSSText = function buildCSSText(nested, paren_less, result) {
      result += 'max(';
      var first_iteration = true;
      for (var _iterator5 = _createForOfIteratorHelperLoose$2(this.numericValues()), _step5; !(_step5 = _iterator5()).done;) {
        var value = _step5.value;
        if (!first_iteration) result += ', ';
        first_iteration = false;
        result = value.buildCSSText(Nested$1.kYes, ParenLess$1.kYes, result);
      }
      result += ')';
      return result;
    };
    return CSSMathMax;
  }(CSSMathVariadic$1);
  /**
   * Represents the sum of one or more CSSNumericValues.
   * @see https://drafts.css-houdini.org/css-typed-om/#cssmathsum
   */
  var CSSMathSum$1 = /*#__PURE__*/function (_CSSMathVariadic3) {
    _inheritsLoose$9(CSSMathSum, _CSSMathVariadic3);
    function CSSMathSum() {
      var _this9;
      for (var _len11 = arguments.length, args = new Array(_len11), _key11 = 0; _key11 < _len11; _key11++) {
        args[_key11] = arguments[_key11];
      }
      _this9 = _CSSMathVariadic3.call.apply(_CSSMathVariadic3, [this].concat(args)) || this;
      _this9.operator = 'sum';
      return _this9;
    }
    CSSMathSum.create = function create(values) {
      var error = false;
      var finalType = typeCheck$1(values, CSSNumericValueType$1.add, error);
      return  new CSSMathSum(values, finalType);
    };
    CSSMathSum.numericTypeFromUnitMap = function numericTypeFromUnitMap(units) {
      var type = new CSSNumericValueType$1();
      Object.keys(units).forEach(function (key) {
        var exp = units[key];
        var error = false;
        type = CSSNumericValueType$1.multiply(type, new CSSNumericValueType$1(Number(key), exp), error);
        DCHECK$1(!error);
      });
      return type;
    };
    CSSMathSum.canCreateNumericTypeFromSumValue = function canCreateNumericTypeFromSumValue(sum) {
      var _this10 = this;
      DCHECK$1(!!sum.length);
      var first_type = this.numericTypeFromUnitMap(sum[0].units);
      return sum.every(function (term) {
        var error = false;
        CSSNumericValueType$1.add(first_type, _this10.numericTypeFromUnitMap(term.units), error);
        return !error;
      });
    };
    var _proto10 = CSSMathSum.prototype;
    _proto10.clone = function clone() {
      return new CSSMathSum(this.values, this.type_);
    };
    _proto10.getType = function getType() {
      return CSSStyleValueType$1.kSumType;
    }
    // toCalcExpressionNode() {
    //   return this.toCalcExporessionNodeForVariadic(CSSMathOperator.kAdd);
    // }
    ;
    _proto10.sumValue = function sumValue() {
      var sum = [];
      this.numericValues().forEach(function (value) {
        var child_sum = value.sumValue();
        if (!child_sum.length) {
          return null;
        }
        child_sum.forEach(function (term) {
          var index = sum.findIndex(function (s) {
            return JSON.stringify(s.units) === JSON.stringify(term.units);
          });
          if (index === -1) {
            sum.push(_extends$3({}, term));
          } else {
            sum[index].value += term.value;
          }
        });
      });
      if (!CSSMathSum.canCreateNumericTypeFromSumValue(sum)) return null;
      return sum;
    };
    _proto10.buildCSSText = function buildCSSText(nested, paren_less, result) {
      if (paren_less == ParenLess$1.kNo) {
        result += nested === Nested$1.kYes ? '(' : 'calc(';
      }
      var values = this.numericValues();
      result = values[0].buildCSSText(Nested$1.kYes, ParenLess$1.kNo, result);
      for (var i = 1; i < values.length; i++) {
        var arg = values[i];
        if (arg.getType() === CSSStyleValueType$1.kNegateType) {
          result += ' - ';
          result = arg.value.buildCSSText(Nested$1.kYes, ParenLess$1.kNo, result);
        } else {
          result += ' + ';
          result = arg.buildCSSText(Nested$1.kYes, ParenLess$1.kNo, result);
        }
      }
      if (paren_less === ParenLess$1.kNo) {
        result += ')';
      }
      return result;
    };
    return CSSMathSum;
  }(CSSMathVariadic$1);
  /**
   * Represents the product of one or more CSSNumericValues.
   * @see https://drafts.css-houdini.org/css-typed-om/#cssmathproduct
   */
  var CSSMathProduct$1 = /*#__PURE__*/function (_CSSMathVariadic4) {
    _inheritsLoose$9(CSSMathProduct, _CSSMathVariadic4);
    function CSSMathProduct() {
      var _this11;
      for (var _len12 = arguments.length, args = new Array(_len12), _key12 = 0; _key12 < _len12; _key12++) {
        args[_key12] = arguments[_key12];
      }
      _this11 = _CSSMathVariadic4.call.apply(_CSSMathVariadic4, [this].concat(args)) || this;
      _this11.operator = 'product';
      return _this11;
    }
    CSSMathProduct.create = function create(values) {
      var error = false;
      var finalType = typeCheck$1(values, CSSNumericValueType$1.multiply, error);
      return  new CSSMathProduct(values, finalType);
    };
    CSSMathProduct.multiplyUnitMaps = function multiplyUnitMaps(a, b) {
      Object.keys(b).forEach(function (key) {
        DCHECK_NE$1(b[key].value, 0);
        var old_value = key in a ? a[key].value : 0;
        if (old_value + b[key].value === 0) {
          delete a[key];
        } else {
          a[key] = old_value + b[key].value;
        }
      });
      return a;
    };
    var _proto11 = CSSMathProduct.prototype;
    _proto11.clone = function clone() {
      return new CSSMathProduct(this.values, this.type_);
    };
    _proto11.getType = function getType() {
      return CSSStyleValueType$1.kProductType;
    }
    // toCalcExpressionNode() {
    //   return this.toCalcExporessionNodeForVariadic(CSSMathOperator.kMultiply);
    // }
    ;
    _proto11.sumValue = function sumValue() {
      var sum = [{
        value: 1,
        units: {}
      }];
      this.numericValues().forEach(function (value) {
        var child_sum = value.sumValue();
        if (!child_sum.length) {
          return null;
        }
        var new_sum = [];
        sum.forEach(function (a) {
          child_sum.forEach(function (b) {
            new_sum.push({
              value: a.value * b.value,
              units: CSSMathProduct.multiplyUnitMaps(a.units, b.units)
            });
          });
        });
        sum = new_sum;
      });
      return sum;
    };
    _proto11.buildCSSText = function buildCSSText(nested, paren_less, result) {
      if (paren_less == ParenLess$1.kNo) {
        result += nested === Nested$1.kYes ? '(' : 'calc(';
      }
      var values = this.numericValues();
      result = values[0].buildCSSText(Nested$1.kYes, ParenLess$1.kNo, result);
      for (var i = 1; i < values.length; i++) {
        var arg = values[i];
        if (arg.getType() === CSSStyleValueType$1.kInvertType) {
          result += ' / ';
          result = arg.value.buildCSSText(Nested$1.kYes, ParenLess$1.kNo, result);
        } else {
          result += ' * ';
          result = arg.buildCSSText(Nested$1.kYes, ParenLess$1.kNo, result);
        }
      }
      if (paren_less === ParenLess$1.kNo) {
        result += ')';
      }
      return result;
    };
    return CSSMathProduct;
  }(CSSMathVariadic$1);
  var Opx$1 = new CSSUnitValue$1(0, 'px');
  var Lpx$1 = new CSSUnitValue$1(1, 'px');
  var Odeg$1 = new CSSUnitValue$1(0, 'deg');

  /**
   * The CSSRGB class represents the CSS rgb()/rgba() functions.
   *
   * @see https://drafts.css-houdini.org/css-typed-om-1/#cssrgb
   */
  var CSSRGB$1 = /*#__PURE__*/function (_CSSColorValue) {
    _inheritsLoose$9(CSSRGB, _CSSColorValue);
    function CSSRGB(r, g, b, alpha,
    /**
     * 'transparent' & 'none' has the same rgba data
     */
    isNone) {
      var _this;
      if (alpha === void 0) {
        alpha = 1;
      }
      if (isNone === void 0) {
        isNone = false;
      }
      _this = _CSSColorValue.call(this, 'rgb') || this;
      _this.r = void 0;
      _this.g = void 0;
      _this.b = void 0;
      _this.alpha = void 0;
      _this.isNone = void 0;
      _this.r = r;
      _this.g = g;
      _this.b = b;
      _this.alpha = alpha;
      _this.isNone = isNone;
      return _this;
    }
    var _proto = CSSRGB.prototype;
    _proto.clone = function clone() {
      return new CSSRGB(this.r, this.g, this.b, this.alpha);
    };
    _proto.buildCSSText = function buildCSSText(n, p, result) {
      return result + ("rgba(" + this.r + "," + this.g + "," + this.b + "," + this.alpha + ")");
    };
    return CSSRGB;
  }(CSSColorValue$1);

  /**
   * CSSKeywordValue
   */
  var unsetKeywordValue$1 = new CSSKeywordValue$1('unset');
  var initialKeywordValue$1 = new CSSKeywordValue$1('initial');
  var inheritKeywordValue$1 = new CSSKeywordValue$1('inherit');
  var keywordCache$1 = {
    '': unsetKeywordValue$1,
    unset: unsetKeywordValue$1,
    initial: initialKeywordValue$1,
    inherit: inheritKeywordValue$1
  };
  var getOrCreateKeyword$1 = function getOrCreateKeyword(name) {
    if (!keywordCache$1[name]) {
      keywordCache$1[name] = new CSSKeywordValue$1(name);
    }
    return keywordCache$1[name];
  };
  /**
   * CSSColor
   */
  var noneColor$1 = new CSSRGB$1(0, 0, 0, 0, true);
  var transparentColor$1 = new CSSRGB$1(0, 0, 0, 0);
  var getOrCreateRGBA$1 = memoize$1(function (r, g, b, a) {
    return new CSSRGB$1(r, g, b, a);
  }, function (r, g, b, a) {
    return "rgba(" + r + "," + g + "," + b + "," + a + ")";
  });
  var getOrCreateUnitValue$1 = memoize$1(function (value, unitOrName) {
    if (unitOrName === void 0) {
      unitOrName = UnitType$1.kNumber;
    }
    return new CSSUnitValue$1(value, unitOrName);
  }, function (value, unitOrName) {
    if (unitOrName === void 0) {
      unitOrName = UnitType$1.kNumber;
    }
    return "" + value + unitOrName;
  });

  var regexLG$1 = /^l\s*\(\s*([\d.]+)\s*\)\s*(.*)/i;
  var regexRG$1 = /^r\s*\(\s*([\d.]+)\s*,\s*([\d.]+)\s*,\s*([\d.]+)\s*\)\s*(.*)/i;
  var regexPR$1 = /^p\s*\(\s*([axyn])\s*\)\s*(.*)/i;
  var regexColorStop$1 = /[\d.]+:(#[^\s]+|[^\)]+\))/gi;
  function spaceColorStops$1(colorStops) {
    var _colorStops$length;
    var length = colorStops.length;
    colorStops[length - 1].length = (_colorStops$length = colorStops[length - 1].length) !== null && _colorStops$length !== void 0 ? _colorStops$length : {
      type: '%',
      value: '100'
    };
    if (length > 1) {
      var _colorStops$0$length;
      colorStops[0].length = (_colorStops$0$length = colorStops[0].length) !== null && _colorStops$0$length !== void 0 ? _colorStops$0$length : {
        type: '%',
        value: '0'
      };
    }
    var previousIndex = 0;
    var previousOffset = Number(colorStops[0].length.value);
    for (var i = 1; i < length; i++) {
      var _colorStops$i$length;
      // support '%' & 'px'
      var offset = (_colorStops$i$length = colorStops[i].length) === null || _colorStops$i$length === void 0 ? void 0 : _colorStops$i$length.value;
      if (!isNil$1(offset) && !isNil$1(previousOffset)) {
        for (var j = 1; j < i - previousIndex; j++) {
          colorStops[previousIndex + j].length = {
            type: '%',
            value: "" + (previousOffset + (Number(offset) - previousOffset) * j / (i - previousIndex))
          };
        }
        previousIndex = i;
        previousOffset = Number(offset);
      }
    }
  }
  // The position of the gradient line's starting point.
  // different from CSS side(to top) @see https://developer.mozilla.org/en-US/docs/Web/CSS/gradient/linear-gradient#values
  var SideOrCornerToDegMap$1 = {
    left: 270 - 90,
    top: 0 - 90,
    bottom: 180 - 90,
    right: 90 - 90,
    'left top': 315 - 90,
    'top left': 315 - 90,
    'left bottom': 225 - 90,
    'bottom left': 225 - 90,
    'right top': 45 - 90,
    'top right': 45 - 90,
    'right bottom': 135 - 90,
    'bottom right': 135 - 90
  };
  var angleToDeg$1 = memoize$1(function (orientation) {
    var angle;
    if (orientation.type === 'angular') {
      angle = Number(orientation.value);
    } else {
      angle = SideOrCornerToDegMap$1[orientation.value] || 0;
    }
    return getOrCreateUnitValue$1(angle, 'deg');
  });
  var positonToCSSUnitValue$1 = memoize$1(function (position) {
    var cx = 50;
    var cy = 50;
    var unitX = '%';
    var unitY = '%';
    if ((position === null || position === void 0 ? void 0 : position.type) === 'position') {
      var _position$value = position.value,
        x = _position$value.x,
        y = _position$value.y;
      if ((x === null || x === void 0 ? void 0 : x.type) === 'position-keyword') {
        if (x.value === 'left') {
          cx = 0;
        } else if (x.value === 'center') {
          cx = 50;
        } else if (x.value === 'right') {
          cx = 100;
        } else if (x.value === 'top') {
          cy = 0;
        } else if (x.value === 'bottom') {
          cy = 100;
        }
      }
      if ((y === null || y === void 0 ? void 0 : y.type) === 'position-keyword') {
        if (y.value === 'left') {
          cx = 0;
        } else if (y.value === 'center') {
          cy = 50;
        } else if (y.value === 'right') {
          cx = 100;
        } else if (y.value === 'top') {
          cy = 0;
        } else if (y.value === 'bottom') {
          cy = 100;
        }
      }
      if ((x === null || x === void 0 ? void 0 : x.type) === 'px' || (x === null || x === void 0 ? void 0 : x.type) === '%' || (x === null || x === void 0 ? void 0 : x.type) === 'em') {
        unitX = x === null || x === void 0 ? void 0 : x.type;
        cx = Number(x.value);
      }
      if ((y === null || y === void 0 ? void 0 : y.type) === 'px' || (y === null || y === void 0 ? void 0 : y.type) === '%' || (y === null || y === void 0 ? void 0 : y.type) === 'em') {
        unitY = y === null || y === void 0 ? void 0 : y.type;
        cy = Number(y.value);
      }
    }
    return {
      cx: getOrCreateUnitValue$1(cx, unitX),
      cy: getOrCreateUnitValue$1(cy, unitY)
    };
  });
  var parseGradient$2 = memoize$1(function (colorStr) {
    if (colorStr.indexOf('linear') > -1 || colorStr.indexOf('radial') > -1) {
      var ast = parseGradient$1$1(colorStr);
      return ast.map(function (_ref) {
        var type = _ref.type,
          orientation = _ref.orientation,
          colorStops = _ref.colorStops;
        spaceColorStops$1(colorStops);
        var steps = colorStops.map(function (colorStop) {
          // TODO: only support % for now, should calc percentage of axis length when using px/em
          return {
            offset: getOrCreateUnitValue$1(colorStop.length.value, '%'),
            color: colorStopToString$1(colorStop)
          };
        });
        if (type === 'linear-gradient') {
          return new CSSGradientValue$1(GradientType$1.LinearGradient, {
            angle: orientation ? angleToDeg$1(orientation) : Odeg$1,
            steps: steps
          });
        } else if (type === 'radial-gradient') {
          if (!orientation) {
            orientation = [{
              type: 'shape',
              value: 'circle'
            }];
          }
          if (orientation[0].type === 'shape' && orientation[0].value === 'circle') {
            var _positonToCSSUnitValu = positonToCSSUnitValue$1(orientation[0].at),
              cx = _positonToCSSUnitValu.cx,
              cy = _positonToCSSUnitValu.cy;
            var size;
            if (orientation[0].style) {
              var _orientation$0$style = orientation[0].style,
                _type = _orientation$0$style.type,
                value = _orientation$0$style.value;
              if (_type === 'extent-keyword') {
                size = getOrCreateKeyword$1(value);
              } else {
                size = getOrCreateUnitValue$1(value, _type);
              }
            }
            return new CSSGradientValue$1(GradientType$1.RadialGradient, {
              cx: cx,
              cy: cy,
              size: size,
              steps: steps
            });
          }
          // TODO: support ellipse shape
          // TODO: repeating-linear-gradient & repeating-radial-gradient
          // } else if (type === 'repeating-linear-gradient') {
          // } else if (type === 'repeating-radial-gradient') {
        }
      });
    }
    // legacy format, should be deprecated later
    var type = colorStr[0];
    if (colorStr[1] === '(' || colorStr[2] === '(') {
      if (type === 'l') {
        var arr = regexLG$1.exec(colorStr);
        if (arr) {
          var _arr$2$match;
          var steps = ((_arr$2$match = arr[2].match(regexColorStop$1)) === null || _arr$2$match === void 0 ? void 0 : _arr$2$match.map(function (stop) {
            return stop.split(':');
          })) || [];
          return [new CSSGradientValue$1(GradientType$1.LinearGradient, {
            angle: getOrCreateUnitValue$1(parseFloat(arr[1]), 'deg'),
            steps: steps.map(function (_ref2) {
              var offset = _ref2[0],
                color = _ref2[1];
              return {
                offset: getOrCreateUnitValue$1(Number(offset) * 100, '%'),
                color: color
              };
            })
          })];
        }
      } else if (type === 'r') {
        var parsedRadialGradient = parseRadialGradient$1(colorStr);
        if (parsedRadialGradient) {
          if (isString$1(parsedRadialGradient)) {
            colorStr = parsedRadialGradient;
          } else {
            return [new CSSGradientValue$1(GradientType$1.RadialGradient, parsedRadialGradient)];
          }
        }
      } else if (type === 'p') {
        return parsePattern$1(colorStr);
      }
    }
  });
  function parseRadialGradient$1(gradientStr) {
    var arr = regexRG$1.exec(gradientStr);
    if (arr) {
      var _arr$4$match;
      var steps = ((_arr$4$match = arr[4].match(regexColorStop$1)) === null || _arr$4$match === void 0 ? void 0 : _arr$4$match.map(function (stop) {
        return stop.split(':');
      })) || [];
      return {
        cx: getOrCreateUnitValue$1(50, '%'),
        cy: getOrCreateUnitValue$1(50, '%'),
        steps: steps.map(function (_ref3) {
          var offset = _ref3[0],
            color = _ref3[1];
          return {
            offset: getOrCreateUnitValue$1(Number(offset) * 100, '%'),
            color: color
          };
        })
      };
    }
    return null;
  }
  function parsePattern$1(patternStr) {
    var arr = regexPR$1.exec(patternStr);
    if (arr) {
      var repetition = arr[1];
      var src = arr[2];
      switch (repetition) {
        case 'a':
          repetition = 'repeat';
          break;
        case 'x':
          repetition = 'repeat-x';
          break;
        case 'y':
          repetition = 'repeat-y';
          break;
        case 'n':
          repetition = 'no-repeat';
          break;
        default:
          repetition = 'no-repeat';
      }
      return {
        image: src,
        // @ts-ignore
        repetition: repetition
      };
    }
    return null;
  }

  function isPattern$1(object) {
    return isObject$1(object) && !!object.image;
  }
  /**
   * @see https://github.com/WebKit/WebKit/blob/main/Source/WebCore/css/parser/CSSParser.cpp#L97
   */
  var parseColor$1 = memoize$1(function (colorStr) {
    if (isPattern$1(colorStr)) {
      return _extends$3({
        repetition: 'repeat'
      }, colorStr);
    }
    if (isNil$1(colorStr)) {
      colorStr = '';
    }
    if (colorStr === 'transparent') {
      // transparent black
      return transparentColor$1;
    } else if (colorStr === 'currentColor') {
      // @see https://github.com/adobe-webplatform/Snap.svg/issues/526
      colorStr = 'black';
    }
    // support CSS gradient syntax
    var g = parseGradient$2(colorStr);
    if (g) {
      return g;
    }
    // constants
    var color$1 = color$2(colorStr);
    var rgba = [0, 0, 0, 0];
    if (color$1 !== null) {
      rgba[0] = color$1.r || 0;
      rgba[1] = color$1.g || 0;
      rgba[2] = color$1.b || 0;
      rgba[3] = color$1.opacity;
    }
    // return new CSSRGB(...rgba);
    return getOrCreateRGBA$1.apply(void 0, rgba);
  });
  function mergeColors$1(left, right) {
    // only support constant value, exclude gradient & pattern
    if (!(left instanceof CSSRGB$1) || !(right instanceof CSSRGB$1)) {
      return;
    }
    return [[Number(left.r), Number(left.g), Number(left.b), Number(left.alpha)], [Number(right.r), Number(right.g), Number(right.b), Number(right.alpha)], function (color) {
      var rgba = color.slice();
      if (rgba[3]) {
        for (var i = 0; i < 3; i++) {
          rgba[i] = Math.round(clamp$1(rgba[i], 0, 255));
        }
      }
      rgba[3] = clamp$1(rgba[3], 0, 1);
      return "rgba(" + rgba.join(',') + ")";
    }];
  }

  function parseDimension$1(unitRegExp, string) {
    if (isNil$1(string)) {
      return getOrCreateUnitValue$1(0, 'px');
    }
    string = ("" + string).trim().toLowerCase();
    if (isFinite(Number(string))) {
      if ('px'.search(unitRegExp) >= 0) {
        return getOrCreateUnitValue$1(Number(string), 'px');
      } else if ('deg'.search(unitRegExp) >= 0) {
        return getOrCreateUnitValue$1(Number(string), 'deg');
      }
    }
    var matchedUnits = [];
    string = string.replace(unitRegExp, function (match) {
      matchedUnits.push(match);
      return 'U' + match;
    });
    var taggedUnitRegExp = 'U(' + unitRegExp.source + ')';
    return matchedUnits.map(function (unit) {
      return getOrCreateUnitValue$1(Number(string.replace(new RegExp('U' + unit, 'g'), '').replace(new RegExp(taggedUnitRegExp, 'g'), '*0')), unit);
    })[0];
  }
  /**
   * <length>
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/length
   * length with only absolute unit, eg. 1px
   */
  var parseLength$1 = memoize$1(function (css) {
    return parseDimension$1(new RegExp('px', 'g'), css);
  });
  /**
   * <percentage>
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/percentage
   */
  var parserPercentage$1 = memoize$1(function (css) {
    return parseDimension$1(new RegExp('%', 'g'), css);
  });
  /**
   * length with absolute or relative unit,
   * eg. 1px, 0.7em, 50%, calc(100% - 200px);
   *
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/length-percentage
   */
  var parseLengthOrPercentage$1 = memoize$1(function (css) {
    if (isNumber$1(css) || isFinite(Number(css))) {
      return getOrCreateUnitValue$1(Number(css), 'px');
    }
    return parseDimension$1(new RegExp('px|%|em|rem', 'g'), css);
  });
  var parseAngle$1 = memoize$1(function (css) {
    return parseDimension$1(new RegExp('deg|rad|grad|turn', 'g'), css);
  });
  /**
   * merge CSSUnitValue
   *
   * @example
   * 10px + 20px = 30px
   * 10deg + 10rad
   * 10% + 20% = 30%
   */
  function mergeDimensions$1(left, right, target, nonNegative, index) {
    if (index === void 0) {
      index = 0;
    }
    var unit = '';
    var leftValue = left.value || 0;
    var rightValue = right.value || 0;
    var canonicalUnit = toCanonicalUnit$1(left.unit);
    var leftCanonicalUnitValue = left.convertTo(canonicalUnit);
    var rightCanonicalUnitValue = right.convertTo(canonicalUnit);
    if (leftCanonicalUnitValue && rightCanonicalUnitValue) {
      leftValue = leftCanonicalUnitValue.value;
      rightValue = rightCanonicalUnitValue.value;
      unit = unitTypeToString$1(left.unit);
    } else {
      // format '%' to 'px'
      if (CSSUnitValue$1.isLength(left.unit) || CSSUnitValue$1.isLength(right.unit)) {
        leftValue = convertPercentUnit$1(left, index, target);
        rightValue = convertPercentUnit$1(right, index, target);
        unit = 'px';
      }
    }
    // // format 'rad' 'turn' to 'deg'
    // if (CSSUnitValue.isAngle(left.unit) || CSSUnitValue.isAngle(right.unit)) {
    //   leftValue = convertAngleUnit(left);
    //   rightValue = convertAngleUnit(right);
    //   unit = 'deg';
    // }
    return [leftValue, rightValue, function (value) {
      if (nonNegative) {
        value = Math.max(value, 0);
      }
      return value + unit;
    }];
  }
  function convertAngleUnit$1(value) {
    var deg = 0;
    if (value.unit === UnitType$1.kDegrees) {
      deg = value.value;
    } else if (value.unit === UnitType$1.kRadians) {
      deg = rad2deg$1(Number(value.value));
    } else if (value.unit === UnitType$1.kTurns) {
      deg = turn2deg$1(Number(value.value));
    }
    return deg;
  }
  function parseDimensionArray$1(string) {
    if (isString$1(string)) {
      // "1px 2px 3px"
      return string.split(' ').map(function (segment) {
        return parseLengthOrPercentage$1(segment);
      });
    } else {
      // [1, '2px', 3]
      return string.map(function (segment) {
        return parseLengthOrPercentage$1(segment.toString());
      });
    }
  }
  // export function mergeDimensionList(
  //   left: CSSUnitValue[],
  //   right: CSSUnitValue[],
  //   target: IElement | null,
  // ): [number[], number[], (list: number[]) => string] | undefined {
  //   if (left.length !== right.length) {
  //     return;
  //   }
  //   const unit = left[0].unit;
  //   return [
  //     left.map((l) => l.value),
  //     right.map((l) => l.value),
  //     (values: number[]) => {
  //       return values.map((n) => new CSSUnitValue(n, unit)).join(' ');
  //     },
  //   ];
  // }
  function convertPercentUnit$1(valueWithUnit, vec3Index, target) {
    if (valueWithUnit.unit === UnitType$1.kPixels) {
      return Number(valueWithUnit.value);
    } else if (valueWithUnit.unit === UnitType$1.kPercentage && target) {
      var bounds = target.nodeName === Shape$2.GROUP ? target.getLocalBounds() : target.getGeometryBounds();
      var size = 0;
      if (!AABB$1.isEmpty(bounds)) {
        size = bounds.halfExtents[vec3Index] * 2;
      }
      return Number(valueWithUnit.value) / 100 * size;
    }
    return 0;
  }

  var parseParam$1 = function parseParam(css) {
    return parseDimension$1(/deg|rad|grad|turn|px|%/g, css);
  };
  var supportedFilters$1 = ['blur', 'brightness', 'drop-shadow', 'contrast', 'grayscale', 'sepia', 'saturate', 'hue-rotate', 'invert'];
  function parseFilter$1(filterStr) {
    if (filterStr === void 0) {
      filterStr = '';
    }
    filterStr = filterStr.toLowerCase().trim();
    if (filterStr === 'none') {
      return [];
    }
    var filterRegExp = /\s*([\w-]+)\(([^)]*)\)/g;
    var result = [];
    var match;
    var prevLastIndex = 0;
    while (match = filterRegExp.exec(filterStr)) {
      if (match.index !== prevLastIndex) {
        return [];
      }
      prevLastIndex = match.index + match[0].length;
      if (supportedFilters$1.indexOf(match[1]) > -1) {
        result.push({
          name: match[1],
          params: match[2].split(' ').map(function (p) {
            return parseParam$1(p) || parseColor$1(p);
          })
        });
      }
      if (filterRegExp.lastIndex === filterStr.length) {
        return result;
      }
    }
    return [];
  }

  function numberToString$1(x) {
    // scale(0.00000001) -> scale(0)
    // return x.toFixed(6).replace(/0+$/, '').replace(/\.$/, '');
    return x.toString();
  }
  /**
   * parse string or number to CSSUnitValue(numeric)
   *
   * eg.
   * * 0 -> CSSUnitValue(0)
   * * '2' -> CSSUnitValue(2)
   */
  var parseNumber$1 = memoize$1(function (string) {
    if (typeof string === 'number') {
      return getOrCreateUnitValue$1(string);
    }
    if (/^\s*[-+]?(\d*\.)?\d+\s*$/.test(string)) {
      return getOrCreateUnitValue$1(Number(string));
    } else {
      return getOrCreateUnitValue$1(0);
    }
  });
  /**
   * separate string to array
   * eg.
   * * [0.5, 0.5] -> [CSSUnitValue, CSSUnitValue]
   */
  var parseNumberList$1 = memoize$1(function (string) {
    if (isString$1(string)) {
      return string.split(' ').map(parseNumber$1);
    } else {
      return string.map(parseNumber$1);
    }
  });
  function mergeNumbers$1(left, right) {
    return [left, right, numberToString$1];
  }
  function clampedMergeNumbers$1(min, max) {
    return function (left, right) {
      return [left, right, function (x) {
        return numberToString$1(clamp$1(x, min, max));
      }];
    };
  }
  function mergeNumberLists$1(left, right) {
    if (left.length !== right.length) {
      return;
    }
    return [left, right, function (numberList) {
      return numberList;
    }];
  }

  var internalParsePath$1 = function internalParsePath(path) {
    // empty path
    if (path === '' || Array.isArray(path) && path.length === 0) {
      return {
        absolutePath: [],
        hasArc: false,
        segments: [],
        polygons: [],
        polylines: [],
        curve: null,
        totalLength: 0,
        rect: {
          x: 0,
          y: 0,
          width: 0,
          height: 0
        }
      };
    }
    var absolutePath;
    try {
      absolutePath = normalizePath$1(path);
    } catch (e) {
      absolutePath = normalizePath$1('');
      console.error("[g]: Invalid SVG Path definition: " + path);
    }
    var hasArc = hasArcOrBezier$1(absolutePath);
    var _extractPolygons = extractPolygons$1(absolutePath),
      polygons = _extractPolygons.polygons,
      polylines = _extractPolygons.polylines;
    // for later use
    var segments = path2Segments$1(absolutePath);
    // Only calculate bbox here since we don't need length now.
    var _getPathBBox = getPathBBox$1(segments, 0),
      x = _getPathBBox.x,
      y = _getPathBBox.y,
      width = _getPathBBox.width,
      height = _getPathBBox.height;
    return {
      absolutePath: absolutePath,
      hasArc: hasArc,
      segments: segments,
      polygons: polygons,
      polylines: polylines,
      // curve,
      // Delay the calculation of length.
      totalLength: 0,
      rect: {
        x: Number.isFinite(x) ? x : 0,
        y: Number.isFinite(y) ? y : 0,
        width: Number.isFinite(width) ? width : 0,
        height: Number.isFinite(height) ? height : 0
      }
    };
  };
  var memoizedParsePath$1 = memoize$1(internalParsePath$1);
  function parsePath$1(path, object) {
    var result = isString$1(path) ? memoizedParsePath$1(path) : internalParsePath$1(path);
    if (object) {
      object.parsedStyle.defX = result.rect.x;
      object.parsedStyle.defY = result.rect.y;
    }
    return result;
  }
  function mergePaths$1(left, right, object) {
    var curve1 = left.curve;
    var curve2 = right.curve;
    if (!curve1 || curve1.length === 0) {
      // convert to curves to do morphing & picking later
      // @see http://thednp.github.io/kute.js/svgCubicMorph.html
      curve1 = path2Curve$1(left.absolutePath, false);
      left.curve = curve1;
    }
    if (!curve2 || curve2.length === 0) {
      curve2 = path2Curve$1(right.absolutePath, false);
      right.curve = curve2;
    }
    var curves = [curve1, curve2];
    if (curve1.length !== curve2.length) {
      curves = equalizeSegments$1(curve1, curve2);
    }
    var curve0 = getDrawDirection$1(curves[0]) !== getDrawDirection$1(curves[1]) ? reverseCurve$1(curves[0]) : clonePath$1(curves[0]);
    return [curve0, getRotatedCurve$1(curves[1], curve0), function (pathArray) {
      // need converting to path string?
      return pathArray;
    }];
  }

  /**
   * @see https://developer.mozilla.org/zh-CN/docs/Web/SVG/Attribute/points
   *
   * @example
   * points="100,10 250,150 200,110"
   */
  function parsePoints$1(pointsOrStr, object) {
    var points;
    if (isString$1(pointsOrStr)) {
      points = pointsOrStr.split(' ').map(function (pointStr) {
        var _pointStr$split = pointStr.split(','),
          x = _pointStr$split[0],
          y = _pointStr$split[1];
        return [Number(x), Number(y)];
      });
    } else {
      points = pointsOrStr;
    }
    var segments = [];
    var tempLength = 0;
    var segmentT;
    var segmentL;
    var totalLength = polyline$1.length(points);
    points.forEach(function (p, i) {
      if (points[i + 1]) {
        segmentT = [0, 0];
        segmentT[0] = tempLength / totalLength;
        segmentL = line$3.length(p[0], p[1], points[i + 1][0], points[i + 1][1]);
        tempLength += segmentL;
        segmentT[1] = tempLength / totalLength;
        segments.push(segmentT);
      }
    });
    var minX = Math.min.apply(Math, points.map(function (point) {
      return point[0];
    }));
    var minY = Math.min.apply(Math, points.map(function (point) {
      return point[1];
    }));
    if (object) {
      object.parsedStyle.defX = minX;
      object.parsedStyle.defY = minY;
    }
    return {
      points: points,
      totalLength: totalLength,
      segments: segments
    };
  }

  var _$1 = null;
  function cast$1(pattern) {
    return function (contents) {
      var i = 0;
      return pattern.map(function (x) {
        return x === _$1 ? contents[i++] : x;
      });
    };
  }
  function id$1(x) {
    return x;
  }
  // type: [argTypes, convertTo3D, convertTo2D]
  // In the argument types string, lowercase characters represent optional arguments
  var transformFunctions$1 = {
    // @ts-ignore
    matrix: ['NNNNNN', [_$1, _$1, 0, 0, _$1, _$1, 0, 0, 0, 0, 1, 0, _$1, _$1, 0, 1], id$1],
    matrix3d: ['NNNNNNNNNNNNNNNN', id$1],
    rotate: ['A'],
    rotatex: ['A'],
    rotatey: ['A'],
    rotatez: ['A'],
    rotate3d: ['NNNA'],
    perspective: ['L'],
    scale: ['Nn', cast$1([_$1, _$1, new CSSUnitValue$1(1)]), id$1],
    scalex: ['N', cast$1([_$1, new CSSUnitValue$1(1), new CSSUnitValue$1(1)]), cast$1([_$1, new CSSUnitValue$1(1)])],
    scaley: ['N', cast$1([new CSSUnitValue$1(1), _$1, new CSSUnitValue$1(1)]), cast$1([new CSSUnitValue$1(1), _$1])],
    scalez: ['N', cast$1([new CSSUnitValue$1(1), new CSSUnitValue$1(1), _$1])],
    scale3d: ['NNN', id$1],
    skew: ['Aa', null, id$1],
    skewx: ['A', null, cast$1([_$1, Odeg$1])],
    skewy: ['A', null, cast$1([Odeg$1, _$1])],
    translate: ['Tt', cast$1([_$1, _$1, Opx$1]), id$1],
    translatex: ['T', cast$1([_$1, Opx$1, Opx$1]), cast$1([_$1, Opx$1])],
    translatey: ['T', cast$1([Opx$1, _$1, Opx$1]), cast$1([Opx$1, _$1])],
    translatez: ['L', cast$1([Opx$1, Opx$1, _$1])],
    translate3d: ['TTL', id$1]
  };
  /**
   * none
   * scale(1) scale(1, 2)
   * scaleX(1)
   */
  function parseTransform$1(string) {
    string = (string || 'none').toLowerCase().trim();
    if (string === 'none') {
      return [];
    }
    var transformRegExp = /\s*(\w+)\(([^)]*)\)/g;
    var result = [];
    var match;
    var prevLastIndex = 0;
    while (match = transformRegExp.exec(string)) {
      if (match.index !== prevLastIndex) {
        return [];
      }
      prevLastIndex = match.index + match[0].length;
      var functionName = match[1]; // scale
      var functionData = transformFunctions$1[functionName]; // scale(1, 2)
      if (!functionData) {
        // invalid, eg. scale()
        return [];
      }
      var args = match[2].split(','); // 1,2
      var argTypes = functionData[0]; // Nn
      if (argTypes.length < args.length) {
        // scale(N, n)
        return [];
      }
      var parsedArgs = [];
      for (var i = 0; i < argTypes.length; i++) {
        var arg = args[i];
        var type = argTypes[i];
        var parsedArg = void 0;
        if (!arg) {
          // @ts-ignore
          parsedArg = {
            a: Odeg$1,
            n: parsedArgs[0],
            t: Opx$1
          }[type];
        } else {
          // @ts-ignore
          parsedArg = {
            A: function A(s) {
              return s.trim() === '0' ? Odeg$1 : parseAngle$1(s);
            },
            N: parseNumber$1,
            T: parseLengthOrPercentage$1,
            L: parseLength$1
          }[type.toUpperCase()](arg);
        }
        if (parsedArg === undefined) {
          return [];
        }
        parsedArgs.push(parsedArg);
      }
      result.push({
        t: functionName,
        d: parsedArgs
      }); // { t: scale, d: [1, 2] }
      if (transformRegExp.lastIndex === string.length) {
        return result;
      }
    }
    return [];
  }
  function convertItemToMatrix$1(item) {
    var x;
    var y;
    var z;
    var angle;
    switch (item.t) {
      case 'rotatex':
        angle = deg2rad$1(convertAngleUnit$1(item.d[0]));
        return [1, 0, 0, 0, 0, Math.cos(angle), Math.sin(angle), 0, 0, -Math.sin(angle), Math.cos(angle), 0, 0, 0, 0, 1];
      case 'rotatey':
        angle = deg2rad$1(convertAngleUnit$1(item.d[0]));
        return [Math.cos(angle), 0, -Math.sin(angle), 0, 0, 1, 0, 0, Math.sin(angle), 0, Math.cos(angle), 0, 0, 0, 0, 1];
      case 'rotate':
      case 'rotatez':
        angle = deg2rad$1(convertAngleUnit$1(item.d[0]));
        return [Math.cos(angle), Math.sin(angle), 0, 0, -Math.sin(angle), Math.cos(angle), 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'rotate3d':
        x = item.d[0].value;
        y = item.d[1].value;
        z = item.d[2].value;
        angle = deg2rad$1(convertAngleUnit$1(item.d[3]));
        var sqrLength = x * x + y * y + z * z;
        if (sqrLength === 0) {
          x = 1;
          y = 0;
          z = 0;
        } else if (sqrLength !== 1) {
          var length = Math.sqrt(sqrLength);
          x /= length;
          y /= length;
          z /= length;
        }
        var s = Math.sin(angle / 2);
        var sc = s * Math.cos(angle / 2);
        var sq = s * s;
        return [1 - 2 * (y * y + z * z) * sq, 2 * (x * y * sq + z * sc), 2 * (x * z * sq - y * sc), 0, 2 * (x * y * sq - z * sc), 1 - 2 * (x * x + z * z) * sq, 2 * (y * z * sq + x * sc), 0, 2 * (x * z * sq + y * sc), 2 * (y * z * sq - x * sc), 1 - 2 * (x * x + y * y) * sq, 0, 0, 0, 0, 1];
      case 'scale':
        return [item.d[0].value, 0, 0, 0, 0, item.d[1].value, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'scalex':
        return [item.d[0].value, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'scaley':
        return [1, 0, 0, 0, 0, item.d[0].value, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'scalez':
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, item.d[0].value, 0, 0, 0, 0, 1];
      case 'scale3d':
        return [item.d[0].value, 0, 0, 0, 0, item.d[1].value, 0, 0, 0, 0, item.d[2].value, 0, 0, 0, 0, 1];
      case 'skew':
        var xAngle = deg2rad$1(convertAngleUnit$1(item.d[0]));
        var yAngle = deg2rad$1(convertAngleUnit$1(item.d[1]));
        return [1, Math.tan(yAngle), 0, 0, Math.tan(xAngle), 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'skewx':
        angle = deg2rad$1(convertAngleUnit$1(item.d[0]));
        return [1, 0, 0, 0, Math.tan(angle), 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'skewy':
        angle = deg2rad$1(convertAngleUnit$1(item.d[0]));
        return [1, Math.tan(angle), 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
      case 'translate':
        // TODO: pass target
        x = convertPercentUnit$1(item.d[0], 0, null) || 0;
        y = convertPercentUnit$1(item.d[1], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, x, y, 0, 1];
      case 'translatex':
        x = convertPercentUnit$1(item.d[0], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, x, 0, 0, 1];
      case 'translatey':
        y = convertPercentUnit$1(item.d[0], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, y, 0, 1];
      case 'translatez':
        z = convertPercentUnit$1(item.d[0], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, z, 1];
      case 'translate3d':
        x = convertPercentUnit$1(item.d[0], 0, null) || 0;
        y = convertPercentUnit$1(item.d[1], 0, null) || 0;
        z = convertPercentUnit$1(item.d[2], 0, null) || 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, x, y, z, 1];
      case 'perspective':
        var t = convertPercentUnit$1(item.d[0], 0, null) || 0;
        var p = t ? -1 / t : 0;
        return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, p, 0, 0, 0, 1];
      case 'matrix':
        return [item.d[0].value, item.d[1].value, 0, 0, item.d[2].value, item.d[3].value, 0, 0, 0, 0, 1, 0, item.d[4].value, item.d[5].value, 0, 1];
      case 'matrix3d':
        return item.d.map(function (d) {
          return d.value;
        });
    }
  }
  function multiplyMatrices$1(a, b) {
    return [a[0] * b[0] + a[4] * b[1] + a[8] * b[2] + a[12] * b[3], a[1] * b[0] + a[5] * b[1] + a[9] * b[2] + a[13] * b[3], a[2] * b[0] + a[6] * b[1] + a[10] * b[2] + a[14] * b[3], a[3] * b[0] + a[7] * b[1] + a[11] * b[2] + a[15] * b[3], a[0] * b[4] + a[4] * b[5] + a[8] * b[6] + a[12] * b[7], a[1] * b[4] + a[5] * b[5] + a[9] * b[6] + a[13] * b[7], a[2] * b[4] + a[6] * b[5] + a[10] * b[6] + a[14] * b[7], a[3] * b[4] + a[7] * b[5] + a[11] * b[6] + a[15] * b[7], a[0] * b[8] + a[4] * b[9] + a[8] * b[10] + a[12] * b[11], a[1] * b[8] + a[5] * b[9] + a[9] * b[10] + a[13] * b[11], a[2] * b[8] + a[6] * b[9] + a[10] * b[10] + a[14] * b[11], a[3] * b[8] + a[7] * b[9] + a[11] * b[10] + a[15] * b[11], a[0] * b[12] + a[4] * b[13] + a[8] * b[14] + a[12] * b[15], a[1] * b[12] + a[5] * b[13] + a[9] * b[14] + a[13] * b[15], a[2] * b[12] + a[6] * b[13] + a[10] * b[14] + a[14] * b[15], a[3] * b[12] + a[7] * b[13] + a[11] * b[14] + a[15] * b[15]];
  }
  function convertToMatrix$1(transformList) {
    if (transformList.length === 0) {
      return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
    }
    return transformList.map(convertItemToMatrix$1).reduce(multiplyMatrices$1);
  }
  function makeMatrixDecomposition$1(transformList) {
    var translate = [0, 0, 0];
    var scale = [1, 1, 1];
    var skew = [0, 0, 0];
    var perspective = [0, 0, 0, 1];
    var quaternion = [0, 0, 0, 1];
    // @ts-ignore
    decomposeMat4$1(convertToMatrix$1(transformList), translate, scale, skew, perspective, quaternion);
    return [[translate, scale, skew, quaternion, perspective]];
  }
  var composeMatrix$1 = function () {
    function multiply(a, b) {
      var result = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0]];
      for (var i = 0; i < 4; i++) {
        for (var j = 0; j < 4; j++) {
          for (var k = 0; k < 4; k++) {
            result[i][j] += b[i][k] * a[k][j];
          }
        }
      }
      return result;
    }
    function is2D(m) {
      return m[0][2] == 0 && m[0][3] == 0 && m[1][2] == 0 && m[1][3] == 0 && m[2][0] == 0 && m[2][1] == 0 && m[2][2] == 1 && m[2][3] == 0 && m[3][2] == 0 && m[3][3] == 1;
    }
    function composeMatrix(translate, scale, skew, quat, perspective) {
      var matrix = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]];
      for (var i = 0; i < 4; i++) {
        matrix[i][3] = perspective[i];
      }
      for (var _i = 0; _i < 3; _i++) {
        for (var j = 0; j < 3; j++) {
          matrix[3][_i] += translate[j] * matrix[j][_i];
        }
      }
      var x = quat[0],
        y = quat[1],
        z = quat[2],
        w = quat[3];
      var rotMatrix = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]];
      rotMatrix[0][0] = 1 - 2 * (y * y + z * z);
      rotMatrix[0][1] = 2 * (x * y - z * w);
      rotMatrix[0][2] = 2 * (x * z + y * w);
      rotMatrix[1][0] = 2 * (x * y + z * w);
      rotMatrix[1][1] = 1 - 2 * (x * x + z * z);
      rotMatrix[1][2] = 2 * (y * z - x * w);
      rotMatrix[2][0] = 2 * (x * z - y * w);
      rotMatrix[2][1] = 2 * (y * z + x * w);
      rotMatrix[2][2] = 1 - 2 * (x * x + y * y);
      matrix = multiply(matrix, rotMatrix);
      var temp = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]];
      if (skew[2]) {
        temp[2][1] = skew[2];
        matrix = multiply(matrix, temp);
      }
      if (skew[1]) {
        temp[2][1] = 0;
        temp[2][0] = skew[0];
        matrix = multiply(matrix, temp);
      }
      if (skew[0]) {
        temp[2][0] = 0;
        temp[1][0] = skew[0];
        matrix = multiply(matrix, temp);
      }
      for (var _i2 = 0; _i2 < 3; _i2++) {
        for (var _j = 0; _j < 3; _j++) {
          matrix[_i2][_j] *= scale[_i2];
        }
      }
      if (is2D(matrix)) {
        return [matrix[0][0], matrix[0][1], matrix[1][0], matrix[1][1], matrix[3][0], matrix[3][1]];
      }
      return matrix[0].concat(matrix[1], matrix[2], matrix[3]);
    }
    return composeMatrix;
  }();
  function numberToLongString$1(x) {
    return x.toFixed(6).replace('.000000', '');
  }
  function mergeMatrices$1(left, right) {
    var leftArgs;
    var rightArgs;
    // @ts-ignore
    if (left.decompositionPair !== right) {
      // @ts-ignore
      left.decompositionPair = right;
      // @ts-ignore
      leftArgs = makeMatrixDecomposition$1(left);
    }
    // @ts-ignore
    if (right.decompositionPair !== left) {
      // @ts-ignore
      right.decompositionPair = left;
      // @ts-ignore
      rightArgs = makeMatrixDecomposition$1(right);
    }
    if (leftArgs[0] === null || rightArgs[0] === null) return [
    // @ts-ignore
    [false],
    // @ts-ignore
    [true],
    // @ts-ignore
    function (x) {
      return x ? right[0].d : left[0].d;
    }];
    leftArgs[0].push(0);
    rightArgs[0].push(1);
    return [leftArgs, rightArgs,
    // @ts-ignore
    function (list) {
      // @ts-ignore
      var q = quat$1(leftArgs[0][3], rightArgs[0][3], list[5]);
      var mat = composeMatrix$1(list[0], list[1], list[2], q, list[4]);
      var stringifiedArgs = mat.map(numberToLongString$1).join(',');
      return stringifiedArgs;
    }];
  }
  function dot$5(v1, v2) {
    var result = 0;
    for (var i = 0; i < v1.length; i++) {
      result += v1[i] * v2[i];
    }
    return result;
  }
  function quat$1(fromQ, toQ, f) {
    var product = dot$5(fromQ, toQ);
    product = clamp$1(product, -1.0, 1.0);
    var quat = [];
    if (product === 1.0) {
      quat = fromQ;
    } else {
      var theta = Math.acos(product);
      var w = Math.sin(f * theta) * 1 / Math.sqrt(1 - product * product);
      for (var i = 0; i < 4; i++) {
        quat.push(fromQ[i] * (Math.cos(f * theta) - product * w) + toQ[i] * w);
      }
    }
    return quat;
  }
  // scalex/y/z -> scale
  function typeTo2D$1(type) {
    return type.replace(/[xy]/, '');
  }
  // scalex/y/z -> scale3d
  function typeTo3D$1(type) {
    return type.replace(/(x|y|z|3d)?$/, '3d');
  }
  var isMatrixOrPerspective$1 = function isMatrixOrPerspective(lt, rt) {
    return lt === 'perspective' && rt === 'perspective' || (lt === 'matrix' || lt === 'matrix3d') && (rt === 'matrix' || rt === 'matrix3d');
  };
  function mergeTransforms$1(left, right, target) {
    var flipResults = false;
    // padding empty transform, eg. merge 'scale(10)' with 'none' -> scale(1)
    if (!left.length || !right.length) {
      if (!left.length) {
        flipResults = true;
        left = right;
        right = [];
      }
      var _loop = function _loop(i) {
        var _left$i = left[i],
          type = _left$i.t,
          args = _left$i.d;
        // none -> scale(1)/translateX(0)
        var defaultValue = type.substring(0, 5) === 'scale' ? 1 : 0;
        right.push({
          t: type,
          d: args.map(function (arg) {
            if (typeof arg === 'number') {
              return getOrCreateUnitValue$1(defaultValue);
            }
            return getOrCreateUnitValue$1(defaultValue, arg.unit);
            //   {
            //     unit: arg.unit,
            //     value: defaultValue,
            //   };
          })
        });
      };
      for (var i = 0; i < left.length; i++) {
        _loop(i);
      }
    }
    var leftResult = [];
    var rightResult = [];
    var types = [];
    // merge matrix() with matrix3d()
    if (left.length !== right.length) {
      var merged = mergeMatrices$1(left, right);
      // @ts-ignore
      leftResult = [merged[0]];
      // @ts-ignore
      rightResult = [merged[1]];
      types = [['matrix', [merged[2]]]];
    } else {
      for (var _i3 = 0; _i3 < left.length; _i3++) {
        var leftType = left[_i3].t;
        var rightType = right[_i3].t;
        var leftArgs = left[_i3].d;
        var rightArgs = right[_i3].d;
        var leftFunctionData = transformFunctions$1[leftType];
        var rightFunctionData = transformFunctions$1[rightType];
        var type = void 0;
        if (isMatrixOrPerspective$1(leftType, rightType)) {
          var _merged = mergeMatrices$1([left[_i3]], [right[_i3]]);
          // @ts-ignore
          leftResult.push(_merged[0]);
          // @ts-ignore
          rightResult.push(_merged[1]);
          types.push(['matrix', [_merged[2]]]);
          continue;
        } else if (leftType === rightType) {
          type = leftType;
        } else if (leftFunctionData[2] && rightFunctionData[2] && typeTo2D$1(leftType) === typeTo2D$1(rightType)) {
          type = typeTo2D$1(leftType);
          // @ts-ignore
          leftArgs = leftFunctionData[2](leftArgs);
          // @ts-ignore
          rightArgs = rightFunctionData[2](rightArgs);
        } else if (leftFunctionData[1] && rightFunctionData[1] && typeTo3D$1(leftType) === typeTo3D$1(rightType)) {
          type = typeTo3D$1(leftType);
          // @ts-ignore
          leftArgs = leftFunctionData[1](leftArgs);
          // @ts-ignore
          rightArgs = rightFunctionData[1](rightArgs);
        } else {
          var _merged2 = mergeMatrices$1(left, right);
          // @ts-ignore
          leftResult = [_merged2[0]];
          // @ts-ignore
          rightResult = [_merged2[1]];
          types = [['matrix', [_merged2[2]]]];
          break;
        }
        var leftArgsCopy = [];
        var rightArgsCopy = [];
        var stringConversions = [];
        for (var j = 0; j < leftArgs.length; j++) {
          // const merge = leftArgs[j].unit === UnitType.kNumber ? mergeDimensions : mergeDimensions;
          var _merged3 = mergeDimensions$1(leftArgs[j], rightArgs[j], target, false, j);
          leftArgsCopy[j] = _merged3[0];
          rightArgsCopy[j] = _merged3[1];
          stringConversions.push(_merged3[2]);
        }
        leftResult.push(leftArgsCopy);
        rightResult.push(rightArgsCopy);
        types.push([type, stringConversions]);
      }
    }
    if (flipResults) {
      var tmp = leftResult;
      leftResult = rightResult;
      rightResult = tmp;
    }
    return [leftResult, rightResult, function (list) {
      return list.map(function (args, i) {
        var stringifiedArgs = args.map(function (arg, j) {
          return types[i][1][j](arg);
        }).join(',');
        if (types[i][0] === 'matrix' && stringifiedArgs.split(',').length === 16) {
          types[i][0] = 'matrix3d';
        }
        if (types[i][0] === 'matrix3d' && stringifiedArgs.split(',').length === 6) {
          types[i][0] = 'matrix';
        }
        return types[i][0] + '(' + stringifiedArgs + ')';
      }).join(' ');
    }];
  }

  /**
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/transform-origin
   * eg. 'center' 'top left' '50px 50px'
   */
  var parseTransformOrigin$1 = memoize$1(function (value) {
    if (isString$1(value)) {
      if (value === 'text-anchor') {
        return [getOrCreateUnitValue$1(0, 'px'), getOrCreateUnitValue$1(0, 'px')];
      }
      var values = value.split(' ');
      if (values.length === 1) {
        if (values[0] === 'top' || values[0] === 'bottom') {
          // 'top' -> 'center top'
          values[1] = values[0];
          values[0] = 'center';
        } else {
          // '50px' -> '50px center'
          values[1] = 'center';
        }
      }
      if (values.length !== 2) {
        return null;
      }
      // eg. center bottom
      return [parseLengthOrPercentage$1(convertKeyword2Percent$1(values[0])), parseLengthOrPercentage$1(convertKeyword2Percent$1(values[1]))];
    } else {
      return [getOrCreateUnitValue$1(value[0] || 0, 'px'), getOrCreateUnitValue$1(value[1] || 0, 'px')];
    }
  });
  function convertKeyword2Percent$1(keyword) {
    if (keyword === 'center') {
      return '50%';
    } else if (keyword === 'left' || keyword === 'top') {
      return '0';
    } else if (keyword === 'right' || keyword === 'bottom') {
      return '100%';
    }
    return keyword;
  }

  var CSSPropertyAngle$1 = /*#__PURE__*/function () {
    function CSSPropertyAngle() {
      this.parser = parseAngle$1;
      this.mixer = mergeNumbers$1;
    }
    var _proto = CSSPropertyAngle.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      return convertAngleUnit$1(parsed);
    };
    return CSSPropertyAngle;
  }();

  /**
   * clipPath / textPath / offsetPath
   */
  var CSSPropertyClipPath$1 = /*#__PURE__*/function () {
    function CSSPropertyClipPath() {}
    var _proto = CSSPropertyClipPath.prototype;
    _proto.calculator = function calculator(name, oldPath, newPath, object) {
      // unset
      if (newPath instanceof CSSKeywordValue$1) {
        newPath = null;
      }
      runtime$2.sceneGraphService.updateDisplayObjectDependency(name, oldPath, newPath, object);
      if (name === 'clipPath') {
        // should affect children
        object.forEach(function (leaf) {
          if (leaf.childNodes.length === 0) {
            runtime$2.sceneGraphService.dirtifyToRoot(leaf);
          }
        });
      }
      return newPath;
    };
    return CSSPropertyClipPath;
  }();

  var CSSPropertyColor$1 = /*#__PURE__*/function () {
    function CSSPropertyColor() {
      this.parser = parseColor$1;
      this.mixer = mergeColors$1;
    }
    var _proto = CSSPropertyColor.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      if (parsed instanceof CSSKeywordValue$1) {
        // 'unset' 'none'
        return parsed.value === 'none' ? noneColor$1 : transparentColor$1;
      }
      return parsed;
    };
    return CSSPropertyColor;
  }();

  var CSSPropertyFilter$1 = /*#__PURE__*/function () {
    function CSSPropertyFilter() {
      this.parser = parseFilter$1;
    }
    var _proto = CSSPropertyFilter.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed) {
      // unset or none
      if (parsed instanceof CSSKeywordValue$1) {
        return [];
      }
      return parsed;
    };
    return CSSPropertyFilter;
  }();

  function getFontSize$1(object) {
    var fontSize = object.parsedStyle.fontSize;
    return isNil$1(fontSize) ? null : fontSize;
  }
  /**
   * <length> & <percentage>
   */
  var CSSPropertyLengthOrPercentage$1 = /*#__PURE__*/function () {
    function CSSPropertyLengthOrPercentage() {
      this.parser = parseLengthOrPercentage$1;
      this.mixer = mergeNumbers$1;
    }
    var _proto = CSSPropertyLengthOrPercentage.prototype;
    /**
     * according to parent's bounds
     *
     * @example
     * CSS.percent(50) -> CSS.px(0.5 * parent.width)
     */
    _proto.calculator = function calculator(name, oldParsed, computed, object, registry) {
      if (CSSUnitValue$1.isRelativeUnit(computed.unit)) {
        if (computed.unit === UnitType$1.kPercentage) {
          // TODO: merge dimensions
          return 0;
        } else if (computed.unit === UnitType$1.kEms) {
          if (object.parentNode) {
            var fontSize = getFontSize$1(object.parentNode);
            if (fontSize) {
              fontSize *= computed.value;
              return fontSize;
            } else {
              registry.addUnresolveProperty(object, name);
            }
          } else {
            registry.addUnresolveProperty(object, name);
          }
          return 0;
        } else if (computed.unit === UnitType$1.kRems) {
          var _object$ownerDocument;
          if (object === null || object === void 0 ? void 0 : (_object$ownerDocument = object.ownerDocument) === null || _object$ownerDocument === void 0 ? void 0 : _object$ownerDocument.documentElement) {
            var _fontSize = getFontSize$1(object.ownerDocument.documentElement);
            if (_fontSize) {
              _fontSize *= computed.value;
              return _fontSize;
            } else {
              registry.addUnresolveProperty(object, name);
            }
          } else {
            registry.addUnresolveProperty(object, name);
          }
          return 0;
        }
      } else {
        // remove listener if exists
        // registry.unregisterParentGeometryBoundsChangedHandler(object, name);
        // return absolute value
        return computed.value;
      }
    };
    return CSSPropertyLengthOrPercentage;
  }();

  /**
   * format to Tuple2<CSSUnitValue>
   *
   * @example
   * rect.style.lineDash = 10;
   * rect.style.lineDash = [10, 10];
   * rect.style.lineDash = '10 10';
   */
  var CSSPropertyLengthOrPercentage12$1 = /*#__PURE__*/function () {
    function CSSPropertyLengthOrPercentage12() {
      this.mixer = mergeNumberLists$1;
    }
    var _proto = CSSPropertyLengthOrPercentage12.prototype;
    _proto.parser = function parser(radius) {
      var parsed = parseDimensionArray$1(isNumber$1(radius) ? [radius] : radius);
      var formatted;
      if (parsed.length === 1) {
        formatted = [parsed[0], parsed[0]];
      } else {
        formatted = [parsed[0], parsed[1]];
      }
      return formatted;
    };
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.map(function (c) {
        return c.value;
      });
    };
    return CSSPropertyLengthOrPercentage12;
  }();

  /**
   * used in rounded rect
   *
   * @example
   * rect.style.radius = 10;
   * rect.style.radius = '10 10';
   * rect.style.radius = '10 10 10 10';
   */
  var CSSPropertyLengthOrPercentage14$1 = /*#__PURE__*/function () {
    function CSSPropertyLengthOrPercentage14() {
      this.mixer = mergeNumberLists$1;
    }
    var _proto = CSSPropertyLengthOrPercentage14.prototype;
    _proto.parser = function parser(radius) {
      var parsed = parseDimensionArray$1(isNumber$1(radius) ? [radius] : radius);
      var formatted;
      // format to Tuple<CSSUnitValue>
      if (parsed.length === 1) {
        formatted = [parsed[0], parsed[0], parsed[0], parsed[0]];
      } else if (parsed.length === 2) {
        formatted = [parsed[0], parsed[1], parsed[0], parsed[1]];
      } else if (parsed.length === 3) {
        formatted = [parsed[0], parsed[1], parsed[2], parsed[1]];
      } else {
        formatted = [parsed[0], parsed[1], parsed[2], parsed[3]];
      }
      return formatted;
    };
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.map(function (c) {
        return c.value;
      });
    };
    return CSSPropertyLengthOrPercentage14;
  }();

  /**
   * local position
   */
  var CSSPropertyLocalPosition$1 = /*#__PURE__*/function (_CSSPropertyLengthOrP) {
    _inheritsLoose$9(CSSPropertyLocalPosition, _CSSPropertyLengthOrP);
    function CSSPropertyLocalPosition() {
      return _CSSPropertyLengthOrP.apply(this, arguments) || this;
    }
    var _proto = CSSPropertyLocalPosition.prototype;
    /**
     * update local position
     */
    _proto.postProcessor = function postProcessor(object) {
      var x;
      var y;
      var z;
      switch (object.nodeName) {
        case Shape$2.CIRCLE:
        case Shape$2.ELLIPSE:
          var _object$parsedStyle = object.parsedStyle,
            cx = _object$parsedStyle.cx,
            cy = _object$parsedStyle.cy;
          if (!isNil$1(cx)) {
            x = cx;
          }
          if (!isNil$1(cy)) {
            y = cy;
          }
          break;
        case Shape$2.LINE:
          var _object$parsedStyle2 = object.parsedStyle,
            x1 = _object$parsedStyle2.x1,
            x2 = _object$parsedStyle2.x2,
            y1 = _object$parsedStyle2.y1,
            y2 = _object$parsedStyle2.y2;
          var minX = Math.min(x1, x2);
          var minY = Math.min(y1, y2);
          x = minX;
          y = minY;
          z = 0;
          break;
        case Shape$2.RECT:
        case Shape$2.IMAGE:
        case Shape$2.GROUP:
        case Shape$2.HTML:
        case Shape$2.TEXT:
        case Shape$2.MESH:
          if (!isNil$1(object.parsedStyle.x)) {
            x = object.parsedStyle.x;
          }
          if (!isNil$1(object.parsedStyle.y)) {
            y = object.parsedStyle.y;
          }
          break;
      }
      if (object.nodeName !== Shape$2.PATH && object.nodeName !== Shape$2.POLYLINE && object.nodeName !== Shape$2.POLYGON) {
        object.parsedStyle.defX = x || 0;
        object.parsedStyle.defY = y || 0;
      }
      var needResetLocalPosition = !isNil$1(x) || !isNil$1(y) || !isNil$1(z);
      if (needResetLocalPosition) {
        var _object$getLocalPosit = object.getLocalPosition(),
          ox = _object$getLocalPosit[0],
          oy = _object$getLocalPosit[1],
          oz = _object$getLocalPosit[2];
        object.setLocalPosition(isNil$1(x) ? ox : x, isNil$1(y) ? oy : y, isNil$1(z) ? oz : z);
      }
    };
    return CSSPropertyLocalPosition;
  }(CSSPropertyLengthOrPercentage$1);

  var CSSPropertyMarker$1 = /*#__PURE__*/function () {
    function CSSPropertyMarker() {}
    var _proto = CSSPropertyMarker.prototype;
    _proto.calculator = function calculator(name, oldMarker, newMarker, object) {
      var _newMarker;
      // unset
      if (newMarker instanceof CSSKeywordValue$1) {
        newMarker = null;
      }
      var cloned = (_newMarker = newMarker) === null || _newMarker === void 0 ? void 0 : _newMarker.cloneNode(true);
      if (cloned) {
        // FIXME: SVG should not inherit parent's style, add a flag here
        cloned.style.isMarker = true;
      }
      return cloned;
    };
    return CSSPropertyMarker;
  }();

  var CSSPropertyNumber$1 = /*#__PURE__*/function () {
    function CSSPropertyNumber() {
      this.mixer = mergeNumbers$1;
      this.parser = parseNumber$1;
    }
    var _proto = CSSPropertyNumber.prototype;
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.value;
    };
    return CSSPropertyNumber;
  }();

  var CSSPropertyOffsetDistance$1 = /*#__PURE__*/function () {
    function CSSPropertyOffsetDistance() {
      this.parser = parseNumber$1;
      this.mixer = clampedMergeNumbers$1(0, 1);
    }
    var _proto = CSSPropertyOffsetDistance.prototype;
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.value;
    };
    _proto.postProcessor = function postProcessor(object) {
      var _object$parsedStyle = object.parsedStyle,
        offsetPath = _object$parsedStyle.offsetPath,
        offsetDistance = _object$parsedStyle.offsetDistance;
      if (!offsetPath) {
        return;
      }
      var nodeName = offsetPath.nodeName;
      if (nodeName === Shape$2.LINE || nodeName === Shape$2.PATH || nodeName === Shape$2.POLYLINE) {
        // set position in world space
        var point = offsetPath.getPoint(offsetDistance);
        if (point) {
          object.parsedStyle.defX = point.x;
          object.parsedStyle.defY = point.y;
          object.setLocalPosition(point.x, point.y);
        }
      }
    };
    return CSSPropertyOffsetDistance;
  }();

  /**
   * opacity
   */
  var CSSPropertyOpacity$1 = /*#__PURE__*/function () {
    function CSSPropertyOpacity() {
      this.parser = parseNumber$1;
      this.mixer = clampedMergeNumbers$1(0, 1);
    }
    var _proto = CSSPropertyOpacity.prototype;
    _proto.calculator = function calculator(name, oldParsed, computed) {
      return computed.value;
    };
    return CSSPropertyOpacity;
  }();

  var CSSPropertyPath$1 = /*#__PURE__*/function () {
    function CSSPropertyPath() {
      this.parser = parsePath$1;
      this.mixer = mergePaths$1;
    }
    var _proto = CSSPropertyPath.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed) {
      // unset
      if (parsed instanceof CSSKeywordValue$1 && parsed.value === 'unset') {
        return {
          absolutePath: [],
          hasArc: false,
          segments: [],
          polygons: [],
          polylines: [],
          curve: null,
          totalLength: 0,
          rect: new Rectangle$1(0, 0, 0, 0)
        };
      }
      return parsed;
    };
    /**
     * update local position
     */
    _proto.postProcessor = function postProcessor(object) {
      if (object.nodeName === Shape$2.PATH) {
        var _object$parsedStyle = object.parsedStyle,
          _object$parsedStyle$d = _object$parsedStyle.defX,
          defX = _object$parsedStyle$d === void 0 ? 0 : _object$parsedStyle$d,
          _object$parsedStyle$d2 = _object$parsedStyle.defY,
          defY = _object$parsedStyle$d2 === void 0 ? 0 : _object$parsedStyle$d2;
        object.setLocalPosition(defX, defY);
      }
    };
    return CSSPropertyPath;
  }();

  var CSSPropertyPoints$1 = /*#__PURE__*/function () {
    function CSSPropertyPoints() {
      this.parser = parsePoints$1;
    }
    var _proto = CSSPropertyPoints.prototype;
    /**
     * update local position
     */
    _proto.postProcessor = function postProcessor(object) {
      if (object.nodeName === Shape$2.POLYGON || object.nodeName === Shape$2.POLYLINE) {
        var _object$parsedStyle = object.parsedStyle,
          defX = _object$parsedStyle.defX,
          defY = _object$parsedStyle.defY;
        object.setLocalPosition(defX, defY);
      }
    };
    return CSSPropertyPoints;
  }();

  var CSSPropertyShadowBlur$1 = /*#__PURE__*/function (_CSSPropertyLengthOrP) {
    _inheritsLoose$9(CSSPropertyShadowBlur, _CSSPropertyLengthOrP);
    function CSSPropertyShadowBlur() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _CSSPropertyLengthOrP.call.apply(_CSSPropertyLengthOrP, [this].concat(args)) || this;
      _this.mixer = clampedMergeNumbers$1(0, Infinity);
      return _this;
    }
    return CSSPropertyShadowBlur;
  }(CSSPropertyLengthOrPercentage$1);

  var CSSPropertyText$1 = /*#__PURE__*/function () {
    function CSSPropertyText() {}
    var _proto = CSSPropertyText.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      if (parsed instanceof CSSKeywordValue$1) {
        if (parsed.value === 'unset') {
          return '';
        } else {
          return parsed.value;
        }
      }
      // allow number as valid text content
      return "" + parsed;
    };
    _proto.postProcessor = function postProcessor(object) {
      object.nodeValue = "" + object.parsedStyle.text || '';
    };
    return CSSPropertyText;
  }();

  /**
   * it must transform after text get parsed
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/text-transform
   */
  var CSSPropertyTextTransform$1 = /*#__PURE__*/function () {
    function CSSPropertyTextTransform() {}
    var _proto = CSSPropertyTextTransform.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      var rawText = object.getAttribute('text');
      if (rawText) {
        var transformedText = rawText;
        if (parsed.value === 'capitalize') {
          transformedText = rawText.charAt(0).toUpperCase() + rawText.slice(1);
        } else if (parsed.value === 'lowercase') {
          transformedText = rawText.toLowerCase();
        } else if (parsed.value === 'uppercase') {
          transformedText = rawText.toUpperCase();
        }
        object.parsedStyle.text = transformedText;
      }
      return parsed.value;
    };
    return CSSPropertyTextTransform;
  }();

  /**
   * @see /zh/docs/api/animation#支持变换的属性
   *
   * support the following formats like CSS Transform:
   *
   * scale
   * * scale(x, y)
   * * scaleX(x)
   * * scaleY(x)
   * * scaleZ(z)
   * * scale3d(x, y, z)
   *
   * translate (unit: none, px, %(relative to its bounds))
   * * translate(x, y) eg. translate(0, 0) translate(0, 30px) translate(100%, 100%)
   * * translateX(0)
   * * translateY(0)
   * * translateZ(0)
   * * translate3d(0, 0, 0)
   *
   * rotate (unit: deg rad turn)
   * * rotate(0.5turn) rotate(30deg) rotate(1rad)
   *
   * none
   *
   * unsupported for now:
   * * calc() eg. translate(calc(100% + 10px))
   * * matrix/matrix3d()
   * * skew/skewX/skewY
   * * perspective
   */
  var CSSPropertyTransform$1 = /*#__PURE__*/function () {
    function CSSPropertyTransform() {
      this.parser = parseTransform$1;
      this.mixer = mergeTransforms$1;
    }
    var _proto = CSSPropertyTransform.prototype;
    _proto.calculator = function calculator(name, oldParsed, parsed, object) {
      // 'none'
      if (parsed instanceof CSSKeywordValue$1) {
        return [];
      }
      return parsed;
    };
    _proto.postProcessor = function postProcessor(object) {
      var transform = object.parsedStyle.transform;
      parsedTransformToMat4(transform, object);
    };
    return CSSPropertyTransform;
  }();

  /**
   * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/transform-origin
   * @example
   * [10px, 10px] [10%, 10%]
   */
  var CSSPropertyTransformOrigin$1 = function CSSPropertyTransformOrigin() {
    this.parser = parseTransformOrigin$1;
  };

  var CSSPropertyZIndex$1 = /*#__PURE__*/function () {
    function CSSPropertyZIndex() {
      this.parser = parseNumber$1;
    }
    var _proto = CSSPropertyZIndex.prototype;
    _proto.calculator = function calculator(name, oldParsed, computed, object) {
      return computed.value;
    };
    _proto.postProcessor = function postProcessor(object) {
      if (object.parentNode) {
        var parentEntity = object.parentNode;
        var parentRenderable = parentEntity.renderable;
        var parentSortable = parentEntity.sortable;
        if (parentRenderable) {
          parentRenderable.dirty = true;
        }
        // need re-sort on parent
        if (parentSortable) {
          parentSortable.dirty = true;
        }
      }
    };
    return CSSPropertyZIndex;
  }();

  /**
   * canvas.customElements
   *
   * @see https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry
   */
  var CustomElementRegistry$1 = /*#__PURE__*/function () {
    function CustomElementRegistry() {
      this.registry = {};
      this.define(Shape$2.CIRCLE, Circle$1);
      this.define(Shape$2.ELLIPSE, Ellipse$1);
      this.define(Shape$2.RECT, Rect$3);
      this.define(Shape$2.IMAGE, Image$1);
      this.define(Shape$2.LINE, Line$2);
      this.define(Shape$2.GROUP, Group$1);
      this.define(Shape$2.PATH, Path$1);
      this.define(Shape$2.POLYGON, Polygon$1);
      this.define(Shape$2.POLYLINE, Polyline$1);
      this.define(Shape$2.TEXT, Text$1);
      this.define(Shape$2.HTML, HTML$1);
    }
    var _proto = CustomElementRegistry.prototype;
    _proto.define = function define(name, constructor) {
      this.registry[name] = constructor;
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry/get
     */;
    _proto.get = function get(name) {
      return this.registry[name];
    };
    return CustomElementRegistry;
  }();

  /**
   * An DOM-compatible synthetic event implementation that is "forwarded" on behalf of an original
   * FederatedEvent or native {@link https://dom.spec.whatwg.org/#event Event}.
   */
  var FederatedEvent$1 = /*#__PURE__*/function () {
    /**
     * The event boundary which manages this event. Propagation can only occur
     *  within the boundary's jurisdiction.
     */
    function FederatedEvent(manager) {
      this.type = void 0;
      this.eventPhase = FederatedEvent.prototype.NONE;
      this.target = void 0;
      this.bubbles = true;
      this.cancelBubble = true;
      this.cancelable = false;
      this.currentTarget = void 0;
      this.defaultPrevented = false;
      this.timeStamp = void 0;
      this.nativeEvent = void 0;
      this.originalEvent = void 0;
      this.propagationStopped = false;
      this.propagationImmediatelyStopped = false;
      this.manager = void 0;
      this.detail = void 0;
      this.layer = new Point$1();
      this.page = new Point$1();
      this.canvas = new Point$1();
      this.viewport = new Point$1();
      this.path = void 0;
      this.view = void 0;
      this.which = void 0;
      this.returnValue = void 0;
      this.srcElement = void 0;
      this.composed = false;
      this.isTrusted = void 0;
      this.NONE = 0;
      this.CAPTURING_PHASE = 1;
      this.AT_TARGET = 2;
      this.BUBBLING_PHASE = 3;
      this.manager = manager;
    }
    var _proto = FederatedEvent.prototype;
    /**
     * The propagation path for this event
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Event/composedPath
     *
     * So composedPath()[0] represents the original target.
     * @see https://polymer-library.polymer-project.org/3.0/docs/devguide/events#retargeting
     */
    _proto.composedPath = function composedPath() {
      if (this.manager && (!this.path || this.path[0] !== this.target)) {
        this.path = this.target ? this.manager.propagationPath(this.target) : [];
      }
      return this.path;
    }
    /**
     * @deprecated
     */;
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Event/preventDefault
     */
    _proto.preventDefault = function preventDefault() {
      if (this.nativeEvent instanceof Event && this.nativeEvent.cancelable) {
        this.nativeEvent.preventDefault();
      }
      this.defaultPrevented = true;
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Event/stopImmediatePropagation
     */;
    _proto.stopImmediatePropagation = function stopImmediatePropagation() {
      this.propagationImmediatelyStopped = true;
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Event/stopPropagation
     */;
    _proto.stopPropagation = function stopPropagation() {
      this.propagationStopped = true;
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/UIEvent/view
     */;
    /**
     * added for compatibility with DOM Event,
     * deprecated props and methods
     */
    _proto.initEvent = function initEvent() {};
    _proto.initUIEvent = function initUIEvent() {};
    _proto.clone = function clone() {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _createClass$5(FederatedEvent, [{
      key: "name",
      get:
      /**
       * The type of event, supports the following:
       * * pointerdown
       * * touchstart
       * * mousedown
       * * rightdown
       * * ...
       */

      /**
       * @deprecated
       */
      function get() {
        return this.type;
      }
      /**
       * The propagation phase.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Event/eventPhase
       */
    }, {
      key: "layerX",
      get: function get() {
        return this.layer.x;
      }
    }, {
      key: "layerY",
      get: function get() {
        return this.layer.y;
      }
      /**
       * The coordinates of the event relative to the DOM document.
       * This is a non-standard property.
       * relative to the DOM document.
       * @see https://developer.mozilla.org/zh-CN/docs/Web/API/MouseEvent/pageX
       */
    }, {
      key: "pageX",
      get: function get() {
        return this.page.x;
      }
    }, {
      key: "pageY",
      get: function get() {
        return this.page.y;
      }
      /**
       * relative to Canvas, origin is left-top
       */
    }, {
      key: "x",
      get: function get() {
        return this.canvas.x;
      }
    }, {
      key: "y",
      get: function get() {
        return this.canvas.y;
      }
    }, {
      key: "canvasX",
      get: function get() {
        return this.canvas.x;
      }
    }, {
      key: "canvasY",
      get: function get() {
        return this.canvas.y;
      }
      /**
       * relative to Viewport, account for Camera
       */
    }, {
      key: "viewportX",
      get: function get() {
        return this.viewport.x;
      }
    }, {
      key: "viewportY",
      get: function get() {
        return this.viewport.y;
      }
    }, {
      key: "propagationPath",
      get: function get() {
        return this.composedPath();
      }
    }]);
    return FederatedEvent;
  }();

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/Events/Creating_and_triggering_events
   *
   * @example
    const event = new CustomEvent('build', { detail: { prop1: 'xx' } });
    circle.addEventListener('build', (e) => {
      e.target; // circle
      e.detail; // { prop1: 'xx' }
    });

    circle.dispatchEvent(event);
   */
  var CustomEvent$1 = /*#__PURE__*/function (_FederatedEvent) {
    _inheritsLoose$9(CustomEvent, _FederatedEvent);
    // eslint-disable-next-line @typescript-eslint/ban-types
    function CustomEvent(eventName, object) {
      var _this;
      _this = _FederatedEvent.call(this, null) || this;
      _this.type = eventName;
      _this.detail = object;
      // compatible with G 3.0
      Object.assign(_assertThisInitialized$5(_this), object);
      return _this;
    }
    return CustomEvent;
  }(FederatedEvent$1);

  var DELEGATION_SPLITTER$1 = ':';
  /**
   * Objects that can receive events and may have listeners for them.
   * eg. Element, Canvas, DisplayObject
   * @see https://developer.mozilla.org/en-US/docs/Web/API/EventTarget
   */
  var EventTarget$1 = /*#__PURE__*/function () {
    function EventTarget() {
      this.emitter = new eventemitter3$1();
    }
    var _proto = EventTarget.prototype;
    /**
     * @deprecated
     * @alias addEventListener
     */
    _proto.on = function on(type, listener, options) {
      this.addEventListener(type, listener, options);
      return this;
    }
    /**
     * support `capture` & `once` in options
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/EventTarget/addEventListener
     */;
    _proto.addEventListener = function addEventListener(type, listener, options) {
      var capture = isBoolean$1(options) && options || isObject$1(options) && options.capture;
      var once = isObject$1(options) && options.once;
      var context = isFunction$3(listener) ? undefined : listener;
      // compatible with G 3.0
      // support using delegate name in event type, eg. 'node:click'
      var useDelegatedName = false;
      var delegatedName = '';
      if (type.indexOf(DELEGATION_SPLITTER$1) > -1) {
        var _type$split = type.split(DELEGATION_SPLITTER$1),
          name = _type$split[0],
          eventType = _type$split[1];
        type = eventType;
        delegatedName = name;
        useDelegatedName = true;
      }
      type = capture ? type + "capture" : type;
      listener = isFunction$3(listener) ? listener : listener.handleEvent;
      // compatible with G 3.0
      if (useDelegatedName) {
        var originListener = listener;
        listener = function listener() {
          var _args$0$target;
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }
          if (((_args$0$target = args[0].target) === null || _args$0$target === void 0 ? void 0 : _args$0$target.name) !== delegatedName) {
            return;
          }
          // @ts-ignore
          originListener.apply(void 0, args);
        };
      }
      if (once) {
        this.emitter.once(type, listener, context);
      } else {
        this.emitter.on(type, listener, context);
      }
      return this;
    }
    /**
     * @deprecated
     * @alias removeEventListener
     */;
    _proto.off = function off(type, listener, options) {
      if (type) {
        this.removeEventListener(type, listener, options);
      } else {
        // remove all listeners
        this.removeAllEventListeners();
      }
      return this;
    };
    _proto.removeAllEventListeners = function removeAllEventListeners() {
      this.emitter.removeAllListeners();
    };
    _proto.removeEventListener = function removeEventListener(type, listener, options) {
      var _listener;
      var capture = isBoolean$1(options) && options || isObject$1(options) && options.capture;
      var context = isFunction$3(listener) ? undefined : listener;
      type = capture ? type + "capture" : type;
      listener = isFunction$3(listener) ? listener : (_listener = listener) === null || _listener === void 0 ? void 0 : _listener.handleEvent;
      this.emitter.off(type, listener, context);
      return this;
    }
    /**
     * @deprecated
     * @alias dispatchEvent
     */
    // eslint-disable-next-line @typescript-eslint/ban-types
    ;
    _proto.emit = function emit(eventName, object) {
      this.dispatchEvent(new CustomEvent$1(eventName, object));
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/dispatchEvent
     */;
    _proto.dispatchEvent = function dispatchEvent(e, skipPropagate) {
      if (skipPropagate === void 0) {
        skipPropagate = false;
      }
      if (!(e instanceof FederatedEvent$1)) {
        throw new Error('DisplayObject cannot propagate events outside of the Federated Events API');
      }
      // should account for Element / Document / Canvas
      var canvas;
      // @ts-ignore
      if (this.document) {
        canvas = this;
        // @ts-ignore
      } else if (this.defaultView) {
        canvas = this.defaultView;
      } else {
        var _this$ownerDocument;
        canvas = (_this$ownerDocument = this.ownerDocument) === null || _this$ownerDocument === void 0 ? void 0 : _this$ownerDocument.defaultView;
      }
      // assign event manager
      if (canvas) {
        var _e$manager;
        e.manager = canvas.getEventService() || null;
        if (!e.manager) {
          return false;
        }
        e.defaultPrevented = false;
        e.path = [];
        if (!skipPropagate) {
          e.target = this;
        }
        (_e$manager = e.manager) === null || _e$manager === void 0 ? void 0 : _e$manager.dispatchEvent(e, e.type, skipPropagate);
      }
      return !e.defaultPrevented;
    };
    return EventTarget;
  }();

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/API/Node
   */
  var Node$2 = /*#__PURE__*/function (_EventTarget) {
    _inheritsLoose$9(Node, _EventTarget);
    function Node() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _EventTarget.call.apply(_EventTarget, [this].concat(args)) || this;
      _this.shadow = false;
      _this.ownerDocument = null;
      _this.isConnected = false;
      _this.baseURI = '';
      _this.childNodes = [];
      _this.nodeType = 0;
      _this.nodeName = '';
      _this.nodeValue = null;
      _this.parentNode = null;
      return _this;
    }
    Node.isNode = function isNode(target) {
      return !!target.childNodes;
    };
    var _proto = Node.prototype;
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Node/getRootNode
     */
    _proto.getRootNode = function getRootNode(opts) {
      if (opts === void 0) {
        opts = {};
      }
      if (this.parentNode) {
        return this.parentNode.getRootNode(opts);
      }
      if (opts.composed && this.host) {
        return this.host.getRootNode(opts);
      }
      return this;
    };
    _proto.hasChildNodes = function hasChildNodes() {
      return this.childNodes.length > 0;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.isDefaultNamespace = function isDefaultNamespace(namespace) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.lookupNamespaceURI = function lookupNamespaceURI(prefix) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.lookupPrefix = function lookupPrefix(namespace) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.normalize = function normalize() {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Node/isEqualNode
     */;
    _proto.isEqualNode = function isEqualNode(otherNode) {
      // TODO: compare 2 nodes, not sameness
      return this === otherNode;
    };
    _proto.isSameNode = function isSameNode(otherNode) {
      return this.isEqualNode(otherNode);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/ParentNode
     */;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Node/compareDocumentPosition
     * @see https://github.com/b-fuze/deno-dom/blob/master/src/dom/node.ts#L338
     */
    _proto.compareDocumentPosition = function compareDocumentPosition(other) {
      if (other === this) {
        // same node
        return 0;
      }
      if (!(other instanceof Node)) {
        throw new TypeError('Node.compareDocumentPosition: Argument 1 does not implement interface Node.');
      }
      var node1Root = other;
      // eslint-disable-next-line @typescript-eslint/no-this-alias
      var node2Root = this;
      var node1Hierarchy = [node1Root];
      var node2Hierarchy = [node2Root];
      while ((_node1Root$parentNode = node1Root.parentNode) !== null && _node1Root$parentNode !== void 0 ? _node1Root$parentNode : node2Root.parentNode) {
        var _node1Root$parentNode;
        node1Root = node1Root.parentNode ? (node1Hierarchy.push(node1Root.parentNode), node1Root.parentNode) : node1Root;
        node2Root = node2Root.parentNode ? (node2Hierarchy.push(node2Root.parentNode), node2Root.parentNode) : node2Root;
      }
      // Check if they don't share the same root node
      if (node1Root !== node2Root) {
        return Node.DOCUMENT_POSITION_DISCONNECTED | Node.DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC | Node.DOCUMENT_POSITION_PRECEDING;
      }
      var longerHierarchy = node1Hierarchy.length > node2Hierarchy.length ? node1Hierarchy : node2Hierarchy;
      var shorterHierarchy = longerHierarchy === node1Hierarchy ? node2Hierarchy : node1Hierarchy;
      // Check if either is a container of the other
      if (longerHierarchy[longerHierarchy.length - shorterHierarchy.length] === shorterHierarchy[0]) {
        return longerHierarchy === node1Hierarchy ?
        // other is a child of this
        Node.DOCUMENT_POSITION_CONTAINED_BY | Node.DOCUMENT_POSITION_FOLLOWING :
        // this is a child of other
        Node.DOCUMENT_POSITION_CONTAINS | Node.DOCUMENT_POSITION_PRECEDING;
      }
      // Find their first common ancestor and see whether they
      // are preceding or following
      var longerStart = longerHierarchy.length - shorterHierarchy.length;
      for (var i = shorterHierarchy.length - 1; i >= 0; i--) {
        var shorterHierarchyNode = shorterHierarchy[i];
        var longerHierarchyNode = longerHierarchy[longerStart + i];
        // We found the first common ancestor
        if (longerHierarchyNode !== shorterHierarchyNode) {
          var siblings = shorterHierarchyNode.parentNode.childNodes;
          if (siblings.indexOf(shorterHierarchyNode) < siblings.indexOf(longerHierarchyNode)) {
            // Shorter is before longer
            if (shorterHierarchy === node1Hierarchy) {
              // Other is before this
              return Node.DOCUMENT_POSITION_PRECEDING;
            } else {
              // This is before other
              return Node.DOCUMENT_POSITION_FOLLOWING;
            }
          } else {
            // Longer is before shorter
            if (longerHierarchy === node1Hierarchy) {
              // Other is before this
              return Node.DOCUMENT_POSITION_PRECEDING;
            } else {
              // Other is after this
              return Node.DOCUMENT_POSITION_FOLLOWING;
            }
          }
        }
      }
      return Node.DOCUMENT_POSITION_FOLLOWING;
    }
    /**
     * @deprecated
     * @alias contains
     */;
    _proto.contain = function contain(other) {
      return this.contains(other);
    };
    _proto.contains = function contains(other) {
      // the node itself, one of its direct children
      var tmp = other;
      // @see https://developer.mozilla.org/en-US/docs/Web/API/Node/contains
      while (tmp && this !== tmp) {
        tmp = tmp.parentNode;
      }
      return !!tmp;
    };
    _proto.getAncestor = function getAncestor(n) {
      // eslint-disable-next-line @typescript-eslint/no-this-alias
      var temp = this;
      while (n > 0 && temp) {
        temp = temp.parentNode;
        n--;
      }
      return temp;
    };
    _proto.forEach = function forEach(callback) {
      if (!callback(this)) {
        this.childNodes.forEach(function (child) {
          child.forEach(callback);
        });
      }
    };
    _createClass$5(Node, [{
      key: "textContent",
      get:
      /**
       * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Node/textContent
       */
      function get() {
        var out = '';
        if (this.nodeName === Shape$2.TEXT) {
          // @ts-ignore
          out += this.style.text;
        }
        for (var _iterator = _createForOfIteratorHelperLoose$2(this.childNodes), _step; !(_step = _iterator()).done;) {
          var child = _step.value;
          if (child.nodeName === Shape$2.TEXT) {
            out += child.nodeValue;
          } else {
            out += child.textContent;
          }
        }
        return out;
      },
      set: function set(content) {
        var _this2 = this;
        // remove all children
        this.childNodes.slice().forEach(function (child) {
          _this2.removeChild(child);
        });
        if (this.nodeName === Shape$2.TEXT) {
          // @ts-ignore
          this.style.text = "" + content;
        }
      }
    }, {
      key: "parent",
      get:
      /**
       * @deprecated
       * @alias parentNode
       */
      function get() {
        return this.parentNode;
      }
    }, {
      key: "parentElement",
      get: function get() {
        return null;
      }
    }, {
      key: "nextSibling",
      get: function get() {
        return null;
      }
    }, {
      key: "previousSibling",
      get: function get() {
        return null;
      }
    }, {
      key: "firstChild",
      get: function get() {
        return this.childNodes.length > 0 ? this.childNodes[0] : null;
      }
    }, {
      key: "lastChild",
      get: function get() {
        return this.childNodes.length > 0 ? this.childNodes[this.childNodes.length - 1] : null;
      }
    }]);
    return Node;
  }(EventTarget$1);
  Node$2.DOCUMENT_POSITION_DISCONNECTED = 1;
  Node$2.DOCUMENT_POSITION_PRECEDING = 2;
  Node$2.DOCUMENT_POSITION_FOLLOWING = 4;
  Node$2.DOCUMENT_POSITION_CONTAINS = 8;
  Node$2.DOCUMENT_POSITION_CONTAINED_BY = 16;
  Node$2.DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC = 32;

  /**
   * the entry of DOM tree
   * Document -> Node -> EventTarget
   * @see https://developer.mozilla.org/en-US/docs/Web/API/Document
   */
  var Document$1 = /*#__PURE__*/function (_Node) {
    _inheritsLoose$9(Document, _Node);
    function Document() {
      var _this;
      _this = _Node.call(this) || this;
      _this.defaultView = null;
      _this.documentElement = void 0;
      _this.timeline = void 0;
      _this.ownerDocument = null;
      _this.nodeName = 'document';
      // create timeline
      try {
        _this.timeline = new runtime$2.AnimationTimeline(_assertThisInitialized$5(_this));
      } catch (e) {}
      /**
       * for inherited properties, the initial value is used on the root element only,
       * as long as no specified value is supplied.
       * @see https://developer.mozilla.org/en-US/docs/Web/CSS/initial_value
       */
      var initialStyle = {};
      BUILT_IN_PROPERTIES$1.forEach(function (_ref) {
        var n = _ref.n,
          inh = _ref.inh,
          d = _ref.d;
        if (inh && d) {
          initialStyle[n] = isFunction$3(d) ? d(Shape$2.GROUP) : d;
        }
      });
      // like <html> in DOM tree
      _this.documentElement = new Group$1({
        id: 'g-root',
        style: initialStyle
      });
      _this.documentElement.ownerDocument = _assertThisInitialized$5(_this);
      _this.documentElement.parentNode = _assertThisInitialized$5(_this);
      _this.childNodes = [_this.documentElement];
      return _this;
    }
    var _proto = Document.prototype;
    /**
     * @example const circle = document.createElement('circle', { style: { r: 10 } });
     */
    _proto.createElement = function createElement(tagName, options) {
      // @observablehq/plot will create <svg>
      if (tagName === 'svg') {
        return this.documentElement;
      }
      // d3 will use <tspan>
      var clazz = this.defaultView.customElements.get(tagName);
      if (!clazz) {
        console.warn('Unsupported tagName: ', tagName);
        clazz = tagName === 'tspan' ? Text$1 : Group$1;
      }
      var shape = new clazz(options);
      shape.ownerDocument = this;
      return shape;
    };
    _proto.createElementNS = function createElementNS(namespaceURI, tagName, options) {
      return this.createElement(tagName, options);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.cloneNode = function cloneNode(deep) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.destroy = function destroy() {
      try {
        this.documentElement.destroy();
        this.timeline.destroy();
      } catch (e) {}
    }
    /**
     * Picking 2D graphics with RBush based on BBox, fast but inaccurate.
     */;
    _proto.elementsFromBBox = function elementsFromBBox(minX, minY, maxX, maxY) {
      var rBush = this.defaultView.context.rBushRoot;
      var rBushNodes = rBush.search({
        minX: minX,
        minY: minY,
        maxX: maxX,
        maxY: maxY
      });
      var hitTestList = [];
      rBushNodes.forEach(function (_ref2) {
        var id = _ref2.id;
        var displayObject = runtime$2.displayObjectPool.getByEntity(id);
        var pointerEvents = displayObject.parsedStyle.pointerEvents;
        // account for `visibility`
        // @see https://developer.mozilla.org/en-US/docs/Web/CSS/pointer-events
        var isVisibilityAffected = ['auto', 'visiblepainted', 'visiblefill', 'visiblestroke', 'visible'].includes(pointerEvents);
        if ((!isVisibilityAffected || isVisibilityAffected && displayObject.isVisible()) && !displayObject.isCulled() && displayObject.isInteractive()) {
          hitTestList.push(displayObject);
        }
      });
      // find group with max z-index
      hitTestList.sort(function (a, b) {
        return b.sortable.renderOrder - a.sortable.renderOrder;
      });
      return hitTestList;
    };
    _proto.elementFromPointSync = function elementFromPointSync(x, y) {
      var _this$defaultView$can = this.defaultView.canvas2Viewport({
          x: x,
          y: y
        }),
        viewportX = _this$defaultView$can.x,
        viewportY = _this$defaultView$can.y;
      var _this$defaultView$get = this.defaultView.getConfig(),
        width = _this$defaultView$get.width,
        height = _this$defaultView$get.height;
      // outside canvas' viewport
      if (viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height) {
        return null;
      }
      var _this$defaultView$vie = this.defaultView.viewport2Client({
          x: viewportX,
          y: viewportY
        }),
        clientX = _this$defaultView$vie.x,
        clientY = _this$defaultView$vie.y;
      var _this$defaultView$get2 = this.defaultView.getRenderingService().hooks.pickSync.call({
          topmost: true,
          position: {
            x: x,
            y: y,
            viewportX: viewportX,
            viewportY: viewportY,
            clientX: clientX,
            clientY: clientY
          },
          picked: []
        }),
        picked = _this$defaultView$get2.picked;
      return picked && picked[0] || this.documentElement;
    }
    /**
     * Do picking with API instead of triggering interactive events.
     *
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/elementFromPoint
     */;
    _proto.elementFromPoint =
    /*#__PURE__*/
    function () {
      var _elementFromPoint = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee(x, y) {
        var _this$defaultView$can2, viewportX, viewportY, _this$defaultView$get3, width, height, _this$defaultView$vie2, clientX, clientY, _yield$this$defaultVi, picked;
        return _regeneratorRuntime$6().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _this$defaultView$can2 = this.defaultView.canvas2Viewport({
                  x: x,
                  y: y
                }), viewportX = _this$defaultView$can2.x, viewportY = _this$defaultView$can2.y;
                _this$defaultView$get3 = this.defaultView.getConfig(), width = _this$defaultView$get3.width, height = _this$defaultView$get3.height; // outside canvas' viewport
                if (!(viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height)) {
                  _context.next = 4;
                  break;
                }
                return _context.abrupt("return", null);
              case 4:
                _this$defaultView$vie2 = this.defaultView.viewport2Client({
                  x: viewportX,
                  y: viewportY
                }), clientX = _this$defaultView$vie2.x, clientY = _this$defaultView$vie2.y;
                _context.next = 7;
                return this.defaultView.getRenderingService().hooks.pick.promise({
                  topmost: true,
                  position: {
                    x: x,
                    y: y,
                    viewportX: viewportX,
                    viewportY: viewportY,
                    clientX: clientX,
                    clientY: clientY
                  },
                  picked: []
                });
              case 7:
                _yield$this$defaultVi = _context.sent;
                picked = _yield$this$defaultVi.picked;
                return _context.abrupt("return", picked && picked[0] || this.documentElement);
              case 10:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
      function elementFromPoint(_x, _x2) {
        return _elementFromPoint.apply(this, arguments);
      }
      return elementFromPoint;
    }();
    _proto.elementsFromPointSync = function elementsFromPointSync(x, y) {
      var _this$defaultView$can3 = this.defaultView.canvas2Viewport({
          x: x,
          y: y
        }),
        viewportX = _this$defaultView$can3.x,
        viewportY = _this$defaultView$can3.y;
      var _this$defaultView$get4 = this.defaultView.getConfig(),
        width = _this$defaultView$get4.width,
        height = _this$defaultView$get4.height;
      // outside canvas' viewport
      if (viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height) {
        return [];
      }
      var _this$defaultView$vie3 = this.defaultView.viewport2Client({
          x: viewportX,
          y: viewportY
        }),
        clientX = _this$defaultView$vie3.x,
        clientY = _this$defaultView$vie3.y;
      var _this$defaultView$get5 = this.defaultView.getRenderingService().hooks.pickSync.call({
          topmost: false,
          position: {
            x: x,
            y: y,
            viewportX: viewportX,
            viewportY: viewportY,
            clientX: clientX,
            clientY: clientY
          },
          picked: []
        }),
        picked = _this$defaultView$get5.picked;
      if (picked[picked.length - 1] !== this.documentElement) {
        picked.push(this.documentElement);
      }
      return picked;
    }
    /**
     * Do picking with API instead of triggering interactive events.
     *
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Document/elementsFromPoint
     */;
    _proto.elementsFromPoint =
    /*#__PURE__*/
    function () {
      var _elementsFromPoint = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee2(x, y) {
        var _this$defaultView$can4, viewportX, viewportY, _this$defaultView$get6, width, height, _this$defaultView$vie4, clientX, clientY, _yield$this$defaultVi2, picked;
        return _regeneratorRuntime$6().wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _this$defaultView$can4 = this.defaultView.canvas2Viewport({
                  x: x,
                  y: y
                }), viewportX = _this$defaultView$can4.x, viewportY = _this$defaultView$can4.y;
                _this$defaultView$get6 = this.defaultView.getConfig(), width = _this$defaultView$get6.width, height = _this$defaultView$get6.height; // outside canvas' viewport
                if (!(viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height)) {
                  _context2.next = 4;
                  break;
                }
                return _context2.abrupt("return", []);
              case 4:
                _this$defaultView$vie4 = this.defaultView.viewport2Client({
                  x: viewportX,
                  y: viewportY
                }), clientX = _this$defaultView$vie4.x, clientY = _this$defaultView$vie4.y;
                _context2.next = 7;
                return this.defaultView.getRenderingService().hooks.pick.promise({
                  topmost: false,
                  position: {
                    x: x,
                    y: y,
                    viewportX: viewportX,
                    viewportY: viewportY,
                    clientX: clientX,
                    clientY: clientY
                  },
                  picked: []
                });
              case 7:
                _yield$this$defaultVi2 = _context2.sent;
                picked = _yield$this$defaultVi2.picked;
                if (picked[picked.length - 1] !== this.documentElement) {
                  picked.push(this.documentElement);
                }
                return _context2.abrupt("return", picked);
              case 11:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
      function elementsFromPoint(_x3, _x4) {
        return _elementsFromPoint.apply(this, arguments);
      }
      return elementsFromPoint;
    }()
    /**
     * eg. Uncaught DOMException: Failed to execute 'appendChild' on 'Node': Only one element on document allowed.
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.appendChild = function appendChild(newChild, index) {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT$1);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.insertBefore = function insertBefore(newChild, refChild) {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT$1);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.removeChild = function removeChild(oldChild, destroy) {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT$1);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.replaceChild = function replaceChild(newChild, oldChild, destroy) {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT$1);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.append = function append() {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT$1);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.prepend = function prepend() {
      throw new Error(ERROR_MSG_USE_DOCUMENT_ELEMENT$1);
    }
    /**
     * Execute query on documentElement.
     */;
    _proto.getElementById = function getElementById(id) {
      return this.documentElement.getElementById(id);
    };
    _proto.getElementsByName = function getElementsByName(name) {
      return this.documentElement.getElementsByName(name);
    };
    _proto.getElementsByTagName = function getElementsByTagName(tagName) {
      return this.documentElement.getElementsByTagName(tagName);
    };
    _proto.getElementsByClassName = function getElementsByClassName(className) {
      return this.documentElement.getElementsByClassName(className);
    };
    _proto.querySelector = function querySelector(selectors) {
      return this.documentElement.querySelector(selectors);
    };
    _proto.querySelectorAll = function querySelectorAll(selectors) {
      return this.documentElement.querySelectorAll(selectors);
    };
    _proto.find = function find(filter) {
      return this.documentElement.find(filter);
    };
    _proto.findAll = function findAll(filter) {
      return this.documentElement.findAll(filter);
    };
    _createClass$5(Document, [{
      key: "children",
      get: function get() {
        return this.childNodes;
      }
    }, {
      key: "childElementCount",
      get: function get() {
        return this.childNodes.length;
      }
    }, {
      key: "firstElementChild",
      get: function get() {
        return this.firstChild;
      }
    }, {
      key: "lastElementChild",
      get: function get() {
        return this.lastChild;
      }
      /**
       * only document has defaultView, points to canvas,
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Document/defaultView
       */
    }]);
    return Document;
  }(Node$2);

  /**
   * @see https://doc.babylonjs.com/how_to/optimizing_your_scene#changing-mesh-culling-strategy
   */
  var Strategy$1;
  (function (Strategy) {
    Strategy[Strategy["Standard"] = 0] = "Standard";
  })(Strategy$1 || (Strategy$1 = {}));

  /**
   * built-in events for element
   * @see https://developer.mozilla.org/en-US/docs/Web/API/MutationEvent
   *
   * TODO: use MutationObserver instead
   * @see https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver
   */
  var ElementEvent$1;
  (function (ElementEvent) {
    ElementEvent["DESTROY"] = "destroy";
    /**
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#event-type-DOMAttrModified
     */
    ElementEvent["ATTR_MODIFIED"] = "DOMAttrModified";
    /**
     * it has been inserted
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#event-type-DOMNodeInserted
     */
    ElementEvent["INSERTED"] = "DOMNodeInserted";
    /**
     * it is being removed
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#event-type-DOMNodeRemoved
     */
    ElementEvent["REMOVED"] = "removed";
    /**
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#domnodeinsertedintodocument
     */
    ElementEvent["MOUNTED"] = "DOMNodeInsertedIntoDocument";
    /**
     * @see https://www.w3.org/TR/DOM-Level-3-Events/#domnoderemovedfromdocument
     */
    ElementEvent["UNMOUNTED"] = "DOMNodeRemovedFromDocument";
    ElementEvent["BOUNDS_CHANGED"] = "bounds-changed";
    // GEOMETRY_BOUNDS_CHANGED = 'geometry-bounds-changed',
    /**
     * trigger when z-index changed
     */
    ElementEvent["RENDER_ORDER_CHANGED"] = "render-order-changed";
    ElementEvent["CULLED"] = "culled";
  })(ElementEvent$1 || (ElementEvent$1 = {}));

  var MutationEvent$1 = /*#__PURE__*/function (_FederatedEvent) {
    _inheritsLoose$9(MutationEvent, _FederatedEvent);
    function MutationEvent(typeArg, relatedNode, prevValue, newValue, attrName, attrChange, prevParsedValue, newParsedValue) {
      var _this;
      _this = _FederatedEvent.call(this, null) || this;
      _this.relatedNode = void 0;
      _this.prevValue = void 0;
      _this.newValue = void 0;
      _this.attrName = void 0;
      _this.attrChange = void 0;
      _this.prevParsedValue = void 0;
      _this.newParsedValue = void 0;
      _this.relatedNode = relatedNode;
      _this.prevValue = prevValue;
      _this.newValue = newValue;
      _this.attrName = attrName;
      _this.attrChange = attrChange;
      _this.prevParsedValue = prevParsedValue;
      _this.newParsedValue = newParsedValue;
      _this.type = typeArg;
      return _this;
    }
    return MutationEvent;
  }(FederatedEvent$1);
  MutationEvent$1.ADDITION = 2;
  MutationEvent$1.MODIFICATION = 1;
  MutationEvent$1.REMOVAL = 3;

  var entityCounter$1 = 0;
  var insertedEvent$1 = new MutationEvent$1(ElementEvent$1.INSERTED, null, '', '', '', 0, '', '');
  var removedEvent$1 = new MutationEvent$1(ElementEvent$1.REMOVED, null, '', '', '', 0, '', '');
  var destroyEvent$1 = new CustomEvent$1(ElementEvent$1.DESTROY);
  /**
   * Has following capabilities:
   * * Node insert/remove, eg. appendChild, removeChild, remove...
   * * Query eg. querySelector getElementById...
   * * Animation
   */
  var Element$1 = /*#__PURE__*/function (_Node) {
    _inheritsLoose$9(Element, _Node);
    function Element() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _Node.call.apply(_Node, [this].concat(args)) || this;
      _this.entity = entityCounter$1++;
      _this.renderable = {
        bounds: undefined,
        boundsDirty: true,
        renderBounds: undefined,
        renderBoundsDirty: true,
        dirtyRenderBounds: undefined,
        dirty: false,
        proxyNodeName: undefined
      };
      _this.cullable = {
        strategy: Strategy$1.Standard,
        visibilityPlaneMask: -1,
        visible: true,
        enable: true
      };
      _this.transformable = {
        dirtyFlag: false,
        localDirtyFlag: false,
        frozen: false,
        localPosition: fromValues$8(0, 0, 0),
        localRotation: fromValues$a(0, 0, 0, 1),
        localScale: fromValues$8(1, 1, 1),
        localTransform: create$7(),
        localSkew: fromValues$b(0, 0),
        position: fromValues$8(0, 0, 0),
        rotation: fromValues$a(0, 0, 0, 1),
        scaling: fromValues$8(1, 1, 1),
        worldTransform: create$7(),
        origin: fromValues$8(0, 0, 0)
      };
      _this.sortable = {
        dirty: false,
        sorted: undefined,
        lastSortedIndex: undefined,
        renderOrder: 0
      };
      _this.geometry = {
        contentBounds: undefined,
        renderBounds: undefined
      };
      _this.rBushNode = {
        aabb: undefined
      };
      _this.id = void 0;
      _this.name = void 0;
      _this.namespaceURI = 'g';
      _this.scrollLeft = 0;
      _this.scrollTop = 0;
      _this.clientTop = 0;
      _this.clientLeft = 0;
      _this.destroyed = false;
      _this.style = {};
      _this.computedStyle = {
        anchor: unsetKeywordValue$1,
        opacity: unsetKeywordValue$1,
        fillOpacity: unsetKeywordValue$1,
        strokeOpacity: unsetKeywordValue$1,
        fill: unsetKeywordValue$1,
        stroke: unsetKeywordValue$1,
        transform: unsetKeywordValue$1,
        transformOrigin: unsetKeywordValue$1,
        visibility: unsetKeywordValue$1,
        pointerEvents: unsetKeywordValue$1,
        lineWidth: unsetKeywordValue$1,
        lineCap: unsetKeywordValue$1,
        lineJoin: unsetKeywordValue$1,
        increasedLineWidthForHitTesting: unsetKeywordValue$1,
        fontSize: unsetKeywordValue$1,
        fontFamily: unsetKeywordValue$1,
        fontStyle: unsetKeywordValue$1,
        fontWeight: unsetKeywordValue$1,
        fontVariant: unsetKeywordValue$1,
        textAlign: unsetKeywordValue$1,
        textBaseline: unsetKeywordValue$1,
        textTransform: unsetKeywordValue$1,
        zIndex: unsetKeywordValue$1,
        filter: unsetKeywordValue$1,
        shadowType: unsetKeywordValue$1
      };
      _this.parsedStyle = {
        // opacity: '',
        // fillOpacity: '',
        // strokeOpacity: '',
        // transformOrigin: '',
        // visibility: '',
        // pointerEvents: '',
        // lineWidth: '',
        // lineCap: '',
        // lineJoin: '',
        // increasedLineWidthForHitTesting: '',
        // fontSize: '',
        // fontFamily: '',
        // fontStyle: '',
        // fontWeight: '',
        // fontVariant: '',
        // textAlign: '',
        // textBaseline: '',
        // textTransform: '',
      };
      _this.attributes = {};
      return _this;
    }
    Element.isElement = function isElement(target) {
      return !!target.getAttribute;
    }
    /**
     * Unique id.
     */;
    var _proto = Element.prototype;
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    _proto.cloneNode = function cloneNode(deep) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.appendChild = function appendChild(child, index) {
      var _this$ownerDocument;
      if (child.destroyed) {
        throw new Error(ERROR_MSG_APPEND_DESTROYED_ELEMENT$1);
      }
      runtime$2.sceneGraphService.attach(child, this, index);
      if ((_this$ownerDocument = this.ownerDocument) === null || _this$ownerDocument === void 0 ? void 0 : _this$ownerDocument.defaultView) {
        this.ownerDocument.defaultView.mountChildren(child);
      }
      insertedEvent$1.relatedNode = this;
      child.dispatchEvent(insertedEvent$1);
      return child;
    };
    _proto.insertBefore = function insertBefore(newChild, refChild) {
      if (!refChild) {
        this.appendChild(newChild);
      } else {
        var index = this.childNodes.indexOf(refChild);
        this.appendChild(newChild, index - 1);
      }
      return newChild;
    };
    _proto.replaceChild = function replaceChild(newChild, oldChild) {
      var index = this.childNodes.indexOf(oldChild);
      this.removeChild(oldChild);
      this.appendChild(newChild, index);
      return oldChild;
    };
    _proto.removeChild = function removeChild(child) {
      var _child$ownerDocument;
      // should emit on itself before detach
      removedEvent$1.relatedNode = this;
      child.dispatchEvent(removedEvent$1);
      if ((_child$ownerDocument = child.ownerDocument) === null || _child$ownerDocument === void 0 ? void 0 : _child$ownerDocument.defaultView) {
        child.ownerDocument.defaultView.unmountChildren(child);
      }
      // remove from scene graph
      runtime$2.sceneGraphService.detach(child);
      return child;
    }
    /**
     * Remove all children which can be appended to its original parent later again.
     */;
    _proto.removeChildren = function removeChildren() {
      var _this2 = this;
      this.childNodes.slice().forEach(function (child) {
        _this2.removeChild(child);
      });
    }
    /**
     * Recursively destroy all children which can not be appended to its original parent later again.
     */;
    _proto.destroyChildren = function destroyChildren() {
      this.childNodes.slice().forEach(function (child) {
        child.destroyChildren();
        child.destroy();
      });
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/matches
     */;
    _proto.matches = function matches(selector) {
      return runtime$2.sceneGraphService.matches(selector, this);
    };
    _proto.getElementById = function getElementById(id) {
      return runtime$2.sceneGraphService.querySelector("#" + id, this);
    };
    _proto.getElementsByName = function getElementsByName(name) {
      return runtime$2.sceneGraphService.querySelectorAll("[name=\"" + name + "\"]", this);
    };
    _proto.getElementsByClassName = function getElementsByClassName(className) {
      return runtime$2.sceneGraphService.querySelectorAll("." + className, this);
    };
    _proto.getElementsByTagName = function getElementsByTagName(tagName) {
      return runtime$2.sceneGraphService.querySelectorAll(tagName, this);
    };
    _proto.querySelector = function querySelector(selectors) {
      return runtime$2.sceneGraphService.querySelector(selectors, this);
    };
    _proto.querySelectorAll = function querySelectorAll(selectors) {
      return runtime$2.sceneGraphService.querySelectorAll(selectors, this);
    }
    /**
     * should traverses the element and its parents (heading toward the document root)
     * until it finds a node that matches the specified CSS selector.
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/closest
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/closest#polyfill
     */;
    _proto.closest = function closest(selectors) {
      var el = this;
      do {
        if (runtime$2.sceneGraphService.matches(selectors, el)) return el;
        el = el.parentElement;
      } while (el !== null);
      return null;
    }
    /**
     * search in scene group, but should not include itself
     */;
    _proto.find = function find(filter) {
      var _this3 = this;
      var target = null;
      this.forEach(function (object) {
        if (object !== _this3 && filter(object)) {
          target = object;
          return true;
        }
        return false;
      });
      return target;
    };
    _proto.findAll = function findAll(filter) {
      var _this4 = this;
      var objects = [];
      this.forEach(function (object) {
        if (object !== _this4 && filter(object)) {
          objects.push(object);
        }
      });
      return objects;
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/after
     */;
    _proto.after = function after() {
      var _this5 = this;
      if (this.parentNode) {
        var index = this.parentNode.childNodes.indexOf(this);
        for (var _len2 = arguments.length, nodes = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          nodes[_key2] = arguments[_key2];
        }
        nodes.forEach(function (node, i) {
          var _this5$parentNode;
          return (_this5$parentNode = _this5.parentNode) === null || _this5$parentNode === void 0 ? void 0 : _this5$parentNode.appendChild(node, index + i + 1);
        });
      }
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/before
     */;
    _proto.before = function before() {
      if (this.parentNode) {
        var index = this.parentNode.childNodes.indexOf(this);
        for (var _len3 = arguments.length, nodes = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
          nodes[_key3] = arguments[_key3];
        }
        var first = nodes[0],
          rest = nodes.slice(1);
        this.parentNode.appendChild(first, index);
        first.after.apply(first, rest);
      }
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/replaceWith
     */;
    _proto.replaceWith = function replaceWith() {
      this.after.apply(this, arguments);
      this.remove();
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/append
     */;
    _proto.append = function append() {
      var _this6 = this;
      for (var _len4 = arguments.length, nodes = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        nodes[_key4] = arguments[_key4];
      }
      nodes.forEach(function (node) {
        return _this6.appendChild(node);
      });
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/prepend
     */;
    _proto.prepend = function prepend() {
      var _this7 = this;
      for (var _len5 = arguments.length, nodes = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        nodes[_key5] = arguments[_key5];
      }
      nodes.forEach(function (node, i) {
        return _this7.appendChild(node, i);
      });
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/replaceChildren
     */;
    _proto.replaceChildren = function replaceChildren() {
      while (this.childNodes.length && this.firstChild) {
        this.removeChild(this.firstChild);
      }
      this.append.apply(this, arguments);
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/remove
     */;
    _proto.remove = function remove() {
      if (this.parentNode) {
        return this.parentNode.removeChild(this);
      }
      return this;
    }
    /**
     * is destroyed or not
     */;
    _proto.destroy = function destroy() {
      // destroy itself before remove
      this.dispatchEvent(destroyEvent$1);
      // remove from scenegraph first
      this.remove();
      // remove event listeners
      this.emitter.removeAllListeners();
      this.destroyed = true;
    };
    _proto.getGeometryBounds = function getGeometryBounds() {
      return runtime$2.sceneGraphService.getGeometryBounds(this);
    };
    _proto.getRenderBounds = function getRenderBounds() {
      return runtime$2.sceneGraphService.getBounds(this, true);
    }
    /**
     * get bounds in world space, account for children
     */;
    _proto.getBounds = function getBounds() {
      return runtime$2.sceneGraphService.getBounds(this);
    }
    /**
     * get bounds in local space, account for children
     */;
    _proto.getLocalBounds = function getLocalBounds() {
      return runtime$2.sceneGraphService.getLocalBounds(this);
    }
    /**
     * account for context's bounds in client space,
     * but not accounting for children
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/getBoundingClientRect
     */;
    _proto.getBoundingClientRect = function getBoundingClientRect() {
      return runtime$2.sceneGraphService.getBoundingClientRect(this);
    }
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/getClientRects
     */;
    _proto.getClientRects = function getClientRects() {
      return [this.getBoundingClientRect()];
    }
    /**
     * compatible with `style`
     * @see https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement/style
     */;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/computedStyleMap
     * eg. circle.computedStyleMap().get('fill');
     */
    _proto.computedStyleMap = function computedStyleMap() {
      return new Map(Object.entries(this.computedStyle));
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/attributes
     */;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/getAttributeNames
     */
    _proto.getAttributeNames = function getAttributeNames() {
      return Object.keys(this.attributes);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/getAttribute
     */;
    _proto.getAttribute = function getAttribute(name) {
      var value = this.attributes[name];
      if (value === undefined) {
        var attributeName = formatAttributeName$1(name);
        value = this.attributes[attributeName];
        // if the given attribute does not exist, the value returned will either be null or ""
        return isNil$1(value) ? null : value;
      } else {
        return value;
      }
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/hasAttribute
     */;
    _proto.hasAttribute = function hasAttribute(qualifiedName) {
      return this.getAttributeNames().includes(qualifiedName);
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/hasAttributes
     */;
    _proto.hasAttributes = function hasAttributes() {
      return !!this.getAttributeNames().length;
    }
    /**
     * should use removeAttribute() instead of setting the attribute value to null either directly or using setAttribute(). Many attributes will not behave as expected if you set them to null.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/removeAttribute
     */;
    _proto.removeAttribute = function removeAttribute(attributeName) {
      this.setAttribute(attributeName, null);
      delete this.attributes[attributeName];
    }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/setAttribute
     */;
    _proto.setAttribute = function setAttribute(attributeName, value,
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    force) {
      this.attributes[attributeName] = value;
    };
    _proto.getAttributeNS = function getAttributeNS(namespace, localName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.getAttributeNode = function getAttributeNode(qualifiedName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.getAttributeNodeNS = function getAttributeNodeNS(namespace, localName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.hasAttributeNS = function hasAttributeNS(namespace, localName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.removeAttributeNS = function removeAttributeNS(namespace, localName) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.removeAttributeNode = function removeAttributeNode(attr) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.setAttributeNS = function setAttributeNS(namespace, qualifiedName, value) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.setAttributeNode = function setAttributeNode(attr) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.setAttributeNodeNS = function setAttributeNodeNS(attr) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.toggleAttribute = function toggleAttribute(qualifiedName, force) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _createClass$5(Element, [{
      key: "className",
      get:
      /**
       * used in `getElementsByClassName`
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Document/getElementsByClassName
       */
      function get() {
        // @ts-ignore
        return this.getAttribute('class') || '';
      },
      set: function set(className) {
        this.setAttribute('class', className);
      }
      /**
       * used in `getElementsByName`
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Document/getElementsByName
       */
    }, {
      key: "classList",
      get:
      /**
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/classList
       */
      function get() {
        return this.className.split(' ').filter(function (c) {
          return c !== '';
        });
      }
    }, {
      key: "tagName",
      get: function get() {
        return this.nodeName;
      }
    }, {
      key: "children",
      get: function get() {
        return this.childNodes;
      }
    }, {
      key: "childElementCount",
      get: function get() {
        return this.childNodes.length;
      }
    }, {
      key: "firstElementChild",
      get: function get() {
        return this.firstChild;
      }
    }, {
      key: "lastElementChild",
      get: function get() {
        return this.lastChild;
      }
    }, {
      key: "parentElement",
      get: function get() {
        return this.parentNode;
      }
    }, {
      key: "nextSibling",
      get: function get() {
        if (this.parentNode) {
          var index = this.parentNode.childNodes.indexOf(this);
          return this.parentNode.childNodes[index + 1] || null;
        }
        return null;
      }
    }, {
      key: "previousSibling",
      get: function get() {
        if (this.parentNode) {
          var index = this.parentNode.childNodes.indexOf(this);
          return this.parentNode.childNodes[index - 1] || null;
        }
        return null;
      }
    }]);
    return Element;
  }(Node$2);

  var FederatedMouseEvent$1 = /*#__PURE__*/function (_FederatedEvent) {
    _inheritsLoose$9(FederatedMouseEvent, _FederatedEvent);
    function FederatedMouseEvent() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _FederatedEvent.call.apply(_FederatedEvent, [this].concat(args)) || this;
      _this.altKey = void 0;
      _this.button = void 0;
      _this.buttons = void 0;
      _this.ctrlKey = void 0;
      _this.metaKey = void 0;
      _this.relatedTarget = void 0;
      _this.shiftKey = void 0;
      _this.client = new Point$1();
      _this.movement = new Point$1();
      _this.offset = new Point$1();
      _this.global = new Point$1();
      _this.screen = new Point$1();
      return _this;
    }
    var _proto = FederatedMouseEvent.prototype;
    _proto.getModifierState = function getModifierState(key) {
      return 'getModifierState' in this.nativeEvent && this.nativeEvent.getModifierState(key);
    };
    _proto.initMouseEvent = function initMouseEvent() {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _createClass$5(FederatedMouseEvent, [{
      key: "clientX",
      get: function get() {
        return this.client.x;
      }
    }, {
      key: "clientY",
      get: function get() {
        return this.client.y;
      }
      /**
       * The movement in this pointer relative to the last `mousemove` event.
       */
    }, {
      key: "movementX",
      get: function get() {
        return this.movement.x;
      }
    }, {
      key: "movementY",
      get: function get() {
        return this.movement.y;
      }
      /**
       * The offset of the pointer coordinates w.r.t. target DisplayObject in world space. This is
       * not supported at the moment.
       */
    }, {
      key: "offsetX",
      get: function get() {
        return this.offset.x;
      }
    }, {
      key: "offsetY",
      get: function get() {
        return this.offset.y;
      }
      /**
       * The pointer coordinates in world space.
       */
    }, {
      key: "globalX",
      get: function get() {
        return this.global.x;
      }
    }, {
      key: "globalY",
      get: function get() {
        return this.global.y;
      }
      /**
       * The pointer coordinates in sceen space.
       */
    }, {
      key: "screenX",
      get: function get() {
        return this.screen.x;
      }
    }, {
      key: "screenY",
      get: function get() {
        return this.screen.y;
      }
    }]);
    return FederatedMouseEvent;
  }(FederatedEvent$1);

  var FederatedPointerEvent$1 = /*#__PURE__*/function (_FederatedMouseEvent) {
    _inheritsLoose$9(FederatedPointerEvent, _FederatedMouseEvent);
    function FederatedPointerEvent() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _FederatedMouseEvent.call.apply(_FederatedMouseEvent, [this].concat(args)) || this;
      _this.pointerId = void 0;
      _this.width = 0;
      _this.height = 0;
      _this.isPrimary = false;
      _this.pointerType = void 0;
      _this.pressure = void 0;
      _this.tangentialPressure = void 0;
      _this.tiltX = void 0;
      _this.tiltY = void 0;
      _this.twist = void 0;
      return _this;
    }
    var _proto = FederatedPointerEvent.prototype;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/getCoalescedEvents
     */
    _proto.getCoalescedEvents = function getCoalescedEvents() {
      if (this.type === 'pointermove' || this.type === 'mousemove' || this.type === 'touchmove') {
        return [this];
      }
      return [];
    }
    /**
     * @see https://chromestatus.com/feature/5765569655603200
     */;
    _proto.getPredictedEvents = function getPredictedEvents() {
      throw new Error('getPredictedEvents is not supported!');
    }
    /**
     * @see https://github.com/antvis/G/issues/1115
     * We currently reuses event objects in the event system,
     * avoiding the creation of a large number of event objects.
     * Reused objects are only used to carry different data,
     * such as coordinate information, native event objects,
     * and therefore the lifecycle is limited to the event handler,
     * which can lead to unintended consequences if an attempt is made to cache the entire event object.
     *
     * Therefore, while keeping the above performance considerations in mind, it is possible to provide a clone method that creates a new object when the user really wants to cache it, e.g.
     */;
    _proto.clone = function clone() {
      return this.manager.clonePointerEvent(this);
    };
    return FederatedPointerEvent;
  }(FederatedMouseEvent$1);

  var FederatedWheelEvent$1 = /*#__PURE__*/function (_FederatedMouseEvent) {
    _inheritsLoose$9(FederatedWheelEvent, _FederatedMouseEvent);
    function FederatedWheelEvent() {
      var _this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      _this = _FederatedMouseEvent.call.apply(_FederatedMouseEvent, [this].concat(args)) || this;
      _this.deltaMode = void 0;
      _this.deltaX = void 0;
      _this.deltaY = void 0;
      _this.deltaZ = void 0;
      _this.DOM_DELTA_LINE = 0;
      _this.DOM_DELTA_PAGE = 1;
      _this.DOM_DELTA_PIXEL = 2;
      return _this;
    }
    var _proto = FederatedWheelEvent.prototype;
    _proto.clone = function clone() {
      return this.manager.cloneWheelEvent(this);
    };
    return FederatedWheelEvent;
  }(FederatedMouseEvent$1);

  var mutationEvent$1 = new MutationEvent$1(ElementEvent$1.ATTR_MODIFIED, null, null, null, null, MutationEvent$1.MODIFICATION, null, null);
  var DEFAULT_STYLE_PROPS$1 = {
    anchor: '',
    opacity: '',
    fillOpacity: '',
    strokeOpacity: '',
    fill: '',
    stroke: '',
    transform: '',
    transformOrigin: '',
    visibility: '',
    pointerEvents: '',
    lineWidth: '',
    lineCap: '',
    lineJoin: '',
    increasedLineWidthForHitTesting: '',
    fontSize: '',
    fontFamily: '',
    fontStyle: '',
    fontWeight: '',
    fontVariant: '',
    textAlign: '',
    textBaseline: '',
    textTransform: '',
    zIndex: '',
    filter: '',
    shadowType: ''
  };
  var DEFAULT_PARSED_STYLE_PROPS$1 = {
    anchor: [0, 0],
    fill: noneColor$1,
    stroke: noneColor$1,
    transform: [],
    zIndex: 0,
    filter: [],
    shadowType: 'outer'
  };
  var INHERITABLE_STYLE_PROPS$1 = ['opacity', 'fillOpacity', 'strokeOpacity', 'transformOrigin', 'visibility', 'pointerEvents', 'lineWidth', 'lineCap', 'lineJoin', 'increasedLineWidthForHitTesting', 'fontSize', 'fontFamily', 'fontStyle', 'fontWeight', 'fontVariant', 'textAlign', 'textBaseline', 'textTransform'];
  /**
   * prototype chains: DisplayObject -> Element -> Node -> EventTarget
   *
   * mixins: Animatable, Transformable, Visible
   * @see https://github.com/tannerntannern/ts-mixer/blob/master/README.md#mixing-generic-classes
   *
   * Provide abilities in scene graph, such as:
   * * transform `translate/rotate/scale`
   * * add/remove child
   * * visibility and z-index
   *
   * Those abilities are implemented with those components: `Transform/Sortable/Visible`.
   *
   * Emit following events:
   * * init
   * * destroy
   * * attributeChanged
   */
  var DisplayObject$1 = /*#__PURE__*/function (_Element) {
    _inheritsLoose$9(DisplayObject, _Element);
    /**
     * contains style props in constructor's params, eg. fill, stroke...
     */

    /**
     * push to active animations after calling `animate()`
     */

    function DisplayObject(config) {
      var _this$config$capture;
      var _this;
      _this = _Element.call(this) || this;
      // assign name, id to config
      // eg. group.get('name')
      _this.config = void 0;
      _this.isCustomElement = false;
      _this.isMutationObserved = false;
      _this.activeAnimations = [];
      _this.getClip = function () {
        return this.style.clipPath || null;
      };
      _this.config = config;
      // compatible with G 3.0
      _this.config.interactive = (_this$config$capture = _this.config.capture) !== null && _this$config$capture !== void 0 ? _this$config$capture : _this.config.interactive;
      // init scene graph node
      _this.id = _this.config.id || '';
      _this.name = _this.config.name || '';
      if (_this.config.className || _this.config.class) {
        _this.className = _this.config.className || _this.config.class;
      }
      _this.nodeName = _this.config.type || Shape$2.GROUP;
      // compatible with G 3.0
      _this.config.style = _extends$3({}, _this.config.style, _this.config.attrs);
      if (_this.config.visible != null) {
        _this.config.style.visibility = _this.config.visible === false ? 'hidden' : 'visible';
      }
      if (_this.config.interactive != null) {
        _this.config.style.pointerEvents = _this.config.interactive === false ? 'none' : 'auto';
      }
      // merge parsed value
      Object.assign(_this.parsedStyle, DEFAULT_PARSED_STYLE_PROPS$1, _this.config.initialParsedStyle);
      Object.assign(_this.attributes, DEFAULT_STYLE_PROPS$1);
      // start to process attributes
      _this.initAttributes(_this.config.style);
      _this.style = new Proxy(
      // @ts-ignore
      {
        // ...this.attributes,
        setProperty: function setProperty(propertyName, value) {
          _this.setAttribute(propertyName, value);
        },
        getPropertyValue: function getPropertyValue(propertyName) {
          return _this.getAttribute(propertyName);
        },
        removeProperty: function removeProperty(propertyName) {
          _this.removeAttribute(propertyName);
        },
        item: function item() {
          return '';
        }
      }, {
        get: function get(target, name) {
          if (target[name] !== undefined) {
            // if (name in target) {
            return target[name];
          }
          return _this.getAttribute(name);
        },
        set: function set(_, prop, value) {
          _this.setAttribute(prop, value);
          return true;
        }
      });
      // insert this group into pool
      runtime$2.displayObjectPool.add(_this.entity, _assertThisInitialized$5(_this));
      return _this;
    }
    var _proto = DisplayObject.prototype;
    _proto.destroy = function destroy() {
      _Element.prototype.destroy.call(this);
      // remove from into pool
      runtime$2.displayObjectPool.remove(this.entity);
      // stop all active animations
      this.getAnimations().forEach(function (animation) {
        animation.cancel();
      });
    };
    _proto.cloneNode = function cloneNode(deep, customCloneFunc) {
      var clonedStyle = _extends$3({}, this.attributes);
      for (var attributeName in clonedStyle) {
        var attribute = clonedStyle[attributeName];
        // @see https://github.com/antvis/G/issues/1095
        if (attribute instanceof DisplayObject &&
        // share the same clipPath if possible
        attributeName !== 'clipPath' && attributeName !== 'offsetPath' && attributeName !== 'textPath') {
          clonedStyle[attributeName] = attribute.cloneNode(deep);
        }
        // TODO: clone other type
        if (customCloneFunc) {
          clonedStyle[attributeName] = customCloneFunc(attributeName, attribute);
        }
      }
      var cloned = new this.constructor({
        // copy id & name
        // @see https://developer.mozilla.org/en-US/docs/Web/API/Node/cloneNode#notes
        id: this.id,
        name: this.name,
        className: this.name,
        interactive: this.interactive,
        style: clonedStyle
      });
      // apply transform
      cloned.setLocalTransform(this.getLocalTransform());
      if (deep) {
        this.children.forEach(function (child) {
          // skip marker
          if (!child.style.isMarker) {
            var clonedChild = child.cloneNode(deep);
            cloned.appendChild(clonedChild);
          }
        });
      }
      return cloned;
    };
    _proto.initAttributes = function initAttributes(attributes) {
      if (attributes === void 0) {
        attributes = {};
      }
      var renderable = this.renderable;
      // account for FCP, process properties as less as possible
      runtime$2.styleValueRegistry.processProperties(this, attributes, {
        forceUpdateGeometry: true,
        usedAttributes: INHERITABLE_STYLE_PROPS$1
      });
      // redraw at next frame
      renderable.dirty = true;
    };
    _proto.setAttribute = function setAttribute(name, value, force) {
      if (force === void 0) {
        force = false;
      }
      var attributeName = formatAttributeName$1(name);
      // ignore undefined value
      if (isUndefined$2(value)) {
        return;
      }
      if (force || value !== this.attributes[attributeName]) {
        this.internalSetAttribute(attributeName, value);
        _Element.prototype.setAttribute.call(this, attributeName, value);
      }
    }
    /**
     * called when attributes get changed or initialized
     */;
    _proto.internalSetAttribute = function internalSetAttribute(name, value, parseOptions) {
      var _runtime$styleValueRe;
      if (parseOptions === void 0) {
        parseOptions = {};
      }
      var renderable = this.renderable;
      var oldValue = this.attributes[name];
      var oldParsedValue = this.parsedStyle[name];
      runtime$2.styleValueRegistry.processProperties(this, (_runtime$styleValueRe = {}, _runtime$styleValueRe[name] = value, _runtime$styleValueRe), parseOptions);
      // redraw at next frame
      renderable.dirty = true;
      var newParsedValue = this.parsedStyle[name];
      if (this.isConnected) {
        mutationEvent$1.relatedNode = this;
        mutationEvent$1.prevValue = oldValue;
        mutationEvent$1.newValue = value;
        mutationEvent$1.attrName = name;
        mutationEvent$1.prevParsedValue = oldParsedValue;
        mutationEvent$1.newParsedValue = newParsedValue;
        if (this.isMutationObserved) {
          this.dispatchEvent(mutationEvent$1);
        } else {
          mutationEvent$1.target = this;
          this.ownerDocument.defaultView.dispatchEvent(mutationEvent$1, true);
        }
      }
      if ((this.isCustomElement && this.isConnected || !this.isCustomElement) && this.attributeChangedCallback) {
        this.attributeChangedCallback(name, oldValue, value, oldParsedValue, newParsedValue);
      }
    }
    // #region transformable
    /**
     * returns different values than getBoundingClientRect(), as the latter returns value relative to the viewport
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGGraphicsElement/getBBox
     *
     * FIXME: It is worth noting that getBBox responds to original untransformed values of a drawn object.
     * @see https://www.w3.org/Graphics/SVG/IG/resources/svgprimer.html#getBBox
     */;
    _proto.getBBox = function getBBox() {
      var aabb = this.getBounds();
      var _aabb$getMin = aabb.getMin(),
        left = _aabb$getMin[0],
        top = _aabb$getMin[1];
      var _aabb$getMax = aabb.getMax(),
        right = _aabb$getMax[0],
        bottom = _aabb$getMax[1];
      return new Rectangle$1(left, top, right - left, bottom - top);
    };
    _proto.setOrigin = function setOrigin(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime$2.sceneGraphService.setOrigin(this, createVec3$1(position, y, z));
      return this;
    };
    _proto.getOrigin = function getOrigin() {
      return runtime$2.sceneGraphService.getOrigin(this);
    }
    /**
     * set position in world space
     */;
    _proto.setPosition = function setPosition(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime$2.sceneGraphService.setPosition(this, createVec3$1(position, y, z));
      return this;
    }
    /**
     * set position in local space
     */;
    _proto.setLocalPosition = function setLocalPosition(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime$2.sceneGraphService.setLocalPosition(this, createVec3$1(position, y, z));
      return this;
    }
    /**
     * translate in world space
     */;
    _proto.translate = function translate(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime$2.sceneGraphService.translate(this, createVec3$1(position, y, z));
      return this;
    }
    /**
     * translate in local space
     */;
    _proto.translateLocal = function translateLocal(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      runtime$2.sceneGraphService.translateLocal(this, createVec3$1(position, y, z));
      return this;
    };
    _proto.getPosition = function getPosition() {
      return runtime$2.sceneGraphService.getPosition(this);
    };
    _proto.getLocalPosition = function getLocalPosition() {
      return runtime$2.sceneGraphService.getLocalPosition(this);
    }
    /**
     * compatible with G 3.0
     *
     * scaling in local space
     * scale(10) = scale(10, 10, 10)
     *
     * we can't set scale in world space
     */;
    _proto.scale = function scale(scaling, y, z) {
      return this.scaleLocal(scaling, y, z);
    };
    _proto.scaleLocal = function scaleLocal(scaling, y, z) {
      if (typeof scaling === 'number') {
        y = y || scaling;
        z = z || scaling;
        scaling = createVec3$1(scaling, y, z);
      }
      runtime$2.sceneGraphService.scaleLocal(this, scaling);
      return this;
    }
    /**
     * set scaling in local space
     */;
    _proto.setLocalScale = function setLocalScale(scaling, y, z) {
      if (typeof scaling === 'number') {
        y = y || scaling;
        z = z || scaling;
        scaling = createVec3$1(scaling, y, z);
      }
      runtime$2.sceneGraphService.setLocalScale(this, scaling);
      return this;
    }
    /**
     * get scaling in local space
     */;
    _proto.getLocalScale = function getLocalScale() {
      return runtime$2.sceneGraphService.getLocalScale(this);
    }
    /**
     * get scaling in world space
     */;
    _proto.getScale = function getScale() {
      return runtime$2.sceneGraphService.getScale(this);
    }
    /**
     * only return degrees of Z axis in world space
     */;
    _proto.getEulerAngles = function getEulerAngles() {
      var _getEuler = getEuler$1(create$8(), runtime$2.sceneGraphService.getWorldTransform(this)),
        ez = _getEuler[2];
      return rad2deg$1(ez);
    }
    /**
     * only return degrees of Z axis in local space
     */;
    _proto.getLocalEulerAngles = function getLocalEulerAngles() {
      var _getEuler2 = getEuler$1(create$8(), runtime$2.sceneGraphService.getLocalRotation(this)),
        ez = _getEuler2[2];
      return rad2deg$1(ez);
    }
    /**
     * set euler angles(degrees) in world space
     */;
    _proto.setEulerAngles = function setEulerAngles(z) {
      runtime$2.sceneGraphService.setEulerAngles(this, 0, 0, z);
      return this;
    }
    /**
     * set euler angles(degrees) in local space
     */;
    _proto.setLocalEulerAngles = function setLocalEulerAngles(z) {
      runtime$2.sceneGraphService.setLocalEulerAngles(this, 0, 0, z);
      return this;
    };
    _proto.rotateLocal = function rotateLocal(x, y, z) {
      if (isNil$1(y) && isNil$1(z)) {
        runtime$2.sceneGraphService.rotateLocal(this, 0, 0, x);
      } else {
        runtime$2.sceneGraphService.rotateLocal(this, x, y, z);
      }
      return this;
    };
    _proto.rotate = function rotate(x, y, z) {
      if (isNil$1(y) && isNil$1(z)) {
        runtime$2.sceneGraphService.rotate(this, 0, 0, x);
      } else {
        runtime$2.sceneGraphService.rotate(this, x, y, z);
      }
      return this;
    };
    _proto.setRotation = function setRotation(rotation, y, z, w) {
      runtime$2.sceneGraphService.setRotation(this, rotation, y, z, w);
      return this;
    };
    _proto.setLocalRotation = function setLocalRotation(rotation, y, z, w) {
      runtime$2.sceneGraphService.setLocalRotation(this, rotation, y, z, w);
      return this;
    };
    _proto.setLocalSkew = function setLocalSkew(skew, y) {
      runtime$2.sceneGraphService.setLocalSkew(this, skew, y);
      return this;
    };
    _proto.getRotation = function getRotation() {
      return runtime$2.sceneGraphService.getRotation(this);
    };
    _proto.getLocalRotation = function getLocalRotation() {
      return runtime$2.sceneGraphService.getLocalRotation(this);
    };
    _proto.getLocalSkew = function getLocalSkew() {
      return runtime$2.sceneGraphService.getLocalSkew(this);
    };
    _proto.getLocalTransform = function getLocalTransform() {
      return runtime$2.sceneGraphService.getLocalTransform(this);
    };
    _proto.getWorldTransform = function getWorldTransform() {
      return runtime$2.sceneGraphService.getWorldTransform(this);
    };
    _proto.setLocalTransform = function setLocalTransform(transform) {
      runtime$2.sceneGraphService.setLocalTransform(this, transform);
      return this;
    };
    _proto.resetLocalTransform = function resetLocalTransform() {
      runtime$2.sceneGraphService.resetLocalTransform(this);
    }
    // #endregion transformable
    // #region animatable
    /**
     * returns an array of all Animation objects affecting this element
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/getAnimations
     */;
    _proto.getAnimations = function getAnimations() {
      return this.activeAnimations;
    }
    /**
     * create an animation with WAAPI
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/animate
     */;
    _proto.animate = function animate(keyframes, options) {
      var _this$ownerDocument;
      var timeline = (_this$ownerDocument = this.ownerDocument) === null || _this$ownerDocument === void 0 ? void 0 : _this$ownerDocument.timeline;
      if (timeline) {
        return timeline.play(this, keyframes, options);
      }
      return null;
    }
    // #endregion animatable
    // #region visible
    /**
     * shortcut for Used value of `visibility`
     */;
    _proto.isVisible = function isVisible() {
      var _this$parsedStyle;
      return ((_this$parsedStyle = this.parsedStyle) === null || _this$parsedStyle === void 0 ? void 0 : _this$parsedStyle.visibility) === 'visible';
    };
    _proto.isInteractive = function isInteractive() {
      var _this$parsedStyle2;
      return ((_this$parsedStyle2 = this.parsedStyle) === null || _this$parsedStyle2 === void 0 ? void 0 : _this$parsedStyle2.pointerEvents) !== 'none';
    };
    _proto.isCulled = function isCulled() {
      return !!(this.cullable && this.cullable.enable && !this.cullable.visible);
    }
    /**
     * bring to front in current group
     */;
    _proto.toFront = function toFront() {
      if (this.parentNode) {
        this.style.zIndex = Math.max.apply(Math, this.parentNode.children.map(function (child) {
          return Number(child.style.zIndex);
        })) + 1;
      }
      return this;
    }
    /**
     * send to back in current group
     */;
    _proto.toBack = function toBack() {
      if (this.parentNode) {
        this.style.zIndex = Math.min.apply(Math, this.parentNode.children.map(function (child) {
          return Number(child.style.zIndex);
        })) - 1;
      }
      return this;
    }
    // #endregion visible
    // #region deprecated
    /**
     * compatible with G 3.0
     * @alias object.config
     * @deprecated
     */;
    _proto.getConfig = function getConfig() {
      return this.config;
    };
    _proto.attr = function attr() {
      var _this2 = this;
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      var name = args[0],
        value = args[1];
      if (!name) {
        return this.attributes;
      }
      if (isObject$1(name)) {
        Object.keys(name).forEach(function (key) {
          _this2.setAttribute(key, name[key]);
        });
        return this;
      }
      if (args.length === 2) {
        this.setAttribute(name, value);
        return this;
      }
      return this.attributes[name];
    }
    /**
     * return 3x3 matrix in world space
     * @deprecated
     */;
    _proto.getMatrix = function getMatrix(transformMat4) {
      var transform = transformMat4 || this.getWorldTransform();
      var _mat4$getTranslation = getTranslation$1(create$8(), transform),
        tx = _mat4$getTranslation[0],
        ty = _mat4$getTranslation[1];
      var _mat4$getScaling = getScaling$1(create$8(), transform),
        sx = _mat4$getScaling[0],
        sy = _mat4$getScaling[1];
      var rotation = getRotation$1(create$a(), transform);
      var _getEuler3 = getEuler$1(create$8(), rotation),
        eux = _getEuler3[0],
        euz = _getEuler3[2];
      // gimbal lock at 90 degrees
      return fromRotationTranslationScale$3(eux || euz, tx, ty, sx, sy);
    }
    /**
     * return 3x3 matrix in local space
     * @deprecated
     */;
    _proto.getLocalMatrix = function getLocalMatrix() {
      return this.getMatrix(this.getLocalTransform());
    }
    /**
     * set 3x3 matrix in world space
     * @deprecated
     */;
    _proto.setMatrix = function setMatrix(mat) {
      var _decompose = decompose$1(mat),
        tx = _decompose[0],
        ty = _decompose[1],
        scalingX = _decompose[2],
        scalingY = _decompose[3],
        angle = _decompose[4];
      this.setEulerAngles(angle).setPosition(tx, ty).setLocalScale(scalingX, scalingY);
    }
    /**
     * set 3x3 matrix in local space
     * @deprecated
     */;
    _proto.setLocalMatrix = function setLocalMatrix(mat) {
      var _decompose2 = decompose$1(mat),
        tx = _decompose2[0],
        ty = _decompose2[1],
        scalingX = _decompose2[2],
        scalingY = _decompose2[3],
        angle = _decompose2[4];
      this.setLocalEulerAngles(angle).setLocalPosition(tx, ty).setLocalScale(scalingX, scalingY);
    }
    /**
     * Use `visibility: visible` instead.
     * @deprecated
     */;
    _proto.show = function show() {
      this.style.visibility = 'visible';
    }
    /**
     * Use `visibility: hidden` instead.
     * @deprecated
     */;
    _proto.hide = function hide() {
      this.style.visibility = 'hidden';
    }
    /**
     * Use `childElementCount` instead.
     * @deprecated
     */;
    _proto.getCount = function getCount() {
      return this.childElementCount;
    }
    /**
     * Use `parentElement` instead.
     * @deprecated
     */;
    _proto.getParent = function getParent() {
      return this.parentElement;
    }
    /**
     * Use `children` instead.
     * @deprecated
     */;
    _proto.getChildren = function getChildren() {
      return this.children;
    }
    /**
     * Use `firstElementChild` instead.
     * @deprecated
     */;
    _proto.getFirst = function getFirst() {
      return this.firstElementChild;
    }
    /**
     * Use `lastElementChild` instead.
     * @deprecated
     */;
    _proto.getLast = function getLast() {
      return this.lastElementChild;
    }
    /**
     * Use `this.children[index]` instead.
     * @deprecated
     */;
    _proto.getChildByIndex = function getChildByIndex(index) {
      return this.children[index] || null;
    }
    /**
     * Use `appendChild` instead.
     * @deprecated
     */;
    _proto.add = function add(child, index) {
      return this.appendChild(child, index);
    }
    /**
     * Use `this.style.clipPath` instead.
     * @deprecated
     */;
    _proto.setClip = function setClip(clipPath) {
      this.style.clipPath = clipPath;
    }
    /**
     * Use `this.style.clipPath` instead.
     * @deprecated
     */;
    /**
     * @deprecated
     */
    _proto.set = function set(name, value) {
      // @ts-ignore
      this.config[name] = value;
    }
    /**
     * @deprecated
     */;
    _proto.get = function get(name) {
      return this.config[name];
    }
    /**
     * Use `setPosition` instead.
     * @deprecated
     */;
    _proto.moveTo = function moveTo(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      this.setPosition(position, y, z);
      return this;
    }
    /**
     * Use `setPosition` instead.
     * @deprecated
     */;
    _proto.move = function move(position, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      this.setPosition(position, y, z);
      return this;
    }
    /**
     * Use `this.style.zIndex` instead.
     * @deprecated
     */;
    _proto.setZIndex = function setZIndex(zIndex) {
      this.style.zIndex = zIndex;
      return this;
    };
    _createClass$5(DisplayObject, [{
      key: "interactive",
      get: function get() {
        return this.isInteractive();
      },
      set: function set(b) {
        this.style.pointerEvents = b ? 'auto' : 'none';
      }
    }]);
    return DisplayObject;
  }(Element$1);

  var _excluded$d = ["style"];
  var Circle$1 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Circle, _DisplayObject);
    function Circle(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$d);
      return _DisplayObject.call(this, _extends$3({
        type: Shape$2.CIRCLE,
        style: _extends$3({
          cx: '',
          cy: '',
          r: ''
        }, style),
        initialParsedStyle: {
          anchor: [0.5, 0.5]
        }
      }, rest)) || this;
    }
    return Circle;
  }(DisplayObject$1);

  var pool$1 = {};
  var htmlPool$1 = [];
  var DisplayObjectPool$1 = /*#__PURE__*/function () {
    function DisplayObjectPool() {}
    var _proto = DisplayObjectPool.prototype;
    _proto.getByEntity = function getByEntity(entity) {
      return pool$1[entity];
    };
    _proto.getAll = function getAll() {
      return Object.keys(pool$1).map(function (entity) {
        return pool$1[entity];
      });
    };
    _proto.add = function add(entity, groupOrShape) {
      pool$1[entity] = groupOrShape;
      if (groupOrShape.nodeName === Shape$2.HTML) {
        htmlPool$1.push(groupOrShape);
      }
    };
    _proto.remove = function remove(entity) {
      var existed = pool$1[entity];
      if (existed) {
        delete pool$1[entity];
        if (existed.nodeName === Shape$2.HTML) {
          var index = htmlPool$1.indexOf(existed);
          htmlPool$1.splice(index, 1);
        }
      }
    };
    _proto.getHTMLs = function getHTMLs() {
      return htmlPool$1;
    };
    return DisplayObjectPool;
  }();

  var _excluded$2$1 = ["style"];
  var Ellipse$1 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Ellipse, _DisplayObject);
    function Ellipse(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$2$1);
      return _DisplayObject.call(this, _extends$3({
        type: Shape$2.ELLIPSE,
        style: _extends$3({
          cx: '',
          cy: '',
          rx: '',
          ry: ''
        }, style),
        initialParsedStyle: {
          anchor: [0.5, 0.5]
        }
      }, rest)) || this;
    }
    return Ellipse;
  }(DisplayObject$1);

  var _excluded$3$1 = ["style"];
  /**
   * its attributes are inherited by its children.
   * @see https://developer.mozilla.org/zh-CN/docs/Web/SVG/Element/g
   *
   * @example
   * <g fill="white" stroke="green" stroke-width="5">
      <circle cx="40" cy="40" r="25" />
      <circle cx="60" cy="60" r="25" />
    </g>
   */
  var Group$1 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Group, _DisplayObject);
    function Group(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$3$1);
      return _DisplayObject.call(this, _extends$3({
        type: Shape$2.GROUP,
        style: _extends$3({
          x: '',
          y: '',
          width: '',
          height: ''
        }, style)
      }, rest)) || this;
    }
    return Group;
  }(DisplayObject$1);

  var _excluded$4$1 = ["style"];
  /**
   * HTML container
   * @see https://github.com/pmndrs/drei#html
   */
  var HTML$1 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(HTML, _DisplayObject);
    function HTML(_temp) {
      var _this;
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$4$1);
      _this = _DisplayObject.call(this, _extends$3({
        type: Shape$2.HTML,
        style: _extends$3({
          x: '',
          y: '',
          width: 'auto',
          height: 'auto',
          innerHTML: ''
        }, style)
      }, rest)) || this;
      _this.cullable.enable = false;
      return _this;
    }
    /**
     * return wrapper HTMLElement
     * * <div> in g-webgl/canvas
     * * <foreignObject> in g-svg
     */
    var _proto = HTML.prototype;
    _proto.getDomElement = function getDomElement() {
      return this.parsedStyle.$el;
    }
    /**
     * override with $el.getBoundingClientRect
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Element/getBoundingClientRect
     */;
    _proto.getBoundingClientRect = function getBoundingClientRect() {
      return this.parsedStyle.$el.getBoundingClientRect();
    };
    _proto.getClientRects = function getClientRects() {
      return [this.getBoundingClientRect()];
    };
    _proto.getBounds = function getBounds() {
      var _this$ownerDocument, _this$ownerDocument$d;
      var clientRect = this.getBoundingClientRect();
      // calc context's offset
      // @ts-ignore
      var canvasRect = (_this$ownerDocument = this.ownerDocument) === null || _this$ownerDocument === void 0 ? void 0 : (_this$ownerDocument$d = _this$ownerDocument.defaultView) === null || _this$ownerDocument$d === void 0 ? void 0 : _this$ownerDocument$d.getContextService().getBoundingClientRect();
      if (canvasRect) {
        var minX = clientRect.left - canvasRect.left;
        var minY = clientRect.top - canvasRect.top;
        var aabb = new AABB$1();
        aabb.setMinMax(fromValues$8(minX, minY, 0), fromValues$8(minX + clientRect.width, minY + clientRect.height, 0));
        return aabb;
      }
      return null;
    };
    _proto.getLocalBounds = function getLocalBounds() {
      if (this.parentNode) {
        var parentInvert = invert$2(create$7(), this.parentNode.getWorldTransform());
        var bounds = this.getBounds();
        if (!AABB$1.isEmpty(bounds)) {
          var localBounds = new AABB$1();
          localBounds.setFromTransformedAABB(bounds, parentInvert);
          return localBounds;
        }
      }
      return this.getBounds();
    };
    return HTML;
  }(DisplayObject$1);

  var _excluded$5$1 = ["style"];
  var Image$1 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Image, _DisplayObject);
    function Image(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$5$1);
      return _DisplayObject.call(this, _extends$3({
        type: Shape$2.IMAGE,
        style: _extends$3({
          x: '',
          y: '',
          img: '',
          width: '',
          height: ''
        }, style)
      }, rest)) || this;
    }
    return Image;
  }(DisplayObject$1);

  var _excluded$6$1 = ["style"];
  /**
   * Create a line connecting two points.
   * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Element/line
   *
   * Also support for using marker.
   */
  var Line$2 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Line, _DisplayObject);
    function Line(_temp) {
      var _this;
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$6$1);
      _this = _DisplayObject.call(this, _extends$3({
        type: Shape$2.LINE,
        style: _extends$3({
          x1: 0,
          y1: 0,
          x2: 0,
          y2: 0,
          z1: 0,
          z2: 0,
          isBillboard: false
        }, style)
      }, rest)) || this;
      _this.markerStartAngle = 0;
      _this.markerEndAngle = 0;
      var _this$parsedStyle = _this.parsedStyle,
        markerStart = _this$parsedStyle.markerStart,
        markerEnd = _this$parsedStyle.markerEnd;
      if (markerStart && markerStart instanceof DisplayObject$1) {
        _this.markerStartAngle = markerStart.getLocalEulerAngles();
        _this.appendChild(markerStart);
      }
      if (markerEnd && markerEnd instanceof DisplayObject$1) {
        _this.markerEndAngle = markerEnd.getLocalEulerAngles();
        _this.appendChild(markerEnd);
      }
      _this.transformMarker(true);
      _this.transformMarker(false);
      return _this;
    }
    var _proto = Line.prototype;
    _proto.attributeChangedCallback = function attributeChangedCallback(attrName, oldValue, newValue, prevParsedValue, newParsedValue) {
      if (attrName === 'x1' || attrName === 'y1' || attrName === 'x2' || attrName === 'y2' || attrName === 'markerStartOffset' || attrName === 'markerEndOffset') {
        this.transformMarker(true);
        this.transformMarker(false);
      } else if (attrName === 'markerStart') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject$1) {
          this.markerStartAngle = 0;
          prevParsedValue.remove();
        }
        // CSSKeyword 'unset'
        if (newParsedValue && newParsedValue instanceof DisplayObject$1) {
          this.markerStartAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(true);
        }
      } else if (attrName === 'markerEnd') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject$1) {
          this.markerEndAngle = 0;
          prevParsedValue.remove();
        }
        if (newParsedValue && newParsedValue instanceof DisplayObject$1) {
          this.markerEndAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(false);
        }
      }
    };
    _proto.transformMarker = function transformMarker(isStart) {
      var _this$parsedStyle2 = this.parsedStyle,
        markerStart = _this$parsedStyle2.markerStart,
        markerEnd = _this$parsedStyle2.markerEnd,
        markerStartOffset = _this$parsedStyle2.markerStartOffset,
        markerEndOffset = _this$parsedStyle2.markerEndOffset,
        x1 = _this$parsedStyle2.x1,
        x2 = _this$parsedStyle2.x2,
        y1 = _this$parsedStyle2.y1,
        y2 = _this$parsedStyle2.y2,
        defX = _this$parsedStyle2.defX,
        defY = _this$parsedStyle2.defY;
      var marker = isStart ? markerStart : markerEnd;
      if (!marker || !(marker instanceof DisplayObject$1)) {
        return;
      }
      var rad = 0;
      var x;
      var y;
      var ox;
      var oy;
      var offset;
      var originalAngle;
      if (isStart) {
        ox = x1 - defX;
        oy = y1 - defY;
        x = x2 - x1;
        y = y2 - y1;
        offset = markerStartOffset || 0;
        originalAngle = this.markerStartAngle;
      } else {
        ox = x2 - defX;
        oy = y2 - defY;
        x = x1 - x2;
        y = y1 - y2;
        offset = markerEndOffset || 0;
        originalAngle = this.markerEndAngle;
      }
      rad = Math.atan2(y, x);
      // account for markerOffset
      marker.setLocalEulerAngles(rad * 180 / Math.PI + originalAngle);
      marker.setLocalPosition(ox + Math.cos(rad) * offset, oy + Math.sin(rad) * offset);
    };
    _proto.getPoint = function getPoint(ratio, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      // TODO: account for z1/z2 in 3D line
      var _this$parsedStyle3 = this.parsedStyle,
        x1 = _this$parsedStyle3.x1,
        y1 = _this$parsedStyle3.y1,
        x2 = _this$parsedStyle3.x2,
        y2 = _this$parsedStyle3.y2,
        defX = _this$parsedStyle3.defX,
        defY = _this$parsedStyle3.defY;
      var _LineUtil$pointAt = line$3.pointAt(x1, y1, x2, y2, ratio),
        x = _LineUtil$pointAt.x,
        y = _LineUtil$pointAt.y;
      var transformed = transformMat4$2(create$8(), fromValues$8(x - defX, y - defY, 0), inWorldSpace ? this.getWorldTransform() : this.getLocalTransform());
      // apply local transformation
      return new Point$1(transformed[0], transformed[1]);
    };
    _proto.getPointAtLength = function getPointAtLength(distance, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      return this.getPoint(distance / this.getTotalLength(), inWorldSpace);
    };
    _proto.getTotalLength = function getTotalLength() {
      // TODO: account for z1/z2 in 3D line
      var _this$parsedStyle4 = this.parsedStyle,
        x1 = _this$parsedStyle4.x1,
        y1 = _this$parsedStyle4.y1,
        x2 = _this$parsedStyle4.x2,
        y2 = _this$parsedStyle4.y2;
      return line$3.length(x1, y1, x2, y2);
    };
    return Line;
  }(DisplayObject$1);

  var _excluded$7$1 = ["style"];
  var Path$1 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Path, _DisplayObject);
    /**
     * markers placed at the mid
     */

    function Path(_temp) {
      var _this;
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$7$1);
      _this = _DisplayObject.call(this, _extends$3({
        type: Shape$2.PATH,
        style: _extends$3({
          path: '',
          miterLimit: ''
        }, style)
      }, rest)) || this;
      _this.markerStartAngle = 0;
      _this.markerEndAngle = 0;
      _this.markerMidList = [];
      var _this$parsedStyle = _this.parsedStyle,
        markerStart = _this$parsedStyle.markerStart,
        markerEnd = _this$parsedStyle.markerEnd,
        markerMid = _this$parsedStyle.markerMid;
      if (markerStart && markerStart instanceof DisplayObject$1) {
        _this.markerStartAngle = markerStart.getLocalEulerAngles();
        _this.appendChild(markerStart);
      }
      if (markerMid && markerMid instanceof DisplayObject$1) {
        _this.placeMarkerMid(markerMid);
      }
      if (markerEnd && markerEnd instanceof DisplayObject$1) {
        _this.markerEndAngle = markerEnd.getLocalEulerAngles();
        _this.appendChild(markerEnd);
      }
      _this.transformMarker(true);
      _this.transformMarker(false);
      return _this;
    }
    var _proto = Path.prototype;
    _proto.attributeChangedCallback = function attributeChangedCallback(attrName, oldValue, newValue, prevParsedValue, newParsedValue) {
      if (attrName === 'path') {
        // recalc markers
        this.transformMarker(true);
        this.transformMarker(false);
        this.placeMarkerMid(this.parsedStyle.markerMid);
      } else if (attrName === 'markerStartOffset' || attrName === 'markerEndOffset') {
        this.transformMarker(true);
        this.transformMarker(false);
      } else if (attrName === 'markerStart') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject$1) {
          this.markerStartAngle = 0;
          prevParsedValue.remove();
        }
        // CSSKeyword 'unset'
        if (newParsedValue && newParsedValue instanceof DisplayObject$1) {
          this.markerStartAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(true);
        }
      } else if (attrName === 'markerEnd') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject$1) {
          this.markerEndAngle = 0;
          prevParsedValue.remove();
        }
        if (newParsedValue && newParsedValue instanceof DisplayObject$1) {
          this.markerEndAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(false);
        }
      } else if (attrName === 'markerMid') {
        this.placeMarkerMid(newParsedValue);
      }
    };
    _proto.transformMarker = function transformMarker(isStart) {
      var _this$parsedStyle2 = this.parsedStyle,
        markerStart = _this$parsedStyle2.markerStart,
        markerEnd = _this$parsedStyle2.markerEnd,
        markerStartOffset = _this$parsedStyle2.markerStartOffset,
        markerEndOffset = _this$parsedStyle2.markerEndOffset,
        defX = _this$parsedStyle2.defX,
        defY = _this$parsedStyle2.defY;
      var marker = isStart ? markerStart : markerEnd;
      if (!marker || !(marker instanceof DisplayObject$1)) {
        return;
      }
      var rad = 0;
      var x;
      var y;
      var ox;
      var oy;
      var offset;
      var originalAngle;
      if (isStart) {
        var _this$getStartTangent = this.getStartTangent(),
          p1 = _this$getStartTangent[0],
          p2 = _this$getStartTangent[1];
        ox = p2[0] - defX;
        oy = p2[1] - defY;
        x = p1[0] - p2[0];
        y = p1[1] - p2[1];
        offset = markerStartOffset || 0;
        originalAngle = this.markerStartAngle;
      } else {
        var _this$getEndTangent = this.getEndTangent(),
          _p = _this$getEndTangent[0],
          _p2 = _this$getEndTangent[1];
        ox = _p2[0] - defX;
        oy = _p2[1] - defY;
        x = _p[0] - _p2[0];
        y = _p[1] - _p2[1];
        offset = markerEndOffset || 0;
        originalAngle = this.markerEndAngle;
      }
      rad = Math.atan2(y, x);
      // account for markerOffset
      marker.setLocalEulerAngles(rad * 180 / Math.PI + originalAngle);
      marker.setLocalPosition(ox + Math.cos(rad) * offset, oy + Math.sin(rad) * offset);
    };
    _proto.placeMarkerMid = function placeMarkerMid(marker) {
      var _this$parsedStyle3 = this.parsedStyle,
        segments = _this$parsedStyle3.path.segments,
        defX = _this$parsedStyle3.defX,
        defY = _this$parsedStyle3.defY;
      // clear all existed markers
      this.markerMidList.forEach(function (marker) {
        marker.remove();
      });
      if (marker && marker instanceof DisplayObject$1) {
        for (var i = 1; i < segments.length - 1; i++) {
          var _segments$i$currentPo = segments[i].currentPoint,
            ox = _segments$i$currentPo[0],
            oy = _segments$i$currentPo[1];
          var cloned = i === 1 ? marker : marker.cloneNode(true);
          this.markerMidList.push(cloned);
          this.appendChild(cloned);
          cloned.setLocalPosition(ox - defX, oy - defY);
          // TODO: orient of marker
        }
      }
    }
    /**
     * Returns the total length of the path.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGGeometryElement/getTotalLength
     */;
    _proto.getTotalLength = function getTotalLength() {
      return getOrCalculatePathTotalLength$1(this);
    }
    /**
     * Returns the point at a given distance along the path.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGGeometryElement/getPointAtLength
     */;
    _proto.getPointAtLength = function getPointAtLength$1$1(distance, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      var _this$parsedStyle4 = this.parsedStyle,
        defX = _this$parsedStyle4.defX,
        defY = _this$parsedStyle4.defY,
        absolutePath = _this$parsedStyle4.path.absolutePath;
      var _getPointAtLength2 = getPointAtLength$1(absolutePath, distance),
        x = _getPointAtLength2.x,
        y = _getPointAtLength2.y;
      var transformed = transformMat4$2(create$8(), fromValues$8(x - defX, y - defY, 0), inWorldSpace ? this.getWorldTransform() : this.getLocalTransform());
      // apply local transformation
      return new Point$1(transformed[0], transformed[1]);
    }
    /**
     * Returns the point at a given ratio of the total length in path.
     */;
    _proto.getPoint = function getPoint(ratio, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      return this.getPointAtLength(ratio * getOrCalculatePathTotalLength$1(this), inWorldSpace);
    }
    /**
     * Get start tangent vector
     */;
    _proto.getStartTangent = function getStartTangent() {
      var segments = this.parsedStyle.path.segments;
      var result = [];
      if (segments.length > 1) {
        var startPoint = segments[0].currentPoint;
        var endPoint = segments[1].currentPoint;
        var tangent = segments[1].startTangent;
        result = [];
        if (tangent) {
          result.push([startPoint[0] - tangent[0], startPoint[1] - tangent[1]]);
          result.push([startPoint[0], startPoint[1]]);
        } else {
          result.push([endPoint[0], endPoint[1]]);
          result.push([startPoint[0], startPoint[1]]);
        }
      }
      return result;
    }
    /**
     * Get end tangent vector
     */;
    _proto.getEndTangent = function getEndTangent() {
      var segments = this.parsedStyle.path.segments;
      var length = segments.length;
      var result = [];
      if (length > 1) {
        var startPoint = segments[length - 2].currentPoint;
        var endPoint = segments[length - 1].currentPoint;
        var tangent = segments[length - 1].endTangent;
        result = [];
        if (tangent) {
          result.push([endPoint[0] - tangent[0], endPoint[1] - tangent[1]]);
          result.push([endPoint[0], endPoint[1]]);
        } else {
          result.push([startPoint[0], startPoint[1]]);
          result.push([endPoint[0], endPoint[1]]);
        }
      }
      return result;
    };
    return Path;
  }(DisplayObject$1);

  var _excluded$8$1 = ["style"];
  var Polygon$1 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Polygon, _DisplayObject);
    /**
     * markers placed at the mid
     */

    function Polygon(_temp) {
      var _this;
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$8$1);
      _this = _DisplayObject.call(this, _extends$3({
        type: Shape$2.POLYGON,
        style: _extends$3({
          points: '',
          miterLimit: '',
          isClosed: true
        }, style)
      }, rest)) || this;
      _this.markerStartAngle = 0;
      _this.markerEndAngle = 0;
      _this.markerMidList = [];
      var _this$parsedStyle = _this.parsedStyle,
        markerStart = _this$parsedStyle.markerStart,
        markerEnd = _this$parsedStyle.markerEnd,
        markerMid = _this$parsedStyle.markerMid;
      if (markerStart && markerStart instanceof DisplayObject$1) {
        _this.markerStartAngle = markerStart.getLocalEulerAngles();
        _this.appendChild(markerStart);
      }
      if (markerMid && markerMid instanceof DisplayObject$1) {
        _this.placeMarkerMid(markerMid);
      }
      if (markerEnd && markerEnd instanceof DisplayObject$1) {
        _this.markerEndAngle = markerEnd.getLocalEulerAngles();
        _this.appendChild(markerEnd);
      }
      _this.transformMarker(true);
      _this.transformMarker(false);
      return _this;
    }
    var _proto = Polygon.prototype;
    _proto.attributeChangedCallback = function attributeChangedCallback(attrName, oldValue, newValue, prevParsedValue, newParsedValue) {
      if (attrName === 'points') {
        // recalc markers
        this.transformMarker(true);
        this.transformMarker(false);
        this.placeMarkerMid(this.parsedStyle.markerMid);
      } else if (attrName === 'markerStartOffset' || attrName === 'markerEndOffset') {
        this.transformMarker(true);
        this.transformMarker(false);
      } else if (attrName === 'markerStart') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject$1) {
          this.markerStartAngle = 0;
          prevParsedValue.remove();
        }
        // CSSKeyword 'unset'
        if (newParsedValue && newParsedValue instanceof DisplayObject$1) {
          this.markerStartAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(true);
        }
      } else if (attrName === 'markerEnd') {
        if (prevParsedValue && prevParsedValue instanceof DisplayObject$1) {
          this.markerEndAngle = 0;
          prevParsedValue.remove();
        }
        if (newParsedValue && newParsedValue instanceof DisplayObject$1) {
          this.markerEndAngle = newParsedValue.getLocalEulerAngles();
          this.appendChild(newParsedValue);
          this.transformMarker(false);
        }
      } else if (attrName === 'markerMid') {
        this.placeMarkerMid(newParsedValue);
      }
    };
    _proto.transformMarker = function transformMarker(isStart) {
      var _this$parsedStyle2 = this.parsedStyle,
        markerStart = _this$parsedStyle2.markerStart,
        markerEnd = _this$parsedStyle2.markerEnd,
        markerStartOffset = _this$parsedStyle2.markerStartOffset,
        markerEndOffset = _this$parsedStyle2.markerEndOffset,
        points = _this$parsedStyle2.points.points,
        defX = _this$parsedStyle2.defX,
        defY = _this$parsedStyle2.defY;
      var marker = isStart ? markerStart : markerEnd;
      if (!marker || !(marker instanceof DisplayObject$1)) {
        return;
      }
      var rad = 0;
      var x;
      var y;
      var ox;
      var oy;
      var offset;
      var originalAngle;
      ox = points[0][0] - defX;
      oy = points[0][1] - defY;
      if (isStart) {
        x = points[1][0] - points[0][0];
        y = points[1][1] - points[0][1];
        offset = markerStartOffset || 0;
        originalAngle = this.markerStartAngle;
      } else {
        var length = points.length;
        if (!this.style.isClosed) {
          ox = points[length - 1][0] - defX;
          oy = points[length - 1][1] - defY;
          x = points[length - 2][0] - points[length - 1][0];
          y = points[length - 2][1] - points[length - 1][1];
        } else {
          x = points[length - 1][0] - points[0][0];
          y = points[length - 1][1] - points[0][1];
        }
        offset = markerEndOffset || 0;
        originalAngle = this.markerEndAngle;
      }
      rad = Math.atan2(y, x);
      // account for markerOffset
      marker.setLocalEulerAngles(rad * 180 / Math.PI + originalAngle);
      marker.setLocalPosition(ox + Math.cos(rad) * offset, oy + Math.sin(rad) * offset);
    };
    _proto.placeMarkerMid = function placeMarkerMid(marker) {
      var _this$parsedStyle3 = this.parsedStyle,
        points = _this$parsedStyle3.points.points,
        defX = _this$parsedStyle3.defX,
        defY = _this$parsedStyle3.defY;
      // clear all existed markers
      this.markerMidList.forEach(function (marker) {
        marker.remove();
      });
      this.markerMidList = [];
      if (marker && marker instanceof DisplayObject$1) {
        for (var i = 1; i < (this.style.isClosed ? points.length : points.length - 1); i++) {
          var ox = points[i][0] - defX;
          var oy = points[i][1] - defY;
          var cloned = i === 1 ? marker : marker.cloneNode(true);
          this.markerMidList.push(cloned);
          this.appendChild(cloned);
          cloned.setLocalPosition(ox, oy);
          // TODO: orient of marker
        }
      }
    };
    return Polygon;
  }(DisplayObject$1);

  var _excluded$9$1 = ["style"];
  /**
   * Polyline inherits the marker-related capabilities of Polygon.
   */
  var Polyline$1 = /*#__PURE__*/function (_Polygon) {
    _inheritsLoose$9(Polyline, _Polygon);
    function Polyline(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$9$1);
      return _Polygon.call(this, _extends$3({
        type: Shape$2.POLYLINE,
        style: _extends$3({
          points: '',
          miterLimit: '',
          isClosed: false
        }, style)
      }, rest)) || this;
    }
    var _proto = Polyline.prototype;
    _proto.getTotalLength = function getTotalLength() {
      return this.parsedStyle.points.totalLength;
    };
    _proto.getPointAtLength = function getPointAtLength(distance, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      return this.getPoint(distance / this.getTotalLength(), inWorldSpace);
    };
    _proto.getPoint = function getPoint(ratio, inWorldSpace) {
      if (inWorldSpace === void 0) {
        inWorldSpace = false;
      }
      var _this$parsedStyle = this.parsedStyle,
        defX = _this$parsedStyle.defX,
        defY = _this$parsedStyle.defY,
        _this$parsedStyle$poi = _this$parsedStyle.points,
        points = _this$parsedStyle$poi.points,
        segments = _this$parsedStyle$poi.segments;
      var subt = 0;
      var index = 0;
      segments.forEach(function (v, i) {
        if (ratio >= v[0] && ratio <= v[1]) {
          subt = (ratio - v[0]) / (v[1] - v[0]);
          index = i;
        }
      });
      var _LineUtil$pointAt = line$3.pointAt(points[index][0], points[index][1], points[index + 1][0], points[index + 1][1], subt),
        x = _LineUtil$pointAt.x,
        y = _LineUtil$pointAt.y;
      var transformed = transformMat4$2(create$8(), fromValues$8(x - defX, y - defY, 0), inWorldSpace ? this.getWorldTransform() : this.getLocalTransform());
      // apply local transformation
      return new Point$1(transformed[0], transformed[1]);
    };
    _proto.getStartTangent = function getStartTangent() {
      var points = this.parsedStyle.points.points;
      var result = [];
      result.push([points[1][0], points[1][1]]);
      result.push([points[0][0], points[0][1]]);
      return result;
    };
    _proto.getEndTangent = function getEndTangent() {
      var points = this.parsedStyle.points.points;
      var l = points.length - 1;
      var result = [];
      result.push([points[l - 1][0], points[l - 1][1]]);
      result.push([points[l][0], points[l][1]]);
      return result;
    };
    return Polyline;
  }(Polygon$1);

  var _excluded$a$1 = ["style"];
  var Rect$3 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Rect, _DisplayObject);
    function Rect(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$a$1);
      return _DisplayObject.call(this, _extends$3({
        type: Shape$2.RECT,
        style: _extends$3({
          x: '',
          y: '',
          width: '',
          height: '',
          radius: ''
        }, style)
      }, rest)) || this;
    }
    return Rect;
  }(DisplayObject$1);

  var _excluded$b$1 = ["style"];
  /**
   * <text> @see https://developer.mozilla.org/en-US/docs/Web/API/SVGTextElement
   */
  var Text$1 = /*#__PURE__*/function (_DisplayObject) {
    _inheritsLoose$9(Text, _DisplayObject);
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGTextContentElement#constants
     */
    // LENGTHADJUST_SPACING: number = 1;
    // LENGTHADJUST_SPACINGANDGLYPHS: number = 2;
    // LENGTHADJUST_UNKNOWN: number = 0;
    function Text(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
        style = _ref.style,
        rest = _objectWithoutPropertiesLoose$2(_ref, _excluded$b$1);
      return _DisplayObject.call(this, _extends$3({
        type: Shape$2.TEXT,
        style: _extends$3({
          x: '',
          y: '',
          text: '',
          fontSize: '',
          fontFamily: '',
          fontStyle: '',
          fontWeight: '',
          fontVariant: '',
          textAlign: '',
          textBaseline: '',
          textTransform: '',
          fill: 'black',
          letterSpacing: '',
          lineHeight: '',
          miterLimit: '',
          // whiteSpace: 'pre',
          wordWrap: false,
          wordWrapWidth: 0,
          leading: 0,
          dx: '',
          dy: ''
        }, style)
      }, rest)) || this;
    }
    // lengthAdjust: SVGAnimatedEnumeration;
    // textLength: SVGAnimatedLength;
    // getCharNumAtPosition(point?: DOMPointInit): number {
    //   throw new Error('Method not implemented.');
    // }
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/SVGTextContentElement
     */
    var _proto = Text.prototype;
    _proto.getComputedTextLength = function getComputedTextLength() {
      var _this$parsedStyle$met;
      return ((_this$parsedStyle$met = this.parsedStyle.metrics) === null || _this$parsedStyle$met === void 0 ? void 0 : _this$parsedStyle$met.maxLineWidth) || 0;
    }
    // getEndPositionOfChar(charnum: number): DOMPoint {
    //   throw new Error('Method not implemented.');
    // }
    // getExtentOfChar(charnum: number): DOMRect {
    //   throw new Error('Method not implemented.');
    // }
    // getNumberOfChars(): number {
    //   throw new Error('Method not implemented.');
    // }
    // getRotationOfChar(charnum: number): number {
    //   throw new Error('Method not implemented.');
    // }
    // getStartPositionOfChar(charnum: number): DOMPoint {
    //   throw new Error('Method not implemented.');
    // }
    // getSubStringLength(charnum: number, nchars: number): number {
    //   throw new Error('Method not implemented.');
    // }
    // selectSubString(charnum: number, nchars: number): void {
    //   throw new Error('Method not implemented.');
    // }
    ;
    _proto.getLineBoundingRects = function getLineBoundingRects() {
      var _this$parsedStyle$met2;
      return ((_this$parsedStyle$met2 = this.parsedStyle.metrics) === null || _this$parsedStyle$met2 === void 0 ? void 0 : _this$parsedStyle$met2.lineMetrics) || [];
    };
    _proto.isOverflowing = function isOverflowing() {
      return !!this.parsedStyle.isOverflowing;
    };
    return Text;
  }(DisplayObject$1);

  function getAngle$3(angle) {
    if (angle === undefined) {
      return 0;
    } else if (angle > 360 || angle < -360) {
      return angle % 360;
    }
    return angle;
  }
  function createVec3$1(x, y, z) {
    if (y === void 0) {
      y = 0;
    }
    if (z === void 0) {
      z = 0;
    }
    if (isNumber$1(x)) {
      return fromValues$8(x, y, z);
    }
    if (Array.isArray(x) && x.length === 3) {
      return clone$3(x);
    }
    return fromValues$8(x[0], x[1] || y, x[2] || z);
  }
  function deg2rad$1(deg) {
    return deg * (Math.PI / 180);
  }
  function rad2deg$1(rad) {
    return rad * (180 / Math.PI);
  }
  function turn2deg$1(turn) {
    return 360 * turn;
  }
  function getEulerFromQuat$1(out, quat) {
    var x = quat[0];
    var y = quat[1];
    var z = quat[2];
    var w = quat[3];
    var x2 = x * x;
    var y2 = y * y;
    var z2 = z * z;
    var w2 = w * w;
    var unit = x2 + y2 + z2 + w2;
    var test = x * w - y * z;
    if (test > 0.499995 * unit) {
      // TODO: Use glmatrix.EPSILON
      // singularity at the north pole
      out[0] = Math.PI / 2;
      out[1] = 2 * Math.atan2(y, x);
      out[2] = 0;
    } else if (test < -0.499995 * unit) {
      //TODO: Use glmatrix.EPSILON
      // singularity at the south pole
      out[0] = -Math.PI / 2;
      out[1] = 2 * Math.atan2(y, x);
      out[2] = 0;
    } else {
      out[0] = Math.asin(2 * (x * z - w * y));
      out[1] = Math.atan2(2 * (x * w + y * z), 1 - 2 * (z2 + w2));
      out[2] = Math.atan2(2 * (x * y + z * w), 1 - 2 * (y2 + z2));
    }
    // TODO: Return them as degrees and not as radians
    return out;
  }
  function getEulerFromMat4$1(out, m) {
    var x;
    var z;
    var halfPi = Math.PI * 0.5;
    var _mat4$getScaling = getScaling$1(create$8(), m),
      sx = _mat4$getScaling[0],
      sy = _mat4$getScaling[1],
      sz = _mat4$getScaling[2];
    var y = Math.asin(-m[2] / sx);
    if (y < halfPi) {
      if (y > -halfPi) {
        x = Math.atan2(m[6] / sy, m[10] / sz);
        z = Math.atan2(m[1] / sx, m[0] / sx);
      } else {
        // Not a unique solution
        z = 0;
        x = -Math.atan2(m[4] / sy, m[5] / sy);
      }
    } else {
      // Not a unique solution
      z = 0;
      x = Math.atan2(m[4] / sy, m[5] / sy);
    }
    out[0] = x;
    out[1] = y;
    out[2] = z;
    return out;
  }
  /**
   * @see https://github.com/toji/gl-matrix/issues/329
   * @see https://doc.babylonjs.com/divingDeeper/mesh/transforms/center_origin/rotation_conventions
   */
  function getEuler$1(out, quat) {
    if (quat.length === 16) {
      return getEulerFromMat4$1(out, quat);
    } else {
      return getEulerFromQuat$1(out, quat);
    }
  }
  function fromRotationTranslationScale$3(rotation, x, y, scaleX, scaleY) {
    var cos = Math.cos(rotation);
    var sin = Math.sin(rotation);
    return fromValues$6(scaleX * cos, scaleY * sin, 0, -scaleX * sin, scaleY * cos, 0, x, y, 1);
  }
  function makePerspective$1(out, left, right, top, bottom, near, far) {
    var x = 2 * near / (right - left);
    var y = 2 * near / (top - bottom);
    var a = (right + left) / (right - left);
    var b = (top + bottom) / (top - bottom);
    var c = -(far + near) / (far - near);
    var d = -2 * far * near / (far - near);
    out[0] = x;
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[4] = 0;
    out[5] = y;
    out[6] = 0;
    out[7] = 0;
    out[8] = a;
    out[9] = b;
    out[10] = c;
    out[11] = -1;
    out[12] = 0;
    out[13] = 0;
    out[14] = d;
    out[15] = 0;
    return out;
  }
  function decompose$1(mat) {
    var row0x = mat[0];
    var row0y = mat[1];
    var row1x = mat[3];
    var row1y = mat[4];
    // decompose 3x3 matrix
    // @see https://www.w3.org/TR/css-transforms-1/#decomposing-a-2d-matrix
    var scalingX = Math.sqrt(row0x * row0x + row0y * row0y);
    var scalingY = Math.sqrt(row1x * row1x + row1y * row1y);
    // If determinant is negative, one axis was flipped.
    var determinant = row0x * row1y - row0y * row1x;
    if (determinant < 0) {
      // Flip axis with minimum unit vector dot product.
      if (row0x < row1y) {
        scalingX = -scalingX;
      } else {
        scalingY = -scalingY;
      }
    }
    // Renormalize matrix to remove scale.
    if (scalingX) {
      row0x *= 1 / scalingX;
      row0y *= 1 / scalingX;
    }
    if (scalingY) {
      row1x *= 1 / scalingY;
      row1y *= 1 / scalingY;
    }
    // Compute rotation and renormalize matrix.
    var rotation = Math.atan2(row0y, row0x);
    var angle = rad2deg$1(rotation);
    return [mat[6], mat[7], scalingX, scalingY, angle];
  }
  var tmp$1 = create$7();
  var perspectiveMatrix$1 = create$7();
  var tmpVec4$1 = create$9();
  var row$1 = [create$8(), create$8(), create$8()];
  var pdum3$1 = create$8();
  /*
  Input:  matrix      ; a 4x4 matrix
  Output: translation ; a 3 component vector
          scale       ; a 3 component vector
          skew        ; skew factors XY,XZ,YZ represented as a 3 component vector
          perspective ; a 4 component vector
          quaternion  ; a 4 component vector
  Returns false if the matrix cannot be decomposed, true if it can


  References:
  https://github.com/kamicane/matrix3d/blob/master/lib/Matrix3d.js
  https://github.com/ChromiumWebApps/chromium/blob/master/ui/gfx/transform_util.cc
  http://www.w3.org/TR/css3-transforms/#decomposing-a-3d-matrix
  */
  function decomposeMat4$1(matrix, translation, scale, skew, perspective, quaternion) {
    //normalize, if not possible then bail out early
    if (!normalize$9(tmp$1, matrix)) return false;
    // perspectiveMatrix is used to solve for perspective, but it also provides
    // an easy way to test for singularity of the upper 3x3 component.
    copy$5(perspectiveMatrix$1, tmp$1);
    perspectiveMatrix$1[3] = 0;
    perspectiveMatrix$1[7] = 0;
    perspectiveMatrix$1[11] = 0;
    perspectiveMatrix$1[15] = 1;
    // If the perspectiveMatrix is not invertible, we are also unable to
    // decompose, so we'll bail early. Constant taken from SkMatrix44::invert.
    if (Math.abs(determinant$1(perspectiveMatrix$1)) < 1e-8) return false;
    var a03 = tmp$1[3],
      a13 = tmp$1[7],
      a23 = tmp$1[11],
      a30 = tmp$1[12],
      a31 = tmp$1[13],
      a32 = tmp$1[14],
      a33 = tmp$1[15];
    // First, isolate perspective.
    if (a03 !== 0 || a13 !== 0 || a23 !== 0) {
      tmpVec4$1[0] = a03;
      tmpVec4$1[1] = a13;
      tmpVec4$1[2] = a23;
      tmpVec4$1[3] = a33;
      // Solve the equation by inverting perspectiveMatrix and multiplying
      // rightHandSide by the inverse.
      // resuing the perspectiveMatrix here since it's no longer needed
      var ret = invert$2(perspectiveMatrix$1, perspectiveMatrix$1);
      if (!ret) return false;
      transpose$1(perspectiveMatrix$1, perspectiveMatrix$1);
      //multiply by transposed inverse perspective matrix, into perspective vec4
      transformMat4$3(perspective, tmpVec4$1, perspectiveMatrix$1);
    } else {
      //no perspective
      perspective[0] = perspective[1] = perspective[2] = 0;
      perspective[3] = 1;
    }
    // Next take care of translation
    translation[0] = a30;
    translation[1] = a31;
    translation[2] = a32;
    // Now get scale and shear. 'row' is a 3 element array of 3 component vectors
    mat3from4$1(row$1, tmp$1);
    // Compute X scale factor and normalize first row.
    scale[0] = length$1(row$1[0]);
    normalize$5(row$1[0], row$1[0]);
    // Compute XY shear factor and make 2nd row orthogonal to 1st.
    skew[0] = dot$3(row$1[0], row$1[1]);
    combine$1(row$1[1], row$1[1], row$1[0], 1.0, -skew[0]);
    // Now, compute Y scale and normalize 2nd row.
    scale[1] = length$1(row$1[1]);
    normalize$5(row$1[1], row$1[1]);
    skew[0] /= scale[1];
    // Compute XZ and YZ shears, orthogonalize 3rd row
    skew[1] = dot$3(row$1[0], row$1[2]);
    combine$1(row$1[2], row$1[2], row$1[0], 1.0, -skew[1]);
    skew[2] = dot$3(row$1[1], row$1[2]);
    combine$1(row$1[2], row$1[2], row$1[1], 1.0, -skew[2]);
    // Next, get Z scale and normalize 3rd row.
    scale[2] = length$1(row$1[2]);
    normalize$5(row$1[2], row$1[2]);
    skew[1] /= scale[2];
    skew[2] /= scale[2];
    // At this point, the matrix (in rows) is orthonormal.
    // Check for a coordinate system flip.  If the determinant
    // is -1, then negate the matrix and the scaling factors.
    cross$1(pdum3$1, row$1[1], row$1[2]);
    if (dot$3(row$1[0], pdum3$1) < 0) {
      for (var i = 0; i < 3; i++) {
        scale[i] *= -1;
        row$1[i][0] *= -1;
        row$1[i][1] *= -1;
        row$1[i][2] *= -1;
      }
    }
    // Now, get the rotations out
    quaternion[0] = 0.5 * Math.sqrt(Math.max(1 + row$1[0][0] - row$1[1][1] - row$1[2][2], 0));
    quaternion[1] = 0.5 * Math.sqrt(Math.max(1 - row$1[0][0] + row$1[1][1] - row$1[2][2], 0));
    quaternion[2] = 0.5 * Math.sqrt(Math.max(1 - row$1[0][0] - row$1[1][1] + row$1[2][2], 0));
    quaternion[3] = 0.5 * Math.sqrt(Math.max(1 + row$1[0][0] + row$1[1][1] + row$1[2][2], 0));
    if (row$1[2][1] > row$1[1][2]) quaternion[0] = -quaternion[0];
    if (row$1[0][2] > row$1[2][0]) quaternion[1] = -quaternion[1];
    if (row$1[1][0] > row$1[0][1]) quaternion[2] = -quaternion[2];
    return true;
  }
  function normalize$9(out, mat) {
    var m44 = mat[15];
    // Cannot normalize.
    if (m44 === 0) return false;
    var scale = 1 / m44;
    for (var i = 0; i < 16; i++) {
      out[i] = mat[i] * scale;
    }
    return true;
  }
  //gets upper-left of a 4x4 matrix into a 3x3 of vectors
  function mat3from4$1(out, mat4x4) {
    out[0][0] = mat4x4[0];
    out[0][1] = mat4x4[1];
    out[0][2] = mat4x4[2];
    out[1][0] = mat4x4[4];
    out[1][1] = mat4x4[5];
    out[1][2] = mat4x4[6];
    out[2][0] = mat4x4[8];
    out[2][1] = mat4x4[9];
    out[2][2] = mat4x4[10];
  }
  function combine$1(out, a, b, scale1, scale2) {
    out[0] = a[0] * scale1 + b[0] * scale2;
    out[1] = a[1] * scale1 + b[1] * scale2;
    out[2] = a[2] * scale1 + b[2] * scale2;
  }
  var tmpMat4$1 = create$7();
  function parsedTransformToMat4(transform, object) {
    if (transform && transform.length) {
      var defX = 0;
      var defY = 0;
      if (object) {
        defX = object.parsedStyle.defX || 0;
        defY = object.parsedStyle.defY || 0;
        // reset transform
        object.resetLocalTransform();
        object.setLocalPosition(defX, defY);
      } else {
        object = new DisplayObject$1({});
      }
      transform.forEach(function (parsed) {
        var t = parsed.t,
          d = parsed.d;
        if (t === 'scale') {
          // scale(1) scale(1, 1)
          var newScale = (d === null || d === void 0 ? void 0 : d.map(function (s) {
            return s.value;
          })) || [1, 1];
          object.scaleLocal(newScale[0], newScale[1], 1);
        } else if (t === 'scalex') {
          var _newScale = (d === null || d === void 0 ? void 0 : d.map(function (s) {
            return s.value;
          })) || [1];
          object.scaleLocal(_newScale[0], 1, 1);
        } else if (t === 'scaley') {
          var _newScale2 = (d === null || d === void 0 ? void 0 : d.map(function (s) {
            return s.value;
          })) || [1];
          object.scaleLocal(1, _newScale2[0], 1);
        } else if (t === 'scalez') {
          var _newScale3 = (d === null || d === void 0 ? void 0 : d.map(function (s) {
            return s.value;
          })) || [1];
          object.scaleLocal(1, 1, _newScale3[0]);
        } else if (t === 'scale3d') {
          var _newScale4 = (d === null || d === void 0 ? void 0 : d.map(function (s) {
            return s.value;
          })) || [1, 1, 1];
          object.scaleLocal(_newScale4[0], _newScale4[1], _newScale4[2]);
        } else if (t === 'translate') {
          var newTranslation = d || [Opx$1, Opx$1];
          object.translateLocal(newTranslation[0].value, newTranslation[1].value, 0);
        } else if (t === 'translatex') {
          var _newTranslation = d || [Opx$1];
          object.translateLocal(_newTranslation[0].value, 0, 0);
        } else if (t === 'translatey') {
          var _newTranslation2 = d || [Opx$1];
          object.translateLocal(0, _newTranslation2[0].value, 0);
        } else if (t === 'translatez') {
          var _newTranslation3 = d || [Opx$1];
          object.translateLocal(0, 0, _newTranslation3[0].value);
        } else if (t === 'translate3d') {
          var _newTranslation4 = d || [Opx$1, Opx$1, Opx$1];
          object.translateLocal(_newTranslation4[0].value, _newTranslation4[1].value, _newTranslation4[2].value);
        } else if (t === 'rotate') {
          var newAngles = d || [Odeg$1];
          object.rotateLocal(0, 0, newAngles[0].value);
        } else if (t === 'rotatex') {
          var _newAngles = d || [Odeg$1];
          object.rotateLocal(_newAngles[0].value, 0, 0);
        } else if (t === 'rotatey') {
          var _newAngles2 = d || [Odeg$1];
          object.rotateLocal(0, _newAngles2[0].value, 0);
        } else if (t === 'rotatez') {
          var _newAngles3 = d || [Odeg$1];
          object.rotateLocal(0, 0, _newAngles3[0].value);
        } else if (t === 'rotate3d') ; else if (t === 'skew') {
          var newSkew = (d === null || d === void 0 ? void 0 : d.map(function (s) {
            return s.value;
          })) || [0, 0];
          object.setLocalSkew(deg2rad$1(newSkew[0]), deg2rad$1(newSkew[1]));
        } else if (t === 'skewx') {
          var _newSkew = (d === null || d === void 0 ? void 0 : d.map(function (s) {
            return s.value;
          })) || [0];
          object.setLocalSkew(deg2rad$1(_newSkew[0]), object.getLocalSkew()[1]);
        } else if (t === 'skewy') {
          var _newSkew2 = (d === null || d === void 0 ? void 0 : d.map(function (s) {
            return s.value;
          })) || [0];
          object.setLocalSkew(object.getLocalSkew()[0], deg2rad$1(_newSkew2[0]));
        } else if (t === 'matrix') {
          var _d$map = d.map(function (s) {
              return s.value;
            }),
            a = _d$map[0],
            b = _d$map[1],
            c = _d$map[2],
            dd = _d$map[3],
            tx = _d$map[4],
            ty = _d$map[5];
          object.setLocalTransform(set$2(tmpMat4$1, a, b, 0, 0, c, dd, 0, 0, 0, 0, 1, 0, tx + defX, ty + defY, 0, 1));
        } else if (t === 'matrix3d') {
          // @ts-ignore
          set$2.apply(mat4$1, [tmpMat4$1].concat(d.map(function (s) {
            return s.value;
          })));
          tmpMat4$1[12] += defX;
          tmpMat4$1[13] += defY;
          object.setLocalTransform(tmpMat4$1);
        }
      });
    }
    return object.getLocalTransform();
  }

  /**
   * borrow from gradient-parser, but we delete some browser compatible prefix such as `-webkit-`
   * @see https://github.com/rafaelcaricio/gradient-parser
   */
  function colorStopToString$1(colorStop) {
    var type = colorStop.type,
      value = colorStop.value;
    if (type === 'hex') {
      return "#" + value;
    } else if (type === 'literal') {
      return value;
    } else if (type === 'rgb') {
      return "rgb(" + value.join(',') + ")";
    } else {
      return "rgba(" + value.join(',') + ")";
    }
  }
  var parseGradient$1$1 = function () {
    var tokens = {
      linearGradient: /^(linear\-gradient)/i,
      repeatingLinearGradient: /^(repeating\-linear\-gradient)/i,
      radialGradient: /^(radial\-gradient)/i,
      repeatingRadialGradient: /^(repeating\-radial\-gradient)/i,
      /**
       * @see https://projects.verou.me/conic-gradient/
       */
      conicGradient: /^(conic\-gradient)/i,
      sideOrCorner: /^to (left (top|bottom)|right (top|bottom)|top (left|right)|bottom (left|right)|left|right|top|bottom)/i,
      extentKeywords: /^(closest\-side|closest\-corner|farthest\-side|farthest\-corner|contain|cover)/,
      positionKeywords: /^(left|center|right|top|bottom)/i,
      pixelValue: /^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))px/,
      percentageValue: /^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))\%/,
      emValue: /^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))em/,
      angleValue: /^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))deg/,
      startCall: /^\(/,
      endCall: /^\)/,
      comma: /^,/,
      hexColor: /^\#([0-9a-fA-F]+)/,
      literalColor: /^([a-zA-Z]+)/,
      rgbColor: /^rgb/i,
      rgbaColor: /^rgba/i,
      number: /^(([0-9]*\.[0-9]+)|([0-9]+\.?))/
    };
    var input = '';
    function error(msg) {
      throw new Error(input + ': ' + msg);
    }
    function getAST() {
      var ast = matchListDefinitions();
      if (input.length > 0) {
        error('Invalid input not EOF');
      }
      return ast;
    }
    function matchListDefinitions() {
      return matchListing(matchDefinition);
    }
    function matchDefinition() {
      return matchGradient('linear-gradient', tokens.linearGradient, matchLinearOrientation) || matchGradient('repeating-linear-gradient', tokens.repeatingLinearGradient, matchLinearOrientation) || matchGradient('radial-gradient', tokens.radialGradient, matchListRadialOrientations) || matchGradient('repeating-radial-gradient', tokens.repeatingRadialGradient, matchListRadialOrientations) || matchGradient('conic-gradient', tokens.conicGradient, matchListRadialOrientations);
    }
    function matchGradient(gradientType, pattern, orientationMatcher) {
      return matchCall(pattern, function (captures) {
        var orientation = orientationMatcher();
        if (orientation) {
          if (!scan(tokens.comma)) {
            error('Missing comma before color stops');
          }
        }
        return {
          type: gradientType,
          orientation: orientation,
          colorStops: matchListing(matchColorStop)
        };
      });
    }
    function matchCall(pattern, callback) {
      var captures = scan(pattern);
      if (captures) {
        if (!scan(tokens.startCall)) {
          error('Missing (');
        }
        var result = callback(captures);
        if (!scan(tokens.endCall)) {
          error('Missing )');
        }
        return result;
      }
    }
    function matchLinearOrientation() {
      return matchSideOrCorner() || matchAngle();
    }
    function matchSideOrCorner() {
      return match('directional', tokens.sideOrCorner, 1);
    }
    function matchAngle() {
      return match('angular', tokens.angleValue, 1);
    }
    function matchListRadialOrientations() {
      var radialOrientations,
        radialOrientation = matchRadialOrientation(),
        lookaheadCache;
      if (radialOrientation) {
        radialOrientations = [];
        radialOrientations.push(radialOrientation);
        lookaheadCache = input;
        if (scan(tokens.comma)) {
          radialOrientation = matchRadialOrientation();
          if (radialOrientation) {
            radialOrientations.push(radialOrientation);
          } else {
            input = lookaheadCache;
          }
        }
      }
      return radialOrientations;
    }
    function matchRadialOrientation() {
      var radialType = matchCircle() || matchEllipse();
      if (radialType) {
        // @ts-ignore
        radialType.at = matchAtPosition();
      } else {
        var extent = matchExtentKeyword();
        if (extent) {
          radialType = extent;
          var positionAt = matchAtPosition();
          if (positionAt) {
            // @ts-ignore
            radialType.at = positionAt;
          }
        } else {
          var defaultPosition = matchPositioning();
          if (defaultPosition) {
            radialType = {
              type: 'default-radial',
              // @ts-ignore
              at: defaultPosition
            };
          }
        }
      }
      return radialType;
    }
    function matchCircle() {
      var circle = match('shape', /^(circle)/i, 0);
      if (circle) {
        // @ts-ignore
        circle.style = matchLength() || matchExtentKeyword();
      }
      return circle;
    }
    function matchEllipse() {
      var ellipse = match('shape', /^(ellipse)/i, 0);
      if (ellipse) {
        // @ts-ignore
        ellipse.style = matchDistance() || matchExtentKeyword();
      }
      return ellipse;
    }
    function matchExtentKeyword() {
      return match('extent-keyword', tokens.extentKeywords, 1);
    }
    function matchAtPosition() {
      if (match('position', /^at/, 0)) {
        var positioning = matchPositioning();
        if (!positioning) {
          error('Missing positioning value');
        }
        return positioning;
      }
    }
    function matchPositioning() {
      var location = matchCoordinates();
      if (location.x || location.y) {
        return {
          type: 'position',
          value: location
        };
      }
    }
    function matchCoordinates() {
      return {
        x: matchDistance(),
        y: matchDistance()
      };
    }
    function matchListing(matcher) {
      var captures = matcher();
      var result = [];
      if (captures) {
        result.push(captures);
        while (scan(tokens.comma)) {
          captures = matcher();
          if (captures) {
            result.push(captures);
          } else {
            error('One extra comma');
          }
        }
      }
      return result;
    }
    function matchColorStop() {
      var color = matchColor();
      if (!color) {
        error('Expected color definition');
      }
      color.length = matchDistance();
      return color;
    }
    function matchColor() {
      return matchHexColor() || matchRGBAColor() || matchRGBColor() || matchLiteralColor();
    }
    function matchLiteralColor() {
      return match('literal', tokens.literalColor, 0);
    }
    function matchHexColor() {
      return match('hex', tokens.hexColor, 1);
    }
    function matchRGBColor() {
      return matchCall(tokens.rgbColor, function () {
        return {
          type: 'rgb',
          value: matchListing(matchNumber)
        };
      });
    }
    function matchRGBAColor() {
      return matchCall(tokens.rgbaColor, function () {
        return {
          type: 'rgba',
          value: matchListing(matchNumber)
        };
      });
    }
    function matchNumber() {
      return scan(tokens.number)[1];
    }
    function matchDistance() {
      return match('%', tokens.percentageValue, 1) || matchPositionKeyword() || matchLength();
    }
    function matchPositionKeyword() {
      return match('position-keyword', tokens.positionKeywords, 1);
    }
    function matchLength() {
      return match('px', tokens.pixelValue, 1) || match('em', tokens.emValue, 1);
    }
    function match(type, pattern, captureIndex) {
      var captures = scan(pattern);
      if (captures) {
        return {
          type: type,
          value: captures[captureIndex]
        };
      }
    }
    function scan(regexp) {
      var blankCaptures = /^[\n\r\t\s]+/.exec(input);
      if (blankCaptures) {
        consume(blankCaptures[0].length);
      }
      var captures = regexp.exec(input);
      if (captures) {
        consume(captures[0].length);
      }
      return captures;
    }
    function consume(size) {
      input = input.substring(size);
    }
    return function (code) {
      input = code;
      return getAST();
    };
  }();

  function getOrCalculatePathTotalLength$1(path) {
    if (path.parsedStyle.path.totalLength === 0) {
      path.parsedStyle.path.totalLength = getTotalLength$1(path.parsedStyle.path.absolutePath);
    }
    return path.parsedStyle.path.totalLength;
  }
  function hasArcOrBezier$1(path) {
    var hasArc = false;
    var count = path.length;
    for (var i = 0; i < count; i++) {
      var params = path[i];
      var cmd = params[0];
      if (cmd === 'C' || cmd === 'A' || cmd === 'Q') {
        hasArc = true;
        break;
      }
    }
    return hasArc;
  }
  function extractPolygons$1(pathArray) {
    var polygons = [];
    var polylines = [];
    var points = []; // 防止第一个命令不是 'M'
    for (var i = 0; i < pathArray.length; i++) {
      var params = pathArray[i];
      var cmd = params[0];
      if (cmd === 'M') {
        // 遇到 'M' 判定是否是新数组，新数组中没有点
        if (points.length) {
          // 如果存在点，则说明没有遇到 'Z'，开始了一个新的多边形
          polylines.push(points);
          points = []; // 创建新的点
        }

        points.push([params[1], params[2]]);
      } else if (cmd === 'Z') {
        if (points.length) {
          // 存在点
          polygons.push(points);
          points = []; // 开始新的点集合
        }
        // 如果不存在点，同时 'Z'，则说明是错误，不处理
      } else {
        points.push([params[1], params[2]]);
      }
    }
    // 说明 points 未放入 polygons 或者 polyline
    // 仅当只有一个 M，没有 Z 时会发生这种情况
    if (points.length > 0) {
      polylines.push(points);
    }
    return {
      polygons: polygons,
      polylines: polylines
    };
  }
  function isSamePoint$1(point1, point2) {
    return point1[0] === point2[0] && point1[1] === point2[1];
  }
  function getPathBBox$1(segments, lineWidth) {
    var xArr = [];
    var yArr = [];
    var segmentsWithAngle = [];
    for (var i = 0; i < segments.length; i++) {
      var segment = segments[i];
      var currentPoint = segment.currentPoint,
        params = segment.params,
        prePoint = segment.prePoint;
      var box = void 0;
      switch (segment.command) {
        case 'Q':
          box = quadratic$2.box(prePoint[0], prePoint[1], params[1], params[2], params[3], params[4]);
          break;
        case 'C':
          box = cubic$2.box(prePoint[0], prePoint[1], params[1], params[2], params[3], params[4], params[5], params[6]);
          break;
        case 'A':
          var arcParams = segment.arcParams;
          box = arc$1.box(arcParams.cx, arcParams.cy, arcParams.rx, arcParams.ry, arcParams.xRotation, arcParams.startAngle, arcParams.endAngle);
          break;
        default:
          xArr.push(currentPoint[0]);
          yArr.push(currentPoint[1]);
          break;
      }
      if (box) {
        segment.box = box;
        xArr.push(box.x, box.x + box.width);
        yArr.push(box.y, box.y + box.height);
      }
      if (lineWidth && (segment.command === 'L' || segment.command === 'M') && segment.prePoint && segment.nextPoint) {
        segmentsWithAngle.push(segment);
      }
    }
    // bbox calculation should ignore NaN for path attribute
    // ref: https://github.com/antvis/g/issues/210
    // ref: https://github.com/antvis/G2/issues/3109
    xArr = xArr.filter(function (item) {
      return !Number.isNaN(item) && item !== Infinity && item !== -Infinity;
    });
    yArr = yArr.filter(function (item) {
      return !Number.isNaN(item) && item !== Infinity && item !== -Infinity;
    });
    var minX = min$4(xArr);
    var minY = min$4(yArr);
    var maxX = max$4(xArr);
    var maxY = max$4(yArr);
    if (segmentsWithAngle.length === 0) {
      return {
        x: minX,
        y: minY,
        width: maxX - minX,
        height: maxY - minY
      };
    }
    for (var _i = 0; _i < segmentsWithAngle.length; _i++) {
      var _segment = segmentsWithAngle[_i];
      var _currentPoint = _segment.currentPoint;
      var extra = void 0;
      if (_currentPoint[0] === minX) {
        extra = getExtraFromSegmentWithAngle$1(_segment, lineWidth);
        minX = minX - extra.xExtra;
      } else if (_currentPoint[0] === maxX) {
        extra = getExtraFromSegmentWithAngle$1(_segment, lineWidth);
        maxX = maxX + extra.xExtra;
      }
      if (_currentPoint[1] === minY) {
        extra = getExtraFromSegmentWithAngle$1(_segment, lineWidth);
        minY = minY - extra.yExtra;
      } else if (_currentPoint[1] === maxY) {
        extra = getExtraFromSegmentWithAngle$1(_segment, lineWidth);
        maxY = maxY + extra.yExtra;
      }
    }
    return {
      x: minX,
      y: minY,
      width: maxX - minX,
      height: maxY - minY
    };
  }
  function getExtraFromSegmentWithAngle$1(segment, lineWidth) {
    var prePoint = segment.prePoint,
      currentPoint = segment.currentPoint,
      nextPoint = segment.nextPoint;
    var currentAndPre = Math.pow(currentPoint[0] - prePoint[0], 2) + Math.pow(currentPoint[1] - prePoint[1], 2);
    var currentAndNext = Math.pow(currentPoint[0] - nextPoint[0], 2) + Math.pow(currentPoint[1] - nextPoint[1], 2);
    var preAndNext = Math.pow(prePoint[0] - nextPoint[0], 2) + Math.pow(prePoint[1] - nextPoint[1], 2);
    // 以 currentPoint 为顶点的夹角
    var currentAngle = Math.acos((currentAndPre + currentAndNext - preAndNext) / (2 * Math.sqrt(currentAndPre) * Math.sqrt(currentAndNext)));
    // 夹角为空、 0 或 PI 时，不需要计算夹角处的额外宽度
    // 注意: 由于计算精度问题，夹角为 0 的情况计算出来的角度可能是一个很小的值，还需要判断其与 0 是否近似相等
    if (!currentAngle || Math.sin(currentAngle) === 0 || isNumberEqual$3(currentAngle, 0)) {
      return {
        xExtra: 0,
        yExtra: 0
      };
    }
    var xAngle = Math.abs(Math.atan2(nextPoint[1] - currentPoint[1], nextPoint[0] - currentPoint[0]));
    var yAngle = Math.abs(Math.atan2(nextPoint[0] - currentPoint[0], nextPoint[1] - currentPoint[1]));
    // 将夹角转为锐角
    xAngle = xAngle > Math.PI / 2 ? Math.PI - xAngle : xAngle;
    yAngle = yAngle > Math.PI / 2 ? Math.PI - yAngle : yAngle;
    // 这里不考虑在水平和垂直方向的投影，直接使用最大差值
    // 由于上层统一加减了二分之一线宽，这里需要进行弥补
    var extra = {
      // 水平方向投影
      xExtra: Math.cos(currentAngle / 2 - xAngle) * (lineWidth / 2 * (1 / Math.sin(currentAngle / 2))) - lineWidth / 2 || 0,
      // 垂直方向投影
      yExtra: Math.cos(yAngle - currentAngle / 2) * (lineWidth / 2 * (1 / Math.sin(currentAngle / 2))) - lineWidth / 2 || 0
    };
    return extra;
  }
  // 点对称
  function toSymmetry$1(point, center) {
    return [center[0] + (center[0] - point[0]), center[1] + (center[1] - point[1])];
  }
  var angleBetween$3 = function angleBetween(v0, v1) {
    var p = v0.x * v1.x + v0.y * v1.y;
    var n = Math.sqrt((Math.pow(v0.x, 2) + Math.pow(v0.y, 2)) * (Math.pow(v1.x, 2) + Math.pow(v1.y, 2)));
    var sign = v0.x * v1.y - v0.y * v1.x < 0 ? -1 : 1;
    var angle = sign * Math.acos(p / n);
    return angle;
  };
  /**
   * @see https://github.com/rveciana/svg-path-properties/blob/b6bd9a322966f6ef7a311872d80c56e3718de861/src/arc.ts#L121
   */
  var pointOnEllipticalArc$1 = function pointOnEllipticalArc(p0, rx, ry, xAxisRotation, largeArcFlag, sweepFlag, p1, t) {
    // In accordance to: http://www.w3.org/TR/SVG/implnote.html#ArcOutOfRangeParameters
    rx = Math.abs(rx);
    ry = Math.abs(ry);
    xAxisRotation = mod$1(xAxisRotation, 360);
    var xAxisRotationRadians = deg2rad$1(xAxisRotation);
    // If the endpoints are identical, then this is equivalent to omitting the elliptical arc segment entirely.
    if (p0.x === p1.x && p0.y === p1.y) {
      return {
        x: p0.x,
        y: p0.y,
        ellipticalArcAngle: 0
      }; // Check if angle is correct
    }
    // If rx = 0 or ry = 0 then this arc is treated as a straight line segment joining the endpoints.
    if (rx === 0 || ry === 0) {
      //return this.pointOnLine(p0, p1, t);
      return {
        x: 0,
        y: 0,
        ellipticalArcAngle: 0
      }; // Check if angle is correct
    }
    // Following "Conversion from endpoint to center parameterization"
    // http://www.w3.org/TR/SVG/implnote.html#ArcConversionEndpointToCenter
    // Step #1: Compute transformedPoint
    var dx = (p0.x - p1.x) / 2;
    var dy = (p0.y - p1.y) / 2;
    var transformedPoint = {
      x: Math.cos(xAxisRotationRadians) * dx + Math.sin(xAxisRotationRadians) * dy,
      y: -Math.sin(xAxisRotationRadians) * dx + Math.cos(xAxisRotationRadians) * dy
    };
    // Ensure radii are large enough
    var radiiCheck = Math.pow(transformedPoint.x, 2) / Math.pow(rx, 2) + Math.pow(transformedPoint.y, 2) / Math.pow(ry, 2);
    if (radiiCheck > 1) {
      rx = Math.sqrt(radiiCheck) * rx;
      ry = Math.sqrt(radiiCheck) * ry;
    }
    // Step #2: Compute transformedCenter
    var cSquareNumerator = Math.pow(rx, 2) * Math.pow(ry, 2) - Math.pow(rx, 2) * Math.pow(transformedPoint.y, 2) - Math.pow(ry, 2) * Math.pow(transformedPoint.x, 2);
    var cSquareRootDenom = Math.pow(rx, 2) * Math.pow(transformedPoint.y, 2) + Math.pow(ry, 2) * Math.pow(transformedPoint.x, 2);
    var cRadicand = cSquareNumerator / cSquareRootDenom;
    // Make sure this never drops below zero because of precision
    cRadicand = cRadicand < 0 ? 0 : cRadicand;
    var cCoef = (largeArcFlag !== sweepFlag ? 1 : -1) * Math.sqrt(cRadicand);
    var transformedCenter = {
      x: cCoef * (rx * transformedPoint.y / ry),
      y: cCoef * (-(ry * transformedPoint.x) / rx)
    };
    // Step #3: Compute center
    var center = {
      x: Math.cos(xAxisRotationRadians) * transformedCenter.x - Math.sin(xAxisRotationRadians) * transformedCenter.y + (p0.x + p1.x) / 2,
      y: Math.sin(xAxisRotationRadians) * transformedCenter.x + Math.cos(xAxisRotationRadians) * transformedCenter.y + (p0.y + p1.y) / 2
    };
    // Step #4: Compute start/sweep angles
    // Start angle of the elliptical arc prior to the stretch and rotate operations.
    // Difference between the start and end angles
    var startVector = {
      x: (transformedPoint.x - transformedCenter.x) / rx,
      y: (transformedPoint.y - transformedCenter.y) / ry
    };
    var startAngle = angleBetween$3({
      x: 1,
      y: 0
    }, startVector);
    var endVector = {
      x: (-transformedPoint.x - transformedCenter.x) / rx,
      y: (-transformedPoint.y - transformedCenter.y) / ry
    };
    var sweepAngle = angleBetween$3(startVector, endVector);
    if (!sweepFlag && sweepAngle > 0) {
      sweepAngle -= 2 * Math.PI;
    } else if (sweepFlag && sweepAngle < 0) {
      sweepAngle += 2 * Math.PI;
    }
    // We use % instead of `mod(..)` because we want it to be -360deg to 360deg(but actually in radians)
    sweepAngle %= 2 * Math.PI;
    // From http://www.w3.org/TR/SVG/implnote.html#ArcParameterizationAlternatives
    var angle = startAngle + sweepAngle * t;
    var ellipseComponentX = rx * Math.cos(angle);
    var ellipseComponentY = ry * Math.sin(angle);
    var point = {
      x: Math.cos(xAxisRotationRadians) * ellipseComponentX - Math.sin(xAxisRotationRadians) * ellipseComponentY + center.x,
      y: Math.sin(xAxisRotationRadians) * ellipseComponentX + Math.cos(xAxisRotationRadians) * ellipseComponentY + center.y,
      ellipticalArcStartAngle: startAngle,
      ellipticalArcEndAngle: startAngle + sweepAngle,
      ellipticalArcAngle: angle,
      ellipticalArcCenter: center,
      resultantRx: rx,
      resultantRy: ry
    };
    return point;
  };
  function path2Segments$1(path) {
    var segments = [];
    var currentPoint = null; // 当前图形
    var nextParams = null; // 下一节点的 path 参数
    var startMovePoint = null; // 开始 M 的点，可能会有多个
    var lastStartMovePointIndex = 0; // 最近一个开始点 M 的索引
    var count = path.length;
    for (var i = 0; i < count; i++) {
      var params = path[i];
      nextParams = path[i + 1];
      var command = params[0];
      // 数学定义上的参数，便于后面的计算
      var segment = {
        command: command,
        prePoint: currentPoint,
        params: params,
        startTangent: null,
        endTangent: null,
        currentPoint: null,
        nextPoint: null,
        arcParams: null,
        box: null,
        cubicParams: null
      };
      switch (command) {
        case 'M':
          startMovePoint = [params[1], params[2]];
          lastStartMovePointIndex = i;
          break;
        case 'A':
          var arcParams = getArcParams$1(currentPoint, params);
          segment.arcParams = arcParams;
          break;
      }
      if (command === 'Z') {
        // 有了 Z 后，当前节点从开始 M 的点开始
        currentPoint = startMovePoint;
        // 如果当前点的命令为 Z，相当于当前点为最近一个 M 点，则下一个点直接指向最近一个 M 点的下一个点
        nextParams = path[lastStartMovePointIndex + 1];
      } else {
        var len = params.length;
        currentPoint = [params[len - 2], params[len - 1]];
      }
      if (nextParams && nextParams[0] === 'Z') {
        // 如果下一个点的命令为 Z，则下一个点直接指向最近一个 M 点
        nextParams = path[lastStartMovePointIndex];
        if (segments[lastStartMovePointIndex]) {
          // 如果下一个点的命令为 Z，则最近一个 M 点的前一个点为当前点
          segments[lastStartMovePointIndex].prePoint = currentPoint;
        }
      }
      segment.currentPoint = currentPoint;
      // 如果当前点与最近一个 M 点相同，则最近一个 M 点的前一个点为当前点的前一个点
      if (segments[lastStartMovePointIndex] && isSamePoint$1(currentPoint, segments[lastStartMovePointIndex].currentPoint)) {
        segments[lastStartMovePointIndex].prePoint = segment.prePoint;
      }
      var nextPoint = nextParams ? [nextParams[nextParams.length - 2], nextParams[nextParams.length - 1]] : null;
      segment.nextPoint = nextPoint;
      // Add startTangent and endTangent
      var prePoint = segment.prePoint;
      if (['L', 'H', 'V'].includes(command)) {
        segment.startTangent = [prePoint[0] - currentPoint[0], prePoint[1] - currentPoint[1]];
        segment.endTangent = [currentPoint[0] - prePoint[0], currentPoint[1] - prePoint[1]];
      } else if (command === 'Q') {
        // 二次贝塞尔曲线只有一个控制点
        var cp = [params[1], params[2]];
        // 二次贝塞尔曲线的终点为 currentPoint
        segment.startTangent = [prePoint[0] - cp[0], prePoint[1] - cp[1]];
        segment.endTangent = [currentPoint[0] - cp[0], currentPoint[1] - cp[1]];
      } else if (command === 'T') {
        var preSegment = segments[i - 1];
        var _cp = toSymmetry$1(preSegment.currentPoint, prePoint);
        if (preSegment.command === 'Q') {
          segment.command = 'Q';
          segment.startTangent = [prePoint[0] - _cp[0], prePoint[1] - _cp[1]];
          segment.endTangent = [currentPoint[0] - _cp[0], currentPoint[1] - _cp[1]];
        } else {
          // @ts-ignore
          segment.command = 'TL';
          segment.startTangent = [prePoint[0] - currentPoint[0], prePoint[1] - currentPoint[1]];
          segment.endTangent = [currentPoint[0] - prePoint[0], currentPoint[1] - prePoint[1]];
        }
      } else if (command === 'C') {
        // 三次贝塞尔曲线有两个控制点
        var cp1 = [params[1], params[2]];
        var cp2 = [params[3], params[4]];
        segment.startTangent = [prePoint[0] - cp1[0], prePoint[1] - cp1[1]];
        segment.endTangent = [currentPoint[0] - cp2[0], currentPoint[1] - cp2[1]];
        // horizontal line, eg. ['C', 100, 100, 100, 100, 200, 200]
        if (segment.startTangent[0] === 0 && segment.startTangent[1] === 0) {
          segment.startTangent = [cp1[0] - cp2[0], cp1[1] - cp2[1]];
        }
        if (segment.endTangent[0] === 0 && segment.endTangent[1] === 0) {
          segment.endTangent = [cp2[0] - cp1[0], cp2[1] - cp1[1]];
        }
      } else if (command === 'S') {
        var _preSegment = segments[i - 1];
        var _cp2 = toSymmetry$1(_preSegment.currentPoint, prePoint);
        var _cp3 = [params[1], params[2]];
        if (_preSegment.command === 'C') {
          segment.command = 'C'; // 将 S 命令变换为 C 命令
          segment.startTangent = [prePoint[0] - _cp2[0], prePoint[1] - _cp2[1]];
          segment.endTangent = [currentPoint[0] - _cp3[0], currentPoint[1] - _cp3[1]];
        } else {
          // @ts-ignore
          segment.command = 'SQ'; // 将 S 命令变换为 SQ 命令
          segment.startTangent = [prePoint[0] - _cp3[0], prePoint[1] - _cp3[1]];
          segment.endTangent = [currentPoint[0] - _cp3[0], currentPoint[1] - _cp3[1]];
        }
      } else if (command === 'A') {
        var _getTangentAtRatio = getTangentAtRatio$1(segment, 0),
          dx1 = _getTangentAtRatio.x,
          dy1 = _getTangentAtRatio.y;
        var _getTangentAtRatio2 = getTangentAtRatio$1(segment, 1, false),
          dx2 = _getTangentAtRatio2.x,
          dy2 = _getTangentAtRatio2.y;
        segment.startTangent = [dx1, dy1];
        segment.endTangent = [dx2, dy2];
      }
      segments.push(segment);
    }
    return segments;
  }
  /**
   * Use length instead of ratio
   */
  function getTangentAtRatio$1(segment, ratio, sign) {
    if (sign === void 0) {
      sign = true;
    }
    var _segment$arcParams = segment.arcParams,
      _segment$arcParams$rx = _segment$arcParams.rx,
      rx = _segment$arcParams$rx === void 0 ? 0 : _segment$arcParams$rx,
      _segment$arcParams$ry = _segment$arcParams.ry,
      ry = _segment$arcParams$ry === void 0 ? 0 : _segment$arcParams$ry,
      xRotation = _segment$arcParams.xRotation,
      arcFlag = _segment$arcParams.arcFlag,
      sweepFlag = _segment$arcParams.sweepFlag;
    var p1 = pointOnEllipticalArc$1({
      x: segment.prePoint[0],
      y: segment.prePoint[1]
    }, rx, ry, xRotation, !!arcFlag, !!sweepFlag, {
      x: segment.currentPoint[0],
      y: segment.currentPoint[1]
    }, ratio);
    var p2 = pointOnEllipticalArc$1({
      x: segment.prePoint[0],
      y: segment.prePoint[1]
    }, rx, ry, xRotation, !!arcFlag, !!sweepFlag, {
      x: segment.currentPoint[0],
      y: segment.currentPoint[1]
    }, sign ? ratio + 0.005 : ratio - 0.005);
    var xDist = p2.x - p1.x;
    var yDist = p2.y - p1.y;
    var dist = Math.sqrt(xDist * xDist + yDist * yDist);
    return {
      x: -xDist / dist,
      y: -yDist / dist
    };
  }
  // 向量长度
  function vMag$1(v) {
    return Math.sqrt(v[0] * v[0] + v[1] * v[1]);
  }
  // u.v/|u||v|，计算夹角的余弦值
  function vRatio$1(u, v) {
    // 当存在一个向量的长度为 0 时，夹角也为 0，即夹角的余弦值为 1
    return vMag$1(u) * vMag$1(v) ? (u[0] * v[0] + u[1] * v[1]) / (vMag$1(u) * vMag$1(v)) : 1;
  }
  // 向量角度
  function vAngle$1(u, v) {
    return (u[0] * v[1] < u[1] * v[0] ? -1 : 1) * Math.acos(vRatio$1(u, v));
  }
  function getArcParams$1(startPoint, params) {
    var rx = params[1];
    var ry = params[2];
    var xRotation = mod$1(deg2rad$1(params[3]), Math.PI * 2);
    var arcFlag = params[4];
    var sweepFlag = params[5];
    // 弧形起点坐标
    var x1 = startPoint[0];
    var y1 = startPoint[1];
    // 弧形终点坐标
    var x2 = params[6];
    var y2 = params[7];
    var xp = Math.cos(xRotation) * (x1 - x2) / 2.0 + Math.sin(xRotation) * (y1 - y2) / 2.0;
    var yp = -1 * Math.sin(xRotation) * (x1 - x2) / 2.0 + Math.cos(xRotation) * (y1 - y2) / 2.0;
    var lambda = xp * xp / (rx * rx) + yp * yp / (ry * ry);
    if (lambda > 1) {
      rx *= Math.sqrt(lambda);
      ry *= Math.sqrt(lambda);
    }
    var diff = rx * rx * (yp * yp) + ry * ry * (xp * xp);
    var f = diff ? Math.sqrt((rx * rx * (ry * ry) - diff) / diff) : 1;
    if (arcFlag === sweepFlag) {
      f *= -1;
    }
    if (isNaN(f)) {
      f = 0;
    }
    // 旋转前的起点坐标，且当长半轴和短半轴的长度为 0 时，坐标按 (0, 0) 处理
    var cxp = ry ? f * rx * yp / ry : 0;
    var cyp = rx ? f * -ry * xp / rx : 0;
    // 椭圆圆心坐标
    var cx = (x1 + x2) / 2.0 + Math.cos(xRotation) * cxp - Math.sin(xRotation) * cyp;
    var cy = (y1 + y2) / 2.0 + Math.sin(xRotation) * cxp + Math.cos(xRotation) * cyp;
    // 起始点的单位向量
    var u = [(xp - cxp) / rx, (yp - cyp) / ry];
    // 终止点的单位向量
    var v = [(-1 * xp - cxp) / rx, (-1 * yp - cyp) / ry];
    // 计算起始点和圆心的连线，与 x 轴正方向的夹角
    var theta = vAngle$1([1, 0], u);
    // 计算圆弧起始点和终止点与椭圆圆心连线的夹角
    var dTheta = vAngle$1(u, v);
    if (vRatio$1(u, v) <= -1) {
      dTheta = Math.PI;
    }
    if (vRatio$1(u, v) >= 1) {
      dTheta = 0;
    }
    if (sweepFlag === 0 && dTheta > 0) {
      dTheta = dTheta - 2 * Math.PI;
    }
    if (sweepFlag === 1 && dTheta < 0) {
      dTheta = dTheta + 2 * Math.PI;
    }
    return {
      cx: cx,
      cy: cy,
      // 弧形的起点和终点相同时，长轴和短轴的长度按 0 处理
      rx: isSamePoint$1(startPoint, [x2, y2]) ? 0 : rx,
      ry: isSamePoint$1(startPoint, [x2, y2]) ? 0 : ry,
      startAngle: theta,
      endAngle: theta + dTheta,
      xRotation: xRotation,
      arcFlag: arcFlag,
      sweepFlag: sweepFlag
    };
  }

  // @ts-nocheck
  // interface HookInterceptor<T, R, AdditionalOptions = UnsetAdditionalOptions> {
  //   name?: string;
  //   tap?: (tap: FullTap & IfSet<AdditionalOptions>) => void;
  //   call?: (...args: any[]) => void;
  //   loop?: (...args: any[]) => void;
  //   error?: (err: Error) => void;
  //   result?: (result: R) => void;
  //   done?: () => void;
  //   register?: (tap: FullTap & IfSet<AdditionalOptions>) => FullTap & IfSet<AdditionalOptions>;
  // }
  // type ArgumentNames<T extends any[]> = FixedSizeArray<T['length'], string>;
  // export declare class AsyncHook<T, R, AdditionalOptions = UnsetAdditionalOptions> extends Hook<
  //   T,
  //   R,
  //   AdditionalOptions
  // > {
  //   tapAsync(
  //     options: string | (Tap & IfSet<AdditionalOptions>),
  //     fn: (...args: Append<AsArray<T>, InnerCallback<Error, R>>) => void,
  //   ): void;
  //   tapPromise(
  //     options: string | (Tap & IfSet<AdditionalOptions>),
  //     fn: (...args: AsArray<T>) => Promise<R>,
  //   ): void;
  // }
  var CALL_DELEGATE$1 = function CALL_DELEGATE() {
    this.call = this._createCall('sync');
    return this.call.apply(this, arguments);
  };
  var CALL_ASYNC_DELEGATE$1 = function CALL_ASYNC_DELEGATE() {
    this.callAsync = this._createCall('async');
    return this.callAsync.apply(this, arguments);
  };
  var PROMISE_DELEGATE$1 = function PROMISE_DELEGATE() {
    this.promise = this._createCall('promise');
    return this.promise.apply(this, arguments);
  };
  var Hook$1 = /*#__PURE__*/function () {
    function Hook(args, name) {
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      this.name = void 0;
      this.taps = void 0;
      this.promise = void 0;
      this._promise = void 0;
      this._args = args;
      this.name = name;
      this.taps = [];
      this.interceptors = [];
      this._call = CALL_DELEGATE$1;
      this.call = CALL_DELEGATE$1;
      this._callAsync = CALL_ASYNC_DELEGATE$1;
      this.callAsync = CALL_ASYNC_DELEGATE$1;
      this._promise = PROMISE_DELEGATE$1;
      this.promise = PROMISE_DELEGATE$1;
      this._x = undefined;
      // this.compile = this.compile;
      // this.tap = this.tap;
      // this.tapAsync = this.tapAsync;
      // this.tapPromise = this.tapPromise;
    }
    var _proto = Hook.prototype;
    _proto.compile = function compile(options) {
      throw new Error('Abstract: should be overridden');
    };
    _proto._createCall = function _createCall(type) {
      return this.compile({
        taps: this.taps,
        interceptors: this.interceptors,
        args: this._args,
        type: type
      });
    };
    _proto._tap = function _tap(type, options, fn) {
      if (typeof options === 'string') {
        options = {
          name: options.trim()
        };
      } else if (typeof options !== 'object' || options === null) {
        throw new Error('Invalid tap options');
      }
      if (typeof options.name !== 'string' || options.name === '') {
        throw new Error('Missing name for tap');
      }
      // if (typeof options.context !== "undefined") {
      // 	deprecateContext();
      // }
      options = Object.assign({
        type: type,
        fn: fn
      }, options);
      options = this._runRegisterInterceptors(options);
      this._insert(options);
    };
    _proto.tap = function tap(options, fn) {
      this._tap('sync', options, fn);
    };
    _proto.tapAsync = function tapAsync(options, fn) {
      this._tap('async', options, fn);
    };
    _proto.tapPromise = function tapPromise(options, fn) {
      this._tap('promise', options, fn);
    };
    _proto._runRegisterInterceptors = function _runRegisterInterceptors(options) {
      for (var _iterator = _createForOfIteratorHelperLoose$2(this.interceptors), _step; !(_step = _iterator()).done;) {
        var interceptor = _step.value;
        if (interceptor.register) {
          var newOptions = interceptor.register(options);
          if (newOptions !== undefined) {
            options = newOptions;
          }
        }
      }
      return options;
    };
    _proto.withOptions = function withOptions(options) {
      var _this = this;
      var mergeOptions = function mergeOptions(opt) {
        return Object.assign({}, options, typeof opt === 'string' ? {
          name: opt
        } : opt);
      };
      return {
        name: this.name,
        tap: function tap(opt, fn) {
          return _this.tap(mergeOptions(opt), fn);
        },
        tapAsync: function tapAsync(opt, fn) {
          return _this.tapAsync(mergeOptions(opt), fn);
        },
        tapPromise: function tapPromise(opt, fn) {
          return _this.tapPromise(mergeOptions(opt), fn);
        },
        // intercept: (interceptor) => this.intercept(interceptor),
        isUsed: function isUsed() {
          return _this.isUsed();
        },
        withOptions: function withOptions(opt) {
          return _this.withOptions(mergeOptions(opt));
        }
      };
    };
    _proto.isUsed = function isUsed() {
      return this.taps.length > 0 || this.interceptors.length > 0;
    }
    // intercept(interceptor: HookInterceptor<T, R, AdditionalOptions>) {
    //   this._resetCompilation();
    //   this.interceptors.push(Object.assign({}, interceptor));
    //   if (interceptor.register) {
    //     for (let i = 0; i < this.taps.length; i++) {
    //       this.taps[i] = interceptor.register(this.taps[i]);
    //     }
    //   }
    // }
    ;
    _proto._resetCompilation = function _resetCompilation() {
      this.call = this._call;
      this.callAsync = this._callAsync;
      this.promise = this._promise;
    };
    _proto._insert = function _insert(item) {
      this._resetCompilation();
      var before;
      if (typeof item.before === 'string') {
        before = new Set([item.before]);
      } else if (Array.isArray(item.before)) {
        before = new Set(item.before);
      }
      var stage = 0;
      if (typeof item.stage === 'number') {
        stage = item.stage;
      }
      var i = this.taps.length;
      while (i > 0) {
        i--;
        var x = this.taps[i];
        this.taps[i + 1] = x;
        var xStage = x.stage || 0;
        if (before) {
          if (before.has(x.name)) {
            before.delete(x.name);
            continue;
          }
          if (before.size > 0) {
            continue;
          }
        }
        if (xStage > stage) {
          continue;
        }
        i++;
        break;
      }
      this.taps[i] = item;
    };
    return Hook;
  }();
  // Object.setPrototypeOf(Hook.prototype, null);

  // @ts-nocheck
  var HookCodeFactory$1 = /*#__PURE__*/function () {
    function HookCodeFactory(config) {
      this.config = config;
      this.options = undefined;
      this._args = undefined;
    }
    var _proto = HookCodeFactory.prototype;
    _proto.create = function create(options) {
      this.init(options);
      var fn;
      switch (this.options.type) {
        case 'sync':
          fn = new Function(this.args(), '"use strict";\n' + this.header() + this.contentWithInterceptors({
            onError: function onError(err) {
              return "throw " + err + ";\n";
            },
            onResult: function onResult(result) {
              return "return " + result + ";\n";
            },
            resultReturns: true,
            onDone: function onDone() {
              return '';
            },
            rethrowIfPossible: true
          }));
          break;
        case 'async':
          fn = new Function(this.args({
            after: '_callback'
          }), '"use strict";\n' + this.header() + this.contentWithInterceptors({
            onError: function onError(err) {
              return "_callback(" + err + ");\n";
            },
            onResult: function onResult(result) {
              return "_callback(null, " + result + ");\n";
            },
            onDone: function onDone() {
              return '_callback();\n';
            }
          }));
          break;
        case 'promise':
          var errorHelperUsed = false;
          var content = this.contentWithInterceptors({
            onError: function onError(err) {
              errorHelperUsed = true;
              return "_error(" + err + ");\n";
            },
            onResult: function onResult(result) {
              return "_resolve(" + result + ");\n";
            },
            onDone: function onDone() {
              return '_resolve();\n';
            }
          });
          var code = '';
          code += '"use strict";\n';
          code += this.header();
          code += 'return new Promise((function(_resolve, _reject) {\n';
          if (errorHelperUsed) {
            code += 'var _sync = true;\n';
            code += 'function _error(_err) {\n';
            code += 'if(_sync)\n';
            code += '_resolve(Promise.resolve().then((function() { throw _err; })));\n';
            code += 'else\n';
            code += '_reject(_err);\n';
            code += '};\n';
          }
          code += content;
          if (errorHelperUsed) {
            code += '_sync = false;\n';
          }
          code += '}));\n';
          fn = new Function(this.args(), code);
          break;
      }
      this.deinit();
      return fn;
    };
    _proto.setup = function setup(instance, options) {
      instance._x = options.taps.map(function (t) {
        return t.fn;
      });
    }
    /**
     * @param {{ type: "sync" | "promise" | "async", taps: Array<Tap>, interceptors: Array<Interceptor> }} options
     */;
    _proto.init = function init(options) {
      this.options = options;
      this._args = options.args.slice();
    };
    _proto.deinit = function deinit() {
      this.options = undefined;
      this._args = undefined;
    };
    _proto.contentWithInterceptors = function contentWithInterceptors(options) {
      var _this = this;
      if (this.options.interceptors.length > 0) {
        var onError = options.onError;
        var onResult = options.onResult;
        var onDone = options.onDone;
        var code = '';
        for (var i = 0; i < this.options.interceptors.length; i++) {
          var interceptor = this.options.interceptors[i];
          if (interceptor.call) {
            code += this.getInterceptor(i) + ".call(" + this.args({
              before: interceptor.context ? '_context' : undefined
            }) + ");\n";
          }
        }
        code += this.content(Object.assign(options, {
          onError: onError && function (err) {
            var code = '';
            for (var _i = 0; _i < _this.options.interceptors.length; _i++) {
              var _interceptor = _this.options.interceptors[_i];
              if (_interceptor.error) {
                code += _this.getInterceptor(_i) + ".error(" + err + ");\n";
              }
            }
            code += onError(err);
            return code;
          },
          onResult: onResult && function (result) {
            var code = '';
            for (var _i2 = 0; _i2 < _this.options.interceptors.length; _i2++) {
              var _interceptor2 = _this.options.interceptors[_i2];
              if (_interceptor2.result) {
                code += _this.getInterceptor(_i2) + ".result(" + result + ");\n";
              }
            }
            code += onResult(result);
            return code;
          },
          onDone: onDone && function () {
            var code = '';
            for (var _i3 = 0; _i3 < _this.options.interceptors.length; _i3++) {
              var _interceptor3 = _this.options.interceptors[_i3];
              if (_interceptor3.done) {
                code += _this.getInterceptor(_i3) + ".done();\n";
              }
            }
            code += onDone();
            return code;
          }
        }));
        return code;
      } else {
        return this.content(options);
      }
    };
    _proto.header = function header() {
      var code = '';
      if (this.needContext()) {
        code += 'var _context = {};\n';
      } else {
        code += 'var _context;\n';
      }
      code += 'var _x = this._x;\n';
      if (this.options.interceptors.length > 0) {
        code += 'var _taps = this.taps;\n';
        code += 'var _interceptors = this.interceptors;\n';
      }
      return code;
    };
    _proto.needContext = function needContext() {
      for (var _iterator = _createForOfIteratorHelperLoose$2(this.options.taps), _step; !(_step = _iterator()).done;) {
        var tap = _step.value;
        if (tap.context) return true;
      }
      return false;
    };
    _proto.callTap = function callTap(tapIndex, _ref) {
      var onError = _ref.onError,
        onResult = _ref.onResult,
        onDone = _ref.onDone,
        rethrowIfPossible = _ref.rethrowIfPossible;
      var code = '';
      var hasTapCached = false;
      for (var i = 0; i < this.options.interceptors.length; i++) {
        var interceptor = this.options.interceptors[i];
        if (interceptor.tap) {
          if (!hasTapCached) {
            code += "var _tap" + tapIndex + " = " + this.getTap(tapIndex) + ";\n";
            hasTapCached = true;
          }
          code += this.getInterceptor(i) + ".tap(" + (interceptor.context ? '_context, ' : '') + "_tap" + tapIndex + ");\n";
        }
      }
      code += "var _fn" + tapIndex + " = " + this.getTapFn(tapIndex) + ";\n";
      var tap = this.options.taps[tapIndex];
      switch (tap.type) {
        case 'sync':
          if (!rethrowIfPossible) {
            code += "var _hasError" + tapIndex + " = false;\n";
            code += 'try {\n';
          }
          if (onResult) {
            code += "var _result" + tapIndex + " = _fn" + tapIndex + "(" + this.args({
              before: tap.context ? '_context' : undefined
            }) + ");\n";
          } else {
            code += "_fn" + tapIndex + "(" + this.args({
              before: tap.context ? '_context' : undefined
            }) + ");\n";
          }
          if (!rethrowIfPossible) {
            code += '} catch(_err) {\n';
            code += "_hasError" + tapIndex + " = true;\n";
            code += onError('_err');
            code += '}\n';
            code += "if(!_hasError" + tapIndex + ") {\n";
          }
          if (onResult) {
            code += onResult("_result" + tapIndex);
          }
          if (onDone) {
            code += onDone();
          }
          if (!rethrowIfPossible) {
            code += '}\n';
          }
          break;
        case 'async':
          var cbCode = '';
          if (onResult) cbCode += "(function(_err" + tapIndex + ", _result" + tapIndex + ") {\n";else cbCode += "(function(_err" + tapIndex + ") {\n";
          cbCode += "if(_err" + tapIndex + ") {\n";
          cbCode += onError("_err" + tapIndex);
          cbCode += '} else {\n';
          if (onResult) {
            cbCode += onResult("_result" + tapIndex);
          }
          if (onDone) {
            cbCode += onDone();
          }
          cbCode += '}\n';
          cbCode += '})';
          code += "_fn" + tapIndex + "(" + this.args({
            before: tap.context ? '_context' : undefined,
            after: cbCode
          }) + ");\n";
          break;
        case 'promise':
          code += "var _hasResult" + tapIndex + " = false;\n";
          code += "var _promise" + tapIndex + " = _fn" + tapIndex + "(" + this.args({
            before: tap.context ? '_context' : undefined
          }) + ");\n";
          code += "if (!_promise" + tapIndex + " || !_promise" + tapIndex + ".then)\n";
          code += "  throw new Error('Tap function (tapPromise) did not return promise (returned ' + _promise" + tapIndex + " + ')');\n";
          code += "_promise" + tapIndex + ".then((function(_result" + tapIndex + ") {\n";
          code += "_hasResult" + tapIndex + " = true;\n";
          if (onResult) {
            code += onResult("_result" + tapIndex);
          }
          if (onDone) {
            code += onDone();
          }
          code += "}), function(_err" + tapIndex + ") {\n";
          code += "if(_hasResult" + tapIndex + ") throw _err" + tapIndex + ";\n";
          code += onError("_err" + tapIndex);
          code += '});\n';
          break;
      }
      return code;
    };
    _proto.callTapsSeries = function callTapsSeries(_ref2) {
      var _this2 = this;
      var _onError = _ref2.onError,
        onResult = _ref2.onResult,
        resultReturns = _ref2.resultReturns,
        onDone = _ref2.onDone,
        doneReturns = _ref2.doneReturns,
        rethrowIfPossible = _ref2.rethrowIfPossible;
      if (this.options.taps.length === 0) return onDone();
      var firstAsync = this.options.taps.findIndex(function (t) {
        return t.type !== 'sync';
      });
      var somethingReturns = resultReturns || doneReturns;
      var code = '';
      var current = onDone;
      var unrollCounter = 0;
      var _loop = function _loop(j) {
        var i = j;
        var unroll = current !== onDone && (_this2.options.taps[i].type !== 'sync' || unrollCounter++ > 20);
        if (unroll) {
          unrollCounter = 0;
          code += "function _next" + i + "() {\n";
          code += current();
          code += "}\n";
          current = function current() {
            return (somethingReturns ? 'return ' : '') + "_next" + i + "();\n";
          };
        }
        var done = current;
        var doneBreak = function doneBreak(skipDone) {
          if (skipDone) return '';
          return onDone();
        };
        var content = _this2.callTap(i, {
          onError: function onError(error) {
            return _onError(i, error, done, doneBreak);
          },
          onResult: onResult && function (result) {
            return onResult(i, result, done, doneBreak);
          },
          onDone: !onResult && done,
          rethrowIfPossible: rethrowIfPossible && (firstAsync < 0 || i < firstAsync)
        });
        current = function current() {
          return content;
        };
      };
      for (var j = this.options.taps.length - 1; j >= 0; j--) {
        _loop(j);
      }
      code += current();
      return code;
    };
    _proto.callTapsLooping = function callTapsLooping(_ref3) {
      var onError = _ref3.onError,
        onDone = _ref3.onDone,
        rethrowIfPossible = _ref3.rethrowIfPossible;
      if (this.options.taps.length === 0) return onDone();
      var syncOnly = this.options.taps.every(function (t) {
        return t.type === 'sync';
      });
      var code = '';
      if (!syncOnly) {
        code += 'var _looper = (function() {\n';
        code += 'var _loopAsync = false;\n';
      }
      code += 'var _loop;\n';
      code += 'do {\n';
      code += '_loop = false;\n';
      for (var i = 0; i < this.options.interceptors.length; i++) {
        var interceptor = this.options.interceptors[i];
        if (interceptor.loop) {
          code += this.getInterceptor(i) + ".loop(" + this.args({
            before: interceptor.context ? '_context' : undefined
          }) + ");\n";
        }
      }
      code += this.callTapsSeries({
        onError: onError,
        onResult: function onResult(i, result, next, doneBreak) {
          var code = '';
          code += "if(" + result + " !== undefined) {\n";
          code += '_loop = true;\n';
          if (!syncOnly) code += 'if(_loopAsync) _looper();\n';
          code += doneBreak(true);
          code += "} else {\n";
          code += next();
          code += "}\n";
          return code;
        },
        onDone: onDone && function () {
          var code = '';
          code += 'if(!_loop) {\n';
          code += onDone();
          code += '}\n';
          return code;
        },
        rethrowIfPossible: rethrowIfPossible && syncOnly
      });
      code += '} while(_loop);\n';
      if (!syncOnly) {
        code += '_loopAsync = true;\n';
        code += '});\n';
        code += '_looper();\n';
      }
      return code;
    };
    _proto.callTapsParallel = function callTapsParallel(_ref4) {
      var _this3 = this;
      var _onError2 = _ref4.onError,
        onResult = _ref4.onResult,
        onDone = _ref4.onDone,
        rethrowIfPossible = _ref4.rethrowIfPossible,
        _ref4$onTap = _ref4.onTap,
        onTap = _ref4$onTap === void 0 ? function (i, run) {
          return run();
        } : _ref4$onTap;
      if (this.options.taps.length <= 1) {
        return this.callTapsSeries({
          onError: _onError2,
          onResult: onResult,
          onDone: onDone,
          rethrowIfPossible: rethrowIfPossible
        });
      }
      var code = '';
      code += 'do {\n';
      code += "var _counter = " + this.options.taps.length + ";\n";
      if (onDone) {
        code += 'var _done = (function() {\n';
        code += onDone();
        code += '});\n';
      }
      var _loop2 = function _loop2(i) {
        var done = function done() {
          if (onDone) return 'if(--_counter === 0) _done();\n';else return '--_counter;';
        };
        var doneBreak = function doneBreak(skipDone) {
          if (skipDone || !onDone) return '_counter = 0;\n';else return '_counter = 0;\n_done();\n';
        };
        code += 'if(_counter <= 0) break;\n';
        code += onTap(i, function () {
          return _this3.callTap(i, {
            onError: function onError(error) {
              var code = '';
              code += 'if(_counter > 0) {\n';
              code += _onError2(i, error, done, doneBreak);
              code += '}\n';
              return code;
            },
            onResult: onResult && function (result) {
              var code = '';
              code += 'if(_counter > 0) {\n';
              code += onResult(i, result, done, doneBreak);
              code += '}\n';
              return code;
            },
            onDone: !onResult && function () {
              return done();
            },
            rethrowIfPossible: rethrowIfPossible
          });
        }, done, doneBreak);
      };
      for (var i = 0; i < this.options.taps.length; i++) {
        _loop2(i);
      }
      code += '} while(false);\n';
      return code;
    };
    _proto.args = function args(_temp) {
      var _ref5 = _temp === void 0 ? {} : _temp,
        before = _ref5.before,
        after = _ref5.after;
      var allArgs = this._args;
      if (before) allArgs = [before].concat(allArgs);
      if (after) allArgs = allArgs.concat(after);
      if (allArgs.length === 0) {
        return '';
      } else {
        return allArgs.join(', ');
      }
    };
    _proto.getTapFn = function getTapFn(idx) {
      return "_x[" + idx + "]";
    };
    _proto.getTap = function getTap(idx) {
      return "_taps[" + idx + "]";
    };
    _proto.getInterceptor = function getInterceptor(idx) {
      return "_interceptors[" + idx + "]";
    };
    return HookCodeFactory;
  }();

  var AsyncParallelHookCodeFactory$1 = /*#__PURE__*/function (_HookCodeFactory) {
    _inheritsLoose$9(AsyncParallelHookCodeFactory, _HookCodeFactory);
    function AsyncParallelHookCodeFactory() {
      return _HookCodeFactory.apply(this, arguments) || this;
    }
    var _proto = AsyncParallelHookCodeFactory.prototype;
    _proto.content = function content(_ref) {
      var _onError = _ref.onError,
        onDone = _ref.onDone;
      return this.callTapsParallel({
        onError: function onError(i, err, done, doneBreak) {
          return _onError(err) + doneBreak(true);
        },
        onDone: onDone
      });
    };
    return AsyncParallelHookCodeFactory;
  }(HookCodeFactory$1);
  var factory$4 = new AsyncParallelHookCodeFactory$1();
  var COMPILE$4 = function COMPILE(options) {
    factory$4.setup(this, options);
    return factory$4.create(options);
  };
  var AsyncParallelHook$1 = /*#__PURE__*/function (_Hook) {
    _inheritsLoose$9(AsyncParallelHook, _Hook);
    function AsyncParallelHook(args, name) {
      var _this;
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      _this = _Hook.call(this, args, name) || this;
      _this.compile = COMPILE$4;
      _this._call = undefined;
      _this.call = undefined;
      return _this;
    }
    return AsyncParallelHook;
  }(Hook$1);
  // export function AsyncParallelHook(args = [], name = undefined) {
  //   const hook = new Hook(args, name);
  //   hook.constructor = AsyncParallelHook;
  //   hook.compile = COMPILE;
  //   hook._call = undefined;
  //   hook.call = undefined;
  //   return hook;
  // }
  // AsyncParallelHook.prototype = null;

  var AsyncSeriesWaterfallHookCodeFactory$1 = /*#__PURE__*/function (_HookCodeFactory) {
    _inheritsLoose$9(AsyncSeriesWaterfallHookCodeFactory, _HookCodeFactory);
    function AsyncSeriesWaterfallHookCodeFactory() {
      return _HookCodeFactory.apply(this, arguments) || this;
    }
    var _proto = AsyncSeriesWaterfallHookCodeFactory.prototype;
    _proto.content = function content(_ref) {
      var _this = this;
      var _onError = _ref.onError,
        onResult = _ref.onResult,
        onDone = _ref.onDone;
      return this.callTapsSeries({
        onError: function onError(i, err, next, doneBreak) {
          return _onError(err) + doneBreak(true);
        },
        onResult: function onResult(i, result, next) {
          var code = '';
          code += "if(" + result + " !== undefined) {\n";
          code += _this._args[0] + " = " + result + ";\n";
          code += "}\n";
          code += next();
          return code;
        },
        onDone: function onDone() {
          return onResult(_this._args[0]);
        }
      });
    };
    return AsyncSeriesWaterfallHookCodeFactory;
  }(HookCodeFactory$1);
  var factory$1$1 = new AsyncSeriesWaterfallHookCodeFactory$1();
  var COMPILE$1$1 = function COMPILE(options) {
    factory$1$1.setup(this, options);
    return factory$1$1.create(options);
  };
  // export function AsyncSeriesWaterfallHook(args = [], name = undefined) {
  //   if (args.length < 1) throw new Error('Waterfall hooks must have at least one argument');
  //   const hook = new Hook(args, name);
  //   hook.constructor = AsyncSeriesWaterfallHook;
  //   hook.compile = COMPILE;
  //   hook._call = undefined;
  //   hook.call = undefined;
  //   return hook;
  // }
  // AsyncSeriesWaterfallHook.prototype = null;
  var AsyncSeriesWaterfallHook$1 = /*#__PURE__*/function (_Hook) {
    _inheritsLoose$9(AsyncSeriesWaterfallHook, _Hook);
    function AsyncSeriesWaterfallHook(args, name) {
      var _this2;
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      _this2 = _Hook.call(this, args, name) || this;
      _this2.compile = COMPILE$1$1;
      _this2._call = undefined;
      _this2.call = undefined;
      return _this2;
    }
    return AsyncSeriesWaterfallHook;
  }(Hook$1);

  var SyncHookCodeFactory$1 = /*#__PURE__*/function (_HookCodeFactory) {
    _inheritsLoose$9(SyncHookCodeFactory, _HookCodeFactory);
    function SyncHookCodeFactory() {
      return _HookCodeFactory.apply(this, arguments) || this;
    }
    var _proto = SyncHookCodeFactory.prototype;
    _proto.content = function content(_ref) {
      var _onError = _ref.onError,
        onDone = _ref.onDone,
        rethrowIfPossible = _ref.rethrowIfPossible;
      return this.callTapsSeries({
        onError: function onError(i, err) {
          return _onError(err);
        },
        onDone: onDone,
        rethrowIfPossible: rethrowIfPossible
      });
    };
    return SyncHookCodeFactory;
  }(HookCodeFactory$1);
  var factory$2$1 = new SyncHookCodeFactory$1();
  var TAP_ASYNC$2 = function TAP_ASYNC() {
    throw new Error('tapAsync is not supported on a SyncHook');
  };
  var TAP_PROMISE$2 = function TAP_PROMISE() {
    throw new Error('tapPromise is not supported on a SyncHook');
  };
  var COMPILE$2$1 = function COMPILE(options) {
    factory$2$1.setup(this, options);
    return factory$2$1.create(options);
  };
  var SyncHook$1 = /*#__PURE__*/function (_Hook) {
    _inheritsLoose$9(SyncHook, _Hook);
    function SyncHook(args, name) {
      var _this;
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      _this = _Hook.call(this, args, name) || this;
      _this.tapAsync = TAP_ASYNC$2;
      _this.tapPromise = TAP_PROMISE$2;
      _this.compile = COMPILE$2$1;
      return _this;
    }
    return SyncHook;
  }(Hook$1);
  // export function SyncHook(args = [], name = undefined) {
  //   const hook = new Hook(args, name);
  //   hook.constructor = SyncHook;
  //   hook.tapAsync = TAP_ASYNC;
  //   hook.tapPromise = TAP_PROMISE;
  //   hook.compile = COMPILE;
  //   return hook;
  // }
  // SyncHook.prototype = null;

  var SyncWaterfallHookCodeFactory$1 = /*#__PURE__*/function (_HookCodeFactory) {
    _inheritsLoose$9(SyncWaterfallHookCodeFactory, _HookCodeFactory);
    function SyncWaterfallHookCodeFactory() {
      return _HookCodeFactory.apply(this, arguments) || this;
    }
    var _proto = SyncWaterfallHookCodeFactory.prototype;
    _proto.content = function content(_ref) {
      var _this = this;
      var _onError = _ref.onError,
        onResult = _ref.onResult,
        resultReturns = _ref.resultReturns,
        rethrowIfPossible = _ref.rethrowIfPossible;
      return this.callTapsSeries({
        onError: function onError(i, err) {
          return _onError(err);
        },
        onResult: function onResult(i, result, next) {
          var code = '';
          code += "if(" + result + " !== undefined) {\n";
          code += _this._args[0] + " = " + result + ";\n";
          code += "}\n";
          code += next();
          return code;
        },
        onDone: function onDone() {
          return onResult(_this._args[0]);
        },
        doneReturns: resultReturns,
        rethrowIfPossible: rethrowIfPossible
      });
    };
    return SyncWaterfallHookCodeFactory;
  }(HookCodeFactory$1);
  var factory$3$1 = new SyncWaterfallHookCodeFactory$1();
  var TAP_ASYNC$1$1 = function TAP_ASYNC() {
    throw new Error('tapAsync is not supported on a SyncWaterfallHook');
  };
  var TAP_PROMISE$1$1 = function TAP_PROMISE() {
    throw new Error('tapPromise is not supported on a SyncWaterfallHook');
  };
  var COMPILE$3$1 = function COMPILE(options) {
    factory$3$1.setup(this, options);
    return factory$3$1.create(options);
  };
  var SyncWaterfallHook$1 = /*#__PURE__*/function (_SyncHook) {
    _inheritsLoose$9(SyncWaterfallHook, _SyncHook);
    function SyncWaterfallHook(args, name) {
      var _this2;
      if (args === void 0) {
        args = [];
      }
      if (name === void 0) {
        name = undefined;
      }
      _this2 = _SyncHook.call(this, args, name) || this;
      _this2.tapAsync = TAP_ASYNC$1$1;
      _this2.tapPromise = TAP_PROMISE$1$1;
      _this2.compile = COMPILE$3$1;
      return _this2;
    }
    return SyncWaterfallHook;
  }(SyncHook$1);
  // export function SyncWaterfallHook(args = [], name = undefined) {
  //   if (args.length < 1) throw new Error('Waterfall hooks must have at least one argument');
  //   const hook = new Hook(args, name);
  //   hook.constructor = SyncWaterfallHook;
  //   hook.tapAsync = TAP_ASYNC;
  //   hook.tapPromise = TAP_PROMISE;
  //   hook.compile = COMPILE;
  //   return hook;
  // }
  // SyncWaterfallHook.prototype = null;

  var genericFontFamilies$1 = ['serif', 'sans-serif', 'monospace', 'cursive', 'fantasy', 'system-ui'];
  var stringRegExp$1 = /([\"\'])[^\'\"]+\1/;
  function toFontString$1(attributes) {
    var fontSize = attributes.fontSize,
      fontFamily = attributes.fontFamily,
      fontStyle = attributes.fontStyle,
      fontVariant = attributes.fontVariant,
      fontWeight = attributes.fontWeight;
    // build canvas api font setting from individual components. Convert a numeric this.fontSize to px
    // const fontSizeString: string = isNumber(fontSize) ? `${fontSize}px` : fontSize.toString();
    var fontSizeString = isNumber$1(fontSize) && fontSize + "px" || '16px';
    // Clean-up fontFamily property by quoting each font name
    // this will support font names with spaces
    // @ts-ignore
    var fontFamilies = isString$1(fontFamily) ? fontFamily.split(',') : [fontFamily];
    for (var i = fontFamilies.length - 1; i >= 0; i--) {
      // Trim any extra white-space
      var _fontFamily = fontFamilies[i].trim();
      // Check if font already contains strings
      if (!stringRegExp$1.test(_fontFamily) && genericFontFamilies$1.indexOf(_fontFamily) < 0) {
        _fontFamily = "\"" + _fontFamily + "\"";
      }
      fontFamilies[i] = _fontFamily;
    }
    return fontStyle + " " + fontVariant + " " + fontWeight + " " + fontSizeString + " " + fontFamilies.join(',');
  }

  /**
   * Blink used them in code generation(css_properties.json5)
   */
  var BUILT_IN_PROPERTIES$1 = [{
    /**
     * used in CSS Layout API
     * eg. `display: 'flex'`
     */
    n: 'display',
    k: ['none']
  }, {
    /**
     * range [0.0, 1.0]
     * @see https://developer.mozilla.org/en-US/docs/Web/CSS/opacity
     */
    n: 'opacity',
    int: true,
    inh: true,
    d: '1',
    syntax: PropertySyntax$1.OPACITY_VALUE
  }, {
    /**
     * inheritable, range [0.0, 1.0]
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/fill-opacity
     * @see https://svgwg.org/svg2-draft/painting.html#FillOpacity
     */
    n: 'fillOpacity',
    int: true,
    inh: true,
    d: '1',
    syntax: PropertySyntax$1.OPACITY_VALUE
  }, {
    /**
     * inheritable, range [0.0, 1.0]
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-opacity
     * @see https://svgwg.org/svg2-draft/painting.html#StrokeOpacity
     */
    n: 'strokeOpacity',
    int: true,
    inh: true,
    d: '1',
    syntax: PropertySyntax$1.OPACITY_VALUE
  }, {
    /**
     * background-color is not inheritable
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Tutorial/Fills_and_Strokes
     */
    n: 'fill',
    int: true,
    k: ['none'],
    d: 'none',
    syntax: PropertySyntax$1.PAINT
  }, {
    n: 'fillRule',
    k: ['nonzero', 'evenodd'],
    d: 'nonzero'
  },
  /**
   * default to none
   * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke#usage_notes
   */
  {
    n: 'stroke',
    int: true,
    k: ['none'],
    d: 'none',
    syntax: PropertySyntax$1.PAINT
  }, {
    n: 'shadowType',
    k: ['inner', 'outer', 'both'],
    d: 'outer',
    l: true
  }, {
    n: 'shadowColor',
    int: true,
    syntax: PropertySyntax$1.COLOR
  }, {
    n: 'shadowOffsetX',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'shadowOffsetY',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'shadowBlur',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.SHADOW_BLUR
  }, {
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-width
     */
    n: 'lineWidth',
    int: true,
    inh: true,
    d: '1',
    l: true,
    a: ['strokeWidth'],
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'increasedLineWidthForHitTesting',
    inh: true,
    d: '0',
    l: true,
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'lineJoin',
    inh: true,
    l: true,
    a: ['strokeLinejoin'],
    k: ['miter', 'bevel', 'round'],
    d: 'miter'
  }, {
    n: 'lineCap',
    inh: true,
    l: true,
    a: ['strokeLinecap'],
    k: ['butt', 'round', 'square'],
    d: 'butt'
  }, {
    n: 'lineDash',
    int: true,
    inh: true,
    k: ['none'],
    a: ['strokeDasharray'],
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE_12
  }, {
    n: 'lineDashOffset',
    int: true,
    inh: true,
    d: '0',
    a: ['strokeDashoffset'],
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'offsetPath',
    syntax: PropertySyntax$1.DEFINED_PATH
  }, {
    n: 'offsetDistance',
    int: true,
    syntax: PropertySyntax$1.OFFSET_DISTANCE
  }, {
    n: 'dx',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'dy',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'zIndex',
    ind: true,
    int: true,
    d: '0',
    k: ['auto'],
    syntax: PropertySyntax$1.Z_INDEX
  }, {
    n: 'visibility',
    k: ['visible', 'hidden'],
    ind: true,
    inh: true,
    /**
     * TODO: support interpolation
     * @see https://developer.mozilla.org/en-US/docs/Web/CSS/visibility#interpolation
     */
    // int: true,
    d: 'visible'
  }, {
    n: 'pointerEvents',
    inh: true,
    k: ['none', 'auto', 'stroke', 'fill', 'painted', 'visible', 'visiblestroke', 'visiblefill', 'visiblepainted',
    // 'bounding-box',
    'all'],
    d: 'auto'
  }, {
    n: 'filter',
    ind: true,
    l: true,
    k: ['none'],
    d: 'none',
    syntax: PropertySyntax$1.FILTER
  }, {
    n: 'clipPath',
    syntax: PropertySyntax$1.DEFINED_PATH
  }, {
    n: 'textPath',
    syntax: PropertySyntax$1.DEFINED_PATH
  }, {
    n: 'textPathSide',
    k: ['left', 'right'],
    d: 'left'
  }, {
    n: 'textPathStartOffset',
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'transform',
    p: 100,
    int: true,
    k: ['none'],
    d: 'none',
    syntax: PropertySyntax$1.TRANSFORM
  }, {
    n: 'transformOrigin',
    p: 100,
    // int: true,
    d: function d(nodeName) {
      if (nodeName === Shape$2.CIRCLE || nodeName === Shape$2.ELLIPSE) {
        return 'center';
      }
      if (nodeName === Shape$2.TEXT) {
        return 'text-anchor';
      }
      return 'left top';
    },
    l: true,
    syntax: PropertySyntax$1.TRANSFORM_ORIGIN
  }, {
    n: 'anchor',
    p: 99,
    d: function d(nodeName) {
      if (nodeName === Shape$2.CIRCLE || nodeName === Shape$2.ELLIPSE) {
        return '0.5 0.5';
      }
      return '0 0';
    },
    l: true,
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE_12
  },
  // <circle> & <ellipse>
  {
    n: 'cx',
    int: true,
    d: '0',
    syntax: PropertySyntax$1.COORDINATE
  }, {
    n: 'cy',
    int: true,
    d: '0',
    syntax: PropertySyntax$1.COORDINATE
  }, {
    n: 'r',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'rx',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'ry',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  },
  // Rect Image Group
  {
    // x in local space
    n: 'x',
    int: true,
    d: '0',
    syntax: PropertySyntax$1.COORDINATE
  }, {
    // y in local space
    n: 'y',
    int: true,
    d: '0',
    syntax: PropertySyntax$1.COORDINATE
  }, {
    // z in local space
    n: 'z',
    int: true,
    d: '0',
    syntax: PropertySyntax$1.COORDINATE
  }, {
    n: 'width',
    int: true,
    l: true,
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/width
     */
    k: ['auto', 'fit-content', 'min-content', 'max-content'],
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'height',
    int: true,
    l: true,
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/height
     */
    k: ['auto', 'fit-content', 'min-content', 'max-content'],
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'radius',
    int: true,
    l: true,
    d: '0',
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE_14
  },
  // Line
  {
    n: 'x1',
    int: true,
    l: true,
    syntax: PropertySyntax$1.COORDINATE
  }, {
    n: 'y1',
    int: true,
    l: true,
    syntax: PropertySyntax$1.COORDINATE
  }, {
    n: 'z1',
    int: true,
    l: true,
    syntax: PropertySyntax$1.COORDINATE
  }, {
    n: 'x2',
    int: true,
    l: true,
    syntax: PropertySyntax$1.COORDINATE
  }, {
    n: 'y2',
    int: true,
    l: true,
    syntax: PropertySyntax$1.COORDINATE
  }, {
    n: 'z2',
    int: true,
    l: true,
    syntax: PropertySyntax$1.COORDINATE
  },
  // Path
  {
    n: 'path',
    int: true,
    l: true,
    d: '',
    a: ['d'],
    syntax: PropertySyntax$1.PATH,
    p: 50
  },
  // Polyline & Polygon
  {
    n: 'points',
    l: true,
    syntax: PropertySyntax$1.LIST_OF_POINTS,
    p: 50
  },
  // Text
  {
    n: 'text',
    l: true,
    d: '',
    syntax: PropertySyntax$1.TEXT,
    p: 50
  }, {
    n: 'textTransform',
    l: true,
    inh: true,
    k: ['capitalize', 'uppercase', 'lowercase', 'none'],
    d: 'none',
    syntax: PropertySyntax$1.TEXT_TRANSFORM,
    p: 51 // it must get parsed after text
  }, {
    n: 'font',
    l: true
  }, {
    n: 'fontSize',
    int: true,
    inh: true,
    /**
     * @see https://www.w3schools.com/css/css_font_size.asp
     */
    d: '16px',
    l: true,
    syntax: PropertySyntax$1.LENGTH_PERCENTAGE
  }, {
    n: 'fontFamily',
    l: true,
    inh: true,
    d: 'sans-serif'
  }, {
    n: 'fontStyle',
    l: true,
    inh: true,
    k: ['normal', 'italic', 'oblique'],
    d: 'normal'
  }, {
    n: 'fontWeight',
    l: true,
    inh: true,
    k: ['normal', 'bold', 'bolder', 'lighter'],
    d: 'normal'
  }, {
    n: 'fontVariant',
    l: true,
    inh: true,
    k: ['normal', 'small-caps'],
    d: 'normal'
  }, {
    n: 'lineHeight',
    l: true,
    syntax: PropertySyntax$1.LENGTH,
    int: true,
    d: '0'
  }, {
    n: 'letterSpacing',
    l: true,
    syntax: PropertySyntax$1.LENGTH,
    int: true,
    d: '0'
  }, {
    n: 'miterLimit',
    l: true,
    syntax: PropertySyntax$1.NUMBER,
    d: function d(nodeName) {
      if (nodeName === Shape$2.PATH || nodeName === Shape$2.POLYGON || nodeName === Shape$2.POLYLINE) {
        return '4';
      }
      return '10';
    }
  }, {
    n: 'wordWrap',
    l: true
  }, {
    n: 'wordWrapWidth',
    l: true
  }, {
    n: 'maxLines',
    l: true
  }, {
    n: 'textOverflow',
    l: true,
    d: 'clip'
  }, {
    n: 'leading',
    l: true
  }, {
    n: 'textBaseline',
    l: true,
    inh: true,
    k: ['top', 'hanging', 'middle', 'alphabetic', 'ideographic', 'bottom'],
    d: 'alphabetic'
  }, {
    n: 'textAlign',
    l: true,
    inh: true,
    k: ['start', 'center', 'middle', 'end', 'left', 'right'],
    d: 'start'
  },
  // {
  //   n: 'whiteSpace',
  //   l: true,
  // },
  {
    n: 'markerStart',
    syntax: PropertySyntax$1.MARKER
  }, {
    n: 'markerEnd',
    syntax: PropertySyntax$1.MARKER
  }, {
    n: 'markerMid',
    syntax: PropertySyntax$1.MARKER
  }, {
    n: 'markerStartOffset',
    syntax: PropertySyntax$1.LENGTH,
    l: true,
    int: true,
    d: '0'
  }, {
    n: 'markerEndOffset',
    syntax: PropertySyntax$1.LENGTH,
    l: true,
    int: true,
    d: '0'
  }];
  var propertyMetadataCache$1 = {};
  var unresolvedProperties$1 = new WeakMap();
  var priorityMap$1 = {};
  var sortAttributeNames$1 = memoize$1(function (attributeNames) {
    return attributeNames.sort(function (a, b) {
      return priorityMap$1[a] - priorityMap$1[b];
    });
  });
  var tmpVec3a$2 = create$8();
  var tmpVec3b$2 = create$8();
  var tmpVec3c$2 = create$8();
  var isPropertyResolved$1 = function isPropertyResolved(object, name) {
    var properties = unresolvedProperties$1.get(object);
    if (!properties || properties.length === 0) {
      return true;
    }
    return properties.includes(name);
  };
  var DefaultStyleValueRegistry$1 = /*#__PURE__*/function () {
    /**
     * need recalc later
     */
    // dirty = false;
    function DefaultStyleValueRegistry() {
      var _this = this;
      BUILT_IN_PROPERTIES$1.forEach(function (property) {
        _this.registerMetadata(property);
      });
    }
    var _proto = DefaultStyleValueRegistry.prototype;
    _proto.registerMetadata = function registerMetadata(metadata) {
      [metadata.n].concat(metadata.a || []).forEach(function (name) {
        propertyMetadataCache$1[name] = metadata;
        priorityMap$1[name] = metadata.p || 0;
      });
    };
    _proto.unregisterMetadata = function unregisterMetadata(name) {
      delete propertyMetadataCache$1[name];
    };
    _proto.getPropertySyntax = function getPropertySyntax(syntax) {
      return runtime$2.CSSPropertySyntaxFactory[syntax];
    }
    /**
     * * parse value, eg.
     * fill: 'red' => CSSRGB
     * translateX: '10px' => CSSUnitValue { unit: 'px', value: 10 }
     * fontSize: '2em' => { unit: 'px', value: 32 }
     *
     * * calculate used value
     * * post process
     */;
    _proto.processProperties = function processProperties(object, attributes, options) {
      var _this2 = this;
      if (options === void 0) {
        options = {
          skipUpdateAttribute: false,
          skipParse: false,
          forceUpdateGeometry: false,
          usedAttributes: []
        };
      }
      var _options = options,
        skipUpdateAttribute = _options.skipUpdateAttribute,
        skipParse = _options.skipParse,
        forceUpdateGeometry = _options.forceUpdateGeometry,
        usedAttributes = _options.usedAttributes;
      var needUpdateGeometry = forceUpdateGeometry;
      var attributeNames = Object.keys(attributes);
      attributeNames.forEach(function (attributeName) {
        var _propertyMetadataCach;
        var name = formatAttributeName$1(attributeName);
        if (!skipUpdateAttribute) {
          object.attributes[name] = attributes[attributeName];
        }
        if (!needUpdateGeometry && ((_propertyMetadataCach = propertyMetadataCache$1[name]) === null || _propertyMetadataCach === void 0 ? void 0 : _propertyMetadataCach.l)) {
          needUpdateGeometry = true;
        }
      });
      if (!skipParse) {
        attributeNames.forEach(function (name) {
          object.computedStyle[name] = _this2.parseProperty(name, object.attributes[name], object);
        });
      }
      // let hasUnresolvedProperties = false;
      // parse according to priority
      attributeNames = sortAttributeNames$1((usedAttributes === null || usedAttributes === void 0 ? void 0 : usedAttributes.length) ? Array.from(new Set(attributeNames.concat(usedAttributes))) : attributeNames);
      // attributeNames.sort((a, b) => priorityMap[a] - priorityMap[b]);
      attributeNames.forEach(function (name) {
        // some style props maybe deleted after parsing such as `anchor` in Text
        if (name in object.computedStyle) {
          object.parsedStyle[name] = _this2.computeProperty(name, object.computedStyle[name], object);
        }
      });
      // if (hasUnresolvedProperties) {
      //   this.dirty = true;
      //   return;
      // }
      // update geometry
      if (needUpdateGeometry) {
        this.updateGeometry(object);
      }
      attributeNames.forEach(function (name) {
        if (name in object.parsedStyle) {
          _this2.postProcessProperty(name, object);
        }
      });
      if (object.children.length) {
        attributeNames.forEach(function (name) {
          if (name in object.parsedStyle && _this2.isPropertyInheritable(name)) {
            // update children's inheritable
            object.children.forEach(function (child) {
              child.internalSetAttribute(name, null, {
                skipUpdateAttribute: true,
                skipParse: true
              });
            });
          }
        });
      }
    }
    /**
     * string -> parsed value
     */;
    _proto.parseProperty = function parseProperty(name, value, object) {
      var metadata = propertyMetadataCache$1[name];
      var computed = value;
      if (value === '' || isNil$1(value)) {
        value = 'unset';
      }
      if (value === 'unset' || value === 'initial' || value === 'inherit') {
        // computed = new CSSKeywordValue(value);
        computed = getOrCreateKeyword$1(value);
      } else {
        if (metadata) {
          var keywords = metadata.k,
            syntax = metadata.syntax;
          var handler = syntax && this.getPropertySyntax(syntax);
          // use keywords
          if (keywords && keywords.indexOf(value) > -1) {
            // computed = new CSSKeywordValue(value);
            computed = getOrCreateKeyword$1(value);
          } else if (handler && handler.parser) {
            // try to parse it to CSSStyleValue, eg. '10px' -> CSS.px(10)
            computed = handler.parser(value, object);
          }
        }
      }
      return computed;
    }
    /**
     * computed value -> used value
     */;
    _proto.computeProperty = function computeProperty(name, computed, object) {
      var metadata = propertyMetadataCache$1[name];
      var isDocumentElement = object.id === 'g-root';
      // let used: CSSStyleValue = computed instanceof CSSStyleValue ? computed.clone() : computed;
      var used = computed;
      if (metadata) {
        var syntax = metadata.syntax,
          inherited = metadata.inh,
          defaultValue = metadata.d;
        if (computed instanceof CSSKeywordValue$1) {
          var value = computed.value;
          /**
           * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/unset
           */
          if (value === 'unset') {
            if (inherited && !isDocumentElement) {
              value = 'inherit';
            } else {
              value = 'initial';
            }
          }
          if (value === 'initial') {
            // @see https://developer.mozilla.org/en-US/docs/Web/CSS/initial
            if (!isNil$1(defaultValue)) {
              computed = this.parseProperty(name, isFunction$3(defaultValue) ? defaultValue(object.nodeName) : defaultValue, object);
            }
          } else if (value === 'inherit') {
            // @see https://developer.mozilla.org/en-US/docs/Web/CSS/inherit
            // behave like `inherit`
            var resolved = this.tryToResolveProperty(object, name, {
              inherited: true
            });
            if (!isNil$1(resolved)) {
              // object.parsedStyle[name] = resolved;
              // return false;
              return resolved;
            } else {
              this.addUnresolveProperty(object, name);
              return;
            }
          }
        }
        var handler = syntax && this.getPropertySyntax(syntax);
        if (handler && handler.calculator) {
          // convert computed value to used value
          var oldParsedValue = object.parsedStyle[name];
          used = handler.calculator(name, oldParsedValue, computed, object, this);
        } else if (computed instanceof CSSKeywordValue$1) {
          used = computed.value;
        } else {
          used = computed;
        }
      }
      // object.parsedStyle[name] = used;
      // return false;
      return used;
    };
    _proto.postProcessProperty = function postProcessProperty(name, object) {
      var metadata = propertyMetadataCache$1[name];
      if (metadata && metadata.syntax) {
        var handler = metadata.syntax && this.getPropertySyntax(metadata.syntax);
        var propertyHandler = handler;
        if (propertyHandler && propertyHandler.postProcessor) {
          propertyHandler.postProcessor(object);
        }
      }
    }
    /**
     * resolve later
     */;
    _proto.addUnresolveProperty = function addUnresolveProperty(object, name) {
      var properties = unresolvedProperties$1.get(object);
      if (!properties) {
        unresolvedProperties$1.set(object, []);
        properties = unresolvedProperties$1.get(object);
      }
      if (properties.indexOf(name) === -1) {
        properties.push(name);
      }
    };
    _proto.tryToResolveProperty = function tryToResolveProperty(object, name, options) {
      if (options === void 0) {
        options = {};
      }
      var _options2 = options,
        inherited = _options2.inherited;
      if (inherited) {
        if (object.parentElement && isPropertyResolved$1(object.parentElement, name)) {
          // const computedValue = object.parentElement.computedStyle[name];
          var usedValue = object.parentElement.parsedStyle[name];
          if (
          // usedValue instanceof CSSKeywordValue &&
          usedValue === 'unset' || usedValue === 'initial' || usedValue === 'inherit') {
            return;
          }
          // else if (
          //   usedValue instanceof CSSUnitValue &&
          //   CSSUnitValue.isRelativeUnit(usedValue.unit)
          // ) {
          //   return false;
          // }
          return usedValue;
        }
      }
      return;
    };
    _proto.recalc = function recalc(object) {
      var properties = unresolvedProperties$1.get(object);
      if (properties && properties.length) {
        var attributes = {};
        properties.forEach(function (property) {
          attributes[property] = object.attributes[property];
        });
        this.processProperties(object, attributes);
        unresolvedProperties$1.delete(object);
      }
    }
    /**
     * update geometry when relative props changed,
     * eg. r of Circle, width/height of Rect
     */;
    _proto.updateGeometry = function updateGeometry(object) {
      var geometryUpdater = runtime$2.geometryUpdaterFactory[object.nodeName];
      if (geometryUpdater) {
        var geometry = object.geometry;
        if (!geometry.contentBounds) {
          geometry.contentBounds = new AABB$1();
        }
        if (!geometry.renderBounds) {
          geometry.renderBounds = new AABB$1();
        }
        var parsedStyle = object.parsedStyle;
        var _geometryUpdater$upda = geometryUpdater.update(parsedStyle, object),
          width = _geometryUpdater$upda.width,
          height = _geometryUpdater$upda.height,
          _geometryUpdater$upda2 = _geometryUpdater$upda.depth,
          depth = _geometryUpdater$upda2 === void 0 ? 0 : _geometryUpdater$upda2,
          _geometryUpdater$upda3 = _geometryUpdater$upda.offsetX,
          offsetX = _geometryUpdater$upda3 === void 0 ? 0 : _geometryUpdater$upda3,
          _geometryUpdater$upda4 = _geometryUpdater$upda.offsetY,
          offsetY = _geometryUpdater$upda4 === void 0 ? 0 : _geometryUpdater$upda4,
          _geometryUpdater$upda5 = _geometryUpdater$upda.offsetZ,
          offsetZ = _geometryUpdater$upda5 === void 0 ? 0 : _geometryUpdater$upda5;
        // account for negative width / height of Rect
        // @see https://github.com/antvis/g/issues/957
        var flipY = width < 0;
        var flipX = height < 0;
        // init with content box
        var halfExtents = set$3(tmpVec3a$2, Math.abs(width) / 2, Math.abs(height) / 2, depth / 2);
        // anchor is center by default, don't account for lineWidth here
        var stroke = parsedStyle.stroke,
          lineWidth = parsedStyle.lineWidth,
          increasedLineWidthForHitTesting = parsedStyle.increasedLineWidthForHitTesting,
          shadowType = parsedStyle.shadowType,
          shadowColor = parsedStyle.shadowColor,
          _parsedStyle$filter = parsedStyle.filter,
          filter = _parsedStyle$filter === void 0 ? [] : _parsedStyle$filter,
          transformOrigin = parsedStyle.transformOrigin;
        var anchor = parsedStyle.anchor;
        // <Text> use textAlign & textBaseline instead of anchor
        if (object.nodeName === Shape$2.TEXT) {
          delete parsedStyle.anchor;
        }
        var center = set$3(tmpVec3b$2, (1 - (anchor && anchor[0] || 0) * 2) * width / 2 + offsetX, (1 - (anchor && anchor[1] || 0) * 2) * height / 2 + offsetY, (1 - (anchor && anchor[2] || 0) * 2) * halfExtents[2] + offsetZ);
        // update geometry's AABB
        geometry.contentBounds.update(center, halfExtents);
        // @see https://github.molgen.mpg.de/git-mirror/cairo/blob/master/src/cairo-stroke-style.c#L97..L128
        var expansion = object.nodeName === Shape$2.POLYLINE || object.nodeName === Shape$2.POLYGON || object.nodeName === Shape$2.PATH ? Math.SQRT2 : 0.5;
        // if (lineCap?.value === 'square') {
        //   expansion = Math.SQRT1_2;
        // }
        // if (lineJoin?.value === 'miter' && expansion < Math.SQRT2 * miterLimit) {
        //   expansion = Math.SQRT1_2 * miterLimit;
        // }
        // append border only if stroke existed
        var hasStroke = stroke && !stroke.isNone;
        if (hasStroke) {
          var halfLineWidth = ((lineWidth || 0) + (increasedLineWidthForHitTesting || 0)) * expansion;
          // halfExtents[0] += halfLineWidth[0];
          // halfExtents[1] += halfLineWidth[1];
          add$4(halfExtents, halfExtents, set$3(tmpVec3c$2, halfLineWidth, halfLineWidth, 0));
        }
        geometry.renderBounds.update(center, halfExtents);
        // account for shadow, only support constant value now
        if (shadowColor && shadowType && shadowType !== 'inner') {
          var _geometry$renderBound = geometry.renderBounds,
            min = _geometry$renderBound.min,
            max = _geometry$renderBound.max;
          var shadowBlur = parsedStyle.shadowBlur,
            shadowOffsetX = parsedStyle.shadowOffsetX,
            shadowOffsetY = parsedStyle.shadowOffsetY;
          var shadowBlurInPixels = shadowBlur || 0;
          var shadowOffsetXInPixels = shadowOffsetX || 0;
          var shadowOffsetYInPixels = shadowOffsetY || 0;
          var shadowLeft = min[0] - shadowBlurInPixels + shadowOffsetXInPixels;
          var shadowRight = max[0] + shadowBlurInPixels + shadowOffsetXInPixels;
          var shadowTop = min[1] - shadowBlurInPixels + shadowOffsetYInPixels;
          var shadowBottom = max[1] + shadowBlurInPixels + shadowOffsetYInPixels;
          min[0] = Math.min(min[0], shadowLeft);
          max[0] = Math.max(max[0], shadowRight);
          min[1] = Math.min(min[1], shadowTop);
          max[1] = Math.max(max[1], shadowBottom);
          geometry.renderBounds.setMinMax(min, max);
        }
        // account for filter, eg. blur(5px), drop-shadow()
        filter.forEach(function (_ref) {
          var name = _ref.name,
            params = _ref.params;
          if (name === 'blur') {
            var blurRadius = params[0].value;
            geometry.renderBounds.update(geometry.renderBounds.center, add$4(geometry.renderBounds.halfExtents, geometry.renderBounds.halfExtents, fromValues$8(blurRadius, blurRadius, 0)));
          } else if (name === 'drop-shadow') {
            var _shadowOffsetX = params[0].value;
            var _shadowOffsetY = params[1].value;
            var _shadowBlur = params[2].value;
            var _geometry$renderBound2 = geometry.renderBounds,
              _min = _geometry$renderBound2.min,
              _max = _geometry$renderBound2.max;
            var _shadowLeft = _min[0] - _shadowBlur + _shadowOffsetX;
            var _shadowRight = _max[0] + _shadowBlur + _shadowOffsetX;
            var _shadowTop = _min[1] - _shadowBlur + _shadowOffsetY;
            var _shadowBottom = _max[1] + _shadowBlur + _shadowOffsetY;
            _min[0] = Math.min(_min[0], _shadowLeft);
            _max[0] = Math.max(_max[0], _shadowRight);
            _min[1] = Math.min(_min[1], _shadowTop);
            _max[1] = Math.max(_max[1], _shadowBottom);
            geometry.renderBounds.setMinMax(_min, _max);
          }
        });
        anchor = parsedStyle.anchor;
        // set transform origin
        var usedOriginXValue = (flipY ? -1 : 1) * convertPercentUnit$1(transformOrigin[0], 0, object);
        var usedOriginYValue = (flipX ? -1 : 1) * convertPercentUnit$1(transformOrigin[1], 1, object);
        usedOriginXValue = usedOriginXValue - (flipY ? -1 : 1) * (anchor && anchor[0] || 0) * geometry.contentBounds.halfExtents[0] * 2;
        usedOriginYValue = usedOriginYValue - (flipX ? -1 : 1) * (anchor && anchor[1] || 0) * geometry.contentBounds.halfExtents[1] * 2;
        object.setOrigin(usedOriginXValue, usedOriginYValue);
        runtime$2.sceneGraphService.dirtifyToRoot(object);
      }
    };
    _proto.isPropertyInheritable = function isPropertyInheritable(name) {
      var metadata = propertyMetadataCache$1[name];
      if (!metadata) {
        return false;
      }
      return metadata.inh;
    };
    return DefaultStyleValueRegistry;
  }();

  /**
   * Different type of cameras, eg. simple camera used in 2D scene or
   * advanced camera which can do actions & switch between landmarks.
   */
  var CameraType$1;
  (function (CameraType) {
    /**
     * Performs all the rotational operations with the focal point instead of the camera position.
     * This type of camera is useful in applications(like CAD) where 3D objects are being designed or explored.
     * Camera cannot orbits over the north & south poles.
     * @see http://voxelent.com/tutorial-cameras/
     *
     * In Three.js it's used in OrbitControls.
     * @see https://threejs.org/docs/#examples/zh/controls/OrbitControls
     */
    CameraType[CameraType["ORBITING"] = 0] = "ORBITING";
    /**
     * It's similar to the ORBITING camera, but it allows the camera to orbit over the north or south poles.
     *
     * In Three.js it's used in OrbitControls.
     * @see https://threejs.org/docs/#examples/en/controls/TrackballControls
     */
    CameraType[CameraType["EXPLORING"] = 1] = "EXPLORING";
    /**
     * Performs all the rotational operations with the camera position.
     * It's useful in first person shooting games.
     * Camera cannot orbits over the north & south poles.
     *
     * In Three.js it's used in FirstPersonControls.
     * @see https://threejs.org/docs/#examples/en/controls/FirstPersonControls
     */
    CameraType[CameraType["TRACKING"] = 2] = "TRACKING";
  })(CameraType$1 || (CameraType$1 = {}));
  /**
   * CameraType must be TRACKING
   */
  var CameraTrackingMode$1;
  (function (CameraTrackingMode) {
    CameraTrackingMode[CameraTrackingMode["DEFAULT"] = 0] = "DEFAULT";
    CameraTrackingMode[CameraTrackingMode["ROTATIONAL"] = 1] = "ROTATIONAL";
    CameraTrackingMode[CameraTrackingMode["TRANSLATIONAL"] = 2] = "TRANSLATIONAL";
    CameraTrackingMode[CameraTrackingMode["CINEMATIC"] = 3] = "CINEMATIC";
  })(CameraTrackingMode$1 || (CameraTrackingMode$1 = {}));
  var CameraProjectionMode$1;
  (function (CameraProjectionMode) {
    CameraProjectionMode[CameraProjectionMode["ORTHOGRAPHIC"] = 0] = "ORTHOGRAPHIC";
    CameraProjectionMode[CameraProjectionMode["PERSPECTIVE"] = 1] = "PERSPECTIVE";
  })(CameraProjectionMode$1 || (CameraProjectionMode$1 = {}));
  var CameraEvent$1 = {
    UPDATED: 'updated'
  };

  var MIN_DISTANCE$1 = 0.0002;
  /**
   * 参考「WebGL Insights - 23.Designing Cameras for WebGL Applications」，基于 Responsible Camera 思路设计
   * @see https://github.com/d13g0/nucleo.js/blob/master/source/camera/Camera.js
   *
   * 保存相机参数，定义相机动作：
   * 1. dolly 沿 n 轴移动
   * 2. pan 沿 u v 轴移动
   * 3. rotate 以方位角旋转
   * 4. 移动到 Landmark，具有平滑的动画效果，其间禁止其他用户交互
   */
  var Camera$1 = /*#__PURE__*/function () {
    function Camera() {
      this.canvas = void 0;
      this.eventEmitter = new eventemitter3$1();
      this.matrix = create$7();
      this.right = fromValues$8(1, 0, 0);
      this.up = fromValues$8(0, 1, 0);
      this.forward = fromValues$8(0, 0, 1);
      this.position = fromValues$8(0, 0, 1);
      this.focalPoint = fromValues$8(0, 0, 0);
      this.distanceVector = fromValues$8(0, 0, -1);
      this.distance = 1;
      this.azimuth = 0;
      this.elevation = 0;
      this.roll = 0;
      this.relAzimuth = 0;
      this.relElevation = 0;
      this.relRoll = 0;
      this.dollyingStep = 0;
      this.maxDistance = Infinity;
      this.minDistance = -Infinity;
      this.zoom = 1;
      this.rotateWorld = false;
      this.fov = 30;
      this.near = 0.1;
      this.far = 1000;
      this.aspect = 1;
      this.left = void 0;
      this.rright = void 0;
      this.top = void 0;
      this.bottom = void 0;
      this.projectionMatrix = create$7();
      this.projectionMatrixInverse = create$7();
      this.jitteredProjectionMatrix = undefined;
      this.view = void 0;
      this.enableUpdate = true;
      this.type = CameraType$1.EXPLORING;
      this.trackingMode = CameraTrackingMode$1.DEFAULT;
      this.projectionMode = CameraProjectionMode$1.PERSPECTIVE;
      this.frustum = new Frustum$1();
      this.orthoMatrix = create$7();
    }
    var _proto = Camera.prototype;
    // constructor(type = CameraType.EXPLORING, trackingMode = CameraTrackingMode.DEFAULT) {
    //   this.setType(type, trackingMode);
    // }
    _proto.isOrtho = function isOrtho() {
      return this.projectionMode === CameraProjectionMode$1.ORTHOGRAPHIC;
    };
    _proto.getProjectionMode = function getProjectionMode() {
      return this.projectionMode;
    };
    _proto.getPerspective = function getPerspective() {
      // account for TAA
      return this.jitteredProjectionMatrix || this.projectionMatrix;
    };
    _proto.getPerspectiveInverse = function getPerspectiveInverse() {
      return this.projectionMatrixInverse;
    };
    _proto.getFrustum = function getFrustum() {
      return this.frustum;
    };
    _proto.getPosition = function getPosition() {
      return this.position;
    };
    _proto.getFocalPoint = function getFocalPoint() {
      return this.focalPoint;
    };
    _proto.getDollyingStep = function getDollyingStep() {
      return this.dollyingStep;
    };
    _proto.getNear = function getNear() {
      return this.near;
    };
    _proto.getFar = function getFar() {
      return this.far;
    };
    _proto.getZoom = function getZoom() {
      return this.zoom;
    };
    _proto.getOrthoMatrix = function getOrthoMatrix() {
      return this.orthoMatrix;
    };
    _proto.getView = function getView() {
      return this.view;
    };
    _proto.setEnableUpdate = function setEnableUpdate(enabled) {
      this.enableUpdate = enabled;
    };
    _proto.setType = function setType(type, trackingMode) {
      this.type = type;
      if (this.type === CameraType$1.EXPLORING) {
        this.setWorldRotation(true);
      } else {
        this.setWorldRotation(false);
      }
      this._getAngles();
      if (this.type === CameraType$1.TRACKING && trackingMode !== undefined) {
        this.setTrackingMode(trackingMode);
      }
      return this;
    };
    _proto.setProjectionMode = function setProjectionMode(projectionMode) {
      this.projectionMode = projectionMode;
      return this;
    };
    _proto.setTrackingMode = function setTrackingMode(trackingMode) {
      if (this.type !== CameraType$1.TRACKING) {
        throw new Error('Impossible to set a tracking mode if the camera is not of tracking type');
      }
      this.trackingMode = trackingMode;
      return this;
    }
    /**
     * If flag is true, it reverses the azimuth and elevation angles.
     * Subsequent calls to rotate, setAzimuth, setElevation,
     * changeAzimuth or changeElevation will cause the inverted effect.
     * setRoll or changeRoll is not affected by this method.
     *
     * This inversion is useful when one wants to simulate that the world
     * is moving, instead of the camera.
     *
     * By default the camera angles are not reversed.
     * @param {Boolean} flag the boolean flag to reverse the angles.
     */;
    _proto.setWorldRotation = function setWorldRotation(flag) {
      this.rotateWorld = flag;
      this._getAngles();
      return this;
    }
    /**
     * 计算 MV 矩阵，为相机矩阵的逆矩阵
     */;
    _proto.getViewTransform = function getViewTransform() {
      return invert$2(create$7(), this.matrix);
    };
    _proto.getWorldTransform = function getWorldTransform() {
      return this.matrix;
    };
    _proto.jitterProjectionMatrix = function jitterProjectionMatrix(x, y) {
      var translation = fromTranslation$1(create$7(), [x, y, 0]);
      this.jitteredProjectionMatrix = multiply$3(create$7(), translation, this.projectionMatrix);
    };
    _proto.clearJitterProjectionMatrix = function clearJitterProjectionMatrix() {
      this.jitteredProjectionMatrix = undefined;
    }
    /**
     * 设置相机矩阵
     */;
    _proto.setMatrix = function setMatrix(matrix) {
      this.matrix = matrix;
      this._update();
      return this;
    };
    _proto.setFov = function setFov(fov) {
      this.setPerspective(this.near, this.far, fov, this.aspect);
      return this;
    };
    _proto.setAspect = function setAspect(aspect) {
      this.setPerspective(this.near, this.far, this.fov, aspect);
      return this;
    };
    _proto.setNear = function setNear(near) {
      if (this.projectionMode === CameraProjectionMode$1.PERSPECTIVE) {
        this.setPerspective(near, this.far, this.fov, this.aspect);
      } else {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, near, this.far);
      }
      return this;
    };
    _proto.setFar = function setFar(far) {
      if (this.projectionMode === CameraProjectionMode$1.PERSPECTIVE) {
        this.setPerspective(this.near, far, this.fov, this.aspect);
      } else {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, this.near, far);
      }
      return this;
    }
    /**
     * Sets an offset in a larger frustum, used in PixelPicking
     */;
    _proto.setViewOffset = function setViewOffset(fullWidth, fullHeight, x, y, width, height) {
      this.aspect = fullWidth / fullHeight;
      if (this.view === undefined) {
        this.view = {
          enabled: true,
          fullWidth: 1,
          fullHeight: 1,
          offsetX: 0,
          offsetY: 0,
          width: 1,
          height: 1
        };
      }
      this.view.enabled = true;
      this.view.fullWidth = fullWidth;
      this.view.fullHeight = fullHeight;
      this.view.offsetX = x;
      this.view.offsetY = y;
      this.view.width = width;
      this.view.height = height;
      if (this.projectionMode === CameraProjectionMode$1.PERSPECTIVE) {
        this.setPerspective(this.near, this.far, this.fov, this.aspect);
      } else {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, this.near, this.far);
      }
      return this;
    };
    _proto.clearViewOffset = function clearViewOffset() {
      if (this.view !== undefined) {
        this.view.enabled = false;
      }
      if (this.projectionMode === CameraProjectionMode$1.PERSPECTIVE) {
        this.setPerspective(this.near, this.far, this.fov, this.aspect);
      } else {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, this.near, this.far);
      }
      return this;
    };
    _proto.setZoom = function setZoom(zoom) {
      this.zoom = zoom;
      if (this.projectionMode === CameraProjectionMode$1.ORTHOGRAPHIC) {
        this.setOrthographic(this.left, this.rright, this.top, this.bottom, this.near, this.far);
      } else if (this.projectionMode === CameraProjectionMode$1.PERSPECTIVE) {
        this.setPerspective(this.near, this.far, this.fov, this.aspect);
      }
      return this;
    }
    /**
     * Zoom by specified point in viewport coordinates.
     */;
    _proto.setZoomByViewportPoint = function setZoomByViewportPoint(zoom, viewportPoint) {
      var _this$canvas$viewport = this.canvas.viewport2Canvas({
          x: viewportPoint[0],
          y: viewportPoint[1]
        }),
        ox = _this$canvas$viewport.x,
        oy = _this$canvas$viewport.y;
      var roll = this.roll;
      this.rotate(0, 0, -roll);
      this.setPosition(ox, oy);
      this.setFocalPoint(ox, oy);
      this.setZoom(zoom);
      this.rotate(0, 0, roll);
      var _this$canvas$viewport2 = this.canvas.viewport2Canvas({
          x: viewportPoint[0],
          y: viewportPoint[1]
        }),
        cx = _this$canvas$viewport2.x,
        cy = _this$canvas$viewport2.y;
      // project to rotated axis
      var dvec = fromValues$8(cx - ox, cy - oy, 0);
      var dx = dot$3(dvec, this.right) / length$1(this.right);
      var dy = dot$3(dvec, this.up) / length$1(this.up);
      this.pan(-dx, -dy);
      return this;
    };
    _proto.setPerspective = function setPerspective(near, far, fov, aspect) {
      var _this$view;
      this.projectionMode = CameraProjectionMode$1.PERSPECTIVE;
      this.fov = fov;
      this.near = near;
      this.far = far;
      this.aspect = aspect;
      var top = this.near * Math.tan(deg2rad$1(0.5 * this.fov)) / this.zoom;
      var height = 2 * top;
      var width = this.aspect * height;
      var left = -0.5 * width;
      if ((_this$view = this.view) === null || _this$view === void 0 ? void 0 : _this$view.enabled) {
        var fullWidth = this.view.fullWidth;
        var fullHeight = this.view.fullHeight;
        left += this.view.offsetX * width / fullWidth;
        top -= this.view.offsetY * height / fullHeight;
        width *= this.view.width / fullWidth;
        height *= this.view.height / fullHeight;
      }
      makePerspective$1(this.projectionMatrix, left, left + width, top, top - height, near, this.far);
      // flipY since the origin of OpenGL/WebGL is bottom-left compared with top-left in Canvas2D
      scale$3(this.projectionMatrix, this.projectionMatrix, fromValues$8(1, -1, 1));
      invert$2(this.projectionMatrixInverse, this.projectionMatrix);
      this.triggerUpdate();
      return this;
    };
    _proto.setOrthographic = function setOrthographic(l, r, t, b, near, far) {
      var _this$view2;
      this.projectionMode = CameraProjectionMode$1.ORTHOGRAPHIC;
      this.rright = r;
      this.left = l;
      this.top = t;
      this.bottom = b;
      this.near = near;
      this.far = far;
      var dx = (this.rright - this.left) / (2 * this.zoom);
      var dy = (this.top - this.bottom) / (2 * this.zoom);
      var cx = (this.rright + this.left) / 2;
      var cy = (this.top + this.bottom) / 2;
      var left = cx - dx;
      var right = cx + dx;
      var top = cy + dy;
      var bottom = cy - dy;
      if ((_this$view2 = this.view) === null || _this$view2 === void 0 ? void 0 : _this$view2.enabled) {
        var scaleW = (this.rright - this.left) / this.view.fullWidth / this.zoom;
        var scaleH = (this.top - this.bottom) / this.view.fullHeight / this.zoom;
        left += scaleW * this.view.offsetX;
        right = left + scaleW * this.view.width;
        top -= scaleH * this.view.offsetY;
        bottom = top - scaleH * this.view.height;
      }
      ortho$1(this.projectionMatrix, left, right, bottom, top, near, far);
      // flipY since the origin of OpenGL/WebGL is bottom-left compared with top-left in Canvas2D
      scale$3(this.projectionMatrix, this.projectionMatrix, fromValues$8(1, -1, 1));
      invert$2(this.projectionMatrixInverse, this.projectionMatrix);
      this._getOrthoMatrix();
      this.triggerUpdate();
      return this;
    }
    /**
     * Move the camera in world coordinates.
     * It will keep looking at the current focal point.
     *
     * support scalars or vectors.
     * @example
     * setPosition(1, 2, 3);
     * setPosition([1, 2, 3]);
     */;
    _proto.setPosition = function setPosition(x, y, z) {
      if (y === void 0) {
        y = this.position[1];
      }
      if (z === void 0) {
        z = this.position[2];
      }
      var position = createVec3$1(x, y, z);
      this._setPosition(position);
      this.setFocalPoint(this.focalPoint);
      this.triggerUpdate();
      return this;
    }
    /**
     * Sets the focal point of this camera in world coordinates.
     *
     * support scalars or vectors.
     * @example
     * setFocalPoint(1, 2, 3);
     * setFocalPoint([1, 2, 3]);
     */;
    _proto.setFocalPoint = function setFocalPoint(x, y, z) {
      if (y === void 0) {
        y = this.focalPoint[1];
      }
      if (z === void 0) {
        z = this.focalPoint[2];
      }
      var up = fromValues$8(0, 1, 0);
      this.focalPoint = createVec3$1(x, y, z);
      if (this.trackingMode === CameraTrackingMode$1.CINEMATIC) {
        var d = subtract$4(create$8(), this.focalPoint, this.position);
        x = d[0];
        y = d[1];
        z = d[2];
        var r = length$1(d);
        var el = rad2deg$1(Math.asin(y / r));
        var az = 90 + rad2deg$1(Math.atan2(z, x));
        var m = create$7();
        rotateY$1(m, m, deg2rad$1(az));
        rotateX$1(m, m, deg2rad$1(el));
        up = transformMat4$2(create$8(), [0, 1, 0], m);
      }
      invert$2(this.matrix, lookAt$1(create$7(), this.position, this.focalPoint, up));
      this._getAxes();
      this._getDistance();
      this._getAngles();
      this.triggerUpdate();
      return this;
    };
    _proto.getDistance = function getDistance() {
      return this.distance;
    };
    _proto.getDistanceVector = function getDistanceVector() {
      return this.distanceVector;
    }
    /**
     * Moves the camera towards/from the focal point.
     */;
    _proto.setDistance = function setDistance(d) {
      if (this.distance === d || d < 0) {
        return this;
      }
      this.distance = d;
      if (this.distance < MIN_DISTANCE$1) {
        this.distance = MIN_DISTANCE$1;
      }
      this.dollyingStep = this.distance / 100;
      var pos = create$8();
      d = this.distance;
      var n = this.forward;
      var f = this.focalPoint;
      pos[0] = d * n[0] + f[0];
      pos[1] = d * n[1] + f[1];
      pos[2] = d * n[2] + f[2];
      this._setPosition(pos);
      this.triggerUpdate();
      return this;
    };
    _proto.setMaxDistance = function setMaxDistance(d) {
      this.maxDistance = d;
      return this;
    };
    _proto.setMinDistance = function setMinDistance(d) {
      this.minDistance = d;
      return this;
    }
    /**
     * 设置相机方位角，不同相机模式下需要重新计算相机位置或者是视点位置
     * the azimuth in degrees
     */;
    _proto.setAzimuth = function setAzimuth(az) {
      this.azimuth = getAngle$3(az);
      this.computeMatrix();
      this._getAxes();
      if (this.type === CameraType$1.ORBITING || this.type === CameraType$1.EXPLORING) {
        this._getPosition();
      } else if (this.type === CameraType$1.TRACKING) {
        this._getFocalPoint();
      }
      this.triggerUpdate();
      return this;
    };
    _proto.getAzimuth = function getAzimuth() {
      return this.azimuth;
    }
    /**
     * 设置相机方位角，不同相机模式下需要重新计算相机位置或者是视点位置
     */;
    _proto.setElevation = function setElevation(el) {
      this.elevation = getAngle$3(el);
      this.computeMatrix();
      this._getAxes();
      if (this.type === CameraType$1.ORBITING || this.type === CameraType$1.EXPLORING) {
        this._getPosition();
      } else if (this.type === CameraType$1.TRACKING) {
        this._getFocalPoint();
      }
      this.triggerUpdate();
      return this;
    };
    _proto.getElevation = function getElevation() {
      return this.elevation;
    }
    /**
     * 设置相机方位角，不同相机模式下需要重新计算相机位置或者是视点位置
     */;
    _proto.setRoll = function setRoll(angle) {
      this.roll = getAngle$3(angle);
      this.computeMatrix();
      this._getAxes();
      if (this.type === CameraType$1.ORBITING || this.type === CameraType$1.EXPLORING) {
        this._getPosition();
      } else if (this.type === CameraType$1.TRACKING) {
        this._getFocalPoint();
      }
      this.triggerUpdate();
      return this;
    };
    _proto.getRoll = function getRoll() {
      return this.roll;
    }
    /**
     * 根据相机矩阵重新计算各种相机参数
     */;
    _proto._update = function _update() {
      this._getAxes();
      this._getPosition();
      this._getDistance();
      this._getAngles();
      this._getOrthoMatrix();
      this.triggerUpdate();
    }
    /**
     * 计算相机矩阵
     */;
    _proto.computeMatrix = function computeMatrix() {
      // 使用四元数描述 3D 旋转
      // @see https://xiaoiver.github.io/coding/2018/12/28/Camera-%E8%AE%BE%E8%AE%A1-%E4%B8%80.html
      var rotZ = setAxisAngle$1(create$a(), [0, 0, 1], deg2rad$1(this.roll));
      identity$1(this.matrix);
      // only consider HCS for EXPLORING and ORBITING cameras
      var rotX = setAxisAngle$1(create$a(), [1, 0, 0], deg2rad$1((this.rotateWorld && this.type !== CameraType$1.TRACKING || this.type === CameraType$1.TRACKING ? 1 : -1) * this.elevation));
      var rotY = setAxisAngle$1(create$a(), [0, 1, 0], deg2rad$1((this.rotateWorld && this.type !== CameraType$1.TRACKING || this.type === CameraType$1.TRACKING ? 1 : -1) * this.azimuth));
      var rotQ = multiply$5(create$a(), rotY, rotX);
      rotQ = multiply$5(create$a(), rotQ, rotZ);
      var rotMatrix = fromQuat$1(create$7(), rotQ);
      if (this.type === CameraType$1.ORBITING || this.type === CameraType$1.EXPLORING) {
        translate$1(this.matrix, this.matrix, this.focalPoint);
        multiply$3(this.matrix, this.matrix, rotMatrix);
        translate$1(this.matrix, this.matrix, [0, 0, this.distance]);
      } else if (this.type === CameraType$1.TRACKING) {
        translate$1(this.matrix, this.matrix, this.position);
        multiply$3(this.matrix, this.matrix, rotMatrix);
      }
    }
    /**
     * Sets the camera position in the camera matrix
     */;
    _proto._setPosition = function _setPosition(x, y, z) {
      this.position = createVec3$1(x, y, z);
      var m = this.matrix;
      m[12] = this.position[0];
      m[13] = this.position[1];
      m[14] = this.position[2];
      m[15] = 1;
      this._getOrthoMatrix();
    }
    /**
     * Recalculates axes based on the current matrix
     */;
    _proto._getAxes = function _getAxes() {
      copy$6(this.right, createVec3$1(transformMat4$3(create$9(), [1, 0, 0, 0], this.matrix)));
      copy$6(this.up, createVec3$1(transformMat4$3(create$9(), [0, 1, 0, 0], this.matrix)));
      copy$6(this.forward, createVec3$1(transformMat4$3(create$9(), [0, 0, 1, 0], this.matrix)));
      normalize$5(this.right, this.right);
      normalize$5(this.up, this.up);
      normalize$5(this.forward, this.forward);
    }
    /**
     * Recalculates euler angles based on the current state
     */;
    _proto._getAngles = function _getAngles() {
      // Recalculates angles
      var x = this.distanceVector[0];
      var y = this.distanceVector[1];
      var z = this.distanceVector[2];
      var r = length$1(this.distanceVector);
      // FAST FAIL: If there is no distance we cannot compute angles
      if (r === 0) {
        this.elevation = 0;
        this.azimuth = 0;
        return;
      }
      if (this.type === CameraType$1.TRACKING) {
        this.elevation = rad2deg$1(Math.asin(y / r));
        this.azimuth = rad2deg$1(Math.atan2(-x, -z));
      } else {
        if (this.rotateWorld) {
          this.elevation = rad2deg$1(Math.asin(y / r));
          this.azimuth = rad2deg$1(Math.atan2(-x, -z));
        } else {
          this.elevation = -rad2deg$1(Math.asin(y / r));
          this.azimuth = -rad2deg$1(Math.atan2(-x, -z));
        }
      }
    }
    /**
     * 重新计算相机位置，只有 ORBITING 模式相机位置才会发生变化
     */;
    _proto._getPosition = function _getPosition() {
      copy$6(this.position, createVec3$1(transformMat4$3(create$9(), [0, 0, 0, 1], this.matrix)));
      // 相机位置变化，需要重新计算视距
      this._getDistance();
    }
    /**
     * 重新计算视点，只有 TRACKING 模式视点才会发生变化
     */;
    _proto._getFocalPoint = function _getFocalPoint() {
      transformMat3$1(this.distanceVector, [0, 0, -this.distance], fromMat4$1(create$6(), this.matrix));
      add$4(this.focalPoint, this.position, this.distanceVector);
      // 视点变化，需要重新计算视距
      this._getDistance();
    }
    /**
     * 重新计算视距
     */;
    _proto._getDistance = function _getDistance() {
      this.distanceVector = subtract$4(create$8(), this.focalPoint, this.position);
      this.distance = length$1(this.distanceVector);
      this.dollyingStep = this.distance / 100;
    };
    _proto._getOrthoMatrix = function _getOrthoMatrix() {
      if (this.projectionMode !== CameraProjectionMode$1.ORTHOGRAPHIC) {
        return;
      }
      var position = this.position;
      var rotZ = setAxisAngle$1(create$a(), [0, 0, 1], -this.roll * Math.PI / 180);
      fromRotationTranslationScaleOrigin$1(this.orthoMatrix, rotZ, fromValues$8((this.rright - this.left) / 2 - position[0], (this.top - this.bottom) / 2 - position[1], 0), fromValues$8(this.zoom, this.zoom, 1), position);
    };
    _proto.triggerUpdate = function triggerUpdate() {
      if (this.enableUpdate) {
        // update frustum
        var viewMatrix = this.getViewTransform();
        var vpMatrix = multiply$3(create$7(), this.getPerspective(), viewMatrix);
        this.getFrustum().extractFromVPMatrix(vpMatrix);
        this.eventEmitter.emit(CameraEvent$1.UPDATED);
      }
    };
    _proto.rotate = function rotate(azimuth, elevation, roll) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.pan = function pan(tx, ty) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.dolly = function dolly(value) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.createLandmark = function createLandmark(name, params) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    _proto.gotoLandmark = function gotoLandmark(name, options) {
      throw new Error(ERROR_MSG_METHOD_NOT_IMPLEMENTED$1);
    };
    return Camera;
  }();

  var CircleUpdater$1 = /*#__PURE__*/function () {
    function CircleUpdater() {}
    var _proto = CircleUpdater.prototype;
    _proto.update = function update(parsedStyle, object) {
      var r = parsedStyle.r;
      var width = r * 2;
      var height = r * 2;
      return {
        width: width,
        height: height
      };
    };
    return CircleUpdater;
  }();

  var EllipseUpdater$1 = /*#__PURE__*/function () {
    function EllipseUpdater() {}
    var _proto = EllipseUpdater.prototype;
    _proto.update = function update(parsedStyle, object) {
      var rx = parsedStyle.rx,
        ry = parsedStyle.ry;
      var width = rx * 2;
      var height = ry * 2;
      return {
        width: width,
        height: height
      };
    };
    return EllipseUpdater;
  }();

  var LineUpdater$1 = /*#__PURE__*/function () {
    function LineUpdater() {}
    var _proto = LineUpdater.prototype;
    _proto.update = function update(parsedStyle) {
      var x1 = parsedStyle.x1,
        y1 = parsedStyle.y1,
        x2 = parsedStyle.x2,
        y2 = parsedStyle.y2;
      var minX = Math.min(x1, x2);
      var maxX = Math.max(x1, x2);
      var minY = Math.min(y1, y2);
      var maxY = Math.max(y1, y2);
      var width = maxX - minX;
      var height = maxY - minY;
      return {
        width: width,
        height: height
      };
    };
    return LineUpdater;
  }();

  var PathUpdater$1 = /*#__PURE__*/function () {
    function PathUpdater() {}
    var _proto = PathUpdater.prototype;
    _proto.update = function update(parsedStyle) {
      var path = parsedStyle.path;
      var _path$rect = path.rect,
        width = _path$rect.width,
        height = _path$rect.height;
      return {
        width: width,
        height: height
      };
    };
    return PathUpdater;
  }();

  var PolylineUpdater$1 = /*#__PURE__*/function () {
    function PolylineUpdater() {}
    var _proto = PolylineUpdater.prototype;
    _proto.update = function update(parsedStyle) {
      var points = parsedStyle.points.points;
      // FIXME: account for miter lineJoin
      var minX = Math.min.apply(Math, points.map(function (point) {
        return point[0];
      }));
      var maxX = Math.max.apply(Math, points.map(function (point) {
        return point[0];
      }));
      var minY = Math.min.apply(Math, points.map(function (point) {
        return point[1];
      }));
      var maxY = Math.max.apply(Math, points.map(function (point) {
        return point[1];
      }));
      var width = maxX - minX;
      var height = maxY - minY;
      return {
        width: width,
        height: height
      };
    };
    return PolylineUpdater;
  }();

  var RectUpdater$1 = /*#__PURE__*/function () {
    function RectUpdater() {}
    var _proto = RectUpdater.prototype;
    _proto.update = function update(parsedStyle, object) {
      var img = parsedStyle.img,
        _parsedStyle$width = parsedStyle.width,
        width = _parsedStyle$width === void 0 ? 0 : _parsedStyle$width,
        _parsedStyle$height = parsedStyle.height,
        height = _parsedStyle$height === void 0 ? 0 : _parsedStyle$height;
      var contentWidth = width;
      var contentHeight = height;
      // resize with HTMLImageElement's size
      if (img && !isString$1(img)) {
        if (!contentWidth) {
          contentWidth = img.width;
          parsedStyle.width = contentWidth;
        }
        if (!contentHeight) {
          contentHeight = img.height;
          parsedStyle.height = contentHeight;
        }
      }
      return {
        width: contentWidth,
        height: contentHeight
      };
    };
    return RectUpdater;
  }();

  var TextUpdater$1 = /*#__PURE__*/function () {
    function TextUpdater(globalRuntime) {
      this.globalRuntime = void 0;
      this.globalRuntime = globalRuntime;
    }
    var _proto = TextUpdater.prototype;
    _proto.isReadyToMeasure = function isReadyToMeasure(parsedStyle, object) {
      var text = parsedStyle.text,
        textAlign = parsedStyle.textAlign,
        textBaseline = parsedStyle.textBaseline,
        fontSize = parsedStyle.fontSize,
        fontStyle = parsedStyle.fontStyle,
        fontWeight = parsedStyle.fontWeight,
        fontVariant = parsedStyle.fontVariant,
        lineWidth = parsedStyle.lineWidth;
      return text && fontSize && fontStyle && fontWeight && fontVariant && textAlign && textBaseline && !isNil$1(lineWidth);
    };
    _proto.update = function update(parsedStyle, object) {
      var _object$ownerDocument, _object$ownerDocument2;
      var text = parsedStyle.text,
        textAlign = parsedStyle.textAlign,
        lineWidth = parsedStyle.lineWidth,
        textBaseline = parsedStyle.textBaseline,
        dx = parsedStyle.dx,
        dy = parsedStyle.dy;
      var _ref = (object === null || object === void 0 ? void 0 : (_object$ownerDocument = object.ownerDocument) === null || _object$ownerDocument === void 0 ? void 0 : (_object$ownerDocument2 = _object$ownerDocument.defaultView) === null || _object$ownerDocument2 === void 0 ? void 0 : _object$ownerDocument2.getConfig()) || {},
        offscreenCanvas = _ref.offscreenCanvas;
      if (!this.isReadyToMeasure(parsedStyle, object)) {
        parsedStyle.metrics = {
          font: '',
          width: 0,
          height: 0,
          lines: [],
          lineWidths: [],
          lineHeight: 0,
          maxLineWidth: 0,
          fontProperties: {
            ascent: 0,
            descent: 0,
            fontSize: 0
          },
          lineMetrics: []
        };
        return {
          width: 0,
          height: 0,
          x: 0,
          y: 0,
          offsetX: 0,
          offsetY: 0
        };
      }
      var metrics = this.globalRuntime.textService.measureText(text, parsedStyle, offscreenCanvas);
      parsedStyle.metrics = metrics;
      var width = metrics.width,
        height = metrics.height,
        lineHeight = metrics.lineHeight,
        fontProperties = metrics.fontProperties;
      // anchor is left-top by default
      var halfExtents = [width / 2, height / 2, 0];
      // default 'left'
      var anchor = [0, 1];
      var lineXOffset = 0;
      if (textAlign === 'center' || textAlign === 'middle') {
        lineXOffset = lineWidth / 2;
        anchor = [0.5, 1];
      } else if (textAlign === 'right' || textAlign === 'end') {
        lineXOffset = lineWidth;
        anchor = [1, 1];
      }
      var lineYOffset = 0;
      if (textBaseline === 'middle') {
        // eslint-disable-next-line prefer-destructuring
        lineYOffset = halfExtents[1];
      } else if (textBaseline === 'top' || textBaseline === 'hanging') {
        lineYOffset = halfExtents[1] * 2;
      } else if (textBaseline === 'alphabetic') {
        lineYOffset = lineHeight - fontProperties.ascent;
      } else if (textBaseline === 'bottom' || textBaseline === 'ideographic') {
        lineYOffset = 0;
      }
      // TODO: ideographic & bottom
      if (dx) {
        lineXOffset += dx;
      }
      if (dy) {
        lineYOffset += dy;
      }
      // update anchor
      parsedStyle.anchor = [anchor[0], anchor[1], 0];
      return {
        width: halfExtents[0] * 2,
        height: halfExtents[1] * 2,
        offsetX: lineXOffset,
        offsetY: lineYOffset
      };
    };
    return TextUpdater;
  }();

  var PROPAGATION_LIMIT$1 = 2048;
  var EventService$1 = /*#__PURE__*/function () {
    function EventService(globalRuntime, context) {
      var _this = this;
      this.globalRuntime = void 0;
      this.context = void 0;
      this.rootTarget = void 0;
      this.emitter = new eventemitter3$1();
      this.cursor = 'default';
      this.mappingTable = {};
      this.mappingState = {
        trackingData: {}
      };
      this.eventPool = new Map();
      this.pickHandler = void 0;
      this.tmpMatrix = create$7();
      this.tmpVec3 = create$8();
      this.onPointerDown = /*#__PURE__*/function () {
        var _ref = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee(from) {
          var e, isRightButton, trackingData;
          return _regeneratorRuntime$6().wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent$1) {
                    _context.next = 2;
                    break;
                  }
                  return _context.abrupt("return");
                case 2:
                  _context.next = 4;
                  return _this.createPointerEvent(from);
                case 4:
                  e = _context.sent;
                  _this.dispatchEvent(e, 'pointerdown');
                  if (e.pointerType === 'touch') {
                    _this.dispatchEvent(e, 'touchstart');
                  } else if (e.pointerType === 'mouse' || e.pointerType === 'pen') {
                    isRightButton = e.button === 2;
                    _this.dispatchEvent(e, isRightButton ? 'rightdown' : 'mousedown');
                  }
                  trackingData = _this.trackingData(from.pointerId);
                  trackingData.pressTargetsByButton[from.button] = e.composedPath();
                  _this.freeEvent(e);
                case 10:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));
        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }();
      this.onPointerUp = /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee2(from) {
          var now, e, isRightButton, trackingData, pressTarget, clickTarget, currentTarget, _isRightButton, _e$detail, clickEvent, clickHistory;
          return _regeneratorRuntime$6().wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent$1) {
                    _context2.next = 2;
                    break;
                  }
                  return _context2.abrupt("return");
                case 2:
                  now = performance.now();
                  _context2.next = 5;
                  return _this.createPointerEvent(from);
                case 5:
                  e = _context2.sent;
                  _this.dispatchEvent(e, 'pointerup');
                  if (e.pointerType === 'touch') {
                    _this.dispatchEvent(e, 'touchend');
                  } else if (e.pointerType === 'mouse' || e.pointerType === 'pen') {
                    isRightButton = e.button === 2;
                    _this.dispatchEvent(e, isRightButton ? 'rightup' : 'mouseup');
                  }
                  trackingData = _this.trackingData(from.pointerId);
                  pressTarget = _this.findMountedTarget(trackingData.pressTargetsByButton[from.button]);
                  clickTarget = pressTarget; // pointerupoutside only bubbles. It only bubbles upto the parent that doesn't contain
                  // the pointerup location.
                  if (pressTarget && !e.composedPath().includes(pressTarget)) {
                    currentTarget = pressTarget;
                    while (currentTarget && !e.composedPath().includes(currentTarget)) {
                      e.currentTarget = currentTarget;
                      _this.notifyTarget(e, 'pointerupoutside');
                      if (e.pointerType === 'touch') {
                        _this.notifyTarget(e, 'touchendoutside');
                      } else if (e.pointerType === 'mouse' || e.pointerType === 'pen') {
                        _isRightButton = e.button === 2;
                        _this.notifyTarget(e, _isRightButton ? 'rightupoutside' : 'mouseupoutside');
                      }
                      if (Node$2.isNode(currentTarget)) {
                        currentTarget = currentTarget.parentNode;
                      }
                    }
                    delete trackingData.pressTargetsByButton[from.button];
                    // currentTarget is the most specific ancestor holding both the pointerdown and pointerup
                    // targets. That is - it's our click target!
                    clickTarget = currentTarget;
                  }
                  if (clickTarget) {
                    clickEvent = _this.clonePointerEvent(e, 'click');
                    clickEvent.target = clickTarget;
                    clickEvent.path = [];
                    if (!trackingData.clicksByButton[from.button]) {
                      trackingData.clicksByButton[from.button] = {
                        clickCount: 0,
                        target: clickEvent.target,
                        timeStamp: now
                      };
                    }
                    clickHistory = trackingData.clicksByButton[from.button];
                    if (clickHistory.target === clickEvent.target && now - clickHistory.timeStamp < 200) {
                      ++clickHistory.clickCount;
                    } else {
                      clickHistory.clickCount = 1;
                    }
                    clickHistory.target = clickEvent.target;
                    clickHistory.timeStamp = now;
                    clickEvent.detail = clickHistory.clickCount;
                    // @see https://github.com/antvis/G/issues/1091
                    if (!((_e$detail = e.detail) === null || _e$detail === void 0 ? void 0 : _e$detail.preventClick)) {
                      if (clickEvent.pointerType === 'mouse' || clickEvent.pointerType === 'touch') {
                        _this.dispatchEvent(clickEvent, 'click');
                      }
                      _this.dispatchEvent(clickEvent, 'pointertap');
                    }
                    _this.freeEvent(clickEvent);
                  }
                  _this.freeEvent(e);
                case 14:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2);
        }));
        return function (_x2) {
          return _ref2.apply(this, arguments);
        };
      }();
      this.onPointerMove = /*#__PURE__*/function () {
        var _ref3 = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee3(from) {
          var e, isMouse, trackingData, outTarget, outType, outEvent, leaveEvent, overType, overEvent, overTargetAncestor, didPointerEnter, enterEvent;
          return _regeneratorRuntime$6().wrap(function _callee3$(_context3) {
            while (1) {
              switch (_context3.prev = _context3.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent$1) {
                    _context3.next = 2;
                    break;
                  }
                  return _context3.abrupt("return");
                case 2:
                  _context3.next = 4;
                  return _this.createPointerEvent(from);
                case 4:
                  e = _context3.sent;
                  isMouse = e.pointerType === 'mouse' || e.pointerType === 'pen';
                  trackingData = _this.trackingData(from.pointerId);
                  outTarget = _this.findMountedTarget(trackingData.overTargets); // First pointerout/pointerleave
                  if (!(trackingData.overTargets && outTarget !== e.target)) {
                    _context3.next = 23;
                    break;
                  }
                  // pointerout always occurs on the overTarget when the pointer hovers over another element.
                  outType = from.type === 'mousemove' ? 'mouseout' : 'pointerout';
                  _context3.next = 12;
                  return _this.createPointerEvent(from, outType, outTarget || undefined);
                case 12:
                  outEvent = _context3.sent;
                  _this.dispatchEvent(outEvent, 'pointerout');
                  if (isMouse) _this.dispatchEvent(outEvent, 'mouseout');
                  // If the pointer exits overTarget and its descendants, then a pointerleave event is also fired. This event
                  // is dispatched to all ancestors that no longer capture the pointer.
                  if (e.composedPath().includes(outTarget)) {
                    _context3.next = 22;
                    break;
                  }
                  _context3.next = 18;
                  return _this.createPointerEvent(from, 'pointerleave', outTarget || undefined);
                case 18:
                  leaveEvent = _context3.sent;
                  leaveEvent.eventPhase = leaveEvent.AT_TARGET;
                  while (leaveEvent.target && !e.composedPath().includes(leaveEvent.target)) {
                    leaveEvent.currentTarget = leaveEvent.target;
                    _this.notifyTarget(leaveEvent);
                    if (isMouse) {
                      _this.notifyTarget(leaveEvent, 'mouseleave');
                    }
                    if (Node$2.isNode(leaveEvent.target)) {
                      leaveEvent.target = leaveEvent.target.parentNode;
                    }
                  }
                  _this.freeEvent(leaveEvent);
                case 22:
                  _this.freeEvent(outEvent);
                case 23:
                  if (!(outTarget !== e.target)) {
                    _context3.next = 38;
                    break;
                  }
                  // pointerover always occurs on the new overTarget
                  overType = from.type === 'mousemove' ? 'mouseover' : 'pointerover';
                  overEvent = _this.clonePointerEvent(e, overType); // clone faster
                  _this.dispatchEvent(overEvent, 'pointerover');
                  if (isMouse) _this.dispatchEvent(overEvent, 'mouseover');
                  // Probe whether the newly hovered Node is an ancestor of the original overTarget.
                  overTargetAncestor = outTarget && Node$2.isNode(outTarget) && outTarget.parentNode;
                case 29:
                  if (!(overTargetAncestor && overTargetAncestor !== (Node$2.isNode(_this.rootTarget) && _this.rootTarget.parentNode))) {
                    _context3.next = 35;
                    break;
                  }
                  if (!(overTargetAncestor === e.target)) {
                    _context3.next = 32;
                    break;
                  }
                  return _context3.abrupt("break", 35);
                case 32:
                  overTargetAncestor = overTargetAncestor.parentNode;
                  _context3.next = 29;
                  break;
                case 35:
                  // The pointer has entered a non-ancestor of the original overTarget. This means we need a pointerentered
                  // event.
                  didPointerEnter = !overTargetAncestor || overTargetAncestor === (Node$2.isNode(_this.rootTarget) && _this.rootTarget.parentNode);
                  if (didPointerEnter) {
                    enterEvent = _this.clonePointerEvent(e, 'pointerenter');
                    enterEvent.eventPhase = enterEvent.AT_TARGET;
                    while (enterEvent.target && enterEvent.target !== outTarget && enterEvent.target !== (Node$2.isNode(_this.rootTarget) && _this.rootTarget.parentNode)) {
                      enterEvent.currentTarget = enterEvent.target;
                      _this.notifyTarget(enterEvent);
                      if (isMouse) _this.notifyTarget(enterEvent, 'mouseenter');
                      if (Node$2.isNode(enterEvent.target)) {
                        enterEvent.target = enterEvent.target.parentNode;
                      }
                    }
                    _this.freeEvent(enterEvent);
                  }
                  _this.freeEvent(overEvent);
                case 38:
                  // Then pointermove
                  _this.dispatchEvent(e, 'pointermove');
                  if (e.pointerType === 'touch') _this.dispatchEvent(e, 'touchmove');
                  if (isMouse) {
                    _this.dispatchEvent(e, 'mousemove');
                    _this.cursor = _this.getCursor(e.target);
                  }
                  trackingData.overTargets = e.composedPath();
                  _this.freeEvent(e);
                case 43:
                case "end":
                  return _context3.stop();
              }
            }
          }, _callee3);
        }));
        return function (_x3) {
          return _ref3.apply(this, arguments);
        };
      }();
      this.onPointerOut = /*#__PURE__*/function () {
        var _ref4 = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee4(from) {
          var trackingData, isMouse, outTarget, outEvent, leaveEvent;
          return _regeneratorRuntime$6().wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent$1) {
                    _context4.next = 2;
                    break;
                  }
                  return _context4.abrupt("return");
                case 2:
                  trackingData = _this.trackingData(from.pointerId);
                  if (!trackingData.overTargets) {
                    _context4.next = 19;
                    break;
                  }
                  isMouse = from.pointerType === 'mouse' || from.pointerType === 'pen';
                  outTarget = _this.findMountedTarget(trackingData.overTargets); // pointerout first
                  _context4.next = 8;
                  return _this.createPointerEvent(from, 'pointerout', outTarget || undefined);
                case 8:
                  outEvent = _context4.sent;
                  _this.dispatchEvent(outEvent);
                  if (isMouse) _this.dispatchEvent(outEvent, 'mouseout');
                  // pointerleave(s) are also dispatched b/c the pointer must've left rootTarget and its descendants to
                  // get an upstream pointerout event (upstream events do not know rootTarget has descendants).
                  _context4.next = 13;
                  return _this.createPointerEvent(from, 'pointerleave', outTarget || undefined);
                case 13:
                  leaveEvent = _context4.sent;
                  leaveEvent.eventPhase = leaveEvent.AT_TARGET;
                  while (leaveEvent.target && leaveEvent.target !== (Node$2.isNode(_this.rootTarget) && _this.rootTarget.parentNode)) {
                    leaveEvent.currentTarget = leaveEvent.target;
                    _this.notifyTarget(leaveEvent);
                    if (isMouse) {
                      _this.notifyTarget(leaveEvent, 'mouseleave');
                    }
                    if (Node$2.isNode(leaveEvent.target)) {
                      leaveEvent.target = leaveEvent.target.parentNode;
                    }
                  }
                  trackingData.overTargets = null;
                  _this.freeEvent(outEvent);
                  _this.freeEvent(leaveEvent);
                case 19:
                  _this.cursor = null;
                case 20:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4);
        }));
        return function (_x4) {
          return _ref4.apply(this, arguments);
        };
      }();
      this.onPointerOver = /*#__PURE__*/function () {
        var _ref5 = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee5(from) {
          var trackingData, e, isMouse, enterEvent;
          return _regeneratorRuntime$6().wrap(function _callee5$(_context5) {
            while (1) {
              switch (_context5.prev = _context5.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent$1) {
                    _context5.next = 2;
                    break;
                  }
                  return _context5.abrupt("return");
                case 2:
                  trackingData = _this.trackingData(from.pointerId);
                  _context5.next = 5;
                  return _this.createPointerEvent(from);
                case 5:
                  e = _context5.sent;
                  isMouse = e.pointerType === 'mouse' || e.pointerType === 'pen';
                  _this.dispatchEvent(e, 'pointerover');
                  if (isMouse) _this.dispatchEvent(e, 'mouseover');
                  if (e.pointerType === 'mouse') _this.cursor = _this.getCursor(e.target);
                  // pointerenter events must be fired since the pointer entered from upstream.
                  enterEvent = _this.clonePointerEvent(e, 'pointerenter');
                  enterEvent.eventPhase = enterEvent.AT_TARGET;
                  while (enterEvent.target && enterEvent.target !== (Node$2.isNode(_this.rootTarget) && _this.rootTarget.parentNode)) {
                    enterEvent.currentTarget = enterEvent.target;
                    _this.notifyTarget(enterEvent);
                    if (isMouse) {
                      // mouseenter should not bubble
                      // @see https://developer.mozilla.org/en-US/docs/Web/API/Element/mouseenter_event#usage_notes
                      _this.notifyTarget(enterEvent, 'mouseenter');
                    }
                    if (Node$2.isNode(enterEvent.target)) {
                      enterEvent.target = enterEvent.target.parentNode;
                    }
                  }
                  trackingData.overTargets = e.composedPath();
                  _this.freeEvent(e);
                  _this.freeEvent(enterEvent);
                case 16:
                case "end":
                  return _context5.stop();
              }
            }
          }, _callee5);
        }));
        return function (_x5) {
          return _ref5.apply(this, arguments);
        };
      }();
      this.onPointerUpOutside = /*#__PURE__*/function () {
        var _ref6 = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee6(from) {
          var trackingData, pressTarget, e, currentTarget;
          return _regeneratorRuntime$6().wrap(function _callee6$(_context6) {
            while (1) {
              switch (_context6.prev = _context6.next) {
                case 0:
                  if (from instanceof FederatedPointerEvent$1) {
                    _context6.next = 2;
                    break;
                  }
                  return _context6.abrupt("return");
                case 2:
                  trackingData = _this.trackingData(from.pointerId);
                  pressTarget = _this.findMountedTarget(trackingData.pressTargetsByButton[from.button]);
                  _context6.next = 6;
                  return _this.createPointerEvent(from);
                case 6:
                  e = _context6.sent;
                  if (pressTarget) {
                    currentTarget = pressTarget;
                    while (currentTarget) {
                      e.currentTarget = currentTarget;
                      _this.notifyTarget(e, 'pointerupoutside');
                      if (e.pointerType === 'touch') ; else if (e.pointerType === 'mouse' || e.pointerType === 'pen') {
                        _this.notifyTarget(e, e.button === 2 ? 'rightupoutside' : 'mouseupoutside');
                      }
                      if (Node$2.isNode(currentTarget)) {
                        currentTarget = currentTarget.parentNode;
                      }
                    }
                    delete trackingData.pressTargetsByButton[from.button];
                  }
                  _this.freeEvent(e);
                case 9:
                case "end":
                  return _context6.stop();
              }
            }
          }, _callee6);
        }));
        return function (_x6) {
          return _ref6.apply(this, arguments);
        };
      }();
      this.onWheel = /*#__PURE__*/function () {
        var _ref7 = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee7(from) {
          var wheelEvent;
          return _regeneratorRuntime$6().wrap(function _callee7$(_context7) {
            while (1) {
              switch (_context7.prev = _context7.next) {
                case 0:
                  if (from instanceof FederatedWheelEvent$1) {
                    _context7.next = 2;
                    break;
                  }
                  return _context7.abrupt("return");
                case 2:
                  _context7.next = 4;
                  return _this.createWheelEvent(from);
                case 4:
                  wheelEvent = _context7.sent;
                  _this.dispatchEvent(wheelEvent);
                  _this.freeEvent(wheelEvent);
                case 7:
                case "end":
                  return _context7.stop();
              }
            }
          }, _callee7);
        }));
        return function (_x7) {
          return _ref7.apply(this, arguments);
        };
      }();
      this.globalRuntime = globalRuntime;
      this.context = context;
    }
    var _proto = EventService.prototype;
    _proto.init = function init() {
      this.rootTarget = this.context.renderingContext.root.parentNode; // document
      this.addEventMapping('pointerdown', this.onPointerDown);
      this.addEventMapping('pointerup', this.onPointerUp);
      this.addEventMapping('pointermove', this.onPointerMove);
      this.addEventMapping('pointerout', this.onPointerOut);
      this.addEventMapping('pointerleave', this.onPointerOut);
      this.addEventMapping('pointerover', this.onPointerOver);
      this.addEventMapping('pointerupoutside', this.onPointerUpOutside);
      this.addEventMapping('wheel', this.onWheel);
    };
    _proto.client2Viewport = function client2Viewport(client) {
      var bbox = this.context.contextService.getBoundingClientRect();
      return new Point$1(client.x - ((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0), client.y - ((bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0));
    };
    _proto.viewport2Client = function viewport2Client(canvas) {
      var bbox = this.context.contextService.getBoundingClientRect();
      return new Point$1(canvas.x + ((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0), canvas.y + ((bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0));
    };
    _proto.viewport2Canvas = function viewport2Canvas(_ref8) {
      var x = _ref8.x,
        y = _ref8.y;
      var canvas = this.rootTarget.defaultView;
      var camera = canvas.getCamera();
      var _this$context$config = this.context.config,
        width = _this$context$config.width,
        height = _this$context$config.height;
      var projectionMatrixInverse = camera.getPerspectiveInverse();
      var worldMatrix = camera.getWorldTransform();
      var vpMatrix = multiply$3(this.tmpMatrix, worldMatrix, projectionMatrixInverse);
      var viewport = set$3(this.tmpVec3, x / width * 2 - 1, (1 - y / height) * 2 - 1, 0);
      transformMat4$2(viewport, viewport, vpMatrix);
      return new Point$1(viewport[0], viewport[1]);
    };
    _proto.canvas2Viewport = function canvas2Viewport(canvasP) {
      var canvas = this.rootTarget.defaultView;
      var camera = canvas.getCamera();
      // World -> Clip
      var projectionMatrix = camera.getPerspective();
      var viewMatrix = camera.getViewTransform();
      var vpMatrix = multiply$3(this.tmpMatrix, projectionMatrix, viewMatrix);
      var clip = set$3(this.tmpVec3, canvasP.x, canvasP.y, 0);
      transformMat4$2(this.tmpVec3, this.tmpVec3, vpMatrix);
      // Clip -> NDC -> Viewport, flip Y
      var _this$context$config2 = this.context.config,
        width = _this$context$config2.width,
        height = _this$context$config2.height;
      return new Point$1((clip[0] + 1) / 2 * width, (1 - (clip[1] + 1) / 2) * height);
    };
    _proto.setPickHandler = function setPickHandler(pickHandler) {
      this.pickHandler = pickHandler;
    };
    _proto.addEventMapping = function addEventMapping(type, fn) {
      if (!this.mappingTable[type]) {
        this.mappingTable[type] = [];
      }
      this.mappingTable[type].push({
        fn: fn,
        priority: 0
      });
      this.mappingTable[type].sort(function (a, b) {
        return a.priority - b.priority;
      });
    };
    _proto.mapEvent = function mapEvent(e) {
      if (!this.rootTarget) {
        return;
      }
      var mappers = this.mappingTable[e.type];
      if (mappers) {
        for (var i = 0, j = mappers.length; i < j; i++) {
          mappers[i].fn(e);
        }
      } else {
        console.warn("[EventService]: Event mapping not defined for " + e.type);
      }
    };
    _proto.dispatchEvent = function dispatchEvent(e, type, skipPropagate) {
      // Canvas should skip
      if (!skipPropagate) {
        e.propagationStopped = false;
        e.propagationImmediatelyStopped = false;
        this.propagate(e, type);
      } else {
        // target phase
        e.eventPhase = e.AT_TARGET;
        var canvas = this.rootTarget.defaultView || null;
        e.currentTarget = canvas;
        this.notifyListeners(e, type);
      }
      this.emitter.emit(type || e.type, e);
    };
    _proto.propagate = function propagate(e, type) {
      if (!e.target) {
        return;
      }
      // [target, parent, root, Canvas]
      var composedPath = e.composedPath();
      // event flow: capture -> target -> bubbling
      // capture phase
      e.eventPhase = e.CAPTURING_PHASE;
      for (var i = composedPath.length - 1; i >= 1; i--) {
        e.currentTarget = composedPath[i];
        this.notifyTarget(e, type);
        if (e.propagationStopped || e.propagationImmediatelyStopped) return;
      }
      // target phase
      e.eventPhase = e.AT_TARGET;
      e.currentTarget = e.target;
      this.notifyTarget(e, type);
      if (e.propagationStopped || e.propagationImmediatelyStopped) return;
      // find current target in composed path
      var index = composedPath.indexOf(e.currentTarget);
      // bubbling phase
      e.eventPhase = e.BUBBLING_PHASE;
      for (var _i = index + 1; _i < composedPath.length; _i++) {
        e.currentTarget = composedPath[_i];
        this.notifyTarget(e, type);
        if (e.propagationStopped || e.propagationImmediatelyStopped) return;
      }
    };
    _proto.propagationPath = function propagationPath(target) {
      var propagationPath = [target];
      var canvas = this.rootTarget.defaultView || null;
      if (canvas && canvas === target) {
        propagationPath.unshift(canvas.document);
        return propagationPath;
      }
      for (var i = 0; i < PROPAGATION_LIMIT$1 && target !== this.rootTarget; i++) {
        // if (Node.isNode(target) && !target.parentNode) {
        //   throw new Error('Cannot find propagation path to disconnected target');
        // }
        if (Node$2.isNode(target) && target.parentNode) {
          // [target, parent, parent, root]
          propagationPath.push(target.parentNode);
          target = target.parentNode;
        }
      }
      if (canvas) {
        // @ts-ignore
        propagationPath.push(canvas);
      }
      return propagationPath;
    };
    _proto.hitTest = /*#__PURE__*/function () {
      var _hitTest = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee8(position) {
        var viewportX, viewportY, _this$context$config3, width, height;
        return _regeneratorRuntime$6().wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                viewportX = position.viewportX, viewportY = position.viewportY;
                _this$context$config3 = this.context.config, width = _this$context$config3.width, height = _this$context$config3.height; // outside canvas
                if (!(viewportX < 0 || viewportY < 0 || viewportX > width || viewportY > height)) {
                  _context8.next = 4;
                  break;
                }
                return _context8.abrupt("return", null);
              case 4:
                _context8.next = 6;
                return this.pickHandler(position);
              case 6:
                _context8.t1 = _context8.sent;
                if (_context8.t1) {
                  _context8.next = 9;
                  break;
                }
                _context8.t1 = this.rootTarget;
              case 9:
                _context8.t0 = _context8.t1;
                if (_context8.t0) {
                  _context8.next = 12;
                  break;
                }
                _context8.t0 =
                // return Document
                null;
              case 12:
                return _context8.abrupt("return", _context8.t0);
              case 13:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));
      function hitTest(_x8) {
        return _hitTest.apply(this, arguments);
      }
      return hitTest;
    }()
    /**
     * whether the native event trigger came from Canvas,
     * should account for HTML shape
     */
    ;
    _proto.isNativeEventFromCanvas = function isNativeEventFromCanvas(event) {
      var _event$nativeEvent;
      var $el = this.context.contextService.getDomElement();
      var target = (_event$nativeEvent = event.nativeEvent) === null || _event$nativeEvent === void 0 ? void 0 : _event$nativeEvent.target;
      if (target) {
        // from <canvas>
        if (target === $el) {
          return true;
        }
        // from <svg>
        if ($el && $el.contains) {
          return $el.contains(target);
        }
      }
      if (event.nativeEvent.composedPath) {
        return event.nativeEvent.composedPath().indexOf($el) > -1;
      }
      // account for Touch
      return false;
    };
    _proto.getExistedHTML = function getExistedHTML(event) {
      if (event.nativeEvent.composedPath) {
        var htmls = this.globalRuntime.displayObjectPool.getHTMLs();
        for (var _iterator = _createForOfIteratorHelperLoose$2(htmls), _step; !(_step = _iterator()).done;) {
          var html = _step.value;
          if (event.nativeEvent.composedPath().indexOf(html) > -1) {
            return html;
          }
        }
      }
      return null;
    };
    _proto.pickTarget = /*#__PURE__*/function () {
      var _pickTarget = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee9(event) {
        return _regeneratorRuntime$6().wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                return _context9.abrupt("return", this.hitTest({
                  clientX: event.clientX,
                  clientY: event.clientY,
                  viewportX: event.viewportX,
                  viewportY: event.viewportY,
                  x: event.canvasX,
                  y: event.canvasY
                }));
              case 1:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));
      function pickTarget(_x9) {
        return _pickTarget.apply(this, arguments);
      }
      return pickTarget;
    }();
    _proto.createPointerEvent = /*#__PURE__*/function () {
      var _createPointerEvent = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee10(from, type, target) {
        var event, existedHTML;
        return _regeneratorRuntime$6().wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                event = this.allocateEvent(FederatedPointerEvent$1);
                this.copyPointerData(from, event);
                this.copyMouseData(from, event);
                this.copyData(from, event);
                event.nativeEvent = from.nativeEvent;
                event.originalEvent = from;
                existedHTML = this.getExistedHTML(event);
                if (!(target !== null && target !== void 0)) {
                  _context10.next = 11;
                  break;
                }
                _context10.t0 = target;
                _context10.next = 20;
                break;
              case 11:
                _context10.t1 = existedHTML;
                if (_context10.t1) {
                  _context10.next = 19;
                  break;
                }
                _context10.t2 = this.isNativeEventFromCanvas(event);
                if (!_context10.t2) {
                  _context10.next = 18;
                  break;
                }
                _context10.next = 17;
                return this.pickTarget(event);
              case 17:
                _context10.t2 = _context10.sent;
              case 18:
                _context10.t1 = _context10.t2;
              case 19:
                _context10.t0 = _context10.t1;
              case 20:
                event.target = _context10.t0;
                if (typeof type === 'string') {
                  event.type = type;
                }
                return _context10.abrupt("return", event);
              case 23:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));
      function createPointerEvent(_x10, _x11, _x12) {
        return _createPointerEvent.apply(this, arguments);
      }
      return createPointerEvent;
    }();
    _proto.createWheelEvent = /*#__PURE__*/function () {
      var _createWheelEvent = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee11(from) {
        var event, existedHTML;
        return _regeneratorRuntime$6().wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                event = this.allocateEvent(FederatedWheelEvent$1);
                this.copyWheelData(from, event);
                this.copyMouseData(from, event);
                this.copyData(from, event);
                event.nativeEvent = from.nativeEvent;
                event.originalEvent = from;
                existedHTML = this.getExistedHTML(event);
                _context11.t0 = existedHTML;
                if (_context11.t0) {
                  _context11.next = 15;
                  break;
                }
                _context11.t1 = this.isNativeEventFromCanvas(event);
                if (!_context11.t1) {
                  _context11.next = 14;
                  break;
                }
                _context11.next = 13;
                return this.pickTarget(event);
              case 13:
                _context11.t1 = _context11.sent;
              case 14:
                _context11.t0 = _context11.t1;
              case 15:
                event.target = _context11.t0;
                return _context11.abrupt("return", event);
              case 17:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));
      function createWheelEvent(_x13) {
        return _createWheelEvent.apply(this, arguments);
      }
      return createWheelEvent;
    }();
    _proto.trackingData = function trackingData(id) {
      if (!this.mappingState.trackingData[id]) {
        this.mappingState.trackingData[id] = {
          pressTargetsByButton: {},
          clicksByButton: {},
          overTarget: null
        };
      }
      return this.mappingState.trackingData[id];
    };
    _proto.cloneWheelEvent = function cloneWheelEvent(from) {
      var event = this.allocateEvent(FederatedWheelEvent$1);
      event.nativeEvent = from.nativeEvent;
      event.originalEvent = from.originalEvent;
      this.copyWheelData(from, event);
      this.copyMouseData(from, event);
      this.copyData(from, event);
      event.target = from.target;
      event.path = from.composedPath().slice();
      event.type = from.type;
      return event;
    };
    _proto.clonePointerEvent = function clonePointerEvent(from, type) {
      var event = this.allocateEvent(FederatedPointerEvent$1);
      event.nativeEvent = from.nativeEvent;
      event.originalEvent = from.originalEvent;
      this.copyPointerData(from, event);
      this.copyMouseData(from, event);
      this.copyData(from, event);
      event.target = from.target;
      event.path = from.composedPath().slice();
      event.type = type !== null && type !== void 0 ? type : event.type;
      return event;
    };
    _proto.copyPointerData = function copyPointerData(from, to) {
      if (!(from instanceof FederatedPointerEvent$1 && to instanceof FederatedPointerEvent$1)) return;
      to.pointerId = from.pointerId;
      to.width = from.width;
      to.height = from.height;
      to.isPrimary = from.isPrimary;
      to.pointerType = from.pointerType;
      to.pressure = from.pressure;
      to.tangentialPressure = from.tangentialPressure;
      to.tiltX = from.tiltX;
      to.tiltY = from.tiltY;
      to.twist = from.twist;
    };
    _proto.copyMouseData = function copyMouseData(from, to) {
      if (!(from instanceof FederatedMouseEvent$1 && to instanceof FederatedMouseEvent$1)) return;
      to.altKey = from.altKey;
      to.button = from.button;
      to.buttons = from.buttons;
      to.ctrlKey = from.ctrlKey;
      to.metaKey = from.metaKey;
      to.shiftKey = from.shiftKey;
      to.client.copyFrom(from.client);
      to.movement.copyFrom(from.movement);
      to.canvas.copyFrom(from.canvas);
      to.screen.copyFrom(from.screen);
      to.global.copyFrom(from.global);
      to.offset.copyFrom(from.offset);
    };
    _proto.copyWheelData = function copyWheelData(from, to) {
      to.deltaMode = from.deltaMode;
      to.deltaX = from.deltaX;
      to.deltaY = from.deltaY;
      to.deltaZ = from.deltaZ;
    };
    _proto.copyData = function copyData(from, to) {
      to.isTrusted = from.isTrusted;
      to.timeStamp = performance.now();
      to.type = from.type;
      to.detail = from.detail;
      to.view = from.view;
      to.page.copyFrom(from.page);
      to.viewport.copyFrom(from.viewport);
    };
    _proto.allocateEvent = function allocateEvent(constructor) {
      if (!this.eventPool.has(constructor)) {
        this.eventPool.set(constructor, []);
      }
      // @ts-ignore
      var event = this.eventPool.get(constructor).pop() || new constructor(this);
      event.eventPhase = event.NONE;
      event.currentTarget = null;
      event.path = [];
      event.target = null;
      return event;
    };
    _proto.freeEvent = function freeEvent(event) {
      if (event.manager !== this) throw new Error('It is illegal to free an event not managed by this EventBoundary!');
      var constructor = event.constructor;
      if (!this.eventPool.has(constructor)) {
        this.eventPool.set(constructor, []);
      }
      // @ts-ignore
      this.eventPool.get(constructor).push(event);
    };
    _proto.notifyTarget = function notifyTarget(e, type) {
      var _type;
      type = (_type = type) !== null && _type !== void 0 ? _type : e.type;
      var key = e.eventPhase === e.CAPTURING_PHASE || e.eventPhase === e.AT_TARGET ? type + "capture" : type;
      this.notifyListeners(e, key);
      if (e.eventPhase === e.AT_TARGET) {
        this.notifyListeners(e, type);
      }
    };
    _proto.notifyListeners = function notifyListeners(e, type) {
      // hack EventEmitter, stops if the `propagationImmediatelyStopped` flag is set
      // @ts-ignore
      var emitter = e.currentTarget.emitter;
      // @ts-ignore
      var listeners = emitter._events[type];
      if (!listeners) return;
      if ('fn' in listeners) {
        if (listeners.once) {
          emitter.removeListener(type, listeners.fn, undefined, true);
        }
        listeners.fn.call(e.currentTarget || listeners.context, e);
        // listeners.fn.call(listeners.context, e);
      } else {
        for (var i = 0; i < listeners.length && !e.propagationImmediatelyStopped; i++) {
          if (listeners[i].once) {
            emitter.removeListener(type, listeners[i].fn, undefined, true);
          }
          listeners[i].fn.call(e.currentTarget || listeners[i].context, e);
          // listeners[i].fn.call(listeners[i].context, e);
        }
      }
    }
    /**
     * some detached nodes may exist in propagation path, need to skip them
     */;
    _proto.findMountedTarget = function findMountedTarget(propagationPath) {
      if (!propagationPath) {
        return null;
      }
      var currentTarget = propagationPath[propagationPath.length - 1];
      for (var i = propagationPath.length - 2; i >= 0; i--) {
        var target = propagationPath[i];
        if (target === this.rootTarget || Node$2.isNode(target) && target.parentNode === currentTarget) {
          currentTarget = propagationPath[i];
        } else {
          break;
        }
      }
      return currentTarget;
    };
    _proto.getCursor = function getCursor(target) {
      var tmp = target;
      while (tmp) {
        var cursor = Element$1.isElement(tmp) && tmp.getAttribute('cursor');
        if (cursor) {
          return cursor;
        }
        tmp = Node$2.isNode(tmp) && tmp.parentNode;
      }
    };
    return EventService;
  }();

  /**
   * used in following scenes:
   * - g `ctx.measureText`
   * - g-plugin-canvas-picker `ctx.isPointInPath`
   * - g-plugin-device-renderer `ctx.createLinearGradient` and generate texture
   *
   * @see https://blog.scottlogic.com/2020/03/19/offscreen-canvas.html
   */
  var OffscreenCanvasCreator$1 = /*#__PURE__*/function () {
    function OffscreenCanvasCreator() {
      this.canvas = void 0;
      this.context = void 0;
    }
    var _proto = OffscreenCanvasCreator.prototype;
    _proto.getOrCreateCanvas = function getOrCreateCanvas(offscreenCanvas) {
      if (this.canvas) {
        return this.canvas;
      }
      // user-defined offscreen canvas
      if (offscreenCanvas) {
        this.canvas = offscreenCanvas;
        this.context = this.canvas.getContext('2d');
      } else {
        try {
          // OffscreenCanvas2D measureText can be up to 40% faster.
          this.canvas = new window.OffscreenCanvas(0, 0);
          this.context = this.canvas.getContext('2d');
          if (!this.context || !this.context.measureText) {
            this.canvas = document.createElement('canvas');
            this.context = this.canvas.getContext('2d');
          }
        } catch (ex) {
          this.canvas = document.createElement('canvas');
          this.context = this.canvas.getContext('2d');
        }
      }
      this.canvas.width = 10;
      this.canvas.height = 10;
      return this.canvas;
    };
    _proto.getOrCreateContext = function getOrCreateContext(offscreenCanvas) {
      if (this.context) {
        return this.context;
      }
      this.getOrCreateCanvas(offscreenCanvas);
      return this.context;
    };
    return OffscreenCanvasCreator;
  }();

  /**
   * why we need re-render
   */
  var RenderReason$1;
  (function (RenderReason) {
    RenderReason[RenderReason["CAMERA_CHANGED"] = 0] = "CAMERA_CHANGED";
    RenderReason[RenderReason["DISPLAY_OBJECT_CHANGED"] = 1] = "DISPLAY_OBJECT_CHANGED";
    RenderReason[RenderReason["NONE"] = 2] = "NONE";
  })(RenderReason$1 || (RenderReason$1 = {}));

  /**
   * Use frame renderer implemented by `g-canvas/svg/webgl`, in every frame we do followings:
   * * update & merge dirty rectangles
   * * begin frame
   * * filter by visible
   * * sort by z-index in scene graph
   * * culling with strategies registered in `g-canvas/webgl`
   * * end frame
   */
  var RenderingService$1 = /*#__PURE__*/function () {
    function RenderingService(globalRuntime, context) {
      this.globalRuntime = void 0;
      this.context = void 0;
      this.inited = false;
      this.stats = {
        /**
         * total display objects in scenegraph
         */
        total: 0,
        /**
         * number of display objects need to render in current frame
         */
        rendered: 0
      };
      this.zIndexCounter = 0;
      this.renderOrderChangedEvent = new CustomEvent$1(ElementEvent$1.RENDER_ORDER_CHANGED);
      this.hooks = {
        /**
         * called before any frame rendered
         */
        init: new AsyncParallelHook$1(),
        /**
         * only dirty object which has sth changed will be rendered
         */
        dirtycheck: new SyncWaterfallHook$1(['object']),
        /**
         * do culling
         */
        cull: new SyncWaterfallHook$1(['object', 'camera']),
        /**
         * called at beginning of each frame, won't get called if nothing to re-render
         */
        beginFrame: new SyncHook$1([]),
        /**
         * called before every dirty object get rendered
         */
        beforeRender: new SyncHook$1(['objectToRender']),
        /**
         * called when every dirty object rendering even it's culled
         */
        render: new SyncHook$1(['objectToRender']),
        /**
         * called after every dirty object get rendered
         */
        afterRender: new SyncHook$1(['objectToRender']),
        endFrame: new SyncHook$1([]),
        destroy: new SyncHook$1([]),
        /**
         * use async but faster method such as GPU-based picking in `g-plugin-device-renderer`
         */
        pick: new AsyncSeriesWaterfallHook$1(['result']),
        /**
         * Unsafe but sync version of pick.
         */
        pickSync: new SyncWaterfallHook$1(['result']),
        /**
         * used in event system
         */
        pointerDown: new SyncHook$1(['event']),
        pointerUp: new SyncHook$1(['event']),
        pointerMove: new SyncHook$1(['event']),
        pointerOut: new SyncHook$1(['event']),
        pointerOver: new SyncHook$1(['event']),
        pointerWheel: new SyncHook$1(['event']),
        pointerCancel: new SyncHook$1(['event'])
      };
      this.globalRuntime = globalRuntime;
      this.context = context;
    }
    var _proto = RenderingService.prototype;
    _proto.init = /*#__PURE__*/function () {
      var _init = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee() {
        var context;
        return _regeneratorRuntime$6().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                context = _extends$3({}, this.globalRuntime, this.context); // register rendering plugins
                this.context.renderingPlugins.forEach(function (plugin) {
                  plugin.apply(context);
                });
                // await this.hooks.init.callPromise();
                _context.next = 4;
                return this.hooks.init.promise();
              case 4:
                this.inited = true;
              case 5:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
      function init() {
        return _init.apply(this, arguments);
      }
      return init;
    }();
    _proto.getStats = function getStats() {
      return this.stats;
    }
    /**
     * Meet the following conditions:
     * * disable DirtyRectangleRendering
     * * camera changed
     */;
    _proto.disableDirtyRectangleRendering = function disableDirtyRectangleRendering() {
      var renderer = this.context.config.renderer;
      var _renderer$getConfig = renderer.getConfig(),
        enableDirtyRectangleRendering = _renderer$getConfig.enableDirtyRectangleRendering;
      return !enableDirtyRectangleRendering || this.context.renderingContext.renderReasons.has(RenderReason$1.CAMERA_CHANGED);
    };
    _proto.render = function render(canvasConfig) {
      var _this = this;
      this.stats.total = 0;
      this.stats.rendered = 0;
      this.zIndexCounter = 0;
      var renderingContext = this.context.renderingContext;
      this.globalRuntime.sceneGraphService.syncHierarchy(renderingContext.root);
      this.globalRuntime.sceneGraphService.triggerPendingEvents();
      if (renderingContext.renderReasons.size && this.inited) {
        this.renderDisplayObject(renderingContext.root, canvasConfig, renderingContext);
        this.hooks.beginFrame.call();
        renderingContext.renderListCurrentFrame.forEach(function (object) {
          _this.hooks.beforeRender.call(object);
          _this.hooks.render.call(object);
          _this.hooks.afterRender.call(object);
        });
        this.hooks.endFrame.call();
        renderingContext.renderListCurrentFrame = [];
        renderingContext.renderReasons.clear();
      }
      // console.log('stats', this.stats);
    };
    _proto.renderDisplayObject = function renderDisplayObject(displayObject, canvasConfig, renderingContext) {
      var _this2 = this;
      var _canvasConfig$rendere = canvasConfig.renderer.getConfig(),
        enableDirtyCheck = _canvasConfig$rendere.enableDirtyCheck,
        enableCulling = _canvasConfig$rendere.enableCulling;
      // recalc style values
      this.globalRuntime.styleValueRegistry.recalc(displayObject);
      // TODO: relayout
      // dirtycheck first
      var objectChanged = enableDirtyCheck ? this.hooks.dirtycheck.call(displayObject) : displayObject;
      if (objectChanged) {
        var objectToRender = enableCulling ? this.hooks.cull.call(objectChanged, this.context.camera) : objectChanged;
        if (objectToRender) {
          this.stats.rendered++;
          renderingContext.renderListCurrentFrame.push(objectToRender);
        }
      }
      displayObject.renderable.dirty = false;
      displayObject.sortable.renderOrder = this.zIndexCounter++;
      this.stats.total++;
      // sort is very expensive, use cached result if posible
      var sortable = displayObject.sortable;
      var renderOrderChanged = false;
      if (sortable.dirty) {
        sortable.sorted = displayObject.childNodes.slice().sort(sortByZIndex$1);
        renderOrderChanged = true;
        sortable.dirty = false;
      }
      // recursive rendering its children
      (sortable.sorted || displayObject.childNodes).forEach(function (child) {
        _this2.renderDisplayObject(child, canvasConfig, renderingContext);
      });
      if (renderOrderChanged) {
        displayObject.forEach(function (child) {
          _this2.renderOrderChangedEvent.target = child;
          _this2.renderOrderChangedEvent.detail = {
            renderOrder: child.sortable.renderOrder
          };
          child.ownerDocument.defaultView.dispatchEvent(_this2.renderOrderChangedEvent, true);
        });
      }
    };
    _proto.destroy = function destroy() {
      this.inited = false;
      this.hooks.destroy.call();
    };
    _proto.dirtify = function dirtify() {
      // need re-render
      this.context.renderingContext.renderReasons.add(RenderReason$1.DISPLAY_OBJECT_CHANGED);
    };
    return RenderingService;
  }();

  var ATTRIBUTE_REGEXP$1 = /\[\s*(.*)=(.*)\s*\]/;
  /**
   * support the following DOM API:
   * * getElementById
   * * getElementsByClassName
   * * getElementsByName
   * * getElementsByTag
   * * querySelector
   * * querySelectorAll
   */
  var DefaultSceneGraphSelector$1 = /*#__PURE__*/function () {
    function DefaultSceneGraphSelector() {}
    var _proto = DefaultSceneGraphSelector.prototype;
    _proto.selectOne = function selectOne(query, root) {
      var _this = this;
      if (query.startsWith('.')) {
        return root.find(function (node) {
          // return !node.shadow && node.id === query.substring(1);
          return ((node === null || node === void 0 ? void 0 : node.classList) || []).indexOf(_this.getIdOrClassname(query)) > -1;
        });
      } else if (query.startsWith('#')) {
        // getElementById('id')
        return root.find(function (node) {
          // return !node.shadow && node.id === query.substring(1);
          return node.id === _this.getIdOrClassname(query);
        });
      } else if (query.startsWith('[')) {
        var _this$getAttribute = this.getAttribute(query),
          name = _this$getAttribute.name,
          value = _this$getAttribute.value;
        if (name) {
          // getElementByName();
          return root.find(function (node) {
            return root !== node && (name === 'name' ? node.name === value : _this.attributeToString(node, name) === value);
          });
        } else {
          return null;
        }
      } else {
        // getElementsByTag('circle');
        return root.find(function (node) {
          return root !== node && node.nodeName === query;
        });
      }
    };
    _proto.selectAll = function selectAll(query, root) {
      var _this2 = this;
      // only support `[name="${name}"]` `.className` `#id`
      if (query.startsWith('.')) {
        // getElementsByClassName('className');
        // should not include itself
        return root.findAll(function (node) {
          return root !== node && ((node === null || node === void 0 ? void 0 : node.classList) || []).indexOf(_this2.getIdOrClassname(query)) > -1;
        });
      } else if (query.startsWith('#')) {
        return root.findAll(function (node) {
          return root !== node && node.id === _this2.getIdOrClassname(query);
        });
      } else if (query.startsWith('[')) {
        var _this$getAttribute2 = this.getAttribute(query),
          name = _this$getAttribute2.name,
          value = _this$getAttribute2.value;
        if (name) {
          // getElementsByName();
          return root.findAll(function (node) {
            return root !== node && (name === 'name' ? node.name === value : _this2.attributeToString(node, name) === value);
          });
        } else {
          return [];
        }
      } else {
        // getElementsByTag('circle');
        return root.findAll(function (node) {
          return root !== node && node.nodeName === query;
        });
      }
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ;
    _proto.is = function is(query, node) {
      // a simple `matches` implementation
      if (query.startsWith('.')) {
        return node.className === this.getIdOrClassname(query);
      } else if (query.startsWith('#')) {
        return node.id === this.getIdOrClassname(query);
      } else if (query.startsWith('[')) {
        var _this$getAttribute3 = this.getAttribute(query),
          name = _this$getAttribute3.name,
          value = _this$getAttribute3.value;
        return name === 'name' ? node.name === value : this.attributeToString(node, name) === value;
      } else {
        return node.nodeName === query;
      }
    };
    _proto.getIdOrClassname = function getIdOrClassname(query) {
      return query.substring(1);
    };
    _proto.getAttribute = function getAttribute(query) {
      var matches = query.match(ATTRIBUTE_REGEXP$1);
      var name = '';
      var value = '';
      if (matches && matches.length > 2) {
        name = matches[1].replace(/"/g, '');
        value = matches[2].replace(/"/g, '');
      }
      return {
        name: name,
        value: value
      };
    };
    _proto.attributeToString = function attributeToString(node, name) {
      if (!node.getAttribute) {
        return '';
      }
      var value = node.getAttribute(name);
      if (isNil$1(value)) {
        return '';
      }
      if (value.toString) {
        return value.toString();
      }
      return '';
    };
    return DefaultSceneGraphSelector;
  }();

  /**
   * update transform in scene graph
   *
   * @see https://community.khronos.org/t/scene-graphs/50542/7
   */
  var DefaultSceneGraphService$1 = /*#__PURE__*/function () {
    function DefaultSceneGraphService(runtime) {
      var _this = this;
      this.runtime = void 0;
      this.pendingEvents = [];
      this.boundsChangedEvent = new CustomEvent$1(ElementEvent$1.BOUNDS_CHANGED);
      this.rotate = function () {
        var parentInvertRotation = create$a();
        return function (element, degrees, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof degrees === 'number') {
            degrees = fromValues$8(degrees, y, z);
          }
          var transform = element.transformable;
          if (element.parentNode === null || !element.parentNode.transformable) {
            _this.rotateLocal(element, degrees);
          } else {
            var rotation = create$a();
            fromEuler$1(rotation, degrees[0], degrees[1], degrees[2]);
            var rot = _this.getRotation(element);
            var parentRot = _this.getRotation(element.parentNode);
            copy$8(parentInvertRotation, parentRot);
            invert$3(parentInvertRotation, parentInvertRotation);
            multiply$5(rotation, parentInvertRotation, rotation);
            multiply$5(transform.localRotation, rotation, rot);
            normalize$7(transform.localRotation, transform.localRotation);
            _this.dirtifyLocal(element, transform);
          }
        };
      }();
      this.rotateLocal = function () {
        var rotation = create$a();
        return function (element, degrees, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof degrees === 'number') {
            degrees = fromValues$8(degrees, y, z);
          }
          var transform = element.transformable;
          fromEuler$1(rotation, degrees[0], degrees[1], degrees[2]);
          mul$3(transform.localRotation, transform.localRotation, rotation);
          _this.dirtifyLocal(element, transform);
        };
      }();
      this.setEulerAngles = function () {
        var invParentRot = create$a();
        return function (element, degrees, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof degrees === 'number') {
            degrees = fromValues$8(degrees, y, z);
          }
          var transform = element.transformable;
          if (element.parentNode === null || !element.parentNode.transformable) {
            _this.setLocalEulerAngles(element, degrees);
          } else {
            fromEuler$1(transform.localRotation, degrees[0], degrees[1], degrees[2]);
            var parentRotation = _this.getRotation(element.parentNode);
            copy$8(invParentRot, invert$3(create$a(), parentRotation));
            mul$3(transform.localRotation, transform.localRotation, invParentRot);
            _this.dirtifyLocal(element, transform);
          }
        };
      }();
      this.translateLocal = function () {
        return function (element, translation, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof translation === 'number') {
            translation = fromValues$8(translation, y, z);
          }
          var transform = element.transformable;
          if (equals$3(translation, create$8())) {
            return;
          }
          transformQuat$1(translation, translation, transform.localRotation);
          add$4(transform.localPosition, transform.localPosition, translation);
          _this.dirtifyLocal(element, transform);
        };
      }();
      this.setPosition = function () {
        var parentInvertMatrix = create$7();
        var tmpPosition = create$8();
        return function (element, position) {
          var transform = element.transformable;
          tmpPosition[0] = position[0];
          tmpPosition[1] = position[1];
          tmpPosition[2] = position[2] || 0;
          if (equals$3(_this.getPosition(element), tmpPosition)) {
            return;
          }
          copy$6(transform.position, tmpPosition);
          if (element.parentNode === null || !element.parentNode.transformable) {
            copy$6(transform.localPosition, tmpPosition);
          } else {
            var parentTransform = element.parentNode.transformable;
            copy$5(parentInvertMatrix, parentTransform.worldTransform);
            invert$2(parentInvertMatrix, parentInvertMatrix);
            transformMat4$2(transform.localPosition, tmpPosition, parentInvertMatrix);
          }
          _this.dirtifyLocal(element, transform);
        };
      }();
      this.setLocalPosition = function () {
        var tmpPosition = create$8();
        return function (element, position) {
          var transform = element.transformable;
          tmpPosition[0] = position[0];
          tmpPosition[1] = position[1];
          tmpPosition[2] = position[2] || 0;
          if (equals$3(transform.localPosition, tmpPosition)) {
            return;
          }
          copy$6(transform.localPosition, tmpPosition);
          _this.dirtifyLocal(element, transform);
        };
      }();
      this.translate = function () {
        var zeroVec3 = create$8();
        var tmpVec3 = create$8();
        var tr = create$8();
        return function (element, translation, y, z) {
          if (y === void 0) {
            y = 0;
          }
          if (z === void 0) {
            z = 0;
          }
          if (typeof translation === 'number') {
            translation = set$3(tmpVec3, translation, y, z);
          }
          if (equals$3(translation, zeroVec3)) {
            return;
          }
          add$4(tr, _this.getPosition(element), translation);
          _this.setPosition(element, tr);
        };
      }();
      this.setRotation = function () {
        var parentInvertRotation = create$a();
        return function (element, rotation, y, z, w) {
          var transform = element.transformable;
          if (typeof rotation === 'number') {
            rotation = fromValues$a(rotation, y, z, w);
          }
          if (element.parentNode === null || !element.parentNode.transformable) {
            _this.setLocalRotation(element, rotation);
          } else {
            var parentRot = _this.getRotation(element.parentNode);
            copy$8(parentInvertRotation, parentRot);
            invert$3(parentInvertRotation, parentInvertRotation);
            multiply$5(transform.localRotation, parentInvertRotation, rotation);
            normalize$7(transform.localRotation, transform.localRotation);
            _this.dirtifyLocal(element, transform);
          }
        };
      };
      this.displayObjectDependencyMap = new WeakMap();
      this.calcLocalTransform = function () {
        var tmpMat = create$7();
        var tmpPosition = create$8();
        var tmpQuat = fromValues$a(0, 0, 0, 1);
        return function (transform) {
          var hasSkew = transform.localSkew[0] !== 0 || transform.localSkew[1] !== 0;
          if (hasSkew) {
            fromRotationTranslationScaleOrigin$1(transform.localTransform, transform.localRotation, transform.localPosition, fromValues$8(1, 1, 1), transform.origin);
            // apply skew2D
            if (transform.localSkew[0] !== 0 || transform.localSkew[1] !== 0) {
              var tmpMat4 = identity$1(tmpMat);
              tmpMat4[4] = Math.tan(transform.localSkew[0]);
              tmpMat4[1] = Math.tan(transform.localSkew[1]);
              multiply$3(transform.localTransform, transform.localTransform, tmpMat4);
            }
            var scaling = fromRotationTranslationScaleOrigin$1(tmpMat, tmpQuat, tmpPosition, transform.localScale, transform.origin);
            multiply$3(transform.localTransform, transform.localTransform, scaling);
          } else {
            // @see https://github.com/mattdesl/css-mat4/blob/master/index.js
            fromRotationTranslationScaleOrigin$1(transform.localTransform, transform.localRotation, transform.localPosition, transform.localScale, transform.origin);
          }
        };
      }();
      this.runtime = runtime;
    }
    var _proto = DefaultSceneGraphService.prototype;
    _proto.matches = function matches(query, root) {
      return this.runtime.sceneGraphSelector.is(query, root);
    };
    _proto.querySelector = function querySelector(query, root) {
      return this.runtime.sceneGraphSelector.selectOne(query, root);
    };
    _proto.querySelectorAll = function querySelectorAll(query, root) {
      return this.runtime.sceneGraphSelector.selectAll(query, root);
      // .filter((node) => !node.shadow);
    };
    _proto.attach = function attach(child, parent, index) {
      var _sortable$sorted, _child$style;
      if (child.parentNode) {
        this.detach(child);
      }
      child.parentNode = parent;
      if (!isNil$1(index)) {
        child.parentNode.childNodes.splice(index, 0, child);
      } else {
        child.parentNode.childNodes.push(child);
      }
      // parent needs re-sort
      var sortable = parent.sortable;
      if ((sortable === null || sortable === void 0 ? void 0 : (_sortable$sorted = sortable.sorted) === null || _sortable$sorted === void 0 ? void 0 : _sortable$sorted.length) || ((_child$style = child.style) === null || _child$style === void 0 ? void 0 : _child$style.zIndex)) {
        // if (sortable) {
        // only child has z-Index
        sortable.dirty = true;
      }
      // this.updateGraphDepth(child);
      var transform = child.transformable;
      if (transform) {
        this.dirtifyWorld(child, transform);
      }
      if (transform.frozen) {
        this.unfreezeParentToRoot(child);
      }
    };
    _proto.detach = function detach(child) {
      if (child.parentNode) {
        var _sortable$sorted2, _child$style2;
        var transform = child.transformable;
        // if (transform) {
        //   const worldTransform = this.getWorldTransform(child, transform);
        //   mat4.getScaling(transform.localScale, worldTransform);
        //   mat4.getTranslation(transform.localPosition, worldTransform);
        //   mat4.getRotation(transform.localRotation, worldTransform);
        //   transform.localDirtyFlag = true;
        // }
        // parent needs re-sort
        var sortable = child.parentNode.sortable;
        // if (sortable) {
        if ((sortable === null || sortable === void 0 ? void 0 : (_sortable$sorted2 = sortable.sorted) === null || _sortable$sorted2 === void 0 ? void 0 : _sortable$sorted2.length) || ((_child$style2 = child.style) === null || _child$style2 === void 0 ? void 0 : _child$style2.zIndex)) {
          sortable.dirty = true;
        }
        var index = child.parentNode.childNodes.indexOf(child);
        if (index > -1) {
          child.parentNode.childNodes.splice(index, 1);
        }
        if (transform) {
          this.dirtifyWorld(child, transform);
        }
        child.parentNode = null;
      }
    };
    _proto.getOrigin = function getOrigin(element) {
      return element.transformable.origin;
    }
    /**
     * same as pivot in Pixi.js
     *
     * @see https://stackoverflow.com/questions/40748452/how-to-change-css-transform-origin-but-preserve-transformation
     */;
    _proto.setOrigin = function setOrigin(element, origin, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      if (typeof origin === 'number') {
        origin = fromValues$8(origin, y, z);
      }
      var transform = element.transformable;
      if (equals$3(origin, transform.origin)) {
        return;
      }
      var originVec = transform.origin;
      // const delta = vec3.subtract(vec3.create(), origin, originVec);
      // vec3.add(transform.localPosition, transform.localPosition, delta);
      // update origin
      originVec[0] = origin[0];
      originVec[1] = origin[1];
      originVec[2] = origin[2] || 0;
      this.dirtifyLocal(element, transform);
    }
    /**
     * rotate in world space
     */;
    /**
     * set euler angles(degrees) in local space
     */
    _proto.setLocalEulerAngles = function setLocalEulerAngles(element, degrees, y, z) {
      if (y === void 0) {
        y = 0;
      }
      if (z === void 0) {
        z = 0;
      }
      if (typeof degrees === 'number') {
        degrees = fromValues$8(degrees, y, z);
      }
      var transform = element.transformable;
      fromEuler$1(transform.localRotation, degrees[0], degrees[1], degrees[2]);
      this.dirtifyLocal(element, transform);
    }
    /**
     * translate in local space
     *
     * @example
     * ```
     * translateLocal(x, y, z)
     * translateLocal(vec3(x, y, z))
     * ```
     */;
    /**
     * scale in local space
     */
    _proto.scaleLocal = function scaleLocal(element, scaling) {
      var transform = element.transformable;
      multiply$4(transform.localScale, transform.localScale, fromValues$8(scaling[0], scaling[1], scaling[2] || 1));
      this.dirtifyLocal(element, transform);
    };
    _proto.setLocalScale = function setLocalScale(element, scaling) {
      var transform = element.transformable;
      var updatedScaling = fromValues$8(scaling[0], scaling[1], scaling[2] || transform.localScale[2]);
      if (equals$3(updatedScaling, transform.localScale)) {
        return;
      }
      copy$6(transform.localScale, updatedScaling);
      this.dirtifyLocal(element, transform);
    }
    /**
     * translate in world space
     *
     * @example
     * ```
     * translate(x, y, z)
     * translate(vec3(x, y, z))
     * ```
     *
     * 对应 g 原版的 translate 2D
     * @see https://github.com/antvis/g/blob/master/packages/g-base/src/abstract/element.ts#L665-L676
     */;
    _proto.setLocalRotation = function setLocalRotation(element, rotation, y, z, w) {
      if (typeof rotation === 'number') {
        rotation = fromValues$a(rotation, y, z, w);
      }
      var transform = element.transformable;
      copy$8(transform.localRotation, rotation);
      this.dirtifyLocal(element, transform);
    };
    _proto.setLocalSkew = function setLocalSkew(element, skew, y) {
      if (typeof skew === 'number') {
        skew = fromValues$b(skew, y);
      }
      var transform = element.transformable;
      copy$9(transform.localSkew, skew);
      this.dirtifyLocal(element, transform);
    };
    _proto.dirtifyLocal = function dirtifyLocal(element, transform) {
      if (!transform.localDirtyFlag) {
        transform.localDirtyFlag = true;
        if (!transform.dirtyFlag) {
          this.dirtifyWorld(element, transform);
        }
      }
    };
    _proto.dirtifyWorld = function dirtifyWorld(element, transform) {
      if (!transform.dirtyFlag) {
        this.unfreezeParentToRoot(element);
      }
      this.dirtifyWorldInternal(element, transform);
      this.dirtifyToRoot(element, true);
    };
    _proto.triggerPendingEvents = function triggerPendingEvents() {
      var _this2 = this;
      var set = new Set();
      var trigger = function trigger(element, detail) {
        if (element.isConnected && !set.has(element.entity)) {
          _this2.boundsChangedEvent.detail = detail;
          _this2.boundsChangedEvent.target = element;
          element.ownerDocument.defaultView.dispatchEvent(_this2.boundsChangedEvent, true);
          set.add(element.entity);
        }
      };
      this.pendingEvents.forEach(function (_ref) {
        var element = _ref[0],
          detail = _ref[1];
        if (detail.affectChildren) {
          element.forEach(function (e) {
            trigger(e, detail);
          });
        } else {
          trigger(element, detail);
        }
      });
      this.pendingEvents = [];
      set.clear();
    };
    _proto.dirtifyToRoot = function dirtifyToRoot(element, affectChildren) {
      if (affectChildren === void 0) {
        affectChildren = false;
      }
      var p = element;
      // only need to re-render itself
      if (p.renderable) {
        p.renderable.dirty = true;
      }
      while (p) {
        var renderable = p.renderable;
        if (renderable) {
          renderable.renderBoundsDirty = true;
          renderable.boundsDirty = true;
        }
        p = p.parentNode;
      }
      // inform dependencies
      this.informDependentDisplayObjects(element);
      // reuse the same custom event
      this.pendingEvents.push([element, {
        affectChildren: affectChildren
      }]);
    };
    _proto.updateDisplayObjectDependency = function updateDisplayObjectDependency(name, oldPath, newPath, object) {
      // clear ref to old clip path
      if (oldPath && oldPath !== newPath) {
        var oldDependencyMap = this.displayObjectDependencyMap.get(oldPath);
        if (oldDependencyMap && oldDependencyMap[name]) {
          var index = oldDependencyMap[name].indexOf(object);
          oldDependencyMap[name].splice(index, 1);
        }
      }
      if (newPath) {
        var newDependencyMap = this.displayObjectDependencyMap.get(newPath);
        if (!newDependencyMap) {
          this.displayObjectDependencyMap.set(newPath, {});
          newDependencyMap = this.displayObjectDependencyMap.get(newPath);
        }
        if (!newDependencyMap[name]) {
          newDependencyMap[name] = [];
        }
        newDependencyMap[name].push(object);
      }
    };
    _proto.informDependentDisplayObjects = function informDependentDisplayObjects(object) {
      var _this3 = this;
      var dependencyMap = this.displayObjectDependencyMap.get(object);
      if (dependencyMap) {
        Object.keys(dependencyMap).forEach(function (name) {
          dependencyMap[name].forEach(function (target) {
            _this3.dirtifyToRoot(target);
            target.dispatchEvent(new MutationEvent$1(ElementEvent$1.ATTR_MODIFIED, target, _this3, _this3, name, MutationEvent$1.MODIFICATION, _this3, _this3));
            if (target.isCustomElement && target.isConnected) {
              if (target.attributeChangedCallback) {
                target.attributeChangedCallback(name, _this3, _this3);
              }
            }
          });
        });
      }
    };
    _proto.getPosition = function getPosition(element) {
      var transform = element.transformable;
      return getTranslation$1(transform.position, this.getWorldTransform(element, transform));
    };
    _proto.getRotation = function getRotation(element) {
      var transform = element.transformable;
      return getRotation$1(transform.rotation, this.getWorldTransform(element, transform));
    };
    _proto.getScale = function getScale(element) {
      var transform = element.transformable;
      return getScaling$1(transform.scaling, this.getWorldTransform(element, transform));
    };
    _proto.getWorldTransform = function getWorldTransform(element, transform) {
      if (transform === void 0) {
        transform = element.transformable;
      }
      if (!transform.localDirtyFlag && !transform.dirtyFlag) {
        return transform.worldTransform;
      }
      if (element.parentNode && element.parentNode.transformable) {
        this.getWorldTransform(element.parentNode);
      }
      this.sync(element, transform);
      return transform.worldTransform;
    };
    _proto.getLocalPosition = function getLocalPosition(element) {
      return element.transformable.localPosition;
    };
    _proto.getLocalRotation = function getLocalRotation(element) {
      return element.transformable.localRotation;
    };
    _proto.getLocalScale = function getLocalScale(element) {
      return element.transformable.localScale;
    };
    _proto.getLocalSkew = function getLocalSkew(element) {
      return element.transformable.localSkew;
    };
    _proto.getLocalTransform = function getLocalTransform(element) {
      var transform = element.transformable;
      if (transform.localDirtyFlag) {
        this.calcLocalTransform(transform);
        transform.localDirtyFlag = false;
      }
      return transform.localTransform;
    };
    _proto.setLocalTransform = function setLocalTransform(element, transform) {
      var t = getTranslation$1(create$8(), transform);
      var r = getRotation$1(create$a(), transform);
      var s = getScaling$1(create$8(), transform);
      this.setLocalScale(element, s);
      this.setLocalPosition(element, t);
      this.setLocalRotation(element, r);
    };
    _proto.resetLocalTransform = function resetLocalTransform(element) {
      this.setLocalScale(element, [1, 1, 1]);
      this.setLocalPosition(element, [0, 0, 0]);
      this.setLocalEulerAngles(element, [0, 0, 0]);
      this.setLocalSkew(element, [0, 0]);
    };
    _proto.getTransformedGeometryBounds = function getTransformedGeometryBounds(element, render, existedAABB) {
      if (render === void 0) {
        render = false;
      }
      var bounds = this.getGeometryBounds(element, render);
      if (!AABB$1.isEmpty(bounds)) {
        var aabb = existedAABB || new AABB$1();
        aabb.setFromTransformedAABB(bounds, this.getWorldTransform(element));
        return aabb;
      } else {
        return null;
      }
    }
    /**
     * won't account for children
     */;
    _proto.getGeometryBounds = function getGeometryBounds(element, render) {
      if (render === void 0) {
        render = false;
      }
      var geometry = element.geometry;
      var bounds = render ? geometry.renderBounds : geometry.contentBounds || null;
      // return (bounds && new AABB(bounds.center, bounds.halfExtents)) || new AABB();
      return bounds || new AABB$1();
    }
    /**
     * account for children in world space
     */;
    _proto.getBounds = function getBounds(element, render) {
      var _this4 = this;
      if (render === void 0) {
        render = false;
      }
      var renderable = element.renderable;
      if (!renderable.boundsDirty && !render && renderable.bounds) {
        return renderable.bounds;
      }
      if (!renderable.renderBoundsDirty && render && renderable.renderBounds) {
        return renderable.renderBounds;
      }
      // reuse existed if possible
      var existedAABB = render ? renderable.renderBounds : renderable.bounds;
      // reset with geometry's aabb
      var aabb = this.getTransformedGeometryBounds(element, render, existedAABB);
      // merge children's aabbs
      var children = element.childNodes;
      children.forEach(function (child) {
        var childBounds = _this4.getBounds(child, render);
        if (childBounds) {
          if (!aabb) {
            aabb = existedAABB || new AABB$1();
            aabb.update(childBounds.center, childBounds.halfExtents);
          } else {
            aabb.add(childBounds);
          }
        }
      });
      if (render) {
        // FIXME: account for clip path
        var clipped = findClosestClipPathTarget$1(element);
        if (clipped) {
          // use bounds under world space
          var clipPathBounds = clipped.style.clipPath.getBounds(render);
          if (!aabb) {
            aabb = clipPathBounds;
          } else if (clipPathBounds) {
            aabb = clipPathBounds.intersection(aabb);
          }
        }
      }
      if (!aabb) {
        aabb = new AABB$1();
      }
      if (aabb) {
        if (render) {
          renderable.renderBounds = aabb;
        } else {
          renderable.bounds = aabb;
        }
      }
      if (render) {
        renderable.renderBoundsDirty = false;
      } else {
        renderable.boundsDirty = false;
      }
      return aabb;
    }
    /**
     * account for children in local space
     */;
    _proto.getLocalBounds = function getLocalBounds(element) {
      if (element.parentNode) {
        var parentInvert = create$7();
        if (element.parentNode.transformable) {
          parentInvert = invert$2(create$7(), this.getWorldTransform(element.parentNode));
        }
        var bounds = this.getBounds(element);
        if (!AABB$1.isEmpty(bounds)) {
          var localBounds = new AABB$1();
          localBounds.setFromTransformedAABB(bounds, parentInvert);
          return localBounds;
        }
      }
      return this.getBounds(element);
    };
    _proto.getBoundingClientRect = function getBoundingClientRect(element) {
      var _element$ownerDocumen, _element$ownerDocumen2;
      var aabb;
      var bounds = this.getGeometryBounds(element);
      if (!AABB$1.isEmpty(bounds)) {
        aabb = new AABB$1();
        // apply transformation to aabb
        aabb.setFromTransformedAABB(bounds, this.getWorldTransform(element));
      }
      // calc context's offset
      var bbox = (_element$ownerDocumen = element.ownerDocument) === null || _element$ownerDocumen === void 0 ? void 0 : (_element$ownerDocumen2 = _element$ownerDocumen.defaultView) === null || _element$ownerDocumen2 === void 0 ? void 0 : _element$ownerDocumen2.getContextService().getBoundingClientRect();
      if (aabb) {
        var _aabb$getMin = aabb.getMin(),
          left = _aabb$getMin[0],
          top = _aabb$getMin[1];
        var _aabb$getMax = aabb.getMax(),
          right = _aabb$getMax[0],
          bottom = _aabb$getMax[1];
        return new Rectangle$1(left + ((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0), top + ((bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0), right - left, bottom - top);
      }
      return new Rectangle$1((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0, (bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0, 0, 0);
    };
    _proto.dirtifyWorldInternal = function dirtifyWorldInternal(element, transform) {
      var _this5 = this;
      if (!transform.dirtyFlag) {
        transform.dirtyFlag = true;
        transform.frozen = false;
        element.childNodes.forEach(function (child) {
          var childTransform = child.transformable;
          if (!childTransform.dirtyFlag) {
            _this5.dirtifyWorldInternal(child, childTransform);
          }
        });
        var renderable = element.renderable;
        if (renderable) {
          renderable.renderBoundsDirty = true;
          renderable.boundsDirty = true;
          renderable.dirty = true;
        }
      }
    };
    _proto.syncHierarchy = function syncHierarchy(element) {
      var transform = element.transformable;
      if (transform.frozen) {
        return;
      }
      transform.frozen = true;
      if (transform.localDirtyFlag || transform.dirtyFlag) {
        this.sync(element, transform);
      }
      var children = element.childNodes;
      for (var i = 0; i < children.length; i++) {
        this.syncHierarchy(children[i]);
      }
    };
    _proto.sync = function sync(element, transform) {
      if (transform.localDirtyFlag) {
        this.calcLocalTransform(transform);
        transform.localDirtyFlag = false;
      }
      if (transform.dirtyFlag) {
        var parent = element.parentNode;
        var parentTransform = parent && parent.transformable;
        if (parent === null || !parentTransform) {
          copy$5(transform.worldTransform, transform.localTransform);
        } else {
          // TODO: should we support scale compensation?
          // @see https://github.com/playcanvas/engine/issues/1077#issuecomment-359765557
          multiply$3(transform.worldTransform, parentTransform.worldTransform, transform.localTransform);
        }
        transform.dirtyFlag = false;
      }
    };
    _proto.unfreezeParentToRoot = function unfreezeParentToRoot(child) {
      var p = child.parentNode;
      while (p) {
        var transform = p.transformable;
        if (transform) {
          transform.frozen = false;
        }
        p = p.parentNode;
      }
    };
    return DefaultSceneGraphService;
  }();

  var TEXT_METRICS$1 = {
    MetricsString: '|ÉqÅ',
    BaselineSymbol: 'M',
    BaselineMultiplier: 1.4,
    HeightMultiplier: 2,
    Newlines: [0x000a, 0x000d // carriage return
    ],

    BreakingSpaces: [0x0009, 0x0020, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005, 0x2006, 0x2008, 0x2009, 0x200a, 0x205f, 0x3000 // ideographic space
    ]
  };

  var LATIN_REGEX$1 = /[a-zA-Z0-9\u00C0-\u00D6\u00D8-\u00f6\u00f8-\u00ff!"#$%&'()*+,-./:;]/;
  // Line breaking rules in CJK (Kinsoku Shori)
  // Refer from https://en.wikipedia.org/wiki/Line_breaking_rules_in_East_Asian_languages
  var regexCannotStartZhCn$1 = /[!%),.:;?\]}¢°·'""†‡›℃∶、。〃〆〕〗〞﹚﹜！＂％＇），．：；？！］｝～]/;
  var regexCannotEndZhCn$1 = /[$(£¥·'"〈《「『【〔〖〝﹙﹛＄（．［｛￡￥]/;
  var regexCannotStartZhTw$1 = /[!),.:;?\]}¢·–—'"•"、。〆〞〕〉》」︰︱︲︳﹐﹑﹒﹓﹔﹕﹖﹘﹚﹜！），．：；？︶︸︺︼︾﹀﹂﹗］｜｝､]/;
  var regexCannotEndZhTw$1 = /[([{£¥'"‵〈《「『〔〝︴﹙﹛（｛︵︷︹︻︽︿﹁﹃﹏]/;
  var regexCannotStartJaJp$1 = /[)\]｝〕〉》」』】〙〗〟'"｠»ヽヾーァィゥェォッャュョヮヵヶぁぃぅぇぉっゃゅょゎゕゖㇰㇱㇲㇳㇴㇵㇶㇷㇸㇹㇺㇻㇼㇽㇾㇿ々〻‐゠–〜?!‼⁇⁈⁉・、:;,。.]/;
  var regexCannotEndJaJp$1 = /[([｛〔〈《「『【〘〖〝'"｟«—...‥〳〴〵]/;
  var regexCannotStartKoKr$1 = /[!%),.:;?\]}¢°'"†‡℃〆〈《「『〕！％），．：；？］｝]/;
  var regexCannotEndKoKr$1 = /[$([{£¥'"々〇〉》」〔＄（［｛｠￥￦#]/;
  var regexCannotStart$1 = new RegExp(regexCannotStartZhCn$1.source + "|" + regexCannotStartZhTw$1.source + "|" + regexCannotStartJaJp$1.source + "|" + regexCannotStartKoKr$1.source);
  var regexCannotEnd$1 = new RegExp(regexCannotEndZhCn$1.source + "|" + regexCannotEndZhTw$1.source + "|" + regexCannotEndJaJp$1.source + "|" + regexCannotEndKoKr$1.source);
  /**
   * Borrow from pixi/packages/text/src/TextMetrics.ts
   */
  var TextService$1 = /*#__PURE__*/function () {
    function TextService(runtime) {
      var _this = this;
      this.runtime = void 0;
      this.fontMetricsCache = {};
      this.shouldBreakByKinsokuShorui = function (char, nextChar) {
        if (_this.isBreakingSpace(nextChar)) return false;
        if (char) {
          // Line breaking rules in CJK (Kinsoku Shori)
          if (regexCannotEnd$1.exec(nextChar) || regexCannotStart$1.exec(char)) {
            return true;
          }
        }
        return false;
      };
      this.trimByKinsokuShorui = function (prev) {
        var next = [].concat(prev);
        var prevLine = next[next.length - 2];
        if (!prevLine) {
          return prev;
        }
        var lastChar = prevLine[prevLine.length - 1];
        next[next.length - 2] = prevLine.slice(0, -1);
        next[next.length - 1] = lastChar + next[next.length - 1];
        return next;
      };
      this.runtime = runtime;
    }
    /**
     * font metrics cache
     */
    var _proto = TextService.prototype;
    /**
     * Calculates the ascent, descent and fontSize of a given font-style.
     */
    _proto.measureFont = function measureFont(font, offscreenCanvas) {
      // as this method is used for preparing assets, don't recalculate things if we don't need to
      if (this.fontMetricsCache[font]) {
        return this.fontMetricsCache[font];
      }
      var properties = {
        ascent: 0,
        descent: 0,
        fontSize: 0
      };
      var canvas = this.runtime.offscreenCanvas.getOrCreateCanvas(offscreenCanvas);
      var context = this.runtime.offscreenCanvas.getOrCreateContext(offscreenCanvas);
      context.font = font;
      var metricsString = TEXT_METRICS$1.MetricsString + TEXT_METRICS$1.BaselineSymbol;
      var width = Math.ceil(context.measureText(metricsString).width);
      var baseline = Math.ceil(context.measureText(TEXT_METRICS$1.BaselineSymbol).width);
      var height = TEXT_METRICS$1.HeightMultiplier * baseline;
      baseline = baseline * TEXT_METRICS$1.BaselineMultiplier | 0;
      // @ts-ignore
      canvas.width = width;
      // @ts-ignore
      canvas.height = height;
      context.fillStyle = '#f00';
      context.fillRect(0, 0, width, height);
      context.font = font;
      context.textBaseline = 'alphabetic';
      context.fillStyle = '#000';
      context.fillText(metricsString, 0, baseline);
      var imagedata = context.getImageData(0, 0, width || 1, height || 1).data;
      var pixels = imagedata.length;
      var line = width * 4;
      var i = 0;
      var idx = 0;
      var stop = false;
      // ascent. scan from top to bottom until we find a non red pixel
      for (i = 0; i < baseline; ++i) {
        for (var j = 0; j < line; j += 4) {
          if (imagedata[idx + j] !== 255) {
            stop = true;
            break;
          }
        }
        if (!stop) {
          idx += line;
        } else {
          break;
        }
      }
      properties.ascent = baseline - i;
      idx = pixels - line;
      stop = false;
      // descent. scan from bottom to top until we find a non red pixel
      for (i = height; i > baseline; --i) {
        for (var _j = 0; _j < line; _j += 4) {
          if (imagedata[idx + _j] !== 255) {
            stop = true;
            break;
          }
        }
        if (!stop) {
          idx -= line;
        } else {
          break;
        }
      }
      properties.descent = i - baseline;
      properties.fontSize = properties.ascent + properties.descent;
      this.fontMetricsCache[font] = properties;
      return properties;
    };
    _proto.measureText = function measureText(text, parsedStyle, offscreenCanvas) {
      var fontSize = parsedStyle.fontSize,
        wordWrap = parsedStyle.wordWrap,
        strokeHeight = parsedStyle.lineHeight,
        lineWidth = parsedStyle.lineWidth,
        textBaseline = parsedStyle.textBaseline,
        textAlign = parsedStyle.textAlign,
        letterSpacing = parsedStyle.letterSpacing,
        textPath = parsedStyle.textPath,
        textPathSide = parsedStyle.textPathSide,
        textPathStartOffset = parsedStyle.textPathStartOffset,
        _parsedStyle$leading = parsedStyle.leading,
        leading = _parsedStyle$leading === void 0 ? 0 : _parsedStyle$leading;
      var font = toFontString$1(parsedStyle);
      var fontProperties = this.measureFont(font, offscreenCanvas);
      // fallback in case UA disallow canvas data extraction
      // (toDataURI, getImageData functions)
      if (fontProperties.fontSize === 0) {
        fontProperties.fontSize = fontSize;
        fontProperties.ascent = fontSize;
      }
      var context = this.runtime.offscreenCanvas.getOrCreateContext(offscreenCanvas);
      context.font = font;
      // no overflowing by default
      parsedStyle.isOverflowing = false;
      var outputText = wordWrap ? this.wordWrap(text, parsedStyle, offscreenCanvas) : text;
      var lines = outputText.split(/(?:\r\n|\r|\n)/);
      var lineWidths = new Array(lines.length);
      var maxLineWidth = 0;
      // account for textPath
      if (textPath) {
        var totalPathLength = textPath.getTotalLength();
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        var startingPoint = textPath.getPoint(0);
        for (var i = 0; i < lines.length; i++) {
          var width = context.measureText(lines[i]).width + (lines[i].length - 1) * letterSpacing;
          // for (
          //   let i = reverse ? lines[0].length - 1 : 0;
          //   reverse ? i >= 0 : i < lines[0].length;
          //   reverse ? i-- : i++
          // ) {
          //   graphemeInfo = lineBounds[i];
          //   if (positionInPath > totalPathLength) {
          //     positionInPath %= totalPathLength;
          //   } else if (positionInPath < 0) {
          //     positionInPath += totalPathLength;
          //   }
          //   // it would probably much faster to send all the grapheme position for a line
          //   // and calculate path position/angle at once.
          //   this.setGraphemeOnPath(
          //     positionInPath,
          //     graphemeInfo,
          //     startingPoint
          //   );
          //   positionInPath += graphemeInfo.kernedWidth;
          // }
        }
      } else {
        for (var _i = 0; _i < lines.length; _i++) {
          // char width + letterSpacing
          var _lineWidth = context.measureText(lines[_i]).width + (lines[_i].length - 1) * letterSpacing;
          lineWidths[_i] = _lineWidth;
          maxLineWidth = Math.max(maxLineWidth, _lineWidth);
        }
        var _width = maxLineWidth + lineWidth;
        // if (dropShadow) {
        //   width += dropShadowDistance;
        // }
        var lineHeight = strokeHeight || fontProperties.fontSize + lineWidth;
        var height = Math.max(lineHeight, fontProperties.fontSize + lineWidth) + (lines.length - 1) * (lineHeight + leading);
        // if (dropShadow) {
        //   height += dropShadowDistance;
        // }
        lineHeight += leading;
        // handle vertical text baseline
        var offsetY = 0;
        if (textBaseline === 'middle') {
          offsetY = -height / 2;
        } else if (textBaseline === 'bottom' || textBaseline === 'alphabetic' || textBaseline === 'ideographic') {
          offsetY = -height;
        } else if (textBaseline === 'top' || textBaseline === 'hanging') {
          offsetY = 0;
        }
        return {
          font: font,
          width: _width,
          height: height,
          lines: lines,
          lineWidths: lineWidths,
          lineHeight: lineHeight,
          maxLineWidth: maxLineWidth,
          fontProperties: fontProperties,
          lineMetrics: lineWidths.map(function (width, i) {
            var offsetX = 0;
            // handle horizontal text align
            if (textAlign === 'center' || textAlign === 'middle') {
              offsetX -= width / 2;
            } else if (textAlign === 'right' || textAlign === 'end') {
              offsetX -= width;
            }
            return new Rectangle$1(offsetX - lineWidth / 2, offsetY + i * lineHeight, width + lineWidth, lineHeight);
          })
        };
      }
    };
    _proto.setGraphemeOnPath = function setGraphemeOnPath() {};
    _proto.wordWrap = function wordWrap(text, parsedStyle, offscreenCanvas) {
      var _this2 = this;
      var _parsedStyle$wordWrap = parsedStyle.wordWrapWidth,
        wordWrapWidth = _parsedStyle$wordWrap === void 0 ? 0 : _parsedStyle$wordWrap,
        letterSpacing = parsedStyle.letterSpacing,
        _parsedStyle$maxLines = parsedStyle.maxLines,
        maxLines = _parsedStyle$maxLines === void 0 ? Infinity : _parsedStyle$maxLines,
        textOverflow = parsedStyle.textOverflow;
      var context = this.runtime.offscreenCanvas.getOrCreateContext(offscreenCanvas);
      var maxWidth = wordWrapWidth + letterSpacing;
      var ellipsis = '';
      if (textOverflow === 'ellipsis') {
        ellipsis = '...';
      } else if (textOverflow && textOverflow !== 'clip') {
        ellipsis = textOverflow;
      }
      var lines = [];
      var currentIndex = 0;
      var currentWidth = 0;
      var cache = {};
      var calcWidth = function calcWidth(char) {
        return _this2.getFromCache(char, letterSpacing, cache, context);
      };
      var ellipsisWidth = Array.from(ellipsis).reduce(function (prev, cur) {
        return prev + calcWidth(cur);
      }, 0);
      var chars = Array.from(text);
      for (var i = 0; i < chars.length; i++) {
        var char = chars[i];
        var prevChar = text[i - 1];
        var nextChar = text[i + 1];
        var charWidth = calcWidth(char);
        if (this.isNewline(char)) {
          currentIndex++;
          // exceed maxLines, break immediately
          if (currentIndex >= maxLines) {
            parsedStyle.isOverflowing = true;
            break;
          }
          currentWidth = 0;
          lines[currentIndex] = '';
          continue;
        }
        if (currentWidth > 0 && currentWidth + charWidth > maxWidth) {
          if (currentIndex + 1 >= maxLines) {
            parsedStyle.isOverflowing = true;
            // If there is not enough space to display the string itself, it is clipped.
            // @see https://developer.mozilla.org/en-US/docs/Web/CSS/text-overflow#values
            if (ellipsisWidth > 0 && ellipsisWidth <= maxWidth) {
              // Backspace from line's end.
              var lastLineWidth = 0;
              var lastLineIndex = 0;
              for (var _i2 = 0; _i2 < lines[currentIndex].length; _i2++) {
                var width = calcWidth(lines[currentIndex][_i2]);
                if (lastLineWidth + width + ellipsisWidth > maxWidth) {
                  lastLineIndex = _i2;
                  break;
                }
                lastLineWidth += width;
              }
              lines[currentIndex] = (lines[currentIndex] || '').slice(0, lastLineIndex) + ellipsis;
            }
            break;
          }
          currentIndex++;
          currentWidth = 0;
          lines[currentIndex] = '';
          if (this.isBreakingSpace(char)) {
            continue;
          }
          if (!this.canBreakInLastChar(char)) {
            lines = this.trimToBreakable(lines);
            currentWidth = this.sumTextWidthByCache(lines[currentIndex] || '', cache);
          }
          if (this.shouldBreakByKinsokuShorui(char, nextChar)) {
            lines = this.trimByKinsokuShorui(lines);
            currentWidth += calcWidth(prevChar || '');
          }
        }
        currentWidth += charWidth;
        lines[currentIndex] = (lines[currentIndex] || '') + char;
      }
      return lines.join('\n');
    };
    _proto.isBreakingSpace = function isBreakingSpace(char) {
      if (typeof char !== 'string') {
        return false;
      }
      return TEXT_METRICS$1.BreakingSpaces.indexOf(char.charCodeAt(0)) >= 0;
    };
    _proto.isNewline = function isNewline(char) {
      if (typeof char !== 'string') {
        return false;
      }
      return TEXT_METRICS$1.Newlines.indexOf(char.charCodeAt(0)) >= 0;
    };
    _proto.trimToBreakable = function trimToBreakable(prev) {
      var next = [].concat(prev);
      var prevLine = next[next.length - 2];
      var index = this.findBreakableIndex(prevLine);
      if (index === -1 || !prevLine) return next;
      var trimmedChar = prevLine.slice(index, index + 1);
      var isTrimmedWithSpace = this.isBreakingSpace(trimmedChar);
      var trimFrom = index + 1;
      var trimTo = index + (isTrimmedWithSpace ? 0 : 1);
      next[next.length - 1] += prevLine.slice(trimFrom, prevLine.length);
      next[next.length - 2] = prevLine.slice(0, trimTo);
      return next;
    };
    _proto.canBreakInLastChar = function canBreakInLastChar(char) {
      if (char && LATIN_REGEX$1.test(char)) return false;
      return true;
    };
    _proto.sumTextWidthByCache = function sumTextWidthByCache(text, cache) {
      return text.split('').reduce(function (sum, c) {
        if (!cache[c]) throw Error('cannot count the word without cache');
        return sum + cache[c];
      }, 0);
    };
    _proto.findBreakableIndex = function findBreakableIndex(line) {
      for (var i = line.length - 1; i >= 0; i--) {
        if (!LATIN_REGEX$1.test(line[i])) return i;
      }
      return -1;
    };
    _proto.getFromCache = function getFromCache(key, letterSpacing, cache, context) {
      var width = cache[key];
      if (typeof width !== 'number') {
        var spacing = key.length * letterSpacing;
        width = context.measureText(key).width + spacing;
        cache[key] = width;
      }
      return width;
    };
    return TextService;
  }();

  var runtime$2 = {};
  /**
   * Replace with IoC container
   */
  var geometryUpdaterFactory$1 = function () {
    var _ref;
    var rectUpdater = new RectUpdater$1();
    var polylineUpdater = new PolylineUpdater$1();
    return _ref = {}, _ref[Shape$2.CIRCLE] = new CircleUpdater$1(), _ref[Shape$2.ELLIPSE] = new EllipseUpdater$1(), _ref[Shape$2.RECT] = rectUpdater, _ref[Shape$2.IMAGE] = rectUpdater, _ref[Shape$2.GROUP] = rectUpdater, _ref[Shape$2.LINE] = new LineUpdater$1(), _ref[Shape$2.TEXT] = new TextUpdater$1(runtime$2), _ref[Shape$2.POLYLINE] = polylineUpdater, _ref[Shape$2.POLYGON] = polylineUpdater, _ref[Shape$2.PATH] = new PathUpdater$1(), _ref[Shape$2.HTML] = null, _ref[Shape$2.MESH] = null, _ref;
  }();
  var CSSPropertySyntaxFactory$1 = function () {
    var _ref2;
    var color = new CSSPropertyColor$1();
    var length = new CSSPropertyLengthOrPercentage$1();
    return _ref2 = {}, _ref2[PropertySyntax$1.PERCENTAGE] = null, _ref2[PropertySyntax$1.NUMBER] = new CSSPropertyNumber$1(), _ref2[PropertySyntax$1.ANGLE] = new CSSPropertyAngle$1(), _ref2[PropertySyntax$1.DEFINED_PATH] = new CSSPropertyClipPath$1(), _ref2[PropertySyntax$1.PAINT] = color, _ref2[PropertySyntax$1.COLOR] = color, _ref2[PropertySyntax$1.FILTER] = new CSSPropertyFilter$1(), _ref2[PropertySyntax$1.LENGTH] = length, _ref2[PropertySyntax$1.LENGTH_PERCENTAGE] = length, _ref2[PropertySyntax$1.LENGTH_PERCENTAGE_12] = new CSSPropertyLengthOrPercentage12$1(), _ref2[PropertySyntax$1.LENGTH_PERCENTAGE_14] = new CSSPropertyLengthOrPercentage14$1(), _ref2[PropertySyntax$1.COORDINATE] = new CSSPropertyLocalPosition$1(), _ref2[PropertySyntax$1.OFFSET_DISTANCE] = new CSSPropertyOffsetDistance$1(), _ref2[PropertySyntax$1.OPACITY_VALUE] = new CSSPropertyOpacity$1(), _ref2[PropertySyntax$1.PATH] = new CSSPropertyPath$1(), _ref2[PropertySyntax$1.LIST_OF_POINTS] = new CSSPropertyPoints$1(), _ref2[PropertySyntax$1.SHADOW_BLUR] = new CSSPropertyShadowBlur$1(), _ref2[PropertySyntax$1.TEXT] = new CSSPropertyText$1(), _ref2[PropertySyntax$1.TEXT_TRANSFORM] = new CSSPropertyTextTransform$1(), _ref2[PropertySyntax$1.TRANSFORM] = new CSSPropertyTransform$1(), _ref2[PropertySyntax$1.TRANSFORM_ORIGIN] = new CSSPropertyTransformOrigin$1(), _ref2[PropertySyntax$1.Z_INDEX] = new CSSPropertyZIndex$1(), _ref2[PropertySyntax$1.MARKER] = new CSSPropertyMarker$1(), _ref2;
  }();
  /**
   * Camera
   * `g-camera-api` will provide an advanced implementation
   */
  runtime$2.CameraContribution = Camera$1;
  /**
   * `g-web-animations-api` will provide an AnimationTimeline
   */
  runtime$2.AnimationTimeline = null;
  runtime$2.EasingFunction = null;
  runtime$2.offscreenCanvas = new OffscreenCanvasCreator$1();
  runtime$2.displayObjectPool = new DisplayObjectPool$1();
  runtime$2.sceneGraphSelector = new DefaultSceneGraphSelector$1();
  runtime$2.sceneGraphService = new DefaultSceneGraphService$1(runtime$2);
  runtime$2.textService = new TextService$1(runtime$2);
  runtime$2.geometryUpdaterFactory = geometryUpdaterFactory$1;
  runtime$2.CSSPropertySyntaxFactory = CSSPropertySyntaxFactory$1;
  runtime$2.styleValueRegistry = new DefaultStyleValueRegistry$1();
  runtime$2.layoutRegistry = null;

  /**
   * apply following rules:
   * 1. `visibility` in scenegraph node
   * 2. other custom culling strategies, eg. frustum culling
   */
  var CullingPlugin$1 = /*#__PURE__*/function () {
    function CullingPlugin(strategies) {
      this.strategies = void 0;
      this.strategies = strategies;
    }
    var _proto = CullingPlugin.prototype;
    _proto.apply = function apply(context) {
      var camera = context.camera,
        renderingService = context.renderingService,
        renderingContext = context.renderingContext;
      var strategies = this.strategies;
      renderingService.hooks.cull.tap(CullingPlugin.tag, function (object) {
        if (object) {
          var cullable = object.cullable;
          // cullable.visible = true;
          // const renderBounds = object.getRenderBounds();
          // if (AABB.isEmpty(renderBounds)) {
          //   cullable.visible = false;
          // } else {
          //   const isShape2D = shape2D.indexOf(object.nodeName as Shape) > -1;
          //   const [p0, p1, p2, p3] = camera.getFrustum().planes;
          //   tmpAABB.setMinMax([-p1.distance, -p3.distance, 0], [p0.distance, p2.distance, 0]);
          //   cullable.visible = isShape2D ? renderBounds.intersects(tmpAABB) : true;
          // }
          if (strategies.length === 0) {
            cullable.visible = renderingContext.unculledEntities.indexOf(object.entity) > -1;
          } else {
            // eg. implemented by g-webgl(frustum culling)
            cullable.visible = strategies.every(function (strategy) {
              return strategy.isVisible(camera, object);
            });
          }
          if (!object.isCulled() && object.isVisible()) {
            return object;
          } else {
            // if (this.renderingContext.renderListLastFrame.indexOf(object) > -1) {
            object.dispatchEvent(new CustomEvent$1(ElementEvent$1.CULLED));
            // }
          }

          return null;
        }
        return object;
      });
      renderingService.hooks.afterRender.tap(CullingPlugin.tag, function (object) {
        object.cullable.visibilityPlaneMask = -1;
      });
    };
    return CullingPlugin;
  }();
  CullingPlugin$1.tag = 'Culling';

  /**
   * Filter dirty renderables and calculate the "dirty rectangle" which will be clear when frame began
   */
  var DirtyCheckPlugin$1 = /*#__PURE__*/function () {
    function DirtyCheckPlugin() {}
    var _proto = DirtyCheckPlugin.prototype;
    _proto.apply = function apply(context) {
      var renderingService = context.renderingService;
      renderingService.hooks.dirtycheck.tap(DirtyCheckPlugin.tag, function (object) {
        if (object) {
          var renderable = object.renderable;
          var isDirty = renderable.dirty || renderingService.disableDirtyRectangleRendering();
          if (isDirty) {
            return object;
          } else {
            return null;
          }
        }
        return object;
      });
    };
    return DirtyCheckPlugin;
  }();
  DirtyCheckPlugin$1.tag = 'DirtyCheck';

  /**
   * support mouse & touch events
   * @see https://github.com/pixijs/pixi.js/blob/dev/packages/interaction/README.md
   *
   * also provide some extra events such as `drag`
   */
  var EventPlugin$1 = /*#__PURE__*/function () {
    function EventPlugin() {
      var _this = this;
      this.autoPreventDefault = false;
      this.rootPointerEvent = new FederatedPointerEvent$1(null);
      this.rootWheelEvent = new FederatedWheelEvent$1(null);
      this.context = void 0;
      this.onPointerMove = function (nativeEvent) {
        var _this$context$renderi, _this$context$renderi2;
        var canvas = (_this$context$renderi = _this.context.renderingContext.root) === null || _this$context$renderi === void 0 ? void 0 : (_this$context$renderi2 = _this$context$renderi.ownerDocument) === null || _this$context$renderi2 === void 0 ? void 0 : _this$context$renderi2.defaultView;
        if (canvas.supportsTouchEvents && nativeEvent.pointerType === 'touch') return;
        var normalizedEvents = _this.normalizeToPointerEvent(nativeEvent, canvas);
        for (var _iterator = _createForOfIteratorHelperLoose$2(normalizedEvents), _step; !(_step = _iterator()).done;) {
          var normalizedEvent = _step.value;
          var event = _this.bootstrapEvent(_this.rootPointerEvent, normalizedEvent, canvas);
          _this.context.eventService.mapEvent(event);
        }
        _this.setCursor(_this.context.eventService.cursor);
      };
    }
    var _proto = EventPlugin.prototype;
    _proto.apply = function apply(context) {
      var _this2 = this;
      this.context = context;
      var renderingService = context.renderingService;
      var canvas = this.context.renderingContext.root.ownerDocument.defaultView;
      this.context.eventService.setPickHandler( /*#__PURE__*/function () {
        var _ref = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee(position) {
          var _yield$_this2$context, picked;
          return _regeneratorRuntime$6().wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  _context.next = 2;
                  return _this2.context.renderingService.hooks.pick.promise({
                    position: position,
                    picked: [],
                    topmost: true // we only concern the topmost element
                  });
                case 2:
                  _yield$_this2$context = _context.sent;
                  picked = _yield$_this2$context.picked;
                  return _context.abrupt("return", picked[0] || null);
                case 5:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));
        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }());
      renderingService.hooks.pointerWheel.tap(EventPlugin.tag, function (nativeEvent) {
        var wheelEvent = _this2.normalizeWheelEvent(nativeEvent);
        _this2.context.eventService.mapEvent(wheelEvent);
      });
      renderingService.hooks.pointerDown.tap(EventPlugin.tag, function (nativeEvent) {
        if (canvas.supportsTouchEvents && nativeEvent.pointerType === 'touch') return;
        var events = _this2.normalizeToPointerEvent(nativeEvent, canvas);
        if (_this2.autoPreventDefault && events[0].isNormalized) {
          var cancelable = nativeEvent.cancelable || !('cancelable' in nativeEvent);
          if (cancelable) {
            nativeEvent.preventDefault();
          }
        }
        for (var _iterator2 = _createForOfIteratorHelperLoose$2(events), _step2; !(_step2 = _iterator2()).done;) {
          var event = _step2.value;
          var federatedEvent = _this2.bootstrapEvent(_this2.rootPointerEvent, event, canvas);
          _this2.context.eventService.mapEvent(federatedEvent);
        }
        _this2.setCursor(_this2.context.eventService.cursor);
      });
      renderingService.hooks.pointerUp.tap(EventPlugin.tag, function (nativeEvent) {
        if (canvas.supportsTouchEvents && nativeEvent.pointerType === 'touch') return;
        // account for element in SVG
        var $element = _this2.context.contextService.getDomElement();
        var outside = $element && nativeEvent.target && nativeEvent.target !== $element &&
        // @ts-ignore
        $element.contains &&
        // @ts-ignore
        !$element.contains(nativeEvent.target) ? 'outside' : '';
        var normalizedEvents = _this2.normalizeToPointerEvent(nativeEvent, canvas);
        for (var _iterator3 = _createForOfIteratorHelperLoose$2(normalizedEvents), _step3; !(_step3 = _iterator3()).done;) {
          var normalizedEvent = _step3.value;
          var event = _this2.bootstrapEvent(_this2.rootPointerEvent, normalizedEvent, canvas);
          event.type += outside;
          _this2.context.eventService.mapEvent(event);
        }
        _this2.setCursor(_this2.context.eventService.cursor);
      });
      renderingService.hooks.pointerMove.tap(EventPlugin.tag, this.onPointerMove);
      renderingService.hooks.pointerOver.tap(EventPlugin.tag, this.onPointerMove);
      renderingService.hooks.pointerOut.tap(EventPlugin.tag, this.onPointerMove);
    };
    _proto.getViewportXY = function getViewportXY(nativeEvent) {
      var x;
      var y;
      /**
       * Should account for CSS Transform applied on container.
       * @see https://github.com/antvis/G/issues/1161
       * @see https://developer.mozilla.org/zh-CN/docs/Web/API/MouseEvent/offsetX
       */
      var offsetX = nativeEvent.offsetX,
        offsetY = nativeEvent.offsetY,
        clientX = nativeEvent.clientX,
        clientY = nativeEvent.clientY;
      if (this.context.config.supportsCSSTransform && !isNil$1(offsetX) && !isNil$1(offsetY)) {
        x = offsetX;
        y = offsetY;
      } else {
        var point = this.context.eventService.client2Viewport(new Point$1(clientX, clientY));
        x = point.x;
        y = point.y;
      }
      return {
        x: x,
        y: y
      };
    };
    _proto.bootstrapEvent = function bootstrapEvent(event, nativeEvent, view) {
      event.view = view;
      event.originalEvent = null;
      event.nativeEvent = nativeEvent;
      event.pointerId = nativeEvent.pointerId;
      event.width = nativeEvent.width;
      event.height = nativeEvent.height;
      event.isPrimary = nativeEvent.isPrimary;
      event.pointerType = nativeEvent.pointerType;
      event.pressure = nativeEvent.pressure;
      event.tangentialPressure = nativeEvent.tangentialPressure;
      event.tiltX = nativeEvent.tiltX;
      event.tiltY = nativeEvent.tiltY;
      event.twist = nativeEvent.twist;
      this.transferMouseData(event, nativeEvent);
      var _this$getViewportXY = this.getViewportXY(nativeEvent),
        x = _this$getViewportXY.x,
        y = _this$getViewportXY.y;
      event.viewport.x = x;
      event.viewport.y = y;
      var _this$context$eventSe = this.context.eventService.viewport2Canvas(event.viewport),
        canvasX = _this$context$eventSe.x,
        canvasY = _this$context$eventSe.y;
      event.canvas.x = canvasX;
      event.canvas.y = canvasY;
      event.global.copyFrom(event.canvas);
      event.offset.copyFrom(event.canvas);
      event.isTrusted = nativeEvent.isTrusted;
      if (event.type === 'pointerleave') {
        event.type = 'pointerout';
      }
      if (event.type.startsWith('mouse')) {
        event.type = event.type.replace('mouse', 'pointer');
      }
      if (event.type.startsWith('touch')) {
        event.type = TOUCH_TO_POINTER$1[event.type] || event.type;
      }
      return event;
    };
    _proto.normalizeWheelEvent = function normalizeWheelEvent(nativeEvent) {
      var event = this.rootWheelEvent;
      this.transferMouseData(event, nativeEvent);
      event.deltaMode = nativeEvent.deltaMode;
      event.deltaX = nativeEvent.deltaX;
      event.deltaY = nativeEvent.deltaY;
      event.deltaZ = nativeEvent.deltaZ;
      var _this$getViewportXY2 = this.getViewportXY(nativeEvent),
        x = _this$getViewportXY2.x,
        y = _this$getViewportXY2.y;
      event.viewport.x = x;
      event.viewport.y = y;
      var _this$context$eventSe2 = this.context.eventService.viewport2Canvas(event.viewport),
        canvasX = _this$context$eventSe2.x,
        canvasY = _this$context$eventSe2.y;
      event.canvas.x = canvasX;
      event.canvas.y = canvasY;
      event.global.copyFrom(event.canvas);
      event.offset.copyFrom(event.canvas);
      event.nativeEvent = nativeEvent;
      event.type = nativeEvent.type;
      return event;
    }
    /**
     * Transfers base & mouse event data from the nativeEvent to the federated event.
     */;
    _proto.transferMouseData = function transferMouseData(event, nativeEvent) {
      event.isTrusted = nativeEvent.isTrusted;
      event.srcElement = nativeEvent.srcElement;
      event.timeStamp = performance.now();
      event.type = nativeEvent.type;
      event.altKey = nativeEvent.altKey;
      event.metaKey = nativeEvent.metaKey;
      event.shiftKey = nativeEvent.shiftKey;
      event.ctrlKey = nativeEvent.ctrlKey;
      event.button = nativeEvent.button;
      event.buttons = nativeEvent.buttons;
      event.client.x = nativeEvent.clientX;
      event.client.y = nativeEvent.clientY;
      event.movement.x = nativeEvent.movementX;
      event.movement.y = nativeEvent.movementY;
      event.page.x = nativeEvent.pageX;
      event.page.y = nativeEvent.pageY;
      event.screen.x = nativeEvent.screenX;
      event.screen.y = nativeEvent.screenY;
      event.relatedTarget = null;
    };
    _proto.setCursor = function setCursor(cursor) {
      this.context.contextService.applyCursorStyle(cursor || this.context.config.cursor || 'default');
    };
    _proto.normalizeToPointerEvent = function normalizeToPointerEvent(event, canvas) {
      var normalizedEvents = [];
      if (canvas.isTouchEvent(event)) {
        for (var i = 0; i < event.changedTouches.length; i++) {
          var touch = event.changedTouches[i];
          // use changedTouches instead of touches since touchend has no touches
          // @see https://stackoverflow.com/a/10079076
          if (isUndefined$2(touch.button)) touch.button = 0;
          if (isUndefined$2(touch.buttons)) touch.buttons = 1;
          if (isUndefined$2(touch.isPrimary)) {
            touch.isPrimary = event.touches.length === 1 && event.type === 'touchstart';
          }
          if (isUndefined$2(touch.width)) touch.width = touch.radiusX || 1;
          if (isUndefined$2(touch.height)) touch.height = touch.radiusY || 1;
          if (isUndefined$2(touch.tiltX)) touch.tiltX = 0;
          if (isUndefined$2(touch.tiltY)) touch.tiltY = 0;
          if (isUndefined$2(touch.pointerType)) touch.pointerType = 'touch';
          // @see https://developer.mozilla.org/zh-CN/docs/Web/API/Touch/identifier
          if (isUndefined$2(touch.pointerId)) touch.pointerId = touch.identifier || 0;
          if (isUndefined$2(touch.pressure)) touch.pressure = touch.force || 0.5;
          if (isUndefined$2(touch.twist)) touch.twist = 0;
          if (isUndefined$2(touch.tangentialPressure)) touch.tangentialPressure = 0;
          touch.isNormalized = true;
          touch.type = event.type;
          normalizedEvents.push(touch);
        }
      } else if (canvas.isMouseEvent(event)) {
        var tempEvent = event;
        if (isUndefined$2(tempEvent.isPrimary)) tempEvent.isPrimary = true;
        if (isUndefined$2(tempEvent.width)) tempEvent.width = 1;
        if (isUndefined$2(tempEvent.height)) tempEvent.height = 1;
        if (isUndefined$2(tempEvent.tiltX)) tempEvent.tiltX = 0;
        if (isUndefined$2(tempEvent.tiltY)) tempEvent.tiltY = 0;
        if (isUndefined$2(tempEvent.pointerType)) tempEvent.pointerType = 'mouse';
        if (isUndefined$2(tempEvent.pointerId)) tempEvent.pointerId = MOUSE_POINTER_ID$1;
        if (isUndefined$2(tempEvent.pressure)) tempEvent.pressure = 0.5;
        if (isUndefined$2(tempEvent.twist)) tempEvent.twist = 0;
        if (isUndefined$2(tempEvent.tangentialPressure)) tempEvent.tangentialPressure = 0;
        tempEvent.isNormalized = true;
        normalizedEvents.push(tempEvent);
      } else {
        normalizedEvents.push(event);
      }
      return normalizedEvents;
    };
    return EventPlugin;
  }();
  EventPlugin$1.tag = 'Event';

  // group is not a 2d shape
  var shape2D$1 = [Shape$2.CIRCLE, Shape$2.ELLIPSE, Shape$2.IMAGE, Shape$2.RECT, Shape$2.LINE, Shape$2.POLYLINE, Shape$2.POLYGON, Shape$2.TEXT, Shape$2.PATH, Shape$2.HTML];
  var FrustumCullingStrategy$1 = /*#__PURE__*/function () {
    function FrustumCullingStrategy() {}
    var _proto = FrustumCullingStrategy.prototype;
    _proto.isVisible = function isVisible(camera, object) {
      var _object$parentNode, _object$parentNode$cu;
      // return true;
      var cullable = object.cullable;
      if (!cullable.enable) {
        return true;
      }
      var renderBounds = object.getRenderBounds();
      if (AABB$1.isEmpty(renderBounds)) {
        return false;
      }
      // get VP matrix from camera
      var frustum = camera.getFrustum();
      var parentVisibilityPlaneMask = (_object$parentNode = object.parentNode) === null || _object$parentNode === void 0 ? void 0 : (_object$parentNode$cu = _object$parentNode.cullable) === null || _object$parentNode$cu === void 0 ? void 0 : _object$parentNode$cu.visibilityPlaneMask;
      cullable.visibilityPlaneMask = this.computeVisibilityWithPlaneMask(object, renderBounds, parentVisibilityPlaneMask || Mask$1.INDETERMINATE, frustum.planes);
      cullable.visible = cullable.visibilityPlaneMask !== Mask$1.OUTSIDE;
      return cullable.visible;
    }
    /**
     *
     * @see「Optimized View Frustum Culling Algorithms for Bounding Boxes」
     * @see https://github.com/antvis/GWebGPUEngine/issues/3
     *
     * * 基础相交测试 the basic intersection test
     * * 标记 masking @see https://cesium.com/blog/2015/08/04/fast-hierarchical-culling/
     * * TODO: 平面一致性测试 the plane-coherency test
     * * TODO: 支持 mesh 指定自身的剔除策略，参考 Babylon.js @see https://doc.babylonjs.com/how_to/optimizing_your_scene#changing-mesh-culling-strategy
     *
     * @param aabb aabb
     * @param parentPlaneMask mask of parent
     * @param planes planes of frustum
     */;
    _proto.computeVisibilityWithPlaneMask = function computeVisibilityWithPlaneMask(object, aabb, parentPlaneMask, planes) {
      if (parentPlaneMask === Mask$1.OUTSIDE || parentPlaneMask === Mask$1.INSIDE) {
        // 父节点完全位于视锥内或者外部，直接返回
        return parentPlaneMask;
      }
      // Start with MASK_INSIDE (all zeros) so that after the loop, the return value can be compared with MASK_INSIDE.
      // (Because if there are fewer than 31 planes, the upper bits wont be changed.)
      var mask = Mask$1.INSIDE;
      var isShape2D = shape2D$1.indexOf(object.nodeName) > -1;
      // Use viewport culling for 2D shapes
      // @see https://github.com/antvis/g/issues/914
      for (var k = 0, len = planes.length; k < len; ++k) {
        // For k greater than 31 (since 31 is the maximum number of INSIDE/INTERSECTING bits we can store), skip the optimization.
        var flag = 1 << k;
        if ((parentPlaneMask & flag) === 0) {
          // 父节点处于当前面内部，可以跳过
          continue;
        }
        // skip near & far planes when testing 2D shapes
        if (isShape2D && (k === 4 || k === 5)) {
          continue;
        }
        // p-vertex n-vertex <-|plane p-vertex n-vertex
        // 使用 p-vertex 和 n-vertex 加速，避免进行平面和 aabb 全部顶点的相交检测
        var _planes$k = planes[k],
          normal = _planes$k.normal,
          distance = _planes$k.distance;
        if (dot$3(normal, aabb.getPositiveFarPoint(planes[k])) + distance < 0) {
          return Mask$1.OUTSIDE;
        }
        if (dot$3(normal, aabb.getNegativeFarPoint(planes[k])) + distance < 0) {
          // 和当前面相交，对应位置为1，继续检测下一个面
          mask |= flag;
        }
      }
      return mask;
    };
    return FrustumCullingStrategy;
  }();

  var PrepareRendererPlugin$1 = /*#__PURE__*/function () {
    function PrepareRendererPlugin() {
      this.rBush = void 0;
      this.toSync = new Set();
    }
    var _proto = PrepareRendererPlugin.prototype;
    // private isFirstTimeRendering = true;
    _proto.apply = function apply(context) {
      var _this = this;
      var renderingService = context.renderingService,
        renderingContext = context.renderingContext,
        rBushRoot = context.rBushRoot;
      var canvas = renderingContext.root.ownerDocument.defaultView;
      this.rBush = rBushRoot;
      var handleAttributeChanged = function handleAttributeChanged(e) {
        var object = e.target;
        object.renderable.dirty = true;
        renderingService.dirtify();
      };
      var handleBoundsChanged = function handleBoundsChanged(e) {
        var affectChildren = e.detail.affectChildren;
        var object = e.target;
        if (affectChildren) {
          object.forEach(function (node) {
            _this.toSync.add(node);
          });
        }
        var p = object;
        while (p) {
          if (p.renderable) {
            _this.toSync.add(p);
          }
          p = p.parentElement;
        }
        // this.pushToSync(e.composedPath().slice(0, -2) as DisplayObject[]);
        renderingService.dirtify();
      };
      var handleMounted = function handleMounted(e) {
        var object = e.target;
        // recalc style values
        runtime$2.styleValueRegistry.recalc(object);
        runtime$2.sceneGraphService.dirtifyToRoot(object);
        renderingService.dirtify();
      };
      var handleUnmounted = function handleUnmounted(e) {
        var object = e.target;
        var rBushNode = object.rBushNode;
        if (rBushNode.aabb) {
          _this.rBush.remove(rBushNode.aabb);
        }
        _this.toSync.delete(object);
        runtime$2.sceneGraphService.dirtifyToRoot(object);
        renderingService.dirtify();
      };
      renderingService.hooks.init.tapPromise(PrepareRendererPlugin.tag, /*#__PURE__*/_asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee() {
        return _regeneratorRuntime$6().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                canvas.addEventListener(ElementEvent$1.MOUNTED, handleMounted);
                canvas.addEventListener(ElementEvent$1.UNMOUNTED, handleUnmounted);
                canvas.addEventListener(ElementEvent$1.ATTR_MODIFIED, handleAttributeChanged);
                canvas.addEventListener(ElementEvent$1.BOUNDS_CHANGED, handleBoundsChanged);
              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      })));
      renderingService.hooks.destroy.tap(PrepareRendererPlugin.tag, function () {
        canvas.removeEventListener(ElementEvent$1.MOUNTED, handleMounted);
        canvas.removeEventListener(ElementEvent$1.UNMOUNTED, handleUnmounted);
        canvas.removeEventListener(ElementEvent$1.ATTR_MODIFIED, handleAttributeChanged);
        canvas.removeEventListener(ElementEvent$1.BOUNDS_CHANGED, handleBoundsChanged);
      });
      renderingService.hooks.endFrame.tap(PrepareRendererPlugin.tag, function () {
        // if (this.isFirstTimeRendering) {
        //   // @see https://github.com/antvis/G/issues/1117
        //   setTimeout(() => this.syncRTree());
        //   this.isFirstTimeRendering = false;
        // } else {
        //   this.syncRTree();
        // }
        _this.syncRTree();
      });
    };
    _proto.syncRTree = function syncRTree() {
      var _this2 = this;
      // bounds changed, need re-inserting its children
      var bulk = [];
      Array.from(this.toSync)
      // some objects may be removed since last frame
      .filter(function (object) {
        return object.isConnected;
      }).forEach(function (node) {
        var rBushNode = node.rBushNode;
        // clear dirty node
        if (rBushNode && rBushNode.aabb) {
          _this2.rBush.remove(rBushNode.aabb);
        }
        var renderBounds = node.getRenderBounds();
        if (renderBounds) {
          var _renderBounds$getMin = renderBounds.getMin(),
            minX = _renderBounds$getMin[0],
            minY = _renderBounds$getMin[1];
          var _renderBounds$getMax = renderBounds.getMax(),
            maxX = _renderBounds$getMax[0],
            maxY = _renderBounds$getMax[1];
          if (!rBushNode.aabb) {
            // @ts-ignore
            rBushNode.aabb = {};
          }
          rBushNode.aabb.id = node.entity;
          rBushNode.aabb.minX = minX;
          rBushNode.aabb.minY = minY;
          rBushNode.aabb.maxX = maxX;
          rBushNode.aabb.maxY = maxY;
        }
        if (rBushNode.aabb) {
          // TODO: NaN occurs when width/height of Rect is 0
          if (!isNaN(rBushNode.aabb.maxX) && !isNaN(rBushNode.aabb.maxX) && !isNaN(rBushNode.aabb.minX) && !isNaN(rBushNode.aabb.minY)) {
            bulk.push(rBushNode.aabb);
          }
        }
      });
      // use bulk inserting, which is ~2-3 times faster
      // @see https://github.com/mourner/rbush#bulk-inserting-data
      this.rBush.load(bulk);
      bulk.length = 0;
      this.toSync.clear();
    };
    return PrepareRendererPlugin;
  }();
  PrepareRendererPlugin$1.tag = 'Prepare';

  var CanvasEvent$1;
  (function (CanvasEvent) {
    CanvasEvent["READY"] = "ready";
    CanvasEvent["BEFORE_RENDER"] = "beforerender";
    CanvasEvent["AFTER_RENDER"] = "afterrender";
    CanvasEvent["BEFORE_DESTROY"] = "beforedestroy";
    CanvasEvent["AFTER_DESTROY"] = "afterdestroy";
    CanvasEvent["RESIZE"] = "resize";
    CanvasEvent["DIRTY_RECTANGLE"] = "dirtyrectangle";
  })(CanvasEvent$1 || (CanvasEvent$1 = {}));
  var DEFAULT_CAMERA_Z$1 = 500;
  var DEFAULT_CAMERA_NEAR$1 = 0.1;
  var DEFAULT_CAMERA_FAR$1 = 1000;
  /**
   * reuse custom event preventing from re-create them in every frame
   */
  var mountedEvent$1 = new CustomEvent$1(ElementEvent$1.MOUNTED);
  var unmountedEvent$1 = new CustomEvent$1(ElementEvent$1.UNMOUNTED);
  var beforeRenderEvent$1 = new CustomEvent$1(CanvasEvent$1.BEFORE_RENDER);
  var afterRenderEvent$1 = new CustomEvent$1(CanvasEvent$1.AFTER_RENDER);
  /**
   * can be treated like Window in DOM
   * provide some extra methods like `window`, such as:
   * * `window.requestAnimationFrame`
   * * `window.devicePixelRatio`
   *
   * prototype chains: Canvas(Window) -> EventTarget
   */
  var Canvas$2 = /*#__PURE__*/function (_EventTarget) {
    _inheritsLoose$9(Canvas, _EventTarget);
    /**
     * window.document
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/window/requestAnimationFrame
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/cancelAnimationFrame
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio
     */

    /**
     * whether the runtime supports PointerEvent?
     * if not, the event system won't trigger pointer events like `pointerdown`
     */

    /**
     * whether the runtime supports TouchEvent?
     * if not, the event system won't trigger touch events like `touchstart`
     */

    /**
     * is this native event a TouchEvent?
     */

    /**
     * is this native event a MouseEvent?
     */

    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element
     */

    /**
     * rAF in auto rendering
     */

    function Canvas(config) {
      var _this;
      _this = _EventTarget.call(this) || this;
      // create document
      _this.document = void 0;
      _this.customElements = void 0;
      _this.requestAnimationFrame = void 0;
      _this.cancelAnimationFrame = void 0;
      _this.devicePixelRatio = void 0;
      _this.supportsPointerEvents = void 0;
      _this.supportsTouchEvents = void 0;
      _this.isTouchEvent = void 0;
      _this.isMouseEvent = void 0;
      _this.Element = DisplayObject$1;
      _this.frameId = void 0;
      _this.inited = false;
      _this.readyPromise = void 0;
      _this.resolveReadyPromise = void 0;
      _this.context = {};
      _this.document = new Document$1();
      _this.document.defaultView = _assertThisInitialized$5(_this);
      // create registry of custom elements
      _this.customElements = new CustomElementRegistry$1();
      var container = config.container,
        canvas = config.canvas,
        offscreenCanvas = config.offscreenCanvas,
        width = config.width,
        height = config.height,
        devicePixelRatio = config.devicePixelRatio,
        renderer = config.renderer,
        background = config.background,
        cursor = config.cursor,
        document = config.document,
        requestAnimationFrame = config.requestAnimationFrame,
        cancelAnimationFrame = config.cancelAnimationFrame,
        createImage = config.createImage,
        supportsPointerEvents = config.supportsPointerEvents,
        supportsTouchEvents = config.supportsTouchEvents,
        supportsCSSTransform = config.supportsCSSTransform,
        isTouchEvent = config.isTouchEvent,
        isMouseEvent = config.isMouseEvent;
      cleanExistedCanvas$1(container, _assertThisInitialized$5(_this));
      var canvasWidth = width;
      var canvasHeight = height;
      var dpr = devicePixelRatio;
      // use user-defined <canvas> or OffscreenCanvas
      if (canvas) {
        // infer width & height with dpr
        dpr = devicePixelRatio || isBrowser$1 && window.devicePixelRatio || 1;
        dpr = dpr >= 1 ? Math.ceil(dpr) : 1;
        canvasWidth = width || getWidth$1(canvas) || canvas.width / dpr;
        canvasHeight = height || getHeight$1(canvas) || canvas.height / dpr;
      }
      /**
       * implements `Window` interface
       */
      _this.devicePixelRatio = dpr;
      _this.requestAnimationFrame = requestAnimationFrame !== null && requestAnimationFrame !== void 0 ? requestAnimationFrame : raf$1.bind(globalThis);
      _this.cancelAnimationFrame = cancelAnimationFrame !== null && cancelAnimationFrame !== void 0 ? cancelAnimationFrame : caf$1.bind(globalThis);
      /**
       * limits query
       */
      // the following feature-detect from hammer.js
      // @see https://github.com/hammerjs/hammer.js/blob/master/src/inputjs/input-consts.js#L5
      _this.supportsTouchEvents = supportsTouchEvents !== null && supportsTouchEvents !== void 0 ? supportsTouchEvents : 'ontouchstart' in globalThis;
      _this.supportsPointerEvents = supportsPointerEvents !== null && supportsPointerEvents !== void 0 ? supportsPointerEvents : !!globalThis.PointerEvent;
      _this.isTouchEvent = isTouchEvent !== null && isTouchEvent !== void 0 ? isTouchEvent : function (event) {
        return _this.supportsTouchEvents && event instanceof globalThis.TouchEvent;
      };
      _this.isMouseEvent = isMouseEvent !== null && isMouseEvent !== void 0 ? isMouseEvent : function (event) {
        return !globalThis.MouseEvent || event instanceof globalThis.MouseEvent && (!_this.supportsPointerEvents || !(event instanceof globalThis.PointerEvent));
      };
      _this.initRenderingContext({
        container: container,
        canvas: canvas,
        width: canvasWidth,
        height: canvasHeight,
        renderer: renderer,
        offscreenCanvas: offscreenCanvas,
        devicePixelRatio: dpr,
        cursor: cursor || 'default',
        background: background || 'transparent',
        createImage: createImage,
        document: document,
        supportsCSSTransform: supportsCSSTransform
      });
      _this.initDefaultCamera(canvasWidth, canvasHeight);
      _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee() {
        return _regeneratorRuntime$6().wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return _this.initRenderer(renderer);
              case 2:
                _this.dispatchEvent(new CustomEvent$1(CanvasEvent$1.READY));
                if (_this.readyPromise) {
                  _this.resolveReadyPromise();
                }
              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }))();
      return _this;
    }
    var _proto = Canvas.prototype;
    _proto.initRenderingContext = function initRenderingContext(mergedConfig) {
      this.context.config = mergedConfig;
      // bind rendering context, shared by all renderers
      this.context.renderingContext = {
        /**
         * the root node in scene graph
         */
        root: this.document.documentElement,
        renderListCurrentFrame: [],
        unculledEntities: [],
        renderReasons: new Set(),
        force: false,
        dirty: false
      };
    };
    _proto.initDefaultCamera = function initDefaultCamera(width, height) {
      var _this2 = this;
      // set a default ortho camera
      var camera = new runtime$2.CameraContribution();
      camera.setType(CameraType$1.EXPLORING, CameraTrackingMode$1.DEFAULT).setPosition(width / 2, height / 2, DEFAULT_CAMERA_Z$1).setFocalPoint(width / 2, height / 2, 0).setOrthographic(width / -2, width / 2, height / 2, height / -2, DEFAULT_CAMERA_NEAR$1, DEFAULT_CAMERA_FAR$1);
      // keep ref since it will use raf in camera animation
      camera.canvas = this;
      // redraw when camera changed
      camera.eventEmitter.on(CameraEvent$1.UPDATED, function () {
        _this2.context.renderingContext.renderReasons.add(RenderReason$1.CAMERA_CHANGED);
      });
      // bind camera
      this.context.camera = camera;
    };
    _proto.getConfig = function getConfig() {
      return this.context.config;
    }
    /**
     * get the root displayObject in scenegraph
     * @alias this.document.documentElement
     */;
    _proto.getRoot = function getRoot() {
      return this.document.documentElement;
    }
    /**
     * get the camera of canvas
     */;
    _proto.getCamera = function getCamera() {
      return this.context.camera;
    };
    _proto.getContextService = function getContextService() {
      return this.context.contextService;
    };
    _proto.getEventService = function getEventService() {
      return this.context.eventService;
    };
    _proto.getRenderingService = function getRenderingService() {
      return this.context.renderingService;
    };
    _proto.getRenderingContext = function getRenderingContext() {
      return this.context.renderingContext;
    };
    _proto.getStats = function getStats() {
      return this.getRenderingService().getStats();
    }
    // /**
    //  * @see https://developer.mozilla.org/zh-CN/docs/Web/API/Window/getComputedStyle
    //  */
    // getComputedStyle(node: DisplayObject) {
    //   return node.computedStyle;
    // }
    ;
    _proto.destroy = function destroy(destroyScenegraph, skipTriggerEvent) {
      if (destroyScenegraph === void 0) {
        destroyScenegraph = true;
      }
      if (skipTriggerEvent === void 0) {
        skipTriggerEvent = false;
      }
      if (!skipTriggerEvent) {
        this.dispatchEvent(new CustomEvent$1(CanvasEvent$1.BEFORE_DESTROY));
      }
      if (this.frameId) {
        var _cancelRAF = this.getConfig().cancelAnimationFrame || cancelAnimationFrame;
        _cancelRAF(this.frameId);
      }
      // unmount all children
      var root = this.getRoot();
      this.unmountChildren(root);
      // root.children.forEach((child: DisplayObject) => {
      //   this.unmountChildren(child);
      // });
      if (destroyScenegraph) {
        // destroy Document
        this.document.destroy();
      }
      // destroy services
      this.getRenderingService().destroy();
      this.getContextService().destroy();
      if (!skipTriggerEvent) {
        this.dispatchEvent(new CustomEvent$1(CanvasEvent$1.AFTER_DESTROY));
      }
    }
    /**
     * compatible with G 3.0
     * @deprecated
     * @alias resize
     */;
    _proto.changeSize = function changeSize(width, height) {
      this.resize(width, height);
    };
    _proto.resize = function resize(width, height) {
      // update canvas' config
      var canvasConfig = this.context.config;
      canvasConfig.width = width;
      canvasConfig.height = height;
      // resize context
      this.getContextService().resize(width, height);
      // resize camera
      var camera = this.context.camera;
      var projectionMode = camera.getProjectionMode();
      camera.setPosition(width / 2, height / 2, DEFAULT_CAMERA_Z$1).setFocalPoint(width / 2, height / 2, 0);
      if (projectionMode === CameraProjectionMode$1.ORTHOGRAPHIC) {
        camera.setOrthographic(width / -2, width / 2, height / 2, height / -2, camera.getNear(), camera.getFar());
      } else {
        camera.setAspect(width / height);
      }
      this.dispatchEvent(new CustomEvent$1(CanvasEvent$1.RESIZE, {
        width: width,
        height: height
      }));
    }
    // proxy to document.documentElement
    ;
    _proto.appendChild = function appendChild(child, index) {
      return this.document.documentElement.appendChild(child, index);
    };
    _proto.insertBefore = function insertBefore(newChild, refChild) {
      return this.document.documentElement.insertBefore(newChild, refChild);
    };
    _proto.removeChild = function removeChild(child) {
      return this.document.documentElement.removeChild(child);
    }
    /**
     * Remove all children which can be appended to its original parent later again.
     */;
    _proto.removeChildren = function removeChildren() {
      this.document.documentElement.removeChildren();
    }
    /**
     * Recursively destroy all children which can not be appended to its original parent later again.
     */;
    _proto.destroyChildren = function destroyChildren() {
      this.document.documentElement.destroyChildren();
    };
    _proto.render = function render() {
      this.dispatchEvent(beforeRenderEvent$1);
      var renderingService = this.getRenderingService();
      renderingService.render(this.getConfig());
      this.dispatchEvent(afterRenderEvent$1);
    };
    _proto.run = function run() {
      var _this3 = this;
      var tick = function tick() {
        _this3.render();
        _this3.frameId = requestAnimationFrame(tick);
      };
      tick();
    };
    _proto.initRenderer = /*#__PURE__*/function () {
      var _initRenderer = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee2(renderer) {
        return _regeneratorRuntime$6().wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (renderer) {
                  _context2.next = 2;
                  break;
                }
                throw new Error('Renderer is required.');
              case 2:
                // reset
                this.inited = false;
                this.readyPromise = undefined;
                // FIXME: should re-create here?
                this.context.rBushRoot = new RBush$1();
                // reset rendering plugins
                this.context.renderingPlugins = [];
                this.context.renderingPlugins.push(new EventPlugin$1(), new PrepareRendererPlugin$1(), new DirtyCheckPlugin$1(), new CullingPlugin$1([new FrustumCullingStrategy$1()]));
                //
                this.loadRendererContainerModule(renderer);
                // init context service
                this.context.contextService = new this.context.ContextService(_extends$3({}, runtime$2, this.context));
                // init rendering service
                this.context.renderingService = new RenderingService$1(runtime$2, this.context);
                // init event service
                this.context.eventService = new EventService$1(runtime$2, this.context);
                this.context.eventService.init();
                _context2.next = 14;
                return this.context.contextService.init();
              case 14:
                _context2.next = 16;
                return this.context.renderingService.init();
              case 16:
                this.inited = true;
                this.getRoot().forEach(function (node) {
                  var renderable = node.renderable;
                  if (renderable) {
                    renderable.renderBoundsDirty = true;
                    renderable.boundsDirty = true;
                    renderable.dirty = true;
                  }
                });
                // keep current scenegraph unchanged, just trigger mounted event
                this.mountChildren(this.getRoot());
                if (renderer.getConfig().enableAutoRendering) {
                  this.run();
                }
              case 20:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
      function initRenderer(_x) {
        return _initRenderer.apply(this, arguments);
      }
      return initRenderer;
    }();
    _proto.loadRendererContainerModule = function loadRendererContainerModule(renderer) {
      var _this4 = this;
      // load other container modules provided by g-canvas/g-svg/g-webgl
      var plugins = renderer.getPlugins();
      plugins.forEach(function (plugin) {
        plugin.context = _this4.context;
        plugin.init();
      });
    };
    _proto.setRenderer = /*#__PURE__*/function () {
      var _setRenderer = _asyncToGenerator$7( /*#__PURE__*/_regeneratorRuntime$6().mark(function _callee3(renderer) {
        var canvasConfig, oldRenderer;
        return _regeneratorRuntime$6().wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                // update canvas' config
                canvasConfig = this.getConfig();
                if (!(canvasConfig.renderer === renderer)) {
                  _context3.next = 3;
                  break;
                }
                return _context3.abrupt("return");
              case 3:
                oldRenderer = canvasConfig.renderer;
                canvasConfig.renderer = renderer;
                // keep all children undestroyed
                this.destroy(false, true);
                // destroy all plugins, reverse will mutate origin array
                [].concat(oldRenderer === null || oldRenderer === void 0 ? void 0 : oldRenderer.getPlugins()).reverse().forEach(function (plugin) {
                  plugin.destroy();
                });
                _context3.next = 9;
                return this.initRenderer(renderer);
              case 9:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));
      function setRenderer(_x2) {
        return _setRenderer.apply(this, arguments);
      }
      return setRenderer;
    }();
    _proto.setCursor = function setCursor(cursor) {
      var canvasConfig = this.getConfig();
      canvasConfig.cursor = cursor;
      this.getContextService().applyCursorStyle(cursor);
    };
    _proto.unmountChildren = function unmountChildren(parent) {
      var _this5 = this;
      // unmountChildren recursively
      parent.childNodes.forEach(function (child) {
        _this5.unmountChildren(child);
      });
      if (this.inited) {
        if (parent.isMutationObserved) {
          parent.dispatchEvent(unmountedEvent$1);
        } else {
          unmountedEvent$1.target = parent;
          this.dispatchEvent(unmountedEvent$1, true);
        }
        // skip document.documentElement
        if (parent !== this.document.documentElement) {
          parent.ownerDocument = null;
        }
        parent.isConnected = false;
      }
      // trigger after unmounted
      if (parent.isCustomElement) {
        if (parent.disconnectedCallback) {
          parent.disconnectedCallback();
        }
      }
    };
    _proto.mountChildren = function mountChildren(parent) {
      var _this6 = this;
      if (this.inited) {
        if (!parent.isConnected) {
          parent.ownerDocument = this.document;
          parent.isConnected = true;
          if (parent.isMutationObserved) {
            parent.dispatchEvent(mountedEvent$1);
          } else {
            mountedEvent$1.target = parent;
            this.dispatchEvent(mountedEvent$1, true);
          }
        }
      } else {
        console.warn("[g]: You are trying to call `canvas.appendChild` before canvas' initialization finished. You can either await `canvas.ready` or listen to `CanvasEvent.READY` manually.", 'appended child: ', parent.nodeName);
      }
      // recursively mount children
      parent.childNodes.forEach(function (child) {
        _this6.mountChildren(child);
      });
      // trigger after mounted
      if (parent.isCustomElement) {
        if (parent.connectedCallback) {
          parent.connectedCallback();
        }
      }
    };
    _proto.client2Viewport = function client2Viewport(client) {
      return this.getEventService().client2Viewport(client);
    };
    _proto.viewport2Client = function viewport2Client(canvas) {
      return this.getEventService().viewport2Client(canvas);
    };
    _proto.viewport2Canvas = function viewport2Canvas(viewport) {
      return this.getEventService().viewport2Canvas(viewport);
    };
    _proto.canvas2Viewport = function canvas2Viewport(canvas) {
      return this.getEventService().canvas2Viewport(canvas);
    }
    /**
     * @deprecated
     * @alias client2Viewport
     */;
    _proto.getPointByClient = function getPointByClient(clientX, clientY) {
      return this.client2Viewport({
        x: clientX,
        y: clientY
      });
    }
    /**
     * @deprecated
     * @alias viewport2Client
     */;
    _proto.getClientByPoint = function getClientByPoint(x, y) {
      return this.viewport2Client({
        x: x,
        y: y
      });
    };
    _createClass$5(Canvas, [{
      key: "ready",
      get: function get() {
        var _this7 = this;
        if (!this.readyPromise) {
          this.readyPromise = new Promise(function (resolve) {
            _this7.resolveReadyPromise = function () {
              resolve(_this7);
            };
          });
          if (this.inited) {
            this.resolveReadyPromise();
          }
        }
        return this.readyPromise;
      }
    }]);
    return Canvas;
  }(EventTarget$1);

  function _extends$4() {
    _extends$4 = Object.assign ? Object.assign.bind() : function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];
        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }
      return target;
    };
    return _extends$4.apply(this, arguments);
  }
  function _objectWithoutPropertiesLoose$3(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }
    return target;
  }

  var _excluded$e = ["offset", "easing"],
    _excluded2 = ["ignore", "easing", "offset"];
  var eps = 0.0001;
  /**
   * Provides some control methods like:
   * - play
   * - pause
   * - stop
   * - goToAndStop
   * - goToAndPlay
   * @see https://github.com/airbnb/lottie-web/blob/master/player/js/animation/AnimationItem.js
   */
  var LottieAnimation = /*#__PURE__*/function () {
    function LottieAnimation(width, height, elements, context) {
      var _this = this;
      this.width = void 0;
      this.height = void 0;
      this.elements = void 0;
      this.context = void 0;
      this.displayObjects = void 0;
      this.keyframeAnimationMap = new WeakMap();
      this.displayObjectElementMap = new WeakMap();
      this.animations = [];
      this.isPaused = false;
      this.direction = 1;
      this.width = width;
      this.height = height;
      this.elements = elements;
      this.context = context;
      this.displayObjects = elements.map(function (element) {
        return _this.buildHierachy(element);
      });
      // TODO: preload images
      // TODO: preload fonts
    }
    var _proto = LottieAnimation.prototype;
    _proto.generateTransform = function generateTransform(tx, ty, scaleX, scaleY, rotation) {
      var transformStr = '';
      if (tx !== 0 || ty !== 0) {
        transformStr += "translate(" + tx + ", " + ty + ")";
      }
      if (scaleX !== 1 || scaleY !== 1) {
        transformStr += " scale(" + (scaleX === 0 ? eps : scaleX) + ", " + (scaleY === 0 ? eps : scaleY) + ")";
      }
      if (rotation !== 0) {
        transformStr += " rotate(" + rotation + "deg)";
      }
      return transformStr;
    };
    _proto.buildHierachy = function buildHierachy(element) {
      var _this2 = this;
      var type = element.type,
        name = element.name,
        _element$anchorX = element.anchorX,
        anchorX = _element$anchorX === void 0 ? 0 : _element$anchorX,
        _element$anchorY = element.anchorY,
        anchorY = _element$anchorY === void 0 ? 0 : _element$anchorY,
        _element$rotation = element.rotation,
        rotation = _element$rotation === void 0 ? 0 : _element$rotation,
        _element$scaleX = element.scaleX,
        scaleX = _element$scaleX === void 0 ? 1 : _element$scaleX,
        _element$scaleY = element.scaleY,
        scaleY = _element$scaleY === void 0 ? 1 : _element$scaleY,
        _element$x = element.x,
        x = _element$x === void 0 ? 0 : _element$x,
        _element$y = element.y,
        y = _element$y === void 0 ? 0 : _element$y,
        children = element.children,
        shape = element.shape,
        style = element.style,
        keyframeAnimation = element.keyframeAnimation;
      var displayObject;
      var transform = this.generateTransform(x - anchorX, y - anchorY, scaleX, scaleY, rotation);
      // const transformMat = mat4.fromRotationTranslationScaleOrigin(
      //   mat4.create(),
      //   quat.fromEuler(quat.create(), 0, 0, rotation),
      //   [x - anchorX, y - anchorY, 0],
      //   [scaleX, scaleY, 1],
      //   [anchorX, anchorY, 0],
      // );
      // TODO: repeater @see https://lottiefiles.github.io/lottie-docs/shapes/#repeater
      // @see https://lottiefiles.github.io/lottie-docs/shapes/#shape
      // TODO: polystar, convert to Bezier @see https://lottiefiles.github.io/lottie-docs/rendering/#polystar
      if (type === Shape$2.GROUP) {
        displayObject = new Group$1({
          style: {
            transformOrigin: anchorX + "px " + anchorY + "px",
            transform: transform
          }
        });
      } else if (type === Shape$2.ELLIPSE) {
        var cx = shape.cx,
          cy = shape.cy,
          rx = shape.rx,
          ry = shape.ry;
        // const center = vec3.fromValues(cx, cy, 0);
        // vec3.transformMat4(center, center, transformMat);
        displayObject = new Ellipse$1({
          style: {
            // cx: center[0],
            // cy: center[1],
            cx: cx,
            cy: cy,
            rx: rx,
            ry: ry,
            // reset transform-origin based on anchor & center
            transformOrigin: anchorX - cx + rx + "px " + (anchorY - cy + ry) + "px",
            transform: transform
          }
        });
      } else if (type === Shape$2.PATH) {
        var d = this.generatePathFromShape(shape);
        displayObject = new Path$1({
          style: {
            d: d,
            transformOrigin: anchorX + "px " + anchorY + "px",
            transform: transform
          }
        });
      } else if (type === Shape$2.RECT) {
        // @see https://lottiefiles.github.io/lottie-docs/shapes/#rectangle
        var _cx = shape.x,
          _cy = shape.y,
          width = shape.width,
          height = shape.height,
          r = shape.r;
        displayObject = new Rect$3({
          style: {
            x: _cx,
            y: _cy,
            width: width,
            height: height,
            anchor: [0.5, 0.5],
            radius: r,
            transformOrigin: anchorX - _cx + width / 2 + "px " + (anchorY - _cy + height / 2) + "px",
            transform: transform
          }
        });
      } else if (type === Shape$2.IMAGE) {
        var _width = shape.width,
          _height = shape.height,
          src = shape.src;
        displayObject = new Image$1({
          style: {
            x: 0,
            y: 0,
            width: _width,
            height: _height,
            src: src,
            transformOrigin: anchorX + "px " + anchorY + "px",
            transform: transform
          }
        });
      }
      if (name) {
        displayObject.name = name;
      }
      // TODO: match name `mn`, used in expressions
      if (style) {
        // { fill, fillOpacity, fillRule, opacity, lineDash, lineDashOffset }
        displayObject.attr(style);
      }
      if (keyframeAnimation) {
        this.keyframeAnimationMap.set(displayObject, keyframeAnimation);
      }
      if (children) {
        var _displayObject;
        var childNodes = children.map(function (child) {
          return _this2.buildHierachy(child);
        });
        (_displayObject = displayObject).append.apply(_displayObject, childNodes);
      }
      this.displayObjectElementMap.set(displayObject, element);
      return displayObject;
    }
    /**
     * Returns the animation duration in seconds or frames.
     * @see https://github.com/airbnb/lottie-web#getdurationinframes
     */;
    _proto.getDuration = function getDuration(inFrames) {
      if (inFrames === void 0) {
        inFrames = false;
      }
      return (inFrames ? this.fps() : 1) * (this.context.endFrame - this.context.startFrame) * this.context.frameTime;
    }
    /**
     * Returns the animation frame rate (frames / second).
     */;
    _proto.fps = function fps() {
      return this.context.fps;
    };
    _proto.isSameKeyframeOptions = function isSameKeyframeOptions(options1, options2) {
      return options1.delay === options2.delay && options1.duration === options2.duration && options1.easing === options2.easing;
    };
    _proto.isSameKeyframes = function isSameKeyframes(keyframe1, keyframe2) {
      return keyframe1.easing === keyframe2.easing && keyframe1.offset === keyframe2.offset;
    };
    _proto.generatePathFromShape = function generatePathFromShape(shape) {
      // @see https://lottiefiles.github.io/lottie-docs/shapes/#path
      var close = shape.close,
        v = shape.v,
        i = shape.in,
        out = shape.out;
      var d = [];
      d.push(['M', v[0][0], v[0][1]]);
      for (var n = 1; n < v.length; n++) {
        // @see https://lottiefiles.github.io/lottie-docs/concepts/#bezier
        // The nth bezier segment is defined as:
        // v[n], v[n]+o[n], v[n+1]+i[n+1], v[n+1]
        d.push(['C', out[n - 1][0], out[n - 1][1], i[n][0], i[n][1], v[n][0], v[n][1]]);
      }
      if (close) {
        d.push(['C', out[v.length - 1][0], out[v.length - 1][1], i[0][0], i[0][1], v[0][0], v[0][1]]);
        d.push(['Z']);
      }
      return d;
    }
    /**
     * render Lottie Group to canvas or a mounted display object
     */;
    _proto.render = function render(canvasOrDisplayObject) {
      var _this3 = this;
      var wrapper = new Group$1();
      wrapper.append.apply(wrapper, this.displayObjects);
      if (canvasOrDisplayObject instanceof Canvas$2) {
        canvasOrDisplayObject.appendChild(wrapper);
      } else if (canvasOrDisplayObject instanceof DisplayObject$1) {
        if (!canvasOrDisplayObject.isConnected) {
          throw new Error('[g-lottie-player]: Cannot render Lottie to an unmounted DisplayObject.');
        } else {
          canvasOrDisplayObject.appendChild(wrapper);
        }
      } else {
        throw new Error('[g-lottie-player]: We should render Lottie to a mounted DisplayObject or Canvas.');
      }
      this.displayObjects.forEach(function (parent) {
        parent.forEach(function (child) {
          var keyframeAnimation = _this3.keyframeAnimationMap.get(child);
          var element = _this3.displayObjectElementMap.get(child);
          if (element && element.clipPath) {
            var _element$clipPath = element.clipPath,
              shape = _element$clipPath.shape,
              _keyframeAnimation = _element$clipPath.keyframeAnimation;
            var clipPath = new Path$1();
            // use clipPath as target's siblings
            child.parentElement.appendChild(clipPath);
            child.style.clipPath = clipPath;
            if (shape) {
              clipPath.style.d = _this3.generatePathFromShape(shape);
            }
            // TODO: only support one clipPath now
            if (_keyframeAnimation && _keyframeAnimation.length) {
              var _keyframeAnimation$ = _keyframeAnimation[0],
                delay = _keyframeAnimation$.delay,
                duration = _keyframeAnimation$.duration,
                easing = _keyframeAnimation$.easing,
                keyframes = _keyframeAnimation$.keyframes;
              // animate clipPath with its `d` property
              var clipPathAnimation = clipPath.animate(keyframes.map(function (_ref) {
                var offset = _ref.offset,
                  shape = _ref.shape,
                  easing = _ref.easing;
                return {
                  offset: offset,
                  d: path2String(_this3.generatePathFromShape(shape)),
                  easing: easing
                };
              }),
              //
              {
                delay: delay,
                duration: duration,
                easing: easing,
                iterations: _this3.context.iterations
              });
              _this3.animations.push(clipPathAnimation);
            }
          }
          if (keyframeAnimation && keyframeAnimation.length) {
            var _this3$animations;
            var keyframesOptions = [];
            keyframeAnimation.map(function (_ref2) {
              var _ref2$delay = _ref2.delay,
                delay = _ref2$delay === void 0 ? 0 : _ref2$delay,
                duration = _ref2.duration,
                easing = _ref2.easing,
                keyframes = _ref2.keyframes;
              var formattedKeyframes = keyframes.map(function (keyframe) {
                return definedProps(keyframe);
              });
              var options = definedProps({
                delay: delay,
                duration: duration,
                easing: easing,
                iterations: _this3.context.iterations
              });
              keyframesOptions.push([formattedKeyframes, options]);
            });
            var mergedKeyframesOptions = [keyframesOptions[0]];
            // merge [{ offset: 0, cx: 1 }, { offset: 0, cy: 1 }] into { offset: 0, cx: 1, cy: 1 }
            var _loop = function _loop(i) {
              var _keyframesOptions$i = keyframesOptions[i],
                currentKeyframes = _keyframesOptions$i[0],
                currentOptions = _keyframesOptions$i[1];
              // can merge options?
              var existedKeyframeOptions = mergedKeyframesOptions.find(function (_ref4) {
                var keyframes = _ref4[0],
                  options = _ref4[1];
                return keyframes.length === currentKeyframes.length && _this3.isSameKeyframeOptions(currentOptions, options);
              });
              if (existedKeyframeOptions) {
                currentKeyframes.forEach(function (currentKeyframe) {
                  var existedKeyframe = existedKeyframeOptions[0].find(function (keyframe) {
                    return _this3.isSameKeyframes(currentKeyframe, keyframe);
                  });
                  if (existedKeyframe) {
                    var offset = currentKeyframe.offset,
                      _currentKeyframe$easi = currentKeyframe.easing,
                      rest = _objectWithoutPropertiesLoose$3(currentKeyframe, _excluded$e);
                    // merge interpolated properties
                    Object.assign(existedKeyframe, rest);
                  } else {
                    // append if cannot be merged
                    existedKeyframeOptions[0].push(currentKeyframe);
                  }
                });
              } else {
                // cannot be merged since options are different
                mergedKeyframesOptions.push(keyframesOptions[i]);
              }
            };
            for (var i = 1; i < keyframesOptions.length; i++) {
              _loop(i);
            }
            // restore animations for later use
            (_this3$animations = _this3.animations).push.apply(_this3$animations, mergedKeyframesOptions.map(function (_ref3) {
              var merged = _ref3[0],
                options = _ref3[1];
              // format interpolated properties, e.g. scaleX -> transform
              var formatted = _this3.formatKeyframes(merged, child);
              if (formatted.length) {
                // console.log(child, formatted);
                var animation = child.animate(formatted, options);
                if (!_this3.context.autoplay) {
                  animation.pause();
                }
                return animation;
              }
            }).filter(function (animation) {
              return !!animation;
            }));
          }
        });
      });
      return wrapper;
    };
    _proto.formatKeyframes = function formatKeyframes(keyframes, object) {
      keyframes.forEach(function (keyframe) {
        // if ('offsetPath' in keyframe) {
        //   if (!object.style.offsetPath) {
        //     const [ox, oy] = object.getOrigin();
        //     (keyframe.offsetPath as AbsoluteArray).forEach((segment) => {
        //       if (segment[0] === 'M') {
        //         segment[1] -= ox;
        //         segment[2] -= oy;
        //       } else if (segment[0] === 'C') {
        //         segment[1] -= ox;
        //         segment[2] -= oy;
        //         segment[3] -= ox;
        //         segment[4] -= oy;
        //         segment[5] -= ox;
        //         segment[6] -= oy;
        //       }
        //     });
        //     const offsetPath = new Path({
        //       style: {
        //         d: keyframe.offsetPath,
        //       },
        //     });
        //     object.style.offsetPath = offsetPath;
        //   }
        //   delete keyframe.offsetPath;
        //   // offsetPath should override x/y
        //   delete keyframe.x;
        //   delete keyframe.y;
        // }
        // // should keep transform during initialization
        // if (!object.style.offsetPath) {
        //   keyframe.transform = object.style.transform || '';
        // }
        keyframe.transform = object.style.transform || '';
        // TODO: transforms with different easing functions will conflict
        if ('scaleX' in keyframe) {
          keyframe.transform = (keyframe.transform || '') + (" scaleX(" + (keyframe.scaleX === 0 ? eps : keyframe.scaleX) + ")");
          delete keyframe.scaleX;
        }
        if ('scaleY' in keyframe) {
          keyframe.transform = (keyframe.transform || '') + (" scaleY(" + (keyframe.scaleY === 0 ? eps : keyframe.scaleY) + ")");
          delete keyframe.scaleY;
        }
        if ('rotation' in keyframe) {
          keyframe.transform = (keyframe.transform || '') + (" rotate(" + keyframe.rotation + "deg)");
          delete keyframe.rotation;
        }
        // TODO: skew & skewAxis
        // if ('skew' in keyframe) {
        //   keyframe.transform = (keyframe.transform || '') + ` skew(${keyframe.skew}deg)`;
        //   delete keyframe.skew;
        // }
        if ('x' in keyframe) {
          keyframe.transform = (keyframe.transform || '') + (" translateX(" + keyframe.x + "px)");
          delete keyframe.x;
        }
        if ('y' in keyframe) {
          keyframe.transform = (keyframe.transform || '') + (" translateY(" + keyframe.y + "px)");
          delete keyframe.y;
        }
        // { style: { opacity: 1 } }
        if ('style' in keyframe) {
          Object.keys(keyframe.style).forEach(function (name) {
            keyframe[name] = keyframe.style[name];
          });
          delete keyframe.style;
        }
      });
      // ignore empty interpolable attributes
      keyframes = keyframes.filter(function (keyframe) {
        // TODO: support negative offset
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        var ignore = keyframe.ignore,
          easing = keyframe.easing,
          offset = keyframe.offset,
          rest = _objectWithoutPropertiesLoose$3(keyframe, _excluded2);
        return offset >= 0 && Object.keys(rest).length > 0;
        // return Object.keys(rest).length > 0;
      });

      if (keyframes.length) {
        // padding offset = 1
        if (keyframes[keyframes.length - 1].offset !== 1) {
          keyframes.push(_extends$4({}, keyframes[keyframes.length - 1], {
            offset: 1
          }));
        }
      }
      // sort by offset
      keyframes.sort(function (a, b) {
        return a.offset - b.offset;
      });
      return keyframes;
    }
    /**
     * Destroy all internal displayobjects.
     */;
    _proto.destroy = function destroy() {
      this.displayObjects.forEach(function (object) {
        object.destroy();
      });
    }
    /**
     * Return the size of this animation.
     * @param outputSize - If provided, the size will be copied into here as width, height.
     */;
    _proto.size = function size(outputSize) {
      return {
        width: this.width,
        height: this.height
      };
    }
    /**
     * Bodymovin version
     */;
    _proto.version = function version() {
      return this.context.version;
    };
    _proto.play = function play() {
      this.isPaused = false;
      this.animations.forEach(function (animation) {
        animation.play();
      });
    };
    _proto.pause = function pause() {
      this.isPaused = true;
      this.animations.forEach(function (animation) {
        animation.pause();
      });
    }
    /**
     *
     */;
    _proto.togglePause = function togglePause() {
      if (this.isPaused) {
        this.play();
      } else {
        this.pause();
      }
    }
    /**
     * split goToAndStop/Play into goTo & stop/play
     * @see https://github.com/airbnb/lottie-web
     */;
    _proto.goTo = function goTo(value, isFrame) {
      var _this4 = this;
      if (isFrame === void 0) {
        isFrame = false;
      }
      if (isFrame) {
        this.animations.forEach(function (animation) {
          animation.currentTime = value / _this4.fps() * _this4.getDuration();
        });
      } else {
        this.animations.forEach(function (animation) {
          animation.currentTime = value;
        });
      }
    }
    /**
     * @see https://github.com/airbnb/lottie-web#stop
     */;
    _proto.stop = function stop() {
      this.animations.forEach(function (animation) {
        animation.finish();
      });
    }
    /**
     * 1 is normal speed.
     * @see https://github.com/airbnb/lottie-web#setspeedspeed
     */;
    _proto.setSpeed = function setSpeed(speed) {
      var _this5 = this;
      this.animations.forEach(function (animation) {
        animation.playbackRate = speed * _this5.direction;
      });
    };
    /**
     * 1 is forward, -1 is reverse.
     * @see https://github.com/airbnb/lottie-web#setdirectiondirection
     */
    _proto.setDirection = function setDirection(direction) {
      this.direction = direction;
      this.animations.forEach(function (animation) {
        animation.playbackRate *= direction;
      });
    };
    return LottieAnimation;
  }();

  /**
   * borrow from https://github.com/airbnb/lottie-web/blob/master/player/js/utils/DataManager.js#L40-L493
   */
  function completeLayers(layers, comps) {
    var layerData;
    var i;
    var len = layers.length;
    var j;
    var jLen;
    var k;
    var kLen;
    for (i = 0; i < len; i += 1) {
      layerData = layers[i];
      if ('ks' in layerData && !layerData.completed) {
        layerData.completed = true;
        if (layerData.tt) {
          layers[i - 1].td = layerData.tt;
        }
        if (layerData.hasMask) {
          var maskProps = layerData.masksProperties;
          jLen = maskProps.length;
          for (j = 0; j < jLen; j += 1) {
            if (maskProps[j].pt.k.i) {
              convertPathsToAbsoluteValues(maskProps[j].pt.k);
            } else {
              kLen = maskProps[j].pt.k.length;
              for (k = 0; k < kLen; k += 1) {
                if (maskProps[j].pt.k[k].s) {
                  convertPathsToAbsoluteValues(maskProps[j].pt.k[k].s[0]);
                }
                if (maskProps[j].pt.k[k].e) {
                  convertPathsToAbsoluteValues(maskProps[j].pt.k[k].e[0]);
                }
              }
            }
          }
        }
        if (layerData.ty === 0) {
          layerData.layers = findCompLayers(layerData.refId, comps);
          completeLayers(layerData.layers, comps);
        } else if (layerData.ty === 4) {
          completeShapes(layerData.shapes);
        } else if (layerData.ty === 5) {
          completeText(layerData);
        }
      }
    }
  }
  function completeChars(chars, assets) {
    if (chars) {
      var i = 0;
      var len = chars.length;
      for (i = 0; i < len; i += 1) {
        if (chars[i].t === 1) {
          // var compData = findComp(chars[i].data.refId, assets);
          chars[i].data.layers = findCompLayers(chars[i].data.refId, assets);
          // chars[i].data.ip = 0;
          // chars[i].data.op = 99999;
          // chars[i].data.st = 0;
          // chars[i].data.sr = 1;
          // chars[i].w = compData.w;
          // chars[i].data.ks = {
          //   a: { k: [0, 0, 0], a: 0 },
          //   p: { k: [0, -compData.h, 0], a: 0 },
          //   r: { k: 0, a: 0 },
          //   s: { k: [100, 100], a: 0 },
          //   o: { k: 100, a: 0 },
          // };
          completeLayers(chars[i].data.layers, assets);
        }
      }
    }
  }
  function findComp(id, comps) {
    var i = 0;
    var len = comps.length;
    while (i < len) {
      if (comps[i].id === id) {
        return comps[i];
      }
      i += 1;
    }
    return null;
  }
  function findCompLayers(id, comps) {
    var comp = findComp(id, comps);
    if (comp) {
      if (!comp.layers.__used) {
        comp.layers.__used = true;
        return comp.layers;
      }
      return JSON.parse(JSON.stringify(comp.layers));
    }
    return null;
  }
  function completeShapes(arr) {
    var i;
    var len = arr.length;
    var j;
    var jLen;
    for (i = len - 1; i >= 0; i -= 1) {
      if (arr[i].ty === 'sh') {
        if (arr[i].ks.k.i) {
          convertPathsToAbsoluteValues(arr[i].ks.k);
        } else {
          jLen = arr[i].ks.k.length;
          for (j = 0; j < jLen; j += 1) {
            if (arr[i].ks.k[j].s) {
              convertPathsToAbsoluteValues(arr[i].ks.k[j].s[0]);
            }
            if (arr[i].ks.k[j].e) {
              convertPathsToAbsoluteValues(arr[i].ks.k[j].e[0]);
            }
          }
        }
      } else if (arr[i].ty === 'gr') {
        completeShapes(arr[i].it);
      }
    }
  }
  function convertPathsToAbsoluteValues(path) {
    var i;
    var len = path.i.length;
    for (i = 0; i < len; i += 1) {
      path.i[i][0] += path.v[i][0];
      path.i[i][1] += path.v[i][1];
      path.o[i][0] += path.v[i][0];
      path.o[i][1] += path.v[i][1];
    }
  }
  function checkVersion(minimum, animVersionString) {
    var animVersion = animVersionString ? animVersionString.split('.') : [100, 100, 100];
    if (minimum[0] > animVersion[0]) {
      return true;
    }
    if (animVersion[0] > minimum[0]) {
      return false;
    }
    if (minimum[1] > animVersion[1]) {
      return true;
    }
    if (animVersion[1] > minimum[1]) {
      return false;
    }
    if (minimum[2] > animVersion[2]) {
      return true;
    }
    if (animVersion[2] > minimum[2]) {
      return false;
    }
    return null;
  }
  var checkText = function () {
    var minimumVersion = [4, 4, 14];
    function updateTextLayer(textLayer) {
      var documentData = textLayer.t.d;
      textLayer.t.d = {
        k: [{
          s: documentData,
          t: 0
        }]
      };
    }
    function iterateLayers(layers) {
      var i;
      var len = layers.length;
      for (i = 0; i < len; i += 1) {
        if (layers[i].ty === 5) {
          updateTextLayer(layers[i]);
        }
      }
    }
    return function (animationData) {
      if (checkVersion(minimumVersion, animationData.v)) {
        iterateLayers(animationData.layers);
        if (animationData.assets) {
          var i;
          var len = animationData.assets.length;
          for (i = 0; i < len; i += 1) {
            if (animationData.assets[i].layers) {
              iterateLayers(animationData.assets[i].layers);
            }
          }
        }
      }
    };
  }();
  var checkChars = function () {
    var minimumVersion = [4, 7, 99];
    return function (animationData) {
      if (animationData.chars && !checkVersion(minimumVersion, animationData.v)) {
        var i;
        var len = animationData.chars.length;
        for (i = 0; i < len; i += 1) {
          var charData = animationData.chars[i];
          if (charData.data && charData.data.shapes) {
            completeShapes(charData.data.shapes);
            charData.data.ip = 0;
            charData.data.op = 99999;
            charData.data.st = 0;
            charData.data.sr = 1;
            charData.data.ks = {
              p: {
                k: [0, 0],
                a: 0
              },
              s: {
                k: [100, 100],
                a: 0
              },
              a: {
                k: [0, 0],
                a: 0
              },
              r: {
                k: 0,
                a: 0
              },
              o: {
                k: 100,
                a: 0
              }
            };
            if (!animationData.chars[i].t) {
              charData.data.shapes.push({
                ty: 'no'
              });
              charData.data.shapes[0].it.push({
                p: {
                  k: [0, 0],
                  a: 0
                },
                s: {
                  k: [100, 100],
                  a: 0
                },
                a: {
                  k: [0, 0],
                  a: 0
                },
                r: {
                  k: 0,
                  a: 0
                },
                o: {
                  k: 100,
                  a: 0
                },
                sk: {
                  k: 0,
                  a: 0
                },
                sa: {
                  k: 0,
                  a: 0
                },
                ty: 'tr'
              });
            }
          }
        }
      }
    };
  }();
  var checkPathProperties = function () {
    var minimumVersion = [5, 7, 15];
    function updateTextLayer(textLayer) {
      var pathData = textLayer.t.p;
      if (typeof pathData.a === 'number') {
        pathData.a = {
          a: 0,
          k: pathData.a
        };
      }
      if (typeof pathData.p === 'number') {
        pathData.p = {
          a: 0,
          k: pathData.p
        };
      }
      if (typeof pathData.r === 'number') {
        pathData.r = {
          a: 0,
          k: pathData.r
        };
      }
    }
    function iterateLayers(layers) {
      var i;
      var len = layers.length;
      for (i = 0; i < len; i += 1) {
        if (layers[i].ty === 5) {
          updateTextLayer(layers[i]);
        }
      }
    }
    return function (animationData) {
      if (checkVersion(minimumVersion, animationData.v)) {
        iterateLayers(animationData.layers);
        if (animationData.assets) {
          var i;
          var len = animationData.assets.length;
          for (i = 0; i < len; i += 1) {
            if (animationData.assets[i].layers) {
              iterateLayers(animationData.assets[i].layers);
            }
          }
        }
      }
    };
  }();
  var checkColors = function () {
    var minimumVersion = [4, 1, 9];
    function iterateShapes(shapes) {
      var i;
      var len = shapes.length;
      var j;
      var jLen;
      for (i = 0; i < len; i += 1) {
        if (shapes[i].ty === 'gr') {
          iterateShapes(shapes[i].it);
        } else if (shapes[i].ty === 'fl' || shapes[i].ty === 'st') {
          if (shapes[i].c.k && shapes[i].c.k[0].i) {
            jLen = shapes[i].c.k.length;
            for (j = 0; j < jLen; j += 1) {
              if (shapes[i].c.k[j].s) {
                shapes[i].c.k[j].s[0] /= 255;
                shapes[i].c.k[j].s[1] /= 255;
                shapes[i].c.k[j].s[2] /= 255;
                shapes[i].c.k[j].s[3] /= 255;
              }
              if (shapes[i].c.k[j].e) {
                shapes[i].c.k[j].e[0] /= 255;
                shapes[i].c.k[j].e[1] /= 255;
                shapes[i].c.k[j].e[2] /= 255;
                shapes[i].c.k[j].e[3] /= 255;
              }
            }
          } else {
            shapes[i].c.k[0] /= 255;
            shapes[i].c.k[1] /= 255;
            shapes[i].c.k[2] /= 255;
            shapes[i].c.k[3] /= 255;
          }
        }
      }
    }
    function iterateLayers(layers) {
      var i;
      var len = layers.length;
      for (i = 0; i < len; i += 1) {
        if (layers[i].ty === 4) {
          iterateShapes(layers[i].shapes);
        }
      }
    }
    return function (animationData) {
      if (checkVersion(minimumVersion, animationData.v)) {
        iterateLayers(animationData.layers);
        if (animationData.assets) {
          var i;
          var len = animationData.assets.length;
          for (i = 0; i < len; i += 1) {
            if (animationData.assets[i].layers) {
              iterateLayers(animationData.assets[i].layers);
            }
          }
        }
      }
    };
  }();
  var checkShapes = function () {
    var minimumVersion = [4, 4, 18];
    function completeClosingShapes(arr) {
      var i;
      var len = arr.length;
      var j;
      var jLen;
      for (i = len - 1; i >= 0; i -= 1) {
        if (arr[i].ty === 'sh') {
          if (arr[i].ks.k.i) {
            arr[i].ks.k.c = arr[i].closed;
          } else {
            jLen = arr[i].ks.k.length;
            for (j = 0; j < jLen; j += 1) {
              if (arr[i].ks.k[j].s) {
                arr[i].ks.k[j].s[0].c = arr[i].closed;
              }
              if (arr[i].ks.k[j].e) {
                arr[i].ks.k[j].e[0].c = arr[i].closed;
              }
            }
          }
        } else if (arr[i].ty === 'gr') {
          completeClosingShapes(arr[i].it);
        }
      }
    }
    function iterateLayers(layers) {
      var layerData;
      var i;
      var len = layers.length;
      var j;
      var jLen;
      var k;
      var kLen;
      for (i = 0; i < len; i += 1) {
        layerData = layers[i];
        if (layerData.hasMask) {
          var maskProps = layerData.masksProperties;
          jLen = maskProps.length;
          for (j = 0; j < jLen; j += 1) {
            if (maskProps[j].pt.k.i) {
              maskProps[j].pt.k.c = maskProps[j].cl;
            } else {
              kLen = maskProps[j].pt.k.length;
              for (k = 0; k < kLen; k += 1) {
                if (maskProps[j].pt.k[k].s) {
                  maskProps[j].pt.k[k].s[0].c = maskProps[j].cl;
                }
                if (maskProps[j].pt.k[k].e) {
                  maskProps[j].pt.k[k].e[0].c = maskProps[j].cl;
                }
              }
            }
          }
        }
        if (layerData.ty === 4) {
          completeClosingShapes(layerData.shapes);
        }
      }
    }
    return function (animationData) {
      if (checkVersion(minimumVersion, animationData.v)) {
        iterateLayers(animationData.layers);
        if (animationData.assets) {
          var i;
          var len = animationData.assets.length;
          for (i = 0; i < len; i += 1) {
            if (animationData.assets[i].layers) {
              iterateLayers(animationData.assets[i].layers);
            }
          }
        }
      }
    };
  }();
  function completeData(animationData) {
    if (animationData.__complete) {
      return;
    }
    checkColors(animationData);
    checkText(animationData);
    checkChars(animationData);
    checkPathProperties(animationData);
    checkShapes(animationData);
    completeLayers(animationData.layers, animationData.assets);
    completeChars(animationData.chars, animationData.assets);
    animationData.__complete = true;
  }
  function completeText(data) {
    if (data.t.a.length === 0 && !('m' in data.t.p)) ;
  }

  /**
   * https://lottiefiles.github.io/lottie-docs/constants/
   */
  var BlendMode;
  (function (BlendMode) {
    BlendMode[BlendMode["Normal"] = 0] = "Normal";
    BlendMode[BlendMode["Multiply"] = 1] = "Multiply";
    BlendMode[BlendMode["Screen"] = 2] = "Screen";
    BlendMode[BlendMode["Overlay"] = 3] = "Overlay";
    BlendMode[BlendMode["Darken"] = 4] = "Darken";
    BlendMode[BlendMode["Lighten"] = 5] = "Lighten";
    BlendMode[BlendMode["ColorDodge"] = 6] = "ColorDodge";
    BlendMode[BlendMode["ColorBurn"] = 7] = "ColorBurn";
    BlendMode[BlendMode["HardLight"] = 8] = "HardLight";
    BlendMode[BlendMode["SoftLight"] = 9] = "SoftLight";
    BlendMode[BlendMode["Difference"] = 10] = "Difference";
    BlendMode[BlendMode["Exclusion"] = 11] = "Exclusion";
    BlendMode[BlendMode["Hue"] = 12] = "Hue";
    BlendMode[BlendMode["Saturation"] = 13] = "Saturation";
    BlendMode[BlendMode["Color"] = 14] = "Color";
    BlendMode[BlendMode["Luminosity"] = 15] = "Luminosity";
    BlendMode[BlendMode["Add"] = 16] = "Add";
    BlendMode[BlendMode["HardMix"] = 17] = "HardMix";
  })(BlendMode || (BlendMode = {}));
  /**
   * @see https://lottiefiles.github.io/lottie-docs/constants/#mattemode
   */
  var MatteMode;
  (function (MatteMode) {
    MatteMode[MatteMode["Normal"] = 0] = "Normal";
    MatteMode[MatteMode["Alpha"] = 1] = "Alpha";
    MatteMode[MatteMode["InvertedAlpha"] = 2] = "InvertedAlpha";
    MatteMode[MatteMode["Luma"] = 3] = "Luma";
    MatteMode[MatteMode["InvertedLuma"] = 4] = "InvertedLuma";
  })(MatteMode || (MatteMode = {}));
  var Layer3DMode;
  (function (Layer3DMode) {
    Layer3DMode[Layer3DMode["Off"] = 0] = "Off";
    Layer3DMode[Layer3DMode["On"] = 1] = "On";
  })(Layer3DMode || (Layer3DMode = {}));
  var AutoOrientMode;
  (function (AutoOrientMode) {
    AutoOrientMode[AutoOrientMode["Off"] = 0] = "Off";
    AutoOrientMode[AutoOrientMode["On"] = 1] = "On";
  })(AutoOrientMode || (AutoOrientMode = {}));
  var EffectValueType;
  (function (EffectValueType) {
    EffectValueType[EffectValueType["Number"] = 0] = "Number";
    EffectValueType[EffectValueType["Color"] = 2] = "Color";
    EffectValueType[EffectValueType["MultiDimensional"] = 3] = "MultiDimensional";
    EffectValueType[EffectValueType["Boolean"] = 7] = "Boolean";
  })(EffectValueType || (EffectValueType = {}));
  var EffectType;
  (function (EffectType) {
    EffectType[EffectType["Transform"] = 5] = "Transform";
    EffectType[EffectType["DropShadow"] = 25] = "DropShadow";
  })(EffectType || (EffectType = {}));
  var LayerType;
  (function (LayerType) {
    LayerType[LayerType["precomp"] = 0] = "precomp";
    LayerType[LayerType["solid"] = 1] = "solid";
    LayerType[LayerType["image"] = 2] = "image";
    LayerType[LayerType["null"] = 3] = "null";
    LayerType[LayerType["shape"] = 4] = "shape";
    LayerType[LayerType["text"] = 5] = "text";
    LayerType[LayerType["audio"] = 6] = "audio";
    LayerType[LayerType["pholderVideo"] = 7] = "pholderVideo";
    LayerType[LayerType["imageSeq"] = 8] = "imageSeq";
    LayerType[LayerType["video"] = 9] = "video";
    LayerType[LayerType["pholderStill"] = 10] = "pholderStill";
    LayerType[LayerType["guide"] = 11] = "guide";
    LayerType[LayerType["adjustment"] = 12] = "adjustment";
    LayerType[LayerType["camera"] = 13] = "camera";
    LayerType[LayerType["light"] = 14] = "light";
    LayerType[LayerType["data"] = 15] = "data";
  })(LayerType || (LayerType = {}));
  var TextJustify;
  (function (TextJustify) {
    TextJustify[TextJustify["Left"] = 0] = "Left";
    TextJustify[TextJustify["Right"] = 1] = "Right";
    TextJustify[TextJustify["Center"] = 2] = "Center";
  })(TextJustify || (TextJustify = {}));
  var VerticalJustify;
  (function (VerticalJustify) {
    VerticalJustify[VerticalJustify["Top"] = 0] = "Top";
    VerticalJustify[VerticalJustify["Center"] = 1] = "Center";
    VerticalJustify[VerticalJustify["Bottom"] = 2] = "Bottom";
  })(VerticalJustify || (VerticalJustify = {}));
  /**
   * @see https://lottiefiles.github.io/lottie-docs/shapes/#shape-types
   */
  var ShapeType;
  (function (ShapeType) {
    ShapeType["Rectangle"] = "rc";
    ShapeType["Ellipse"] = "el";
    ShapeType["PolyStar"] = "sr";
    ShapeType["Path"] = "sh";
    ShapeType["Fill"] = "fl";
    ShapeType["Stroke"] = "st";
    ShapeType["GradientFill"] = "gf";
    ShapeType["GradientStroke"] = "gs";
    ShapeType["NoStyle"] = "no";
    ShapeType["Group"] = "gr";
    ShapeType["Transform"] = "tr";
    ShapeType["Repeater"] = "rp";
    ShapeType["Trim"] = "tm";
    ShapeType["RoundedCorners"] = "rd";
    ShapeType["PuckerOrBloat"] = "pb";
    ShapeType["Merge"] = "mm";
    ShapeType["Twist"] = "tw";
    ShapeType["OffsetPath"] = "op";
    ShapeType["ZigZag"] = "zz";
  })(ShapeType || (ShapeType = {}));
  /**
   * @see https://lottiefiles.github.io/lottie-docs/constants/#fillrule
   */
  var FillRule;
  (function (FillRule) {
    FillRule[FillRule["NonZero"] = 1] = "NonZero";
    FillRule[FillRule["EvenOdd"] = 2] = "EvenOdd";
  })(FillRule || (FillRule = {}));
  /**
   * @see https://lottiefiles.github.io/lottie-docs/constants/#linejoin
   */
  var LineJoin;
  (function (LineJoin) {
    LineJoin[LineJoin["Miter"] = 1] = "Miter";
    LineJoin[LineJoin["Round"] = 2] = "Round";
    LineJoin[LineJoin["Bevel"] = 3] = "Bevel";
  })(LineJoin || (LineJoin = {}));
  /**
   * @see https://lottiefiles.github.io/lottie-docs/constants/#linecap
   */
  var LineCap;
  (function (LineCap) {
    LineCap[LineCap["Butt"] = 1] = "Butt";
    LineCap[LineCap["Round"] = 2] = "Round";
    LineCap[LineCap["Square"] = 3] = "Square";
  })(LineCap || (LineCap = {}));
  /**
   * @see https://lottiefiles.github.io/lottie-docs/constants/#gradienttype
   */
  var GradientType$2;
  (function (GradientType) {
    GradientType[GradientType["Linear"] = 1] = "Linear";
    GradientType[GradientType["Radial"] = 2] = "Radial";
  })(GradientType$2 || (GradientType$2 = {}));
  var FontPathOrigin;
  (function (FontPathOrigin) {
    FontPathOrigin[FontPathOrigin["CssUrl"] = 1] = "CssUrl";
    FontPathOrigin[FontPathOrigin["ScriptUrl"] = 2] = "ScriptUrl";
    FontPathOrigin[FontPathOrigin["FontUrl"] = 3] = "FontUrl";
  })(FontPathOrigin || (FontPathOrigin = {}));

  var ParseContext = function ParseContext() {
    this.fps = void 0;
    this.frameTime = 1000 / 30;
    this.startFrame = 0;
    this.endFrame = void 0;
    this.version = void 0;
    this.autoplay = false;
    this.iterations = 0;
    this.assetsMap = new Map();
    this.layerOffsetTime = void 0;
  };
  function isNumberArray$1(val) {
    return Array.isArray(val) && typeof val[0] === 'number';
  }
  function isMultiDimensionalValue(val) {
    return isNumberArray$1(val === null || val === void 0 ? void 0 : val.k);
  }
  function isMultiDimensionalKeyframedValue(val) {
    var k = val === null || val === void 0 ? void 0 : val.k;
    return Array.isArray(k) && k[0].t !== undefined && isNumberArray$1(k[0].s);
  }
  function isValue(val) {
    // TODO is [100] sort of value?
    return typeof (val === null || val === void 0 ? void 0 : val.k) === 'number';
  }
  function isKeyframedValue(val) {
    var k = val === null || val === void 0 ? void 0 : val.k;
    return Array.isArray(k) && k[0].t !== undefined && typeof k[0].s === 'number';
  }
  function toColorString(val) {
    var opacity = getMultiDimensionValue(val, 3);
    return "rgba(" + [Math.round(getMultiDimensionValue(val, 0) * 255), Math.round(getMultiDimensionValue(val, 1) * 255), Math.round(getMultiDimensionValue(val, 2) * 255), !isNil$1(opacity) ? opacity : 1].join(',') + ")";
  }
  function getMultiDimensionValue(val, dimIndex) {
    return val != null ? typeof val === 'number' ? val : val[dimIndex || 0] : NaN;
  }
  /**
   * @see https://lottiefiles.github.io/lottie-docs/concepts/#easing-handles
   */
  function getMultiDimensionEasingBezierString(kf, nextKf, dimIndex) {
    var _kf$o, _kf$o2, _kf$i, _kf$i2;
    var bezierEasing = [];
    bezierEasing.push(((_kf$o = kf.o) === null || _kf$o === void 0 ? void 0 : _kf$o.x) && (getMultiDimensionValue(kf.o.x, dimIndex) || getMultiDimensionValue(kf.o.x, 0)) || 0, ((_kf$o2 = kf.o) === null || _kf$o2 === void 0 ? void 0 : _kf$o2.y) && (getMultiDimensionValue(kf.o.y, dimIndex) || getMultiDimensionValue(kf.o.y, 0)) || 0, ((_kf$i = kf.i) === null || _kf$i === void 0 ? void 0 : _kf$i.x) && (getMultiDimensionValue(kf.i.x, dimIndex) || getMultiDimensionValue(kf.i.x, 0)) || 1, ((_kf$i2 = kf.i) === null || _kf$i2 === void 0 ? void 0 : _kf$i2.y) && (getMultiDimensionValue(kf.i.y, dimIndex) || getMultiDimensionValue(kf.i.y, 0)) || 1);
    // linear by default
    if (!(bezierEasing[0] === 0 && bezierEasing[1] === 0 && bezierEasing[2] === 1 && bezierEasing[3] === 1)) {
      return "cubic-bezier(" + bezierEasing.join(',') + ")";
    }
    return;
  }
  /**
   * @see https://lottiefiles.github.io/lottie-docs/concepts/#keyframe
   */
  function parseKeyframe(kfs, bezierEasingDimIndex, context, setVal) {
    var kfsLen = kfs.length;
    // const offset = context.layerStartTime;
    var duration = context.endFrame - context.startFrame;
    var out = {
      duration: 0,
      delay: 0,
      keyframes: []
    };
    var prevKf;
    for (var i = 0; i < kfsLen; i++) {
      var _prevKf;
      var kf = kfs[i];
      var nextKf = kfs[i + 1];
      // If h is present and it's 1, you don't need i and o,
      // as the property will keep the same value until the next keyframe.
      var isDiscrete = kf.h === 1;
      var offset = (kf.t + context.layerOffsetTime - context.startFrame) / duration;
      var outKeyframe = {
        offset: offset
      };
      if (!isDiscrete) {
        outKeyframe.easing = getMultiDimensionEasingBezierString(kf, nextKf, bezierEasingDimIndex);
      }
      // Use end state of later frame if start state not exits.
      // @see https://lottiefiles.github.io/lottie-docs/concepts/#old-lottie-keyframes
      var startVal = kf.s || ((_prevKf = prevKf) === null || _prevKf === void 0 ? void 0 : _prevKf.e);
      if (startVal) {
        setVal(outKeyframe, startVal);
      }
      if (outKeyframe.offset > 0 && i === 0) {
        // Set initial
        var initialKeyframe = {
          offset: 0
        };
        if (startVal) {
          setVal(initialKeyframe, startVal);
        }
        out.keyframes.push(initialKeyframe);
      }
      out.keyframes.push(outKeyframe);
      if (isDiscrete && nextKf) {
        // Use two keyframe to simulate the discrete animation.
        var extraKeyframe = {
          offset: Math.max((nextKf.t + context.layerOffsetTime - context.startFrame) / duration, 0)
        };
        setVal(extraKeyframe, startVal);
        out.keyframes.push(extraKeyframe);
      }
      prevKf = kf;
    }
    if (kfsLen) {
      out.duration = context.frameTime * duration;
    }
    return out;
  }
  function parseOffsetKeyframe(kfs, targetPropName, propNames, keyframeAnimations, context, convertVal) {
    var _loop = function _loop(dimIndex) {
      var propName = propNames[dimIndex];
      var keyframeAnim = parseKeyframe(kfs, dimIndex, context, function (outKeyframe, startVal) {
        var val = getMultiDimensionValue(startVal, dimIndex);
        if (convertVal) {
          val = convertVal(val);
        }
        (targetPropName ? outKeyframe[targetPropName] = {} : outKeyframe)[propName] = val;
      });
      // moving position around a curved path
      var needOffsetPath = kfs.some(function (kf) {
        return kf.ti && kf.to;
      });
      if (needOffsetPath) {
        var offsetPath = [];
        kfs.forEach(function (kf, i) {
          keyframeAnim.keyframes[i].offsetPath = offsetPath;
          // convert to & ti(Tangent for values (eg: moving position around a curved path)) to offsetPath & offsetDistance
          // @see https://lottiefiles.github.io/lottie-docs/concepts/#animated-position
          if (kf.ti && kf.to) {
            if (i === 0) {
              offsetPath.push(['M', kf.s[0], kf.s[1]]);
            }
            keyframeAnim.keyframes[i].segmentLength = getTotalLength$1(offsetPath);
            // @see https://lottiefiles.github.io/lottie-docs/concepts/#bezier
            // The nth bezier segment is defined as:
            // v[n], v[n]+o[n], v[n+1]+i[n+1], v[n+1]
            offsetPath.push(['C', kf.s[0] + kf.to[0], kf.s[1] + kf.to[1], kf.s[0] + kf.ti[0], kf.s[1] + kf.ti[1], kf.e[0], kf.e[1]]);
          }
        });
        // calculate offsetDistance: segmentLength / totalLength
        var totalLength = getTotalLength$1(offsetPath);
        keyframeAnim.keyframes.forEach(function (kf) {
          kf.offsetDistance = isNil$1(kf.segmentLength) ? 1 : kf.segmentLength / totalLength;
          delete kf.segmentLength;
        });
      }
      if (keyframeAnim.keyframes.length) {
        keyframeAnimations.push(keyframeAnim);
      }
    };
    for (var dimIndex = 0; dimIndex < propNames.length; dimIndex++) {
      _loop(dimIndex);
    }
  }
  function parseColorOffsetKeyframe(kfs, targetPropName, propName, keyframeAnimations, context) {
    var keyframeAnim = parseKeyframe(kfs, 0, context, function (outKeyframe, startVal) {
      (targetPropName ? outKeyframe[targetPropName] = {} : outKeyframe)[propName] = toColorString(startVal);
    });
    if (keyframeAnim.keyframes.length) {
      keyframeAnimations.push(keyframeAnim);
    }
  }
  function parseValue(lottieVal, attrs, targetPropName, propNames, animations, context, convertVal) {
    if (targetPropName) {
      attrs[targetPropName] = attrs[targetPropName] || {};
    }
    var target = targetPropName ? attrs[targetPropName] : attrs;
    if (isValue(lottieVal)) {
      var val = lottieVal.k;
      target[propNames[0]] = convertVal ? convertVal(val) : val;
    } else if (isKeyframedValue(lottieVal)) {
      parseOffsetKeyframe(lottieVal.k, targetPropName, propNames, animations, context, convertVal);
    } else if (isMultiDimensionalValue(lottieVal)) {
      for (var i = 0; i < propNames.length; i++) {
        var _val = getMultiDimensionValue(lottieVal.k, i);
        target[propNames[i]] = convertVal ? convertVal(_val) : _val;
      }
    } else if (isMultiDimensionalKeyframedValue(lottieVal)) {
      // TODO Merge dimensions
      parseOffsetKeyframe(lottieVal.k, targetPropName, propNames, animations, context, convertVal);
    }
  }
  /**
   * @see https://lottiefiles.github.io/lottie-docs/concepts/#transform
   */
  function parseTransforms(ks, attrs, animations, context, targetProp, transformProps) {
    if (targetProp === void 0) {
      targetProp = '';
    }
    if (transformProps === void 0) {
      transformProps = {
        x: 'x',
        y: 'y',
        rotation: 'rotation',
        scaleX: 'scaleX',
        scaleY: 'scaleY',
        anchorX: 'anchorX',
        anchorY: 'anchorY',
        skew: 'skew',
        skewAxis: 'skewAxis'
      };
    }
    // @see https://lottiefiles.github.io/lottie-docs/concepts/#split-vector
    if (ks.p.s) {
      parseValue(ks.p.x, attrs, targetProp, [transformProps.x], animations, context);
      parseValue(ks.p.y, attrs, targetProp, [transformProps.y], animations, context);
    } else {
      parseValue(ks.p, attrs, targetProp, [transformProps.x, transformProps.y], animations, context);
    }
    parseValue(ks.s, attrs, targetProp, [transformProps.scaleX, transformProps.scaleY], animations, context, function (val) {
      return val / 100;
    });
    parseValue(ks.r, attrs, targetProp, [transformProps.rotation], animations, context);
    parseValue(ks.a, attrs, targetProp, [transformProps.anchorX, transformProps.anchorY], animations, context);
    parseValue(ks.sk, attrs, targetProp, [transformProps.skew], animations, context);
    parseValue(ks.sa, attrs, targetProp, [transformProps.skewAxis], animations, context);
  }
  function isGradientFillOrStroke(fl) {
    return fl.g && fl.s && fl.e;
  }
  function convertColorStops(arr, count) {
    var colorStops = [];
    for (var i = 0; i < count * 4;) {
      var offset = arr[i++];
      var r = Math.round(arr[i++] * 255);
      var g = Math.round(arr[i++] * 255);
      var b = Math.round(arr[i++] * 255);
      colorStops.push({
        offset: offset,
        color: "rgb(" + r + ", " + g + ", " + b + ")"
      });
    }
    return colorStops;
  }
  function joinColorStops(colorStops) {
    return "" + colorStops.map(function (_ref) {
      var offset = _ref.offset,
        color = _ref.color;
      return color + " " + offset * 100 + "%";
    }).join(', ');
  }
  /**
   * TODO:
   * * Transition
   * * Highlight length & angle in Radial Gradient
   *
   * @see https://lottiefiles.github.io/lottie-docs/concepts/#gradients
   * @see https://lottiefiles.github.io/lottie-docs/shapes/#gradients
   */
  function parseGradient$3(shape) {
    var colorArr = shape.g.k.k;
    var colorStops = convertColorStops(colorArr, shape.g.p);
    // @see https://lottiefiles.github.io/lottie-docs/constants/#gradienttype
    if (shape.t === GradientType$2.Linear) {
      var angle = rad2deg$1(Math.atan2(shape.e.k[1] - shape.s.k[1], shape.e.k[0] - shape.s.k[0]));
      // @see https://g-next.antv.vision/zh/docs/api/css/css-properties-values-api#linear-gradient
      return "linear-gradient(" + angle + "deg, " + joinColorStops(colorStops) + ")";
    } else if (shape.t === GradientType$2.Radial) {
      // TODO: highlight length & angle (h & a)
      // Highlight Length, as a percentage between s and e
      // Highlight Angle, relative to the direction from s to e
      var size = distanceSquareRoot$1(shape.e.k, shape.s.k);
      // @see https://g-next.antv.vision/zh/docs/api/css/css-properties-values-api#radial-gradient
      return "radial-gradient(circle " + size + "px at " + shape.s.k[0] + "px " + shape.s.k[1] + "px, " + joinColorStops(colorStops) + ")";
    } else {
      // Invalid gradient
      return '#000';
    }
  }
  function parseFill(fl, attrs, animations, context) {
    attrs.style = attrs.style || {};
    // Color
    if (isGradientFillOrStroke(fl)) {
      attrs.style.fill = parseGradient$3(fl);
    } else {
      if (isMultiDimensionalValue(fl.c)) {
        attrs.style.fill = toColorString(fl.c.k);
      } else if (isMultiDimensionalKeyframedValue(fl.c)) {
        parseColorOffsetKeyframe(fl.c.k, 'style', 'fill', animations, context);
      }
    }
    // FillRule @see https://lottiefiles.github.io/lottie-docs/constants/#fillrule
    attrs.style.fillRule = fl.r === FillRule.EvenOdd ? 'evenodd' : 'nonzero';
    // Opacity
    parseValue(fl.o, attrs, 'style', ['fillOpacity'], animations, context, function (opacity) {
      return opacity / 100;
    });
  }
  function parseStroke(st, attrs, animations, context) {
    attrs.style = attrs.style || {};
    // Color
    if (isGradientFillOrStroke(st)) {
      attrs.style.stroke = parseGradient$3(st);
    } else {
      if (isMultiDimensionalValue(st.c)) {
        attrs.style.stroke = toColorString(st.c.k);
      } else if (isMultiDimensionalKeyframedValue(st.c)) {
        parseColorOffsetKeyframe(st.c.k, 'style', 'stroke', animations, context);
      }
    }
    // Opacity
    parseValue(st.o, attrs, 'style', ['strokeOpacity'], animations, context, function (opacity) {
      return opacity / 100;
    });
    // Line width
    parseValue(st.w, attrs, 'style', ['lineWidth'], animations, context);
    switch (st.lj) {
      case LineJoin.Bevel:
        attrs.style.lineJoin = 'bevel';
        break;
      case LineJoin.Round:
        attrs.style.lineJoin = 'round';
        break;
      case LineJoin.Miter:
        attrs.style.lineJoin = 'miter';
        break;
    }
    switch (st.lc) {
      case LineCap.Butt:
        attrs.style.lineCap = 'butt';
        break;
      case LineCap.Round:
        attrs.style.lineCap = 'round';
        break;
      case LineCap.Square:
        attrs.style.lineCap = 'square';
        break;
    }
    // Line dash
    var dashArray = [];
    var dashOffset = 0;
    if (st.d) {
      st.d.forEach(function (item) {
        if (item.n !== 'o') {
          dashArray.push(item.v.k);
        } else {
          dashOffset = item.v.k;
        }
      });
      attrs.style.lineDash = dashArray;
      attrs.style.lineDashOffset = dashOffset;
    }
  }
  function isBezier(k) {
    return k && k.i && k.o && k.v;
  }
  /**
   * @see https://lottiefiles.github.io/lottie-docs/shapes/#path
   */
  function parseShapePaths(shape, animations, context) {
    var attrs = {
      type: Shape$2.PATH,
      // Should have no fill and stroke by default
      style: {
        fill: 'none',
        stroke: 'none'
      }
    };
    // @see https://lottiefiles.github.io/lottie-docs/concepts/#bezier
    if (isBezier(shape.ks.k)) {
      attrs.shape = {
        in: shape.ks.k.i,
        out: shape.ks.k.o,
        v: shape.ks.k.v,
        close: shape.ks.k.c
      };
    } else if (Array.isArray(shape.ks.k)) {
      var keyframeAnim = parseKeyframe(shape.ks.k, 0, context, function (outKeyframe, startVal) {
        outKeyframe.shape = {
          in: startVal[0].i,
          out: startVal[0].o,
          v: startVal[0].v,
          close: startVal[0].c
        };
      });
      if (keyframeAnim.keyframes.length) {
        animations.push(keyframeAnim);
      }
    }
    return attrs;
  }
  /**
   * @see https://lottiefiles.github.io/lottie-docs/shapes/#rectangle
   */
  function parseShapeRect(shape, animations, context) {
    var attrs = {
      type: Shape$2.RECT,
      // Should have no fill and stroke by default
      style: {
        fill: 'none',
        stroke: 'none'
      },
      shape: {}
    };
    parseValue(shape.p, attrs, 'shape', ['x', 'y'], animations, context);
    parseValue(shape.s, attrs, 'shape', ['width', 'height'], animations, context);
    parseValue(shape.r, attrs, 'shape', ['r'], animations, context);
    return attrs;
  }
  /**
   * @see https://lottiefiles.github.io/lottie-docs/layers/#image-layer
   */
  function parseImageLayer(layer, context) {
    var attrs = {
      type: Shape$2.IMAGE,
      style: {},
      shape: {
        width: 0,
        height: 0,
        src: ''
      }
    };
    var asset = context.assetsMap.get(layer.refId);
    if (asset) {
      attrs.shape.width = asset.w;
      attrs.shape.height = asset.h;
      // TODO: url to fetch
      attrs.shape.src = asset.p;
    }
    return attrs;
  }
  /**
   * @see https://lottiefiles.github.io/lottie-docs/shapes/#ellipse
   */
  function parseShapeEllipse(shape, animations, context) {
    var attrs = {
      type: Shape$2.ELLIPSE,
      // Should have no fill and stroke by default
      style: {
        fill: 'none',
        stroke: 'none'
      },
      shape: {}
    };
    parseValue(shape.p, attrs, 'shape', ['cx', 'cy'], animations, context);
    parseValue(shape.s, attrs, 'shape', ['rx', 'ry'], animations, context, function (val) {
      return val / 2;
    });
    return attrs;
  }
  function parseShapeLayer(layer, context) {
    function tryCreateShape(shape, keyframeAnimations) {
      var ecEl;
      // eslint-disable-next-line @typescript-eslint/switch-exhaustiveness-check
      switch (shape.ty) {
        case ShapeType.Path:
          ecEl = parseShapePaths(shape, keyframeAnimations, context);
          break;
        case ShapeType.Ellipse:
          ecEl = parseShapeEllipse(shape, keyframeAnimations, context);
          break;
        case ShapeType.Rectangle:
          ecEl = parseShapeRect(shape, keyframeAnimations, context);
          break;
        case ShapeType.PolyStar:
          // TODO: parseShapePolyStar
          break;
      }
      return ecEl;
    }
    function parseModifiers(shapes, modifiers) {
      shapes.forEach(function (shape) {
        if (shape.hd) {
          return;
        }
        // eslint-disable-next-line @typescript-eslint/switch-exhaustiveness-check
        switch (shape.ty) {
          case ShapeType.Repeater:
            parseValue(shape.c, modifiers.attrs, 'shape', ['repeat'], modifiers.keyframeAnimations, context);
            parseTransforms(shape.tr, modifiers.attrs, modifiers.keyframeAnimations, context, 'shape', {
              x: 'repeatX',
              y: 'repeatY',
              rotation: 'repeatRot',
              scaleX: 'repeatScaleX',
              scaleY: 'repeatScaleY',
              anchorX: 'repeatAnchorX',
              anchorY: 'repeatAnchorY',
              skew: 'repeatSkew',
              skewAxis: 'repeatSkewAxis'
            });
            break;
          case ShapeType.Trim:
            parseValue(shape.s, modifiers.attrs, 'shape', ['trimStart'], modifiers.keyframeAnimations, context);
            parseValue(shape.e, modifiers.attrs, 'shape', ['trimEnd'], modifiers.keyframeAnimations, context);
            break;
        }
      });
    }
    function parseIterations(shapes, modifiers) {
      var ecEls = [];
      var attrs = {};
      var keyframeAnimations = [];
      // Order is reversed
      shapes = shapes.slice().reverse();
      // Modifiers first:
      parseModifiers(shapes, modifiers);
      shapes.forEach(function (shape) {
        if (shape.hd) {
          return;
        }
        var ecEl;
        switch (shape.ty) {
          case ShapeType.Group:
            ecEl = {
              type: Shape$2.GROUP,
              children: parseIterations(shape.it,
              // Modifiers will be applied to all childrens.
              modifiers)
            };
            break;
          // TODO Multiple fill and stroke
          case ShapeType.Fill:
          case ShapeType.GradientFill:
            parseFill(shape, attrs, keyframeAnimations, context);
            break;
          case ShapeType.Stroke:
          case ShapeType.GradientStroke:
            parseStroke(shape, attrs, keyframeAnimations, context);
            break;
          case ShapeType.Transform:
            parseTransforms(shape, attrs, keyframeAnimations, context);
            break;
          // TODO Multiple shapes.
          default:
            ecEl = tryCreateShape(shape, keyframeAnimations);
        }
        if (ecEl) {
          ecEl.name = shape.nm;
          ecEls.push(ecEl);
        }
      });
      ecEls.forEach(function (el, idx) {
        // Apply modifiers first
        el = _extends$4({}, el, definedProps(modifiers.attrs), attrs);
        if (keyframeAnimations.length || modifiers.keyframeAnimations.length) {
          el.keyframeAnimation = [].concat(modifiers.keyframeAnimations, keyframeAnimations);
        }
        ecEls[idx] = el;
      });
      return ecEls;
    }
    return {
      type: Shape$2.GROUP,
      children: parseIterations(layer.shapes, {
        attrs: {},
        keyframeAnimations: []
      })
    };
  }
  function traverse(el, cb) {
    cb(el);
    if (el.type === Shape$2.GROUP) {
      var _el$children;
      (_el$children = el.children) === null || _el$children === void 0 ? void 0 : _el$children.forEach(function (child) {
        traverse(child, cb);
      });
    }
  }
  function addLayerOpacity(layer, layerGroup, context) {
    var _layer$ks;
    var opacityAttrs = {};
    var opacityAnimations = [];
    if ((_layer$ks = layer.ks) === null || _layer$ks === void 0 ? void 0 : _layer$ks.o) {
      var _opacityAttrs$style;
      parseValue(layer.ks.o, opacityAttrs, 'style', ['opacity'], opacityAnimations, context, function (val) {
        return val / 100;
      });
      if (((_opacityAttrs$style = opacityAttrs.style) === null || _opacityAttrs$style === void 0 ? void 0 : _opacityAttrs$style.opacity) || opacityAnimations.length) {
        // apply opacity to group's children
        traverse(layerGroup, function (el) {
          if (el.type !== Shape$2.GROUP && el.style) {
            Object.assign(el.style, opacityAttrs.style);
            if (opacityAnimations.length) {
              el.keyframeAnimation = (el.keyframeAnimation || []).concat(opacityAnimations);
            }
          }
        });
      }
    }
  }
  function parseSolidShape(layer) {
    return {
      type: Shape$2.RECT,
      shape: {
        x: 0,
        y: 0,
        width: layer.sw,
        height: layer.sh
      },
      style: {
        fill: layer.sc
      }
    };
  }
  function parseLayers(layers, context, precompLayerTl) {
    var _layers;
    var elements = [];
    // Order is reversed
    layers = layers.slice().reverse();
    var layerIndexMap = new Map();
    var offsetTime = (precompLayerTl === null || precompLayerTl === void 0 ? void 0 : precompLayerTl.st) || 0;
    (_layers = layers) === null || _layers === void 0 ? void 0 : _layers.forEach(function (layer) {
      var _context$assetsMap$ge;
      // Layer time is offseted by the precomp layer.
      // Use the ip, op, st of ref from.
      // const layerIp = offsetTime + layer.ip;
      // const layerOp = offsetTime + layer.op;
      var layerSt = offsetTime + layer.st;
      context.layerOffsetTime = offsetTime;
      var layerGroup;
      // eslint-disable-next-line @typescript-eslint/switch-exhaustiveness-check
      switch (layer.ty) {
        case LayerType.shape:
          // @see https://lottiefiles.github.io/lottie-docs/layers/#shape-layer
          layerGroup = parseShapeLayer(layer, context);
          break;
        case LayerType.null:
          // @see https://lottiefiles.github.io/lottie-docs/layers/#null-layer
          layerGroup = {
            type: Shape$2.GROUP,
            children: []
          };
          break;
        case LayerType.solid:
          // @see https://lottiefiles.github.io/lottie-docs/layers/#solid-color-layer
          layerGroup = {
            type: Shape$2.GROUP,
            children: []
          };
          // Anything you can do with solid layers, you can do better with a shape layer and a rectangle shape
          // since none of this layer's own properties can be animated.
          if (layer.sc) {
            layerGroup.children.push(parseSolidShape(layer));
          }
          break;
        case LayerType.precomp:
          // @see https://lottiefiles.github.io/lottie-docs/layers/#precomposition-layer
          layerGroup = {
            type: Shape$2.GROUP,
            children: parseLayers(((_context$assetsMap$ge = context.assetsMap.get(layer.refId)) === null || _context$assetsMap$ge === void 0 ? void 0 : _context$assetsMap$ge.layers) || [], context, {
              st: layerSt
            })
          };
          break;
        case LayerType.text:
          // TODO: https://lottiefiles.github.io/lottie-docs/layers/#text-layer
          break;
        case LayerType.image:
          // TODO: https://lottiefiles.github.io/lottie-docs/layers/#image-layer
          layerGroup = layerGroup = {
            type: Shape$2.GROUP,
            children: [parseImageLayer(layer, context)]
          };
          break;
      }
      if (layerGroup) {
        var _layer$masksPropertie;
        var keyframeAnimations = [];
        var attrs = {
          name: layer.nm
        };
        if (layer.ks) {
          parseTransforms(layer.ks, attrs, keyframeAnimations, context);
        }
        Object.assign(layerGroup, attrs);
        if (layer.ind != null) {
          layerIndexMap.set(layer.ind, layerGroup);
        }
        layerGroup.extra = {
          layerParent: layer.parent
        };
        // Masks @see https://lottiefiles.github.io/lottie-docs/layers/#masks
        // @see https://lottie-animation-community.github.io/docs/specs/layers/common/#clipping-masks
        // TODO: not support alpha and other modes.
        // @see https://lottie-animation-community.github.io/docs/specs/properties/mask-mode-types/
        if (layer.hasMask && ((_layer$masksPropertie = layer.masksProperties) === null || _layer$masksPropertie === void 0 ? void 0 : _layer$masksPropertie.length)) {
          var maskKeyframeAnimations = [];
          // TODO: Only support one mask now.
          var _attrs = parseShapePaths({
            ks: layer.masksProperties[0].pt
          }, maskKeyframeAnimations, context);
          layerGroup.clipPath = _extends$4({
            type: Shape$2.PATH
          }, _attrs);
          if (maskKeyframeAnimations.length) {
            layerGroup.clipPath.keyframeAnimation = maskKeyframeAnimations;
          }
        }
        addLayerOpacity(layer, layerGroup, context);
        // Update in and out animation.
        // if (
        //   layerIp != null &&
        //   layerOp != null &&
        //   (layerIp > context.startFrame || layerOp < context.endFrame)
        // ) {
        //   const duration = context.endFrame - context.startFrame;
        //   const enterAndLeaveAnim = {
        //     duration: duration * context.frameTime,
        //     keyframes: [
        //       {
        //         ignore: false,
        //         offset: (layerIp - context.startFrame) / duration,
        //       },
        //     ],
        //   };
        //   if (layerIp > context.startFrame) {
        //     // Add initial keyframe.
        //     // NOTE: layerIp may be earlier than startFrame. In this case the first frame has negative percent.
        //     enterAndLeaveAnim.keyframes.unshift({
        //       ignore: true,
        //       offset: 0,
        //     });
        //   }
        //   if ((layerOp - context.startFrame) / duration < 1) {
        //     enterAndLeaveAnim.keyframes.push({
        //       ignore: true,
        //       offset: (layerOp - context.startFrame) / duration,
        //     });
        //   }
        //   keyframeAnimations.push(enterAndLeaveAnim);
        // }
        if (keyframeAnimations.length) {
          layerGroup.keyframeAnimation = keyframeAnimations;
        }
        elements.push(layerGroup);
      }
    });
    // Build hierarchy
    return elements.filter(function (el) {
      var _el$extra;
      var parentLayer = layerIndexMap.get((_el$extra = el.extra) === null || _el$extra === void 0 ? void 0 : _el$extra.layerParent);
      if (parentLayer) {
        var _parentLayer$children;
        (_parentLayer$children = parentLayer.children) === null || _parentLayer$children === void 0 ? void 0 : _parentLayer$children.push(el);
        return false;
      }
      return true;
    });
  }
  function parse(data, options) {
    var _data$assets;
    if (options === void 0) {
      options = {
        loop: true,
        autoplay: false
      };
    }
    completeData(data);
    var _options = options,
      loop = _options.loop,
      autoplay = _options.autoplay;
    var context = new ParseContext();
    context.fps = data.fr || 30;
    context.frameTime = 1000 / context.fps;
    context.startFrame = data.ip;
    context.endFrame = data.op;
    context.version = data.v;
    context.autoplay = !!autoplay;
    context.iterations = isNumber$1(loop) ? loop : loop ? Infinity : 1;
    // @see https://lottiefiles.github.io/lottie-docs/assets/
    (_data$assets = data.assets) === null || _data$assets === void 0 ? void 0 : _data$assets.forEach(function (asset) {
      context.assetsMap.set(asset.id, asset);
    });
    var elements = parseLayers(data.layers || [], context);
    return {
      width: data.w,
      height: data.h,
      elements: elements,
      context: context
    };
  }

  /**
   * @see https://github.com/airbnb/lottie-web/wiki/loadAnimation-options
   * @see https://github.com/airbnb/lottie-web#other-loading-options
   */
  function loadAnimation(data, options) {
    var _parse = parse(data, options),
      width = _parse.width,
      height = _parse.height,
      elements = _parse.elements,
      context = _parse.context;
    return new LottieAnimation(width, height, elements, context);
  }

  var Lottie = /*#__PURE__*/function (_Component) {
    _inherits$2(Lottie, _Component);
    var _super = _createSuper$2(Lottie);
    function Lottie(props) {
      var _this;
      _classCallCheck$2(this, Lottie);
      _this = _super.call(this, props);
      _this.updateSize = function () {
        var _this$size = _this.size,
          currentWidth = _this$size.width,
          currentHeight = _this$size.height;
        var style = _this.props.style;
        if (!style) return;
        var _style$width = style.width,
          width = _style$width === void 0 ? currentWidth : _style$width,
          _style$height = style.height,
          height = _style$height === void 0 ? currentHeight : _style$height;
        _this.ref.current.scale(width / currentWidth, height / currentHeight);
        _this.size = {
          width: width,
          height: height
        };
      };
      _this.ref = createRef();
      return _this;
    }
    _createClass$4(Lottie, [{
      key: "didMount",
      value: function didMount() {
        var _this2 = this;
        var props = this.props,
          context = this.context;
        var data = props.data,
          options = props.options;
        var canvas = context.canvas;
        // 文档流后挂载lottie
        canvas.ready.then(function () {
          var animation = loadAnimation(data, options);
          animation.render(_this2.ref.current);
          _this2.size = animation.size();
          _this2.updateSize();
        });
      }
    }, {
      key: "willUpdate",
      value: function willUpdate() {
        var _this3 = this;
        var context = this.context;
        var canvas = context.canvas;
        canvas.ready.then(function () {
          _this3.updateSize();
        });
      }
    }, {
      key: "render",
      value: function render() {
        var _this$props = this.props,
          style = _this$props.style,
          animation = _this$props.animation;
        return jsx("group", {
          ref: this.ref,
          style: style,
          animation: animation
        });
      }
    }]);
    return Lottie;
  }(Component);

  var defaultProps$2 = {
    offsetX: 0,
    offsetY: 0,
    points: [],
    data: '',
    animation: null,
    options: {
      loop: true,
      autoplay: true
    }
  };
  var LottieGuideView = (function (props, context) {
    var cfg = deepMix({}, defaultProps$2, props);
    var points = cfg.points,
      style = cfg.style,
      offsetX = cfg.offsetX,
      offsetY = cfg.offsetY,
      lottieJson = cfg.lottieJson,
      animation = cfg.animation,
      option = cfg.option;
    var _ref = points[0] || {},
      x = _ref.x,
      y = _ref.y;
    var _style$height = style.height,
      height = _style$height === void 0 ? 0 : _style$height,
      _style$width = style.width,
      width = _style$width === void 0 ? 0 : _style$width;
    var offsetXNum = context.px2hd(offsetX);
    var offsetYNum = context.px2hd(offsetY);
    var posX = x + (offsetXNum || 0) - height / 2;
    var posY = y + (offsetYNum || 0) - width / 2;
    return jsx(Lottie, {
      data: lottieJson,
      options: option,
      style: {
        x: posX,
        y: posY,
        width: width,
        height: height
      },
      animation: deepMix({
        update: {
          easing: 'linear',
          duration: 450,
          property: ['x', 'y']
        }
      }, animation)
    });
  });

  var DefaultGuideView = function DefaultGuideView() {
    return null;
  };
  var TextGuide = withGuide(TextGuideView);
  var PointGuide = withGuide(PointGuideView);
  var LineGuide = withGuide(LineGuideView);
  var ArcGuide = withGuide(ArcGuideView);
  var RectGuide = withGuide(RectGuideView);
  var ImageGuide = withGuide(ImageGuideView);
  var TagGuide = withGuide(TagGuideView);
  var LottieGuide = withGuide(LottieGuideView);
  var index$6 = withGuide(DefaultGuideView);

  var withTooltip = (function (View) {
    return /*#__PURE__*/function (_Component) {
      _inherits$1(Tooltip, _Component);
      var _super = _createSuper$1(Tooltip);
      function Tooltip(props) {
        var _this;
        _classCallCheck$1(this, Tooltip);
        _this = _super.call(this, props);
        _this._triggerOn = function (ev) {
          var points = ev.points;
          _this.show(points[0], ev);
        };
        _this._triggerOff = function () {
          var _assertThisInitialize = _assertThisInitialized$3(_this),
            _assertThisInitialize2 = _assertThisInitialize.props.alwaysShow,
            alwaysShow = _assertThisInitialize2 === void 0 ? false : _assertThisInitialize2;
          if (!alwaysShow) {
            _this.hide();
          }
        };
        _this.state = {
          records: null
        };
        return _this;
      }
      _createClass$3(Tooltip, [{
        key: "updateCoord",
        value: function updateCoord() {
          var props = this.props,
            context = this.context;
          var _props$padding = props.padding,
            padding = _props$padding === void 0 ? '10px' : _props$padding,
            chart = props.chart;
          chart.updateCoordFor(this, {
            position: 'top',
            width: 0,
            height: context.px2hd(padding)
          });
        }
      }, {
        key: "willMount",
        value: function willMount() {
          this.updateCoord();
        }
      }, {
        key: "didMount",
        value: function didMount() {
          this._initShow();
          this._initEvent();
        }
      }, {
        key: "willReceiveProps",
        value: function willReceiveProps(nextProps) {
          var nextDefaultItem = nextProps.defaultItem,
            nextCoord = nextProps.coord;
          var _this$props = this.props,
            lastDefaultItem = _this$props.defaultItem,
            lastCoord = _this$props.coord;
          // 默认元素或坐标有变动，均需重新渲染
          if (!equal(nextDefaultItem, lastDefaultItem) || !equal(nextCoord, lastCoord)) {
            this._showByData(nextDefaultItem);
          }
        }
      }, {
        key: "didUnmount",
        value: function didUnmount() {
          this._clearEvents();
        }
      }, {
        key: "_clearEvents",
        value: function _clearEvents() {
          var props = this.props,
            context = this.context;
          var _props$triggerOn = props.triggerOn,
            triggerOn = _props$triggerOn === void 0 ? 'press' : _props$triggerOn,
            _props$triggerOff = props.triggerOff,
            triggerOff = _props$triggerOff === void 0 ? 'pressend' : _props$triggerOff;
          // 解绑事件
          context.gesture.off(triggerOn, this._triggerOn);
          context.gesture.off(triggerOff, this._triggerOff);
        }
      }, {
        key: "_initShow",
        value: function _initShow() {
          var props = this.props;
          var defaultItem = props.defaultItem;
          this._showByData(defaultItem);
        }
      }, {
        key: "_showByData",
        value: function _showByData(dataItem) {
          var _this2 = this;
          if (!dataItem) return;
          var props = this.props;
          var chart = props.chart;
          // 因为 tooltip 有可能在 geometry 之前，所以需要等 geometry render 完后再执行
          setTimeout(function () {
            var point = chart.getPosition(dataItem);
            _this2.show(point);
          }, 0);
        }
      }, {
        key: "_initEvent",
        value: function _initEvent() {
          var _this3 = this;
          var context = this.context,
            props = this.props;
          var _props$triggerOn2 = props.triggerOn,
            triggerOn = _props$triggerOn2 === void 0 ? 'press' : _props$triggerOn2,
            _props$triggerOff2 = props.triggerOff,
            triggerOff = _props$triggerOff2 === void 0 ? 'pressend' : _props$triggerOff2,
            _props$alwaysShow = props.alwaysShow,
            alwaysShow = _props$alwaysShow === void 0 ? false : _props$alwaysShow;
          context.gesture.on(triggerOn, function (ev) {
            var points = ev.points;
            _this3.show(points[0], ev);
          });
          context.gesture.on(triggerOff, function (_ev) {
            if (!alwaysShow) {
              _this3.hide();
            }
          });
        }
      }, {
        key: "show",
        value: function show(point, _ev) {
          var props = this.props;
          var chart = props.chart,
            onChange = props.onChange;
          var snapRecords = chart.getSnapRecords(point, true); // 超出边界会自动调整
          if (!snapRecords || !snapRecords.length) return;
          var legendItems = chart.getLegendItems();
          var _snapRecords$ = snapRecords[0],
            xField = _snapRecords$.xField,
            yField = _snapRecords$.yField;
          var xScale = chart.getScale(xField);
          var yScale = chart.getScale(yField);
          var records = snapRecords.map(function (record) {
            var origin = record.origin,
              xField = record.xField,
              yField = record.yField;
            var value = yScale.getText(origin[yField]);
            // 默认取 alias 的配置
            var name = yScale.alias;
            if (!name) {
              name = xScale.getText(origin[xField]);
              if (legendItems && legendItems.length) {
                var item = find$1(legendItems, function (item) {
                  var field = item.field,
                    tickValue = item.tickValue;
                  return origin[field] === tickValue;
                });
                if (item && item.name) {
                  name = item.name;
                }
              }
            }
            return _objectSpread(_objectSpread({}, record), {}, {
              name: name,
              value: value
            });
          });
          if (!isArray$1(records) || !records.length) {
            return;
          }
          this.setState({
            records: records
          });
          if (isFunction$2(onChange)) {
            onChange(records);
          }
        }
      }, {
        key: "hide",
        value: function hide() {
          this.setState({
            records: null
          });
        }
      }, {
        key: "render",
        value: function render() {
          var props = this.props,
            state = this.state;
          var visible = props.visible;
          if (visible === false) {
            return null;
          }
          var records = state.records;
          if (!records || !records.length) return null;
          return jsx(View, _objectSpread(_objectSpread({}, props), {}, {
            records: records
          }));
        }
      }]);
      return Tooltip;
    }(Component);
  });

  var asyncToGenerator = createCommonjsModule(function (module) {
  function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }
    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }
  function _asyncToGenerator(fn) {
    return function () {
      var self = this,
        args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);
        function _next(value) {
          asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
        }
        function _throw(err) {
          asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
        }
        _next(undefined);
      });
    };
  }
  module.exports = _asyncToGenerator, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _asyncToGenerator$8 = /*@__PURE__*/getDefaultExportFromCjs(asyncToGenerator);

  var regeneratorRuntime$2 = createCommonjsModule(function (module) {
  var _typeof = _typeof_1$1["default"];
  function _regeneratorRuntime() {
    module.exports = _regeneratorRuntime = function _regeneratorRuntime() {
      return exports;
    }, module.exports.__esModule = true, module.exports["default"] = module.exports;
    var exports = {},
      Op = Object.prototype,
      hasOwn = Op.hasOwnProperty,
      defineProperty = Object.defineProperty || function (obj, key, desc) {
        obj[key] = desc.value;
      },
      $Symbol = "function" == typeof Symbol ? Symbol : {},
      iteratorSymbol = $Symbol.iterator || "@@iterator",
      asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
      toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
    function define(obj, key, value) {
      return Object.defineProperty(obj, key, {
        value: value,
        enumerable: !0,
        configurable: !0,
        writable: !0
      }), obj[key];
    }
    try {
      define({}, "");
    } catch (err) {
      define = function define(obj, key, value) {
        return obj[key] = value;
      };
    }
    function wrap(innerFn, outerFn, self, tryLocsList) {
      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
        generator = Object.create(protoGenerator.prototype),
        context = new Context(tryLocsList || []);
      return defineProperty(generator, "_invoke", {
        value: makeInvokeMethod(innerFn, self, context)
      }), generator;
    }
    function tryCatch(fn, obj, arg) {
      try {
        return {
          type: "normal",
          arg: fn.call(obj, arg)
        };
      } catch (err) {
        return {
          type: "throw",
          arg: err
        };
      }
    }
    exports.wrap = wrap;
    var ContinueSentinel = {};
    function Generator() {}
    function GeneratorFunction() {}
    function GeneratorFunctionPrototype() {}
    var IteratorPrototype = {};
    define(IteratorPrototype, iteratorSymbol, function () {
      return this;
    });
    var getProto = Object.getPrototypeOf,
      NativeIteratorPrototype = getProto && getProto(getProto(values([])));
    NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
    function defineIteratorMethods(prototype) {
      ["next", "throw", "return"].forEach(function (method) {
        define(prototype, method, function (arg) {
          return this._invoke(method, arg);
        });
      });
    }
    function AsyncIterator(generator, PromiseImpl) {
      function invoke(method, arg, resolve, reject) {
        var record = tryCatch(generator[method], generator, arg);
        if ("throw" !== record.type) {
          var result = record.arg,
            value = result.value;
          return value && "object" == _typeof(value) && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
            invoke("next", value, resolve, reject);
          }, function (err) {
            invoke("throw", err, resolve, reject);
          }) : PromiseImpl.resolve(value).then(function (unwrapped) {
            result.value = unwrapped, resolve(result);
          }, function (error) {
            return invoke("throw", error, resolve, reject);
          });
        }
        reject(record.arg);
      }
      var previousPromise;
      defineProperty(this, "_invoke", {
        value: function value(method, arg) {
          function callInvokeWithMethodAndArg() {
            return new PromiseImpl(function (resolve, reject) {
              invoke(method, arg, resolve, reject);
            });
          }
          return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
        }
      });
    }
    function makeInvokeMethod(innerFn, self, context) {
      var state = "suspendedStart";
      return function (method, arg) {
        if ("executing" === state) throw new Error("Generator is already running");
        if ("completed" === state) {
          if ("throw" === method) throw arg;
          return doneResult();
        }
        for (context.method = method, context.arg = arg;;) {
          var delegate = context.delegate;
          if (delegate) {
            var delegateResult = maybeInvokeDelegate(delegate, context);
            if (delegateResult) {
              if (delegateResult === ContinueSentinel) continue;
              return delegateResult;
            }
          }
          if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
            if ("suspendedStart" === state) throw state = "completed", context.arg;
            context.dispatchException(context.arg);
          } else "return" === context.method && context.abrupt("return", context.arg);
          state = "executing";
          var record = tryCatch(innerFn, self, context);
          if ("normal" === record.type) {
            if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
            return {
              value: record.arg,
              done: context.done
            };
          }
          "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
        }
      };
    }
    function maybeInvokeDelegate(delegate, context) {
      var method = delegate.iterator[context.method];
      if (undefined === method) {
        if (context.delegate = null, "throw" === context.method) {
          if (delegate.iterator["return"] && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
          context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
        }
        return ContinueSentinel;
      }
      var record = tryCatch(method, delegate.iterator, context.arg);
      if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
      var info = record.arg;
      return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
    }
    function pushTryEntry(locs) {
      var entry = {
        tryLoc: locs[0]
      };
      1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
    }
    function resetTryEntry(entry) {
      var record = entry.completion || {};
      record.type = "normal", delete record.arg, entry.completion = record;
    }
    function Context(tryLocsList) {
      this.tryEntries = [{
        tryLoc: "root"
      }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
    }
    function values(iterable) {
      if (iterable) {
        var iteratorMethod = iterable[iteratorSymbol];
        if (iteratorMethod) return iteratorMethod.call(iterable);
        if ("function" == typeof iterable.next) return iterable;
        if (!isNaN(iterable.length)) {
          var i = -1,
            next = function next() {
              for (; ++i < iterable.length;) {
                if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
              }
              return next.value = undefined, next.done = !0, next;
            };
          return next.next = next;
        }
      }
      return {
        next: doneResult
      };
    }
    function doneResult() {
      return {
        value: undefined,
        done: !0
      };
    }
    return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
      value: GeneratorFunctionPrototype,
      configurable: !0
    }), defineProperty(GeneratorFunctionPrototype, "constructor", {
      value: GeneratorFunction,
      configurable: !0
    }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
      var ctor = "function" == typeof genFun && genFun.constructor;
      return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
    }, exports.mark = function (genFun) {
      return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
    }, exports.awrap = function (arg) {
      return {
        __await: arg
      };
    }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
      return this;
    }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
      void 0 === PromiseImpl && (PromiseImpl = Promise);
      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
      return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
        return result.done ? result.value : iter.next();
      });
    }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
      return this;
    }), define(Gp, "toString", function () {
      return "[object Generator]";
    }), exports.keys = function (val) {
      var object = Object(val),
        keys = [];
      for (var key in object) {
        keys.push(key);
      }
      return keys.reverse(), function next() {
        for (; keys.length;) {
          var key = keys.pop();
          if (key in object) return next.value = key, next.done = !1, next;
        }
        return next.done = !0, next;
      };
    }, exports.values = values, Context.prototype = {
      constructor: Context,
      reset: function reset(skipTempReset) {
        if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) {
          "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
        }
      },
      stop: function stop() {
        this.done = !0;
        var rootRecord = this.tryEntries[0].completion;
        if ("throw" === rootRecord.type) throw rootRecord.arg;
        return this.rval;
      },
      dispatchException: function dispatchException(exception) {
        if (this.done) throw exception;
        var context = this;
        function handle(loc, caught) {
          return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
        }
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i],
            record = entry.completion;
          if ("root" === entry.tryLoc) return handle("end");
          if (entry.tryLoc <= this.prev) {
            var hasCatch = hasOwn.call(entry, "catchLoc"),
              hasFinally = hasOwn.call(entry, "finallyLoc");
            if (hasCatch && hasFinally) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            } else if (hasCatch) {
              if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            } else {
              if (!hasFinally) throw new Error("try statement without catch or finally");
              if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
            }
          }
        }
      },
      abrupt: function abrupt(type, arg) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
            var finallyEntry = entry;
            break;
          }
        }
        finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
        var record = finallyEntry ? finallyEntry.completion : {};
        return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
      },
      complete: function complete(record, afterLoc) {
        if ("throw" === record.type) throw record.arg;
        return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
      },
      finish: function finish(finallyLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
        }
      },
      "catch": function _catch(tryLoc) {
        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
          var entry = this.tryEntries[i];
          if (entry.tryLoc === tryLoc) {
            var record = entry.completion;
            if ("throw" === record.type) {
              var thrown = record.arg;
              resetTryEntry(entry);
            }
            return thrown;
          }
        }
        throw new Error("illegal catch attempt");
      },
      delegateYield: function delegateYield(iterable, resultName, nextLoc) {
        return this.delegate = {
          iterator: values(iterable),
          resultName: resultName,
          nextLoc: nextLoc
        }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
      }
    }, exports;
  }
  module.exports = _regeneratorRuntime, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  // TODO(Babel 8): Remove this file.

  var runtime$3 = regeneratorRuntime$2();
  var regenerator$1 = runtime$3;

  // Copied from https://github.com/facebook/regenerator/blob/main/packages/runtime/runtime.js#L736=
  try {
    regeneratorRuntime = runtime$3;
  } catch (accidentalStrictMode) {
    if (typeof globalThis === "object") {
      globalThis.regeneratorRuntime = runtime$3;
    } else {
      Function("r", "regeneratorRuntime = r")(runtime$3);
    }
  }

  // view 的默认配置
  var defaultStyle$1 = {
    showTitle: false,
    showCrosshairs: false,
    crosshairsType: 'y',
    crosshairsStyle: {
      stroke: 'rgba(0, 0, 0, 0.25)',
      lineWidth: '2px'
    },
    showTooltipMarker: false,
    markerBackgroundStyle: {
      fill: '#CCD6EC',
      opacity: 0.3,
      padding: '6px'
    },
    tooltipMarkerStyle: {
      fill: '#fff',
      lineWidth: '3px'
    },
    background: {
      radius: '4px',
      fill: 'rgba(0, 0, 0, 0.65)',
      padding: ['6px', '10px']
    },
    titleStyle: {
      fontSize: '24px',
      fill: '#fff',
      textAlign: 'start',
      textBaseline: 'top'
    },
    nameStyle: {
      fontSize: '24px',
      fill: 'rgba(255, 255, 255, 0.65)',
      textAlign: 'start',
      textBaseline: 'middle'
    },
    valueStyle: {
      fontSize: '24px',
      fill: '#fff',
      textAlign: 'start',
      textBaseline: 'middle'
    },
    joinString: ': ',
    showItemMarker: true,
    itemMarkerStyle: {
      width: '12px',
      radius: '6px',
      symbol: 'circle',
      lineWidth: '2px',
      stroke: '#fff'
    },
    layout: 'horizontal',
    snap: false,
    xTipTextStyle: {
      fontSize: '24px',
      fill: '#fff'
    },
    yTipTextStyle: {
      fontSize: '24px',
      fill: '#fff'
    },
    xTipBackground: {
      radius: '4px',
      fill: 'rgba(0, 0, 0, 0.65)',
      padding: ['6px', '10px'],
      marginLeft: '-50%',
      marginTop: '6px'
    },
    yTipBackground: {
      radius: '4px',
      fill: 'rgba(0, 0, 0, 0.65)',
      padding: ['6px', '10px'],
      marginLeft: '-100%',
      marginTop: '-50%'
    }
  };
  function directionEnabled(mode, dir) {
    if (mode === undefined) {
      return true;
    } else if (typeof mode === 'string') {
      return mode.indexOf(dir) !== -1;
    }
    return false;
  }
  var RenderItemMarker = function RenderItemMarker(props) {
    var records = props.records,
      coord = props.coord,
      context = props.context,
      markerBackgroundStyle = props.markerBackgroundStyle;
    var point = coord.convertPoint({
      x: 1,
      y: 1
    });
    var padding = context.px2hd(markerBackgroundStyle.padding || '6px');
    var xPoints = [].concat(_toConsumableArray$1(records.map(function (record) {
      return record.xMin;
    })), _toConsumableArray$1(records.map(function (record) {
      return record.xMax;
    })));
    var yPoints = [].concat(_toConsumableArray$1(records.map(function (record) {
      return record.yMin;
    })), _toConsumableArray$1(records.map(function (record) {
      return record.yMax;
    })));
    if (coord.transposed) {
      xPoints.push(point.x);
    } else {
      yPoints.push(point.y);
    }
    var xMin = Math.min.apply(null, xPoints);
    var xMax = Math.max.apply(null, xPoints);
    var yMin = Math.min.apply(null, yPoints);
    var yMax = Math.max.apply(null, yPoints);
    var x = coord.transposed ? xMin : xMin - padding;
    var y = coord.transposed ? yMin - padding : yMin;
    var width = coord.transposed ? xMax - xMin : xMax - xMin + 2 * padding;
    var height = coord.transposed ? yMax - yMin + 2 * padding : yMax - yMin;
    return jsx("rect", {
      attrs: _objectSpread({
        x: x,
        y: y,
        width: width,
        height: height
      }, markerBackgroundStyle)
    });
  };
  var RenderCrosshairs = function RenderCrosshairs(props) {
    var records = props.records,
      coord = props.coord,
      chart = props.chart,
      crosshairsType = props.crosshairsType,
      crosshairsStyle = props.crosshairsStyle;
    var coordLeft = coord.left,
      coordTop = coord.top,
      coordRight = coord.right,
      coordBottom = coord.bottom,
      center = coord.center;
    var firstRecord = records[0];
    var x = firstRecord.x,
      y = firstRecord.y,
      origin = firstRecord.origin,
      xField = firstRecord.xField;
    if (coord.isPolar) {
      // 极坐标下的辅助线
      var xScale = chart.getScale(xField);
      var ticks = xScale.getTicks();
      var tick = find$1(ticks, function (tick) {
        return origin[xField] === tick.tickValue;
      });
      var end = coord.convertPoint({
        x: tick.value,
        y: 1
      });
      return jsx("line", {
        attrs: _objectSpread({
          x1: center.x,
          y1: center.y,
          x2: end.x,
          y2: end.y
        }, crosshairsStyle)
      });
    }
    return jsx("group", null, directionEnabled(crosshairsType, 'x') ? jsx("line", {
      attrs: _objectSpread({
        x1: coordLeft,
        y1: y,
        x2: coordRight,
        y2: y
      }, crosshairsStyle)
    }) : null, directionEnabled(crosshairsType, 'y') ? jsx("line", {
      attrs: _objectSpread({
        x1: x,
        y1: coordTop,
        x2: x,
        y2: coordBottom
      }, crosshairsStyle)
    }) : null);
  };
  var TooltipView = /*#__PURE__*/function (_Component) {
    _inherits$1(TooltipView, _Component);
    var _super = _createSuper$1(TooltipView);
    function TooltipView(props) {
      var _this;
      _classCallCheck$1(this, TooltipView);
      _this = _super.call(this, props);
      _this.rootRef = createRef();
      _this.arrowRef = createRef();
      return _this;
    }
    // 调整 显示的位置
    _createClass$3(TooltipView, [{
      key: "_position",
      value: function () {
        var _position2 = _asyncToGenerator$8( /*#__PURE__*/regenerator$1.mark(function _callee() {
          var _rootRef$current;
          var props, context, rootRef, arrowRef, canvas, rect, records, coord, arrowWidth, record, x, coordLeft, coordWidth, _rect$getBBox, y, width, height, radius, halfWidth, offsetX, offset;
          return regenerator$1.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  props = this.props, context = this.context, rootRef = this.rootRef, arrowRef = this.arrowRef;
                  canvas = context.canvas;
                  _context.next = 4;
                  return canvas.ready;
                case 4:
                  rect = (_rootRef$current = rootRef.current) === null || _rootRef$current === void 0 ? void 0 : _rootRef$current.childNodes[0];
                  if (rect) {
                    _context.next = 7;
                    break;
                  }
                  return _context.abrupt("return");
                case 7:
                  records = props.records, coord = props.coord;
                  arrowWidth = context.px2hd('6px');
                  record = records[0]; // 中心点
                  x = record.x;
                  coordLeft = coord.left, coordWidth = coord.width;
                  _rect$getBBox = rect.getBBox(), y = _rect$getBBox.y, width = _rect$getBBox.width, height = _rect$getBBox.height;
                  radius = rect.getAttribute('radius');
                  halfWidth = width / 2; // 让 tooltip 限制在 coord 的显示范围内
                  offsetX = Math.min(Math.max(x - coordLeft - halfWidth, -arrowWidth - radius), coordWidth - width + arrowWidth + radius); // 因为默认是从 coord 的范围内显示的，所以要往上移，移出 coord，避免挡住 geometry
                  offset = Math.min(y, height + arrowWidth); // 因为不能超出 canvas 画布区域，所以最大只能是 y
                  rect.setLocalPosition(offsetX, -offset);
                  arrowRef.current.translate(0, height - offset);
                case 19:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, this);
        }));
        function _position() {
          return _position2.apply(this, arguments);
        }
        return _position;
      }()
    }, {
      key: "didMount",
      value: function didMount() {
        this._position();
      }
    }, {
      key: "didUpdate",
      value: function didUpdate() {
        this._position();
      }
    }, {
      key: "render",
      value: function render() {
        var props = this.props,
          context = this.context;
        var records = props.records,
          coord = props.coord;
        var coordLeft = coord.left,
          coordTop = coord.top,
          coordBottom = coord.bottom;
        var firstRecord = records[0];
        var x = firstRecord.x,
          y = firstRecord.y;
        var xFirstText = firstRecord.name,
          yFirstText = firstRecord.value;
        var chart = props.chart,
          customBackground = props.background,
          _props$showTooltipMar = props.showTooltipMarker,
          showTooltipMarker = _props$showTooltipMar === void 0 ? defaultStyle$1.showTooltipMarker : _props$showTooltipMar,
          _props$markerBackgrou = props.markerBackgroundStyle,
          markerBackgroundStyle = _props$markerBackgrou === void 0 ? defaultStyle$1.markerBackgroundStyle : _props$markerBackgrou,
          _props$showItemMarker = props.showItemMarker,
          showItemMarker = _props$showItemMarker === void 0 ? defaultStyle$1.showItemMarker : _props$showItemMarker,
          customItemMarkerStyle = props.itemMarkerStyle,
          nameStyle = props.nameStyle,
          valueStyle = props.valueStyle,
          _props$joinString = props.joinString,
          joinString = _props$joinString === void 0 ? defaultStyle$1.joinString : _props$joinString,
          _props$showCrosshairs = props.showCrosshairs,
          showCrosshairs = _props$showCrosshairs === void 0 ? defaultStyle$1.showCrosshairs : _props$showCrosshairs,
          crosshairsStyle = props.crosshairsStyle,
          _props$crosshairsType = props.crosshairsType,
          crosshairsType = _props$crosshairsType === void 0 ? defaultStyle$1.crosshairsType : _props$crosshairsType,
          _props$snap = props.snap,
          snap = _props$snap === void 0 ? defaultStyle$1.snap : _props$snap,
          _props$tooltipMarkerS = props.tooltipMarkerStyle,
          tooltipMarkerStyle = _props$tooltipMarkerS === void 0 ? defaultStyle$1.tooltipMarkerStyle : _props$tooltipMarkerS,
          showXTip = props.showXTip,
          showYTip = props.showYTip,
          xTip = props.xTip,
          yTip = props.yTip,
          _props$xTipTextStyle = props.xTipTextStyle,
          xTipTextStyle = _props$xTipTextStyle === void 0 ? defaultStyle$1.xTipTextStyle : _props$xTipTextStyle,
          _props$yTipTextStyle = props.yTipTextStyle,
          yTipTextStyle = _props$yTipTextStyle === void 0 ? defaultStyle$1.yTipTextStyle : _props$yTipTextStyle,
          _props$xTipBackground = props.xTipBackground,
          xTipBackground = _props$xTipBackground === void 0 ? defaultStyle$1.xTipBackground : _props$xTipBackground,
          _props$yTipBackground = props.yTipBackground,
          yTipBackground = _props$yTipBackground === void 0 ? defaultStyle$1.yTipBackground : _props$yTipBackground,
          _props$custom = props.custom,
          custom = _props$custom === void 0 ? false : _props$custom,
          customText = props.customText;
        var itemMarkerStyle = _objectSpread(_objectSpread({}, customItemMarkerStyle), defaultStyle$1.itemMarkerStyle);
        var background = _objectSpread(_objectSpread({}, defaultStyle$1.background), customBackground);
        var arrowWidth = context.px2hd('6px');
        return jsx("group", {
          style: {
            display: 'flex',
            flexDirection: 'row'
          }
        }, jsx("group", null, !custom && jsx("group", null, jsx("group", {
          ref: this.rootRef,
          style: {
            x: coordLeft,
            y: coordTop
          }
        }, jsx("rect", {
          style: _objectSpread({
            display: 'flex',
            flexDirection: 'row',
            flexWrap: 'wrap',
            padding: [0, 0, 0, '6px']
          }, background)
        }, records.map(function (record) {
          var name = record.name,
            value = record.value;
          return jsx("group", {
            style: {
              display: 'flex',
              flexDirection: 'row',
              alignItems: 'center',
              padding: [0, '6px', 0, 0]
            }
          }, showItemMarker ? jsx("marker", {
            style: {
              width: itemMarkerStyle.width,
              marginRight: '6px'
            },
            attrs: _objectSpread(_objectSpread({}, itemMarkerStyle), {}, {
              fill: record.color
            })
          }) : null, customText && isFunction$2(customText) ? customText(record) : jsx("group", {
            style: {
              display: 'flex',
              flexDirection: 'row'
            }
          }, jsx("text", {
            attrs: _objectSpread(_objectSpread(_objectSpread({}, defaultStyle$1.nameStyle), nameStyle), {}, {
              text: value ? "".concat(name).concat(joinString) : name
            })
          }), jsx("text", {
            attrs: _objectSpread(_objectSpread(_objectSpread({}, defaultStyle$1.valueStyle), valueStyle), {}, {
              text: value
            })
          })));
        }))), jsx("polygon", {
          ref: this.arrowRef,
          attrs: {
            points: [[x - arrowWidth, coordTop], [x + arrowWidth, coordTop], [x, arrowWidth + coordTop]],
            fill: background.fill
          }
        })), showTooltipMarker ? jsx(RenderItemMarker, {
          coord: coord,
          context: context,
          records: records,
          markerBackgroundStyle: markerBackgroundStyle
        }) : null, showCrosshairs ? jsx(RenderCrosshairs, {
          chart: chart,
          coord: coord,
          records: records,
          crosshairsType: crosshairsType,
          crosshairsStyle: _objectSpread(_objectSpread({}, defaultStyle$1.crosshairsStyle), crosshairsStyle)
        }) : null, snap ? records.map(function (item) {
          var x = item.x,
            y = item.y,
            color = item.color,
            shape = item.shape;
          return jsx("circle", {
            attrs: _objectSpread(_objectSpread({
              cx: x,
              cy: y,
              r: '6px',
              stroke: color,
              fill: color
            }, shape), tooltipMarkerStyle)
          });
        }) : null), showXTip && jsx("group", {
          style: _objectSpread(_objectSpread({
            left: x,
            top: coordBottom
          }, defaultStyle$1.xTipBackground), xTipBackground),
          attrs: _objectSpread(_objectSpread({}, defaultStyle$1.xTipBackground), xTipBackground)
        }, jsx("text", {
          attrs: _objectSpread(_objectSpread(_objectSpread({}, defaultStyle$1.xTipTextStyle), xTipTextStyle), {}, {
            text: isFunction$2(xTip) ? xTip(xFirstText) : xFirstText
          })
        })), showYTip && jsx("group", {
          style: _objectSpread(_objectSpread({
            left: coordLeft,
            top: y
          }, defaultStyle$1.yTipBackground), yTipBackground),
          attrs: _objectSpread(_objectSpread({}, defaultStyle$1.yTipBackground), yTipBackground)
        }, jsx("text", {
          attrs: _objectSpread(_objectSpread(_objectSpread({}, defaultStyle$1.yTipTextStyle), yTipTextStyle), {}, {
            text: isFunction$2(yTip) ? yTip(yFirstText) : yFirstText
          })
        })));
      }
    }]);
    return TooltipView;
  }(Component);

  var index$7 = withTooltip(TooltipView);

  function count(node) {
    var sum = 0,
      children = node.children,
      i = children && children.length;
    if (!i) sum = 1;else while (--i >= 0) {
      sum += children[i].value;
    }
    node.value = sum;
  }
  function node_count () {
    return this.eachAfter(count);
  }

  var createForOfIteratorHelper = createCommonjsModule(function (module) {
  function _createForOfIteratorHelper(o, allowArrayLike) {
    var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
    if (!it) {
      if (Array.isArray(o) || (it = unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
        if (it) o = it;
        var i = 0;
        var F = function F() {};
        return {
          s: F,
          n: function n() {
            if (i >= o.length) return {
              done: true
            };
            return {
              done: false,
              value: o[i++]
            };
          },
          e: function e(_e) {
            throw _e;
          },
          f: F
        };
      }
      throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
    }
    var normalCompletion = true,
      didErr = false,
      err;
    return {
      s: function s() {
        it = it.call(o);
      },
      n: function n() {
        var step = it.next();
        normalCompletion = step.done;
        return step;
      },
      e: function e(_e2) {
        didErr = true;
        err = _e2;
      },
      f: function f() {
        try {
          if (!normalCompletion && it["return"] != null) it["return"]();
        } finally {
          if (didErr) throw err;
        }
      }
    };
  }
  module.exports = _createForOfIteratorHelper, module.exports.__esModule = true, module.exports["default"] = module.exports;
  });

  var _createForOfIteratorHelper = /*@__PURE__*/getDefaultExportFromCjs(createForOfIteratorHelper);

  function node_each (callback, that) {
    var index = -1;
    var _iterator = _createForOfIteratorHelper(this),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var node = _step.value;
        callback.call(that, node, ++index, this);
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
    return this;
  }

  function node_eachBefore (callback, that) {
    var node = this,
      nodes = [node],
      children,
      i,
      index = -1;
    while (node = nodes.pop()) {
      callback.call(that, node, ++index, this);
      if (children = node.children) {
        for (i = children.length - 1; i >= 0; --i) {
          nodes.push(children[i]);
        }
      }
    }
    return this;
  }

  function node_eachAfter (callback, that) {
    var node = this,
      nodes = [node],
      next = [],
      children,
      i,
      n,
      index = -1;
    while (node = nodes.pop()) {
      next.push(node);
      if (children = node.children) {
        for (i = 0, n = children.length; i < n; ++i) {
          nodes.push(children[i]);
        }
      }
    }
    while (node = next.pop()) {
      callback.call(that, node, ++index, this);
    }
    return this;
  }

  function node_find (callback, that) {
    var index = -1;
    var _iterator = _createForOfIteratorHelper(this),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var node = _step.value;
        if (callback.call(that, node, ++index, this)) {
          return node;
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  }

  function node_sum (value) {
    return this.eachAfter(function (node) {
      var sum = +value(node.data) || 0,
        children = node.children,
        i = children && children.length;
      while (--i >= 0) {
        sum += children[i].value;
      }
      node.value = sum;
    });
  }

  function node_sort (compare) {
    return this.eachBefore(function (node) {
      if (node.children) {
        node.children.sort(compare);
      }
    });
  }

  function node_path (end) {
    var start = this,
      ancestor = leastCommonAncestor(start, end),
      nodes = [start];
    while (start !== ancestor) {
      start = start.parent;
      nodes.push(start);
    }
    var k = nodes.length;
    while (end !== ancestor) {
      nodes.splice(k, 0, end);
      end = end.parent;
    }
    return nodes;
  }
  function leastCommonAncestor(a, b) {
    if (a === b) return a;
    var aNodes = a.ancestors(),
      bNodes = b.ancestors(),
      c = null;
    a = aNodes.pop();
    b = bNodes.pop();
    while (a === b) {
      c = a;
      a = aNodes.pop();
      b = bNodes.pop();
    }
    return c;
  }

  function node_ancestors () {
    var node = this,
      nodes = [node];
    while (node = node.parent) {
      nodes.push(node);
    }
    return nodes;
  }

  function node_descendants () {
    return Array.from(this);
  }

  function node_leaves () {
    var leaves = [];
    this.eachBefore(function (node) {
      if (!node.children) {
        leaves.push(node);
      }
    });
    return leaves;
  }

  function node_links () {
    var root = this,
      links = [];
    root.each(function (node) {
      if (node !== root) {
        // Don’t include the root’s parent, if any.
        links.push({
          source: node.parent,
          target: node
        });
      }
    });
    return links;
  }

  var _marked = /*#__PURE__*/regenerator$1.mark(_callee);
  function _callee() {
    var node, current, next, children, i, n;
    return regenerator$1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            node = this, next = [node];
          case 1:
            current = next.reverse(), next = [];
          case 2:
            if (!(node = current.pop())) {
              _context.next = 8;
              break;
            }
            _context.next = 5;
            return node;
          case 5:
            if (children = node.children) {
              for (i = 0, n = children.length; i < n; ++i) {
                next.push(children[i]);
              }
            }
            _context.next = 2;
            break;
          case 8:
            if (next.length) {
              _context.next = 1;
              break;
            }
          case 9:
          case "end":
            return _context.stop();
        }
      }
    }, _marked, this);
  }

  function hierarchy(data, children) {
    if (data instanceof Map) {
      data = [undefined, data];
      if (children === undefined) children = mapChildren;
    } else if (children === undefined) {
      children = objectChildren;
    }
    var root = new Node$3(data),
      node,
      nodes = [root],
      child,
      childs,
      i,
      n;
    while (node = nodes.pop()) {
      if ((childs = children(node.data)) && (n = (childs = Array.from(childs)).length)) {
        node.children = childs;
        for (i = n - 1; i >= 0; --i) {
          nodes.push(child = childs[i] = new Node$3(childs[i]));
          child.parent = node;
          child.depth = node.depth + 1;
        }
      }
    }
    return root.eachBefore(computeHeight);
  }
  function node_copy() {
    return hierarchy(this).eachBefore(copyData);
  }
  function objectChildren(d) {
    return d.children;
  }
  function mapChildren(d) {
    return Array.isArray(d) ? d[1] : null;
  }
  function copyData(node) {
    if (node.data.value !== undefined) node.value = node.data.value;
    node.data = node.data.data;
  }
  function computeHeight(node) {
    var height = 0;
    do {
      node.height = height;
    } while ((node = node.parent) && node.height < ++height);
  }
  function Node$3(data) {
    this.data = data;
    this.depth = this.height = 0;
    this.parent = null;
  }
  Node$3.prototype = hierarchy.prototype = _defineProperty$1({
    constructor: Node$3,
    count: node_count,
    each: node_each,
    eachAfter: node_eachAfter,
    eachBefore: node_eachBefore,
    find: node_find,
    sum: node_sum,
    sort: node_sort,
    path: node_path,
    ancestors: node_ancestors,
    descendants: node_descendants,
    leaves: node_leaves,
    links: node_links,
    copy: node_copy
  }, Symbol.iterator, _callee);

  function required(f) {
    if (typeof f !== "function") throw new Error();
    return f;
  }

  function constantZero() {
    return 0;
  }
  function constant$1 (x) {
    return function () {
      return x;
    };
  }

  function roundNode (node) {
    node.x0 = Math.round(node.x0);
    node.y0 = Math.round(node.y0);
    node.x1 = Math.round(node.x1);
    node.y1 = Math.round(node.y1);
  }

  function treemapDice (parent, x0, y0, x1, y1) {
    var nodes = parent.children,
      node,
      i = -1,
      n = nodes.length,
      k = parent.value && (x1 - x0) / parent.value;
    while (++i < n) {
      node = nodes[i], node.y0 = y0, node.y1 = y1;
      node.x0 = x0, node.x1 = x0 += node.value * k;
    }
  }

  function partition () {
    var dx = 1,
      dy = 1,
      padding = 0,
      round = false;
    function partition(root) {
      var n = root.height + 1;
      root.x0 = root.y0 = padding;
      root.x1 = dx;
      root.y1 = dy / n;
      root.eachBefore(positionNode(dy, n));
      if (round) root.eachBefore(roundNode);
      return root;
    }
    function positionNode(dy, n) {
      return function (node) {
        if (node.children) {
          treemapDice(node, node.x0, dy * (node.depth + 1) / n, node.x1, dy * (node.depth + 2) / n);
        }
        var x0 = node.x0,
          y0 = node.y0,
          x1 = node.x1 - padding,
          y1 = node.y1 - padding;
        if (x1 < x0) x0 = x1 = (x0 + x1) / 2;
        if (y1 < y0) y0 = y1 = (y0 + y1) / 2;
        node.x0 = x0;
        node.y0 = y0;
        node.x1 = x1;
        node.y1 = y1;
      };
    }
    partition.round = function (x) {
      return arguments.length ? (round = !!x, partition) : round;
    };
    partition.size = function (x) {
      return arguments.length ? (dx = +x[0], dy = +x[1], partition) : [dx, dy];
    };
    partition.padding = function (x) {
      return arguments.length ? (padding = +x, partition) : padding;
    };
    return partition;
  }

  function treemapSlice (parent, x0, y0, x1, y1) {
    var nodes = parent.children,
      node,
      i = -1,
      n = nodes.length,
      k = parent.value && (y1 - y0) / parent.value;
    while (++i < n) {
      node = nodes[i], node.x0 = x0, node.x1 = x1;
      node.y0 = y0, node.y1 = y0 += node.value * k;
    }
  }

  var phi = (1 + Math.sqrt(5)) / 2;
  function squarifyRatio(ratio, parent, x0, y0, x1, y1) {
    var rows = [],
      nodes = parent.children,
      row,
      nodeValue,
      i0 = 0,
      i1 = 0,
      n = nodes.length,
      dx,
      dy,
      value = parent.value,
      sumValue,
      minValue,
      maxValue,
      newRatio,
      minRatio,
      alpha,
      beta;
    while (i0 < n) {
      dx = x1 - x0, dy = y1 - y0;

      // Find the next non-empty node.
      do {
        sumValue = nodes[i1++].value;
      } while (!sumValue && i1 < n);
      minValue = maxValue = sumValue;
      alpha = Math.max(dy / dx, dx / dy) / (value * ratio);
      beta = sumValue * sumValue * alpha;
      minRatio = Math.max(maxValue / beta, beta / minValue);

      // Keep adding nodes while the aspect ratio maintains or improves.
      for (; i1 < n; ++i1) {
        sumValue += nodeValue = nodes[i1].value;
        if (nodeValue < minValue) minValue = nodeValue;
        if (nodeValue > maxValue) maxValue = nodeValue;
        beta = sumValue * sumValue * alpha;
        newRatio = Math.max(maxValue / beta, beta / minValue);
        if (newRatio > minRatio) {
          sumValue -= nodeValue;
          break;
        }
        minRatio = newRatio;
      }

      // Position and record the row orientation.
      rows.push(row = {
        value: sumValue,
        dice: dx < dy,
        children: nodes.slice(i0, i1)
      });
      if (row.dice) treemapDice(row, x0, y0, x1, value ? y0 += dy * sumValue / value : y1);else treemapSlice(row, x0, y0, value ? x0 += dx * sumValue / value : x1, y1);
      value -= sumValue, i0 = i1;
    }
    return rows;
  }
  var squarify = (function custom(ratio) {
    function squarify(parent, x0, y0, x1, y1) {
      squarifyRatio(ratio, parent, x0, y0, x1, y1);
    }
    squarify.ratio = function (x) {
      return custom((x = +x) > 1 ? x : 1);
    };
    return squarify;
  })(phi);

  function treemap () {
    var tile = squarify,
      round = false,
      dx = 1,
      dy = 1,
      paddingStack = [0],
      paddingInner = constantZero,
      paddingTop = constantZero,
      paddingRight = constantZero,
      paddingBottom = constantZero,
      paddingLeft = constantZero;
    function treemap(root) {
      root.x0 = root.y0 = 0;
      root.x1 = dx;
      root.y1 = dy;
      root.eachBefore(positionNode);
      paddingStack = [0];
      if (round) root.eachBefore(roundNode);
      return root;
    }
    function positionNode(node) {
      var p = paddingStack[node.depth],
        x0 = node.x0 + p,
        y0 = node.y0 + p,
        x1 = node.x1 - p,
        y1 = node.y1 - p;
      if (x1 < x0) x0 = x1 = (x0 + x1) / 2;
      if (y1 < y0) y0 = y1 = (y0 + y1) / 2;
      node.x0 = x0;
      node.y0 = y0;
      node.x1 = x1;
      node.y1 = y1;
      if (node.children) {
        p = paddingStack[node.depth + 1] = paddingInner(node) / 2;
        x0 += paddingLeft(node) - p;
        y0 += paddingTop(node) - p;
        x1 -= paddingRight(node) - p;
        y1 -= paddingBottom(node) - p;
        if (x1 < x0) x0 = x1 = (x0 + x1) / 2;
        if (y1 < y0) y0 = y1 = (y0 + y1) / 2;
        tile(node, x0, y0, x1, y1);
      }
    }
    treemap.round = function (x) {
      return arguments.length ? (round = !!x, treemap) : round;
    };
    treemap.size = function (x) {
      return arguments.length ? (dx = +x[0], dy = +x[1], treemap) : [dx, dy];
    };
    treemap.tile = function (x) {
      return arguments.length ? (tile = required(x), treemap) : tile;
    };
    treemap.padding = function (x) {
      return arguments.length ? treemap.paddingInner(x).paddingOuter(x) : treemap.paddingInner();
    };
    treemap.paddingInner = function (x) {
      return arguments.length ? (paddingInner = typeof x === "function" ? x : constant$1(+x), treemap) : paddingInner;
    };
    treemap.paddingOuter = function (x) {
      return arguments.length ? treemap.paddingTop(x).paddingRight(x).paddingBottom(x).paddingLeft(x) : treemap.paddingTop();
    };
    treemap.paddingTop = function (x) {
      return arguments.length ? (paddingTop = typeof x === "function" ? x : constant$1(+x), treemap) : paddingTop;
    };
    treemap.paddingRight = function (x) {
      return arguments.length ? (paddingRight = typeof x === "function" ? x : constant$1(+x), treemap) : paddingRight;
    };
    treemap.paddingBottom = function (x) {
      return arguments.length ? (paddingBottom = typeof x === "function" ? x : constant$1(+x), treemap) : paddingBottom;
    };
    treemap.paddingLeft = function (x) {
      return arguments.length ? (paddingLeft = typeof x === "function" ? x : constant$1(+x), treemap) : paddingLeft;
    };
    return treemap;
  }

  function treemapBinary (parent, x0, y0, x1, y1) {
    var nodes = parent.children,
      i,
      n = nodes.length,
      sum,
      sums = new Array(n + 1);
    for (sums[0] = sum = i = 0; i < n; ++i) {
      sums[i + 1] = sum += nodes[i].value;
    }
    partition(0, n, parent.value, x0, y0, x1, y1);
    function partition(i, j, value, x0, y0, x1, y1) {
      if (i >= j - 1) {
        var node = nodes[i];
        node.x0 = x0, node.y0 = y0;
        node.x1 = x1, node.y1 = y1;
        return;
      }
      var valueOffset = sums[i],
        valueTarget = value / 2 + valueOffset,
        k = i + 1,
        hi = j - 1;
      while (k < hi) {
        var mid = k + hi >>> 1;
        if (sums[mid] < valueTarget) k = mid + 1;else hi = mid;
      }
      if (valueTarget - sums[k - 1] < sums[k] - valueTarget && i + 1 < k) --k;
      var valueLeft = sums[k] - valueOffset,
        valueRight = value - valueLeft;
      if (x1 - x0 > y1 - y0) {
        var xk = value ? (x0 * valueRight + x1 * valueLeft) / value : x1;
        partition(i, k, valueLeft, x0, y0, xk, y1);
        partition(k, j, valueRight, xk, y0, x1, y1);
      } else {
        var yk = value ? (y0 * valueRight + y1 * valueLeft) / value : y1;
        partition(i, k, valueLeft, x0, y0, x1, yk);
        partition(k, j, valueRight, x0, yk, x1, y1);
      }
    }
  }

  var withTreemap = (function (View) {
    return /*#__PURE__*/function (_Component) {
      _inherits$1(Treemap, _Component);
      var _super = _createSuper$1(Treemap);
      function Treemap(props, context) {
        var _this;
        _classCallCheck$1(this, Treemap);
        _this = _super.call(this, props, context);
        var color = props.color,
          data = props.data;
        _this.coord = new coordController();
        _this.color = new Category$1(_objectSpread(_objectSpread({
          range: Theme.colors
        }, color), {}, {
          data: data
        }));
        return _this;
      }
      _createClass$3(Treemap, [{
        key: "willMount",
        value: function willMount() {
          var props = this.props,
            coord = this.coord,
            layout = this.layout;
          var coordOption = props.coord;
          coord.updateLayout(layout);
          coord.create(coordOption);
        }
      }, {
        key: "treemapLayout",
        value: function treemapLayout() {
          var props = this.props,
            coord = this.coord,
            colorAttr = this.color;
          var data = props.data,
            value = props.value;
          var root = hierarchy({
            children: data
          }).sum(function (d) {
            return d[value];
          }).sort(function (a, b) {
            return b[value] - a[value];
          });
          var treemapLayout = treemap()
          // 默认treemapSquarify
          .tile(treemapBinary)
          // .size([1, 1])
          // @ts-ignore
          .round(false);
          // .padding(space)
          // .paddingInner(space);
          // .paddingOuter(options.paddingOuter)
          // .paddingTop(options.paddingTop)
          // .paddingRight(options.paddingRight)
          // .paddingBottom(options.paddingBottom)
          // .paddingLeft(options.paddingLeft);
          var nodes = treemapLayout(root);
          return nodes.children.map(function (item) {
            var data = item.data,
              x0 = item.x0,
              y0 = item.y0,
              x1 = item.x1,
              y1 = item.y1;
            var color = colorAttr.mapping(data[colorAttr.field]);
            var rect = coord.getCoord().convertRect({
              x: [x0, x1],
              y: [y0, y1]
            });
            return _objectSpread({
              key: data.key,
              origin: data,
              color: color
            }, rect);
          });
        }
      }, {
        key: "render",
        value: function render() {
          var nodes = this.treemapLayout();
          var props = this.props,
            coord = this.coord;
          return jsx(View, _objectSpread(_objectSpread({
            nodes: nodes
          }, props), {}, {
            coord: coord.getCoord()
          }));
        }
      }]);
      return Treemap;
    }(Component);
  });

  var TreemapView = (function (props) {
    var nodes = props.nodes,
      coord = props.coord;
    if (coord.isPolar) {
      var center = coord.center;
      var x = center.x,
        y = center.y;
      return jsx("group", null, nodes.map(function (node) {
        var xMin = node.xMin,
          xMax = node.xMax,
          yMin = node.yMin,
          yMax = node.yMax,
          color = node.color;
        return jsx("sector", {
          attrs: {
            x: x,
            y: y,
            lineWidth: '1px',
            stroke: '#fff',
            startAngle: xMin,
            endAngle: xMax,
            r0: yMin,
            r: yMax,
            fill: color
          }
        });
      }));
    }
    return jsx("group", null, nodes.map(function (node) {
      var key = node.key,
        xMin = node.xMin,
        xMax = node.xMax,
        yMin = node.yMin,
        yMax = node.yMax,
        color = node.color;
      return jsx("rect", {
        key: key,
        attrs: {
          x: xMin,
          y: yMin,
          width: xMax - xMin,
          height: yMax - yMin,
          fill: color,
          lineWidth: '4px',
          stroke: '#fff',
          radius: '8px'
        },
        animation: {
          appear: {
            easing: 'linear',
            duration: 450,
            property: ['fillOpacity', 'strokeOpacity'],
            start: {
              fillOpacity: 0,
              strokeOpacity: 0
            },
            end: {
              fillOpacity: 1,
              strokeOpacity: 1
            }
          },
          update: {
            easing: 'linear',
            duration: 450,
            property: ['x', 'y', 'width', 'height', 'radius', 'lineWidth']
          }
        }
      });
    }));
  });

  var index$8 = withTreemap(TreemapView);

  function rootParent(data) {
    var d = data;
    while (d.depth > 1) {
      d = d.parent;
    }
    return d;
  }
  var withSunburst = (function (View) {
    return /*#__PURE__*/function (_Component) {
      _inherits$1(Sunburst, _Component);
      var _super = _createSuper$1(Sunburst);
      function Sunburst(props, context) {
        var _this;
        _classCallCheck$1(this, Sunburst);
        _this = _super.call(this, props, context);
        var color = props.color,
          data = props.data;
        _this.coord = new coordController();
        _this.color = new Category$1(_objectSpread(_objectSpread({
          range: Theme.colors
        }, color), {}, {
          data: data
        }));
        return _this;
      }
      _createClass$3(Sunburst, [{
        key: "willMount",
        value: function willMount() {
          var props = this.props,
            coord = this.coord,
            layout = this.layout;
          var coordOption = props.coord;
          coord.updateLayout(layout);
          coord.create(coordOption);
        }
      }, {
        key: "didMount",
        value: function didMount() {
          var _this2 = this;
          var props = this.props,
            context = this.context;
          var onClick = props.onClick;
          this.triggerRef = [];
          context.gesture.on('click', function (ev) {
            var x = ev.x,
              y = ev.y;
            var shape = _this2.triggerRef.find(function (ref) {
              return isInBBox(ref.current.getBBox(), {
                x: x,
                y: y
              });
            });
            if (shape) {
              ev.shape = shape;
              // @ts-ignore
              ev.payload = shape.payload;
              onClick && onClick(ev);
            }
          });
        }
      }, {
        key: "_mapping",
        value: function _mapping(children) {
          var colorAttr = this.color,
            coord = this.coord;
          for (var i = 0, len = children.length; i < len; i++) {
            var node = children[i];
            var root = rootParent(node);
            var color = colorAttr.mapping(root.data[colorAttr.field]);
            node.color = color;
            var x0 = node.x0,
              x1 = node.x1,
              y0 = node.y0,
              y1 = node.y1;
            var rect = coord.getCoord().convertRect({
              x: [x0, x1],
              y: [y0, y1]
            });
            mix(node, rect);
            // 递归处理
            if (node.children && node.children.length) {
              this._mapping(node.children);
            }
          }
        }
      }, {
        key: "sunburst",
        value: function sunburst() {
          var props = this.props;
          var data = props.data,
            value = props.value,
            _props$sort = props.sort,
            sort = _props$sort === void 0 ? true : _props$sort;
          var root = hierarchy({
            children: data
          }).sum(function (d) {
            return d[value];
          });
          // 内置按value大小顺序排序，支持传入sort函数
          if (sort === true || isFunction$2(sort)) {
            var sortFn = isFunction$2(sort) ? sort : function (a, b) {
              return b[value] - a[value];
            };
            root.sort(sortFn);
          }
          var nodes = partition()(root);
          var children = nodes.children;
          this._mapping(children);
          return nodes;
        }
      }, {
        key: "render",
        value: function render() {
          var node = this.sunburst();
          var coord = this.coord,
            props = this.props;
          return jsx(View, _objectSpread(_objectSpread({}, props), {}, {
            coord: coord.getCoord(),
            node: node,
            triggerRef: this.triggerRef
          }));
        }
      }]);
      return Sunburst;
    }(Component);
  });

  var SunburstView = (function (props) {
    var coord = props.coord,
      node = props.node;
    var children = node.children;
    var _coord$center = coord.center,
      x = _coord$center.x,
      y = _coord$center.y;
    var renderNodes = function renderNodes(nodes) {
      return jsx("group", null, nodes.map(function (node) {
        var xMin = node.xMin,
          xMax = node.xMax,
          yMin = node.yMin,
          yMax = node.yMax,
          color = node.color,
          children = node.children;
        return jsx("group", null, jsx("sector", {
          attrs: {
            x: x,
            y: y,
            lineWidth: '1px',
            stroke: '#fff',
            startAngle: "".concat(xMin, " rad"),
            endAngle: "".concat(xMax, " rad"),
            r0: yMin,
            r: yMax,
            fill: color
          }
        }), children && children.length ? renderNodes(children) : null);
      }));
    };
    return renderNodes(children);
  });

  var IcicleView = (function (props) {
    var node = props.node;
    var children = node.children;
    var renderNodes = function renderNodes(nodes) {
      return jsx("group", null, nodes.map(function (node) {
        var xMin = node.xMin,
          xMax = node.xMax,
          yMin = node.yMin,
          yMax = node.yMax,
          color = node.color,
          children = node.children;
        return jsx("group", null, jsx("rect", {
          attrs: {
            x: xMin,
            y: yMin,
            width: xMax - xMin,
            height: yMax - yMin,
            lineWidth: '1px',
            stroke: '#fff',
            fill: color
          }
        }), children && children.length ? renderNodes(children) : null);
      }));
    };
    return renderNodes(children);
  });

  var View = (function (props) {
    var coord = props.coord;
    if (coord.type === 'polar') {
      return jsx(SunburstView, _objectSpread({}, props));
    }
    return jsx(IcicleView, _objectSpread({}, props));
  });

  var index$9 = withSunburst(View);

  var DEFAULT_CONFIG = {
    anchorOffset: '10px',
    inflectionOffset: '30px',
    sidePadding: '15px',
    height: '64px',
    adjustOffset: '30',
    triggerOn: 'click',
    // activeShape: false, // 当有图形被选中的时候，是否激活图形
    // activeStyle: {
    //   offset: '1px',
    //   appendRadius: '8px',
    //   fillOpacity: 0.5,
    // },
    label1OffsetY: '-4px',
    label2OffsetY: '4px'
  };
  function getEndPoint(center, angle, r) {
    return {
      x: center.x + r * Math.cos(angle),
      y: center.y + r * Math.sin(angle)
    };
  }
  // 计算中间角度
  function getMiddleAngle(startAngle, endAngle) {
    if (endAngle < startAngle) {
      endAngle += Math.PI * 2;
    }
    return (endAngle + startAngle) / 2;
  }
  function move(from, to, count, center) {
    var x = center.x;
    var sort = from.sort(function (a, b) {
      var aDistance = Math.abs(a.x - x);
      var bDistance = Math.abs(b.x - x);
      return bDistance - aDistance;
    });
    return [sort.slice(0, sort.length - count), sort.slice(sort.length - count).concat(to)];
  }
  // 第一象限
  function isFirstQuadrant(angle) {
    return angle >= -Math.PI / 2 && angle < 0;
  }
  // 第二象限
  function isSecondQuadrant(angle) {
    return angle >= 0 && angle < Math.PI / 2;
  }
  function isThirdQuadrant(angle) {
    return angle >= Math.PI / 2 && angle < Math.PI;
  }
  function isFourthQuadrant(angle) {
    return angle >= Math.PI && angle < Math.PI * 3 / 2;
  }
  function findShapeByClassName(shape, point, className) {
    var targetShapes = shape.getElementsByClassName(className);
    for (var i = 0, len = targetShapes.length; i < len; i++) {
      var _shape = targetShapes[i];
      if (isInBBox(_shape.getBBox(), point)) {
        return _shape;
      }
    }
  }
  var withPieLabel = (function (View) {
    return /*#__PURE__*/function (_Component) {
      _inherits$1(PieLabel, _Component);
      var _super = _createSuper$1(PieLabel);
      function PieLabel(props) {
        var _this;
        _classCallCheck$1(this, PieLabel);
        _this = _super.call(this, props);
        _this._handleEvent = function (ev) {
          var _this$props = _this.props,
            chart = _this$props.chart,
            onClick = _this$props.onClick;
          var ele = _this.triggerRef.current;
          var points = ev.points,
            x = ev.x,
            y = ev.y;
          var point = points ? points[0] : {
            x: x,
            y: y
          };
          var shape = findShapeByClassName(ele, point, 'click');
          var pieData = chart.getSnapRecords(point);
          if (typeof onClick === 'function') {
            // 点击label
            if (shape) {
              onClick(shape.get('data'));
            }
            // 点击饼图
            else if (isArray$1(pieData) && pieData.length > 0) {
              onClick(pieData);
            }
          }
        };
        _this.triggerRef = {};
        return _this;
      }
      _createClass$3(PieLabel, [{
        key: "willMount",
        value: function willMount() {}
        /**
         * 绑定事件
         */
      }, {
        key: "didMount",
        value: function didMount() {
          this._initEvent();
        }
      }, {
        key: "getLabels",
        value: function getLabels(props) {
          var chart = props.chart,
            coord = props.coord,
            anchorOffset = props.anchorOffset,
            inflectionOffset = props.inflectionOffset,
            label1 = props.label1,
            label2 = props.label2,
            itemHeight = props.height,
            sidePadding = props.sidePadding;
          var center = coord.center,
            radius = coord.radius,
            coordWidth = coord.width,
            coordHeight = coord.height,
            coordLeft = coord.left,
            coordRight = coord.right,
            coordTop = coord.top;
          var maxCountForOneSide = Math.floor(coordHeight / itemHeight);
          var maxCount = maxCountForOneSide * 2;
          var geometry = chart.getGeometrys()[0];
          var records = geometry.flatRecords()
          // 按角度大到小排序
          .sort(function (a, b) {
            var angle1 = a.xMax - a.xMin;
            var angle2 = b.xMax - b.xMin;
            return angle2 - angle1;
          })
          // 只取前 maxCount 个显示
          .slice(0, maxCount);
          // 存储左右 labels
          var halves = [[], [] // right
          ];

          records.forEach(function (record) {
            var xMin = record.xMin,
              xMax = record.xMax,
              color = record.color,
              origin = record.origin;
            // 锚点角度
            var anchorAngle = getMiddleAngle(xMin, xMax);
            // 锚点坐标
            var anchorPoint = getEndPoint(center, anchorAngle, radius + anchorOffset);
            // 拐点坐标
            var inflectionPoint = getEndPoint(center, anchorAngle, radius + inflectionOffset);
            // 锚点方向
            var side = anchorPoint.x < center.x ? 'left' : 'right';
            var label = {
              origin: origin,
              angle: anchorAngle,
              anchor: anchorPoint,
              inflection: inflectionPoint,
              side: side,
              x: inflectionPoint.x,
              y: inflectionPoint.y,
              r: radius + inflectionOffset,
              color: color,
              label1: isFunction$2(label1) ? label1(origin, record) : label1,
              label2: isFunction$2(label2) ? label2(origin, record) : label2
            };
            // 判断文本的方向
            if (side === 'left') {
              halves[0].push(label);
            } else {
              halves[1].push(label);
            }
          });
          // 判断是有一边超过了显示的最大
          if (halves[0].length > maxCountForOneSide) {
            halves = move(halves[0], halves[1], halves[0].length - maxCountForOneSide, center);
          } else if (halves[1].length > maxCountForOneSide) {
            var _move = move(halves[1], halves[0], halves[1].length - maxCountForOneSide, center),
              _move2 = _slicedToArray$1(_move, 2),
              right = _move2[0],
              left = _move2[1];
            halves = [left, right];
          }
          // label 的最大宽度
          var labelWidth = coordWidth / 2 - radius - anchorOffset - inflectionOffset - 2 * sidePadding;
          var labels = [];
          halves.forEach(function (half, index) {
            var showSide = index === 0 ? 'left' : 'right';
            // 顺时针方向排序
            half.sort(function (a, b) {
              var aAngle = a.angle;
              var bAngle = b.angle;
              if (showSide === 'left') {
                // 是否在第一象限
                aAngle = isFirstQuadrant(aAngle) ? aAngle + Math.PI * 2 : aAngle;
                bAngle = isFirstQuadrant(bAngle) ? bAngle + Math.PI * 2 : bAngle;
                return bAngle - aAngle;
              } else {
                // 是否在第四象限
                aAngle = isFourthQuadrant(aAngle) ? aAngle - Math.PI * 2 : aAngle;
                bAngle = isFourthQuadrant(bAngle) ? bAngle - Math.PI * 2 : bAngle;
                return aAngle - bAngle;
              }
            });
            var pointsY = half.map(function (label) {
              return label.y;
            });
            var maxY = Math.max.apply(null, pointsY);
            var minY = Math.min.apply(null, pointsY);
            // 每个 label 占用的高度
            var labelCount = half.length;
            var labelHeight = coordHeight / labelCount;
            var halfLabelHeight = labelHeight / 2;
            // 线之间的间隔
            var lineInterval = 2;
            if (showSide === 'left') {
              half.forEach(function (label, index) {
                var anchor = label.anchor,
                  inflection = label.inflection,
                  angle = label.angle,
                  x = label.x,
                  y = label.y;
                var points = [anchor, inflection];
                var endX = coordLeft + sidePadding;
                var endY = coordTop + halfLabelHeight + labelHeight * index;
                // 文本开始点
                var labelStart = {
                  x: endX + labelWidth + lineInterval * index,
                  y: endY
                };
                // 文本结束点
                var labelEnd = {
                  x: endX,
                  y: endY
                };
                // 第四象限
                if (isFirstQuadrant(angle)) {
                  var pointY = minY - lineInterval * (labelCount - index);
                  points.push({
                    x: x,
                    y: pointY
                  });
                  points.push({
                    x: labelStart.x,
                    y: pointY
                  });
                } else if (isThirdQuadrant(angle) || isFourthQuadrant(angle)) {
                  points.push({
                    x: labelStart.x,
                    y: y
                  });
                } else if (isSecondQuadrant(angle)) {
                  var _pointY = maxY + lineInterval * index;
                  points.push({
                    x: x,
                    y: _pointY
                  });
                  points.push({
                    x: labelStart.x,
                    y: _pointY
                  });
                }
                points.push(labelStart);
                points.push(labelEnd);
                label.points = points;
                label.side = showSide;
                labels.push(label);
              });
            } else {
              half.forEach(function (label, index) {
                var anchor = label.anchor,
                  inflection = label.inflection,
                  angle = label.angle,
                  x = label.x,
                  y = label.y;
                // 折线的点
                var points = [anchor, inflection];
                var endX = coordRight - sidePadding;
                var endY = coordTop + halfLabelHeight + labelHeight * index;
                // 文本开始点
                var labelStart = {
                  x: endX - labelWidth - lineInterval * index,
                  y: endY
                };
                // 文本结束点
                var labelEnd = {
                  x: endX,
                  y: endY
                };
                // 第四象限
                if (isFourthQuadrant(angle)) {
                  var pointY = minY - lineInterval * (labelCount - index);
                  points.push({
                    x: x,
                    y: pointY
                  });
                  points.push({
                    x: labelStart.x,
                    y: pointY
                  });
                } else if (isFirstQuadrant(angle) || isSecondQuadrant(angle)) {
                  points.push({
                    x: labelStart.x,
                    y: y
                  });
                } else if (isThirdQuadrant(angle)) {
                  var _pointY2 = maxY + lineInterval * index;
                  points.push({
                    x: x,
                    y: _pointY2
                  });
                  points.push({
                    x: labelStart.x,
                    y: _pointY2
                  });
                }
                points.push(labelStart);
                points.push(labelEnd);
                label.points = points;
                label.side = showSide;
                labels.push(label);
              });
            }
          });
          return labels;
        }
      }, {
        key: "_initEvent",
        value: function _initEvent() {
          var context = this.context,
            props = this.props;
          var _props$triggerOn = props.triggerOn,
            triggerOn = _props$triggerOn === void 0 ? DEFAULT_CONFIG.triggerOn : _props$triggerOn;
          context.gesture.on(triggerOn, this._handleEvent);
        }
      }, {
        key: "render",
        value: function render() {
          var context = this.context;
          var props = context.px2hd(deepMix({}, DEFAULT_CONFIG, this.props));
          var labels = this.getLabels(props);
          return jsx(View, _objectSpread(_objectSpread({
            labels: labels
          }, props), {}, {
            triggerRef: this.triggerRef
          }));
        }
      }]);
      return PieLabel;
    }(Component);
  });

  var PieLabelView = (function (props) {
    var lineStyle = props.lineStyle,
      anchorStyle = props.anchorStyle,
      labels = props.labels,
      label1OffsetY = props.label1OffsetY,
      label2OffsetY = props.label2OffsetY,
      triggerRef = props.triggerRef;
    return jsx("group", {
      ref: triggerRef
    }, labels.map(function (label) {
      var origin = label.origin,
        anchor = label.anchor,
        side = label.side,
        color = label.color,
        label1 = label.label1,
        label2 = label.label2,
        points = label.points;
      var end = points[points.length - 1];
      return jsx("group", null, jsx("circle", {
        attrs: _objectSpread({
          r: '4px',
          cx: anchor.x,
          cy: anchor.y,
          fill: color
        }, anchorStyle)
      }), jsx("polyline", {
        attrs: _objectSpread({
          points: points.map(function (d) {
            return [d.x, d.y];
          }),
          lineWidth: '2px',
          stroke: color
        }, lineStyle)
      }), jsx("text", {
        className: "click",
        attrs: _objectSpread({
          x: end.x,
          y: end.y + label1OffsetY,
          fontSize: '24px',
          lineHeight: '24px',
          fill: color,
          textBaseline: 'bottom',
          textAlign: side === 'left' ? 'left' : 'right'
        }, label1),
        data: origin
      }), jsx("text", {
        className: "click",
        attrs: _objectSpread({
          x: end.x,
          y: end.y + label2OffsetY,
          fontSize: '24px',
          lineHeight: '24px',
          fill: '#808080',
          textBaseline: 'top',
          textAlign: side === 'left' ? 'left' : 'right'
        }, label2),
        data: origin
      }));
    }));
  });

  var index$a = withPieLabel(PieLabelView);

  var getPoint$3 = function getPoint(cener, angle, r) {
    var x = cener.x + Math.cos(angle) * r;
    var y = cener.y + Math.sin(angle) * r;
    return {
      x: x,
      y: y
    };
  };
  var getTicks = function getTicks(start, end, tickCount, center, r, tickOffset, tickLength) {
    var ticks = [];
    var diff = end - start;
    for (var i = 0; i <= tickCount; i++) {
      var tickValue = start + diff * i / tickCount;
      var startPoint = getPoint$3(center, tickValue, r + tickOffset - tickLength);
      var endPoint = getPoint$3(center, tickValue, r + tickOffset);
      ticks.push({
        tickValue: tickValue,
        start: startPoint,
        end: endPoint
      });
    }
    return ticks;
  };
  var withGauge = (function (View) {
    return /*#__PURE__*/function (_Component) {
      _inherits$1(Guage, _Component);
      var _super = _createSuper$1(Guage);
      function Guage() {
        _classCallCheck$1(this, Guage);
        return _super.apply(this, arguments);
      }
      _createClass$3(Guage, [{
        key: "render",
        value: function render() {
          var props = this.props,
            context = this.context;
          var startAngle = props.startAngle,
            endAngle = props.endAngle,
            tickCount = props.tickCount,
            center = props.center,
            r = props.r,
            tickOffset = props.tickOffset,
            tickLength = props.tickLength;
          var ticks = getTicks(startAngle, endAngle, tickCount, center, context.px2hd(r), context.px2hd(tickOffset), context.px2hd(tickLength));
          return jsx(View, _objectSpread(_objectSpread({}, props), {}, {
            ticks: ticks
          }));
        }
      }]);
      return Guage;
    }(Component);
  });

  var GaugeView = (function (props) {
    var center = props.center,
      startAngle = props.startAngle,
      endAngle = props.endAngle,
      r = props.r,
      percent = props.percent,
      ticks = props.ticks;
    var x = center.x,
      y = center.y;
    var diff = endAngle - startAngle;
    return jsx("group", null, jsx("arc", {
      attrs: {
        x: x,
        y: y,
        r: r,
        startAngle: "".concat(startAngle, " rad"),
        endAngle: "".concat(endAngle, " rad"),
        lineWidth: '20px',
        lineCap: 'round',
        stroke: '#e7e7e7'
      }
    }), jsx("arc", {
      attrs: {
        x: x,
        y: y,
        r: r,
        startAngle: "".concat(startAngle, " rad"),
        endAngle: "".concat(startAngle, " rad"),
        lineWidth: '40px',
        lineCap: 'round',
        stroke: '#0075ff'
      },
      animation: {
        appear: {
          easing: 'linear',
          duration: 500,
          property: ['endAngle'],
          start: {
            endAngle: "".concat(startAngle, " rad")
          },
          end: {
            endAngle: "".concat(startAngle + diff * percent, " rad")
          }
        }
      }
    }), ticks.map(function (tick) {
      var start = tick.start,
        end = tick.end;
      return jsx("line", {
        attrs: {
          x1: start.x,
          y1: start.y,
          x2: end.x,
          y2: end.y,
          lineWidth: '6px',
          lineCap: 'round',
          stroke: '#e7e7e7'
        }
      });
    }));
  });

  var index$b = withGauge(GaugeView);

  // 判断新老values是否相等，这里只要判断前后是否相等即可
  function isValuesEqual(values, newValues) {
    if (values.length !== newValues.length) {
      return false;
    }
    var lastIndex = values.length - 1;
    return values[0] === newValues[0] && values[lastIndex] === newValues[lastIndex];
  }
  function updateCategoryRange(scale, originScale, range) {
    var currentValues = scale.values,
      currentTicks = scale.ticks,
      tickMethod = scale.tickMethod,
      tickCount = scale.tickCount;
    var originValues = originScale.values;
    var _range = _slicedToArray$1(range, 2),
      start = _range[0],
      end = _range[1];
    var len = originValues.length;
    var valueStart = start * len;
    var valueEnd = end * len;
    // 保持滑动时个数的稳定
    var count = Math.round(valueEnd - valueStart);
    var sliceSatrt = Math.round(valueStart);
    // 从原始数据里截取需要显示的数据
    var newValues = originValues.slice(sliceSatrt, sliceSatrt + count);
    // 根据当前数据的比例，和定义的tickCount计算应该需要多少个ticks
    var newTickCount = Math.round(tickCount * originValues.length / newValues.length);
    // 计算新的ticks
    var catTicks = getTickMethod(tickMethod);
    var newTicks = catTicks({
      tickCount: newTickCount,
      values: originValues
    });
    // 如果新数组和当前显示的数组相同，则不更新
    if (isValuesEqual(currentValues, newValues) && isValuesEqual(currentTicks, newTicks)) {
      return;
    }
    scale.change({
      values: newValues,
      ticks: newTicks
    });
    return scale;
  }
  function updateLinearRange(scale, originScale, range) {
    var min = originScale.min,
      max = originScale.max;
    var _range2 = _slicedToArray$1(range, 2),
      start = _range2[0],
      end = _range2[1];
    var newMin = min + (max - min) * start;
    var newMax = min + (max - min) * end;
    scale.change({
      min: newMin,
      max: newMax,
      nice: false
    });
  }
  function updateScale(scale, values) {
    var isLinear = scale.isLinear;
    if (isLinear) {
      var _getRange = getRange(values),
        min = _getRange.min,
        max = _getRange.max;
      return scale.change({
        min: min,
        max: max,
        nice: true
      });
    }
  }
  function updateRange(scale, originScale, range) {
    var isCategory = scale.isCategory,
      isLinear = scale.isLinear;
    if (isCategory) {
      return updateCategoryRange(scale, originScale, range);
    }
    if (isLinear) {
      return updateLinearRange(scale, originScale, range);
    }
  }
  function updateFollow(scales, mainScale, data) {
    var mainField = mainScale.field,
      mainType = mainScale.type,
      mainValues = mainScale.values;
    // 转成 map 提高查询性能
    var mainValuesMap = {};
    mainValues.forEach(function (item) {
      mainValuesMap[item] = true;
    });
    return scales.map(function (scale) {
      var followField = scale.field;
      var values = [];
      data.forEach(function (item) {
        var value = mainType === 'timeCat' ? toTimeStamp$1(item[mainField]) : item[mainField];
        if (mainValuesMap[value]) {
          values.push(item[followField]);
        }
      });
      return updateScale(scale, values);
    });
  }

  function lerp$2(min, max, fraction) {
    return (max - min) * fraction + min;
  }
  function isEqualRange(aRange, bRange) {
    for (var i in aRange) {
      if (!isNumberEqual$3(aRange[i], bRange[i])) return false;
    }
    return true;
  }
  function cloneScale$1(scale, scaleConfig) {
    // @ts-ignore
    return new scale.constructor(_objectSpread(_objectSpread({}, scale.__cfg__), scaleConfig));
  }
  // 缩放
  var Zoom = /*#__PURE__*/function (_Component) {
    _inherits$1(Zoom, _Component);
    var _super = _createSuper$1(Zoom);
    function Zoom(props) {
      var _this;
      _classCallCheck$1(this, Zoom);
      var defaultProps = {
        onPanStart: function onPanStart() {},
        onPinchStart: function onPinchStart() {},
        onPan: function onPan() {},
        onPinch: function onPinch() {},
        onInit: function onInit() {},
        onPanEnd: function onPanEnd() {},
        onPinchEnd: function onPinchEnd() {},
        minCount: 10
      };
      _this = _super.call(this, _objectSpread(_objectSpread({}, defaultProps), props));
      _this.scale = {};
      _this.originScale = {};
      //swipe end x y
      _this.swipeEnd = {
        startX: 0,
        startY: 0,
        endX: 0,
        endY: 0
      };
      _this.onStart = function () {
        var _assertThisInitialize = _assertThisInitialized$3(_this),
          state = _assertThisInitialize.state;
        var range = state.range;
        _this.startRange = range;
        _this.loop && cancelAnimationFrame(_this.loop);
      };
      _this.onPan = function (ev) {
        var _assertThisInitialize2 = _assertThisInitialized$3(_this),
          dims = _assertThisInitialize2.dims;
        var range = {};
        each$1(dims, function (dim) {
          if (dim === 'x') {
            range['x'] = _this._doXPan(ev);
            return;
          }
          if (dim === 'y') {
            range['y'] = _this._doYPan(ev);
            return;
          }
        });
        if (isEqualRange(range, _this.state.range)) return;
        _this.setState({
          range: range
        });
      };
      _this.onSwipe = function (ev) {
        var swipe = _this.props.swipe;
        if (_this.props.mode.length < 2 || !swipe) return;
        var _ev$velocityX = ev.velocityX,
          velocityX = _ev$velocityX === void 0 ? 0 : _ev$velocityX,
          _ev$velocityY = ev.velocityY,
          velocityY = _ev$velocityY === void 0 ? 0 : _ev$velocityY,
          points = ev.points;
        var range = _this.state.range;
        var _points$ = points[0],
          x = _points$.x,
          y = _points$.y;
        // 边界处理
        if (Math.abs((range === null || range === void 0 ? void 0 : range.x[0]) - 0) < 0.0005 && velocityX > 0) return;
        if (Math.abs((range === null || range === void 0 ? void 0 : range.x[1]) - 1) < 0.0005 && velocityX < 0) return;
        if (Math.abs((range === null || range === void 0 ? void 0 : range.y[0]) - 0) < 0.0005 && velocityY < 0) return;
        if (Math.abs((range === null || range === void 0 ? void 0 : range.x[1]) - 1) < 0.0005 && velocityY > 0) return;
        _this.swipeEnd = {
          startX: x,
          startY: y,
          endX: x + velocityX * 50,
          endY: y - velocityY * 50
        };
        _this.onStart();
        _this.update();
      };
      _this.onPinch = function (ev) {
        var _assertThisInitialize3 = _assertThisInitialized$3(_this),
          dims = _assertThisInitialize3.dims;
        var range = {};
        each$1(dims, function (dim) {
          if (dim === 'x') {
            range['x'] = _this._doXPinch(ev);
            return;
          }
          if (dim === 'y') {
            range['y'] = _this._doYPinch(ev);
            return;
          }
        });
        if (isEqualRange(range, _this.state.range)) return;
        _this.setState({
          range: range
        });
      };
      _this.onEnd = function () {
        _this.startRange = null;
      };
      var _props$range = props.range,
        mode = props.mode;
      _this.dims = mode instanceof Array ? mode : [mode];
      return _this;
    }
    _createClass$3(Zoom, [{
      key: "didMount",
      value: function didMount() {
        this._bindEvents();
      }
    }, {
      key: "willReceiveProps",
      value: function willReceiveProps(nextProps) {
        var nextRange = nextProps.range;
        var lastRange = this.props.range;
        if (!equal(nextRange, lastRange)) {
          var cacheRange = {};
          each$1(this.dims, function (dim) {
            cacheRange[dim] = nextRange;
          });
          this.state = {
            range: cacheRange
          };
        }
      }
    }, {
      key: "willMount",
      value: function willMount() {
        var _this2 = this;
        var props = this.props,
          dims = this.dims,
          state = this.state;
        var minCount = props.minCount,
          range = props.range;
        // const { range } = state;
        var valueLength = Number.MIN_VALUE;
        var cacheRange = {};
        each$1(dims, function (dim) {
          var scale = _this2._getScale(dim);
          var values = scale.values;
          valueLength = values.length > valueLength ? values.length : valueLength;
          _this2.scale[dim] = scale;
          _this2.originScale[dim] = cloneScale$1(scale);
          _this2.updateRange(range, dim);
          cacheRange[dim] = range;
        });
        // 图表上最少显示 MIN_COUNT 个数据
        this.minScale = minCount / valueLength;
        this.state = {
          range: cacheRange
        };
      }
    }, {
      key: "didUnmount",
      value: function didUnmount() {
        this._clearEvents();
      }
    }, {
      key: "update",
      value: function update() {
        var _this3 = this;
        var _this$swipeEnd = this.swipeEnd,
          startX = _this$swipeEnd.startX,
          startY = _this$swipeEnd.startY,
          endX = _this$swipeEnd.endX,
          endY = _this$swipeEnd.endY;
        var x = lerp$2(startX, endX, 0.05);
        var y = lerp$2(startY, endY, 0.05);
        this.swipeEnd = {
          startX: x,
          startY: y,
          endX: endX,
          endY: endY
        };
        var props = this.props;
        var coord = props.coord;
        var coordWidth = coord.width,
          coordHeight = coord.height;
        var range = {};
        range['x'] = this._doPan((x - startX) / coordWidth, 'x');
        range['y'] = this._doPan((y - startY) / coordHeight, 'y');
        this.setState({
          range: range
        });
        this.startRange = range;
        this.loop = requestAnimationFrame(function () {
          return _this3.update();
        });
        if (Math.abs(x - endX) < 0.0005 && Math.abs(y - endY) < 0.0005) {
          this.onEnd();
          cancelAnimationFrame(this.loop);
        }
      }
    }, {
      key: "_doXPan",
      value: function _doXPan(ev) {
        var direction = ev.direction,
          deltaX = ev.deltaX;
        if (this.props.mode.length === 1 && (direction === 'up' || direction === 'down')) {
          return;
        }
        ev.preventDefault && ev.preventDefault();
        var props = this.props;
        var coord = props.coord,
          _props$panSensitive = props.panSensitive,
          panSensitive = _props$panSensitive === void 0 ? 1 : _props$panSensitive;
        var coordWidth = coord.width;
        var ratio = deltaX / coordWidth * panSensitive;
        var newRange = this._doPan(ratio, 'x');
        return newRange;
      }
    }, {
      key: "_doYPan",
      value: function _doYPan(ev) {
        var direction = ev.direction,
          deltaY = ev.deltaY;
        if (this.props.mode.length === 1 && (direction === 'left' || direction === 'right')) {
          return;
        }
        ev.preventDefault && ev.preventDefault();
        var props = this.props;
        var coord = props.coord,
          _props$panSensitive2 = props.panSensitive,
          panSensitive = _props$panSensitive2 === void 0 ? 1 : _props$panSensitive2;
        var coordHeight = coord.height;
        var ratio = -deltaY / coordHeight * panSensitive;
        var newRange = this._doPan(ratio, 'y');
        return newRange;
      }
    }, {
      key: "_doPan",
      value: function _doPan(ratio, dim) {
        var startRange = this.startRange;
        var _startRange$dim = _slicedToArray$1(startRange[dim], 2),
          start = _startRange$dim[0],
          end = _startRange$dim[1];
        var rangeLen = end - start;
        var rangeOffset = rangeLen * ratio;
        var newStart = start - rangeOffset;
        var newEnd = end - rangeOffset;
        var newRange = this.updateRange([newStart, newEnd], dim);
        return newRange;
      }
    }, {
      key: "_doXPinch",
      value: function _doXPinch(ev) {
        ev.preventDefault && ev.preventDefault();
        var zoom = ev.zoom,
          center = ev.center;
        var props = this.props;
        var coord = props.coord;
        var coordWidth = coord.width,
          left = coord.left,
          right = coord.right;
        var leftLen = Math.abs(center.x - left);
        var rightLen = Math.abs(right - center.x);
        // 计算左右缩放的比例
        var leftZoom = leftLen / coordWidth;
        var rightZoom = rightLen / coordWidth;
        var newRange = this._doPinch(leftZoom, rightZoom, zoom, 'x');
        return newRange;
      }
    }, {
      key: "_doYPinch",
      value: function _doYPinch(ev) {
        ev.preventDefault && ev.preventDefault();
        var zoom = ev.zoom,
          center = ev.center;
        var props = this.props;
        var coord = props.coord;
        var coordHeight = coord.height,
          top = coord.top,
          bottom = coord.bottom;
        var topLen = Math.abs(center.y - top);
        var bottomLen = Math.abs(bottom - center.y);
        // 计算左右缩放的比例
        var topZoom = topLen / coordHeight;
        var bottomZoom = bottomLen / coordHeight;
        var newRange = this._doPinch(topZoom, bottomZoom, zoom, 'y');
        return newRange;
      }
    }, {
      key: "_doPinch",
      value: function _doPinch(startRatio, endRatio, zoom, dim) {
        var startRange = this.startRange,
          minScale = this.minScale,
          props = this.props;
        var _props$pinchSensitive = props.pinchSensitive,
          pinchSensitive = _props$pinchSensitive === void 0 ? 1 : _props$pinchSensitive;
        var _startRange$dim2 = _slicedToArray$1(startRange[dim], 2),
          start = _startRange$dim2[0],
          end = _startRange$dim2[1];
        var zoomOffset = (1 - zoom) * pinchSensitive;
        var rangeLen = end - start;
        var rangeOffset = rangeLen * zoomOffset;
        var startOffset = rangeOffset * startRatio;
        var endOffset = rangeOffset * endRatio;
        var newStart = Math.max(0, start - startOffset);
        var newEnd = Math.min(1, end + endOffset);
        var newRange = [newStart, newEnd];
        // 如果已经到了最小比例，则不能再继续再放大
        if (newEnd - newStart < minScale) {
          return this.state.range[dim];
        }
        return this.updateRange(newRange, dim);
      }
    }, {
      key: "updateRange",
      value: function updateRange$1(originalRange, dim) {
        if (!originalRange) return;
        var _originalRange = _slicedToArray$1(originalRange, 2),
          start = _originalRange[0],
          end = _originalRange[1];
        var rangeLength = end - start;
        // 处理边界值
        var newRange;
        if (start < 0) {
          newRange = [0, rangeLength];
        } else if (end > 1) {
          newRange = [1 - rangeLength, 1];
        } else {
          newRange = originalRange;
        }
        var props = this.props,
          scale = this.scale,
          originScale = this.originScale,
          state = this.state;
        var chart = props.chart,
          data = props.data,
          autoFit = props.autoFit;
        var range = state.range;
        if (range && isEqualRange(newRange, range[dim])) return newRange;
        // 更新主 scale
        updateRange(scale[dim], originScale[dim], newRange);
        if (autoFit) {
          var followScale = this._getFollowScales(dim);
          this.updateFollow(followScale, scale[dim], data);
        }
        // 手势变化不执行动画
        var animate = chart.animate;
        chart.setAnimate(false);
        chart.forceUpdate(function () {
          chart.setAnimate(animate);
        });
        return newRange;
      }
    }, {
      key: "updateFollow",
      value: function updateFollow$1(scales, mainScale, data) {
        updateFollow(scales, mainScale, data);
      }
    }, {
      key: "_getScale",
      value: function _getScale(dim) {
        var _this$props = this.props,
          coord = _this$props.coord,
          chart = _this$props.chart;
        if (dim === 'x') {
          return coord.transposed ? chart.getYScales()[0] : chart.getXScales()[0];
        } else {
          return coord.transposed ? chart.getXScales()[0] : chart.getYScales()[0];
        }
      }
    }, {
      key: "_getFollowScales",
      value: function _getFollowScales(dim) {
        var _this$props2 = this.props,
          coord = _this$props2.coord,
          chart = _this$props2.chart;
        if (dim === 'x') {
          return coord.transposed ? chart.getXScales() : chart.getYScales();
        }
        if (dim === 'y') {
          return coord.transposed ? chart.getYScales() : chart.getXScales();
        }
      }
    }, {
      key: "_bindEvents",
      value: function _bindEvents() {
        var _this4 = this;
        var context = this.context,
          props = this.props,
          scale = this.scale;
        var onPinchStart = props.onPinchStart,
          onPanStart = props.onPanStart,
          onPanEnd = props.onPanEnd,
          pan = props.pan,
          pinch = props.pinch,
          swipe = props.swipe,
          onInit = props.onInit,
          onPan = props.onPan,
          onPinch = props.onPinch;
        // 统一绑定事件
        if (pan !== false) {
          var _context$gesture, _context$gesture2, _context$gesture3;
          (_context$gesture = context.gesture) === null || _context$gesture === void 0 ? void 0 : _context$gesture.on('panstart', function () {
            _this4.onStart();
            onPanStart({
              scale: scale
            });
          });
          (_context$gesture2 = context.gesture) === null || _context$gesture2 === void 0 ? void 0 : _context$gesture2.on('pan', function (ev) {
            _this4.onPan(ev);
            onPan(ev);
          });
          (_context$gesture3 = context.gesture) === null || _context$gesture3 === void 0 ? void 0 : _context$gesture3.on('panend', function () {
            _this4.onEnd();
            onPanEnd({
              scale: scale
            });
          });
        }
        if (pinch !== false) {
          var _context$gesture4, _context$gesture5, _context$gesture6;
          (_context$gesture4 = context.gesture) === null || _context$gesture4 === void 0 ? void 0 : _context$gesture4.on('pinchstart', function () {
            _this4.onStart();
            onPinchStart();
          });
          (_context$gesture5 = context.gesture) === null || _context$gesture5 === void 0 ? void 0 : _context$gesture5.on('pinch', function (ev) {
            _this4.onPinch(ev);
            onPinch(ev);
          });
          (_context$gesture6 = context.gesture) === null || _context$gesture6 === void 0 ? void 0 : _context$gesture6.on('pinchend', function () {
            _this4.onEnd();
            onPanEnd({
              scale: scale
            });
          });
        }
        if (swipe !== false) {
          var _context$gesture7;
          (_context$gesture7 = context.gesture) === null || _context$gesture7 === void 0 ? void 0 : _context$gesture7.on('swipe', this.onSwipe);
        }
        onInit({
          scale: scale
        });
      }
    }, {
      key: "_clearEvents",
      value: function _clearEvents() {
        var _this5 = this;
        var context = this.context,
          props = this.props,
          scale = this.scale;
        var canvas = context.canvas;
        var onPinchEnd = props.onPinchEnd,
          onPanEnd = props.onPanEnd,
          onPinchStart = props.onPinchStart,
          pan = props.pan,
          pinch = props.pinch,
          onPan = props.onPan,
          onPinch = props.onPinch,
          swipe = props.swipe;
        // 统一解绑事件
        if (pan !== false) {
          canvas.off('panstart', function () {
            _this5.onStart();
            onPinchStart();
          });
          canvas.off('pan', function (ev) {
            _this5.onPan(ev);
            onPan(ev);
          });
          canvas.off('panend', function () {
            _this5.onEnd();
            onPanEnd({
              scale: scale
            });
          });
        }
        if (pinch !== false) {
          canvas.off('pinchstart', function () {
            _this5.onStart();
            onPinchStart();
          });
          canvas.off('pinch', function (ev) {
            _this5.onPinch(ev);
            onPinch(ev);
          });
          canvas.off('pinchend', function () {
            _this5.onEnd();
            onPinchEnd({
              scale: scale
            });
          });
        }
        if (swipe !== false) {
          canvas.off('swipe', this.onSwipe);
        }
      }
    }]);
    return Zoom;
  }(Component);

  var withScrollBar = (function (View) {
    return /*#__PURE__*/function (_Zoom) {
      _inherits$1(ScrollBar, _Zoom);
      var _super = _createSuper$1(ScrollBar);
      function ScrollBar() {
        _classCallCheck$1(this, ScrollBar);
        return _super.apply(this, arguments);
      }
      _createClass$3(ScrollBar, [{
        key: "willMount",
        value: function willMount() {
          _get$1(_getPrototypeOf$1(ScrollBar.prototype), "willMount", this).call(this);
          var context = this.context,
            props = this.props;
          var visible = props.visible,
            _props$position = props.position,
            position = _props$position === void 0 ? 'bottom' : _props$position,
            _props$margin = props.margin,
            margin = _props$margin === void 0 ? '16px' : _props$margin,
            chart = props.chart;
          var marginNumber = context.px2hd(margin);
          if (visible === false) {
            return null;
          }
          chart.updateCoordFor(this, {
            position: position,
            width: position === 'left' || position === 'right' ? marginNumber : 0,
            height: position === 'bottom' || position === 'top' ? marginNumber : 0
          });
        }
      }, {
        key: "render",
        value: function render() {
          var props = this.props,
            state = this.state;
          var visible = props.visible;
          if (visible === false) {
            return null;
          }
          return jsx(View, _objectSpread(_objectSpread({
            position: "bottom"
          }, props), state));
        }
      }]);
      return ScrollBar;
    }(Zoom);
  });

  var Horizontal = (function (props, context) {
    var coord = props.coord,
      range = props.range,
      position = props.position,
      layout = props.layout;
    var left = coord.left,
      width = coord.width;
    var top = layout.top,
      height = layout.height;
    var _ref = (range === null || range === void 0 ? void 0 : range.x) || (range === null || range === void 0 ? void 0 : range.y),
      _ref2 = _slicedToArray$1(_ref, 2),
      start = _ref2[0],
      end = _ref2[1];
    var barLeft = width * start;
    var barWidth = width * (end - start);
    if (isNaN(barWidth)) return;
    return jsx("group", {
      style: {
        display: 'flex',
        left: left,
        top: position === 'top' ? top - context.px2hd('8px') : top + height
      }
    }, jsx("line", {
      style: {
        display: 'flex',
        position: 'absolute',
        left: 0,
        width: width,
        height: 0,
        stroke: 'rgba(202, 215, 239, .2)',
        lineCap: 'round',
        lineWidth: '8px'
      }
    }), jsx("line", {
      style: {
        display: 'flex',
        position: 'absolute',
        left: barLeft,
        width: barWidth,
        height: 0,
        stroke: 'rgba(202, 215, 239, .5)',
        lineCap: 'round',
        lineWidth: '8px'
      }
    }));
  });

  var Vertical = (function (props, context) {
    var coord = props.coord,
      range = props.range,
      position = props.position,
      layout = props.layout;
    var top = coord.top,
      height = coord.height;
    var left = layout.left,
      width = layout.width;
    var _ref = (range === null || range === void 0 ? void 0 : range.y) || (range === null || range === void 0 ? void 0 : range.x),
      _ref2 = _slicedToArray$1(_ref, 2),
      start = _ref2[0],
      end = _ref2[1];
    var barTop = height * start;
    var barHeight = height * (end - start);
    return jsx("group", {
      style: {
        display: 'flex',
        top: top,
        left: position === 'left' ? left - context.px2hd('8px') : left + width
      }
    }, jsx("line", {
      style: {
        position: 'absolute',
        top: 0,
        left: 0,
        width: 0,
        height: height,
        stroke: 'rgba(202, 215, 239, .2)',
        lineCap: 'round',
        lineWidth: '8px'
      }
    }), jsx("line", {
      style: {
        position: 'absolute',
        top: barTop,
        width: 0,
        height: barHeight,
        stroke: 'rgba(202, 215, 239, .5)',
        lineCap: 'round',
        lineWidth: '8px'
      }
    }));
  });

  var ScrollBarView = (function (props) {
    var position = props.position,
      mode = props.mode;
    if (mode.length > 1) {
      return jsx("group", null, jsx(Vertical, _objectSpread({}, props)), jsx(Horizontal, _objectSpread({}, props)));
    }
    if (position === 'left' || position === 'right') {
      return jsx(Vertical, _objectSpread({}, props));
    }
    return jsx(Horizontal, _objectSpread({}, props));
  });

  var index$c = withScrollBar(ScrollBarView);

  exports.ArcGuide = ArcGuide;
  exports.Area = index$1;
  exports.AreaView = AreaView;
  exports.Axis = index$4;
  exports.AxisView = AxisView;
  exports.Canvas = Canvas$1;
  exports.CanvasRenderer = Renderer;
  exports.Chart = Chart;
  exports.Children = Children;
  exports.Component = Component;
  exports.Fragment = fragment;
  exports.Gauge = index$b;
  exports.GaugeView = GaugeView;
  exports.Geometry = Geometry;
  exports.Guide = index$6;
  exports.ImageGuide = ImageGuide;
  exports.Interval = index$2;
  exports.IntervalView = intervalView;
  exports.Legend = index$5;
  exports.LegendView = LegendView;
  exports.Line = index;
  exports.LineGuide = LineGuide;
  exports.LineView = LineView;
  exports.LottieGuide = LottieGuide;
  exports.PieLabel = index$a;
  exports.PieLabelView = PieLabelView;
  exports.Point = index$3;
  exports.PointGuide = PointGuide;
  exports.PointView = PointView;
  exports.RectGuide = RectGuide;
  exports.ScrollBar = index$c;
  exports.ScrollBarView = ScrollBarView;
  exports.Sunburst = index$9;
  exports.SunburstView = SunburstView;
  exports.TagGuide = TagGuide;
  exports.TextGuide = TextGuide;
  exports.Timeline = Timeline;
  exports.Tooltip = index$7;
  exports.TooltipView = TooltipView;
  exports.Treemap = index$8;
  exports.TreemapView = TreemapView;
  exports.Zoom = Zoom;
  exports.computeLayout = computeLayout$1;
  exports.createElement = jsx;
  exports.createRef = createRef;
  exports.jsx = jsx;
  exports.withArea = withArea;
  exports.withAxis = withAxis;
  exports.withGauge = withGauge;
  exports.withGuide = withGuide;
  exports.withInterval = withInterval;
  exports.withLegend = withLegend;
  exports.withLine = withLine;
  exports.withPieLabel = withPieLabel;
  exports.withPoint = withPoint;
  exports.withScrollBar = withScrollBar;
  exports.withSunburst = withSunburst;
  exports.withTooltip = withTooltip;
  exports.withTreemap = withTreemap;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
